@@ -46,13 +46,13 @@ const unsigned int KIOInputStream_impl::PACKET_COUNT = 10;
 KIOInputStream_impl::KIOInputStream_impl() : m_packetSize(2048)
 {
 	m_job = 0;
-	m_data = 0;
 	m_finished = false;
 	m_firstBuffer = false;
 	m_packetBuffer = 16;
 	m_streamStarted = false;
 	m_streamSuspended = false;
 	m_streamPulled = false;
+	m_size = 0;
 }
 
 KIOInputStream_impl::~KIOInputStream_impl()
@@ -65,18 +65,19 @@ void KIOInputStream_impl::streamStart()
 {
 	// prevent kill/reconnect
 	if (m_streamStarted) {
-		kdDebug(400) << "not restarting stream!\n";
+		kdDebug() << "not restarting stream!\n";
 		if (m_job->isSuspended())
 			m_job->resume();
 		return;
 	}
 
-	kdDebug(400) << "(re)starting stream\n";
+	kdDebug() << "(re)starting stream\n";
 
 	if(m_job != 0)
 		m_job->kill();
 	m_job = KIO::get(m_url, false, false);
-	m_job->addMetaData("accept", "audio/x-mp3, video/mpeg, application/x-ogg");
+
+	m_job->addMetaData("accept", "audio/x-mp3, video/mpeg, application/ogg");
 	m_job->addMetaData("UserAgent", QString::fromLatin1("aRts/") + QString::fromLatin1(ARTS_VERSION));
 
 	QObject::connect(m_job, SIGNAL(data(KIO::Job *, const QByteArray &)),
@@ -85,13 +86,15 @@ void KIOInputStream_impl::streamStart()
 			 this, SLOT(slotResult(KIO::Job *)));		     
 	QObject::connect(m_job, SIGNAL(mimetype(KIO::Job *, const QString &)),
 			 this, SLOT(slotScanMimeType(KIO::Job *, const QString &)));
+	QObject::connect(m_job, SIGNAL(totalSize( KIO::Job *, KIO::filesize_t)),
+			 this, SLOT(slotTotalSize(KIO::Job *, KIO::filesize_t)));
 
 	m_streamStarted = true;
 }
 
 void KIOInputStream_impl::streamEnd()
 {
-	kdDebug(400) << "streamEnd()\n";
+	kdDebug() << "streamEnd()\n";
 
 	if(m_job != 0)
 	{
@@ -101,6 +104,8 @@ void KIOInputStream_impl::streamEnd()
 						this, SLOT(slotResult(KIO::Job *)));		     
 		QObject::disconnect(m_job, SIGNAL(mimetype(KIO::Job *, const QString &)),
 				 this, SLOT(slotScanMimeType(KIO::Job *, const QString &)));
+		QObject::disconnect(m_job, SIGNAL(totalSize( KIO::Job *, KIO::filesize_t)),
+				 this, SLOT(slotTotalSize(KIO::Job *, KIO::filesize_t)));
 
 		if ( m_streamPulled )
 			outdata.endPull();
@@ -115,6 +120,7 @@ void KIOInputStream_impl::streamEnd()
 bool KIOInputStream_impl::openURL(const std::string& url)
 {
 	m_url = KURL(url.c_str());
+	m_size = 0;
 	return true;
 }
 
@@ -125,7 +131,7 @@ void KIOInputStream_impl::slotData(KIO::Job *, const QByteArray &data)
 
 	QDataStream dataStream(m_data, IO_WriteOnly | IO_Append);
 	dataStream.writeRawBytes(data.data(), data.size());
-	//kdDebug() << "STREAMING: buffersize = " << m_data.size() << " bytes" << endl;
+	kdDebug() << "STREAMING: buffersize = " << m_data.size() << " bytes" << endl;
 	
 	processQueue();
 }
@@ -147,10 +153,15 @@ void KIOInputStream_impl::slotResult(KIO::Job *job)
 
 void KIOInputStream_impl::slotScanMimeType(KIO::Job *, const QString &mimetype)
 {
-	kdDebug(400) << "got mimetype: " << mimetype << endl;
+	kdDebug() << "got mimetype: " << mimetype << endl;
 	emit mimeTypeFound(mimetype);
 }
 
+void KIOInputStream_impl::slotTotalSize(KIO::Job *, KIO::filesize_t size)
+{
+	m_size = size;
+}
+
 bool KIOInputStream_impl::eof()
 {
 	return (m_finished && m_data.size() == 0);
@@ -163,7 +174,7 @@ bool KIOInputStream_impl::seekOk()
 
 long KIOInputStream_impl::size()
 {
-	return m_data.size();
+	return m_size ? m_size : m_data.size();
 }
 
 long KIOInputStream_impl::seek(long)
@@ -177,19 +188,19 @@ void KIOInputStream_impl::processQueue()
 	{
 		if(m_data.size() > (m_packetBuffer * m_packetSize * 2) && !m_job->isSuspended())
 		{
-			kdDebug(400) << "STREAMING: suspend job" << endl;
+			kdDebug() << "STREAMING: suspend job" << endl;
 			m_job->suspend();
 		}
 		else if(m_data.size() < (m_packetBuffer * m_packetSize) && m_job->isSuspended())
 		{
-			kdDebug(400) << "STREAMING: resume job" << endl;
+			kdDebug() << "STREAMING: resume job" << endl;
 			m_job->resume();
 		}
 	}
 
 	if (!m_firstBuffer) {
 		if(m_data.size() < (m_packetBuffer * m_packetSize * 2) ) {
-			//kdDebug() << "STREAMING: Buffering in progress... (Needed bytes before it starts to play: " << ((m_packetBuffer * m_packetSize * 2) - m_data.size()) << ")" << endl;
+			kdDebug() << "STREAMING: Buffering in progress... (Needed bytes before it starts to play: " << ((m_packetBuffer * m_packetSize * 2) - m_data.size()) << ")" << endl;
 			return;
 		} else {
 			m_firstBuffer = true;
