@@ -61,7 +61,7 @@ extern char *strchr(const char *, int);
  * Each transport is assigned a unique transport id.
  *
  * New transports can be added by adding an entry in this table.
- * For compatiblity, the transport ids should never be renumbered.
+ * For compatibility, the transport ids should never be renumbered.
  * Always add to the end of the list.
  */
 
@@ -202,8 +202,8 @@ TRANS(ParseAddress) (char *address, char **protocol, char **host, char **port)
 {
     /*
      * For the font library, the address is a string formatted
-     * as "protocol/host:port[/catalogue]".  Note that the catologue
-     * is optional.  At this time, the catologue info is ignored, but
+     * as "protocol/host:port[/catalog]".  Note that the catalog
+     * is optional.  At this time, the catalog info is ignored, but
      * we have to parse it anyways.
      *
      * Other than fontlib, the address is a string formatted
@@ -211,7 +211,6 @@ TRANS(ParseAddress) (char *address, char **protocol, char **host, char **port)
      *
      * If the protocol part is missing, then assume INET.
      * If the protocol part and host part are missing, then assume local.
-     * If a "::" is found then assume DNET.
      */
 
     char	*mybuf, *tmpptr;
@@ -233,7 +232,7 @@ TRANS(ParseAddress) (char *address, char **protocol, char **host, char **port)
 
     if ((mybuf = strpbrk (mybuf,"/:")) == NULL)
     {
-	/* adress is in a bad format */
+	/* address is in a bad format */
 	*protocol = NULL;
 	*host = NULL;
 	*port = NULL;
@@ -282,7 +281,7 @@ TRANS(ParseAddress) (char *address, char **protocol, char **host, char **port)
 
     _host = mybuf;
 
-    if ((mybuf = strchr (mybuf,':')) == NULL)
+    if ((mybuf = strrchr (mybuf,':')) == NULL)
     {
 	*protocol = NULL;
 	*host = NULL;
@@ -299,28 +298,20 @@ TRANS(ParseAddress) (char *address, char **protocol, char **host, char **port)
 	_host = hostnamebuf;
     }
 
-    /* Check for DECnet */
-
-    if (*mybuf == ':')
-    {
-	_protocol = "dnet";
-	mybuf++;
-    }
-
     /* Get the port */
 
     _port = mybuf;
 
 #if defined(FONT_t) || defined(FS_t)
     /*
-     * Is there an optional catalogue list?
+     * Is there an optional catalog list?
      */
 
     if ((mybuf = strchr (mybuf,'/')) != NULL)
 	*mybuf ++= '\0';
 
     /*
-     * The rest, if any, is the (currently unused) catalogue list.
+     * The rest, if any, is the (currently unused) catalog list.
      *
      * _catalogue = mybuf;
      */
@@ -1025,7 +1016,7 @@ TRANS(MakeAllCOTSServerListeners) (char *port, int *partial, int *count_ret,
 	if (trans->flags&TRANS_ALIAS || trans->flags&TRANS_NOLISTEN)
 	    continue;
 
-	snprintf(buffer, 256, "%s/:%s", trans->TransName, port ? port : "");
+	snprintf(buffer, sizeof(buffer), "%s/:%s", trans->TransName, port ? port : "");
 
 	PRMSG (5,"MakeAllCOTSServerListeners: opening %s\n",
 	       buffer, 0, 0);
@@ -1127,7 +1118,7 @@ TRANS(MakeAllCLTSServerListeners) (char *port, int *partial, int *count_ret,
 	if (trans->flags&TRANS_ALIAS || trans->flags&TRANS_NOLISTEN)
 	    continue;
 
-	snprintf(buffer, 256, "%s/:%s", trans->TransName, port ? port : "");
+	snprintf(buffer, sizeof(buffer), "%s/:%s", trans->TransName, port ? port : "");
 
 	PRMSG (5,"MakeAllCLTSServerListeners: opening %s\n",
 	    buffer, 0, 0);
