@@ -50,9 +50,10 @@ from The Open Group.
 #ifndef _XTRANS_H_
 #define _XTRANS_H_
 
-#ifndef QWS
-#include <X11/Xfuncproto.h>
-#include <X11/Xos.h>
+#include "config.h"
+#if defined Q_WS_X11 && ! defined K_WS_QTONLY
+#include <X11/Xfuncproto.h> // schroder
+#include <X11/Xos.h> // schroder
 #else
 #include <KDE-ICE/Xfuncproto.h>
 #include <fcntl.h>
@@ -120,12 +121,12 @@ static char* __xtransname = "_FontTrans";
 
 #ifdef ICE_t
 #if (defined(__STDC__) && !defined(UNIXCPP)) || defined(ANSICPP)
-#define TRANS(func) _KDE_IceTrans##func
+#define TRANS(func) _kde_IceTrans##func
 #else
-#define TRANS(func) _KDE_IceTrans/**/func
+#define TRANS(func) _kde_IceTrans/**/func
 #endif
 #ifdef XTRANSDEBUG
-static char* __xtransname = "_KDE_IceTrans";
+static char* __xtransname = "_kde_IceTrans";
 #endif
 #endif /* ICE_t */
 
