@@ -43,6 +43,11 @@ CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 
 #include "marshall.cpp"
 
+#if defined Q_WS_X11
+#include <X11/Xlib.h>
+#include <X11/Xatom.h>
+#endif
+
 typedef QMap<QString, QString> UserList;
 
 static DCOPClient* dcop = 0;
@@ -167,10 +172,10 @@ int callFunction( const char* app, const char* obj, const char* func, const QCSt
 	if ( !ok && args.isEmpty() )
 	    goto doit;
 	if ( !ok )
-        {
+	{
 	    qWarning( "object not accessible" );
 	    return( 1 );
-        }
+	}
 	for ( QCStringList::Iterator it = funcs.begin(); it != funcs.end(); ++it ) {
 	    int l = (*it).find( '(' );
 	    int s = (*it).find( ' ');
@@ -256,11 +261,11 @@ int callFunction( const char* app, const char* obj, const char* func, const QCSt
 	}
 	QString fc = f.left( left );
 	fc += '(';
-	bool first = TRUE;
+	bool first = true;
 	for ( QStringList::Iterator it = types.begin(); it != types.end(); ++it ) {
 	    if ( !first )
 		fc +=",";
-	    first = FALSE;
+	    first = false;
 	    fc += *it;
 	}
 	fc += ')';
@@ -291,7 +296,7 @@ int callFunction( const char* app, const char* obj, const char* func, const QCSt
         {
             QCString replyString = demarshal( reply, replyType );
             if ( !replyString.isEmpty() )
-            printf( "%s\n", replyString.data() );
+                printf( "%s\n", replyString.data() );
             else
                 printf("\n");
         }
@@ -304,20 +309,32 @@ int callFunction( const char* app, const char* obj, const char* func, const QCSt
  */
 void showHelp( int exitCode = 0 )
 {
-    cout_ << "Usage: dcop [options] [application [object [function [arg1] [arg2] ... ] ] ]" << endl
-	 << "" << endl
+#ifdef DCOPQUIT
+   cout_ << "Usage: dcopquit [options] [application]" << endl
+#else
+   cout_ << "Usage: dcop [options] [application [object [function [arg1] [arg2] ... ] ] ]" << endl
+#endif
+         << "" << endl
 	 << "Console DCOP client" << endl
 	 << "" << endl
 	 << "Generic options:" << endl
 	 << "  --help          Show help about options" << endl
 	 << "" << endl
 	 << "Options:" << endl
-	 << "  --pipe          Call DCOP for each line read from stdin" << endl
-	 << "                  This is roughly equivalent to calling" << endl
-	 << "                      'while read line ; do dcop $line ; done'" << endl
-	 << "                  but because no new dcop instance has to be started for" << endl
-	 << "                  each line this is generally much faster, especially" << endl
-	 << "                  for the slower GNU dynamic linkers." << endl
+	 << "  --pipe          Call DCOP for each line read from stdin. The string '%1'" << endl
+	 << "                  will be used in the argument list as a placeholder for" << endl
+	 << "                  the substituted line." << endl
+	 << "                  For example," << endl
+	 << "                      dcop --pipe konqueror html-widget1 evalJS %1" << endl
+	 << "                  is equivalent to calling" << endl
+	 << "                      while read line ; do" << endl
+	 << "                          dcop konqueror html-widget1 evalJS \"$line\"" << endl
+	 << "                      done" << endl
+	 << "                  in bash, but because no new dcop instance has to be started" << endl
+	 << "                  for each line this is generally much faster, especially for" << endl
+	 << "                  the slower GNU dynamic linkers." << endl
+	 << "                  The '%1' placeholder cannot be used to replace e.g. the" << endl
+	 << "                  program, object or method name." << endl
 	 << "  --user <user>   Connect to the given user's DCOP server. This option will" << endl
 	 << "                  ignore the values of the environment vars $DCOPSERVER and" << endl
 	 << "                  $ICEAUTHORITY, even if they are set." << endl
@@ -333,6 +350,9 @@ void showHelp( int exitCode = 0 )
 	 << "  --all-sessions  Send to all sessions found. Only works with the --user" << endl
 	 << "                  and --all-users options." << endl
 	 << "  --list-sessions List all active KDE session for a user or all users." << endl
+	 << "  --no-user-time  Don't update the user activity timestamp in the called" << endl
+	 << "                  application (for usage in scripts running" << endl
+	 << "                  in the background)." << endl
 	 << endl;
 
     exit( exitCode );
@@ -405,11 +425,36 @@ QStringList dcopSessionList( const QString &user, const QString &home )
     return result;
 }
 
+void sendUserTime( const char* app )
+{
+#if defined Q_WS_X11
+    static unsigned long time = 0;
+    if( time == 0 )
+    {
+        Display* dpy = XOpenDisplay( NULL );
+        if( dpy != NULL )
+        {
+            Window w = XCreateSimpleWindow( dpy, DefaultRootWindow( dpy ), 0, 0, 1, 1, 0, 0, 0 );
+            XSelectInput( dpy, w, PropertyChangeMask );
+            unsigned char data[ 1 ];
+            XChangeProperty( dpy, w, XA_ATOM, XA_ATOM, 8, PropModeAppend, data, 1 );
+            XEvent ev;
+            XWindowEvent( dpy, w, PropertyChangeMask, &ev );
+            time = ev.xproperty.time;
+            XDestroyWindow( dpy, w );
+        }
+    }
+    DCOPRef( app, "MainApplication-Interface" ).call( "updateUserTimestamp", time );
+#else
+// ...
+#endif
+}
+
 /**
  * Do the actual DCOP call
  */
 int runDCOP( QCStringList args, UserList users, Session session,
-              const QString sessionName, bool readStdin )
+              const QString sessionName, bool readStdin, bool updateUserTime )
 {
     bool DCOPrefmode=false;
     QCString app;
@@ -464,7 +509,7 @@ int runDCOP( QCStringList args, UserList users, Session session,
 //    char *dcopStr = 0L;
     QString dcopServer;
 
-    for( it = users.begin(); it != users.end() || firstRun; it++ )
+    for( it = users.begin(); it != users.end() || firstRun; ++it )
     {
 	firstRun = false;
 
@@ -490,8 +535,8 @@ int runDCOP( QCStringList args, UserList users, Session session,
 		    cout_ << "for user " << *it << " ";
 		cout_ << ":" << endl;
 
-		QStringList::Iterator sIt;
-		for( sIt = sessions.begin(); sIt != sessions.end(); sIt++ )
+		QStringList::Iterator sIt = sessions.begin();
+		for( ; sIt != sessions.end(); ++sIt )
 		    cout_ << "  " << *sIt << endl;
 
 		cout_ << endl;
@@ -593,7 +638,7 @@ int runDCOP( QCStringList args, UserList users, Session session,
 	// in user. In this case we don't have a session, but still want
 	// to iterate the loop once.
 	QStringList::Iterator sIt = sessions.begin();
-	for( ; sIt != sessions.end() || users.isEmpty(); sIt++ )
+	for( ; sIt != sessions.end() || users.isEmpty(); ++sIt )
 	{
 	    if( !presetDCOPServer && !users.isEmpty() )
 	    {
@@ -656,12 +701,14 @@ int runDCOP( QCStringList args, UserList users, Session session,
 		break;
 	    case 3:
 	    default:
+                if( updateUserTime )
+                    sendUserTime( app );
 		if( readStdin )
 		{
 		    QCStringList::Iterator replaceArg = params.end();
 
-		    QCStringList::Iterator it;
-		    for( it = params.begin(); it != params.end(); it++ )
+		    QCStringList::Iterator it = params.begin();
+		    for( ; it != params.end(); ++it )
 			if( *it == "%1" )
 			    replaceArg = it;
 
@@ -711,6 +758,7 @@ int main( int argc, char** argv )
     QString user;
     Session session = DefaultSession;
     QString sessionName;
+    bool updateUserTime = true;
 
     cin_.setEncoding( QTextStream::Locale );
 
@@ -777,6 +825,11 @@ int main( int argc, char** argv )
 	    session = AllSessions;
 	    numOptions ++;
 	}
+        else if( strcmp( argv[ pos ], "--no-user-time" ) == 0 )
+        {
+            updateUserTime = false;
+            numOptions ++;
+        }
 	else if( argv[ pos ][ 0 ] == '-' )
 	{
 	    cerr_ << "Unknown command-line option '" << argv[ pos ]
@@ -790,8 +843,34 @@ int main( int argc, char** argv )
     argc -= numOptions;
 
     QCStringList args;
+    
+#ifdef DCOPQUIT
+    if (argc > 1)
+    {
+       QCString prog = argv[ numOptions + 1 ];
+       
+       if (!prog.isEmpty())
+       {
+          args.append( prog );
+       
+          // Pass as-is if it ends with a wildcard
+          if (prog[prog.length()-1] != '*')
+          {
+             // Strip a trailing -<PID> part.
+             int i = prog.findRev('-');
+             if ((i >= 0) && prog.mid(i+1).toLong())
+             {
+                prog = prog.left(i);      
+             }
+             args.append( "qt/"+prog ); 
+             args.append( "quit()" );
+          }
+       }
+    }
+#else
     for( int i = numOptions; i < argc + numOptions - 1; i++ )
-	args.append( argv[ i + 1 ] );
+       args.append( argv[ i + 1 ] );
+#endif
 
     if( readStdin && args.count() < 3 )
     {
@@ -832,7 +911,7 @@ int main( int argc, char** argv )
     else if( !user.isEmpty() )
 	users[ user ] = userList()[ user ];
 
-    int retval = runDCOP( args, users, session, sessionName, readStdin );
+    int retval = runDCOP( args, users, session, sessionName, readStdin, updateUserTime );
 
     return retval;
 }
