@@ -107,11 +107,11 @@ bool findObject( const char* app, const char* obj, const char* func, QCStringLis
 	}
 	QString fc = f.left( left );
 	fc += '(';
-	bool first = TRUE;
+	bool first = true;
 	for ( QStringList::Iterator it = types.begin(); it != types.end(); ++it ) {
 	    if ( !first )
 		fc +=",";
-	    first = FALSE;
+	    first = false;
 	    fc += *it;
 	}
 	fc += ')';
