@@ -26,15 +26,15 @@ CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 bool mkBool( const QString& s )
 {
     if ( s.lower()  == "true" )
-	return TRUE;
+	return true;
     if ( s.lower()  == "yes" )
-	return TRUE;
+	return true;
     if ( s.lower()  == "on" )
-	return TRUE;
+	return true;
     if ( s.toInt() != 0 )
-	return TRUE;
+	return true;
 
-    return FALSE;
+    return false;
 }
 
 QPoint mkPoint( const QString &str )
@@ -106,27 +106,31 @@ QCString demarshal( QDataStream &stream, const QString &type )
     {
         int i;
         stream >> i;
-        result.sprintf( "%d", i );
-    } else if ( type == "uint" )
+        result.setNum( i );
+    } else if ( type == "uint" || type == "Q_UINT32" )
     {
         uint i;
         stream >> i;
-        result.sprintf( "%d", i );
+        result.setNum( i );
     } else if ( type == "long" )
     {
         long l;
         stream >> l;
-        result.sprintf( "%ld", l );
+        result.setNum( l );
     } else if ( type == "float" )
     {
         float f;
         stream >> f;
-        result.sprintf( "%f", (double)f );
+        result.setNum( f, 'f' );
     } else if ( type == "double" )
     {
         double d;
         stream >> d;
-        result.sprintf( "%f", d );
+        result.setNum( d, 'f' );
+    } else if ( type == "Q_UINT64" ) {
+        Q_UINT64 i;
+        stream >> i;
+        result.sprintf( "%llu", i );
     } else if ( type == "bool" )
     {
         bool b;
@@ -271,6 +275,12 @@ void marshall( QDataStream &arg, QCStringList args, uint &i, QString type )
 	arg << s.toUInt();
     else if ( type == "unsigned int" )
 	arg << s.toUInt();
+    else if ( type == "Q_UINT32" )
+	arg << s.toUInt();
+    else if ( type == "Q_UINT64" ) {
+	QVariant qv = QVariant( s );
+	arg << qv.toULongLong();
+    }
     else if ( type == "long" )
 	arg << s.toLong();
     else if ( type == "long int" )
@@ -314,8 +324,12 @@ void marshall( QDataStream &arg, QCStringList args, uint &i, QString type )
 	    arg << QVariant( mkColor( s.mid(7, s.length()-8) ) );
 	else
 	    arg << QVariant( s );
-    } else if ( type.startsWith("QValueList<")) {
-	type = type.mid(11, type.length() - 12);
+    } else if ( type.startsWith("QValueList<") ||
+	        type == "KURL::List" ) {
+	if ( type == "KURL::List" )
+            type = "KURL";
+        else
+	    type = type.mid(11, type.length() - 12);
 	QStringList list;
 	QString delim = s;
 	if (delim == "[")
