@@ -28,6 +28,7 @@ CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 #include <qvaluelist.h>
 #include <qstring.h>
 #include <kdatastream.h> // needed for proper bool marshalling
+#include <kdemacros.h>
 
 class DCOPObjectProxy;
 class DCOPClientPrivate;
@@ -40,26 +41,26 @@ typedef QValueList<QCString> QCStringList;
  * for KDE applications.
  *
  * This class provides IPC and RPC for KDE applications.  Usually you
- * will not have to instantiate one yourself because @ref KApplication
+ * will not have to instantiate one yourself because KApplication
  * contains a method to return a pointer to a DCOPClient object which
  * can be used for your whole application.
  *
  * Before being able to send or receive any DCOP messages, you will have
  * to attach your client object to the DCOP server, and then register
- * your application with a specific name. See @ref attach()
- * and @ref registerAs() for
+ * your application with a specific name. See attach()
+ * and registerAs() for
  * more information.
  *
- * Data to be sent should be serialized into a @ref QDataStream which was
- * initialized with the @ref QByteArray that you actually intend to send
+ * Data to be sent should be serialized into a QDataStream which was
+ * initialized with the QByteArray that you actually intend to send
  * the data in.  An example of how you might do this:
  *
- * <pre>
+ * \code
  *   QByteArray data;
  *   QDataStream arg(data, IO_WriteOnly);
  *   arg << QString("This is text I am serializing");
- *   client->send("someApp", "someObject", "someFunction", arg);
- * </pre>
+ *   client->send("someApp", "someObject", "someFunction(QString)", data);
+ * \endcode
  *
  * @see KApplication::dcopClient()
  * @author Preston Brown <pbrown@kde.org>, Matthias Ettrich <ettrich@kde.org>
@@ -95,12 +96,12 @@ class DCOPClient : public QObject
    *
    * Naturally, only attached application can use DCOP services.
    *
-   * If a @ref QApplication object exists then client registers itself as
-   * @ref QApplication::name() + "-" + <pid>.
-   * If no @ref QApplication object exists the client registers itself as
+   * If a QApplication object exists then client registers itself as
+   * QApplication::name() + "-" + \<pid\>.
+   * If no QApplication object exists the client registers itself as
    * "anonymous".
    *
-   * If you want to register differently, you should use @ref registerAs()
+   * If you want to register differently, you should use registerAs()
    * instead.
    *
    * @return true if attaching was successful.
@@ -109,9 +110,9 @@ class DCOPClient : public QObject
 
   /**
    * @internal
-   * Internal function for @ref KUniqueApplication to register the @ref DCOPClient
+   * Internal function for KUniqueApplication to register the DCOPClient
    * with the application in case the application didn't exist at the
-   * time the @ref DCOPClient was created.
+   * time the DCOPClient was created.
    */
   void bindToApp();
 
@@ -152,7 +153,7 @@ class DCOPClient : public QObject
    * Returns whether the DCOP - Qt bridge is enabled.
    * By default the DCOP - Qt bridge is enabled.
    * @return true if Qt objects are accessible over DCOP
-   * @since 3.1
+   * @since 3.0.2
    */
   bool qtBridgeEnabled(); // ### KDE 4.0: make const
 
@@ -161,7 +162,7 @@ class DCOPClient : public QObject
    * via DCOP.
    * By default the DCOP - Qt bridge is enabled.
    * @param b true to make Qt objects accessible over DCOP
-   * @since 3.1
+   * @since 3.0.2
    */
   void setQtBridgeEnabled(bool b);
 
@@ -169,7 +170,7 @@ class DCOPClient : public QObject
    * Registers at the DCOP server.
    *
    * If the application was already registered,
-   * the registration will be re-done with the new @ref appId.
+   * the registration will be re-done with the new appId.
    *
    * @p appId is a @p unique application/program id that the server
    * will use to associate requests with. If there is already an application
@@ -179,9 +180,9 @@ class DCOPClient : public QObject
    *
    * Registration is necessary if you want to allow other clients to talk
    * to you.  They can do so using your @p appId as first parameter
-   * for @ref send() or @ref call(). If you just want to talk to
+   * for send() or call(). If you just want to talk to
    *  other clients, you
-   * do not need to register at the server. In that case @ref attach() is
+   * do not need to register at the server. In that case attach() is
    * enough.
    * It will implicitly register you as "anonymous".
    *
@@ -201,7 +202,7 @@ class DCOPClient : public QObject
   /**
    * Returns the current app id or a null string if the application
    * hasn't yet been registered.
-   * @param the application id, or QString::null if not registered
+   * @return the application id, or QString::null if not registered
    */
   QCString appId() const;
 
@@ -213,7 +214,7 @@ class DCOPClient : public QObject
 
   /**
    * Temporarily suspends processing of DCOP events.
-   * This can be usefull if you need to show e.g. a dialog before
+   * This can be useful if you need to show e.g. a dialog before
    * your application is ready to accept DCOP requests. Normally the
    * dialog would start an event loop and in this event loop DCOP
    * requests will be handled.
@@ -255,8 +256,8 @@ class DCOPClient : public QObject
 
   /**
    * This function acts exactly the same as the above, but the data
-   * parameter can be specified as a @ref QString for convenience.
-   * 
+   * parameter can be specified as a QString for convenience.
+   *
    * @param remApp The remote application id.
    * @param remObj The name of the remote object.
    * @param remFun The remote function in the specified object to call.
@@ -271,7 +272,7 @@ class DCOPClient : public QObject
    * Performs a synchronous send and receive.
    *
    *  The parameters are the same as for send, with the exception of
-   *  another @ref QByteArray being provided for results to be
+   *  another QByteArray being provided for results to be
    *  (optionally) returned in.
    *
    * A call blocks the application until the process receives the
@@ -289,14 +290,58 @@ class DCOPClient : public QObject
    * @param replyData the data of the reply will be written here
    * @param useEventLoop if true the event loop will be started when
    *         the call blocks too long
+   * @param timeout timeout for the call in miliseconds, or -1 for no timeout
    * @return true if successful, false otherwise
+   *
+   * @since 3.2
+   *
    * @see send()
    */
+  bool call(const QCString &remApp, const QCString &remObj,
+	    const QCString &remFun, const QByteArray &data,
+	    QCString& replyType, QByteArray &replyData,
+	    bool useEventLoop/*=false*/, int timeout/*=-1*/);
+  /**
+   * @deprecated
+   */
+  // KDE4 merge with above
   bool call(const QCString &remApp, const QCString &remObj,
 	    const QCString &remFun, const QByteArray &data,
 	    QCString& replyType, QByteArray &replyData,
 	    bool useEventLoop=false);
 
+  /**
+   * Performs a asynchronous send with receive callback.
+   *
+   *  The first four parameters are the same as for send.
+   *
+   * @p callBackObj and @p callBackSlot specify a call back
+   * slot that is called when an answer is received.
+   *
+   * The slot should have the following signature:
+   * callBackSlot(int callId, const QCString& replyType,
+   *              const QByteArray &replyData);
+   *
+   *
+   * @param remApp the remote application's id
+   * @param remObj the remote object id
+   * @param remFun the remote function id
+   * @param data the data to send
+   * @param callBackObj object to call back
+   * @param callBackSlot slot to call back
+   * @return 0 on failure, on success a callId > 0 is
+   * returned that will be passed as first argument of
+   * the result call back
+   *
+   * @see send()
+   * @see callback()
+   *
+   * @since 3.2
+   */
+  int callAsync(const QCString &remApp, const QCString &remObj,
+                const QCString &remFun, const QByteArray &data,
+                QObject *callBackObj, const char *callBackSlot);
+
   /**
    * Searches for an object which matches a criteria.
    *
@@ -325,10 +370,23 @@ class DCOPClient : public QObject
    * @param foundObj The remote object that matched the criteria.
    * @param useEventLoop if true the event loop will be started when
    *         the call blocks too long
+   * @param timeout timeout for the call in miliseconds, or -1 for no timeout
    * @return true is returned when an object was found for which @p remFun
    *         returned true. If no such object is the function returns false.
+   *
+   * @since 3.2
+   *
    * @see send()
    */
+  bool findObject(const QCString &remApp, const QCString &remObj,
+	    const QCString &remFun, const QByteArray &data,
+	    QCString &foundApp, QCString &foundObj,
+	    bool useEventLoop/*=false*/, int timeout/*=-1*/);
+
+  /**
+   * @deprecated
+   */
+  // KDE4 merge with above
   bool findObject(const QCString &remApp, const QCString &remObj,
 	    const QCString &remFun, const QByteArray &data,
 	    QCString &foundApp, QCString &foundObj,
@@ -370,13 +428,13 @@ class DCOPClient : public QObject
                           const QCString &receiverObj, const QCString &slot,
                           bool Volatile);
 
-  /** 
+  /**
    * @deprecated
-   * For backwards compatibility 
+   * For backwards compatibility
    */
   bool connectDCOPSignal( const QCString &sender, const QCString &signal,
                           const QCString &receiverObj, const QCString &slot,
-                          bool Volatile);
+                          bool Volatile) KDE_DEPRECATED;
 
   /**
    * Disconnects a DCOP signal.
@@ -401,20 +459,20 @@ class DCOPClient : public QObject
 
   /**
    * @deprecated
-   * For backwards compatibility 
+   * For backwards compatibility
    */
   bool disconnectDCOPSignal( const QCString &sender, const QCString &signal,
-                          const QCString &receiverObj, const QCString &slot);
+                          const QCString &receiverObj, const QCString &slot) KDE_DEPRECATED;
 
   /**
    * Reimplement this function to handle app-wide function calls unassociated w/an object.
    *
-   * Note that @p fun is normalized. See @ref normalizeFunctionSignature().
+   * Note that @p fun is normalized. See normalizeFunctionSignature().
    *
    * If you do not want to reimplement this function for whatever reason,
-   * you can also use a default object  or a @ref DCOPObjectProxy.
+   * you can also use a default object  or a DCOPObjectProxy.
    *
-   * @param the normalized function signature
+   * @param fun the normalized function signature
    * @param data the received data
    * @param replyType write the reply type in this string
    * @param replyData write the reply data in this array
@@ -426,18 +484,18 @@ class DCOPClient : public QObject
 
   /**
    * Delays the reply of the current function call
-   * until @ref endTransaction() is called.
+   * until endTransaction() is called.
    *
    * This allows a server to queue requests.
    *
-   * Note: Should be called from inside @ref process() only!
+   * Note: Should be called from inside process() only!
    * @see endTransaction()
    */
   DCOPClientTransaction *beginTransaction( );
 
   /**
    * Sends the delayed reply of a function call.
-   * @param t the transaction as received from @ref beginTransaction()
+   * @param t the transaction as received from beginTransaction()
    * @param replyType write the reply type in this string
    * @param replyData write the reply data in this array
    * @see beginTransaction()
@@ -447,7 +505,7 @@ class DCOPClient : public QObject
   /**
    * Test whether the current function call is delayed.
    *
-   * Note: Should be called from inside @ref process() only!
+   * Note: Should be called from inside process() only!
    * @return The ID of the current transaction or
    *         0 if no transaction is going on.
    * @see process()
@@ -471,7 +529,7 @@ class DCOPClient : public QObject
 
   /**
    * Retrieves the list of objects of the remote application @p remApp.
-   * @param tremAPp he id of the application
+   * @param remApp he id of the application
    * @param ok if not null, the function sets @p ok to true if successful
    *           and false if an error occurred
    * @return the list of object ids
@@ -538,11 +596,11 @@ class DCOPClient : public QObject
    * anymore. The remaining whitespace consists of single blanks only (0x20).
    *
    * Example for a normalized signature:
-   * <pre>
+   * \code
    *   "someFunction(QString,int)"
-   * </pre>
+   * \endcode
    *
-   * When using @ref send() or @ref call(), normalization is done
+   * When using send() or call(), normalization is done
    * automatically for you.
    *
    * @param fun the function signature to normalize
@@ -579,14 +637,14 @@ class DCOPClient : public QObject
   QCString defaultObject() const;
 
   /**
-   * Enables / disables the @ref applicationRegistered() /
-   * @ref applicationRemoved() signals.
+   * Enables / disables the applicationRegistered() /
+   * applicationRemoved() signals.
    * Note that a counter is maintained about how often this method
    * was called. If this method is called twice with @p enabled set to
    * true, notifications will be enabled until it was called with
    * @p enabled set to false as often.
    *
-   * They are disabled by default.  
+   * They are disabled by default.
    * @param enabled true to enable notifications, false to disable
    */
   void setNotifications( bool enabled );
@@ -601,11 +659,18 @@ class DCOPClient : public QObject
    */
   void setDaemonMode( bool daemonMode );
 
+  /**
+   * @internal
+   *
+   * Switch to priority call mode.
+   */
+  void setPriorityCall(bool);
+
   /**
    * Returns the application's main dcop client. The main client can
    * be used by objects that do not have any specific access to a dcop
    * client. In KDE applications, the main client usually is the same
-   * as KAppliction::dcopClient().
+   * as KApplication::dcopClient().
    * @return the application's main dcop client
    */
   static DCOPClient* mainClient();
@@ -614,11 +679,18 @@ class DCOPClient : public QObject
    * Sets the application's main dcop client. The main client can
    * be used by objects that do not have any specific access to a dcop
    * client. In KDE applications, the main client usually is the same
-   * as KAppliction::dcopClient().
+   * as KApplication::dcopClient().
    * @param mainClient the new main dcop client
    */
   static void setMainClient( DCOPClient* mainClient);
 
+  /**
+   * Look for the given client only in this process. This can be used
+   * to check whether a given client (by name) is running in the same
+   * process or in another one.
+   */
+  static DCOPClient* findLocalClient( const QCString &_appId );
+
   /**
     * @internal Do not use.
     */
@@ -630,9 +702,9 @@ class DCOPClient : public QObject
     * Provides information about the last DCOP call for debugging purposes.
     */
   static const char *postMortemSender();
-  /// @internal
+  /** @internal */
   static const char *postMortemObject();
-  /// @internal
+  /** @internal */
   static const char *postMortemFunction();
 
   /**
@@ -648,14 +720,19 @@ class DCOPClient : public QObject
    * @deprecated
    * For backwards compatibility with KDE 2.x
    */
-  static QCString dcopServerFileOld(const QCString &hostname=0);
+  static QCString dcopServerFileOld(const QCString &hostname=0) KDE_DEPRECATED;
+
+  /**
+   * Return the path of iceauth or an empty string if not found.
+   */
+  static QCString iceauthPath();
 
 signals:
   /**
    * Indicates that the application @p appId has been registered with
    * the server we are attached to.
    *
-   * You need to call @ref setNotifications() first, to tell the DCOP server
+   * You need to call setNotifications() first, to tell the DCOP server
    * that you want to get these events.
    * @param appId the id of the new application
    */
@@ -664,7 +741,7 @@ signals:
    * Indicates that the formerly registered application @p appId has
    * been removed.
    *
-   * You need to call @ref setNotifications() first, to tell the
+   * You need to call setNotifications() first, to tell the
    * DCOP server that you want to get these events.
    * @param appId the id of the removed application
    */
@@ -689,7 +766,7 @@ signals:
    * synchronous call. In that case, it will enter a local event
    * loop to keep the GUI updated until finally an answer arrives.
    *
-   * In KDE, the @ref KApplication object connects to this signal to be
+   * In KDE, the KApplication object connects to this signal to be
    * able to block any user input (i.e. mouse and key events) while
    * we are waiting for an answer. If we did not do this, the
    * application might end up in an illegal state, as a keyboard
@@ -699,6 +776,17 @@ signals:
    */
   void blockUserInput( bool block );
 
+  /**
+   * @internal
+   *
+   * Signal used for callbacks of async calls.
+   * This signal is automatically connected to the call back
+   * slot specified in the async call.
+   * @see callAsync()
+   * @since 3.2
+   */
+  void callBack(int, const QCString&, const QByteArray &);
+
 public slots:
 
 protected slots:
@@ -710,17 +798,33 @@ protected slots:
 
 private slots:
   void processPostedMessagesInternal();
+  void asyncReplyReady();
 
-protected:
+public:
+  class ReplyStruct;
+
+  /**
+   * @internal
+   **/
+  void handleAsyncReply(ReplyStruct *replyStruct);
 
 private:
 
-  bool attachInternal( bool registerAsAnonymous = TRUE );
+  bool isLocalTransactionFinished(Q_INT32 id, QCString &replyType, QByteArray &replyData);
+
+  bool attachInternal( bool registerAsAnonymous = true );
 
   bool callInternal(const QCString &remApp, const QCString &remObj,
 	    const QCString &remFun, const QByteArray &data,
 	    QCString& replyType, QByteArray &replyData,
-	    bool useEventLoop, int minor_opcode);
+	    bool useEventLoop, int timeout, int minor_opcode);
+
+
+  bool callInternal(const QCString &remApp, const QCString &remObjId,
+            const QCString &remFun, const QByteArray &data,
+            ReplyStruct *replyStruct,
+            bool useEventLoop, int timeout, int minor_opcode);
+
 protected:
   virtual void virtual_hook( int id, void* data );
 private:
