@@ -17,6 +17,11 @@ class Test2;
 typedef QMap<KIO::Job *, KIODownload> KIODownloadMap;
 class Test3;
 
+/**
+ * This is a stupid comment that might help 
+ * geiseri at some point in the near future.
+ * blah di blah
+ **/
 class Test : public MyNamespace::MyParentClass, virtual public DCOPObject,
              public QValueList<QString>
 {
@@ -28,6 +33,7 @@ public:
     };
     class SomeForwardFooPrivate;
     Test();
+    Test() : DCOPObject("name") {}
     ~Test();
     int getPageOfObj( int obj );
     void setCell( KSpreadTable *table,
@@ -38,12 +44,17 @@ public:
     static const int AUDIO_CD = 1 << 6;
 k_dcop:
     virtual QString url();
+    virtual QString constTest() const;
     virtual DCOPRef firstView();
     virtual DCOPRef nextView();
     virtual int getNumObjects();
     virtual DCOPRef getObject( int num );
     virtual int getNumPages();
     virtual DCOPRef getPage( int num );
+    /** 
+     * a function comment
+     * @see blah di blah also
+     **/
     int getPageOfObj( int obj );
     void setCell( const int& point = 3 );
     QValueList<DCOPRef> getWindows();
@@ -73,7 +84,7 @@ public:
     void yadda();
 };
 
-#endif // end
+#endif // end of line comment
 
 /* Test for line numbers */
 
