@@ -42,12 +42,12 @@ int main( int argc, char** argv )
 {
 
     if ( *qVersion() == '1' ) {
-        fprintf( stderr, "dcopidl2cpp appears to be linked to Qt 1 instead of Qt 2 ! Aborting.\n" );
+        fprintf( stderr, "dcopidl2cpp appears to be linked to Qt 1 instead of Qt >= 2 ! Aborting.\n" );
         exit(1);
     }
     int argpos = 1;
-    bool generate_skel    = TRUE;
-    bool generate_stub    = TRUE;
+    bool generate_skel    = true;
+    bool generate_stub    = true;
 
     QString suffix = "cpp";
 
@@ -55,13 +55,13 @@ int main( int argc, char** argv )
 
 	if ( strcmp( argv[argpos], "--no-skel" ) == 0 )
 	{
-	    generate_skel = FALSE;
+	    generate_skel = false;
 	    for (int i = argpos; i < argc - 1; i++) argv[i] = argv[i+1];
 	    argc--;
 	}
 	else if ( strcmp( argv[argpos], "--no-stub" ) == 0 )
 	{
-	    generate_stub = FALSE;
+	    generate_stub = false;
 	    for (int i = argpos; i < argc - 1; i++) argv[i] = argv[i+1];
 	    argc--;
 	}
