@@ -59,11 +59,10 @@ CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 #include <dcopglobal.h>
 #include "dcop-path.h"
 
-// We don't do tcp in the first place.
-// #define HAVE_KDE_ICETRANSNOLISTEN 1
-
 // #define DCOP_DEBUG
 
+DCOPServer* the_server;
+
 template class QDict<DCOPConnection>;
 template class QPtrDict<DCOPConnection>;
 template class QPtrList<DCOPListener>;
@@ -75,8 +74,6 @@ template class QPtrList<DCOPListener>;
 #define _DCOPIceSendEnd()	\
    fcntl(fd, F_SETFL, fd_fl);
 
-static bool only_local = false;
-
 static QCString findDcopserverShutdown()
 {
    QCString path = getenv("PATH");
@@ -99,12 +96,12 @@ static QCString findDcopserverShutdown()
 
 static Bool HostBasedAuthProc ( char* /*hostname*/)
 {
-    return only_local; // no host based authentication
+    return false; // no host based authentication
 }
 
 extern "C" {
-extern IceWriteHandler _KDE_IceWriteHandler;
-extern IceIOErrorHandler _KDE_IceIOErrorHandler;
+extern IceWriteHandler _kde_IceWriteHandler;
+extern IceIOErrorHandler _kde_IceIOErrorHandler;
 void DCOPIceWriteChar(register IceConn iceConn, unsigned long nbytes, char *ptr);
 }
 
@@ -206,7 +203,7 @@ static unsigned long writeIceData(IceConn iceConn, unsigned long nbytes, char *p
 		}
 	    }
 
-	    (*_KDE_IceIOErrorHandler) (iceConn);
+	    (*_kde_IceIOErrorHandler) (iceConn);
 	    return 0;
 	}
 
@@ -331,7 +328,7 @@ qWarning("DCOPServer: slotOutputReady() %d bytes written", nwritten);
    {
       if ((e == EINTR) || (e == EAGAIN))
          return;
-      (*_KDE_IceIOErrorHandler) (iceConn);
+      (*_kde_IceIOErrorHandler) (iceConn);
       return;
    }
    outputBufferStart += nwritten;
@@ -370,8 +367,6 @@ qWarning("DCOPServer: Flushing data, fd = %d", IceConnectionNumber(_iceConn));
    DCOPIceWrite(_iceConn, _data);
 }
 
-DCOPServer* the_server = 0;
-
 class DCOPListener : public QSocketNotifier
 {
 public:
@@ -412,11 +407,11 @@ DCOPConnection::signalConnectionList()
     return _signalConnectionList;
 }
 
-IceAuthDataEntry *authDataEntries = 0;
-static char *addAuthFile = 0;
+static IceAuthDataEntry *authDataEntries;
+static char *addAuthFile;
 
-static IceListenObj *listenObjs = 0;
-int numTransports = 0;
+static IceListenObj *listenObjs;
+static int numTransports;
 static int ready[2];
 
 
@@ -491,8 +486,7 @@ static char *unique_filename (const char *path, const char *prefix, int *pFd)
 #endif
 }
 
-#define MAGIC_COOKIE_LEN 16
-
+#if 0
 Status SetAuthentication_local (int count, IceListenObj *listenObjs)
 {
     int i;
@@ -522,6 +516,9 @@ Status SetAuthentication_local (int count, IceListenObj *listenObjs)
     }
     return 1;
 }
+#endif
+
+#define MAGIC_COOKIE_LEN 16
 
 Status
 SetAuthentication (int count, IceListenObj *_listenObjs,
@@ -530,8 +527,8 @@ SetAuthentication (int count, IceListenObj *_listenObjs,
     FILE        *addfp = NULL;
     const char  *path;
     int         original_umask;
-    char        command[PATH_MAX + 32];
     int         i;
+    QCString command;    
 #ifdef HAVE_MKSTEMP
     int         fd;
 #endif
@@ -589,7 +586,16 @@ SetAuthentication (int count, IceListenObj *_listenObjs,
 
     umask (original_umask);
 
-    snprintf (command, PATH_MAX + 32, "iceauth source %s", addAuthFile);
+    command = DCOPClient::iceauthPath();
+    
+    if (command.isEmpty())
+    {
+       fprintf( stderr, "dcopserver: 'iceauth' not found in path, aborting.\n" );
+       exit(1);
+    }
+    
+    command += " source ";
+    command += addAuthFile;
     system (command);
 
     unlink(addAuthFile);
@@ -620,9 +626,6 @@ FreeAuthenticationData(int count, IceAuthDataEntry *_authDataEntries)
     /* Each transport has entries for ICE and XSMP */
     int i;
 
-    if (only_local)
-	return;
-
     for (i = 0; i < count * 2; i++) {
 	free (_authDataEntries[i].network_id);
 	free (_authDataEntries[i].auth_data);
@@ -773,7 +776,7 @@ if (opcode == DCOPCall)
 	    } else {
 		QCString replyType;
 		QByteArray replyData;
-		bool b = FALSE;
+		bool b = false;
 		// DCOPServer itself does not do DCOPFind.
 		if ( (opcode == DCOPCall) && (toApp == "DCOPServer") ) {
    		    QCString obj = readQCString(ds);
@@ -862,11 +865,11 @@ if (opcode == DCOPCall)
     }
 }
 
-IcePaVersionRec DCOPServerVersions[] = {
+static const IcePaVersionRec DCOPServerVersions[] = {
     { DCOPVersionMajor, DCOPVersionMinor,  DCOPProcessMessage }
 };
 
-IcePoVersionRec DUMMYVersions[] = {
+static const IcePoVersionRec DUMMYVersions[] = {
     { DCOPVersionMajor, DCOPVersionMinor, 0 }
 };
 
@@ -928,6 +931,7 @@ static Status DCOPServerProtocolSetupProc ( IceConn iceConn,
     return 1;
 }
 
+static int pipeOfDeath[2];
 
 static void sighandler(int sig)
 {
@@ -936,47 +940,34 @@ static void sighandler(int sig)
 	return;
     }
 
-    qApp->quit();
-    //exit(0);
+    write(pipeOfDeath[1], "x", 1);
 }
 
-#ifdef HAVE_KDE_ICETRANSNOLISTEN
-extern "C" int _KDE_IceTransNoListen(const char *protocol);
-#endif
-
-DCOPServer::DCOPServer(bool _only_local, bool _suicide)
+DCOPServer::DCOPServer(bool _suicide)
     : QObject(0,0), currentClientNumber(0), appIds(263), clients(263)
 {
     serverKey = 42;
 
-    only_local = _only_local;
     suicide = _suicide;
 
-#ifdef HAVE_KDE_ICETRANSNOLISTEN
-    if (only_local)
-	_KDE_IceTransNoListen("tcp");
-#else
-    only_local = false;
-#endif
-
     dcopSignals = new DCOPSignals;
 
-    extern int _KDE_IceLastMajorOpcode; // from libICE
-    if (_KDE_IceLastMajorOpcode < 1 )
+    extern int _kde_IceLastMajorOpcode; // from libICE
+    if (_kde_IceLastMajorOpcode < 1 )
         IceRegisterForProtocolSetup(const_cast<char *>("DUMMY"),
 				    const_cast<char *>("DUMMY"),
 				    const_cast<char *>("DUMMY"),
-				    1, DUMMYVersions,
+				    1, const_cast<IcePoVersionRec *>(DUMMYVersions),
 				    DCOPAuthCount, const_cast<char **>(DCOPAuthNames),
 				    DCOPClientAuthProcs, 0);
-    if (_KDE_IceLastMajorOpcode < 1 )
+    if (_kde_IceLastMajorOpcode < 1 )
 	qWarning("DCOPServer Error: incorrect major opcode!");
 
     the_server = this;
     if (( majorOpcode = IceRegisterForProtocolReply (const_cast<char *>("DCOP"),
 						     const_cast<char *>(DCOPVendorString),
 						     const_cast<char *>(DCOPReleaseString),
-						     1, DCOPServerVersions,
+						     1, const_cast<IcePaVersionRec *>(DCOPServerVersions),
 						     1, const_cast<char **>(DCOPAuthNames),
 						     DCOPServerAuthProcs,
 						     HostBasedAuthProc,
@@ -1010,28 +1001,30 @@ DCOPServer::DCOPServer(bool _only_local, bool _suicide)
 	    }
 	    char *idlist = IceComposeNetworkIdList(numTransports, listenObjs);
 	    if (idlist != 0) {
-	        fprintf(f, idlist);
+	        fprintf(f, "%s", idlist);
 		free(idlist);
 	    }
 	    fprintf(f, "\n%i\n", getpid());
 	    fclose(f);
-            // Create a link named like the old-style (KDE 2.x) naming
-            QCString compatName = DCOPClient::dcopServerFileOld();
-            ::symlink(fName,compatName);
+	    if (QCString(getenv("DCOPAUTHORITY")).isEmpty())
+	    {
+                // Create a link named like the old-style (KDE 2.x) naming
+                QCString compatName = DCOPClient::dcopServerFileOld();
+                ::symlink(fName,compatName);
+            }
 	}
 
-    if (only_local) {
+#if 0
 	if (!SetAuthentication_local(numTransports, listenObjs))
 	    qFatal("DCOPSERVER: authentication setup failed.");
-    } else {
-	if (!SetAuthentication(numTransports, listenObjs, &authDataEntries))
-	    qFatal("DCOPSERVER: authentication setup failed.");
-    }
+#endif
+    if (!SetAuthentication(numTransports, listenObjs, &authDataEntries))
+        qFatal("DCOPSERVER: authentication setup failed.");
 
     IceAddConnectionWatch (DCOPWatchProc, static_cast<IcePointer>(this));
     _IceWriteHandler = DCOPIceWriteChar;
 
-    listener.setAutoDelete( TRUE );
+    listener.setAutoDelete( true );
     DCOPListener* con;
     for ( int i = 0; i < numTransports; i++) {
 	con = new DCOPListener( listenObjs[i] );
@@ -1243,6 +1236,7 @@ void DCOPServer::slotTerminate()
     dcopSignals->emitSignal(0L /* dcopserver */, "terminateKDE()", data, false);
     disconnect( m_timer, SIGNAL(timeout()), this, SLOT(slotTerminate()) );
     connect( m_timer, SIGNAL(timeout()), this, SLOT(slotSuicide()) );
+    system(findDcopserverShutdown()+" --nokill");
 }
 
 void DCOPServer::slotSuicide()
@@ -1368,7 +1362,7 @@ bool DCOPServer::receive(const QCString &/*app*/, const QCString &obj,
 	    }
 	    replyType = "QCString";
 	    reply << conn->appId;
-	    return TRUE;
+	    return true;
 	}
     }
     else if ( fun == "registeredApplications()" ) {
@@ -1381,7 +1375,7 @@ bool DCOPServer::receive(const QCString &/*app*/, const QCString &obj,
 	}
 	replyType = "QCStringList";
 	reply << applications;
-	return TRUE;
+	return true;
     } else if ( fun == "isApplicationRegistered(QCString)" ) {
 	QDataStream args( data, IO_ReadOnly );
 	if (!args.atEnd()) {
@@ -1390,7 +1384,7 @@ bool DCOPServer::receive(const QCString &/*app*/, const QCString &obj,
 	    int b = ( findApp( s ) != 0 );
 	    replyType = "bool";
 	    reply << b;
-	    return TRUE;
+	    return true;
 	}
     } else if ( fun == "setNotifications(bool)" ) {
 	QDataStream args( data, IO_ReadOnly );
@@ -1405,7 +1399,7 @@ bool DCOPServer::receive(const QCString &/*app*/, const QCString &obj,
 		    conn->notifyRegister--;
 	    }
 	    replyType = "void";
-	    return TRUE;
+	    return true;
 	}
     } else if ( fun == "connectSignal(QCString,QCString,QCString,QCString,QCString,bool)") {
         DCOPConnection* conn = clients.find( iceConn );
@@ -1424,7 +1418,7 @@ bool DCOPServer::receive(const QCString &/*app*/, const QCString &obj,
         replyType = "bool";
         QDataStream reply( replyData, IO_WriteOnly );
         reply << (Q_INT8) (b?1:0);
-        return TRUE;
+        return true;
     } else if ( fun == "disconnectSignal(QCString,QCString,QCString,QCString,QCString)") {
         DCOPConnection* conn = clients.find( iceConn );
         if (!conn) return false;
@@ -1440,10 +1434,10 @@ bool DCOPServer::receive(const QCString &/*app*/, const QCString &obj,
         replyType = "bool";
         QDataStream reply( replyData, IO_WriteOnly );
         reply << (Q_INT8) (b?1:0);
-        return TRUE;
+        return true;
     }
 
-    return FALSE;
+    return false;
 }
 
 void DCOPServer::broadcastApplicationRegistration( DCOPConnection* conn, const QCString type,
@@ -1538,7 +1532,7 @@ static bool isRunning(const QCString &fName, bool printNetworkId = false)
 }
 
 const char* const ABOUT =
-"Usage: dcopserver [--nofork] [--nosid] [--nolocal] [--help]\n"
+"Usage: dcopserver [--nofork] [--nosid] [--help]\n"
 "       dcopserver --serverid\n"
 "\n"
 "DCOP is KDE's Desktop Communications Protocol. It is a lightweight IPC/RPC\n"
@@ -1548,12 +1542,11 @@ const char* const ABOUT =
 "Copyright (C) 1999-2001, The KDE Developers <http://www.kde.org>\n"
 ;
 
-int main( int argc, char* argv[] )
+extern "C" int kdemain( int argc, char* argv[] )
 {
     bool serverid = false;
     bool nofork = false;
     bool nosid = false;
-    bool nolocal = false;
     bool suicide = false;
     for(int i = 1; i < argc; i++) {
 	if (strcmp(argv[i], "--nofork") == 0)
@@ -1561,7 +1554,7 @@ int main( int argc, char* argv[] )
 	else if (strcmp(argv[i], "--nosid") == 0)
 	    nosid = true;
 	else if (strcmp(argv[i], "--nolocal") == 0)
-	    nolocal = true;
+	    ; // Ignore
 	else if (strcmp(argv[i], "--suicide") == 0)
 	    suicide = true;
 	else if (strcmp(argv[i], "--serverid") == 0)
@@ -1582,7 +1575,8 @@ int main( int argc, char* argv[] )
     // check if we are already running
     if (isRunning(DCOPClient::dcopServerFile()))
        return 0;
-    if (isRunning(DCOPClient::dcopServerFileOld()))
+    if (QCString(getenv("DCOPAUTHORITY")).isEmpty() &&
+        isRunning(DCOPClient::dcopServerFileOld()))
     {
        // Make symlink for compatibility
        QCString oldFile = DCOPClient::dcopServerFileOld();
@@ -1639,6 +1633,8 @@ int main( int argc, char* argv[] )
 	    return 0; // get rid of controlling terminal
     }
 
+    pipe(pipeOfDeath);
+
     signal(SIGHUP, sighandler);
     signal(SIGTERM, sighandler);
     signal(SIGPIPE, SIG_IGN);
@@ -1646,9 +1642,12 @@ int main( int argc, char* argv[] )
     putenv(strdup("SESSION_MANAGER="));
 
     QApplication a( argc, argv, false );
-
+    
+    QSocketNotifier DEATH(pipeOfDeath[0], QSocketNotifier::Read, 0, 0);
+    a.connect(&DEATH, SIGNAL(activated(int)), SLOT(quit()));
+    
     IceSetIOErrorHandler (IoErrorHandler );
-    DCOPServer *server = new DCOPServer(!nolocal, suicide); // this sets the_server
+    DCOPServer *server = new DCOPServer(suicide); // this sets the_server
 
     int ret = a.exec();
     delete server;
