@@ -20,7 +20,7 @@ AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */
 #ifndef DCOPSERVER_H
-#define DCOPSERVER_H "$Id: dcopserver.h,v 1.33 2002/01/04 11:03:18 lunakl Exp $"
+#define DCOPSERVER_H "$Id: dcopserver.h,v 1.35 2003/03/19 16:26:40 waba Exp $"
 
 #include <qobject.h>
 #include <qstring.h>
@@ -106,7 +106,7 @@ class DCOPServer : public QObject
 {
     Q_OBJECT
 public:
-    DCOPServer(bool _only_local, bool _suicide);
+    DCOPServer(bool _suicide);
     ~DCOPServer();
 
     void* watchConnection( IceConn iceConn );
@@ -152,5 +152,4 @@ private:
 };
 
 extern DCOPServer* the_server;
-
 #endif
