@@ -20,7 +20,7 @@ CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */
 
 #ifndef DCOPSIGNALS_H
-#define DCOPSIGNALS_H "$Id: dcopsignals.h,v 1.5 2002/09/29 13:05:01 tjansen Exp $"
+#define DCOPSIGNALS_H "$Id: dcopsignals.h,v 1.7 2003/08/15 20:49:19 coolo Exp $"
 
 #include <qdict.h>
 #include <qptrlist.h>
@@ -28,10 +28,10 @@ CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 
 class DCOPConnection;
 
-//
-// This requires:
-// DCOPSignalConnectionList * DCOPConnection::signalConnectionList()
-/// @internal
+/** \internal
+  * This requires:
+  * DCOPSignalConnectionList * DCOPConnection::signalConnectionList()
+*/
 class DCOPSignalConnection
 {
 public:
@@ -45,7 +45,7 @@ public:
    QCString slot;        // Function to send to in the object.
 };
 
-/// @internal
+/** \internal */
 class DCOPSignalConnectionList : public QPtrList<DCOPSignalConnection>
 {
 public:
@@ -96,7 +96,7 @@ public:
     * If "receiverObj" is empty, every object is disconnected.
     * If "slot" is empty, every slot is disconnected.
     *
-    * Returns true if successfull, false if no connection was found.
+    * Returns true if successful, false if no connection was found.
     */
    bool disconnectSignal( const QCString &sender, const QCString &senderObj,
                        const QCString &signal,
