@@ -19,13 +19,14 @@
 #ifndef __ktexteditor_encodinginterface_h__
 #define __ktexteditor_encodinginterface_h__
 
+class QString;
 class QCString;
 
 namespace KTextEditor
 {
 
-/*
-*  This is an interface for the KTextEditor::Document class !!!
+/**
+*  This is an interface to the text encoding of a Document class.
 */
 class EncodingInterface
 {
