@@ -10,7 +10,7 @@
          This code may be linked against any version of the Qt toolkit
          from Troll Tech, Norway. $
 
-   $Id: qconfigDB.cc,v 1.27 2001/11/03 00:23:56 matz Exp $	 
+   $Id: qconfigDB.cc,v 1.29 2003/09/10 00:12:12 wheeler Exp $	 
 */
 
 #include "qconfigDB.h"
@@ -40,44 +40,7 @@ extern "C" {
 
 #define KAB_KDEBUG_AREA 800
 
-// the global functions:
-void evaluate_assertion(bool cond, const char* file, int line, const char* text)
-{
-  if(!cond)
-    {
-      kdDebug() << file << ":" << line << ": " << text << endl;
-    }
-}
-
-// QCString AuthorEmailAddress; // assign your email address to this string
-
-QCString ReadLineFromStream(QTextStream& stream)
-{
-  register bool GUARD; GUARD=false;
-  // ############################################################################
-  kdDebug(GUARD, KAB_KDEBUG_AREA) <<  "ReadLineFromStream:: reading line." << endl;
-  QCString line;
-  // -----
-  while(!stream.eof())
-    {
-      line=stream.readLine().ascii();
-      if(!line.isEmpty())
-	{
-	  if(isComment(line))
-	    {
-	      line="";
-	      continue;
-	    }
-	}
-      break;
-    }
-  // -----
-  kdDebug(GUARD, KAB_KDEBUG_AREA) <<  "ReadLineFromStream:: line \"" << line.data() << "\" read.\n";
-  return line;
-  // ############################################################################
-}
-
-bool isComment(QCString line)
+static bool isComment(QCString line)
 {
   // ############################################################################
   line=line.stripWhiteSpace();
@@ -90,35 +53,7 @@ bool isComment(QCString line)
   // ############################################################################
 }
 
-bool htmlizeString(const QCString& orig, QCString& target)
-{
-  register bool GUARD; GUARD=false;
-  kdDebug(GUARD, KAB_KDEBUG_AREA) <<  "htmlizeString: called." << endl;
-  // ############################################################################
-  QCString temp;
-  char current;
-  unsigned int index;
-  // -----
-  for(index=0; index<orig.length(); index++)
-    { // ----- the following code is character set depending
-      //       and will possibly need recompilation for EVERY
-      //       system using another charset than the machine
-      //       it was compiled on! Lets see.
-      current=orig[index];
-      switch(current)
-	{
-	case '':
-	default: temp+=current;
-	}
-    }
-  target=temp;
-  // -----
-  kdDebug(GUARD, KAB_KDEBUG_AREA) <<  "htmlizeString: done." << endl;
-  return false; // not implemented
-  // ############################################################################
-}
-
-void tokenize(list<QCString>& res, const QCString& text, char tr, bool strict)
+static void tokenize(list<QCString>& res, const QCString& text, char tr, bool strict=false)
 {
   register bool GUARD; GUARD=false;
   // ############################################################################
@@ -160,6 +95,34 @@ void tokenize(list<QCString>& res, const QCString& text, char tr, bool strict)
   // ############################################################################
 }
 
+// QCString AuthorEmailAddress; // assign your email address to this string
+
+static QCString ReadLineFromStream(QTextStream& stream)
+{
+  register bool GUARD; GUARD=false;
+  // ############################################################################
+  kdDebug(GUARD, KAB_KDEBUG_AREA) <<  "ReadLineFromStream:: reading line." << endl;
+  QCString line;
+  // -----
+  while(!stream.eof())
+    {
+      line=stream.readLine().ascii();
+      if(!line.isEmpty())
+	{
+	  if(isComment(line))
+	    {
+	      line="";
+	      continue;
+	    }
+	}
+      break;
+    }
+  // -----
+  kdDebug(GUARD, KAB_KDEBUG_AREA) <<  "ReadLineFromStream:: line \"" << line.data() << "\" read.\n";
+  return line;
+  // ############################################################################
+}
+
 // class implementations:
 
 list<QString> QConfigDB::LockFiles; // the lockfiles created by this session
@@ -289,7 +252,7 @@ KeyValueMap::save(const QString& filename, bool force)
     {
       QTextStream stream(&file);
       stream.setEncoding(QTextStream::Latin1); // no conversion
-      stream << "# saved by KeyValueMap object ($Revision: 1.27 $)" << endl;
+      stream << "# saved by KeyValueMap object ($Revision: 1.29 $)" << endl;
       for(pos=data->begin(); pos!=data->end(); ++pos)
 	{ // values do not get coded here
 	  stream << (*pos).first << '=' << (*pos).second << endl;
@@ -762,14 +725,14 @@ KeyValueMap::get(const QCString& key, bool& value) const
   if(v=="true")
     {
 	kdDebug(GUARD, KAB_KDEBUG_AREA) <<  "KeyValueMap::get[bool]: success, "
-	    "value is TRUE." << endl;
+	    "value is true." << endl;
       value=true;
       return true;
     }
   if(v=="false")
     {
       kdDebug(GUARD, KAB_KDEBUG_AREA) <<  "KeyValueMap::get[bool]: success, "
-      "value is FALSE." << endl;
+      "value is false." << endl;
       value=false;
       return true;
     }
