@@ -139,7 +139,7 @@ class DistributionListManager
 
     /**
       Insert distribution list. If a list with this name already exists, nothing
-      happens.
+      happens. The passed object is deleted by the manager.
     */
     void insert( DistributionList * );
 
@@ -172,29 +172,28 @@ class DistributionListManager
 
 /**
   @short Watchdog for distribution lists
- 
-  This class provides a @ref changed() signal that i emitted when the
+
+  This class provides a changed() signal that i emitted when the
   distribution lists has changed in some way.
 
   Exapmle:
-  
-  <pre>
+
+  \code
   KABC::DistributionListWatcher *watchdog = KABC::DistributionListWatcher::self()
 
   connect( watchdog, SIGNAL( changed() ), SLOT( doSomething() ) );
-  </pre>
+  \endcode
 */
 
 class DistributionListWatcher : public QObject
 {
   Q_OBJECT
-  
+
   public:
     /**
      * Returns the watcher object.
      */
     static DistributionListWatcher *self();
-    
 
   signals:
     /**
