@@ -22,10 +22,10 @@
 #include <qlayout.h>
 #include <qpushbutton.h>
 #include <qcombobox.h>
-#include <qinputdialog.h>
 #include <qbuttongroup.h>
 #include <qradiobutton.h>
 
+#include <kinputdialog.h>
 #include <klocale.h>
 #include <kdebug.h>
 
@@ -140,7 +140,7 @@ DistributionListEditor::DistributionListEditor( AddressBook *addressBook, QWidge
   mEntryView = new QListView( this );
   mEntryView->addColumn( i18n("Name") );
   mEntryView->addColumn( i18n("Email") );
-  mEntryView->addColumn( i18n("Use preferred") );
+  mEntryView->addColumn( i18n("Use Preferred") );
   topLayout->addWidget( mEntryView );
   connect(mEntryView,SIGNAL(selectionChanged ()),this, SLOT(slotSelectionEntryViewChanged()));
 
@@ -182,8 +182,8 @@ DistributionListEditor::~DistributionListEditor()
 
 void DistributionListEditor::slotSelectionEntryViewChanged()
 {
-    EditEntryItem *entryItem =dynamic_cast<EditEntryItem *>( mEntryView->selectedItem() );
-    bool state=entryItem;
+    EditEntryItem *entryItem = dynamic_cast<EditEntryItem *>( mEntryView->selectedItem() );
+    bool state = (entryItem != 0L);
 
     changeEmailButton->setEnabled(state);
     removeEntryButton->setEnabled(state);
@@ -192,11 +192,11 @@ void DistributionListEditor::slotSelectionEntryViewChanged()
 void DistributionListEditor::newList()
 {
   bool ok = false;
-  QString name = QInputDialog::getText( i18n("New Distribution List"),
-                                        i18n("Please enter name."),
-                                        QLineEdit::Normal, QString::null, &ok,
-                                        this );
-  if ( !ok || name.isEmpty() ) return;
+  QString name = KInputDialog::getText( i18n("New Distribution List"),
+                                        i18n("Please enter name:"),
+                                        QString::null, &ok, this );
+  if ( !ok )
+    return;
 
   new DistributionList( mManager, name );
 
@@ -207,7 +207,7 @@ void DistributionListEditor::newList()
 
 void DistributionListEditor::removeList()
 {
-  delete mManager->list( mNameCombo->currentText() );
+  mManager->remove( mManager->list( mNameCombo->currentText() ) );
   mNameCombo->removeItem( mNameCombo->currentItem() );
   removeButton->setEnabled(!mManager->listNames().isEmpty());
   addEntryButton->setEnabled( !mNameCombo->currentText().isEmpty());
@@ -276,8 +276,8 @@ void DistributionListEditor::updateEntryView()
   for( it = entries.begin(); it != entries.end(); ++it ) {
     new EditEntryItem( mEntryView, (*it).addressee, (*it).email );
   }
-   EditEntryItem *entryItem =dynamic_cast<EditEntryItem *>( mEntryView->selectedItem() );
-   bool state=entryItem;
+   EditEntryItem *entryItem = dynamic_cast<EditEntryItem *>( mEntryView->selectedItem() );
+   bool state = (entryItem != 0L);
 
    changeEmailButton->setEnabled(state);
    removeEntryButton->setEnabled(state);
@@ -304,6 +304,6 @@ void DistributionListEditor::slotSelectionAddresseeViewChanged()
 {
     AddresseeItem *addresseeItem =
         dynamic_cast<AddresseeItem *>( mAddresseeView->selectedItem() );
-    bool state=addresseeItem;
+    bool state = (addresseeItem != 0L);
     addEntryButton->setEnabled( state && !mNameCombo->currentText().isEmpty());
 }
