@@ -38,10 +38,10 @@ using namespace KABC;
 
 static const KCmdLineOptions options[] =
 {
-  {"disable-autostart", I18N_NOOP("Disable automatic startup on login."), 0},
+  {"disable-autostart", I18N_NOOP("Disable automatic startup on login"), 0},
   {"o", 0, 0},
-  {"override", I18N_NOOP("Override existing entries."),"1"},
-  {0,0,0}
+  {"override", I18N_NOOP("Override existing entries"),"1"},
+  KCmdLineLastOption
 };
 
 void readKMailEntry( const QString &kmailEntry, KABC::AddressBook *ab )
@@ -287,7 +287,9 @@ void readKAddressBookEntries( const QString &dataString, Addressee &a )
 
 void importKab( KABC::AddressBook *ab, bool override )
 {
-  if (!QFile::exists(locateLocal("data", "kab/addressbook.kab") )) {
+  QString fileName = KGlobal::dirs()->saveLocation( "data", "kab/" );
+  fileName += "addressbook.kab";
+  if (!QFile::exists( fileName )) {
     kdDebug() << "No KDE 2 addressbook found." << endl;
     return;
   }
@@ -363,7 +365,7 @@ void importKab( KABC::AddressBook *ab, bool override )
     QStringList::ConstIterator phoneIt;
     for( phoneIt = entry.telephone.begin(); phoneIt != entry.telephone.end(); ++phoneIt ) {
       int kabType = (*phoneIt++).toInt();
-      if ( phoneIt == entry.telephone.end() ) break;
+      if( phoneIt == entry.telephone.end() ) break;
       QString number = *phoneIt;
       int type = 0;
       if ( kabType == ::AddressBook::Fixed ) type = PhoneNumber::Voice;
