@@ -20,22 +20,31 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
  */
 
-#include <kmdcodec.h>
-#include <qpixmap.h>
+
+
+#include <qfile.h>
 #include <qimage.h>
 #include <qlabel.h>
-#include <qfile.h>
+#include <qpixmap.h>
+#include <qtextstream.h>
+#include <qurl.h>
 
-#include <kprotocolinfo.h>
-#include <kconfig.h>
 #include <kapplication.h>
-
+#include <kconfig.h>
 #include <kdebug.h>
+#include <kmdcodec.h>
+#include <kprotocolinfo.h>
 
 #include "ldapclient.h"
 
 using namespace KABC;
 
+class LdapClient::LdapClientPrivate{
+public:
+  QString bindDN;
+  QString pwdBindDN;
+};
+
 QString LdapObject::toString() const
 {
   QString result = QString::fromLatin1( "\ndn: %1\n" ).arg( dn );
@@ -58,12 +67,12 @@ QString LdapObject::toString() const
         //l->show();
 #endif
       } else {
-        result += QString("%1: %2\n").arg( attr).arg( *it2 );
+        result += QString("%1: %2\n").arg(attr).arg(QString::fromUtf8(*it2));
       }
     }
   }
 
-  return result;  
+  return result;
 }
 
 void LdapObject::clear()
@@ -77,17 +86,19 @@ void LdapObject::assign( const LdapObject& that )
   if ( &that != this ) {
     dn = that.dn;
     attrs = that.attrs;
-  }    
+  }
 }
 
 LdapClient::LdapClient( QObject* parent, const char* name )
   : QObject( parent, name ), mJob( 0 ), mActive( false )
 {
+  d = new LdapClientPrivate;
 }
 
 LdapClient::~LdapClient()
 {
   cancelQuery();
+  delete d; d = 0;
 }
 
 void LdapClient::setHost( const QString& host )
@@ -105,6 +116,16 @@ void LdapClient::setBase( const QString& base )
   mBase = base;
 }
 
+void LdapClient::setBindDN( const QString& bindDN )
+{
+  d->bindDN = bindDN;
+}
+
+void LdapClient::setPwdBindDN( const QString& pwdBindDN )
+{
+  d->pwdBindDN = pwdBindDN;
+}
+
 void LdapClient::setAttrs( const QStringList& attrs )
 {
   mAttrs = attrs;
@@ -117,6 +138,19 @@ void LdapClient::startQuery( const QString& filter )
   if ( mScope.isEmpty() )
     mScope = "sub";
 
+  QString auth;
+  QString encodedPassword;
+  if ( !d->bindDN.isEmpty() ) {
+    auth = d->bindDN;
+    QUrl::encode( auth );
+    if ( !d->pwdBindDN.isEmpty() ) {
+      encodedPassword = d->pwdBindDN;
+      QUrl::encode( encodedPassword );
+      auth += ":" + encodedPassword;
+    }
+    auth += "@";
+  }
+
   QString host = mHost;
   if ( !mPort.isEmpty() ) {
     host += ':';
@@ -124,12 +158,11 @@ void LdapClient::startQuery( const QString& filter )
   }
 
   if ( mAttrs.empty() ) {
-    query = QString("ldap://%1/%2?*?%3?(%4)").arg( host ).arg( mBase ).arg( mScope ).arg( filter );
+    QTextOStream(&query) << "ldap://" << auth << host << "/" << mBase << "?*?" << mScope << "?(" << filter << ")";
   } else {
-    query = QString("ldap://%1/%2?%3?%4?(%5)").arg( host ).arg( mBase )
-      .arg( mAttrs.join(",") ).arg( mScope ).arg( filter );
+    QTextOStream(&query) << "ldap://" << auth << host << "/" << mBase << "?" << mAttrs.join(",") << "?" << mScope << "?(" << filter << ")";
   }
-  kdDebug(5700) << "Doing query" << query.latin1() << endl;
+  kdDebug(5700) << "Doing query " << query << endl;
 
   startParseLDIF();
   mActive = true;
@@ -178,7 +211,7 @@ void LdapClient::slotDone()
   int err = mJob->error();
   if ( err ) {
     emit error( KIO::buildErrorString( err, QString("%1:%2").arg( mHost ).arg( mPort ) ) );
-  } 
+  }
   emit done();
 }
 
@@ -208,9 +241,9 @@ void LdapClient::endParseLDIF()
 }
 
 void LdapClient::parseLDIF( const QByteArray& data )
-{  
-  //qDebug("%s", data.data());
-  if ( data.isNull() || data.isEmpty() )
+{
+  // qDebug("%s: %s", __FUNCTION__, data.data());
+  if ( data.isEmpty() )
     return;
   mBuf += QCString( data, data.size() + 1 ); // collect data in buffer
   int nl;
@@ -238,7 +271,7 @@ void LdapClient::parseLDIF( const QByteArray& data )
       continue;
 
     int colon = line.find(':');
-    if ( colon != -1 ) { // Found new attribute	
+    if ( colon != -1 ) { // Found new attribute
       if ( mLastAttrName == "dn" ) { // New object, store the current
         if ( !mCurrentObject.dn.isNull() ) {
           emit result( mCurrentObject );
@@ -276,6 +309,16 @@ void LdapClient::parseLDIF( const QByteArray& data )
   }
 }
 
+QString LdapClient::bindDN() const
+{
+  return d->bindDN;
+}
+
+QString LdapClient::pwdBindDN() const
+{
+  return d->pwdBindDN;
+}
+
 LdapSearch::LdapSearch()
     : mActiveClients( 0 ), mNoLDAPLookup( false )
 {
@@ -285,18 +328,18 @@ LdapSearch::LdapSearch()
   }
 
   // stolen from KAddressBook
-  KConfig config( "kaddressbookrc", true );
+  KConfig config( "kabldaprc", true );
   config.setGroup( "LDAP" );
-  int numHosts = config.readUnsignedNumEntry( "NumSelectedHosts"); 
+  int numHosts = config.readUnsignedNumEntry( "NumSelectedHosts");
   if ( !numHosts ) {
     mNoLDAPLookup = true;
     return;
   } else {
     for ( int j = 0; j < numHosts; j++ ) {
       LdapClient* ldapClient = new LdapClient( this );
-    
+
       QString host =  config.readEntry( QString( "SelectedHost%1" ).arg( j ), "" ).stripWhiteSpace();
-      if ( host != "" )
+      if ( !host.isEmpty() )
         ldapClient->setHost( host );
 
       QString port = QString::number( config.readUnsignedNumEntry( QString( "SelectedPort%1" ).arg( j ) ) );
@@ -304,9 +347,17 @@ LdapSearch::LdapSearch()
         ldapClient->setPort( port );
 
       QString base = config.readEntry( QString( "SelectedBase%1" ).arg( j ), "" ).stripWhiteSpace();
-      if ( base != "" )
+      if ( !base.isEmpty() )
         ldapClient->setBase( base );
 
+      QString bindDN = config.readEntry( QString( "SelectedBind%1" ).arg( j ) ).stripWhiteSpace();
+      if ( !bindDN.isEmpty() )
+        ldapClient->setBindDN( bindDN );
+
+      QString pwdBindDN = config.readEntry( QString( "SelectedPwdBind%1" ).arg( j ) ).stripWhiteSpace();
+      if ( !pwdBindDN.isEmpty() )
+        ldapClient->setPwdBindDN( pwdBindDN );
+
       QStringList attrs;
       attrs << "cn" << "mail" << "givenname" << "sn";
       ldapClient->setAttrs( attrs );
@@ -314,14 +365,14 @@ LdapSearch::LdapSearch()
       connect( ldapClient, SIGNAL( result( const KABC::LdapObject& ) ),
                this, SLOT( slotLDAPResult( const KABC::LdapObject& ) ) );
       connect( ldapClient, SIGNAL( done() ),
-               this, SLOT( slotLDAPDone() ) ); 
+               this, SLOT( slotLDAPDone() ) );
       connect( ldapClient, SIGNAL( error( const QString& ) ),
                this, SLOT( slotLDAPError( const QString& ) ) );
 
-      mClients.append( ldapClient );     
+      mClients.append( ldapClient );
     }
   }
-  
+
   connect( &mDataTimer, SIGNAL( timeout() ), SLOT( slotDataTimer() ) );
 }
 
@@ -331,6 +382,7 @@ void LdapSearch::startSearch( const QString& txt )
     return;
 
   cancelSearch();
+
   int pos = txt.find( '\"' );
   if( pos >= 0 )
   {
@@ -407,14 +459,15 @@ QStringList LdapSearch::makeSearchData()
 
     LdapAttrMap::ConstIterator it2;
     for ( it2 = (*it1).attrs.begin(); it2 != (*it1).attrs.end(); ++it2 ) {
+      QString tmp = QString::fromUtf8((*it2).first());
       if ( it2.key() == "cn" )
-        name = QString( "%1" ).arg( (*it2).first() ); // TODO loop?
-      else if( it2.key() == "mail" ) 
-        mail = QString( "%1" ).arg( (*it2).first() );
-	    else if( it2.key() == "givenName" )
-        givenname = QString( "%1" ).arg( (*it2).first() );
-	    else if( it2.key() == "sn" )
-        sn = QString( "%1" ).arg( (*it2).first() );
+        name = tmp; // TODO loop?
+      else if( it2.key() == "mail" )
+        mail = tmp;
+      else if( it2.key() == "givenName" )
+        givenname = tmp;
+      else if( it2.key() == "sn" )
+        sn = tmp;
     }
 
     if( mail.isEmpty())
@@ -422,13 +475,15 @@ QStringList LdapSearch::makeSearchData()
     else if ( name.isEmpty() )
       ret.append( mail );
     else {
-        kdDebug() << "<" << name << "><" << mail << ">" << endl;
+        kdDebug(5700) << "<" << name << "><" << mail << ">" << endl;
       ret.append( QString( "%1 <%2>" ).arg( name ).arg( mail ) );
+#if 0
       // this sucks
       if ( givenname.upper().startsWith( search_text_upper ) )
         ret.append( QString( "$$%1$%2 <%3>" ).arg( givenname ).arg( name ).arg( mail ) );
       if ( sn.upper().startsWith( search_text_upper ) )
         ret.append( QString( "$$%1$%2 <%3>" ).arg( sn ).arg( name ).arg( mail ) );
+#endif
     }
   }
 
@@ -442,4 +497,6 @@ bool LdapSearch::isAvailable() const
   return !mNoLDAPLookup;
 }
 
+
+
 #include "ldapclient.moc"
