@@ -43,7 +43,7 @@ PhoneNumber::~PhoneNumber()
 
 void PhoneNumber::init()
 {
-  mId = KApplication::randomString(8);
+  mId = KApplication::randomString( 8 );
 }
 
 bool PhoneNumber::operator==( const PhoneNumber &p ) const
@@ -130,6 +130,9 @@ QString PhoneNumber::label( int type )
 
 QString PhoneNumber::typeLabel( int type )
 {
+  if ( type & Pref )
+    return i18n( "Preferred phone", "Preferred" );
+
   switch ( type ) {
     case Home:
       return i18n("Home phone", "Home");
