@@ -1,6 +1,7 @@
 /*
     This file is part of libkabc.
     Copyright (c) 2001 Cornelius Schumacher <schumacher@kde.org>
+    Copyright (c) 2003 Carsten Pfeiffer <pfeiffer@kde.org>
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Library General Public
@@ -17,20 +18,28 @@
     the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
     Boston, MA 02111-1307, USA.
 */
+
+#include <qregexp.h>
+
 #include <ksharedptr.h>
 #include <kdebug.h>
 #include <kapplication.h>
 #include <klocale.h>
 
+#include "addresseehelper.h"
+#include "field.h"
 #include "resource.h"
+
 #include "addressee.h"
 
 using namespace KABC;
 
-bool matchBinaryPattern( int value, int pattern, int max );
+static bool matchBinaryPattern( int value, int pattern );
+KABC::Field *Addressee::mSortField = 0;
 
 struct Addressee::AddresseeData : public KShared
 {
+  QString uid;
   --VARIABLES--
 
   PhoneNumber::List phoneNumbers;
@@ -42,17 +51,25 @@ struct Addressee::AddresseeData : public KShared
 
   Resource *resource;
 
-  bool empty;
-  bool changed;
+  bool empty    :1;
+  bool changed  :1;
 };
 
+Addressee::AddresseeData* Addressee::shared_null = 0;
+
+Addressee::AddresseeData* Addressee::makeSharedNull()
+{
+  Addressee::shared_null = new AddresseeData;
+  shared_null->_KShared_ref(); //just in case (we should add KSD)
+  shared_null->empty = true;
+  shared_null->changed = false;
+  shared_null->resource = 0;
+  return shared_null;
+}
+
 Addressee::Addressee()
 {
-  mData = new AddresseeData;
-  mData->uid = KApplication::randomString( 10 );
-  mData->empty = true;
-  mData->changed = false;
-  mData->resource = 0;
+  mData = shared_null ? shared_null : makeSharedNull();
 }
 
 Addressee::~Addressee()
@@ -70,21 +87,23 @@ Addressee &Addressee::operator=( const Addressee &a )
   return (*this);
 }
 
-Addressee Addressee::copy()
-{
-  Addressee a;
-  *(a.mData) = *mData;
-  return a;
-}
-
 void Addressee::detach()
 {
-  if ( mData.count() == 1 ) return;
-  *this = copy();
+  if ( mData.data() == shared_null ) {
+    mData = new AddresseeData;
+    mData->empty = true;
+    mData->changed = false;
+    mData->resource = 0;
+    return;
+  } else if ( mData.count() == 1 ) return;
+
+  AddresseeData data = *mData;
+  mData = new AddresseeData( data );
 }
 
 bool Addressee::operator==( const Addressee &a ) const
 {
+  if ( uid() != a.uid() ) return false;
   --EQUALSTEST--
   if ( ( mData->url.isValid() || a.mData->url.isValid() ) &&
        ( mData->url != a.mData->url ) ) return false;
@@ -108,6 +127,27 @@ bool Addressee::isEmpty() const
   return mData->empty;
 }
 
+void Addressee::setUid( const QString &id )
+{
+  if ( id == mData->uid ) return;
+  detach();
+  mData->empty = false;
+  mData->uid = id;
+}
+
+QString Addressee::uid() const
+{
+  if ( mData->uid.isEmpty() )
+    mData->uid = KApplication::randomString( 10 );
+
+  return mData->uid;
+}
+
+QString Addressee::uidLabel()
+{
+  return i18n("Unique Identifier");
+}
+
 --DEFINITIONS--
 
 void Addressee::setNameFromString( const QString &str )
@@ -115,46 +155,30 @@ void Addressee::setNameFromString( const QString &str )
   setFormattedName( str );
   setName( str );
 
-  QStringList titles;
-  titles += i18n( "Dr." );
-  titles += i18n( "Miss" );
-  titles += i18n( "Mr." );
-  titles += i18n( "Mrs." );
-  titles += i18n( "Ms." );
-  titles += i18n( "Prof." );
-
-  QStringList suffixes;
-  suffixes += i18n( "I" );
-  suffixes += i18n( "II" );
-  suffixes += i18n( "III" );
-  suffixes += i18n( "Jr." );
-  suffixes += i18n( "Sr." );
-
-  QStringList prefixes;
-  prefixes += "van";
-  prefixes += "von";
-  prefixes += "de";
-
   // clear all name parts
-  setPrefix( "" );
-  setGivenName( "" );
-  setAdditionalName( "" );
-  setFamilyName( "" );
-  setSuffix( "" );
+  setPrefix( QString::null );
+  setGivenName( QString::null );
+  setAdditionalName( QString::null );
+  setFamilyName( QString::null );
+  setSuffix( QString::null );
 
   if ( str.isEmpty() )
     return;
 
-  int i = str.find(',');
+  QString spaceStr = " ";
+  QString emptyStr = "";
+  AddresseeHelper *helper = AddresseeHelper::self();
+      
+  int i = str.find( ',' );
   if( i < 0 ) {
-    QStringList parts = QStringList::split( " ", str );
+    QStringList parts = QStringList::split( spaceStr, str );
     int leftOffset = 0;
     int rightOffset = parts.count() - 1;
 
     QString suffix;
     while ( rightOffset >= 0 ) {
-      if ( suffixes.contains( parts[ rightOffset ] ) ) {
-        suffix.prepend(parts[ rightOffset ] + (suffix.isEmpty() ? "" : " "));
+      if ( helper->containsSuffix( parts[ rightOffset ] ) ) {
+        suffix.prepend(parts[ rightOffset ] + (suffix.isEmpty() ? emptyStr : spaceStr));
         rightOffset--;
       } else
         break;
@@ -164,16 +188,16 @@ void Addressee::setNameFromString( const QString &str )
     if ( rightOffset < 0 )
       return;
 
-    if ( rightOffset - 1 >= 0 && prefixes.contains( parts[ rightOffset - 1 ].lower() ) ) {
-      setFamilyName( parts[ rightOffset - 1 ] + " " + parts[ rightOffset ] );
+    if ( rightOffset - 1 >= 0 && helper->containsPrefix( parts[ rightOffset - 1 ].lower() ) ) {
+      setFamilyName( parts[ rightOffset - 1 ] + spaceStr + parts[ rightOffset ] );
       rightOffset--;
     } else
       setFamilyName( parts[ rightOffset ] );
 
     QString prefix;
     while ( leftOffset < rightOffset ) {
-      if ( titles.contains( parts[ leftOffset ] ) ) {
-        prefix.append( ( prefix.isEmpty() ? "" : " ") + parts[ leftOffset ] );
+      if ( helper->containsTitle( parts[ leftOffset ] ) ) {
+        prefix.append( ( prefix.isEmpty() ? emptyStr : spaceStr) + parts[ leftOffset ] );
         leftOffset++;
       } else
         break;
@@ -187,7 +211,7 @@ void Addressee::setNameFromString( const QString &str )
 
     QString additionalName;
     while ( leftOffset < rightOffset ) {
-      additionalName.append( ( additionalName.isEmpty() ? "" : " ") + parts[ leftOffset ] );
+      additionalName.append( ( additionalName.isEmpty() ? emptyStr : spaceStr) + parts[ leftOffset ] );
       leftOffset++;
     }
     setAdditionalName( additionalName );
@@ -195,60 +219,74 @@ void Addressee::setNameFromString( const QString &str )
     QString part1 = str.left( i );
     QString part2 = str.mid( i + 1 );
 
-    QStringList parts = QStringList::split( " ", part1 );
+    QStringList parts = QStringList::split( spaceStr, part1 );
     int leftOffset = 0;
     int rightOffset = parts.count() - 1;
 
-    QString suffix;
-    while ( rightOffset >= 0 ) {
-      if ( suffixes.contains( parts[ rightOffset ] ) ) {
-        suffix.prepend(parts[ rightOffset ] + (suffix.isEmpty() ? "" : " "));
+    if ( parts.count() > 0 ) {
+
+      QString suffix;
+      while ( rightOffset >= 0 ) {
+        if ( helper->containsSuffix( parts[ rightOffset ] ) ) {
+          suffix.prepend(parts[ rightOffset ] + (suffix.isEmpty() ? emptyStr : spaceStr));
+          rightOffset--;
+        } else
+          break;
+      }
+      setSuffix( suffix );
+
+      if ( rightOffset - 1 >= 0 && helper->containsPrefix( parts[ rightOffset - 1 ].lower() ) ) {
+        setFamilyName( parts[ rightOffset - 1 ] + spaceStr + parts[ rightOffset ] );
         rightOffset--;
       } else
-        break;
-    }
-    setSuffix( suffix );
+        setFamilyName( parts[ rightOffset ] );
 
-    if ( rightOffset - 1 >= 0 && prefixes.contains( parts[ rightOffset - 1 ].lower() ) ) {
-      setFamilyName( parts[ rightOffset - 1 ] + " " + parts[ rightOffset ] );
-      rightOffset--;
-    } else
-      setFamilyName( parts[ rightOffset ] );
-
-    QString prefix;
-    while ( leftOffset < rightOffset ) {
-      if ( titles.contains( parts[ leftOffset ] ) ) {
-        prefix.append( ( prefix.isEmpty() ? "" : " ") + parts[ leftOffset ] );
-        leftOffset++;
-      } else
-        break;
+      QString prefix;
+      while ( leftOffset < rightOffset ) {
+        if ( helper->containsTitle( parts[ leftOffset ] ) ) {
+          prefix.append( ( prefix.isEmpty() ? emptyStr : spaceStr) + parts[ leftOffset ] );
+          leftOffset++;
+        } else
+          break;
+      }
+    } else {
+      setPrefix( "" );
+      setFamilyName( "" );
+      setSuffix( "" );
     }
 
-    parts = QStringList::split( " ", part2 );
+    parts = QStringList::split( spaceStr, part2 );
 
     leftOffset = 0;
     rightOffset = parts.count();
 
-    while ( leftOffset < rightOffset ) {
-      if ( titles.contains( parts[ leftOffset ] ) ) {
-        prefix.append( ( prefix.isEmpty() ? "" : " ") + parts[ leftOffset ] );
+    if ( parts.count() > 0 ) {
+
+      QString prefix;
+      while ( leftOffset < rightOffset ) {
+        if ( helper->containsTitle( parts[ leftOffset ] ) ) {
+          prefix.append( ( prefix.isEmpty() ? emptyStr : spaceStr) + parts[ leftOffset ] );
+          leftOffset++;
+        } else
+          break;
+      }
+      setPrefix( prefix );
+
+      if ( leftOffset < rightOffset ) {
+        setGivenName( parts[ leftOffset ] );
         leftOffset++;
-      } else
-        break;
-    }
-    setPrefix( prefix );
+      }
 
-    if ( leftOffset < rightOffset ) {
-      setGivenName( parts[ leftOffset ] );
-      leftOffset++;
+      QString additionalName;
+      while ( leftOffset < rightOffset ) {
+        additionalName.append( ( additionalName.isEmpty() ? emptyStr : spaceStr) + parts[ leftOffset ] );
+        leftOffset++;
+      }
+      setAdditionalName( additionalName );
+    } else {
+      setGivenName( "" );
+      setAdditionalName( "" );
     }
-
-    QString additionalName;
-    while ( leftOffset < rightOffset ) {
-      additionalName.append( ( additionalName.isEmpty() ? "" : " ") + parts[ leftOffset ] );
-      leftOffset++;
-    }
-    setAdditionalName( additionalName );
   }
 }
 
@@ -261,7 +299,7 @@ QString Addressee::realName() const
 
   if ( n.isEmpty() )
     n = name();
-  
+
   return n;
 }
 
@@ -282,18 +320,25 @@ QString Addressee::fullEmail( const QString &email ) const
     e = email;
   }
   if ( e.isEmpty() ) return QString::null;
-  
+
   QString text;
   if ( realName().isEmpty() )
     text = e;
-  else
-    text = assembledName() + " <" + e + ">";
+  else {
+    QRegExp needQuotes( "[^ 0-9A-Za-z\\x0080-\\xFFFF]" );
+    if ( realName().find( needQuotes ) != -1 )
+      text = "\"" + realName() + "\" <" + e + ">";
+    else
+      text = realName() + " <" + e + ">";
+  }
 
   return text;
 }
 
 void Addressee::insertEmail( const QString &email, bool preferred )
 {
+  if (email.simplifyWhiteSpace().isEmpty())
+    return;
   detach();
 
   QStringList::Iterator it = mData->emails.find( email );
@@ -331,7 +376,10 @@ QStringList Addressee::emails() const
 {
   return mData->emails;
 }
-
+void Addressee::setEmails( const QStringList& emails ) {
+    detach();
+    mData->emails = emails;
+}
 void Addressee::insertPhoneNumber( const PhoneNumber &phoneNumber )
 {
   detach();
@@ -344,7 +392,8 @@ void Addressee::insertPhoneNumber( const PhoneNumber &phoneNumber )
       return;
     }
   }
-  mData->phoneNumbers.append( phoneNumber );
+  if ( !phoneNumber.number().simplifyWhiteSpace().isEmpty() )
+    mData->phoneNumbers.append( phoneNumber );
 }
 
 void Addressee::removePhoneNumber( const PhoneNumber &phoneNumber )
@@ -365,7 +414,7 @@ PhoneNumber Addressee::phoneNumber( int type ) const
   PhoneNumber phoneNumber( "", type );
   PhoneNumber::List::ConstIterator it;
   for( it = mData->phoneNumbers.begin(); it != mData->phoneNumbers.end(); ++it ) {
-    if ( matchBinaryPattern( (*it).type(), type, PhoneNumber::Pager ) ) {
+    if ( matchBinaryPattern( (*it).type(), type ) ) {
       if ( (*it).type() & PhoneNumber::Pref )
         return (*it);
       else if ( phoneNumber.number().isEmpty() )
@@ -387,7 +436,7 @@ PhoneNumber::List Addressee::phoneNumbers( int type ) const
 
   PhoneNumber::List::ConstIterator it;
   for( it = mData->phoneNumbers.begin(); it != mData->phoneNumbers.end(); ++it ) {
-    if ( matchBinaryPattern( (*it).type(), type, PhoneNumber::Pager ) ) {
+    if ( matchBinaryPattern( (*it).type(), type ) ) {
       list.append( *it );
     }
   }
@@ -453,6 +502,12 @@ Key Addressee::key( int type, QString customTypeString ) const
   return Key( QString(), type );
 }
 
+void Addressee::setKeys( const Key::List& list )
+{
+  detach();
+  mData->keys = list;
+}
+
 Key::List Addressee::keys() const
 {
   return mData->keys;
@@ -467,13 +522,13 @@ Key::List Addressee::keys( int type, QString customTypeString ) const
     if ( (*it).type() == type ) {
       if ( type == Key::Custom ) {
         if ( customTypeString.isEmpty() ) {
-          list.append(*it);
+          list.append( *it );
         } else {
           if ( (*it).customTypeString() == customTypeString )
-            list.append(*it);
+            list.append( *it );
         }
       } else {
-        list.append(*it);
+        list.append( *it );
       }
     }
   }
@@ -493,15 +548,17 @@ Key Addressee::findKey( const QString &id ) const
 
 QString Addressee::asString() const
 {
-  return "Smith, special agent Smith...";
+  return "Smith, agent Smith...";
 }
 
 void Addressee::dump() const
 {
   kdDebug(5700) << "Addressee {" << endl;
 
+  kdDebug(5700) << "  Uid: '" << uid() << "'" << endl;
+
   --DEBUG--
-  
+
   kdDebug(5700) << "  Emails {" << endl;
   QStringList e = emails();
   QStringList::ConstIterator it;
@@ -571,7 +628,7 @@ Address Addressee::address( int type ) const
   Address address( type );
   Address::List::ConstIterator it;
   for( it = mData->addresses.begin(); it != mData->addresses.end(); ++it ) {
-    if ( matchBinaryPattern( (*it).type(), type, Address::Pref ) ) {
+    if ( matchBinaryPattern( (*it).type(), type ) ) {
       if ( (*it).type() & Address::Pref )
         return (*it);
       else if ( address.isEmpty() )
@@ -593,7 +650,7 @@ Address::List Addressee::addresses( int type ) const
 
   Address::List::ConstIterator it;
   for( it = mData->addresses.begin(); it != mData->addresses.end(); ++it ) {
-    if ( matchBinaryPattern( (*it).type(), type , Address::Pref ) ) {
+    if ( matchBinaryPattern( (*it).type(), type ) ) {
       list.append( *it );
     }
   }
@@ -617,7 +674,7 @@ void Addressee::insertCategory( const QString &c )
   detach();
   mData->empty = false;
 
-  if ( mData->categories.contains( c ) ) return;
+  if ( mData->categories.findIndex( c ) != -1 ) return;
 
   mData->categories.append( c );
 }
@@ -634,7 +691,7 @@ void Addressee::removeCategory( const QString &c )
 
 bool Addressee::hasCategory( const QString &c ) const
 {
-  return ( mData->categories.contains( c ) );
+  return ( mData->categories.findIndex( c ) != -1 );
 }
 
 void Addressee::setCategories( const QStringList &c )
@@ -657,9 +714,9 @@ void Addressee::insertCustom( const QString &app, const QString &name,
 
   detach();
   mData->empty = false;
-  
+
   QString qualifiedName = app + "-" + name + ":";
-  
+
   QStringList::Iterator it;
   for( it = mData->custom.begin(); it != mData->custom.end(); ++it ) {
     if ( (*it).startsWith( qualifiedName ) ) {
@@ -667,16 +724,16 @@ void Addressee::insertCustom( const QString &app, const QString &name,
       return;
     }
   }
-  
+
   mData->custom.append( qualifiedName + value );
 }
 
 void Addressee::removeCustom( const QString &app, const QString &name)
 {
   detach();
-  
+
   QString qualifiedName = app + "-" + name + ":";
-  
+
   QStringList::Iterator it;
   for( it = mData->custom.begin(); it != mData->custom.end(); ++it ) {
     if ( (*it).startsWith( qualifiedName ) ) {
@@ -690,7 +747,7 @@ QString Addressee::custom( const QString &app, const QString &name ) const
 {
   QString qualifiedName = app + "-" + name + ":";
   QString value;
-  
+
   QStringList::ConstIterator it;
   for( it = mData->custom.begin(); it != mData->custom.end(); ++it ) {
     if ( (*it).startsWith( qualifiedName ) ) {
@@ -698,7 +755,7 @@ QString Addressee::custom( const QString &app, const QString &name ) const
       break;
     }
   }
-  
+
   return value;
 }
 
@@ -706,7 +763,7 @@ void Addressee::setCustoms( const QStringList &l )
 {
   detach();
   mData->empty = false;
-  
+
   mData->custom = l;
 }
 
@@ -715,87 +772,108 @@ QStringList Addressee::customs() const
   return mData->custom;
 }
 
-void Addressee::parseEmailAddress( const QString &rawEmail, QString &fullName, 
+void Addressee::parseEmailAddress( const QString &rawEmail, QString &fullName,
                                    QString &email)
 {
   int startPos, endPos, len;
   QString partA, partB, result;
   char endCh = '>';
-  
-  startPos = rawEmail.find('<');
-  if (startPos < 0)
-  {
-    startPos = rawEmail.find('(');
+
+  startPos = rawEmail.find( '<' );
+  if ( startPos < 0 ) {
+    startPos = rawEmail.find( '(' );
     endCh = ')';
   }
-  if (startPos < 0)
-  {
+
+  if ( startPos < 0 ) {
     // We couldn't find any separators, so we assume the whole string
     // is the email address
     email = rawEmail;
     fullName = "";
-  }
-  else 
-  {
+  } else {
     // We have a start position, try to find an end
-    endPos = rawEmail.find(endCh, startPos+1);
-    
-    if (endPos < 0) 
-    {
+    endPos = rawEmail.find( endCh, startPos + 1 );
+
+    if ( endPos < 0 ) {
       // We couldn't find the end of the email address. We can only
       // assume the entire string is the email address.
       email = rawEmail;
       fullName = "";
-    }
-    else
-    {
+    } else {
       // We have a start and end to the email address
-      
+
       // Grab the name part
-      fullName = rawEmail.left(startPos).stripWhiteSpace();
-      
+      QString left = rawEmail.left( startPos ).stripWhiteSpace();
       // grab the email part
-      email = rawEmail.mid(startPos+1, endPos-startPos-1).stripWhiteSpace();
+      QString right = rawEmail.mid( startPos + 1, endPos - startPos - 1 )
+                              .stripWhiteSpace();
+
+      // Either "Name <email>" or "email (Name)"
+      if ( endCh == '>' ) {
+        fullName = left;
+        email = right;
+      } else { // endCh == ')'
+        fullName = right;
+        email = left;
+      }
 
       // Check that we do not have any extra characters on the end of the
       // strings
       len = fullName.length();
-      if (fullName[0]=='"' && fullName[len-1]=='"')
-        fullName = fullName.mid(1, len-2);
-      else if (fullName[0]=='<' && fullName[len-1]=='>')
-        fullName = fullName.mid(1, len-2);
-      else if (fullName[0]=='(' && fullName[len-1]==')')
-        fullName = fullName.mid(1, len-2);
+      if ( fullName[ 0 ] == '"' && fullName[ len - 1 ] == '"' )
+        fullName = fullName.mid( 1, len - 2 );
+      else if ( fullName[ 0 ] == '(' && fullName[ len - 1 ] == ')' )
+        fullName = fullName.mid( 1, len - 2 );
+
+      len = email.length();
+      if ( email[ 0 ] == '<' && email[ len - 1 ] == '>' )
+        email = email.mid( 1, len - 2 );
     }
   }
 }
 
 void Addressee::setResource( Resource *resource )
 {
-    detach();
-    mData->resource = resource;
-} 
+  detach();
+  mData->resource = resource;
+}
 
 Resource *Addressee::resource() const
 {
-    return mData->resource;
+  return mData->resource;
 }
 
 void Addressee::setChanged( bool value )
 {
-    detach();
-    mData->changed = value;
+  detach();
+  mData->changed = value;
 }
 
 bool Addressee::changed() const
 {
-    return mData->changed;
+  return mData->changed;
+}
+
+void Addressee::setSortKey( KABC::Field *field )
+{
+  mSortField = field;
+}
+
+bool Addressee::operator< ( const Addressee &addr )
+{
+  if ( !mSortField )
+    return false;
+  else
+    return ( QString::localeAwareCompare( mSortField->value( *this ).lower(),
+                                          mSortField->value( addr ).lower() ) < 0 );
 }
 
 QDataStream &KABC::operator<<( QDataStream &s, const Addressee &a )
 {
   if (!a.mData) return s;
 
+  s << a.uid();
+
   --STREAMOUT--
   s << a.mData->phoneNumbers;
   s << a.mData->addresses;
@@ -810,6 +888,8 @@ QDataStream &KABC::operator>>( QDataStream &s, Addressee &a )
 {
   if (!a.mData) return s;
 
+  s >> a.mData->uid;
+
   --STREAMIN--
   s >> a.mData->phoneNumbers;
   s >> a.mData->addresses;
@@ -823,25 +903,16 @@ QDataStream &KABC::operator>>( QDataStream &s, Addressee &a )
   return s;
 }
 
-bool matchBinaryPattern( int value, int pattern, int max )
+bool matchBinaryPattern( int value, int pattern )
 {
-  if ( pattern == 0 ) {
-    if ( value != 0 )
-      return false;
-    else
-      return true;
-  }
-
-  int counter = 0;
-  while ( 1 ) {
-    if ( ( pattern & ( 1 << counter ) ) && !( value & ( 1 << counter ) ) )
-      return false;
-
-    if ( ( 1 << counter ) == max )
-      break;
-
-    counter++;
-  }
-
-  return true;
+  /**
+    We want to match all telephonnumbers/addresses which have the bits in the
+    pattern set. More are allowed.
+    if pattern == 0 we have a special handling, then we want only those with
+    exactly no bit set.
+   */
+  if ( pattern == 0 )
+    return ( value == 0 );
+  else
+    return ( pattern == ( pattern & value ) );
 }
