@@ -1,5 +1,6 @@
 /* This file is part of the KDE libraries
    Copyright (c) 2000 Waldo Bastian <bastian@kde.org>
+   Copyright (C) 2002, 2003 Christoph Cullmann <cullmann@kde.org>
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
@@ -20,15 +21,17 @@
 #define _KATE_BUFFER_H_
 
 #include "katetextline.h"
-#include "katecodefoldinghelpers.h"
 
-#include <qstring.h>
 #include <qptrlist.h>
 #include <qobject.h>
 #include <qtimer.h>
 
+class KateCodeFoldingTree;
+class KateLineInfo;
 class KateBufBlock;
 class KateBufFileLoader;
+class KateDocument;
+class Highlight;
 
 class KVMAllocator;
 
@@ -52,23 +55,30 @@ class KateBuffer : public QObject
     /**
      * Create an empty buffer.
      */
-    KateBuffer(class KateDocument *doc);
-   
+    KateBuffer(KateDocument *doc);
+
     /**
      * Goodbye buffer
      */
     ~KateBuffer();
 
+    KateDocument* document() const { return m_doc; }
+
   public slots:
     /**
      * Open a file, use the given filename + codec (internal use of qtextstream)
      */
-    bool openFile (const QString &m_file, QTextCodec *codec);
-   
+    bool openFile (const QString &m_file);
+
+    /**
+     * Can the current codec handle all chars
+     */
+    bool canEncode ();
+
     /**
      * Save the buffer to a file, use the given filename + codec + end of line chars (internal use of qtextstream)
      */
-    bool saveFile (const QString &m_file, QTextCodec *codec, const QString &eol);
+    bool saveFile (const QString &m_file);
 
     /**
      * Return the total number of lines in the buffer.
@@ -78,31 +88,16 @@ class KateBuffer : public QObject
       return m_lines;
     }
 
-    inline uint countVisible ()
-    {
-      return m_lines - m_regionTree->getHiddenLinesCount(m_lines);
-    }
+    uint countVisible ();
 
-    inline uint lineNumber (uint visibleLine)
-    {
-      return m_regionTree->getRealLine (visibleLine);
-    }
+    uint lineNumber (uint visibleLine);
 
-    inline uint lineVisibleNumber (uint line)
-    {
-      return m_regionTree->getVirtualLine (line);
-    }
+    uint lineVisibleNumber (uint line);
 
-    inline void lineInfo (KateLineInfo *info, unsigned int line)
-    {
-      m_regionTree->getLineInfo(info,line);
-    }
+    void lineInfo (KateLineInfo *info, unsigned int line);
+
+    KateCodeFoldingTree *foldingTree ();
 
-    inline KateCodeFoldingTree *foldingTree ()
-    {
-      return m_regionTree;
-    }
-    
     inline void setHlUpdate (bool b)
     {
       m_hlUpdate = b;
@@ -114,7 +109,7 @@ class KateBuffer : public QObject
      * Return line @p i
      */
     TextLine::Ptr line(uint i);
-   
+
     /**
      * Return line @p i without triggering highlighting
      */
@@ -123,7 +118,7 @@ class KateBuffer : public QObject
     /**
      * Return text from line @p i without triggering highlighting
      */
-    QString textLine(uint i);
+    QString textLine(uint i, bool withoutTrailingSpaces = false);
 
     /**
      * Insert @p line in front of line @p i
@@ -151,15 +146,17 @@ class KateBuffer : public QObject
      * @p highlight may be 0 in which case highlighting
      * will be disabled.
      */
-    void setHighlight(class Highlight *highlight);
-   
+    void setHighlight (Highlight *highlight);
+
+    Highlight *highlight () { return m_highlight; };
+
     /**
      * Update the highlighting.
      *
-     * PRE-condition: 
+     * PRE-condition:
      *   All lines prior to @p from have been highlighted already.
      *
-     * POST-condition: 
+     * POST-condition:
      *   All lines till at least @p to haven been highlighted.
      */
     void updateHighlighting(uint from, uint to, bool invalidate);
@@ -178,43 +175,46 @@ class KateBuffer : public QObject
      * Get the text between the two given positions.
      */
     QString text(uint startLine, uint startCol, uint endLine, uint endCol, bool blockwise = false);
-   
+
     uint length ();
-    
+
     int lineLength ( uint line );
-    
+
     /**
      * change the visibility of a given line
      */
     void setLineVisible (unsigned int lineNr, bool visible);
-   
+
+    /**
+     * was the last loading broken because of not enough tmp disk space ?
+     * (will be reseted on successful save of the file, user gets warning if he really wants to do it)
+     */
+    bool loadingBorked () const
+    {
+      return m_loadingBorked;
+    }
+
+    void setTabWidth (uint w);
+    
+    inline uint tabWidth () const { return m_tabWidth; }
+
   signals:
     /**
      * Emitted during loading when the line count changes.
      */
     void linesChanged(int lines);
-   
-    /**
-     * Emitted when some code folding related attributes changed
-     */
-    void foldingUpdate(unsigned int , QMemArray<signed char>* ,bool *changed,bool foldingChanged);
 
     /**
      * Emittend if codefolding returned with a changed list
      */
     void codeFoldingUpdated();
-    
+
     /**
      * Emitted when the highlighting of a certain range has
      * changed.
      */
     void tagLines(int start, int end);
 
-    /**
-     * Advice to update highlighting a certain range.
-     */
-    void pleaseHighlight(uint from, uint to);
-
     /**
      * Loading of the file finished
      */
@@ -240,7 +240,7 @@ class KateBuffer : public QObject
      * Find the block containing line @p i
      */
     KateBufBlock *findBlock(uint i);
-   
+
     void checkLoadedMax ();
     void checkCleanMax ();
     void checkDirtyMax ();
@@ -257,53 +257,58 @@ class KateBuffer : public QObject
      * false otherwise.
      */
     bool needHighlight(KateBufBlock *buf, uint from, uint to);
- 
+
+  private:
+    void pleaseHighlight (uint,uint);
+
   private slots:
-    /**
-     * Load a part of the file that is currently loading.
-     */
-    void loadFilePart();
-    
-    void slotBufferUpdateHighlight (uint,uint);
-    void slotBufferUpdateHighlight ();
+    void pleaseHighlight ();
 
   private:
     bool m_hlUpdate;
-  
+
     uint m_lines;
     uint m_highlightedTo; // The highest line with correct highlight info
     uint m_highlightedRequested; // The highest line that we requested highlight for
-   
+
     uint m_lastInSyncBlock;  // last block where the start/end line is in sync with real life
 
-    class Highlight *m_highlight;
-    class KateDocument *m_doc;
-      
+    Highlight *m_highlight;
+    KateDocument *m_doc;
+
     // stuff we need to load a file
     KateBufFileLoader *m_loader;
     QTimer m_loadTimer;
 
     // ALL blocks
     QPtrList<KateBufBlock> m_blocks;
-   
+
     // List of blocks that can be swapped out.
     QPtrList<KateBufBlock> m_loadedBlocks;
-   
+
     // List of blocks that can be disposed.
     QPtrList<KateBufBlock> m_cleanBlocks;
-   
+
     // List of blocks that are dirty.
     QPtrList<KateBufBlock> m_dirtyBlocks;
-   
-    class KVMAllocator *m_vm;
+
+    KVMAllocator *m_vm;
 
     // folding tree
     KateCodeFoldingTree *m_regionTree;
-    
+
     QTimer m_highlightTimer;
-    
+
     uint m_highlightedTill;
     uint m_highlightedEnd;
+    uint m_highlightedSteps;
+
+    bool m_cacheReadError;
+    bool m_cacheWriteError;
+    bool m_loadingBorked;
+
+    // for the scrapty indent sensitive langs
+    uint m_tabWidth;
 };
 
 #endif
