@@ -16,16 +16,17 @@
    Boston, MA 02111-1307, USA.
 */
 
-// $Id: katecodefoldinghelpers.cpp,v 1.39.2.1 2003/01/28 18:44:41 waba Exp $
-
 #include "katecodefoldinghelpers.h"
 #include "katecodefoldinghelpers.moc"
-#include <qstring.h>
+
 #include <kdebug.h>
-  
+
+#include <qstring.h>
 
 #define JW_DEBUG 0
 
+bool KateCodeFoldingTree::trueVal = true;
+
 KateCodeFoldingNode::KateCodeFoldingNode() :
     parentNode(0),
     startLineRel(0),
@@ -43,7 +44,7 @@ KateCodeFoldingNode::KateCodeFoldingNode() :
 KateCodeFoldingNode::KateCodeFoldingNode(KateCodeFoldingNode *par, signed char typ, unsigned int sLRel):
     parentNode(par),
     startLineRel(sLRel),
-    endLineRel(20000),
+    endLineRel(10000),
     startLineValid(true),
     endLineValid(false),
     type(typ),
@@ -64,33 +65,31 @@ KateCodeFoldingNode::~KateCodeFoldingNode()
 
 KateCodeFoldingTree::KateCodeFoldingTree(QObject *par): QObject(par), KateCodeFoldingNode()
 {
-  clear(); 
+  clear();
+}
+
+void KateCodeFoldingTree::fixRoot(int endLRel)
+{
+  endLineRel = endLRel;
 }
 
 void KateCodeFoldingTree::clear()
 {
   if (m_childnodes)
-  {
-	m_childnodes->setAutoDelete(true);
-	m_childnodes->clear();
-	m_childnodes->setAutoDelete(false);
-  }
-  dontIgnoreUnchangedLines.setAutoDelete(true);
-  dontIgnoreUnchangedLines.clear();
-  lineMapping.setAutoDelete(true);
-  lineMapping.clear();
-  hiddenLinesCountCacheValid=false;
+    m_childnodes->clear();
 
   // initialize the root "special" node
-  startLineRel=0;
   startLineValid=true;
   endLineValid=true; // temporary, should be false;
-  endLineRel=60000;   // temporary;
+  endLineRel=1;      // temporary;
 
-  markedForDeleting.clear();
-  nodesForLine.clear();
+  hiddenLinesCountCacheValid=false;
+  lineMapping.setAutoDelete(true);
   hiddenLines.clear();
-
+  lineMapping.clear();
+  nodesForLine.clear();
+  markedForDeleting.clear();
+  dontIgnoreUnchangedLines.clear();
 }
 
 KateCodeFoldingTree::~KateCodeFoldingTree()
@@ -104,7 +103,7 @@ bool KateCodeFoldingTree::isTopLevel(unsigned int line)
 
   // look if a given lines belongs to a sub node
   for ( KateCodeFoldingNode *node = m_childnodes->first(); node; node = m_childnodes->next() )
-  { 
+  {
     if ((node->startLineRel<=line) && (line<=node->startLineRel+node->endLineRel))
       return false;  // the line is within the range of a subnode -> return toplevel=false
   }
@@ -124,7 +123,7 @@ void KateCodeFoldingTree::getLineInfo(KateLineInfo *info, unsigned int line)
 
   if (!hasChildNodes())
     return;
-  
+
   //let's look for some information
   for ( KateCodeFoldingNode *node = m_childnodes->first(); node; node = m_childnodes->next() )
   {
@@ -132,7 +131,7 @@ void KateCodeFoldingTree::getLineInfo(KateLineInfo *info, unsigned int line)
     {
       info->topLevel = false; //we are definitly not toplevel
       findAllNodesOpenedOrClosedAt(line); //lookup all nodes, which start or and at the given line
-      
+
       for ( KateCodeFoldingNode *node = nodesForLine.first(); node; node = nodesForLine.next() )
       {
         uint startLine = getStartLine(node);
@@ -194,7 +193,7 @@ KateCodeFoldingNode *KateCodeFoldingTree::findNodeForLineDescending ( KateCodeFo
       {
         // a subnode contains the line.
         // if oneStepOnly is true, we don't want to search for the deepest node, just return the found one
-        
+
         if (oneStepOnly)
           return subNode;
         else
@@ -202,7 +201,7 @@ KateCodeFoldingNode *KateCodeFoldingTree::findNodeForLineDescending ( KateCodeFo
       }
     }
   }
-  
+
   return node; // the current node has no sub nodes, or the line couldn'te be found within a subregion
 }
 
@@ -246,7 +245,7 @@ void KateCodeFoldingTree::updateLine(unsigned int line,
     else
       return;
   }
- 
+
   something_changed = false;
 
   findAndMarkAllNodesforRemovalOpenedOrClosedAt(line);
@@ -259,6 +258,13 @@ void KateCodeFoldingTree::updateLine(unsigned int line,
   }
   else
   {
+    for (unsigned int i=0;i<regionChanges->size() / 2;i++) {
+        signed char tmp=(*regionChanges)[regionChanges->size()-1-i];
+        (*regionChanges)[regionChanges->size()-1-i]=(*regionChanges)[i];
+        (*regionChanges)[i]=tmp;
+    }
+
+
     signed char data= (*regionChanges)[regionChanges->size()-1];
     regionChanges->resize (regionChanges->size()-1);
 
@@ -270,7 +276,7 @@ void KateCodeFoldingTree::updateLine(unsigned int line,
       //  if (insertPos==-1)
       {
         unsigned int tmpLine=line-getStartLine(node);
-        
+
         for (int i=0; i<(int)node->childnodes()->count(); i++)
         {
           if (node->childnodes()->at(i)->startLineRel >= tmpLine)
@@ -333,7 +339,7 @@ void KateCodeFoldingTree::updateLine(unsigned int line,
           if (node->childnodes()->at(insertPos)->startLineRel == line-startLine)
           {
             addOpening(node->childnodes()->at(insertPos), data, regionChanges, line);
-            insertPos++;;
+            insertPos++;
           }
           else
           {
@@ -356,26 +362,26 @@ void KateCodeFoldingTree::updateLine(unsigned int line,
       }
     } while (data!=0);
   }
-  
+
   cleanupUnneededNodes(line);
 //  if (something_changed) emit regionBeginEndAddedRemoved(line);
   (*updated) = something_changed;
 }
 
 
-void KateCodeFoldingTree::removeOpening(KateCodeFoldingNode *node,unsigned int line)
+bool KateCodeFoldingTree::removeOpening(KateCodeFoldingNode *node,unsigned int line)
 {
   signed char type;
   if ((type=node->type) == 0)
   {
     dontDeleteOpening(node);
     dontDeleteEnding(node);
-    return;
+    return false;
   }
-  
+
   if (!node->visible)
   {
-	toggleRegionVisibility(getStartLine(node));
+  toggleRegionVisibility(getStartLine(node));
   }
 
   KateCodeFoldingNode *parent = node->parentNode;
@@ -397,27 +403,32 @@ void KateCodeFoldingTree::removeOpening(KateCodeFoldingNode *node,unsigned int l
   //mypos = parent->childnodes()->find(node);
   bool endLineValid = node->endLineValid;
   int endLineRel = node->endLineRel;
-  
+
   // removes + deletes, as autodelete is on
   parent->childnodes()->remove(mypos);
-  
+
   if ((type>0) && (endLineValid))
     correctEndings(-type, parent, line+endLineRel/*+1*/, mypos); // why the hell did I add a +1 here ?
-       }
+  }
+
+  return true;
 }
 
-void KateCodeFoldingTree::removeEnding(KateCodeFoldingNode *node,unsigned int /* line */)
+bool KateCodeFoldingTree::removeEnding(KateCodeFoldingNode *node,unsigned int /* line */)
 {
   KateCodeFoldingNode *parent = node->parentNode;
 
+  if (!parent)
+    return false;
+  
   if (node->type == 0)
-    return;
+    return false;
 
   if (node->type < 0)
   {
           // removes + deletes, as autodelete is on
     parent->childnodes()->remove (node);
-    return;
+    return true;
   }
 
   int mypos = parent->childnodes()->find(node);
@@ -441,7 +452,7 @@ void KateCodeFoldingTree::removeEnding(KateCodeFoldingNode *node,unsigned int /*
           node->childnodes()->append(tmp);
         }
       }
-      return;
+      return false;
     }
   }
 
@@ -456,22 +467,23 @@ void KateCodeFoldingTree::removeEnding(KateCodeFoldingNode *node,unsigned int /*
     }
 
     // this should fix the bug of wrongly closed nodes
-    if (!parent->parentNode) node->endLineValid=false;
-	else 
-		node->endLineValid = parent->endLineValid;
+    if (!parent->parentNode)
+      node->endLineValid=false;
+    else
+      node->endLineValid = parent->endLineValid;
+
     node->endLineRel = parent->endLineRel-node->startLineRel;
 
     if (node->endLineValid)
-      removeEnding(parent, getStartLine(parent)+parent->endLineRel);
+      return removeEnding(parent, getStartLine(parent)+parent->endLineRel);
 
-    return;
+    return false;
   }
 
   node->endLineValid = false;
   node->endLineRel = parent->endLineRel - node->startLineRel;
-//  if (parent && (!parent->parentNode)) {
-//	kdDebug(13000)<<"removing ending of toplevel node"<<endl;
-//  }
+
+  return false;
 }
 
 
@@ -493,7 +505,7 @@ bool KateCodeFoldingTree::correctEndings(signed char data, KateCodeFoldingNode *
     newNode->startLineValid = false;
     newNode->endLineValid = true;
     newNode->endLineRel = 0;
-      
+
     if ((insertPos==-1) || (insertPos==(int)node->childnodes()->count()))
       node->childnodes()->append(newNode);
     else
@@ -505,20 +517,16 @@ bool KateCodeFoldingTree::correctEndings(signed char data, KateCodeFoldingNode *
   else
   {
     something_changed = true;
-    dontDeleteEnding(node);  
+    dontDeleteEnding(node);
 
     // valid closing region
-    uint startLine = getStartLine(node);
     if (!node->endLineValid)
     {
       node->endLineValid = true;
       node->endLineRel = line - startLine;
       //moving
-#if JW_DEBUG
-	kdDebug(13000)<<"Move subnodes up, if needed"<<endl;
-#endif
-	moveSubNodesUp(node);
 
+      moveSubNodesUp(node);
     }
     else
     {
@@ -543,8 +551,8 @@ bool KateCodeFoldingTree::correctEndings(signed char data, KateCodeFoldingNode *
         kdDebug(13000)<< "reclosed node had childnodes()"<<endl;
         kdDebug(13000)<<"It could be, that childnodes() need to be moved up"<<endl;
 #endif
-	moveSubNodesUp(node);
-                        
+  moveSubNodesUp(node);
+
         if (node->parentNode)
         {
           correctEndings(data,node->parentNode,bakEndLine, node->parentNode->childnodes()->find(node)+1); // ????
@@ -699,7 +707,7 @@ void KateCodeFoldingTree::addOpening(KateCodeFoldingNode *node,signed char nType
          break;
       }
     }
-    
+
     int current;
     if (insert_position==-1)
     {
@@ -711,7 +719,7 @@ void KateCodeFoldingTree::addOpening(KateCodeFoldingNode *node,signed char nType
       node->childnodes()->insert(insert_position, newNode);
       current = insert_position;
     }
-    
+
 //    if (node->type==newNode->type)
 //    {
 //      newNode->endLineValid=true;
@@ -740,7 +748,7 @@ void KateCodeFoldingTree::addOpening(KateCodeFoldingNode *node,signed char nType
         else
         {
           node->endLineValid = false;
-          node->endLineRel = 20000;
+          node->endLineRel = 10000;
         }
         if (count > 0)
         {
@@ -753,7 +761,7 @@ void KateCodeFoldingTree::addOpening(KateCodeFoldingNode *node,signed char nType
         }
 //      }
     }
-    
+
     addOpening(newNode, nType, list, line);
 
     addOpening_further_iterations(node, node->type, list, line, current, startLine);
@@ -843,9 +851,9 @@ unsigned int KateCodeFoldingTree::getStartLine(KateCodeFoldingNode *node)
 void KateCodeFoldingTree::lineHasBeenRemoved(unsigned int line)
 {
   lineMapping.clear();
-  dontIgnoreUnchangedLines.insert(line,new bool(true));
-  dontIgnoreUnchangedLines.insert(line-1,new bool(true));
-  dontIgnoreUnchangedLines.insert(line+1,new bool(true));
+  dontIgnoreUnchangedLines.insert(line, &trueVal);
+  dontIgnoreUnchangedLines.insert(line-1, &trueVal);
+  dontIgnoreUnchangedLines.insert(line+1, &trueVal);
   hiddenLinesCountCacheValid = false;
 #if JW_DEBUG
   kdDebug(13000)<<QString("KateCodeFoldingTree::lineHasBeenRemoved: %1").arg(line)<<endl;
@@ -878,6 +886,14 @@ void KateCodeFoldingTree::lineHasBeenRemoved(unsigned int line)
 
   if (node->parentNode)
     decrementBy1(node->parentNode, node);
+
+  for (QValueList<hiddenLineBlock>::Iterator it=hiddenLines.begin(); it!=hiddenLines.end(); ++it)
+  {
+    if ((*it).start > line)
+      (*it).start--;
+    else if ((*it).start+(*it).length > line)
+      (*it).length--;
+  }
 }
 
 
@@ -900,14 +916,14 @@ void KateCodeFoldingTree::decrementBy1(KateCodeFoldingNode *node, KateCodeFoldin
 void KateCodeFoldingTree::lineHasBeenInserted(unsigned int line)
 {
   lineMapping.clear();
-  dontIgnoreUnchangedLines.insert(line,new bool(true));
-  dontIgnoreUnchangedLines.insert(line-1,new bool(true));
-  dontIgnoreUnchangedLines.insert(line+1,new bool(true));
+  dontIgnoreUnchangedLines.insert(line, &trueVal);
+  dontIgnoreUnchangedLines.insert(line-1, &trueVal);
+  dontIgnoreUnchangedLines.insert(line+1, &trueVal);
   hiddenLinesCountCacheValid = false;
 //return;
 #if JW_DEBUG
   kdDebug(13000)<<QString("KateCodeFoldingTree::lineHasBeenInserted: %1").arg(line)<<endl;
-#endif 
+#endif
 
 //  findAndMarkAllNodesforRemovalOpenedOrClosedAt(line);
 //  cleanupUnneededNodes(line);
@@ -980,17 +996,28 @@ void KateCodeFoldingTree::findAndMarkAllNodesforRemovalOpenedOrClosedAt(unsigned
 
 void KateCodeFoldingTree::addNodeToRemoveList(KateCodeFoldingNode *node,unsigned int line)
 {
+  bool add=false;
 #ifdef __GNUC__
 #warning "FIXME:  make this multiple region changes per line save";
 #endif
   unsigned int startLine=getStartLine(node);
   if ((startLine==line) && (node->startLineValid))
+  {
+    add=true;
     node->deleteOpening = true;
-
+  }
   if ((startLine+node->endLineRel==line) || ((node->endLineValid==false) && (node->deleteOpening)))
+  {
+    int myPos=node->parentNode->childnodes()->find(node); // this has to be implemented nicely
+    if ((int)node->parentNode->childnodes()->count()>myPos+1)
+     addNodeToRemoveList(node->parentNode->childnodes()->at(myPos+1),line);
+    add=true;
     node->deleteEnding = true;
+  }
 
+  if(add)
   markedForDeleting.append(node);
+
 }
 
 
@@ -1021,14 +1048,16 @@ void KateCodeFoldingTree::findAllNodesOpenedOrClosedAt(unsigned int line)
 void KateCodeFoldingTree::addNodeToFoundList(KateCodeFoldingNode *node,unsigned int line,int childpos)
 {
   unsigned int startLine = getStartLine(node);
+
   if ((startLine==line) && (node->type!=0))
     nodesForLine.append(node);
   else if ((startLine+node->endLineRel==line) && (node->type!=0))
     nodesForLine.append(node);
-  
+
   for (int i=childpos+1; i<(int)node->childnodes()->count(); i++)
   {
     KateCodeFoldingNode *child = node->childnodes()->at(i);
+
     if (startLine+child->startLineRel == line)
     {
       nodesForLine.append(child);
@@ -1049,16 +1078,14 @@ void KateCodeFoldingTree::cleanupUnneededNodes(unsigned int line)
 //  return;
   if (markedForDeleting.isEmpty())
     return;
-  
+
   for (int i=0; i<(int)markedForDeleting.count(); i++)
   {
     KateCodeFoldingNode *node = markedForDeleting.at(i);
-#if JW_DEBUG
     if (node->deleteOpening)
       kdDebug(13000)<<"DELETE OPENING SET"<<endl;
     if (node->deleteEnding)
       kdDebug(13000)<<"DELETE ENDING SET"<<endl;
-#endif
 
     if ((node->deleteOpening) && (node->deleteEnding))
     {
@@ -1083,30 +1110,24 @@ void KateCodeFoldingTree::cleanupUnneededNodes(unsigned int line)
 #if JW_DEBUG
         kdDebug(13000)<<"calling removeOpening"<<endl;
 #endif
-        if (node->type > 0)
-          removeOpening(node, line);
+        removeOpening(node, line);
         something_changed = true;
-        dontDeleteOpening(node);
-
       }
       else
       {
         dontDeleteOpening(node);
+
         if ((node->deleteEnding) && (node->endLineValid))
         {
-//#if JW_DEBUG
-          kdDebug(13000)<<"calling removeEnding"<<endl;
-//#endif
+          dontDeleteEnding(node);
           removeEnding(node, line);
           something_changed = true;
-          dontDeleteEnding(node);
         }
         else
           dontDeleteEnding(node);
       }
     }
   }
-//  debugDump();
 }
 
 void KateCodeFoldingTree::dontDeleteEnding(KateCodeFoldingNode* node)
@@ -1125,9 +1146,7 @@ void KateCodeFoldingTree::toggleRegionVisibility(unsigned int line)
 {
   lineMapping.clear();
   hiddenLinesCountCacheValid = false;
-#if JW_DEBUG
   kdDebug(13000)<<QString("KateCodeFoldingTree::toggleRegionVisibility() %1").arg(line)<<endl;
-#endif
 
   findAllNodesOpenedOrClosedAt(line);
   for (int i=0; i<(int)nodesForLine.count(); i++)
@@ -1182,41 +1201,62 @@ void KateCodeFoldingTree::updateHiddenSubNodes(KateCodeFoldingNode *node)
     else
       updateHiddenSubNodes(iter);
   }
-  return;
 }
 
 void KateCodeFoldingTree::addHiddenLineBlock(KateCodeFoldingNode *node,unsigned int line)
 {
   struct hiddenLineBlock data;
   data.start = line+1;
-  data.length = node->endLineRel;
+  data.length = node->endLineRel-(existsOpeningAtLineAfter(line+node->endLineRel,node)?1:0); // without -1;
   bool inserted = false;
 
   for (QValueList<hiddenLineBlock>::Iterator it=hiddenLines.begin(); it!=hiddenLines.end(); ++it)
   {
     if (((*it).start>=data.start) && ((*it).start<=data.start+data.length-1)) // another hidden block starting at the within this block already exits -> adapt new block
     {
-      // the existing block can't have lines behind the new one, because a newly hidden 
+      // the existing block can't have lines behind the new one, because a newly hidden
       //  block has to encapsulate already hidden ones
       it=hiddenLines.remove(it);
       --it;
     }
     else
+    {
       if ((*it).start > line)
       {
         hiddenLines.insert(it, data);
         inserted = true;
+
         break;
       }
+    }
   }
 
   if (!inserted)
     hiddenLines.append(data);
 
-  for (unsigned int i=line+1; i<=node->endLineRel+line; i++)
+  for (unsigned int i = line+1; i <= (node->endLineRel+line); i++)
     emit(setLineVisible(i,false));
 }
 
+bool KateCodeFoldingTree::existsOpeningAtLineAfter(unsigned int line, KateCodeFoldingNode *node)
+{
+  for(KateCodeFoldingNode *tmp = node->parentNode; tmp; tmp=tmp->parentNode)
+  {
+    KateCodeFoldingNode *tmp2;
+    unsigned int startLine=getStartLine(tmp);
+
+    if ((tmp2 = tmp->childnodes()->at(tmp->childnodes()->find(node) + 1))
+         && ((tmp2->startLineRel + startLine) == line))
+      return true;
+
+    if ((startLine + tmp->endLineRel) > line)
+      return false;
+  }
+
+  return false;
+}
+
+
 //
 // get the real line number for a virtual line
 //
@@ -1227,7 +1267,7 @@ unsigned int KateCodeFoldingTree::getRealLine(unsigned int virtualLine)
     return virtualLine;
 
   // kdDebug(13000)<<QString("VirtualLine %1").arg(virtualLine)<<endl;
-  
+
   unsigned int *real=lineMapping[virtualLine];
   if (real)
     return (*real);
@@ -1267,7 +1307,7 @@ unsigned int KateCodeFoldingTree::getVirtualLine(unsigned int realLine)
   }
 
   // kdDebug(13000)<<QString("-->virtual Line %1").arg(realLine)<<endl;
-  
+
   return realLine;
 }
 
@@ -1288,7 +1328,7 @@ unsigned int KateCodeFoldingTree::getHiddenLinesCount(unsigned int doclen)
 
   for (QValueList<hiddenLineBlock>::ConstIterator it=hiddenLines.begin(); it!=hiddenLines.end(); ++it)
   {
-    if ((*it).start+(*it).length<=doclen) 
+    if ((*it).start+(*it).length<=doclen)
       hiddenLinesCountCache += (*it).length;
     else
     {
@@ -1296,6 +1336,143 @@ unsigned int KateCodeFoldingTree::getHiddenLinesCount(unsigned int doclen)
       break;
     }
   }
-  
+
   return hiddenLinesCountCache;
 }
+
+
+void KateCodeFoldingTree::collapseToplevelNodes()
+{
+  if( !hasChildNodes ())
+    return;
+
+  for (uint i=0; i<m_childnodes->count(); i++)
+  {
+    KateCodeFoldingNode *node = m_childnodes->at(i);
+    if (node->visible && node->startLineValid && node->endLineValid)
+    {
+        node->visible=false;
+        lineMapping.clear();
+        hiddenLinesCountCacheValid = false;
+        addHiddenLineBlock(node,node->startLineRel);
+        emit regionVisibilityChangedAt(node->startLineRel);
+    }
+  }
+}
+
+void KateCodeFoldingTree::expandToplevelNodes(int numLines)
+{
+  KateLineInfo line;
+  for (int i = 0; i < numLines; i++) {
+    getLineInfo(&line, i);
+
+    if (line.startsInVisibleBlock)
+      toggleRegionVisibility(i);
+  }
+}
+
+int KateCodeFoldingTree::collapseOne(int realLine)
+{
+  KateLineInfo line;
+  int unrelatedBlocks = 0;
+  for (int i = realLine; i >= 0; i--) {
+    getLineInfo(&line, i);
+
+    if (line.topLevel && !line.endsBlock)
+      // optimisation
+      break;
+
+    if (line.endsBlock  && i != realLine) {
+      unrelatedBlocks++;
+    }
+
+    if (line.startsVisibleBlock) {
+      unrelatedBlocks--;
+      if (unrelatedBlocks == -1) {
+        toggleRegionVisibility(i);
+        return i;
+      }
+    }
+  }
+  return -1;
+}
+
+void KateCodeFoldingTree::expandOne(int realLine, int numLines)
+{
+  KateLineInfo line;
+  int blockTrack = 0;
+  for (int i = realLine; i >= 0; i--) {
+    getLineInfo(&line, i);
+
+    if (line.topLevel)
+      // done
+      break;
+
+    if (line.startsInVisibleBlock && i != realLine) {
+      if (blockTrack == 0)
+        toggleRegionVisibility(i);
+
+      blockTrack--;
+    }
+
+    if (line.endsBlock)
+      blockTrack++;
+
+    if (blockTrack < 0)
+      // too shallow
+      break;
+  }
+
+  blockTrack = 0;
+  for (int i = realLine; i < numLines; i++) {
+    getLineInfo(&line, i);
+
+    if (line.topLevel)
+      // done
+      break;
+
+    if (line.startsInVisibleBlock) {
+      if (blockTrack == 0)
+        toggleRegionVisibility(i);
+
+      blockTrack++;
+    }
+
+    if (line.endsBlock)
+      blockTrack--;
+
+    if (blockTrack < 0)
+      // too shallow
+      break;
+  }
+}
+
+void KateCodeFoldingTree::ensureVisible( uint line )
+{
+  // first have a look, if the line is really hidden
+  bool found=false;
+  for (QValueList<hiddenLineBlock>::ConstIterator it=hiddenLines.begin();it!=hiddenLines.end();++it)
+  {
+    if ( ((*it).start<=line)  && ((*it).start+(*it).length>line) )
+    {
+      found=true;
+      break;
+    }
+  }
+
+
+  if (!found) return;
+
+  kdDebug()<<"line "<<line<<" is really hidden ->show block"<<endl;
+
+  // it looks like we really have to ensure visibility
+  KateCodeFoldingNode *n = findNodeForLine( line );
+  do {
+    if ( ! n->visible )
+      toggleRegionVisibility( getStartLine( n ) );
+    n = n->parentNode;
+  } while( n );
+
+}
+
+// kate: space-indent on; indent-width 2; replace-tabs on;
