@@ -21,80 +21,63 @@
 #ifndef kate_document_h
 #define kate_document_h
 
+#include "katesupercursor.h"
+#include "katetextline.h"
+
+#include "../interfaces/document.h"
+
 #include <ktexteditor/configinterfaceextension.h>
 #include <ktexteditor/encodinginterface.h>
 #include <ktexteditor/sessionconfiginterface.h>
-#include <ktexteditor/plugin.h>
+#include <ktexteditor/editinterfaceext.h>
 
-#include "katecursor.h"
-#include "katebuffer.h"
-#include "katefont.h"
-#include "katesearch.h"
-#include "katetextline.h"
-#include "katebrowserextension.h"
-#include "../interfaces/document.h"
-
-#include <ktrader.h>
-#include <kservice.h>
-
-#include <qobject.h>
-#include <qptrlist.h>
-#include <qdialog.h>
+#include <dcopobject.h>
 
 #include <qintdict.h>
 #include <qdatetime.h>
 
+namespace KTextEditor { class Plugin; }
+
+namespace KIO { class TransferJob; }
+
 class KateUndoGroup;
 class KateCmd;
+class KateAttribute;
+class KateAutoIndent;
 class KateCodeFoldingTree;
 class KateBuffer;
 class KateView;
 class KateViewInternal;
+class LineRange;
+class KateArbitraryHighlight;
+class KateSuperRange;
+class KateLineInfo;
+class KateBrowserExtension;
+class KateDocumentConfig;
+class Highlight;
+class KatePartPluginItem;
+class KatePartPluginInfo;
 
-namespace Kate
-{
-  class PluginInfo
-  {
-    public:
-      bool load;
-      KService::Ptr service;
-      KTextEditor::Plugin *plugin;
-  };
+class KSpell;
+class KTempFile;
 
-  typedef QPtrList<PluginInfo> PluginList;
-}
-
-class LineRange
-{
-  public:
-    LineRange();
-    
-    void clear();  
-  
-    int line;
-    int visibleLine;
-    int startCol;
-    int endCol;
-    int startX;
-    int endX;
-    bool dirty;
-    int viewLine;
-    bool wrap;
-};
+class QTimer;
 
 //
 // Kate KTextEditor::Document class (and even KTextEditor::Editor ;)
 //
-class KateDocument : public Kate::Document, public KTextEditor::ConfigInterfaceExtension,
-                     public KTextEditor::EncodingInterface, public KTextEditor::SessionConfigInterface
+class KateDocument : public Kate::Document,
+                     public KTextEditor::ConfigInterfaceExtension,
+                     public KTextEditor::EncodingInterface,
+                     public KTextEditor::SessionConfigInterface,
+                     public KTextEditor::EditInterfaceExt,
+                     public DCOPObject
 {
+  K_DCOP
   Q_OBJECT
 
   friend class KateViewInternal;
-  friend class KateView;
-  friend class KateIconBorder;
-  friend class ColorConfig;
-  friend class ViewDefaultsConfig;
+  friend class KateRenderer;
 
   public:
     KateDocument (bool bSingleViewMode=false, bool bBrowserView=false, bool bReadOnly=false,
@@ -103,16 +86,32 @@ class KateDocument : public Kate::Document, public KTextEditor::ConfigInterfaceE
 
     bool closeURL();
 
-    Kate::PluginList *plugins () { return &m_plugins; };
+  //
+  // Plugins section
+  //
+  public:
+    void unloadAllPlugins ();
 
-    void loadAllEnabledPlugins ();
     void enableAllPluginsGUI (KateView *view);
+    void disableAllPluginsGUI (KateView *view);
 
-    void loadPlugin (Kate::PluginInfo *item);
-    void unloadPlugin (Kate::PluginInfo *item);
-    void enablePluginGUI (Kate::PluginInfo *item, KateView *view);
-    void enablePluginGUI (Kate::PluginInfo *item);
-    void disablePluginGUI (Kate::PluginInfo *item);
+    void loadPlugin (uint pluginIndex);
+    void unloadPlugin (uint pluginIndex);
+
+    void enablePluginGUI (KTextEditor::Plugin *plugin, KateView *view);
+    void enablePluginGUI (KTextEditor::Plugin *plugin);
+
+    void disablePluginGUI (KTextEditor::Plugin *plugin, KateView *view);
+    void disablePluginGUI (KTextEditor::Plugin *plugin);
+
+  private:
+     QMemArray<KTextEditor::Plugin *> m_plugins;
+
+  public:
+    bool readOnly () const { return m_bReadOnly; }
+    bool browserView () const { return m_bBrowserView; }
+    bool singleViewMode () const { return m_bSingleViewMode; }
+    KateBrowserExtension *browserExtension () { return m_extension; }
 
   private:
     // only to make part work, don't change it !
@@ -120,7 +119,6 @@ class KateDocument : public Kate::Document, public KTextEditor::ConfigInterfaceE
     bool m_bBrowserView;
     bool m_bReadOnly;
     KateBrowserExtension *m_extension;
-    Kate::PluginList m_plugins;
 
   //
   // KTextEditor::Document stuff
@@ -129,7 +127,7 @@ class KateDocument : public Kate::Document, public KTextEditor::ConfigInterfaceE
     KTextEditor::View *createView( QWidget *parent, const char *name );
     QPtrList<KTextEditor::View> views () const;
 
-     inline KateView *activeView () const { return m_activeView; }
+    inline KateView *activeView () const { return m_activeView; }
 
   private:
     QPtrList<KateView> m_views;
@@ -183,6 +181,7 @@ class KateDocument : public Kate::Document, public KTextEditor::ConfigInterfaceE
     //
     // start edit / end edit (start/end undo, cursor update, view update)
     //
+    void editBegin () { editStart(); }
     void editStart (bool withUndo = true);
     void editEnd ();
 
@@ -192,13 +191,54 @@ class KateDocument : public Kate::Document, public KTextEditor::ConfigInterfaceE
     bool editInsertText ( uint line, uint col, const QString &s );
     bool editRemoveText ( uint line, uint col, uint len );
 
-    bool editWrapLine ( uint line, uint col , bool autowrap = false);
-    bool editUnWrapLine ( uint line, uint col);
+    bool editMarkLineAutoWrapped ( uint line, bool autowrapped );
+
+    bool editWrapLine ( uint line, uint col, bool newLine = true, bool *newLineAdded = 0 );
+    bool editUnWrapLine ( uint line, bool removeLine = true, uint length = 0 );
 
     bool editInsertLine ( uint line, const QString &s );
     bool editRemoveLine ( uint line );
 
-    bool wrapText (uint startLine, uint endLine, uint col);
+    bool wrapText (uint startLine, uint endLine);
+
+  signals:
+    /**
+     * Emitted each time text is inserted into a pre-existing line, including appends.
+     * Does not include newly inserted lines at the moment. ?need
+     */
+    void editTextInserted ( uint line, uint col, uint len);
+
+    /**
+     * Emitted each time text is removed from a line, including truncates and space removal.
+     */
+    void editTextRemoved ( uint line, uint col, uint len);
+
+    /**
+     * Emmitted when text from @p line was wrapped at position pos onto line @p nextLine.
+     */
+    void editLineWrapped ( uint line, uint col, uint len );
+
+    /**
+     * Emitted each time text from @p nextLine was upwrapped onto @p line.
+     */
+    void editLineUnWrapped ( uint line, uint col );
+
+    /**
+     * Emitted whenever a line is inserted before @p line, becoming itself line @ line.
+     */
+    void editLineInserted ( uint line );
+
+    /**
+     * Emitted when a line is deleted.
+     */
+    void editLineRemoved ( uint line );
+
+  private:
+    void undoStart();
+    void undoEnd();
+
+  private slots:
+    void undoCancel();
 
   private:
     void editAddUndo (uint type, uint line, uint col, uint len, const QString &text);
@@ -212,18 +252,19 @@ class KateDocument : public Kate::Document, public KTextEditor::ConfigInterfaceE
     bool editWithUndo;
     uint editTagLineStart;
     uint editTagLineEnd;
-    KateUndoGroup *editCurrentUndo;
+    bool editTagFrom;
+    KateUndoGroup* m_editCurrentUndo;
 
   //
   // KTextEditor::SelectionInterface stuff
   //
   public slots:
     bool setSelection ( const KateTextCursor & start,
-			const KateTextCursor & end );
+      const KateTextCursor & end );
     bool setSelection ( uint startLine, uint startCol,
-			uint endLine, uint endCol );
+      uint endLine, uint endCol );
     bool clearSelection ();
-    bool clearSelection (bool redraw);
+    bool clearSelection (bool redraw, bool finishedChangingSelection = true);
 
     bool hasSelection () const;
     QString selection () const ;
@@ -235,10 +276,10 @@ class KateDocument : public Kate::Document, public KTextEditor::ConfigInterfaceE
     //
     // KTextEditor::SelectionInterfaceExt
     //
-    int selStartLine() { return selectStart.line; };
-    int selStartCol()  { return selectStart.col; };
-    int selEndLine()   { return selectEnd.line; };
-    int selEndCol()    { return selectEnd.col; };
+    int selStartLine() { return selectStart.line(); };
+    int selStartCol()  { return selectStart.col(); };
+    int selEndLine()   { return selectEnd.line(); };
+    int selEndCol()    { return selectEnd.col(); };
 
   private:
     // some internal functions to get selection state of a line/col
@@ -248,14 +289,11 @@ class KateDocument : public Kate::Document, public KTextEditor::ConfigInterfaceE
     bool lineHasSelected (int line);
     bool lineIsSelection (int line);
 
-    // stores the current selection
-    KateTextCursor selectStart;
-    KateTextCursor selectEnd;
-    KateTextCursor oldSelectStart;
-    KateTextCursor oldSelectEnd;
+    QPtrList<KateSuperCursor> m_superCursors;
 
-    // only to make the selection from the view easier
-    KateTextCursor selectAnchor;
+    // stores the current selection
+    KateSuperCursor selectStart;
+    KateSuperCursor selectEnd;
 
   signals:
     void selectionChanged ();
@@ -293,13 +331,14 @@ class KateDocument : public Kate::Document, public KTextEditor::ConfigInterfaceE
     //
     QPtrList<KateUndoGroup> undoItems;
     QPtrList<KateUndoGroup> redoItems;
+    bool m_undoDontMerge;
+    bool m_undoIgnoreCancel;
+    QTimer* m_undoMergeTimer;
     // these two variables are for resetting the document to
     // non-modified if all changes have been undone...
     KateUndoGroup* lastUndoGroupWhenSaved;
     bool docWasSavedWhenUndoWasEmpty;
 
-    uint myUndoSteps;
-
     // this sets
     void updateModified();
 
@@ -344,6 +383,15 @@ class KateDocument : public Kate::Document, public KTextEditor::ConfigInterfaceE
   signals:
     void hlChanged ();
 
+  //
+  // Kate::ArbitraryHighlightingInterface stuff
+  //
+  public:
+    KateArbitraryHighlight* arbitraryHL() const { return m_arbitraryHL; };
+
+  private slots:
+    void tagArbitraryLines(KateView* view, KateSuperRange* range);
+
   //
   // KTextEditor::ConfigInterface stuff
   //
@@ -374,7 +422,8 @@ class KateDocument : public Kate::Document, public KTextEditor::ConfigInterfaceE
     void setPixmap( MarkInterface::MarkTypes, const QPixmap& );
     void setDescription( MarkInterface::MarkTypes, const QString& );
     QString markDescription( MarkInterface::MarkTypes );
-    QPixmap markPixmap( MarkInterface::MarkTypes );
+    QPixmap *markPixmap( MarkInterface::MarkTypes );
+    QColor markColor( MarkInterface::MarkTypes );
 
     void setMarksUserChangable( uint markMask );
     uint editableMarks();
@@ -387,7 +436,6 @@ class KateDocument : public Kate::Document, public KTextEditor::ConfigInterfaceE
     QIntDict<KTextEditor::Mark> m_marks;
     QIntDict<QPixmap>           m_markPixmaps;
     QIntDict<QString>           m_markDescriptions;
-    bool                        restoreMarks;
     uint                        m_editableMarks;
 
   //
@@ -397,18 +445,38 @@ class KateDocument : public Kate::Document, public KTextEditor::ConfigInterfaceE
     bool printDialog ();
     bool print ();
 
+  //
   //
   // KParts::ReadWrite stuff
   //
   public:
+    bool openURL( const KURL &url );
+
+    /* Anders:
+      I reimplemented this, since i need to check if backup succeeded
+      if requested */
+    bool save();
+
+    bool openFile (KIO::Job * job);
     bool openFile ();
+
     bool saveFile ();
 
-    void setReadWrite( bool );
-    bool isReadWrite() const;
+    void setReadWrite ( bool rw = true );
 
-    void setModified(bool);
-    bool isModified() const;
+    void setModified( bool m );
+
+  private slots:
+    void slotDataKate ( KIO::Job* kio_job, const QByteArray &data );
+    void slotFinishedKate ( KIO::Job * job );
+
+  private:
+    void abortLoadKate();
+
+    void activateDirWatch ();
+    void deactivateDirWatch ();
+
+    QString m_dirWatchFile;
 
   //
   // Kate::Document stuff
@@ -422,55 +490,16 @@ class KateDocument : public Kate::Document, public KTextEditor::ConfigInterfaceE
     Kate::ConfigPage *keysConfigPage (QWidget *);
     Kate::ConfigPage *hlConfigPage (QWidget *);
     Kate::ConfigPage *viewDefaultsConfigPage (QWidget *);
+    Kate::ConfigPage *saveConfigPage( QWidget * );
 
     Kate::ActionMenu *hlActionMenu (const QString& text, QObject* parent = 0, const char* name = 0);
     Kate::ActionMenu *exportActionMenu (const QString& text, QObject* parent = 0, const char* name = 0);
 
-  //
-  // displaying related stuff
-  //
   public:
-     // use different fonts for screen and printing
-    enum WhichFont
-    {
-      ViewFont = 1,
-      PrintFont = 2
-    };
-
-    // ultimate paintLine function (supports startcol/endcol, startx/endx, draw of cursor, tabs + selections)
-    bool paintTextLine ( QPainter &, const LineRange& range, int xPos, int y,
-                                int xStart, int xEnd, int showCursor, bool replaceCursor, int cursorXPos,
-                                bool showSelections, bool showTabs,WhichFont wf=ViewFont, bool currentLine = false,
-                                bool printerfriendly = false, const BracketMark& bm = BracketMark(), int startColX = 0 );
-
-    uint textWidth(const TextLine::Ptr &, int cursorCol, WhichFont wf=ViewFont);
-    uint textWidth(const TextLine::Ptr &textLine, uint startcol, uint maxwidth, uint wrapsymwidth, WhichFont wf, bool *needWrap, int *endX = 0);
-    uint textWidth(KateTextCursor &cursor);
-    uint textWidth(KateTextCursor &cursor, int xPos,WhichFont wf=ViewFont, uint startCol = 0);
-    uint textPos(uint line, int xPos, WhichFont wf=ViewFont, uint startCol = 0);
-    uint textPos(const TextLine::Ptr &, int xPos,WhichFont wf=ViewFont, uint startCol = 0);
-    uint textHeight(WhichFont wf=ViewFont);
-
-    QColor &backCol(int x, int y);
-    QColor &cursorCol(int x, int y);
-
-    const FontStruct& getFontStruct( WhichFont wf );
-    void setFont( WhichFont wf, QFont font );
-    const QFont& getFont( WhichFont wf );
-    const KateFontMetrics& getFontMetrics( WhichFont wf );
-
-    bool selectBounds(uint line, uint &start, uint &end, uint lineLength);
-
-  private:
-    // fonts structures for the view + printing font
-    FontStruct viewFont;
-    FontStruct printFont;
-
-  public:
-    //
-    // internal edit stuff (mostly for view)
-    //
-    bool insertChars ( int line, int col, const QString &chars, KateView *view );
+    /**
+     * Type chars in a view
+     */
+    bool typeChars ( KateView *type, const QString &chars );
 
     /**
      * gets the last line number (numLines() -1)
@@ -478,56 +507,46 @@ class KateDocument : public Kate::Document, public KTextEditor::ConfigInterfaceE
     uint lastLine() const { return numLines()-1;}
 
     TextLine::Ptr kateTextLine(uint i);
+    TextLine::Ptr plainKateTextLine(uint i);
 
-    void setTabWidth(int);
-    int tabWidth() {return tabChars;}
-    void setNewDoc( bool );
-    bool isNewDoc() const;
+    uint configFlags ();
+    void setConfigFlags (uint flags);
 
     /**
        Tag the lines in the current selection.
      */
-    void tagSelection();
-    
+    void tagSelection(const KateTextCursor &oldSelectStart, const KateTextCursor &oldSelectEnd);
+
     // Repaint all of all of the views
     void repaintViews(bool paintOnlyDirty = true);
 
+    Highlight *highlight () { return m_highlight; }
+
   public slots:    //please keep prototypes and implementations in same order
     void tagLines(int start, int end);
     void tagLines(KateTextCursor start, KateTextCursor end);
 
-   //export feature
-   public slots:
-   	void exportAs(const QString&);
+  //export feature
+  public slots:
+     void exportAs(const QString&);
 
-   private: //the following things should become plugins
-   bool exportDocumentToHTML(QTextStream *outputStream,const QString &name);
-   QString HTMLEncode(QChar theChar);
+  private: //the following things should become plugins
+    bool exportDocumentToHTML (QTextStream *outputStream,const QString &name);
+    QString HTMLEncode (QChar theChar);
 
   signals:
     void modifiedChanged ();
     void preHighlightChanged(uint);
 
-  private:
-    Attribute *attribute (uint pos);
-
-  public:
-    class Highlight *highlight() { return m_highlight; }
-
-  private:
-    void makeAttribs();
-    void updateFontData();
-
   private slots:
     void internalHlChanged();
-    void slotLoadingFinished();
 
   public:
     void addView(KTextEditor::View *);
     void removeView(KTextEditor::View *);
 
-    void addCursor(KTextEditor::Cursor *);
-    void removeCursor(KTextEditor::Cursor *);
+    void addSuperCursor(class KateSuperCursor *, bool privateC);
+    void removeSuperCursor(class KateSuperCursor *, bool privateC);
 
     bool ownedView(KateView *);
     bool isLastView(int numViews);
@@ -539,46 +558,58 @@ class KateDocument : public Kate::Document, public KTextEditor::ConfigInterfaceE
     void transpose(     const KateTextCursor& );
     void cut();
     void copy();
-    void paste( const KateTextCursor& cursor, KateView* view );
+    void paste ( KateView* view );
 
-    void selectTo(     const KateTextCursor& from, const KateTextCursor& to );
     void selectWord(   const KateTextCursor& cursor );
     void selectLine(   const KateTextCursor& cursor );
     void selectLength( const KateTextCursor& cursor, int length );
 
-    void indent(      uint line ) { doIndent( line,  1 );  }
-    void unIndent(    uint line ) { doIndent( line, -1 );  }
-    void cleanIndent( uint line ) { doIndent( line,  0 );  }
-    void comment(     uint line ) { doComment( line,  1 ); }
-    void unComment(   uint line ) { doComment( line, -1 ); }
-    private:
-    void doIndent( uint line, int change );
-    void optimizeLeadingSpace( uint line, int flags, int change );
-    void doComment( uint line, int change );
-    public:
-
-    QString getWord( const KateTextCursor& cursor );
-
   public:
-    void tagAll();
-    void updateLines(int startLine, int endLine);
-    void updateLines();
-    void updateViews();
+    void insertIndentChars ( KateView *view );
 
-    void newBracketMark( const KateTextCursor& start, BracketMark& );
-    bool findMatchingBracket( KateTextCursor& start, KateTextCursor& end );
+    void indent ( KateView *view, uint line, int change );
+    void comment ( KateView *view, uint line, int change );
+
+    enum TextTransform { Uppercase, Lowercase, Capitalize };
+
+    /**
+      Handling uppercase, lowercase and capitalize for the view.
+
+      If there is a selection, that is transformed, otherwise for uppercase or
+      lowercase the character right of the cursor is transformed, for capitalize
+      the word under the cursor is transformed.
+    */
+    void transform ( KateView *view, const KateTextCursor &, TextTransform );
+    /**
+      Unwrap a range of lines.
+    */
+    void joinLines( uint first, uint last );
 
   private:
-    void guiActivateEvent( KParts::GUIActivateEvent *ev );
+    void optimizeLeadingSpace( uint line, int flags, int change );
+    void replaceWithOptimizedSpace( uint line, uint upto_column, uint space, int flags );
 
-  private:
-    //
-    // Comment, uncomment methods
-    //
     bool removeStringFromBegining(int line, QString &str);
     bool removeStringFromEnd(int line, QString &str);
 
+    /**
+      Find the position (line and col) of the next char
+      that is not a space. If found line and col point to the found character.
+      Otherwise they have both the value -1.
+      @param line Line of the character which is examined first.
+      @param col Column of the character which is examined first.
+      @returns True if the specified or a following character is not a space
+               Otherwise false.
+    */
     bool nextNonSpaceCharPos(int &line, int &col);
+
+    /**
+      Find the position (line and col) of the previous char
+      that is not a space. If found line and col point to the found character.
+      Otherwise they have both the value -1.
+      @returns True if the specified or a preceding character is not a space.
+               Otherwise false.
+    */
     bool previousNonSpaceCharPos(int &line, int &col);
 
     void addStartLineCommentToSingleLine(int line);
@@ -593,6 +624,19 @@ class KateDocument : public Kate::Document, public KTextEditor::ConfigInterfaceE
     bool removeStartStopCommentFromSelection();
     bool removeStartLineCommentFromSelection();
 
+  public:
+    QString getWord( const KateTextCursor& cursor );
+
+  public:
+    void tagAll();
+    void updateViews();
+
+    void newBracketMark( const KateTextCursor& start, KateTextRange& bm );
+    bool findMatchingBracket( KateTextCursor& start, KateTextCursor& end );
+
+  private:
+    void guiActivateEvent( KParts::GUIActivateEvent *ev );
+
   private slots:
     void slotBufferChanged();
 
@@ -608,15 +652,9 @@ class KateDocument : public Kate::Document, public KTextEditor::ConfigInterfaceE
 
     void setDocName (QString docName);
 
-		inline void lineInfo (KateLineInfo *info, unsigned int line)
-	 {
-	   buffer->lineInfo(info,line);
-	 }
+    void lineInfo (KateLineInfo *info, unsigned int line);
 
-	 inline KateCodeFoldingTree *foldingTree ()
-	 {
-	   return buffer->foldingTree();
-	 }
+    KateCodeFoldingTree *foldingTree ();
 
   public slots:
     /**
@@ -624,22 +662,20 @@ class KateDocument : public Kate::Document, public KTextEditor::ConfigInterfaceE
      */
     void reloadFile();
 
-  private slots:
-    void slotModChanged ();
-
-  public:
-    KateCmd *cmd () { return myCmd; };
-
   public slots:
-    void setEncoding (const QString &e) { myEncoding = e; };
-    QString encoding() const { return myEncoding; };
+    void setEncoding (const QString &e);
+    QString encoding() const;
 
   public slots:
     void setWordWrap (bool on);
-    bool wordWrap () { return myWordWrap; };
+    bool wordWrap ();
 
     void setWordWrapAt (uint col);
-    uint wordWrapAt () { return myWordWrapAt; };
+    uint wordWrapAt ();
+
+  public slots:
+    void setPageUpDownMovesCursor(bool on);
+    bool pageUpDownMovesCursor();
 
   signals:
     void modStateChanged (Kate::Document *doc);
@@ -656,45 +692,46 @@ class KateDocument : public Kate::Document, public KTextEditor::ConfigInterfaceE
      */
     void fileNameChanged ();
 
-  public:
-    //end of line settings
-    enum Eol_settings {eolUnix=0,eolDos=1,eolMacintosh=2};
-
-  public:
-    // wrap the text of the document at the column col
-    void wrapText (uint col);
-
   public slots:
      void applyWordWrap ();
 
   public:
-    uint configFlags ();
-    void setConfigFlags (uint flags);
 
  // code folding
   public:
-        unsigned int getRealLine(unsigned int virtualLine);
-        unsigned int getVirtualLine(unsigned int realLine);
-        unsigned int visibleLines ();
+    unsigned int getRealLine(unsigned int virtualLine);
+    unsigned int getVirtualLine(unsigned int realLine);
+    unsigned int visibleLines ();
 
   signals:
-	void codeFoldingUpdated();
+    void codeFoldingUpdated();
+
   public slots:
-	void dumpRegionTree();
+    void dumpRegionTree();
+
+  private slots:
+    void slotModOnHdDirty (const QString &path);
+    void slotModOnHdCreated (const QString &path);
+    void slotModOnHdDeleted (const QString &path);
+
+  private:
+    /**
+     * create a MD5 digest of the file, if it is a local file,
+     * and fill it into the string @p result.
+     * This is using KMD5::hexDigest().
+     *
+     * @return wheather the operation was attempted and succeded.
+     */
+    bool createDigest( QCString &result );
 
-  //
-  // Some flags for internal ONLY use !
-  //
   public:
+    // should cursor be wrapped ? take config + blockselection state in account
+    bool wrapCursor ();
 
-    // result flags for katepart's internal dialogs
-    enum DialogResults
-    {
-      srYes=QDialog::Accepted,
-      srNo=10,
-      srAll,
-      srCancel=QDialog::Rejected
-    };
+  public:
+    void updateFileType (int newType, bool user = false);
+
+    int fileType () const { return m_fileType; };
 
   //
   // REALLY internal data ;)
@@ -703,61 +740,127 @@ class KateDocument : public Kate::Document, public KTextEditor::ConfigInterfaceE
     // text buffer
     KateBuffer *buffer;
 
-    QColor colors[4];
-    class HlManager *hlManager;
-    class Highlight *m_highlight;
+    Highlight *m_highlight;
 
-    int eolMode;
-    int tabChars;
+    KateArbitraryHighlight* m_arbitraryHL;
 
-    bool readOnly;
-    bool newDoc;          // True if the file is a new document (used to determine whether
-                          // to check for overwriting files on save)
-    bool modified;
-
-    bool myWordWrap;
-    uint myWordWrapAt;
+    KateAutoIndent *m_indenter;
 
     bool hlSetByUser;
 
-    QString myEncoding;
+    bool m_modOnHd;
+    unsigned char m_modOnHdReason;
+    QCString m_digest; // MD5 digest, updated on load/save
 
-    /**
-     * updates mTime to reflect file on fs.
-     * called from constructor and from saveFile.
-     */
-    void setMTime();
-    class QFileInfo* fileInfo;
-    class QDateTime mTime;
     QString m_docName;
+    int m_docNameNumber;
 
-    class KateCmd *myCmd;
+    // file type !!!
+    int m_fileType;
+    bool m_fileTypeSetByUser;
 
-    QMemArray<Attribute> myAttribs;
+  public slots:
+    void spellcheck();
+    void ready(KSpell *);
+    void misspelling( const QString&, const QStringList&, unsigned int );
+    void corrected  ( const QString&, const QString&, unsigned int);
+    void spellResult( const QString& );
+    void spellCleanDone();
 
-    //
-    // core katedocument config !
-    //
-    uint _configFlags;
+
+    void slotQueryClose_save(bool *handled, bool* abortClosing);
 
   private:
-    // defaults for all views !!!
-    bool m_dynWordWrap;
-    bool m_lineNumbers;
-    bool m_iconBar;
-    bool m_foldingBar;
-    int m_bookmarkSort;
+    void makeAttribs ();
+
+    void locatePosition( uint pos, uint& line, uint& col );
+    KSpell*         m_kspell;
+    int             m_mispellCount;
+    int             m_replaceCount;
+    bool            m_reloading;
 
   public:
-    void updateViewDefaults ();
+    static bool checkOverwrite( KURL u );
 
+    static void setDefaultEncoding (const QString &encoding);
+
+  /**
+   * Configuration
+   */
   public:
+    inline KateDocumentConfig *config () { return m_config; };
+
+    void updateConfig ();
+
+  private:
+    KateDocumentConfig *m_config;
+
+  /**
+   * Variable Reader
+   * TODO add register functionality/ktexteditor interface
+   */
+  private:
     /**
-      Allow the HlManager to fill the array
+      Reads all the variables in the document.
+      Called when opening/saving a document
     */
-    QMemArray<Attribute> *attribs() { return &myAttribs; }
+    void readVariables(bool onlyViewAndRenderer = false);
+
+    /**
+      Reads and applies the variables in a single line
+      TODO registered variables gets saved in a [map]
+    */
+    void readVariableLine( QString t, bool onlyViewAndRenderer = false );
+    /**
+      Sets a view variable in all the views.
+    */
+    void setViewVariable( QString var, QString val );
+    /**
+      @return weather a string value could be converted
+      to a bool value as supported.
+      The value is put in *result.
+    */
+    static bool checkBoolValue( QString value, bool *result );
+    /**
+      @return weather a string value could be converted
+      to a integer value.
+      The value is put in *result.
+    */
+    static bool checkIntValue( QString value, int *result );
+    /*
+      Feeds value into @p col using QColor::setNamedColor() and returns
+      wheather the color is valid
+    */
+    static bool checkColorValue( QString value, QColor &col );
+
+    static QRegExp kvLine;
+    static QRegExp kvVar;
+
+    KIO::TransferJob *m_job;
+    KTempFile *m_tempFile;
+
+  //
+  // IM input stuff
+  //
+  public:
+    void setIMSelectionValue( uint imStartLine, uint imStart, uint imEnd,
+                              uint imSelStart, uint imSelEnd, bool m_imComposeEvent );
+    void getIMSelectionValue( uint *imStartLine, uint *imStart, uint *imEnd,
+                              uint *imSelStart, uint *imSelEnd );
+
+  private:
+    uint m_imStartLine;
+    uint m_imStart;
+    uint m_imEnd;
+    uint m_imSelStart;
+    uint m_imSelEnd;
+    bool m_imComposeEvent;
+
+  k_dcop:
+    uint documentNumber () const;
 };
 
 #endif
 
+// kate: space-indent on; indent-width 2; replace-tabs on;
 
