@@ -1,5 +1,5 @@
 /* This file is part of the KDE libraries
-   Copyright (C) 2001-2002 Christoph Cullmann <cullmann@kde.org>
+   Copyright (C) 2001-2003 Christoph Cullmann <cullmann@kde.org>
    Copyright (C) 2002 Joseph Wenninger <jowenn@kde.org>
 
    Based on:
@@ -20,13 +20,12 @@
    Boston, MA 02111-1307, USA.
 */
 
-// $Id: katetextline.cpp,v 1.55 2002/10/31 18:22:07 faure Exp $
-
 #include "katetextline.h"
 
 #include <qregexp.h>
+#include <kglobal.h>
 
-TextLine::TextLine()
+TextLine::TextLine ()
   : m_flags(TextLine::flagVisible)
 {
 }
@@ -42,42 +41,29 @@ void TextLine::insertText (uint pos, uint insLen, const QChar *insText, uchar *i
     return;
 
   // calc new textLen, store old
-  uint textLen = m_text.size();
-  uint oldTextLen = textLen;
+  uint oldTextLen = m_text.length();
+  m_text.insert (pos, insText, insLen);
+  uint textLen = m_text.length();
 
-  if (pos <= textLen)
-    textLen += insLen;
-  else if (pos > textLen)
-    textLen = pos + insLen;
-
-  // resize the arrays
-  m_text.resize (textLen);
+  // resize the array
   m_attributes.resize (textLen);
 
   // HA, insert behind text end, fill with spaces
   if (pos >= oldTextLen)
   {
     for (uint z = oldTextLen; z < pos; z++)
-    {
-      m_text[z] = QChar(' ');
       m_attributes[z] = 0;
-    }
   }
   // HA, insert in text, move the old text behind pos
   else if (oldTextLen > 0)
   {
     for (int z = oldTextLen -1; z >= (int) pos; z--)
-    {
-      m_text[z+insLen] = m_text[z];
       m_attributes[z+insLen] = m_attributes[z];
-    }
   }
 
   // BUH, actually insert the new text
   for (uint z = 0; z < insLen; z++)
   {
-    m_text[z+pos] = insText[z];
-
     if (insAttribs == 0)
       m_attributes[z+pos] = 0;
     else
@@ -91,7 +77,7 @@ void TextLine::removeText (uint pos, uint delLen)
   if (delLen == 0)
     return;
 
-  uint textLen = m_text.size();
+  uint textLen = m_text.length();
 
   if (textLen == 0)
     return; // uh, again nothing real to do ;)
@@ -104,72 +90,50 @@ void TextLine::removeText (uint pos, uint delLen)
 
   // BU, MOVE THE OLD TEXT AROUND
   for (uint z = pos; z < textLen - delLen; z++)
-  {
-      m_text[z] = m_text[z+delLen];
-      m_attributes[z] = m_attributes[z+delLen];
-  }
+    m_attributes[z] = m_attributes[z+delLen];
 
-  // resize the stuff
-  if (delLen < textLen)
-    textLen -= delLen;
-  else // seems to happen when hitting Backspace at the beginning of a line...
-    textLen = 0;
-
-  m_text.resize (textLen);
+  m_text.remove (pos, delLen);
+  textLen = m_text.length ();
   m_attributes.resize (textLen);
 }
 
 void TextLine::append(const QChar *s, uint l)
 {
-  insertText (m_text.size(), l, s, 0);
+  insertText (m_text.length(), l, s, 0);
 }
 
 void TextLine::truncate(uint newLen)
 {
-  if (newLen < m_text.size())
+  if (newLen < m_text.length())
   {
     m_text.truncate (newLen);
     m_attributes.truncate (newLen);
   }
 }
 
-void TextLine::wrap(TextLine::Ptr nextLine, uint pos)
-{
-  int l = m_text.size() - pos;
-
-  if (l > 0)
-  {
-    nextLine->insertText (0, l, &m_text[pos], &m_attributes[pos]);
-    truncate(pos);
-  }
-}
-
-void TextLine::unWrap(uint pos, TextLine::Ptr nextLine, uint len)
-{
-  insertText (pos, len, nextLine->m_text.data(), nextLine->m_attributes.data());
-  nextLine->removeText (0, len);
-}
-
 int TextLine::nextNonSpaceChar(uint pos) const
 {
-    for(int i = pos; i < (int)m_text.size(); i++) {
-        if(!m_text[i].isSpace())
-            return i;
-    }
-    return -1;
+  for(int i = pos; i < (int)m_text.length(); i++)
+  {
+    if(!m_text[i].isSpace())
+      return i;
+  }
+
+  return -1;
 }
 
 int TextLine::previousNonSpaceChar(uint pos) const
 {
-    if(pos >= m_text.size()) {
-        pos = m_text.size() - 1;
-    }
+  if (pos >= m_text.length())
+    pos = m_text.length() - 1;
 
-    for(int i = pos; i >= 0; i--) {
-        if(!m_text[i].isSpace())
-            return i;
-    }
-    return -1;
+  for(int i = pos; i >= 0; i--)
+  {
+    if(!m_text[i].isSpace())
+      return i;
+  }
+
+  return -1;
 }
 
 int TextLine::firstChar() const
@@ -179,64 +143,77 @@ int TextLine::firstChar() const
 
 int TextLine::lastChar() const
 {
-    return previousNonSpaceChar(m_text.size() - 1);
+  return previousNonSpaceChar(m_text.length() - 1);
 }
 
-QString TextLine::string (uint startCol, uint length) const
+QString TextLine::withoutTrailingSpaces()
 {
-  if ( startCol >= m_text.size() )
-    return QString ();
-
-  if ( (startCol + length) > m_text.size())
-    length = m_text.size() - startCol;
-
-  return QString (m_text.data() + startCol, length);
+  return m_text.left (lastChar() + 1);
 }
 
-void TextLine::removeSpaces()
-{
-  truncate(lastChar() + 1);
-}
-
-QChar *TextLine::firstNonSpace() const
+const QChar *TextLine::firstNonSpace() const
 {
   int first = firstChar();
-  return (first > -1) ? &m_text[first] : m_text.data();
+  return (first > -1) ? ((QChar*)m_text.unicode())+first : m_text.unicode();
+}
+
+uint TextLine::indentDepth (uint tabwidth) const
+{
+  uint d = 0;
+
+  for(uint i = 0; i < m_text.length(); i++)
+  {
+    if(m_text[i].isSpace())
+    {
+      if (m_text[i] == QChar('\t'))
+        d += tabwidth - (d % tabwidth);
+      else
+        d++;
+    }
+    else
+      return d;
+  }
+
+  return d;
 }
 
 bool TextLine::stringAtPos(uint pos, const QString& match) const
 {
-  return (QConstString (this->m_text.data(), this->m_text.size()).string().mid(pos, match.length()) == match);
+  return (m_text.mid(pos, match.length()) == match);
 }
 
 bool TextLine::startingWith(const QString& match) const
 {
-  return (QConstString (this->m_text.data(), this->m_text.size()).string().left(match.length()) == match);
+  return (m_text.left(match.length()) == match);
 }
 
 bool TextLine::endingWith(const QString& match) const
 {
-  return (QConstString (this->m_text.data(), this->m_text.size()).string().right(match.length()) == match);
+  return (m_text.right(match.length()) == match);
 }
 
 int TextLine::cursorX(uint pos, uint tabChars) const
 {
-  int l, x, z;
+  uint x = 0;
 
-  l = (pos < m_text.size()) ? pos : m_text.size();
-  x = 0;
-  for (z = 0; z < l; z++) {
-    if (m_text[z] == QChar('\t')) x += tabChars - (x % tabChars); else x++;
+  for (uint z = 0; z < kMin (pos, m_text.length()); z++)
+  {
+    if (m_text[z] == QChar('\t'))
+      x += tabChars - (x % tabChars);
+    else
+      x++;
   }
-  x += pos - l;
+
   return x;
 }
 
-void TextLine::setAttribs(uchar attribute, uint start, uint end) {
-  uint z;
+void TextLine::setAttribs(uchar attribute, uint start, uint end)
+{
+  if (end > m_text.length())
+    end = m_text.length();
 
-  if (end > m_text.size()) end = m_text.size();
-  for (z = start; z < end; z++) m_attributes[z] = attribute;
+  for (uint z = start; z < end; z++)
+    m_attributes[z] = attribute;
 }
 
 bool TextLine::searchText (uint startCol, const QString &text, uint *foundAtCol, uint *matchLen, bool casesensitive, bool backwards)
@@ -244,15 +221,15 @@ bool TextLine::searchText (uint startCol, const QString &text, uint *foundAtCol,
   int index;
 
   if (backwards)
-    index = QConstString (this->m_text.data(), this->m_text.size()).string().findRev (text, startCol, casesensitive);
+    index = m_text.findRev (text, startCol, casesensitive);
   else
-    index = QConstString (this->m_text.data(), this->m_text.size()).string().find (text, startCol, casesensitive);
+    index = m_text.find (text, startCol, casesensitive);
 
-   if (index > -1)
-	{
-	  (*foundAtCol) = index;
-		(*matchLen)=text.length();
-		return true;
+  if (index > -1)
+  {
+    (*foundAtCol) = index;
+    (*matchLen)=text.length();
+    return true;
   }
 
   return false;
@@ -263,15 +240,15 @@ bool TextLine::searchText (uint startCol, const QRegExp &regexp, uint *foundAtCo
   int index;
 
   if (backwards)
-    index = regexp.searchRev (QConstString (this->m_text.data(), this->m_text.size()).string(), startCol);
+    index = regexp.searchRev (m_text, startCol);
   else
-    index = regexp.search (QConstString (this->m_text.data(), this->m_text.size()).string(), startCol);
+    index = regexp.search (m_text, startCol);
 
-   if (index > -1)
-	{
-	  (*foundAtCol) = index;
-		(*matchLen)=regexp.matchedLength();
-		return true;
+  if (index > -1)
+  {
+    (*foundAtCol) = index;
+    (*matchLen)=regexp.matchedLength();
+    return true;
   }
 
   return false;
@@ -297,19 +274,20 @@ uint TextLine::dumpSize () const
     }
   }
 
-  return (4*sizeof(uint)) + (m_text.size()*sizeof(QChar)) + (attributesLen * sizeof(uchar)) + (attributesLen * sizeof(uint)) + 1 + (m_ctx.size() * sizeof(uint)) + (m_foldingList.size() * sizeof(signed char));
+  return (1 + 5*sizeof(uint)) + (m_text.length()*sizeof(QChar)) + (attributesLen * (sizeof(uchar) + sizeof(uint))) + (m_ctx.size() * sizeof(short)) + (m_foldingList.size() * sizeof(signed char) + (m_indentationDepth.size() * sizeof(unsigned short)));
 }
 
 char *TextLine::dump (char *buf) const
 {
-  uint l = m_text.size();
+  uint l = m_text.length();
   uint lctx = m_ctx.size();
   uint lfold = m_foldingList.size();
+  uint lind = m_indentationDepth.size();
 
   memcpy(buf, &l, sizeof(uint));
   buf += sizeof(uint);
 
-  memcpy(buf, (char *) m_text.data(), sizeof(QChar)*l);
+  memcpy(buf, (char *) m_text.unicode(), sizeof(QChar)*l);
   buf += sizeof(QChar)*l;
 
   memcpy(buf, (char *) &m_flags, 1);
@@ -324,6 +302,9 @@ char *TextLine::dump (char *buf) const
   memcpy(buf, &lfold, sizeof(uint));
   buf += sizeof(uint);
 
+  memcpy(buf, &lind, sizeof(uint));
+  buf += sizeof(uint);
+
   // hl size runlength encoding START
 
   uint attributesLen = 0;
@@ -368,12 +349,15 @@ char *TextLine::dump (char *buf) const
 
   // hl size runlength encoding STOP
 
-  memcpy(buf, (char *)m_ctx.data(), sizeof(uint) * lctx);
-  buf += sizeof (uint) * lctx;
+  memcpy(buf, (char *)m_ctx.data(), sizeof(short) * lctx);
+  buf += sizeof (short) * lctx;
 
   memcpy(buf, (char *)m_foldingList.data(), lfold);
   buf += sizeof (signed char) * lfold;
 
+  memcpy(buf, (char *)m_indentationDepth.data(), sizeof(unsigned short) * lind);
+  buf += sizeof (unsigned short) * lind;
+
   return buf;
 }
 
@@ -383,13 +367,14 @@ char *TextLine::restore (char *buf)
   uint lattrib = 0;
   uint lctx = 0;
   uint lfold = 0;
+  uint lind = 0;
 
   // text + context length read
   memcpy((char *) &l, buf, sizeof(uint));
   buf += sizeof(uint);
 
   // text + attributes
-  m_text.duplicate ((QChar *) buf, l);
+  m_text.setUnicode ((QChar *) buf, l);
   buf += sizeof(QChar)*l;
 
   m_attributes.resize (l);
@@ -415,6 +400,9 @@ char *TextLine::restore (char *buf)
   memcpy((char *) &lfold, buf, sizeof(uint));
   buf += sizeof(uint);
 
+  memcpy((char *) &lind, buf, sizeof(uint));
+  buf += sizeof(uint);
+
   // hl size runlength encoding START
 
   uchar *attr = m_attributes.data();
@@ -445,18 +433,16 @@ char *TextLine::restore (char *buf)
 
   // hl size runlength encoding STOP
 
-  m_ctx.duplicate ((uint *) buf, lctx);
-  buf += sizeof(uint) * lctx;
+  m_ctx.duplicate ((short *) buf, lctx);
+  buf += sizeof(short) * lctx;
 
   m_foldingList.duplicate ((signed char *) buf, lfold);
   buf += lfold;
 
+  m_indentationDepth.duplicate ((unsigned short *) buf, lind);
+  buf += sizeof(unsigned short) * lind;
+
   return buf;
 }
 
-
-
-
-
-
-
+// kate: space-indent on; indent-width 2; replace-tabs on;
