@@ -1,7 +1,7 @@
 /* This file is part of the KDE libraries
    Copyright (C) 2002 John Firebaugh <jfirebaugh@kde.org>
    Copyright (C) 2001 Christoph Cullmann <cullmann@kde.org>
-   Copyright (C) 2001 Joseph Wenninger <jowenn@kde.org>  
+   Copyright (C) 2001 Joseph Wenninger <jowenn@kde.org>
    Copyright (C) 1999 Jochen Wilhelmy <digisnap@cs.tu-berlin.de>
 
    This library is free software; you can redistribute it and/or
@@ -22,44 +22,51 @@
 #ifndef kate_view_h
 #define kate_view_h
 
-#include "../interfaces/view.h"
-
 #include "katedocument.h"
-#include "katesearch.h"
 #include "kateviewinternal.h"
-#include "katecodecompletion.h"
+#include "kateconfig.h"
+
+#include "../interfaces/view.h"
 
 #include <ktexteditor/sessionconfiginterface.h>
 #include <ktexteditor/viewstatusmsginterface.h>
-#include <ktexteditor/plugin.h>
+#include <ktexteditor/texthintinterface.h>
 
-#include <qlayout.h>
+#include <qguardedptr.h>
+
+class KateDocument;
+class KateBookmarks;
+class KateSearch;
+class KateCmdLine;
+class KateCodeCompletion;
+class KateViewConfig;
+class KateViewSchemaAction;
+class KateRenderer;
 
 class KToggleAction;
 class KAction;
 class KRecentFilesAction;
 class KSelectAction;
-class KateDocument;
-class KateBookmarks;
+
+class QGridLayout;
 
 //
 // Kate KTextEditor::View class ;)
 //
 class KateView : public Kate::View,
                  public KTextEditor::SessionConfigInterface,
-                 public KTextEditor::ViewStatusMsgInterface
+                 public KTextEditor::ViewStatusMsgInterface,
+                 public KTextEditor::TextHintInterface
 {
     Q_OBJECT
+
     friend class KateViewInternal;
-    friend class KateDocument;
-    friend class KateUndoGroup;
-    friend class KateUndo;
     friend class KateIconBorder;
     friend class KateCodeCompletion;
 
   public:
     KateView( KateDocument* doc, QWidget* parent = 0L, const char* name = 0 );
-    ~KateView ();  
+    ~KateView ();
 
   //
   // KTextEditor::View
@@ -82,7 +89,7 @@ class KateView : public Kate::View,
   public:
     void installPopup( QPopupMenu* menu ) { m_rmbMenu = menu; }
     QPopupMenu* popup() const             { return m_rmbMenu; }
-    
+
   //
   // KTextEditor::ViewCursorInterface
   //
@@ -94,32 +101,42 @@ class KateView : public Kate::View,
     void cursorPositionReal( uint* l, uint* c )
         { if( l ) *l = cursorLine(); if( c ) *c = cursorColumnReal(); }
     bool setCursorPosition( uint line, uint col )
-        { return setCursorPositionInternal( line, col, tabWidth() );  }
+        { return setCursorPositionInternal( line, col, tabWidth(), true );  }
     bool setCursorPositionReal( uint line, uint col)
-        { return setCursorPositionInternal( line, col, 1 );           }
+        { return setCursorPositionInternal( line, col, 1, true );           }
     uint cursorLine()
-        { return m_viewInternal->getCursor().line;                    }
-    uint cursorColumn()
-        { return m_doc->currentColumn(m_viewInternal->getCursor());   }
+        { return m_viewInternal->getCursor().line();                    }
+    uint cursorColumn();
     uint cursorColumnReal()
-        { return m_viewInternal->getCursor().col;                     }
+        { return m_viewInternal->getCursor().col();                     }
+
   signals:
     void cursorPositionChanged();
 
   //
   // KTextEditor::CodeCompletionInterface
   //
-  public:
-    void showArgHint( QStringList arg1, const QString& arg2, const QString& arg3 )
-        { m_codeCompletion->showArgHint( arg1, arg2, arg3 ); }
-    void showCompletionBox( QValueList<KTextEditor::CompletionEntry> arg1, int offset = 0, bool cs = true )
-        { m_codeCompletion->showCompletionBox( arg1, offset, cs ); }
+  public slots:
+    void showArgHint( QStringList arg1, const QString& arg2, const QString& arg3 );
+    void showCompletionBox( QValueList<KTextEditor::CompletionEntry> arg1, int offset = 0, bool cs = true );
+
   signals:
     void completionAborted();
     void completionDone();
     void argHintHidden();
     void completionDone(KTextEditor::CompletionEntry);
     void filterInsertString(KTextEditor::CompletionEntry*,QString *);
+    void aboutToShowCompletionBox();
+
+  //
+  // KTextEditor::TextHintInterface
+  //
+  public:
+    void enableTextHints(int timeout);
+    void disableTextHints();
+
+  signals:
+    void needTextHint(int line, int col, QString &text);
 
   //
   // KTextEditor::DynWordWrapInterface
@@ -128,28 +145,46 @@ class KateView : public Kate::View,
     void setDynWordWrap( bool b );
     bool dynWordWrap() const      { return m_hasWrap; }
 
+  // BEGIN EDIT STUFF
+  public:
+    void editStart ();
+    void editEnd (int editTagLineStart, int editTagLineEnd, bool tagFrom);
+
+    void editSetCursor (const KateTextCursor &cursor);
+  // END
+
+  // BEGIN TAG & CLEAR
+  public:
+    bool tagLine (const KateTextCursor& virtualCursor);
+
+    bool tagLines (int start, int end, bool realLines = false );
+    bool tagLines (KateTextCursor start, KateTextCursor end, bool realCursors = false);
+
+    void tagAll ();
+
+    void clear ();
+
+    void repaintText (bool paintOnlyDirty = false);
+
+    void updateView (bool changed = false);
+  // END
+
   //
   // Kate::View
   //
   public:
-    bool isOverwriteMode() const
-        { return m_doc->_configFlags & KateDocument::cfOvr; }
+    bool isOverwriteMode() const;
     void setOverwriteMode( bool b );
-    // TODO: As this method can be implemented in terms of KTextEditor
-    // methods, it should be dropped.
+
     QString currentTextLine()
         { return getDoc()->textLine( cursorLine() ); }
     QString currentWord()
         { return m_doc->getWord( m_viewInternal->getCursor() ); }
-//    QString word(int x, int y)
-//        { return m_doc->getWord( x, y ); }
-    // TODO: As this method can be implemented in terms of KTextEditor
-    // methods, it should be dropped.
     void insertText( const QString& text )
         { getDoc()->insertText( cursorLine(), cursorColumnReal(), text ); }
     bool canDiscard();
-    int tabWidth()                { return m_doc->tabChars;      }
-    void setTabWidth( int w )     { m_doc->setTabWidth(w);       }
+    int tabWidth()                { return m_doc->config()->tabWidth(); }
+    void setTabWidth( int w )     { m_doc->config()->setTabWidth(w);  }
     void setEncoding( QString e ) { m_doc->setEncoding(e);       }
     bool isLastView()             { return m_doc->isLastView(1); }
 
@@ -157,14 +192,33 @@ class KateView : public Kate::View,
     void flush();
     saveResult save();
     saveResult saveAs();
-    
-    void indent()             { m_doc->indent( cursorLine() );       }
-    void unIndent()           { m_doc->unIndent( cursorLine() );     }
-    void cleanIndent()        { m_doc->cleanIndent( cursorLine() );  }
-    void comment()            { m_doc->comment( cursorLine() );      }
-    void uncomment()          { m_doc->unComment( cursorLine() );    }
+
+    void indent()             { m_doc->indent( this, cursorLine(), 1 );       }
+    void unIndent()           { m_doc->indent( this, cursorLine(), -1 );     }
+    void cleanIndent()        { m_doc->indent( this, cursorLine(), 0 );  }
+    void comment()            { m_doc->comment( this, cursorLine(), 1 );      }
+    void uncomment()          { m_doc->comment( this, cursorLine(), -1 );    }
     void killLine()           { m_doc->removeLine( cursorLine() );     }
-    
+
+    /**
+      Uppercases selected text, or an alphabetic character next to the cursor.
+    */
+    void uppercase() { m_doc->transform( this, m_viewInternal->cursor, KateDocument::Uppercase ); }
+    /**
+      Lowercases selected text, or an alphabetic character next to the cursor.
+    */
+    void lowercase() { m_doc->transform( this, m_viewInternal->cursor, KateDocument::Lowercase ); }
+    /**
+      Capitalizes the selection (makes each word start with an uppercase) or
+      the word under the cursor.
+    */
+    void capitalize() { m_doc->transform( this, m_viewInternal->cursor, KateDocument::Capitalize ); }
+    /**
+      Joins lines touched by the selection
+    */
+    void joinLines();
+
+
     void keyReturn()          { m_viewInternal->doReturn();          }
     void backspace()          { m_viewInternal->doBackspace();       }
     void deleteWordLeft()     { m_viewInternal->doDeleteWordLeft();  }
@@ -190,7 +244,9 @@ class KateView : public Kate::View,
     void scrollUp()           { m_viewInternal->scrollUp();          }
     void scrollDown()         { m_viewInternal->scrollDown();        }
     void topOfView()          { m_viewInternal->topOfView();         }
+    void shiftTopOfView()     { m_viewInternal->topOfView(true);     }
     void bottomOfView()       { m_viewInternal->bottomOfView();      }
+    void shiftBottomOfView()  { m_viewInternal->bottomOfView(true);  }
     void pageUp()             { m_viewInternal->pageUp();            }
     void shiftPageUp()        { m_viewInternal->pageUp(true);        }
     void pageDown()           { m_viewInternal->pageDown();          }
@@ -208,39 +264,43 @@ class KateView : public Kate::View,
   // config file / session management functions
   public:
     void readSessionConfig(KConfig *);
-    void writeSessionConfig(KConfig *);  
+    void writeSessionConfig(KConfig *);
 
   public slots:
-    int getEol()                  { return m_doc->eolMode; }
+    int getEol();
     void setEol( int eol );
-    void find()                   { m_search->find();            }
-    void replace()                { m_search->replace();         }
-    void findAgain( bool back )   { m_search->findAgain( back ); }
+    void find();
+    void replace();
+    void findAgain( bool back );
     void findAgain()              { findAgain( false );          }
     void findPrev()               { findAgain( true );           }
-    void slotEditCommand();
 
     void setFoldingMarkersOn( bool enable ); // Not in Kate::View, but should be
     void setIconBorder( bool enable );
     void setLineNumbersOn( bool enable );
+    void showCmdLine ( bool enable );
     void toggleFoldingMarkers();
     void toggleIconBorder();
     void toggleLineNumbersOn();
     void toggleDynWordWrap ();
+    void toggleCmdLine ();
+    void setDynWrapIndicators(int mode);
 
   public:
+    KateRenderer *renderer ();
+
     bool iconBorder();
     bool lineNumbersOn();
+    int dynWrapIndicators();
     bool foldingMarkersOn();
     Kate::Document* getDoc()    { return m_doc; }
-    
+
     void setActive( bool b )    { m_active = b; }
     bool isActive()             { return m_active; }
-    
+
   public slots:
-    void slotIncFontSizes();
-    void slotDecFontSizes();
-    void gotoMark( KTextEditor::Mark* mark ) { setCursorPositionReal( mark->line, 0 ); }
+    void gotoMark( KTextEditor::Mark* mark ) { setCursorPositionInternal ( mark->line, 0, 1 ); }
+    void selectionChanged ();
 
   signals:
     void gotFocus( Kate::View* );
@@ -255,34 +315,41 @@ class KateView : public Kate::View,
     KateDocument*  doc() const       { return m_doc; }
 
     KActionCollection* editActionCollection() const { return m_editActions; }
-    
+
   public slots:
     void slotNewUndo();
     void slotUpdate();
     void toggleInsert();
     void reloadFile();
+    void toggleWWMarker();
+    void toggleWriteLock();
+    void switchToCmdLine ();
+    void slotReadWriteChanged ();
 
   signals:
     void dropEventPass(QDropEvent*);
     void viewStatusMsg (const QString &msg);
-    
+
+  public:
+    bool setCursorPositionInternal( uint line, uint col, uint tabwidth = 1, bool calledExternally = false );
+
   protected:
-    void customEvent( QCustomEvent* );
     void contextMenuEvent( QContextMenuEvent* );
     bool checkOverwrite( KURL );
 
+  public slots:
+    void slotSelectionTypeChanged();
+
   private slots:
     void slotGotFocus();
     void slotLostFocus();
     void slotDropEventPass( QDropEvent* ev );
     void slotSetEncoding( const QString& descriptiveName );
-    void updateFoldingMarkersAction();
     void slotStatusMsg();
-    void slotSelectionTypeChanged();
     void slotSaveCanceled( const QString& error );
-    
-  public:
-    void updateViewDefaults ();
+    void slotExpandToplevel();
+    void slotCollapseLocal();
+    void slotExpandLocal();
 
   private:
     void setupConnections();
@@ -290,33 +357,78 @@ class KateView : public Kate::View,
     void setupEditActions();
     void setupCodeFolding();
     void setupCodeCompletion();
-    void setupViewPlugins();
-    
-    bool setCursorPositionInternal( uint line, uint col, uint tabwidth );
 
     KActionCollection*     m_editActions;
     KAction*               m_editUndo;
     KAction*               m_editRedo;
-    KRecentFilesAction*    m_fileRecent; 
+    KRecentFilesAction*    m_fileRecent;
     KToggleAction*         m_toggleFoldingMarkers;
     KToggleAction*         m_toggleIconBar;
     KToggleAction*         m_toggleLineNumbers;
     KToggleAction*         m_toggleDynWrap;
+    KSelectAction*         m_setDynWrapIndicators;
+    KToggleAction*         m_toggleWWMarker;
+    KToggleAction*         m_toggleCmdLine;
+
     KSelectAction*         m_setEndOfLine;
+
     KSelectAction*         m_setEncoding;
     Kate::ActionMenu*      m_setHighlight;
+    Kate::ActionMenu*      m_setFileType;
+    KToggleAction*         m_toggleWriteLock;
+    KateViewSchemaAction*  m_schemaMenu;
+
+    KAction *m_cut;
+    KAction *m_copy;
+    KAction *m_paste;
+    KAction *m_selectAll;
+    KAction *m_deSelect;
+
+    KToggleAction *m_toggleBlockSelection;
+    KToggleAction *m_toggleInsert;
 
     KateDocument*          m_doc;
     KateViewInternal*      m_viewInternal;
+    KateRenderer*          m_renderer;
     KateSearch*            m_search;
     KateBookmarks*         m_bookmarks;
-    QPopupMenu*            m_rmbMenu;
+    QGuardedPtr<QPopupMenu>  m_rmbMenu;
     KateCodeCompletion*    m_codeCompletion;
-    
+
+    KateCmdLine *m_cmdLine;
+    bool m_cmdLineOn;
+
     QGridLayout *m_grid;
 
     bool       m_active;
     bool       m_hasWrap;
+
+    private slots:
+        void slotNeedTextHint(int line, int col, QString &text);
+
+        void slotClipboardDataChanged();
+
+        void slotHlChanged();
+  /**
+   * Configuration
+   */
+  public:
+    inline KateViewConfig *config () { return m_config; };
+
+    void updateConfig ();
+
+    void updateDocumentConfig();
+
+    void updateRendererConfig();
+
+  private slots:
+    void updateFoldingConfig ();
+    void toggleBlockSelectionMode ();
+
+  private:
+    KateViewConfig *m_config;
+    bool m_startingUp;
+    bool m_updatingDocumentConfig;
 };
 
 #endif
