@@ -25,20 +25,27 @@
 #define _KATE_VIEW_INTERNAL_
 
 #include "katecursor.h"
-#include "katedocument.h"
+#include "katesupercursor.h"
+#include "katelinerange.h"
+#include "katetextline.h"
 
-#include <qpoint.h>
-#include <qlayout.h>
 #include <qscrollbar.h>
+#include <qpoint.h>
+#include <qtimer.h>
 
 class KateView;
+class KateDocument;
 class KateIconBorder;
 
-  enum Bias {
+class QHBoxLayout;
+class QVBoxLayout;
+
+enum Bias
+{
     left  = -1,
     none  =  0,
     right =  1
-  };
+};
 
 /**
  * This class is required because QScrollBar's sliderMoved() signal is
@@ -49,62 +56,93 @@ class KateScrollBar : public QScrollBar
 {
   Q_OBJECT
 
-public:
-  KateScrollBar(Orientation orientation, QWidget* parent, const char* name = 0L);
+  public:
+    KateScrollBar(Orientation orientation, QWidget* parent, const char* name = 0L);
 
-signals:
-  void sliderMMBMoved(int value);
+  signals:
+    void sliderMMBMoved(int value);
 
-protected:
-  virtual void mousePressEvent(QMouseEvent* e);
-  virtual void mouseReleaseEvent(QMouseEvent* e);
+  protected:
+    virtual void mousePressEvent(QMouseEvent* e);
+    virtual void mouseReleaseEvent(QMouseEvent* e);
 
-protected slots:
-  void sliderMaybeMoved(int value);
+  protected slots:
+    void sliderMaybeMoved(int value);
 
-private:
+  private:
   bool m_middleMouseDown;
 };
 
 class KateViewInternal : public QWidget
 {
     Q_OBJECT
-    friend class KateDocument;
-    friend class KateUndoGroup;
-    friend class KateUndo;
+
     friend class KateView;
     friend class KateIconBorder;
+    friend class CalculatingCursor;
+    friend class BoundedCursor;
+    friend class WrappingCursor;
 
   public:
     KateViewInternal ( KateView *view, KateDocument *doc );
     ~KateViewInternal ();
-    
+
+  // BEGIN EDIT STUFF
+  public:
+    void editStart ();
+    void editEnd (int editTagLineStart, int editTagLineEnd, bool tagFrom);
+
+    void editSetCursor (const KateTextCursor &cursor);
+
+  private:
+    uint editSessionNumber;
+    bool editIsRunning;
+    KateTextCursor editOldCursor;
+  // END
+
+  // BEGIN TAG & CLEAR & UPDATE STUFF
+  public:
+    bool tagLine (const KateTextCursor& virtualCursor);
+
+    bool tagLines (int start, int end, bool realLines = false);
+    bool tagLines (KateTextCursor start, KateTextCursor end, bool realCursors = false);
+
+    void tagAll ();
+
+    void clear ();
+  // END
+
+  private:
+    void updateView (bool changed = false, int viewLinesScrolled = 0);
+    void makeVisible (const KateTextCursor& c, uint endCol, bool force = false, bool center = false, bool calledExternally = false);
+
   public:
     inline const KateTextCursor& startPos() const { return m_startPos; }
-    inline uint startLine () const { return m_startPos.line; }
+    inline uint startLine () const { return m_startPos.line(); }
     inline uint startX () const { return m_startX; }
-  
+
     KateTextCursor endPos () const;
     uint endLine () const;
 
-    LineRange yToLineRange ( uint y ) const { return lineRanges[y / m_doc->viewFont.fontHeight]; };
+    LineRange yToLineRange(uint y) const;
 
     void prepareForDynWrapChange();
     void dynWrapChanged();
-             
+
+  public slots:
+    void slotIncFontSizes();
+    void slotDecFontSizes();
+
   private slots:
-    void scrollLines(int line); // connected to the sliderMoved of the m_lineScroll  
+    void scrollLines(int line); // connected to the sliderMoved of the m_lineScroll
     void scrollViewLines(int offset);
     void scrollNextPage();
     void scrollPrevPage();
     void scrollPrevLine();
     void scrollNextLine();
     void scrollColumns (int x); // connected to the valueChanged of the m_columnScroll
-  
-  //public slots:
-    void updateView (bool changed = false, int viewLinesScrolled = 0);
-    void makeVisible (const KateTextCursor& c, uint endCol, bool force = false);
-    
+    void docSelectionChanged ();
+
   public:
     void doReturn();
     void doDelete();
@@ -113,7 +151,7 @@ class KateViewInternal : public QWidget
     void doTranspose();
     void doDeleteWordLeft();
     void doDeleteWordRight();
-    
+
     void cursorLeft(bool sel=false);
     void cursorRight(bool sel=false);
     void wordLeft(bool sel=false);
@@ -134,30 +172,11 @@ class KateViewInternal : public QWidget
     void top_home(bool sel=false);
     void bottom_end(bool sel=false);
 
-    void clear();
-    
     inline const KateTextCursor& getCursor() { return cursor; }
     QPoint cursorCoordinates();
-    
-    void setTagLinesFrom(int line);
 
-    void editStart();
-    void editEnd(int editTagLineStart, int editTagLineEnd);
-
-    void editRemoveText(int line, int col, int len);
-    void removeSelectedText(KateTextCursor & start);
-
-    /**
-       Set the tagLinesFrom member if usefull. 
-    */
-    void setViewTagLinesFrom(int line);
-
-    void editWrapLine(int line, int col, int len);
-    void editUnWrapLine(int line, int col);
-    void editRemoveLine(int line);
-    
     void paintText (int x, int y, int width, int height, bool paintOnlyDirty = false);
- 
+
   // EVENT HANDLING STUFF - IMPORTANT
   protected:
     void paintEvent(QPaintEvent *e);
@@ -169,24 +188,25 @@ class KateViewInternal : public QWidget
     void mouseDoubleClickEvent( QMouseEvent* );
     void mouseReleaseEvent(     QMouseEvent* );
     void mouseMoveEvent(        QMouseEvent* );
-    void timerEvent( QTimerEvent* );
     void dragEnterEvent( QDragEnterEvent* );
     void dragMoveEvent( QDragMoveEvent* );
-    void dropEvent( QDropEvent* );   
+    void dropEvent( QDropEvent* );
     void showEvent ( QShowEvent *);
     void wheelEvent(QWheelEvent* e);
     void focusInEvent (QFocusEvent *);
     void focusOutEvent (QFocusEvent *);
-    
+
+    void imStartEvent( QIMEvent *e );
+    void imComposeEvent( QIMEvent *e );
+    void imEndEvent( QIMEvent *e );
+
   private slots:
-    void tripleClickTimeout();  
-    
+    void tripleClickTimeout();
+
   signals:
     // emitted when KateViewInternal is not handling its own URI drops
     void dropEventPass(QDropEvent*);
-    
-    
-    
+
   private slots:
     void slotRegionVisibilityChangedAt(unsigned int);
     void slotRegionBeginEndAddedRemoved(unsigned int);
@@ -197,21 +217,16 @@ class KateViewInternal : public QWidget
     void moveWord( Bias bias, bool sel );
     void moveEdge( Bias bias, bool sel );
     KateTextCursor maxStartPos(bool changed = false);
-    void scrollPos(KateTextCursor& c, bool force = false);
+    void scrollPos(KateTextCursor& c, bool force = false, bool calledExternally = false);
     void scrollLines( int lines, bool sel );
-    
+
     uint linesDisplayed() const;
 
-    inline int lineToY ( uint viewLine ) const { return (viewLine-startLine()) * m_doc->viewFont.fontHeight; }
+    int lineToY(uint viewLine) const;
 
     void updateSelection( const KateTextCursor&, bool keepSel );
-    void updateCursor( const KateTextCursor& newCursor );
+    void updateCursor( const KateTextCursor& newCursor, bool force = false, bool center = false, bool calledExternally = false );
     void updateBracketMarks();
-    
-    bool tagLine(const KateTextCursor& virtualCursor);
-    bool tagLines(int start, int end, bool realLines = false );
-    bool tagLines(KateTextCursor start, KateTextCursor end, bool realCursors = false);
-    void tagAll();
 
     void paintCursor();
 
@@ -223,33 +238,21 @@ class KateViewInternal : public QWidget
     KateView *m_view;
     KateDocument* m_doc;
     class KateIconBorder *leftBorder;
-    
+
     int mouseX;
     int mouseY;
     int scrollX;
     int scrollY;
-    int scrollTimer;
 
-    KateTextCursor cursor;
+    KateSuperCursor cursor;
     KateTextCursor displayCursor;
-    bool cursorOn;
-    int cursorTimer;
     int cXPos;
 
     bool possibleTripleClick;
 
-    // for use from doc: tag lines from here (if larger than -1)
-    int tagLinesFrom;
+    // Bracket mark
+    KateTextRange bm;
 
-    //
-    // cursor cache for document
-    // here stores the document the view's cursor pos while editing before update
-    //
-    KateTextCursor cursorCache;
-    bool cursorCacheChanged;
-
-    BracketMark bm;
-    
     enum DragState { diNone, diPending, diDragging };
 
     struct _dragInfo {
@@ -259,23 +262,29 @@ class KateViewInternal : public QWidget
     } dragInfo;
 
     uint iconBorderHeight;
-    
+
     //
     // line scrollbar + first visible (virtual) line in the current view
     //
     QScrollBar *m_lineScroll;
     QWidget* m_dummy;
     QVBoxLayout* m_lineLayout;
-    
+    QHBoxLayout* m_colLayout;
+
     // These are now cursors to account for word-wrap.
     KateTextCursor m_startPos;
     KateTextCursor m_oldStartPos;
-    
+
     // This is set to false on resize or scroll (other than that called by makeVisible),
     // so that makeVisible is again called when a key is pressed and the cursor is in the same spot
     bool m_madeVisible;
     bool m_shiftKeyPressed;
-    
+
+    // How many lines to should be kept visible above/below the cursor when possible
+    void setAutoCenterLines(int viewLines, bool updateView = true);
+    int m_autoCenterLines;
+    int m_minLinesVisible;
+
     //
     // column scrollbar + x position
     //
@@ -283,59 +292,63 @@ class KateViewInternal : public QWidget
     bool m_columnScrollDisplayed;
     int m_startX;
     int m_oldStartX;
-    
+
     // has selection changed while your mouse or shift key is pressed
-    bool m_selChangedByUser; 
-    
+    bool m_selChangedByUser;
+    KateTextCursor selectAnchor;
+
     //
     // lines Ranges, mostly useful to speedup + dyn. word wrap
     //
     QMemArray<LineRange> lineRanges;
-    
+
     // Used to determine if the scrollbar will appear/disappear in non-wrapped mode
     bool scrollbarVisible(uint startLine);
     int maxLen(uint startLine);
-    
+
     // returns the maximum X value / col value a cursor can take for a specific line range
     int lineMaxCursorX(const LineRange& range);
     int lineMaxCol(const LineRange& range);
-    
+
     // get the values for a specific range.
     // specify lastLine to get the next line of a range.
     LineRange range(int realLine, const LineRange* previous = 0L);
-    
+
     LineRange currentRange();
     LineRange previousRange();
     LineRange nextRange();
-    
+
     // Finds the lineRange currently occupied by the cursor.
     LineRange range(const KateTextCursor& realCursor);
-    
+
     // Returns the lineRange of the specified realLine + viewLine.
     LineRange range(uint realLine, int viewLine);
-    
+
     // find the view line of cursor c (0 = same line, 1 = down one, etc.)
     uint viewLine(const KateTextCursor& realCursor);
-    
+
     // find the view line of the cursor, relative to the display (0 = top line of view, 1 = second line, etc.)
     // if limitToVisible is true, only lines which are currently visible will be searched for, and -1 returned if the line is not visible.
     int displayViewLine(const KateTextCursor& virtualCursor, bool limitToVisible = false);
-    
+
     // find the index of the last view line for a specific line
     uint lastViewLine(uint realLine);
-    
+
     // count the number of view lines for a real line
     uint viewLineCount(uint realLine);
-    
+
     // find the cursor offset by (offset) view lines from a cursor.
     // when keepX is true, the column position will be calculated based on the x
     // position of the specified cursor.
     KateTextCursor viewLineOffset(const KateTextCursor& virtualCursor, int offset, bool keepX = false);
-    
+
     // These variable holds the most recent maximum real & visible column number
     bool m_preserveMaxX;
     int m_currentMaxX;
-    
+
+    bool m_usePlainLines; // accept non-highlighted lines if this is set
+    TextLine::Ptr textLine(int realLine);
+
     bool m_updatingView;
     int m_wrapChangeViewLine;
     KateTextCursor m_cachedMaxStartPos;
@@ -346,14 +359,37 @@ class KateViewInternal : public QWidget
     void stopDragScroll();
 
   private:
-    // Timer for drag & scroll
+    // Timers
     QTimer m_dragScrollTimer;
+    QTimer m_scrollTimer;
+    QTimer m_cursorTimer;
+    QTimer m_textHintTimer;
 
     static const int scrollTime = 30;
     static const int scrollMargin = 16;
-    
+
     // needed to stop the column scroll bar from hiding / unhiding during a dragScroll.
     bool m_suppressColumnScrollBar;
+
+  private slots:
+    void scrollTimeout ();
+    void cursorTimeout ();
+    void textHintTimeout ();
+
+  //TextHint
+ public:
+   void enableTextHints(int timeout);
+   void disableTextHints();
+
+ private:
+   bool m_textHintEnabled;
+   int m_textHintTimeout;
+   int m_textHintMouseX;
+   int m_textHintMouseY;
+
+   int m_imPreeditStartLine;
+   int m_imPreeditStart;
+   int m_imPreeditLength;
 };
 
 #endif
