@@ -1,6 +1,6 @@
 /* This file is part of the KDE libraries
    Copyright (C) 2002 Anders Lund <anders@alweb.dk>
-   
+
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License version 2 as published by the Free Software Foundation.
@@ -14,8 +14,6 @@
    along with this library; see the file COPYING.LIB.  If not, write to
    the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
    Boston, MA 02111-1307, USA.
-
-   $Id: insertfileplugin.cpp,v 1.9.2.1 2003/07/15 10:42:16 mueller Exp $
 */
 
 #include "insertfileplugin.h"
@@ -88,18 +86,17 @@ void InsertFilePluginView::slotInsertFile()
                                              i18n("Choose File to Insert") ).url();
   if ( _file.isEmpty() ) return;
 
-  if ( KURL( _file ).isLocalFile() ) {
-    _tmpfile = _file;
+  if ( _file.isLocalFile() ) {
+    _tmpfile = _file.path();
     insertFile();
   }
   else {
-    KURL url( _file );
     KTempFile tempFile( QString::null );
     _tmpfile = tempFile.name();
 
     KURL destURL;
     destURL.setPath( _tmpfile );
-    _job = KIO::file_copy( url, destURL, 0600, true, false, true );
+    _job = KIO::file_copy( _file, destURL, 0600, true, false, true );
     connect( _job, SIGNAL( result( KIO::Job * ) ), this, SLOT( slotFinished ( KIO::Job * ) ) );
   }
 }
@@ -109,7 +106,7 @@ void InsertFilePluginView::slotFinished( KIO::Job *job )
   assert( job == _job );
   _job = 0;
   if ( job->error() )
-    KMessageBox::error( (QWidget*)parent(), i18n("Failed to load file:\n\n") + job->errorString(), i18n("Insert File error") );
+    KMessageBox::error( (QWidget*)parent(), i18n("Failed to load file:\n\n") + job->errorString(), i18n("Insert File Error") );
   else
     insertFile();
 }
@@ -118,16 +115,16 @@ void InsertFilePluginView::insertFile()
 {
   QString error;
   if ( _tmpfile.isEmpty() )
-    error = i18n("<p>The file <strong>%1</strong> is empty, aborting.").arg(_file);
+    return;
 
   QFileInfo fi;
   fi.setFile( _tmpfile );
   if (!fi.exists() || !fi.isReadable())
-    error = i18n("<p>The file <strong>%1</strong> does not exist or is not readable, aborting.").arg(_file);
+    error = i18n("<p>The file <strong>%1</strong> does not exist or is not readable, aborting.").arg(_file.fileName());
 
   QFile f( _tmpfile );
-  if ( ! f.open(IO_ReadOnly) )
-    error = i18n("<p>Unable to open file <strong>%1</strong>, aborting.").arg(_file);
+  if ( !f.open(IO_ReadOnly) )
+    error = i18n("<p>Unable to open file <strong>%1</strong>, aborting.").arg(_file.fileName());
 
   if ( ! error.isEmpty() ) {
     KMessageBox::sorry( (QWidget*)parent(), error, i18n("Insert file error") );
@@ -150,7 +147,7 @@ void InsertFilePluginView::insertFile()
   f.close();
 
   if ( str.isEmpty() )
-    error = i18n("<p>File <strong>%1</strong> had no contents.").arg(_file);
+    error = i18n("<p>File <strong>%1</strong> had no contents.").arg(_file.fileName());
   if ( ! error.isEmpty() ) {
     KMessageBox::sorry( (QWidget*)parent(), error, i18n("Insert file error") );
     return;
@@ -170,7 +167,7 @@ void InsertFilePluginView::insertFile()
   ci->setCursorPositionReal( line + numlines - 1, numlines > 1 ? len : col + len  );
 
   // clean up
-  _file.truncate( 0 );
+  _file = KURL ();
   _tmpfile.truncate( 0 );
   v = 0;
   ei = 0;
