@@ -80,6 +80,7 @@ public:
         {};
     const char *translatorName;
     const char *translatorEmail;
+    const char *productName;
 };
 
 
@@ -104,6 +105,7 @@ KAboutData::KAboutData( const char *appName,
   mBugEmailAddress( bugsEmailAddress )
 {
    d = new KAboutDataPrivate;
+   d->productName = 0;
 
    if( appName ) {
      const char *p = strrchr(appName, '/');
@@ -155,6 +157,11 @@ KAboutData::setLicenseTextFile( const QString &file )
   mLicenseKey = License_File;
 }
 
+void
+KAboutData::setProductName( const char *productName )
+{
+  d->productName = productName;
+}
 
 const char *
 KAboutData::appName() const
@@ -162,6 +169,15 @@ KAboutData::appName() const
    return mAppName;
 }
 
+const char *
+KAboutData::productName() const
+{
+   if (d->productName)
+      return d->productName;
+   else
+      return appName();
+}
+
 QString
 KAboutData::programName() const
 {
@@ -282,6 +298,7 @@ KAboutData::otherText() const
 QString
 KAboutData::license() const
 {
+  QString result = copyrightStatement() + "\n\n";
   QString l;
   QString f;
   switch ( mLicenseKey )
@@ -314,14 +331,14 @@ KAboutData::license() const
           return( i18n(mLicenseText) );
        // fall through
     default:
-       return i18n("No licensing terms for this program have been specified.\n"
+       result += i18n("No licensing terms for this program have been specified.\n"
                    "Please check the documentation or the source for any\n"
                    "licensing terms.\n");
-      }
+       return result;
+  }
 
-  QString result;
   if (!l.isEmpty())
-     result = i18n("This program is distributed under the terms of the %1.").arg( l );
+     result += i18n("This program is distributed under the terms of the %1.").arg( l );
 
   if (!f.isEmpty())
   {
