@@ -33,7 +33,7 @@ class KAboutDataPrivate;
  * This structure is used to store information about a person or developer.
  * It can store the person's name, a task, an email address and a
  * link to a home page. This class is intended for use in the
- * @ref KAboutData class, but it can be used elsewhere as well.
+ * KAboutData class, but it can be used elsewhere as well.
  * Normally you should at least define the person's name.
 */
 class KAboutPerson
@@ -61,7 +61,8 @@ public:
       mWebAddress = webAddress;
     }
     /**
-     * @internal - don't use. Required by QValueList
+     * @internal
+     * Don't use. Required by QValueList
      */
     KAboutPerson() {}
 
@@ -106,12 +107,12 @@ class KAboutTranslatorPrivate;
 /**
  * This structure is used to store information about a translator.
  * It can store the translator's name and an email address.
- * This class is intended for use in the @ref KAboutData class,
+ * This class is intended for use in the KAboutData class,
  * but it can be used elsewhere as well.
  * Normally you should at least define the translator's name.
  *
- * It's not possible to use @ref KAboutPerson for this, because
- * @ref KAboutPerson stores internally only const char* pointers, but the
+ * It's not possible to use KAboutPerson for this, because
+ * KAboutPerson stores internally only const char* pointers, but the
  * translator information is generated dynamically from the translation
  * of a dummy string.
 */
@@ -153,16 +154,16 @@ private:
  * This class is used to store information about a program. It can store
  * such values as version number, program name, home page, email address
  * for bug reporting, multiple authors and contributors
- * (using @ref KAboutPerson), license and copyright information.
+ * (using KAboutPerson), license and copyright information.
  *
  * Currently, the values set here are shown by the "About" box
- * (see @ref KAboutDialog), used by the bug report dialog (see @ref KBugReport),
- * and by the help shown on command line (see @ref KCmdLineArgs).
+ * (see KAboutDialog), used by the bug report dialog (see KBugReport),
+ * and by the help shown on command line (see KCmdLineArgs).
  *
  * @short Holds information needed by the "About" box and other
  * classes.
  * @author Espen Sand (espen@kde.org), David Faure (faure@kde.org)
- * @version $Id: kaboutdata.h,v 1.26 2002/11/14 18:13:28 lukas Exp $
+ * @version $Id: kaboutdata.h,v 1.35 2003/11/24 22:19:37 tokoe Exp $
  */
 class KAboutData
 {
@@ -200,12 +201,12 @@ class KAboutData
      *        This string should be marked for translation.
      *        Example: I18N_NOOP("A simple text editor.")
      *
-     * @param licenseType The license identifier. Use @ref setLicenseText if
+     * @param licenseType The license identifier. Use setLicenseText if
      *        you use a license not predefined here.
      *
      * @param copyrightStatement A copyright statement, that can look like this:
      *        "(c) 1999-2000, Name". The string specified here is not modified
-     *        in any manner. The author information from @ref addAuthor is not
+     *        in any manner. The author information from addAuthor is not
      *        used.
      *
      * @param text Some free form text, that can contain any kind of
@@ -286,10 +287,10 @@ class KAboutData
      * on the language, just use a dummy text marked for translation.
      *
      * For example:
-     * <pre>
-     * setTranslator(I18N_NOOP("_: NAME OF TRANSLATORS\nYour names")
-     * ,I18N_NOOP("_: EMAIL OF TRANSLATORS\nYour emails"));
-     * </pre>
+     * \code
+     * setTranslator(I18N_NOOP("_: NAME OF TRANSLATORS\\nYour names")
+     * ,I18N_NOOP("_: EMAIL OF TRANSLATORS\\nYour emails"));
+     * \endcode
      *
      * The translator can then translate this dummy text with his name
      * or with a list of names separated with ",".
@@ -308,15 +309,15 @@ class KAboutData
     void setTranslator(const char* name, const char* emailAddress);
 
     /**
-     * Defines a licence text.
+     * Defines a license text.
      *
      * The text will be translated if it got marked for
      * translations with the I18N_NOOP() macro.
      *
      * Example:
-     * <pre>
+     * \code
      * setLicenseText( I18N_NOOP("This is my license"));
-     * </pre>
+     * \endcode
      *
      * NOTE: No copy of the text is made.
      *
@@ -325,18 +326,36 @@ class KAboutData
     void setLicenseText( const char *license );
 
     /**
-     * Defines a licence text.
+     * Defines a license text.
      *
      * @param file File containing the license text.
      */
     void setLicenseTextFile( const QString &file );
 
+    /**
+     * Defines the product name wich will be used in the KBugReport dialog.
+     * By default it's the appName, but you can overwrite it here to provide
+     * support for special components e.g. 'product/component' like
+     * 'kontact/summary'.
+     *
+     * @param name The name of product
+     */
+    void setProductName( const char *name );
+
     /**
      * Returns the application's internal name.
      * @return the internal program name.
      */
     const char *appName() const;
 
+    /**
+     * Returns the application's product name, which will be used in KBugReport
+     * dialog. By default it returns appName(), otherwise the one which is set
+     * with setProductName()
+     * 
+     * @return the product name.
+     */
+    const char *productName() const;
     /**
      * Returns the translated program name.
      * @return the program name (translated).
@@ -401,7 +420,7 @@ class KAboutData
 
     /**
      * Returns the license. If the licenseType argument of the constructor has been
-     * used, any text defined by @ref setLicenseText is ignored,
+     * used, any text defined by setLicenseText is ignored,
      * and the standard text for the chosen license will be returned.
      *
      * @return The license text.
@@ -412,8 +431,7 @@ class KAboutData
      * Returns the copyright statement.
      * @return the copyright statement. Can be QString::null if not set.
      */
-    QString copyrightStatement() const { return( i18n(mCopyrightStatement )); }
-
+    QString copyrightStatement() const;
 
   private:
     const char *mAppName;
