@@ -18,13 +18,14 @@
     Boston, MA 02111-1307, USA.
 */
 
-// $Id: kapplication.h,v 1.278.2.1 2003/03/03 14:39:42 lunakl Exp $
+// $Id: kapplication.h,v 1.306.2.1 2004/04/08 09:56:29 lunakl Exp $
 
 #ifndef _KAPP_H
 #define _KAPP_H
 
 // Version macros. Never put this further down.
 #include "kdeversion.h"
+#include "kdemacros.h"
 
 class KConfig;
 class KCharsets;
@@ -55,18 +56,22 @@ class KApplicationPrivate;
 * Controls and provides information to all KDE applications.
 *
 * Only one object of this class can be instantiated in a single app.
-* This instance is always accessible via the @ref kApplication() method.
+* This instance is always accessible via the kApplication() method.
 *
 * This class provides the following services to all KDE applications.
 *
-* @li It controls the event queue (see @ref QApplication ).
+* @li It controls the event queue (see QApplication ).
 * @li It provides the application with KDE resources such as
-* accelerators, common menu entries, a @ref KConfig object. session
+* accelerators, common menu entries, a KConfig object. session
 * management events, help invocation etc.
 * @li Installs a signal handler for the SIGCHLD signal in order to
 * avoid zombie children. If you want to catch this signal yourself or
 * don't want it to be caught at all, you have set a new signal handler
 * (or SIG_IGN) after KApplication's constructor has run.
+* @li Installs an empty signal handler for the SIGPIPE signal using
+* installSigpipeHandler(). If you want to catch this signal
+* yourself, you have set a new signal handler after KApplication's
+* constructor has run.
 * @li It can start new services
 *
 *
@@ -83,10 +88,10 @@ class KApplicationPrivate;
 * @li X-DCOP-ServiceType=Unique
 *    Only start the service if it isn't already running,
 *    wait until the service has registered with dcop.
-* 
+*
 * @short Controls and provides information to all KDE applications.
 * @author Matthias Kalle Dalheimer <kalle@kde.org>
-* @version $Id: kapplication.h,v 1.278.2.1 2003/03/03 14:39:42 lunakl Exp $
+* @version $Id: kapplication.h,v 1.306.2.1 2004/04/08 09:56:29 lunakl Exp $
 */
 class KApplication : public QApplication, public KInstance
 {
@@ -97,13 +102,13 @@ public:
 
   /**
    * This constructor takes aboutData and command line
-   *  arguments from @ref KCmdLineArgs.
+   *  arguments from KCmdLineArgs.
    *
    * @param allowStyles Set to false to disable the loading on plugin based
    * styles. This is only useful to applications that do not display a GUI
    * normally. If you do create an application with @p allowStyles set to false
    * it normally runs in the background but under special circumstances
-   * displays widgets.  Call @ref enableStyles() before displaying any widgets.
+   * displays widgets.  Call enableStyles() before displaying any widgets.
    *
    * @param GUIenabled Set to false to disable all GUI stuff. This implies
    * no styles either.
@@ -118,6 +123,10 @@ public:
    * @param display Will be passed to Qt as the X display. The display must be valid and already
    * opened.
    *
+   * @param argc command line argument count
+   *
+   * @param argv command line argument value(s)
+   *
    * @param rAppName application name. Will be used for finding the
    * associated message files and icon files, and as the default
    * registration name for DCOP. This is a mandatory parameter.
@@ -126,7 +135,7 @@ public:
    * styles. This is only useful to applications that do not display a GUI
    * normally. If you do create an application with @p allowStyles set to false
    * that normally runs in the background but under special circumstances
-   * displays widgets call @ref enableStyles() before displaying any widgets.
+   * displays widgets call enableStyles() before displaying any widgets.
    *
    * @param GUIenabled Set to false to disable all GUI stuff. This implies
    * no styles either.
@@ -136,12 +145,14 @@ public:
 #endif
 
   /**
-   * @deprecated
-   * // REMOVE FOR KDE 4.0 - using it only gives crashing applications because
-   * // KCmdLineArgs::init isn't called
+   * @deprecated do not use it at all, it will make your application crash, use KCmdineArgs
    *
    * Constructor. Parses command-line arguments.
    *
+   * @param argc command line argument count
+   *
+   * @param argv command line argument value(s)
+   *
    * @param rAppName application name. Will be used for finding the
    * associated message files and icon files, and as the default
    * registration name for DCOP. This is a mandatory parameter.
@@ -150,13 +161,15 @@ public:
    * styles. This is only useful to applications that do not display a GUI
    * normally. If you do create an application with @p allowStyles set to false
    * that normally runs in the background but under special circumstances
-   * displays widgets call @ref enableStyles() before displaying any widgets.
+   * displays widgets call enableStyles() before displaying any widgets.
    *
    * @param GUIenabled Set to false to disable all GUI stuff. This implies
    * no styles either.
    */
-  KApplication(int& argc, char** argv,
-              const QCString& rAppName, bool allowStyles=true, bool GUIenabled=true);
+  // REMOVE FOR KDE 4.0 - using it only gives crashing applications because
+  // KCmdLineArgs::init isn't called
+ KApplication(int& argc, char** argv,
+              const QCString& rAppName, bool allowStyles=true, bool GUIenabled=true) KDE_DEPRECATED;
 
   /**
     * Add Qt and KDE command line options to KCmdLineArgs.
@@ -168,7 +181,7 @@ public:
   /**
    * Returns the current application object.
    *
-   * This is similar to the global @ref QApplication pointer qApp. It
+   * This is similar to the global QApplication pointer qApp. It
    * allows access to the single global KApplication object, since
    * more than one cannot be created in the same application. It
    * saves you the trouble of having to pass the pointer explicitly
@@ -181,7 +194,7 @@ public:
    * Returns the application session config object.
    *
    * @return A pointer to the application's instance specific
-   * @ref KConfig object.
+   * KConfig object.
    * @see KConfig
    */
   KConfig* sessionConfig();
@@ -191,8 +204,8 @@ public:
    *
    * @return If true, this application was restored by the session manager.
    *    Note that this may mean the config object returned by
-   * @ref sessionConfig() contains data saved by a session closedown.
-   * @see #sessionConfig()
+   * sessionConfig() contains data saved by a session closedown.
+   * @see sessionConfig()
    */
   bool isRestored() const { return QApplication::isSessionRestored(); }
 
@@ -204,6 +217,14 @@ public:
    */
   void disableSessionManagement();
 
+  /**
+   * Enables again session management for this application, formerly
+   * disabled by calling disableSessionManagement(). You usually
+   * shouldn't call this function, as the session management is enabled
+   * by default.
+   */
+  void enableSessionManagement();
+
   /**
    * The possible values for the @p confirm parameter of requestShutDown().
    */
@@ -280,10 +301,10 @@ public:
    * will work with any session manager compliant with it.
    *
    * @param confirm Whether to ask the user if he really wants to log out.
-   *   @ref ShutdownConfirm
-   * @param sdtype The action to take after logging out. @ref ShutdownType
-   * @param sdmode If/When the action should be taken. @ref ShutdownMode
-   * @return true on sucess, false if the session manager could not be
+   * ShutdownConfirm
+   * @param sdtype The action to take after logging out. ShutdownType
+   * @param sdmode If/When the action should be taken. ShutdownMode
+   * @return true on success, false if the session manager could not be
    * contacted.
    */
   bool requestShutDown( ShutdownConfirm confirm = ShutdownConfirmDefault,
@@ -307,7 +328,7 @@ public:
 
     /*
      * Reimplemented for internal purposes, mainly the highlevel
-     *  handling of session management with @ref KSessionManaged.
+     *  handling of session management with KSessionManaged.
      * @internal
      */
   void commitData( QSessionManager& sm );
@@ -318,7 +339,7 @@ public:
      * @internal
      */
   void saveState( QSessionManager& sm );
-  
+
   /**
    * Returns true if the application is currently saving its session
    * data (most probably before KDE logout). This is intended for use
@@ -331,7 +352,7 @@ public:
   bool sessionSaving() const;
 
   /**
-   * Returns a pointer to a @ref DCOPClient for the application.
+   * Returns a pointer to a DCOPClient for the application.
    * If a client does not exist yet, it is created when this
    * function is called.
    * @return the DCOPClient for the application
@@ -345,7 +366,7 @@ public:
   static void disableAutoDcopRegistration();
 
   /**
-   * Returns a @ref QPixmap with the application icon.
+   * Returns a QPixmap with the application icon.
    * @return the application icon
    */
   QPixmap icon() const;
@@ -357,7 +378,7 @@ public:
   QString iconName() const;
 
   /**
-   * Returns the mini-icon for the application as a @ref QPixmap.
+   * Returns the mini-icon for the application as a QPixmap.
    * @return the application's mini icon
    */
   QPixmap miniIcon() const;
@@ -372,7 +393,7 @@ public:
    *  Sets the top widget of the application.
    *  This means basically applying the right window caption and
    *  icon. An application may have several top widgets. You don't
-   *  need to call this function manually when using @ref KMainWindow.
+   *  need to call this function manually when using KMainWindow.
    *
    *  @param topWidget A top widget of the application.
    *
@@ -389,7 +410,14 @@ public:
    * @param appname     This allows you to show the help of another
    *                    application. If empty the current name() is
    *                    used
+   * @param startup_id for app startup notification, "0" for none,
+   *           "" ( empty string ) is the default
    */
+  void invokeHelp( const QString& anchor,
+                   const QString& appname,
+                   const QCString& startup_id ) const;
+
+  // KDE4 merge with above with startup_id = ""
   void invokeHelp( const QString& anchor = QString::null,
                    const QString& appname = QString::null ) const;
 
@@ -407,21 +435,29 @@ public:
    *                   value will be appended to the filename,
    *                   prefixed with a "#" (hash) character.
    */
-  void invokeHTMLHelp( const QString& aFilename, const QString& aTopic = QString::null ) const;
+  void invokeHTMLHelp( const QString& aFilename, const QString& aTopic = QString::null ) const KDE_DEPRECATED;
 
   /**
    * Convenience method; invokes the standard email application.
    *
    * @param address The destination address
    * @param subject Subject string. Can be QString::null.
+   * @param startup_id for app startup notification, "0" for none,
+   *           "" ( empty string ) is the default
    */
+  void invokeMailer( const QString &address, const QString &subject, const QCString& startup_id );
+  // KDE4 merge with above with startup_id = ""
   void invokeMailer( const QString &address, const QString &subject );
 
   /**
    * Invokes the standard email application.
    *
    * @param mailtoURL A mailto URL.
+   * @param startup_id for app startup notification, "0" for none,
+   *           "" ( empty string ) is the default
    */
+  void invokeMailer( const KURL &mailtoURL, const QCString& startup_id );
+  // KDE4 merge with above with startup_id = ""
   void invokeMailer( const KURL &mailtoURL );
 
   /**
@@ -436,7 +472,14 @@ public:
    * @param body        A string containing the body of the mail (exclusive with messageFile)
    * @param messageFile A file (URL) containing the body of the mail (exclusive with body) - currently unsupported
    * @param attachURLs  List of URLs to be attached to the mail.
+   * @param startup_id for app startup notification, "0" for none,
+   *           "" ( empty string ) is the default
    */
+  void invokeMailer(const QString &to, const QString &cc, const QString &bcc,
+                    const QString &subject, const QString &body,
+                    const QString &messageFile, const QStringList &attachURLs,
+                    const QCString& startup_id );
+  // KDE4 merge with above with startup_id = ""
   void invokeMailer(const QString &to, const QString &cc, const QString &bcc,
                     const QString &subject, const QString &body,
                     const QString &messageFile = QString::null, const QStringList &attachURLs = QStringList());
@@ -449,9 +492,76 @@ public slots:
    * or anything else than directory or HTML, prefer to use new KRun( url ).
    *
    * @param url The destination address
+   * @param startup_id for app startup notification, "0" for none,
+   *           "" ( empty string ) is the default
    */
+  void invokeBrowser( const QString &url, const QCString& startup_id );
+  // KDE4 merge with above with startup_id = ""
   void invokeBrowser( const QString &url );
 
+  /**
+   * If the widget with focus provides a cut() slot, call that slot.  Thus for a
+   * simple application cut can be implemented as:
+   * \code
+   * KStdAction::cut( kapp, SLOT( cut() ), actionCollection() );
+   * \endcode
+   */
+  void cut();
+
+  /**
+   * If the widget with focus provides a copy() slot, call that slot.  Thus for a
+   * simple application copy can be implemented as:
+   * \code
+   * KStdAction::copy( kapp, SLOT( copy() ), actionCollection() );
+   * \endcode
+   */
+  void copy();
+
+  /**
+   * If the widget with focus provides a paste() slot, call that slot.  Thus for a
+   * simple application copy can be implemented as:
+   * \code
+   * KStdAction::paste( kapp, SLOT( paste() ), actionCollection() );
+   * \endcode
+   */
+  void paste();
+
+  /**
+   * If the widget with focus provides a clear() slot, call that slot.  Thus for a
+   * simple application clear() can be implemented as:
+   * \code
+   * new KAction( i18n( "Clear" ), "editclear", 0, kapp, SLOT( clear() ), actionCollection(), "clear" );
+   * \endcode
+   *
+   * Note that for some widgets, this may not provide the intended bahavior.  For
+   * example if you make use of the code above and a KListView has the focus, clear()
+   * will clear all of the items in the list.  If this is not the intened behavior
+   * and you want to make use of this slot, you can subclass KListView and reimplement
+   * this slot.  For example the following code would implement a KListView without this
+   * behavior:
+   *
+   * \code
+   * class MyListView : public KListView {
+   *   Q_OBJECT
+   * public:
+   *   MyListView( QWidget * parent = 0, const char * name = 0, WFlags f = 0 ) : KListView( parent, name, f ) {}
+   *   virtual ~MyListView() {}
+   * public slots:
+   *   virtual void clear() {}
+   * };
+   * \endcode
+   */
+  void clear();
+
+  /**
+   * If the widget with focus provides a selectAll() slot, call that slot.  Thus for a
+   * simple application select all can be implemented as:
+   * \code
+   * KStdAction::selectAll( kapp, SLOT( selectAll() ), actionCollection() );
+   * \endcode
+   */
+  void selectAll();
+
 public:
   /**
    * Returns the DCOP name of the service launcher. This will be something like
@@ -464,12 +574,12 @@ public:
    * Starts a service based on the (translated) name of the service.
    * E.g. "Web Browser"
    *
-   * @param name the name of the service
+   * @param _name the name of the service
    * @param URL if not empty this URL is passed to the service
    * @param error On failure, @p error contains a description of the error
    *         that occurred. If the pointer is 0, the argument will be
    *         ignored
-   * @param dcopService On success, @p dcopService contains the DCOP name 
+   * @param dcopService On success, @p dcopService contains the DCOP name
    *         under which this service is available. If empty, the service does
    *         not provide DCOP services. If the pointer is 0 the argument
    *         will be ignored
@@ -487,12 +597,12 @@ public:
    * Starts a service based on the (translated) name of the service.
    * E.g. "Web Browser"
    *
-   * @param name the name of the service
+   * @param _name the name of the service
    * @param URLs if not empty these URLs will be passed to the service
    * @param error On failure, @p error contains a description of the error
    *         that occurred. If the pointer is 0, the argument will be
    *         ignored
-   * @param dcopService On success, @p dcopService contains the DCOP name 
+   * @param dcopService On success, @p dcopService contains the DCOP name
    *         under which this service is available. If empty, the service does
    *         not provide DCOP services. If the pointer is 0 the argument
    *         will be ignored
@@ -510,12 +620,12 @@ public:
    * Starts a service based on the desktop path of the service.
    * E.g. "Applications/konqueror.desktop" or "/home/user/bla/myfile.desktop"
    *
-   * @param name the path of the desktop file
+   * @param _name the path of the desktop file
    * @param URL if not empty this URL is passed to the service
    * @param error On failure, @p error contains a description of the error
    *         that occurred. If the pointer is 0, the argument will be
    *         ignored
-   * @param dcopService On success, @p dcopService contains the DCOP name 
+   * @param dcopService On success, @p dcopService contains the DCOP name
    *         under which this service is available. If empty, the service does
    *         not provide DCOP services. If the pointer is 0 the argument
    *         will be ignored
@@ -533,12 +643,12 @@ public:
    * Starts a service based on the desktop path of the service.
    * E.g. "Applications/konqueror.desktop" or "/home/user/bla/myfile.desktop"
    *
-   * @param name the path of the desktop file
+   * @param _name the path of the desktop file
    * @param URLs if not empty these URLs will be passed to the service
    * @param error On failure, @p error contains a description of the error
    *         that occurred. If the pointer is 0, the argument will be
    *         ignored
-   * @param dcopService On success, @p dcopService contains the DCOP name 
+   * @param dcopService On success, @p dcopService contains the DCOP name
    *         under which this service is available. If empty, the service does
    *         not provide DCOP services. If the pointer is 0 the argument
    *         will be ignored
@@ -556,12 +666,12 @@ public:
    * Starts a service based on the desktop name of the service.
    * E.g. "konqueror"
    *
-   * @param name the desktop name of the service
+   * @param _name the desktop name of the service
    * @param URL if not empty this URL is passed to the service
    * @param error On failure, @p error contains a description of the error
    *         that occurred. If the pointer is 0, the argument will be
    *         ignored
-   * @param dcopService On success, @p dcopService contains the DCOP name 
+   * @param dcopService On success, @p dcopService contains the DCOP name
    *         under which this service is available. If empty, the service does
    *         not provide DCOP services. If the pointer is 0 the argument
    *         will be ignored
@@ -579,12 +689,12 @@ public:
    * Starts a service based on the desktop name of the service.
    * E.g. "konqueror"
    *
-   * @param name the desktop name of the service
+   * @param _name the desktop name of the service
    * @param URLs if not empty these URLs will be passed to the service
    * @param error On failure, @p error contains a description of the error
    *         that occurred. If the pointer is 0, the argument will be
    *         ignored
-   * @param dcopService On success, @p dcopService contains the DCOP name 
+   * @param dcopService On success, @p dcopService contains the DCOP name
    *         under which this service is available. If empty, the service does
    *         not provide DCOP services. If the pointer is 0 the argument
    *         will be ignored
@@ -613,15 +723,20 @@ public:
    *         ignored
    * @param pid On success, the process id of the new service will be written
    *        here. If the pointer is 0, the argument will be ignored.
+   * @param startup_id for app startup notification, "0" for none,
+   *           "" ( empty string ) is the default
    * @return an error code indicating success (== 0) or failure (> 0).
    */
+  static int kdeinitExec( const QString& name, const QStringList &args,
+                QString *error, int *pid, const QCString& startup_id );
+  // KDE4 merge with above with startup_id = ""
   static int kdeinitExec( const QString& name, const QStringList &args=QStringList(),
                 QString *error=0, int *pid = 0 );
 
   /**
    * Starts a program via kdeinit and wait for it to finish.
    *
-   * Like @ref kdeinitExec(), but it waits till the program is finished.
+   * Like kdeinitExec(), but it waits till the program is finished.
    * As such it behaves similar to the system(...) function.
    *
    * @param name Name of the program to start
@@ -631,8 +746,13 @@ public:
    *         ignored
    * @param pid On success, the process id of the new service will be written
    *        here. If the pointer is 0, the argument will be ignored.
+   * @param startup_id for app startup notification, "0" for none,
+   *           "" ( empty string ) is the default
    * @return an error code indicating success (== 0) or failure (> 0).
    */
+  static int kdeinitExecWait( const QString& name, const QStringList &args,
+                QString *error, int *pid, const QCString& startup_id );
+  // KDE4 merge with above with startup_id = ""
   static int kdeinitExecWait( const QString& name, const QStringList &args=QStringList(),
                 QString *error=0, int *pid = 0 );
 
@@ -649,11 +769,11 @@ public:
   /**
    * @deprecated
    */
-  KStyle* kstyle() const { return 0; }
+  KDE_DEPRECATED KStyle* kstyle() const { return 0; }
 
   /**
    * Builds a caption that contains the application name along with the
-   * @ref userCaption() using a standard layout.
+   * userCaption using a standard layout.
    *
    * To make a compliant caption
    * for your window, simply do: @p setCaption(kapp->makeStdCaption(yourCaption));
@@ -713,7 +833,7 @@ public:
    *
    * Current style plugins do not get unloaded.
    *
-   * This is only useful when used in combination with @ref enableStyles().
+   * This is only useful when used in combination with enableStyles().
    */
   void disableStyles();
 
@@ -721,13 +841,19 @@ public:
    *  Installs widget filter as global X11 event filter.
    *
    * The widget
-   *  filter receives XEvents in its standard @ref QWidget::x11Event() function.
+   *  filter receives XEvents in its standard QWidget::x11Event() function.
    *
    *  Warning: Only do this when absolutely necessary. An installed X11 filter
    *  can slow things down.
    **/
   void installX11EventFilter( QWidget* filter );
 
+  /**
+   * Removes global X11 event filter previously installed by
+   * installX11EventFilter().
+   */
+  void removeX11EventFilter( const QWidget* filter );
+
   /**
    * Generates a uniform random number.
    * @return A truly unpredictable number in the range [0, RAND_MAX)
@@ -745,7 +871,7 @@ public:
    * Adds a message type to the KIPC event mask. You can only add "system
    * messages" to the event mask. These are the messages with id < 32.
    * Messages with id >= 32 are user messages.
-   * @param id The message id. See @ref KIPC::Message.
+   * @param id The message id. See KIPC::Message.
    * @see KIPC
    * @see removeKipcEventMask()
    * @see kipcMessage()
@@ -753,7 +879,7 @@ public:
   void addKipcEventMask(int id);
 
   /**
-   * Removes a message type from the @ref KIPC event mask. This message will 
+   * Removes a message type from the KIPC event mask. This message will
    * not be handled anymore.
    * @param id The message id.
    * @see KIPC
@@ -763,7 +889,7 @@ public:
   void removeKipcEventMask(int id);
 
   /**
-   * Returns the app startup notification identifier for this running 
+   * Returns the app startup notification identifier for this running
    * application.
    * @return the startup notification identifier
    */
@@ -772,10 +898,25 @@ public:
   /**
    * Sets a new value for the application startup notification window property for newly
    * created toplevel windows.
-   * @param the startup notification identifier
+   * @param startup_id the startup notification identifier
    */
   void setStartupId( const QCString& startup_id );
 
+  /**
+   * Updates the last user action timestamp to the given time, or to the current time,
+   * if 0 is given. Do not use unless you're really sure what you're doing.
+   * Consult focus stealing prevention section in kdebase/kwin/README.
+   * @since 3.2
+   */
+  void updateUserTimestamp( unsigned long time = 0 );
+  
+  /**
+   * Returns the last user action timestamp or 0 if no user activity has taken place yet.
+   * @since 3.2.3
+   * @see updateuserTimestamp
+   */
+  unsigned long userTimestamp() const;
+
     /**
     * Returns the argument to --geometry if any, so the geometry can be set
     * wherever necessary
@@ -800,7 +941,7 @@ public:
    * Returns whether a certain KAction is authorized.
    *
    * @param action The name of a KAction action. The name is prepended
-   * with "action/" before being passed to @ref authorize()
+   * with "action/" before being passed to authorize()
    * @return true if the KAction is authorized
    */
   bool authorizeKAction(const char *action);
@@ -819,7 +960,37 @@ public:
    * @since 3.1
    */
   bool authorizeURLAction(const QString &action, const KURL &baseURL, const KURL &destURL);
-                   
+
+  /**
+   * Allow a certain URL action. This can be useful if your application
+   * needs to ensure access to an application specific directory that may 
+   * otherwise be subject to KIOSK restrictions.
+   * @param action The name of the action.
+   * @param baseURL The url where the action originates from
+   * @param destURL The object of the action
+   * @since 3.2
+   */
+  void allowURLAction(const QString &action, const KURL &_baseURL, const KURL &_destURL);
+
+  /**
+   * Returns whether access to a certain control module is authorized.
+   *
+   * @param menuId identifying the control module, e.g. kde-mouse.desktop
+   * @return true if access to the module is authorized, false otherwise.
+   * @since 3.2
+   */
+  bool authorizeControlModule(const QString &menuId);
+  
+  /**
+   * Returns whether access to a certain control modules is authorized.
+   *
+   * @param menuIds list of menu-ids of control module, 
+   * an example of a menu-id is kde-mouse.desktop.
+   * @return Those control modules for which access has been authorized.
+   * @since 3.2
+   */
+  QStringList authorizeControlModules(const QStringList &menuIds);
+
 
   // Same values as ShiftMask etc. in X.h
   enum { ShiftModifier = 1<<0,
@@ -833,7 +1004,7 @@ public:
   /**
    * Returns the currently pressed keyboard modifiers (e.g. shift, control, etc.)
    * Usually you simply want to test for those in key events, in which case
-   * @ref QKeyEvent::state() does the job (or @ref QKeyEvent::key() to 
+   * QKeyEvent::state() does the job (or QKeyEvent::key() to
    * notice when a modifier is pressed alone).
    * But it can be useful to query for the status of the modifiers at another moment
    * (e.g. some KDE apps do that upon a drop event).
@@ -851,10 +1022,10 @@ public:
   /**
    * Returns the currently pressed mouse buttons.
    * Usually you simply want to test for those in mouse events, in which case
-   * @ref QMouseEvent::button() does the job (or @ref QMouseEvent::state()).
+   * QMouseEvent::button() does the job (or QMouseEvent::state()).
    * But it can be useful to query for the status of the mouse buttons at another moment.
-   * To query for the mouse pointer position, use @ref QCursor::pos().
-   * @return the currently pressed mouse buttons 
+   * To query for the mouse pointer position, use QCursor::pos().
+   * @return the currently pressed mouse buttons
    * @since 3.1
    */
   static uint mouseState();
@@ -870,7 +1041,7 @@ public slots:
   void ref();
 
   /**
-   * Tells KApplication that one operation such as those described in @ref just finished.
+   * Tells KApplication that one operation such as those described in ref() just finished.
    * The application exits if the counter is back to 0.
    */
   void deref();
@@ -896,10 +1067,44 @@ protected:
   static KApplication *KApp;
   int pArgc;
 
+  /**
+   * This method is used internally to determine which edit slots are implemented
+   * by the widget that has the focus, and to invoke those slots if available.
+   *
+   * @param slot is the slot as returned using the SLOT() macro, for example SLOT( cut() )
+   *
+   * This method can be used in KApplication subclasses to implement application wide
+   * edit actions not supported by the KApplication class.  For example (in your subclass):
+   *
+   * \code
+   * void MyApplication::deselect()
+   * {
+   *   invokeEditSlot( SLOT( deselect() ) );
+   * }
+   * \endcode
+   *
+   * Now in your application calls to MyApplication::deselect() will call this slot on the
+   * focused widget if it provides this slot.  You can combine this with KAction with:
+   *
+   * \code
+   * KStdAction::deselect( static_cast<MyApplication *>( kapp ), SLOT( cut() ), actionCollection() );
+   * \endcode
+   *
+   * @see cut()
+   * @see copy()
+   * @see paste()
+   * @see clear()
+   * @see selectAll()
+   *
+   * @since 3.2
+   */
+  void invokeEditSlot( const char *slot );
+
 private slots:
   void dcopFailure(const QString &);
   void dcopBlockUserInput( bool );
   void x11FilterDestroyed();
+  void checkAppStartedSlot();
 
 private:
   QString sessionConfigName() const;
@@ -908,8 +1113,8 @@ private:
   static bool s_dcopClientNeedsPostInit;
   QString aCaption; // the name for the window title
   bool bSessionManagement;
-  QPixmap aIconPixmap;
-  QPixmap aMiniIconPixmap;
+  QPixmap aIconPixmap; // BIC (KDE 4) make a pointer, to avoid creating if noGUI
+  QPixmap aMiniIconPixmap; // BIC (KDE 4) make a pointer, to avoid creating if noGUI
   QString aIconName;
   QString aMiniIconName;
   bool useStyles;
@@ -924,6 +1129,7 @@ private:
   void dcopAutoRegistration();
   void dcopClientPostInit();
   void initUrlActionRestrictions();
+
 public:
   /**
    * @internal
@@ -933,7 +1139,12 @@ public:
   /**
       @internal
     */
-  int xioErrhandler();
+  int xErrhandler( Display*, void* );
+
+  /**
+      @internal
+    */
+  int xioErrhandler( Display* );
 
   /**
    * @internal
@@ -958,7 +1169,7 @@ public:
 
   /**
    * Used to obtain the QPalette that will be used to set the application palette.
-   * 
+   *
    * This is only useful for configuration modules such as krdb and should not be
    * used in normal circumstances.
    * @return the QPalette
@@ -972,6 +1183,23 @@ public:
    */
   static QPalette createApplicationPalette( KConfig *config, int contrast );
 
+  /**
+   * Installs a handler for the SIGPIPE signal. It is thrown when you write to
+   * a pipe or socket that has been closed.
+   * The handler is installed automatically in the constructor, but you may
+   * need it if your application or component does not have a KApplication
+   * instance.
+   */
+  static void installSigpipeHandler();
+
+  /**
+   * @internal
+   * Whether widgets can be used. 
+   *
+   * @since 3.2
+   */
+  static bool guiEnabled();
+
 signals:
   /**
    * Emitted when KApplication has changed its palette due to a KControl request.
@@ -996,6 +1224,10 @@ signals:
    * Normally widgets will update their fonts automatically, but you should
    * connect to this to monitor global font changes, especially if you are
    * using explicit fonts.
+   *
+   * Note: If you derive from a QWidget-based class, a faster method is to
+   *       reimplement QWidget::fontChange(). This is the preferred way
+   *       to get informed about font updates.
    */
   void kdisplayFontChanged();
 
@@ -1047,14 +1279,14 @@ signals:
       Session management asks you to save the state of your application.
 
      This signal is provided for compatibility only. For new
-     applications, simply use KMainWindow. By reimplementing @ref
-     KMainWindow::queryClose(), @ref KMainWindow::saveProperties() and
-     @ref KMainWindow::readProperties() you can simply handle session
+     applications, simply use KMainWindow. By reimplementing 
+     KMainWindow::queryClose(), KMainWindow::saveProperties() and
+ KMainWindow::readProperties() you can simply handle session
      management for applications with multiple toplevel windows.
 
      For purposes without KMainWindow, create an instance of
-     KSessionManaged and reimplement the functions @ref
-     KSessionManaged::commitData() and/or @ref
+     KSessionManaged and reimplement the functions 
+     KSessionManaged::commitData() and/or 
      KSessionManaged::saveState()
 
      If you still want to use this signal, here is what you should do:
@@ -1063,7 +1295,7 @@ signals:
      manipulate the UI in that slot, it is blocked by the session
      manager.
 
-     Use the @ref #sessionConfig() KConfig object to store all your
+     Use the sessionConfig() KConfig object to store all your
      instance specific data.
 
      Do not do any closing at this point! The user may still select
@@ -1089,6 +1321,7 @@ private:
   void kdisplaySetStyle();
   void kdisplaySetFont();
   void applyGUIStyle();
+  static void sigpipeHandler(int);
 
   int captionLayout;
 
@@ -1101,7 +1334,9 @@ private:
 };
 
 
-/** Check, if a file may be accessed in a given mode.
+/**
+ * \relates KGlobal
+ * Check, if a file may be accessed in a given mode.
  * This is a wrapper around the access() system call.
  * checkAccess() calls access() with the given parameters.
  * If this is OK, checkAccess() returns true. If not, and W_OK
@@ -1118,7 +1353,6 @@ private:
  */
 bool checkAccess(const QString& pathname, int mode);
 
-
 class KSessionManagedPrivate;
 
 /**
@@ -1126,13 +1360,13 @@ class KSessionManagedPrivate;
    base.
 
    KSessionManaged makes it possible to provide implementations for
-   @ref QApplication::commitData() and @ref QApplication::saveState(), without
+ QApplication::commitData() and QApplication::saveState(), without
    subclassing KApplication. KMainWindow internally makes use of this.
 
    You don't need to do anything with this class when using
-   KMainWindow. Instead, use @ref KMainWindow::saveProperties(),
-   @ref KMainWindow::readProperties(), @ref KMainWindow::queryClose(),
-   @ref KMainWindow::queryExit() and friends.
+   KMainWindow. Instead, use KMainWindow::saveProperties(),
+ KMainWindow::readProperties(), KMainWindow::queryClose(),
+ KMainWindow::queryExit() and friends.
 
   @short Highlevel access to session management.
   @author Matthias Ettrich <ettrich@kde.org>
@@ -1144,7 +1378,7 @@ public:
   virtual ~KSessionManaged();
 
     /**
-       See @ref QApplication::saveState() for documentation.
+       See QApplication::saveState() for documentation.
 
        This function is just a convenience version to avoid subclassing KApplication.
 
@@ -1154,7 +1388,7 @@ public:
      */
   virtual bool saveState( QSessionManager& sm );
     /**
-       See @ref QApplication::commitData() for documentation.
+       See QApplication::commitData() for documentation.
 
        This function is just a convenience version to avoid subclassing KApplication.
 
