@@ -1,6 +1,6 @@
 /* This file is part of the KDE libraries
     Copyright (C) 1999 Lars Knoll (knoll@kde.org)
-    $Id: kcharsets.cpp,v 1.134.2.3 2003/06/29 21:42:59 deller Exp $
+    $Id: kcharsets.cpp,v 1.144.2.1 2004/03/12 09:18:03 kalass Exp $
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Library General Public
@@ -191,12 +191,12 @@ static struct Builtin
     { "unicode-1-1-utf-7", "utf-7" },
     { "ucs2", "iso-10646-ucs-2" },
     { "iso10646-1", "iso-10646-ucs-2" },
-    { "big5-0", "big5" },
     { "gb18030.2000-1", "gb18030" },
     { "gb18030.2000-0", "gb18030" },
     { "gbk-0", "gbk" },
     { "gb2312", "gbk" },
     { "gb2312.1980-0", "gbk" },
+    { "big5-0", "big5" },
     { "euc-kr", "euckr" },
     { "x-euc-kr", "euckr" },
     { "euc-jp", "eucjp" },
@@ -312,7 +312,7 @@ static struct ConversionHints
 // search an array of items index/data, index is const char*, data is T, find first matching index
 // and return data, or return 0
 template< typename T, typename Data >
-Data kcharsets_array_search( const T* start, const char* entry )
+static Data kcharsets_array_search( const T* start, const char* entry )
 {
     for( const T* pos = start;
          pos->index != 0;
@@ -353,7 +353,7 @@ KCharsets::~KCharsets()
     delete d;
 }
 
-QChar KCharsets::fromEntity(const QString &str) 
+QChar KCharsets::fromEntity(const QString &str)
 {
     QChar res = QChar::null;
 
@@ -377,7 +377,7 @@ QChar KCharsets::fromEntity(const QString &str)
         return res;
     }
 
-    const entity *e = findEntity(str.ascii(), str.length());
+    const entity *e = kde_findEntity(str.ascii(), str.length());
 
     if(!e)
     {
@@ -389,7 +389,7 @@ QChar KCharsets::fromEntity(const QString &str)
     return QChar(e->code);
 }
 
-QChar KCharsets::fromEntity(const QString &str, int &len) 
+QChar KCharsets::fromEntity(const QString &str, int &len)
 {
     // entities are never longer than 8 chars... we start from
     // that length and work backwards...
@@ -405,7 +405,7 @@ QChar KCharsets::fromEntity(const QString &str, int &len)
 }
 
 
-QString KCharsets::toEntity(const QChar &ch) 
+QString KCharsets::toEntity(const QChar &ch)
 {
     QString ent;
     ent.sprintf("&#0x%x;", ch.unicode());
@@ -421,7 +421,7 @@ QString KCharsets::resolveEntities( const QString &input )
     bool scanForSemicolon = false;
 
     for ( ; p < end; ++p ) {
-        QChar ch = *p;
+        const QChar ch = *p;
 
         if ( ch == '&' ) {
             ampersand = p;
@@ -438,15 +438,15 @@ QString KCharsets::resolveEntities( const QString &input )
 
         const QChar *entityBegin = ampersand + 1;
 
-        uint entityLength = p - entityBegin;
+        const uint entityLength = p - entityBegin;
         if ( entityLength == 0 )
             continue;
 
-        QChar entityValue = KCharsets::fromEntity( QConstString( entityBegin, entityLength ).string() );
+        const QChar entityValue = KCharsets::fromEntity( QConstString( entityBegin, entityLength ).string() );
         if ( entityValue.isNull() )
             continue;
 
-        uint ampersandPos = ampersand - text.unicode();
+        const uint ampersandPos = ampersand - text.unicode();
 
         text[ ampersandPos ] = entityValue;
         text.remove( ampersandPos + 1, entityLength + 1 );
@@ -497,9 +497,19 @@ QString KCharsets::languageForEncoding( const QString &encoding )
 
 QString KCharsets::encodingForName( const QString &descriptiveName )
 {
-    int left = descriptiveName.find( "( " );
-    return descriptiveName.mid( left + 2, descriptiveName.find( " )" )
-      - left - 2 );
+    const int left = descriptiveName.findRev( '(' );
+    
+    if (left<0) // No parenthesis, so assume it is a normal encoding name
+	return descriptiveName.stripWhiteSpace();
+    
+    QString name(descriptiveName.mid(left+1));
+    
+    const int right = name.findRev( ')' );
+    
+    if (right<0) 
+        return name;
+
+    return name.left(right).stripWhiteSpace();
 }
 
 QStringList KCharsets::descriptiveEncodingNames()
@@ -508,7 +518,7 @@ QStringList KCharsets::descriptiveEncodingNames()
   QStringList::Iterator it;
   for( it = encodings.begin(); it != encodings.end(); ++it ) {
       QString lang = KGlobal::charsets()->languageForEncoding( *it );
-      *it = lang + " ( " + *it + " )";
+      *it = i18n("Descriptive Encoding Name", "%1 ( %2 )") .arg(lang) .arg(*it);
   }
   encodings.sort();
   return encodings;
@@ -553,7 +563,7 @@ QTextCodec *KCharsets::codecForName(const QString &n, bool &ok) const
     }
 
     // these codecs are built into Qt, but the name given for the codec is different,
-    // so QTextCodec did not recognise it.
+    // so QTextCodec did not recognize it.
     QCString cname = kcharsets_array_search< Builtin, const char* >( builtin, name.data());
 
     if(!cname.isEmpty())
