@@ -1,6 +1,6 @@
 /* This file is part of the KDE libraries
     Copyright (C) 1999 Lars Knoll (knoll@kde.org)
-    $Id: kcharsets.h,v 1.53.2.1 2003/02/09 13:55:28 rnolden Exp $
+    $Id: kcharsets.h,v 1.57 2003/10/06 07:45:39 bhards Exp $
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Library General Public
@@ -39,7 +39,7 @@ class QTextCodec;
  * in languages which use non iso-8859-1 charsets.
  *
  * @author Lars Knoll <knoll@kde.org>
- * @version $Id: kcharsets.h,v 1.53.2.1 2003/02/09 13:55:28 rnolden Exp $
+ * @version $Id: kcharsets.h,v 1.57 2003/10/06 07:45:39 bhards Exp $
  */
 class KCharsets
 {
@@ -47,7 +47,7 @@ class KCharsets
 
 protected:
     /** Protected constructor. If you need the kcharsets object, use
-        @ref KGlobal::charsets() instead.
+ KGlobal::charsets() instead.
     */
     KCharsets();
 
@@ -65,7 +65,7 @@ public:
      *         it returns a default (Latin-1) codec
      */
     QTextCodec *codecForName(const QString &name) const;
-    
+
     /**
      * Tries to find a QTextCodec to convert the given encoding from and to
      * Unicode. If no codec could be found the latin1 codec will be returned an
@@ -84,7 +84,7 @@ public:
     static QChar fromEntity(const QString &str);
     /**
      * Overloaded member function. Tries to find an entity in the
-     * @ref QString str.
+     * QString str.
      * @param str the string containing entified
      * @param len is a return value, that gives the length of the decoded
      * entity.
@@ -123,20 +123,7 @@ public:
     QStringList descriptiveEncodingNames();
 
     /**
-     * Lists all languages.
-     * @return a list of all languages
-     */
-    QStringList languages();
-    
-    /**
-     * Lists all available encodings for language name 'language'.
-     * @param language to language to search for
-     * @return the list of encodings for the language
-     */
-    QStringList encodingsForLanguage( const QString &language );
-    
-    /**
-     * Returns the language the encoding is used for. 
+     * Returns the language the encoding is used for.
      * @param encoding the encoding for the language
      * @return the language of the encoding
      */
@@ -144,11 +131,11 @@ public:
 
     /**
      * Returns the encoding for a string obtained with descriptiveEncodingNames().
-     * @param the descriptive name for the encoding
+     * @param descriptiveName the descriptive name for the encoding
      * @return the name of the encoding
      */
     QString encodingForName( const QString &descriptiveName );
-	    
+
 private:
     KCharsetsPrivate *d;
 };
