@@ -118,6 +118,15 @@ bool KCmdLineArgs::ignoreUnknown = false;
 // Static functions
 //
 
+void
+KCmdLineArgs::init(int _argc, char **_argv, const char *_appname, const char* programName,
+                   const char *_description, const char *_version, bool noKApp)
+{
+   init(_argc, _argv,
+        new KAboutData(_appname, programName, _version, _description),
+        noKApp);
+}
+
 void
 KCmdLineArgs::init(int _argc, char **_argv, const char *_appname,
                    const char *_description, const char *_version, bool noKApp)
@@ -138,7 +147,9 @@ KCmdLineArgs::initIgnore(int _argc, char **_argv, const char *_appname )
 void
 KCmdLineArgs::init(const KAboutData* ab)
 {
-   init(0,0,ab, true);
+   char **_argv = (char **) malloc(sizeof(char *));
+   _argv[0] = (char *) ab->appName();
+   init(1,_argv,ab, true);
 }
 
 
@@ -170,7 +181,7 @@ KCmdLineArgs::init(int _argc, char **_argv, const KAboutData *_about, bool noKAp
 
    about = _about;
    parsed = false;
-   mCwd = mCwdd.setObject(new char [PATH_MAX+1], true);
+   mCwd = mCwdd.setObject(mCwd, new char [PATH_MAX+1], true);
    getcwd(mCwd, PATH_MAX);
    if (!noKApp)
       KApplication::addCmdLineOptions();
@@ -240,6 +251,7 @@ KCmdLineArgs::saveAppArgs( QDataStream &ds)
    KCmdLineArgs *args;
    for(args = argsList->first(); args; args = argsList->next())
    {
+      ds << QCString(args->id);
       args->save(ds);
    }
 }
@@ -251,9 +263,11 @@ KCmdLineArgs::loadAppArgs( QDataStream &ds)
    removeArgs("qt");
    removeArgs("kde");
 
+   if (ds.atEnd())
+      return;
+
    KCmdLineArgs *args;
-   if (argsList)
-   {
+   if ( argsList ) {
       for(args = argsList->first(); args; args = argsList->next())
       {
          args->clear();
@@ -262,28 +276,27 @@ KCmdLineArgs::loadAppArgs( QDataStream &ds)
 
    QCString qCwd;
    ds >> qCwd;
-   if (mCwd)
-      delete [] mCwd;
+   delete [] mCwd;
 
-   mCwd = mCwdd.setObject(new char[qCwd.length()+1], true);
+   mCwd = mCwdd.setObject(mCwd, new char[qCwd.length()+1], true);
    strncpy(mCwd, qCwd.data(), qCwd.length()+1);
 
    uint count;
    ds >> count;
 
-   if (count == 0)
-      return;
-
-   if (!argsList || (count != argsList->count()))
+   while(count--)
    {
-      fprintf(stderr, "loadAppArgs:: Unexpected number of command line sets "
-                      "(%d instead of %d)\n", count, argsList ? argsList->count() : 0);
-      return;
-   }
-
-   for(args = argsList->first(); args; args = argsList->next())
-   {
-      args->load(ds);
+     QCString id;
+     ds >> id;
+     assert( argsList );
+     for(args = argsList->first(); args; args = argsList->next())
+     {
+       if (args->id  == id)
+       {
+          args->load(ds);
+          break;
+       }
+     }
    }
 }
 
@@ -306,7 +319,7 @@ KCmdLineArgs *KCmdLineArgs::parsedArgs(const char *id)
 #ifndef NDEBUG
       fprintf(stderr, "WARNING (KCmdLineArgs):\n");
       fprintf(stderr, "Application requests for parsedArgs(\"%s\") without a prior call\n", id?id:"null");
-      fprintf(stderr, "to addCmdLineOptions( ..., \"%s\")\n\n", id?id:"null");
+      fprintf(stderr, "to addCmdLineOptions( ..., \"%s\"), or after a reset()\n\n", id?id:"null");
 #endif
    }
    return args;
@@ -351,6 +364,12 @@ findOption(const KCmdLineOptions *options, QCString &opt,
       result = 0;
       inverse = false;
       opt_name = options->name;
+      if ((opt_name[0] == ':') || (opt_name[0] == 0))
+      {
+         options++;
+         continue;
+      }
+
       if (opt_name[0] == '!')
       {
          opt_name++;
@@ -430,7 +449,7 @@ KCmdLineArgs::findOption(const char *_opt, QCString opt, int &i, bool _enabled,
    }
    if (!args && (_opt[0] == '-') && _opt[1] && (_opt[1] != '-'))
    {
-      // Option not found check if it is a valid option 
+      // Option not found check if it is a valid option
       // in the style of -Pprinter1 or ps -aux
       int p = 1;
       while (true)
@@ -451,7 +470,7 @@ KCmdLineArgs::findOption(const char *_opt, QCString opt, int &i, bool _enabled,
          p++;
          if (result == 1) // Single option
          {
-            args->setOption(singleCharOption, enabled); 
+            args->setOption(singleCharOption, enabled);
             if (_opt[p])
                continue; // Next option
             else
@@ -591,7 +610,10 @@ KCmdLineArgs::parseAllArgs()
 		 if ( !authors.isEmpty() ) {
                      QString authorlist;
 		     for (QValueList<KAboutPerson>::ConstIterator it = authors.begin(); it != authors.end(); ++it ) {
-			 authorlist += QString("    ") + (*it).name() + " <" + (*it).emailAddress() + ">\n";
+			 QString email;
+			 if ( !(*it).emailAddress().isEmpty() )
+				 email = " <" + (*it).emailAddress() + ">";
+			 authorlist += QString("    ") + (*it).name() + email + "\n";
 		     }
 		     printQ( i18n("the 2nd argument is a list of name+address, one on each line","%1 was written by\n%2").arg ( QString(about->programName()) ).arg( authorlist ) );
 		 }
@@ -600,10 +622,10 @@ KCmdLineArgs::parseAllArgs()
 	     }
 	     if (!about->bugAddress().isEmpty())
 	     {
-	        QString bugReporting = i18n( "Please use http://bugs.kde.org to report bugs, do not mail the authors directly.\n" );
-		if (about->bugAddress() != "submit@bugs.kde.org")
-		    bugReporting.replace("http://bugs.kde.org", about->bugAddress());
-		printQ(bugReporting);
+		if (about->bugAddress() == "submit@bugs.kde.org")
+		    printQ( i18n( "Please use http://bugs.kde.org to report bugs, do not mail the authors directly.\n" ) );
+		else
+		    printQ( i18n( "Please use %1 to report bugs, do not mail the authors directly.\n" ).arg(about->bugAddress()) );
 	     }
 	     exit(0);
          } else {
@@ -750,7 +772,7 @@ KCmdLineArgs::usage(const char *id)
    {
       if (args->name)
       {
-         usage = QString(i18n("[%1-options]")).arg(args->name)+" "+usage;
+         usage = i18n("[%1-options]").arg(args->name)+" "+usage;
       }
       args = argsList->prev();
    }
@@ -813,6 +835,12 @@ KCmdLineArgs::usage(const char *id)
    {
      bool hasArgs = false;
      bool hasOptions = false;
+     QString optionsHeader;
+     if (args->name)
+        optionsHeader = optionHeaderString.arg(i18n("%1 options").arg(QString::fromLatin1(args->name)));
+     else
+        optionsHeader = i18n("\nOptions:\n");
+
      while (args)
      {
        const KCmdLineOptions *option = args->options;
@@ -823,6 +851,36 @@ KCmdLineArgs::usage(const char *id)
          QString description;
          QString descriptionRest;
          QStringList dl;
+
+         // Option header
+         if (option->name[0] == ':')
+         {
+            if (option->description)
+            {
+               optionsHeader = "\n"+i18n(option->description);
+               if (!optionsHeader.endsWith("\n"))
+                  optionsHeader.append("\n");
+               hasOptions = false;
+            }
+            option++;
+            continue;
+         }
+
+         // Free-form comment
+         if (option->name[0] == 0)
+         {
+            if (option->description)
+            {
+               QString tmp = "\n"+i18n(option->description);
+               if (!tmp.endsWith("\n"))
+                  tmp.append("\n");
+               printQ(tmp);
+            }
+            option++;
+            continue;
+         }
+
+         // Options
          if (option->description)
          {
             description = i18n(option->description);
@@ -852,10 +910,7 @@ KCmdLineArgs::usage(const char *id)
          {
             if (!hasOptions)
             {
-               if (!args->name)
-                  printQ(i18n("\nOptions:\n"));
-               else if (args->name)
-                  printQ(optionHeaderString.arg(i18n("%1 options").arg(QString::fromLatin1(args->name))));
+               printQ(optionsHeader);
                hasOptions = true;
             }
 
@@ -885,7 +940,7 @@ KCmdLineArgs::usage(const char *id)
          }
          for(QStringList::Iterator it = dl.begin();
              it != dl.end();
-             it++)
+             ++it)
          {
             printQ(optionFormatString.arg("", -25).arg(*it));
          }
@@ -927,14 +982,7 @@ KCmdLineArgs::~KCmdLineArgs()
   delete parsedOptionList;
   delete parsedArgList;
   if (argsList)
-  {
      argsList->removeRef(this);
-     if (argsList->count() == 0)
-     {
-        delete argsList;
-        argsList = 0;
-     }
-  }
 }
 
 void
@@ -946,6 +994,18 @@ KCmdLineArgs::clear()
    parsedOptionList = 0;
 }
 
+void
+KCmdLineArgs::reset()
+{
+   if ( argsList ) {
+      argsList->setAutoDelete( true );
+      argsList->clear();
+      delete argsList;
+      argsList = 0;
+   }
+   parsed = false;
+}
+
 void
 KCmdLineArgs::save( QDataStream &ds) const
 {
