@@ -1,20 +1,20 @@
 /* This file is part of the KDE libraries
-    Copyright (C) 1999,2000 Carsten Pfeiffer <pfeiffer@kde.org>
+   Copyright (C) 1999,2000 Carsten Pfeiffer <pfeiffer@kde.org>
 
-    This library is free software; you can redistribute it and/or
-    modify it under the terms of the GNU Library General Public
-    License as published by the Free Software Foundation; either
-    version 2 of the License, or (at your option) any later version.
+   This library is free software; you can redistribute it and/or
+   modify it under the terms of the GNU Library General Public
+   License as published by the Free Software Foundation; either
+   version 2 of the License, or (at your option) any later version.
 
-    This library is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-    Library General Public License for more details.
+   This library is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+   Library General Public License for more details.
 
-    You should have received a copy of the GNU Library General Public License
-    along with this library; see the file COPYING.LIB.  If not, write to
-    the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-    Boston, MA 02111-1307, USA.
+   You should have received a copy of the GNU Library General Public License
+   along with this library; see the file COPYING.LIB.  If not, write to
+   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+   Boston, MA 02111-1307, USA.
 */
 
 #ifndef KCOMPLETION_H
@@ -27,6 +27,7 @@
 #include <qstringlist.h>
 #include <qguardedptr.h>
 
+#include "kdemacros.h"
 #include <kglobalsettings.h>
 #include <ksortablevaluelist.h>
 #include <kshortcut.h>
@@ -39,22 +40,24 @@ class KCompletionMatches;
 class QPopupMenu;
 
 /**
+ * @short A generic class for completing QStrings
+ *
  * This class offers easy use of "auto-completion", "manual-completion" or
  * "shell completion" on QString objects. A common use is completing filenames
- * or URLs (see @ref KURLCompletion()).
+ * or URLs (see KURLCompletion()).
  * But it is not limited to URL-completion -- everything should be completable!
  * The user should be able to complete email-addresses, telephone-numbers,
  * commands, SQL queries, ...
  * Every time your program knows what the user can type into an edit-field, you
  * should offer completion. With KCompletion, this is very easy, and if you are
- * using a line edit widget (@ref KLineEdit), it is even more easy.
+ * using a line edit widget ( KLineEdit), it is even more easy.
  * Basically, you tell a KCompletion object what strings should be completable
- * and whenever completion should be invoked, you call @ref makeCompletion().
+ * and whenever completion should be invoked, you call makeCompletion().
  * KLineEdit and (an editable) KComboBox even do this automatically for you.
  *
- * KCompletion offers the completed string via the signal @ref #match() and
+ * KCompletion offers the completed string via the signal match() and
  * all matching strings (when the result is ambiguous) via the method
- * @ref allMatches().
+ * allMatches().
  *
  * Notice: auto-completion, shell completion and manual completion work
  *         slightly differently:
@@ -62,7 +65,7 @@ class QPopupMenu;
  * @li auto-completion always returns a complete item as match.
  *     When more than one matching items are available, it will deliver just
  *     the first (depending on sorting order) item. Iterating over all matches
- *     is possible via @ref nextMatch() and @ref previousMatch().
+ *     is possible via nextMatch() and previousMatch().
  *
  * @li popup-completion works in the same way, the only difference being that
  *     the completed items are not put into the edit-widget, but into a
@@ -74,23 +77,23 @@ class QPopupMenu;
  *     of manual and auto-completion is therefore only visible in UI classes,
  *     KCompletion needs to know whether to deliver partial matches
  *     (shell completion) or whole matches (auto/manual completion), therefore
- *     @ref KGlobalSettings::CompletionMan and
- *     @ref KGlobalSettings::CompletionAuto have the exact same effect in
+ * KGlobalSettings::CompletionMan and
+ * KGlobalSettings::CompletionAuto have the exact same effect in
  *     KCompletion.
  *
  * @li shell completion works like how shells complete filenames:
  *     when multiple matches are available, the longest possible string of all
  *     matches is returned (i.e. only a partial item).
  *     Iterating over all matching items (complete, not partial) is possible
- *     via @ref nextMatch() and @ref previousMatch().
+ *     via nextMatch() and previousMatch().
  *
  * You don't have to worry much about that though, KCompletion handles
- * that for you, according to the setting @ref setCompletionMode().
+ * that for you, according to the setting setCompletionMode().
  * The default setting is globally configured by the user and read
- * from @ref KGlobalSettings::completionMode().
+ * from KGlobalSettings::completionMode().
  *
  * A short example:
- * <pre>
+ * \code
  * KCompletion completion;
  * completion.setOrder( KCompletion::Sorted );
  * completion.addItem( "pfeiffer@kde.org" );
@@ -99,13 +102,14 @@ class QPopupMenu;
  * completion.addItem( "carp@cs.tu-berlin.de" );
  *
  * cout << completion.makeCompletion( "ca" ).latin1() << endl;
- * </pre>
+ * \endcode
+ *
  * In shell-completion-mode, this will be "carp"; in auto-completion-
- * mode it will be "carp@cs.tu-berlin.de", as that is alphabetically
+ * mode it will be "carp\@cs.tu-berlin.de", as that is alphabetically
  * smaller.
- * If setOrder was set to Insertion, "carpdjih@sp.zrz.tu-berlin.de"
+ * If setOrder was set to Insertion, "carpdjih\@sp.zrz.tu-berlin.de"
  * would be completed in auto-completion-mode, as that was inserted before
- * "carp@cs.tu-berlin.de".
+ * "carp\@cs.tu-berlin.de".
  *
  * You can dynamically update the completable items by removing and adding them
  * whenever you want.
@@ -117,15 +121,14 @@ class QPopupMenu;
  * Note: KCompletion does not work with strings that contain 0x0 characters
  *       (unicode nul), as this is used internally as a delimiter.
  *
- * You may inherit from KCompletion and override @ref makeCompletion() in
+ * You may inherit from KCompletion and override makeCompletion() in
  * special cases (like reading directories/urls and then supplying the
  * contents to KCompletion, as KURLCompletion does), but generally, this is
  * not necessary.
  *
  *
- * @short A generic class for completing QStrings
  * @author Carsten Pfeiffer <pfeiffer@kde.org>
- * @version $Id: kcompletion.h,v 1.65.2.1 2003/02/09 13:55:28 rnolden Exp $
+ * @version $Id: kcompletion.h,v 1.73 2003/10/06 07:45:39 bhards Exp $
  */
 class KCompletion : public QObject
 {
@@ -141,8 +144,8 @@ public:
      * completion-lookups.
      */
     enum CompOrder { Sorted,    ///< Use order of insertion
-		     Insertion, ///< Use alphabetically sorted order
-		     Weighted   ///< Use weighted order
+                     Insertion, ///< Use alphabetically sorted order
+                     Weighted   ///< Use weighted order
     };
 
     /**
@@ -150,7 +153,7 @@ public:
      */
     KCompletion();
 
-    // FIXME: copy constructor, assignment constructor...
+    // FIXME: copy constructor, assignment operator...
 
     /**
      * Destructor, nothing special here, either.
@@ -164,9 +167,9 @@ public:
      * found.
      *
      * In the latter case, a sound will be issued, depending on
-     * @ref isSoundsEnabled().
+     * isSoundsEnabled().
      * If a match was found, it will also be emitted via the signal
-     * @ref #match().
+     * match().
      *
      * If this is called twice or more often with the same string while no
      * items were added or removed in the meantime, all available completions
@@ -176,8 +179,8 @@ public:
      * @param string the string to complete
      * @return the matching item, or QString::null if there is no matching
      * item.
-     * @see #slotMakeCompletion
-     * @see #substringCompletion
+     * @see slotMakeCompletion
+     * @see substringCompletion
      */
     virtual QString makeCompletion( const QString& string );
 
@@ -187,18 +190,18 @@ public:
      * @return a list of items which all contain @p text as a substring,
      * i.e. not necessarily at the beginning.
      *
-     * @see #makeCompletion
+     * @see makeCompletion
      */
     QStringList substringCompletion( const QString& string ) const;
 
     /**
      * Returns the next item from the matching-items-list.
      * When reaching the beginning, the list is rotated so it will return the
-     * last match and a sound is issued (depending on @ref isSoundsEnabled()).
+     * last match and a sound is issued (depending on isSoundsEnabled()).
      * @return the next item from the matching-items-list.
      * When there is no match, QString::null is returned and
      * a sound is be issued.
-     * @see #slotPreviousMatch
+     * @see slotPreviousMatch
      */
     QString previousMatch();
 
@@ -206,17 +209,17 @@ public:
      * Returns the next item from the matching-items-list.
      * When reaching the last item, the list is rotated, so it will return
      * the first match and a sound is issued (depending on
-     * @ref isSoundsEnabled()).
-     * @return the next item from the matching-items-list.  When there is no 
+     * isSoundsEnabled()).
+     * @return the next item from the matching-items-list.  When there is no
      * match, QString::null is returned and a sound is issued
-     * @see #slotNextMatch
+     * @see slotNextMatch
      */
     QString nextMatch();
 
     /**
      * Returns the last match. Might be useful if you need to check whether
      * a completion is different from the last one.
-     * @return the last match. QString::null is returned when there is no 
+     * @return the last match. QString::null is returned when there is no
      *         last match.
      */
     virtual const QString& lastMatch() const { return myLastMatch; }
@@ -226,18 +229,18 @@ public:
      * if you need to save the state of a KCompletion object and restore it
      * later.
      *
-     * Important note: when @ref order() == Weighted, then every item in the
+     * Important note: when order() == Weighted, then every item in the
      * stringlist has its weight appended, delimited by a colon. E.g. an item
      * "www.kde.org" might look like "www.kde.org:4", where 4 is the weight.
      *
      * This is necessary so that you can save the items along with its
-     * weighting on disk and load them back with @ref setItems(), restoring its
+     * weighting on disk and load them back with setItems(), restoring its
      * weight as well. If you really don't want the appended weightings, call
-     * @ref setOrder( KCompletion::Insertion )
+     * setOrder( KCompletion::Insertion )
      * before calling items().
      *
      * @return a list of all items
-     * @see #setItems
+     * @see setItems
      */
     QStringList items() const;
 
@@ -245,22 +248,22 @@ public:
      * Sets the completion mode to Auto/Manual, Shell or None.
      * If you don't set the mode explicitly, the global default value
      * KGlobalSettings::completionMode() is used.
-     * @ref KGlobalSettings::CompletionNone disables completion.
+     * KGlobalSettings::CompletionNone disables completion.
      * @param mode the completion mode
-     * @see #completionMode
-     * @see #KGlobalSettings::completionMode
+     * @see completionMode
+     * @see KGlobalSettings::completionMode
      */
     virtual void setCompletionMode( KGlobalSettings::Completion mode );
 
     /**
      * Return the current completion mode.
-     * May be different from @ref KGlobalSettings::completionMode(), if you
-     * explicitly called @ref setCompletionMode().
+     * May be different from KGlobalSettings::completionMode(), if you
+     * explicitly called setCompletionMode().
      * @return the current completion mode
-     * @see #setCompletionMode
+     * @see setCompletionMode
      */
     KGlobalSettings::Completion completionMode() const {
-	return myCompletionMode;
+        return myCompletionMode;
     }
 
     /**
@@ -281,23 +284,23 @@ public:
      *
      * Default is insertion order.
      * @param order the new order
-     * @see #order
+     * @see order
      */
     virtual void setOrder( CompOrder order );
 
     /**
      * Returns the completion order.
      * @return the current completion order.
-     * @see #setOrder
+     * @see setOrder
      */
     CompOrder order() const { return myOrder; }
 
     /**
      * Setting this to true makes KCompletion behave case insensitively.
-     * E.g. makeCompletion( "CA" ); might return "carp@cs.tu-berlin.de".
+     * E.g. makeCompletion( "CA" ); might return "carp\@cs.tu-berlin.de".
      * Default is false (case sensitive).
      * @param ignoreCase true to ignore the case
-     * @see #ignoreCase
+     * @see ignoreCase
      */
     virtual void setIgnoreCase( bool ignoreCase );
 
@@ -305,7 +308,7 @@ public:
      * Return whether KCompletion acts case insensitively or not.
      * Default is false (case sensitive).
      * @return true if the case will be ignored
-     * @see #setIgnoreCase
+     * @see setIgnoreCase
      */
     bool ignoreCase() const { return myIgnoreCase; }
 
@@ -313,7 +316,7 @@ public:
      * Returns a list of all items matching the last completed string.
      * Might take some time, when you have LOTS of items.
      * @return a list of all matches for the last completed string.
-     * @see #substringCompletion
+     * @see substringCompletion
      */
     QStringList allMatches();
 
@@ -334,7 +337,7 @@ public:
      * after that in order to have the matches ordered correctly.
      *
      * @return a list of all completion matches
-     * @see #substringCompletion
+     * @see substringCompletion
      */
     KCompletionMatches allWeightedMatches();
 
@@ -347,16 +350,16 @@ public:
 
     /**
      * Enables/disables playing a sound when
-     * @li @ref makeCompletion() can't find a match
+     * @li makeCompletion() can't find a match
      * @li there is a partial completion (= multiple matches in
      *     Shell-completion mode)
-     * @li @ref nextMatch() or @ref previousMatch() hit the last possible
+     * @li nextMatch() or previousMatch() hit the last possible
      *     match -> rotation
      *
-     * For playing the sounds, @ref KNotifyClient() is used.
+     * For playing the sounds, KNotifyClient() is used.
      *
      * @param enable true to enable sounds
-     * @see #isSoundsEnabled
+     * @see isSoundsEnabled
      */
     virtual void setEnableSounds( bool enable ) { myBeep = enable; }
 
@@ -364,64 +367,64 @@ public:
      * Tells you whether KCompletion will play sounds on certain occasions.
      * Default is enabled.
      * @return true if sounds are enabled
-     * @see #enableSounds
-     * @see #disableSounds
+     * @see enableSounds
+     * @see disableSounds
      */
     bool isSoundsEnabled() const { return myBeep; }
 
     /**
      * Returns true when more than one match is found.
      * @return true if there are more than one match
-     * @see #multipleMatches
+     * @see multipleMatches
      */
     bool hasMultipleMatches() const { return myHasMultipleMatches; }
 
 #ifndef KDE_NO_COMPAT
     /**
      * @deprecated
-     * @see #setEnableSounds
+     * @see setEnableSounds
      */
     void enableSounds() { myBeep = true; }
 
     /**
      * @deprecated
-     * @see #setEnableSounds
+     * @see setEnableSounds
      */
     void disableSounds() { myBeep = false; }
 #endif
 
 public slots:
     /**
-     * Attempts to complete "string" and emits the completion via @ref match().
-     * Same as @ref makeCompletion() (just as a slot).
+     * Attempts to complete "string" and emits the completion via match().
+     * Same as makeCompletion() (just as a slot).
      * @param string the string to complete
-     * @see #makeCompletion
+     * @see makeCompletion
      */
     void slotMakeCompletion( const QString& string ) {
-	(void) makeCompletion( string );
+        (void) makeCompletion( string );
     }
 
     /**
-     * Searches the previous matching item and emits it via @ref match().
-     * Same as @ref previousMatch() (just as a slot).
-     * @see #previousMatch
+     * Searches the previous matching item and emits it via match().
+     * Same as previousMatch() (just as a slot).
+     * @see previousMatch
      */
     void slotPreviousMatch() {
-	(void) previousMatch();
+        (void) previousMatch();
     }
 
     /**
-     * Searches the next matching item and emits it via @ref match().
-     * Same as @ref nextMatch() (just as a slot).
-     * @see #nextMatch
+     * Searches the next matching item and emits it via match().
+     * Same as nextMatch() (just as a slot).
+     * @see nextMatch
      */
     void slotNextMatch() {
-	(void) nextMatch();
+        (void) nextMatch();
     }
 
     /**
      * Inserts @p items into the list of possible completions.
-     * Does the same as @ref setItems(), but does not call @ref clear() before.
+     * Does the same as setItems(), but does not call clear() before.
      * @param items the items to insert
      */
     void insertItems( const QStringList& items );
@@ -439,13 +442,13 @@ public slots:
      * before calling setItems().
      *
      * @param list the list of items that are available for completion
-     * @see #items
+     * @see items
      */
     virtual void setItems( const QStringList& list);
 
     /**
      * Adds an item to the list of available completions.
-     * Resets the current item-state (@ref previousMatch() and @ref nextMatch()
+     * Resets the current item-state ( previousMatch() and nextMatch()
      * won't work anymore).
      * @param item the item to add
      */
@@ -453,7 +456,7 @@ public slots:
 
     /**
      * Adds an item to the list of available completions.
-     * Resets the current item-state (@ref previousMatch() and @ref nextMatch()
+     * Resets the current item-state ( previousMatch() and nextMatch()
      * won't work anymore).
      *
      * Sets the weighting of the item to @p weight or adds it to the current
@@ -466,7 +469,7 @@ public slots:
 
     /**
      * Removes an item from the list of available completions.
-     * Resets the current item-state (@ref previousMatch() and @ref nextMatch()
+     * Resets the current item-state ( previousMatch() and nextMatch()
      * won't work anymore).
      * @param item the item to remove
      */
@@ -480,15 +483,15 @@ public slots:
 
 signals:
     /**
-     * The matching item. Will be emitted by @ref makeCompletion(),
-     * @ref previousMatch() or @ref nextMatch(). May be QString::null if there
+     * The matching item. Will be emitted by makeCompletion(),
+     * previousMatch() or nextMatch(). May be QString::null if there
      * is no matching item.
      * @param item the match, or QString::null if there is none
      */
     void match( const QString& item);
 
     /**
-     * All matching items. Will be emitted by @ref makeCompletion() in shell-
+     * All matching items. Will be emitted by makeCompletion() in shell-
      * completion-mode, when the same string is passed to makeCompletion twice
      * or more often.
      * @param matchlist the list of matches
@@ -496,9 +499,9 @@ signals:
     void matches( const QStringList& matchlist);
 
     /**
-     * This signal is emitted, when calling @ref makeCompletion() and more than
+     * This signal is emitted, when calling makeCompletion() and more than
      * one matching item is found.
-     * @see #hasMultipleMatches
+     * @see hasMultipleMatches
      */
     void multipleMatches();
 
@@ -507,26 +510,26 @@ protected:
      * This method is called after a completion is found and before the
      * matching string is emitted. You can override this method to modify the
      * string that will be emitted.
-     * This is necessary e.g. in @ref KURLCompletion(), where files with spaces
+     * This is necessary e.g. in KURLCompletion(), where files with spaces
      * in their names are shown escaped ("filename\ with\ spaces"), but stored
      * unescaped inside KCompletion.
      * Never delete that pointer!
      *
      * Default implementation does nothing.
      * @param match the match to process
-     * @see #postProcessMatches
+     * @see postProcessMatches
      */
     virtual void postProcessMatch( QString *match ) const { Q_UNUSED(match) }
 
     /**
      * This method is called before a list of all available completions is
      * emitted via #matches. You can override this method to modify the
-     * found items before @ref match() or #matches are emitted.
+     * found items before match() or #matches are emitted.
      * Never delete that pointer!
      *
      * Default implementation does nothing.
      * @param matches the matches to process
-     * @see #postProcessMatch
+     * @see postProcessMatch
      */
     virtual void postProcessMatches( QStringList * matches ) const { Q_UNUSED(matches)}
 
@@ -538,16 +541,16 @@ protected:
      *
      * Default implementation does nothing.
      * @param matches the matches to process
-     * @see #postProcessMatch
+     * @see postProcessMatch
      */
     virtual void postProcessMatches( KCompletionMatches * matches ) const {Q_UNUSED(matches)}
 
 private:
-    void 		addWeightedItem( const QString& );
-    QString 		findCompletion( const QString& string );
-    void        	findAllCompletions( const QString&,
-                                            KCompletionMatchesWrapper *matches,
-                                            bool& hasMultipleMatches ) const;
+    void            addWeightedItem( const QString& );
+    QString         findCompletion( const QString& string );
+    void            findAllCompletions( const QString&,
+                                        KCompletionMatchesWrapper *matches,
+                                        bool& hasMultipleMatches ) const;
 
     void extractStringsFromNode( const KCompTreeNode *,
                                  const QString& beginning,
@@ -558,21 +561,21 @@ private:
                                    const QString& restString,
                                    KCompletionMatchesWrapper *matches) const;
 
-    enum 		BeepMode { NoMatch, PartialMatch, Rotation };
-    void 		doBeep( BeepMode ) const;
+    enum        BeepMode { NoMatch, PartialMatch, Rotation };
+    void        doBeep( BeepMode ) const;
 
     KGlobalSettings::Completion myCompletionMode;
 
-    CompOrder 		myOrder;
-    QString             myLastString;
-    QString 		myLastMatch;
-    QString 		myCurrentMatch;
-    KCompTreeNode *     myTreeRoot;
-    QStringList 	myRotations;
-    bool                myBeep;
-    bool 		myIgnoreCase;
-    bool 		myHasMultipleMatches;
-    uint 		myRotationIndex;
+    CompOrder       myOrder;
+    QString         myLastString;
+    QString         myLastMatch;
+    QString         myCurrentMatch;
+    KCompTreeNode * myTreeRoot;
+    QStringList     myRotations;
+    bool            myBeep;
+    bool            myIgnoreCase;
+    bool            myHasMultipleMatches;
+    uint            myRotationIndex;
 
 
 protected:
@@ -586,25 +589,24 @@ typedef KSortableValueList<QString> KCompletionMatchesList;
 class KCompletionMatchesPrivate;
 
 /**
- * This structure is returned by @ref KCompletion::allWeightedMatches .
+ * This structure is returned by KCompletion::allWeightedMatches .
  * It also keeps the weight of the matches, allowing
  * you to modify some matches or merge them with matches
  * from another call to allWeightedMatches(), and sort the matches
  * after that in order to have the matches ordered correctly
  *
  * Example (a simplified example of what Konqueror's completion does):
- * <pre>
+ * \code
  * KCompletionMatches matches = completion->allWeightedMatches( location );
  * if( !location.startsWith( "www." ))
-       matches += completion->allWeightedmatches( "www." + location" );
+ matches += completion->allWeightedmatches( "www." + location" );
  * matches.removeDuplicates();
  * QStringList list = matches.list();
- * </pre>
+ * \endcode
  *
  * @short List for keeping matches returned from KCompletion
  */
-class KCompletionMatches
-    : public KCompletionMatchesList
+class KCompletionMatches : public KCompletionMatchesList
 {
 public:
     KCompletionMatches( bool sort );
@@ -645,7 +647,7 @@ private:
  * This is a convenience class that provides the basic functions
  * needed to add text completion support into widgets.  All that
  * is required is an implementation for the pure virtual function
- * @ref setCompletedText.  Refer to @ref KLineEdit or @ref KComboBox
+ * setCompletedText.  Refer to KLineEdit or KComboBox
  * to see how easily such support can be added using this as a base
  * class.
  *
@@ -654,13 +656,11 @@ private:
  */
 class KCompletionBase
 {
-
 public:
-
     /**
      * Constants that represent the items whose short-cut
      * key-binding is programmable.  The default key-bindings
-     * for these items are defined in @ref KStdAccel.
+     * for these items are defined in KStdAccel.
      */
     enum KeyBindingType {
         /**
@@ -703,14 +703,14 @@ public:
      * here is used by default to handle the signals
      * internally.  It is also deleted when this object's
      * destructor is invoked.  If you do not want these
-     * default settings, use @ref setAutoDeleteCompletionObject
-     * and @ref setHandleSignals to change the behavior.
+     * default settings, use setAutoDeleteCompletionObject
+     * and setHandleSignals to change the behavior.
      * Alternatively, you can set the boolean parameter to
      * false to disable the automatic handling of the signals
      * by this object.  Note that the boolean argument will be
      * ignored if there already exists a completion object since
      * no new object needs to be created. You need to use either
-     * @ref setHandleSignals or @ref setCompletionObject for
+     * setHandleSignals or setCompletionObject for
      * such cases depending on your requirement.
      *
      * @param hsig if true, handles signals internally.
@@ -729,7 +729,7 @@ public:
      *
      * The object assigned through this method is not deleted
      * when this object's destructor is invoked unless you
-     * explicitly call @ref setAutoDeleteCompletionObject after
+     * explicitly call setAutoDeleteCompletionObject after
      * calling this method.  Also if you do not want the signals
      * to be handled by an internal implementation, be sure to
      * set the bool argument to false.
@@ -737,10 +737,10 @@ public:
      * This method is also called when a completion-object is created
      * automatically, when completionObject() is called the first time.
      *
-     * @param compObj a @ref KCompletion() or a derived child object.
+     * @param compObj a KCompletion() or a derived child object.
      * @param hsig if true, handles signals internally.
      */
-    virtual void setCompletionObject( KCompletion* /*compObj*/, bool hsig = true );
+    virtual void setCompletionObject( KCompletion* compObj, bool hsig = true );
 
     /**
      * Enables this object to handle completion and rotation
@@ -754,16 +754,16 @@ public:
      *
      * @param handle if true, handle completion & rotation internally.
      */
-    virtual void setHandleSignals( bool /*handle*/ );
+    virtual void setHandleSignals( bool handle );
 
     /**
      * Returns true if the completion object is deleted
      * upon this widget's destruction.
      *
-     * See @ref setCompletionObject() and @ref enableCompletion()
+     * See setCompletionObject() and enableCompletion()
      * for details.
      *
-     * @return true if the completion object will be deleted 
+     * @return true if the completion object will be deleted
      *              automatically
      */
     bool isCompletionObjectAutoDeleted() const {
@@ -778,7 +778,7 @@ public:
      * is deleted when this widget's destructor is called.
      *
      * @param autoDelete if true, delete completion object on destruction.
-    */
+     */
     void setAutoDeleteCompletionObject( bool autoDelete ) {
         if ( m_delegate )
             m_delegate->setAutoDeleteCompletionObject( autoDelete );
@@ -793,16 +793,16 @@ public:
      * Invoking this function with @p enable set to @p false will
      * cause the completion & rotation signals not to be emitted.
      * However, unlike setting the completion object to @p NULL
-     * using @ref setCompletionObject, disabling the emition of
+     * using setCompletionObject, disabling the emition of
      * the signals through this method does not affect the current
      * completion object.
      *
      * There is no need to invoke this function by default.  When a
-     * completion object is created through @ref completionObject or
-     * @ref setCompletionObject, these signals are set to emit
+     * completion object is created through completionObject or
+     * setCompletionObject, these signals are set to emit
      * automatically.  Also note that disabling this signals will not
      * necessarily interfere with the objects ability to handle these
-     * events internally.  See @ref setHandleSignals.
+     * events internally.  See setHandleSignals.
      *
      * @param enable if false, disables the emition of completion & rotation signals.
      */
@@ -831,7 +831,7 @@ public:
      * Sets the type of completion to be used.
      *
      * The completion modes supported are those defined in
-     * @ref KGlobalSettings().  See below.
+     * KGlobalSettings().  See below.
      *
      * @param mode Completion type:
      *   @li CompletionNone:  Disables completion feature.
@@ -852,8 +852,8 @@ public:
     /**
      * Returns the current completion mode.
      *
-     * The return values are of type @ref KGlobalSettings::Completion.
-     * See @ref setCompletionMode() for details.
+     * The return values are of type KGlobalSettings::Completion.
+     * See setCompletionMode() for details.
      *
      * @return the completion mode.
      */
@@ -883,15 +883,15 @@ public:
      * function.
      *
      * @param item the feature whose key-binding needs to be set:
-     *   @li TextCompletion	the manual completion key-binding.
-     *   @li PrevCompletionMatch	the previous match key for multiple completion.
-     *   @li NextCompletionMatch	the next match key for for multiple completion.
+     *   @li TextCompletion the manual completion key-binding.
+     *   @li PrevCompletionMatch    the previous match key for multiple completion.
+     *   @li NextCompletionMatch    the next match key for for multiple completion.
      *   @li SubstringCompletion  the key for substring completion
      * @param key key-binding used to rotate down in a list.
-     * @return  true if key-binding can successfully be set.
-     * @see #getKeyBinding
+     * @return true if key-binding can successfully be set.
+     * @see getKeyBinding
      */
-    bool setKeyBinding( KeyBindingType /*item*/ , const KShortcut& cut );
+    bool setKeyBinding( KeyBindingType item , const KShortcut& key );
 
     /**
      * Returns the key-binding used for the specified item.
@@ -903,7 +903,7 @@ public:
      *
      * @param item the item to check
      * @return the key-binding used for the feature given by @p item.
-     * @see #setKeyBinding
+     * @see setKeyBinding
      */
     const KShortcut& getKeyBinding( KeyBindingType item ) const {
         return m_delegate ? m_delegate->getKeyBinding( item ) : m_keyMap[ item ];
@@ -930,7 +930,7 @@ public:
      * implementations to set completed text appropriately.  It
      * is mostly relevant when the completion mode is set to
      * CompletionAuto and CompletionManual modes. See
-     * @ref KCompletionBase::setCompletedText.
+     * KCompletionBase::setCompletedText.
      * Does nothing in CompletionPopup mode, as all available
      * matches will be shown in the popup.
      *
@@ -948,7 +948,7 @@ public:
     /**
      * Returns a pointer to the completion object.
      *
-     * This method is only different from @ref completionObject()
+     * This method is only different from completionObject()
      * in that it does not create a new KCompletion object even if
      * the internal pointer is @p NULL. Use this method to get the
      * pointer to a completion object when inheriting so that you
@@ -962,7 +962,7 @@ protected:
     /**
      * Returns a key-binding map.
      *
-     * This method is the same as @ref getKeyBinding() except it
+     * This method is the same as getKeyBinding() except it
      * returns the whole keymap containing the key-bindings.
      *
      * @return the key-binding used for the feature given by @p item.
