@@ -19,7 +19,7 @@
   Boston, MA 02111-1307, USA.
 */
 
-// $Id: kconfig.cpp,v 1.69 2002/08/06 22:46:58 waba Exp $
+// $Id: kconfig.cpp,v 1.79.2.1 2004/05/21 21:05:08 waba Exp $
 
 #include <config.h>
 
@@ -55,6 +55,7 @@ KConfig::KConfig( const QString& fileName,
 						      fileName,
                                                       resType,
 						      bUseKderc);
+
   // set the object's back end pointer to this new backend
   backEnd = aBackEnd;
 
@@ -73,6 +74,15 @@ KConfig::KConfig( const QString& fileName,
       reparseConfiguration();
 }
 
+KConfig::KConfig(KConfigBackEnd *aBackEnd, bool bReadOnly)
+    : bGroupImmutable(false), bFileImmutable(false),
+    bForceGlobal(false)
+{
+  setReadOnly(bReadOnly);
+  backEnd = aBackEnd;
+  reparseConfiguration();
+}
+
 KConfig::~KConfig()
 {
   sync();
@@ -149,6 +159,10 @@ QMap<QString, QString> KConfig::entryMap(const QString &pGroup) const
 
 void KConfig::reparseConfiguration()
 {
+  // Don't lose pending changes
+  if (!isReadOnly() && backEnd && bDirty)
+    backEnd->sync();
+
   aEntryMap.clear();
 
   // add the "default group" marker to the map
@@ -186,7 +200,7 @@ KEntryMap KConfig::internalEntryMap(const QString &pGroup) const
 
 void KConfig::putData(const KEntryKey &_key, const KEntry &_data, bool _checkGroup)
 {
-  if (bFileImmutable)
+  if (bFileImmutable && !_key.bDefault)
     return;
 
   // check to see if the special group key is present,
@@ -197,16 +211,17 @@ void KConfig::putData(const KEntryKey &_key, const KEntry &_data, bool _checkGro
     KEntry &entry = aEntryMap[groupKey];
     bGroupImmutable = entry.bImmutable;
   }
-  if (bGroupImmutable)
+  if (bGroupImmutable && !_key.bDefault)
     return;
 
   // now either add or replace the data
   KEntry &entry = aEntryMap[_key];
-  if (entry.bImmutable)
+  bool immutable = entry.bImmutable;
+  if (immutable && !_key.bDefault)
     return;
 
   entry = _data;
-
+  entry.bImmutable |= immutable;
   entry.bGlobal |= bForceGlobal; // force to kdeglobals
 
   if (_key.bDefault)
@@ -277,9 +292,62 @@ void KConfig::checkUpdate(const QString &id, const QString &updateFile)
   setGroup(oldGroup);
 }
 
+KConfig* KConfig::copyTo(const QString &file, KConfig *config) const
+{
+  if (!config)
+     config = new KConfig(QString::null, false, false);
+  config->backEnd->changeFileName(file, "config", false);
+  config->setReadOnly(false);
+  config->bFileImmutable = false;
+  config->backEnd->mConfigState = ReadWrite;
+
+  QStringList groups = groupList();
+  for(QStringList::ConstIterator it = groups.begin();
+      it != groups.end(); ++it)
+  {
+     QMap<QString, QString> map = entryMap(*it);
+     config->setGroup(*it);
+     for (QMap<QString,QString>::Iterator it2  = map.begin();
+          it2 != map.end(); ++it2)
+     {
+        config->writeEntry(it2.key(), it2.data());
+     }
+
+  }
+  return config;
+}
+
 void KConfig::virtual_hook( int id, void* data )
 { KConfigBase::virtual_hook( id, data ); }
 
+QValueList<KSharedConfig*> *KSharedConfig::s_list = 0;
 
+KSharedConfig::Ptr KSharedConfig::openConfig(const QString& fileName, bool immutable, bool useKDEGlobals )
+{
+  if (s_list)
+  {
+     for(QValueList<KSharedConfig*>::ConstIterator it = s_list->begin();
+         it != s_list->end(); ++it)
+     {
+        if ((*it)->backEnd->fileName() == fileName)
+           return (*it);
+     }
+  }
+  return new KSharedConfig(fileName, immutable, useKDEGlobals);
+}
+
+KSharedConfig::KSharedConfig( const QString& fileName, bool readonly, bool usekdeglobals)
+ : KConfig(fileName, readonly, usekdeglobals)
+{
+  if (!s_list)
+     s_list = new QValueList<KSharedConfig*>;
+     
+  s_list->append(this);
+}
+
+KSharedConfig::~KSharedConfig()
+{
+  s_list->remove(this);
+}
 
 #include "kconfig.moc"
