@@ -32,8 +32,8 @@
 #endif
 #include <fcntl.h>
 #include <signal.h>
+#include <setjmp.h>
 
-#undef Unsorted
 #include <qdir.h>
 #include <qfileinfo.h>
 #include <qtextcodec.h>
@@ -41,7 +41,9 @@
 
 #include "kconfigbackend.h"
 #include "kconfigbase.h"
+#include <kapplication.h>
 #include <kglobal.h>
+#include <kprocess.h>
 #include <klocale.h>
 #include <kstandarddirs.h>
 #include <ksavefile.h>
@@ -160,6 +162,61 @@ static QCString stringToPrintable(const QCString& str){
   return result;
 }
 
+static QCString decodeGroup(const char*s, int l)
+{
+  QCString result(l);
+  register char *r = result.data();
+
+  l--; // Correct for trailing \0
+  while(l)
+  {
+    if ((*s == '[') && (l > 1))
+    {
+       if ((*(s+1) == '['))
+       {
+          l--;
+          s++;
+       }
+    }
+    if ((*s == ']') && (l > 1))
+    {
+       if ((*(s+1) == ']'))
+       {
+          l--;
+          s++;
+       }
+    }
+    *r++ = *s++;
+    l--;
+  }
+  result.truncate(r - result.data());
+  return result;
+}
+
+static QCString encodeGroup(const QCString &str)
+{
+  int l = str.length();
+  QCString result(l*2+1);
+  register char *r = result.data();
+  register char *s = str.data();
+  while(l)
+  {
+    if ((*s == '[') || (*s == ']'))
+       *r++ = *s;
+    *r++ = *s++;
+    l--;
+  }
+  result.truncate(r - result.data());
+  return result;
+}
+
+class KConfigBackEnd::KConfigBackEndPrivate
+{
+public:
+   QDateTime localLastModified;
+   uint      localLastSize;
+};
+
 void KConfigBackEnd::changeFileName(const QString &_fileName,
                                     const char * _resType,
                                     bool _useKDEGlobals)
@@ -179,6 +236,9 @@ void KConfigBackEnd::changeFileName(const QString &_fileName,
 	      QString::fromLatin1("kdeglobals");
    else
       mGlobalFileName = QString::null;
+
+   d->localLastModified = QDateTime();
+   d->localLastSize = 0;
 }
 
 KConfigBackEnd::KConfigBackEnd(KConfigBase *_config,
@@ -187,9 +247,15 @@ KConfigBackEnd::KConfigBackEnd(KConfigBase *_config,
 			       bool _useKDEGlobals)
   : pConfig(_config), bFileImmutable(false), mConfigState(KConfigBase::NoAccess), mFileMode(-1)
 {
+   d = new KConfigBackEndPrivate;
    changeFileName(_fileName, _resType, _useKDEGlobals);
 }
 
+KConfigBackEnd::~KConfigBackEnd()
+{
+   delete d;
+}
+
 void KConfigBackEnd::setFileWriteMode(int mode)
 {
   mFileMode = mode;
@@ -218,6 +284,9 @@ bool KConfigINIBackEnd::parseConfigFiles()
            mConfigState = KConfigBase::ReadWrite;
         }
      }
+     QFileInfo info(mLocalFileName);
+     d->localLastModified = info.lastModified();
+     d->localLastSize = info.size();
   }
 
   // Parse all desired files from the least to the most specific.
@@ -259,8 +328,11 @@ bool KConfigINIBackEnd::parseConfigFiles()
     }
 
     bFileImmutable = false;
-    QStringList list = KGlobal::dirs()->
-      findAllResources(resType, mfileName);
+    QStringList list;
+    if ( mfileName[0] == '/' )
+      list << mfileName;
+    else
+      list = KGlobal::dirs()->findAllResources(resType, mfileName);
 
     QStringList::ConstIterator it;
 
@@ -299,22 +371,27 @@ bool KConfigINIBackEnd::parseConfigFiles()
 
 #ifdef HAVE_MMAP
 #ifdef SIGBUS
-static const char **mmap_pEof;
+static sigjmp_buf mmap_jmpbuf;
+struct sigaction mmap_old_sigact;
 
-static void mmap_sigbus_handler(int)
-{
-   *mmap_pEof = 0;
-   write(2, "SIGBUS\n", 7);
-   signal(SIGBUS, mmap_sigbus_handler);
+extern "C" {
+   static void mmap_sigbus_handler(int)
+   {
+      siglongjmp (mmap_jmpbuf, 1);
+   }
 }
 #endif
 #endif
 
+extern bool kde_kiosk_exception;
+
 void KConfigINIBackEnd::parseSingleConfigFile(QFile &rFile,
 					      KEntryMap *pWriteBackMap,
 					      bool bGlobal, bool bDefault)
 {
-   void (*old_sighandler)(int) = 0;
+   const char *s; // May get clobbered by sigsetjump, but we don't use them afterwards.
+   const char *eof; // May get clobbered by sigsetjump, but we don't use them afterwards.
+   QByteArray data;
 
    if (!rFile.isOpen()) // come back, if you have real work for us ;->
       return;
@@ -326,23 +403,35 @@ void KConfigINIBackEnd::parseSingleConfigFile(QFile &rFile,
 
    QCString aCurrentGroup("<default>");
 
-   const char *s, *eof;
-   QByteArray data;
-
    unsigned int ll = localeString.length();
 
 #ifdef HAVE_MMAP
-   const char *map = ( const char* ) mmap(0, rFile.size(), PROT_READ, MAP_PRIVATE,
+   static volatile const char *map;
+   map = ( const char* ) mmap(0, rFile.size(), PROT_READ, MAP_PRIVATE,
                                           rFile.handle(), 0);
 
    if (map)
    {
-      s = map;
+      s = (const char*) map;
       eof = s + rFile.size();
 
-#ifdef SIGBUS      
-      mmap_pEof = &eof;
-      old_sighandler = signal(SIGBUS, mmap_sigbus_handler);
+#ifdef SIGBUS
+      struct sigaction act;
+      act.sa_handler = mmap_sigbus_handler;
+      sigemptyset( &act.sa_mask );
+#ifdef SA_ONESHOT
+      act.sa_flags = SA_ONESHOT;
+#else
+      act.sa_flags = SA_RESETHAND;
+#endif      
+      sigaction( SIGBUS, &act, &mmap_old_sigact );
+
+      if (sigsetjmp (mmap_jmpbuf, 1))
+      {
+         munmap(( char* )map, rFile.size());
+         sigaction (SIGBUS, &mmap_old_sigact, 0);
+         return;
+      }
 #endif
    }
    else
@@ -378,7 +467,19 @@ void KConfigINIBackEnd::parseSingleConfigFile(QFile &rFile,
 
       if (*s == '[')  //group
       {
-         while ((s < eof) && (*s != '\n') && (*s != ']')) s++; // Search till end of group
+         // In a group [[ and ]] have a special meaning
+         while ((s < eof) && (*s != '\n')) 
+         {
+            if (*s == ']')
+            {
+               if ((s+1 < eof) && (*(s+1) == ']'))
+                  s++; // Skip "]]"
+               else
+                  break;
+            }
+
+            s++; // Search till end of group
+         }
          const char *e = s;
          while ((s < eof) && (*s != '\n')) s++; // Search till end of line / end of file
          if ((e >= eof) || (*e != ']'))
@@ -392,11 +493,12 @@ void KConfigINIBackEnd::parseSingleConfigFile(QFile &rFile,
              (startLine[1] == '$') &&
              (startLine[2] == 'i'))
          {
-            fileOptionImmutable = true;
+            if (!kde_kiosk_exception)
+               fileOptionImmutable = true;
             continue;
          }
 
-         aCurrentGroup = QCString(startLine + 1, e - startLine);
+         aCurrentGroup = decodeGroup(startLine + 1, e - startLine);
          //cout<<"found group ["<<aCurrentGroup<<"]"<<endl;
 
          // Backwards compatibility
@@ -408,7 +510,7 @@ void KConfigINIBackEnd::parseSingleConfigFile(QFile &rFile,
          e++;
          if ((e+2 < eof) && (*e++ == '[') && (*e++ == '$')) // Option follows
          {
-            if (*e == 'i')
+            if ((*e == 'i') && !kde_kiosk_exception)
             {
                groupOptionImmutable = true;
             }
@@ -418,10 +520,10 @@ void KConfigINIBackEnd::parseSingleConfigFile(QFile &rFile,
          KEntry entry = pConfig->lookupData(groupKey);
          groupSkip = entry.bImmutable;
 
-         if (groupSkip)
+         if (groupSkip && !bDefault)
             continue;
 
-         entry.bImmutable = groupOptionImmutable;
+         entry.bImmutable |= groupOptionImmutable;
          pConfig->putData(groupKey, entry, false);
 
          if (pWriteBackMap)
@@ -432,7 +534,7 @@ void KConfigINIBackEnd::parseSingleConfigFile(QFile &rFile,
 
          continue;
       }
-      if (groupSkip)
+      if (groupSkip && !bDefault)
         goto sktoeol; // Skip entry
 
       bool optionImmutable = groupOptionImmutable;
@@ -479,7 +581,7 @@ void KConfigINIBackEnd::parseSingleConfigFile(QFile &rFile,
               while (option < eoption)
               {
                  option++;
-                 if (*option == 'i')
+                 if ((*option == 'i') && !kde_kiosk_exception)
                     optionImmutable = true;
                  else if (*option == 'e')
                     optionExpand = true;
@@ -517,7 +619,7 @@ void KConfigINIBackEnd::parseSingleConfigFile(QFile &rFile,
           if ((ll != cl) || memcmp(locale, localeString.data(), ll))
           {
               // backward compatibility. C == en_US
-              if ( cl != 1 || ll != 5 || memcmp(locale, "C", 1) || memcmp(localeString.data(), "en_US", 5)) {
+              if ( cl != 1 || ll != 5 || *locale != 'C' || memcmp(localeString.data(), "en_US", 5)) {
                   //cout<<"mismatched locale '"<<QCString(locale, elocale-locale +1)<<"'"<<endl;
                   // We can ignore this one
                   if (!pWriteBackMap)
@@ -564,8 +666,8 @@ void KConfigINIBackEnd::parseSingleConfigFile(QFile &rFile,
    if (map)
    {
       munmap(( char* )map, rFile.size());
-#ifdef SIGBUS      
-      signal(SIGBUS, old_sighandler);
+#ifdef SIGBUS
+      sigaction (SIGBUS, &mmap_old_sigact, 0);
 #endif
    }
 #endif
@@ -598,8 +700,43 @@ void KConfigINIBackEnd::sync(bool bMerge)
     // check if the user would be allowed to write if
     // it wasn't SUID.
     if (checkAccess(mLocalFileName, W_OK)) {
-      // is it writable?
-      bEntriesLeft = writeConfigFile( mLocalFileName, false, bMerge );
+      // File is writable
+
+      bool mergeLocalFile = bMerge;
+      // Check if the file has been updated since.
+      if (mergeLocalFile)
+      {
+         QFileInfo info(mLocalFileName);
+         if ((d->localLastSize == info.size()) &&
+             (d->localLastModified == info.lastModified()))
+         {
+            // Not changed, don't merge.
+            mergeLocalFile = false;
+         }
+         else
+         {
+            // Changed...
+            d->localLastModified = QDateTime();
+            d->localLastSize = 0;
+         }
+      }
+
+      bEntriesLeft = writeConfigFile( mLocalFileName, false, mergeLocalFile );
+      
+      // Only if we didn't have to merge anything can we use our in-memory state
+      // the next time around. Otherwise the config-file may contain entries
+      // that are different from our in-memory state which means we will have to 
+      // do a merge from then on. 
+      // We do not automatically update the in-memory state with the on-disk 
+      // state when writing the config to disk. We only do so when 
+      // KCOnfig::reparseConfiguration() is called.
+      // For KDE 4.0 we may wish to reconsider that.
+      if (!mergeLocalFile)
+      {
+         QFileInfo info(mLocalFileName);
+         d->localLastModified = info.lastModified();
+         d->localLastSize = info.size();
+      }
     }
   }
 
@@ -608,10 +745,9 @@ void KConfigINIBackEnd::sync(bool bMerge)
   // the useKDEGlobals flag is set.
   if (bEntriesLeft && useKDEGlobals) {
 
-
     // can we allow the write? (see above)
     if (checkAccess ( mGlobalFileName, W_OK )) {
-      writeConfigFile( mGlobalFileName, true, bMerge );
+      writeConfigFile( mGlobalFileName, true, bMerge ); // Always merge
     }
   }
 
@@ -668,7 +804,7 @@ static void writeEntries(FILE *pStream, const KEntryMap& entryMap, bool defaultG
 	if (!firstEntry)
 	    fprintf(pStream, "\n");
 	currentGroup = key.mGroup;
-	fprintf(pStream, "[%s]\n", currentGroup.data());
+	fprintf(pStream, "[%s]\n", encodeGroup(currentGroup).data());
      }
 
      firstEntry = false;
@@ -696,7 +832,7 @@ static void writeEntries(FILE *pStream, const KEntryMap& entryMap, bool defaultG
               fputc('i', pStream);
            if (currentEntry.bExpand)
               fputc('e', pStream);
-           
+
            fputc(']', pStream);
         }
         fputc('=', pStream);
@@ -706,71 +842,72 @@ static void writeEntries(FILE *pStream, const KEntryMap& entryMap, bool defaultG
   } // for loop
 }
 
+bool KConfigINIBackEnd::getEntryMap(KEntryMap &aTempMap, bool bGlobal,
+                                    QFile *mergeFile)
+{
+  bool bEntriesLeft = false;
+  bFileImmutable = false;
+
+  // Read entries from disk
+  if (mergeFile && mergeFile->open(IO_ReadOnly))
+  {
+     // fill the temporary structure with entries from the file
+     parseSingleConfigFile(*mergeFile, &aTempMap, bGlobal, false );
+
+     if (bFileImmutable) // File has become immutable on disk
+        return bEntriesLeft;
+  }
+
+  KEntryMap aMap = pConfig->internalEntryMap();
+
+  // augment this structure with the dirty entries from the config object
+  for (KEntryMapIterator aIt = aMap.begin();
+       aIt != aMap.end(); ++aIt)
+  {
+    const KEntry &currentEntry = *aIt;
+    if(aIt.key().bDefault)
+    {
+       aTempMap.replace(aIt.key(), currentEntry);
+       continue;
+    }
+
+    if (mergeFile && !currentEntry.bDirty)
+       continue;
+
+    // only write back entries that have the same
+    // "globality" as the file
+    if (currentEntry.bGlobal != bGlobal)
+    {
+       // wrong "globality" - might have to be saved later
+       bEntriesLeft = true;
+       continue;
+    }
+
+    // put this entry from the config object into the
+    // temporary map, possibly replacing an existing entry
+    KEntryMapIterator aIt2 = aTempMap.find(aIt.key());
+    if (aIt2 != aTempMap.end() && (*aIt2).bImmutable)
+       continue; // Bail out if the on-disk entry is immutable
+
+    aTempMap.insert(aIt.key(), currentEntry, true);
+  } // loop
+
+  return bEntriesLeft;
+}
+
+/* antlarr: KDE 4.0:  make the first parameter "const QString &" */
 bool KConfigINIBackEnd::writeConfigFile(QString filename, bool bGlobal,
 					bool bMerge)
 {
-  KEntryMap aTempMap;
-  bool bEntriesLeft = false;
-
   // is the config object read-only?
   if (pConfig->isReadOnly())
     return true; // pretend we wrote it
 
-  bFileImmutable = false;
-  if (bMerge)
-  {
-    // Read entries from disk
-    QFile rConfigFile( filename );
-    if (rConfigFile.open(IO_ReadOnly))
-    {
-       // fill the temporary structure with entries from the file
-       parseSingleConfigFile( rConfigFile, &aTempMap, bGlobal, false );
-       rConfigFile.close();
-       
-       if (bFileImmutable) // File has become immutable on disk
-          return true; // pretend we wrote it
-    }
-
-    KEntryMap aMap = pConfig->internalEntryMap();
-
-    // augment this structure with the dirty entries from the config object
-    for (KEntryMapIterator aIt = aMap.begin();
-          aIt != aMap.end(); ++aIt)
-    {
-      const KEntry &currentEntry = *aIt;
-      if(aIt.key().bDefault)
-      {
-         aTempMap.replace(aIt.key(), currentEntry);
-         continue;
-      }
-
-      if (!currentEntry.bDirty)
-         continue;
-
-      // only write back entries that have the same
-      // "globality" as the file
-      if (currentEntry.bGlobal != bGlobal)
-      {
-        // wrong "globality" - might have to be saved later
-        bEntriesLeft = true;
-        continue;
-      }
-
-      // put this entry from the config object into the
-      // temporary map, possibly replacing an existing entry
-      KEntryMapIterator aIt2 = aTempMap.find(aIt.key());
-      if (aIt2 != aTempMap.end() && (*aIt2).bImmutable)
-        continue; // Bail out if the on-disk entry is immutable
-
-      aTempMap.insert(aIt.key(), currentEntry, true);
-    } // loop
-  }
-  else
-  {
-    // don't merge, just get the regular entry map and use that.
-    aTempMap = pConfig->internalEntryMap();
-    bEntriesLeft = true; // maybe not true, but we aren't sure
-  }
+  KEntryMap aTempMap;
+  QFile *mergeFile = (bMerge ? new QFile(filename) : 0);
+  bool bEntriesLeft = getEntryMap(aTempMap, bGlobal, mergeFile);
+  delete mergeFile;
+  if (bFileImmutable) return true; // pretend we wrote it
 
   // OK now the temporary map should be full of ALL entries.
   // write it out to disk.
@@ -842,17 +979,22 @@ bool KConfigINIBackEnd::writeConfigFile(QString filename, bool bGlobal,
      }
   }
 
-  bool firstEntry = true;
-
-  // Write default group
-  writeEntries(pStream, aTempMap, true, firstEntry, localeString);
-
-  // Write all other groups
-  writeEntries(pStream, aTempMap, false, firstEntry, localeString);
+  writeEntries(pStream, aTempMap);
 
   if (pConfigFile)
   {
-     pConfigFile->close();
+     bool bEmptyFile = (ftell(pStream) == 0);
+     if ( bEmptyFile && ((fileMode == -1) || (fileMode == 0600)) )
+     {
+        // File is empty and doesn't have special permissions: delete it.
+        ::unlink(QFile::encodeName(filename));
+        pConfigFile->abort();
+     }
+     else
+     {
+        // Normal case: Close the file
+        pConfigFile->close();
+     }
      delete pConfigFile;
   }
   else
@@ -863,6 +1005,16 @@ bool KConfigINIBackEnd::writeConfigFile(QString filename, bool bGlobal,
   return bEntriesLeft;
 }
 
+void KConfigINIBackEnd::writeEntries(FILE *pStream, const KEntryMap &aTempMap)
+{
+  bool firstEntry = true;
+
+  // Write default group
+  ::writeEntries(pStream, aTempMap, true, firstEntry, localeString);
+
+  // Write all other groups
+  ::writeEntries(pStream, aTempMap, false, firstEntry, localeString);
+}
 
 void KConfigBackEnd::virtual_hook( int, void* )
 { /*BASE::virtual_hook( id, data );*/ }
@@ -870,3 +1022,36 @@ void KConfigBackEnd::virtual_hook( int, void* )
 void KConfigINIBackEnd::virtual_hook( int id, void* data )
 { KConfigBackEnd::virtual_hook( id, data ); }
 
+bool KConfigBackEnd::checkConfigFilesWritable(bool warnUser)
+{
+  // WARNING: Do NOT use the event loop as it may not exist at this time.
+  bool allWritable = true;
+  QString errorMsg( i18n("Will not save configuration.\n") );
+  if ( !mLocalFileName.isEmpty() && !bFileImmutable && !checkAccess(mLocalFileName,W_OK) )
+  {
+    allWritable = false;
+    errorMsg += i18n("Configuration file \"%1\" not writable.\n").arg(mLocalFileName);
+  }
+  // We do not have an immutability flag for kdeglobals. However, making kdeglobals mutable while making
+  // the local config file immutable is senseless.
+  if ( !mGlobalFileName.isEmpty() && useKDEGlobals && !bFileImmutable && !checkAccess(mGlobalFileName,W_OK) )
+  {
+    errorMsg += i18n("Configuration file \"%1\" not writable.\n").arg(mGlobalFileName);
+    allWritable = false;
+  }
+
+  if (warnUser && !allWritable)
+  {
+    // Note: We don't ask the user if we should not ask this question again because we can't save the answer.
+    errorMsg += i18n("Please contact your system administrator.");
+    QString cmdToExec = KStandardDirs::findExe(QString("kdialog"));
+    KApplication *app = kapp;
+    if (!cmdToExec.isEmpty() && app)
+    {
+      KProcess lprocess;
+      lprocess << cmdToExec << "--title" << app->instanceName() << "--msgbox" << errorMsg.local8Bit();
+      lprocess.start( KProcess::Block );
+    }
+  }
+  return allWritable;
+}
