@@ -2,17 +2,17 @@
    This file is part of the KDE libraries
    Copyright (c) 1999 Preston Brown <pbrown@kde.org>
    Portions copyright (c) 1997 Matthias Kalle Dalheimer <kalle@kde.org>
-   
+
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.
-   
+
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.
-   
+
    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
    the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
@@ -24,6 +24,7 @@
 
 #include "kconfigdata.h"
 #include <kconfigbase.h>
+#include "kdemacros.h"
 
 class QFile;
 class KConfigBackEndPrivate;
@@ -38,13 +39,14 @@ class KConfigBackEndPrivate;
  * is one to read/write to INI-style files, but in the future, other
  * formats may be available, such as XML or a database.
  *
- * @author Preston Brown <pbrown@kde.org>, 
+ * @author Preston Brown <pbrown@kde.org>,
  *         Matthias Kalle Dalheimer <kalle@kde.org>
- * @version $Id: kconfigbackend.h,v 1.22 2002/09/17 21:24:43 tjansen Exp $
- * @short KDE Configuration file loading/saving abstract base class 
+ * @version $Id: kconfigbackend.h,v 1.32 2003/11/26 23:28:41 ossi Exp $
+ * @short KDE Configuration file loading/saving abstract base class
  */
 class KConfigBackEnd
 {
+  friend class KConfig;
 public:
   /**
    * Constructs a configuration back end.
@@ -67,7 +69,7 @@ public:
   /**
    * Destructs the configuration backend.
    */
-  virtual ~KConfigBackEnd() {};
+  virtual ~KConfigBackEnd();
 
   /**
    * Parses all configuration files for a configuration object.  This
@@ -134,13 +136,21 @@ public:
    */
   void setFileWriteMode(int mode);
 
+  /**
+   * Check whether the config files are writable.
+   * @param warnUser Warn the user if the configuration files are not writable.
+   * @return Indicates that all of the configuration files used are writable.
+   * @since 3.2
+   */
+  bool checkConfigFilesWritable(bool warnUser);
+
 #ifdef KDE_NO_COMPAT
 private:
 #endif
   /**
    * @deprecated Use @see fileName() instead
    */
-  QString filename() const { return mfileName; }
+  KDE_DEPRECATED QString filename() const { return mfileName; }
 
 
 protected:
@@ -158,7 +168,8 @@ protected:
 
 protected:
   virtual void virtual_hook( int id, void* data );
-private:
+protected:
+  class KConfigBackEndPrivate;
   KConfigBackEndPrivate *d;
 };
 
@@ -169,7 +180,7 @@ class KConfigINIBackEndPrivate;
  *
  * @author Preston Brown <pbrown@kde.org>,
  *         Matthias Kalle Dalheimer <kalle@kde.org>
- * @version $Id: kconfigbackend.h,v 1.22 2002/09/17 21:24:43 tjansen Exp $
+ * @version $Id: kconfigbackend.h,v 1.32 2003/11/26 23:28:41 ossi Exp $
  */
 class KConfigINIBackEnd : public KConfigBackEnd
 {
@@ -250,10 +261,27 @@ protected:
    */
   bool writeConfigFile(QString filename, bool bGlobal = false, bool bMerge = true);
 
+  /** Get the entry map.
+   *
+   * @param map the entries will be stored in this object.
+   * @param bGlobal Specifies whether to get only entries which
+   *        are marked as belonging to the global KDE config file.
+   *        If this is false, it skips those entries.
+   * @param mergeFile if not null, the dirty entries for this file will
+   * be merged.
+   *
+   * @return Whether there will be some entries left for writing to other
+   * files.
+   */
+  bool getEntryMap(KEntryMap &map, bool bGlobal, QFile *mergeFile);
+
+  /** Write the entries in @e aTempMap to the file stream.*/
+  void writeEntries(FILE *pStream, const KEntryMap &aTempMap);
+
 protected:
   virtual void virtual_hook( int id, void* data );
 private:
-  KConfigINIBackEndPrivate *d;
+  KConfigINIBackEndPrivate *not_d;
 };
 
 #endif
