@@ -36,99 +36,27 @@
 #include "kconfigbackend.h"
 #include "kdebug.h"
 #include "kstandarddirs.h"
-#undef Bool
+#include "kstringhandler.h"
 
-static bool isUtf8(const char *buf) {
-  int i, n;
-  register unsigned char c;
-  bool gotone = false;
-
-#define F 0   /* character never appears in text */
-#define T 1   /* character appears in plain ASCII text */
-#define I 2   /* character appears in ISO-8859 text */
-#define X 3   /* character appears in non-ISO extended ASCII (Mac, IBM PC) */
-
-  static const unsigned char text_chars[256] = {
-  /*                  BEL BS HT LF    FF CR    */
-        F, F, F, F, F, F, F, T, T, T, T, F, T, T, F, F,  /* 0x0X */
-        /*                              ESC          */
-        F, F, F, F, F, F, F, F, F, F, F, T, F, F, F, F,  /* 0x1X */
-        T, T, T, T, T, T, T, T, T, T, T, T, T, T, T, T,  /* 0x2X */
-        T, T, T, T, T, T, T, T, T, T, T, T, T, T, T, T,  /* 0x3X */
-        T, T, T, T, T, T, T, T, T, T, T, T, T, T, T, T,  /* 0x4X */
-        T, T, T, T, T, T, T, T, T, T, T, T, T, T, T, T,  /* 0x5X */
-        T, T, T, T, T, T, T, T, T, T, T, T, T, T, T, T,  /* 0x6X */
-        T, T, T, T, T, T, T, T, T, T, T, T, T, T, T, F,  /* 0x7X */
-        /*            NEL                            */
-        X, X, X, X, X, T, X, X, X, X, X, X, X, X, X, X,  /* 0x8X */
-        X, X, X, X, X, X, X, X, X, X, X, X, X, X, X, X,  /* 0x9X */
-        I, I, I, I, I, I, I, I, I, I, I, I, I, I, I, I,  /* 0xaX */
-        I, I, I, I, I, I, I, I, I, I, I, I, I, I, I, I,  /* 0xbX */
-        I, I, I, I, I, I, I, I, I, I, I, I, I, I, I, I,  /* 0xcX */
-        I, I, I, I, I, I, I, I, I, I, I, I, I, I, I, I,  /* 0xdX */
-        I, I, I, I, I, I, I, I, I, I, I, I, I, I, I, I,  /* 0xeX */
-        I, I, I, I, I, I, I, I, I, I, I, I, I, I, I, I   /* 0xfX */
-  };
-
-  /* *ulen = 0; */
-  for (i = 0; (c = buf[i]); i++) {
-    if ((c & 0x80) == 0) {        /* 0xxxxxxx is plain ASCII */
-      /*
-       * Even if the whole file is valid UTF-8 sequences,
-       * still reject it if it uses weird control characters.
-       */
-
-      if (text_chars[c] != T)
-        return false;
-
-    } else if ((c & 0x40) == 0) { /* 10xxxxxx never 1st byte */
-      return false;
-    } else {                           /* 11xxxxxx begins UTF-8 */
-      int following;
-
-    if ((c & 0x20) == 0) {             /* 110xxxxx */
-      following = 1;
-    } else if ((c & 0x10) == 0) {      /* 1110xxxx */
-      following = 2;
-    } else if ((c & 0x08) == 0) {      /* 11110xxx */
-      following = 3;
-    } else if ((c & 0x04) == 0) {      /* 111110xx */
-      following = 4;
-    } else if ((c & 0x02) == 0) {      /* 1111110x */
-      following = 5;
-    } else
-      return false;
-
-      for (n = 0; n < following; n++) {
-        i++;
-        if (!(c = buf[i]))
-          goto done;
-
-        if ((c & 0x80) == 0 || (c & 0x40))
-          return false;
-      }
-      gotone = true;
-    }
-  }
-done:
-  return gotone;   /* don't claim it's UTF-8 if it's all 7-bit */
-}
-
-#undef F
-#undef T
-#undef I
-#undef X
+class KConfigBase::KConfigBasePrivate
+{
+public:
+     KConfigBasePrivate() : readDefaults(false) { };
 
+public:
+     bool readDefaults;
+};
 
 KConfigBase::KConfigBase()
   : backEnd(0L), bDirty(false), bLocaleInitialized(false),
-    bReadOnly(false), bExpand(false)
+    bReadOnly(false), bExpand(false), d(0)
 {
     setGroup(QString::null);
 }
 
 KConfigBase::~KConfigBase()
 {
+    delete d;
 }
 
 void KConfigBase::setLocale()
@@ -187,6 +115,7 @@ bool KConfigBase::hasKey(const char *pKey) const
 {
   KEntryKey aEntryKey(mGroup, 0);
   aEntryKey.c_key = pKey;
+  aEntryKey.bDefault = readDefaults();
 
   if (!locale().isNull()) {
     // try the localized key first
@@ -298,25 +227,13 @@ QString KConfigBase::readEntry( const char *pKey,
   KEntry aEntryData;
   KEntryKey entryKey(mGroup, 0);
   entryKey.c_key = pKey;
+  entryKey.bDefault = readDefaults();
   entryKey.bLocal = true;
   aEntryData = lookupData(entryKey);
   if (!aEntryData.mValue.isNull()) {
-
     // for GNOME .desktop
-    // const char *data = aEntryData.mValue.char();
-    if ( isUtf8(aEntryData.mValue.data() ) )
-      aValue = QString::fromUtf8( aEntryData.mValue.data() );
-    else
-      aValue = QString::fromLocal8Bit(aEntryData.mValue.data());
+    aValue = KStringHandler::from8Bit( aEntryData.mValue.data() );
     expand = aEntryData.bExpand;
-
-    // Ok this sucks. QString::fromUtf8("").isNull() is true,
-    // but QString::fromLatin1("").isNull() returns false.
-    if (aValue.isNull())
-    {
-      static const QString &emptyString = KGlobal::staticQString("");
-      aValue = emptyString;
-    }
   } else {
     entryKey.bLocal = false;
     aEntryData = lookupData(entryKey);
@@ -383,10 +300,7 @@ QString KConfigBase::readEntry( const char *pKey,
 	    // !!! Sergey A. Sukiyazov <corwin@micom.don.ru> !!!
 	    // A environment variables may contain values in 8bit
 	    // locale cpecified encoding or in UTF8 encoding.
-	    if (isUtf8( pEnv ))
-		aValue.replace( nDollarPos, nEndPos-nDollarPos, QString::fromUtf8(pEnv) );
-	    else
-		aValue.replace( nDollarPos, nEndPos-nDollarPos, QString::fromLocal8Bit(pEnv) );
+	    aValue.replace( nDollarPos, nEndPos-nDollarPos, KStringHandler::from8Bit( pEnv ) );
           } else
             aValue.remove( nDollarPos, nEndPos-nDollarPos );
         } else {
@@ -405,6 +319,7 @@ QCString KConfigBase::readEntryUtf8( const char *pKey) const
 {
   // We don't try the localized key
   KEntryKey entryKey(mGroup, 0);
+  entryKey.bDefault = readDefaults();
   entryKey.c_key = pKey;
   KEntry aEntryData = lookupData(entryKey);
   if (aEntryData.bExpand)
@@ -477,6 +392,10 @@ QVariant KConfigBase::readPropertyEntry( const char *pKey,
           return QVariant( readNumEntry( pKey, aDefault.toInt() ) );
       case QVariant::UInt:
           return QVariant( readUnsignedNumEntry( pKey, aDefault.toUInt() ) );
+      case QVariant::LongLong:
+          return QVariant( readNum64Entry( pKey, aDefault.toLongLong() ) );
+      case QVariant::ULongLong:
+          return QVariant( readUnsignedNum64Entry( pKey, aDefault.toULongLong() ) );
       case QVariant::Bool:
           return QVariant( readBoolEntry( pKey, aDefault.toBool() ), 0 );
       case QVariant::Double:
@@ -538,7 +457,8 @@ int KConfigBase::readListEntry( const char *pKey,
     }
     if (str_list[i] == '\\') {
       i++;
-      value += str_list[i];
+      if ( i < len )
+        value += str_list[i];
       continue;
     }
     // if we fell through to here, we are at a separator.  Append
@@ -562,14 +482,18 @@ QStringList KConfigBase::readListEntry( const QString& pKey, char sep ) const
 
 QStringList KConfigBase::readListEntry( const char *pKey, char sep ) const
 {
+  static const QString& emptyString = KGlobal::staticQString("");
+
   QStringList list;
   if( !hasKey( pKey ) )
     return list;
   QString str_list = readEntry( pKey );
   if( str_list.isEmpty() )
     return list;
-  QString value = "";
+  QString value(emptyString);
   int len = str_list.length();
+ // obviously too big, but faster than letting each += resize the string.
+  value.reserve( len );
   for( int i = 0; i < len; i++ )
     {
       if( str_list[i] != sep && str_list[i] != '\\' )
@@ -580,14 +504,20 @@ QStringList KConfigBase::readListEntry( const char *pKey, char sep ) const
       if( str_list[i] == '\\' )
         {
           i++;
-          value += str_list[i];
+          if ( i < len )
+            value += str_list[i];
           continue;
         }
-      list.append( value );
-      value.truncate(0);
+      QString finalvalue( value );
+      finalvalue.squeeze();
+      list.append( finalvalue );
+      value.truncate( 0 );
     }
   if ( str_list[len-1] != sep || ( len > 1 && str_list[len-2] == '\\' ) )
+  {
+    value.squeeze();
     list.append( value );
+  }
   return list;
 }
 
@@ -713,6 +643,45 @@ unsigned long KConfigBase::readUnsignedLongNumEntry( const char *pKey, unsigned
     }
 }
 
+Q_INT64 KConfigBase::readNum64Entry( const QString& pKey, Q_INT64 nDefault) const
+{
+  return readNum64Entry(pKey.utf8().data(), nDefault);
+}
+
+Q_INT64 KConfigBase::readNum64Entry( const char *pKey, Q_INT64 nDefault) const
+{
+  // Note that QCString::toLongLong() is missing, we muse use a QString instead.
+  QString aValue = readEntry( pKey );
+  if( aValue.isNull() )
+    return nDefault;
+  else
+    {
+      bool ok;
+      Q_INT64 rc = aValue.toLongLong( &ok );
+      return( ok ? rc : nDefault );
+    }
+}
+
+
+Q_UINT64 KConfigBase::readUnsignedNum64Entry( const QString& pKey, Q_UINT64 nDefault) const
+{
+  return readUnsignedNum64Entry(pKey.utf8().data(), nDefault);
+}
+
+Q_UINT64 KConfigBase::readUnsignedNum64Entry( const char *pKey, Q_UINT64 nDefault) const
+{
+  // Note that QCString::toULongLong() is missing, we muse use a QString instead.
+  QString aValue = readEntry( pKey );
+  if( aValue.isNull() )
+    return nDefault;
+  else
+    {
+      bool ok;
+      Q_UINT64 rc = aValue.toULongLong( &ok );
+      return( ok ? rc : nDefault );
+    }
+}
+
 double KConfigBase::readDoubleNumEntry( const QString& pKey, double nDefault) const
 {
   return readDoubleNumEntry(pKey.utf8().data(), nDefault);
@@ -1094,7 +1063,7 @@ static QString translatePath( QString path )
    if (path.isEmpty())
        return path;
 
-   bool startsWithFile = path.left(5).lower() == QString::fromLatin1("file:");
+   bool startsWithFile = path.startsWith("file:", false);
 
    // return original path, if it refers to another type of URL (e.g. http:/), or
    // if the path is already relative to another directory
@@ -1277,6 +1246,12 @@ void KConfigBase::writeEntry ( const char *pKey, const QVariant &prop,
     case QVariant::UInt:
       writeEntry( pKey, prop.toUInt(), bPersistent, bGlobal, bNLS );
       return;
+    case QVariant::LongLong:
+      writeEntry( pKey, prop.toLongLong(), bPersistent, bGlobal, bNLS );
+      return;
+    case QVariant::ULongLong:
+      writeEntry( pKey, prop.toULongLong(), bPersistent, bGlobal, bNLS );
+      return;
     case QVariant::Bool:
       writeEntry( pKey, prop.toBool(), bPersistent, bGlobal, bNLS );
       return;
@@ -1339,10 +1314,7 @@ void KConfigBase::writeEntry ( const char *pKey, const QStrList &list,
       // !!! Sergey A. Sukiyazov <corwin@micom.don.ru> !!!
       // A QStrList may contain values in 8bit locale cpecified
       // encoding or in UTF8 encoding.
-      if (isUtf8(it.current()))
-        value = QString::fromUtf8(it.current());
-      else
-        value = QString::fromLocal8Bit(it.current());
+      value = KStringHandler::from8Bit(it.current());
       for( i = 0; i < value.length(); i++ )
         {
           if( value[i] == sep || value[i] == '\\' )
@@ -1466,6 +1438,34 @@ void KConfigBase::writeEntry( const char *pKey, unsigned long nValue,
   writeEntry( pKey, QString::number(nValue), bPersistent, bGlobal, bNLS );
 }
 
+void KConfigBase::writeEntry( const QString& pKey, Q_INT64 nValue,
+                                 bool bPersistent, bool bGlobal,
+                                 bool bNLS )
+{
+  writeEntry( pKey, QString::number(nValue), bPersistent, bGlobal, bNLS );
+}
+
+void KConfigBase::writeEntry( const char *pKey, Q_INT64 nValue,
+                                 bool bPersistent, bool bGlobal,
+                                 bool bNLS )
+{
+  writeEntry( pKey, QString::number(nValue), bPersistent, bGlobal, bNLS );
+}
+
+
+void KConfigBase::writeEntry( const QString& pKey, Q_UINT64 nValue,
+                                 bool bPersistent, bool bGlobal,
+                                 bool bNLS )
+{
+  writeEntry( pKey, QString::number(nValue), bPersistent, bGlobal, bNLS );
+}
+
+void KConfigBase::writeEntry( const char *pKey, Q_UINT64 nValue,
+                                 bool bPersistent, bool bGlobal,
+                                 bool bNLS )
+{
+  writeEntry( pKey, QString::number(nValue), bPersistent, bGlobal, bNLS );
+}
 
 void KConfigBase::writeEntry( const QString& pKey, double nValue,
                                  bool bPersistent, bool bGlobal,
@@ -1670,6 +1670,75 @@ void KConfigBase::rollback( bool /*bDeep = true*/ )
   bDirty = false;
 }
 
+
+void KConfigBase::setReadDefaults(bool b)
+{
+  if (!d)
+  {
+     if (!b) return;
+     d = new KConfigBasePrivate();
+  }   
+     
+  d->readDefaults = b;
+}
+  
+bool KConfigBase::readDefaults() const
+{
+  return (d && d->readDefaults);
+}
+
+void KConfigBase::revertToDefault(const QString &key)
+{
+  setDirty(true);
+
+  KEntryKey aEntryKey(mGroup, key.utf8());
+  aEntryKey.bDefault = true;
+
+  if (!locale().isNull()) {
+    // try the localized key first
+    aEntryKey.bLocal = true;
+    KEntry entry = lookupData(aEntryKey);
+    if (entry.mValue.isNull())
+        entry.bDeleted = true;
+
+    entry.bDirty = true;
+    putData(aEntryKey, entry, true); // Revert
+    aEntryKey.bLocal = false;
+  }
+
+  // try the non-localized version
+  KEntry entry = lookupData(aEntryKey);
+  if (entry.mValue.isNull())
+     entry.bDeleted = true;
+  entry.bDirty = true;
+  putData(aEntryKey, entry, true); // Revert
+}
+
+bool KConfigBase::hasDefault(const QString &key) const
+{
+  KEntryKey aEntryKey(mGroup, key.utf8());
+  aEntryKey.bDefault = true;
+
+  if (!locale().isNull()) {
+    // try the localized key first
+    aEntryKey.bLocal = true;
+    KEntry entry = lookupData(aEntryKey);
+    if (!entry.mValue.isNull())
+        return true;
+
+    aEntryKey.bLocal = false;
+  }
+
+  // try the non-localized version
+  KEntry entry = lookupData(aEntryKey);
+  if (!entry.mValue.isNull())
+     return true;
+
+  return false;  
+}
+
+
+
 KConfigGroup::KConfigGroup(KConfigBase *master, const QString &group)
 {
   mMaster = master;
@@ -1680,6 +1749,7 @@ KConfigGroup::KConfigGroup(KConfigBase *master, const QString &group)
   bDirty = false; // Not used
   mGroup = group.utf8();
   aLocaleString = mMaster->aLocaleString;
+  setReadDefaults(mMaster->readDefaults());
 }
 
 KConfigGroup::KConfigGroup(KConfigBase *master, const QCString &group)
@@ -1692,6 +1762,7 @@ KConfigGroup::KConfigGroup(KConfigBase *master, const QCString &group)
   bDirty = false; // Not used
   mGroup = group;
   aLocaleString = mMaster->aLocaleString;
+  setReadDefaults(mMaster->readDefaults());
 }
 
 KConfigGroup::KConfigGroup(KConfigBase *master, const char * group)
@@ -1704,6 +1775,7 @@ KConfigGroup::KConfigGroup(KConfigBase *master, const char * group)
   bDirty = false; // Not used
   mGroup = group;
   aLocaleString = mMaster->aLocaleString;
+  setReadDefaults(mMaster->readDefaults());
 }
 
 void KConfigGroup::deleteGroup(bool bGlobal)
@@ -1711,9 +1783,9 @@ void KConfigGroup::deleteGroup(bool bGlobal)
   mMaster->deleteGroup(KConfigBase::group(), false, bGlobal);
 }
 
-void KConfigGroup::setDirty(bool b)
+void KConfigGroup::setDirty(bool _bDirty)
 {
-  mMaster->setDirty(b);
+  mMaster->setDirty(_bDirty);
 }
 
 void KConfigGroup::putData(const KEntryKey &_key, const KEntry &_data, bool _checkGroup)
@@ -1737,5 +1809,12 @@ void KConfigBase::virtual_hook( int, void* )
 void KConfigGroup::virtual_hook( int id, void* data )
 { KConfigBase::virtual_hook( id, data ); }
 
-#include "kconfigbase.moc"
+bool KConfigBase::checkConfigFilesWritable(bool warnUser)
+{
+  if (backEnd)
+    return backEnd->checkConfigFilesWritable(warnUser);
+  else
+    return false;
+}
 
+#include "kconfigbase.moc"
