@@ -20,7 +20,7 @@
    Boston, MA 02111-1307, USA.
 */
 
-// $Id: kconfigbase.h,v 1.83.2.1 2003/06/22 21:39:31 deller Exp $
+// $Id: kconfigbase.h,v 1.104 2003/11/26 23:28:41 ossi Exp $
 
 #ifndef _KCONFIGBASE_H
 #define _KCONFIGBASE_H
@@ -45,8 +45,8 @@ class KConfigGroup;
  *
  * This class forms the base for all KDE configuration. It is an
  * abstract base class, meaning that you cannot directly instantiate
- * objects of this class. Either use @ref KConfig (for usual KDE
- * configuration) or @ref KSimpleConfig (for special needs as in ksamba), or
+ * objects of this class. Either use KConfig (for usual KDE
+ * configuration) or KSimpleConfig (for special needs as in ksamba), or
  * even KSharedConfig (stores values in shared memory).
  *
  * All configuration entries are key, value pairs.  Each entry also
@@ -63,7 +63,7 @@ class KConfigGroup;
  * group name.
  *
  * @author Kalle Dalheimer <kalle@kde.org>, Preston Brown <pbrown@kde.org>
- * @version $Id: kconfigbase.h,v 1.83.2.1 2003/06/22 21:39:31 deller Exp $
+ * @version $Id: kconfigbase.h,v 1.104 2003/11/26 23:28:41 ossi Exp $
  * @see  KGlobal#config()  KConfig  KSimpleConfig
  * @short KDE Configuration Management abstract base class
  */
@@ -90,7 +90,7 @@ public:
    * Specifies the group in which keys will be read and written.
    *
    *  Subsequent
-   * calls to @ref readEntry() and @ref writeEntry() will be aplied only in the
+   * calls to readEntry() and writeEntry() will be aplied only in the
    * activated group.
    *
    * Switch back to the default group by passing a null string.
@@ -200,7 +200,7 @@ public:
    *         found.
    */
   QVariant readPropertyEntry( const QString& pKey,
-                              const QVariant &adefault) const;
+                              const QVariant &aDefault) const;
 
   /**
    * Reads the value of an entry specified by @p pKey in the current group.
@@ -437,6 +437,58 @@ public:
    */
   unsigned long readUnsignedLongNumEntry( const char *pKey, unsigned long nDefault = 0 ) const;
 
+  /**
+   * Reads a 64-bit numerical value.
+   *
+   * Read the value of an entry specified by @p pKey in the current group
+   * and interpret it numerically.
+   *
+   * @param pKey The key to search for.
+   * @param nDefault A default value returned if the key was not found or if
+   * the read value cannot be interpreted.
+   * @return The value for this key.
+   */
+  Q_INT64 readNum64Entry( const QString& pKey, Q_INT64 nDefault = 0 ) const;
+
+  /**
+   * Reads a 64-bit numerical value.
+   *
+   * Read the value of an entry specified by @p pKey in the current group
+   * and interpret it numerically.
+   *
+   * @param pKey The key to search for.
+   * @param nDefault A default value returned if the key was not found or if
+   * the read value cannot be interpreted.
+   * @return The value for this key.
+   */
+  Q_INT64 readNum64Entry( const char *pKey, Q_INT64 nDefault = 0 ) const;
+
+  /**
+   * Read an 64-bit unsigned numerical value.
+   *
+   * Read the value of an entry specified by @p pKey in the current group
+   * and interpret it numerically.
+   *
+   * @param pKey The key to search for.
+   * @param nDefault A default value returned if the key was not found or if
+   * the read value cannot be interpreted.
+   * @return The value for this key.
+   */
+  Q_UINT64 readUnsignedNum64Entry( const QString& pKey, Q_UINT64 nDefault = 0 ) const;
+
+  /**
+   * Read an 64-bit unsigned numerical value.
+   *
+   * Read the value of an entry specified by @p pKey in the current group
+   * and interpret it numerically.
+   *
+   * @param pKey The key to search for.
+   * @param nDefault A default value returned if the key was not found or if
+   * the read value cannot be interpreted.
+   * @return The value for this key.
+   */
+  Q_UINT64 readUnsignedNum64Entry( const char *pKey, Q_UINT64 nDefault = 0 ) const;
+
   /**
    * Reads a floating point value.
    *
@@ -464,7 +516,7 @@ public:
   double readDoubleNumEntry( const char *pKey, double nDefault = 0.0 ) const;
 
   /**
-   * Reads a @ref QFont value.
+   * Reads a QFont value.
    *
    * Read the value of an entry specified by @p pKey in the current group
    * and interpret it as a font object.
@@ -477,7 +529,7 @@ public:
   QFont readFontEntry( const QString& pKey, const QFont* pDefault = 0L ) const;
 
   /**
-   * Reads a @ref QFont value.
+   * Reads a QFont value.
    *
    * Read the value of an entry specified by @p pKey in the current group
    * and interpret it as a font object.
@@ -516,10 +568,10 @@ public:
   bool readBoolEntry( const char *pKey, const bool bDefault = false ) const;
 
   /**
-   * Reads a @ref QRect entry.
+   * Reads a QRect entry.
    *
    * Read the value of an entry specified by pKey in the current group
-   * and interpret it as a @ref QRect object.
+   * and interpret it as a QRect object.
    *
    * @param pKey The key to search for
    * @param pDefault A default value (null QRect by default) returned if the
@@ -529,10 +581,10 @@ public:
   QRect readRectEntry( const QString& pKey, const QRect* pDefault = 0L ) const;
 
   /**
-   * Reads a @ref QRect entry.
+   * Reads a QRect entry.
    *
    * Read the value of an entry specified by pKey in the current group
-   * and interpret it as a @ref QRect object.
+   * and interpret it as a QRect object.
    *
    * @param pKey The key to search for
    * @param pDefault A default value (null QRect by default) returned if the
@@ -542,10 +594,10 @@ public:
   QRect readRectEntry( const char *pKey, const QRect* pDefault = 0L ) const;
 
   /**
-   * Reads a @ref QPoint entry.
+   * Reads a QPoint entry.
    *
    * Read the value of an entry specified by @p pKey in the current group
-   * and interpret it as a @ref QPoint object.
+   * and interpret it as a QPoint object.
    *
    * @param pKey The key to search for
    * @param pDefault A default value (null QPoint by default) returned if the
@@ -555,10 +607,10 @@ public:
   QPoint readPointEntry( const QString& pKey, const QPoint* pDefault = 0L ) const;
 
   /**
-   * Reads a @ref QPoint entry.
+   * Reads a QPoint entry.
    *
    * Read the value of an entry specified by @p pKey in the current group
-   * and interpret it as a @ref QPoint object.
+   * and interpret it as a QPoint object.
    *
    * @param pKey The key to search for
    * @param pDefault A default value (null QPoint by default) returned if the
@@ -568,10 +620,10 @@ public:
   QPoint readPointEntry( const char *pKey, const QPoint* pDefault = 0L ) const;
 
   /**
-   * Reads a @ref QSize entry.
+   * Reads a QSize entry.
    *
    * Read the value of an entry specified by @p pKey in the current group
-   * and interpret it as a @ref QSize object.
+   * and interpret it as a QSize object.
    *
    * @param pKey The key to search for
    * @param pDefault A default value (null QSize by default) returned if the
@@ -581,10 +633,10 @@ public:
   QSize readSizeEntry( const QString& pKey, const QSize* pDefault = 0L ) const;
 
   /**
-   * Reads a @ref QSize entry.
+   * Reads a QSize entry.
    *
    * Read the value of an entry specified by @p pKey in the current group
-   * and interpret it as a @ref QSize object.
+   * and interpret it as a QSize object.
    *
    * @param pKey The key to search for
    * @param pDefault A default value (null QSize by default) returned if the
@@ -595,7 +647,7 @@ public:
 
 
   /**
-   * Reads a @ref QColor entry.
+   * Reads a QColor entry.
    *
    * Read the value of an entry specified by @p pKey in the current group
    * and interpret it as a color.
@@ -608,7 +660,7 @@ public:
   QColor readColorEntry( const QString& pKey, const QColor* pDefault = 0L ) const;
 
   /**
-   * Reads a @ref QColor entry.
+   * Reads a QColor entry.
    *
    * Read the value of an entry specified by @p pKey in the current group
    * and interpret it as a color.
@@ -621,13 +673,13 @@ public:
   QColor readColorEntry( const char *pKey, const QColor* pDefault = 0L ) const;
 
   /**
-   * Reads a @ref QDateTime entry.
+   * Reads a QDateTime entry.
    *
    * Read the value of an entry specified by @p pKey in the current group
    * and interpret it as a date and time.
    *
    * @param pKey The key to search for.
-   * @param pDefault A default value (@ref currentDateTime() by default)
+   * @param pDefault A default value ( currentDateTime() by default)
    * returned if the key was not found or if the read value cannot be
    * interpreted.
    * @return The value for this key.
@@ -635,13 +687,13 @@ public:
   QDateTime readDateTimeEntry( const QString& pKey, const QDateTime* pDefault = 0L ) const;
 
   /**
-   * Reads a @ref QDateTime entry.
+   * Reads a QDateTime entry.
    *
    * Read the value of an entry specified by @p pKey in the current group
    * and interpret it as a date and time.
    *
    * @param pKey The key to search for.
-   * @param pDefault A default value (@ref currentDateTime() by default)
+   * @param pDefault A default value ( currentDateTime() by default)
    * returned if the key was not found or if the read value cannot be
    * interpreted.
    * @return The value for this key.
@@ -674,7 +726,7 @@ public:
    * Writes a key/value pair.
    *
    * This is stored in the most specific config file when destroying the
-   * config object or when calling @ref sync().
+   * config object or when calling sync().
    *
    * @param pKey         The key to write.
    * @param pValue       The value to write.
@@ -695,7 +747,7 @@ public:
    * Writes a key/value pair.
    *
    * This is stored in the most specific config file when destroying the
-   * config object or when calling @ref sync().
+   * config object or when calling sync().
    *
    * @param pKey         The key to write.
    * @param pValue       The value to write.
@@ -713,9 +765,9 @@ public:
                       bool bNLS = false );
 
   /**
-   * @ref writeEntry() Overridden to accept a property.
+   * writeEntry() Overridden to accept a property.
    *
-   * Note: Unlike the other @ref writeEntry() functions, the old value is
+   * Note: Unlike the other writeEntry() functions, the old value is
    * _not_ returned here!
    *
    * @param pKey The key to write
@@ -735,9 +787,9 @@ public:
                     bool bPersistent = true, bool bGlobal = false,
                     bool bNLS = false );
   /**
-   * @ref writeEntry() Overridden to accept a property.
+   * writeEntry() Overridden to accept a property.
    *
-   * Note: Unlike the other @ref writeEntry() functions, the old value is
+   * Note: Unlike the other writeEntry() functions, the old value is
    * _not_ returned here!
    *
    * @param pKey The key to write
@@ -758,13 +810,14 @@ public:
                     bool bNLS = false );
 
   /**
-   * @ref writeEntry() overridden to accept a list of strings.
+   * writeEntry() overridden to accept a list of strings.
    *
-   * Note: Unlike the other @ref writeEntry() functions, the old value is
+   * Note: Unlike the other writeEntry() functions, the old value is
    * _not_ returned here!
    *
    * @param pKey The key to write
    * @param rValue The list to write
+   * @param sep  The list separator (default is ",").
    * @param bPersistent If @p bPersistent is false, the entry's dirty flag
    *                    will not be set and thus the entry will not be
    *                    written to disk at deletion time.
@@ -779,13 +832,14 @@ public:
   void writeEntry( const QString& pKey, const QStrList &rValue,
 		   char sep = ',', bool bPersistent = true, bool bGlobal = false, bool bNLS = false );
   /**
-   * @ref writeEntry() overridden to accept a list of strings.
+   * writeEntry() overridden to accept a list of strings.
    *
-   * Note: Unlike the other @ref writeEntry() functions, the old value is
+   * Note: Unlike the other writeEntry() functions, the old value is
    * _not_ returned here!
    *
    * @param pKey The key to write
    * @param rValue The list to write
+   * @param sep  The list separator (default is ",").
    * @param bPersistent If @p bPersistent is false, the entry's dirty flag
    *                    will not be set and thus the entry will not be
    *                    written to disk at deletion time.
@@ -801,13 +855,14 @@ public:
 		   char sep = ',', bool bPersistent = true, bool bGlobal = false, bool bNLS = false );
 
   /**
-   * @ref writeEntry() overridden to accept a list of strings.
+   * writeEntry() overridden to accept a list of strings.
    *
-   * Note: Unlike the other @ref writeEntry() functions, the old value is
+   * Note: Unlike the other writeEntry() functions, the old value is
    * _not_ returned here!
    *
    * @param pKey The key to write
    * @param rValue The list to write
+   * @param sep  The list separator (default is ",").
    * @param bPersistent If @p bPersistent is false, the entry's dirty flag
    *                    will not be set and thus the entry will not be
    *                    written to disk at deletion time.
@@ -822,13 +877,14 @@ public:
   void writeEntry( const QString& pKey, const QStringList &rValue,
 		   char sep = ',', bool bPersistent = true, bool bGlobal = false, bool bNLS = false );
   /**
-   * @ref writeEntry() overridden to accept a list of strings.
+   * writeEntry() overridden to accept a list of strings.
    *
-   * Note: Unlike the other @ref writeEntry() functions, the old value is
+   * Note: Unlike the other writeEntry() functions, the old value is
    * _not_ returned here!
    *
    * @param pKey The key to write
    * @param rValue The list to write
+   * @param sep  The list separator (default is ",").
    * @param bPersistent If @p bPersistent is false, the entry's dirty flag
    *                    will not be set and thus the entry will not be
    *                    written to disk at deletion time.
@@ -845,9 +901,9 @@ public:
 
 
  /**
-   * @ref writeEntry() overridden to accept a list of Integers.
+   * writeEntry() overridden to accept a list of Integers.
    *
-   * Note: Unlike the other @ref writeEntry() functions, the old value is
+   * Note: Unlike the other writeEntry() functions, the old value is
    * _not_ returned here!
    *
    * @param pKey The key to write
@@ -866,9 +922,9 @@ public:
   void writeEntry( const QString& pKey, const QValueList<int>& rValue,
 		   bool bPersistent = true, bool bGlobal = false, bool bNLS = false );
  /**
-   * @ref writeEntry() overridden to accept a list of Integers.
+   * writeEntry() overridden to accept a list of Integers.
    *
-   * Note: Unlike the other @ref writeEntry() functions, the old value is
+   * Note: Unlike the other writeEntry() functions, the old value is
    * _not_ returned here!
    *
    * @param pKey The key to write
@@ -891,7 +947,7 @@ public:
    * Write a (key/value) pair.
    *
    * This is stored to the most specific config file when destroying the
-   * config object or when calling @ref sync().
+   * config object or when calling sync().
    *
    *  @param pKey               The key to write.
    *  @param pValue     The value to write.
@@ -912,7 +968,7 @@ public:
    * Write a (key/value) pair.
    *
    * This is stored to the most specific config file when destroying the
-   * config object or when calling @ref sync().
+   * config object or when calling sync().
    *
    *  @param pKey               The key to write.
    *  @param pValue     The value to write.
@@ -1074,6 +1130,76 @@ public:
                       bool bPersistent = true, bool bGlobal = false,
                       bool bNLS = false );
 
+  /**
+   * Writes a (key/value) pair.
+   * Same as above, but write a 64-bit numerical value.
+   *
+   * @param pKey The key to write.
+   * @param nValue The value to write.
+   * @param bPersistent If @p bPersistent is false, the entry's dirty
+   * flag will not be set and thus the entry will not be written to
+   * disk at deletion time.
+   * @param bGlobal     If @p bGlobal is true, the pair is not saved to the
+   *  application specific config file, but to the global KDE config file.
+   * @param bNLS        If @p bNLS is true, the locale tag is added to the key
+   *  when writing it back.
+   */
+  void writeEntry( const QString& pKey, Q_LLONG nValue,
+                      bool bPersistent = true, bool bGlobal = false,
+                      bool bNLS = false );
+  /**
+   * Writes a (key/value) pair.
+   * Same as above, but write a long numerical value.
+   *
+   * @param pKey The key to write.
+   * @param nValue The value to write.
+   * @param bPersistent If @p bPersistent is false, the entry's dirty
+   * flag will not be set and thus the entry will not be written to
+   * disk at deletion time.
+   * @param bGlobal     If @p bGlobal is true, the pair is not saved to the
+   *  application specific config file, but to the global KDE config file.
+   * @param bNLS        If @p bNLS is true, the locale tag is added to the key
+   *  when writing it back.
+   */
+  void writeEntry( const char *pKey, Q_LLONG nValue,
+                      bool bPersistent = true, bool bGlobal = false,
+                      bool bNLS = false );
+
+  /**
+   * Writes a (key/value) pair.
+   * Same as above, but writes an unsigned 64-bit numerical value.
+   *
+   * @param pKey The key to write.
+   * @param nValue The value to write.
+   * @param bPersistent If @p bPersistent is false, the entry's dirty
+   * flag will not be set and thus the entry will not be written to
+   * disk at deletion time.
+   * @param bGlobal     If @p bGlobal is true, the pair is not saved to the
+   *  application specific config file, but to the global KDE config file.
+   * @param bNLS        If @p bNLS is true, the locale tag is added to the key
+   *  when writing it back.
+   */
+  void writeEntry( const QString& pKey, Q_ULLONG nValue,
+                      bool bPersistent = true, bool bGlobal = false,
+                      bool bNLS = false );
+  /**
+   * Writes a (key/value) pair.
+   * Same as above, but writes an unsigned 64-bit numerical value.
+   *
+   * @param pKey The key to write.
+   * @param nValue The value to write.
+   * @param bPersistent If @p bPersistent is false, the entry's dirty
+   * flag will not be set and thus the entry will not be written to
+   * disk at deletion time.
+   * @param bGlobal     If @p bGlobal is true, the pair is not saved to the
+   *  application specific config file, but to the global KDE config file.
+   * @param bNLS        If @p bNLS is true, the locale tag is added to the key
+   *  when writing it back.
+   */
+  void writeEntry( const char *pKey, Q_ULLONG nValue,
+                      bool bPersistent = true, bool bGlobal = false,
+                      bool bNLS = false );
+
   /**
    * Writes a (key/value) pair.
    * Same as above, but writes a floating-point value.
@@ -1191,11 +1317,11 @@ public:
    * Writes a (key/value) pair.
    * Same as above, but write a color entry.
    *
-   * Note: Unlike the other @ref writeEntry() functions, the old value is
+   * Note: Unlike the other writeEntry() functions, the old value is
    * _not_ returned here!
    *
    * @param pKey The key to write.
-   * @param rValue The color value to write.
+   * @param rColor The color value to write.
    * @param bPersistent If @p bPersistent is false, the entry's dirty
    * flag will not be set and thus the entry will not be written to
    * disk at deletion time.
@@ -1211,11 +1337,11 @@ public:
    * Writes a (key/value) pair.
    * Same as above, but write a color entry.
    *
-   * Note: Unlike the other @ref writeEntry() functions, the old value is
+   * Note: Unlike the other writeEntry() functions, the old value is
    * _not_ returned here!
    *
    * @param pKey The key to write.
-   * @param rValue The color value to write.
+   * @param rColor The color value to write.
    * @param bPersistent If @p bPersistent is false, the entry's dirty
    * flag will not be set and thus the entry will not be written to
    * disk at deletion time.
@@ -1232,11 +1358,11 @@ public:
    * Writes a (key/value) pair.
    * Same as above, but writes a date and time entry.
    *
-   * Note: Unlike the other @ref writeEntry() functions, the old value is
+   * Note: Unlike the other writeEntry() functions, the old value is
    * @em not returned here!
    *
    * @param pKey The key to write.
-   * @param rValue The date and time value to write.
+   * @param rDateTime The date and time value to write.
    * @param bPersistent If @p bPersistent is false, the entry's dirty
    * flag will not be set and thus the entry will not be written to
    * disk at deletion time.
@@ -1252,11 +1378,11 @@ public:
    * Writes a (key/value) pair.
    * Same as above, but writes a date and time entry.
    *
-   * Note: Unlike the other @ref writeEntry() functions, the old value is
+   * Note: Unlike the other writeEntry() functions, the old value is
    * @em not returned here!
    *
    * @param pKey The key to write.
-   * @param rValue The date and time value to write.
+   * @param rDateTime The date and time value to write.
    * @param bPersistent If @p bPersistent is false, the entry's dirty
    * flag will not be set and thus the entry will not be written to
    * disk at deletion time.
@@ -1274,7 +1400,7 @@ public:
    * Writes a (key/value) pair.
    * Same as above, but writes a rectangle.
    *
-   * Note: Unlike the other @ref writeEntry() functions, the old value is
+   * Note: Unlike the other writeEntry() functions, the old value is
    * _not_ returned here!
    *
    * @param pKey The key to write.
@@ -1294,7 +1420,7 @@ public:
    * Writes a (key/value) pair.
    * Same as above, but writes a rectangle.
    *
-   * Note: Unlike the other @ref writeEntry() functions, the old value is
+   * Note: Unlike the other writeEntry() functions, the old value is
    * _not_ returned here!
    *
    * @param pKey The key to write.
@@ -1315,7 +1441,7 @@ public:
    * Writes a (key/value) pair.
    * Same as above, but writes a point.
    *
-   * Note: Unlike the other @ref writeEntry() functions, the old value is
+   * Note: Unlike the other writeEntry() functions, the old value is
    * _not_ returned here!
    *
    * @param pKey The key to write.
@@ -1335,7 +1461,7 @@ public:
    * Writes a (key/value) pair.
    * Same as above, but writes a point.
    *
-   * Note: Unlike the other @ref writeEntry() functions, the old value is
+   * Note: Unlike the other writeEntry() functions, the old value is
    * _not_ returned here!
    *
    * @param pKey The key to write.
@@ -1356,7 +1482,7 @@ public:
    * Writes a (key/value) pair.
    * Same as above, but writes a size.
    *
-   * Note: Unlike the other @ref writeEntry() functions, the old value is
+   * Note: Unlike the other writeEntry() functions, the old value is
    * _not_ returned here!
    *
    * @param pKey The key to write.
@@ -1376,7 +1502,7 @@ public:
    * Writes a (key/value) pair.
    * Same as above, but writes a size.
    *
-   * Note: Unlike the other @ref writeEntry() functions, the old value is
+   * Note: Unlike the other writeEntry() functions, the old value is
    * _not_ returned here!
    *
    * @param pKey The key to write.
@@ -1398,7 +1524,7 @@ public:
    *
    * It is checked whether the path is located under $HOME. If so the
    * path is written out with the user's home-directory replaced with
-   * $HOME. The path should be read back with @ref readPathEntry()
+   * $HOME. The path should be read back with readPathEntry()
    *
    * @param pKey The key to write.
    * @param path The path to write.
@@ -1418,7 +1544,7 @@ public:
    *
    * It is checked whether the path is located under $HOME. If so the
    * path is written out with the user's home-directory replaced with
-   * $HOME. The path should be read back with @ref readPathEntry()
+   * $HOME. The path should be read back with readPathEntry()
    *
    * @param pKey The key to write.
    * @param path The path to write.
@@ -1435,14 +1561,15 @@ public:
                        bool bNLS = false );
 
   /**
-   * @ref writePathEntry() overridden to accept a list of paths (strings).
+   * writePathEntry() overridden to accept a list of paths (strings).
    *
-   * It is checked whether the paths are located under $HOME. If so each of 
+   * It is checked whether the paths are located under $HOME. If so each of
    * the paths are written out with the user's home-directory replaced with
-   * $HOME. The paths should be read back with @ref readPathListEntry()
+   * $HOME. The paths should be read back with readPathListEntry()
    *
    * @param pKey The key to write
    * @param rValue The list to write
+   * @param sep  The list separator (default is ",").
    * @param bPersistent If @p bPersistent is false, the entry's dirty flag
    *                    will not be set and thus the entry will not be
    *                    written to disk at deletion time.
@@ -1459,14 +1586,15 @@ public:
   void writePathEntry( const QString& pKey, const QStringList &rValue,
 		   char sep = ',', bool bPersistent = true, bool bGlobal = false, bool bNLS = false );
   /**
-   * @ref writePathEntry() overridden to accept a list of paths (strings).
+   * writePathEntry() overridden to accept a list of paths (strings).
    *
-   * It is checked whether the paths are located under $HOME. If so each of 
+   * It is checked whether the paths are located under $HOME. If so each of
    * the paths are written out with the user's home-directory replaced with
-   * $HOME. The paths should be read back with @ref readPathListEntry()
+   * $HOME. The paths should be read back with readPathListEntry()
    *
    * @param pKey The key to write
    * @param rValue The list to write
+   * @param sep  The list separator (default is ",").
    * @param bPersistent If @p bPersistent is false, the entry's dirty flag
    *                    will not be set and thus the entry will not be
    *                    written to disk at deletion time.
@@ -1545,12 +1673,12 @@ public:
    * Mark the config object as "clean," i.e. don't write dirty entries
    * at destruction time. If @p bDeep is false, only the global dirty
    * flag of the KConfig object gets cleared. If you then call
-   * @ref writeEntry() again, the global dirty flag is set again and all
-   * dirty entries will be written at a subsequent @ref sync() call.
+   * writeEntry() again, the global dirty flag is set again and all
+   * dirty entries will be written at a subsequent sync() call.
    *
    * Classes that derive from KConfigBase should override this
    * method and implement storage-specific behavior, as well as
-   * calling the @ref KConfigBase::rollback() explicitly in the initializer.
+   * calling the KConfigBase::rollback() explicitly in the initializer.
    *
    * @param bDeep If true, the dirty flags of all entries are cleared,
    *        as well as the global dirty flag.
@@ -1563,12 +1691,12 @@ public:
    * written to the most specific file available.
    *
    * Asks the back end to flush out all pending writes, and then calls
-   * @ref rollback().  No changes are made if the object has @p readOnly
+   * rollback().  No changes are made if the object has @p readOnly
    * status.
    *
    * You should call this from your destructor in derivative classes.
    *
-   * @see #rollback(), #isReadOnly()
+   * @see rollback(), #isReadOnly()
    */
   virtual void sync();
 
@@ -1582,7 +1710,7 @@ public:
    * Sets the config object's read-only status.
    *
    * @param _ro If true, the config object will not write out any
-   *        changes to disk even if it is destroyed or @ref sync() is called.
+   *        changes to disk even if it is destroyed or sync() is called.
    *
    */
   virtual void setReadOnly(bool _ro) { bReadOnly = _ro; }
@@ -1600,7 +1728,7 @@ public:
    * group (hasKey() returns false). Keys with null data are considered
    * nonexistent.
    *
-   * @param pKey The key to search for.
+   * @param key The key to search for.
    * @return If true, the key is available.
    */
   bool hasKey( const QString& key ) const;
@@ -1624,7 +1752,7 @@ public:
    * appends the default group.
    *
    * Derivative classes should clear any internal data structures and
-   * then simply call @ref parseConfigFiles() when implementing this
+   * then simply call parseConfigFiles() when implementing this
    * method.
    *
    * @see  parseConfigFiles()
@@ -1647,14 +1775,14 @@ public:
 
   /**
    * Checks whether it is possible to change the given entry.
-   * @param the key to check
+   * @param key the key to check
    * @return whether the entry @p key may be changed in the current group
    * in this configuration file.
    */
   bool entryIsImmutable(const QString &key) const;
 
   /**
-   * Possible return values for @ref getConfigState().
+   * Possible return values for getConfigState().
    *
    * @see  getConfigState()
    */
@@ -1675,11 +1803,69 @@ public:
    */
   ConfigState getConfigState() const;
 
+  /**
+   * Check whether the config files are writable.
+   * @param warnUser Warn the user if the configuration files are not writable.
+   * @return Indicates that all of the configuration files used are writable.
+   * @since 3.2
+   */
+  bool checkConfigFilesWritable(bool warnUser);
+
+  /**
+   * When set, all readEntry and readXXXEntry calls return the system
+   * wide (default) values instead of the user's preference.
+   * This is off by default.
+   * @since 3.2
+   */
+  void setReadDefaults(bool b);
+
+  /**
+   * @returns true if all readEntry and readXXXEntry calls return the system
+   * wide (default) values instead of the user's preference.
+   * @since 3.2
+   */
+  bool readDefaults() const;
+
+  /**
+   * Reverts the entry with key @p key in the current group in the
+   * application specific config file to either the system wide (default)
+   * value or the value specified in the global KDE config file.
+   *
+   * To revert entries in the global KDE config file, the global KDE config
+   * file should be opened explicitly in a separate config object.
+   *
+   * @param key The key of the entry to revert.
+   * @since 3.2
+   */
+  void revertToDefault(const QString &key);
+
+  /**
+   * Returns whether a default is specified for an entry in either the
+   * system wide configuration file or the global KDE config file.
+   *
+   * If an application uses a dynamically determined default value for
+   * a certain entry, it may wish to make the following check before
+   * writing back changes:
+   * \code
+   * if ( (value == dynamicDefault) && !config->hasDefault(key) )
+   *    revertToDefault(key)
+   * else
+   *    writeEntry(key, value)
+   * \endcode
+   *
+   * This ensures that as long as the entry is not modified to differ
+   * from the dynamically determined default, the application will
+   * follow changes of the default.
+   * @param key The key of the entry to check.
+   * @since 3.2
+   */
+  bool hasDefault(const QString &key) const;
+
 protected:
   /**
    * Reads the locale and put in the configuration data struct.
    * Note that this should be done in the constructor, but this is not
-   * possible due to some mutual dependencies in @ref KApplication::init()
+   * possible due to some mutual dependencies in KApplication::init()
    */
   void setLocale();
 
@@ -1754,7 +1940,7 @@ protected:
    *
    * @param _key The key to look up  It contains information both on
    *        the group of the key and the entry's key itself.
-   * @return The @ref KEntry value (data) found for the key.  @p KEntry.aValue
+   * @return The KEntry value (data) found for the key.  @p KEntry.aValue
    * will be the null string if nothing was located.
    * @internal
    */
@@ -1799,20 +1985,21 @@ protected:
 protected:
   virtual void virtual_hook( int id, void* data );
 private:
+  class KConfigBasePrivate;
   KConfigBasePrivate *d;
 };
 
 class KConfigGroupSaverPrivate;
 
 /**
-  * Helper class to facilitate working with @ref KConfig / @ref KSimpleConfig
+  * Helper class to facilitate working with KConfig / KSimpleConfig
   * groups.
   *
   * Careful programmers always set the group of a
-  * @ref KConfig @ref KSimpleConfig object to the group they want to read from
+  * KConfig KSimpleConfig object to the group they want to read from
   * and set it back to the old one of afterwards. This is usually
   * written as:
-  * <pre>
+  * \code
   *
   * QString oldgroup config->group();
   * config->setGroup( "TheGroupThatIWant" );
@@ -1820,7 +2007,7 @@ class KConfigGroupSaverPrivate;
   * config->writeEntry( "Blah", "Blubb" );
   *
   * config->setGroup( oldgroup );
-  * </pre>
+  * \endcode
   *
   * In order to facilitate this task, you can use
   * KConfigGroupSaver. Simply construct such an object ON THE STACK
@@ -1832,7 +2019,7 @@ class KConfigGroupSaverPrivate;
   * in one block.
   *
   * @author Matthias Kalle Dalheimer <kalle@kde.org>
-  * @version $Id: kconfigbase.h,v 1.83.2.1 2003/06/22 21:39:31 deller Exp $
+  * @version $Id: kconfigbase.h,v 1.104 2003/11/26 23:28:41 ossi Exp $
   * @see KConfigBase, KConfig, KSimpleConfig
   * @short Helper class for easier use of KConfig/KSimpleConfig groups
   */
@@ -1844,11 +2031,13 @@ public:
    * Constructor. You pass a pointer to the KConfigBase-derived
    * object you want to work with and a string indicating the _new_
    * group.
+   *
    * @param config The KConfigBase-derived object this
    *               KConfigGroupSaver works on.
    * @param group  The new group that the config object should switch to.
    */
   KConfigGroupSaver( KConfigBase* config, QString group )
+      /* KDE 4 : make the second parameter const QString & */
       : _config(config), _oldgroup(config->group())
         { _config->setGroup( group ); }
 
@@ -1876,6 +2065,9 @@ private:
 
 class KConfigGroupPrivate;
 
+/**
+ * A KConfigBase derived class for one specific group in a KConfig object.
+ */
 class KConfigGroup: public KConfigBase
 {
 public:
