@@ -1,5 +1,6 @@
 /* This file is part of the KDE libraries
     Copyright (C) 1997 Matthias Kalle Dalheimer (kalle@kde.org)
+                  2002 Holger Freyther (freyther@kde.org)
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Library General Public
@@ -17,10 +18,6 @@
     Boston, MA 02111-1307, USA.
 */
 
-// Include our header without NDEBUG defined to avoid having the kDebugInfo
-// functions inlined to noops (which would then conflict with their definition
-// here).
-
 #include "kdebug.h"
 
 #ifdef NDEBUG
@@ -28,16 +25,28 @@
 #undef kdBacktrace
 #endif
 
+#include "kdebugdcopiface.h"
+
 #include "kapplication.h"
 #include "kglobal.h"
 #include "kinstance.h"
 #include "kstandarddirs.h"
+
 #include <qmessagebox.h>
 #include <klocale.h>
 #include <qfile.h>
 #include <qintdict.h>
 #include <qstring.h>
-#include <qtextstream.h>
+#include <qdatetime.h>
+#include <qpoint.h>
+#include <qrect.h>
+#include <qregion.h>
+#include <qstringlist.h>
+#include <qpen.h>
+#include <qbrush.h>
+#include <qsize.h>
+
+#include <kurl.h>
 
 #include <stdlib.h>	// abort
 #include <unistd.h>	// getpid
@@ -59,19 +68,19 @@ class KDebugEntry;
 class KDebugEntry
 {
 public:
-    KDebugEntry (int n, QString d) {number=n; descr=d;}
+    KDebugEntry (int n, const QCString& d) {number=n; descr=d;}
     unsigned int number;
-    QString descr;
+    QCString descr;
 };
 
 static QIntDict<KDebugEntry> *KDebugCache;
 
 static KStaticDeleter< QIntDict<KDebugEntry> > kdd;
 
-static QString getDescrFromNum(unsigned int _num)
+static QCString getDescrFromNum(unsigned int _num)
 {
   if (!KDebugCache) {
-    kdd.setObject(KDebugCache, new QIntDict<KDebugEntry>);
+    kdd.setObject(KDebugCache, new QIntDict<KDebugEntry>( 601 ));
     // Do not call this deleter from ~KApplication
     KGlobal::unregisterStaticDeleter(&kdd);
     KDebugCache->setAutoDelete(true);
@@ -82,66 +91,59 @@ static QString getDescrFromNum(unsigned int _num)
     return ent->descr;
 
   if ( !KDebugCache->isEmpty() ) // areas already loaded
-    return QString::null;
+    return QCString();
 
   QString filename(locate("config","kdebug.areas"));
+  if (filename.isEmpty())
+      return QCString();
+
   QFile file(filename);
   if (!file.open(IO_ReadOnly)) {
     qWarning("Couldn't open %s", filename.local8Bit().data());
     file.close();
-    return "";
+    return QCString();
   }
 
-  unsigned long number = 0;
-  bool longOK;
+  uint lineNumber=0;
+  QCString line(1024);
+  int len;
 
-  QTextStream *ts = new QTextStream(&file);
-  ts->setEncoding( QTextStream::Latin1 );
-  while (!ts->eof()) {
-    const QString data(ts->readLine());
-    int i = 0;
-    int len = data.length();
+  while (( len = file.readLine(line.data(),line.size()-1) ) > 0) {
+      int i=0;
+      ++lineNumber;
 
-    QChar ch = data[0];
-    if (ch == '#' || ch.isNull()) {
-      continue;
-    }
-    while (ch.isSpace()) {
-      if (!(i < len))
-	continue;
-      ++i;
-      ch = data[i];
-    }
-    if (ch.isNumber()) {
-	int numStart = i ;
-	while (ch.isNumber())  {
-	  if (!(i < len))
-	    continue;
-	  ++i;
-	  ch = data[i];
-	}
-	number = data.mid(numStart,i).toULong(&longOK);
-    }
-    while (ch.isSpace()) {
-      if (!(i < len))
-	continue;
-      ++i;
-      ch = data[i];
-    }
-    const QString description(data.mid(i, len));
-    //qDebug("number: [%i] description: [%s]", number, description.latin1());
+      while (line[i] && line[i] <= ' ')
+        i++;
 
-    KDebugCache->insert(number, new KDebugEntry(number,description));
+      unsigned char ch=line[i];
+
+      if ( !ch || ch =='#' || ch =='\n')
+          continue; // We have an eof, a comment or an empty line
+
+      if (ch < '0' && ch > '9') {
+          qWarning("Syntax error: no number (line %u)",lineNumber);
+          continue;
+      }
+
+      const int numStart=i;
+      do {
+          ch=line[++i];
+      } while ( ch >= '0' && ch <= '9');
+
+      const Q_ULONG number =line.mid(numStart,i).toULong();
+
+      while (line[i] && line[i] <= ' ')
+        i++;
+
+      KDebugCache->insert(number, new KDebugEntry(number, line.mid(i, len-i-1)));
   }
-
-  delete ts;
   file.close();
 
   ent = KDebugCache->find( _num );
   if ( ent )
       return ent->descr;
 
-  return QString::null;
+  return QCString();
 }
 
 enum DebugLevels {
@@ -153,18 +155,20 @@ enum DebugLevels {
 
 
 struct kDebugPrivate {
-  kDebugPrivate() : 
+  kDebugPrivate() :
   	oldarea(0), config(0) { }
-  	
+
   ~kDebugPrivate() { delete config; }
 
-  QString aAreaName;
+  QCString aAreaName;
   unsigned int oldarea;
   KConfig *config;
 };
 
 static kDebugPrivate *kDebug_data = 0;
 static KStaticDeleter<kDebugPrivate> pcd;
+static KStaticDeleter<KDebugDCOPIface> dcopsd;
+static KDebugDCOPIface* kDebugDCOPIface = 0;
 
 static void kDebugBackend( unsigned short nLevel, unsigned int nArea, const char *data)
 {
@@ -173,12 +177,23 @@ static void kDebugBackend( unsigned short nLevel, unsigned int nArea, const char
       pcd.setObject(kDebug_data, new kDebugPrivate());
       // Do not call this deleter from ~KApplication
       KGlobal::unregisterStaticDeleter(&pcd);
+
+      // create the dcop interface if it has not been created yet
+      if (!kDebugDCOPIface)
+      {
+          kDebugDCOPIface = dcopsd.setObject(kDebugDCOPIface, new KDebugDCOPIface);
+      }
   }
 
   if (!kDebug_data->config && KGlobal::_instance )
   {
       kDebug_data->config = new KConfig("kdebugrc", false, false);
       kDebug_data->config->setGroup("0");
+
+      //AB: this is necessary here, otherwise all output with area 0 won't be
+      //prefixed with anything, unless something with area != 0 is called before
+      if ( KGlobal::_instance )
+        kDebug_data->aAreaName = KGlobal::instance()->instanceName();
   }
 
   if (kDebug_data->config && kDebug_data->oldarea != nArea) {
@@ -196,32 +211,32 @@ static void kDebugBackend( unsigned short nLevel, unsigned int nArea, const char
 
     /* Determine output */
 
-    QString key;
-    switch( nLevel )
-      {
-      case KDEBUG_INFO:
-	key = "InfoOutput";
-	aCaption = "Info";
-	nPriority = LOG_INFO;
+  QString key;
+  switch( nLevel )
+  {
+  case KDEBUG_INFO:
+      key = "InfoOutput";
+      aCaption = "Info";
+      nPriority = LOG_INFO;
+      break;
+  case KDEBUG_WARN:
+      key = "WarnOutput";
+      aCaption = "Warning";
+      nPriority = LOG_WARNING;
 	break;
-      case KDEBUG_WARN:
-	key = "WarnOutput";
-	aCaption = "Warning";
-	nPriority = LOG_WARNING;
-	break;
-      case KDEBUG_FATAL:
-	key = "FatalOutput";
-	aCaption = "Fatal Error";
-	nPriority = LOG_CRIT;
-	break;
-      case KDEBUG_ERROR:
-      default:
-	/* Programmer error, use "Error" as default */
-	key = "ErrorOutput";
-	aCaption = "Error";
-	nPriority = LOG_ERR;
-	break;
-      }
+  case KDEBUG_FATAL:
+      key = "FatalOutput";
+      aCaption = "Fatal Error";
+      nPriority = LOG_CRIT;
+      break;
+  case KDEBUG_ERROR:
+  default:
+      /* Programmer error, use "Error" as default */
+      key = "ErrorOutput";
+      aCaption = "Error";
+      nPriority = LOG_ERR;
+      break;
+  }
 
   short nOutput = kDebug_data->config ? kDebug_data->config->readNumEntry(key, 2) : 2;
 
@@ -229,79 +244,73 @@ static void kDebugBackend( unsigned short nLevel, unsigned int nArea, const char
   // a messagebox.
   if (!kapp && (nOutput == 1))
     nOutput = 2;
+  else if ( nOutput == 4 && nLevel != KDEBUG_FATAL )
+      return;
+
+  const int BUFSIZE = 4096;
+  char buf[BUFSIZE];
+  int nSize;
+  if ( !kDebug_data->aAreaName.isEmpty() ) {
+      strlcpy( buf, kDebug_data->aAreaName.data(), BUFSIZE );
+      strlcat( buf, ": ", BUFSIZE );
+      strlcat( buf, data, BUFSIZE );
+      nSize = strlen( buf );
+  }
+  else
+      nSize = strlcpy( buf, data, BUFSIZE );
+
 
   // Output
   switch( nOutput )
-        {
-        case 0: // File
-          {
-                QString aKey;
-                switch( nLevel )
-                {
-                    case KDEBUG_INFO:
-                        aKey = "InfoFilename";
-                        break;
-                    case KDEBUG_WARN:
-                        aKey = "WarnFilename";
-                        break;
-                    case KDEBUG_FATAL:
-                        aKey = "FatalFilename";
-                        break;
-                    case KDEBUG_ERROR:
-                    default:
-                        aKey = "ErrorFilename";
-                        break;
-                }
-                QString aOutputFileName = kDebug_data->config->readPathEntry(aKey, "kdebug.dbg");
-
-                const int BUFSIZE = 4096;
-                char buf[BUFSIZE] = "";
-                buf[BUFSIZE-1] = '\0';
-		int nSize;
-                if ( !kDebug_data->aAreaName.isEmpty() )
-		    nSize = snprintf( buf, BUFSIZE-1, "%s: %s", kDebug_data->aAreaName.ascii(), data);
-		else
-		    nSize = snprintf( buf, BUFSIZE-1, "%s", data);
-
-                QFile aOutputFile( aOutputFileName );
-                aOutputFile.open( IO_WriteOnly | IO_Append );
-                if ( ( nSize == -1 ) || ( nSize >= BUFSIZE ) )
-                    aOutputFile.writeBlock( buf, BUFSIZE-1 );
-                else
-                    aOutputFile.writeBlock( buf, nSize );
-                aOutputFile.close();
-                break;
-          }
-        case 1: // Message Box
-          {
-                // Since we are in kdecore here, we cannot use KMsgBox and use
-                // QMessageBox instead
-	      if ( !kDebug_data->aAreaName.isEmpty() ) aCaption += QString("(")+kDebug_data->aAreaName+")";
-	      QMessageBox::warning( 0L, aCaption, data, i18n("&OK") );
-	      break;
-          }
-        case 2: // Shell
-          {
-              FILE *output;
-              /* we used to use stdout for debug
-              if (nPriority == LOG_INFO)
-                  output = stderr;
-              else */
-                  output = stderr;
-              // Uncomment this to get the pid of the app in the output (useful for e.g. kioslaves)
-	      // if ( !kDebug_data->aAreaName.isEmpty() ) fprintf( output, "%d %s: ", (int)getpid(), kDebug_data->aAreaName.ascii() );
-	      if ( !kDebug_data->aAreaName.isEmpty() ) fprintf( output, "%s: ", kDebug_data->aAreaName.ascii() );
-	      fputs(  data, output);
-	      break;
-          }
-        case 3: // syslog
-          {
-	      syslog( nPriority, "%s", data);
-          }
-        case 4: // nothing
-          {
-          }
-        }
+  {
+  case 0: // File
+  {
+      const char* aKey;
+      switch( nLevel )
+      {
+      case KDEBUG_INFO:
+          aKey = "InfoFilename";
+          break;
+      case KDEBUG_WARN:
+          aKey = "WarnFilename";
+          break;
+      case KDEBUG_FATAL:
+          aKey = "FatalFilename";
+          break;
+      case KDEBUG_ERROR:
+      default:
+          aKey = "ErrorFilename";
+          break;
+      }
+      QFile aOutputFile( kDebug_data->config->readPathEntry(aKey, "kdebug.dbg") );
+      aOutputFile.open( IO_WriteOnly | IO_Append | IO_Raw );
+      if ( ( nSize == -1 ) || ( nSize >= BUFSIZE ) )
+          aOutputFile.writeBlock( buf, BUFSIZE-1 );
+      else
+          aOutputFile.writeBlock( buf, nSize );
+      aOutputFile.close();
+      break;
+  }
+  case 1: // Message Box
+  {
+      // Since we are in kdecore here, we cannot use KMsgBox and use
+      // QMessageBox instead
+      if ( !kDebug_data->aAreaName.isEmpty() )
+          aCaption += QString("(%1)").arg( kDebug_data->aAreaName );
+      QMessageBox::warning( 0L, aCaption, data, i18n("&OK") );
+      break;
+  }
+  case 2: // Shell
+  {
+      write( 2, buf, nSize ); //fputs( buf, stderr );
+      break;
+  }
+  case 3: // syslog
+  {
+      syslog( nPriority, "%s", buf);
+      break;
+  }
+  }
 
   // check if we should abort
   if( ( nLevel == KDEBUG_FATAL )
@@ -332,8 +341,7 @@ kdbgstream &kdbgstream::form(const char *format, ...)
     char buf[4096];
     va_list arguments;
     va_start( arguments, format );
-    buf[sizeof(buf)-1] = '\0';
-    vsnprintf( buf, sizeof(buf)-1, format, arguments );
+    vsnprintf( buf, sizeof(buf), format, arguments );
     va_end(arguments);
     *this << buf;
     return *this;
@@ -342,6 +350,7 @@ kdbgstream &kdbgstream::form(const char *format, ...)
 kdbgstream::~kdbgstream() {
     if (!output.isEmpty()) {
 	fprintf(stderr, "ASSERT: debug output not ended with \\n\n");
+        fprintf(stderr, "%s", kdBacktrace().latin1());
 	*this << "\n";
     }
 }
@@ -359,6 +368,11 @@ kdbgstream& kdbgstream::operator << (char ch)
 }
 
 kdbgstream& kdbgstream::operator << (QWidget* widget)
+{
+    return *this << const_cast< const QWidget* >( widget );
+}
+
+kdbgstream& kdbgstream::operator << (const QWidget* widget)
 {
   QString string, temp;
   // -----
@@ -393,6 +407,108 @@ kdbgstream& kdbgstream::operator << (QWidget* widget)
     }
   return *this;
 }
+/*
+ * either use 'output' directly and do the flush if needed
+ * or use the QString operator which calls the char* operator
+ *
+ */
+kdbgstream& kdbgstream::operator<<( const QDateTime& time) {
+    *this << time.toString();
+    return *this;
+}
+kdbgstream& kdbgstream::operator<<( const QDate& date) {
+    *this << date.toString();
+
+    return *this;
+}
+kdbgstream& kdbgstream::operator<<( const QTime& time ) {
+    *this << time.toString();
+    return *this;
+}
+kdbgstream& kdbgstream::operator<<( const QPoint& p ) {
+    *this << "(" << p.x() << ", " << p.y() << ")";
+    return *this;
+}
+kdbgstream& kdbgstream::operator<<( const QSize& s ) {
+    *this << "[" << s.width() << "x" << s.height() << "]";
+    return *this;
+}
+kdbgstream& kdbgstream::operator<<( const QRect& r ) {
+    *this << "[" << r.x() << "," << r.y() << " - " << r.width() << "x" << r.height() << "]";
+    return *this;
+}
+kdbgstream& kdbgstream::operator<<( const QRegion& reg ) {
+    *this<< "[ ";
+
+    QMemArray<QRect>rs=reg.rects();
+    for (uint i=0;i<rs.size();++i)
+        *this << QString("[%1,%2 - %3x%4] ").arg(rs[i].x()).arg(rs[i].y()).arg(rs[i].width()).arg(rs[i].height() ) ;
+
+    *this <<"]";
+    return *this;
+}
+kdbgstream& kdbgstream::operator<<( const KURL& u ) {
+    *this << u.prettyURL();
+    return *this;
+}
+kdbgstream& kdbgstream::operator<<( const QStringList& l ) {
+    *this << "(";
+    *this << l.join(",");
+    *this << ")";
+
+    return *this;
+}
+kdbgstream& kdbgstream::operator<<( const QColor& c ) {
+    if ( c.isValid() )
+        *this <<c.name();
+    else
+        *this << "(invalid/default)";
+    return *this;
+}
+kdbgstream& kdbgstream::operator<<( const QPen& p ) {
+    static const char* const s_penStyles[] = {
+        "NoPen", "SolidLine", "DashLine", "DotLine", "DashDotLine",
+        "DashDotDotLine" };
+    static const char* const s_capStyles[] = {
+        "FlatCap", "SquareCap", "RoundCap" };
+    *this << "[ style:";
+    *this << s_penStyles[ p.style() ];
+    *this << " width:";
+    *this << p.width();
+    *this << " color:";
+    if ( p.color().isValid() )
+        *this << p.color().name();
+    else
+        *this <<"(invalid/default)";
+    if ( p.width() > 0 ) // cap style doesn't matter, otherwise
+    {
+        *this << " capstyle:";
+        *this << s_capStyles[ p.capStyle() >> 4 ];
+        // join style omitted
+    }
+    *this <<" ]";
+    return *this;
+}
+kdbgstream& kdbgstream::operator<<( const QBrush& b) {
+    static const char* const s_brushStyles[] = {
+        "NoBrush", "SolidPattern", "Dense1Pattern", "Dense2Pattern", "Dense3Pattern",
+        "Dense4Pattern", "Dense5Pattern", "Dense6Pattern", "Dense7Pattern",
+        "HorPattern", "VerPattern", "CrossPattern", "BDiagPattern", "FDiagPattern",
+        "DiagCrossPattern" };
+
+    *this <<"[ style: ";
+    *this <<s_brushStyles[ b.style() ];
+    *this <<" color: ";
+    // can't use operator<<(str, b.color()) because that terminates a kdbgstream (flushes)
+    if ( b.color().isValid() )
+        *this <<b.color().name() ;
+    else
+        *this <<"(invalid/default)";
+    if ( b.pixmap() )
+        *this <<" has a pixmap";
+    *this <<" ]";
+    return *this;
+}
 
 QString kdBacktrace(int levels)
 {
@@ -400,6 +516,8 @@ QString kdBacktrace(int levels)
 #ifdef HAVE_BACKTRACE
     void* trace[256];
     int n = backtrace(trace, 256);
+    if (!n)
+	return s;
     char** strings = backtrace_symbols (trace, n);
 
     if ( levels != -1 )
@@ -411,7 +529,8 @@ QString kdBacktrace(int levels)
              QString::fromLatin1(": ") +
              QString::fromLatin1(strings[i]) + QString::fromLatin1("\n");
     s += "]\n";
-    free (strings);
+    if (strings)
+        free (strings);
 #endif
     return s;
 }
@@ -421,6 +540,13 @@ QString kdBacktrace()
     return kdBacktrace(-1 /*all*/);
 }
 
+void kdClearDebugConfig()
+{
+    delete kDebug_data->config;
+    kDebug_data->config = 0;
+}
+
+
 // Needed for --enable-final
 #ifdef NDEBUG
 #define kdDebug kndDebug
