@@ -1,6 +1,7 @@
 /* This file is part of the KDE libraries
     Copyright (C) 1997 Matthias Kalle Dalheimer (kalle@kde.org)
                   2000-2002 Stephan Kulow (coolo@kde.org)
+                  2002 Holger Freyther (freyther@kde.org)
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Library General Public
@@ -19,13 +20,34 @@
 */
 
 #ifndef _KDEBUG_H_
-#define _KDEBUG_H_ "$Id: kdebug.h,v 1.61 2002/11/12 18:13:27 mutz Exp $"
+#define _KDEBUG_H_
 
 #include <qstring.h>
+
 class QWidget;
+class QDateTime;
+class QDate;
+class QTime;
+class QPoint;
+class QSize;
+class QRect;
+class QRegion;
+class KURL;
+class QStringList;
+class QColor;
+class QPen;
+class QBrush;
 
 class kdbgstream;
 class kndbgstream;
+
+/**
+ * \addtogroup kdebug Debug message generators
+ *  @{
+ * KDE debug message streams let you and the user control just how many debug
+ * messages you see.
+ */
+
 typedef kdbgstream & (*KDBGFUNC)(kdbgstream &); // manipulator function
 typedef kndbgstream & (*KNDBGFUNC)(kndbgstream &); // manipulator function
 
@@ -40,15 +62,15 @@ typedef kndbgstream & (*KNDBGFUNC)(kndbgstream &); // manipulator function
 class kdbgstreamprivate;
 /**
  * kdbgstream is a text stream that allows you to print debug messages.
- * Using the overloaded "<<" operator you can send messages. Usually 
- * you do not create the kdbgstream yourself, but use @ref kdDebug()
- * @ref kdWarning(), @ref kdError() or @ref kdFatal to obtain one.
+ * Using the overloaded "<<" operator you can send messages. Usually
+ * you do not create the kdbgstream yourself, but use kdDebug()
+ * kdWarning(), kdError() or kdFatal to obtain one.
  *
  * Example:
- * <pre>
+ * \code
  *    int i = 5;
  *    kdDebug() << "The value of i is " << i << endl;
- * </pre>
+ * \endcode
  * @see kndbgstream
  */
 class kdbgstream {
@@ -153,7 +175,8 @@ class kdbgstream {
     /**
      * Flushes the output.
      */
-    void flush();
+    void flush(); //AB: maybe this should be virtual! would save some trouble for some 3rd party projects
+
     /**
      * Prints the given value.
      * @param string the string to print
@@ -220,13 +243,106 @@ class kdbgstream {
      * @param format the printf-style format
      * @return this stream
      */
-    kdbgstream &form(const char *format, ...);
+    kdbgstream &form(const char *format, ...)
+#ifdef __GNUC__
+      __attribute__ ( ( format ( printf, 2, 3 ) ) )
+#endif
+     ;
+
     /** Operator to print out basic information about a QWidget.
      *  Output of class names only works if the class is moc'ified.
      * @param widget the widget to print
      * @return this stream
      */
-    kdbgstream& operator << (QWidget* widget);
+    kdbgstream& operator << (const QWidget* widget);
+    kdbgstream& operator << (QWidget* widget); // KDE4 merge
+
+    /**
+     * Prints the given value.
+     * @param dateTime the datetime to print
+     * @return this stream
+     */
+    kdbgstream& operator << ( const QDateTime& dateTime );
+
+    /**
+     * Prints the given value.
+     * @param date the date to print
+     * @return this stream
+     */
+    kdbgstream& operator << ( const QDate& date );
+
+    /**
+     * Prints the given value.
+     * @param time the time to print
+     * @return this stream
+     */
+    kdbgstream& operator << ( const QTime& time );
+
+    /**
+     * Prints the given value.
+     * @param point the point to print
+     * @return this stream
+     */
+    kdbgstream& operator << ( const QPoint& point );
+
+    /**
+     * Prints the given value.
+     * @param size the QSize to print
+     * @return this stream
+     */
+    kdbgstream& operator << ( const QSize& size );
+
+    /**
+     * Prints the given value.
+     * @param rect the QRect to print
+     * @return this stream
+     */
+    kdbgstream& operator << ( const QRect& rect);
+
+    /**
+     * Prints the given value.
+     * @param region the QRegion to print
+     * @return this stream
+     */
+    kdbgstream& operator << ( const QRegion& region);
+
+    /**
+     * Prints the given value.
+     * @param url the url to print
+     * @return this stream
+     */
+    kdbgstream& operator << ( const KURL& url );
+
+    /**
+     * Prints the given value.
+     * @param list the stringlist to print
+     * @return this stream
+     */
+    kdbgstream& operator << ( const QStringList& list);
+
+    /**
+     * Prints the given value.
+     * @param color the color to print
+     * @return this stream
+     */
+    kdbgstream& operator << ( const QColor& color);
+
+    /**
+     * Prints the given value.
+     * @param pen the pen to print
+     * @return this stream
+     * @since 3.2
+     */
+    kdbgstream& operator << ( const QPen& pen );
+
+    /**
+     * Prints the given value.
+     * @param brush the brush to print
+     * @return this stream
+     */
+    kdbgstream& operator << ( const QBrush& brush );
+
+
  private:
     QString output;
     unsigned int area, level;
@@ -235,27 +351,32 @@ class kdbgstream {
 };
 
 /**
- * Prints an "\n". 
+ * \relates KGlobal
+ * Prints an "\n".
  * @param s the debug stream to write to
  * @return the debug stream (@p s)
  */
 inline kdbgstream &endl( kdbgstream &s) { s << "\n"; return s; }
+
 /**
+ * \relates KGlobal
  * Flushes the stream.
  * @param s the debug stream to write to
  * @return the debug stream (@p s)
  */
 inline kdbgstream &flush( kdbgstream &s) { s.flush(); return s; }
+
 kdbgstream &perror( kdbgstream &s);
 
 /**
- * kndbgstream is a dummy variant of @ref kdbgstream. All functions do
+ * \relates KGlobal
+ * kndbgstream is a dummy variant of kdbgstream. All functions do
  * nothing.
  * @see kndDebug()
  */
 class kndbgstream {
  public:
-  /// EMpty constructor.
+    /// Default constructor.
     kndbgstream() {}
     ~kndbgstream() {}
     /**
@@ -341,29 +462,44 @@ class kndbgstream {
      * Does nothing.
      * @return this stream
      */
-    kndbgstream& operator << (QWidget*) { return *this; }
+    kndbgstream& operator << (const QWidget*) { return *this; }
+    kndbgstream& operator << (QWidget*) { return *this; } // KDE4 merge
     /**
      * Does nothing.
      * @return this stream
      */
     kndbgstream &form(const char *, ...) { return *this; }
+
+    kndbgstream& operator<<( const QDateTime& ) { return *this; }
+    kndbgstream& operator<<( const QDate&     ) { return *this; }
+    kndbgstream& operator<<( const QTime&     ) { return *this; }
+    kndbgstream& operator<<( const QPoint & )  { return *this; }
+    kndbgstream& operator<<( const QSize & )  { return *this; }
+    kndbgstream& operator<<( const QRect & )  { return *this; }
+    kndbgstream& operator<<( const QRegion & ) { return *this; }
+    kndbgstream& operator<<( const KURL & )  { return *this; }
+    kndbgstream& operator<<( const QStringList & ) { return *this; }
+    kndbgstream& operator<<( const QColor & ) { return *this; }
+    kndbgstream& operator<<( const QPen & ) { return *this; }
+    kndbgstream& operator<<( const QBrush & ) { return *this; }
 };
 
 /**
  * Does nothing.
- * @param a stream
+ * @param s a stream
  * @return the given @p s
  */
 inline kndbgstream &endl( kndbgstream & s) { return s; }
 /**
  * Does nothing.
- * @param a stream
+ * @param s a stream
  * @return the given @p s
  */
 inline kndbgstream &flush( kndbgstream & s) { return s; }
 inline kndbgstream &perror( kndbgstream & s) { return s; }
 
 /**
+ * \relates KGlobal
  * Returns a debug stream. You can use it to print debug
  * information.
  * @param area an id to identify the output, 0 for default
@@ -372,11 +508,13 @@ inline kndbgstream &perror( kndbgstream & s) { return s; }
 kdbgstream kdDebug(int area = 0);
 kdbgstream kdDebug(bool cond, int area = 0);
 /**
+ * \relates KGlobal
  * Returns a backtrace.
  * @return a backtrace
  */
 QString kdBacktrace();
 /**
+ * \relates KGlobal
  * Returns a backtrace.
  * @param levels the number of levels of the backtrace
  * @return a backtrace
@@ -388,11 +526,13 @@ QString kdBacktrace(int levels);
  * @param area an id to identify the output, 0 for default
  * @see kdDebug()
  */
-inline kndbgstream kndDebug(int = 0) { return kndbgstream(); }
+inline kndbgstream kndDebug(int area = 0) { Q_UNUSED(area); return kndbgstream(); }
 inline kndbgstream kndDebug(bool , int  = 0) { return kndbgstream(); }
 inline QString kndBacktrace() { return QString::null; }
+inline QString kndBacktrace(int) { return QString::null; }
 
 /**
+ * \relates KGlobal
  * Returns a warning stream. You can use it to print warning
  * information.
  * @param area an id to identify the output, 0 for default
@@ -400,6 +540,7 @@ inline QString kndBacktrace() { return QString::null; }
 kdbgstream kdWarning(int area = 0);
 kdbgstream kdWarning(bool cond, int area = 0);
 /**
+ * \relates KGlobal
  * Returns an error stream. You can use it to print error
  * information.
  * @param area an id to identify the output, 0 for default
@@ -407,6 +548,7 @@ kdbgstream kdWarning(bool cond, int area = 0);
 kdbgstream kdError(int area = 0);
 kdbgstream kdError(bool cond, int area = 0);
 /**
+ * \relates KGlobal
  * Returns a fatal error stream. You can use it to print fatal error
  * information.
  * @param area an id to identify the output, 0 for default
@@ -414,6 +556,15 @@ kdbgstream kdError(bool cond, int area = 0);
 kdbgstream kdFatal(int area = 0);
 kdbgstream kdFatal(bool cond, int area = 0);
 
+/**
+ * \relates KGlobal
+ * Deletes the kdebugrc cache and therefore forces KDebug to reread the
+ * config file
+ */
+void kdClearDebugConfig();
+
+/** @} */
+
 #ifdef NDEBUG
 #define kdDebug kndDebug
 #define kdBacktrace kndBacktrace
