@@ -2,7 +2,7 @@
  *  This file is part of the KDE libraries
  *  Copyright (C) 2000-2002 Thiago Macieira <thiagom@mail.com>
  *
- *  $Id: kextsock.cpp,v 1.44.2.8 2003/06/16 09:31:43 waba Exp $
+ *  $Id: kextsock.cpp,v 1.67.2.2 2004/03/01 13:18:40 waba Exp $
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Library General Public
@@ -26,11 +26,13 @@
 #include <sys/socket.h>
 #include <sys/times.h>
 #include <netinet/in.h>
+#include <arpa/inet.h>
 #include <sys/un.h>
 
 #ifdef HAVE_RES_INIT
 # include <arpa/nameser.h>
 # include <resolv.h>
+extern "C" int res_init();
 #endif
 
 #include <stdio.h>
@@ -49,6 +51,7 @@
 #include <qdns.h>
 #include <qguardedptr.h>
 
+#include "kidna.h"
 #include "kdebug.h"
 #include "kextsock.h"
 #include "ksockaddr.h"
@@ -82,6 +85,12 @@
 #endif
 #define offsetof(TYPE, MEMBER) ((size_t) &((TYPE *)0)->MEMBER)
 
+#ifdef Q_OS_FREEBSD
+#ifndef EAI_NODATA
+#define EAI_NODATA EAI_NONAME
+#endif
+#endif
+
 //
 // Internal class definitions
 //
@@ -265,7 +274,7 @@ static int skipData(int fd, unsigned len)
 // calls the correct deallocation routine
 // also uses by-reference parameter to simplify caller routines, because
 // we set the parameter to NULL after deallocation
-void local_freeaddrinfo(kde_addrinfo *&p)
+static void local_freeaddrinfo(kde_addrinfo *&p)
 {
   if (p == NULL)
     return;
@@ -283,8 +292,10 @@ void local_freeaddrinfo(kde_addrinfo *&p)
  */
 kde_addrinfo* KExtendedSocketLookup::results()
 {
-  QValueList<QHostAddress> v4 = dnsIpv4.addresses(),
-    v6 = dnsIpv6.addresses();
+  QValueList<QHostAddress> v4 = dnsIpv4.addresses();
+#ifdef AF_INET6
+  QValueList<QHostAddress> v6 = dnsIpv6.addresses();
+#endif    
   addrinfo *p = NULL;
   kde_addrinfo *res = new kde_addrinfo;
   res->origin = KAI_QDNS;
@@ -292,9 +303,10 @@ kde_addrinfo* KExtendedSocketLookup::results()
   unsigned short port;
 
   QString canon = dnsIpv4.canonicalName();
+#ifdef AF_INET6
   if (canon.isNull())
     canon = dnsIpv6.canonicalName();
-
+#endif
   char* canonname;
   if (!canon.isNull())
     canonname = strdup(canon.latin1());
@@ -336,7 +348,7 @@ kde_addrinfo* KExtendedSocketLookup::results()
       q->ai_next = p;
 
       memset(sin6, 0, sizeof(*sin6));
-# ifdef HAVE_SOCKADDR_SA_LEN
+# ifdef HAVE_STRUCT_SOCKADDR_SA_LEN
       sin6->sin6_len = sizeof(*sin6);
 # endif
       sin6->sin6_family = AF_INET6;
@@ -362,7 +374,7 @@ kde_addrinfo* KExtendedSocketLookup::results()
       q->ai_next = p;
 
       memset(sin, 0, sizeof(*sin));
-# ifdef HAVE_SOCKADDR_SA_LEN
+# ifdef HAVE_STRUCT_SOCKADDR_SA_LEN
       sin->sin_len = sizeof(*sin);
 # endif
       sin->sin_family = AF_INET;
@@ -380,7 +392,7 @@ void KExtendedSocketLookup::freeresults(kde_addrinfo *res)
 {
   addrinfo *ai = res->data;
   if (ai == NULL)
-    return;                    // No data? Bizarre, but nonetheless possible
+    return;			// No data? Bizarre, but nonetheless possible
 
   if (ai->ai_canonname)
     free(ai->ai_canonname);
@@ -748,7 +760,7 @@ bool KExtendedSocket::setAddressReusable(int fd, bool enable)
   if (fd == -1)
     return false;
 
-  int on = (int)enable;		// just to be on the safe side
+  int on = enable;		// just to be on the safe side
 
   if (setsockopt(fd, SOL_SOCKET, SO_REUSEADDR, (char*)&on, sizeof(on)) == -1)
     return false;
@@ -863,12 +875,20 @@ bool KExtendedSocket::setBufferSize(int rsize, int wsize)
 
   // LOCK BUFFER MUTEX
 
+  // The input socket notifier is always enabled
+  // That happens because we want to be notified of when the socket gets
+  // closed
+  if (d->qsnIn == NULL)
+    {
+      d->qsnIn = new QSocketNotifier(sockfd, QSocketNotifier::Read);
+      QObject::connect(d->qsnIn, SIGNAL(activated(int)), this, SLOT(socketActivityRead()));
+      d->qsnIn->setEnabled(true);
+    }
+
   if (rsize == 0 && d->flags & inputBufferedSocket)
     {
       // user wants to disable input buffering
       d->flags &= ~inputBufferedSocket;
-      if (d->qsnIn && !d->emitRead)
-	d->qsnIn->setEnabled(false);
 
       consumeReadBuffer(readBufferSize(), NULL, true);
       d->inMaxSize = 0;
@@ -884,11 +904,6 @@ bool KExtendedSocket::setBufferSize(int rsize, int wsize)
 	// input buffer has more data than the new size; discard
 	consumeReadBuffer(readBufferSize() - rsize, NULL, true);
 
-      if (d->qsnIn == NULL)
-	{
-	  d->qsnIn = new QSocketNotifier(sockfd, QSocketNotifier::Read);
-	  QObject::connect(d->qsnIn, SIGNAL(activated(int)), this, SLOT(socketActivityRead()));
-	}
     }
 
   if (wsize == 0 && d->flags & outputBufferedSocket)
@@ -926,11 +941,6 @@ bool KExtendedSocket::setBufferSize(int rsize, int wsize)
   setFlags((mode() & ~IO_Raw) | ((d->flags & bufferedSocket) ? 0 : IO_Raw));
 
   // check we didn't turn something off we shouldn't
-  if (d->emitRead && d->qsnIn == NULL)
-    {
-      d->qsnIn = new QSocketNotifier(sockfd, QSocketNotifier::Read);
-      QObject::connect(d->qsnIn, SIGNAL(activated(int)), this, SLOT(socketActivityRead()));
-    }
   if (d->emitWrite && d->qsnOut == NULL)
     {
       d->qsnOut = new QSocketNotifier(sockfd, QSocketNotifier::Write);
@@ -958,7 +968,7 @@ const KSocketAddress *KExtendedSocket::localAddress()
 /*
  * Same thing, but for peer address. Which means this does not work on
  * passiveSocket and that we require to be connected already. Also note that
- * the behaviour on connectionless sockets is not defined here.
+ * the behavior on connectionless sockets is not defined here.
  */
 const KSocketAddress* KExtendedSocket::peerAddress()
 {
@@ -1142,6 +1152,8 @@ int KExtendedSocket::listen(int N)
 	  kdDebug(170) << "Failed to create: " << perror << endl;
 	  continue;
 	}
+	
+      fcntl(sockfd, F_SETFD, FD_CLOEXEC);
 
       if (d->addressReusable)
 	setAddressReusable(sockfd, true);
@@ -1237,6 +1249,8 @@ int KExtendedSocket::accept(KExtendedSocket *&sock)
       return -1;
     }
 
+  fcntl(newfd, F_SETFD, FD_CLOEXEC);
+
   //kdDebug(170).form("Socket %d accepted socket %d\n", sockfd, newfd);
 
   setBlockingMode(block);	// restore blocking mode
@@ -1323,6 +1337,7 @@ int KExtendedSocket::connect()
 	  setError(IO_ConnectError, errno);
 	  if (sockfd == -1)
 	    continue;		// cannot create this socket
+          fcntl(sockfd, F_SETFD, FD_CLOEXEC);
 	  if (d->addressReusable)
 	    setAddressReusable(sockfd, true);
 	  setIPv6Only(d->ipv6only);
@@ -1344,6 +1359,7 @@ int KExtendedSocket::connect()
 	      setError(IO_ConnectError, errno);
 	      continue;
 	    }
+          fcntl(sockfd, F_SETFD, FD_CLOEXEC);
 	  if (d->addressReusable)
 	    setAddressReusable(sockfd, true);
 	  setIPv6Only(d->ipv6only);
@@ -1665,8 +1681,8 @@ void KExtendedSocket::flush()
       while (a && count + (a->size() - offset) < buf.size())
 	{
 	  memcpy(buf.data() + count, a->data() + offset, a->size() - offset);
-	  offset = 0;
 	  count += a->size() - offset;
+	  offset = 0;
 	  a = outBuf.next();
 	}
 
@@ -1699,8 +1715,6 @@ Q_LONG KExtendedSocket::readBlock(char *data, Q_ULONG maxlen)
   cleanError();
   if (d->status < connected || d->flags & passiveSocket)
     return -2;
-  if (sockfd == -1)
-    return -2;
 
   int retval;
 
@@ -1709,6 +1723,8 @@ Q_LONG KExtendedSocket::readBlock(char *data, Q_ULONG maxlen)
       // we aren't buffering this socket, so just pass along
       // the call to the real read method
 
+      if (sockfd == -1)
+	return -2;
       if (data)
 	retval = KSocks::self()->read(sockfd, data, maxlen);
       else
@@ -1727,6 +1743,8 @@ Q_LONG KExtendedSocket::readBlock(char *data, Q_ULONG maxlen)
 	{
 	  // consumeReadBuffer returns 0 only if the buffer is
 	  // empty
+	  if (sockfd == -1)
+	    return 0;		// buffer is clear now, indicate EOF
 	  setError(IO_ReadError, EWOULDBLOCK);
 	  retval = -1;
 	}
@@ -1815,7 +1833,7 @@ int KExtendedSocket::unreadBlock(const char *, uint)
 
 int KExtendedSocket::bytesAvailable() const
 {
-  if (d->status < connected || d->status >= closing || d->flags & passiveSocket)
+  if (d->status < connected || d->flags & passiveSocket)
     return -2;
 
   // as of now, we don't do any extra processing
@@ -1847,7 +1865,7 @@ int KExtendedSocket::waitForMore(int msecs)
       setError(IO_FatalError, errno);
       return -1;
     }
-  else if (retval == 0)
+  else if (retval != 0)
     socketActivityRead();	// do read processing
 
   return bytesAvailable();
@@ -1875,11 +1893,10 @@ int KExtendedSocket::doLookup(const QString &host, const QString &serv, addrinfo
 {
   int err;
 
-  // FIXME! What is the encoding?
-  const char *_host = NULL,
-    *_serv = NULL;
+  QCString _host;
+  QCString _serv;
   if (!host.isNull())
-    _host = host.latin1();
+    _host = KIDNA::toAsciiCString(host);
   if (!serv.isNull())
     _serv = serv.latin1();
   // Please read the comments before kde_getaddrinfo in netsupp.cpp
@@ -2008,6 +2025,27 @@ void KExtendedSocket::socketActivityRead()
 
       // UNLOCK MUTEX
     }
+  else
+    {
+      // No input buffering, but the notifier fired
+      // That means that either there is data to be read or that the 
+      // socket closed.
+
+      // try to read one byte. If we can't, then the socket got closed
+
+      char c;
+      int len = KSocks::self()->recv(sockfd, &c, sizeof(c), MSG_PEEK);
+      if (len == 0)
+	{
+	  // yes, it's an EOF condition
+	  d->qsnIn->setEnabled(false);
+	  ::close(sockfd);
+	  sockfd = -1;
+	  d->status = done;
+	  emit closed(involuntary);
+	  return;
+	}
+    }
 
   if (d->emitRead)
     emit readyRead();
@@ -2137,6 +2175,7 @@ void KExtendedSocket::connectionEvent()
 	  errcode = errno;
 	  if (sockfd == -1)
 	    continue;		// cannot create this socket
+          fcntl(sockfd, F_SETFD, FD_CLOEXEC);
 	  if (d->addressReusable)
 	    setAddressReusable(sockfd, true);
 	  setIPv6Only(d->ipv6only);
@@ -2158,6 +2197,7 @@ void KExtendedSocket::connectionEvent()
 	      errcode = errno;
 	      continue;
 	    }
+          fcntl(sockfd, F_SETFD, FD_CLOEXEC);
 	  if (d->addressReusable)
 	    setAddressReusable(sockfd, true);
 	  setIPv6Only(d->ipv6only);
@@ -2240,7 +2280,17 @@ void KExtendedSocket::dnsResultsReady()
 	n++;
     }
 
-  d->status = lookupDone;
+  if (n)
+    {
+      d->status = lookupDone;
+      cleanError();
+    }
+  else
+    {
+      d->status = nothing;
+      setError(IO_LookupError, EAI_NODATA);
+    }
+
   emit lookupFinished(n);
 
   return;
@@ -2248,7 +2298,10 @@ void KExtendedSocket::dnsResultsReady()
 
 void KExtendedSocket::startAsyncConnectSlot()
 {
-  startAsyncConnect();
+  QObject::disconnect(this, SIGNAL(lookupFinished(int)), this, SLOT(startAsyncConnectSlot()));
+
+  if (d->status == lookupDone)
+    startAsyncConnect();
 }
 
 int KExtendedSocket::resolve(sockaddr *sock, ksocklen_t len, QString &host,
@@ -2328,14 +2381,14 @@ KSocketAddress *KExtendedSocket::localAddress(int fd)
 
   /* was it enough? */
   if (len > sizeof(static_sa)
-#ifdef HAVE_SOCKADDR_SA_LEN
+#ifdef HAVE_STRUCT_SOCKADDR_SA_LEN
       || sa->sa_len > sizeof(static_sa)
 #endif
       )
     {
       /* nope, malloc a new socket with the proper size */
 
-#ifdef HAVE_SOCKADDR_SA_LEN
+#ifdef HAVE_STRUCT_SOCKADDR_SA_LEN
       if (sa->sa_len != len)
         len = sa->sa_len;
 #endif
@@ -2376,14 +2429,14 @@ KSocketAddress *KExtendedSocket::peerAddress(int fd)
 
   /* was it enough? */
   if (len > sizeof(static_sa)
-#ifdef HAVE_SOCKADDR_SA_LEN
+#ifdef HAVE_STRUCT_SOCKADDR_SA_LEN
       || sa->sa_len > sizeof(static_sa)
 #endif
       )
     {
       /* nope, malloc a new socket with the proper size */
 
-#ifdef HAVE_SOCKADDR_SA_LEN
+#ifdef HAVE_STRUCT_SOCKADDR_SA_LEN
       if (sa->sa_len != len)
         len = sa->sa_len;
 #endif
