@@ -50,19 +50,19 @@ class KExtendedSocketPrivate;
 /**
  * The extended socket class.
  *
- * This class should be used instead of @ref KSocket whenever the user needs
+ * This class should be used instead of KSocket whenever the user needs
  * fine-grained control over the socket being created. Unlike KSocket, which
  * does everything at once, without much intervention, KExtendedSocket allows
  * intervention at every step of the process and the setting of parameters.
  *
  * This class allows for the creation of both server and client sockets. The
  * only difference is that the passiveSocket flag must be passed either to
- * the constructor or to @ref setSocketFlags(). If passiveSocket is used, the class will
- * enable functions @ref listen() and @ref accept() and related signals, and will
- * also disable @ref readBlock() and @ref writeBlock().
+ * the constructor or to setSocketFlags(). If passiveSocket is used, the class will
+ * enable functions listen() and accept() and related signals, and will
+ * also disable readBlock() and writeBlock().
  *
  * To create a Unix socket, one would pass flag unixSocket to the constructor
- * or @ref setSocketFlags(). The hostname and service/port can be set to whatever is
+ * or setSocketFlags(). The hostname and service/port can be set to whatever is
  * necessary. If no hostname is given, but a service/port is, the socket created
  * will be implementation dependant (usually in /tmp). In any other case, the
  * fields will be concatenated.
@@ -77,7 +77,7 @@ class KExtendedSocketPrivate;
  * support newer IP protocols, ipv4Socket | ipv6Socket is your guy.
  *
  * @author Thiago Macieira <thiagom@mail.com>
- * @version $Id: kextsock.h,v 1.30.2.1 2003/01/03 00:14:05 malte Exp $
+ * @version $Id: kextsock.h,v 1.42 2003/10/07 22:09:16 mueller Exp $
  * @short an extended socket
  */
 class KExtendedSocket: public KBufferedIO // public QObject, public QIODevice
@@ -162,7 +162,7 @@ public:
   /**
    * Creates a socket with the given hostname and service.
    * @param host	the hostname
-   * @param serv	the service
+   * @param service	the service
    * @param flags	flags
    */
   KExtendedSocket(const QString& host, const QString& service, int flags = 0);
@@ -200,7 +200,7 @@ public:
   int systemError() const;
 
   /**
-   * Sets the given flags. 
+   * Sets the given flags.
    * @param flags	the flags to be set
    * @return the new flags status, or -1 if flags can no longer be set
    */
@@ -237,7 +237,7 @@ public:
    * @param port	the port
    * @return true if successful, false on error (e.g. connection already established)
    */
-  bool setPort(const QString& service);
+  bool setPort(const QString& port);
 
   /**
    * Returns the port/service. If it is a port, the string contains a number.
@@ -319,7 +319,7 @@ public:
    * Sets both host and service to which we will bind the socket. Will return
    * false if this is a passiveSocket.
    * @param host	the hostname
-   * @param serv	the service
+   * @param service	the service
    * @return true if successful, false on error (e.g. connection already established)
    */
   bool setBindAddress(const QString& host, const QString& service);
@@ -383,6 +383,34 @@ public:
    */
   bool addressReusable();
 
+  /**
+   * Sets/unsets the v6-only flag for IPv6 sockets.
+   *
+   * When an IPv6 socket is in use, communication with IPv4 sockets is
+   * guaranteed by translating those IPv4 addresses into IPv6 ones
+   * (specifically, the v4-mapped addresses). This flag allows that
+   * behavior to be turned on and off.
+   *
+   * Note that this does not have any effect on sockets that are not
+   * IPv6 and the function will always return false in those cases.
+   * Also note that this flag defaults to off in order to accommodate
+   * existing applications.
+   *
+   * @param enable	if true, no IPv4 translation will be performed;
+   *			this socket will be restricted to IPv6 communication
+   * @returns true on success, false on failure.
+   * @see localAddress to find out if this is an IPv6 socket
+   */
+  bool setIPv6Only(bool enable);
+
+  /**
+   * Returns the status of the v6-only flag for IPv6 sockets.
+   * @returns true if the flag is set to on; false if it is not. If this
+   * socket is not an IPv6 one, the return value is false.
+   * @see setIPv6Only
+   */
+  bool isIPv6Only();
+
   /**
    * Sets the buffer sizes for this socket.
    *
@@ -395,10 +423,6 @@ public:
    * buffer's to be discarded. Likewise, setting the size to a value less than
    * the current size will cause the buffer to be shrunk to the wanted value,
    * as if the data had been read.
-   *
-   * Note 2: The closed() signal will only be emitted for a connection closed
-   * by the remote end if we are doing input buffering. That happens because if we aren't
-   * reading from the connection, we don't know when it closed.
    * @param rsize	read buffer size
    * @param wsize	write buffer size
    * @return true on success, false if this is not possible in this state (e.g. connection
@@ -447,14 +471,14 @@ public:
    * When the lookup is done, the lookupReady signal will be emitted.
    *
    * Note that, depending on the parameters for the lookup, this function might
-   * know the results without the need for blocking or queueing an
+   * know the results without the need for blocking or queuing an
    * asynchronous lookup. That means that the lookupReady signal might be
    * emitted by this function, so your code should be prepared for that.
    *
    * One such case is when noResolve flag is set.
-   * If this function is able to determine the results without queueing
+   * If this function is able to determine the results without queuing
    * and the lookup failed, this function will return -1.
-   * 
+   *
    * @return 0 on success or -1 on error. Note that
    * returning 0 means that either we are in the process of doing
    * lookup or that it has finished already.
@@ -468,7 +492,7 @@ public:
 
   /**
    * Place the socket in listen mode. The parameters are the same as for
-   * the system listen() call. 
+   * the system listen() call.
    * @param N		the queue length for pending connections
    * @return 0 on success, -1 on system error (errno
    *         available) and -2 if this is not a passiveSocket.
@@ -492,17 +516,17 @@ public:
   virtual int accept(KExtendedSocket *&sock);
 
   /**
-   * Attempts to connect to the remote host. 
+   * Attempts to connect to the remote host.
    * After successful connection (return value 0), the socket will be ready
    * for I/O operations. Note, however, that not all signals may be enabled
    * for emission by this socket:
-   * @li @ref readyRead and @ref readyWrite signals will be enabled only if
-   *    @ref enableRead or @ref enableWrite were called. You can still enable
+   * @li readyRead and readyWrite signals will be enabled only if
+   * enableRead or enableWrite were called. You can still enable
    *    them by calling those functions, of course.
    * @li #closed() will only be sent if we are indeed reading from the input
-   *    stream. That is, if this socket is buffering the input. See @ref setBufferSize
+   *    stream. That is, if this socket is buffering the input. See setBufferSize
    *
-   * Note that, in general, functions inherited/overriden from KBufferedIO will only
+   * Note that, in general, functions inherited/overridden from KBufferedIO will only
    * work on buffered sockets, like bytesAvailable and bytesToWrite.
    * @return    The return values are:
    * @li 0: success
@@ -518,13 +542,17 @@ public:
 
   /**
    * Starts an asynchronous connect. This works exactly the same as #connect,
-   * except that the connection result won't be returned. 
+   * except that the connection result won't be returned.
    *
    * Note that those signals might be emitted before this function returns, so your
    * code should be prepared for that condition.
-   * @return 0 on successful queueing of the connect or -1 on error. 
-   *         If this function returns 0, then the @ref connectionSuccess() or the 
-   *         @ref connectionFailed() signals will be emitted.
+   *
+   * You must call cancelAsyncConnect() before you delete the socket if you
+   * call this.  Otherwise you will have crashes.
+   *
+   * @return 0 on successful queuing of the connect or -1 on error.
+   *         If this function returns 0, then the connectionSuccess() or the
+   * connectionFailed() signals will be emitted.
    */
   virtual int startAsyncConnect();
 
@@ -534,7 +562,7 @@ public:
   virtual void cancelAsyncConnect();
 
   /**
-   * Implementation of @ref QIODevice::open() pure virtual function.
+   * Implementation of QIODevice::open() pure virtual function.
    * This depends on the target host address already being there.
    * If this is a passiveSocket, this is identical to call listen(); else, if
    * this is not a passiveSocket and no connection attempt is in progress, this
@@ -549,8 +577,8 @@ public:
    * Closes the socket. If we have data still in the write buffer yet to be
    * sent, the socket won't be closed right now. It'll be closed after we managed
    * to send everything out.
-   * If you want to close the socket now, you may want to call @ref flush() first,
-   * and then @ref closeNow().
+   * If you want to close the socket now, you may want to call flush() first,
+   * and then closeNow().
    */
   virtual void close();
 
@@ -588,12 +616,12 @@ public:
    * data as possible, but it will stop as soon as the kernel cannot receive
    * any more data, and would possibly block.
    *
-   * By repeatedly calling this function, the behaviour will be like that of
+   * By repeatedly calling this function, the behavior will be like that of
    * a blocking socket. Indeed, if this function is called with the kernel not
    * ready to receive data, it will block, unless this is a non-blocking socket.
    *
    * This function does not touch the read buffer. You can empty it by calling
-   * @ref readBlock() with a null destination buffer.
+   * readBlock() with a null destination buffer.
    */
   virtual void flush();
 
@@ -632,7 +660,7 @@ public:
    *
    * If the socket is not buffered, this function will simply call the underlying
    * read method. This function will block if the socket is not on non-blocking mode
-   * (see @ref setBlockingMode) and there is not enough data to be read in the
+   * (see setBlockingMode) and there is not enough data to be read in the
    * Operating System yet. If we are in non-blocking operation, the call will
    * fail in this case.
    *
@@ -694,7 +722,7 @@ public:
    *
    * @param data	where to store the data
    * @param maxlen	how many bytes to copy, at most
-   * @return the number of bytes copied. 0 does not mean end-of-file 
+   * @return the number of bytes copied. 0 does not mean end-of-file
    *         condition.
    */
   virtual int peekBlock(char *data, uint maxlen);
@@ -708,8 +736,8 @@ public:
   virtual int unreadBlock(const char *data, uint len);
 
   /**
-   * Returns the number of available bytes yet to be read via @ref readBlock
-   * and family of functions. 
+   * Returns the number of available bytes yet to be read via readBlock
+   * and family of functions.
    *
    * Note: as of now, this only works on input-buffered sockets. This will
    * change in the future
@@ -724,7 +752,7 @@ public:
    * read in the read buffer.
    *
    * @param msec	milliseconds to wait
-   * @return -1 in case of system error and -2 in case of invalid socket 
+   * @return -1 in case of system error and -2 in case of invalid socket
    *         state
    */
   virtual int waitForMore(int msec);
@@ -775,7 +803,7 @@ public:
 signals:
   /**
    * This signal is emitted whenever an asynchronous lookup process is done.
-   * The parameter @p count tells 
+   * The parameter @p count tells
    * @param count the number of results
    */
   void lookupFinished(int count);
@@ -849,7 +877,8 @@ public:
    * That is, tries to resolve the raw form of the socket address into a textual
    * representation.
    *
-   * @param sockaddr	the socket address
+   * @param sock	the socket address
+   * @param len         the length of the socket address
    * @param host	where the hostname will be written
    * @param port	where the service-port will be written
    * @param flags	the same flags as getnameinfo()
@@ -863,7 +892,7 @@ public:
    * That is, tries to resolve the raw form of the socket address into a textual
    * representation.
    *
-   * @param sockaddr	the socket address
+   * @param sock	the socket address
    * @param host	where the hostname will be written
    * @param port	where the service-port will be written
    * @param flags	the same flags as getnameinfo()
@@ -874,20 +903,20 @@ public:
   /**
    * Performs lookup on the given hostname/port combination and returns a list
    * of matching addresses.
-   * The error code can be transformed into string by @ref KExtendedSocket::strError()
+   * The error code can be transformed into string by KExtendedSocket::strError()
    * with code of IO_LookupError.
    *
-   * IMPORTANT: the result values of the @ref QPtrList must be deleted after use. So,
-   * if you don't copy the @ref KAddressInfo objects, the best way to assure that
+   * IMPORTANT: the result values of the QPtrList must be deleted after use. So,
+   * if you don't copy the KAddressInfo objects, the best way to assure that
    * is to call setAutoDelete(true) on the list right after this function
    * returns. If you do copy the results out, you must assure that the objects
    * get deleted when they are not needed any more.
    *
    * @param host	the hostname to look up
    * @param port	the port/service to look up
-   * @param flags	flags to be used when looking up, @ref Flags
+   * @param flags	flags to be used when looking up, Flags
    * @param error	pointer to a variable holding the error code
-   * @return a list of @ref KAddressInfos
+   * @return a list of KAddressInfos
    */
   static QPtrList<KAddressInfo> lookup(const QString& host, const QString& port, int flags = 0, int *error = 0);
 
@@ -957,7 +986,7 @@ public:
   ~KAddressInfo();
 
   /**
-   * Returns the KAddressInfo's @ref KSocketAddress.
+   * Returns the KAddressInfo's KSocketAddress.
    * Only valid as long as the KAddressInfo exists.
    */
   inline operator const KSocketAddress*() const
@@ -970,14 +999,14 @@ public:
   { return *ai; }
 
   /**
-   * Returns a pointer to KAddressInfo's addrinfo. 
+   * Returns a pointer to KAddressInfo's addrinfo.
    * Only valid as long as the KAddressInfo exists.
    */
   inline operator const addrinfo*() const
   { return ai; }
 
   /**
-   * Returns the KAddressInfo's @ref KSocketAddress.
+   * Returns the KAddressInfo's KSocketAddress.
    * Only valid as long as the KAddressInfo exists.
    * @return the KAddressInfo's KSocketAddress.
    */
