@@ -18,7 +18,7 @@
 /*
 * kglobal.cpp -- Implementation of class KGlobal.
 * Author:	Sirtaj Singh Kang
-* Version:	$Id: kglobal.cpp,v 1.66 2002/01/26 12:14:35 coolo Exp $
+* Version:	$Id: kglobal.cpp,v 1.68 2003/09/24 13:05:09 waba Exp $
 * Generated:	Sat May  1 02:08:43 EST 1999
 */
 
@@ -64,6 +64,13 @@ KConfig	*KGlobal::config()
     return _instance->config();
 }
 
+KSharedConfig *KGlobal::sharedConfig()
+{
+    MYASSERT(_instance);
+
+    return _instance->sharedConfig();
+}
+
 KIconLoader *KGlobal::iconLoader()
 {
     MYASSERT(_instance);
@@ -123,7 +130,7 @@ KGlobal::staticQString(const char *str)
 class KStringDict : public QDict<QString>
 {
 public:
-   KStringDict() : QDict<QString>() { };
+   KStringDict() : QDict<QString>(139) { };
 };
 
 /**
