@@ -18,29 +18,29 @@
 #ifndef _KGLOBAL_H
 #define _KGLOBAL_H
 
-#include <qstring.h>
+#include "kdemacros.h"
+#include <kinstance.h> // KDE4: class KInstance is enough here
 
-class KStandardDirs;
-class KConfig;
-class KLocale;
-class KIconLoader;
 class KCharsets;
-class QFont;
-class KInstance;
-class KStringDict;
+class KConfig;
+class KSharedConfig;
+class KIconLoader;
+class KLocale;
+class KStandardDirs;
 class KStaticDeleterBase;
 class KStaticDeleterList;
-#include <kinstance.h>
+class KStringDict;
+class QString;
 
 /**
  * Access to the KDE global objects.
- * KGlobal provides you with pointers of many central 
+ * KGlobal provides you with pointers of many central
  * objects that exist only once in the process. It is also
- * responsible for managing instances of @ref KStaticDeleterBase.
+ * responsible for managing instances of KStaticDeleterBase.
  *
  * @see KStaticDeleterBase
  * @author Sirtaj Singh Kang (taj@kde.org)
- * @version $Id: kglobal.h,v 1.44.2.2 2003/09/02 02:27:45 raabe Exp $
+ * @version $Id: kglobal.h,v 1.55 2003/09/24 13:05:09 waba Exp $
  */
 class KGlobal
 {
@@ -66,6 +66,12 @@ public:
      */
     static KConfig		*config();
 
+    /**
+     *  Returns the general config object.
+     * @return the global configuration object.
+     */
+    static KSharedConfig        *sharedConfig();
+
     /**
      *  Returns an iconloader object.
      * @return the global iconloader object
@@ -88,14 +94,14 @@ public:
      * Creates a static QString.
      *
      * To be used inside functions(!) like:
-     * <pre>
+     * \code
      * static const QString &myString = KGlobal::staticQString("myText");
-     * </pre>
+     * \endcode
      *
      * !!! Do _NOT_ use: !!!
-     * <pre>
+     * \code
      * static QString myString = KGlobal::staticQString("myText");
-     * </pre>
+     * \endcode
      * This creates a static object (instead of a static reference)
      * and as you know static objects are EVIL.
      * @param str the string to create
@@ -107,14 +113,14 @@ public:
      * Creates a static QString.
      *
      * To be used inside functions(!) like:
-     * <pre>
+     * \code
      * static const QString &myString = KGlobal::staticQString(i18n("My Text"));
-     * </pre>
+     * \endcode
      *
      * !!! Do _NOT_ use: !!!
-     * <pre>
+     * \code
      * static QString myString = KGlobal::staticQString(i18n("myText"));
-     * </pre>
+     * \endcode
      * This creates a static object (instead of a static reference)
      * and as you know static objects are EVIL.
      * @param str the string to create
@@ -165,20 +171,22 @@ public:
     static  KInstance           *_activeInstance;
 };
 
-/** @ref KGlobal
+/**
+ * \relates KGlobal
  * A typesafe function to find the minimum of the two arguments.
  */
 #define KMIN(a,b)	kMin(a,b)
-/** @ref KGlobal
+/**
+ * \relates KGlobal
  * A typesafe function to find the maximum of the two arguments.
  */
 #define KMAX(a,b)	kMax(a,b)
-/** @ref KGlobal
+/**
+ * \relates KGlobal
  * A typesafe function to determine the absolute value of the argument.
  */
 #define KABS(a)	kAbs(a)
 
-
 template<class T>
 inline const T& kMin (const T& a, const T& b) { return a < b ? a : b; }
 
@@ -189,7 +197,7 @@ template<class T>
 inline T kAbs (const T& a) { return a < 0 ? -a : a; }
 
 /**
- * @libdoc The KDE Core Functionality Library
+ * \mainpage The KDE Core Functionality Library
  *
  * All KDE programs use this library to provide basic functionality such
  * as the configuration system, IPC, internationalization and locale
@@ -197,7 +205,7 @@ inline T kAbs (const T& a) { return a < 0 ? -a : a; }
  * of other (but no less important) things.
  *
  * All KDE applications should link to the kdecore library. Also, using a
- * @ref KApplication derived class instead of @ref QApplication is almost
+ * KApplication derived class instead of QApplication is almost
  * mandatory if you expect your application to behave nicely within the
  * KDE environment.
  */
