@@ -15,17 +15,22 @@
    the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
    Boston, MA 02111-1307, USA.
 */
-
+#include "config.h"
 #include "kglobalsettings.h"
 
 #include <qdir.h>
 #include <qpixmap.h>
 #include <qfontdatabase.h>
+#include <qcursor.h>
 
 #include <kconfig.h>
 #include <ksimpleconfig.h>
 #include <kapplication.h>
-#include <kipc.h>
+
+#if defined Q_WS_X11 && ! defined K_WS_QTONLY
+#include <kipc.h> // schroder
+#endif
+
 #include <kdebug.h>
 #include <kglobal.h>
 #include <kshortcut.h>
@@ -35,10 +40,12 @@
 #include <klocale.h>
 #include <qfontinfo.h>
 #include <stdlib.h>
+#include <kprotocolinfo.h>
 
-#ifdef Q_WS_X11
+#if defined Q_WS_X11 && ! defined K_WS_QTONLY
+//#ifdef Q_WS_X11
 //#include <X11/X.h>
-#include <X11/Xlib.h>
+#include <X11/Xlib.h> // schroder
 //#include <X11/Xutil.h>
 #endif
 
@@ -76,10 +83,12 @@ bool KGlobalSettings::singleClick()
 KGlobalSettings::TearOffHandle KGlobalSettings::insertTearOffHandle()
 {
     int tearoff;
+    bool effectsenabled;
     KConfig *c = KGlobal::config();
     KConfigGroupSaver cgs( c, "KDE" );
+    effectsenabled = c->readBoolEntry( "EffectsEnabled", false);
     tearoff =c->readNumEntry("InsertTearOffHandle", KDE_DEFAULT_INSERTTEAROFFHANDLES);
-    return (TearOffHandle) tearoff;
+    return effectsenabled ? (TearOffHandle) tearoff : Disable;
 }
 
 bool KGlobalSettings::changeCursorOverIcon()
@@ -280,8 +289,9 @@ QFont KGlobalSettings::generalFont()
     if (_generalFont)
         return *_generalFont;
 
+    // Sync default with kdebase/kcontrol/fonts/fonts.cpp
     _generalFont = new QFont("helvetica", 12);
-    _generalFont->setPixelSize(12);
+    _generalFont->setPointSize(12);
     _generalFont->setStyleHint(QFont::SansSerif);
 
     KConfig *c = KGlobal::config();
@@ -296,8 +306,9 @@ QFont KGlobalSettings::fixedFont()
     if (_fixedFont)
         return *_fixedFont;
 
+    // Sync default with kdebase/kcontrol/fonts/fonts.cpp
     _fixedFont = new QFont("courier", 12);
-    _fixedFont->setPixelSize(12);
+    _fixedFont->setPointSize(12);
     _fixedFont->setStyleHint(QFont::TypeWriter);
 
     KConfig *c = KGlobal::config();
@@ -312,8 +323,9 @@ QFont KGlobalSettings::toolBarFont()
     if(_toolBarFont)
         return *_toolBarFont;
 
+    // Sync default with kdebase/kcontrol/fonts/fonts.cpp
     _toolBarFont = new QFont("helvetica", 10);
-    _toolBarFont->setPixelSize(10);
+    _toolBarFont->setPointSize(10);
     _toolBarFont->setStyleHint(QFont::SansSerif);
 
     KConfig *c = KGlobal::config();
@@ -328,8 +340,9 @@ QFont KGlobalSettings::menuFont()
     if(_menuFont)
         return *_menuFont;
 
+    // Sync default with kdebase/kcontrol/fonts/fonts.cpp
     _menuFont = new QFont("helvetica", 12);
-    _menuFont->setPixelSize(12);
+    _menuFont->setPointSize(12);
     _menuFont->setStyleHint(QFont::SansSerif);
 
     KConfig *c = KGlobal::config();
@@ -344,8 +357,9 @@ QFont KGlobalSettings::windowTitleFont()
     if(_windowTitleFont)
         return *_windowTitleFont;
 
+    // Sync default with kdebase/kcontrol/fonts/fonts.cpp
     _windowTitleFont = new QFont("helvetica", 12, QFont::Bold);
-    _windowTitleFont->setPixelSize(12);
+    _windowTitleFont->setPointSize(12);
     _windowTitleFont->setStyleHint(QFont::SansSerif);
 
     KConfig *c = KGlobal::config();
@@ -360,8 +374,9 @@ QFont KGlobalSettings::taskbarFont()
     if(_taskbarFont)
         return *_taskbarFont;
 
+    // Sync default with kdebase/kcontrol/fonts/fonts.cpp
     _taskbarFont = new QFont("helvetica", 11);
-    _taskbarFont->setPixelSize(11);
+    _taskbarFont->setPointSize(11);
     _taskbarFont->setStyleHint(QFont::SansSerif);
 
     KConfig *c = KGlobal::config();
@@ -441,27 +456,25 @@ void KGlobalSettings::initStatic() // should be called initPaths(). Don't put an
     s_documentPath = new QString();
 
     KConfig *config = KGlobal::config();
-    bool dollarExpansion = config->isDollarExpansion();
-    config->setDollarExpansion(true);
     KConfigGroupSaver cgs( config, "Paths" );
 
     // Desktop Path
-    *s_desktopPath = QDir::homeDirPath() + "/" + "Desktop" + "/";
+    *s_desktopPath = QDir::homeDirPath() + "/Desktop/";
     *s_desktopPath = config->readPathEntry( "Desktop", *s_desktopPath);
-    if ( (*s_desktopPath)[0] != '/' )
+    if ( !s_desktopPath->startsWith("/") )
       s_desktopPath->prepend( QDir::homeDirPath() + "/" );
     *s_desktopPath = QDir::cleanDirPath( *s_desktopPath );
-    if ( s_desktopPath->right(1) != "/")
-        *s_desktopPath += "/";
+    if ( !s_desktopPath->endsWith("/") )
+      s_desktopPath->append('/');
 
     // Trash Path
     *s_trashPath = *s_desktopPath + i18n("Trash") + "/";
     *s_trashPath = config->readPathEntry( "Trash" , *s_trashPath);
-    if ( (*s_trashPath)[0] != '/' )
+    if ( !s_trashPath->startsWith("/") )
       s_trashPath->prepend( QDir::homeDirPath() + "/" );
     *s_trashPath = QDir::cleanDirPath( *s_trashPath );
-    if ( s_trashPath->right(1) != "/")
-        *s_trashPath += "/";
+    if ( !s_trashPath->endsWith("/") )
+      s_trashPath->append('/');
     // We need to save it in any case, in case the language changes later on,
     if ( !config->hasKey( "Trash" ) )
     {
@@ -470,28 +483,27 @@ void KGlobalSettings::initStatic() // should be called initPaths(). Don't put an
     }
 
     // Autostart Path
-    *s_autostartPath = KGlobal::dirs()->localkdedir() + "Autostart" + "/";
+    *s_autostartPath = KGlobal::dirs()->localkdedir() + "Autostart/";
     *s_autostartPath = config->readPathEntry( "Autostart" , *s_autostartPath);
-    if ( (*s_autostartPath)[0] != '/' )
+    if ( !s_autostartPath->startsWith("/") )
       s_autostartPath->prepend( QDir::homeDirPath() + "/" );
     *s_autostartPath = QDir::cleanDirPath( *s_autostartPath );
-    if ( s_autostartPath->right(1) != "/")
-        *s_autostartPath += "/";
+    if ( !s_autostartPath->endsWith("/") )
+      s_autostartPath->append('/');
 
     // Document Path
-    *s_documentPath = QString::null;
-    *s_documentPath = config->readPathEntry( "Documents" , *s_documentPath);
-    if ( (*s_documentPath)[0] != '/' )
+    *s_documentPath = config->readPathEntry( "Documents" );
+    if ( !s_documentPath->startsWith("/") )
       s_documentPath->prepend( QDir::homeDirPath() + "/" );
     *s_documentPath = QDir::cleanDirPath( *s_documentPath );
-    if ( s_documentPath->right(1) != "/")
-        *s_documentPath += "/";
-
-    config->setDollarExpansion(dollarExpansion);
+    if ( !s_documentPath->endsWith("/"))
+      s_documentPath->append('/');
 
     // Make sure this app gets the notifications about those paths
+#if defined Q_WS_X11 && ! defined K_WS_QTONLY
     if (kapp)
         kapp->addKipcEventMask(KIPC::SettingsChanged);
+#endif
 }
 
 void KGlobalSettings::initColors()
@@ -550,7 +562,7 @@ KGlobalSettings::KMouseSettings & KGlobalSettings::mouseSettings()
             s.handed = KMouseSettings::LeftHanded;
         else
         {
-#ifdef Q_WS_X11
+#if defined Q_WS_X11 && ! defined K_WS_QTONLY
             // get settings from X server
             // This is a simplified version of the code in input/mouse.cpp
             // Keep in sync !
@@ -599,6 +611,86 @@ bool KGlobalSettings::isMultiHead()
 
 bool KGlobalSettings::wheelMouseZooms()
 {
-    KConfigGroup group( KGlobal::config(), "KDE" );
-    return group.readBoolEntry( "WheelMouseZooms", KDE_DEFAULT_WHEEL_ZOOM );
+    KConfig *c = KGlobal::config();
+    KConfigGroupSaver cgs( c, "KDE" );
+    return c->readBoolEntry( "WheelMouseZooms", KDE_DEFAULT_WHEEL_ZOOM );
+}
+
+QRect KGlobalSettings::splashScreenDesktopGeometry()
+{
+    QDesktopWidget *dw = QApplication::desktop();
+
+    if (dw->isVirtualDesktop()) {
+        KConfigGroup group(KGlobal::config(), "Windows");
+        int scr = group.readNumEntry("Unmanaged", -3);
+        if (group.readBoolEntry("XineramaEnabled", true) && scr != -2) {
+            if (scr == -3)
+                scr = dw->screenNumber(QCursor::pos());
+            return dw->screenGeometry(scr);
+        } else {
+            return dw->geometry();
+        }
+    } else {
+        return dw->geometry();
+    }
+}
+
+QRect KGlobalSettings::desktopGeometry(const QPoint& point)
+{
+    QDesktopWidget *dw = QApplication::desktop();
+
+    if (dw->isVirtualDesktop()) {
+        KConfigGroup group(KGlobal::config(), "Windows");
+        if (group.readBoolEntry("XineramaEnabled", true) &&
+            group.readBoolEntry("XineramaPlacementEnabled", true)) {
+            return dw->screenGeometry(dw->screenNumber(point));
+        } else {
+            return dw->geometry();
+        }
+    } else {
+        return dw->geometry();
+    }
+}
+
+QRect KGlobalSettings::desktopGeometry(QWidget* w)
+{
+    QDesktopWidget *dw = QApplication::desktop();
+
+    if (dw->isVirtualDesktop()) {
+        KConfigGroup group(KGlobal::config(), "Windows");
+        if (group.readBoolEntry("XineramaEnabled", true) &&
+            group.readBoolEntry("XineramaPlacementEnabled", true)) {
+            if (w)
+                return dw->screenGeometry(dw->screenNumber(w));
+            else return dw->screenGeometry(-1);
+        } else {
+            return dw->geometry();
+        }
+    } else {
+        return dw->geometry();
+    }
+}
+
+bool KGlobalSettings::showIconsOnPushButtons()
+{
+    KConfig *c = KGlobal::config();
+    KConfigGroupSaver cgs( c, "KDE" );
+    return c->readBoolEntry("ShowIconsOnPushButtons",
+        KDE_DEFAULT_ICON_ON_PUSHBUTTON);
+}
+
+bool KGlobalSettings::showFilePreview(const KURL &url)
+{
+    KConfigGroup g(KGlobal::config(), "PreviewSettings");
+    QString protocol = url.protocol();
+    bool defaultSetting = KProtocolInfo::showFilePreview( protocol );
+    return g.readBoolEntry(protocol, defaultSetting );
+}
+
+bool KGlobalSettings::opaqueResize()
+{
+    KConfig *c = KGlobal::config();
+    KConfigGroupSaver cgs( c, "KDE" );
+    return c->readBoolEntry("OpaqueResize",
+        KDE_DEFAULT_OPAQUE_RESIZE);
 }
