@@ -1,6 +1,6 @@
 /* vi: ts=8 sts=4 sw=4
  *
- * $Id: kiconloader.cpp,v 1.207.2.3 2003/06/26 07:39:21 staikos Exp $
+ * $Id: kiconloader.cpp,v 1.217 2003/11/20 21:41:54 antlarr Exp $
  *
  * This file is part of the KDE project, module kdecore.
  * Copyright (C) 2000 Geert Jansen <jansen@kde.org>
@@ -42,6 +42,7 @@
 #include <unistd.h>     //for readlink
 #include <dirent.h>
 #include <config.h>
+#include <assert.h>
 
 #ifdef HAVE_LIBART
 #include "svgicons/ksvgiconengine.h"
@@ -226,7 +227,7 @@ void KIconLoader::init( const QString& _appname, KStandardDirs *_dirs )
     QStringList dirs;
     dirs += d->mpDirs->resourceDirs("icon");
     dirs += d->mpDirs->resourceDirs("pixmap");
-    for (QStringList::ConstIterator it = dirs.begin(); it != dirs.end(); it++)
+    for (QStringList::ConstIterator it = dirs.begin(); it != dirs.end(); ++it)
 	d->mpDirs->addResourceDir("appicon", *it);
 
 #ifndef NDEBUG
@@ -290,7 +291,7 @@ void KIconLoader::addBaseThemes(KIconThemeNode *node, const QString &appname)
     QStringList lst = node->theme->inherits();
     QStringList::ConstIterator it;
 
-    for (it=lst.begin(); it!=lst.end(); it++)
+    for (it=lst.begin(); it!=lst.end(); ++it)
     {
 	if (!d->mThemeList.contains(*it) ||
 	    ( d->mThemesInTree.contains(*it) && (*it) != "hicolor"))
@@ -316,14 +317,14 @@ void KIconLoader::addExtraDesktopThemes()
     QStringList::ConstIterator it;
     char buf[1000];
     int r;
-    for (it=icnlibs.begin(); it!=icnlibs.end(); it++)
+    for (it=icnlibs.begin(); it!=icnlibs.end(); ++it)
     {
 	QDir dir(*it);
 	if (!dir.exists())
 	    continue;
 	QStringList lst = dir.entryList("default.*", QDir::Dirs);
 	QStringList::ConstIterator it2;
-	for (it2=lst.begin(); it2!=lst.end(); it2++)
+	for (it2=lst.begin(); it2!=lst.end(); ++it2)
 	{
 	    if (!KStandardDirs::exists(*it + *it2 + "/index.desktop")
 		&& !KStandardDirs::exists(*it + *it2 + "/index.theme"))
@@ -334,19 +335,19 @@ void KIconLoader::addExtraDesktopThemes()
 	      buf[r]=0;
 	      QDir dir2( buf );
 	      QString themeName=dir2.dirName();
-	
+
 	      if (!list.contains(themeName))
 		list.append(themeName);
-	    }	
+	    }
 	}
     }
 
-    for (it=list.begin(); it!=list.end(); it++)
+    for (it=list.begin(); it!=list.end(); ++it)
     {
 	if ( d->mThemesInTree.contains(*it) )
 		continue;
 	if ( *it == QString("default.kde") ) continue;
-			
+
 	KIconTheme *def = new KIconTheme( *it, "" );
 	KIconThemeNode* node = new KIconThemeNode(def);
 	d->mThemesInTree.append(*it);
@@ -355,7 +356,7 @@ void KIconLoader::addExtraDesktopThemes()
     }
 
     d->extraDesktopIconsLoaded=true;
-   
+
 }
 
 bool KIconLoader::extraDesktopThemesAdded() const
@@ -390,7 +391,7 @@ QString KIconLoader::removeIconExtension(const QString &name) const
     {
 #ifndef NDEBUG
 	kdDebug(264) << "Application " << KGlobal::instance()->instanceName()
-                     << " loads icon " << name << " with extension.\n";
+                     << " loads icon " << name << " with extension." << endl;
 #endif
 
 	return name.left(name.length() - extensionLength);
@@ -465,7 +466,7 @@ inline QString KIconLoader::unknownIconPath( int size ) const
     if (!icon.isValid())
     {
         kdDebug(264) << "Warning: could not find \"Unknown\" icon for size = "
-                     << size << "\n";
+                     << size << endl;
         return QString::null;
     }
     return icon.path;
@@ -503,10 +504,10 @@ QString KIconLoader::iconPath(const QString& _name, int group_or_size,
 	     path = d->mpDirs->findResource("appicon", name + xpm_ext);
 	return path;
     }
-    
+
     if (group_or_size >= KIcon::LastGroup)
     {
-	kdDebug(264) << "Illegal icon group: " << group_or_size << "\n";
+	kdDebug(264) << "Illegal icon group: " << group_or_size << endl;
 	return path;
     }
 
@@ -531,7 +532,7 @@ QString KIconLoader::iconPath(const QString& _name, int group_or_size,
 	path = iconPath(name, KIcon::User, true);
 	if (!path.isEmpty() || canReturnNull)
 	    return path;
-	
+
 	if (canReturnNull)
 	    return QString::null;
         else
@@ -565,6 +566,7 @@ QPixmap KIconLoader::loadIcon(const QString& _name, KIcon::Group group, int size
     if (group == KIcon::User)
     {
 	key = "$kicou_";
+        key += QString::number(size); key += '_';
 	key += name;
 	bool inCache = QPixmapCache::find(key, pix);
 	if (inCache && (path_store == 0L))
@@ -580,7 +582,7 @@ QPixmap KIconLoader::loadIcon(const QString& _name, KIcon::Group group, int size
 	    path = iconPath(str_unknown, KIcon::Small, true);
 	    if (path.isEmpty())
 	    {
-		kdDebug(264) << "Warning: Cannot find \"unknown\" icon.\n";
+		kdDebug(264) << "Warning: Cannot find \"unknown\" icon." << endl;
 		return pix;
 	    }
 	}
@@ -589,7 +591,11 @@ QPixmap KIconLoader::loadIcon(const QString& _name, KIcon::Group group, int size
 	    *path_store = path;
 	if (inCache)
 	    return pix;
-	pix.load(path);
+	QImage img(path);
+	if (size != 0)
+	    img=img.smoothScale(size,size);
+
+	pix.convertFromImage(img);
 	QPixmapCache::insert(key, pix);
 	return pix;
     }
@@ -598,7 +604,7 @@ QPixmap KIconLoader::loadIcon(const QString& _name, KIcon::Group group, int size
 
     if ((group < -1) || (group >= KIcon::LastGroup))
     {
-	kdDebug(264) << "Illegal icon group: " << group << "\n";
+	kdDebug(264) << "Illegal icon group: " << group << endl;
 	group = KIcon::Desktop;
     }
 
@@ -606,13 +612,13 @@ QPixmap KIconLoader::loadIcon(const QString& _name, KIcon::Group group, int size
     state &= ~KIcon::OverlayMask;
     if ((state < 0) || (state >= KIcon::LastState))
     {
-	kdDebug(264) << "Illegal icon state: " << state << "\n";
+	kdDebug(264) << "Illegal icon state: " << state << endl;
 	state = KIcon::DefaultState;
     }
 
-    if ((size == 0) && (group < 0))
+    if (size == 0 && group < 0)
     {
-	kdDebug(264) << "Neither size nor group specified!\n";
+	kdDebug(264) << "Neither size nor group specified!" << endl;
 	group = KIcon::Desktop;
     }
 
@@ -629,7 +635,7 @@ QPixmap KIconLoader::loadIcon(const QString& _name, KIcon::Group group, int size
     {
 	size = d->mpGroups[group].size;
     }
-    favIconOverlay = favIconOverlay && (size > 22);
+    favIconOverlay = favIconOverlay && size > 22;
 
     // Generate a unique cache key for the icon.
 
@@ -689,7 +695,7 @@ QPixmap KIconLoader::loadIcon(const QString& _name, KIcon::Group group, int size
                 {
                     kdDebug(264)
                         << "Warning: could not find \"Unknown\" icon for size = "
-                        << size << "\n";
+                        << size << endl;
                     return pix;
                 }
             }
@@ -767,16 +773,16 @@ QPixmap KIconLoader::loadIcon(const QString& _name, KIcon::Group group, int size
     }
 
     // Scale the icon and apply effects if necessary
-    if ((iconType == KIcon::Scalable) && (size != img->width()))
+    if (iconType == KIcon::Scalable && size != img->width())
     {
         *img = img->smoothScale(size, size);
     }
-    if ((iconType == KIcon::Threshold) && (size != img->width()))
+    if (iconType == KIcon::Threshold && size != img->width())
     {
 	if ( abs(size-img->width())>iconThreshold )
 	    *img = img->smoothScale(size, size);
     }
-    if ((group >= 0) && d->mpGroups[group].dblPixels)
+    if (group >= 0 && d->mpGroups[group].dblPixels)
     {
 	*img = d->mpEffect.doublePixels(*img);
     }
@@ -804,7 +810,7 @@ QPixmap KIconLoader::loadIcon(const QString& _name, KIcon::Group group, int size
 		// expensive, but works
 		fmask = favIcon.createHeuristicMask();
 	    }
-		
+
             bitBlt(&mask, x, y, &fmask,
                    0, 0, favIcon.width(), favIcon.height(),
                    favIcon.mask() ? Qt::OrROP : Qt::SetROP);
@@ -827,7 +833,7 @@ QImage *KIconLoader::loadOverlay(const QString &name, int size) const
     KIcon icon = findMatchingIcon(name, size);
     if (!icon.isValid())
     {
-	kdDebug(264) << "Overlay " << name << "not found.\n";
+	kdDebug(264) << "Overlay " << name << "not found." << endl;
 	return 0L;
     }
     image = new QImage(icon.path);
@@ -853,14 +859,14 @@ QString KIconLoader::moviePath(const QString& name, KIcon::Group group, int size
 {
     if (!d->mpGroups) return QString::null;
 
-    if ( ((group < -1) || (group >= KIcon::LastGroup)) && (group != KIcon::User) )
+    if ( (group < -1 || group >= KIcon::LastGroup) && group != KIcon::User )
     {
-	kdDebug(264) << "Illegal icon group: " << group << "\n";
+	kdDebug(264) << "Illegal icon group: " << group << endl;
 	group = KIcon::Desktop;
     }
-    if ((size == 0) && (group < 0))
+    if (size == 0 && group < 0)
     {
-	kdDebug(264) << "Neither size nor group specified!\n";
+	kdDebug(264) << "Neither size nor group specified!" << endl;
 	group = KIcon::Desktop;
     }
 
@@ -875,13 +881,27 @@ QString KIconLoader::moviePath(const QString& name, KIcon::Group group, int size
 	    size = d->mpGroups[group].size;
 
         KIcon icon;
-        icon = d->mpThemeRoot->findIcon(file, size, KIcon::MatchExact);
-        if (!icon.isValid())
-        {
-           icon = d->mpThemeRoot->findIcon(file, size, KIcon::MatchBest);
-        }
+	
+	for ( KIconThemeNode *themeNode = d->links.first() ; themeNode ;
+		themeNode = d->links.next() )
+	{
+	    icon = themeNode->theme->iconPath(file, size, KIcon::MatchExact);
+	    if (icon.isValid())
+		break;
+	}
+	
+	if ( !icon.isValid() )
+	{
+	    for ( KIconThemeNode *themeNode = d->links.first() ; themeNode ;
+		    themeNode = d->links.next() )
+	    {
+		icon = themeNode->theme->iconPath(file, size, KIcon::MatchBest);
+		if (icon.isValid())
+		    break;
+	    }
+	}
+	
 	file = icon.isValid() ? icon.path : QString::null;
-
     }
     return file;
 }
@@ -895,12 +915,12 @@ QStringList KIconLoader::loadAnimated(const QString& name, KIcon::Group group, i
 
     if ((group < -1) || (group >= KIcon::LastGroup))
     {
-	kdDebug(264) << "Illegal icon group: " << group << "\n";
+	kdDebug(264) << "Illegal icon group: " << group << endl;
 	group = KIcon::Desktop;
     }
     if ((size == 0) && (group < 0))
     {
-	kdDebug(264) << "Neither size nor group specified!\n";
+	kdDebug(264) << "Neither size nor group specified!" << endl;
 	group = KIcon::Desktop;
     }
 
@@ -948,9 +968,9 @@ int KIconLoader::currentSize(KIcon::Group group) const
 {
     if (!d->mpGroups) return -1;
 
-    if ((group < 0) || (group >= KIcon::LastGroup))
+    if (group < 0 || group >= KIcon::LastGroup)
     {
-	kdDebug(264) << "Illegal icon group: " << group << "\n";
+	kdDebug(264) << "Illegal icon group: " << group << endl;
 	return -1;
     }
     return d->mpGroups[group].size;
@@ -962,7 +982,7 @@ QStringList KIconLoader::queryIconsByDir( const QString& iconsDir ) const
   QStringList lst = dir.entryList("*.png;*.xpm", QDir::Files);
   QStringList result;
   QStringList::ConstIterator it;
-  for (it=lst.begin(); it!=lst.end(); it++)
+  for (it=lst.begin(); it!=lst.end(); ++it)
     result += iconsDir + "/" + *it;
   return result;
 }
@@ -973,7 +993,7 @@ QStringList KIconLoader::queryIconsByContext(int group_or_size,
     QStringList result;
     if (group_or_size >= KIcon::LastGroup)
     {
-	kdDebug(264) << "Illegal icon group: " << group_or_size << "\n";
+	kdDebug(264) << "Illegal icon group: " << group_or_size << endl;
 	return result;
     }
     int size;
@@ -990,7 +1010,7 @@ QStringList KIconLoader::queryIconsByContext(int group_or_size,
     QString name;
     QStringList res2, entries;
     QStringList::ConstIterator it;
-    for (it=result.begin(); it!=result.end(); it++)
+    for (it=result.begin(); it!=result.end(); ++it)
     {
 	int n = (*it).findRev('/');
 	if (n == -1)
@@ -1012,7 +1032,7 @@ QStringList KIconLoader::queryIcons(int group_or_size, KIcon::Context context) c
     QStringList result;
     if (group_or_size >= KIcon::LastGroup)
     {
-	kdDebug(264) << "Illegal icon group: " << group_or_size << "\n";
+	kdDebug(264) << "Illegal icon group: " << group_or_size << endl;
 	return result;
     }
     int size;
@@ -1029,7 +1049,7 @@ QStringList KIconLoader::queryIcons(int group_or_size, KIcon::Context context) c
     QString name;
     QStringList res2, entries;
     QStringList::ConstIterator it;
-    for (it=result.begin(); it!=result.end(); it++)
+    for (it=result.begin(); it!=result.end(); ++it)
     {
 	int n = (*it).findRev('/');
 	if (n == -1)
@@ -1054,9 +1074,9 @@ bool KIconLoader::alphaBlending(KIcon::Group group) const
 {
     if (!d->mpGroups) return -1;
 
-    if ((group < 0) || (group >= KIcon::LastGroup))
+    if (group < 0 || group >= KIcon::LastGroup)
     {
-	kdDebug(264) << "Illegal icon group: " << group << "\n";
+	kdDebug(264) << "Illegal icon group: " << group << endl;
 	return -1;
     }
     return d->mpGroups[group].alphaBlending;
@@ -1084,41 +1104,47 @@ class KIconFactory
     };
 
 
-QIconSet KIconLoader::loadIconSet( const QString& name, KIcon::Group group, int size,
+QIconSet KIconLoader::loadIconSet( const QString& name, KIcon::Group g, int s,
     bool canReturnNull)
 {
     if ( !d->delayedLoading )
-        return loadIconSetNonDelayed( name, group, size, canReturnNull );
+        return loadIconSetNonDelayed( name, g, s, canReturnNull );
+
+    if (g < -1 || g > 6) {
+        kdDebug() << "KIconLoader::loadIconSet " << name << " " << (int)g << " " << s << endl;
+        qDebug("%s", kdBacktrace().latin1());
+        abort();
+    }
 
     if(canReturnNull)
     { // we need to find out if the icon actually exists
-        QPixmap pm = loadIcon( name, group, size, KIcon::DefaultState, NULL, true );
+        QPixmap pm = loadIcon( name, g, s, KIcon::DefaultState, NULL, true );
         if( pm.isNull())
             return QIconSet();
 
         QIconSet ret( pm );
-        ret.installIconFactory( new KIconFactory( name, group, size, this ));
+        ret.installIconFactory( new KIconFactory( name, g, s, this ));
         return ret;
     }
-    
+
     QIconSet ret;
-    ret.installIconFactory( new KIconFactory( name, group, size, this ));
+    ret.installIconFactory( new KIconFactory( name, g, s, this ));
     return ret;
 }
 
-QIconSet KIconLoader::loadIconSetNonDelayed( const QString& name, 
-                                             KIcon::Group group,
-                                             int size, bool canReturnNull )
+QIconSet KIconLoader::loadIconSetNonDelayed( const QString& name,
+                                             KIcon::Group g,
+                                             int s, bool canReturnNull )
 {
     QIconSet iconset;
-    QPixmap tmp = loadIcon(name, group, size, KIcon::ActiveState, NULL, canReturnNull);
+    QPixmap tmp = loadIcon(name, g, s, KIcon::ActiveState, NULL, canReturnNull);
     iconset.setPixmap( tmp, QIconSet::Small, QIconSet::Active );
     // we don't use QIconSet's resizing anyway
     iconset.setPixmap( tmp, QIconSet::Large, QIconSet::Active );
-    tmp = loadIcon(name, group, size, KIcon::DisabledState, NULL, canReturnNull);
+    tmp = loadIcon(name, g, s, KIcon::DisabledState, NULL, canReturnNull);
     iconset.setPixmap( tmp, QIconSet::Small, QIconSet::Disabled );
     iconset.setPixmap( tmp, QIconSet::Large, QIconSet::Disabled );
-    tmp = loadIcon(name, group, size, KIcon::DefaultState, NULL, canReturnNull);
+    tmp = loadIcon(name, g, s, KIcon::DefaultState, NULL, canReturnNull);
     iconset.setPixmap( tmp, QIconSet::Small, QIconSet::Normal );
     iconset.setPixmap( tmp, QIconSet::Large, QIconSet::Normal );
     return iconset;
@@ -1142,7 +1168,7 @@ QPixmap* KIconFactory::createPixmap( const QIconSet&, QIconSet::Size, QIconSet::
     { // active and normal icon are usually the same
 	if( loader->iconEffect()->fingerprint(group, KIcon::ActiveState )
             == loader->iconEffect()->fingerprint(group, KIcon::DefaultState ))
-            return NULL; // so let QIconSet simply duplicate it
+            return 0; // so let QIconSet simply duplicate it
     }
     // ignore passed size
     // ignore passed state (i.e. on/off)
@@ -1256,7 +1282,7 @@ QPixmap KIconLoader::unknown()
     QString path = KGlobal::iconLoader()->iconPath("unknown", KIcon::Small, true);
     if (path.isEmpty())
     {
-	kdDebug(264) << "Warning: Cannot find \"unknown\" icon.\n";
+	kdDebug(264) << "Warning: Cannot find \"unknown\" icon." << endl;
 	pix.resize(32,32);
     } else
     {
