@@ -1,6 +1,6 @@
 /* vi: ts=8 sts=4 sw=4
  *
- * $Id: kicontheme.cpp,v 1.51.2.2 2003/06/29 21:42:59 deller Exp $
+ * $Id: kicontheme.cpp,v 1.56.2.1 2004/05/29 15:22:08 djurban Exp $
  *
  * This file is part of the KDE project, module kdecore.
  * Copyright (C) 2000 Geert Jansen <jansen@kde.org>
@@ -42,7 +42,7 @@ class KIconThemePrivate
 public:
     QString example, screenshot;
     QString linkOverlay, lockOverlay, zipOverlay, shareOverlay;
-    bool hidden; 
+    bool hidden;
 };
 
 /**
@@ -91,11 +91,11 @@ KIconTheme::KIconTheme(const QString& name, const QString& appName)
     // "hicolor" icon themes. For these, the _global_ theme description
     // files are used..
 
-    if (!appName.isEmpty() && 
+    if (!appName.isEmpty() &&
        ( name == "crystalsvg" || name== "hicolor" || name == "locolor" ) )
     {
 	icnlibs = KGlobal::dirs()->resourceDirs("data");
-	for (it=icnlibs.begin(); it!=icnlibs.end(); it++)
+	for (it=icnlibs.begin(); it!=icnlibs.end(); ++it)
 	{
 	    cDir = *it + appName + "/icons/" + name;
 	    if (QFile::exists( cDir ))
@@ -105,7 +105,7 @@ KIconTheme::KIconTheme(const QString& name, const QString& appName)
     // Find the theme description file. These are always global.
 
     icnlibs = KGlobal::dirs()->resourceDirs("icon");
-    for (it=icnlibs.begin(); it!=icnlibs.end(); it++)
+    for (it=icnlibs.begin(); it!=icnlibs.end(); ++it)
     {
         cDir = *it + name + "/";
         if (KStandardDirs::exists(cDir))
@@ -151,16 +151,19 @@ KIconTheme::KIconTheme(const QString& name, const QString& appName)
 
     QStringList dirs = cfg.readPathListEntry("Directories");
     mDirs.setAutoDelete(true);
-    for (it=dirs.begin(); it!=dirs.end(); it++)
+    for (it=dirs.begin(); it!=dirs.end(); ++it)
     {
 	cfg.setGroup(*it);
-	for (itDir=themeDirs.begin(); itDir!=themeDirs.end(); itDir++)
+	for (itDir=themeDirs.begin(); itDir!=themeDirs.end(); ++itDir)
 	{
 	    if (KStandardDirs::exists(*itDir + *it + "/"))
 	    {
 	        KIconThemeDir *dir = new KIconThemeDir(*itDir + *it, &cfg);
 	        if (!dir->isValid())
+	        {
+	            kdWarning(264) << "Icon directory " << *itDir << " group " << *it << " not valid.\n";
 	            delete dir;
+	        }
 	        else
 	            mDirs.append(dir);
             }
@@ -189,12 +192,12 @@ KIconTheme::KIconTheme(const QString& name, const QString& appName)
     groups += "Panel";
     const int defDefSizes[] = { 32, 22, 22, 16, 32 };
     cfg.setGroup(mainSection);
-    for (it=groups.begin(), i=0; it!=groups.end(); it++, i++)
+    for (it=groups.begin(), i=0; it!=groups.end(); ++it, i++)
     {
         mDefSize[i] = cfg.readNumEntry(*it + "Default", defDefSizes[i]);
-        QValueList<int> lst = cfg.readIntListEntry(*it + "Sizes"), exp;
+        QValueList<int> exp, lst = cfg.readIntListEntry(*it + "Sizes");
         QValueList<int>::ConstIterator it2;
-        for (it2=lst.begin(); it2!=lst.end(); it2++)
+        for (it2=lst.begin(); it2!=lst.end(); ++it2)
         {
             if (scIcons.contains(*it2))
                 exp += scIcons[*it2];
@@ -263,13 +266,13 @@ QStringList KIconTheme::queryIcons(int size, KIcon::Context context) const
         dir = dirs.current();
         if ((context != KIcon::Any) && (context != dir->context()))
             continue;
-        if ((dir->type() == KIcon::Fixed) && (dir->size() == size)) 
+        if ((dir->type() == KIcon::Fixed) && (dir->size() == size))
         {
             result += dir->iconList();
             continue;
         }
         if ((dir->type() == KIcon::Scalable) &&
-            (size >= dir->minSize()) && (size <= dir->maxSize())) 
+            (size >= dir->minSize()) && (size <= dir->maxSize()))
         {
             result += dir->iconList();
             continue;
@@ -280,7 +283,7 @@ QStringList KIconTheme::queryIcons(int size, KIcon::Context context) const
     }
 
     return result;
-        
+
     dirs.toFirst();
 
     // Find close match
@@ -310,7 +313,8 @@ QStringList KIconTheme::queryIconsByContext(int size, KIcon::Context context) co
 
     // We want all the icons for a given context, but we prefer icons
     // of size size . Note that this may (will) include duplicate icons
-    QStringList iconlist[34]; // 33 == 48-16+1
+    //QStringList iconlist[34]; // 33 == 48-16+1
+    QStringList iconlist[128]; // 33 == 48-16+1
     // Usually, only the 0, 6 (22-16), 10 (32-22), 16 (48-32 or 32-16),
     // 26 (48-22) and 32 (48-16) will be used, but who knows if someone
     // will make icon themes with different icon sizes.
@@ -321,11 +325,11 @@ QStringList KIconTheme::queryIconsByContext(int size, KIcon::Context context) co
         if ((context != KIcon::Any) && (context != dir->context()))
             continue;
         dw = abs(dir->size() - size);
-        iconlist[(dw<33)?dw:33]+=dir->iconList();
+        iconlist[(dw<127)?dw:127]+=dir->iconList();
     }
 
     QStringList iconlistResult;
-    for (int i=0; i<34; i++) iconlistResult+=iconlist[i];
+    for (int i=0; i<128; i++) iconlistResult+=iconlist[i];
 
     return iconlistResult;
 }
@@ -355,10 +359,34 @@ KIcon KIconTheme::iconPath(const QString& name, int size, KIcon::MatchType match
                 continue;
         } else
         {
+          // dw < 0 means need to scale up to get an icon of the requested size
+          if (dir->type() == KIcon::Fixed)
+          {
             dw = dir->size() - size;
-            if (dir->type() != KIcon::Threshold &&
-               ((dw > 7) || (abs(dw) >= abs(delta))))
-                continue;
+          } else if (dir->type() == KIcon::Scalable)
+          {
+            if (size < dir->minSize())
+              dw = dir->minSize() - size;
+            else if (size > dir->maxSize())
+              dw = dir->maxSize() - size;
+            else
+              dw = 0;
+          } else if (dir->type() == KIcon::Threshold)
+          {
+            if (size < dir->size() - dir->threshold())
+              dw = dir->size() - dir->threshold() - size;
+            else if (size > dir->size() + dir->threshold())
+              dw = dir->size() + dir->threshold() - size;
+            else
+              dw = 0;
+          }
+          /* Skip this if we've found a closer one, unless
+             it's a downscale, and we only had upscales befores.
+             This is to avoid scaling up unless we have to,
+             since that looks very ugly */
+          if ((abs(dw) >= abs(delta)) &&
+              !(delta > 0 && dw < 0))
+            continue;
         }
 
         path = dir->iconPath(name);
@@ -420,20 +448,23 @@ QStringList KIconTheme::list()
     _theme_list = new QStringList();
     QStringList icnlibs = KGlobal::dirs()->resourceDirs("icon");
     QStringList::ConstIterator it;
-    for (it=icnlibs.begin(); it!=icnlibs.end(); it++)
+    for (it=icnlibs.begin(); it!=icnlibs.end(); ++it)
     {
         QDir dir(*it);
         if (!dir.exists())
             continue;
         QStringList lst = dir.entryList(QDir::Dirs);
         QStringList::ConstIterator it2;
-        for (it2=lst.begin(); it2!=lst.end(); it2++)
+        for (it2=lst.begin(); it2!=lst.end(); ++it2)
         {
             if ((*it2 == ".") || (*it2 == "..") || (*it2).startsWith("default.") )
                 continue;
             if (!KStandardDirs::exists(*it + *it2 + "/index.desktop") && !KStandardDirs::exists(*it + *it2 + "/index.theme"))
                 continue;
-            if (!_theme_list->contains(*it2))
+		KIconTheme oink(*it2);
+	    if (!oink.isValid()) continue;
+
+	    if (!_theme_list->contains(*it2))
                 _theme_list->append(*it2);
         }
     }
@@ -526,7 +557,7 @@ QStringList KIconThemeDir::iconList() const
 #endif
     QStringList result;
     QStringList::ConstIterator it;
-    for (it=lst.begin(); it!=lst.end(); it++)
+    for (it=lst.begin(); it!=lst.end(); ++it)
         result += mDir + "/" + *it;
     return result;
 }
