@@ -17,18 +17,36 @@
 */
 #include "kinstance.h"
 
+#include <unistd.h>
+
 #include "kconfig.h"
 #include "klocale.h"
 #include "kcharsets.h"
 #include "kiconloader.h"
 #include "kaboutdata.h"
 #include "kstandarddirs.h"
+#include "kuser.h"
 #include "kdebug.h"
 #include "kglobal.h"
 #include "kmimesourcefactory.h"
 
 #include <qfont.h>
 
+#include "config.h"
+#ifndef NDEBUG
+  #include <assert.h>
+  #include <qptrdict.h>
+  static QPtrList<KInstance> *allInstances = 0;
+  static QPtrDict<QCString> *allOldInstances = 0;
+  #define DEBUG_ADD do { if (!allInstances) { allInstances = new QPtrList<KInstance>(); allOldInstances = new QPtrDict<QCString>(); } allInstances->append(this); allOldInstances->insert( this, new QCString( _name)); } while (false);
+  #define DEBUG_REMOVE do { allInstances->removeRef(this); } while (false);
+  #define DEBUG_CHECK_ALIVE do { if (!allInstances->contains((KInstance*)this)) { QCString *old = allOldInstances->find((KInstance*)this); qWarning("ACCESSING DELETED KINSTANCE! (%s)", old ? old->data() : "<unknown>"); assert(false); } } while (false);
+#else
+  #define DEBUG_ADD
+  #define DEBUG_REMOVE
+  #define DEBUG_CHECK_ALIVE
+#endif
+
 class KInstancePrivate
 {
 public:
@@ -45,6 +63,7 @@ public:
     KMimeSourceFactory* mimeSourceFactory;
     QString configName;
     bool ownAboutdata;
+    KSharedConfig::Ptr sharedConfig;
 };
 
 KInstance::KInstance( const QCString& name)
@@ -53,6 +72,7 @@ KInstance::KInstance( const QCString& name)
     _iconLoader (0L),
     _name( name ), _aboutData( new KAboutData( name, "", 0 ) )
 {
+    DEBUG_ADD
     Q_ASSERT(!name.isEmpty());
     if (!KGlobal::_instance)
     {
@@ -70,6 +90,7 @@ KInstance::KInstance( const KAboutData * aboutData )
     _iconLoader (0L),
     _name( aboutData->appName() ), _aboutData( aboutData )
 {
+    DEBUG_ADD
     Q_ASSERT(!_name.isEmpty());
 
     if (!KGlobal::_instance)
@@ -88,6 +109,7 @@ KInstance::KInstance( KInstance* src )
     _iconLoader ( src->_iconLoader ),
     _name( src->_name ), _aboutData( src->_aboutData )
 {
+    DEBUG_ADD
     Q_ASSERT(!_name.isEmpty());
 
     if (!KGlobal::_instance || KGlobal::_instance == src )
@@ -98,6 +120,7 @@ KInstance::KInstance( KInstance* src )
 
     d = new KInstancePrivate ();
     d->ownAboutdata = src->d->ownAboutdata;
+    d->sharedConfig = src->d->sharedConfig;
 
     src->_dirs = 0L;
     src->_config = 0L;
@@ -108,6 +131,9 @@ KInstance::KInstance( KInstance* src )
 
 KInstance::~KInstance()
 {
+    DEBUG_CHECK_ALIVE
+    DEBUG_REMOVE
+
     if (d->ownAboutdata)
         delete _aboutData;
     _aboutData = 0;
@@ -117,7 +143,8 @@ KInstance::~KInstance()
 
     delete _iconLoader;
     _iconLoader = 0;
-    delete _config;
+
+    // delete _config; // Do not delete, stored in d->sharedConfig
     _config = 0;
     delete _dirs;
     _dirs = 0;
@@ -131,6 +158,7 @@ KInstance::~KInstance()
 
 KStandardDirs *KInstance::dirs() const
 {
+    DEBUG_CHECK_ALIVE
     if( _dirs == 0 ) {
 	_dirs = new KStandardDirs( );
         if (_config)
@@ -141,33 +169,62 @@ KStandardDirs *KInstance::dirs() const
     return _dirs;
 }
 
+extern bool kde_kiosk_exception;
+
 KConfig	*KInstance::config() const
 {
+    DEBUG_CHECK_ALIVE
     if( _config == 0 ) {
         if ( !d->configName.isEmpty() )
         {
-            _config = new KConfig( d->configName );
+            d->sharedConfig = KSharedConfig::openConfig( d->configName );
+
             // Check whether custom config files are allowed.
-            _config->setGroup( "KDE Action Restrictions" );
-            if (_config->readBoolEntry( "custom_config", true))
+            d->sharedConfig->setGroup( "KDE Action Restrictions" );
+            QString kioskException = d->sharedConfig->readEntry("kiosk_exception");
+            if (d->sharedConfig->readBoolEntry( "custom_config", true))
             {
-               _config->setGroup(QString::null);
+               d->sharedConfig->setGroup(QString::null);
             }
             else
             {
-               delete _config;
-               _config = 0;
+               d->sharedConfig = 0;
             }
 
         }
 
-        if ( _config == 0 )
+        if ( d->sharedConfig == 0 )
         {
 	    if ( !_name.isEmpty() )
-	        _config = new KConfig( _name + "rc");
+	        d->sharedConfig = KSharedConfig::openConfig( _name + "rc");
 	    else
-	        _config = new KConfig();
+	        d->sharedConfig = KSharedConfig::openConfig( QString::null );
 	}
+        d->sharedConfig->setGroup( "KDE Action Restrictions" );
+        QString kioskException = d->sharedConfig->readEntry("kiosk_exception");
+        d->sharedConfig->setGroup( QString::null );
+        if (!kioskException.isEmpty() && !kde_kiosk_exception)
+        {
+            int i = kioskException.find(':');
+            QString user = kioskException.left(i);
+            QString host = kioskException.mid(i+1);
+
+            KUser thisUser;
+            char hostname[ 256 ];
+            hostname[ 0 ] = '\0';
+            if (!gethostname( hostname, 255 ))
+                hostname[sizeof(hostname)-1] = '\0';
+                       
+            if ((user == thisUser.loginName()) &&
+                (host.isEmpty() || (host == hostname)))
+            {
+                kde_kiosk_exception = true;
+                d->sharedConfig = 0;
+                return config(); // Reread...
+            }
+        }
+	
+	_config = d->sharedConfig;
         if (_dirs)
             if (_dirs->addCustomized(_config))
                 _config->reparseConfiguration();
@@ -176,13 +233,24 @@ KConfig	*KInstance::config() const
     return _config;
 }
 
+KSharedConfig *KInstance::sharedConfig() const
+{
+    DEBUG_CHECK_ALIVE
+    if (_config == 0)
+       (void) config(); // Initialize config
+
+    return d->sharedConfig;
+}
+
 void KInstance::setConfigName(const QString &configName)
 {
+    DEBUG_CHECK_ALIVE
     d->configName = configName;
 }
 
 KIconLoader *KInstance::iconLoader() const
 {
+    DEBUG_CHECK_ALIVE
     if( _iconLoader == 0 ) {
 	_iconLoader = new KIconLoader( _name, dirs() );
     _iconLoader->enableDelayedIconSetLoading( true );
@@ -193,22 +261,26 @@ KIconLoader *KInstance::iconLoader() const
 
 void KInstance::newIconLoader() const
 {
+    DEBUG_CHECK_ALIVE
     KIconTheme::reconfigure();
     _iconLoader->reconfigure( _name, dirs() );
 }
 
 const KAboutData * KInstance::aboutData() const
 {
+    DEBUG_CHECK_ALIVE
     return _aboutData;
 }
 
 QCString KInstance::instanceName() const
 {
+    DEBUG_CHECK_ALIVE
     return _name;
 }
 
 KMimeSourceFactory* KInstance::mimeSourceFactory () const
 {
+  DEBUG_CHECK_ALIVE
   if (!d->mimeSourceFactory)
   {
     d->mimeSourceFactory = new KMimeSourceFactory(iconLoader());
