@@ -23,25 +23,22 @@
 /*
  * kipc.cpp: Send a message to one/all KDE apps.
  *
- * $Id: kipc.cpp,v 1.11 2001/08/29 15:25:38 bero Exp $
+ * $Id: kipc.cpp,v 1.13 2003/09/03 15:33:13 schroder Exp $
  */
+#include "config.h"
 
 #include <qwindowdefs.h>
 
-#ifdef Q_WS_X11
-#include <X11/X.h>
-#include <X11/Xlib.h>
+#if defined Q_WS_X11 && ! defined K_WS_QTONLY
+#include <X11/X.h> // schroder
+#include <X11/Xlib.h> // schroder
+#include <kxerrorhandler.h> // schroder
 #endif
 
 #include <kipc.h>
 
 
-#ifndef Q_WS_QWS
-static int dropError(Display *, XErrorEvent *)
-{
-    return 0;
-}
-
+#if defined Q_WS_X11 && ! defined K_WS_QTONLY
 static long getSimpleProperty(Window w, Atom a)
 {
     Atom real_type;
@@ -57,10 +54,11 @@ static long getSimpleProperty(Window w, Atom a)
     if (p) XFree((char *) p);
     return res;
 }
-
+#endif
 
 void KIPC::sendMessage(Message msg, WId w, int data)
 {
+#if defined Q_WS_X11 && ! defined K_WS_QTONLY
     static Atom a = 0;
     if (a == 0)
 	a = XInternAtom(qt_xdisplay(), "KIPC_COMM_ATOM", False);
@@ -83,19 +81,19 @@ void KIPC::sendMessage(Message msg, WId w, int data)
 	XSendEvent(qt_xdisplay(), (Window) w, False, 0L, &ev);
     }
 
+#endif
 }
 
 
 void KIPC::sendMessageAll(Message msg, int data)
 {
+#if defined Q_WS_X11 && ! defined K_WS_QTONLY
     unsigned int i, nrootwins;
     Window dw1, dw2, *rootwins = 0;
-    int (*defaultHandler)(Display *, XErrorEvent *);
     Display *dpy = qt_xdisplay();
     int screen_count = ScreenCount(dpy);
 
-    defaultHandler = XSetErrorHandler(&dropError);
-
+    KXErrorHandler handler;
     for (int s = 0; s < screen_count; s++) {
 	Window root = RootWindow(dpy, s);
 
@@ -106,12 +104,9 @@ void KIPC::sendMessageAll(Message msg, int data)
 		if (getSimpleProperty(rootwins[i], a) != 0L)
 		    sendMessage(msg, rootwins[i], data);
 	    }
+        XFree((char *) rootwins);
     }
-
-    XFlush(dpy);
-    XSetErrorHandler(defaultHandler);
-    XFree((char *) rootwins);
-}
-#else
-	// FIXME(E): Implement in Qt Embedded
+    XSync(dpy,False);
 #endif
+}
+
