@@ -22,10 +22,12 @@
 
 #include <kshortcut.h>
 
+#ifdef Q_WS_X11
 typedef union _XEvent XEvent;
+#endif
 
 class KKeyNativePrivate;
-/** 
+/**
  * Representation of a key in the format native of the windowing system (eg. X11).
  * @see KKey
  */
@@ -40,15 +42,17 @@ class KKeyNative
 	 */
 	KKeyNative();
 
+#ifdef Q_WS_X11
 	/**
 	 * Extracts a new native key from the given xevent.
 	 * @param xevent the XEvent that contains the key
 	 */
 	KKeyNative( const XEvent* xevent );
+#endif
 
 	/**
 	 * Creates a new native key for the given KKey code.
-	 * @param key the @ref KKey that contains the generic key
+	 * @param key the KKey that contains the generic key
 	 */
 	KKeyNative( const KKey& key );
 
@@ -73,21 +77,21 @@ class KKeyNative
 	 * Initializes the native key by extracting the information
 	 * from the given xevent.
 	 * @param xevent the XEvent that contains the key
-	 * @return true if succesful, false otherwise
+	 * @return true if successful, false otherwise
 	 */
 	bool init( const XEvent* xevent );
 
 	/**
 	 * Creates a new native key for the given KKey code.
-	 * @param key the @ref KKey that contains the generic key
-	 * @return true if succesful, false otherwise
+	 * @param key the KKey that contains the generic key
+	 * @return true if successful, false otherwise
 	 */
 	bool init( const KKey& key );
 
 	/**
 	 * Copies the given key into this key.
 	 * @param key the key to copy
-	 * @return true if succesful, false otherwise
+	 * @return true if successful, false otherwise
 	 */
 	bool init( const KKeyNative& key );
 
@@ -95,7 +99,7 @@ class KKeyNative
 	 * Copies the given key into this key.
 	 * @param key the key to copy
 	 * @return this key
-	 */	
+	 */
 	KKeyNative& operator =( const KKeyNative& key )
 		{ init( key ); return *this; }
 
@@ -104,7 +108,7 @@ class KKeyNative
 	 * @return the qt key code or 0 if there is no key set.
 	 * @see Qt::Key
 	 */
-	
+
 	int keyCodeQt() const;
 
 	/**
@@ -139,7 +143,7 @@ class KKeyNative
 	uint sym() const;
 
 	/**
-	 * Returns true if the key is null (after @ref clear() or empty
+	 * Returns true if the key is null (after clear() or empty
 	 * constructor).
 	 * @return true if the key is null
 	 * @see clear()
@@ -148,14 +152,14 @@ class KKeyNative
 	bool isNull() const;
 
 	/**
-	 * Compares this key with the given KKeyNative object. Returns a 
-	 * negative number if the given KKeyNative is larger, 0 if they 
-	 * are equal and a positive number this KKeyNative is larger. The 
+	 * Compares this key with the given KKeyNative object. Returns a
+	 * negative number if the given KKeyNative is larger, 0 if they
+	 * are equal and a positive number this KKeyNative is larger. The
 	 * returned value is the difference between the symbol, modifier
 	 * or code, whatever is non-zero first.
 	 *
 	 * @param key the key to compare with this key
-	 * @return a negative number if the given KKeyNative is larger, 0 if 
+	 * @return a negative number if the given KKeyNative is larger, 0 if
 	 * they are equal and a positive number this KKeyNative is larger
 	 */
 	int compare( const KKeyNative& key ) const;
