@@ -16,6 +16,8 @@
    the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
    Boston, MA 02111-1307, USA.
 */
+#include "config.h"
+
 #include <config.h>
 #include <qclipboard.h>
 #include <qfile.h>
@@ -32,9 +34,9 @@
 
 #include "ltdl.h"
 
-#ifdef Q_WS_X11
-#include <X11/Xlib.h>
-#include <X11/Xatom.h>
+#if defined Q_WS_X11 && ! defined K_WS_QTONLY
+#include <X11/Xlib.h> // schroder
+#include <X11/Xatom.h> // schroder
 #endif
 
 template class QAsciiDict<KLibrary>;
@@ -62,6 +64,19 @@ extern "C" {
 extern int lt_dlopen_flag;
 }
 
+class KLibLoaderPrivate
+{
+public:
+    QPtrList<KLibWrapPrivate> loaded_stack;
+    QPtrList<KLibWrapPrivate> pending_close;
+    enum {UNKNOWN, UNLOAD, DONT_UNLOAD} unload_mode;
+
+    QString errorMessage;
+};
+
+KLibLoader* KLibLoader::s_self = 0;
+
+// -------------------------------------------------------------------------
 
 KLibFactory::KLibFactory( QObject* parent, const char* name )
     : QObject( parent, name )
@@ -124,6 +139,7 @@ KLibrary::~KLibrary()
     if ( m_factory ) {
 //	kdDebug(150) << " ... deleting the factory " << m_factory << endl;
 	delete m_factory;
+        m_factory = 0L;
     }
 }
 
@@ -148,7 +164,8 @@ KLibFactory* KLibrary::factory()
     void* sym = symbol( symname );
         if ( !sym )
         {
-            kdWarning(150) << "KLibrary: The library " << name() << " does not offer an init_" << name() << " function" << endl;
+            KLibLoader::self()->d->errorMessage = i18n( "The library %1 does not offer an %2 function." ).arg( name() ).arg( "init_" + name() );
+            kdWarning(150) << KLibLoader::self()->d->errorMessage << endl;
             return 0;
     }
 
@@ -158,7 +175,8 @@ KLibFactory* KLibrary::factory()
 
     if( !m_factory )
     {
-        kdWarning(150) << "KLibrary: The library " << name() << " does not offer a KDE compatible factory" << endl;
+        KLibLoader::self()->d->errorMessage = i18n( "The library %1 does not offer a KDE compatible factory." ).arg( name() );
+        kdWarning(150) << KLibLoader::self()->d->errorMessage << endl;
         return 0;
     }
 
@@ -173,7 +191,8 @@ void* KLibrary::symbol( const char* symname ) const
     void* sym = lt_dlsym( (lt_dlhandle) m_handle, symname );
     if ( !sym )
     {
-        kdWarning(150) << "KLibrary: " << lt_dlerror() << endl;
+        KLibLoader::self()->d->errorMessage = "KLibrary: " + QString::fromLatin1( lt_dlerror() );
+        kdWarning(150) << KLibLoader::self()->d->errorMessage << endl;
         return 0;
     }
 
@@ -274,18 +293,6 @@ KLibWrapPrivate::KLibWrapPrivate(KLibrary *l, lt_dlhandle h)
     }
 }
 
-class KLibLoaderPrivate
-{
-public:
-    QPtrList<KLibWrapPrivate> loaded_stack;
-    QPtrList<KLibWrapPrivate> pending_close;
-    enum {UNKNOWN, UNLOAD, DONT_UNLOAD} unload_mode;
-
-    QString errorMessage;
-};
-
-KLibLoader* KLibLoader::s_self = 0;
-
 KLibLoader* KLibLoader::self()
 {
     if ( !s_self )
@@ -299,7 +306,7 @@ void KLibLoader::cleanUp()
     return;
 
   delete s_self;
-  s_self = 0;
+  s_self = 0L;
 }
 
 KLibLoader::KLibLoader( QObject* parent, const char* name )
@@ -331,6 +338,7 @@ KLibLoader::~KLibLoader()
     close_pending(0);
 
     delete d;
+    d = 0L;
 }
 
 //static
@@ -433,7 +441,6 @@ KLibrary* KLibLoader::library( const char *name )
             d->errorMessage = QString::fromLatin1(errmsg);
         else
             d->errorMessage = QString::null;
-        kdWarning(150) << "library=" << name << ": file=" << libfile << ": " << d->errorMessage << endl;
         return 0;
       }
       else
@@ -511,12 +518,16 @@ void KLibLoader::close_pending(KLibWrapPrivate *wrap)
     if (wrap->lib) {
       disconnect( wrap->lib, SIGNAL( destroyed() ),
                   this, SLOT( slotLibraryDestroyed() ) );
-      delete wrap->lib;
-      wrap->lib = 0;
+      KLibrary* to_delete = wrap->lib;
+      wrap->lib = 0L; // unset first, because KLibrary dtor can cause
+      delete to_delete; // recursive call to close_pending()
     }
   }
 
-  if (d->unload_mode == KLibLoaderPrivate::DONT_UNLOAD) return;
+  if (d->unload_mode == KLibLoaderPrivate::DONT_UNLOAD) {
+    d->pending_close.clear();
+    return;
+  }
 
   bool deleted_one = false;
   while ((wrap = d->loaded_stack.first())) {
@@ -539,7 +550,8 @@ void KLibLoader::close_pending(KLibWrapPrivate *wrap)
 
 //    kdDebug(150) << "try to dlclose " << wrap->name << ": yes, done." << endl;
 
-#ifndef Q_WS_QWS
+#if defined Q_WS_X11 && ! defined K_WS_QTONLY
+//#ifndef Q_WS_QWS
     if ( !deleted_one ) {
       /* Only do the hack once in this loop.
          WABA: *HACK*
