@@ -36,13 +36,15 @@ class KLibLoaderPrivate;
 class KLibraryPrivate;
 
 #define K_EXPORT_COMPONENT_FACTORY( libname, factory ) \
-    extern "C" { void *init_##libname() { return new factory; } };
+    extern "C" { void *init_##libname() { return new factory; } }
 
 /**
  * @short Represents a dynamically loaded library.
  *
  * KLibrary allows you to look up symbols of the shared library.
+ * Use KLibLoader to create a new instance of KLibrary.
  *
+ * @see KLibLoader
  * @author Torben Weis <weis@kde.org>
  */
 class KLibrary : public QObject
@@ -54,7 +56,7 @@ class KLibrary : public QObject
 public:
     /**
      * @internal
-     * Don't create KLibrary objects on your own. Instead use @ref KLibLoader.
+     * Don't create KLibrary objects on your own. Instead use KLibLoader.
      */
     KLibrary( const QString& libname, const QString& filename, void * handle );
 
@@ -79,18 +81,18 @@ public:
     /**
      * Looks up a symbol from the library. This is a very low level
      * function that you usually don't want to use. Usually you should
-     * check using @ref hasSymbol() whether the symbol actually exists,
+     * check using hasSymbol() whether the symbol actually exists,
      * otherwise a warning will be printed.
      * @param name the name of the symbol to look up
      * @return the address of the symbol, or 0 if it does not exist
-     * @see #hasSymbol
+     * @see hasSymbol
      */
     void* symbol( const char* name ) const;
 
     /**
      * Looks up a symbol from the library. This is a very low level
      * function that you usually don't want to use.
-     * Unlike @ref #symbol(), this method doesn't warn if the symbol doesn't exist,
+     * Unlike symbol(), this method doesn't warn if the symbol doesn't exist,
      * so if the symbol might or might not exist, better use hasSymbol() before symbol().
      * @param name the name of the symbol to check
      * @return true if the symbol exists
@@ -113,7 +115,7 @@ private slots:
 private:
     /**
      * @internal
-     * Don't destruct KLibrary objects yourself. Instead use @ref unload() instead.
+     * Don't destruct KLibrary objects yourself. Instead use unload() instead.
      */
     ~KLibrary();
 
@@ -133,8 +135,9 @@ class KLibWrapPrivate;
  * Dependent libraries are loaded automatically.
  *
  * KLibLoader follows the singleton pattern. You can not create multiple
- * instances. Use @ref #self() to get a pointer to the loader.
+ * instances. Use self() to get a pointer to the loader.
  *
+ * @see KLibrary
  * @author Torben Weis <weis@kde.org>
  */
 class KLibLoader : public QObject
@@ -166,9 +169,9 @@ public:
      *                 You can, however, give a library name ending in ".so"
      *                 (or whatever is used on your platform), and the library
      *                 will be loaded without resolving dependencies. USE WITH CARE :)
-     * @return the @ref KLibFactory, or 0 if the library does not exist or it does
+     * @return the KLibFactory, or 0 if the library does not exist or it does
      *         not have a factory
-     * @see #library
+     * @see library
      */
     KLibFactory* factory( const char* libname );
 
@@ -187,10 +190,10 @@ public:
      *                 You can, however, give a library name ending in ".so"
      *                 (or whatever is used on your platform), and the library
      *                 will be loaded without resolving dependencies. USE WITH CARE :)
-     * @return @ref KLibrary is invalid (0) when the library couldn't be dlopened. in such
+     * @return KLibrary is invalid (0) when the library couldn't be dlopened. in such
      * a case you can retrieve the error message by calling KLibLoader::lastErrorMessage()
      *
-     * @see #factory
+     * @see factory
      */
     virtual KLibrary* library( const char* libname );
 
@@ -200,7 +203,7 @@ public:
      * globally accessible so libraries with dependencies can be loaded
      * sequentially.
      *
-     * @param libname  This is the library name without extension. Usually that is something like
+     * @param name     This is the library name without extension. Usually that is something like
      *                 "libkspread". The function will then search for a file named
      *                 "libkspread.la" in the KDE library paths.
      *                 The *.la files are created by libtool and contain
@@ -214,13 +217,13 @@ public:
      * @return KLibrariy is invalid (0) when the library couldn't be dlopened. in such
      * a case you can retrieve the error message by calling KLibLoader::lastErrorMessage()
      *
-     * @see #factory
+     * @see factory
      */
     KLibrary* globalLibrary( const char *name );
 
     /*
      * Returns an error message that can be useful to debug the problem.
-     * Returns QString::null if the last call to @ref #library() was successful.
+     * Returns QString::null if the last call to library() was successful.
      * You can call this function more than once. The error message is only
      * reset by a new call to library().
      * @return the last error message, or QString::null if there was no error
@@ -252,7 +255,7 @@ public:
     static KLibLoader* self();
 
     /**
-     * @internal 
+     * @internal
      * Internal Method, called by the KApplication destructor.
      * Do not call it.
      * This is what makes it possible to rely on ~KLibFactory
@@ -267,7 +270,7 @@ public:
      * Made public for code that doesn't use KLibLoader itself, but still
      * wants to open modules.
      * @param name of the library. If it is not a path, the function searches in
-     *             the "module" and "lib" resources. If there is no extension, 
+     *             the "module" and "lib" resources. If there is no extension,
      *             ".la" will be appended.
      * @param instance a KInstance used to get the standard paths
      */
@@ -294,33 +297,33 @@ private:
  * If you develop a library that is to be loaded dynamically at runtime, then
  * you should return a pointer to your factory. The K_EXPORT_COMPONENT_FACTORY
  * macro is provided for this purpose:
- * <pre>
+ * \code
  *   K_EXPORT_COMPONENT_FACTORY( libkspread, KSpreadFactory )
- * </pre>
+ * \endcode
  *
  * The first macro argument is the name of your library, the second specifies the name
  * of your factory.
  *
- * In the constructor of your factory you should create an instance of @ref KInstance
+ * In the constructor of your factory you should create an instance of KInstance
  * like this:
- * <pre>
+ * \code
  *     s_global = new KInstance( "kspread" );
- * </pre>
- * This @ref KInstance is comparable to @ref KGlobal used by normal applications.
+ * \endcode
+ * This KInstance is comparable to KGlobal used by normal applications.
  * It allows you to find resource files (images, XML, sound etc.) belonging
  * to the library.
  *
- * If you want to load a library, use @ref KLibLoader. You can query @ref KLibLoader
- * directly for a pointer to the libraries factory by using the @ref KLibLoader::factory()
+ * If you want to load a library, use KLibLoader. You can query KLibLoader
+ * directly for a pointer to the libraries factory by using the KLibLoader::factory()
  * function.
  *
  * The KLibFactory is used to create the components, the library has to offer.
  * The factory of KSpread for example will create instances of KSpreadDoc,
  * while the Konqueror factory will create KonqView widgets.
- * All objects created by the factory must be derived from @ref QObject, since @ref QObject
+ * All objects created by the factory must be derived from QObject, since QObject
  * offers type safe casting.
  *
- * KLibFactory is an abstract class. Reimplement the @ref
+ * KLibFactory is an abstract class. Reimplement the
  * createObject() method to give it functionality.
  *
  * @author Torben Weis <weis@kde.org>
@@ -347,7 +350,7 @@ public:
      * if asked for a "QWidget", it could create a wrapper widget,
      * that encapsulates the Koffice specific features.
      *
-     * create() automatically emits a signal @ref objectCreated to tell
+     * create() automatically emits a signal objectCreated to tell
      * the library about its newly created object.  This is very
      * important for reference counting, and allows unloading the
      * library automatically once all its objects have been destroyed.
@@ -375,7 +378,7 @@ protected:
      * the requested classname.
      *
      * It is valid behavior to create different kinds of objects
-     * depending on the requested @p classname. For example a koffice
+     * depending on the requested @p className. For example a koffice
      * library may usually return a pointer to KoDocument.  But
      * if asked for a "QWidget", it could create a wrapper widget,
      * that encapsulates the Koffice specific features.
@@ -383,10 +386,12 @@ protected:
      * This function is called by #create()
      * @param parent the parent of the QObject, 0 for no parent
      * @param name the name of the QObject, 0 for no name
-     * @param classname the name of the class
+     * @param className the name of the class
      * @param args a list of arguments
      */
-    virtual QObject* createObject( QObject* parent = 0, const char* name = 0, const char* classname = "QObject", const QStringList &args = QStringList() ) = 0;
+    virtual QObject* createObject( QObject* parent = 0, const char* name = 0,
+                                   const char* className = "QObject",
+                                   const QStringList &args = QStringList() ) = 0;
 
 
 protected:
