@@ -2,7 +2,7 @@
 /* This file is part of the KDE libraries
    Copyright (c) 1997,2001 Stephan Kulow <coolo@kde.org>
    Copyright (c) 1999 Preston Brown <pbrown@kde.org>
-   Copyright (c) 1999-2001 Hans Petter Bieker <bieker@kde.org>
+   Copyright (c) 1999-2002 Hans Petter Bieker <bieker@kde.org>
    Copyright (c) 2002 Lukas Tinkl <lukas@kde.org>
 
    This library is free software; you can redistribute it and/or
@@ -27,7 +27,6 @@
 
 #include <qtextcodec.h>
 #include <qfile.h>
-#undef GrayScale // make --enable-final happy
 #include <qprinter.h>
 #include <qdatetime.h>
 #include <qfileinfo.h>
@@ -40,6 +39,8 @@
 #include "kinstance.h"
 #include "kconfig.h"
 #include "kdebug.h"
+#include "kcalendarsystem.h"
+#include "kcalendarsystemfactory.h"
 #include "klocale.h"
 
 static const char * const SYSTEM_MESSAGES = "kdelibs";
@@ -63,17 +64,23 @@ public:
   KLocale::MeasureSystem measureSystem;
   QStringList langTwoAlpha;
   KConfig *languages;
+
+  QString calendarType;
+  KCalendarSystem * calendar;
+  QString first_language;
+  bool utf8FileEncoding;
 };
 
 static KLocale *this_klocale = 0;
 
-KLocale::KLocale( const QString & catalogue, KConfig * config )
+KLocale::KLocale( const QString & catalog, KConfig * config )
 {
   d = new KLocalePrivate;
   d->config = config;
   d->languages = 0;
+  d->calendar = 0;
 
-  initCatalogue(catalogue);
+  initCatalogue(catalog);
   initEncoding(0);
   initFileNameEncoding(0);
 
@@ -87,35 +94,35 @@ KLocale::KLocale( const QString & catalogue, KConfig * config )
      initLanguage(cfg, config == 0);
 }
 
-
 QString KLocale::_initLanguage(KConfigBase *config)
 {
   if (this_klocale)
   {
+     // ### HPB Why this cast??
      this_klocale->initLanguage((KConfig *) config, true);
      return this_klocale->language();
   }
   return QString::null;
 }
 
-
-void KLocale::initCatalogue(const QString & catalogue)
+void KLocale::initCatalogue(const QString & catalog)
 {
   // Use the first non-null string.
-  QString mainCatalogue = catalogue;
+  QString mainCatalogue = catalog;
   if (maincatalogue)
     mainCatalogue = QString::fromLatin1(maincatalogue);
 
   if (mainCatalogue.isEmpty()) {
     kdDebug(173) << "KLocale instance created called without valid "
-                 << "catalogue! Give an argument or call setMainCatalogue "
+                 << "catalog! Give an argument or call setMainCatalogue "
                  << "before init" << endl;
   }
   else
     d->catalogues.append( KCatalogue(mainCatalogue ) );
 
-  // always include kdelibs.mo
+  // always include kdelibs's mo files
   d->catalogues.append( KCatalogue( SYSTEM_MESSAGES ) );
+  d->catalogues.append( KCatalogue( "kio" ) );
 }
 
 void KLocale::initLanguage(KConfig * config, bool useEnv)
@@ -157,8 +164,8 @@ void KLocale::initLanguage(KConfig * config, bool useEnv)
 	    if (!chrset.isEmpty())
 	      langs.insert(it, ln + '_' + ct + '.' + chrset);
 	  }
-	  langs.insert(it, ln);
 
+          langs.insert(it, ln);
 	}
 
       languageList += langs;
@@ -287,8 +294,7 @@ void KLocale::initFormat()
 
   readConfigEntry("DecimalSymbol", ".", m_decimalSymbol);
   readConfigEntry("ThousandsSeparator", ",", m_thousandsSeparator);
-  m_thousandsSeparator.replace( QRegExp(QString::fromLatin1("\\$0")),
-				QString::null );
+  m_thousandsSeparator.replace( QString::fromLatin1("$0"), QString::null );
   //kdDebug(173) << "m_thousandsSeparator=" << m_thousandsSeparator << endl;
 
   readConfigEntry("PositiveSign", "", m_positiveSign);
@@ -299,8 +305,7 @@ void KLocale::initFormat()
   readConfigEntry("MonetaryDecimalSymbol", ".", m_monetaryDecimalSymbol);
   readConfigEntry("MonetaryThousandsSeparator", ",",
 		  m_monetaryThousandsSeparator);
-  m_monetaryThousandsSeparator.replace(QRegExp(QString::fromLatin1("\\$0")),
-				       QString::null);
+  m_monetaryThousandsSeparator.replace(QString::fromLatin1("$0"), QString::null);
 
   readConfigNumEntry("FracDigits", 2, m_fracDigits, int);
   readConfigBoolEntry("PositivePrefixCurrencySymbol", true,
@@ -327,6 +332,9 @@ void KLocale::initFormat()
   readConfigNumEntry("PageSize", (int)QPrinter::A4, d->pageSize, int);
   readConfigNumEntry("MeasureSystem", (int)Metric, d->measureSystem,
 		     MeasureSystem);
+  readConfigEntry("CalendarSystem", "gregorian", d->calendarType);
+  delete d->calendar;
+  d->calendar = 0; // ### HPB Is this the correct place?
 
   // end of hack
   KGlobal::_locale = lsave;
@@ -346,11 +354,11 @@ bool KLocale::setCountry(const QString & country)
 }
 
 QString KLocale::catalogueFileName(const QString & language,
-				   const KCatalogue & catalogue)
+				   const KCatalogue & catalog)
 {
   QString path = QString::fromLatin1("%1/LC_MESSAGES/%2.mo")
     .arg( language )
-    .arg( catalogue.name() );
+    .arg( catalog.name() );
 
   return locate( "locale", path );
 }
@@ -368,7 +376,7 @@ bool KLocale::isLanguageInstalled(const QString & language) const
       {
 	bRes = !catalogueFileName( language, *it ).isNull();
         if ( !bRes )
-	  kdDebug(173) << "message catalogue not found: "
+	  kdDebug(173) << "message catalog not found: "
 		       << (*it).name() << endl;
       }
 
@@ -377,12 +385,19 @@ bool KLocale::isLanguageInstalled(const QString & language) const
 
 bool KLocale::setLanguage(const QString & language)
 {
-  bool bRes = isLanguageInstalled( language );
+  bool bRes = true;
+
+  if (d->first_language.isNull() || language != d->first_language)
+    bRes = isLanguageInstalled( language );
 
   if ( bRes )
     {
       m_language = language;
 
+      // remember our first time - it will be our true love
+      if (d->first_language.isNull())
+        d->first_language = language;
+
       doBindInit();
     }
 
@@ -564,31 +579,31 @@ QString KLocale::weekDayName (int i, bool shortName) const
   return QString::null;
 }
 
-void KLocale::insertCatalogue( const QString & catalogue )
+void KLocale::insertCatalogue( const QString & catalog )
 {
-  KCatalogue cat( catalogue );
+  KCatalogue cat( catalog );
 
   initCatalogue( cat );
 
   d->catalogues.append( cat );
 }
 
-void KLocale::removeCatalogue(const QString &catalogue)
+void KLocale::removeCatalogue(const QString &catalog)
 {
   for ( QValueList<KCatalogue>::Iterator it = d->catalogues.begin();
 	it != d->catalogues.end(); )
-    if ((*it).name() == catalogue) {
+    if ((*it).name() == catalog) {
       it = d->catalogues.remove(it);
       return;
     } else
       ++it;
 }
 
-void KLocale::setActiveCatalogue(const QString &catalogue)
+void KLocale::setActiveCatalogue(const QString &catalog)
 {
   for ( QValueList<KCatalogue>::Iterator it = d->catalogues.begin();
 	it != d->catalogues.end(); ++it)
-    if ((*it).name() == catalogue) {
+    if ((*it).name() == catalog) {
       KCatalogue save = *it;
       d->catalogues.remove(it);
       d->catalogues.prepend(save);
@@ -596,11 +611,12 @@ void KLocale::setActiveCatalogue(const QString &catalogue)
     }
 }
 
-
 KLocale::~KLocale()
 {
+  delete d->calendar;
   delete d->languages;
   delete d;
+  d = 0L;
 }
 
 QString KLocale::translate_priv(const char *msgid,
@@ -609,7 +625,7 @@ QString KLocale::translate_priv(const char *msgid,
 {
   if (!msgid || !msgid[0])
     {
-      kdWarning() << "KLocale: trying to look up \"\" in catalogue. "
+      kdWarning() << "KLocale: trying to look up \"\" in catalog. "
 		   << "Fix the program" << endl;
       return QString::null;
     }
@@ -646,7 +662,7 @@ QString KLocale::translate( const char *index, const char *fallback) const
 {
   if (!index || !index[0] || !fallback || !fallback[0])
     {
-      kdDebug(173) << "KLocale: trying to look up \"\" in catalogue. "
+      kdDebug(173) << "KLocale: trying to look up \"\" in catalog. "
 		   << "Fix the program" << endl;
       return QString::null;
     }
@@ -663,7 +679,7 @@ QString KLocale::translate( const char *index, const char *fallback) const
   return r;
 }
 
-QString put_n_in(const QString &orig, unsigned long n)
+static QString put_n_in(const QString &orig, unsigned long n)
 {
   QString ret = orig;
   int index = ret.find("%n");
@@ -683,7 +699,7 @@ QString KLocale::translate( const char *singular, const char *plural,
 {
   if (!singular || !singular[0] || !plural || !plural[0])
     {
-      kdWarning() << "KLocale: trying to look up \"\" in catalogue. "
+      kdWarning() << "KLocale: trying to look up \"\" in catalog. "
 		   << "Fix the program" << endl;
       return QString::null;
     }
@@ -829,7 +845,7 @@ QString KLocale::translateQt( const char *context, const char *source,
 			      const char *message) const
 {
   if (!source || !source[0]) {
-    kdWarning() << "KLocale: trying to look up \"\" in catalogue. "
+    kdWarning() << "KLocale: trying to look up \"\" in catalog. "
 		<< "Fix the program" << endl;
     return QString::null;
   }
@@ -965,13 +981,13 @@ KLocale::SignPosition KLocale::negativeMonetarySignPosition() const
   return m_negativeMonetarySignPosition;
 }
 
-inline void put_it_in( QChar *buffer, uint& index, const QString &s )
+static inline void put_it_in( QChar *buffer, uint& index, const QString &s )
 {
   for ( uint l = 0; l < s.length(); l++ )
     buffer[index++] = s.at( l );
 }
 
-inline void put_it_in( QChar *buffer, uint& index, int number )
+static inline void put_it_in( QChar *buffer, uint& index, int number )
 {
   buffer[index++] = number / 10 + '0';
   buffer[index++] = number % 10 + '0';
@@ -1061,6 +1077,11 @@ QString KLocale::formatNumber(double num, int precision) const
   return res;
 }
 
+QString KLocale::formatLong(long num) const
+{
+  return formatNumber((double)num, 0);
+}
+
 QString KLocale::formatNumber(const QString &numStr) const
 {
   return formatNumber(numStr.toDouble());
@@ -1070,12 +1091,12 @@ QString KLocale::formatDate(const QDate &pDate, bool shortFormat) const
 {
   const QString rst = shortFormat?dateFormatShort():dateFormat();
 
-  // I'm rather safe than sorry
-  QChar *buffer = new QChar[rst.length() * 3 / 2 + 50];
+  QString buffer;
 
-  unsigned int index = 0;
   bool escape = false;
-  int number = 0;
+
+  int year = calendar()->year(pDate);
+  int month = calendar()->month(pDate);
 
   for ( uint format_index = 0; format_index < rst.length(); ++format_index )
     {
@@ -1084,69 +1105,64 @@ QString KLocale::formatDate(const QDate &pDate, bool shortFormat) const
 	  if ( rst.at( format_index ).unicode() == '%' )
 	    escape = true;
 	  else
-	    buffer[index++] = rst.at( format_index );
+	    buffer.append(rst.at(format_index));
 	}
       else
 	{
 	  switch ( rst.at( format_index ).unicode() )
 	    {
 	    case '%':
-	      buffer[index++] = '%';
+	      buffer.append('%');
 	      break;
 	    case 'Y':
-	      put_it_in( buffer, index, pDate.year() / 100 );
+	      buffer.append(calendar()->yearString(pDate, false));
+	      break;
 	    case 'y':
-	      put_it_in( buffer, index, pDate.year() % 100 );
+	      buffer.append(calendar()->yearString(pDate, true));
 	      break;
 	    case 'n':
-	      number = pDate.month();
+              buffer.append(calendar()->monthString(pDate, true));
+	      break;
 	    case 'e':
-	      // to share the code
-	      if ( rst.at( format_index ).unicode() == 'e' )
-		number = pDate.day();
-	      if ( number / 10 )
-		buffer[index++] = number / 10 + '0';
-	      buffer[index++] = number % 10 + '0';
+              buffer.append(calendar()->dayString(pDate, true));
 	      break;
 	    case 'm':
-	      put_it_in( buffer, index, pDate.month() );
+              buffer.append(calendar()->monthString(pDate, false));
 	      break;
 	    case 'b':
 	      if (d->nounDeclension && d->dateMonthNamePossessive)
-		put_it_in( buffer, index, monthNamePossessive(pDate.month(), true) );
+		buffer.append(calendar()->monthNamePossessive(month, year, true));
 	      else
-		put_it_in( buffer, index, monthName(pDate.month(), true) );
+		buffer.append(calendar()->monthName(month, year, true));
 	      break;
 	    case 'B':
 	      if (d->nounDeclension && d->dateMonthNamePossessive)
-		put_it_in( buffer, index, monthNamePossessive(pDate.month(), false) );
+		buffer.append(calendar()->monthNamePossessive(month, year, false));
 	      else
-		put_it_in( buffer, index, monthName(pDate.month(), false) );
+		buffer.append(calendar()->monthName(month, year, false));
 	      break;
 	    case 'd':
-	      put_it_in( buffer, index, pDate.day() );
+              buffer.append(calendar()->dayString(pDate, false));
 	      break;
 	    case 'a':
-	      put_it_in( buffer, index, weekDayName(pDate.dayOfWeek(), true) );
+	      buffer.append(calendar()->weekDayName(pDate, true));
 	      break;
 	    case 'A':
-	      put_it_in( buffer, index, weekDayName(pDate.dayOfWeek(), false) );
+	      buffer.append(calendar()->weekDayName(pDate, false));
 	      break;
 	    default:
-	      buffer[index++] = rst.at( format_index );
+	      buffer.append(rst.at(format_index));
 	      break;
 	    }
 	  escape = false;
 	}
     }
-  QString ret( buffer, index );
-  delete [] buffer;
-  return ret;
+  return buffer;
 }
 
-void KLocale::setMainCatalogue(const char *catalogue)
+void KLocale::setMainCatalogue(const char *catalog)
 {
-  maincatalogue = catalogue;
+  maincatalogue = catalog;
 }
 
 double KLocale::readNumber(const QString &_str, bool * ok) const
@@ -1278,7 +1294,6 @@ double KLocale::readMoney(const QString &_str, bool * ok) const
       minior = str.mid(pos + monetaryDecimalSymbol().length());
     }
 
-
   // Remove thousand separators
   int thlen = monetaryThousandsSeparator().length();
   int lastpos = 0;
@@ -1331,14 +1346,14 @@ static int readInt(const QString &str, uint &pos)
 QDate KLocale::readDate(const QString &intstr, bool* ok) const
 {
   QDate date;
-  date = readDate(intstr, true, ok);
+  date = readDate(intstr, ShortFormat, ok);
   if (date.isValid()) return date;
-  return readDate(intstr, false, ok);
+  return readDate(intstr, NormalFormat, ok);
 }
 
-QDate KLocale::readDate(const QString &intstr, bool shortFormat, bool* ok) const
+QDate KLocale::readDate(const QString &intstr, ReadDateFlags flags, bool* ok) const
 {
-  QString fmt = (shortFormat ? dateFormatShort() : dateFormat()).simplifyWhiteSpace();
+  QString fmt = ((flags & ShortFormat) ? dateFormatShort() : dateFormat()).simplifyWhiteSpace();
   return readDate( intstr, fmt, ok );
 }
 
@@ -1348,10 +1363,12 @@ QDate KLocale::readDate(const QString &intstr, const QString &fmt, bool* ok) con
   QString str = intstr.simplifyWhiteSpace().lower();
   int day = -1, month = -1;
   // allow the year to be omitted if not in the format
-  int year = QDate::currentDate().year();
+  int year = calendar()->year(QDate::currentDate());
   uint strpos = 0;
   uint fmtpos = 0;
 
+  int iLength; // Temperary variable used when reading input
+
   bool error = false;
 
   while (fmt.length() > fmtpos && str.length() > strpos && !error)
@@ -1368,7 +1385,7 @@ QDate KLocale::readDate(const QString &intstr, const QString &fmt, bool* ok) con
     else
     {
       int j;
-      // remove space at the begining
+      // remove space at the beginning
       if (str.length() > strpos && str.at(strpos).isSpace())
         strpos++;
 
@@ -1381,7 +1398,7 @@ QDate KLocale::readDate(const QString &intstr, const QString &fmt, bool* ok) con
           error = true;
 	  j = 1;
 	  while (error && (j < 8)) {
-	    QString s = weekDayName(j, c == 'a').lower();
+	    QString s = calendar()->weekDayName(j, c == 'a').lower();
 	    int len = s.length();
 	    if (str.mid(strpos, len) == s)
             {
@@ -1398,7 +1415,7 @@ QDate KLocale::readDate(const QString &intstr, const QString &fmt, bool* ok) con
 	  if (d->nounDeclension && d->dateMonthNamePossessive) {
 	    j = 1;
 	    while (error && (j < 13)) {
-	      QString s = monthNamePossessive(j, c == 'b').lower();
+	      QString s = calendar()->monthNamePossessive(j, year, c == 'b').lower();
 	      int len = s.length();
 	      if (str.mid(strpos, len) == s) {
 	        month = j;
@@ -1410,7 +1427,7 @@ QDate KLocale::readDate(const QString &intstr, const QString &fmt, bool* ok) con
 	  }
 	  j = 1;
 	  while (error && (j < 13)) {
-	    QString s = monthName(j, c == 'b').lower();
+	    QString s = calendar()->monthName(j, year, c == 'b').lower();
 	    int len = s.length();
 	    if (str.mid(strpos, len) == s) {
 	      month = j;
@@ -1422,27 +1439,36 @@ QDate KLocale::readDate(const QString &intstr, const QString &fmt, bool* ok) con
 	  break;
 	case 'd':
 	case 'e':
-	  day = readInt(str, strpos);
-	  error = (day < 1 || day > 31);
+	  day = calendar()->dayStringToInteger(str.mid(strpos), iLength);
+	  strpos += iLength;
+
+	  error = iLength <= 0;
 	  break;
 
 	case 'n':
 	case 'm':
-	  month = readInt(str, strpos);
-	  error = (month < 1 || month > 12);
+	  month = calendar()->monthStringToInteger(str.mid(strpos), iLength);
+	  strpos += iLength;
+
+	  error = iLength <= 0;
 	  break;
 
 	case 'Y':
 	case 'y':
-	  year = readInt(str, strpos);
-	  error = (year < 0);
+	  year = calendar()->yearStringToInteger(str.mid(strpos), iLength);
+	  strpos += iLength;
+
+	  error = iLength <= 0;
+
+	  // ### HPB: This should be moved to the implemtnation classes of KCalendarSystem
 	  // Qt treats a year in the range 0-100 as 1900-1999.
 	  // It is nicer for the user if we treat 0-68 as 2000-2068
-	  if (year < 69)
-	    year += 2000;
-	  else if (c == 'y')
-	    year += 1900;
-
+	  //if (c == 'y' && year < 69)
+	    // eg. gregorian += 2000
+	  //  year += (calendar()->year(QDate::currentDate()) / 100) * 100;
+	  //else if (c == 'y' && year < 100)
+	    // eg. gregorian += 1900
+	  //  year += (calendar()->year(QDate::currentDate()) / 100) * 100 - 100;
 	  break;
       }
     }
@@ -1459,7 +1485,11 @@ QDate KLocale::readDate(const QString &intstr, const QString &fmt, bool* ok) con
   if ( year != -1 && month != -1 && day != -1 && !error)
   {
     if (ok) *ok = true;
-    return QDate(year, month, day);
+
+    QDate result;
+    calendar()->setYMD(result, year, month, day);
+
+    return result;
   }
   else
   {
@@ -1471,19 +1501,20 @@ QDate KLocale::readDate(const QString &intstr, const QString &fmt, bool* ok) con
 QTime KLocale::readTime(const QString &intstr, bool *ok) const
 {
   QTime _time;
-  _time = readTime(intstr, true, ok);
+  _time = readTime(intstr, WithSeconds, ok);
   if (_time.isValid()) return _time;
-  return readTime(intstr, false, ok);
+  return readTime(intstr, WithoutSeconds, ok);
 }
 
-QTime KLocale::readTime(const QString &intstr, bool seconds, bool *ok) const
+QTime KLocale::readTime(const QString &intstr, ReadTimeFlags flags, bool *ok) const
 {
   QString str = intstr.simplifyWhiteSpace().lower();
   QString Format = timeFormat().simplifyWhiteSpace();
-  if (!seconds)
-    Format.replace(QRegExp(QString::fromLatin1(".%S")), QString::null);
+  if (flags & WithoutSeconds)
+    Format.remove(QRegExp(".%S"));
 
-  int hour = -1, minute = -1, second = seconds ? -1 : 0; // don't require seconds
+  int hour = -1, minute = -1;
+  int second = ( flags & WithoutSeconds == 0 ) ? -1 : 0; // don't require seconds
   bool g_12h = false;
   bool pm = false;
   uint strpos = 0;
@@ -1504,7 +1535,7 @@ QTime KLocale::readTime(const QString &intstr, bool seconds, bool *ok) const
 	  continue;
 	}
 
-      // remove space at the begining
+      // remove space at the beginning
       if (str.length() > strpos && str.at(strpos).isSpace())
 	strpos++;
 
@@ -1623,7 +1654,7 @@ QString KLocale::formatTime(const QTime &pTime, bool includeSecs) const
 		put_it_in( buffer, index, pTime.second() );
 	      else if ( index > 0 )
 		{
-		  // we remove the seperator sign before the seconds and
+		  // we remove the separator sign before the seconds and
 		  // assume that works everywhere
 		  --index;
 		  break;
@@ -1788,7 +1819,8 @@ void KLocale::initFileNameEncoding(KConfig *)
 {
   // If the following environment variable is set, assume all filenames
   // are in UTF-8 regardless of the current C locale.
-  if (getenv("KDE_UTF8_FILENAMES") != 0)
+  d->utf8FileEncoding = getenv("KDE_UTF8_FILENAMES") != 0;
+  if (d->utf8FileEncoding)
   {
     QFile::setEncodingFunction(KLocale::encodeFileNameUTF8);
     QFile::setDecodingFunction(KLocale::decodeFileNameUTF8);
@@ -1807,9 +1839,9 @@ QString KLocale::decodeFileNameUTF8( const QCString & localFileName )
   return QString::fromUtf8(localFileName);
 }
 
-void KLocale::initCatalogue( KCatalogue & catalogue )
+void KLocale::initCatalogue( KCatalogue & catalog )
 {
-  catalogue.setFileName( catalogueFileName( language(), catalogue ) );
+  catalog.setFileName( catalogueFileName( language(), catalog ) );
 }
 
 void KLocale::setDateFormat(const QString & format)
@@ -1991,6 +2023,13 @@ int KLocale::encodingMib() const
   return codecForEncoding()->mibEnum();
 }
 
+int KLocale::fileEncodingMib() const
+{
+  if (d->utf8FileEncoding)
+     return 106;
+  return codecForEncoding()->mibEnum();
+}
+
 QTextCodec * KLocale::codecForEncoding() const
 {
   return d->codecForEncoding;
@@ -2049,7 +2088,7 @@ QStringList KLocale::allLanguagesTwoAlpha() const
 {
   if (!d->languages)
     d->languages = new KConfig("all_languages", true, false, "locale");
-  
+
   return d->languages->groupList();
 }
 
@@ -2083,6 +2122,33 @@ QString KLocale::twoAlphaToCountryName(const QString &code) const
   return cfg.readEntry("Name");
 }
 
+void KLocale::setCalendar(const QString & calType)
+{
+  doFormatInit();
+
+  d->calendarType = calType;
+
+  delete d->calendar;
+  d->calendar = 0;
+}
+
+QString KLocale::calendarType() const
+{
+  doFormatInit();
+
+  return d->calendarType;
+}
+
+const KCalendarSystem * KLocale::calendar() const
+{
+  doFormatInit();
+
+  // Check if it's the correct calendar?!?
+  if ( !d->calendar )
+    d->calendar = KCalendarSystemFactory::create( d->calendarType, this );
+
+  return d->calendar;
+}
 
 KLocale::KLocale(const KLocale & rhs)
 {
@@ -2118,6 +2184,11 @@ KLocale & KLocale::operator=(const KLocale & rhs)
   // the assignment operator works here
   *d = *rhs.d;
   d->languages = 0; // Don't copy languages
+  d->calendar = 0; // Don't copy the calendar
 
   return *this;
 }
+
+bool KLocale::setCharset(const QString & ) { return true; }
+QString KLocale::charset() const { return QString::fromLatin1("UTF-8"); }
+
