@@ -1,7 +1,7 @@
 // -*- c-basic-offset: 2 -*-
 /* This file is part of the KDE libraries
     Copyright (C) 1997 Stephan Kulow <coolo@kde.org>
-    Copyright (C) 1999-2001 Hans Petter Bieker <bieker@kde.org>
+    Copyright (C) 1999-2003 Hans Petter Bieker <bieker@kde.org>
     Copyright (c) 2002 Lukas Tinkl <lukas@kde.org>
 
     This library is free software; you can redistribute it and/or
@@ -23,6 +23,7 @@
 #define _KLOCALE_H
 
 #include <qstring.h>
+#include "kdemacros.h"
 
 class QStringList;
 class QTextCodec;
@@ -35,12 +36,19 @@ class KConfig;
 class KConfigBase;
 class KLocalePrivate;
 class KCatalogue;
+class KCalendarSystem;
 
 #ifndef I18N_NOOP
-#define I18N_NOOP(x) (x)
+/**
+ * \relates KLocale
+ * I18N_NOOP marks a string to be translated without translating it.
+ * Do not use this unless you know you need it.
+ */
+#define I18N_NOOP(x) x
 #endif
 
 /**
+ * \relates KLocale
  *  i18n is the function that does everything you need to translate
  *  a string. You just wrap around every user visible string a i18n
  *  call to get a QString with the string in the user's preferred
@@ -54,14 +62,16 @@ class KCatalogue;
 QString i18n(const char *text);
 
 /**
+ * \relates KLocale
  *  If the string is too ambiguous to be translated well to a non-english
  *  language, use this form of i18n to separate lookup string and english
  *  text.
  *  @see translate
  **/
-QString i18n(const char *index, const char *text);
+QString i18n(const char *comment, const char *text);
 
 /**
+ * \relates KLocale
  *  If you want to handle plural forms, use this form of i18n.
  *  The plural has to contain a %n where n fits into.
  *  @see translate
@@ -69,6 +79,7 @@ QString i18n(const char *index, const char *text);
 QString i18n(const char *singular, const char *plural, unsigned long n);
 
 /**
+ * \relates KLocale
  * Qt3's uic generates i18n( "msg", "comment" ) calls which conflict
  * with our i18n method. We use uic -tr tr2i18n to redirect
  * to the right i18n() function
@@ -94,7 +105,7 @@ class KLocale
   friend class KGlobal; // for initInstance()
 public:
   /**
-   * Constructs a KLocale with the given catalogue name.
+   * Constructs a KLocale with the given catalog name.
    * The constructor looks for an entry Locale/Language in the
    * configuration file.
    * If no config file is specified, it will also look for languages
@@ -106,10 +117,10 @@ public:
    * If you specify a configuration file, it has to be valid until
    * the KLocale object is destroyed.
    *
-   * @param catalogue The name of the main language file
+   * @param catalog The name of the main language file
    * @param config The configuration file to use.
    */
-  KLocale( const QString& catalogue, KConfig *config = 0 );
+  KLocale( const QString& catalog, KConfig *config = 0 );
 
   /**
    * Copy constructor.
@@ -133,14 +144,14 @@ public:
    * There is a KDE wide message file that contains the most
    * often used phrases, so we can avoid duplicating the
    * translation of these phrases. If a phrase is not found
-   * in the catalogue given to the constructor, it will search
+   * in the catalog given to the constructor, it will search
    * in the system catalog. This makes it possible to override
    * some phrases for your needs.
    *
-   *  The argument must be an UTF-8 encoded string (If you only use
-   *  characters that are in US-ASCII you're on the safe side. But
-   *  for e.g. german umlauts or french accents should be recoded to
-   *  UTF-8)
+   * The argument must be an UTF-8 encoded string (If you only use
+   * characters that are in US-ASCII you're on the safe side. But
+   * for e.g. german umlauts or french accents should be recoded to
+   * UTF-8)
    *
    * @param index The lookup text and default text, if not found.
    */
@@ -167,11 +178,11 @@ public:
    * The english user will see "Open" in both cases, but the translated
    * version may vary. Of course you can also use i18n()
    *
-   * @param index The lookup text
+   * @param comment the comment. The lookup text is made out of comment + @p fallback
    * @param fallback the default text, if not found
    * @return translation
    */
-  QString translate( const char *index, const char *fallback) const;
+  QString translate( const char *comment, const char *fallback) const;
 
   /**
    * Used to get the correct, translated singular or plural of a
@@ -186,17 +197,6 @@ public:
   QString translate( const char *singular, const char *plural,
 		     unsigned long n) const;
 
-  /**
-   * @deprecated Changes the current charset used for fonts.
-   *
-   * @param charset The new charset
-   *
-   * @return True on success.
-   */
-#ifndef KDE_NO_COMPAT
-  bool setCharset(const QString & charset);
-#endif
-
   /**
    * Changes the current encoding.
    *
@@ -366,23 +366,11 @@ public:
 		      const QString & currency = QString::null,
 		      int digits = -1) const;
 
-  /**
-   * @deprecated This function differs from the above only in that it can take
-   * a QString as the argument for convenience.
-   *
-   * @param numStr The string you want to reformat.
-   *
-   * @return The number of money as a localized string
-   */
-#ifndef KDE_NO_COMPAT
-  QString formatMoney(const QString &numStr) const;
-#endif
-
   /**
    * Given a double, converts that to a numeric string containing
    * the localized numeric equivalent.
    *
-   * e.g. given 123456.78, return "123,456.78" (for some European country).
+   * e.g. given 123456.78F, return "123,456.78" (for some European country).
    * If precision isn't specified, 2 is used.
    *
    * @param num The number to convert
@@ -393,16 +381,17 @@ public:
   QString formatNumber(double num, int precision = -1) const;
 
   /**
-   * @deprecated This function differs from the above only in that it can take
-   * a QString as the argument for convenience.
+   * Given an integer, converts that to a numeric string containing
+   * the localized numeric equivalent.
    *
-   * @param numStr The string you want to convert.
+   * e.g. given 123456L, return "123,456" (for some European country).
    *
-   * @return The number as a formated string
+   * @param num The number to convert
+   *
+   * @return The number as a localized string
+   * @since 3.2
    */
-#ifndef KDE_NO_COMPAT
-  QString formatNumber(const QString &numStr) const;
-#endif
+  QString formatLong(long num) const;
 
   /**
    * Use this to determine whether nouns are declined in
@@ -456,13 +445,13 @@ public:
   /**
    * @deprecated
    *
-   * Please use the @ref weekStartDay method instead.
+   * Please use the weekStartDay method instead.
    *
    * Use this to determine if the user wants the week to start on Monday.
    *
    * @return true if the week starts on Monday
    */
-  bool weekStartsMonday() const; //### remove for KDE 4.0
+  bool weekStartsMonday() const KDE_DEPRECATED; //### remove for KDE 4.0
 
   /**
    * Use this to determine which day is the first day of the week.
@@ -473,17 +462,21 @@ public:
   int weekStartDay() const;
 
   /**
-   * Returns a string containing the name of the month name.
+   * @deprecated
+   *
+   * Returns a string containing the name of the month name used in the Gregorian calendar.
    *
    * @param i the month number of the year starting at 1/January.
    * @param shortName we will return the short version of the string.
    *
    * @return The name of the month
    */
-  QString monthName(int i, bool shortName = false) const;
+  QString monthName(int i, bool shortName = false) const KDE_DEPRECATED;
 
   /**
-   * Returns a string containing the possessive form of the month name.
+   * @deprecated
+   *
+   * Returns a string containing the possessive form of the month name used in the Gregorian calendar.
    * ("of January", "of February", etc.)
    * It's needed in long format dates in some languages.
    *
@@ -493,17 +486,46 @@ public:
    * @return The possessive form of the name of the month
    * @since 3.1
   */
-  QString monthNamePossessive(int i, bool shortName = false) const;
+  QString monthNamePossessive(int i, bool shortName = false) const KDE_DEPRECATED;
 
   /**
-   * Returns a string containing the name of the week day.
+   * @deprecated use calendar()->weekDayName
+   *
+   * Returns a string containing the name of the week day used in the Gregorian calendar.
    *
    * @param i the day number of the week starting at 1/Monday.
    * @param shortName we will return the short version of the string.
    *
    * @return The name of the day
    */
-  QString weekDayName(int i, bool shortName = false) const;
+  QString weekDayName(int i, bool shortName = false) const KDE_DEPRECATED;
+
+  /**
+   * Returns a pointer to the calendar system object.
+   *
+   * @return the current calendar system instance
+   * @since 3.2
+   */
+  const KCalendarSystem * calendar() const;
+
+  /**
+   * Returns the name of the calendar system that is currently being
+   * used by the system.
+   *
+   * @return the name of the calendar system
+   * @since 3.2
+   */
+  QString calendarType() const;
+
+  /**
+   * Changes the current calendar system to the calendar specified.
+   * Currently is "gregorian" and "hijri" supported. If the calendar
+   * system specified is not found, gregorian will be used.
+   *
+   * @param calendarType the name of the calendar type
+   * @since 3.2
+   */
+  void setCalendar(const QString & calendarType);
 
   /**
    * Returns a string formated to the current locale's conventions
@@ -511,7 +533,7 @@ public:
    *
    * @param pDateTime The date and time to be formated.
    * @param shortFormat using the short date format.
-   * @param includeSeconds using the short date format.
+   * @param includeSecs using the short date format.
    *
    * @return The date and time as a string
    */
@@ -559,8 +581,29 @@ public:
    */
   QDate readDate( const QString &intstr, const QString &fmt, bool* ok = 0) const;
 
+  enum ReadDateFlags {
+      NormalFormat = 1,
+      ShortFormat = 2
+  };
+
+  /**
+   * Converts a localized date string to a QDate.
+   * This method is stricter than readDate(str,&ok): it will either accept
+   * a date in full format or a date in short format, depending on @p flags.
+   *
+   * @param str the string we want to convert.
+   * @param flags whether the date string is to be in full format or in short format.
+   * @param ok the boolean that is set to false if it's not a valid date.
+   *           If @p ok is 0, it will be ignored
+   *
+   * @return The string converted to a QDate
+   * @since 3.2
+   */
+  QDate readDate(const QString &str, ReadDateFlags flags, bool *ok = 0) const;
+
   /**
    * Converts a localized time string to a QTime.
+   * This method will try to parse it with seconds, then without seconds.
    * The bool pointed by ok will be false if the time entered was not valid.
    *
    * @param str the string we want to convert.
@@ -571,10 +614,30 @@ public:
    */
   QTime readTime(const QString &str, bool* ok = 0) const;
 
+  enum ReadTimeFlags {
+      WithSeconds = 0, // default (no flag set)
+      WithoutSeconds = 1
+  }; // (maybe use this enum as a bitfield, if adding independent features?)
+  /**
+   * Converts a localized time string to a QTime.
+   * This method is stricter than readTime(str,&ok): it will either accept
+   * a time with seconds or a time without seconds.
+   * Use this method when the format is known by the application.
+   *
+   * @param str the string we want to convert.
+   * @param flags whether the time string is expected to contain seconds or not.
+   * @param ok the boolean that is set to false if it's not a valid time.
+   *           If @p ok is 0, it will be ignored
+   *
+   * @return The string converted to a QTime
+   * @since 3.2
+   */
+  QTime readTime(const QString &str, ReadTimeFlags flags, bool *ok = 0) const;
+
   /**
    * Returns the language used by this object. The domain AND the
    * library translation must be available in this language.
-   * "en_US" is default, if no other available.
+   * defaultLanguage() is returned by default, if no other available.
    *
    * @return The currently used language.
    */
@@ -582,21 +645,12 @@ public:
 
   /**
    * Returns the country code of the country where the user lives.
-   * "C" is default, if no other available
+   * defaultCountry() is returned by default, if no other available.
    *
    * @return The country code for the user.
    */
   QString country() const;
 
-  /**
-   * @deprecated Returns the languages selected by user.
-   *
-   * @return String containing language codes separated by colons
-   */
-#ifndef KDE_NO_COMPAT
-  QString languages() const;
-#endif
-
   /**
    * Returns the preferred languages as ISO 639-1 codes. This means
    * that information about country is removed. If the internal language
@@ -622,20 +676,6 @@ public:
    */
   QStringList languageList() const;
 
-  /**
-   * @deprecated Returns the charset name by selected locale.
-   * This will be the charset defined in the config file.
-   * NOTE: This is no longer the same as encoding.
-   * "iso-8859-1" is default
-   *
-   * @return Name of the preferred charset for fonts
-   *
-   * @see encoding
-   */
-#ifndef KDE_NO_COMPAT
-  QString charset() const;
-#endif
-
   /**
    * Returns the user's preferred encoding.
    *
@@ -665,6 +705,16 @@ public:
    */
   QTextCodec * codecForEncoding() const;
 
+  /**
+   * Returns the file encoding.
+   *
+   * @return The Mib of the file encoding
+   *
+   * @see QFile::encodeName
+   * @see QFile::decodeName
+   */
+  int fileEncodingMib() const;
+
   /**
    * Changes the current date format.
    *
@@ -737,13 +787,14 @@ public:
   /**
    * @deprecated
    *
-   * Please use @ref setWeekStartDay instead.
+   * Please use setWeekStartDay instead.
    *
    * Changes how KLocale defines the first day in week.
    *
    * @param start True if Monday is the first day in the week
    */
-  void setWeekStartsMonday(bool start); //### remove for KDE 4.0
+  void setWeekStartsMonday(bool start) KDE_DEPRECATED; //### remove for KDE 4.0
+
   /**
    * Changes how KLocale defines the first day in week.
    *
@@ -862,7 +913,7 @@ public:
   /**
    * Changes the preferred page size when printing.
    *
-   * @param The new preferred page size in the format QPrinter::PageSize
+   * @param paperFormat the new preferred page size in the format QPrinter::PageSize
    */
   void setPageSize(int paperFormat);
 
@@ -887,29 +938,29 @@ public:
   void setMeasureSystem(MeasureSystem value);
 
   /**
-   * Adds another catalogue to search for translation lookup.
+   * Adds another catalog to search for translation lookup.
    * This function is useful for extern libraries and/or code,
-   * that provides its own messages.
+   * that provide there own messages.
    *
-   * If the catalogue does not exist for the chosen language,
+   * If the catalog does not exist for the chosen language,
    * it will be ignored and en_US will be used.
    *
-   * @param catalogue The catalogue to add.
+   * @param catalog The catalog to add.
    */
-  void insertCatalogue(const QString& catalogue);
+  void insertCatalogue(const QString& catalog);
 
   /**
    * Removes a catalog for translation lookup.
-   * @param catalogue The catalogue to remove.
+   * @param catalog The catalog to remove.
    * @see insertCatalogue()
    */
-  void removeCatalogue(const QString &catalogue);
+  void removeCatalogue(const QString &catalog);
 
   /**
    * Sets the active catalog for translation lookup.
-   * @param catalogue The catalogue to activate.
+   * @param catalog The catalog to activate.
    */
-  void setActiveCatalogue(const QString &catalogue);
+  void setActiveCatalogue(const QString &catalog);
 
   /**
    * Translates a message as a QTranslator is supposed to.
@@ -966,18 +1017,17 @@ public:
 			  QString & charset);
 
   /**
-   * Use this to as main catalogue for *all* KLocales, if not the appname
+   * Use this as main catalog for *all* KLocales, if not the appname
    * will be used. This function is best to be the very first instruction
    * in your program's main function as it only has an effect before the
-   * first KLocale object is created (and this is in common KDE applications
-   * quite early).
+   * first KLocale object is created.
    *
-   * @param catalogue Catalogue to override all other main catalogues.
+   * @param catalog Catalogue to override all other main catalogues.
    */
-  static void setMainCatalogue(const char *catalogue);
+  static void setMainCatalogue(const char *catalog);
 
   /**
-   * Finds localized resource in resourceDir( rtype ) + <lang> + fname.
+   * Finds localized resource in resourceDir( rtype ) + \<lang> + fname.
    *
    * @param fname relative path to find
    * @param rtype resource type to use
@@ -1004,6 +1054,41 @@ public:
    */
   static QString _initLanguage(KConfigBase *config);
 
+#ifdef KDE_NO_COMPAT
+private:
+#endif
+  /**
+   * @deprecated
+   * use formatMoney(double)
+   */
+  QString formatMoney(const QString &numStr) const KDE_DEPRECATED;
+
+  /**
+   * @deprecated
+   * use formatNumber(double)
+   */
+  QString formatNumber(const QString &numStr) const KDE_DEPRECATED;
+
+  /**
+   * @deprecated
+   * Use languageList()
+   *
+   * @return String containing language codes separated by colons
+   */
+  QString languages() const KDE_DEPRECATED;
+
+  /**
+   * @deprecated
+   * @return True
+   */
+  bool setCharset(const QString & charset) KDE_DEPRECATED;
+
+  /**
+   * @deprecated
+   * @see encoding
+   */
+  QString charset() const KDE_DEPRECATED;
+
 protected:
   /**
    * @internal Creates a KLocale object for KGlobal and inits the locale
@@ -1039,9 +1124,9 @@ private:
   /**
    * @internal Figures out which catalogues to use.
    *
-   * @param catalogue The name of the main catalogue
+   * @param catalog The name of the main catalog
    */
-  void initCatalogue(const QString & catalogue);
+  void initCatalogue(const QString & catalog);
 
   /**
    * @internal Figures out which encoding the user prefers for filenames
@@ -1060,20 +1145,10 @@ private:
   static QString decodeFileNameUTF8( const QCString & localFileName );
 
   /**
-   * @internal function used by readTime(const QString &) const.
-   */
-  QTime readTime(const QString &str, bool seconds, bool *ok) const;
-
-  /**
-   * @internal function used by readDate(const QString &) const.
-   */
-  QDate readDate(const QString &str, bool shortFormat, bool *ok) const;
-
-  /**
-   * @internal Changes the file name of the catalogue to the correct
+   * @internal Changes the file name of the catalog to the correct
    * one.
    */
-  void initCatalogue( KCatalogue & catalogue );
+  void initCatalogue( KCatalogue & catalog );
 
   /**
    * @internal Reads the language and format configuration form disk.
@@ -1108,11 +1183,11 @@ private:
   bool isLanguageInstalled(const QString & language) const;
 
   /**
-   * @internal Retrieves the file name of the catalogue, or QString::null
+   * @internal Retrieves the file name of the catalog, or QString::null
    *           if not found.
    */
   static QString catalogueFileName(const QString & language,
-				   const KCatalogue & catalogue);
+				   const KCatalogue & catalog);
 
 private:
   // Numbers and money
