@@ -21,6 +21,7 @@
 #include <qstring.h>
 #include <kprocess.h>
 #include <qstrlist.h>
+#include "kdemacros.h"
 
 class KProcIOPrivate;
 class QTextCodec;
@@ -39,9 +40,9 @@ class QTextCodec;
  * Basically, KProcIO gives you buffered I/O similar to fgets()/fputs().
  *
  * Aside from these, and the fact that start() takes different
- * parameters, use this class just like @ref KProcess.
+ * parameters, use this class just like KProcess.
  *
- * @version $Id: kprocio.h,v 1.15 2002/09/10 19:17:01 tjansen Exp $
+ * @version $Id: kprocio.h,v 1.21 2003/10/31 22:05:58 ossi Exp $
  * @author David Sweet
  * @short A slightly simpler interface to KProcess
  **/
@@ -50,11 +51,27 @@ class QTextCodec;
 class KProcIO : public KProcess
 {
   Q_OBJECT
+
 public:
+  /**
+   * Constructor
+   */
   KProcIO ( QTextCodec *codec = 0 );
-  
+
+  /**
+   * Destructor
+   */
   ~KProcIO();
-  
+
+  /**
+   * Sets the communication mode to be passed to KProcess::start()
+   * by start(). The default communication mode is KProcess::All.
+   * You probably want to use this function in conjunction with
+   * KProcess::setUsePty().
+   * @param comm the communication mode
+   */
+  void setComm (Communication comm);
+
   /**
    *  Starts the process. It will fail in the following cases:
    *  @li The process is already running.
@@ -64,25 +81,25 @@ public:
    *
    *  @param runmode For a detailed description of the
    *  various run modes, have a look at the
-   *  general description of the @ref KProcess class.
-   *  @param includeStderr If true, data from both stdout and stderr is 
+   *  general description of the KProcess class.
+   *  @param includeStderr If true, data from both stdout and stderr is
    *  listened to. If false, only stdout is listened to.
-   *  @return true on success, false on error. 
+   *  @return true on success, false on error.
    **/
   bool start (RunMode  runmode = NotifyOnExit, bool includeStderr = false);
 
   /**
    * Writes text to stdin of the process.
    * @param line Text to write.
-   * @param appendnewline if true, a newline '\n' is appended.
+   * @param appendnewline if true, a newline '\\n' is appended.
    * @return true if successful, false otherwise
    **/
-  bool writeStdin(const QString &line, bool appendnewline=TRUE);
+  bool writeStdin(const QString &line, bool appendnewline=true);
 
   /**
    * Writes text to stdin of the process.
    * @param line Text to write.
-   * @param appendnewline if true, a newline '\n' is appended.
+   * @param appendnewline if true, a newline '\\n' is appended.
    * @return true if successful, false otherwise
    **/
   bool writeStdin(const QCString &line, bool appendnewline);
@@ -97,23 +114,23 @@ public:
   //I like fputs better -- it's the same as writeStdin
   //inline
   /**
-   * This function just calls @ref writeStdin().
-   * 
+   * This function just calls writeStdin().
+   *
    * @param line Text to write.
-   * @param AppendNewLine if true, a newline '\n' is appended.
+   * @param AppendNewLine if true, a newline '\\n' is appended.
    * @return true if successful, false otherwise
    * @deprecated
    **/
-  bool fputs (const QString &line, bool AppendNewLine=TRUE)
+  KDE_DEPRECATED bool fputs (const QString &line, bool AppendNewLine=true)
     { return writeStdin(line, AppendNewLine); }
 
   /**
-   * closes stdin after all data has been send.
-   */ 
+   * Closes stdin after all data has been send.
+   */
   void closeWhenDone();
 
   /**
-   * Reads a line of text (up to and including '\n').
+   * Reads a line of text (up to and including '\\n').
    *
    * Use readln() in response to a readReady() signal.
    * You may use it multiple times if more than one line of data is
@@ -124,28 +141,26 @@ public:
    *
    * readln() never blocks.
    *
-   * autoAck==TRUE makes these functions call ackRead() for you.
+   * autoAck==true makes these functions call ackRead() for you.
    *
    * @param line is used to store the line that was read.
    * @param autoAck when true, ackRead() is called for you.
-   * @param partial when provided the line is returned 
-   * even if it does not contain a '\n'. *partial will be set to
-   * false if the line contains a '\n' and false otherwise.
+   * @param partial when provided the line is returned
+   * even if it does not contain a '\\n'. *partial will be set to
+   * false if the line contains a '\\n' and false otherwise.
    * @return the number of characters read, or -1 if no data is available.
    **/
   int readln (QString &line, bool autoAck=true, bool *partial=0);
 
   /**
-   * This function calls @ref readln().
+   * This function calls readln().
    * @param line is used to store the line that was read.
    * @param autoAck when true, ackRead() is called for you.
-   * @param partial when provided the line is returned 
-   * even if it does not contain a '\n'. *partial will be set to
-   * false if the line contains a '\n' and false otherwise.
    * @return the number of characters read, or -1 if no data is available.
-   * @deprecated
+   * @deprecated use readln. Note that it has an inverted autoAck default,
+   *  though.
    **/
-  int fgets (QString &line, bool autoAck=false)
+  KDE_DEPRECATED int fgets (QString &line, bool autoAck=false)
     { return readln (line, autoAck); }
 
   /**
@@ -165,7 +180,7 @@ public:
   void ackRead ();
 
   /**
-   *  Turns @ref readReady() signals on and off.
+   *  Turns readReady() signals on and off.
    *   You can turn this off at will and not worry about losing any data.
    *   (as long as you turn it back on at some point...)
    * @param enable true to turn the signals on, false to turn them off
