@@ -4,7 +4,7 @@
  *  Copyright (c) 2000-2002 Waldo Bastian <bastian@kde.org>
  *                2002 Rik Hemsley <rik@kde.org>
  *
- * $Id: krfcdate.cpp,v 1.16 2002/10/07 01:07:45 maragato Exp $
+ * $Id: krfcdate.cpp,v 1.18 2003/12/29 19:11:30 adridg Exp $
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Library General Public
@@ -78,7 +78,7 @@ static const char haystack[37]="janfebmaraprmayjunjulaugsepoctnovdec";
 // we follow the recommendation of rfc2822 to consider all
 // obsolete time zones not listed here equivalent to "-0000"
 static const struct {
-    const char *tzName;
+    const char tzName[4];
     int tzOffset;
 } known_zones[] = {
     { "UT", 0 },
@@ -91,7 +91,7 @@ static const struct {
     { "MDT", -360 },
     { "PST", -480 },
     { "PDT", -420 },
-    { 0, 0 }
+    { { 0,0,0,0 }, 0 }
 };
 
 time_t
