@@ -18,7 +18,10 @@
 */
 
 #include <qwidget.h>
-#ifndef Q_WS_QWS
+
+#include "config.h"
+#if defined Q_WS_X11 && ! defined K_WS_QTONLY // not needed anyway :-)
+
 #include "krootprop.h"
 #include "kglobal.h"
 #include "klocale.h"
@@ -26,13 +29,13 @@
 #include "kapplication.h"
 #include <qtextstream.h>
 
-#include <X11/Xlib.h>
-#include <X11/Xatom.h>
+#include <X11/Xlib.h> // schroder
+#include <X11/Xatom.h> // schroder
 
 KRootProp::KRootProp(const QString& rProp )
 {
   atom = 0;
-  dirty = FALSE;
+  dirty = false;
   setProp( rProp );
 }
 
@@ -145,7 +148,7 @@ QString KRootProp::prop() const
 
 void KRootProp::destroy()
 {
-    dirty = FALSE;
+    dirty = false;
     propDict.clear();
     if( atom ) {
 	XDeleteProperty( qt_xdisplay(), qt_xrootwin(), atom );
@@ -211,7 +214,7 @@ QColor KRootProp::readColorEntry( const QString& rKey,
   if( aValue.isNull() )
     return aRetColor;
 
-  // Support #ffffff style colour naming.
+  // Support #ffffff style color naming.
   // Help ease transistion from legacy KDE setups
   if( aValue.find("#") == 0 ) {
     aRetColor.setNamedColor( aValue );
@@ -241,7 +244,7 @@ QColor KRootProp::readColorEntry( const QString& rKey,
 
 QString KRootProp::writeEntry( const QString& rKey, const QString& rValue )
 {
-    dirty = TRUE;
+    dirty = true;
     if ( propDict.contains( rKey ) ) {
 	QString aValue = propDict[ rKey ];
 	propDict.replace( rKey, rValue );
@@ -277,7 +280,7 @@ QString KRootProp::writeEntry( const QString& rKey, const QColor& rColor )
 QString KRootProp::removeEntry(const QString& rKey)
 {
     if (propDict.contains(rKey)) {
-	dirty = TRUE;
+	dirty = true;
 	QString aValue = propDict[rKey];
 	propDict.remove(rKey);
 	return aValue;
