@@ -26,7 +26,7 @@
 class QKeyEvent;
 class KKeyNative;
 
-/*
+/**
 * A KKey object represents a single key with possible modifiers
 * (Shift, Ctrl, Alt, Win).  It can represent both keys which are
 * understood by Qt as well as those which are additionally supported
@@ -44,7 +44,7 @@ class KKey
 	 * @see ModFlag
 	 */
 	enum { MOD_FLAG_COUNT = 4 };
-	enum { QtWIN = (Qt::ALT << 1) };
+	enum { QtWIN = (Qt::META) };
 	/**
 	 * Flags to represent the modifiers. You can combine modifiers
 	 * by ORing them.
@@ -73,7 +73,7 @@ class KKey
 
 	/**
 	 * Creates a new key from the first key code of the given key sequence.
-	 * @param keySeq the key sequence that contains the key 
+	 * @param keySeq the key sequence that contains the key
 	 */
 	KKey( const QKeySequence& keySeq );
 
@@ -90,9 +90,9 @@ class KKey
 
 	/**
 	 * Creates a new key from the given description. The form of the description
-	 * is "[modifier+[modifier+]]+key", for example "e", "CTRL+q" or 
+	 * is "[modifier+[modifier+]]+key", for example "e", "CTRL+q" or
 	 * "CTRL+ALT+DEL". Allowed modifiers are "SHIFT", "CTRL", "ALT", "WIN" and
-	 * "META". "WIN" and "META" are equivalent. Modifiers are not case-sensitive. 
+	 * "META". "WIN" and "META" are equivalent. Modifiers are not case-sensitive.
 	 * @param key the description of the key
 	 * @see KKeyServer::Sym::init()
 	 */
@@ -120,7 +120,7 @@ class KKey
 
 	/**
 	 * Initializes the key with the first key code of the given key sequence.
-	 * @param keySeq the key sequence that contains the key 
+	 * @param keySeq the key sequence that contains the key
 	 * @return true if successful, false otherwise
 	 */
 	bool init( const QKeySequence& keySeq );
@@ -141,14 +141,14 @@ class KKey
 
 	/**
 	 * Initializes the key with the given description. The form of the description
-	 * is "[modifier+[modifier+]]+key", for example "e", "CTRL+q" or 
+	 * is "[modifier+[modifier+]]+key", for example "e", "CTRL+q" or
 	 * "CTRL+ALT+DEL". Allowed modifiers are "SHIFT", "CTRL", "ALT", "WIN" and
-	 * "META". "WIN" and "META" are equivalent. Modifiers are not case-sensitive. 
+	 * "META". "WIN" and "META" are equivalent. Modifiers are not case-sensitive.
 	 * @param key the description of the key
 	 * @return true if successful, false otherwise
 	 * @see KKeyServer::Sym::init()
 	 */
-	bool init( const QString& );
+	bool init( const QString& key);
 
 	/**
 	 * @internal
@@ -163,7 +163,7 @@ class KKey
 
  // Query methods.
 	/**
-	 * Returns true if the key is null (after @ref clear() or empty
+	 * Returns true if the key is null (after clear() or empty
 	 * constructor).
 	 * @return true if the key is null
 	 * @see clear()
@@ -171,16 +171,6 @@ class KKey
 	 */
 	bool isNull() const;
 
-	/**
-	 * @internal
-	 */
-	bool isValidQt() const;
-
-	/**
-	 * @internal
-	 */
-	bool isValidNative() const;
-
 	/**
 	 * @internal
 	 */
@@ -192,13 +182,13 @@ class KKey
 
  // Comparison Methods
 	/**
-	 * Compares this key with the given KKey object. Returns a negative 
+	 * Compares this key with the given KKey object. Returns a negative
 	 * number if the given KKey is larger, 0 if they are equal and
 	 * a positive number this KKey is larger. The returned value
-	 * is the difference between the symbol or, if the symbols 
+	 * is the difference between the symbol or, if the symbols
 	 * are equal, the difference between the encoded modifiers.
 	 * @param key the key to compare with this key
-	 * @return a negative number if the given KKey is larger, 0 if 
+	 * @return a negative number if the given KKey is larger, 0 if
 	 * they are equal and a positive number this KKey is larger
 	 */
 	int compare( const KKey& key ) const;
@@ -229,7 +219,7 @@ class KKey
 	 * @see Qt::Key
 	 */
 	int keyCodeQt() const;
-	
+
 	/**
 	 * Returns a human-readable representation of the key in the form
 	 * "modifier+key".
@@ -247,7 +237,7 @@ class KKey
 	 * @internal
 	 */
 	void simplify();
-	
+
 	/**
 	 * Returns a null key.
 	 * @return the null key
@@ -294,8 +284,8 @@ class KKey
 class KKeySequence
 {
  public:
-        /// Defines the maximum length of the key sequence 
-        enum { MAX_KEYS = 4 }; 
+        /// Defines the maximum length of the key sequence
+        enum { MAX_KEYS = 4 };
 
 	/**
 	 * Create a new null key sequence.
@@ -330,9 +320,9 @@ class KKeySequence
 	KKeySequence( const KKeySequence& keySeq );
 
 	/**
-	 * Creates a new key sequence that contains the given key sequence. 
+	 * Creates a new key sequence that contains the given key sequence.
 	 * The description consists of comma-separated keys as
-	 * required by @ref KKey::KKey(const QString&).
+	 * required by KKey::KKey(const QString&).
 	 * @param keySeq the description of the key
 	 * @see KKeyServer::Sym::init()
 	 * @see KKey::KKey(const QString&)
@@ -342,7 +332,7 @@ class KKeySequence
 	~KKeySequence();
 
 	/**
-	 * Clears the key sequence. The key sequence is null after calling this 
+	 * Clears the key sequence. The key sequence is null after calling this
 	 * function.
 	 * @see isNull()
 	 */
@@ -379,7 +369,7 @@ class KKeySequence
 	/**
 	 * Initializes this key sequence to contain the given key sequence.
 	 * The description consists of comma-separated keys as
-	 * required by @ref KKey::KKey(const QString&).
+	 * required by KKey::KKey(const QString&).
 	 * @param key the description of the key
 	 * @return true if successful, false otherwise
 	 * @see KKeyServer::Sym::init()
@@ -404,7 +394,7 @@ class KKeySequence
 	 * Return the @p i'th key of this sequence, or a null key if there
 	 * are less then i keys.
 	 * @param i the key to retrieve
-	 * @return the @p i'th key, or @ref KKey::null() if there are less
+	 * @return the @p i'th key, or KKey::null() if there are less
 	 *         than i keys
 	 * @see MAX_KEYS
 	 */
@@ -417,21 +407,16 @@ class KKeySequence
 
 	/**
 	 * Sets the @p i'th key of the sequence. You can not introduce gaps
-	 * in a sequence, so you must use an @p i <= @ref count(). Also note that
-	 * the maximum length of a key sequence is @ref MAX_KEYS.
-	 * @param i the position of the new key (<= @ref count(), <= @ref MAX_KEYS)
+	 * in a sequence, so you must use an @p i <= count(). Also note that
+	 * the maximum length of a key sequence is MAX_KEYS.
+	 * @param i the position of the new key (<= count(), <= MAX_KEYS)
 	 * @param key the key to set
 	 * @return true if successful, false otherwise
 	 */
 	bool setKey( uint i, const KKey& key );
 
 	/**
-	 * @internal
-	 */
-	void setTriggerOnRelease( bool );
-
-	/**
-	 * Returns true if the key sequence is null (after @ref clear() or empty
+	 * Returns true if the key sequence is null (after clear() or empty
 	 * constructor).
 	 * @return true if the key sequence is null
 	 * @see clear()
@@ -447,14 +432,14 @@ class KKeySequence
 	bool startsWith( const KKeySequence& keySeq ) const;
 
 	/**
-	 * Compares this object with the given key sequence. Returns a negative 
+	 * Compares this object with the given key sequence. Returns a negative
 	 * number if the given KKeySequence is larger, 0 if they are equal and
 	 * a positive number this KKeySequence is larger. Key sequences are
 	 * compared by comparing the individual keys, starting from the beginning
 	 * until an unequal key has been found. If a sequence contains more
 	 * keys, it is considered larger.
 	 * @param keySeq the key sequence to compare to
-	 * @return a negative number if the given KKeySequence is larger, 0 if 
+	 * @return a negative number if the given KKeySequence is larger, 0 if
 	 * they are equal and a positive number this KKeySequence is larger
 	 * @see KKey::sequence
 	 */
@@ -487,7 +472,7 @@ class KKeySequence
 	 * @return the QKeySequence
 	 */
 	QKeySequence qt() const;
-	
+
 	/**
 	 * Returns the qt key code of the first key.
 	 * @return the qt key code of the first key
@@ -498,7 +483,7 @@ class KKeySequence
 
 	/**
 	 * Returns the key sequence as a number of key presses as
-	 * returned by @ref KKey::toString(), seperated by commas.
+	 * returned by KKey::toString(), separated by commas.
 	 * @return the string represenation of this key sequence
 	 * @see KKey::toString()
 	 */
@@ -528,7 +513,7 @@ class KKeySequence
 	friend class KKeyNative;
 };
 
-/*
+/**
 * The KShortcut class is used to represent a keyboard shortcut to an action.
 * A shortcut is normally a single key with modifiers, such as Ctrl+V.
 * A KShortcut object may also contain an alternate key which will also
@@ -540,9 +525,9 @@ class KShortcut
 {
  public:
         /**
-	 * The maximum number of key sequences that can be contained in 
+	 * The maximum number of key sequences that can be contained in
 	 * a KShortcut.
-         */ 
+         */
 	enum { MAX_SEQUENCES = 2 };
 
 	/**
@@ -591,7 +576,7 @@ class KShortcut
 	/**
 	 * Creates a new key sequence that contains the given key sequence.
 	 * The description consists of semicolon-separated keys as
-	 * used in @ref KKeySequence::KKeySequence(const QString&).
+	 * used in KKeySequence::KKeySequence(const QString&).
 	 * @param shortcut the description of the key
 	 * @see KKeySequence::KKeySequence(const QString&)
 	 */
@@ -600,7 +585,7 @@ class KShortcut
 	/**
 	 * Creates a new key sequence that contains the given key sequence.
 	 * The description consists of semicolon-separated keys as
-	 * used in @ref KKeySequence::KKeySequence(const QString&).
+	 * used in KKeySequence::KKeySequence(const QString&).
 	 * @param shortcut the description of the key
 	 * @see KKeySequence::KKeySequence(const QString&)
 	 */
@@ -608,7 +593,7 @@ class KShortcut
 	~KShortcut();
 
 	/**
-	 * Clears the shortcut. The shortcut is null after calling this 
+	 * Clears the shortcut. The shortcut is null after calling this
 	 * function.
 	 * @see isNull()
 	 */
@@ -649,7 +634,7 @@ class KShortcut
 	/**
 	 * Initializes the key sequence with the given key sequence.
 	 * The description consists of semicolon-separated keys as
-	 * used in @ref KKeySequence::KKeySequence(const QString&).
+	 * used in KKeySequence::KKeySequence(const QString&).
 	 * @param shortcut the description of the key
 	 * @see KKeySequence::KKeySequence(const QString&)
 	 */
@@ -665,21 +650,21 @@ class KShortcut
 	 * Returns the number of sequences that are in this
 	 * shortcut.
 	 * @return the number of sequences
-	 * @ref MAX_SEQUENCES
+	 * MAX_SEQUENCES
 	 */
 	uint count() const;
 
 	/**
 	 * Returns the @p i'th key sequence of this shortcut.
 	 * @param i the number of the key sequence to retrieve
-	 * @return the @p i'th sequence or @ref KKeySequence::null() if
+	 * @return the @p i'th sequence or KKeySequence::null() if
 	 *         there are less than @p i key sequences
-	 * @ref MAX_SEQUENCES
+	 * MAX_SEQUENCES
 	 */
 	const KKeySequence& seq( uint i ) const;
 
 	/**
-	 * Returns the key code of the first key sequence, or 
+	 * Returns the key code of the first key sequence, or
 	 * null if there is no first key sequence.
 	 * @return the key code of the first sequence's first key
 	 * @see Qt::Key
@@ -688,7 +673,7 @@ class KShortcut
 	int keyCodeQt() const;
 
 	/**
-	 * Returns true if the shortcut is null (after @ref clear() or empty
+	 * Returns true if the shortcut is null (after clear() or empty
 	 * constructor).
 	 * @return true if the shortcut is null
 	 * @see clear()
@@ -697,14 +682,14 @@ class KShortcut
 	bool isNull() const;
 
 	/**
-	 * Compares this object with the given shortcut. Returns a negative 
+	 * Compares this object with the given shortcut. Returns a negative
 	 * number if the given shortcut is larger, 0 if they are equal and
 	 * a positive number this shortcut is larger. Shortcuts are
-	 * compared by comparing the individual key sequences, starting from the 
+	 * compared by comparing the individual key sequences, starting from the
 	 * beginning until an unequal key sequences has been found. If a shortcut
 	 * contains more key sequences, it is considered larger.
 	 * @param shortcut the shortcut to compare to
-	 * @return a negative number if the given KShortcut is larger, 0 if 
+	 * @return a negative number if the given KShortcut is larger, 0 if
 	 * they are equal and a positive number this KShortcut is larger
 	 * @see KKey::compare()
 	 * @see KKeyShortcut::compare()
@@ -756,11 +741,11 @@ class KShortcut
 	bool contains( const KKeySequence& keySeq ) const;
 
 	/**
-	 * Sets the @p i'th key sequence of the shortcut. You can not introduce 
-	 * gaps in the list of sequences, so you must use an @i <= @ref count(). 
-	 * Also note that the maximum number of key sequences is @ref MAX_SEQUENCES.
-	 * @param i the position of the new key sequence(<= @ref count(), 
-	 *          <= @ref MAX_SEQUENCES)
+	 * Sets the @p i 'th key sequence of the shortcut. You can not introduce
+	 * gaps in the list of sequences, so you must use an @p i <= count().
+	 * Also note that the maximum number of key sequences is MAX_SEQUENCES.
+	 * @param i the position of the new key sequence(<= count(),
+	 *          <= MAX_SEQUENCES)
 	 * @param keySeq the key sequence to set
 	 * @return true if successful, false otherwise
 	 */
@@ -775,6 +760,25 @@ class KShortcut
 	*/
 	bool append( const KKeySequence& keySeq );
 
+	/**
+	 * Appends the given key
+	 * @param spec the key to add
+	 * @return true if successful, false otherwise
+	 * @see setSeq()
+	 * @see MAX_SEQUENCES
+	 * @since 3.2
+	*/
+	bool append( const KKey& spec );
+
+	/**
+	 * Appends the sequences from the given shortcut.
+	 * @param cut the shortcut to append
+	 * @return true if successful, false otherwise
+	 * @see MAX_SEQUENCES
+	 * @since 3.2
+	*/
+	bool append( const KShortcut& cut );
+
 	/**
 	 * Converts this shortcut to a key sequence. The first key sequence
 	 * will be taken.
@@ -783,7 +787,7 @@ class KShortcut
 
 	/**
 	 * Returns a description of the shortcut as semicolon-separated
-	 * ket sequences, as returned by @ref KKeySequence::toString().
+	 * ket sequences, as returned by KKeySequence::toString().
 	 * @return the string represenation of this shortcut
 	 * @see KKey::toString()
 	 * @see KKeySequence::toString()
