@@ -2,7 +2,7 @@
 #include <qvariant.h>
 
 #include <kaccel.h>
-#include <kaccelaction.h>
+#include "kaccelaction.h"
 #include <kconfig.h>
 #include <kdebug.h>
 #include <kglobal.h>
@@ -28,9 +28,14 @@ bool KShortcutList::isGlobal( uint ) const
 	return false;
 }
 
-// FIXME: implement!
-int KShortcutList::index( const QString& /*sName*/ ) const
+int KShortcutList::index( const QString& sName ) const
 {
+	uint nSize = count();
+        for( uint i = 0;
+             i < nSize;
+             ++i )
+            if( name( i ) == sName )
+                return i;
 	return -1;
 }
 
@@ -109,7 +114,7 @@ bool KShortcutList::writeSettings( const QString &sConfigGroup, KConfigBase* pCo
 
 	QString sGroup = (!sConfigGroup.isEmpty()) ? sConfigGroup : QString("Shortcuts");
 
-	// If it has the depricated group [Keys], remove it
+	// If it has the deprecated group [Keys], remove it
 	if( pConfig->hasGroup( "Keys" ) )
 		pConfig->deleteGroup( "Keys", true );
 
