@@ -17,7 +17,7 @@
    the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
    Boston, MA 02111-1307, USA.
 */
-// $Id: ksimpleconfig.cpp,v 1.47 2002/03/03 21:20:26 lunakl Exp $
+// $Id: ksimpleconfig.cpp,v 1.49 2003/09/06 19:06:30 binner Exp $
 
 #include <config.h>
 
@@ -51,11 +51,15 @@ KSimpleConfig::KSimpleConfig(const QString &fileName, bool bReadOnly)
   reparseConfiguration();
 }
 
+KSimpleConfig::KSimpleConfig(KConfigBackEnd *backEnd, bool bReadOnly)
+  : KConfig(backEnd, bReadOnly)
+{}
+
 KSimpleConfig::~KSimpleConfig()
 {
   // we need to call the KSimpleConfig version of sync.  Relying on the
   // regular KConfig sync is bad, because the KSimpleConfig sync has
-  // different behaviour.  Syncing here will insure that the sync() call
+  // different behavior.  Syncing here will insure that the sync() call
   // in the KConfig destructor doesn't actually do anything.
   sync();
 }
