@@ -2,7 +2,7 @@
  *  This file is part of the KDE libraries
  *  Copyright (C) 2000,2001 Thiago Macieira <thiagom@mail.com>
  *
- *  $Id: ksockaddr.h,v 1.15 2002/09/09 21:11:35 tjansen Exp $
+ *  $Id: ksockaddr.h,v 1.18 2003/08/20 08:01:42 coolo Exp $
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Library General Public
@@ -42,7 +42,7 @@ class KSocketAddressPrivate;
  * This class envelopes almost if not all socket addresses.
  *
  * @author Thiago Macieira <thiagom@mail.com>
- * @version $Id: ksockaddr.h,v 1.15 2002/09/09 21:11:35 tjansen Exp $
+ * @version $Id: ksockaddr.h,v 1.18 2003/08/20 08:01:42 coolo Exp $
  * @short a socket address.
  */
 class KSocketAddress: public QObject
@@ -151,7 +151,7 @@ public:
   { return isCoreEqual(*other); }
 
   /**
-   * Returns the node name of this socket, as @ref KExtendedSocket::lookup expects
+   * Returns the node name of this socket, as KExtendedSocket::lookup expects
    * as the first argument.
    * In the case of Internet sockets, this is the hostname.
    * The default implementation returns QString::null.
@@ -160,7 +160,7 @@ public:
   virtual QString nodeName() const;
 
   /**
-   * Returns the service name for this socket, as @ref KExtendedSocket::lookup expects
+   * Returns the service name for this socket, as KExtendedSocket::lookup expects
    * as the service argument.
    * In the case of Internet sockets, this is the port number.
    * The default implementation returns QString::null.
@@ -187,7 +187,7 @@ public:
    * @param size	new socket address's length
    * @return the new KSocketAddress, or 0 if the function failed
    */
-  static KSocketAddress* newAddress(const struct sockaddr*, ksocklen_t size);
+  static KSocketAddress* newAddress(const struct sockaddr *sa, ksocklen_t size);
 
   /**
    * Returns the IANA family number of the given address family.
@@ -226,11 +226,11 @@ class KInetSocketAddressPrivate;
  *
  * This is an IPv4 or IPv6 address of the Internet
  *
- * This class inherits most of the functionality from @ref KSocketAddress, but
+ * This class inherits most of the functionality from KSocketAddress, but
  * is targeted specifically to Internet addresses
  *
  * @author Thiago Macieira <thiagom@mail.com>
- * @version $Id: ksockaddr.h,v 1.15 2002/09/09 21:11:35 tjansen Exp $
+ * @version $Id: ksockaddr.h,v 1.18 2003/08/20 08:01:42 coolo Exp $
  * @short an Internet socket address
  */
 class KInetSocketAddress: public KSocketAddress
@@ -418,7 +418,7 @@ public:
 
   /**
    * Returns the text representation of the port number.
-   * @param the name of the service (a number)
+   * @return the name of the service (a number)
    */
   virtual QString serviceName() const;
 
@@ -484,7 +484,7 @@ public:
    * Compares two IPv4 addresses.
    * @param s1 the first address to compare
    * @param s2 the second address to compare
-   * @param coreOnly true if only core parts should be compared (only 
+   * @param coreOnly true if only core parts should be compared (only
    *                 the address)
    * @return true if the given addresses are equal.
    * @see areEqualInet6()
@@ -497,7 +497,7 @@ public:
    * Compares two IPv6 addresses.
    * @param s1 the first address to compare
    * @param s2 the second address to compare
-   * @param coreOnly true if only core parts should be compared (only 
+   * @param coreOnly true if only core parts should be compared (only
    *                 the address)
    * @return true if the given addresses are equal.
    * @see areEqualInet()
@@ -582,7 +582,7 @@ class KUnixSocketAddressPrivate;
  * before passing down to the function
  *
  * @author Thiago Macieira <thiagom@mail.com>
- * @version $Id: ksockaddr.h,v 1.15 2002/09/09 21:11:35 tjansen Exp $
+ * @version $Id: ksockaddr.h,v 1.18 2003/08/20 08:01:42 coolo Exp $
  * @short a Unix socket address
  */
 class KUnixSocketAddress: public KSocketAddress
@@ -603,7 +603,7 @@ public:
 
   /**
    * Constructor from pathname.
-   * @param path	pathname
+   * @param pathname	pathname
    */
   KUnixSocketAddress(QCString pathname);
 
@@ -666,8 +666,8 @@ public:
    * Compares two unix socket addresses.
    * @param s1 the first address to compare
    * @param s2 the second address to compare
-   * @param coreOnly true if only core parts should be compared (currently 
-   *                 unused) 
+   * @param coreOnly true if only core parts should be compared (currently
+   *                 unused)
    * @return true if the given addresses are equal.
    * @see KSocketAddress::isEqual()
    * @see KSocketAddress::isCoreEqual()
