@@ -37,7 +37,7 @@ struct sockaddr;
  * is able to provide full internet access behind a firewall.
  * KSocks is a singleton; there can only be one instance at any
  * given time. To obtain a reference to that instance, use
- * @ref self().
+ * self().
  *
  * @short Access to a SOCKS Proxy.
  */
@@ -154,12 +154,15 @@ public:
     * name. Read the appropriate man page.
     */
     int     listen      (int s, int backlog);
+
    /**
     * This is the re-implementation of libc's function of the same
     * name. Read the appropriate man page.
     */
     int     bind        (int sockfd, sockaddr *my_addr, 
                                      ksocklen_t addrlen);
+    int     bind        (int sockfd, const sockaddr *my_addr, 
+                                     ksocklen_t addrlen);
 
     /**
     *  If you're using this, you're probably doing something wrong.
