@@ -20,7 +20,7 @@
 
 /*
  * Author: Stephan Kulow <coolo@kde.org> and Sirtaj Singh Kang <taj@kde.org>
- * Version:	$Id: kstandarddirs.cpp,v 1.151.2.3 2003/06/16 09:31:43 waba Exp $
+ * Version:	$Id: kstandarddirs.cpp,v 1.168.2.2 2004/05/21 11:07:18 waba Exp $
  * Generated:	Thu Mar  5 16:05:28 EST 1998
  */
 
@@ -35,6 +35,7 @@
 #include <sys/types.h>
 #include <dirent.h>
 #include <pwd.h>
+#include <grp.h>
 
 #include <qregexp.h>
 #include <qasciidict.h>
@@ -48,6 +49,8 @@
 #include "kconfig.h"
 #include "kdebug.h"
 #include "kinstance.h"
+#include "kshell.h"
+#include "ksimpleconfig.h"
 #include <sys/param.h>
 #include <unistd.h>
 
@@ -64,18 +67,24 @@ public:
    bool restrictionsActive;
    bool dataRestrictionActive;
    QAsciiDict<bool> restrictions;
+   QStringList xdgdata_prefixes;
+   QStringList xdgconf_prefixes;
 };
 
 static const char* const types[] = {"html", "icon", "apps", "sound",
 			      "data", "locale", "services", "mime",
 			      "servicetypes", "config", "exe",
-			      "wallpaper", "lib", "pixmap", "templates", "module", "qtplugins", 0 };
+			      "wallpaper", "lib", "pixmap", "templates",
+			      "module", "qtplugins",
+			      "xdgdata-apps", "xdgdata-dirs", "xdgconf-menu",
+                              "kcfg", 0 };
 
 static int tokenize( QStringList& token, const QString& str,
 		const QString& delim );
 
-KStandardDirs::KStandardDirs( ) : addedCustoms(false), d(0)
+KStandardDirs::KStandardDirs( ) : addedCustoms(false)
 {
+    d = new KStandardDirsPrivate;
     dircache.setAutoDelete(true);
     relatives.setAutoDelete(true);
     absolutes.setAutoDelete(true);
@@ -86,7 +95,6 @@ KStandardDirs::KStandardDirs( ) : addedCustoms(false), d(0)
 KStandardDirs::~KStandardDirs()
 {
     delete d;
-    d = 0L;
 }
 
 bool KStandardDirs::isRestrictedResource(const char *type, const QString& relPath) const
@@ -131,7 +139,27 @@ QStringList KStandardDirs::allTypes() const
     return list;
 }
 
+static void priorityAdd(QStringList &prefixes, const QString& dir, bool priority)
+{
+    if (priority && !prefixes.isEmpty())
+    {
+        // Add in front but behind $KDEHOME
+        QStringList::iterator it = prefixes.begin();
+        it++;
+        prefixes.insert(it, 1, dir);
+    }
+    else
+    {
+        prefixes.append(dir);
+    }
+}
+
 void KStandardDirs::addPrefix( const QString& _dir )
+{
+    addPrefix(_dir, false);
+}
+
+void KStandardDirs::addPrefix( const QString& _dir, bool priority )
 {
     if (_dir.isNull())
 	return;
@@ -141,7 +169,47 @@ void KStandardDirs::addPrefix( const QString& _dir )
 	dir += '/';
 
     if (!prefixes.contains(dir)) {
-	prefixes.append(dir);
+        priorityAdd(prefixes, dir, priority);
+	dircache.clear();
+    }
+}
+
+void KStandardDirs::addXdgConfigPrefix( const QString& _dir )
+{
+    addXdgConfigPrefix(_dir, false);
+}
+
+void KStandardDirs::addXdgConfigPrefix( const QString& _dir, bool priority )
+{
+    if (_dir.isNull())
+	return;
+
+    QString dir = _dir;
+    if (dir.at(dir.length() - 1) != '/')
+	dir += '/';
+
+    if (!d->xdgconf_prefixes.contains(dir)) {
+        priorityAdd(d->xdgconf_prefixes, dir, priority);
+	dircache.clear();
+    }
+}
+
+void KStandardDirs::addXdgDataPrefix( const QString& _dir )
+{
+    addXdgDataPrefix(_dir, false);
+}
+
+void KStandardDirs::addXdgDataPrefix( const QString& _dir, bool priority )
+{
+    if (_dir.isNull())
+	return;
+
+    QString dir = _dir;
+    if (dir.at(dir.length() - 1) != '/')
+	dir += '/';
+
+    if (!d->xdgdata_prefixes.contains(dir)) {
+	priorityAdd(d->xdgdata_prefixes, dir, priority);
 	dircache.clear();
     }
 }
@@ -153,6 +221,13 @@ QString KStandardDirs::kfsstnd_prefixes()
 
 bool KStandardDirs::addResourceType( const char *type,
 				     const QString& relativename )
+{
+    return addResourceType(type, relativename, true);
+}
+
+bool KStandardDirs::addResourceType( const char *type,
+                                     const QString& relativename,
+                                     bool priority )
 {
     if (relativename.isNull())
        return false;
@@ -166,7 +241,10 @@ bool KStandardDirs::addResourceType( const char *type,
     if (copy.at(copy.length() - 1) != '/')
 	copy += '/';
     if (!rels->contains(copy)) {
-	rels->prepend(copy);
+        if (priority)
+	    rels->prepend(copy);
+	else
+	    rels->append(copy);
 	dircache.remove(type); // clean the cache
 	return true;
     }
@@ -175,6 +253,14 @@ bool KStandardDirs::addResourceType( const char *type,
 
 bool KStandardDirs::addResourceDir( const char *type,
 				    const QString& absdir)
+{
+    // KDE4: change priority to bring in line with addResourceType
+    return addResourceDir(type, absdir, false);
+}
+
+bool KStandardDirs::addResourceDir( const char *type,
+				    const QString& absdir,
+				    bool priority)
 {
     QStringList *paths = absolutes.find(type);
     if (!paths) {
@@ -186,7 +272,10 @@ bool KStandardDirs::addResourceDir( const char *type,
       copy += '/';
 
     if (!paths->contains(copy)) {
-	paths->append(copy);
+        if (priority)
+            paths->prepend(copy);
+        else
+	    paths->append(copy);
 	dircache.remove(type); // clean the cache
 	return true;
     }
@@ -257,14 +346,26 @@ Q_UINT32 KStandardDirs::calcResourceHash( const char *type,
 QStringList KStandardDirs::findDirs( const char *type,
                                      const QString& reldir ) const
 {
+    QDir testdir;
     QStringList list;
+    if (reldir.startsWith("/"))
+    {
+        testdir.setPath(reldir);
+        if (testdir.exists())
+        {
+            if (reldir.endsWith("/"))
+               list.append(reldir);
+            else
+               list.append(reldir+'/');
+        }
+        return list;
+    }
 
     checkConfig();
 
     if (d && d->restrictionsActive && (strcmp(type, "data")==0))
        applyDataRestrictions(reldir);
     QStringList candidates = resourceDirs(type);
-    QDir testdir;
 
     for (QStringList::ConstIterator it = candidates.begin();
          it != candidates.end(); it++) {
@@ -321,7 +422,7 @@ static void lookupDirectory(const QString& path, const QString &relPart,
 			    const QRegExp &regexp,
 			    QStringList& list,
 			    QStringList& relList,
-			    bool recursive, bool uniq)
+			    bool recursive, bool unique)
 {
   QString pattern = regexp.pattern();
   if (recursive || pattern.contains('?') || pattern.contains('*'))
@@ -355,14 +456,14 @@ static void lookupDirectory(const QString& path, const QString &relPart,
       }
       if ( recursive ) {
 	if ( S_ISDIR( buff.st_mode )) {
-	  lookupDirectory(pathfn + '/', relPart + fn + '/', regexp, list, relList, recursive, uniq);
+	  lookupDirectory(pathfn + '/', relPart + fn + '/', regexp, list, relList, recursive, unique);
 	}
         if (!regexp.exactMatch(fn))
 	  continue; // No match
       }
       if ( S_ISREG( buff.st_mode))
       {
-        if (!uniq || !relList.contains(relPart + fn))
+        if (!unique || !relList.contains(relPart + fn))
         {
 	    list.append( pathfn );
 	    relList.append( relPart + fn );
@@ -381,7 +482,7 @@ static void lookupDirectory(const QString& path, const QString &relPart,
         return; // File not found
      if ( S_ISREG( buff.st_mode))
      {
-       if (!uniq || !relList.contains(relPart + fn))
+       if (!unique || !relList.contains(relPart + fn))
        {
          list.append( pathfn );
          relList.append( relPart + fn );
@@ -395,11 +496,11 @@ static void lookupPrefix(const QString& prefix, const QString& relpath,
 			 const QRegExp &regexp,
 			 QStringList& list,
 			 QStringList& relList,
-			 bool recursive, bool uniq)
+			 bool recursive, bool unique)
 {
     if (relpath.isNull()) {
        lookupDirectory(prefix, relPart, regexp, list,
-		       relList, recursive, uniq);
+		       relList, recursive, unique);
        return;
     }
     QString path;
@@ -448,7 +549,7 @@ static void lookupPrefix(const QString& prefix, const QString& relpath,
 		    continue; // Couldn't stat (e.g. no permissions)
 		}
 		if ( S_ISDIR( buff.st_mode ))
-		    lookupPrefix(fn + '/', rest, rfn + '/', regexp, list, relList, recursive, uniq);
+		    lookupPrefix(fn + '/', rest, rfn + '/', regexp, list, relList, recursive, unique);
 	    }
 
 	closedir( dp );
@@ -457,7 +558,7 @@ static void lookupPrefix(const QString& prefix, const QString& relpath,
         // when we try to open it.
         lookupPrefix(prefix + path + '/', rest,
                      relPart + path + '/', regexp, list,
-                     relList, recursive, uniq);
+                     relList, recursive, unique);
     }
 }
 
@@ -465,16 +566,10 @@ QStringList
 KStandardDirs::findAllResources( const char *type,
 			         const QString& filter,
 				 bool recursive,
-			         bool uniq,
+			         bool unique,
                                  QStringList &relList) const
 {
     QStringList list;
-    if (filter.at(0) == '/') // absolute paths we return
-    {
-        list.append( filter);
-	return list;
-    }
-
     QString filterPath;
     QString filterFile;
 
@@ -491,9 +586,18 @@ KStandardDirs::findAllResources( const char *type,
 
     checkConfig();
 
-    if (d && d->restrictionsActive && (strcmp(type, "data")==0))
-       applyDataRestrictions(filter);
-    QStringList candidates = resourceDirs(type);
+    QStringList candidates;
+    if (filterPath.startsWith("/")) // absolute path
+    {
+        filterPath = filterPath.mid(1);
+        candidates << "/";
+    }
+    else
+    {
+        if (d && d->restrictionsActive && (strcmp(type, "data")==0))
+            applyDataRestrictions(filter);
+        candidates = resourceDirs(type);
+    }
     if (filterFile.isEmpty())
 	filterFile = "*";
 
@@ -503,7 +607,7 @@ KStandardDirs::findAllResources( const char *type,
          it != candidates.end(); it++)
     {
         lookupPrefix(*it, filterPath, "", regExp, list,
-                     relList, recursive, uniq);
+                     relList, recursive, unique);
     }
 
     return list;
@@ -513,13 +617,13 @@ QStringList
 KStandardDirs::findAllResources( const char *type,
 			         const QString& filter,
 				 bool recursive,
-			         bool uniq) const
+			         bool unique) const
 {
     QStringList relList;
-    return findAllResources(type, filter, recursive, uniq, relList);
+    return findAllResources(type, filter, recursive, unique, relList);
 }
 
-QString 
+QString
 KStandardDirs::realPath(const QString &dirname)
 {
     char realpath_buffer[MAXPATHLEN + 1];
@@ -537,71 +641,49 @@ KStandardDirs::realPath(const QString &dirname)
     return dirname;
 }
 
+void KStandardDirs::createSpecialResource(const char *type)
+{
+   char hostname[256];
+   hostname[0] = 0;
+   gethostname(hostname, 255);
+   QString dir = QString("%1%2-%3").arg(localkdedir()).arg(type).arg(hostname);
+   char link[1024];
+   link[1023] = 0;
+   int result = readlink(QFile::encodeName(dir).data(), link, 1023);
+   if ((result == -1) && (errno == ENOENT))
+   {
+      QString srv = findExe(QString::fromLatin1("lnusertemp"), KDEDIR+QString::fromLatin1("/bin"));
+      if (srv.isEmpty())
+         srv = findExe(QString::fromLatin1("lnusertemp"));
+      if (!srv.isEmpty())
+      {
+         system(QFile::encodeName(srv)+" "+type);
+         result = readlink(QFile::encodeName(dir).data(), link, 1023);
+      }
+   }
+   if (result > 0)
+   {
+      link[result] = 0;
+      if (link[0] == '/')
+         dir = QFile::decodeName(link);
+      else
+         dir = QDir::cleanDirPath(dir+QFile::decodeName(link));
+   }
+   addResourceDir(type, dir+'/');
+}
+
 QStringList KStandardDirs::resourceDirs(const char *type) const
 {
     QStringList *candidates = dircache.find(type);
 
     if (!candidates) { // filling cache
         if (strcmp(type, "socket") == 0)
-        {
-          char hostname[256];
-          hostname[0] = 0;
-          gethostname(hostname, 255);
-          QString dir = QString("%1socket-%2").arg(localkdedir()).arg(hostname);
-          char link[1024];
-          link[1023] = 0;
-          int result = readlink(QFile::encodeName(dir).data(), link, 1023);
-          if ((result == -1) && (errno == ENOENT))
-          {
-             QString srv = findExe(QString::fromLatin1("lnusertemp"), KDEDIR+QString::fromLatin1("/bin"));
-             if (srv.isEmpty())
-                srv = findExe(QString::fromLatin1("lnusertemp"));
-             if (!srv.isEmpty())
-             {
-                system(QFile::encodeName(srv)+" socket");
-                result = readlink(QFile::encodeName(dir).data(), link, 1023);
-             }
-          }
-          if (result > 0)
-          {
-             link[result] = 0;
-             if (link[0] == '/')
-                dir = QFile::decodeName(link);
-             else
-                dir = QDir::cleanDirPath(dir+QFile::decodeName(link));
-          }
-          const_cast<KStandardDirs *>(this)->addResourceDir("socket", dir+'/');
-        }
-        if (strcmp(type, "tmp") == 0)
-        {
-          char hostname[256];
-          hostname[0] = 0;
-          gethostname(hostname, 255);
-          QString dir = QString("%1tmp-%2").arg(localkdedir()).arg(hostname);
-          char link[1024];
-          link[1023] = 0;
-          int result = readlink(QFile::encodeName(dir).data(), link, 1023);
-          if ((result == -1) && (errno == ENOENT))
-          {
-             QString srv = findExe(QString::fromLatin1("lnusertemp"), KDEDIR+QString::fromLatin1("/bin"));
-             if (srv.isEmpty())
-                srv = findExe(QString::fromLatin1("lnusertemp"));
-             if (!srv.isEmpty())
-             {
-                system(QFile::encodeName(srv)+" tmp");
-                result = readlink(QFile::encodeName(dir).data(), link, 1023);
-             }
-          }
-          if (result > 0)
-          {
-             link[result] = 0;
-             if (link[0] == '/')
-                dir = QFile::decodeName(link);
-             else
-                dir = QDir::cleanDirPath(dir+QFile::decodeName(link));
-          }
-          const_cast<KStandardDirs *>(this)->addResourceDir("tmp", dir+'/');
-        }
+           const_cast<KStandardDirs *>(this)->createSpecialResource(type);
+        else if (strcmp(type, "tmp") == 0)
+           const_cast<KStandardDirs *>(this)->createSpecialResource(type);
+        else if (strcmp(type, "cache") == 0)
+           const_cast<KStandardDirs *>(this)->createSpecialResource(type);
+
         QDir testdir;
 
         candidates = new QStringList();
@@ -623,8 +705,16 @@ QStringList KStandardDirs::resourceDirs(const char *type) const
         if (dirs)
         {
             bool local = true;
-            for (QStringList::ConstIterator pit = prefixes.begin();
-                 pit != prefixes.end();
+            const QStringList *prefixList = 0;
+            if (strncmp(type, "xdgdata-", 8) == 0)
+                prefixList = &(d->xdgdata_prefixes);
+            else if (strncmp(type, "xdgconf-", 8) == 0)
+                prefixList = &(d->xdgconf_prefixes);
+            else
+                prefixList = &prefixes;
+
+            for (QStringList::ConstIterator pit = prefixList->begin();
+                 pit != prefixList->end();
                  pit++)
             {
                 for (QStringList::ConstIterator it = dirs->begin();
@@ -669,6 +759,50 @@ QStringList KStandardDirs::resourceDirs(const char *type) const
   return *candidates;
 }
 
+QStringList KStandardDirs::systemPaths( const QString& pstr )
+{
+    QStringList tokens;
+    QString p = pstr;
+
+    if( p.isNull() ) 
+    {
+	p = getenv( "PATH" );
+    }
+
+    tokenize( tokens, p, ":\b" );
+
+    QStringList exePaths;
+
+    // split path using : or \b as delimiters
+    for( unsigned i = 0; i < tokens.count(); i++ )
+    {
+	p = tokens[ i ];
+
+        if ( p[ 0 ] == '~' )
+        {
+            int len = p.find( '/' );
+            if ( len == -1 )
+                len = p.length();
+            if ( len == 1 )
+            {
+                p.replace( 0, 1, QDir::homeDirPath() );
+            }
+            else
+            {
+                QString user = p.mid( 1, len - 1 );
+                struct passwd *dir = getpwnam( user.local8Bit().data() );
+                if ( dir && strlen( dir->pw_dir ) )
+                    p.replace( 0, len, QString::fromLocal8Bit( dir->pw_dir ) );
+            }
+        }
+
+	exePaths << p;
+    }
+
+    return exePaths;
+}
+
+
 QString KStandardDirs::findExe( const QString& appname,
 				const QString& pstr, bool ignore)
 {
@@ -692,36 +826,10 @@ QString KStandardDirs::findExe( const QString& appname,
          return p;
     }
 
-    QStringList tokens;
-    p = pstr;
-
-    if( p == QString::null ) {
-	p = getenv( "PATH" );
-    }
-
-    tokenize( tokens, p, ":\b" );
-
-    // split path using : or \b as delimiters
-    for( unsigned i = 0; i < tokens.count(); i++ ) {
-	p = tokens[ i ];
-
-        if ( p[ 0 ] == '~' )
-        {
-            int len = p.find( '/' );
-            if ( len == -1 )
-                len = p.length();
-            if ( len == 1 )
-                p.replace( 0, 1, QDir::homeDirPath() );
-            else
-            {
-                QString user = p.mid( 1, len - 1 );
-                struct passwd *dir = getpwnam( user.local8Bit().data() );
-                if ( dir && strlen( dir->pw_dir ) )
-                    p.replace( 0, len, QString::fromLocal8Bit( dir->pw_dir ) );
-            }
-        }
-
-	p += "/";
+    QStringList exePaths = systemPaths( pstr );
+    for (QStringList::ConstIterator it = exePaths.begin(); it != exePaths.end(); it++)
+    {
+	p = (*it) + "/";
 	p += appname;
 
 	// Check for executable in this tokenized path
@@ -742,20 +850,14 @@ QString KStandardDirs::findExe( const QString& appname,
 int KStandardDirs::findAllExe( QStringList& list, const QString& appname,
 			const QString& pstr, bool ignore )
 {
-    QString p = pstr;
     QFileInfo info;
-    QStringList tokens;
-
-    if( p == QString::null ) {
-	p = getenv( "PATH" );
-    }
-
+    QString p;
     list.clear();
-    tokenize( tokens, p, ":\b" );
 
-    for ( unsigned i = 0; i < tokens.count(); i++ ) {
-	p = tokens[ i ];
-	p += "/";
+    QStringList exePaths = systemPaths( pstr );
+    for (QStringList::ConstIterator it = exePaths.begin(); it != exePaths.end(); it++)
+    {
+	p = (*it) + "/";
 	p += appname;
 
 	info.setFile( p );
@@ -764,7 +866,6 @@ int KStandardDirs::findAllExe( QStringList& list, const QString& appname,
 	    && info.isFile() ) {
 	    list.append( p );
 	}
-
     }
 
     return list.count();
@@ -833,6 +934,14 @@ QString KStandardDirs::kde_default(const char *type) {
 	return "lib/kde3/";
     if (!strcmp(type, "qtplugins"))
         return "lib/kde3/plugins";
+    if (!strcmp(type, "xdgdata-apps"))
+        return "applications/";
+    if (!strcmp(type, "xdgdata-dirs"))
+        return "desktop-directories/";
+    if (!strcmp(type, "xdgconf-menu"))
+        return "menus/";
+    if (!strcmp(type, "kcfg"))
+	return "share/config.kcfg";
     qFatal("unknown resource type %s", type);
     return QString::null;
 }
@@ -847,15 +956,23 @@ QString KStandardDirs::saveLocation(const char *type,
     if (!pPath)
     {
        QStringList *dirs = relatives.find(type);
-       if (!dirs && ((strcmp(type, "socket") == 0) || (strcmp(type, "tmp") == 0)))
+       if (!dirs && (
+                     (strcmp(type, "socket") == 0) ||
+                     (strcmp(type, "tmp") == 0) ||
+                     (strcmp(type, "cache") == 0) ))
        {
-          (void) resourceDirs(type); // Generate socket resource.
+          (void) resourceDirs(type); // Generate socket|tmp|cache resource.
           dirs = relatives.find(type); // Search again.
        }
        if (dirs)
        {
-          // Check for existance of typed directory + suffix
-          pPath = new QString(realPath(localkdedir() + dirs->last()));
+          // Check for existence of typed directory + suffix
+          if (strncmp(type, "xdgdata-", 8) == 0)
+             pPath = new QString(realPath(localxdgdatadir() + dirs->last()));
+          else if (strncmp(type, "xdgconf-", 8) == 0)
+             pPath = new QString(realPath(localxdgconfdir() + dirs->last()));
+          else
+             pPath = new QString(realPath(localkdedir() + dirs->last()));
        }
        else {
           dirs = absolutes.find(type);
@@ -874,11 +991,11 @@ QString KStandardDirs::saveLocation(const char *type,
 #ifndef NDEBUG
 	    qDebug("save location %s doesn't exist", fullPath.latin1());
 #endif
-	    return localkdedir()+suffix;
+	    return fullPath;
 	}
 	if(!makeDir(fullPath, 0700)) {
             qWarning("failed to create %s", fullPath.latin1());
-	    return localkdedir()+suffix;
+	    return fullPath;
 	}
         dircache.remove(type);
     }
@@ -955,18 +1072,11 @@ static QString readEnvPath(const char *env)
    return QFile::decodeName(c_path);
 }
 
-static void fixHomeDir(QString &dir)
-{
-   if (dir[0] == '~')
-   {
-      dir = QDir::homeDirPath() + dir.mid(1);
-   }
-}
-
 void KStandardDirs::addKDEDefaults()
 {
     QStringList kdedirList;
 
+    // begin KDEDIRS
     QString kdedirs = readEnvPath("KDEDIRS");
     if (!kdedirs.isEmpty())
     {
@@ -977,7 +1087,7 @@ void KStandardDirs::addKDEDefaults()
 	QString kdedir = readEnvPath("KDEDIR");
 	if (!kdedir.isEmpty())
         {
-           fixHomeDir(kdedir);
+           kdedir = KShell::tildeExpand(kdedir);
 	   kdedirList.append(kdedir);
         }
     }
@@ -989,51 +1099,113 @@ void KStandardDirs::addKDEDefaults()
        kdedirList.append(execPrefix);
 #endif
 
-    QString localKdeDir;
-    if (getuid())
+    // We treat root differently to prevent a "su" shell messing up the
+    // file permissions in the user's home directory.
+    QString localKdeDir = readEnvPath(getuid() ? "KDEHOME" : "KDEROOTHOME");
+    if (!localKdeDir.isEmpty())
     {
-       localKdeDir = readEnvPath("KDEHOME");
-       if (!localKdeDir.isEmpty())
-       {
-          if (localKdeDir[localKdeDir.length()-1] != '/')
-             localKdeDir += '/';
-       }
-       else
-       {
-          localKdeDir =  QDir::homeDirPath() + "/.kde/";
-       }
+       if (localKdeDir[localKdeDir.length()-1] != '/')
+          localKdeDir += '/';
     }
     else
     {
-       // We treat root different to prevent root messing up the
-       // file permissions in the users home directory.
-       localKdeDir = readEnvPath("KDEROOTHOME");
-       if (!localKdeDir.isEmpty())
-       {
-          if (localKdeDir[localKdeDir.length()-1] != '/')
-             localKdeDir += '/';
-       }
-       else
-       {
-          struct passwd *pw = getpwuid(0);
-          localKdeDir =  QFile::decodeName((pw && pw->pw_dir) ? pw->pw_dir : "/root")  + "/.kde/";
-       }
-
+       localKdeDir =  QDir::homeDirPath() + "/.kde/";
     }
 
     if (localKdeDir != "-/")
     {
-        fixHomeDir(localKdeDir);
+        localKdeDir = KShell::tildeExpand(localKdeDir);
         addPrefix(localKdeDir);
     }
 
     for (QStringList::ConstIterator it = kdedirList.begin();
 	 it != kdedirList.end(); it++)
     {
-        QString dir = *it;
-        fixHomeDir(dir);
+        QString dir = KShell::tildeExpand(*it);
 	addPrefix(dir);
     }
+    // end KDEDIRS
+
+    // begin XDG_CONFIG_XXX
+    QStringList xdgdirList;
+    QString xdgdirs = readEnvPath("XDG_CONFIG_DIRS");
+    if (!xdgdirs.isEmpty())
+    {
+	tokenize(xdgdirList, xdgdirs, ":");
+    }
+    else
+    {
+	xdgdirList.clear();
+        xdgdirList.append("/etc/xdg");
+        xdgdirList.append(KDESYSCONFDIR "/xdg");
+    }
+
+    QString localXdgDir = readEnvPath("XDG_CONFIG_HOME");
+    if (!localXdgDir.isEmpty())
+    {
+       if (localXdgDir[localXdgDir.length()-1] != '/')
+          localXdgDir += '/';
+    }
+    else
+    {
+       localXdgDir =  QDir::homeDirPath() + "/.config/";
+    }
+
+    localXdgDir = KShell::tildeExpand(localXdgDir);
+    addXdgConfigPrefix(localXdgDir);
+
+    for (QStringList::ConstIterator it = xdgdirList.begin();
+	 it != xdgdirList.end(); it++)
+    {
+        QString dir = KShell::tildeExpand(*it);
+	addXdgConfigPrefix(dir);
+    }
+    // end XDG_CONFIG_XXX
+
+    // begin XDG_DATA_XXX
+    xdgdirs = readEnvPath("XDG_DATA_DIRS");
+    if (!xdgdirs.isEmpty())
+    {
+	tokenize(xdgdirList, xdgdirs, ":");
+    }
+    else
+    {
+	xdgdirList.clear();
+        for (QStringList::ConstIterator it = kdedirList.begin();
+           it != kdedirList.end(); it++)
+        {
+           QString dir = *it;
+           if (dir[dir.length()-1] != '/')
+             dir += '/';
+           xdgdirList.append(dir+"share/");
+        }
+
+        xdgdirList.append("/usr/local/share/");
+        xdgdirList.append("/usr/share/");
+    }
+
+    localXdgDir = readEnvPath("XDG_DATA_HOME");
+    if (!localXdgDir.isEmpty())
+    {
+       if (localXdgDir[localXdgDir.length()-1] != '/')
+          localXdgDir += '/';
+    }
+    else
+    {
+       localXdgDir = QDir::homeDirPath() + "/.local/share/";
+    }
+
+    localXdgDir = KShell::tildeExpand(localXdgDir);
+    addXdgDataPrefix(localXdgDir);
+
+    for (QStringList::ConstIterator it = xdgdirList.begin();
+	 it != xdgdirList.end(); it++)
+    {
+        QString dir = KShell::tildeExpand(*it);
+	addXdgDataPrefix(dir);
+    }
+    // end XDG_DATA_XXX
+
 
     uint index = 0;
     while (types[index] != 0) {
@@ -1041,9 +1213,6 @@ void KStandardDirs::addKDEDefaults()
 	index++;
     }
 
-    QString dir = QString("%1share/cache/").arg(localKdeDir);
-    addResourceDir("cache", dir);
-
     addResourceDir("home", QDir::homeDirPath());
 }
 
@@ -1053,10 +1222,70 @@ void KStandardDirs::checkConfig() const
         const_cast<KStandardDirs*>(this)->addCustomized(KGlobal::_instance->_config);
 }
 
+static QStringList lookupProfiles(const QString &mapFile)
+{
+    QStringList profiles;
+
+    if (mapFile.isEmpty() || !QFile::exists(mapFile))
+    {
+       profiles << "default";
+       return profiles;
+    }
+
+    struct passwd *pw = getpwuid(geteuid());
+    if (!pw)
+    {
+        profiles << "default";
+        return profiles; // Not good
+    }
+
+    QCString user = pw->pw_name;
+
+    KSimpleConfig mapCfg(mapFile, true);
+    mapCfg.setGroup("Users");
+    if (mapCfg.hasKey(user.data()))
+    {
+        profiles = mapCfg.readListEntry(user.data());
+        return profiles; 
+    }
+        
+    mapCfg.setGroup("General");
+    QStringList groups = mapCfg.readListEntry("groups");
+
+    mapCfg.setGroup("Groups");
+
+    for( QStringList::ConstIterator it = groups.begin();
+         it != groups.end(); ++it )
+    {
+        QCString grp = (*it).utf8();
+        // Check if user is in this group
+        struct group *grp_ent = getgrnam(grp);
+        if (!grp_ent) continue;
+
+        char ** members = grp_ent->gr_mem;
+        for(char * member; (member = *members); ++members)
+        {
+            if (user == member)
+            {
+                // User is in this group --> add profiles
+                profiles += mapCfg.readListEntry(*it);
+                break;
+            }
+        }
+    }
+
+    if (profiles.isEmpty())
+        profiles << "default";
+    return profiles;
+}
+
 bool KStandardDirs::addCustomized(KConfig *config)
 {
     if (addedCustoms) // there are already customized entries
-        return false; // we just quite and hope they are the right ones
+        return false; // we just quit and hope they are the right ones
+
+    // save it for future calls - that will return
+    addedCustoms = true;
 
     // save the numbers of config directories. If this changes,
     // we will return true to give KConfig a chance to reparse
@@ -1064,54 +1293,63 @@ bool KStandardDirs::addCustomized(KConfig *config)
 
     // reading the prefixes in
     QString oldGroup = config->group();
-    config->setGroup("Directories");
+    QString group = QString::fromLatin1("Directories");
+    config->setGroup(group);
+    QString userMapFile = config->readEntry("userProfileMapFile");
 
-    QStringList list;
-    QStringList::ConstIterator it;
-    list = config->readListEntry("prefixes");
-    for (it = list.begin(); it != list.end(); it++)
-	addPrefix(*it);
-
-    // iterating over all entries in the group Directories
-    // to find entries that start with dir_$type
-    QMap<QString, QString> entries = config->entryMap("Directories");
-
-    QMap<QString, QString>::ConstIterator it2;
-    for (it2 = entries.begin(); it2 != entries.end(); it2++)
+    QStringList profiles = lookupProfiles(userMapFile);
+    
+    bool priority = false;
+    while(true)
     {
-	QString key = it2.key();
-	if (key.left(4) == "dir_") {
-	    // generate directory list, there may be more than 1.
-	    QStringList dirs = QStringList::split(',',
-						  *it2);
-	    QStringList::Iterator sIt(dirs.begin());
-	    QString resType = key.mid(4, key.length());
-	    for (; sIt != dirs.end(); ++sIt) {
-		addResourceDir(resType.latin1(), *sIt);
-	    }
+        config->setGroup(group);
+        QStringList list = config->readListEntry("prefixes");
+        for (QStringList::ConstIterator it = list.begin(); it != list.end(); it++)
+        {
+            addPrefix(*it, priority);
+	    addXdgConfigPrefix(*it+"/etc/xdg", priority);
 	}
+
+        // iterating over all entries in the group Directories
+        // to find entries that start with dir_$type
+        QMap<QString, QString> entries = config->entryMap(group);
+        for (QMap<QString, QString>::ConstIterator it2 = entries.begin(); 
+             it2 != entries.end(); it2++)
+        {
+            QString key = it2.key();
+            if (key.startsWith("dir_")) {
+                // generate directory list, there may be more than 1.
+                QStringList dirs = QStringList::split(',',
+						  *it2);
+                QStringList::Iterator sIt(dirs.begin());
+                QString resType = key.mid(4, key.length());
+                for (; sIt != dirs.end(); ++sIt) {
+                    addResourceDir(resType.latin1(), *sIt, priority);
+                }
+            }
+        }
+        if (profiles.isEmpty())
+           break;
+        group = QString::fromLatin1("Directories-%1").arg(profiles.back());
+        profiles.pop_back();
+        priority = true;
     }
 
     // Process KIOSK restrictions.
     config->setGroup("KDE Resource Restrictions");
-    entries = config->entryMap("KDE Resource Restrictions");
-    for (it2 = entries.begin(); it2 != entries.end(); it2++)
+    QMap<QString, QString> entries = config->entryMap("KDE Resource Restrictions");
+    for (QMap<QString, QString>::ConstIterator it2 = entries.begin(); 
+         it2 != entries.end(); it2++)
     {
 	QString key = it2.key();
         if (!config->readBoolEntry(key, true))
         {
-           if (!d)
-           {
-               d = new KStandardDirsPrivate;
-               d->restrictionsActive = true;
-           }
+           d->restrictionsActive = true;
            d->restrictions.insert(key.latin1(), &d->restrictionsActive); // Anything will do
            dircache.remove(key.latin1());
         }
     }
 
-    // save it for future calls - that will return
-    addedCustoms = true;
     config->setGroup(oldGroup);
 
     // return true if the number of config dirs changed
@@ -1124,6 +1362,18 @@ QString KStandardDirs::localkdedir() const
     return prefixes.first();
 }
 
+QString KStandardDirs::localxdgdatadir() const
+{
+    // Return the prefix to use for saving
+    return d->xdgdata_prefixes.first();
+}
+
+QString KStandardDirs::localxdgconfdir() const
+{
+    // Return the prefix to use for saving
+    return d->xdgconf_prefixes.first();
+}
+
 // just to make code more readable without macros
 QString locate( const char *type,
 		const QString& filename, const KInstance* inst )
@@ -1133,15 +1383,21 @@ QString locate( const char *type,
 
 QString locateLocal( const char *type,
 	             const QString& filename, const KInstance* inst )
+{
+    return locateLocal(type, filename, true, inst);
+}
+
+QString locateLocal( const char *type,
+	             const QString& filename, bool createDir, const KInstance* inst )
 {
     // try to find slashes. If there are some, we have to
     // create the subdir first
     int slash = filename.findRev('/')+1;
     if (!slash) // only one filename
-	return inst->dirs()->saveLocation(type) + filename;
+	return inst->dirs()->saveLocation(type, QString::null, createDir) + filename;
 
     // split path from filename
     QString dir = filename.left(slash);
     QString file = filename.mid(slash);
-    return inst->dirs()->saveLocation(type, dir) + file;
+    return inst->dirs()->saveLocation(type, dir, createDir) + file;
 }
