@@ -34,7 +34,7 @@ class KStandardDirsPrivate;
 /**
  * @short Site-independent access to standard KDE directories.
  * @author Stephan Kulow <coolo@kde.org> and Sirtaj Singh Kang <taj@kde.org>
- * @version $Id: kstandarddirs.h,v 1.100.2.1 2003/01/08 16:15:51 waba Exp $
+ * @version $Id: kstandarddirs.h,v 1.111.2.2 2004/05/21 11:07:18 waba Exp $
  *
  * This is one of the most central classes in kdelibs as
  * it provides a basic service: It knows where the files
@@ -46,7 +46,7 @@ class KStandardDirsPrivate;
  * and applications always refer to a file with a resource type
  * (e.g. icon) and a filename (e.g. khexdit.xpm). In an ideal world
  * the application would make no assumption where this file is and
- * leave it up to @ref KStandardDirs::findResource("apps", "Home.desktop")
+ * leave it up to KStandardDirs::findResource("apps", "Home.desktop")
  * to apply this knowledge to return /opt/kde/share/applnk/Home.desktop
  * or ::locate("data", "kgame/background.jpg") to return
  * /opt/kde/share/apps/kgame/background.jpg
@@ -59,7 +59,7 @@ class KStandardDirsPrivate;
  * defined suffixes where specific resource types are to be found.
  * For example, for the resource type "html" the suffixes could be
  * share/doc/HTML and share/doc/kde/HTML.
- * So the search algorithm basicly appends to each prefix each registered
+ * So the search algorithm basically appends to each prefix each registered
  * suffix and tries to locate the file there.
  * To make the thing even more complex, it's also possible to register
  * absolute paths that KStandardDirs looks up after not finding anything
@@ -67,18 +67,19 @@ class KStandardDirsPrivate;
  * specific directories that aren't in his $HOME/.kde directory for,
  * for example, icons.
  *
- * @sect Standard resources that kdelibs allocates are:
+ * <b>Standard resources that kdelibs allocates are:</b>\n
  *
  * @li apps - Applications menu (.desktop files).
  * @li cache - Cached information (e.g. favicons, web-pages)
  * @li cgi - CGIs to run from kdehelp.
  * @li config - Configuration files.
  * @li data - Where applications store data.
- * @li exe - Executables in $prefix/bin. @ref #findExe for a function that takes $PATH into account.
+ * @li exe - Executables in $prefix/bin. findExe() for a function that takes $PATH into account.
  * @li html - HTML documentation.
- * @li icon - Icons, see @ref KIconLoader.
+ * @li icon - Icons, see KIconLoader.
+ * @li kcfg - Configuration meta information files used by KConfig XT.
  * @li lib - Libraries.
- * @li locale - Translation files for @ref KLocale.
+ * @li locale - Translation files for KLocale.
  * @li mime - Mime types.
  * @li module - Module (dynamically loaded library).
  * @li qtplugins - Qt plugins (dynamically loaded objects for Qt)
@@ -88,25 +89,25 @@ class KStandardDirsPrivate;
  * @li sound - Application sounds.
  * @li templates - Templates
  * @li wallpaper - Wallpapers.
- * @li tmp - Temporary files (specfic for both current host and current user)
+ * @li tmp - Temporary files (specific for both current host and current user)
  * @li socket - UNIX Sockets (specific for both current host and current user)
  *
- * A type that is added by the class @ref KApplication if you use it, is
+ * A type that is added by the class KApplication if you use it, is
  * appdata. This one makes the use of the type data a bit easier as it
  * appends the name of the application.
  * So while you had to ::locate("data", "appname/filename") so you can
  * also write ::locate("appdata", "filename") if your KApplication instance
  * is called "appname" (as set via KApplication's constructor or KAboutData, if
- * you use the global KStandardDirs object @ref KGlobal::dirs()).
+ * you use the global KStandardDirs object KGlobal::dirs()).
  * Please note though that you cannot use the "appdata"
  * type if you intend to use it in an applet for Kicker because 'appname' would
  * be "Kicker" instead of the applet's name. Therefore, for applets, you've got
  * to work around this by using ::locate("data", "appletname/filename").
  *
- * @sect KStandardDirs supports the following environment variables:
+ * <b>KStandardDirs supports the following environment variables:</b>
  *
  * @li KDEDIRS: This may set an additional number of directory prefixes to
- *          search for resources. The directories should be seperated
+ *          search for resources. The directories should be separated
  *          by ':'. The directories are searched in the order they are
  *          specified.
  * @li KDEDIR:  Used for backwards compatibility. As KDEDIRS but only a single
@@ -146,12 +147,30 @@ public:
 	 */
 	void addPrefix( const QString& dir );
 
+	/**
+	 * Adds another search dir to front of the XDG_CONFIG_XXX list
+	 * of prefixes.
+	 * This prefix is only used for resources that start with "xdgconf-"
+	 *
+	 * @param dir The directory to append relative paths to.
+	 */
+	void addXdgConfigPrefix( const QString& dir );
+
+	/**
+	 * Adds another search dir to front of the XDG_DATA_XXX list
+	 * of prefixes.
+	 * This prefix is only used for resources that start with "xdgdata-"
+	 *
+	 * @param dir The directory to append relative paths to.
+	 */
+	void addXdgDataPrefix( const QString& dir );
+
 	/**
 	 * Adds suffixes for types.
 	 *
 	 * You may add as many as you need, but it is advised that there
 	 * is exactly one to make writing definite.
-	 * All basic types (@ref kde_default) are added by @ref addKDEDefaults(),
+	 * All basic types ( kde_default) are added by addKDEDefaults(),
 	 * but for those you can add more relative paths as well.
 	 *
 	 * The later a suffix is added, the higher its priority. Note, that the
@@ -188,7 +207,7 @@ public:
 
 	/**
 	 * Tries to find a resource in the following order:
-	 * @li All PREFIX/<relativename> paths (most recent first).
+	 * @li All PREFIX/\<relativename> paths (most recent first).
 	 * @li All absolute paths (most recent first).
 	 *
 	 * The filename should be a filename relative to the base dir
@@ -261,7 +280,7 @@ public:
 
 	/**
 	 * Tries to find the directory the file is in.
-	 * It works the same as @ref findResource(), but it doesn't
+	 * It works the same as findResource(), but it doesn't
 	 * return the filename but the name of the directory.
 	 *
 	 * This way the application can access a couple of files
@@ -291,12 +310,12 @@ public:
 	 *
 	 * @param type The type of resource to locate directories for.
 	 * @param filter Only accept filenames that fit to filter. The filter
-	 *        may consist of an optional directory and a @ref QRegExp
+	 *        may consist of an optional directory and a QRegExp
 	 *        wildcard expression. E.g. "images\*.jpg". Use QString::null
 	 *        if you do not want a filter.
 	 * @param recursive Specifies if the function should decend
 	 *        into subdirectories.
-	 * @param uniq If specified,  only return items which have
+	 * @param unique If specified,  only return items which have
 	 *        unique suffixes - suppressing duplicated filenames.
 	 *
 	 * @return A list of directories matching the resource specified,
@@ -305,7 +324,7 @@ public:
 	QStringList findAllResources( const char *type,
 				       const QString& filter = QString::null,
 				       bool recursive = false,
-				       bool uniq = false) const;
+				       bool unique = false) const;
 
 	/**
 	 * Tries to find all resources with the specified type.
@@ -316,14 +335,13 @@ public:
 	 *
 	 * @param type The type of resource to locate directories for.
 	 * @param filter Only accept filenames that fit to filter. The filter
-	 *        may consist of an optional directory and a @ref QRegExp
+	 *        may consist of an optional directory and a QRegExp
 	 *        wildcard expression. E.g. "images\*.jpg". Use QString::null
 	 *        if you do not want a filter.
 	 * @param recursive Specifies if the function should decend
 	 *        into subdirectories.
-	 * @param uniq If specified,  only return items which have
+	 * @param unique If specified,  only return items which have
 	 *        unique suffixes.
-	 * @param list Of relative paths for the given type.
 	 * @param relPaths The list to store the relative paths into
 	 *        These can be used later to ::locate() the file
 	 *
@@ -333,9 +351,20 @@ public:
 	QStringList findAllResources( const char *type,
 				       const QString& filter,
 				       bool recursive,
-				       bool uniq,
+				       bool unique,
 				       QStringList &relPaths) const;
 
+	/** 
+	 * Returns a QStringList list of pathnames in the system path.
+	 *
+	 * @param pstr  The path which will be searched. If this is
+	 * 		null (default), the $PATH environment variable will
+	 *		be searched.
+	 *
+	 * @return a QStringList list of pathnames in the system path.
+	 */
+	static QStringList systemPaths( const QString& pstr=QString::null );
+
 	/**
 	 * Finds the executable in the system path.
 	 *
@@ -358,7 +387,7 @@ public:
 				bool ignoreExecBit=false );
 
 	/**
-	 * Finds all occurences of an executable in the system path.
+	 * Finds all occurrences of an executable in the system path.
 	 *
 	 * @param list	Will be filled with the pathnames of all the
 	 *		executables found. Will be empty if the executable
@@ -383,16 +412,16 @@ public:
 	 * This function adds the defaults that are used by the current
 	 * KDE version.
 	 *
-	 * It's a series of @ref addResourceTypes()
-	 * and @ref addPrefix() calls.
+	 * It's a series of addResourceTypes()
+	 * and addPrefix() calls.
 	 * You normally wouldn't call this function because it's called
-	 * for you from @ref KGlobal.
+	 * for you from KGlobal.
 	 */
 	void addKDEDefaults();
 
 	/**
 	 * Reads customized entries out of the given config object and add
-	 * them via @ref addResourceDirs().
+	 * them via addResourceDirs().
 	 *
 	 * @param config The object the entries are read from. This should
 	 *        contain global config files
@@ -514,6 +543,18 @@ public:
 	 */
 	QString localkdedir() const;
 
+	/**
+	 * @return $XDG_DATA_HOME
+	 * See also http://www.freedesktop.org/standards/basedir/draft/basedir-spec/basedir-spec.html
+	 */
+	QString localxdgdatadir() const;
+
+	/**
+	 * @return $XDG_CONFIG_HOME
+	 * See also http://www.freedesktop.org/standards/basedir/draft/basedir-spec/basedir-spec.html
+	 */
+	QString localxdgconfdir() const;
+
 	/**
 	 * Checks for existence and accessability.
 	 * Faster than creating a QFileInfo first.
@@ -523,10 +564,10 @@ public:
 	static bool exists(const QString &fullPath);
 
         /**
-         * Expands all symbolic links and resolves references to 
+         * Expands all symbolic links and resolves references to
          * '/./', '/../' and extra  '/' characters in @p dirname
          * and returns the canonicalized absolute pathname.
-         * The resulting path will have no symbolic link, '/./' 
+         * The resulting path will have no symbolic link, '/./'
          * or '/../' components.
 	 * @since 3.1
          */
@@ -553,6 +594,21 @@ public:
 
 	void checkConfig() const;
 	void applyDataRestrictions(const QString &) const;
+	void createSpecialResource(const char*);
+
+        // Like their publi counter parts but with an extra priority argument
+        // If priority is true, the directory is added directly after
+        // $KDEHOME/$XDG_DATA_HOME/$XDG_CONFIG_HOME
+	void addPrefix( const QString& dir, bool priority );
+	void addXdgConfigPrefix( const QString& dir, bool priority );
+	void addXdgDataPrefix( const QString& dir, bool priority );
+
+	// If priority is true, the directory is added before any other,
+	// otherwise after
+	bool addResourceType( const char *type,
+			      const QString& relativename, bool priority );
+	bool addResourceDir( const char *type,
+			     const QString& absdir, bool priority);
 };
 
 /**
@@ -624,6 +680,17 @@ QString locate( const char *type, const QString& filename, const KInstance* inst
  **/
 QString locateLocal( const char *type, const QString& filename, const KInstance* instance = KGlobal::instance() );
 
+/*!
+ * \relates KStandardDirs
+ * This function is much like locate. No check is made if the
+ * specified filename actually exists. Missing directories
+ * are created if @p createDir is true. If filename is only
+ * a directory, without a specific file,
+ * filename must have a trailing slash.
+ *
+ **/
+QString locateLocal( const char *type, const QString& filename, bool createDir, const KInstance* instance = KGlobal::instance() );
+
 /*! @} */
 
 #endif // SSK_KSTDDIRS_H
