@@ -1,8 +1,8 @@
 /****************************************************************************
 
- $Id: kstartupinfo.cpp,v 1.32.2.5 2003/02/25 08:05:12 lunakl Exp $
+ $Id: kstartupinfo.cpp,v 1.50.2.2 2004/03/11 12:10:32 lunakl Exp $
 
- Copyright (C) 2001 Lubos Lunak        <l.lunak@kde.org>
+ Copyright (C) 2001-2003 Lubos Lunak        <l.lunak@kde.org>
 
 Permission is hereby granted, free of charge, to any person obtaining a
 copy of this software and associated documentation files (the "Software"),
@@ -27,10 +27,14 @@ DEALINGS IN THE SOFTWARE.
 // kdDebug() can't be turned off in kdeinit
 #if 0
 #define KSTARTUPINFO_ALL_DEBUG
+#warning Extra KStartupInfo debug messages enabled.
 #endif
 
 #include <qwidget.h>
-#ifdef Q_WS_X11 // FIXME(E): Re-implement in a less X11 specific way
+
+#include "config.h"
+#if defined Q_WS_X11 && ! defined K_WS_QTONLY
+//#ifdef Q_WS_X11 // FIXME(E): Re-implement in a less X11 specific way
 #include <qglobal.h>
 #ifdef HAVE_CONFIG_H
 #include <config.h>
@@ -47,37 +51,35 @@ DEALINGS IN THE SOFTWARE.
 #include <sys/time.h>
 #include <stdlib.h>
 #include <qtimer.h>
-#include <netwm.h>
+#if defined Q_WS_X11 && ! defined K_WS_QTONLY
+#include <netwm.h> // schroder
+#endif
 #include <kdebug.h>
 #include <kapplication.h>
 #include <signal.h>
-#include <kwinmodule.h>
-#include <kxmessages.h>
-#ifndef KDE_USE_FINAL
-#include <X11/Xlibint.h> // cannot be included multiple times
+#if defined Q_WS_X11 && ! defined K_WS_QTONLY
+#include <kwinmodule.h> // schroder
+#include <kxmessages.h> // schroder
+#include <kwin.h>
+extern Time qt_x_time;
 #endif
 
-#undef Data // crappy X11
+static const char* const NET_STARTUP_MSG = "_NET_STARTUP_INFO";
+static const char* const NET_STARTUP_WINDOW = "_NET_STARTUP_ID";
+// DESKTOP_STARTUP_ID is used also in kinit/wrapper.c
+static const char* const NET_STARTUP_ENV = "DESKTOP_STARTUP_ID";
 
-#ifndef None  // CHECKME
-#define None 0
-#endif
+static bool auto_app_started_sending = true;
 
-static const char* const KDE_STARTUP_INFO = "_KDE_STARTUP_INFO";
-static const char* const KDE_STARTUP_ID = "_KDE_STARTUP_ID";
-// KDE_STARTUP_ENV is used also in kinit/wrapper.c
-static const char* const KDE_STARTUP_ENV = "KDE_STARTUP_ENV";
-
-// TODO these two are for backward compatibility with KDE2.x
-static const char* const KDE_STARTUP_INFO_2 = "KDE_STARTUP_INFO";
-static const char* const KDE_STARTUP_ID_2 = "KDE_STARTUP_ID";
-
-static int get_num( const QString& item_P );
+static long get_num( const QString& item_P );
+static unsigned long get_unum( const QString& item_P );
 static QString get_str( const QString& item_P );
 static QCString get_cstr( const QString& item_P );
 static QStringList get_fields( const QString& txt_P );
 static QString escape_str( const QString& str_P );
 
+static Atom utf8_string_atom = None;
+
 class KStartupInfo::Data
     : public KStartupInfoData
     {
@@ -94,13 +96,17 @@ struct KStartupInfoPrivate
         QMap< KStartupInfoId, KStartupInfo::Data > startups;
 	// contains silenced ASN's only if !AnnounceSilencedChanges
         QMap< KStartupInfoId, KStartupInfo::Data > silent_startups;
+#if defined Q_WS_X11 && ! defined K_WS_QTONLY
         KWinModule* wm_module;
         KXMessages msgs;
-        KXMessages msgs_2;
+#endif
 	QTimer* cleanup;
 	int flags;
 	KStartupInfoPrivate( int flags_P )
-    	    : msgs( KDE_STARTUP_INFO ), msgs_2( KDE_STARTUP_INFO_2 ),
+    	    :
+#if defined Q_WS_X11 && ! defined K_WS_QTONLY
+	    msgs( NET_STARTUP_MSG, NULL, false ),
+#endif
 	      flags( flags_P ) {}
     };
 
@@ -121,10 +127,13 @@ KStartupInfo::KStartupInfo( bool clean_on_cantdetect_P, QObject* parent_P, const
 void KStartupInfo::init( int flags_P )
     {
     // d == NULL means "disabled"
-    if (!KApplication::kApplication()) return;
-    if (!KApplication::kApplication()->getDisplay()) return;
+    if( !KApplication::kApplication())
+        return;
+    if( !KApplication::kApplication()->getDisplay())
+        return;
 
     d = new KStartupInfoPrivate( flags_P );
+#if defined Q_WS_X11 && ! defined K_WS_QTONLY
     if( !( d->flags & DisableKWinModule ))
         {
         d->wm_module = new KWinModule( this );
@@ -134,7 +143,7 @@ void KStartupInfo::init( int flags_P )
     else
         d->wm_module = NULL;
     connect( &d->msgs, SIGNAL( gotMessage( const QString& )), SLOT( got_message( const QString& )));
-    connect( &d->msgs_2, SIGNAL( gotMessage( const QString& )), SLOT( got_message( const QString& )));
+#endif
     d->cleanup = new QTimer( this );
     connect( d->cleanup, SIGNAL( timeout()), SLOT( startups_cleanup()));
     }
@@ -146,6 +155,7 @@ KStartupInfo::~KStartupInfo()
 
 void KStartupInfo::got_message( const QString& msg_P )
     {
+// TODO do something with SCREEN= ?
     kdDebug( 172 ) << "got:" << msg_P << endl;
     QString msg = msg_P.stripWhiteSpace();
     if( msg.startsWith( "new:" )) // must match length below
@@ -161,6 +171,7 @@ void KStartupInfo::got_message( const QString& msg_P )
 // actually gets the info about the started application (depends
 // on their order in X11 event filter in KApplication)
 // simply delay info from KWinModule a bit
+// SELI???
 namespace
 {
 class DelayedWindowEvent
@@ -190,17 +201,11 @@ void KStartupInfo::window_added( WId w_P )
     {
     KStartupInfoId id;
     KStartupInfoData data;
-    startup_t ret = check_startup_internal( w_P, &id, &data, false );
+    startup_t ret = check_startup_internal( w_P, &id, &data );
     switch( ret )
         {
         case Match:
             kdDebug( 172 ) << "new window match" << endl;
-	    // HACK
-	    // with KDE's spec, first matching window ends ASN
-	    // but when silenced, ASN shouldn't be ended
-	    // TODO this shouldn't be needed with the new spec?
-	    if( data.silent() != KStartupInfoData::Yes )
-        	remove_startup_info_internal( id );
           break;
         case NoMatch:
           break; // nothing
@@ -223,7 +228,8 @@ void KStartupInfo::got_startup_info( const QString& msg_P, bool update_only_P )
 void KStartupInfo::new_startup_info_internal( const KStartupInfoId& id_P,
     Data& data_P, bool update_only_P )
     {
-    if (!d) return;
+    if( d == NULL )
+        return;
     if( id_P.none())
         return;
     if( d->startups.contains( id_P ))
@@ -290,7 +296,8 @@ void KStartupInfo::got_remove_startup_info( const QString& msg_P )
 
 void KStartupInfo::remove_startup_info_internal( const KStartupInfoId& id_P )
     {
-    if (!d) return;
+    if( d == NULL )
+        return;
     if( d->startups.contains( id_P ))
         {
 	kdDebug( 172 ) << "removing" << endl;
@@ -307,7 +314,8 @@ void KStartupInfo::remove_startup_info_internal( const KStartupInfoId& id_P )
 
 void KStartupInfo::remove_startup_pids( const KStartupInfoData& data_P )
     { // first find the matching info
-    if (!d) return;
+    if( d == NULL )
+        return;
     for( QMap< KStartupInfoId, Data >::Iterator it = d->startups.begin();
          it != d->startups.end();
          ++it )
@@ -324,7 +332,8 @@ void KStartupInfo::remove_startup_pids( const KStartupInfoData& data_P )
 void KStartupInfo::remove_startup_pids( const KStartupInfoId& id_P,
     const KStartupInfoData& data_P )
     {
-    if (!d) return;
+    if( d == NULL )
+        return;
     kdFatal( data_P.pids().count() == 0, 172 );
     Data* data = NULL;
     if( d->startups.contains( id_P ))
@@ -348,8 +357,9 @@ bool KStartupInfo::sendStartup( const KStartupInfoId& id_P, const KStartupInfoDa
     KXMessages msgs;
     QString msg = QString::fromLatin1( "new: %1 %2" )
         .arg( id_P.to_text()).arg( data_P.to_text());
+    msg = check_required_startup_fields( msg, data_P, qt_xscreen());
     kdDebug( 172 ) << "sending " << msg << endl;
-    msgs.broadcastMessage( KDE_STARTUP_INFO, msg );
+    msgs.broadcastMessage( NET_STARTUP_MSG, msg, -1, false );
     return true;
     }
 
@@ -360,10 +370,28 @@ bool KStartupInfo::sendStartupX( Display* disp_P, const KStartupInfoId& id_P,
         return false;
     QString msg = QString::fromLatin1( "new: %1 %2" )
         .arg( id_P.to_text()).arg( data_P.to_text());
+    msg = check_required_startup_fields( msg, data_P, DefaultScreen( disp_P ));
 #ifdef KSTARTUPINFO_ALL_DEBUG
     kdDebug( 172 ) << "sending " << msg << endl;
 #endif
-    return KXMessages::broadcastMessageX( disp_P, KDE_STARTUP_INFO, msg );
+    return KXMessages::broadcastMessageX( disp_P, NET_STARTUP_MSG, msg, -1, false );
+    }
+
+QString KStartupInfo::check_required_startup_fields( const QString& msg, const KStartupInfoData& data_P,
+    int screen )
+    {
+    QString ret = msg;
+    if( data_P.name().isEmpty())
+        {
+//        kdWarning( 172 ) << "NAME not specified in initial startup message" << endl;
+        QString name = data_P.bin();
+        if( name.isEmpty())
+            name = "UNKNOWN";
+        ret += QString( " NAME=\"%1\"" ).arg( escape_str( name ));
+        }
+    if( data_P.screen() == -1 ) // add automatically if needed
+        ret += QString( " SCREEN=%1" ).arg( screen );
+    return ret;
     }
 
 bool KStartupInfo::sendChange( const KStartupInfoId& id_P, const KStartupInfoData& data_P )
@@ -374,7 +402,7 @@ bool KStartupInfo::sendChange( const KStartupInfoId& id_P, const KStartupInfoDat
     QString msg = QString::fromLatin1( "change: %1 %2" )
         .arg( id_P.to_text()).arg( data_P.to_text());
     kdDebug( 172 ) << "sending " << msg << endl;
-    msgs.broadcastMessage( KDE_STARTUP_INFO, msg );
+    msgs.broadcastMessage( NET_STARTUP_MSG, msg, -1, false );
     return true;
     }
 
@@ -388,7 +416,7 @@ bool KStartupInfo::sendChangeX( Display* disp_P, const KStartupInfoId& id_P,
 #ifdef KSTARTUPINFO_ALL_DEBUG
     kdDebug( 172 ) << "sending " << msg << endl;
 #endif
-    return KXMessages::broadcastMessageX( disp_P, KDE_STARTUP_INFO, msg );
+    return KXMessages::broadcastMessageX( disp_P, NET_STARTUP_MSG, msg, -1, false );
     }
 
 bool KStartupInfo::sendFinish( const KStartupInfoId& id_P )
@@ -398,7 +426,7 @@ bool KStartupInfo::sendFinish( const KStartupInfoId& id_P )
     KXMessages msgs;
     QString msg = QString::fromLatin1( "remove: %1" ).arg( id_P.to_text());
     kdDebug( 172 ) << "sending " << msg << endl;
-    msgs.broadcastMessage( KDE_STARTUP_INFO, msg );
+    msgs.broadcastMessage( NET_STARTUP_MSG, msg, -1, false );
     return true;
     }
 
@@ -410,7 +438,7 @@ bool KStartupInfo::sendFinishX( Display* disp_P, const KStartupInfoId& id_P )
 #ifdef KSTARTUPINFO_ALL_DEBUG
     kdDebug( 172 ) << "sending " << msg << endl;
 #endif
-    return KXMessages::broadcastMessageX( disp_P, KDE_STARTUP_INFO, msg );
+    return KXMessages::broadcastMessageX( disp_P, NET_STARTUP_MSG, msg, -1, false );
     }
 
 bool KStartupInfo::sendFinish( const KStartupInfoId& id_P, const KStartupInfoData& data_P )
@@ -421,7 +449,7 @@ bool KStartupInfo::sendFinish( const KStartupInfoId& id_P, const KStartupInfoDat
     QString msg = QString::fromLatin1( "remove: %1 %2" )
         .arg( id_P.to_text()).arg( data_P.to_text());
     kdDebug( 172 ) << "sending " << msg << endl;
-    msgs.broadcastMessage( KDE_STARTUP_INFO, msg );
+    msgs.broadcastMessage( NET_STARTUP_MSG, msg, -1, false );
     return true;
     }
 
@@ -435,68 +463,127 @@ bool KStartupInfo::sendFinishX( Display* disp_P, const KStartupInfoId& id_P,
 #ifdef KSTARTUPINFO_ALL_DEBUG
     kdDebug( 172 ) << "sending " << msg << endl;
 #endif
-    return KXMessages::broadcastMessageX( disp_P, KDE_STARTUP_INFO, msg );
+    return KXMessages::broadcastMessageX( disp_P, NET_STARTUP_MSG, msg, -1, false );
     }
 
 void KStartupInfo::appStarted()
     {
-    if( kapp != NULL ) // KApplication constructor unsets the env. variable
-        {
-        KStartupInfoId id;
-        id.initId( kapp->startupId());
-        if( !id.none())
-            KStartupInfo::sendFinish( id );
-        }
+    if( kapp != NULL )  // KApplication constructor unsets the env. variable
+        appStarted( kapp->startupId());
+    else
+        appStarted( KStartupInfo::currentStartupIdEnv().id());
+    }
+
+void KStartupInfo::appStarted( const QCString& startup_id )
+    {
+    KStartupInfoId id;
+    id.initId( startup_id );
+    if( id.none())
+        return;
+    if( kapp != NULL )
+        KStartupInfo::sendFinish( id );
     else if( getenv( "DISPLAY" ) != NULL ) // don't rely on qt_xdisplay()
         {
-        KStartupInfoId id = KStartupInfo::currentStartupIdEnv();
-        if( !id.none())
+#if defined Q_WS_X11 && ! defined K_WS_QTONLY
+        Display* disp = XOpenDisplay( NULL );
+        if( disp != NULL )
             {
-            Display* disp = XOpenDisplay( NULL );
-            if( disp != NULL )
+            KStartupInfo::sendFinishX( disp, id );
+            XCloseDisplay( disp );
+            }
+#endif
+        }
+    }
+
+void KStartupInfo::disableAutoAppStartedSending( bool disable )
+    {
+    auto_app_started_sending = !disable;
+    }
+
+void KStartupInfo::silenceStartup( bool silence )
+    {
+    KStartupInfoId id;
+    id.initId( kapp->startupId());
+    if( id.none())
+        return;
+    KStartupInfoData data;
+    data.setSilent( silence ? KStartupInfoData::Yes : KStartupInfoData::No );
+    sendChange( id, data );
+    }
+
+void KStartupInfo::handleAutoAppStartedSending()
+    {
+    if( auto_app_started_sending )
+        appStarted();
+    }
+
+void KStartupInfo::setNewStartupId( QWidget* window, const QCString& startup_id )
+    {
+    long activate = true;
+    kapp->setStartupId( startup_id );
+    if( window != NULL )
+        {
+        if( !startup_id.isEmpty() && startup_id != "0" )
+            {
+            NETRootInfo i( qt_xdisplay(), NET::Supported );
+            if( i.isSupported( NET::WM2StartupId ))
                 {
-                KStartupInfo::sendFinishX( disp, id );
-                XCloseDisplay( disp );
+                KStartupInfo::setWindowStartupId( window->winId(), startup_id );
+                activate = false; // WM will take care of it
                 }
             }
+        if( activate )
+        // This is not very nice, but there's no way how to get any
+        // usable timestamp without ASN, so force activating the window.
+        // And even with ASN, it's not possible to get the timestamp here,
+        // so if the WM doesn't have support for ASN, it can't be used either.
+            KWin::forceActiveWindow( window->winId());
         }
+    KStartupInfo::handleAutoAppStartedSending();
     }
 
 KStartupInfo::startup_t KStartupInfo::checkStartup( WId w_P, KStartupInfoId& id_O,
     KStartupInfoData& data_O )
     {
-    return check_startup_internal( w_P, &id_O, &data_O, true );
+    return check_startup_internal( w_P, &id_O, &data_O );
     }
 
 KStartupInfo::startup_t KStartupInfo::checkStartup( WId w_P, KStartupInfoId& id_O )
     {
-    return check_startup_internal( w_P, &id_O, NULL, true );
+    return check_startup_internal( w_P, &id_O, NULL );
     }
 
 KStartupInfo::startup_t KStartupInfo::checkStartup( WId w_P, KStartupInfoData& data_O )
     {
-    return check_startup_internal( w_P, NULL, &data_O, true );
+    return check_startup_internal( w_P, NULL, &data_O );
     }
 
 KStartupInfo::startup_t KStartupInfo::checkStartup( WId w_P )
     {
-    return check_startup_internal( w_P, NULL, NULL, true );
+    return check_startup_internal( w_P, NULL, NULL );
     }
 
 KStartupInfo::startup_t KStartupInfo::check_startup_internal( WId w_P, KStartupInfoId* id_O,
-    KStartupInfoData* data_O, bool remove_P )
+    KStartupInfoData* data_O )
     {
-    if (!d) return NoMatch;
+    if( d == NULL )
+        return NoMatch;
     if( d->startups.count() == 0 )
         return NoMatch; // no startups
+#if defined Q_WS_X11 && ! defined K_WS_QTONLY
+// SELI ???
     NETWinInfo info( qt_xdisplay(),  w_P, qt_xrootwin(),
         NET::WMWindowType | NET::WMPid | NET::WMState );
     // ignore NET::Tool and other special window types
-    if( info.windowType() != NET::Normal
-        && info.windowType() != NET::Override
-        && info.windowType() != NET::Unknown
-        && info.windowType() != NET::Dialog
-        && info.windowType() != NET::Dock )
+    NET::WindowType type = info.windowType( NET::NormalMask | NET::DesktopMask
+        | NET::DockMask | NET::ToolbarMask | NET::MenuMask | NET::DialogMask
+        | NET::OverrideMask | NET::TopMenuMask | NET::UtilityMask | NET::SplashMask );
+    if( type != NET::Normal
+        && type != NET::Override
+        && type != NET::Unknown
+        && type != NET::Dialog
+        && type != NET::Utility )
+//        && type != NET::Dock ) why did I put this here?
 	return NoMatch;
     // lets see if this is a transient
     Window transient_for;
@@ -504,6 +591,7 @@ KStartupInfo::startup_t KStartupInfo::check_startup_internal( WId w_P, KStartupI
         && static_cast< WId >( transient_for ) != qt_xrootwin()
         && transient_for != None )
 	return NoMatch;
+#endif
     // Strategy:
     //
     // Is this a compliant app ?
@@ -520,33 +608,38 @@ KStartupInfo::startup_t KStartupInfo::check_startup_internal( WId w_P, KStartupI
             kdDebug( 172 ) << "ignore" << endl;
             return NoMatch;
             }
-        return find_id( id, id_O, data_O, remove_P ) ? Match : NoMatch;
+        return find_id( id, id_O, data_O ) ? Match : NoMatch;
         }
-    // _NET_WM_PID apps are also considered compliant
+#if defined Q_WS_X11 && ! defined K_WS_QTONLY
     pid_t pid = info.pid();
     if( pid > 0 )
         {
         QCString hostname = get_window_hostname( w_P );
         if( !hostname.isEmpty()
-            && find_pid( pid, hostname, id_O, data_O, remove_P ))
+            && find_pid( pid, hostname, id_O, data_O ))
             return Match;
         // try XClass matching , this PID stuff sucks :(
         }
-    // Hard - this app is not even NET_WM compliant
     XClassHint hint;
     if( XGetClassHint( qt_xdisplay(), w_P, &hint ) != 0 )
         { // We managed to read the class hint
-        if( find_wclass( hint.res_name, hint.res_class, id_O, data_O, remove_P ))
+        QCString res_name = hint.res_name;
+        QCString res_class = hint.res_class;
+        XFree( hint.res_name );
+        XFree( hint.res_class );
+        if( find_wclass( res_name, res_class, id_O, data_O ))
             return Match;
         }
+#endif
     kdDebug( 172 ) << "check_startup:cantdetect" << endl;
     return CantDetect;
     }
 
 bool KStartupInfo::find_id( const QCString& id_P, KStartupInfoId* id_O,
-    KStartupInfoData* data_O, bool remove_P )
+    KStartupInfoData* data_O )
     {
-    if (!d) return false;
+    if( d == NULL )
+        return false;
     kdDebug( 172 ) << "find_id:" << id_P << endl;
     KStartupInfoId id;
     id.initId( id_P );
@@ -556,9 +649,6 @@ bool KStartupInfo::find_id( const QCString& id_P, KStartupInfoId* id_O,
             *id_O = id;
         if( data_O != NULL )
             *data_O = d->startups[ id ];
-        if( remove_P
-	    && d->startups[ id ].silent() != Data::Yes ) // HACK
-            d->startups.remove( id );
         kdDebug( 172 ) << "check_startup_id:match" << endl;
         return true;
         }
@@ -566,9 +656,10 @@ bool KStartupInfo::find_id( const QCString& id_P, KStartupInfoId* id_O,
     }
 
 bool KStartupInfo::find_pid( pid_t pid_P, const QCString& hostname_P,
-    KStartupInfoId* id_O, KStartupInfoData* data_O, bool remove_P )
+    KStartupInfoId* id_O, KStartupInfoData* data_O )
     {
-    if (!d) return false;
+    if( d == NULL )
+        return false;
     kdDebug( 172 ) << "find_pid:" << pid_P << endl;
     for( QMap< KStartupInfoId, Data >::Iterator it = d->startups.begin();
          it != d->startups.end();
@@ -580,9 +671,8 @@ bool KStartupInfo::find_pid( pid_t pid_P, const QCString& hostname_P,
                 *id_O = it.key();
             if( data_O != NULL )
                 *data_O = *it;
-            if( remove_P
-		&& ( *it ).silent() != Data::Yes ) // HACK
-                d->startups.remove( it );
+            // non-compliant, remove on first match
+            remove_startup_info_internal( it.key());
             kdDebug( 172 ) << "check_startup_pid:match" << endl;
             return true;
             }
@@ -591,9 +681,10 @@ bool KStartupInfo::find_pid( pid_t pid_P, const QCString& hostname_P,
     }
 
 bool KStartupInfo::find_wclass( QCString res_name, QCString res_class,
-    KStartupInfoId* id_O, KStartupInfoData* data_O, bool remove_P )
+    KStartupInfoId* id_O, KStartupInfoData* data_O )
     {
-    if (!d) return false;
+    if( d == NULL )
+        return false;
     res_name = res_name.lower();
     res_class = res_class.lower();
     kdDebug( 172 ) << "find_wclass:" << res_name << ":" << res_class << endl;
@@ -608,9 +699,8 @@ bool KStartupInfo::find_wclass( QCString res_name, QCString res_class,
                 *id_O = it.key();
             if( data_O != NULL )
                 *data_O = *it;
-            if( remove_P
-		&& ( *it ).silent() != Data::Yes ) // HACK
-                d->startups.remove( it );
+            // non-compliant, remove on first match
+            remove_startup_info_internal( it.key());
             kdDebug( 172 ) << "check_startup_wclass:match" << endl;
             return true;
             }
@@ -618,53 +708,54 @@ bool KStartupInfo::find_wclass( QCString res_name, QCString res_class,
     return false;
     }
 
-static Atom kde_startup_atom = None;
-static Atom kde_startup_atom_2 = None;
+#if defined Q_WS_X11 && ! defined K_WS_QTONLY
+static Atom net_startup_atom = None;
+#endif
 
 QCString KStartupInfo::windowStartupId( WId w_P )
     {
-    if( kde_startup_atom == None )
-        kde_startup_atom = XInternAtom( qt_xdisplay(), KDE_STARTUP_ID, False );
-    if( kde_startup_atom_2 == None )
-        kde_startup_atom_2 = XInternAtom( qt_xdisplay(), KDE_STARTUP_ID_2, False );
+#if defined Q_WS_X11 && ! defined K_WS_QTONLY
+    if( net_startup_atom == None )
+        net_startup_atom = XInternAtom( qt_xdisplay(), NET_STARTUP_WINDOW, False );
+    if( utf8_string_atom == None )
+        utf8_string_atom = XInternAtom( qt_xdisplay(), "UTF8_STRING", False );
     unsigned char *name_ret;
     QCString ret;
     Atom type_ret;
     int format_ret;
     unsigned long nitems_ret = 0, after_ret = 0;
-    if( XGetWindowProperty( qt_xdisplay(), w_P, kde_startup_atom, 0l, (long) BUFSIZE,
-            False, XA_STRING, &type_ret, &format_ret, &nitems_ret, &after_ret, &name_ret )
+    if( XGetWindowProperty( qt_xdisplay(), w_P, net_startup_atom, 0l, 4096,
+            False, utf8_string_atom, &type_ret, &format_ret, &nitems_ret, &after_ret, &name_ret )
 	    == Success )
         {
-	if( type_ret == XA_STRING && format_ret == 8 && name_ret != NULL )
-	    ret = reinterpret_cast< char* >( name_ret );
-        if ( name_ret != NULL )
-            XFree( name_ret );
-        }
-    if( ret.isNull() && XGetWindowProperty( qt_xdisplay(), w_P, kde_startup_atom_2, 0l, (long) BUFSIZE,
-            False, XA_STRING, &type_ret, &format_ret, &nitems_ret, &after_ret, &name_ret )
-	    == Success )
-        {
-	if( type_ret == XA_STRING && format_ret == 8 && name_ret != NULL )
-	    ret = reinterpret_cast< char* >( name_ret );
+	if( type_ret == utf8_string_atom && format_ret == 8 && name_ret != NULL )
+  	    ret = reinterpret_cast< char* >( name_ret );
         if ( name_ret != NULL )
             XFree( name_ret );
         }
     return ret;
+#else
+    return QCString();
+#endif
     }
 
 void KStartupInfo::setWindowStartupId( WId w_P, const QCString& id_P )
     {
+#if defined Q_WS_X11 && ! defined K_WS_QTONLY
     if( id_P.isNull())
         return;
-    if( kde_startup_atom == None )
-        kde_startup_atom = XInternAtom( qt_xdisplay(), KDE_STARTUP_ID, False );
-    XChangeProperty( qt_xdisplay(), w_P, kde_startup_atom, XA_STRING, 8,
+    if( net_startup_atom == None )
+        net_startup_atom = XInternAtom( qt_xdisplay(), NET_STARTUP_WINDOW, False );
+    if( utf8_string_atom == None )
+        utf8_string_atom = XInternAtom( qt_xdisplay(), "UTF8_STRING", False );
+    XChangeProperty( qt_xdisplay(), w_P, net_startup_atom, utf8_string_atom, 8,
         PropModeReplace, reinterpret_cast< unsigned char* >( id_P.data()), id_P.length());
+#endif
     }
 
 QCString KStartupInfo::get_window_hostname( WId w_P )
     {
+#if defined Q_WS_X11 && ! defined K_WS_QTONLY
     XTextProperty tp;
     char** hh;
     int cnt;
@@ -679,6 +770,7 @@ QCString KStartupInfo::get_window_hostname( WId w_P )
             }
         XFreeStringList( hh );
         }
+#endif
     // no hostname
     return QCString();
     }
@@ -697,7 +789,8 @@ void KStartupInfo::startups_cleanup_no_age()
 
 void KStartupInfo::startups_cleanup()
     {
-    if (!d) return;
+    if( d == NULL )
+        return;
     if( d->startups.count() == 0 && d->silent_startups.count() == 0 )
         {
         d->cleanup->stop();
@@ -708,7 +801,8 @@ void KStartupInfo::startups_cleanup()
 
 void KStartupInfo::startups_cleanup_internal( bool age_P )
     {
-    if (!d) return;
+    if( d == NULL )
+        return;
     for( QMap< KStartupInfoId, Data >::Iterator it = d->startups.begin();
          it != d->startups.end();
          )
@@ -716,7 +810,7 @@ void KStartupInfo::startups_cleanup_internal( bool age_P )
         if( age_P )
             ( *it ).age++;
 	int tout = timeout;
-	if( ( *it ).silent() == Data::Yes ) // TODO 
+	if( ( *it ).silent() == Data::Yes ) // TODO
 	    tout *= 20;
         if( ( *it ).age >= timeout )
             {
@@ -735,7 +829,7 @@ void KStartupInfo::startups_cleanup_internal( bool age_P )
         if( age_P )
             ( *it ).age++;
 	int tout = timeout;
-	if( ( *it ).silent() == Data::Yes ) // TODO 
+	if( ( *it ).silent() == Data::Yes ) // TODO
 	    tout *= 20;
         if( ( *it ).age >= timeout )
             {
@@ -751,7 +845,8 @@ void KStartupInfo::startups_cleanup_internal( bool age_P )
 
 void KStartupInfo::clean_all_noncompliant()
     {
-    if (!d) return;
+    if( d == NULL )
+        return;
     for( QMap< KStartupInfoId, Data >::Iterator it = d->startups.begin();
          it != d->startups.end();
          )
@@ -809,7 +904,7 @@ void KStartupInfoId::initId( const QCString& id_P )
 #endif
         return;
         }
-    const char* startup_env = getenv( KDE_STARTUP_ENV );
+    const char* startup_env = getenv( NET_STARTUP_ENV );
     if( startup_env != NULL && *startup_env != '\0' )
         { // already has id
         d->id = startup_env;
@@ -825,9 +920,10 @@ void KStartupInfoId::initId( const QCString& id_P )
     gettimeofday( &tm, NULL );
     char hostname[ 256 ];
     hostname[ 0 ] = '\0';
-    gethostname( hostname, 255 );
+    if (!gethostname( hostname, 255 ))
+	hostname[sizeof(hostname)-1] = '\0';
     d->id = QString( "%1;%2;%3;%4" ).arg( hostname ).arg( tm.tv_sec )
-        .arg( tm.tv_usec ).arg( getpid()).latin1();
+        .arg( tm.tv_usec ).arg( getpid()).utf8();
 #ifdef KSTARTUPINFO_ALL_DEBUG
     kdDebug( 172 ) << "creating: " << d->id << endl;
 #endif
@@ -837,15 +933,15 @@ bool KStartupInfoId::setupStartupEnv() const
     {
     if( id().isEmpty())
         {
-        unsetenv( KDE_STARTUP_ENV );
+        unsetenv( NET_STARTUP_ENV );
         return false;
         }
-    return setenv( KDE_STARTUP_ENV, id(), true ) == 0;
+    return setenv( NET_STARTUP_ENV, id(), true ) == 0;
     }
 
 KStartupInfoId KStartupInfo::currentStartupIdEnv()
     {
-    const char* startup_env = getenv( KDE_STARTUP_ENV );
+    const char* startup_env = getenv( NET_STARTUP_ENV );
     KStartupInfoId id;
     if( startup_env != NULL && *startup_env != '\0' )
         id.d->id = startup_env;
@@ -856,7 +952,7 @@ KStartupInfoId KStartupInfo::currentStartupIdEnv()
 
 void KStartupInfo::resetStartupEnv()
     {
-    unsetenv( KDE_STARTUP_ENV );
+    unsetenv( NET_STARTUP_ENV );
     }
 
 KStartupInfoId::KStartupInfoId()
@@ -907,15 +1003,18 @@ bool KStartupInfoId::none() const
 struct KStartupInfoDataPrivate
     {
     KStartupInfoDataPrivate() : desktop( 0 ), wmclass( "" ), hostname( "" ),
-	silent( KStartupInfoData::Unknown ) {};
+	silent( KStartupInfoData::Unknown ), timestamp( -1U ), screen( -1 ) {};
     QString bin;
     QString name;
+    QString description;
     QString icon;
     int desktop;
     QValueList< pid_t > pids;
     QCString wmclass;
     QCString hostname;
     KStartupInfoData::TriState silent;
+    unsigned long timestamp;
+    int screen;
     };
 
 QString KStartupInfoData::to_text() const
@@ -925,10 +1024,13 @@ QString KStartupInfoData::to_text() const
         ret += QString::fromLatin1( " BIN=\"%1\"" ).arg( escape_str( d->bin ));
     if( !d->name.isEmpty())
         ret += QString::fromLatin1( " NAME=\"%1\"" ).arg( escape_str( d->name ));
+    if( !d->description.isEmpty())
+        ret += QString::fromLatin1( " DESCRIPTION=\"%1\"" ).arg( escape_str( d->description ));
     if( !d->icon.isEmpty())
         ret += QString::fromLatin1( " ICON=%1" ).arg( d->icon );
     if( d->desktop != 0 )
-        ret += QString::fromLatin1( " DESKTOP=%1" ).arg( d->desktop );
+        ret += QString::fromLatin1( " DESKTOP=%1" )
+            .arg( d->desktop == NET::OnAllDesktops ? NET::OnAllDesktops : d->desktop - 1 ); // spec counts from 0
     if( !d->wmclass.isEmpty())
         ret += QString::fromLatin1( " WMCLASS=%1" ).arg( d->wmclass );
     if( !d->hostname.isEmpty())
@@ -939,6 +1041,10 @@ QString KStartupInfoData::to_text() const
         ret += QString::fromLatin1( " PID=%1" ).arg( *it );
     if( d->silent != Unknown )
 	ret += QString::fromLatin1( " SILENT=%1" ).arg( d->silent == Yes ? 1 : 0 );
+    if( d->timestamp != -1U )
+        ret += QString::fromLatin1( " TIMESTAMP=%1" ).arg( d->timestamp );
+    if( d->screen != -1 )
+        ret += QString::fromLatin1( " SCREEN=%1" ).arg( d->screen );
     return ret;
     }
 
@@ -948,12 +1054,15 @@ KStartupInfoData::KStartupInfoData( const QString& txt_P )
     QStringList items = get_fields( txt_P );
     const QString bin_str = QString::fromLatin1( "BIN=" );
     const QString name_str = QString::fromLatin1( "NAME=" );
+    const QString description_str = QString::fromLatin1( "DESCRIPTION=" );
     const QString icon_str = QString::fromLatin1( "ICON=" );
     const QString desktop_str = QString::fromLatin1( "DESKTOP=" );
     const QString wmclass_str = QString::fromLatin1( "WMCLASS=" );
-    const QString hostname_str = QString::fromLatin1( "HOSTNAME=" );
-    const QString pid_str = QString::fromLatin1( "PID=" );
+    const QString hostname_str = QString::fromLatin1( "HOSTNAME=" ); // SELI nonstd
+    const QString pid_str = QString::fromLatin1( "PID=" );  // SELI nonstd
     const QString silent_str = QString::fromLatin1( "SILENT=" );
+    const QString timestamp_str = QString::fromLatin1( "TIMESTAMP=" );
+    const QString screen_str = QString::fromLatin1( "SCREEN=" );
     for( QStringList::Iterator it = items.begin();
          it != items.end();
          ++it )
@@ -962,10 +1071,16 @@ KStartupInfoData::KStartupInfoData( const QString& txt_P )
             d->bin = get_str( *it );
         else if( ( *it ).startsWith( name_str ))
             d->name = get_str( *it );
+        else if( ( *it ).startsWith( description_str ))
+            d->description = get_str( *it );
         else if( ( *it ).startsWith( icon_str ))
             d->icon = get_str( *it );
         else if( ( *it ).startsWith( desktop_str ))
+            {
             d->desktop = get_num( *it );
+            if( d->desktop != NET::OnAllDesktops )
+                ++d->desktop; // spec counts from 0
+            }
         else if( ( *it ).startsWith( wmclass_str ))
             d->wmclass = get_cstr( *it );
         else if( ( *it ).startsWith( hostname_str ))
@@ -974,6 +1089,10 @@ KStartupInfoData::KStartupInfoData( const QString& txt_P )
             addPid( get_num( *it ));
         else if( ( *it ).startsWith( silent_str ))
             d->silent = get_num( *it ) != 0 ? Yes : No;
+        else if( ( *it ).startsWith( timestamp_str ))
+            d->timestamp = get_unum( *it );
+        else if( ( *it ).startsWith( screen_str ))
+            d->screen = get_num( *it );
         }
     }
 
@@ -997,6 +1116,8 @@ void KStartupInfoData::update( const KStartupInfoData& data_P )
         d->bin = data_P.bin();
     if( !data_P.name().isEmpty() && name().isEmpty()) // don't overwrite
         d->name = data_P.name();
+    if( !data_P.description().isEmpty() && description().isEmpty()) // don't overwrite
+        d->description = data_P.description();
     if( !data_P.icon().isEmpty() && icon().isEmpty()) // don't overwrite
         d->icon = data_P.icon();
     if( data_P.desktop() != 0 && desktop() == 0 ) // don't overwrite
@@ -1011,6 +1132,10 @@ void KStartupInfoData::update( const KStartupInfoData& data_P )
         addPid( *it );
     if( data_P.silent() != Unknown )
 	d->silent = data_P.silent();
+    if( data_P.timestamp() != -1U && timestamp() == -1U ) // don't overwrite
+        d->timestamp = data_P.timestamp();
+    if( data_P.screen() != -1 )
+        d->screen = data_P.screen();
     }
 
 KStartupInfoData::KStartupInfoData()
@@ -1038,6 +1163,11 @@ void KStartupInfoData::setName( const QString& name_P )
     d->name = name_P;
     }
 
+const QString& KStartupInfoData::name() const
+    {
+    return d->name;
+    }
+
 const QString& KStartupInfoData::findName() const
     {
     if( !name().isEmpty())
@@ -1045,9 +1175,21 @@ const QString& KStartupInfoData::findName() const
     return bin();
     }
 
-const QString& KStartupInfoData::name() const
+void KStartupInfoData::setDescription( const QString& desc_P )
     {
-    return d->name;
+    d->description = desc_P;
+    }
+
+const QString& KStartupInfoData::description() const
+    {
+    return d->description;
+    }
+
+const QString& KStartupInfoData::findDescription() const
+    {
+    if( !description().isEmpty())
+        return description();
+    return name();
     }
 
 void KStartupInfoData::setIcon( const QString& icon_P )
@@ -1086,7 +1228,7 @@ const QCString KStartupInfoData::findWMClass() const
     {
     if( !WMClass().isEmpty() && WMClass() != "0" )
         return WMClass();
-    return bin().latin1(); // CHECKME
+    return bin().utf8();
     }
 
 const QCString& KStartupInfoData::WMClass() const
@@ -1102,7 +1244,8 @@ void KStartupInfoData::setHostname( const QCString& hostname_P )
         {
         char tmp[ 256 ];
         tmp[ 0 ] = '\0';
-        gethostname( tmp, 255 );
+        if (!gethostname( tmp, 255 ))
+	    tmp[sizeof(tmp)-1] = '\0';
         d->hostname = tmp;
         }
     }
@@ -1137,17 +1280,44 @@ void KStartupInfoData::setSilent( TriState state_P )
     {
     d->silent = state_P;
     }
-    
+
 KStartupInfoData::TriState KStartupInfoData::silent() const
     {
     return d->silent;
     }
 
+void KStartupInfoData::setTimestamp( unsigned long time )
+    {
+    d->timestamp = time;
+    }
+
+unsigned long KStartupInfoData::timestamp() const
+    {
+    return d->timestamp;
+    }
+
+void KStartupInfoData::setScreen( int screen )
+    {
+    d->screen = screen;
+    }
+
+int KStartupInfoData::screen() const
+    {
+    return d->screen;
+    }
+
 static
-int get_num( const QString& item_P )
+long get_num( const QString& item_P )
     {
     unsigned int pos = item_P.find( '=' );
-    return item_P.mid( pos + 1 ).toInt();
+    return item_P.mid( pos + 1 ).toLong();
+    }
+
+static
+unsigned long get_unum( const QString& item_P )
+    {
+    unsigned int pos = item_P.find( '=' );
+    return item_P.mid( pos + 1 ).toULong();
     }
 
 static
@@ -1167,7 +1337,7 @@ QString get_str( const QString& item_P )
 static
 QCString get_cstr( const QString& item_P )
     {
-    return get_str( item_P ).latin1(); // CHECKME
+    return get_str( item_P ).utf8();
     }
 
 static
@@ -1218,7 +1388,5 @@ static QString escape_str( const QString& str_P )
     return ret;
     }
 
-#undef None
-
 #include "kstartupinfo.moc"
 #endif
