@@ -1,8 +1,8 @@
 /****************************************************************************
 
- $Id: kstartupinfo.h,v 1.15.2.4 2003/02/25 08:05:13 lunakl Exp $
+ $Id: kstartupinfo.h,v 1.28 2003/10/23 17:40:25 lunakl Exp $
 
- Copyright (C) 2001 Lubos Lunak        <l.lunak@kde.org>
+ Copyright (C) 2001-2003 Lubos Lunak        <l.lunak@kde.org>
 
 Permission is hereby granted, free of charge, to any person obtaining a
 copy of this software and associated documentation files (the "Software"),
@@ -29,7 +29,8 @@ DEALINGS IN THE SOFTWARE.
 
 #include <sys/types.h>
 #include <qobject.h>
-#ifdef Q_WS_X11 // FIXME(E): Redo in a less X11-specific way
+
+//#ifdef Q_WS_X11 // FIXME(E): Redo in a less X11-specific way
 #include <qcstring.h>
 #include <qstring.h>
 #include <qvaluelist.h>
@@ -48,7 +49,7 @@ class KStartupInfoPrivate;
  *
  * You usually don't need to use this class for sending the notification
  * information, as KDE libraries should do this when an application is
- * started ( e.g. KRun class ).
+ * started (e.g. KRun class).
  *
  * For receiving the startup notification info, create an instance and connect
  * to its slots. It will automatically detect started applications and when
@@ -58,19 +59,66 @@ class KStartupInfoPrivate;
  * @see KStartupInfoData
  *
  * @author Lubos Lunak <l.lunak@kde.org>
- * @version $Id: kstartupinfo.h,v 1.15.2.4 2003/02/25 08:05:13 lunakl Exp $
+ * @version $Id: kstartupinfo.h,v 1.28 2003/10/23 17:40:25 lunakl Exp $
  */
 class KStartupInfo
     : public QObject
     {
     Q_OBJECT
     public:
+        /**
+         * By default, the startup notification is ended for the application
+         * after it shows its first toplevel window. If you app e.g. has
+         * several toplevel windows after its startup, you can disable
+         * the automatic handling, and call appStarted() manually after
+         * all toplevel windows have been shown.
+         * @since 3.2
+         */
+        static void disableAutoAppStartedSending( bool disable = true );
+
+        /**
+         * Manual notification that the application has started.
+         * If you do not map a (toplevel) window, then startup
+         * notification will not disappear for the application
+         * until a timeout. You can use this as an alternative
+         * method in this case.
+         */
+        static void appStarted();
+
+        /**
+         * Sends explicit notification that the startup notification
+         * with id startup_id should end.
+         * @since 3.2
+         */ 
+        static void appStarted( const QCString& startup_id );
+        
+        /**
+         * Use this function if the application got a request with startup
+         * notification from outside (for example, when KUniqueApplication::newInstance()
+         * is called, or e.g. when khelpcenter opens new URL in its window).
+         * The window can be either an already existing and visible window,
+         * or a new one, before being shown. Note that this function is usually
+         * needed only when a window is reused.
+         * @since 3.2
+         */
+        static void setNewStartupId( QWidget* window, const QCString& startup_id );
+
+        /**
+         * If your application shows temporarily some window during its startup,
+         * for example a dialog, and only after closing it shows the main window,
+         * startup notification would normally be shown while the dialog is visible.
+         * To temporarily suspend and resume the notification, use this function.
+         * Note that this is cumulative, i.e. after suspending twice, you have to
+         * resume twice.
+         * @since 3.2
+         */
+        static void silenceStartup( bool silence );
+
 	enum {
 	    CleanOnCantDetect		= 1 << 0,
 	    DisableKWinModule		= 1 << 1,
 	    AnnounceSilenceChanges	= 1 << 2
 	    };
-	
 	/**
 	 * Creates an instance that will receive the startup notifications.
 	 * The various flags passed may be
@@ -79,7 +127,7 @@ class KStartupInfo
 	 *     the startup protocol are removed
 	 * @li DisableKWinModule - KWinModule, which is normally used to detect
 	 *     new windows, is disabled. With this flag, checkStartup() must be
-	 *     called in order to check newly mapped windows
+	 *     called in order to check newly mapped windows.
 	 * @li AnnounceSilenceChanges - normally, startup notifications are
 	 *     "removed" when they're silenced, and "recreated" when they're resumed.
 	 *     With this flag, the change is normally announced with gotStartupChange().
@@ -106,7 +154,8 @@ class KStartupInfo
 	/**
 	 * Sends given notification data about started application
 	 * with the given startup identification. If no notification for this identification
-	 * exists yet, it is created, otherwise it's updated.
+	 * exists yet, it is created, otherwise it's updated. Note that the name field
+         * in data is required.
 	 * 
 	 * @param id the id of the application
 	 * @param data the application's data
@@ -117,8 +166,9 @@ class KStartupInfo
         static bool sendStartup( const KStartupInfoId& id, const KStartupInfoData& data );
 
 	/**
-	 * Like @ref sendStartup , uses dpy instead of qt_x11display() for sending the info.
-	 * @param dpy the display of the application.
+	 * Like sendStartup , uses dpy instead of qt_x11display() for sending the info.
+	 * @param dpy the display of the application. Note that the name field
+         * in data is required.
 	 * @param id the id of the application
 	 * @param data the application's data
 	 * @return true if successful, false otherwise
@@ -139,7 +189,7 @@ class KStartupInfo
         static bool sendChange( const KStartupInfoId& id, const KStartupInfoData& data );
 
 	/**
-	 * Like @ref sendChange , uses dpy instead of qt_x11display() for sending the info.
+	 * Like sendChange , uses dpy instead of qt_x11display() for sending the info.
 	 * @param dpy the display of the application.
 	 * @param id the id of the application
 	 * @param data the application's data
@@ -156,7 +206,7 @@ class KStartupInfo
         static bool sendFinish( const KStartupInfoId& id );
 
 	/**
-	 * Like @ref sendFinish , uses dpy instead of qt_x11display() for sending the info.
+	 * Like sendFinish , uses dpy instead of qt_x11display() for sending the info.
 	 * @param dpy the display of the application.
 	 * @param id the id of the application
 	 * @return true if successful, false otherwise
@@ -173,7 +223,7 @@ class KStartupInfo
         static bool sendFinish( const KStartupInfoId& id, const KStartupInfoData& data );
 
 	/**
-	 * Like @ref sendFinish , uses dpy instead of qt_x11display() for sending the info.
+	 * Like sendFinish , uses dpy instead of qt_x11display() for sending the info.
 	 * @param dpy the display of the application.
 	 * @param id the id of the application
 	 * @param data the application's data
@@ -185,7 +235,7 @@ class KStartupInfo
 	/**
 	 * Returns the current startup notification identification for the current
 	 * startup notification environment variable. Note that KApplication constructor
-	 * unsets the variable and you have to use @ref KApplication::startupId .
+	 * unsets the variable and you have to use KApplication::startupId .
 	 * @return the current startup notification identification
 	 */
         static KStartupInfoId currentStartupIdEnv();
@@ -193,33 +243,22 @@ class KStartupInfo
 	 * Unsets the startup notification environment variable.
 	 */
         static void resetStartupEnv();
-        /**
-         * Manual notification that the application has started.
-         * If you do not map a (toplevel) window, then startup
-         * notification will not disappear for the application
-         * until a timeout. You can use this as an alternative
-         * method in this case.
-         */
-        static void appStarted();
 	/**
-	 * @li NoMatch    - the window doesn't match any existing startup notification</li>
-	 * @li Match      - the window matches an existing startup notification</li>
+	 * @li NoMatch    - the window doesn't match any existing startup notification
+	 * @li Match      - the window matches an existing startup notification
 	 * @li CantDetect - unable to detect if the window matches any existing
-	 *			startup notification</li>
+	 *			startup notification
 	 */
         enum startup_t { NoMatch, Match, CantDetect };
 	/**
-	 * Checks if the given windows matches any existing startup notification. If yes,
-	 * the startup notification is removed. Prevents emitting gotRemoveStartup()
-         * for the window if the window matches.
+	 * Checks if the given windows matches any existing startup notification.
 	 * @param w the window id to check
 	 * @return the result of the operation
 	 */
         startup_t checkStartup( WId w );
 	/**
 	 * Checks if the given windows matches any existing startup notification, and
-	 * if yes, returns the identification in id, and removes the startup notification.
-         * Prevents emitting gotRemoveStartup() for the window if the window matches.
+	 * if yes, returns the identification in id.
 	 * @param w the window id to check
 	 * @param id if found, the id of the startup notification will be written here
 	 * @return the result of the operation
@@ -227,9 +266,7 @@ class KStartupInfo
         startup_t checkStartup( WId w, KStartupInfoId& id );
 	/**
 	 * Checks if the given windows matches any existing startup notification, and
-	 * if yes, returns the notification data in data, and removes the startup
-	 * notification. Prevents emitting gotRemoveStartup() for the window
-         * if the window matches.
+	 * if yes, returns the notification data in data.
 	 * @param w the window id to check
 	 * @param data if found, the data of the startup notification will be written here
 	 * @return the result of the operation
@@ -237,9 +274,7 @@ class KStartupInfo
         startup_t checkStartup( WId w, KStartupInfoData& data );
 	/**
 	 * Checks if the given windows matches any existing startup notification, and
-	 * if yes, returns the identification in id and notification data in data,
-	 * and removes the startup notification. Prevents emitting gotRemoveStartup()
-         * for the window if the window matches.
+	 * if yes, returns the identification in id and notification data in data.
 	 * @param w the window id to check
 	 * @param id if found, the id of the startup notification will be written here
 	 * @param data if found, the data of the startup notification will be written here
@@ -253,24 +288,28 @@ class KStartupInfo
         void setTimeout( unsigned int secs );
 	/**
 	 * Sets the startup notification window property on the given window.
-	 * @param windows the id of the window
+	 * @param window the id of the window
 	 * @param id the startup notification id
 	 */
         static void setWindowStartupId( WId window, const QCString& id );
 	/**
 	 * Returns startup notification identification of the given window.
-	 * @param windows the id of the window
+	 * @param w the id of the window
 	 * @return the startup notification id. Can be null if not found.
 	 */
         static QCString windowStartupId( WId w );
+        /**
+         * @internal
+         */
+        static void handleAutoAppStartedSending();
 	/**
 	 * @internal
 	 */
         class Data;
     signals:
 	/**
-	 * Emitted when a new startup notification is created ( i.e. a new application is
-	 * being started ).
+	 * Emitted when a new startup notification is created (i.e. a new application is
+	 * being started).
 	 * @param id the notification identification
 	 * @param data the notification data
 	 */
@@ -282,8 +321,8 @@ class KStartupInfo
 	 */
         void gotStartupChange( const KStartupInfoId& id, const KStartupInfoData& data );
 	/**
-	 * Emitted when a startup notification is removed ( either because it was detected
-	 * that the application is ready or because of a timeout ).
+	 * Emitted when a startup notification is removed (either because it was detected
+	 * that the application is ready or because of a timeout).
 	 * @param id the notification identification
 	 * @param data the notification data
 	 */
@@ -306,18 +345,18 @@ class KStartupInfo
         void remove_startup_info_internal( const KStartupInfoId& id_P );
         void remove_startup_pids( const KStartupInfoId& id, const KStartupInfoData& data );
         void remove_startup_pids( const KStartupInfoData& data );
-        startup_t check_startup_internal( WId w, KStartupInfoId* id, KStartupInfoData* data,
-            bool remove );
+        startup_t check_startup_internal( WId w, KStartupInfoId* id, KStartupInfoData* data );
         bool find_id( const QCString& id_P, KStartupInfoId* id_O,
-            KStartupInfoData* data_O, bool remove );
+            KStartupInfoData* data_O );
         bool find_pid( pid_t pid_P, const QCString& hostname, KStartupInfoId* id_O,
-            KStartupInfoData* data_O, bool remove );
+            KStartupInfoData* data_O );
         bool find_wclass( QCString res_name_P, QCString res_class_P,
-            KStartupInfoId* id_O, KStartupInfoData* data_O, bool remove );
-        static QCString get_window_startup_id( WId w_P );
+            KStartupInfoId* id_O, KStartupInfoData* data_O );
         static QCString get_window_hostname( WId w_P );
         void startups_cleanup_internal( bool age_P );
         void clean_all_noncompliant();
+        static QString check_required_startup_fields( const QString& msg,
+            const KStartupInfoData& data, int screen );
         bool clean_on_cantdetect_; // KDE4 remove unused
         unsigned int timeout;
         KStartupInfoPrivate* d;
@@ -335,7 +374,7 @@ class KStartupInfoIdPrivate;
  * @see KStartupInfoData
  *
  * @author Lubos Lunak <l.lunak@kde.org>
- * @version $Id: kstartupinfo.h,v 1.15.2.4 2003/02/25 08:05:13 lunakl Exp $
+ * @version $Id: kstartupinfo.h,v 1.28 2003/10/23 17:40:25 lunakl Exp $
  */
 class KStartupInfoId
     {
@@ -403,7 +442,7 @@ class KStartupInfoDataPrivate;
  * @see KStartupInfoId
  *
  * @author Lubos Lunak <l.lunak@kde.org>
- * @version $Id: kstartupinfo.h,v 1.15.2.4 2003/02/25 08:05:13 lunakl Exp $
+ * @version $Id: kstartupinfo.h,v 1.28 2003/10/23 17:40:25 lunakl Exp $
  */
 class KStartupInfoData
     {
@@ -419,22 +458,42 @@ class KStartupInfoData
 	 */
         const QString& bin() const;
 	/**
-	 * Sets the name for the notification ( e.g. 'Control Center' )
+	 * Sets the name for the notification (e.g. 'Control Center')
 	 */
         void setName( const QString& name );
 	/**
-	 * Returns the name of the startup notification, or if it's not available,
-	 * the binary name.
-	 * @return the name of the startup notification, or the binary name
-	 *         if not available
+	 * Returns the name of the startup notification. If it's not available,
+         * it tries to use other information (binary name).
+	 * @return the name of the startup notification
 	 */
         const QString& findName() const;
 	/**
 	 * Returns the name of the startup notification, or empty if not available.
-	 * @return the name of the startup notificaiton, or an empty string
+	 * @return the name of the startup notification, or an empty string
 	 *         if not set.
 	 */
         const QString& name() const;
+	/**
+	 * Sets the description for the notification (e.g. 'Launching Control Center').
+         * I.e. name() describes what is being started, while description() is
+         * the actual action performed by the starting.
+         * @since 3.2
+	 */
+        void setDescription( const QString& descr );
+	/**
+	 * Returns the description of the startup notification. If it's not available,
+         * it returns name().
+	 * @return the description of the startup notification
+         * @since 3.2
+	 */
+        const QString& findDescription() const;
+	/**
+	 * Returns the name of the startup notification, or empty if not available.
+	 * @return the name of the startup notificaiton, or an empty string
+	 *         if not set.
+         * @since 3.2
+	 */
+        const QString& description() const;
 	/**
 	 * Sets the icon for the startup notification ( e.g. 'kcontrol' )
 	 * @param icon the name of the icon
@@ -529,6 +588,32 @@ class KStartupInfoData
 	 * @since 3.1.1
 	 */
 	TriState silent() const;
+        
+        /**
+         * Sets timestamp for the startup notification. The timestamp is expressed
+         * as XServer time, and is used to prevent activation of the matching
+         * window if user interaction took place after this timestamp.
+         * Value -1 means no timestamp set, value 0 means that the window should
+         * not be activated.
+         */
+        void setTimestamp( unsigned long time );
+        
+        /**
+         * Return the timestamp for the startup notification, or -1 if no timestamp
+         * is set.
+         */
+        unsigned long timestamp() const;
+        
+        /**
+         * The X11 screen on which the startup notification is happening, -1 if unknown.
+         */
+        int screen() const;
+        
+        /**
+         * Sets the X11 screen on which the startup notification should happen.
+         * This is usually not necessary to set, as it's set by default to qt_xscreen().
+         */
+        void setScreen( int screen );
 
 	/**
 	 * Updates the notification data from the given data. Some data, such as the desktop
@@ -536,6 +621,7 @@ class KStartupInfoData
 	 * @param data the data to update
 	 */
         void update( const KStartupInfoData& data );
+
 	/**
 	 * Constructor. Initializes all the data to their default empty values.
 	 */
@@ -557,4 +643,3 @@ class KStartupInfoData
     };
 
 #endif
-#endif
