@@ -17,11 +17,12 @@
     the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
     Boston, MA 02111-1307, USA.
 */
+#define __KSTDACCEL_CPP_ 1
 
 #include "kstdaccel.h"
 
-#include <kaccelaction.h>
-#include <kaccelbase.h>
+#include "kaccelaction.h"
+#include "kaccelbase.h"
 #include <kconfig.h>
 #include <kdebug.h>
 #include <kglobal.h>
@@ -80,6 +81,9 @@ static KStdAccelInfo g_infoStdAccel[] =
 	{ Reload,              I18N_NOOP("Reload"), 0,   Qt::Key_F5, 0, 0, 0, KShortcut(), false },
 	{ PopupMenuContext,    "PopupMenuContext", I18N_NOOP("Popup Menu Context"), Qt::Key_Menu, 0, 0, 0, KShortcut(), false },
 	{ ShowMenubar,         "ShowMenubar", I18N_NOOP("Show Menu Bar"), Qt::CTRL+Qt::Key_M, 0, 0, 0, KShortcut(), false },
+	{ TabNext,             I18N_NOOP("Activate Next Tab"), 0,  Qt::CTRL+Qt::Key_Period, 0, Qt::CTRL+Qt::Key_BracketRight, 0, KShortcut(), false },
+	{ TabPrev,             I18N_NOOP("Activate Previous Tab"), 0,   Qt::CTRL+Qt::Key_Comma, 0, Qt::CTRL+Qt::Key_BracketLeft, 0, KShortcut(), false },
+	{ FullScreen,          "FullScreen", I18N_NOOP("Full Screen Mode"), Qt::CTRL+Qt::SHIFT+Qt::Key_F, 0, 0, 0, KShortcut(), false },
 	{AccelNone,            "Group:Help", I18N_NOOP("Help"), 0, 0, 0, 0, KShortcut(), false },
 	{ Help,                I18N_NOOP("Help"), 0,     Qt::Key_F1, 0, 0, 0, KShortcut(), false },
 	{ WhatsThis,           "WhatsThis", I18N_NOOP("What's This"), Qt::SHIFT+Qt::Key_F1, 0, 0, 0, KShortcut(), false },
@@ -196,7 +200,7 @@ KShortcut shortcutDefault3( StdAccel id )
 		// FIXME: if there is no cutDefault, then this we be made the primary
 		//  instead of alternate shortcut.
 		if( pInfo->cutDefault3B )
-			cut.append( QKeySequence(pInfo->cutDefault3B) );
+			cut.append( KKey(pInfo->cutDefault3B) );
 	}
 
 	return cut;
@@ -268,6 +272,9 @@ const KShortcut& prior()                 { return shortcut( Prior ); }
 const KShortcut& next()                  { return shortcut( Next ); }
 const KShortcut& gotoLine()              { return shortcut( GotoLine ); }
 const KShortcut& addBookmark()           { return shortcut( AddBookmark ); }
+const KShortcut& tabNext()               { return shortcut( TabNext ); }
+const KShortcut& tabPrev()               { return shortcut( TabPrev ); }
+const KShortcut& fullScreen()            { return shortcut( FullScreen ); }
 const KShortcut& zoomIn()                { return shortcut( ZoomIn ); }
 const KShortcut& zoomOut()               { return shortcut( ZoomOut ); }
 const KShortcut& help()                  { return shortcut( Help ); }
@@ -346,7 +353,6 @@ bool ShortcutList::save() const
 	return writeSettings( QString::null, 0, false, true );
 }
 
-#ifndef KDE_NO_COMPAT
 QString action(StdAccel id)
 	{ return name(id); }
 QString description(StdAccel id)
@@ -361,6 +367,7 @@ bool isEqual(const QKeyEvent* ev, int keyQt)
 	KKey key1( ev ), key2( keyQt );
 	return key1 == key2;
 }
-#endif // !KDE_NO_COMPAT
 
 }
+
+#undef __KSTDACCEL_CPP_
