@@ -22,6 +22,7 @@
 #include "config.h"
 
 #include "kstringhandler.h"
+#include "kglobal.h"
 
 QString KStringHandler::word( const QString &text , uint pos )
 {
@@ -397,6 +398,105 @@ QString KStringHandler::rsqueeze( const QString & str, uint maxlen )
   else return str;
 }
 
+QString KStringHandler::lEmSqueeze(const QString &name, const QFontMetrics& fontMetrics, uint maxlen)
+{
+  return lPixelSqueeze(name, fontMetrics, fontMetrics.maxWidth() * maxlen);
+}
+
+static inline int emSqueezeLimit(int delta, int min, int max)
+{
+  if (delta < min) return min;
+  if (delta > max) return max;
+  return delta;
+}
+
+QString KStringHandler::lPixelSqueeze(const QString& name, const QFontMetrics& fontMetrics, uint maxPixels)
+{
+  uint nameWidth = fontMetrics.width(name);
+
+  if (maxPixels < nameWidth)
+  {
+    QString tmp = name;
+    const uint em = fontMetrics.maxWidth();
+    maxPixels -= fontMetrics.width("...");
+
+    while (maxPixels < nameWidth && !tmp.isEmpty())
+    {
+      int delta = (nameWidth - maxPixels) / em;
+      delta = emSqueezeLimit(delta, 1, delta); // no max
+
+      tmp.remove(0, delta);
+      nameWidth = fontMetrics.width(tmp);
+    }
+
+    return ("..." + tmp);
+  }
+
+  return name;
+}
+
+QString KStringHandler::cEmSqueeze(const QString& name, const QFontMetrics& fontMetrics, uint maxlen)
+{
+  return cPixelSqueeze(name, fontMetrics, fontMetrics.maxWidth() * maxlen);
+}
+
+QString KStringHandler::cPixelSqueeze(const QString& name, const QFontMetrics& fontMetrics, uint maxPixels)
+{
+  uint nameWidth = fontMetrics.width(name);
+
+  if (maxPixels < nameWidth)
+  {
+    QString tmp = name;
+    const uint em = fontMetrics.maxWidth();
+    maxPixels -= fontMetrics.width("...");
+
+    while (maxPixels < nameWidth && !tmp.isEmpty())
+    {
+      int length = tmp.length();
+      int delta = (nameWidth - maxPixels) / em;
+      delta = emSqueezeLimit(delta, 1, length) ;
+
+      tmp.remove((length / 2) - (delta / 2), delta);
+      nameWidth = fontMetrics.width(tmp);
+    }
+
+    return tmp.insert((tmp.length() + 1) / 2, "...");
+  }
+
+  return name;
+}
+
+QString KStringHandler::rEmSqueeze(const QString& name, const QFontMetrics& fontMetrics, uint maxlen)
+{
+  return rPixelSqueeze(name, fontMetrics, fontMetrics.maxWidth() * maxlen);
+}
+
+QString KStringHandler::rPixelSqueeze(const QString& name, const QFontMetrics& fontMetrics, uint maxPixels)
+{
+  uint nameWidth = fontMetrics.width(name);
+
+  if (maxPixels < nameWidth)
+  {
+    QString tmp = name;
+    const uint em = fontMetrics.maxWidth();
+    maxPixels -= fontMetrics.width("...");
+
+    while (maxPixels < nameWidth && !tmp.isEmpty())
+    {
+      int length = tmp.length();
+      int delta = (nameWidth - maxPixels) / em;
+      delta = emSqueezeLimit(delta, 1, length) ;
+
+      tmp.remove(length - delta, delta);
+      nameWidth = fontMetrics.width(tmp);
+    }
+
+    return (tmp + "...");
+  }
+
+  return name;
+}
+
 ///// File name patterns (like *.txt)
 
 bool KStringHandler::matchFileName( const QString& filename, const QString& pattern  )
@@ -409,6 +509,11 @@ bool KStringHandler::matchFileName( const QString& filename, const QString& patt
 
    // Patterns like "Makefile*"
    if ( pattern[ pattern_len - 1 ] == '*' && len + 1 >= pattern_len ) {
+      if ( pattern[ 0 ] == '*' )
+      {
+         return filename.find(pattern.mid(1, pattern_len - 2)) != -1;
+      }
+
       const QChar *c1 = pattern.unicode();
       const QChar *c2 = filename.unicode();
       int cnt = 1;
@@ -536,3 +641,109 @@ KStringHandler::tagURLs( const QString& text )
     }
     return richText;
 }
+
+QString KStringHandler::obscure( const QString &str )
+{
+  QString result;
+  const QChar *unicode = str.unicode();
+  for ( uint i = 0; i < str.length(); ++i )
+    result += ( unicode[ i ].unicode() < 0x20 ) ? unicode[ i ] :
+        QChar( 0x1001F - unicode[ i ].unicode() );
+
+  return result;
+}
+
+bool KStringHandler::isUtf8(const char *buf)
+{
+  int i, n;
+  register unsigned char c;
+  bool gotone = false;
+
+#define F 0   /* character never appears in text */
+#define T 1   /* character appears in plain ASCII text */
+#define I 2   /* character appears in ISO-8859 text */
+#define X 3   /* character appears in non-ISO extended ASCII (Mac, IBM PC) */
+
+  static const unsigned char text_chars[256] = {
+        /*                  BEL BS HT LF    FF CR    */
+        F, F, F, F, F, F, F, T, T, T, T, F, T, T, F, F,  /* 0x0X */
+        /*                              ESC          */
+        F, F, F, F, F, F, F, F, F, F, F, T, F, F, F, F,  /* 0x1X */
+        T, T, T, T, T, T, T, T, T, T, T, T, T, T, T, T,  /* 0x2X */
+        T, T, T, T, T, T, T, T, T, T, T, T, T, T, T, T,  /* 0x3X */
+        T, T, T, T, T, T, T, T, T, T, T, T, T, T, T, T,  /* 0x4X */
+        T, T, T, T, T, T, T, T, T, T, T, T, T, T, T, T,  /* 0x5X */
+        T, T, T, T, T, T, T, T, T, T, T, T, T, T, T, T,  /* 0x6X */
+        T, T, T, T, T, T, T, T, T, T, T, T, T, T, T, F,  /* 0x7X */
+        /*            NEL                            */
+        X, X, X, X, X, T, X, X, X, X, X, X, X, X, X, X,  /* 0x8X */
+        X, X, X, X, X, X, X, X, X, X, X, X, X, X, X, X,  /* 0x9X */
+        I, I, I, I, I, I, I, I, I, I, I, I, I, I, I, I,  /* 0xaX */
+        I, I, I, I, I, I, I, I, I, I, I, I, I, I, I, I,  /* 0xbX */
+        I, I, I, I, I, I, I, I, I, I, I, I, I, I, I, I,  /* 0xcX */
+        I, I, I, I, I, I, I, I, I, I, I, I, I, I, I, I,  /* 0xdX */
+        I, I, I, I, I, I, I, I, I, I, I, I, I, I, I, I,  /* 0xeX */
+        I, I, I, I, I, I, I, I, I, I, I, I, I, I, I, I   /* 0xfX */
+  };
+
+  /* *ulen = 0; */
+  for (i = 0; (c = buf[i]); i++) {
+    if ((c & 0x80) == 0) {        /* 0xxxxxxx is plain ASCII */
+      /*
+       * Even if the whole file is valid UTF-8 sequences,
+       * still reject it if it uses weird control characters.
+       */
+
+      if (text_chars[c] != T)
+        return false;
+
+    } else if ((c & 0x40) == 0) { /* 10xxxxxx never 1st byte */
+      return false;
+    } else {                           /* 11xxxxxx begins UTF-8 */
+      int following;
+
+    if ((c & 0x20) == 0) {             /* 110xxxxx */
+      following = 1;
+    } else if ((c & 0x10) == 0) {      /* 1110xxxx */
+      following = 2;
+    } else if ((c & 0x08) == 0) {      /* 11110xxx */
+      following = 3;
+    } else if ((c & 0x04) == 0) {      /* 111110xx */
+      following = 4;
+    } else if ((c & 0x02) == 0) {      /* 1111110x */
+      following = 5;
+    } else
+      return false;
+
+      for (n = 0; n < following; n++) {
+        i++;
+        if (!(c = buf[i]))
+          goto done;
+
+        if ((c & 0x80) == 0 || (c & 0x40))
+          return false;
+      }
+      gotone = true;
+    }
+  }
+done:
+  return gotone;   /* don't claim it's UTF-8 if it's all 7-bit */
+}
+
+#undef F
+#undef T
+#undef I
+#undef X
+
+QString KStringHandler::from8Bit( const char *str )
+{
+  if (!str)
+    return QString::null;
+  if (!*str) {
+    static const QString &emptyString = KGlobal::staticQString("");
+    return emptyString;
+  }
+  return KStringHandler::isUtf8( str ) ?
+             QString::fromUtf8( str ) : 
+             QString::fromLocal8Bit( str );
+}
