@@ -23,15 +23,18 @@
 #include <qstring.h>
 #include <qstringlist.h>
 #include <qregexp.h>            // for the word ranges
-
+#include <qfontmetrics.h>
+#include "kdemacros.h"
 
 /**
- * This class is @em not a substitute for the @ref QString class. What
+ * This class contains utility functions for handling strings.
+ *
+ * This class is @em not a substitute for the QString class. What
  * I tried to do with this class is provide an easy way to
  * cut/slice/splice words inside sentences in whatever order desired.
  * While the main focus of this class are words (ie characters
- * separated by spaces/tabs), the two core functions here (@ref split()
- * and @ref join() ) will function given any char to use as a separator.
+ * separated by spaces/tabs), the two core functions here ( split()
+ * and join() ) will function given any char to use as a separator.
  * This will make it easy to redefine what a 'word' means in the
  * future if needed.
  *
@@ -41,14 +44,14 @@
  *
  * The ranges are a fairly powerful way of getting/stripping words from
  * a string. These ranges function, for the large part, as they would in
- * python. See the @ref word(const QString&, int) and @ref remword(const QString&, int) functions for more detail.
+ * python. See the word(const QString&, int) and remword(const QString&, int) functions for more detail.
  *
  * This class contains no data members of it own. All strings are cut
  * on the fly and returned as new qstrings/qstringlists.
  *
  * Quick example on how to use:
  *
- * <pre>
+ * \code
  * KStringHandler kstr;
  * QString line = "This is a test of the strings";
  *
@@ -56,12 +59,13 @@
  * cout << "2> " << kstr.remrange( line , "2:5" ) << "\n";
  * cout << "2> " << kstr.reverse( line ) << "\n";
  * cout << "2> " << kstr.center( kstr.word( line , 4 ) , 15 ) << "\n";
- * </pre>
+ * \endcode
  *
  * and so forth.
  *
  * @short Class for manipulating words and sentences in strings
  * @author Ian Zepp <icszepp@islc.net>
+ * @see KShell
  */
 class KStringHandler
 {
@@ -69,7 +73,7 @@ public:
     /** Returns the nth word in the string if found
       * Returns a EMPTY (not null) string otherwise.
       * Note that the FIRST index is 0.
-      * @param the the string to search for the words
+      * @param text the string to search for the words
       * @param pos the position of the word to search
       * @return the word, or an empty string if not found
       */
@@ -83,7 +87,7 @@ public:
       * @li ":3" returns everything up to the fourth word
       *
       * If you grok python, you're set.
-      * @param the the string to search for the words
+      * @param text the string to search for the words
       * @param range the words to return (see description)
       * @return the words, or an empty string if not found
       */
@@ -91,7 +95,7 @@ public:
 
     /** Inserts a word into the string, and returns
       * a new string with the word included. the first
-      * index is zero (0). If there are not @p pos words in the original 
+      * index is zero (0). If there are not @p pos words in the original
       * string, the new word will be appended to the end.
       * @param text the original text
       * @param word the word to insert
@@ -102,7 +106,7 @@ public:
 
     /** Replaces a word in the string, and returns
       * a new string with the word included. the first
-      * index is zero (0). If there are not @p pos words in the original 
+      * index is zero (0). If there are not @p pos words in the original
       * string, the new word will be appended to the end.
       * @param text the original text
       * @param word the word to insert
@@ -170,7 +174,7 @@ public:
       */
     static QStringList    reverse( const QStringList &list );
 
-    /** Left-justifies a string and returns a string at least 'width' characters 
+    /** Left-justifies a string and returns a string at least 'width' characters
       * wide.
       * If the string is longer than the @p width, the original
       * string is returned. It is never truncated.
@@ -180,7 +184,7 @@ public:
       */
     static QString        ljust( const QString &text , uint width );
 
-    /** Right-justifies a string and returns a string at least 'width' characters 
+    /** Right-justifies a string and returns a string at least 'width' characters
       * wide.
       * If the string is longer than the @p width, the original
       * string is returned. It is never truncated.
@@ -190,7 +194,7 @@ public:
       */
     static QString        rjust( const QString &text , uint width );
 
-    /** Centers a string and returns a string at least 'width' characters 
+    /** Centers a string and returns a string at least 'width' characters
       * wide.
       * If the string is longer than the @p width, the original
       * string is returned. It is never truncated.
@@ -208,6 +212,33 @@ public:
      */
     static QString        lsqueeze( const QString & str, uint maxlen = 40 );
 
+    /** Substitute characters at the beginning of a string by "...". Similar to
+     * method above, except that it truncates based on pixel width rather than
+     * the number of characters
+     * @param name is the string to modify
+     * @param fontMetrics is the font metrics to use to calculate character sizes
+     * @param maxlen is the maximum length in ems the modified string will have
+     * If the original string is shorter than "maxlen", it is returned verbatim
+     * @return the modified string
+     * @since 3.2
+     */
+    static QString lEmSqueeze( const QString & name,
+                               const QFontMetrics& fontMetrics,
+                               uint maxlen = 30 );
+
+    /** Substitute characters at the beginning of a string by "...". Similar to
+     * method above, except that maxlen is the width in pixels to truncate to
+     * @param name is the string to modify
+     * @param fontMetrics is the font metrics to use to calculate character sizes
+     * @param maxPixels is the maximum pixel length the modified string will have
+     * If the original string is shorter than "maxlen", it is returned verbatim
+     * @return the modified string
+     * @since 3.2
+     */
+    static QString lPixelSqueeze( const QString & name,
+                                  const QFontMetrics& fontMetrics,
+                                  uint maxPixels );
+
     /** Substitute characters at the middle of a string by "...".
      * @param str is the string to modify
      * @param maxlen is the maximum length the modified string will have
@@ -216,6 +247,33 @@ public:
      */
     static QString        csqueeze( const QString & str, uint maxlen = 40 );
 
+    /** Substitute characters in the middle of a string by "...". Similar to
+     * method above, except that it truncates based on pixel width rather than
+     * the number of characters
+     * @param name is the string to modify
+     * @param fontMetrics is the font metrics to use to calculate character sizes
+     * @param maxlen is the maximum length in ems the modified string will have
+     * If the original string is shorter than "maxlen", it is returned verbatim
+     * @return the modified string
+     * @since 3.2
+     */
+    static QString cEmSqueeze( const QString & name,
+                               const QFontMetrics& fontMetrics,
+                               uint maxlen = 30 );
+
+    /** Substitute characters in the middle of a string by "...". Similar to
+     * method above, except that maxlen is the width in pixels to truncate to
+     * @param name is the string to modify
+     * @param fontMetrics is the font metrics to use to calculate character sizes
+     * @param maxPixels is the maximum pixel length the modified string will have
+     * If the original string is shorter than "maxlen", it is returned verbatim
+     * @return the modified string
+     * @since 3.2
+     */
+    static QString cPixelSqueeze( const QString & name,
+                                  const QFontMetrics& fontMetrics,
+                                  uint maxPixels );
+
     /** Substitute characters at the end of a string by "...".
      * @param str is the string to modify
      * @param maxlen is the maximum length the modified string will have
@@ -224,15 +282,43 @@ public:
      */
     static QString        rsqueeze( const QString & str, uint maxlen = 40 );
 
+    /** Substitute characters at the end of a string by "...". Similar to
+     * method above, except that it truncates based on pixel width rather than
+     * the number of characters
+     * @param name is the string to modify
+     * @param fontMetrics is the font metrics to use to calculate character sizes
+     * @param maxlen is the maximum length in ems the modified string will have
+     * If the original string is shorter than "maxlen", it is returned verbatim
+     * @return the modified string
+     * @since 3.2
+     */
+    static QString rEmSqueeze( const QString & name,
+                               const QFontMetrics& fontMetrics,
+                               uint maxlen = 30 );
+
+    /** Substitute characters at the end of a string by "...". Similar to
+     * method above, except that maxlen is the width in pixels to truncate to
+     * @param name is the string to modify
+     * @param fontMetrics is the font metrics to use to calculate character sizes
+     * @param maxPixels is the maximum pixel length the modified string will have
+     * If the original string is shorter than "maxlen", it is returned verbatim
+     * @return the modified string
+     * @since 3.2
+     */
+    static QString rPixelSqueeze( const QString & name,
+                                  const QFontMetrics& fontMetrics,
+                                  uint maxPixels );
+
     /**
      * Match a filename.
      * @param filename is the real decoded filename (or dirname
      *        without trailing '/').
-     * @param pattern is a pattern like *.txt, *.tar.gz, Makefile.*, etc.
+     * @param pattern is a pattern like *.txt, *.tar.gz, Makefile.*, *README*, etc.
      * Patterns with two asterisks like "*.*pk" are not supported.
      * @return true if the given filename matches the given pattern
      */
     static bool matchFileName( const QString& filename, const QString& pattern );
+    // KDE4: move to KShell
 
     /**
      * Split a QString into a QStringList in a similar fashion to the static
@@ -241,12 +327,13 @@ public:
      * will be extracted. The final token will be the remainder of the string.
      *
      * Example:
-     * <pre>
+     * \code
      * perlSplit("__", "some__string__for__you__here", 4)
      * QStringList contains: "some", "string", "for", "you__here"
-     * </pre>
+     * \endcode
      *
      * @param sep is the string to use to delimit s.
+     * @param s is the input string
      * @param max is the maximum number of extractions to perform, or 0.
      * @return A QStringList containing tokens extracted from s.
      */
@@ -260,12 +347,13 @@ public:
      * will be extracted. The final token will be the remainder of the string.
      *
      * Example:
-     * <pre>
+     * \code
      * perlSplit(' ', "kparts reaches the parts other parts can't", 3)
-     * QStringList contains: "kparts", "reaches", "the pats other parts can't"
-     * </pre>
+     * QStringList contains: "kparts", "reaches", "the parts other parts can't"
+     * \endcode
      *
      * @param sep is the character to use to delimit s.
+     * @param s is the input string
      * @param max is the maximum number of extractions to perform, or 0.
      * @return A QStringList containing tokens extracted from s.
      */
@@ -279,12 +367,13 @@ public:
      * will be extracted. The final token will be the remainder of the string.
      *
      * Example:
-     * <pre>
+     * \code
      * perlSplit(QRegExp("[! ]", "Split me up ! I'm bored ! OK ?", 3)
      * QStringList contains: "Split", "me", "up ! I'm bored, OK ?"
-     * </pre>
+     * \endcode
      *
      * @param sep is the regular expression to use to delimit s.
+     * @param s is the input string
      * @param max is the maximum number of extractions to perform, or 0.
      * @return A QStringList containing tokens extracted from s.
      */
@@ -301,13 +390,47 @@ public:
      */
     static QString tagURLs( const QString& text );
 
+    /**
+      Obscure string by using a simple symmetric encryption. Applying the
+      function to a string obscured by this function will result in the original
+      string.
+
+      The function can be used to obscure passwords stored to configuration
+      files. Note that this won't give you any more security than preventing
+      that the password is directly copied and pasted.
+
+      @param str string to be obscured
+      @return obscured string
+      @since 3.2
+    */
+    static QString obscure( const QString &str );
+
+    /**
+      Guess whether a string is UTF8 encoded.
+
+      @param str the string to check
+      @return true if UTF8. If false, the string is probably in Local8Bit.
+      @since 3.2
+     */
+    static bool isUtf8( const char *str );
+
+    /**
+      Construct QString from a c string, guessing whether it is UTF8- or
+      Local8Bit-encoded.
+
+      @param str the input string
+      @return the (hopefully correctly guessed) QString representation of @p str
+      @since 3.2
+     */
+    static QString from8Bit( const char *str );
+
 #ifdef KDE_NO_COMPAT
 private:
 #endif
     /**
      * @deprecated Use @see matchFileName () instead.
      */
-    static bool matchFilename( const QString& filename, const QString& pattern )
+    static KDE_DEPRECATED bool matchFilename( const QString& filename, const QString& pattern )
     {
         return matchFileName (filename, pattern);
     };
