@@ -17,7 +17,7 @@
  **/
 
 #ifndef __ksycoca_h__
-#define __ksycoca_h__ "$Id: ksycoca.h,v 1.63.2.2 2003/05/05 21:03:52 mueller Exp $"
+#define __ksycoca_h__ "$Id: ksycoca.h,v 1.76.2.1 2004/02/28 08:34:06 coolo Exp $"
 
 #include <dcopobject.h>
 #include <qobject.h>
@@ -34,7 +34,7 @@ class KSycocaFactoryList;
  * If the existing file is outdated, it will not get read
  * but instead we'll ask kded to regenerate a new one...
 */
-#define KSYCOCA_VERSION 61
+#define KSYCOCA_VERSION 72
 
 /**
  * @internal
@@ -66,6 +66,8 @@ public:
 
    virtual ~KSycoca();
 
+   static int version();
+
    /**
     * @internal - called by factories in read-only mode
     * This is how factories get a stream to an entry
@@ -103,6 +105,12 @@ public:
     */
    Q_UINT32 updateSignature();
 
+   /**
+    * @internal - returns all directories with information 
+    * stored inside sycoca.
+    */
+   QStringList allResourceDirs();
+
    /**
     * @internal - add a factory
     */
@@ -126,7 +134,7 @@ public:
 
    /**
     * When you receive a "databaseChanged" signal, you can query here if
-    * a change has occured in a specific resource type.
+    * a change has occurred in a specific resource type.
     * @see KStandardDirs for the various resource types.
     */
    static bool isChanged(const char *type);
