@@ -1,7 +1,7 @@
 /* This file is part of the KDE libraries
     Copyright (c) 1999 Preston Brown <pbrown@kde.org>
 
-    $Id: kuniqueapplication.cpp,v 1.52.2.2 2003/06/05 09:12:50 waba Exp $
+    $Id: kuniqueapplication.cpp,v 1.68 2004/01/07 12:13:50 lunakl Exp $
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Library General Public
@@ -37,16 +37,26 @@
 #include <kcmdlineargs.h>
 #include <kstandarddirs.h>
 #include <kaboutdata.h>
-#include <kwin.h>
-#include <kstartupinfo.h>
+
+#if defined Q_WS_X11 && ! defined K_WS_QTONLY
+#include <kwin.h> // schroder
+#include <kstartupinfo.h> // schroder
+#endif
+
 #include <kconfig.h>
 #include "kdebug.h"
 #include "kuniqueapplication.h"
-#ifdef Q_WS_X11
-#include <X11/Xlib.h>
+
+#if defined Q_WS_X11 && ! defined K_WS_QTONLY
+#include <netwm.h> // schroder
+#include <X11/Xlib.h> // schroder
 #define DISPLAY "DISPLAY"
 #else
-#define DISPLAY "QWS_DISPLAY"
+#  ifdef Q_WS_QWS
+#    define DISPLAY "QWS_DISPLAY"
+#  else
+#    define DISPLAY "DISPLAY"
+#  endif
 #endif
 
 bool KUniqueApplication::s_nofork = false;
@@ -56,7 +66,7 @@ bool KUniqueApplication::s_uniqueTestDone = false;
 static KCmdLineOptions kunique_options[] =
 {
   { "nofork", "Don't run in the background.", 0 },
-  { 0, 0, 0 }
+  KCmdLineLastOption
 };
 
 struct DCOPRequest {
@@ -99,7 +109,6 @@ KUniqueApplication::start()
         pid.setNum(getpid());
         appName = appName + "-" + pid;
      }
-     ( void ) dcopClient();
      dcopClient()->registerAs(appName, false );
      // We'll call newInstance in the constructor. Do nothing here.
      return true;
@@ -122,11 +131,7 @@ KUniqueApplication::start()
      // Child
      ::close(fd[0]);
      if (s_multipleInstances)
-     {
-        QCString pid;
-        pid.setNum(getpid());
-        appName = appName + "-" + pid;
-     }
+        appName.append("-").append(QCString().setNum(getpid()));
      dc = dcopClient();
      {
         QCString regName = dc->registerAs(appName, false);
@@ -160,7 +165,9 @@ KUniqueApplication::start()
            delete dc;	// Clean up DCOP commmunication
            ::write(fd[1], &result, 1);
            ::close(fd[1]);
-#ifdef Q_WS_X11
+#if 0
+#if defined Q_WS_X11 && ! defined K_WS_QTONLY
+//#ifdef Q_WS_X11
            // say we're up and running ( probably no new window will appear )
            KStartupInfoId id;
            if( kapp != NULL ) // KApplication constructor unsets the env. variable
@@ -177,13 +184,16 @@ KUniqueApplication::start()
                }
            }
 #else //FIXME(E): implement
+#endif
 #endif
            return false;
         }
+        dc->setPriorityCall(true);
      }
 
      {
-#ifdef Q_WS_X11
+#if defined Q_WS_X11 && ! defined K_WS_QTONLY
+//#ifdef Q_WS_X11
          KStartupInfoId id;
          if( kapp != NULL ) // KApplication constructor unsets the env. variable
              id.initId( kapp->startupId());
@@ -212,11 +222,7 @@ KUniqueApplication::start()
 //     DCOPClient::emergencyClose();
 //     dcopClient()->detach();
      if (s_multipleInstances)
-     {
-        QCString pid;
-        pid.setNum(fork_result);
-        appName = appName + "-" + pid;
-     }
+        appName.append("-").append(QCString().setNum(fork_result));
      ::close(fd[1]);
      for(;;)
      {
@@ -224,7 +230,7 @@ KUniqueApplication::start()
        if (n == 1) break;
        if (n == 0)
        {
-          kdError() << "KUniqueApplication: Pipe closed unexpected." << endl;
+          kdError() << "KUniqueApplication: Pipe closed unexpectedly." << endl;
           ::exit(255);
        }
        if (errno != EINTR)
@@ -236,7 +242,7 @@ KUniqueApplication::start()
      ::close(fd[0]);
 
      if (result != 0)
-        ::exit(result); // Error occured in child.
+        ::exit(result); // Error occurred in child.
 
      dc = new DCOPClient();
      if (!dc->attach())
@@ -248,11 +254,25 @@ KUniqueApplication::start()
      if (!dc->isApplicationRegistered(appName)) {
         kdError() << "KUniqueApplication: Registering failed!" << endl;
      }
+
+     QCString new_asn_id;
+#if defined Q_WS_X11 && ! defined K_WS_QTONLY
+     KStartupInfoId id;
+     if( kapp != NULL ) // KApplication constructor unsets the env. variable
+         id.initId( kapp->startupId());
+     else
+         id = KStartupInfo::currentStartupIdEnv();
+     if( !id.none())
+         new_asn_id = id.id();
+#endif
+     
      QByteArray data, reply;
      QDataStream ds(data, IO_WriteOnly);
 
      KCmdLineArgs::saveAppArgs(ds);
+     ds << new_asn_id;
 
+     dc->setPriorityCall(true);
      QCString replyType;
      if (!dc->call(appName, KCmdLineArgs::about->appName(), "newInstance()", data, replyType, reply))
      {
@@ -260,6 +280,7 @@ KUniqueApplication::start()
         delete dc;	// Clean up DCOP commmunication
         ::exit(255);
      }
+     dc->setPriorityCall(false);
      if (replyType != "int")
      {
         kdError() << "KUniqueApplication: DCOP communication error!" << endl;
@@ -320,6 +341,7 @@ void KUniqueApplication::newInstanceNoFork()
   }
   
   newInstance();
+  KStartupInfo::handleAutoAppStartedSending(); // KDE4 remove?
   // What to do with the return value ?
 }
 
@@ -364,9 +386,17 @@ KUniqueApplication::processDelayed()
      QByteArray replyData;
      QCString replyType;
      if (request->fun == "newInstance()") {
+       dcopClient()->setPriorityCall(false);
        QDataStream ds(request->data, IO_ReadOnly);
        KCmdLineArgs::loadAppArgs(ds);
+       if( !ds.atEnd()) // backwards compatibility
+       {
+           QCString asn_id;
+           ds >> asn_id;
+           setStartupId( asn_id );
+       }
        int exitCode = newInstance();
+       KStartupInfo::handleAutoAppStartedSending(); // KDE4 remove?
        QDataStream rs(replyData, IO_WriteOnly);
        rs << exitCode;
        replyType = "int";
@@ -378,14 +408,26 @@ KUniqueApplication::processDelayed()
   d->processingRequest = false;
 }
 
+#if defined Q_WS_X11 && ! defined K_WS_QTONLY
+//#ifndef Q_WS_QWS // FIXME(E): Implement for Qt/Embedded
+extern Time qt_x_time;
+#endif
+
 int KUniqueApplication::newInstance()
 {
   if (!d->firstInstance)
   {
-#ifndef Q_WS_QWS // FIXME(E): Implement for Qt/Embedded
-     if ( mainWidget() )
-        KWin::setActiveWindow(mainWidget()->winId());
+//#ifndef Q_WS_QWS // FIXME(E): Implement for Qt/Embedded
+    if ( mainWidget() )
+    {
+      mainWidget()->show();
+#if defined Q_WS_X11 && ! defined K_WS_QTONLY
+    // This is the line that handles window activation if necessary,
+    // and what's important, it does it properly. If you reimplement newInstance(),
+    // and don't call the inherited one, use this.
+      KStartupInfo::setNewStartupId( mainWidget(), kapp->startupId());
 #endif
+    }
   }
   d->firstInstance = false;
   return 0; // do nothing in default implementation
