@@ -22,12 +22,15 @@
 #ifndef KDE_QT_ONLY
 #include <kdebug.h>
 #include <kglobal.h>
+#include <kidna.h>
+#include <kprotocolinfo.h>
 #endif
 
 #include <stdio.h>
 #include <assert.h>
 #include <ctype.h>
 #include <stdlib.h>
+#include <unistd.h>
 
 #include <qurl.h>
 #include <qdir.h>
@@ -36,19 +39,23 @@
 #include <qstylesheet.h>
 #include <qmap.h>
 #include <qtextcodec.h>
+#include <qmutex.h>
+
+static const QString fileProt = "file";
 
 static QTextCodec * codecForHint( int encoding_hint /* not 0 ! */ )
 {
     return QTextCodec::codecForMib( encoding_hint );
 }
 
-static QString encode( const QString& segment, bool encode_slash, int encoding_hint )
+// encoding_offset:
+// 0 encode both @ and /
+// 1 encode @ but not /
+// 2 encode neither @ or /
+static QString encode( const QString& segment, int encoding_offset, int encoding_hint )
 {
-  const char *encode_string;
-  if (encode_slash)
-    encode_string = "<>#@\"&%?={}|^~[]\'`\\:+/";
-  else
-    encode_string = "<>#@\"&%?={}|^~[]\'`\\:+";
+  const char *encode_string = "/@<>#\"&%?={}|^~[]\'`\\:+";
+  encode_string += encoding_offset;
 
   QCString local;
   if (encoding_hint==0)
@@ -65,7 +72,7 @@ static QString encode( const QString& segment, bool encode_slash, int encoding_h
   int old_length = local.length();
 
   if ( !old_length )
-    return segment.isNull() ? QString::null : QString(""); // differenciate null and empty
+    return segment.isNull() ? QString::null : QString(""); // differentiate null and empty
 
   // a worst case approximation
   QChar *new_segment = new QChar[ old_length * 3 + 1 ];
@@ -101,6 +108,21 @@ static QString encode( const QString& segment, bool encode_slash, int encoding_h
   return result;
 }
 
+static QString encodeHost( const QString& segment, bool encode_slash, int encoding_hint )
+{
+  // Hostnames are encoded differently
+  // we use the IDNA transformation instead
+
+  // Note: when merging qt-addon, use QResolver::domainToAscii here
+#ifndef KDE_QT_ONLY
+  Q_UNUSED( encode_slash );
+  Q_UNUSED( encoding_hint );
+  return KIDNA::toAscii(segment);
+#else
+  return encode(segment, encode_slash ? 0 : 1, encoding_hint);
+#endif
+}
+
 static int hex2int( unsigned int _char )
 {
   if ( _char >= 'A' && _char <='F')
@@ -122,7 +144,7 @@ static int hex2int( unsigned int _char )
 //      KURL(url.prettyURL()).url()
 
 
-static QString lazy_encode( const QString& segment )
+static QString lazy_encode( const QString& segment, bool encodeAt=true )
 {
   int old_length = segment.length();
 
@@ -144,6 +166,7 @@ static QString lazy_encode( const QString& segment )
           (hex2int(segment[i+1].unicode())!= -1) &&
           (hex2int(segment[i+2].unicode())!= -1)) ||
         (character == '?') || // Start of query delimiter
+        ((character == '@') && encodeAt) || // Username delimiter
         (character == '#') || // Start of reference delimiter
         ((character == 32) && (i+1 == old_length))) // A trailing space
     {
@@ -182,10 +205,14 @@ static void decode( const QString& segment, QString &decoded, QString &encoded,
   if (!textCodec)
       textCodec = QTextCodec::codecForLocale();
 
-  if (!textCodec->canEncode(segment))
-      textCodec = codecForHint( 106 ); // Fall back to utf-8 if it doesn't fit.
-
   QCString csegment = textCodec->fromUnicode(segment);
+  // Check if everything went ok
+  if (textCodec->toUnicode(csegment) != segment)
+  {
+      // Uh oh
+      textCodec = codecForHint( 106 ); // Fall back to utf-8
+      csegment = textCodec->fromUnicode(segment);
+  } 
   old_length = csegment.length();
 
   int new_length = 0;
@@ -271,10 +298,37 @@ static QString decode(const QString &segment, int encoding_hint = 0)
   return result;
 }
 
-static QString cleanpath(const QString &path, bool cleanDirSeparator=true)
+static QString cleanpath(const QString &_path, bool cleanDirSeparator, bool decodeDots)
 {
-  if (path.isEmpty()) return QString::null;
+  if (_path.isEmpty()) return QString::null;
+  
+  if (_path[0] != '/')
+     return _path; // Don't mangle mailto-style URLs
+  
+  QString path = _path;
+
   int len = path.length();
+
+  if (decodeDots)
+  {
+#ifndef KDE_QT_ONLY
+     static const QString &encodedDot = KGlobal::staticQString("%2e");
+#else
+     QString encodedDot("%2e");
+#endif
+     if (path.find(encodedDot, 0, false) != -1)
+     {
+#ifndef KDE_QT_ONLY
+        static const QString &encodedDOT = KGlobal::staticQString("%2E"); // Uppercase!
+#else
+        QString encodedDOT("%2E");
+#endif
+        path.replace(encodedDot, ".");
+        path.replace(encodedDOT, ".");
+        len = path.length();
+     }
+  }
+
   bool slash = (len && path[len-1] == '/') ||
                (len > 1 && path[len-2] == '/' && path[len-1] == '.');
 
@@ -296,7 +350,7 @@ static QString cleanpath(const QString &path, bool cleanDirSeparator=true)
       cdUp++;
     else
     {
-      // Ignore any occurances of '.'
+      // Ignore any occurrences of '.'
       // This includes entries that simply do not make sense like /..../
       if ( (len || !cleanDirSeparator) &&
            (len != 1 || path[pos+1] != '.' ) )
@@ -424,11 +478,15 @@ QDataStream & operator>> (QDataStream & s, KURL & a)
       >> malf >> a.m_iPort;
     a.m_bIsMalformed = (malf != 0);
 
-    if ( QueryFromWire.isEmpty() )
+    if ( QueryFromWire.isNull() )
       a.m_strQuery_encoded = QString::null;
+    else if ( QueryFromWire.length() == 1 ) // empty query
+      a.m_strQuery_encoded = "";
     else
       a.m_strQuery_encoded = QueryFromWire.mid(1);
 
+    a.m_iUriMode = KURL::uriModeForProtocol( a.m_strProtocol );
+
     return s;
 }
 
@@ -441,6 +499,15 @@ KURL::KURL( const QUrl &u )
 
 KURL::KURL( const KURL& _u, const QString& _rel_url, int encoding_hint )
 {
+  if (_u.hasSubURL()) // Operate on the last suburl, not the first
+  {
+    KURL::List lst = split( _u );
+    KURL u(lst.last(), _rel_url, encoding_hint);
+    lst.remove( lst.last() );
+    lst.append( u );
+    *this = join( lst );
+    return;
+  }
   // WORKAROUND THE RFC 1606 LOOPHOLE THAT ALLOWS
   // http:/index.html AS A VALID SYNTAX FOR RELATIVE
   // URLS. ( RFC 2396 section 5.2 item # 3 )
@@ -513,6 +580,7 @@ KURL::KURL( const KURL& _u, const QString& _rel_url, int encoding_hint )
        m_strUser = _u.m_strUser;
        m_strPass = _u.m_strPass;
     }
+    cleanPath(false);
   }
 }
 
@@ -528,6 +596,7 @@ void KURL::reset()
   m_strRef_encoded = QString::null;
   m_bIsMalformed = true;
   m_iPort = 0;
+  m_iUriMode = Auto;
 }
 
 bool KURL::isEmpty() const
@@ -537,22 +606,125 @@ bool KURL::isEmpty() const
 
 void KURL::parse( const QString& _url, int encoding_hint )
 {
-  //kdDebug(126) << "parse " << _url << endl;
-  // Return immediately whenever the given url
-  // is empty or null.
-  if ( _url.isEmpty() )
-  {
-    m_strProtocol = _url;
-    return;
-  }
+    if ( _url.isEmpty() || m_iUriMode == Invalid )
+    {
+	m_strProtocol = _url;
+	m_iUriMode = Invalid;
+	return;
+    }
 
+    const QChar* buf = _url.unicode();
+    const QChar* orig = buf;
+    uint len = _url.length();
+    uint pos = 0;
+
+    // Node 1: Accept alpha or slash
+    QChar x = buf[pos++];
+    if ( x == '/' )
+    {
+	// A slash means we immediately proceed to parse it as a file URL.
+	m_iUriMode = URL;
+	m_strProtocol = fileProt;
+	parseURL( _url, encoding_hint );
+	return;
+    }
+    if ( !isalpha( (int)x ) )
+	goto NodeErr;
+
+    // Node 2: Accept any amount of (alpha|digit|'+'|'-')
+    // '.' is not currently accepted, because current KURL may be confused.
+    // Proceed with :// :/ or :
+    while( pos < len && (isalpha((int)buf[pos]) || isdigit((int)buf[pos]) ||
+			 buf[pos] == '+' || buf[pos] == '-')) pos++;
+
+    if (pos < len && buf[pos] == ':' )
+    {
+	m_strProtocol = QString( orig, pos ).lower();
+	if ( m_iUriMode == Auto )
+	    m_iUriMode = uriModeForProtocol( m_strProtocol );
+	// Proceed to correct parse function.
+	switch ( m_iUriMode )
+  {
+	case RawURI:
+	    parseRawURI( _url );
+	    return;
+	case Mailto:
+	    parseMailto( _url );
+	    return;
+	case URL:
+	    parseURL( _url, encoding_hint );
+	    return;
+	default:
+	    // Unknown URI mode results in an invalid URI.
+	    break;
+	}
+    }
+
+NodeErr:
+    reset();
+    m_strProtocol = _url;
+    m_iUriMode = Invalid;
+}
+
+void KURL::parseRawURI( const QString& _url, int encoding_hint )
+{
+    uint len = _url.length();
+    const QChar* buf = _url.unicode();
+
+    uint pos = 0;
+
+    // Accept any amount of (alpha|digit|'+'|'-')
+    // '.' is not currently accepted, because current KURL may be confused.
+    // Proceed with :
+    while( pos < len && (isalpha((int)buf[pos]) || isdigit((int)buf[pos]) ||
+			 buf[pos] == '+' || buf[pos] == '-')) pos++;
+
+    // Note that m_strProtocol is already set here, so we just skip over the protocol.
+    if (pos < len && buf[pos] == ':' )
+	pos++;
+    else {
+	reset();
+    m_strProtocol = _url;
+	m_iUriMode = Invalid;
+	return;
+    }
+
+    if ( pos == len )
+	m_strPath = QString::null;
+    else
+	m_strPath = decode( QString( buf + pos, len - pos ), encoding_hint );
+
+    m_bIsMalformed = false;
+
+    return;
+}
+
+void KURL::parseMailto( const QString& _url, int encoding_hint )
+{
+    parseURL( _url, encoding_hint);
+    if ( m_bIsMalformed )
+    return;
+    QRegExp mailre("(.+@)(.+)");
+    if ( mailre.exactMatch( m_strPath ) )
+    {
+#ifndef KDE_QT_ONLY
+	QString host = KIDNA::toUnicode( mailre.cap( 2 ) );
+	if (host.isEmpty())
+	    host = mailre.cap( 2 ).lower();
+#else
+	QString host = mailre.cap( 2 ).lower();
+#endif
+	m_strPath = mailre.cap( 1 ) + host;
+  }
+}
+
+void KURL::parseURL( const QString& _url, int encoding_hint )
+{
   QString port;
   bool badHostName = false;
   int start = 0;
   uint len = _url.length();
-  QChar* buf = new QChar[ len + 1 ];
-  QChar* orig = buf;
-  memcpy( buf, _url.unicode(), len * sizeof( QChar ) );
+  const QChar* buf = _url.unicode();
 
   QChar delim;
   QString tmp;
@@ -569,28 +741,16 @@ void KURL::parse( const QString& _url, int encoding_hint )
   // Node 2: Accept any amount of (alpha|digit|'+'|'-')
   // '.' is not currently accepted, because current KURL may be confused.
   // Proceed with :// :/ or :
-  while( (isalpha((int)buf[pos]) || isdigit((int)buf[pos]) ||
-          buf[pos] == '+' || buf[pos] == '-') &&
-         pos < len ) pos++;
-  if ( pos == len - 1 ) // Need to always compare length()-1 otherwise KURL passes "http:" as legal!!! (DA)
-    goto NodeErr;
-  if (buf[pos] == ':' && buf[pos+1] == '/' && buf[pos+2] == '/' )
+  while( pos < len && (isalpha((int)buf[pos]) || isdigit((int)buf[pos]) ||
+          buf[pos] == '+' || buf[pos] == '-')) pos++;
+
+  // Note that m_strProtocol is already set here, so we just skip over the protocol.
+  if ( pos+2 < len && buf[pos] == ':' && buf[pos+1] == '/' && buf[pos+2] == '/' )
     {
-      m_strProtocol = QString( orig, pos ).lower();
       pos += 3;
     }
-  else if (buf[pos] == ':' && buf[pos+1] == '/' )
+  else if (pos+1 < len && buf[pos] == ':' ) // Need to always compare length()-1 otherwise KURL passes "http:" as legal!!
     {
-      m_strProtocol = QString( orig, pos ).lower();
-      //kdDebug(126)<<"setting protocol to "<<m_strProtocol<<endl;
-      pos++;
-      start = pos;
-      goto Node9;
-    }
-  else if ( buf[pos] == ':' )
-    {
-      m_strProtocol = QString( orig, pos ).lower();
-      //kdDebug(126)<<"setting protocol to "<<m_strProtocol<<endl;
       pos++;
       start = pos;
       goto Node9;
@@ -608,18 +768,20 @@ void KURL::parse( const QString& _url, int encoding_hint )
       goto Node8;
   // Terminate on / or @ or ? or # or " or ; or <
   x = buf[pos];
-  while( (x != ':') && (x != '@') && (x != '/') && (x != '?') && (x != '#') &&  (pos < len) )
+  while( (x != ':') && (x != '@') && (x != '/') && (x != '?') && (x != '#') )
   {
      if ((x == '\"') || (x == ';') || (x == '<'))
         badHostName = true;
-     x = buf[++pos];
+     if (++pos == len)
+        break;
+     x = buf[pos];
   }
   if ( pos == len )
     {
       if (badHostName)
          goto NodeErr;
 
-      m_strHost = decode(QString( buf + start, pos - start ), encoding_hint);
+      setHost(decode(QString( buf + start, pos - start ), encoding_hint));
       goto NodeOk;
     }
   if ( x == '@' )
@@ -628,18 +790,12 @@ void KURL::parse( const QString& _url, int encoding_hint )
       pos++;
       goto Node7;
     }
-  /* else if ( x == ':' )
-     {
-     m_strHost = decode(QString( buf + start, pos - start ), encoding_hint);
-     pos++;
-     goto Node8a;
-     } */
   else if ( (x == '/') || (x == '?') || (x == '#'))
     {
       if (badHostName)
          goto NodeErr;
 
-      m_strHost = decode(QString( buf + start, pos - start ), encoding_hint);
+      setHost(decode(QString( buf + start, pos - start ), encoding_hint));
       start = pos;
       goto Node9;
     }
@@ -666,7 +822,7 @@ void KURL::parse( const QString& _url, int encoding_hint )
       // Ok the : was used to separate host and port
       if (badHostName)
          goto NodeErr;
-      m_strHost = m_strUser;
+      setHost(m_strUser);
       m_strUser = QString::null;
       QString tmp( buf + start, pos - start );
       char *endptr;
@@ -675,7 +831,13 @@ void KURL::parse( const QString& _url, int encoding_hint )
         goto NodeOk;
       // there is more after the digits
       pos -= strlen(endptr);
-      start = pos++;
+      if ((buf[pos] != '@') &&
+          (buf[pos] != '/') &&
+          (buf[pos] != '?') &&
+          (buf[pos] != '#'))
+        goto NodeErr;
+
+      start = pos;
       goto Node9;
     }
   m_strPass = decode(QString( buf + start, pos - start), encoding_hint);
@@ -692,76 +854,88 @@ void KURL::parse( const QString& _url, int encoding_hint )
     // IPv6 address
     start = ++pos; // Skip '['
 
-    // Node 8b: Read everything until ] or terminate
+    if (pos == len)
+    {
+       badHostName = true;
+       goto NodeErr;
+    }
+    // Node 8a: Read everything until ] or terminate
     badHostName = false;
     x = buf[pos];
-    while( (x != ']') &&  (pos < len) )
+    while( (x != ']') )
     {
        if ((x == '\"') || (x == ';') || (x == '<'))
           badHostName = true;
-       x = buf[++pos];
+       if (++pos == len)
+       {
+          badHostName = true;
+          break;
+       }
+       x = buf[pos];
     }
     if (badHostName)
        goto NodeErr;
-    m_strHost = decode(QString( buf + start, pos - start ), encoding_hint);
+    setHost(decode(QString( buf + start, pos - start ), encoding_hint));
     if (pos < len) pos++; // Skip ']'
     if (pos == len)
        goto NodeOk;
   }
   else
   {
-    // Non IPv6 address
-    start = pos++;
+    // Non IPv6 address, with a user
+    start = pos;
 
     // Node 8b: Read everything until / : or terminate
     badHostName = false;
     x = buf[pos];
-    while( (x != ':') && (x != '@') && (x != '/') && (x != '?') && (x != '#') &&  (pos < len) )
+    while( (x != ':') && (x != '@') && (x != '/') && (x != '?') && (x != '#') )
     {
        if ((x == '\"') || (x == ';') || (x == '<'))
           badHostName = true;
-       x = buf[++pos];
+       if (++pos == len)
+          break;
+       x = buf[pos];
     }
     if (badHostName)
        goto NodeErr;
     if ( pos == len )
     {
-       m_strHost = decode(QString( buf + start, pos - start ), encoding_hint);
+       setHost(decode(QString( buf + start, pos - start ), encoding_hint));
        goto NodeOk;
     }
-    m_strHost = decode(QString( buf + start, pos - start ), encoding_hint);
+    setHost(decode(QString( buf + start, pos - start ), encoding_hint));
   }
   x = buf[pos];
-  if ( x == '/' )
+  if ( x == '/' || x == '#' || x == '?' )
     {
-      start = pos++;
+      start = pos;
       goto Node9;
     }
   else if ( x != ':' )
     goto NodeErr;
   pos++;
 
-  // Node 8a: Accept at least one digit
+  // Node 8c: Accept at least one digit
   if ( pos == len )
     goto NodeErr;
   start = pos;
   if ( !isdigit( buf[pos++] ) )
     goto NodeErr;
 
-  // Node 8b: Accept any amount of digits
-  while( isdigit( buf[pos] ) && pos < len ) pos++;
+  // Node 8d: Accept any amount of digits
+  while( pos < len && isdigit( buf[pos] ) ) pos++;
   port = QString( buf + start, pos - start );
   m_iPort = port.toUShort();
   if ( pos == len )
     goto NodeOk;
-  start = pos++;
+  start = pos;
 
  Node9: // parse path until query or reference reached
 
-  while( buf[pos] != '#' && buf[pos]!='?' && pos < len ) pos++;
+  while( pos < len && buf[pos] != '#' && buf[pos]!='?' ) pos++;
 
   tmp = QString( buf + start, pos - start );
-  //kdDebug(126)<<" setting encoded path&query to:"<<tmp<<endl;
+  //kdDebug(126)<<" setting encoded path to:"<<tmp<<endl;
   setEncodedPath( tmp, encoding_hint );
 
   if ( pos == len )
@@ -772,11 +946,11 @@ void KURL::parse( const QString& _url, int encoding_hint )
 
   start = pos;
 
-  while(buf[pos]!=delim && pos < len) pos++;
+  while(pos < len && buf[pos]!=delim ) pos++;
 
   tmp = QString(buf + start, pos - start);
   if (delim=='#')
-      setQuery(tmp, encoding_hint);
+      _setQuery(tmp, encoding_hint);
   else
       m_strRef_encoded = tmp;
 
@@ -788,41 +962,25 @@ void KURL::parse( const QString& _url, int encoding_hint )
   if (delim == '#')
       m_strRef_encoded = tmp;
   else
-      setQuery(tmp, encoding_hint);
+      _setQuery(tmp, encoding_hint);
 
  NodeOk:
   //kdDebug(126)<<"parsing finished. m_strProtocol="<<m_strProtocol<<" m_strHost="<<m_strHost<<" m_strPath="<<m_strPath<<endl;
-  delete []orig;
   m_bIsMalformed = false; // Valid URL
-  if (m_strProtocol.isEmpty())
-    m_strProtocol = "file";
 
   //kdDebug()<<"Prot="<<m_strProtocol<<"\nUser="<<m_strUser<<"\nPass="<<m_strPass<<"\nHost="<<m_strHost<<"\nPath="<<m_strPath<<"\nQuery="<<m_strQuery_encoded<<"\nRef="<<m_strRef_encoded<<"\nPort="<<m_iPort<<endl;
-  if (m_strProtocol == "file")
+  if (m_strProtocol.isEmpty())
   {
-    if (!m_strHost.isEmpty())
-    {
-      // File-protocol has a host name..... hmm?
-      if (m_strHost.lower() == "localhost")
-      {
-        m_strHost = QString::null; // We can ignore localhost
-      }
-      else {
-        // Pass the hostname as part of the path. Perhaps system calls
-        // just handle it.
-        m_strPath = "//"+m_strHost+m_strPath;
-        m_strPath_encoded = QString::null;
-        m_strHost = QString::null;
-      }
-    }
+    m_iUriMode = URL;
+    m_strProtocol = fileProt;
   }
   return;
 
  NodeErr:
 //  kdDebug(126) << "KURL couldn't parse URL \"" << _url << "\"" << endl;
-  delete []orig;
   reset();
   m_strProtocol = _url;
+  m_iUriMode = Invalid;
 }
 
 KURL& KURL::operator=( const QString& _url )
@@ -845,10 +1003,11 @@ KURL& KURL::operator=( const char * _url )
 KURL& KURL::operator=( const QUrl & u )
 {
   m_strProtocol = u.protocol();
+  m_iUriMode = Auto;
   m_strUser = u.user();
   m_strPass = u.password();
   m_strHost = u.host();
-  m_strPath = u.path( FALSE );
+  m_strPath = u.path( false );
   m_strPath_encoded = QString::null;
   m_strQuery_encoded = u.query();
   m_strRef_encoded = u.ref();
@@ -871,19 +1030,61 @@ KURL& KURL::operator=( const KURL& _u )
   m_strRef_encoded = _u.m_strRef_encoded;
   m_bIsMalformed = _u.m_bIsMalformed;
   m_iPort = _u.m_iPort;
+  m_iUriMode = _u.m_iUriMode;
 
   return *this;
 }
 
+bool KURL::operator<( const KURL& _u) const
+{
+  int i;
+  if (!_u.isValid())
+  {
+     if (!isValid())
+     {
+        i = m_strProtocol.compare(_u.m_strProtocol);
+        return (i < 0);
+     }
+     return false;
+  }
+  if (!isValid())
+     return true;
+  
+  i = m_strProtocol.compare(_u.m_strProtocol);
+  if (i) return (i < 0);
+
+  i = m_strHost.compare(_u.m_strHost);
+  if (i) return (i < 0);
+
+  if (m_iPort != _u.m_iPort) return (m_iPort < _u.m_iPort);
+
+  i = m_strPath.compare(_u.m_strPath);
+  if (i) return (i < 0);
+
+  i = m_strQuery_encoded.compare(_u.m_strQuery_encoded);
+  if (i) return (i < 0);
+
+  i = m_strRef_encoded.compare(_u.m_strRef_encoded);
+  if (i) return (i < 0);
+
+  i = m_strUser.compare(_u.m_strUser);
+  if (i) return (i < 0);
+  
+  i = m_strPass.compare(_u.m_strPass);
+  if (i) return (i < 0);
+
+  return false;    
+}
+
 bool KURL::operator==( const KURL& _u ) const
 {
-  if ( isMalformed() || _u.isMalformed() )
+  if ( !isValid() || !_u.isValid() )
     return false;
 
   if ( m_strProtocol == _u.m_strProtocol &&
        m_strUser == _u.m_strUser &&
        m_strPass == _u.m_strPass &&
-       m_strHost.lower() == _u.m_strHost.lower() &&
+       m_strHost == _u.m_strHost &&
        m_strPath == _u.m_strPath &&
        // The encoded path may be null, but the URLs are still equal (David)
        ( m_strPath_encoded.isNull() || _u.m_strPath_encoded.isNull() ||
@@ -911,7 +1112,7 @@ bool KURL::cmp( const KURL &u, bool ignore_trailing ) const
 
 bool KURL::equals( const KURL &_u, bool ignore_trailing ) const
 {
-  if ( isMalformed() || _u.isMalformed() )
+  if ( !isValid() || !_u.isValid() )
     return false;
 
   if ( ignore_trailing )
@@ -938,7 +1139,7 @@ bool KURL::equals( const KURL &_u, bool ignore_trailing ) const
 
 bool KURL::isParentOf( const KURL& _u ) const
 {
-  if ( isMalformed() || _u.isMalformed() )
+  if ( !isValid() || !_u.isValid() )
     return false;
 
   if ( m_strProtocol == _u.m_strProtocol &&
@@ -952,10 +1153,10 @@ bool KURL::isParentOf( const KURL& _u ) const
     if ( path().isEmpty() || _u.path().isEmpty() )
         return false; // can't work with implicit paths
 
-    QString p1( cleanpath( path() ) );
+    QString p1( cleanpath( path(), true, false ) );
     if ( p1[p1.length()-1] != '/' )
         p1 += '/';
-    QString p2( cleanpath( _u.path() ) );
+    QString p2( cleanpath( _u.path(), true, false ) );
     if ( p2[p2.length()-1] != '/' )
         p2 += '/';
 
@@ -1009,9 +1210,10 @@ void KURL::setFileName( const QString& _txt )
 
 void KURL::cleanPath( bool cleanDirSeparator ) // taken from the old KURL
 {
-  m_strPath = cleanpath(m_strPath, cleanDirSeparator);
+  if (m_iUriMode != URL) return;
+  m_strPath = cleanpath(m_strPath, cleanDirSeparator, false);
   // WABA: Is this safe when "/../" is encoded with %?
-  m_strPath_encoded = cleanpath(m_strPath_encoded, cleanDirSeparator);
+  m_strPath_encoded = cleanpath(m_strPath_encoded, cleanDirSeparator, true);
 }
 
 static QString trailingSlash( int _trailing, const QString &path )
@@ -1023,9 +1225,7 @@ static QString trailingSlash( int _trailing, const QString &path )
   else if ( _trailing == 1 )
   {
     int len = result.length();
-    if ( len == 0 )
-      result = QString::null;
-    else if ( result[ len - 1 ] != '/' )
+    if ( (len == 0) || (result[ len - 1 ] != '/') )
       result += "/";
     return result;
   }
@@ -1034,7 +1234,7 @@ static QString trailingSlash( int _trailing, const QString &path )
     if ( result == "/" )
       return result;
     int len = result.length();
-    if ( len != 0 && result[ len - 1 ] == '/' )
+    if ( (len != 0) && (result[ len - 1 ] == '/') )
       result.truncate( len - 1 );
     return result;
   }
@@ -1066,7 +1266,14 @@ QString KURL::encodedPathAndQuery( int _trailing, bool _no_empty_path, int encod
      tmp = path( _trailing );
      if ( _no_empty_path && tmp.isEmpty() )
         tmp = "/";
-     tmp = encode( tmp, false, encoding_hint );
+     if (m_iUriMode == Mailto)
+     {
+        tmp = encode( tmp, 2, encoding_hint );
+     }
+     else
+     {
+        tmp = encode( tmp, 1, encoding_hint );
+     }
   }
 
   // TODO apply encoding_hint to the query
@@ -1077,17 +1284,15 @@ QString KURL::encodedPathAndQuery( int _trailing, bool _no_empty_path, int encod
 
 void KURL::setEncodedPath( const QString& _txt, int encoding_hint )
 {
-#ifdef KDE_QT_ONLY
-  QString fileProt = "file";
-#else
-  static const QString & fileProt = KGlobal::staticQString( "file" );
-#endif
   m_strPath_encoded = _txt;
 
   decode( m_strPath_encoded, m_strPath, m_strPath_encoded, encoding_hint );
   // Throw away encoding for local files, makes file-operations faster.
   if (m_strProtocol == fileProt)
      m_strPath_encoded = QString::null;
+
+  if ( m_iUriMode == Auto )
+    m_iUriMode = URL;
 }
 
 
@@ -1102,7 +1307,7 @@ void KURL::setEncodedPathAndQuery( const QString& _txt, int encoding_hint )
   else
   {
     setEncodedPath(_txt.left( pos ), encoding_hint);
-    setQuery(_txt.right(_txt.length() - pos - 1), encoding_hint);
+    _setQuery(_txt.right(_txt.length() - pos - 1), encoding_hint);
   }
 }
 
@@ -1113,12 +1318,21 @@ QString KURL::path( int _trailing ) const
 
 bool KURL::isLocalFile() const
 {
-#ifdef KDE_QT_ONLY
-  QString fileProt = "file";
-#else
-  static const QString & fileProt = KGlobal::staticQString( "file" );
-#endif
-  return ( ( m_strProtocol == fileProt ) && ( m_strHost.isEmpty()) && !hasSubURL() );
+  if ( (m_strProtocol != fileProt ) || hasSubURL() )
+     return false;
+     
+  if (m_strHost.isEmpty() || (m_strHost == "localhost"))
+     return true;
+     
+  char hostname[ 256 ];
+  hostname[ 0 ] = '\0';
+  if (!gethostname( hostname, 255 ))
+     hostname[sizeof(hostname)-1] = '\0';
+     
+  for(char *p = hostname; *p; p++)
+     *p = tolower(*p);
+     
+  return (m_strHost == hostname);
 }
 
 void KURL::setFileEncoding(const QString &encoding)
@@ -1145,9 +1359,9 @@ void KURL::setFileEncoding(const QString &encoding)
      args.append("charset="+encode_string(encoding));
 
   if (args.isEmpty())
-     setQuery(QString::null);
+     _setQuery(QString::null);
   else
-     setQuery(args.join("&"));
+     _setQuery(args.join("&"));
 }
 
 QString KURL::fileEncoding() const
@@ -1189,6 +1403,10 @@ bool KURL::hasSubURL() const
      return true;
   if (m_strRef_encoded.startsWith("tar:"))
      return true;
+  if (m_strRef_encoded.startsWith("ar:"))
+     return true;
+  if (m_strRef_encoded.startsWith("zip:"))
+     return true;
   if ( m_strProtocol == "error" ) // anything that starts with error: has suburls
      return true;
   return false;
@@ -1206,34 +1424,44 @@ QString KURL::url( int _trailing, int encoding_hint ) const
 
   QString u = m_strProtocol;
   if (!u.isEmpty())
-     u += ":";
+    u += ":";
 
   if ( hasHost() )
   {
     u += "//";
     if ( hasUser() )
     {
-      u += encode(m_strUser, true, encoding_hint);
+      u += encode(m_strUser, 0, encoding_hint);
       if ( hasPass() )
       {
         u += ":";
-        u += encode(m_strPass, true, encoding_hint);
+        u += encode(m_strPass, 0, encoding_hint);
       }
       u += "@";
     }
-    bool IPv6 = (m_strHost.find(':') != -1);
-    if (IPv6)
-       u += '[' + m_strHost + ']';
+    if ( m_iUriMode == URL )
+    {
+      bool IPv6 = (m_strHost.find(':') != -1);
+      if (IPv6)
+        u += '[' + m_strHost + ']';
+      else
+        u += encodeHost(m_strHost, true, encoding_hint);
+      if ( m_iPort != 0 ) {
+        QString buffer;
+        buffer.sprintf( ":%u", m_iPort );
+        u += buffer;
+      }
+    }
     else
-       u += encode(m_strHost, true, encoding_hint);
-    if ( m_iPort != 0 ) {
-      QString buffer;
-      buffer.sprintf( ":%u", m_iPort );
-      u += buffer;
+    {
+      u += m_strHost;
     }
   }
 
-  u += encodedPathAndQuery( _trailing, false, encoding_hint );
+  if ( m_iUriMode == URL || m_iUriMode == Mailto )
+    u += encodedPathAndQuery( _trailing, false, encoding_hint );
+  else
+    u += m_strPath;
 
   if ( hasRef() )
   {
@@ -1267,11 +1495,22 @@ QString KURL::prettyURL( int _trailing ) const
       // Don't show password!
       u += "@";
     }
+    if ( m_iUriMode == URL )
+    {
     bool IPv6 = (m_strHost.find(':') != -1);
     if (IPv6)
+    {
        u += '[' + m_strHost + ']';
+    }
     else
+    {
        u += lazy_encode(m_strHost);
+    }
+    }
+    else
+    {
+      u += lazy_encode(m_strHost);
+    }
     if ( m_iPort != 0 ) {
       QString buffer;
       buffer.sprintf( ":%u", m_iPort );
@@ -1279,7 +1518,15 @@ QString KURL::prettyURL( int _trailing ) const
     }
   }
 
-  u += trailingSlash( _trailing, lazy_encode( m_strPath ) );
+  if (m_iUriMode == Mailto)
+  {
+     u += lazy_encode( m_strPath, false );
+  }
+  else
+  {
+     u += trailingSlash( _trailing, lazy_encode( m_strPath ) );
+  }
+
   if (!m_strQuery_encoded.isNull())
       u += '?' + m_strQuery_encoded;
 
@@ -1353,7 +1600,8 @@ KURL KURL::join( const KURL::List & lst )
      KURL u(*it);
      if (it != first)
      {
-        u.m_strRef_encoded = tmp.url();
+        if (!u.m_strRef_encoded) u.m_strRef_encoded = tmp.url();
+        else u.m_strRef_encoded += "#" + tmp.url(); // Support more than one suburl thingy
      }
      tmp = u;
   }
@@ -1364,6 +1612,11 @@ KURL KURL::join( const KURL::List & lst )
 QString KURL::fileName( bool _strip_trailing_slash ) const
 {
   QString fname;
+  if (hasSubURL()) { // If we have a suburl, then return the filename from there
+    KURL::List list = KURL::split(*this);
+    KURL::List::Iterator it = list.fromLast();
+    return (*it).fileName(_strip_trailing_slash);
+  }
   const QString &path = m_strPath;
 
   int len = path.length();
@@ -1412,11 +1665,20 @@ QString KURL::fileName( bool _strip_trailing_slash ) const
   {
      fname = path.mid( i + 1, len - i - 1 ); // TO CHECK
   }
-     return fname;
+  return fname;
 }
 
 void KURL::addPath( const QString& _txt )
 {
+  if (hasSubURL())
+  {
+     KURL::List lst = split( *this );
+     KURL &u = lst.last();
+     u.addPath(_txt);
+     *this = join( lst );
+     return;
+  }
+  
   m_strPath_encoded = QString::null;
 
   if ( _txt.isEmpty() )
@@ -1424,12 +1686,8 @@ void KURL::addPath( const QString& _txt )
 
   int i = 0;
   int len = m_strPath.length();
-  // NB: avoid three '/' when building a new path from nothing
-  if ( len == 0 ) {
-    while( _txt[i] == '/' ) ++i;
-  }
   // Add the trailing '/' if it is missing
-  else if ( _txt[0] != '/' && ( len == 0 || m_strPath[ len - 1 ] != '/' ) )
+  if ( _txt[0] != '/' && ( len == 0 || m_strPath[ len - 1 ] != '/' ) )
     m_strPath += "/";
 
   // No double '/' characters
@@ -1502,7 +1760,7 @@ bool KURL::cd( const QString& _dir )
   }
 
   // Users home directory on the local disk ?
-  if ( ( _dir[0] == '~' ) && ( m_strProtocol == "file" ))
+  if ( ( _dir[0] == '~' ) && ( m_strProtocol == fileProt ))
   {
     m_strPath_encoded = QString::null;
     m_strPath = QDir::homeDirPath();
@@ -1520,7 +1778,7 @@ bool KURL::cd( const QString& _dir )
   // append '/' if necessary
   QString p = path(1);
   p += _dir;
-  p = cleanpath( p );
+  p = cleanpath( p, true, false );
   setPath( p );
 
   setHTMLRef( QString::null );
@@ -1534,7 +1792,7 @@ KURL KURL::upURL( ) const
   if (!query().isEmpty())
   {
      KURL u(*this);
-     u.setQuery(QString::null);
+     u._setQuery(QString::null);
      return u;
   };
 
@@ -1589,13 +1847,13 @@ void KURL::setHTMLRef( const QString& _ref )
 {
   if ( !hasSubURL() )
   {
-    m_strRef_encoded = encode( _ref, true, 0 /*?*/);
+    m_strRef_encoded = encode( _ref, 0, 0 /*?*/);
     return;
   }
 
   List lst = split( *this );
 
-  (*lst.begin()).setRef( encode( _ref, true, 0 /*?*/) );
+  (*lst.begin()).setRef( encode( _ref, 0, 0 /*?*/) );
 
   *this = join( lst );
 }
@@ -1615,6 +1873,7 @@ void
 KURL::setProtocol( const QString& _txt )
 {
    m_strProtocol = _txt;
+   if ( m_iUriMode == Auto ) m_iUriMode = uriModeForProtocol( m_strProtocol );
    m_bIsMalformed = false;
 }
 
@@ -1633,7 +1892,23 @@ KURL::setPass( const QString& _txt )
 void
 KURL::setHost( const QString& _txt )
 {
-   m_strHost = _txt;
+  if ( m_iUriMode == Auto )
+    m_iUriMode = URL;
+  switch ( m_iUriMode )
+  {
+  case URL:
+#ifndef KDE_QT_ONLY
+   m_strHost = KIDNA::toUnicode(_txt);
+   if (m_strHost.isEmpty())
+      m_strHost = _txt.lower(); // Probably an invalid hostname, but...
+#else
+   m_strHost = _txt.lower();
+#endif
+    break;
+  default:
+    m_strHost = _txt;
+    break;
+  }
 }
 
 void
@@ -1647,22 +1922,37 @@ void KURL::setPath( const QString & path )
   if (isEmpty())
     m_bIsMalformed = false;
   if (m_strProtocol.isEmpty())
-    m_strProtocol = "file";
+  {
+    m_strProtocol = fileProt;
+  }
   m_strPath = path;
   m_strPath_encoded = QString::null;
+  if ( m_iUriMode == Auto )
+    m_iUriMode = URL;
+}
+
+void KURL::setDirectory( const QString &dir)
+{
+  if ( dir.endsWith("/"))
+     setPath(dir);
+  else
+     setPath(dir+"/");
 }
 
 void KURL::setQuery( const QString &_txt, int encoding_hint)
 {
-   if (!_txt.length())
-   {
-      m_strQuery_encoded = _txt;
-      return;
-   }
-   if (_txt[0] =='?')
-      m_strQuery_encoded = _txt.mid(1);
+   if (_txt[0] == '?')
+      _setQuery( _txt.mid(1), encoding_hint );
    else
-      m_strQuery_encoded = _txt;
+      _setQuery( _txt, encoding_hint );
+}
+
+// This is a private function that expects a query without '?'
+void KURL::_setQuery( const QString &_txt, int encoding_hint)
+{
+   m_strQuery_encoded = _txt;
+   if (!_txt.length())
+      return;
 
    int l = m_strQuery_encoded.length();
    int i = 0;
@@ -1710,12 +2000,12 @@ QString KURL::decode_string(const QString &str, int encoding_hint)
 
 QString KURL::encode_string(const QString &str, int encoding_hint)
 {
-   return encode(str, false, encoding_hint);
+   return encode(str, 1, encoding_hint);
 }
 
 QString KURL::encode_string_no_slash(const QString &str, int encoding_hint)
 {
-   return encode(str, true, encoding_hint);
+   return encode(str, 0, encoding_hint);
 }
 
 bool urlcmp( const QString& _url1, const QString& _url2 )
@@ -1773,6 +2063,10 @@ bool urlcmp( const QString& _url1, const QString& _url2, bool _ignore_trailing,
 }
 
 QMap< QString, QString > KURL::queryItems( int options ) const {
+  return queryItems(options, 0);
+}
+
+QMap< QString, QString > KURL::queryItems( int options, int encoding_hint ) const {
   if ( m_strQuery_encoded.isEmpty() )
     return QMap<QString,QString>();
 
@@ -1788,9 +2082,9 @@ QMap< QString, QString > KURL::queryItems( int options ) const {
       if ( value.isEmpty() )
 	result.insert( name, QString::fromLatin1("") );
       else {
-	// ### why is decoding name not neccessary?
+	// ### why is decoding name not necessary?
 	value.replace( '+', ' ' ); // + in queries means space
-	result.insert( name, decode_string( value ) );
+	result.insert( name, decode_string( value, encoding_hint ) );
       }
     } else if ( equal_pos < 0 ) { // no =
       QString name = (*it);
@@ -1804,6 +2098,11 @@ QMap< QString, QString > KURL::queryItems( int options ) const {
 }
 
 QString KURL::queryItem( const QString& _item ) const
+{
+  return queryItem( _item, 0 );
+}
+
+QString KURL::queryItem( const QString& _item, int encoding_hint ) const
 {
   QString item = _item + '=';
   if ( m_strQuery_encoded.length() <= 1 )
@@ -1819,7 +2118,7 @@ QString KURL::queryItem( const QString& _item ) const
       {
         QString str = (*it).mid( _len );
         str.replace( '+', ' ' ); // + in queries means space.
-        return decode_string( str );
+        return decode_string( str, encoding_hint );
       }
       else // empty value
         return QString::fromLatin1("");
@@ -1855,7 +2154,7 @@ void KURL::removeQueryItem( const QString& _item )
 void KURL::addQueryItem( const QString& _item, const QString& _value, int encoding_hint )
 {
   QString item = _item + '=';
-  QString value = encode( _value, true, encoding_hint );
+  QString value = encode( _value, 0, encoding_hint );
 
   if (!m_strQuery_encoded.isEmpty())
      m_strQuery_encoded += '&';
@@ -1876,3 +2175,110 @@ KURL KURL::fromPathOrURL( const QString& text )
 
     return url;
 }
+
+static QString _relativePath(const QString &base_dir, const QString &path, bool &isParent)
+{
+   QString _base_dir(QDir::cleanDirPath(base_dir));
+   QString _path(QDir::cleanDirPath(path.isEmpty() || (path[0] != '/') ? _base_dir+"/"+path : path));
+
+   if (_base_dir.isEmpty())
+      return _path;
+
+   if (_base_dir[_base_dir.length()-1] != '/')
+      _base_dir.append('/');
+
+   QStringList list1 = QStringList::split('/', _base_dir);
+   QStringList list2 = QStringList::split('/', _path);
+                                                                   
+   // Find where they meet
+   uint level = 0;
+   uint maxLevel = QMIN(list1.count(), list2.count());
+   while((level < maxLevel) && (list1[level] == list2[level])) level++;
+  
+   QString result;                                                                       
+   // Need to go down out of the first path to the common branch.
+   for(uint i = level; i < list1.count(); i++)
+      result.append("../");
+
+   // Now up up from the common branch to the second path.
+   for(uint i = level; i < list2.count(); i++)
+      result.append(list2[i]).append("/");
+
+   if ((level < list2.count()) && (path[path.length()-1] != '/'))
+      result.truncate(result.length()-1);
+
+   isParent = (level == list1.count());
+
+   return result;
+}
+
+QString KURL::relativePath(const QString &base_dir, const QString &path, bool *isParent)
+{
+   bool parent = false;
+   QString result = _relativePath(base_dir, path, parent);
+   if (parent)
+      result.prepend("./");
+      
+   if (isParent)
+      *isParent = parent;
+   
+   return result;
+}
+
+
+QString KURL::relativeURL(const KURL &base_url, const KURL &url, int encoding_hint)
+{
+   if ((url.protocol() != base_url.protocol()) ||
+       (url.host() != base_url.host()) ||
+       (url.port() && url.port() != base_url.port()) ||
+       (url.hasUser() && url.user() != base_url.user()) ||
+       (url.hasPass() && url.pass() != base_url.pass()))
+   {
+      return url.url(0, encoding_hint);
+   }
+
+   QString relURL;
+   
+   if ((base_url.path() != url.path()) || (base_url.query() != url.query()))
+   {
+      bool dummy;
+      QString basePath = base_url.directory(false, false);
+      relURL = encode( _relativePath(basePath, url.path(), dummy), 1, encoding_hint);
+      relURL += url.query();
+   }
+
+   if ( url.hasRef() )
+   {
+      relURL += "#";
+      relURL += url.ref();
+   }
+
+   if ( relURL.isEmpty() )
+      return "./";
+
+   return relURL;
+}
+
+int KURL::uriMode() const
+{
+  return m_iUriMode;
+}
+
+KURL::URIMode KURL::uriModeForProtocol(const QString& protocol)
+{
+#ifndef KDE_QT_ONLY
+    KURL::URIMode mode = Auto;
+    if (KGlobal::_instance)
+        mode = KProtocolInfo::uriParseMode(protocol);
+    if (mode == Auto ) {
+#else
+        KURL::URIMode mode = Auto;
+#endif
+	if ( protocol == "ed2k" || protocol == "sig2dat" || protocol == "slsk" || protocol == "data" ) mode = RawURI;
+	else if ( protocol == "mailto" ) mode = Mailto;
+	else mode = URL;
+#ifndef KDE_QT_ONLY
+    }
+#endif
+    return mode;
+}
