@@ -18,10 +18,11 @@
  **/
 
 #ifndef __kurl_h__
-#define __kurl_h__ "$Id: kurl.h,v 1.102.2.2 2003/02/22 09:54:36 kalass Exp $"
+#define __kurl_h__ "$Id: kurl.h,v 1.130 2003/12/10 13:55:53 waba Exp $"
 
 #include <qstring.h>
 #include <qvaluelist.h>
+#include "kdemacros.h"
 
 class QUrl;
 class QStringList;
@@ -32,16 +33,81 @@ class KURLPrivate;
  * Represents and parses a URL.
  *
  * A prototypical URL looks like:
- * <pre>
- *   protocol:/user:password@hostname:port/path/to/file.ext#reference
- * </pre>
+ * \code
+ *   protocol://user:password\@hostname:port/path/to/file.ext#reference
+ * \endcode
+ *
+ * KURL handles escaping of URLs. This means that the specification
+ * of a full URL will differ from the corresponding string that would specify a
+ * local file or directory in file-operations like fopen. This is because an URL
+ * doesn't allow certain characters and escapes them. (e.g. '#'->"%23", space->"%20")
+ * (In a URL the hash-character '#' is used to specify a "reference", i.e. the position
+ * within a document).
+ *
+ * The constructor KURL(const QString&) expects a string properly escaped,
+ * or at least non-ambiguous.
+ * For instance a local file or directory "/bar/#foo#" would have the URL
+ * file:/bar/%23foo%23.
+ * If you have the absolute path and need the URL-escaping you should create
+ * KURL via the default-constructor and then call setPath(const QString&).
+ * \code
+ *     KURL kurl;
+ *     kurl.setPath("/bar/#foo#");
+ *     QString url = kurl.url();    // -> "file:/bar/%23foo%23"
+ * \endcode
+ *
+ * If you have the URL of a local file or directory and need the absolute path,
+ * you would use path().
+ * \code
+ *    KURL url( "file:/bar/%23foo%23" ); 
+ *    ...
+ *    if ( url.isLocalFile() )
+ *       QString path = url.path();       // -> "/bar/#foo#"
+ * \endcode
+ *
+ * The other way round: if the user can enter a string, that can be either a
+ * path or a URL, then you need to use KURL::fromPathOrURL() to build a KURL.
+ *
+ * This must also be considered, when you have separated directory and file
+ * strings and need to put them together.
+ * While you can simply concatenate normal path strings, you must take care if
+ * the directory-part is already an escaped URL.
+ * (This might be needed if the user specifies a relative path, and your
+ * program supplies the rest from elsewhere.)
+ *
+ * Wrong:
+ * \code
+ *    QString dirUrl = "file:/bar/";
+ *    QString fileName = "#foo#";
+ *    QString invalidURL = dirUrl + fileName;   // -> "file:/bar/#foo#" won't behave like you would expect. 
+ * \endcode
+ * Instead you should use addPath():
+ * Right:
+ * \code
+ *    KURL url( "file:/bar/" );
+ *    QString fileName = "#foo#";
+ *    url.addPath( fileName );
+ *    QString validURL = url.url();    // -> "file:/bar/%23foo%23"
+ * \endcode
+ *
+ * Also consider that some URLs contain the password, but this shouldn't be
+ * visible. Your program should use prettyURL() every time it displays a
+ * URL, whether in the GUI or in debug output or...
+ *
+ * \code
+ *    KURL url( "ftp://name:password@ftp.faraway.org/bar/%23foo%23");
+ *    QString visibleURL = url.prettyURL(); // -> "ftp://name@ftp.faraway.org/bar/%23foo%23"
+ * \endcode
+ * Note that prettyURL() doesn't change the character escapes (like "%23").
+ * Otherwise the URL would be invalid and the user wouldn't be able to use it in another
+ * context.
  *
  * KURL has some restrictions regarding the path
  * encoding. KURL works internally with the decoded path and
  * and encoded query. For example,
- * <pre>
+ * \code
  * http://localhost/cgi-bin/test%20me.pl?cmd=Hello%20you
- * </pre>
+ * \endcode
  * would result in a decoded path "/cgi-bin/test me.pl"
  * and in the encoded query "?cmd=Hello%20you".
  * Since path is internally always encoded you may @em not use
@@ -58,6 +124,38 @@ public:
     StripFileProtocol = 1
   };
 
+  /**
+   * Defines the type of URI we are processing.
+   */
+  enum URIMode
+  {
+    /**
+     * Automatically detected. Using this mode, an appropriate processing
+     * mode will be selected when the URI is first processed.
+     */
+    Auto,
+    /**
+     * Invalid URI. This is something that can't be parsed as a URI at all.
+     * The contents are accessible through the @ref protocol() method.
+     */
+    Invalid,
+    /**
+     * Raw URI. This type of URI should not be processed in any way.
+     * Contents are accessible through the @ref path() method.
+     */
+    RawURI,
+    /**
+     * Standards compliant URL. Process as a syntactically correct URL.
+     */
+    URL,
+    /**
+     * Mailto URI. path() contains an email address which should have its
+     * domain part processed as a DNS name. The email address is accessible
+     * through the @ref path() method.
+     */
+    Mailto
+  };
+
   /**
    * KURL::List is a QValueList that contains KURLs with a few
    * convenience methods.
@@ -111,7 +209,7 @@ public:
    *             This means that if you have a usual UNIX like path you
    *             should not use this constructor.
    *             Instead create an empty url and set the path by using
-   *             @ref setPath().
+   * setPath().
    * @param encoding_hint MIB of original encoding of URL.
    *             @see QTextCodec::mibEnum()
    */
@@ -136,13 +234,14 @@ public:
    * @see QTextCodec::mibEnum()
    */
   KURL( const QCString& url, int encoding_hint = 0 );
+
   /**
    * Copy constructor.
    * @param u the KURL to copy
    */
   KURL( const KURL& u );
   /**
-   * Converts from a @ref QUrl.
+   * Converts from a QUrl.
    * @param u the QUrl
    */
   KURL( const QUrl &u );
@@ -172,6 +271,13 @@ public:
    **/
   void setProtocol( const QString& _txt );
 
+  /**
+   * Returns the URI processing mode for the URL.
+   * @return the URI processing mode set for this URL.
+   * @since 3.2
+   **/
+  int uriMode() const;
+
   /**
    * Returns the decoded user name (login, user id, ...) included in the URL.
    * @return the user name or QString::null if there is no user name
@@ -191,19 +297,19 @@ public:
   bool hasUser() const { return !m_strUser.isEmpty(); }
 
   /**
-   * Returns the decoded password (corresponding to @ref user()) included in the URL.
+   * Returns the decoded password (corresponding to user()) included in the URL.
    * @return the password or QString::null if it does not exist
    **/
   QString pass() const { return m_strPass; }
   /**
-   * Sets the password (corresponding to @ref user()) included in the URL.
+   * Sets the password (corresponding to user()) included in the URL.
    *
    * Special characters in the password will appear encoded in the URL.
    * Note that a password can only appear in a URL string if you also set
    * a user.
    * @param _txt the password to set or QString::null to remove the password
-   * @see #setUser
-   * @see #hasUser
+   * @see setUser
+   * @see hasUser
    **/
   void setPass( const QString& _txt );
   /**
@@ -217,6 +323,7 @@ public:
    * @return the name of the host or QString::null if no host is set
    **/
   QString host() const { return m_strHost; }
+
   /**
    * Sets the hostname included in the URL.
    *
@@ -254,8 +361,8 @@ public:
    *                  a trailing '/' if there is none yet and 0 returns the
    *                  path unchanged. If the URL has no path, then no '/' is added
    *                  anyway. And on the other side: If the path is "/", then this
-   *                  character won't be stripped. Reason: "ftp://weis@host" means something
-   *                  completely different than "ftp://weis@host/". So adding or stripping
+   *                  character won't be stripped. Reason: "ftp://weis\@host" means something
+   *                  completely different than "ftp://weis\@host/". So adding or stripping
    *                  the '/' would really alter the URL, while "ftp://host/path" and
    *                  "ftp://host/path/" mean the same directory.
    *
@@ -285,7 +392,7 @@ public:
    * Some servers seem not to like the removal of extra '/'
    * even though it is against the specification in RFC 2396.
    *
-   * @param cleanDirSeparator if true, occurrances of consecutive
+   * @param cleanDirSeparator if true, occurrences of consecutive
    * directory separators (e.g. /foo//bar) are cleaned up as well.
    */
   void cleanPath(bool cleanDirSeparator = true);
@@ -296,8 +403,8 @@ public:
    *                  a trailing '/' if there is none yet and 0 returns the
    *                  path unchanged. If the URL has no path, then no '/' is added
    *                  anyway. And on the other side: If the path is "/", then this
-   *                  character won't be stripped. Reason: "ftp://weis@host" means something
-   *                  completely different than "ftp://weis@host/". So adding or stripping
+   *                  character won't be stripped. Reason: "ftp://weis\@host" means something
+   *                  completely different than "ftp://weis\@host/". So adding or stripping
    *                  the '/' would really alter the URL, while "ftp://host/path" and
    *                  "ftp://host/path/" mean the same directory.
    */
@@ -327,8 +434,8 @@ public:
    *                  a trailing '/' if there is none yet and 0 returns the
    *                  path unchanged. If the URL has no path, then no '/' is added
    *                  anyway. And on the other side: If the path is "/", then this
-   *                  character won't be stripped. Reason: "ftp://weis@host" means something
-   *                  completely different than "ftp://weis@host/". So adding or stripping
+   *                  character won't be stripped. Reason: "ftp://weis\@host" means something
+   *                  completely different than "ftp://weis\@host/". So adding or stripping
    *                  the '/' would really alter the URL, while "ftp://host/path" and
    *                  "ftp://host/path/" mean the same directory.
    * @param _no_empty_path If set to true then an empty path is substituted by "/".
@@ -382,9 +489,9 @@ public:
   /**
    * Returns the HTML reference (the part of the URL after "#").
    * @return The HTML-style reference.
-   * @see #split
-   * @see #hasSubURL
-   * @see #encodedHtmlRef
+   * @see split
+   * @see hasSubURL
+   * @see encodedHtmlRef
    */
   QString htmlRef() const;
 
@@ -399,7 +506,7 @@ public:
    * Sets the HTML-style reference.
    *
    * @param _ref The new reference. This is considered to be @em not encoded in
-   *         contrast to @ref setRef(). Use QString::null to remove it.
+   *         contrast to setRef(). Use QString::null to remove it.
    * @see htmlRef()
    */
   void setHTMLRef( const QString& _ref );
@@ -418,9 +525,9 @@ public:
    */
   bool isValid() const  { return !m_bIsMalformed; }
   /**
-   * @deprecated
+   * @deprecated Use !isValid() instead.
    */
-  bool isMalformed() const { return !isValid(); }
+  KDE_DEPRECATED bool isMalformed() const { return !isValid(); }
 
   /**
    * Checks whether the file is local.
@@ -446,10 +553,10 @@ public:
   QString fileEncoding() const;
 
   /**
-   * Checks whether the URL has any sub URLs. See @ref #split()
+   * Checks whether the URL has any sub URLs. See split()
    * for examples for sub URLs.
    * @return true if the file has at least one sub URL.
-   * @see #split
+   * @see split
    */
   bool hasSubURL() const;
 
@@ -461,22 +568,29 @@ public:
    * '/' or not. If there is none, it becomes appended. If @p _txt
    * has a leading '/' then this one is stripped.
    *
-   * @param _txt The text to add. It is considered to be decoded.
+   * @param txt The text to add. It is considered to be decoded.
    */
-  void addPath( const QString& _txt );
+  void addPath( const QString& txt );
+
+  /**
+   * call the function below with encoding_hint = 0
+   * (will be merged for KDE4)
+   */
+  QString queryItem( const QString& item ) const;
 
   /**
    * Returns the value of a certain query item.
    *
-   * @param _item Item whose value we want
+   * @param item Item whose value we want
+   * @param encoding_hint MIB of encoding of query.
    *
    * @return the value of the given query item name or QString::null if the
    * specified item does not exist.
    */
-  QString queryItem( const QString& _item ) const;
+  QString queryItem( const QString& item, int encoding_hint ) const;
 
   /**
-   * Options for @ref #queryItems. Currently, only one option is
+   * Options for queryItems. Currently, only one option is
    * defined:
    *
    * @param CaseInsensitiveKeys normalize query keys to lowercase.
@@ -485,22 +599,28 @@ public:
    **/
   enum QueryItemsOptions { CaseInsensitiveKeys = 1 };
 
+  /**
+   * @internal, override for the below function
+   */
+  QMap< QString, QString > queryItems( int options=0 ) const;
+
   /**
    * Returns the list of query items as a map mapping keys to values.
    *
-   * @param options any of @ref QueryItemsOptions <em>or</or>ed together.
+   * @param options any of QueryItemsOptions <em>or</em>ed together.
+   * @param encoding_hint MIB of encoding of query.
    *
    * @return the map of query items or the empty map if the url has no
    * query items.
    *
    * @since 3.1
    */
-  QMap< QString, QString > queryItems( int options=0 ) const;
+  QMap< QString, QString > queryItems( int options, int encoding_hint ) const;
 
   /**
    * Add an additional query item.
    * To replace an existing query item, the item should first be
-   * removed with @ref removeQueryItem()
+   * removed with removeQueryItem()
    *
    * @param _item Name of item to add
    * @param _value Value of item to add
@@ -518,7 +638,7 @@ public:
 
   /**
    * Sets the filename of the path.
-   * In comparison to @ref addPath() this function does not assume that the current
+   * In comparison to addPath() this function does not assume that the current
    * path is a directory. This is only assumed if the current path ends with '/'.
    *
    * Any reference is reset.
@@ -557,6 +677,11 @@ public:
   QString directory( bool _strip_trailing_slash_from_result = true,
 		     bool _ignore_trailing_slash_in_path = true ) const;
 
+  /**
+   * Set the directory to @p dir, leaving the filename empty.
+   */
+  void setDirectory(const QString &dir);
+
   /**
    * Changes the directory by descending into the given directory.
    * It is assumed the current URL represents a directory.
@@ -573,29 +698,52 @@ public:
   bool cd( const QString& _dir );
 
   /**
-   * Returns the URL as string.
-   * @return The complete URL, with all escape sequences intact, encoded
-   * in a given charset.
+   * Returns the URL as string, with all escape sequences intact,
+   * encoded in a given charset.
    * This is used in particular for encoding URLs in UTF-8 before using them
    * in a drag and drop operation.
+   * Please note that the string returned by url() will include
+   * the password of the URL. If you want to show the URL to the
+   * user, use prettyURL().
    *
    * @param _trailing This may be ( -1, 0 +1 ). -1 strips a trailing '/' from the path, +1 adds
    *                  a trailing '/' if there is none yet and 0 returns the
    *                  path unchanged.
    * @param encoding_hint MIB of encoding to use.
+   * @return The complete URL, with all escape sequences intact, encoded
+   * in a given charset.
    * @see QTextCodec::mibEnum()
+   * @see prettyURL()
    */
   QString url( int _trailing = 0, int encoding_hint = 0) const;
 
   /**
    * Returns the URL as string in human-friendly format.
-   * Example: http://localhost:8080/test.cgi?test=hello world&name=fred
+   * Example:
+   * \code
+   * http://localhost:8080/test.cgi?test=hello world&name=fred
+   * \endcode
+   * @param _trailing -1 to strip a trailing '/' from the path, +1 adds
+   *                  a trailing '/' if there is none yet and 0 returns the
+   *                  path unchanged.
    * @return A human readable URL, with no non-necessary encodings/escaped
    * characters. Password will not be shown.
+   * @see url()
    */
   QString prettyURL( int _trailing = 0) const;
 
-
+  /**
+   * Returns the URL as string, escaped for HTML.
+   * Example:
+   * \code
+   * http://localhost:8080/test.cgi?test=hello world&name=fred
+   * \endcode
+   * @return A human readable URL, with no non-necessary encodings/escaped
+   * characters. Password will not be shown.
+   */
+  QString prettyURL( int _trailing, AdjustementFlags _flags) const;
+  // ### BIC: Merge the two above + spell it as "Adjustment"
+  
   /**
    * Returns the URL as string, escaped for HTML.
    * @return A human readable URL, with no non-necessary encodings/escaped
@@ -604,14 +752,6 @@ public:
    */
   QString htmlURL() const;
 
-  /**
-   * Returns the URL as string, escaped for HTML.
-   * Example: http://localhost:8080/test.cgi?test=hello world&name=fred
-   * @return A human readable URL, with no non-necessary encodings/escaped
-   * characters. Password will not be shown.
-   */
-  QString prettyURL( int _trailing, AdjustementFlags _flags) const;
-  // ### BIC: Merge the two above
 
   /**
    * Test to see if the KURL is empty.
@@ -621,7 +761,7 @@ public:
 
   /**
    * This function is useful to implement the "Up" button in a file manager for example.
-   * @ref cd() never strips a sub-protocol. That means that if you are in
+   * cd() never strips a sub-protocol. That means that if you are in
    * file:/home/x.tgz#gzip:/#tar:/ and hit the up button you expect to see
    * file:/home. The algorithm tries to go up on the right-most URL. If that is not
    * possible it strips the right most URL. It continues stripping URLs.
@@ -629,6 +769,8 @@ public:
    */
   KURL upURL( ) const;
 
+  bool operator<(const KURL& _u) const;
+
   KURL& operator=( const KURL& _u );
   KURL& operator=( const QString& _url );
   KURL& operator=( const char * _url );
@@ -642,23 +784,25 @@ public:
   /**
    * The same as equals(), just with a less obvious name.
    * Compares this url with @p u.
+   * @param u the URL to compare this one with.
    * @param ignore_trailing set to true to ignore trailing '/' characters.
    * @return true if both urls are the same
    * @see operator==. This function should be used if you want to
    * ignore trailing '/' characters.
-   * @deprecated
+   * @deprecated Use equals() instead.
    */
-  bool cmp( const KURL &u, bool ignore_trailing = false ) const;
+  bool cmp( const KURL &u, bool ignore_trailing = false ) const KDE_DEPRECATED;
 
   /**
    * Compares this url with @p u.
+   * @param u the URL to compare this one with.
    * @param ignore_trailing set to true to ignore trailing '/' characters.
    * @return true if both urls are the same
    * @see operator==. This function should be used if you want to
    * ignore trailing '/' characters.
    * @since 3.1
    */
-  bool equals( const KURL &u, bool ignore_trailing = false ) const;
+  bool equals( const KURL &u, bool ignore_trailing = false ) const; // TODO KDE4: add bool _ignore_ref = false
 
   /**
    * Checks whether the given URL is parent of this URL.
@@ -680,7 +824,7 @@ public:
    *
    * @param _url The URL that has to be split.
    * @return An empty list on error or the list of split URLs.
-   * @see #hasSubURL
+   * @see hasSubURL
    */
   static List split( const QString& _url );
 
@@ -697,12 +841,12 @@ public:
    * @return An empty list on error or the list of split URLs.
    *
    * @param _url The URL that has to be split.
-   * @see #hasSubURL
+   * @see hasSubURL
    */
   static List split( const KURL& _url );
 
   /**
-   * Reverses @ref #split(). Only the first URL may have a reference. This reference
+   * Reverses split(). Only the first URL may have a reference. This reference
    * is considered to be HTML-like and is appended at the end of the resulting
    * joined URL.
    * @param _list the list to join
@@ -712,11 +856,11 @@ public:
 
   /**
    * Creates a KURL object from a QString representing either an absolute path
-   * or a real URL. Use this method instead of 
-   * <pre>
+   * or a real URL. Use this method instead of
+   * \code
    * QString someDir = ...
    * KURL url = someDir;
-   * </pre>
+   * \endcode
    *
    * Otherwise some characters (e.g. the '#') won't be encoded properly.
    * @param text the string representation of the URL to convert
@@ -724,7 +868,7 @@ public:
    * @since 3.1
    */
   static KURL fromPathOrURL( const QString& text );
-    
+
 /**
    * Convenience function.
    *
@@ -771,6 +915,42 @@ public:
    */
   static bool isRelativeURL(const QString &_url);
 
+  /**
+   * Convenience function
+   *
+   * Returns a "relative URL" based on @p base_url that points to @p url.
+   *
+   * If no "relative URL" can be created, e.g. because the protocol
+   * and/or hostname differ between @p base_url and @p url an absolute
+   * URL is returned.
+   * Note that if @p base_url represents a directory, it should contain
+   * a trailing slash.
+   * @param base_url the URL to derive from
+   * @param url new URL
+   * @param encoding_hint MIB of original encoding of @p str .
+   * @see QTextCodec::mibEnum()
+   * @see adjustPath()
+   */
+  static QString relativeURL(const KURL &base_url, const KURL &url, int encoding_hint = 0);
+
+  /**
+   * Convenience function
+   *
+   * Returns a relative path based on @p base_dir that points to @p path.
+   * @param base_dir the base directory to derive from
+   * @param path the new target directory
+   * @param isParent A pointer to a boolean which, if provided, will be set to reflect
+   * whether @p path has @p base_dir is a parent dir.
+   */
+  static QString relativePath(const QString &base_dir, const QString &path, bool *isParent=0);
+
+  /**
+   * Determine which URI mode is suitable for processing URIs of a given protocol.
+   * @param protocol The protocol name.
+   * @since 3.2
+   */
+  static URIMode uriModeForProtocol(const QString& protocol);
+
 #ifdef KDE_NO_COMPAT
 private:
 #endif
@@ -781,9 +961,14 @@ private:
 
 protected:
   void reset();
-  void parse( const QString& _url, int encoding_hint = 0);
+  void parseURL( const QString& _url, int encoding_hint = 0 );
+  void parseRawURI( const QString& _url, int encoding_hint = 0 );
+  void parseMailto( const QString& _url, int encoding_hint = 0 );
+  void parse( const QString& _url, int encoding_hint = 0 );
 
 private:
+  void _setQuery( const QString& _txt, int encoding_hint = 0);
+
   QString m_strProtocol;
   QString m_strUser;
   QString m_strPass;
@@ -792,7 +977,8 @@ private:
   QString m_strRef_encoded;
   QString m_strQuery_encoded;
   bool m_bIsMalformed : 1;
-  int freeForUse      : 7;
+  enum URIMode m_iUriMode : 3;
+  uint freeForUse     : 4;
   unsigned short int m_iPort;
   QString m_strPath_encoded;
 
@@ -803,6 +989,7 @@ private:
 };
 
 /**
+ * \relates KURL
  * Compares URLs. They are parsed, split and compared.
  * Two malformed URLs with the same string representation
  * are nevertheless considered to be unequal.
@@ -811,12 +998,15 @@ private:
 bool urlcmp( const QString& _url1, const QString& _url2 );
 
 /**
+ * \relates KURL
  * Compares URLs. They are parsed, split and compared.
  * Two malformed URLs with the same string representation
  * are nevertheless considered to be unequal.
  * That means no malformed URL equals anything else.
  *
- * @param _ignore_trailing Described in @ref KURL::cmp
+ * @param _url1 A reference URL
+ * @param _url2 A URL that will be compared with the reference URL
+ * @param _ignore_trailing Described in KURL::cmp
  * @param _ignore_ref If true, disables comparison of HTML-style references.
  */
 bool urlcmp( const QString& _url1, const QString& _url2, bool _ignore_trailing, bool _ignore_ref );
