@@ -85,7 +85,7 @@ KVMAllocator::allocate(size_t _size)
 {
    if (!d->tempfile)
    {
-      d->tempfile = new KTempFile("/tmp/", "vmdata");
+      d->tempfile = new KTempFile(QString::null, "vmdata");
       d->tempfile->unlink();
    }
    // Search in free list
@@ -151,7 +151,7 @@ KVMAllocator::free(Block *block_p)
    if (before != d->free_blocks.end())
    {
       Block &block_before = before.data();
-      if ((block_before.start + block_before.size) == block.start)
+      if ((block_before.start + off_t(block_before.size)) == block.start)
       {
          // Merge blocks.
          kdDebug(13020) << "VM merging: Block "<< (long)block_before.start<<
@@ -168,7 +168,7 @@ KVMAllocator::free(Block *block_p)
    if (after != d->free_blocks.end())
    {
       Block &block_after = after.data();
-      if ((block.start + block.size) == block_after.start)
+      if ((block.start + off_t(block.size)) == block_after.start)
       {
          // Merge blocks.
          kdDebug(13020) << "VM merging: Block "<< (long)block.start<<
@@ -185,6 +185,12 @@ KVMAllocator::free(Block *block_p)
  */
 void
 KVMAllocator::copy(void *dest, Block *src, int _offset, size_t length)
+{
+   (void) copyBlock(dest, src, _offset, length);
+}
+
+bool
+KVMAllocator::copyBlock(void *dest, Block *src, int _offset, size_t length)
 {
    //kdDebug(13020)<<"VM read: seek "<<(long)src->start<<" +"<<_offset<<":"<<length<<endl;
    lseek(d->tempfile->handle(), src->start+_offset, SEEK_SET);
@@ -193,14 +199,21 @@ KVMAllocator::copy(void *dest, Block *src, int _offset, size_t length)
    int to_go = length;
    int done = 0;
    char *buf = (char *) dest;
-   while(done < to_go)
+   while(to_go > 0)
    {
       int n = read(d->tempfile->handle(), buf+done, to_go);
-      if (n <= 0) return; // End of data or error
+      if (n <= 0) 
+      {
+         if (n < 0)
+            return false; // Error
+         else
+            return true; // End of data
+      }
       done += n;
       to_go -= n;
    }
    // Done.
+   return true;
 }
 
 /**
@@ -208,6 +221,12 @@ KVMAllocator::copy(void *dest, Block *src, int _offset, size_t length)
  */
 void
 KVMAllocator::copy(Block *dest, void *src, int _offset, size_t length)
+{
+   (void) copyBlock(dest, src, _offset, length);
+}
+
+bool
+KVMAllocator::copyBlock(Block *dest, void *src, int _offset, size_t length)
 {
    //kdDebug(13020)<<"VM write: seek "<<(long)dest->start<<" +"<<_offset<< ":" << length << endl;
    lseek(d->tempfile->handle(), dest->start+_offset, SEEK_SET);
@@ -216,14 +235,15 @@ KVMAllocator::copy(Block *dest, void *src, int _offset, size_t length)
    int to_go = length;
    int done = 0;
    char *buf = (char *) src;
-   while(done < to_go)
+   while(to_go > 0)
    {
       int n = write(d->tempfile->handle(), buf+done, to_go);
-      if (n <= 0) return; // End of data or error
+      if (n <= 0) return false; // Error
       done += n;
       to_go -= n;
    }
    // Done.
+   return true;
 }
 
 /**
