@@ -26,6 +26,7 @@
 #define KVMALLOCATOR_H
 
 #include <sys/types.h>
+#include "kdemacros.h"
 
 class KVMAllocatorPrivate;
 
@@ -34,7 +35,7 @@ class KVMAllocatorPrivate;
  * Memory is allocated block-wise in a tmp file.
  * 
  * @author Waldo Bastian <bastian@kde.org>
- * @version $Id: kvmallocator.h,v 1.5 2002/09/14 17:15:10 tjansen Exp $
+ * @version $Id: kvmallocator.h,v 1.8 2003/08/30 07:33:49 raabe Exp $
  */
 class KVMAllocator
 {
@@ -71,8 +72,16 @@ public:
      * @param src the source block
      * @param _offset the offset in the source block
      * @param length the length of the data to copy
+     * @return true on success, false on failure, see errno for details
+     * @since 3.2
      */
-    void copy(void *dest, Block *src, int _offset = 0, size_t length = 0);
+    bool copyBlock(void *dest, Block *src, int _offset = 0, size_t length = 0);
+
+    /**
+     * @deprecated
+     * @see copyBlock
+     */
+    void copy(void *dest, Block *src, int _offset = 0, size_t length = 0) KDE_DEPRECATED;
      
     /**
      * Copy @p length bytes from normal memory at address @p src to 
@@ -81,8 +90,16 @@ public:
      * @param src the source location of the data
      * @param _offset the offset in the destination block
      * @param length the length of the data to copy
+     * @return true on success, false on failure, see errno for details
+     * @since 3.2
      */
-    void copy(Block *dest, void *src, int _offset = 0, size_t length = 0);  
+    bool copyBlock(Block *dest, void *src, int _offset = 0, size_t length = 0);
+
+    /**
+     * @deprecated
+     * @see copyBlock
+     */
+    void copy(Block *dest, void *src, int _offset = 0, size_t length = 0) KDE_DEPRECATED;  
 
     /**
      * Map a virtual memory block in memory
