@@ -2,7 +2,7 @@
  *  This file is part of the KDE libraries
  *  Copyright (C) 2000,2001 Thiago Macieira <thiagom@mail.com>
  *
- *  $Id: netsupp.cpp,v 1.33.2.1 2002/12/10 16:43:58 lunakl Exp $
+ *  $Id: netsupp.cpp,v 1.39.2.1 2004/04/21 12:20:25 waba Exp $
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Library General Public
@@ -158,7 +158,7 @@ make_unix(const char *name, const char *serv)
     }
 
   _sun->sun_family = AF_UNIX;
-# ifdef HAVE_SOCKADDR_SA_LEN
+# ifdef HAVE_STRUCT_SOCKADDR_SA_LEN
   _sun->sun_len = len;
 # endif
   if (*buf == '/')
@@ -188,6 +188,8 @@ static int check_ipv6_stack()
 # ifndef AF_INET6
   return 2;			// how can we check?
 # else
+  if (getenv("KDE_NO_IPV6"))
+     return 2;
   int fd = ::socket(AF_INET6, SOCK_STREAM, 0);
   if (fd == -1)
      return 2;
@@ -205,9 +207,9 @@ static int check_ipv6_stack()
  * I decided to add this wrapper function for getaddrinfo
  * and have this be called by KExtendedSocket instead of
  * the real getaddrinfo so that we can make sure that the
- * behaviour is the desired one.
+ * behavior is the desired one.
  *
- * Currently, the only "undesired" behaviour is getaddrinfo
+ * Currently, the only "undesired" behavior is getaddrinfo
  * not returning PF_UNIX sockets in some implementations.
  *
  * getaddrinfo and family are defined in POSIX 1003.1g
@@ -473,7 +475,7 @@ static int inet_lookup(const char *name, int portnum, int protonum,
     {
       struct sockaddr_in *sin = (sockaddr_in*)q->ai_addr;
       sin->sin_family = AF_INET;
-# ifdef HAVE_SOCKADDR_SA_LEN
+# ifdef HAVE_STRUCT_SOCKADDR_SA_LEN
       sin->sin_len = sizeof(*sin);
 # endif
       sin->sin_port = portnum;
@@ -484,7 +486,7 @@ static int inet_lookup(const char *name, int portnum, int protonum,
     {
       struct sockaddr_in6 *sin6 = (sockaddr_in6*)q->ai_addr;
       sin6->sin6_family = AF_INET6;
-#  ifdef HAVE_SOCKADDR_SA_LEN
+#  ifdef HAVE_STRUCT_SOCKADDR_SA_LEN
       sin6->sin6_len = sizeof(*sin6);
 #  endif
       sin6->sin6_port = portnum;
@@ -524,7 +526,7 @@ static int inet_lookup(const char *name, int portnum, int protonum,
 	{
 	  struct sockaddr_in *sin = (sockaddr_in*)q->ai_addr;
 	  sin->sin_family = AF_INET;
-# ifdef HAVE_SOCKADDR_SA_LEN
+# ifdef HAVE_STRUCT_SOCKADDR_SA_LEN
 	  sin->sin_len = sizeof(*sin);
 # endif
 	  sin->sin_port = portnum;
@@ -535,7 +537,7 @@ static int inet_lookup(const char *name, int portnum, int protonum,
 	{
 	  struct sockaddr_in6 *sin6 = (sockaddr_in6*)q->ai_addr;
 	  sin6->sin6_family = AF_INET6;
-#  ifdef HAVE_SOCKADDR_SA_LEN
+#  ifdef HAVE_STRUCT_SOCKADDR_SA_LEN
 	  sin6->sin6_len = sizeof(*sin6);
 #  endif
 	  sin6->sin6_port = portnum;
@@ -614,7 +616,7 @@ static int make_inet(const char *name, int portnum, int protonum, struct addrinf
 		}
 
 	      sin6->sin6_family = AF_INET6;
-#  ifdef HAVE_SOCKADDR_SA_LEN
+#  ifdef HAVE_STRUCT_SOCKADDR_SA_LEN
 	      sin6->sin6_len = sizeof(*sin6);
 #  endif
 	      sin6->sin6_port = portnum;
@@ -663,7 +665,7 @@ static int make_inet(const char *name, int portnum, int protonum, struct addrinf
 		}
 
 	      sin->sin_family = AF_INET;
-# ifdef HAVE_SOCKADDR_SA_LEN
+# ifdef HAVE_STRUCT_SOCKADDR_SA_LEN
 	      sin->sin_len = sizeof(*sin);
 # endif
 	      sin->sin_port = portnum;
@@ -714,7 +716,7 @@ static int make_inet(const char *name, int portnum, int protonum, struct addrinf
 	    }
 
 	  sin->sin_family = AF_INET;
-# ifdef HAVE_SOCKADDR_SA_LEN
+# ifdef HAVE_STRUCT_SOCKADDR_SA_LEN
 	  sin->sin_len = sizeof(*sin);
 # endif
 	  sin->sin_port = portnum;
@@ -749,7 +751,7 @@ static int make_inet(const char *name, int portnum, int protonum, struct addrinf
 	    }
 
 	  sin6->sin6_family = AF_INET6;
-#  ifdef HAVE_SOCKADDR_SA_LEN
+#  ifdef HAVE_STRUCT_SOCKADDR_SA_LEN
 	  sin6->sin6_len = sizeof(*sin6);
 #  endif
 	  sin6->sin6_port = portnum;
@@ -1173,7 +1175,7 @@ int inet_pton(int af, const char *cp, void *buf)
 	{
 	  if (has_v4 && inet_pton(AF_INET, p, addr + n) != 0)
 	    {
-	      // successfull v4 convertion
+	      // successful v4 convertion
 	      addr[n] = ntohs(addr[n]);
 	      n++;
 	      addr[n] = ntohs(addr[n]);
@@ -1236,5 +1238,6 @@ int inet_pton(int af, const char *cp, void *buf)
 
 namespace KDE
 {
+  /** @internal */
   extern const int resolverFlags = KRF_getaddrinfo | KRF_resolver | KRF_afinet6 | KRF_inet_ntop | KRF_inet_pton;
 }
