@@ -1,8 +1,8 @@
 /*
  *  This file is part of the KDE libraries
- *  Copyright (C) 2000,2001 Thiago Macieira <thiagom@mail.com>
+ *  Copyright (C) 2000-2003 Thiago Macieira <thiagom@mail.com>
  *
- *  $Id: netsupp.h,v 1.21 2002/01/22 18:28:41 thiago Exp $
+ *  $Id: netsupp.h,v 1.25 2003/11/15 22:07:57 thiago Exp $
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Library General Public
@@ -46,27 +46,25 @@
 
 /*** IPv6 structures that might be missing from some implementations ***/
 
-/**
+/** @internal
  * An IPv6 address.
  * This is taken from RFC 2553
- * @internal
  */
 struct kde_in6_addr
 {
   unsigned char __u6_addr[16];
 };
 
-/**
+/** @internal
  * An IPv6 socket address
  * This is taken from RFC 2553.
- * @internal
  */
 struct kde_sockaddr_in6
 {
-#ifdef HAVE_SOCKADDR_SA_LEN
+#ifdef HAVE_STRUCT_SOCKADDR_SA_LEN
   Q_UINT8		sin6_len;
   Q_UINT8		sin6_family;
-#else  //HAVE_SOCKADDR_SA_LEN
+#else  //HAVE_STRUCT_SOCKADDR_SA_LEN
   Q_UINT16		sin6_family;
 #endif
   unsigned short       	sin6_port;	/* RFC says in_port_t */
@@ -108,19 +106,19 @@ struct kde_sockaddr_in6
 	 (((Q_UINT32 *) (a))[3] == ((Q_UINT32 *) (b))[3]))
 
 #define KDE_IN6_IS_ADDR_MC_NODELOCAL(a) \
-	(IN6_IS_ADDR_MULTICAST(a) && ((((Q_UINT8 *) (a))[1] & 0xf) == 0x1))
+	(KDE_IN6_IS_ADDR_MULTICAST(a) && ((((Q_UINT8 *) (a))[1] & 0xf) == 0x1))
 
 #define KDE_IN6_IS_ADDR_MC_LINKLOCAL(a) \
-	(IN6_IS_ADDR_MULTICAST(a) && ((((Q_UINT8 *) (a))[1] & 0xf) == 0x2))
+	(KDE_IN6_IS_ADDR_MULTICAST(a) && ((((Q_UINT8 *) (a))[1] & 0xf) == 0x2))
 
 #define KDE_IN6_IS_ADDR_MC_SITELOCAL(a) \
-	(IN6_IS_ADDR_MULTICAST(a) && ((((Q_UINT8 *) (a))[1] & 0xf) == 0x5))
+	(KDE_IN6_IS_ADDR_MULTICAST(a) && ((((Q_UINT8 *) (a))[1] & 0xf) == 0x5))
 
 #define KDE_IN6_IS_ADDR_MC_ORGLOCAL(a) \
-	(IN6_IS_ADDR_MULTICAST(a) && ((((Q_UINT8 *) (a))[1] & 0xf) == 0x8))
+	(KDE_IN6_IS_ADDR_MULTICAST(a) && ((((Q_UINT8 *) (a))[1] & 0xf) == 0x8))
 
 #define KDE_IN6_IS_ADDR_MC_GLOBAL(a) \
-	(IN6_IS_ADDR_MULTICAST(a) && ((((Q_UINT8 *) (a))[1] & 0xf) == 0xe))
+	(KDE_IN6_IS_ADDR_MULTICAST(a) && ((((Q_UINT8 *) (a))[1] & 0xf) == 0xe))
 
 #ifdef NEED_IN6_TESTS
 # define IN6_IS_ADDR_UNSPECIFIED	KDE_IN6_IS_ADDR_UNSPECIFIED
@@ -168,6 +166,9 @@ extern void kde_freeaddrinfo(struct kde_addrinfo *p);
 #if !defined(HAVE_GETADDRINFO) || defined(HAVE_BROKEN_GETADDRINFO)
 
 # ifndef HAVE_STRUCT_ADDRINFO
+/**
+ * @internal
+ */
 struct addrinfo
 {
   int ai_flags;			/* Input flags.  */
@@ -247,11 +248,15 @@ struct addrinfo
 
 namespace KDE
 {
+  /** \internal */
   extern int getaddrinfo(const char *name, const char *service,
 			 const struct addrinfo* hint,
 			 struct addrinfo** result);
+  /** \internal */
   extern void freeaddrinfo(struct addrinfo* ai);
+  /** \internal */
   extern char *gai_strerror(int errorcode);
+  /** \internal */
   extern int getnameinfo(const struct sockaddr *sa,
 			 unsigned int salen,
 			 char *host, size_t hostlen,
@@ -271,6 +276,7 @@ namespace KDE
 
 namespace KDE
 {
+  /** \internal */
   extern int inet_pton(int af, const char *cp, void* buf);
 }
 
@@ -281,6 +287,7 @@ namespace KDE
 
 namespace KDE
 {
+  /** \internal */
   extern const char* inet_ntop(int af, const void *cp, char *buf, size_t len);
 }
 
