@@ -1,6 +1,7 @@
 /*
 
   Copyright (c) 2000 Troll Tech AS
+  Copyright (c) 2003 Lubos Lunak <l.lunak@kde.org>
 
   Permission is hereby granted, free of charge, to any person obtaining a
   copy of this software and associated documentation files (the "Software"),
@@ -26,6 +27,7 @@
 #ifndef   __net_wm_h
 #define   __net_wm_h
 
+#include "kdemacros.h"
 #include <qwidget.h>
 #ifdef Q_WS_X11
 #include <X11/Xlib.h>
@@ -54,6 +56,14 @@ struct NETWinInfoPrivate;
 
 class NETRootInfo : public NET {
 public:
+    /**
+        Indexes for the properties array.
+        @since 3.2
+    **/
+    // update also NETRootInfoPrivate::properties[] size when extending this
+    enum { PROTOCOLS, WINDOW_TYPES, STATES, PROTOCOLS2, ACTIONS,
+        PROPERTIES_SIZE };
+
     /**
        Window Managers should use this constructor to create a NETRootInfo object,
        which will be used to set/update information stored on the rootWindow.
@@ -69,18 +79,36 @@ public:
        @param wmName A string which should be the window manager's name (ie. "KWin"
        or "Blackbox").
 
-       @param properties An OR'ed list of all properties and protocols the window
-       manager supports (see the NET base class documentation for a description
-       of all properties and protocols).
+       @param properties An array of elements listing all properties and protocols
+       the window manager supports. The elements contain OR'ed values of constants
+       from the NET base class, in the following order: [0]= NET::Property,
+       [1]= NET::WindowTypeMask (not NET::WindowType!), [2]= NET::State,
+       [3]= NET::Property2, [4]= NET::Action.
+       In future versions, the list may be extended. In case you pass less elements,
+       the missing ones will be replaced with default values.
+       
+       @param properties_size The number of elements in the properties array.
 
        @param screen For Window Managers that support multiple screen (ie.
        "multiheaded") displays, the screen number may be explicitly defined.  If
        this argument is omitted, the default screen will be used.
        
        @param doActivate true to activate the window
+       
+       @since 3.2
     **/
     NETRootInfo(Display *display, Window supportWindow, const char *wmName,
-		unsigned long properties, int screen = -1, bool doActivate = true);
+		const unsigned long properties[], int properties_size,
+                int screen = -1, bool doActivate = true);
+
+    /**
+        @deprecated
+        This constructor differs from the above one only in the way it accepts
+        the list of supported properties. The properties argument is equivalent
+        to the first element of the properties array in the above constructor.
+    **/
+    NETRootInfo(Display *display, Window supportWindow, const char *wmName,
+		unsigned long properties, int screen = -1, bool doActivate = true) KDE_DEPRECATED;
 
     /**
        Clients should use this constructor to create a NETRootInfo object, which
@@ -89,15 +117,29 @@ public:
 
        @param display An X11 Display struct.
 
-       @param properties An OR'ed list of all properties and protocols the client
-       supports (see the NET base class documentation for a description of all
-       properties and protocols).
+       @param properties An array of elements listing all protocols the client
+       is interested in. The elements contain OR'ed values of constants
+       from the NET base class, in the following order: [0]= NET::Property,
+       [1]= NET::Property2.
+
+       @param properties_size The number of elements in the properties array.
 
        @param screen For Clients that support multiple screen (ie. "multiheaded")
        displays, the screen number may be explicitly defined. If this argument is
        omitted, the default screen will be used.
        
        @param doActivate true to activate the window
+
+       @since 3.2
+    **/
+    NETRootInfo(Display *display, const unsigned long properties[], int properties_size,
+                int screen = -1, bool doActivate = true);
+
+    /**
+        This constructor differs from the above one only in the way it accepts
+        the list of supported properties. The properties argument is equivalent
+        to the first element of the properties array in the above constructor,
+        and therefore you cannot read all root window properties using it.
     **/
     NETRootInfo(Display *display, unsigned long properties, int screen = -1,
 		bool doActivate = true);
@@ -150,13 +192,72 @@ public:
     int screenNumber() const;
 
     /**
-       Returns an OR'ed list of supported protocols and properties.
+       Returns true if the given property is supported by the window
+       manager. Note that for Client mode, NET::Supported needs
+       to be passed in the properties argument for this to work.
+       @since 3.2
+    **/
+    bool isSupported( NET::Property property ) const;
+    /**
+       @overload
+       @since 3.2
+    **/
+    bool isSupported( NET::Property2 property ) const;
+    /**
+       @overload
+       @since 3.2
+    **/
+    bool isSupported( NET::WindowType type ) const;
+    /**
+       @overload
+       @since 3.2
+    **/
+    bool isSupported( NET::State state ) const;
+
+    /**
+       @overload
+       @since 3.2
+    **/
+    bool isSupported( NET::Action action ) const;
+
+    /**
+       In the Window Manager mode, this is equivalent to the properties
+       argument passed to the constructor. In the Client mode, if
+       NET::Supported was passed in the properties argument, the returned
+       value is array of all protocols and properties supported
+       by the Window Manager. The elements of the array are the same
+       as they would be passed to the Window Manager mode constructor,
+       the size is the maximum array size the constructor accepts.
        
-       @return an OR'ed list of protocols and properties
+       @since 3.2
+    **/
+    const unsigned long* supportedProperties() const;
+
+    /**
+       Returns the properties argument passed to the constructor.
+       The size is the maximum array size the constructor accepts.
+
+       @since 3.2
+    **/
+    // KDE4 better name?
+    const unsigned long* passedProperties() const;
+
+    /**
+       @deprecated
+       
+       Returns an OR'ed list of protocols passed to the constructor.
+       For the constructor used by Window Managers, this is equivalent
+       to the first element of the properties argument, for the constructor
+       for Clients, it's the properties argument.
+       
+       Clients willing to find out all properties and protocols supported
+       by the WindowManager should use supportedProperties().
+       
+       @return an OR'ed list of protocols
 
        @see NET::Property
     **/
-    unsigned long supported() const;
+    unsigned long supported() const KDE_DEPRECATED;
 
     /**
        Returns an array of Window id's, which contain all managed windows.
@@ -308,6 +409,7 @@ public:
 
        @param count The number of windows in the array
     **/
+    // KDE4 'const Window*', also in the others below
     void setClientList(Window *windows, unsigned int count);
 
     /**
@@ -374,7 +476,23 @@ public:
     void setDesktopName(int desktop, const char *desktopName);
 
     /**
-       Sets the active (focused) window the specified window.
+       Requests that the specified window becomes the active (focused) one.
+
+       @param window the id of the new active window
+       @param src whether the request comes from normal application
+          or from a pager or similar tool
+       @param timestamp X server timestamp of the user action that
+          caused the request
+       @param active_window active window of the requesting application, if any
+          
+       @since 3.2
+    **/
+    void setActiveWindow(Window window, NET::RequestSource src,
+        Time timestamp, Window active_window);
+
+    /**
+       Sets the active (focused) window the specified window. This should
+       be used only in the window manager mode.
 
        @param window the if of the new active window
     **/
@@ -415,8 +533,9 @@ public:
 
     /**
        Clients (such as pagers/taskbars) that wish to start a WMMoveResize
-       (where the window manager controls the resize/movement) should call
-       this function.  This will send a request to the Window Manager.
+       (where the window manager controls the resize/movement,
+       i.e. _NET_WM_MOVERESIZE) should call this function.
+       This will send a request to the Window Manager.
 
        @param window The client window that would be resized/moved.
 
@@ -430,10 +549,46 @@ public:
     void moveResizeRequest(Window window, int x_root, int y_root,
 			   Direction direction);
 
+    /**
+       Clients (such as pagers/taskbars) that wish to move/resize a window
+       using WM2MoveResizeWindow (_NET_MOVERESIZE_WINDOW) should call this function.
+       This will send a request to the Window Manager. See _NET_MOVERESIZE_WINDOW
+       description for details.
+
+       @param window The client window that would be resized/moved.
+       
+       @since 3.2
+    **/
+    void moveResizeWindowRequest(Window window, int flags, int x, int y, int width, int height );
+
+    /**
+       @since 3.2
+    **/
+    void restackRequest(Window window, Window above, int detail);
+    
+    /**
+       This function takes the passed XEvent and returns an OR'ed list of
+       NETRootInfo properties that have changed in the properties argument.
+       The new information will be read immediately by the class.
+       The elements of the properties argument are as they would be passed
+       to the constructor, if the array is not large enough,
+       changed properties that don't fit in it won't be listed there
+       (they'll be updated in the class though).
+       
+       @param event the event
+       @param properties properties that changed
+       @param properties_size size of the passed properties array
+       @since 3.2
+
+    **/
+    void event( XEvent* event, unsigned long* properties, int properties_size );
+    
     /**
        This function takes the passed XEvent and returns an OR'ed list of
        NETRootInfo properties that have changed.  The new information will be
-       read immediately by the class.
+       read immediately by the class. This overloaded version returns
+       only a single mask, and therefore cannot check state of all properties
+       like the other variant.
 
        @param event the event
 
@@ -526,14 +681,16 @@ protected:
     virtual void changeCurrentDesktop(int) { }
 
     /**
+       @deprecated Use NETRootInfo2::changeActiveWindow() instead.
        A Window Manager should subclass NETRootInfo and reimplement this function
        when it wants to know when a Client made a request to change the active
-       (focused) window.
+       (focused) window. The changeActiveWindow() method in NETRootInfo2
+       should be used instead.
 
        @param window the id of the window to activate
     **/
     // virtual void changeActiveWindow(Window window) { }
-    virtual void changeActiveWindow(Window) { }
+    virtual KDE_DEPRECATED void changeActiveWindow(Window) { }
 
     /**
        A Window Manager should subclass NETRootInfo and reimplement this function
@@ -563,16 +720,84 @@ protected:
 
 
 private:
-    void update(unsigned long);
-    void setSupported(unsigned long);
+    void update( const unsigned long[] );
+    void setSupported();
+    void setDefaultProperties();
+    void updateSupportedProperties( Atom atom );
     Role role;
 
 protected:
     virtual void virtual_hook( int id, void* data );
 private:
     NETRootInfoPrivate *p;
+    friend class NETRootInfo2;
 };
 
+/**
+ This class is an extension of the NETRootInfo class, and exists solely
+ for binary compatibility reasons (adds new virtual methods). Simply
+ use it instead of NETRootInfo and override also the added virtual methods.
+ @since 3.2
+*/
+class NETRootInfo2
+    : public NETRootInfo
+{
+public:
+    NETRootInfo2(Display *display, Window supportWindow, const char *wmName,
+		unsigned long properties[], int properties_size,
+                int screen = -1, bool doActivate = true);
+    /**
+      Sends a ping with the given timestamp to the window, using
+      the _NET_WM_PING protocol.
+    */
+    void sendPing( Window window, Time timestamp );
+protected:
+    friend class NETRootInfo;
+    /**
+       A Window Manager should subclass NETRootInfo2 and reimplement this function
+       when it wants to receive replies to the _NET_WM_PING protocol.
+       @param window the window from which the reply came
+       @param timestamp timestamp of the ping
+     */
+    // virtual void gotPing( Window window, Time timestamp ) {};
+    virtual void gotPing( Window, Time ) {};
+    /**
+       A Window Manager should subclass NETRootInfo2 and reimplement this function
+       when it wants to know when a Client made a request to change the active
+       (focused) window.
+
+       @param window the id of the window to activate
+       @param src the source from which the request came
+       @param timestamp the timestamp of the user action causing this request
+       @param active_window active window of the requesting application, if any
+    **/
+    // virtual void changeActiveWindow(Window window,NET::RequestSource src,
+    //     Time timestamp, Window active_window ) { }
+    virtual void changeActiveWindow(Window,NET::RequestSource,Time,Window) { }
+    /**
+       A Window Manager should subclass NETRootInfo2 and reimplement this function
+       when it wants to know when a Client made a request to restack a window.
+       See _NET_RESTACK_WINDOW for details.
+
+       @param window the id of the window to restack
+       @param above other window in the restack request
+       @param detail restack detail
+    **/
+    // virtual void restackWindow(Window window, Window above, int detail) { }
+    virtual void restackWindow(Window, Window, int) { }
+
+    /**
+       A Window Manager should subclass NETRootInfo2 and reimplement this function
+       when it wants to know when a pager made a request to move/resize a window.
+       See _NET_MOVERESIZE_WINDOW for details.
+
+       @param window the id of the window to more/resize
+    **/
+    // virtual void moveResizeWindow(Window window, int flags, int x, int y, int width, int height) { }
+    virtual void moveResizeWindow( Window, int, int, int, int, int ) { }
+
+// no private data, use NETRootInfoPrivate
+};
 
 /**
    Common API for application window properties/protocols.
@@ -590,6 +815,13 @@ private:
 
 class NETWinInfo : public NET {
 public:
+    /**
+        Indexes for the properties array.
+        @since 3.2
+    **/
+    // update also NETWinInfoPrivate::properties[] size when extending this
+    enum { PROTOCOLS, PROTOCOLS2,
+        PROPERTIES_SIZE };
     /**
        Create a NETWinInfo object, which will be used to set/read/change
        information stored on an application window.
@@ -600,12 +832,29 @@ public:
 
        @param rootWindow The Window id of the root window.
 
-       @param properties An OR'ed list of all properties and protocols the
-       client/window manager supports (see the NET base class documentation
-       for a description of all properties and protocols).
+       @param properties An array of elements listing all properties the client
+       is interested in.The elements contain OR'ed values of constants
+       from the NET base class, in the following order: [0]= NET::Property,
+       [1]= NET::Property2.
+       In future versions, the list may be extended. In case you pass less elements,
+       the missing ones will be replaced with default values.
+       
+       @param properties_size The number of elements in the properties array.
 
        @param role Select the application role.  If this argument is omitted,
        the role will default to Client.
+       
+       @since 3.2
+    **/
+    NETWinInfo(Display *display, Window window, Window rootWindow,
+               const unsigned long properties[], int properties_size,
+	       Role role = Client);
+
+    /**
+        This constructor differs from the above one only in the way it accepts
+        the list of properties the client is interested in. The properties argument
+        is equivalent to the first element of the properties array
+        in the above constructor.
     **/
     NETWinInfo(Display *display, Window window,
 	       Window rootWindow, unsigned long properties,
@@ -624,13 +873,40 @@ public:
     virtual ~NETWinInfo();
 
     /**
-       Returns an OR'ed list of supported protocols and properties.
+       Assignment operator.  Ensures that the shared data reference counts are
+       correct.
+    **/
+    const NETWinInfo &operator=(const NETWinInfo &wintinfo);
+
+    /**
+       Returns true if the window has any window type set, even if the type
+       itself is not known to this implementation. Presence of a window type
+       as specified by the NETWM spec is considered as the window supporting
+       this specification.
+       @since 3.2
+       @return true if the window has support for the NETWM spec
+    **/
+    bool hasNETSupport() const;
+
+    /**
+       Returns the properties argument passed to the constructor.
+       The size is the maximum array size the constructor accepts.
+
+       @since 3.2
+    **/
+    // KDE4 better name?
+    const unsigned long* passedProperties() const;
+
+    /**
+       @deprecated
        
-       @return an OR'ed list of protocols and properties
+       Returns an OR'ed list of protocols passed to the constructor.
+       
+       @return an OR'ed list of protocols
 
        @see NET::Property
     **/
-    unsigned long properties() const;
+    unsigned long properties() const KDE_DEPRECATED;
 
     /**
        Returns the icon geometry.
@@ -657,10 +933,27 @@ public:
     /**
        Returns the window type for this client (see the NET base class
        documentation for a description of the various window types).
+       Since clients may specify several windows types for a window
+       in order to support backwards compatibility and extensions
+       not available in the NETWM spec, you should specify all
+       window types you application supports (see the NET::WindowTypeMask
+       mask values for various window types). This method will
+       return the first window type that is listed in the supported types,
+       or NET::Unknown if none of the window types is supported.
+
+       @return the type of the window
+       @since 3.2
+    **/
+    WindowType windowType( unsigned long supported_types ) const;
+
+    /**
+       @deprecated
+       Returns the window type for this client (see the NET base class
+       documentation for a description of the various window types).
 
        @return the type of the window
     **/
-    WindowType windowType() const;
+    WindowType windowType() const KDE_DEPRECATED;
 
     /**
        Returns the name of the window in UTF-8 format.
@@ -677,16 +970,24 @@ public:
     const char *visibleName() const;
 
     /**
-       Returns the icon name of the window in UTF-8 format.
+       Returns the iconic name of the window in UTF-8 format. Note that this has
+       nothing to do with icons, but it's for "iconic"
+       representations of the window (taskbars etc.), that should be shown
+       when the window is in iconic state. See description of _NET_WM_ICON_NAME
+       for details.
        
-       @return the name of the icon
+       @return the iconic name
     **/
     const char *iconName() const;
 
     /**
-       Returns the visible icon name as set by the window manager in UTF-8 format.
+       Returns the visible iconic name as set by the window manager in UTF-8 format.
+       Note that this has nothing to do with icons, but it's for "iconic"
+       representations of the window (taskbars etc.), that should be shown
+       when the window is in iconic state. See description of _NET_WM_VISIBLE_ICON_NAME
+       for details.
 
-       @return the visible icon name
+       @return the visible iconic name
     **/
     const char *visibleIconName() const;
 
@@ -756,7 +1057,9 @@ public:
 
     /**
        Set the state for the application window (see the NET base class documentation
-       for a description of window state).
+       for a description of window state). Note that the constructor needs to be
+       passed NET::WMState in properties even if it's only used for setting the state
+       without reading it.
 
        @param state the name state
 
@@ -788,17 +1091,17 @@ public:
     void setVisibleName(const char *visibleName);
 
     /**
-       Sets the icon name for the application window.
+       Sets the iconic name for the application window.
 
-       @param name the new icon name
+       @param name the new iconic name
     **/
     void setIconName(const char *name);
 
     /**
-       For Window Managers only: set the visible icon name ( i.e. xterm, xterm <2>,
+       For Window Managers only: set the visible iconic name ( i.e. xterm, xterm <2>,
        xterm <3>, ... )
 
-       @param visibleName the new visible icon name
+       @param visibleName the new visible iconic name
     **/
     void setVisibleIconName(const char *name);
 
@@ -845,14 +1148,67 @@ public:
        the closest it can find (the next biggest).  If width and height are omitted,
        then the first icon in the list is returned.
 
-       @param width the prefered width for the icon, -1 to ignore
+       @param width the preferred width for the icon, -1 to ignore
 
-       @param height the prefered height for the icon, -1 to ignore
+       @param height the preferred height for the icon, -1 to ignore
 
        @return the icon
     **/
     NETIcon icon(int width = -1, int height = -1) const;
 
+    /*    
+     * Sets user timestamp @p time on the window (property _NET_WM_USER_TIME).
+     * The timestamp is expressed as XServer time. If a window
+     * is shown with user timestamp older than the time of the last
+     * user action, it won't be activated after being shown, with the special
+     * value 0 meaning not to activate the window after being shown.
+     * @since 3.2
+     */
+    void setUserTime( Time time );
+    
+    /**
+     * Returns the time of last user action on the window, or -1 if not set.
+     * @since 3.2
+     */
+    Time userTime() const;
+
+    /*    
+     * Sets the startup notification id @p id on the window.
+     * @since 3.2
+     */
+    void setStartupId( const char* startup_id );
+    
+    /**
+     * Returns the startup notification id of the window.
+     * @since 3.2
+     */
+    const char* startupId() const;
+
+    /**
+     * Sets actions that the window manager allows for the window.
+     * @since 3.2
+     */
+    void setAllowedActions( unsigned long actions );
+
+    /**
+     * Returns actions that the window manager allows for the window.
+     * @since 3.2
+     */
+    unsigned long allowedActions() const;
+
+    /*
+     * Returns the WM_TRANSIENT_FOR property for the window, i.e. the mainwindow
+     * for this window.
+     * @since 3.2
+     */
+    Window transientFor() const;
+
+    /**
+     * Returns the leader window for the group the window is in, if any.
+     * @since 3.2
+     */    
+    Window groupLeader() const;
+    
     /**
        Places the window frame geometry in frame, and the application window
        geometry in window.  Both geometries are relative to the root window.
@@ -863,10 +1219,29 @@ public:
     **/
     void kdeGeometry(NETRect &frame, NETRect &window);
 
+    /**
+       This function takes the passed XEvent and returns an OR'ed list of
+       NETWinInfo properties that have changed in the properties argument.
+       The new information will be read immediately by the class.
+       The elements of the properties argument are as they would be passed
+       to the constructor, if the array is not large enough,
+       changed properties that don't fit in it won't be listed there
+       (they'll be updated in the class though).
+       
+       @param event the event
+       @param properties properties that changed
+       @param properties_size size of the passed properties array
+       @since 3.2
+
+    **/
+    void event( XEvent* event, unsigned long* properties, int properties_size );
+
     /**
        This function takes the pass XEvent and returns an OR'ed list of NETWinInfo
        properties that have changed.  The new information will be read
-       immediately by the class.
+       immediately by the class. This overloaded version returns
+       only a single mask, and therefore cannot check state of all properties
+       like the other variant.
 
        @param event the event
 
@@ -906,7 +1281,8 @@ protected:
     virtual void changeState(unsigned long /*state*/, unsigned long /*mask*/) { }
 
 private:
-    void update(unsigned long);
+    void update( const unsigned long[] );
+    void updateWMState();
     Role role;
 
 protected:
@@ -916,5 +1292,7 @@ private:
 };
 
 
+//#define KWIN_FOCUS
+
 #endif
 #endif // __net_wm_h
