@@ -22,14 +22,11 @@
 #include <qfile.h>
 #include <qcolor.h>
 #include <qimage.h>
-#include <qregexp.h>
 #include <qwmatrix.h>
 
-#include <kdebug.h>
 #include <kmdcodec.h>
 
 #include <zlib.h>
-#include <math.h>
 
 #include "ksvgiconpainter.h"
 #include "ksvgiconengine.h"
@@ -46,61 +43,11 @@ public:
 	{
 	}
 
-	double dpi()
-	{
-	    return 90.0; // TODO: make modal?
-	}
-	
 	double toPixel(const QString &s, bool hmode)
 	{
-		if(s.isEmpty())
-			return 0.0;
-
-		QString check = s;
-
-		double ret = 0.0;
-
-		bool ok = false;
-
-		double value = check.toDouble(&ok);
-
-		if(!ok)
-		{
-			QRegExp reg("[0-9 .-]");
-			check.replace(reg, "");
-
-			if(check.compare("px") == 0)
-				ret = value;
-			else if(check.compare("cm") == 0)
-				ret = (value / 2.54) * dpi();
-			else if(check.compare("pc") == 0)
-				ret = (value / 6.0) * dpi();
-			else if(check.compare("mm") == 0)
-				ret = (value / 25.4) * dpi();
-			else if(check.compare("in") == 0)
-				ret = value * dpi();
-			else if(check.compare("pt") == 0)
-				ret = (value / 72.0) * dpi();
-			else if(check.compare("%") == 0)
-			{
-				ret = value / 100.0;
-
-				if(hmode)
-					ret *= m_engine->width();
-				else
-					ret *= m_engine->height();
-			}
-			else if(check.compare("em") == 0)
-			{
-				ret = (value / 72.0) * dpi() * 12.0; // TODO make this depend on actual font size
-			}
-		}
-		else
-			ret = value;
-
-		return ret;
+		return m_engine->painter()->toPixel(s, hmode);
 	}
-	
+
 	ArtGradientStop *parseGradientStops(QDomElement element, int &offsets)
 	{
 		QMemArray<ArtGradientStop> *stopArray = new QMemArray<ArtGradientStop>();
@@ -109,7 +56,7 @@ public:
 		for(QDomNode node = element.firstChild(); !node.isNull(); node = node.nextSibling())
 		{
 			QDomElement element = node.toElement();
-			
+
 			oldOffset = newOffset;
 			QString temp = element.attribute("offset");
 
@@ -120,7 +67,7 @@ public:
 			}
 			else
 				newOffset = temp.toFloat();
-			
+
 			// Spec  skip double offset specifications
 			if(oldOffset == newOffset)
 				continue;
@@ -130,19 +77,19 @@ public:
 
 			(*stopArray)[offsets].offset = newOffset;
 
-			QString parseOpacity;			
+			QString parseOpacity;
 			QString parseColor;
-			
+
 			if(element.hasAttribute("stop-opacity"))
 				parseOpacity = element.attribute("stop-opacity");
-			
+
 			if(element.hasAttribute("stop-color"))
 				parseColor = element.attribute("stop-color");
-			
+
 			if(parseOpacity.isEmpty() || parseColor.isEmpty())
 			{
 				QString style = element.attribute("style");
-					
+
 				QStringList substyles = QStringList::split(';', style);
 				for(QStringList::Iterator it = substyles.begin(); it != substyles.end(); ++it)
 				{
@@ -155,32 +102,32 @@ public:
 					if(command == "stop-color")
 					{
 						parseColor = params;
-						
+
 						if(!parseOpacity.isEmpty())
 							break;
 					}
 					else if(command == "stop-opacity")
 					{
 						parseOpacity = params;
-						
+
 						if(!parseColor.isEmpty())
 							break;
 					}
 				}
 			}
-	
+
 			// Parse color using KSVGIconPainter (which uses Qt)
 			// Supports all svg-needed color formats
 			QColor qStopColor = m_engine->painter()->parseColor(parseColor);
 
 			// Convert in a libart suitable form
 			Q_UINT32 stopColor = m_engine->painter()->toArtColor(qStopColor);
-		
+
 			int opacity = m_engine->painter()->parseOpacity(parseOpacity);
 
 			Q_UINT32 rgba = (stopColor << 8) | opacity;
-			Q_UINT32 r, g, b, a;				
-				
+			Q_UINT32 r, g, b, a;
+
 			// Convert from separated to premultiplied alpha
 			a = rgba & 0xff;
 			r = (rgba >> 24) * a + 0x80;
@@ -189,7 +136,7 @@ public:
 			g = (g + (g >> 8)) >> 8;
 			b = ((rgba >> 8) & 0xff) * a + 0x80;
 			b = (b + (b >> 8)) >> 8;
-				
+
 			(*stopArray)[offsets].color[0] = ART_PIX_MAX_FROM_8(r);
 			(*stopArray)[offsets].color[1] = ART_PIX_MAX_FROM_8(g);
 			(*stopArray)[offsets].color[2] = ART_PIX_MAX_FROM_8(b);
@@ -209,24 +156,24 @@ public:
 		if(points.contains(",,") || points.contains(", ,"))
 			return QPointArray();
 
-		points.replace(QRegExp(","), " ");
-		points.replace(QRegExp("\r"), "");
-		points.replace(QRegExp("\n"), "");
+		points.replace(',', ' ');
+		points.replace('\r', QString::null);
+		points.replace('\n', QString::null);
 
 		points = points.simplifyWhiteSpace();
 
 		QStringList pointList = QStringList::split(' ', points);
 
 		QPointArray array(pointList.count() / 2);
-		int i = 0;		
-		
+		int i = 0;
+
 		for(QStringList::Iterator it = pointList.begin(); it != pointList.end(); it++)
 		{
 			float x = (*(it++)).toFloat();
 			float y = (*(it)).toFloat();
-			
+
 			array.setPoint(i, static_cast<int>(x), static_cast<int>(y));
-	        i++;
+			i++;
 		}
 
 		return array;
@@ -236,9 +183,9 @@ public:
 	{
 		// Combine new and old matrix
 		QWMatrix matrix = m_engine->painter()->parseTransform(transform);
-		
+
 		QWMatrix *current = m_engine->painter()->worldMatrix();
-		*current *= matrix;
+		*current = matrix * *current;
 	}
 
 	void parseCommonAttributes(QDomNode &node)
@@ -280,7 +227,7 @@ public:
 				else
 					parsePA(name, value);
 			}
-		}	
+		}
 
 		// Apply local attributes
 		QDomNamedNodeMap attr = node.attributes();
@@ -298,8 +245,10 @@ public:
 		}
 	}
 
-	void handleTags(QDomElement element, bool paint)
+	bool handleTags(QDomElement element, bool paint)
 	{
+		if(element.attribute("display") == "none")
+			return false;
 		if(element.tagName() == "linearGradient")
 		{
 			ArtGradientLinear *gradient = new ArtGradientLinear();
@@ -307,7 +256,7 @@ public:
 			int offsets = -1;
 			gradient->stops = parseGradientStops(element, offsets);
 			gradient->n_stops = offsets + 1;
-			
+
 			QString spread = element.attribute("spreadMethod");
 			if(spread == "repeat")
 				gradient->spread = ART_GRADIENT_REPEAT;
@@ -315,10 +264,10 @@ public:
 				gradient->spread = ART_GRADIENT_REFLECT;
 			else
 				gradient->spread = ART_GRADIENT_PAD;
-		
-			m_engine->painter()->addLinearGradient(element.attribute("id"), gradient);	
-			m_engine->painter()->addLinearGradientElement(gradient, element);	
-			return;
+
+			m_engine->painter()->addLinearGradient(element.attribute("id"), gradient);
+			m_engine->painter()->addLinearGradientElement(gradient, element);
+			return true;
 		}
 		else if(element.tagName() == "radialGradient")
 		{
@@ -327,14 +276,14 @@ public:
 			int offsets = -1;
 			gradient->stops = parseGradientStops(element, offsets);
 			gradient->n_stops = offsets + 1;
-				
-			m_engine->painter()->addRadialGradient(element.attribute("id"), gradient);	
-			m_engine->painter()->addRadialGradientElement(gradient, element);	
-			return;
+
+			m_engine->painter()->addRadialGradient(element.attribute("id"), gradient);
+			m_engine->painter()->addRadialGradientElement(gradient, element);
+			return true;
 		}
-		
+
 		if(!paint)
-			return;
+			return true;
 
 		// TODO: Default attribute values
 		if(element.tagName() == "rect")
@@ -348,13 +297,31 @@ public:
 			double ry = 0.0;
 
 			if(element.hasAttribute("rx"))
-				rx = toPixel(element.attribute("rx"), true);		
+				rx = toPixel(element.attribute("rx"), true);
 
 			if(element.hasAttribute("ry"))
-				ry = toPixel(element.attribute("ry"), false);		
+				ry = toPixel(element.attribute("ry"), false);
 
 			m_engine->painter()->drawRectangle(x, y, w, h, rx, ry);
 		}
+		else if(element.tagName() == "switch")
+		{
+			QDomNode iterate = element.firstChild();
+
+			while(!iterate.isNull())
+			{
+				// Reset matrix
+				m_engine->painter()->setWorldMatrix(new QWMatrix(m_initialMatrix));
+
+				// Parse common attributes, style / transform
+				parseCommonAttributes(iterate);
+
+				if(handleTags(iterate.toElement(), true))
+					return true;
+				iterate = iterate.nextSibling();
+			}
+			return true;
+		}
 		else if(element.tagName() == "g" || element.tagName() == "defs")
 		{
 			QDomNode iterate = element.firstChild();
@@ -366,10 +333,11 @@ public:
 
 				// Parse common attributes, style / transform
 				parseCommonAttributes(iterate);
-				
+
 				handleTags(iterate.toElement(), (element.tagName() == "defs") ? false : true);
 				iterate = iterate.nextSibling();
 			}
+			return true;
 		}
 		else if(element.tagName() == "line")
 		{
@@ -379,6 +347,7 @@ public:
 			double y2 = toPixel(element.attribute("y2"), false);
 
 			m_engine->painter()->drawLine(x1, y1, x2, y2);
+			return true;
 		}
 		else if(element.tagName() == "circle")
 		{
@@ -388,6 +357,7 @@ public:
 			double r = toPixel(element.attribute("r"), true); // TODO: horiz correct?
 
 			m_engine->painter()->drawEllipse(cx, cy, r, r);
+			return true;
 		}
 		else if(element.tagName() == "ellipse")
 		{
@@ -396,18 +366,21 @@ public:
 
 			double rx = toPixel(element.attribute("rx"), true);
 			double ry = toPixel(element.attribute("ry"), false);
-				
+
 			m_engine->painter()->drawEllipse(cx, cy, rx, ry);
+			return true;
 		}
 		else if(element.tagName() == "polyline")
 		{
 			QPointArray polyline = parsePoints(element.attribute("points"));
 			m_engine->painter()->drawPolyline(polyline);
+			return true;
 		}
 		else if(element.tagName() == "polygon")
 		{
 			QPointArray polygon = parsePoints(element.attribute("points"));
 			m_engine->painter()->drawPolygon(polygon);
+			return true;
 		}
 		else if(element.tagName() == "path")
 		{
@@ -420,6 +393,7 @@ public:
 				filled = false;
 
 			m_engine->painter()->drawPath(element.attribute("d"), filled);
+			return true;
 		}
 		else if(element.tagName() == "image")
 		{
@@ -429,7 +403,7 @@ public:
 			double h = toPixel(element.attribute("height"), false);
 
 			QString href = element.attribute("xlink:href");
-			
+
 			if(href.startsWith("data:"))
 			{
 				// Get input
@@ -451,9 +425,11 @@ public:
 
 				m_engine->painter()->drawImage(x, y, *image);
 			}
+			return true;
 		}
+		return false;
 	}
-	
+
 	void parseStyle(const QString &style)
 	{
 		QStringList substyles = QStringList::split(';', style);
@@ -464,7 +440,7 @@ public:
 			QString params = substyle[1];
 			command = command.stripWhiteSpace();
 			params = params.stripWhiteSpace();
-			
+
 			parsePA(command, params);
 		}
 	}
@@ -481,9 +457,9 @@ public:
 			m_engine->painter()->setJoinStyle(value);
 		else if(command == "stroke-dashoffset")
 			m_engine->painter()->setStrokeDashOffset(value);
-		else if(command == "stroke-dasharray")
+		else if(command == "stroke-dasharray" && value != "none")
 			m_engine->painter()->setStrokeDashArray(value);
-			else if(command == "stroke")
+		else if(command == "stroke")
 			m_engine->painter()->setStrokeColor(value);
 		else if(command == "fill")
 			m_engine->painter()->setFillColor(value);
@@ -506,7 +482,7 @@ public:
 
 private:
 	friend class KSVGIconEngine;
-	
+
 	KSVGIconEngine *m_engine;
 	QWMatrix m_initialMatrix;
 };
@@ -560,25 +536,26 @@ bool KSVGIconEngine::load(int width, int height, const QString &path)
 
 		QString data;
 		bool done = false;
-		
-		char *buffer = new char[1024];
+
+		QCString buffer(1024);
+		int length = 0;
 
 		while(!done)
 		{
-			memset(buffer, 0, 1024);
-			
-			int ret = gzread(svgz, buffer, 1024);
+			int ret = gzread(svgz, buffer.data() + length, 1024);
 			if(ret == 0)
 				done = true;
 			else if(ret == -1)
 				return false;
-			
-			data += QString::fromUtf8(buffer);
+			else {
+				buffer.resize(buffer.size()+1024);
+				length += ret;
+			}
 		}
-		
+
 		gzclose(svgz);
 
-		svgDocument.setContent(data);
+		svgDocument.setContent(buffer);
 	}
 
 	if(svgDocument.isNull())
@@ -592,22 +569,20 @@ bool KSVGIconEngine::load(int width, int height, const QString &path)
 	// Detect width and height
 	QDomElement rootElement = rootNode.toElement();
 
-	d->width = width;
+	// Create icon painter
+	d->painter = new KSVGIconPainter(width, height);
 
+	d->width = width; // this sets default for no width -> 100% case
 	if(rootElement.hasAttribute("width"))
 		d->width = d->helper->toPixel(rootElement.attribute("width"), true);
-	else
-		d->width = d->helper->toPixel("100%", true);
-
-	d->height = height;
 
+	d->height = height; // this sets default for no height -> 100% case
 	if(rootElement.hasAttribute("height"))
 		d->height = d->helper->toPixel(rootElement.attribute("height"), false);
-	else
-		d->height = d->helper->toPixel("100%", false);
 
 	// Create icon painter
-	d->painter = new KSVGIconPainter(width, height, static_cast<int>(d->width), static_cast<int>(d->height));
+	d->painter->setDrawWidth(static_cast<int>(d->width));
+	d->painter->setDrawHeight(static_cast<int>(d->height));
 
 	// Set viewport clipping rect
 	d->painter->setClippingRect(0, 0, width, height);
@@ -640,11 +615,11 @@ bool KSVGIconEngine::load(int width, int height, const QString &path)
 
 	QWMatrix initialMatrix = *d->painter->worldMatrix();
 	d->helper->m_initialMatrix = initialMatrix;
-	
+
 	// Apply transform
 	if(rootElement.hasAttribute("transform"))
 		d->helper->parseTransform(rootElement.attribute("transform"));
-	
+
 	// Go through all elements
 	QDomNode svgNode = rootElement.firstChild();
 	while(!svgNode.isNull())
@@ -672,6 +647,11 @@ KSVGIconPainter *KSVGIconEngine::painter()
 	return d->painter;
 }
 
+QImage *KSVGIconEngine::image()
+{
+	return d->painter->image();
+}
+
 double KSVGIconEngine::width()
 {
 	return d->width;
