@@ -31,17 +31,14 @@
 
 #include <kdebug.h>
 
-#include <libart_lgpl/art_svp.h>
-#include <libart_lgpl/art_rgb.h>
 #include <libart_lgpl/art_rgba.h>
+#include <libart_lgpl/art_bpath.h>
 #include <libart_lgpl/art_vpath.h>
 #include <libart_lgpl/art_vpath_dash.h>
-#include <libart_lgpl/art_bpath.h>
 #include <libart_lgpl/art_affine.h>
-#include <libart_lgpl/art_svp_ops.h>
-#include <libart_lgpl/art_svp_vpath.h>
 #include <libart_lgpl/art_render_svp.h>
-#include <libart_lgpl/art_vpath_bpath.h>
+#include <libart_lgpl/art_svp.h>
+#include <libart_lgpl/art_svp_vpath.h>
 #include <libart_lgpl/art_svp_intersect.h>
 #include <libart_lgpl/art_svp_vpath_stroke.h>
 
@@ -57,11 +54,11 @@ public:
 	KSVGIconPainterHelper(int width, int height, KSVGIconPainter *painter)
 	{
 		m_painter = painter;
-		
+
 		m_clipSVP = 0;
 
 		m_fillColor = Qt::black;
-		
+
 		m_useFill = true;
 		m_useStroke = false;
 
@@ -69,7 +66,7 @@ public:
 		m_useStrokeGradient = false;
 
 		m_worldMatrix = new QWMatrix();
-		
+
 		// Create new image with alpha support
 		m_image = new QImage(width, height, 32);
 		m_image->setAlphaBuffer(true);
@@ -82,12 +79,12 @@ public:
 		m_opacity = 0xff;
 		m_fillOpacity = 0xff;
 		m_strokeOpacity = 0xff;
-		
+
 		m_fillRule = "nonzero";
-		
+
 		m_width = width;
 		m_height = height;
-		
+
 		m_rowstride = m_width * 4;
 
 		// Make internal libart rendering buffer transparent
@@ -106,13 +103,22 @@ public:
 
 		delete m_image;
 		delete m_worldMatrix;
+
+		for(QMap<QString, ArtGradientLinear *>::Iterator it = m_linearGradientMap.begin(); it != m_linearGradientMap.end(); ++it)
+		{
+			delete it.data();
+		}
+		for(QMap<QString, ArtGradientRadial *>::Iterator it = m_radialGradientMap.begin(); it != m_radialGradientMap.end(); ++it)
+		{
+			delete it.data();
+		}
 	}
 
 	ArtVpath *allocVPath(int number)
 	{
 		return art_new(ArtVpath, number);
 	}
-	
+
 	ArtBpath *allocBPath(int number)
 	{
 		return art_new(ArtBpath, number);
@@ -167,13 +173,13 @@ public:
 		// Re-swap again...
 		art_u8 *temp = m_buffer;
 		m_buffer = m_tempBuffer;
-		m_tempBuffer = temp;		
-		
+		m_tempBuffer = temp;
+
 		art_free(m_tempBuffer);
 		m_tempBuffer = 0;
 	}
 
-	Q_UINT32 toArtColor(QColor color)
+	Q_UINT32 toArtColor(const QColor &color)
 	{
 		// Convert in a libart suitable form
 		QString tempName = color.name();
@@ -198,7 +204,7 @@ public:
 
 		return result;
 	}
-	
+
 	void drawSVP(ArtSVP *svp, Q_UINT32 rgb, int opacity)
 	{
 		if(!svp)
@@ -206,22 +212,36 @@ public:
 
 		ArtRender *render = art_render_new(0, 0, m_width, m_height, m_buffer, m_rowstride, 3, 8, ART_ALPHA_SEPARATE, 0);
 		art_render_svp(render, svp);
-		
+
 		art_render_mask_solid(render, (opacity << 8) + opacity + (opacity >> 7));
 
 		ArtPixMaxDepth color[3];
 		color[0] = ART_PIX_MAX_FROM_8(rgb >> 16);
 		color[1] = ART_PIX_MAX_FROM_8((rgb >> 8) & 0xff);
 		color[2] = ART_PIX_MAX_FROM_8(rgb & 0xff);
-		
+
 		art_render_image_solid(render, color);
 		art_render_invoke(render);
 	}
 
+	void drawBPath(ArtBpath *bpath)
+	{
+		double affine[6];
+		affine[0] = m_worldMatrix->m11();
+		affine[1] = m_worldMatrix->m12();
+		affine[2] = m_worldMatrix->m21();
+		affine[3] = m_worldMatrix->m22();
+		affine[4] = m_worldMatrix->dx();
+		affine[5] = m_worldMatrix->dy();
+
+		ArtBpath *temp = art_bpath_affine_transform(bpath, affine);
+		ArtVpath *vec = art_bez_path_to_vec(temp, 0.25);
+		art_free(temp);
+		drawPathInternal(vec, affine);
+	}
+
 	void drawVPath(ArtVpath *vec)
 	{
-		ArtSVP *svp;
-
 		double affine[6];
 		affine[0] = m_worldMatrix->m11();
 		affine[1] = m_worldMatrix->m12();
@@ -233,7 +253,12 @@ public:
 		ArtVpath *temp = art_vpath_affine_transform(vec, affine);
 		art_free(vec);
 		vec = temp;
+		drawPathInternal(vec, affine);
+	}
 
+	void drawPathInternal(ArtVpath *vec, double *affine)
+	{
+		ArtSVP *svp;
 		ArtSVP *fillSVP = 0, *strokeSVP = 0;
 
 		Q_UINT32 fillColor = 0, strokeColor = 0;
@@ -278,7 +303,7 @@ public:
 		// fix (Rob).
 		if(m_strokeWidth <= 0)
 			m_useStroke = m_useStrokeGradient = false;
-		
+
 		// Stroking
 		if(m_useStroke || m_useStrokeGradient)
 		{
@@ -287,9 +312,9 @@ public:
 			double ratio = sqrt(pow(affine[0], 2) + pow(affine[3], 2)) / sqrt(2.0);
 			double strokeWidth = m_strokeWidth * ratio;
 
-		    ArtPathStrokeJoinType joinStyle = ART_PATH_STROKE_JOIN_MITER;
+			ArtPathStrokeJoinType joinStyle = ART_PATH_STROKE_JOIN_MITER;
 			ArtPathStrokeCapType capStyle = ART_PATH_STROKE_CAP_BUTT;
-			
+
 			if(m_joinStyle == "miter")
 				joinStyle = ART_PATH_STROKE_JOIN_MITER;
 			else if(m_joinStyle == "round")
@@ -306,12 +331,12 @@ public:
 
 			if(m_dashes.length() > 0)
 			{
-				QRegExp reg("[a-zA-Z,(; ]");
+				QRegExp reg("[, ]");
 				QStringList dashList = QStringList::split(reg, m_dashes);
 
 				double *dashes = new double[dashList.count()];
 				for(unsigned int i = 0; i < dashList.count(); i++)
-					dashes[i] = dashList[i].toDouble();
+					dashes[i] = m_painter->toPixel(dashList[i], true);
 
 				ArtVpathDash dash;
 				dash.offset = m_dashOffset;
@@ -336,7 +361,7 @@ public:
 		int fillOpacity = static_cast<int>(m_fillOpacity);
 		int strokeOpacity = static_cast<int>(m_strokeOpacity);
 		int opacity = static_cast<int>(m_opacity);
-		
+
 		// Needed hack, to support both transparent
 		// paths and transparent gradients
 		if(fillOpacity == strokeOpacity && fillOpacity == opacity && !m_useFillGradient && !m_useStrokeGradient)
@@ -347,13 +372,13 @@ public:
 			int temp = fillOpacity * opacity + 0x80;
 			fillOpacity = (temp + (temp >> 8)) >> 8;
 		}
-		
+
 		if(strokeOpacity != 255)
 		{
 			int temp = strokeOpacity * opacity + 0x80;
 			strokeOpacity = (temp + (temp >> 8)) >> 8;
 		}
-		
+
 		// Create temporary buffer if necessary
 		bool tempDone = false;
 		if(m_opacity != 0xff)
@@ -376,7 +401,7 @@ public:
 		// Mix in temporary buffer, if possible
 		if(tempDone)
 			mixBuffer(opacity);
-	
+
 		if(m_clipSVP)
 		{
 			art_svp_free(m_clipSVP);
@@ -393,7 +418,7 @@ public:
 		m_opacity = 255.0;
 		m_fillOpacity = 255.0;
 		m_strokeOpacity = 255.0;
-		
+
 		art_free(vec);
 	}
 
@@ -403,7 +428,7 @@ public:
 		if(linear)
 		{
 			QDomElement element = m_linearGradientElementMap[linear];
-			
+
 			double x1, y1, x2, y2;
 			if(element.hasAttribute("x1"))
 				x1 = m_painter->toPixel(element.attribute("x1"), true);
@@ -447,7 +472,7 @@ public:
 			art_render_svp(render, svp);
 
 			art_render_gradient_linear(render, linear, ART_FILTER_HYPER);
-			art_render_invoke(render);		
+			art_render_invoke(render);
 		}
 	}
 
@@ -458,7 +483,7 @@ public:
 		{
 			QDomElement element = m_radialGradientElementMap[radial];
 
-			double cx, cy, r, fx, fy;				
+			double cx, cy, r, fx, fy;
 			if(element.hasAttribute("cx"))
 				cx = m_painter->toPixel(element.attribute("cx"), true);
 			else
@@ -518,7 +543,7 @@ public:
 			art_render_svp(render, svp);
 
 			art_render_gradient_radial(render, radial, ART_FILTER_HYPER);
-			art_render_invoke(render);		
+			art_render_invoke(render);
 		}
 	}
 
@@ -558,18 +583,18 @@ public:
 							newElement.setAttribute(name, attr.item(i).nodeValue());
 					}
 				}
-				
+
 				applyGradient(svp, element.attribute("xlink:href").mid(1));
 
 				// Restore attributes
 				QDictIterator<QString> itr(refattrs);
 				for(; itr.current(); ++itr)
 					newElement.setAttribute(itr.currentKey(), *(itr.current()));
-				
+
 				return;
 			}
 		}
-		
+
 		ArtGradientRadial *radial = m_radialGradientMap[ref];
 		if(radial)
 		{
@@ -604,7 +629,7 @@ public:
 							newElement.setAttribute(name, attr.item(i).nodeValue());
 					}
 				}
-				
+
 				applyGradient(svp, element.attribute("xlink:href").mid(1));
 
 				// Restore attributes
@@ -620,7 +645,7 @@ public:
 	void applyGradient(ArtSVP *svp, bool fill)
 	{
 		QString ref;
-		
+
 		if(fill)
 		{
 			m_useFillGradient = false;
@@ -638,7 +663,7 @@ public:
 	void blit()
 	{
 		  unsigned char *line = m_buffer;
-		  
+
 		  for(int y = 0; y < m_height; y++)
 		  {
 			  QRgb *sl = reinterpret_cast<QRgb *>(m_image->scanLine(y));
@@ -669,12 +694,12 @@ public:
 			dx = -x / 2.0;
 
 		double dy;
-		
+
 		if(!relative)
 			dy = (cury - y) / 2.0;
 		else
 			dy = -y / 2.0;
-		
+
 		double _x1 =  cos_th * dx + sin_th * dy;
 		double _y1 = -sin_th * dx + cos_th * dy;
 		double Pr1 = r1 * r1;
@@ -702,7 +727,7 @@ public:
 			x1 = a00 * x + a01 * y;
 		else
 			x1 = a00 * (curx + x) + a01 * (cury + y);
-		
+
 		if(!relative)
 			y1 = a10 * x + a11 * y;
 		else
@@ -712,7 +737,7 @@ public:
 		   (x1, y1) is new point in transformed coordinate space.
 
 		   The arc fits a unit-radius circle in this space.
-	     */
+		*/
 
 		d = (x1 - x0) * (x1 - x0) + (y1 - y0) * (y1 - y0);
 
@@ -795,7 +820,7 @@ public:
 		if(!relative)
 			cury = y;
 		else
-			cury += y;	
+			cury += y;
 	}
 
 	// For any docs, see the libart library
@@ -881,7 +906,7 @@ public:
 		{
 			if(vec_n >= vec_n_max)
 				art_expand (vec, ArtVpath, vec_n_max);
-			
+
 			switch (bez[bez_index].code)
 			{
 				case ART_MOVETO_OPEN:
@@ -918,7 +943,7 @@ public:
 					break;
 			}
 		}
-		
+
 		while (bez[bez_index++].code != ART_END);
 		return vec;
 	}
@@ -1097,18 +1122,18 @@ private:
 	QString m_fillRule;
 	QString m_joinStyle;
 	QString m_capStyle;
-	
+
 	int m_strokeMiterLimit;
-	
+
 	QString m_dashes;
 	unsigned short m_dashOffset;
 
 	QColor m_fillColor;
 	QColor m_strokeColor;
-	
+
 	art_u8 *m_buffer;
 	art_u8 *m_tempBuffer;
-	
+
 	int m_width;
 	int m_height;
 
@@ -1117,16 +1142,16 @@ private:
 	double m_opacity;
 	double m_fillOpacity;
 	double m_strokeOpacity;
-	
+
 	bool m_useFill;
 	bool m_useStroke;
 
 	bool m_useFillGradient;
-	bool m_useStrokeGradient;	
-	
+	bool m_useStrokeGradient;
+
 	QString m_fillGradientReference;
-	QString m_strokeGradientReference;	
-	
+	QString m_strokeGradientReference;
+
 	QMap<QString, ArtGradientLinear *> m_linearGradientMap;
 	QMap<ArtGradientLinear *, QDomElement> m_linearGradientElementMap;
 
@@ -1141,23 +1166,33 @@ private:
 struct KSVGIconPainter::Private
 {
 	KSVGIconPainterHelper *helper;
-	
+
 	int drawWidth;
 	int drawHeight;
 };
 
-KSVGIconPainter::KSVGIconPainter(int width, int height, int dwidth, int dheight) : d(new Private())
+KSVGIconPainter::KSVGIconPainter(int width, int height) : d(new Private())
 {
 	d->helper = new KSVGIconPainterHelper(width, height, this);
-	
-	d->drawWidth = dwidth;
-	d->drawHeight = dheight;
+
+	d->drawWidth = width;
+	d->drawHeight = height;
 }
 
 KSVGIconPainter::~KSVGIconPainter()
 {
 	delete d->helper;
-	delete d;	
+	delete d;
+}
+
+void KSVGIconPainter::setDrawWidth(int dwidth)
+{
+	d->drawWidth = dwidth;
+}
+
+void KSVGIconPainter::setDrawHeight(int dheight)
+{
+	d->drawHeight = dheight;
 }
 
 void KSVGIconPainter::finish()
@@ -1219,9 +1254,9 @@ void KSVGIconPainter::setStrokeColor(const QString &stroke)
 	{
 		d->helper->m_useStroke = false;
 		d->helper->m_useStrokeGradient = true;
-		
+
 		QString url = stroke;
-	
+
 		unsigned int start = url.find("#") + 1;
 		unsigned int end = url.findRev(")");
 
@@ -1233,7 +1268,7 @@ void KSVGIconPainter::setStrokeColor(const QString &stroke)
 
 		d->helper->m_useStrokeGradient = false;
 		d->helper->m_strokeGradientReference = QString::null;
-		
+
 		if(stroke.stripWhiteSpace().lower() != "none")
 			setUseStroke(true);
 		else
@@ -1249,7 +1284,7 @@ void KSVGIconPainter::setFillColor(const QString &fill)
 		d->helper->m_useFillGradient = true;
 
 		QString url = fill;
-	
+
 		unsigned int start = url.find("#") + 1;
 		unsigned int end = url.findRev(")");
 
@@ -1261,7 +1296,7 @@ void KSVGIconPainter::setFillColor(const QString &fill)
 
 		d->helper->m_useFillGradient = false;
 		d->helper->m_fillGradientReference = QString::null;
-		
+
 		if(fill.stripWhiteSpace().lower() != "none")
 			setUseFill(true);
 		else
@@ -1274,7 +1309,7 @@ void KSVGIconPainter::setFillRule(const QString &fillRule)
 	d->helper->m_fillRule = fillRule;
 }
 
-Q_UINT32 KSVGIconPainter::parseOpacity(QString data)
+Q_UINT32 KSVGIconPainter::parseOpacity(const QString &data)
 {
 	int opacity = 255;
 
@@ -1491,71 +1526,53 @@ void KSVGIconPainter::drawRectangle(double x, double y, double w, double h, doub
 
 void KSVGIconPainter::drawEllipse(double cx, double cy, double rx, double ry)
 {
-	ArtVpath *vec, *vec2;
-	ArtBpath *temp, *abp;
-	
+	ArtBpath *temp;
+
 	temp = d->helper->allocBPath(6);
-	
-	double x0, y0, x1, y1, x2, y2, x3, y3, len, s, e;
-	double affine[6];
+
+	double x1, y1, x2, y2, x3, y3;
+	double len = 0.55228474983079356;
+	double cos4[] = {1.0, 0.0, -1.0, 0.0, 1.0};
+	double sin4[] = {0.0, 1.0, 0.0, -1.0, 0.0};
 	int i = 0;
 
-	// Use a blowup factor of 10 to make ellipses with small radii look good
-	art_affine_scale(affine, rx * 10.0, ry * 10.0);
-
 	temp[i].code = ART_MOVETO;
-	temp[i].x3 = cos(0.0);
-	temp[i].y3 = sin(0.0);
+	temp[i].x3 = cx + rx;
+	temp[i].y3 = cy;
 
 	i++;
-	
-	for(s = 0; s < 2 * M_PI; s += M_PI_2)
-	{
-		e = s + M_PI_2;
-		if(e > 2 * M_PI)
-			e = 2 * M_PI;
 
-		len = 0.552 * (e - s) / M_PI_2;
-		x0 = cos (s);
-		y0 = sin (s);
-		x1 = x0 + len * cos (s + M_PI_2);
-		y1 = y0 + len * sin (s + M_PI_2);
-		x3 = cos (e);
-		y3 = sin (e);
-		x2 = x3 + len * cos (e - M_PI_2);
-		y2 = y3 + len * sin (e - M_PI_2);
+	while(i < 5)
+	{
+		x1 = cos4[i-1] + len * cos4[i];
+		y1 = sin4[i-1] + len * sin4[i];
+		x2 = cos4[i] + len * cos4[i-1];
+		y2 = sin4[i] + len * sin4[i-1];
+		x3 = cos4[i];
+		y3 = sin4[i];
 
 		temp[i].code = ART_CURVETO;
-		temp[i].x1 = x1;
-		temp[i].y1 = y1;
-		temp[i].x2 = x2;
-		temp[i].y2 = y2;
-		temp[i].x3 = x3;
-		temp[i].y3 = y3;
+		temp[i].x1 = cx + x1 * rx;
+		temp[i].y1 = cy + y1 * ry;
+		temp[i].x2 = cx + x2 * rx;
+		temp[i].y2 = cy + y2 * ry;
+		temp[i].x3 = cx + x3 * rx;
+		temp[i].y3 = cy + y3 * ry;
 
 		i++;
 	}
 
 	temp[i].code = ART_END;
 
-	abp = art_bpath_affine_transform(temp, affine);
-	vec = d->helper->art_bez_path_to_vec(abp, 0.25);
-	art_free(abp);
-	// undo blowup
-	art_affine_scale(affine, 0.1, 0.1);
-	affine[4] = cx;
-	affine[5] = cy;
-	vec2 = art_vpath_affine_transform(vec, affine);
+	d->helper->drawBPath(temp);
 
-	art_free(vec);
-	
-	d->helper->drawVPath(vec2); 
+	art_free(temp);
 }
 
 void KSVGIconPainter::drawLine(double x1, double y1, double x2, double y2)
 {
 	ArtVpath *vec;
-	
+
 	vec = d->helper->allocVPath(3);
 
 	vec[0].code = ART_MOVETO_OPEN;
@@ -1575,7 +1592,7 @@ void KSVGIconPainter::drawPolyline(QPointArray polyArray, int points)
 {
 	if(polyArray.point(0).x() == -1 || polyArray.point(0).y() == -1)
 		return;
-	
+
 	ArtVpath *polyline;
 
 	if(points == -1)
@@ -1583,8 +1600,8 @@ void KSVGIconPainter::drawPolyline(QPointArray polyArray, int points)
 
 	polyline = d->helper->allocVPath(3 + points);
 	polyline[0].code = ART_MOVETO;
-    polyline[0].x = polyArray.point(0).x();
-    polyline[0].y = polyArray.point(0).y();
+	polyline[0].x = polyArray.point(0).x();
+	polyline[0].y = polyArray.point(0).y();
 
 	int index;
 	for(index = 1; index < points; index++)
@@ -1594,14 +1611,14 @@ void KSVGIconPainter::drawPolyline(QPointArray polyArray, int points)
 		polyline[index].x = point.x();
 		polyline[index].y = point.y();
 	}
-	
+
 	if(d->helper->m_useFill) // if the polyline must be filled, inform libart that it should not be closed.
 	{
 		polyline[index].code = (ArtPathcode)ART_END2;
 		polyline[index].x = polyArray.point(0).x();
 		polyline[index++].y = polyArray.point(0).y();
 	}
-	
+
 	polyline[index].code = ART_END;
 
 	d->helper->drawVPath(polyline);
@@ -1610,11 +1627,11 @@ void KSVGIconPainter::drawPolyline(QPointArray polyArray, int points)
 void KSVGIconPainter::drawPolygon(QPointArray polyArray)
 {
 	ArtVpath *polygon;
-	
+
 	polygon = d->helper->allocVPath(3 + polyArray.count());
 	polygon[0].code = ART_MOVETO;
-    polygon[0].x = polyArray.point(0).x();
-    polygon[0].y = polyArray.point(0).y();
+	polygon[0].x = polyArray.point(0).x();
+	polygon[0].y = polyArray.point(0).y();
 
 	unsigned int index;
 	for(index = 1; index < polyArray.count(); index++)
@@ -1628,7 +1645,7 @@ void KSVGIconPainter::drawPolygon(QPointArray polyArray)
 	polygon[index].code = ART_LINETO;
 	polygon[index].x = polyArray.point(0).x();
 	polygon[index].y = polyArray.point(0).y();
-	
+
 	index++;
 	polygon[index].code = ART_END;
 
@@ -1637,7 +1654,7 @@ void KSVGIconPainter::drawPolygon(QPointArray polyArray)
 
 // Path parsing tool
 // parses the coord into number and forwards to the next token
-const char *getCoord(const char *ptr, double &number)
+static const char *getCoord(const char *ptr, double &number)
 {
 	int integer, exponent;
 	double decimal, frac;
@@ -1649,7 +1666,7 @@ const char *getCoord(const char *ptr, double &number)
 	decimal = 0;
 	sign = 1;
 	expsign = 1;
-	
+
 	// read the sign
 	if(*ptr == '+')
 		ptr++;
@@ -1661,18 +1678,18 @@ const char *getCoord(const char *ptr, double &number)
 	// read the integer part
 	while(*ptr != '\0' && *ptr >= '0' && *ptr <= '9')
 		integer = (integer * 10) + *(ptr++) - '0';
-	
+
 	if(*ptr == '.') // read the decimals
-    {
+	{
 		ptr++;
 		while(*ptr != '\0' && *ptr >= '0' && *ptr <= '9')
 			decimal += (*(ptr++) - '0') * (frac *= 0.1);
-    }
-	
+	}
+
 	if(*ptr == 'e' || *ptr == 'E') // read the exponent part
 	{
 		ptr++;
-		
+
 		// read the sign of the exponent
 		if(*ptr == '+')
 			ptr++;
@@ -1681,7 +1698,7 @@ const char *getCoord(const char *ptr, double &number)
 			ptr++;
 			expsign = -1;
 		}
-		
+
 		exponent = 0;
 		while(*ptr != '\0' && *ptr >= '0' && *ptr <= '9')
 		{
@@ -1689,15 +1706,15 @@ const char *getCoord(const char *ptr, double &number)
 			exponent += *ptr - '0';
 			ptr++;
 		}
-    }
-	
+	}
+
 	number = integer + decimal;
 	number *= sign * pow(10.0, expsign * exponent);
 
 	// skip the following space
 	if(*ptr == ' ')
 		ptr++;
-	
+
 	return ptr;
 }
 
@@ -1711,7 +1728,7 @@ void KSVGIconPainter::drawPath(const QString &data, bool filled)
 	double curx = 0.0, cury = 0.0, contrlx = 0.0, contrly = 0.0, xc, yc;
 	unsigned int lastCommand = 0;
 
-	QString _d = value.replace(QRegExp(","), " ");
+	QString _d = value.replace(",", " ");
 	_d = _d.simplifyWhiteSpace();
 	const char *ptr = _d.latin1();
 	const char *end = _d.latin1() + _d.length() + 1;
@@ -1724,7 +1741,7 @@ void KSVGIconPainter::drawPath(const QString &data, bool filled)
 	{
 		if(*ptr == ' ')
 			ptr++;
-		
+
 		switch(command)
 		{
 			case 'm':
@@ -1785,7 +1802,7 @@ void KSVGIconPainter::drawPath(const QString &data, bool filled)
 
 					index++;
 
-					if(vec.size() == (unsigned int) index)							
+					if(vec.size() == (unsigned int) index)
 						vec.resize(index + 1);
 
 					vec[index].code = (ArtPathcode)ART_END2;
@@ -2108,7 +2125,7 @@ void KSVGIconPainter::drawPath(const QString &data, bool filled)
 				contrlx = xc;
 				contrly = yc;
 
-				lastCommand = 'T';					
+				lastCommand = 'T';
 				break;
 			case 'z':
 			case 'Z':
@@ -2154,6 +2171,10 @@ void KSVGIconPainter::drawPath(const QString &data, bool filled)
 				ptr = getCoord(ptr, tox);
 				ptr = getCoord(ptr, toy);
 
+				// Spec: radii are nonnegative numbers
+				rx = fabs(rx);
+				ry = fabs(ry);
+
 				d->helper->calculateArc(true, vec, index, curx, cury, angle, tox, toy, rx, ry, largeArc, sweep);
 
 				lastCommand = 'a';
@@ -2169,6 +2190,10 @@ void KSVGIconPainter::drawPath(const QString &data, bool filled)
 				ptr = getCoord(ptr, tox);
 				ptr = getCoord(ptr, toy);
 
+				// Spec: radii are nonnegative numbers
+				rx = fabs(rx);
+				ry = fabs(ry);
+
 				d->helper->calculateArc(false, vec, index, curx, cury, angle, tox, toy, rx, ry, largeArc, sweep);
 
 				lastCommand = 'A';
@@ -2187,14 +2212,14 @@ void KSVGIconPainter::drawPath(const QString &data, bool filled)
 			command = *(ptr++);
 
 		// Detect reflection points
-		if(lastCommand != 'C' && lastCommand != 'c'
-	      && lastCommand != 'S' && lastCommand != 's'
-		  && lastCommand != 'Q' && lastCommand != 'q'
-		  && lastCommand != 'T' && lastCommand != 't')
+		if(lastCommand != 'C' && lastCommand != 'c' &&
+			lastCommand != 'S' && lastCommand != 's' &&
+			lastCommand != 'Q' && lastCommand != 'q' &&
+			lastCommand != 'T' && lastCommand != 't')
 		{
 			contrlx = curx;
 			contrly = cury;
-		}			
+		}
 	}
 
 	// Find last subpath
@@ -2245,7 +2270,7 @@ void KSVGIconPainter::drawPath(const QString &data, bool filled)
 			cury = vec[find].y3;
 		}
 	}
-	
+
 	// Close
 	index++;
 
@@ -2267,7 +2292,7 @@ void KSVGIconPainter::drawPath(const QString &data, bool filled)
 	}
 
 	if(render)
-		d->helper->drawVPath(d->helper->art_bez_path_to_vec(vec.data(), 0.25));
+		d->helper->drawBPath(vec.data());
 }
 
 void KSVGIconPainter::drawImage(double x, double y, QImage &image)
@@ -2280,8 +2305,8 @@ void KSVGIconPainter::drawImage(double x, double y, QImage &image)
 	affine[1] = d->helper->m_worldMatrix->m12();
 	affine[2] = d->helper->m_worldMatrix->m21();
 	affine[3] = d->helper->m_worldMatrix->m22();
-	affine[4] = x;
-	affine[5] = y;
+	affine[4] = d->helper->m_worldMatrix->dx() + x;
+	affine[5] = d->helper->m_worldMatrix->dy() + y;
 
 	d->helper->art_rgba_rgba_affine(d->helper->m_buffer, 0, 0, d->helper->m_width, d->helper->m_height,
 									d->helper->m_rowstride, image.bits(), image.width(), image.height(),
@@ -2647,8 +2672,7 @@ double KSVGIconPainter::toPixel(const QString &s, bool hmode)
 
 	if(!ok)
 	{
-		QRegExp reg("[0-9 .-]");
-		check.replace(reg, "");
+		check.replace(QRegExp("[0-9 .-]"), QString::null);
 
 		if(check.compare("px") == 0)
 			ret = value;
@@ -2671,6 +2695,10 @@ double KSVGIconPainter::toPixel(const QString &s, bool hmode)
 			else
 				ret *= d->drawHeight;
 		}
+		else if(check.compare("em") == 0)
+		{
+			ret = value * 10.0; // TODO make this depend on actual font size
+		}
 	}
 	else
 		ret = value;
@@ -2682,7 +2710,7 @@ ArtGradientLinear *KSVGIconPainter::linearGradient(const QString &id)
 {
 	return d->helper->m_linearGradientMap[id];
 }
- 
+
 void KSVGIconPainter::addLinearGradient(const QString &id, ArtGradientLinear *gradient)
 {
 	d->helper->m_linearGradientMap.insert(id, gradient);
@@ -2692,7 +2720,7 @@ QDomElement KSVGIconPainter::linearGradientElement(ArtGradientLinear *linear)
 {
 	return d->helper->m_linearGradientElementMap[linear];
 }
- 
+
 void KSVGIconPainter::addLinearGradientElement(ArtGradientLinear *gradient, QDomElement element)
 {
 	d->helper->m_linearGradientElementMap.insert(gradient, element);
@@ -2702,7 +2730,7 @@ ArtGradientRadial *KSVGIconPainter::radialGradient(const QString &id)
 {
 	return d->helper->m_radialGradientMap[id];
 }
- 
+
 void KSVGIconPainter::addRadialGradient(const QString &id, ArtGradientRadial *gradient)
 {
 	d->helper->m_radialGradientMap.insert(id, gradient);
@@ -2712,13 +2740,13 @@ QDomElement KSVGIconPainter::radialGradientElement(ArtGradientRadial *radial)
 {
 	return d->helper->m_radialGradientElementMap[radial];
 }
- 
+
 void KSVGIconPainter::addRadialGradientElement(ArtGradientRadial *gradient, QDomElement element)
 {
 	d->helper->m_radialGradientElementMap.insert(gradient, element);
 }
 
-Q_UINT32 KSVGIconPainter::toArtColor(QColor color)
+Q_UINT32 KSVGIconPainter::toArtColor(const QColor &color)
 {
 	return d->helper->toArtColor(color);
 }
@@ -2726,7 +2754,7 @@ Q_UINT32 KSVGIconPainter::toArtColor(QColor color)
 QWMatrix KSVGIconPainter::parseTransform(const QString &transform)
 {
 	QWMatrix result;
-	
+
 	// Split string for handling 1 transform statement at a time
 	QStringList subtransforms = QStringList::split(')', transform);
 	QStringList::ConstIterator it = subtransforms.begin();
