@@ -1,9 +1,8 @@
-#include <kdebug.h>
 #include <qwidget.h>
 #include <kinstance.h>
-#include <iostream.h>
+#include <iostream>
 #include <qapplication.h>
-#include <kdebugclasses.h>
+#include <kdebug.h>
 
 class TestWidget : public QWidget
 {
