@@ -33,6 +33,8 @@
 
 #include <kmdcodec.h>
 
+using namespace std;
+
 #define TEST_BLOCK_LEN 1000             // Length of test blocks.
 #define TEST_BLOCK_COUNT 10000          // Number of test blocks.
 #define MAX_READ_BUF_SIZE 8192
@@ -327,7 +329,7 @@ int main (int argc, char *argv[])
         { "x", "uudecode the given string or file", 0 },
         { "z", "run a preset message-digest test", 0 },
         { "+command", "[input1, input2,...]", 0 },
-        { 0, 0, 0 }
+        KCmdLineLastOption
     };
 
     KCmdLineArgs::init( argc, argv, "kmdcodectest", description, version );
