@@ -49,7 +49,7 @@ int main(int argc, char *argv[])
        "The Quick Brown Fox Jumped Over The Lazy Bridge. "); 
   check("reverse(test)", 
 	KStringHandler::reverse(test),
-       "bridge. lazy the over jumped fox brown quick The"); 
+       " bridge. lazy the over jumped fox brown quick The"); 
   QString result;
   result = KStringHandler::ljust(test, 70);
   if (result.length() != 70)
@@ -78,5 +78,14 @@ int main(int argc, char *argv[])
   check("center(test, 70)", 
 	result,
         "           The quick brown fox jumped over the lazy bridge.           ");
+
+  test = "Click on http://foo@bar:www.kde.org/yoyo/dyne.html#a1 for info.";
+  check( "tagURLs()", KStringHandler::tagURLs( test ),
+	"Click on <a href=\"http://foo@bar:www.kde.org/yoyo/dyne.html#a1\">http://foo@bar:www.kde.org/yoyo/dyne.html#a1</a> for info." );
+
+  test = "&lt;a href=www.foo.com&gt;";
+  check( "tagURLs()", KStringHandler::tagURLs( test ),
+	 "&lt;a href=<a href=\"www.foo.com\">www.foo.com</a>&gt;" );
+
   printf("\nTest OK !\n");
 }
