@@ -21,6 +21,7 @@
 #include "ksycoca.h"
 #include "ksycocadict.h"
 #include "kresourcelist.h"
+#include "kmimetype.h"
 
 #include <kglobal.h>
 #include <kstandarddirs.h>
@@ -32,13 +33,13 @@
 KBuildServiceFactory::KBuildServiceFactory( KSycocaFactory *serviceTypeFactory,
 	KBuildServiceGroupFactory *serviceGroupFactory ) :
   KServiceFactory(),
+  m_serviceDict(977),
   m_serviceTypeFactory( serviceTypeFactory ),
-  m_serviceGroupFactory( serviceGroupFactory ),
-  m_serviceDict(977)
+  m_serviceGroupFactory( serviceGroupFactory )
 {
    m_resourceList = new KSycocaResourceList();
-   m_resourceList->add( "apps", "*.desktop" );
-   m_resourceList->add( "apps", "*.kdelnk" );
+//   m_resourceList->add( "apps", "*.desktop" );
+//   m_resourceList->add( "apps", "*.kdelnk" );
    m_resourceList->add( "services", "*.desktop" );
    m_resourceList->add( "services", "*.kdelnk" );
 }
@@ -74,14 +75,8 @@ KBuildServiceFactory::createEntry( const QString& file, const char *resource )
   if (name.isEmpty())
      return 0;
 
-  if ( name == ".directory")
-  {
-     m_serviceGroupFactory->addNewEntry(file, resource, 0);
-     return 0;
-  }
-
   // Is it a .desktop file?
-  if ((name.right(8) != ".desktop") && (name.right(7) != ".kdelnk"))
+  if (!name.endsWith(".desktop") && !name.endsWith(".kdelnk"))
       return 0;
 
   KDesktopFile desktopFile(file, true, resource);
@@ -109,6 +104,7 @@ KBuildServiceFactory::saveHeader(QDataStream &str)
    str << (Q_INT32) m_relNameDictOffset;
    str << (Q_INT32) m_offerListOffset;
    str << (Q_INT32) m_initListOffset;
+   str << (Q_INT32) m_menuIdDictOffset;
 }
 
 void
@@ -125,6 +121,9 @@ KBuildServiceFactory::save(QDataStream &str)
    saveOfferList(str);
    saveInitList(str);
 
+   m_menuIdDictOffset = str.device()->at();
+   m_menuIdDict->save(str);
+
    int endOfFactoryData = str.device()->at();
 
    // Update header (pass #3)
@@ -140,29 +139,61 @@ KBuildServiceFactory::saveOfferList(QDataStream &str)
 {
    m_offerListOffset = str.device()->at();
 
+   bool isNumber;
+   for(QDictIterator<KSycocaEntry::Ptr> itserv ( *m_entryDict );
+       itserv.current();
+       ++itserv)
+   {
+      KService *service = (KService *) ((KSycocaEntry *)(*itserv.current()));
+      QStringList serviceTypeList = service->serviceTypes();
+      KServiceType::List serviceTypes;
+      QStringList::ConstIterator it = serviceTypeList.begin();
+      for( ; it != serviceTypeList.end(); ++it )
+      {
+         (*it).toInt(&isNumber);
+         if (isNumber)
+            continue;
+         
+         KServiceType::Ptr serviceType = KServiceType::serviceType(*it);
+         if (!serviceType)
+         {
+           kdDebug() << "'"<< service->desktopEntryPath() << "' specifies undefined mimetype/servicetype '"<< (*it) << "'" << endl;
+           continue;
+         }
+         serviceTypes.append(serviceType);
+      }
+      
+      while(serviceTypes.count())
+      {
+         KServiceType::Ptr serviceType = serviceTypes.first();
+         serviceTypes.pop_front();
+         
+         KServiceType::Ptr parentType = serviceType->parentType();
+         if (parentType)
+            serviceTypes.append(parentType);
+         
+         serviceType->addService(service);
+      }
+   }
+
    // For each entry in servicetypeFactory
    for(QDictIterator<KSycocaEntry::Ptr> it ( *(m_serviceTypeFactory->entryDict()) );
        it.current();
        ++it)
    {
       // export associated services
-      // This means looking for the service type in ALL services
-      // This is SLOW. But it used to be done in every app (in KServiceTypeProfile)
-      // Doing it here saves a lot of time to the clients
-      KSycocaEntry *entry = (*it.current());
-      QString serviceType = entry->name();
-      for(QDictIterator<KSycocaEntry::Ptr> itserv ( *m_entryDict );
-          itserv.current();
-          ++itserv)
+      KServiceType *entry = static_cast<KServiceType*>(static_cast<KSycocaEntry*>(*it.current()));
+      KService::List services = entry->services();
+  
+      for(KService::List::ConstIterator it2 = services.begin();
+          it2 != services.end(); ++it2)
       {
-         KService *service = (KService *) ((KSycocaEntry *)(*itserv.current()));
-         if ( service->hasServiceType( serviceType ) )
-         {
-            str << (Q_INT32) entry->offset();
-            str << (Q_INT32) service->offset();
-         }
+         KService *service = *it2;
+         str << (Q_INT32) entry->offset();
+         str << (Q_INT32) service->offset();
       }
    }
+
    str << (Q_INT32) 0;               // End of list marker (0)
 }
 
@@ -198,9 +229,9 @@ KBuildServiceFactory::addEntry(KSycocaEntry *newEntry, const char *resource)
    KSycocaFactory::addEntry(newEntry, resource);
 
    KService * service = (KService *) newEntry;
+
    if (!service->isDeleted())
    {
-      m_serviceGroupFactory->addNewEntry(service->entryPath(), resource, service);
       QString parent = service->parentApp();
       if (!parent.isEmpty())
          m_serviceGroupFactory->addNewChild(parent, resource, service);
@@ -212,4 +243,7 @@ KBuildServiceFactory::addEntry(KSycocaEntry *newEntry, const char *resource)
 
    QString relName = service->desktopEntryPath();
    m_relNameDict->add( relName, newEntry );
+   QString menuId = service->menuId();
+   if (!menuId.isEmpty())
+      m_menuIdDict->add( menuId, newEntry );
 }
