@@ -1,5 +1,6 @@
 /*  This file is part of the KDE libraries
  *  Copyright (C) 1999 David Faure <faure@kde.org>
+ *  Copyright (C) 2002-2003 Waldo Bastian <bastian@kde.org>
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Library General Public
@@ -18,9 +19,11 @@
 
 #include <qdir.h>
 #include <qeventloop.h>
+#include <config.h>
 
 #include "kbuildsycoca.h"
 #include "kresourcelist.h"
+#include "vfolder_menu.h"
 
 #include <kservice.h>
 #include <kmimetype.h>
@@ -49,18 +52,41 @@
 #include <kcmdlineargs.h>
 #include <kcrash.h>
 
+//#define ANNOYING_POPUP
+
+#ifdef ANNOYING_POPUP
+#include <kmessagebox.h>
+#endif
+
 #include <stdlib.h>
 #include <unistd.h>
 #include <time.h>
 
+typedef QDict<KSycocaEntry> KBSEntryDict;
+typedef QValueList<KSycocaEntry::List> KSycocaEntryListList;
+
 static Q_UINT32 newTimestamp = 0;
 
 static KBuildServiceFactory *g_bsf = 0;
-
+static KBuildServiceGroupFactory *g_bsgf = 0;
+static KSycocaFactory *g_factory = 0;
+static KCTimeInfo *g_ctimeInfo = 0;
+static QDict<Q_UINT32> *g_ctimeDict = 0;
+static const char *g_resource = 0;
+static KBSEntryDict *g_entryDict = 0;
+static KBSEntryDict *g_serviceGroupEntryDict = 0;
+static KSycocaEntryListList *g_allEntries = 0;
 static QStringList *g_changeList = 0;
+static QStringList *g_allResourceDirs = 0;
+static bool g_changed = false;
+static KSycocaEntry::List g_tempStorage;
+static VFolderMenu *g_vfolder = 0;
 
 static const char *cSycocaPath = 0;
 
+static bool bGlobalDatabase = false;
+static bool bMenuTest = false;
+
 void crashHandler(int)
 {
    // If we crash while reading sycoca, we delete the database
@@ -72,15 +98,32 @@ void crashHandler(int)
 static QString sycocaPath()
 {
   QString path;
-  QCString ksycoca_env = getenv("KDESYCOCA");
-  if (ksycoca_env.isEmpty())
-     path = KGlobal::dirs()->saveLocation("tmp")+"ksycoca";
+  
+  if (bGlobalDatabase)
+  {
+     path = KGlobal::dirs()->saveLocation("services")+"ksycoca";
+  }
   else
-     path = QFile::decodeName(ksycoca_env);
+  {
+     QCString ksycoca_env = getenv("KDESYCOCA");
+     if (ksycoca_env.isEmpty())
+        path = KGlobal::dirs()->saveLocation("cache")+"ksycoca";
+     else
+        path = QFile::decodeName(ksycoca_env);
+  }
 
   return path;     
 }
 
+static QString oldSycocaPath()
+{
+  QCString ksycoca_env = getenv("KDESYCOCA");
+  if (ksycoca_env.isEmpty())
+     return KGlobal::dirs()->saveLocation("tmp")+"ksycoca";
+
+  return QString::null;
+}
+
 KBuildSycoca::KBuildSycoca()
   : KSycoca( true )
 {
@@ -138,15 +181,82 @@ void KBuildSycoca::processGnomeVfs()
    fclose( f );
 }
 
-// returns false if the database is up to date
-bool KBuildSycoca::build(KSycocaEntryListList *allEntries,
-                         QDict<Q_UINT32> *ctimeDict)
+KSycocaEntry *KBuildSycoca::createEntry(const QString &file, bool addToFactory)
 {
-  typedef QDict<KSycocaEntry> myEntryDict;
-  typedef QPtrList<myEntryDict> myEntryDictList;
-  myEntryDictList *entryDictList = 0;
+   Q_UINT32 timeStamp = g_ctimeInfo->ctime(file);
+   if (!timeStamp)
+   {
+      timeStamp = KGlobal::dirs()->calcResourceHash( g_resource, file, true);
+   }
+   KSycocaEntry* entry = 0;
+   if (g_allEntries)
+   {
+      assert(g_ctimeDict);
+      Q_UINT32 *timeP = (*g_ctimeDict)[file];
+      Q_UINT32 oldTimestamp = timeP ? *timeP : 0;
 
-  entryDictList = new myEntryDictList();
+      if (timeStamp && (timeStamp == oldTimestamp))
+      {
+         // Re-use old entry
+         if (g_factory == g_bsgf) // Strip .directory from service-group entries
+         {
+            entry = g_entryDict->find(file.left(file.length()-10));
+         }
+         else if (g_factory == g_bsf)
+         {
+            entry = g_entryDict->find(file);
+         }
+         else
+         {
+            entry = g_entryDict->find(file);
+         }
+         // remove from g_ctimeDict; if g_ctimeDict is not empty
+         // after all files have been processed, it means
+         // some files were removed since last time
+         g_ctimeDict->remove( file );
+      }
+      else if (oldTimestamp)
+      {
+         g_changed = true;
+         kdDebug(7021) << "modified: " << file << endl;
+      }
+      else 
+      {
+         g_changed = true;
+         kdDebug(7021) << "new: " << file << endl;
+      }
+   }
+   g_ctimeInfo->addCTime(file, timeStamp );
+   if (!entry)
+   {
+      // Create a new entry
+      entry = g_factory->createEntry( file, g_resource );
+   }
+   if ( entry && entry->isValid() )
+   {
+      if (addToFactory)
+         g_factory->addEntry( entry, g_resource );
+      else
+         g_tempStorage.append(entry);
+      return entry;
+   }
+   return 0;
+}
+
+void KBuildSycoca::slotCreateEntry(const QString &file, KService **service)
+{
+   KSycocaEntry *entry = createEntry(file, false);
+   *service = dynamic_cast<KService *>(entry);
+}
+
+// returns false if the database is up to date
+bool KBuildSycoca::build()
+{
+  typedef QPtrList<KBSEntryDict> KBSEntryDictList;
+  KBSEntryDictList *entryDictList = 0;
+  KBSEntryDict *serviceEntryDict = 0;
+
+  entryDictList = new KBSEntryDictList();
   // Convert for each factory the entryList to a Dict.
   int i = 0;
   // For each factory
@@ -154,10 +264,10 @@ bool KBuildSycoca::build(KSycocaEntryListList *allEntries,
        factory;
        factory = m_lstFactories->next() )
   {
-     myEntryDict *entryDict = new myEntryDict();
-     if (allEntries)
+     KBSEntryDict *entryDict = new KBSEntryDict();
+     if (g_allEntries)
      {
-         KSycocaEntry::List list = (*allEntries)[i++];
+         KSycocaEntry::List list = (*g_allEntries)[i++];
          for( KSycocaEntry::List::Iterator it = list.begin();
             it != list.end();
             ++it)
@@ -165,6 +275,10 @@ bool KBuildSycoca::build(KSycocaEntryListList *allEntries,
             entryDict->insert( (*it)->entryPath(), static_cast<KSycocaEntry *>(*it));
          }
      }
+     if (factory == g_bsf)
+        serviceEntryDict = entryDict;
+     else if (factory == g_bsgf)
+        g_serviceGroupEntryDict = entryDict;
      entryDictList->append(entryDict);
   }
 
@@ -188,19 +302,19 @@ bool KBuildSycoca::build(KSycocaEntryListList *allEntries,
     }
   }
 
-  KCTimeInfo *ctimeInfo = new KCTimeInfo();
+  g_ctimeInfo = new KCTimeInfo(); // This is a build factory too, don't delete!!
   bool uptodate = true;
   // For all resources
   for( QStringList::ConstIterator it1 = allResources.begin();
        it1 != allResources.end();
        ++it1 )
   {
-     bool changed = false;
-     const char *resource = (*it1).ascii();
+     g_changed = false;
+     g_resource = (*it1).ascii();
 
      QStringList relFiles;
 
-     (void) KGlobal::dirs()->findAllResources( resource,
+     (void) KGlobal::dirs()->findAllResources( g_resource,
                                                QString::null,
                                                true, // Recursive!
                                                true, // uniq
@@ -209,14 +323,14 @@ bool KBuildSycoca::build(KSycocaEntryListList *allEntries,
 
      // Now find all factories that use this resource....
      // For each factory
-     myEntryDict *entryDict = entryDictList->first();
-     for (KSycocaFactory *factory = m_lstFactories->first();
-          factory;
-          factory = m_lstFactories->next(),
-          entryDict = entryDictList->next() )
+     g_entryDict = entryDictList->first();
+     for (g_factory = m_lstFactories->first();
+          g_factory;
+          g_factory = m_lstFactories->next(),
+          g_entryDict = entryDictList->next() )
      {
         // For each resource the factory deals with
-        const KSycocaResourceList *list = factory->resourceList();
+        const KSycocaResourceList *list = g_factory->resourceList();
         if (!list) continue;
 
         for( KSycocaResourceList::ConstIterator it2 = list->begin();
@@ -232,64 +346,113 @@ bool KBuildSycoca::build(KSycocaEntryListList *allEntries,
                 ++it3 )
            {
                // Check if file matches filter
-               if (res.filter.search(*it3) == -1) continue;
-
-               Q_UINT32 timeStamp = ctimeInfo->ctime(*it3);
-               if (!timeStamp)
-               {
-                   timeStamp = KGlobal::dirs()->calcResourceHash( resource, *it3, true);
-               }
-               KSycocaEntry* entry = 0;
-               if (allEntries)
-               {
-                   assert(ctimeDict);
-                   Q_UINT32 *timeP = (*ctimeDict)[*it3];
-                   Q_UINT32 oldTimestamp = timeP ? *timeP : 0;
-
-                   if (timeStamp && (timeStamp == oldTimestamp))
-                   {
-                      // Re-use old entry
-                      entry = entryDict->find(*it3);
-                      // remove from ctimeDict; if ctimeDict is not empty
-                      // after all files have been processed, it means
-                      // some files were removed since last time
-                      ctimeDict->remove( *it3 );
-                   }
-                   else if (oldTimestamp)
-                   {
-                      changed = true;
-                      kdDebug(7021) << "modified: " << (*it3) << endl;
-                   }
-                   else 
-                   {
-                      changed = true;
-                      kdDebug(7021) << "new: " << (*it3) << endl;
-                   }
-               }
-               ctimeInfo->addCTime(*it3, timeStamp );
-               if (!entry)
-               {
-                   // Create a new entry
-                   entry = factory->createEntry( *it3, resource );
-               }
-               if ( entry && entry->isValid() )
-                  factory->addEntry( entry, resource );
+               if ((*it3).endsWith(res.extension))
+                   createEntry(*it3, true);
            }
         }
-        if ((factory == g_bsf) && (strcmp(resource, "apps") == 0))
+        if ((g_factory == g_bsf) && (strcmp(g_resource, "services") == 0))
            processGnomeVfs();
      }
-     if (changed || !allEntries)
+     if (g_changed || !g_allEntries)
      {
         uptodate = false;
-        g_changeList->append(resource);
+        g_changeList->append(g_resource);
      }
   }
+
+  bool result = !uptodate || !g_ctimeDict->isEmpty();
   
-  return !uptodate || !ctimeDict->isEmpty();
+  if (result || bMenuTest)
+  {
+     g_resource = "apps";
+     g_factory = g_bsf;
+     g_entryDict = serviceEntryDict;
+     g_changed = false;
+
+     g_vfolder = new VFolderMenu;
+
+     connect(g_vfolder, SIGNAL(newService(const QString &, KService **)),
+             this, SLOT(slotCreateEntry(const QString &, KService **)));
+             
+     VFolderMenu::SubMenu *kdeMenu = g_vfolder->parseMenu("applications.menu", true);
+
+     KServiceGroup *entry = g_bsgf->addNew("/", kdeMenu->directoryFile, 0, false);
+     entry->setLayoutInfo(kdeMenu->layoutList);
+     createMenu(QString::null, QString::null, kdeMenu);
+     
+     (void) existingResourceDirs();
+     *g_allResourceDirs += g_vfolder->allDirectories(); 
+
+     disconnect(g_vfolder, SIGNAL(newService(const QString &, KService **)),
+             this, SLOT(slotCreateEntry(const QString &, KService **)));
+
+     if (g_changed || !g_allEntries)
+     {
+        uptodate = false;
+        g_changeList->append(g_resource);
+     }
+     if (bMenuTest)
+        return false;
+  }
+  
+  return result;
 }
 
-void KBuildSycoca::recreate( KSycocaEntryListList *allEntries, QDict<Q_UINT32> *ctimeDict)
+void KBuildSycoca::createMenu(QString caption, QString name, VFolderMenu::SubMenu *menu)
+{
+  for(VFolderMenu::SubMenu *subMenu = menu->subMenus.first(); subMenu; subMenu = menu->subMenus.next())
+  {
+     QString subName = name+subMenu->name+"/";
+
+     QString directoryFile = subMenu->directoryFile;
+     if (directoryFile.isEmpty())
+        directoryFile = subName+".directory";
+     Q_UINT32 timeStamp = g_ctimeInfo->ctime(directoryFile);
+     if (!timeStamp)
+     {
+        timeStamp = KGlobal::dirs()->calcResourceHash( g_resource, directoryFile, true);
+     }
+
+     KServiceGroup* entry = 0;
+     if (g_allEntries)
+     {
+        Q_UINT32 *timeP = (*g_ctimeDict)[directoryFile];
+        Q_UINT32 oldTimestamp = timeP ? *timeP : 0;
+
+        if (timeStamp && (timeStamp == oldTimestamp))
+        {
+            entry = dynamic_cast<KServiceGroup *> (g_serviceGroupEntryDict->find(subName));
+            if (entry && (entry->directoryEntryPath() != directoryFile))
+                entry = 0; // Can't reuse this one!
+        }
+     }
+     g_ctimeInfo->addCTime(directoryFile, timeStamp);
+
+     entry = g_bsgf->addNew(subName, subMenu->directoryFile, entry, subMenu->isDeleted);
+     entry->setLayoutInfo(subMenu->layoutList);
+     createMenu(caption + entry->caption() + "/", subName, subMenu);
+  }
+  if (caption.isEmpty())
+     caption += "/";
+  if (name.isEmpty())
+     name += "/";
+  for(QDictIterator<KService> it(menu->items); it.current(); ++it)
+  {
+     if (bMenuTest)
+     {
+        if (!menu->isDeleted)
+          printf("%s\t%s\t%s\n", caption.local8Bit().data(), it.current()->menuId().local8Bit().data(), locate("apps", it.current()->desktopEntryPath()).local8Bit().data());
+     }
+     else
+     {
+        // TODO: Check if problematic if same sevice is added twice
+        g_bsf->addEntry( it.current(), g_resource );
+        g_bsgf->addNewEntryTo(name, it.current());
+     }
+  }
+}
+
+void KBuildSycoca::recreate()
 {
   QString path(sycocaPath());
 
@@ -305,17 +468,17 @@ void KBuildSycoca::recreate( KSycocaEntryListList *allEntries, QDict<Q_UINT32> *
 
   m_str = database.dataStream();
 
-  kdDebug(7021) << "Recreating ksycoca file (" << path << ", version " << KSYCOCA_VERSION << ")" << endl;
+  kdDebug(7021) << "Recreating ksycoca file (" << path << ", version " << KSycoca::version() << ")" << endl;
 
   // It is very important to build the servicetype one first
   // Both are registered in KSycoca, no need to keep the pointers
   KSycocaFactory *stf = new KBuildServiceTypeFactory;
-  KBuildServiceGroupFactory *bsgf = new KBuildServiceGroupFactory();
-  g_bsf = new KBuildServiceFactory(stf, bsgf);
+  g_bsgf = new KBuildServiceGroupFactory();
+  g_bsf = new KBuildServiceFactory(stf, g_bsgf);
   (void) new KBuildImageIOFactory();
   (void) new KBuildProtocolInfoFactory();
 
-  if( build(allEntries, ctimeDict)) // Parse dirs
+  if( build()) // Parse dirs
   {
     save(); // Save database
     if (m_str->device()->status())
@@ -332,16 +495,36 @@ void KBuildSycoca::recreate( KSycocaEntryListList *allEntries, QDict<Q_UINT32> *
   {
     m_str = 0L;
     database.abort();
+    if (bMenuTest)
+       return;
     kdDebug(7021) << "Database is up to date" << endl;
   }
 
-  // update the timestamp file
-  QString stamppath = path + "stamp";
-  QFile ksycocastamp(stamppath);
-  ksycocastamp.open( IO_WriteOnly );
-  QDataStream str( &ksycocastamp );
-  str << newTimestamp;
-  str << existingResourceDirs();
+  if (!bGlobalDatabase)
+  {
+    // update the timestamp file
+    QString stamppath = path + "stamp";
+    QFile ksycocastamp(stamppath);
+    ksycocastamp.open( IO_WriteOnly );
+    QDataStream str( &ksycocastamp );
+    str << newTimestamp;
+    str << existingResourceDirs();
+    str << g_vfolder->allDirectories(); // Extra resource dirs
+
+    // Recreate compatibility symlink
+    QString oldPath = oldSycocaPath();
+    if (!oldPath.isEmpty())
+    {
+       KTempFile tmp;
+       if (tmp.status() == 0)
+       {
+          QString tmpFile = tmp.name();
+          tmp.unlink();
+          symlink(QFile::encodeName(path), QFile::encodeName(tmpFile));
+          rename(QFile::encodeName(tmpFile), QFile::encodeName(oldPath));
+       }
+    }
+  }
 }
 
 void KBuildSycoca::save()
@@ -349,7 +532,7 @@ void KBuildSycoca::save()
    // Write header (#pass 1)
    m_str->device()->at(0);
 
-   (*m_str) << (Q_INT32) KSYCOCA_VERSION;
+   (*m_str) << (Q_INT32) KSycoca::version();
    KSycocaFactory * servicetypeFactory = 0L;
    KSycocaFactory * serviceFactory = 0L;
    for(KSycocaFactory *factory = m_lstFactories->first();
@@ -373,6 +556,7 @@ void KBuildSycoca::save()
    (*m_str) << newTimestamp;
    (*m_str) << KGlobal::locale()->language();
    (*m_str) << KGlobal::dirs()->calcResourceHash("services", "update_ksycoca", true);
+   (*m_str) << (*g_allResourceDirs);
 
    // Write factory data....
    for(KSycocaFactory *factory = m_lstFactories->first();
@@ -389,7 +573,7 @@ void KBuildSycoca::save()
    // Write header (#pass 2)
    m_str->device()->at(0);
 
-   (*m_str) << (Q_INT32) KSYCOCA_VERSION;
+   (*m_str) << (Q_INT32) KSycoca::version();
    for(KSycocaFactory *factory = m_lstFactories->first();
        factory;
        factory = m_lstFactories->next())
@@ -445,13 +629,12 @@ bool KBuildSycoca::checkDirTimestamps( const QString& dirname, const QDateTime&
 // and also their directories
 // if all of them all older than the timestamp in file ksycocastamp, this
 // means that there's no need to rebuild ksycoca
-bool KBuildSycoca::checkTimestamps( Q_UINT32 timestamp )
+bool KBuildSycoca::checkTimestamps( Q_UINT32 timestamp, const QStringList &dirs )
 {
    kdDebug( 7021 ) << "checking file timestamps" << endl;
-   QStringList dirs = existingResourceDirs();
    QDateTime stamp;
    stamp.setTime_t( timestamp );
-   for( QStringList::Iterator it = dirs.begin();
+   for( QStringList::ConstIterator it = dirs.begin();
         it != dirs.end();
         ++it )
    {
@@ -468,6 +651,7 @@ QStringList KBuildSycoca::existingResourceDirs()
    if( dirs != NULL )
        return *dirs;
    dirs = new QStringList;
+   g_allResourceDirs = new QStringList;
    // these are all resources cached by ksycoca
    QStringList resources;
    resources += KBuildServiceTypeFactory::resourceTypes();
@@ -481,6 +665,9 @@ QStringList KBuildSycoca::existingResourceDirs()
       *dirs += KGlobal::dirs()->resourceDirs( res.latin1());
       resources.remove( res ); // remove this 'res' and all its duplicates
    }
+   
+   *g_allResourceDirs = *dirs;
+   
    for( QStringList::Iterator it = dirs->begin();
         it != dirs->end(); )
    {
@@ -494,14 +681,16 @@ QStringList KBuildSycoca::existingResourceDirs()
 }
 
 static KCmdLineOptions options[] = {
-   { "nosignal", I18N_NOOP("Don't signal applications."), 0 },
-   { "incremental", I18N_NOOP("Incremental update."), 0 },
+   { "nosignal", I18N_NOOP("Do not signal applications to update."), 0 },
+   { "noincremental", I18N_NOOP("Disable incremental update, re-read everything."), 0 },
    { "checkstamps", I18N_NOOP("Check file timestamps."), 0 },
-   { 0, 0, 0 }
+   { "global", I18N_NOOP("Create global database."), 0 },
+   { "menutest", I18N_NOOP("Perform menu generation test run only."), 0 },
+   KCmdLineLastOption
 };
 
 static const char *appName = "kbuildsycoca";
-static const char *appVersion = "1.0";
+static const char *appVersion = "1.1";
 
 class WaitForSignal : public QObject
 {
@@ -509,7 +698,7 @@ public:
    ~WaitForSignal() { kapp->eventLoop()->exitLoop(); }
 };
 
-int main(int argc, char **argv)
+extern "C" int kdemain(int argc, char **argv)
 {
    KLocale::setMainCatalogue("kdelibs");
    KAboutData d(appName, I18N_NOOP("KBuildSycoca"), appVersion,
@@ -521,16 +710,28 @@ int main(int argc, char **argv)
    KCmdLineArgs::init(argc, argv, &d);
    KCmdLineArgs::addCmdLineOptions(options);
    KCmdLineArgs *args = KCmdLineArgs::parsedArgs();
+   bGlobalDatabase = args->isSet("global");
+   bMenuTest = args->isSet("menutest");
+
+   if (bGlobalDatabase)
+   {
+     setenv("KDEHOME", "-", 1);
+     setenv("KDEROOTHOME", "-", 1);
+   }
 
    KApplication::disableAutoDcopRegistration();
+#ifdef ANNOYING_POPUP
+   KApplication k;
+#else
    KApplication k(false, false);
+#endif
    k.disableSessionManagement();
 
    KCrash::setCrashHandler(KCrash::defaultCrashHandler);
    KCrash::setEmergencySaveFunction(crashHandler);
    KCrash::setApplicationName(QString(appName));
 
-   // this program is in kdelibs so it uses kdelibs as catalogue
+   // this program is in kdelibs so it uses kdelibs as catalog
    KLocale::setMainCatalogue("kdelibs");
    // force generating of KLocale object. if not, the database will get
    // be translated
@@ -565,20 +766,19 @@ int main(int argc, char **argv)
    }
    fprintf(stderr, "%s running...\n", appName);
 
-   bool incremental = args->isSet("incremental");
+
+   bool incremental = !bGlobalDatabase && args->isSet("incremental");
    if (incremental)
    {
-     KSycoca::self()->disableAutoRebuild(); // Don't deadlock
-     QString current_kfsstnd = KGlobal::dirs()->kfsstnd_prefixes();
-     QString ksycoca_kfsstnd = KSycoca::self()->kfsstnd_prefixes();
+     KSycoca::self()->disableAutoRebuild(); // Prevent deadlock
      QString current_language = KGlobal::locale()->language();
      QString ksycoca_language = KSycoca::self()->language();
      Q_UINT32 current_update_sig = KGlobal::dirs()->calcResourceHash("services", "update_ksycoca", true);
      Q_UINT32 ksycoca_update_sig = KSycoca::self()->updateSignature();
      
      if ((current_update_sig != ksycoca_update_sig) ||
-         (current_kfsstnd != ksycoca_kfsstnd) ||
-         (current_language != ksycoca_language))
+         (current_language != ksycoca_language) ||
+         (KSycoca::self()->timeStamp() == 0))
      {
         incremental = false;
         delete KSycoca::self();
@@ -589,38 +789,58 @@ int main(int argc, char **argv)
 
    bool checkstamps = incremental && args->isSet("checkstamps");
    Q_UINT32 filestamp = 0;
+   QStringList oldresourcedirs;
    if( checkstamps && incremental )
-       {
-       QString path = KGlobal::dirs()->saveLocation("tmp")+"ksycocastamp";
+   {
+       QString path = sycocaPath()+"stamp";
+       QCString qPath = QFile::encodeName(path);
+       cSycocaPath = qPath.data(); // Delete timestamps on crash
        QFile ksycocastamp(path);
        if( ksycocastamp.open( IO_ReadOnly ))
-           {
+       {
            QDataStream str( &ksycocastamp );
-           str >> filestamp;
-           QStringList oldresourcedirs;
-           str >> oldresourcedirs;
-           if( oldresourcedirs != KBuildSycoca::existingResourceDirs())
+           if (!str.atEnd())
+               str >> filestamp;
+           if (!str.atEnd())
+           {
+               str >> oldresourcedirs;
+               if( oldresourcedirs != KBuildSycoca::existingResourceDirs())
+                   checkstamps = false;
+           }
+           else
+           {
                checkstamps = false;
            }
+           if (!str.atEnd())
+           {
+               QStringList extraResourceDirs;
+               str >> extraResourceDirs;
+               oldresourcedirs += extraResourceDirs;
+           }
+       }
        else
+       {
            checkstamps = false;
        }
+       cSycocaPath = 0;
+   }
 
    newTimestamp = (Q_UINT32) time(0);
 
-   if( !checkstamps || !KBuildSycoca::checkTimestamps( filestamp ))
+   if( !checkstamps || !KBuildSycoca::checkTimestamps( filestamp, oldresourcedirs ))
    {
       QCString qSycocaPath = QFile::encodeName(sycocaPath());
       cSycocaPath = qSycocaPath.data();
 
-      KBuildSycoca::KSycocaEntryListList *allEntries = 0;
-      QDict<Q_UINT32> *ctimeDict = 0;
+      g_allEntries = 0;
+      g_ctimeDict = 0;
       if (incremental)
       {
+qWarning("Reusing existing ksycoca");
          KSycoca *oldSycoca = KSycoca::self();
          KSycocaFactoryList *factories = new KSycocaFactoryList;
-         allEntries = new KBuildSycoca::KSycocaEntryListList;
-         ctimeDict = new QDict<Q_UINT32>(523);
+         g_allEntries = new KSycocaEntryListList;
+         g_ctimeDict = new QDict<Q_UINT32>(523);
 
          // Must be in same order as in KBuildSycoca::recreate()!
          factories->append( new KServiceTypeFactory );
@@ -636,17 +856,17 @@ int main(int argc, char **argv)
          {
              KSycocaEntry::List list;
              list = factory->allEntries();
-             allEntries->append( list );
+             g_allEntries->append( list );
          }
          delete factories; factories = 0;
          KCTimeInfo *ctimeInfo = new KCTimeInfo;
-         ctimeInfo->fillCTimeDict(*ctimeDict);
+         ctimeInfo->fillCTimeDict(*g_ctimeDict);
          delete oldSycoca;
       }
       cSycocaPath = 0;
 
       KBuildSycoca *sycoca= new KBuildSycoca; // Build data base
-      sycoca->recreate(allEntries, ctimeDict);
+      sycoca->recreate();
    }
 
    if (args->isSet("signal"))
@@ -657,6 +877,9 @@ int main(int argc, char **argv)
      stream << *g_changeList;
      dcopClient->send( "*", "ksycoca", "notifyDatabaseChanged(QStringList)", data );
    }
+#ifdef ANNOYING_POPUP
+   KMessageBox::information(0, i18n("System configuration cache (ksycoca) successfully updated."));
+#endif
    return 0;
 }
 
