@@ -22,8 +22,6 @@
 #include "kded.h"
 #include "kdedmodule.h"
 
-#include <kbuildservicetypefactory.h>
-#include <kbuildservicefactory.h>
 #include <kresourcelist.h>
 #include <kcrash.h>
 
@@ -54,14 +52,26 @@ Kded *Kded::_self = 0;
 
 static bool checkStamps = true;
 
-static void runBuildSycoca()
+static void runBuildSycoca(QObject *callBackObj=0, const char *callBackSlot=0)
 {
    QStringList args;
    args.append("--incremental");
    if(checkStamps)
       args.append("--checkstamps");
-   KApplication::kdeinitExecWait( "kbuildsycoca", args );
    checkStamps = false; // useful only during kded startup
+   if (callBackObj)
+   {
+      QByteArray data;
+      QDataStream dataStream( data, IO_WriteOnly );
+      dataStream << QString("kbuildsycoca") << args;
+      QCString _launcher = KApplication::launcher();
+
+      kapp->dcopClient()->callAsync(_launcher, _launcher, "kdeinit_exec_wait(QString,QStringList)", data, callBackObj, callBackSlot);
+   }
+   else
+   {
+      KApplication::kdeinitExecWait( "kbuildsycoca", args );
+   }
 }
 
 static void runKonfUpdate()
@@ -77,29 +87,8 @@ static void runDontChangeHostname(const QCString &oldName, const QCString &newNa
    KApplication::kdeinitExecWait( "kdontchangethehostname", args );
 }
 
-class KBuildSycoca : public KSycoca
-{
-public:
-   KBuildSycoca() : KSycoca(true) { }
-
-   static KBuildSycoca* createBuildSycoca() 
-   { 
-      if (_self)
-         delete _self;
-      return new KBuildSycoca();
-   }
-
-   KSycocaFactoryList* factoryList() { return m_lstFactories; }
-
-   /**
-    * return true if building
-    */
-   virtual bool isBuilding() { return true; }
-};
-
-
 Kded::Kded(bool checkUpdates)
-  : DCOPObject("kbuildsycoca"), DCOPObjectProxy(), 
+  : DCOPObject("kbuildsycoca"), DCOPObjectProxy(),
     b_checkUpdates(checkUpdates)
 {
   _self = this;
@@ -115,8 +104,11 @@ Kded::Kded(bool checkUpdates)
   QTimer::singleShot(100, this, SLOT(installCrashHandler()));
 
   m_pDirWatch = 0;
-  
+
   m_windowIdList.setAutoDelete(true);
+
+  m_recreateCount = 0;
+  m_recreateBusy = false;
 }
 
 Kded::~Kded()
@@ -129,7 +121,7 @@ Kded::~Kded()
   m_modules.setAutoDelete(true);
 }
 
-bool Kded::process(const QCString &obj, const QCString &fun, 
+bool Kded::process(const QCString &obj, const QCString &fun,
                    const QByteArray &data,
                    QCString &replyType, QByteArray &replyData)
 {
@@ -140,12 +132,41 @@ bool Kded::process(const QCString &obj, const QCString &fun,
 
   KDEDModule *module = loadModule(obj, true);
   if (!module)
-     return false; 
+     return false;
 
   module->setCallingDcopClient(kapp->dcopClient());
   return module->process(fun, data, replyType, replyData);
 }
 
+void Kded::initModules()
+{
+     m_dontLoad.clear();
+     KConfig *config = kapp->config();
+
+     // Preload kded modules.
+     KService::List kdedModules = KServiceType::offers("KDEDModule");
+     for(KService::List::ConstIterator it = kdedModules.begin(); it != kdedModules.end(); ++it)
+     {
+         KService::Ptr service = *it;
+         bool autoload = service->property("X-KDE-Kded-autoload").toBool();
+         config->setGroup(QString("Module-%1").arg(service->desktopEntryName()));
+         autoload = config->readBoolEntry("autoload", autoload);
+         if (autoload)
+            loadModule(service, false);
+
+         bool dontLoad = false;
+         QVariant p = service->property("X-KDE-Kded-load-on-demand");
+         if (p.isValid() && (p.toBool() == false))
+            dontLoad = true;
+         if (dontLoad)
+            noDemandLoad(service->desktopEntryName());
+            
+         if (dontLoad && !autoload)
+            unloadModule(service->desktopEntryName().latin1());
+     }
+}
+
+
 void Kded::noDemandLoad(const QString &obj)
 {
   m_dontLoad.insert(obj.latin1(), this);
@@ -165,6 +186,11 @@ KDEDModule *Kded::loadModule(const KService *s, bool onDemand)
   KDEDModule *module = 0;
   if (s && !s->library().isEmpty())
   {
+    QCString obj = s->desktopEntryName().latin1();
+    KDEDModule *oldModule = m_modules.find(obj);
+    if (oldModule)
+       return oldModule;
+
     if (onDemand)
     {
       QVariant p = s->property("X-KDE-Kded-load-on-demand");
@@ -174,9 +200,8 @@ KDEDModule *Kded::loadModule(const KService *s, bool onDemand)
          return 0;
       }
     }
-    QCString obj = s->desktopEntryName().latin1();
     // get the library loader instance
- 
+
     KLibLoader *loader = KLibLoader::self();
 
     QVariant v = s->property("X-KDE-Factory");
@@ -185,7 +210,7 @@ KDEDModule *Kded::loadModule(const KService *s, bool onDemand)
       factory = s->library();
 
     factory = "create_" + factory;
-    QString libname = "kded_"+s->library(); 
+    QString libname = "kded_"+s->library();
 
     KLibrary *lib = loader->library(QFile::encodeName(libname));
     if (!lib)
@@ -201,7 +226,7 @@ KDEDModule *Kded::loadModule(const KService *s, bool onDemand)
       if (create)
       {
         // create the module
-        KDEDModule* (*func)(const QCString &); 
+        KDEDModule* (*func)(const QCString &);
         func = (KDEDModule* (*)(const QCString &)) create;
         module = func(obj);
         if (module)
@@ -280,12 +305,10 @@ void Kded::slotApplicationRemoved(const QCString &appId)
   }
 }
 
-void Kded::build()
+void Kded::updateDirWatch()
 {
   if (!b_checkUpdates) return;
 
-  KBuildSycoca* kbs = KBuildSycoca::createBuildSycoca();
-
   delete m_pDirWatch;
   m_pDirWatch = new KDirWatch;
 
@@ -296,42 +319,33 @@ void Kded::build()
   QObject::connect( m_pDirWatch, SIGNAL(deleted(const QString&)),
            this, SLOT(dirDeleted(const QString&)));
 
-  // It is very important to build the servicetype one first
-  // Both are registered in KSycoca, no need to keep the pointers
-  KSycocaFactory *stf = new KBuildServiceTypeFactory;
-  KBuildServiceGroupFactory *bsgf = new KBuildServiceGroupFactory;
-  (void) new KBuildServiceFactory(stf, bsgf);
-  // We don't include KImageIOFactory here because it doesn't add
-  // new resourceList entries anyway.
-  //(void) KImageIOFactory::self();
-  // Same for KBuildProtocolInfoFactory
-
-  KSycocaFactoryList *factoryList = kbs->factoryList();
-
-  // For each factory
-  QPtrListIterator<KSycocaFactory> factit ( *factoryList );
-  for (KSycocaFactory *factory = factoryList->first();
-       factory;
-       factory = factoryList->first() )
+  // For each resource
+  for( QStringList::ConstIterator it = m_allResourceDirs.begin();
+       it != m_allResourceDirs.end();
+       ++it )
   {
-    // For each resource the factory deals with
-    for( KSycocaResourceList::ConstIterator it1 = factory->resourceList()->begin();
-         it1 != factory->resourceList()->end();
-         ++it1 )
-    {
-      KSycocaResource res = (*it1);
-      QStringList dirs = KGlobal::dirs()->resourceDirs( res.resource.ascii() );
-      // For each resource the factory deals with
-      for( QStringList::ConstIterator it2 = dirs.begin();
-           it2 != dirs.end();
-           ++it2 )
-      {
-	readDirectory( *it2 );
-      }
-    }
-    factoryList->removeRef(factory);
+     readDirectory( *it );
+  }
+}
+
+void Kded::updateResourceList()
+{
+  delete KSycoca::self();
+
+  if (!b_checkUpdates) return;
+
+  QStringList dirs = KSycoca::self()->allResourceDirs();
+  // For each resource
+  for( QStringList::ConstIterator it = dirs.begin();
+       it != dirs.end();
+       ++it )
+  {
+     if (m_allResourceDirs.find(*it) == m_allResourceDirs.end())
+     {
+        m_allResourceDirs.append(*it);
+        readDirectory(*it);
+     }
   }
-  delete kbs;
 }
 
 void Kded::crashHandler(int)
@@ -339,8 +353,8 @@ void Kded::crashHandler(int)
    DCOPClient::emergencyClose();
    system("kded");
 qWarning("Last DCOP call before KDED crash was from application '%s'\n"
-         "to object '%s', function '%s'.", 
-         DCOPClient::postMortemSender(), 
+         "to object '%s', function '%s'.",
+         DCOPClient::postMortemSender(),
          DCOPClient::postMortemObject(),
          DCOPClient::postMortemFunction());
 }
@@ -352,47 +366,82 @@ void Kded::installCrashHandler()
 
 void Kded::recreate()
 {
+   recreate(true); // Async
+}
+
+void Kded::recreate(bool async)
+{
+   m_recreateBusy = true;
    // Using KLauncher here is difficult since we might not have a
    // database
 
-   build(); // Update tree first, to be sure to miss nothing.
+   updateDirWatch(); // Update tree first, to be sure to miss nothing.
 
-   runBuildSycoca();
-
-   while( !m_requests.isEmpty())
+   if (async)
    {
-      QCString replyType = "void";
-      QByteArray replyData;
-      kapp->dcopClient()->endTransaction(m_requests.first(), replyType, replyData);
-      m_requests.remove(m_requests.begin());
+      runBuildSycoca(this, SLOT(recreateDone()));
+   }
+   else
+   {
+      runBuildSycoca();
+      recreateDone();
    }
 }
 
-void Kded::dirDeleted(const QString& /*path*/)
+void Kded::recreateDone()
 {
-  // We could be smarter here, and find out which factory
-  // deals with that dir, and update only that...
-  // But rebuilding everything is fine for me.
-  m_pTimer->start( 2000, true /* single shot */ );
+   updateResourceList();
+
+   for(; m_recreateCount; m_recreateCount--)
+   {
+      QCString replyType = "void";
+      QByteArray replyData;
+      DCOPClientTransaction *transaction = m_recreateRequests.first();
+      if (transaction)
+         kapp->dcopClient()->endTransaction(transaction, replyType, replyData);
+      m_recreateRequests.remove(m_recreateRequests.begin());
+   }
+   m_recreateBusy = false;
+   
+   // Did a new request come in while building?
+   if (!m_recreateRequests.isEmpty())
+   {
+      m_pTimer->start(2000, true /* single shot */ );
+      m_recreateCount = m_recreateRequests.count();
+   }
+}
+
+void Kded::dirDeleted(const QString& path)
+{
+  update(path);
 }
 
 void Kded::update(const QString& )
 {
-  // We could be smarter here, and find out which factory
-  // deals with that dir, and update only that...
-  // But rebuilding everything is fine for me.
-  m_pTimer->start( 2000, true /* single shot */ );
+  if (!m_recreateBusy)
+  {
+    m_pTimer->start( 2000, true /* single shot */ );
+  }
+  else
+  {
+    m_recreateRequests.append(0);
+  }
 }
 
 bool Kded::process(const QCString &fun, const QByteArray &data,
                            QCString &replyType, QByteArray &replyData)
 {
   if (fun == "recreate()") {
-    if (m_requests.isEmpty())
+    if (!m_recreateBusy)
     {
-       m_pTimer->start(0, true /* single shot */ );
+       if (m_recreateRequests.isEmpty())
+       {
+          m_pTimer->start(0, true /* single shot */ );
+          m_recreateCount = 0;
+       }
+       m_recreateCount++;
     }
-    m_requests.append(kapp->dcopClient()->beginTransaction());
+    m_recreateRequests.append(kapp->dcopClient()->beginTransaction());
     replyType = "void";
     return true;
   } else {
@@ -528,7 +577,7 @@ KUpdateD::~KUpdateD()
 
 void KUpdateD::runKonfUpdate()
 {
-    ::runKonfUpdate();    
+    ::runKonfUpdate();
 }
 
 void KUpdateD::slotNewUpdateFile()
@@ -553,6 +602,7 @@ void KHostnameD::checkHostname()
     char buf[1024+1];
     if (gethostname(buf, 1024) != 0)
        return;
+    buf[sizeof(buf)-1] = '\0';
 
     if (m_hostname.isEmpty())
     {
@@ -566,14 +616,39 @@ void KHostnameD::checkHostname()
     QCString newHostname = buf;
 
     runDontChangeHostname(m_hostname, newHostname);
-    m_hostname = newHostname;    
+    m_hostname = newHostname;
 }
 
 
 static KCmdLineOptions options[] =
 {
-  { "check", I18N_NOOP("Check Sycoca database only once."), 0 },
-  { 0, 0, 0 }
+  { "check", I18N_NOOP("Check Sycoca database only once"), 0 },
+  KCmdLineLastOption
+};
+
+class KDEDQtDCOPObject : public DCOPObject
+{
+public:
+  KDEDQtDCOPObject() : DCOPObject("qt/kded") { }
+  
+  virtual bool process(const QCString &fun, const QByteArray &data,
+                       QCString& replyType, QByteArray &replyData)
+    {
+      if ( kapp && (fun == "quit()") )
+      {
+        kapp->quit();
+        replyType = "void";
+        return true;
+      }
+      return DCOPObject::process(fun, data, replyType, replyData);
+    }                       
+
+  QCStringList functions()
+    {
+       QCStringList res = DCOPObject::functions();
+       res += "void quit()";
+       return res;
+    }
 };
 
 class KDEDApplication : public KUniqueApplication
@@ -599,6 +674,8 @@ public:
        res += "void registerWindowId(long int)";
        res += "void unregisterWindowId(long int)";
        res += "QCStringList loadedModules()";
+       res += "void reconfigure()";
+       res += "void quit()";
        return res;
     }
 
@@ -614,7 +691,7 @@ public:
       QDataStream _replyStream( replyData, IO_WriteOnly );
       _replyStream << result;
       return true;
-    } 
+    }
     else if (fun == "unloadModule(QCString)") {
       QCString module;
       QDataStream arg( data, IO_ReadOnly );
@@ -631,6 +708,7 @@ public:
       arg >> windowId;
       Kded::self()->setCallingDcopClient(callingDcopClient());
       Kded::self()->registerWindowId(windowId);
+      replyType = "void";
       return true;
     }
      else if (fun == "unregisterWindowId(long int)") {
@@ -639,6 +717,7 @@ public:
       arg >> windowId;
       Kded::self()->setCallingDcopClient(callingDcopClient());
       Kded::self()->unregisterWindowId(windowId);
+      replyType = "void";
       return true;
     }
     else if (fun == "loadedModules()") {
@@ -647,30 +726,44 @@ public:
       _replyStream << Kded::self()->loadedModules();
       return true;
     }
+    else if (fun == "reconfigure()") {
+      config()->reparseConfiguration();
+      Kded::self()->initModules();
+      replyType = "void";
+      return true;
+    }
+    else if (fun == "quit()") {
+      quit();
+      replyType = "void";
+      return true;
+    }
     return KUniqueApplication::process(fun, data, replyType, replyData);
   }
 
   bool startup;
+  KDEDQtDCOPObject kdedQtDcopObject;
 };
 
-int main(int argc, char *argv[])
+extern "C" int kdemain(int argc, char *argv[])
 {
      KAboutData aboutData( "kded", I18N_NOOP("KDE Daemon"),
-        "$Id: kded.cpp,v 1.75.2.1 2003/03/14 22:36:21 waba Exp $",
+        "$Id: kded.cpp,v 1.88 2003/11/18 16:38:41 waba Exp $",
         I18N_NOOP("KDE Daemon - triggers Sycoca database updates when needed."));
 
+     KApplication::installSigpipeHandler();
+
      KCmdLineArgs::init(argc, argv, &aboutData);
 
      KUniqueApplication::addCmdLineOptions();
 
      KCmdLineArgs::addCmdLineOptions( options );
 
-     // this program is in kdelibs so it uses kdelibs as catalogue
+     // this program is in kdelibs so it uses kdelibs as catalog
      KLocale::setMainCatalogue("kdelibs");
 
      // WABA: Make sure not to enable session management.
      putenv(strdup("SESSION_MANAGER="));
-     
+
      // Parse command line before checking DCOP
      KCmdLineArgs *args = KCmdLineArgs::parsedArgs();
 
@@ -703,6 +796,8 @@ int main(int argc, char *argv[])
         exit(0);
      }
 
+     KUniqueApplication::dcopClient()->setQtBridgeEnabled(false);
+
      config->setGroup("General");
      int HostnamePollInterval = config->readNumEntry("HostnamePollInterval", 5000);
      bool bCheckSycoca = config->readBoolEntry("CheckSycoca", true);
@@ -716,7 +811,7 @@ int main(int argc, char *argv[])
      signal(SIGHUP, sighandler);
      KDEDApplication k;
 
-     kded->recreate();
+     kded->recreate(false);
 
      if (bCheckUpdates)
         (void) new KUpdateD; // Watch for updates
@@ -732,27 +827,8 @@ int main(int argc, char *argv[])
      client->setNotifications(true);
      client->setDaemonMode( true );
 
-     // Preload kded modules.
-     KService::List kdedModules = KServiceType::offers("KDEDModule");
-     for(KService::List::ConstIterator it = kdedModules.begin(); it != kdedModules.end(); ++it)
-     {
-         KService::Ptr service = *it;
-	     if (!service->property( "X-KDE-Kded-nostart").toBool())
-         {
-            bool autoload = service->property("X-KDE-Kded-autoload").toBool();
-            config->setGroup(QString("Module-%1").arg(service->desktopEntryName()));
-            autoload = config->readBoolEntry("autoload", autoload);
-            if (autoload)
-               kded->loadModule(service, false);
+     kded->initModules();
 
-            bool dontLoad = false;
-            QVariant p = service->property("X-KDE-Kded-load-on-demand");
-            if (p.isValid() && (p.toBool() == false))
-               dontLoad = true;
-            if (dontLoad)
-               kded->noDemandLoad(service->desktopEntryName());
-         }
-     }
 
      // During startup kdesktop waits for KDED to finish.
      // Send a notifyDatabaseChanged signal even if the database hasn't
@@ -765,10 +841,10 @@ int main(int argc, char *argv[])
      client->send( "ksplash", "", "upAndRunning(QString)",  QString("kded"));
 
      int result = k.exec(); // keep running
-       
+
      delete kded;
      delete instance; // Deletes config as well
-       
+
      return result;
 }
 
