@@ -71,6 +71,11 @@ public:
    bool isWindowRegistered(long windowId);
    void registerWindowId(long windowId);
    void unregisterWindowId(long windowId);
+   void recreate(bool async);
+   /**
+    * Loads / unloads modules according to config.
+    */
+   void initModules();
 
 public slots:
 
@@ -79,10 +84,20 @@ public slots:
     */
    void recreate();
 
+   /**
+    * Recreating finished
+    */
+   void recreateDone();
+
    /**
     * Collect all directories to watch
     */
-   void build();
+   void updateDirWatch();
+
+   /**
+    * Update directories to watch
+    */
+   void updateResourceList();
 
    /**
     * An application unregistered itself with DCOP
@@ -112,16 +127,13 @@ protected slots:
    void installCrashHandler();
 
 protected:
-
-
    /**
     * Scans dir for new files and new subdirectories.
     */
    void readDirectory(const QString& dir );
+   
 
    static void crashHandler(int);
-   
-protected:
 
    /**
     * Pointer to the dirwatch class which tells us, when some directories
@@ -139,12 +151,16 @@ protected:
     */
    QTimer* m_pTimer;
    
-   QValueList<DCOPClientTransaction *> m_requests;
+   QValueList<DCOPClientTransaction *> m_recreateRequests;
+   int m_recreateCount;
+   bool m_recreateBusy;
+   
    QAsciiDict<KDEDModule> m_modules;
    QAsciiDict<KLibrary> m_libs;
    QAsciiDict<QObject> m_dontLoad;
    QAsciiDict<QValueList<long> > m_windowIdList;
    QIntDict<long> m_globalWindowIdList;
+   QStringList m_allResourceDirs;
      
    static Kded *_self;
 };
