@@ -26,7 +26,7 @@ THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
 */
 
-// $Id: kimageeffect.h,v 1.25 2002/07/31 23:14:10 orlovich Exp $
+// $Id: kimageeffect.h,v 1.37 2003/10/07 22:40:42 mueller Exp $
 
 #ifndef __KIMAGE_EFFECT_H
 #define __KIMAGE_EFFECT_H
@@ -34,34 +34,91 @@ THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 class QImage;
 class QSize;
 class QColor;
+class QPoint;
+class QRect;
 
 /**
- * This class includes various @ref QImage based graphical effects.
+ * This class includes various QImage based graphical effects.
  *
  * Everything is
  * static, so there is no need to create an instance of this class. You can
  * just call the static methods. They are encapsulated here merely to provide
  * a common namespace.
  */
-
 class KImageEffect
 {
 public:
-    enum GradientType { VerticalGradient, HorizontalGradient,
-                        DiagonalGradient, CrossDiagonalGradient,
-                        PyramidGradient, RectangleGradient,
-                        PipeCrossGradient, EllipticGradient };
-    enum RGBComponent { Red, Green, Blue, Gray, All };
+    /** 
+     * This enum provides a gradient type specification
+     * @see KImageEffect::blend(), KImageEffect::gradient(),
+     * KImageEffect::unbalancedGradient()
+     */
+    enum GradientType { VerticalGradient,
+			HorizontalGradient,
+                        DiagonalGradient,
+			CrossDiagonalGradient,
+                        PyramidGradient,
+			RectangleGradient,
+                        PipeCrossGradient,
+			EllipticGradient
+    };
 
-    enum Lighting {NorthLite, NWLite, WestLite, SWLite,
-                   SouthLite, SELite, EastLite, NELite};
+    /** 
+     * This enum provides a RGB channel specification
+     * @see KImageEffect::blend(), KImageEffect::channelIntensity(),
+     * KImageEffect::modulate()
+     */
+    enum RGBComponent { Red,   //!< Red channel
+			Green, //!< Green channel
+			Blue,  //!< Blue channel
+			Gray,  //!< Grey channel
+			All    //!< All channels
+    };
 
-    enum ModulationType { Intensity, Saturation, HueShift, Contrast };
+    /** 
+     * This enum provides a lighting direction specification
+     * @see KImageEffect::hash()
+     */
+    enum Lighting {NorthLite, //!< Lighting from the top of the image
+		   NWLite,    //!< Lighting from the top left of the image
+		   WestLite,  //!< Lighting from the left of the image
+		   SWLite,    //!< Lighting from the bottom left of the image
+                   SouthLite, //!< Lighting from the bottom of the image
+		   SELite,    //!< Lighting from the bottom right of the image
+		   EastLite,  //!< Lighting from the right of the image
+		   NELite     //!< Lighting from the top right of the image
+    };
 
-    enum NoiseType { UniformNoise=0, GaussianNoise, MultiplicativeGaussianNoise,
-                     ImpulseNoise, LaplacianNoise, PoissonNoise};
+    /** 
+     * This enum provides a modulation type specification
+     * @see KImageEffect::modulate()
+     */
+    enum ModulationType { Intensity,  //!< Modulate image intensity
+			  Saturation, //!< Modulate image saturation
+			  HueShift,   //!< Modulate image hue
+			  Contrast    //!< Modulate image contrast
+    };
 
-    enum RotateDirection{ Rotate90, Rotate180, Rotate270 };
+    /** 
+     * This enum provides a noise type specification
+     * @see KImageEffect::addNoise()
+     */
+    enum NoiseType { UniformNoise=0,              //!< Uniform distribution
+		     GaussianNoise,               //!< Gaussian distribution
+		     MultiplicativeGaussianNoise, //!< Multiplicative Gaussian distribution
+                     ImpulseNoise,                //!< Impulse distribution
+		     LaplacianNoise,              //!< Laplacian distribution
+		     PoissonNoise                 //!< Poisson distribution
+    };  
+
+    /**
+     * This enum provides a rotation specification.
+     * @see KImageEffect::rotate()
+     */
+    enum RotateDirection{ Rotate90,  //!< Rotate 90 degrees to the right.
+			  Rotate180, //!< Rotate 180 degrees.
+			  Rotate270  //!< Rotate 90 degrees to the left.
+    };
 
     /**
      * Create a gradient from color a to color b of the specified type.
@@ -79,7 +136,7 @@ public:
 
     /**
      * Create an unbalanced gradient.
-
+     *
      * An unbalanced gradient is a gradient where the transition from
      * color a to color b is not linear, but in this case, exponential.
      *
@@ -89,7 +146,7 @@ public:
      * @param type The type of gradient.
      * @param xfactor The x decay length. Use a value between -200 and 200.
      * @param yfactor The y decay length.
-     * @param ncols The number of colors. See KPixmapEffect:gradient.
+     * @param ncols The number of colors. See KImageEffect:gradient.
      */
     static QImage unbalancedGradient(const QSize &size, const QColor &ca,
 	    const QColor &cb, GradientType type, int xfactor = 100,
@@ -100,12 +157,16 @@ public:
      * value for blending one into another. Very fast direct pixel
      * manipulation is used.
      *
-     * @author Karol Szwed (gallium@kde.org)
+     * This function uses MMX and SSE2 instructions to blend the
+     * image on processors that support it.
+     *
      * @param clr source color to be blended into the destination image.
      * @param dst destination image in which the source will be blended into.
-     * @param opacity opacity (in percent) which determines how much the source
-     *                color will be blended into the destination image.
+     * @param opacity opacity (between 0.0 and 1.0) which determines how much
+     *             the source color will be blended into the destination image.
      * @return The destination image (dst) containing the result.
+     * @author Karol Szwed (gallium@kde.org)
+     * @author Fredrik H&ouml;glund (fredrik@kde.org)
      */
     static QImage& blend(const QColor& clr, QImage& dst, float opacity);
 
@@ -114,16 +175,20 @@ public:
      * value for blending one into another. Very fast direct pixel
      * manipulation is used.
      *
-     * @author Karol Szwed (gallium@kde.org)
+     * This function uses MMX and SSE2 instructions to blend the
+     * images on processors that support it.
+     *
      * @param src source image to be blended into the destination image.
      * @param dst destination image in which the source will be blended into.
-     * @param opacity opacity (in percent) which determines how much the source
-     *                image will be blended into the destination image.
+     * @param opacity opacity (between 0.0 and 1.0) which determines how much
+     *             the source image will be blended into the destination image.
      * @return The destination image (dst) containing the result.
+     * @author Karol Szwed (gallium@kde.org)
+     * @author Fredrik H&ouml;glund (fredrik@kde.org)
      */
     static QImage& blend(QImage& src, QImage& dst, float opacity);
 
-     /**
+    /**
      * Blend the provided image into a background of the indicated color.
      *
      * @param initial_intensity this parameter takes values from -1 to 1:
@@ -163,7 +228,7 @@ public:
      * @param blendImage If the gray value of of pixel is 0, the result
      *               for this pixel is that of image1; for a gray value
      *               of 1, the pixel of image2 is used; for a value
-     *               inbetween, a corresponding blending is used.
+     *               in between, a corresponding blending is used.
      * @param channel The RBG channel to use for the blending decision.
      */
     static QImage& blend(QImage &image1, QImage &image2,
@@ -171,6 +236,9 @@ public:
 
     /**
      * Blend an image into another one, using alpha in the expected way.
+     * @param upper the "upper" image
+     * @param lower the "lower" image
+     * @param output the target image
      * @author Rik Hemsley (rikkus) <rik@kde.org>
      */
     static bool blend(const QImage & upper, const QImage & lower, QImage & output);
@@ -186,24 +254,107 @@ public:
      * painted on lower, if there has to be some clipping, output's size will
      * be the clipped area and x and y will be set to the correct up-left corner
      * where the clipped rectangle begins.
+     * @param x x-coordinate of lower image
+     * @param y y-coordinate of lower image
+     * @param upper the "upper" image
+     * @param lower the "lower" image
+     * @param output the target image
      */
     static bool blend(int &x, int &y, const QImage & upper, const QImage & lower, QImage & output);
+
     /**
      * Blend an image into another one, using alpha in the expected way and
      * over coordinates @p x and @p y with respect to the lower image.
      * The output is painted in the own @p lower image. This is an optimization
      * of the blend method above provided by convenience.
+     * @param x x-coordinate of lower image
+     * @param y y-coordinate of lower image
+     * @param upper the "upper" image
+     * @param lower the "lower" image, which becomes the output image
      */
     static bool blendOnLower(int x, int y, const QImage & upper, const QImage & lower);
 
+    /** 
+     * Blend part of an image into part of another, using the alpha channel in
+     * the expected way.
+     * Note that the destination rectangle will be correctly clipped.
+     *
+     * @param upper the "upper" image
+     * @param upperOffset Offset for the part of the upper image to be used.
+     * @param lower the "lower" image
+     * @param lowerRect Rectangle for the part of the lower image where the
+     *                  blending will occur.
+     * @since 3.2
+     */
+    static void blendOnLower(const QImage &upper, const QPoint &upperOffset,
+                             QImage &lower, const QRect &lowerRect);
+
+    /** 
+     * Blend part of an image into part of another, using the opacity value
+     * and the alpha channel in the expected way.
+     * Note that the destination rectangle will be correctly clipped.
+     *
+     * @param upper the "upper" image
+     * @param upperOffset Offset for the part of the upper image to be used.
+     * @param lower the "lower" image
+     * @param lowerRect Rectangle for the part of the lower image where the
+     *                  blending will occur.
+     * @param opacity Opacity (between 0.0 and 1.0) which determines how much
+     *             the source image will be blended into the destination image.
+     * @since 3.2
+     */
+    static void blendOnLower(const QImage &upper, const QPoint &upperOffset,
+                             QImage &lower, const QRect &lowerRect, float opacity);
+
+    /** 
+     * Disposition of a source image on top of a destination image.
+     * @see KImageEffect::computeDestinationRect, KImageEffect::blendOnLower
+     * @since 3.2
+     */
+    enum Disposition { NoImage = 0, //!< Don't overlay
+		       Centered,    //!< Center top image on botton image
+		       Tiled,       //!< Tile top image on bottom image
+		       CenterTiled, //!< Center and tile top image on bottom image
+		       CenteredMaxpect, //!< Center and scale aspect
+		       TiledMaxpect, //!< Tile and scale aspect
+		       Scaled,      //!< Scale
+		       CenteredAutoFit //!< Center and scale or scale aspect
+    };
+
+    /**
+     * Compute the destination rectangle where to draw the upper image on top
+     * of another image using the given disposition. For tiled
+     * disposition, the rectangle should be duplicated on the whole area to
+     * obtained the wanted effect.
+     *
+     * @param lowerSize The size of the destination image.
+     * @param disposition The wanted disposition.
+     * @param upper The upper image. Note that this image may be scaled to
+     *               adjust to the requested disposition.
+     *
+     * @return the computed rectangle. Its size may exceed @e lowerSize.
+     * @since 3.2
+     */
+    static QRect computeDestinationRect(const QSize &lowerSize,
+                                      Disposition disposition, QImage &upper);
+
+    /**
+     * Blend an image on top of another using a given disposition and a given
+     * opacity. The alpha channel of the upper image is used in the expected
+     * way. Beware the upper image may be modified.
+     * @since 3.2
+     */
+    static void blendOnLower(QImage &upper, QImage &lower,
+                             Disposition disposition, float opacity);
+
     /**
      * Modifies the intensity of a pixmap's RGB channel component.
      *
-     * @author Daniel M. Duley (mosfet)
      * @param image The QImage to process.
      * @param percent Percent value. Use a negative value to dim.
      * @param channel Which channel(s) should be modified
      * @return The @p image, provided for convenience.
+     * @author Daniel M. Duley (mosfet)
      */
     static QImage& channelIntensity(QImage &image, float percent,
                                     RGBComponent channel);
@@ -214,11 +365,11 @@ public:
      * The number of colors will not be changed.
      *
      * @param image The QImage to process.
-     * @param val The strength of the effect. 0 <= val <= 1.
+     * @param val   The strength of the effect. 0 <= val <= 1.
      * @param color The background color.
-     * @return Returns the @ref image(), provided for convenience.
+     * @return Returns the image(), provided for convenience.
      */
-    static QImage& fade(QImage &img, float val, const QColor &color);
+    static QImage& fade(QImage &image, float val, const QColor &color);
 
 
     /**
@@ -228,17 +379,19 @@ public:
      * @param image A QImage to process.
      * @param ca Color a
      * @param cb Color b
+     * @param ncols The number of colors to dither the image to.
+     * Pass 0 to prevent dithering.
      */
     static QImage& flatten(QImage &image, const QColor &ca,
            const QColor &cb, int ncols=0);
 
     /**
-     * Build a hash on any given @ref QImage
+     * Build a hash on any given QImage
      *
      * @param image The QImage to process
      * @param lite The hash faces the indicated lighting (cardinal poles).
-     * @param spacing How many unmodified pixels inbetween hashes.
-     * @return Returns the @ref image(), provided for convenience.
+     * @param spacing How many unmodified pixels in between hashes.
+     * @return Returns the image(), provided for convenience.
      */
     static QImage& hash(QImage &image, Lighting lite=NorthLite,
                         unsigned int spacing=0);
@@ -247,10 +400,14 @@ public:
      * Either brighten or dim the image by a specified percent.
      * For example, .50 will modify the colors by 50%.
      *
-     * @author Daniel M. Duley (mosfet)
+     * This function uses MMX instructions to process the image
+     * on processors that support it.
+     *
      * @param image The QImage to process.
      * @param percent The percent value. Use a negative value to dim.
-     * @return Returns The @ref image(), provided for convenience.
+     * @return Returns The image(), provided for convenience.
+     * @author Daniel M. Duley (mosfet)
+     * @author Benjamin Roe (ben@benroe.com)
      */
     static QImage& intensity(QImage &image, float percent);
 
@@ -263,7 +420,7 @@ public:
      * @param type The modulation Type to use.
      * @param factor The modulation amplitude; with 0 no effect [-200;200].
      * @param channel The RBG channel of image2 to use for modulation.
-     * @return Returns the @ref image(), provided for convenience.
+     * @return Returns the image(), provided for convenience.
      */
     static QImage& modulate(QImage &image, QImage &modImage, bool reverse,
 		ModulationType type, int factor, RGBComponent channel);
@@ -271,11 +428,11 @@ public:
     /**
      * Convert an image to grayscale.
      *
-     * @author Daniel M. Duley (mosfet)
-     * @param image The @ref QImage to process.
+     * @param image The QImage to process.
      * @param fast Set to @p true in order to use a faster but non-photographic
      * quality algorithm. Appropriate for things such as toolbar icons.
-     * @return Returns the @ref image(), provided for convenience.
+     * @return Returns the image(), provided for convenience.
+     * @author Daniel M. Duley (mosfet)
      */
     static QImage& toGray(QImage &image, bool fast = false);
 
@@ -284,17 +441,17 @@ public:
      *
      * @param image The QImage to process.
      * @param desat A value between 0 and 1 setting the degree of desaturation
-     * @return Returns the @ref image(), provided for convenience.
+     * @return Returns the image(), provided for convenience.
      */
     static QImage& desaturate(QImage &image, float desat = 0.3);
 
     /**
      * Fast, but low quality contrast of an image. Also see contrastHSV.
      *
-     * @author Daniel M. Duley (mosfet)
      * @param image The QImage to process.
      * @param c A contrast value between -255 to 255.
-     * @return The @ref image(), provided for convenience.
+     * @return The image(), provided for convenience.
+     * @author Daniel M. Duley (mosfet)
      */
     static QImage& contrast(QImage &image, int c);
 
@@ -305,9 +462,9 @@ public:
      * @param image The QImage to process.
      * @param palette The color palette to use
      * @param size The size of the palette
-     * @return Returns the @ref image(), provided for convenience.
+     * @return Returns the image(), provided for convenience.
      */
-    static QImage& dither(QImage &img, const QColor *palette, int size);
+    static QImage& dither(QImage &image, const QColor *palette, int size);
 
     /**
      * Calculate the image for a selected image, for instance a selected icon
@@ -323,23 +480,25 @@ public:
      * channels above it and decrementing those below it, but this gives much
      * better results.
      *
-     * @author Daniel M. Duley (mosfet)
      * @param img The QImage to process.
      * @param sharpen If true sharpness is increase, (spiffed). Otherwise
      * it is decreased, (dulled).
+     * @author Daniel M. Duley (mosfet)
      */
     static void contrastHSV(QImage &img, bool sharpen=true);
 
     /**
-     * Normalizes the pixel values to span the full range of color values.
+     * Normalises the pixel values to span the full range of color values.
      * This is a contrast enhancement technique.
+     * @param img the image that is normalised 
      * @author Daniel M. Duley (mosfet)
      */
     static void normalize(QImage &img);
 
     /**
-     * Performs histogram equalization on the reference
+     * Performs histogram equalisation on the reference
      * image.
+     * @param img the image that is equalised
      * @author Daniel M. Duley (mosfet)
      */
     static void equalize(QImage &img);
@@ -349,9 +508,9 @@ public:
      * ThresholdDither in the various QPixmap/QImage convert methods, but this
      * lets you specify a threshold value.
      *
-     * @author Daniel M. Duley (mosfet)
      * @param img The QImage to process.
      * @param value The threshold value.
+     * @author Daniel M. Duley (mosfet)
      */
     static void threshold(QImage &img, unsigned int value=128);
 
@@ -359,9 +518,9 @@ public:
      * Produces a 'solarization' effect seen when exposing a photographic
      * film to light during the development process.
      *
-     * @author Daniel M. Duley (mosfet)
      * @param img The QImage to process.
      * @param factor The extent of the solarization (0-99.9)
+     * @author Daniel M. Duley (mosfet)
      */
     static void solarize(QImage &img, double factor=50.0);
 
@@ -370,9 +529,18 @@ public:
      * and applying various other enhancements in order to get a metal
      * effect.
      *
-     * @author Daniel M. Duley (mosfet)
      * @param src The QImage to process.
+     * @param radius The radius of the gaussian not counting the
+     * center pixel. Use 0 and a suitable radius will be automatically used.
+     * @param sigma The standard deviation of the gaussian. Use 1 if you're not
+     * sure.
      * @return The embossed image. The original is not changed.
+     * @author Daniel M. Duley (mosfet)
+     */
+    static QImage emboss(QImage &src, double radius, double sigma);
+
+    /**
+     * Convenience method.
      */
     static QImage emboss(QImage &src);
 
@@ -380,29 +548,38 @@ public:
      * Minimizes speckle noise in the source image using the 8 hull
      * algorithm.
      *
-     * @author Daniel M. Duley (mosfet)
      * @param src The QImage to process.
      * @return The despeckled image. The original is not changed.
+     * @author Daniel M. Duley (mosfet)
      */
     static QImage despeckle(QImage &src);
 
     /**
      * Produces a neat little "charcoal" effect.
      *
-     * @author Daniel M. Duley (mosfet)
      * @param src The QImage to process.
-     * @param factor The factor for detecting lines (0-99.0).
+     * @param radius The radius of the gaussian not counting the
+     * center pixel. Use 0 and a suitable radius will be automatically used.
+     * @param sigma The standard deviation of the gaussian. Use 1 if you're not
+     * sure.
      * @return The charcoal image. The original is not changed.
+     * @author Daniel M. Duley (mosfet)
      */
-    static QImage charcoal(QImage &src, double factor=50.0);
+    static QImage charcoal(QImage &src, double radius, double sigma);
+
+    /**
+     * This is provided for binary compatability only! Use the above method
+     * with a radius and sigma instead!
+     */
+     static QImage charcoal(QImage &src, double factor=50.0);
 
     /**
      * Rotates the image by the specified amount
      *
-     * @author Daniel M. Duley (mosfet)
      * @param src The QImage to process.
      * @param r The rotate direction.
      * @return The rotated image. The original is not changed.
+     * @author Daniel M. Duley (mosfet)
      */
     static QImage rotate(QImage &src, RotateDirection r);
 
@@ -411,31 +588,40 @@ public:
      * nearly as nice a result as QImage::smoothScale(), but has the
      * advantage of being much faster - only a few milliseconds.
      *
-     * @author Daniel M. Duley (mosfet)
      * @param src The QImage to process.
      * @param w The new width.
      * @param h The new height.
      * @return The scaled image. The original is not changed.
+     * @author Daniel M. Duley (mosfet)
      */
     static QImage sample(QImage &src, int w, int h);
 
     /**
      * Adds noise to an image.
      *
-     * @author Daniel M. Duley (mosfet)
      * @param src The QImage to process.
      * @param type The algorithm used to generate the noise.
      * @return The image with noise added. The original is not changed.
+     * @author Daniel M. Duley (mosfet)
      */
     static QImage addNoise(QImage &src, NoiseType type = GaussianNoise);
 
     /**
      * Blurs an image by convolving pixel neighborhoods.
      *
-     * @author Daniel M. Duley (mosfet)
      * @param src The QImage to process.
-     * @param factor The percent weight to give to the center pixel.
+     * @param radius The radius of the gaussian not counting the
+     * center pixel. Use 0 and a suitable radius will be automatically used.
+     * @param sigma The standard deviation of the gaussian. Use 1 if you're not
+     * sure.
      * @return The blurred image. The original is not changed.
+     * @author Daniel M. Duley (mosfet)
+     */
+    static QImage blur(QImage &src, double radius, double sigma);
+
+    /**
+     * This is provided for binary compatability only! Use the above method
+     * with a radius and sigma instead!
      */
     static QImage blur(QImage &src, double factor=50.0);
 
@@ -443,77 +629,94 @@ public:
      * Detects edges in an image using pixel neighborhoods and an edge
      * detection mask.
      *
-     * @author Daniel M. Duley (mosfet)
      * @param src The QImage to process.
-     * @param factor The percent weight to give to the center pixel.
+     * @param radius The radius of the gaussian not counting the
+     * center pixel. Use 0 and a suitable radius will be automatically used.
      * @return The image with edges detected. The original is not changed.
+     * @author Daniel M. Duley (mosfet)
      */
-    static QImage edge(QImage &src, double factor=50.0);
+    static QImage edge(QImage &src, double radius);
 
     /**
      * Implodes an image by a specified percent.
      *
-     * @author Daniel M. Duley (mosfet)
      * @param src The QImage to process.
      * @param factor The extent of the implosion.
      * @param background An RGBA value to use for the background. After the
      * effect some pixels may be "empty". This value is used for those pixels.
      * @return The imploded image. The original is not changed.
+     * @author Daniel M. Duley (mosfet)
      */
     static QImage implode(QImage &src, double factor=30.0,
                    unsigned int background = 0xFFFFFFFF);
+
     /**
      * Produces an oil painting effect.
      *
-     * @author Daniel M. Duley (mosfet)
      * @param src The QImage to process.
-     * @param radius The radius of the pixel neighborhood used in applying the
-     * effect.
+     * @param radius The radius of the gaussian not counting the
+     * center pixel. Use 0 and a suitable radius will be automatically used.
      * @return The new image. The original is not changed.
+     * @author Daniel M. Duley (mosfet)
+     */
+    static QImage oilPaintConvolve(QImage &src, double radius);
+
+    /**
+     * This is provided for binary compatability only! Use the above method
+     * instead!
      */
     static QImage oilPaint(QImage &src, int radius=3);
 
     /**
      * Sharpens the pixels in the image using pixel neighborhoods.
      *
-     * @author Daniel M. Duley (mosfet)
      * @param src The QImage to process.
-     * @param factor The percent weight to give to the center pixel.
+     * @param radius The radius of the gaussian not counting the
+     * center pixel. Use 0 and a suitable radius will be automatically used.
+     * @param sigma The standard deviation of the gaussian. Use 1 if you're not
+     * sure.
      * @return The sharpened image. The original is not changed.
+     * @author Daniel M. Duley (mosfet)
+     */
+    static QImage sharpen(QImage &src, double radius, double sigma);
+
+    /**
+     * This is provided for binary compatability only! Use the above method
+     * instead!
      */
     static QImage sharpen(QImage &src, double factor=30.0);
 
     /**
      * Randomly displaces pixels.
      *
-     * @author Daniel M. Duley (mosfet)
      * @param src The QImage to process.
      * @param amount The vicinity for choosing a random pixel to swap.
      * @return The image with pixels displaced. The original is not changed.
+     * @author Daniel M. Duley (mosfet)
      */
     static QImage spread(QImage &src, unsigned int amount=3);
 
     /**
      * Shades the image using a distance light source.
      *
-     * @author Daniel M. Duley (mosfet)
      * @param src The QImage to process.
      * @param color_shading If true do color shading, otherwise do grayscale.
      * @param azimuth Determines the light source and direction.
      * @param elevation Determines the light source and direction.
      * @return The shaded image. The original is not changed.
+     * @author Daniel M. Duley (mosfet)
      */
     static QImage shade(QImage &src, bool color_shading=true, double azimuth=30.0,
                         double elevation=30.0);
     /**
      * Swirls the image by a specified amount
      *
-     * @author Daniel M. Duley (mosfet)
      * @param src The QImage to process.
      * @param degrees The tightness of the swirl.
      * @param background An RGBA value to use for the background. After the
      * effect some pixels may be "empty". This value is used for those pixels.
      * @return The swirled image. The original is not changed.
+     * @author Daniel M. Duley (mosfet)
      */
      static QImage swirl(QImage &src, double degrees=50.0, unsigned int background =
                          0xFFFFFFFF);
@@ -521,11 +724,13 @@ public:
      /**
       * Modifies the pixels along a sine wave.
       *
-      * @author Daniel M. Duley (mosfet)
-      * @param src The QImage to process.
-      * @param amplitude The amplitude of the sine wave.
-      * @param wavelength The frequency of the sine wave.
+      * @param src        The QImage to process.
+      * @param amplitude  The amplitude of the sine wave.
+      * @param frequency  The frequency of the sine wave.
+      * @param background An RGBA value to use for the background. After the
+      * effect some pixels may be "empty". This value is used for those pixels.
       * @return The new image. The original is not changed.
+      * @author Daniel M. Duley (mosfet)
       */
      static QImage wave(QImage &src, double amplitude=25.0, double frequency=150.0,
                         unsigned int background = 0xFFFFFFFF);
@@ -550,6 +755,15 @@ private:
     static unsigned int generateNoise(unsigned int pixel, NoiseType type);
     static unsigned int interpolateColor(QImage *image, double x, double y,
                                          unsigned int background);
+    /* Various convolve routines */
+    static int getOptimalKernelWidth(double radius, double sigma);
+    static bool convolveImage(QImage *image, QImage *dest,
+                              const unsigned int order,
+                              const double *kernel);
+    static void blurScanLine(double *kernel, int width,
+                             unsigned int *src, unsigned int *dest,
+                             int columns);
+    static int getBlurKernel(int width, double sigma, double **kernel);
 };
 
 #endif
