@@ -3,7 +3,7 @@
  *  Copyright (C) 1998	Mark Donohoe <donohoe@kde.org>
  * 			Stephan Kulow <coolo@kde.org>
  *
- *  $Id: kpixmap.h,v 1.27 2002/07/31 23:14:10 orlovich Exp $
+ *  $Id: kpixmap.h,v 1.31 2003/10/06 09:07:10 bhards Exp $
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Library General Public
@@ -50,13 +50,29 @@ class KPixmapPrivate;
  * the desktop which are always present use no more than 40 colors.
 
  * @author Mark Donohoe (donohoe@kde.org)
- * @version $Id: kpixmap.h,v 1.27 2002/07/31 23:14:10 orlovich Exp $
+ * @version $Id: kpixmap.h,v 1.31 2003/10/06 09:07:10 bhards Exp $
  */
 class KPixmap : public QPixmap
 {
 public:
-        enum ColorMode { Auto, Color, Mono, LowColor, WebColor };
-        enum GradientMode { Horizontal, Vertical, Diagonal, CrossDiagonal };
+	/**
+	 * This enumeration provides a color pallete specification
+	 * @see KPixmap::convertFromImage(), KPixmap::load()
+	 */
+        enum ColorMode { Auto,   //!< Convert to monochrome if possible
+			 Color,  //!< Native display depth
+			 Mono,   //!< Monochrome pixmap
+			 LowColor, //!< 3x3x3 color cube (or monochrome)
+			 WebColor //!< Netscape pallete (or monochrome)
+	};
+	/**
+	 * This enumeration provides a gradient mode specification
+	 */
+        enum GradientMode { Horizontal,
+			    Vertical,
+			    Diagonal,
+			    CrossDiagonal
+	};
 
 	/**
 	 * Constructs a null pixmap.
@@ -65,8 +81,9 @@ public:
 
 	/**
 	 * Destructs the pixmap.
+         * ### KDE 4: remove
 	 */
-	~KPixmap() {};
+	~KPixmap();
 
 	/**
 	 * Copies the QPixmap @p pix.
@@ -118,12 +135,15 @@ public:
 	 * is being converted to 8-bits. This is the default when
 	 * converting to a pixmap.
 	 * @li AvoidDither - Only dither 32-bit images if the image has
-	 * more than 256 colours and it is being converted to 8-bits.
+	 * more than 256 colors and it is being converted to 8-bits.
 	 * This is the default when an image is converted for the
 	 * purpose of saving to a file.
 	 *
 	 * Passing 0 for @p conversion_flags gives all the default
 	 * options.
+	 *
+	 * @param img the image to convert
+	 * @param conversion_flags bitmask, described above
 	 * @return @p true if successful.
 	 **/
 	bool convertFromImage( const QImage &img, int conversion_flags );
@@ -132,6 +152,9 @@ public:
 	 * This is an overloaded member function, provided for
 	 * convenience. It differs from the above function only in
 	 * what argument(s) it accepts.
+	 * @param img the image to convert
+	 * @param mode a ColorMode to apply
+	 * @return @p true if successful.
 	 **/
 	bool convertFromImage( const QImage &img, ColorMode mode = WebColor );
 
@@ -143,14 +166,15 @@ public:
 	 * specified (default), the loader reads a few bytes from the
 	 * header to guess the file format.
 	 *
-	 * See the @ref convertFromImage() documentation for a description
-	 * of the conversion_flags argument.
-	 *
-	 * The @ref QImageIO documentation lists the supported image
+	 * The QImageIO documentation lists the supported image
 	 * formats and explains how to add extra formats.
 	 *
+	 * @param fileName the name of the file to load the image from
+	 * @param format the format for the image
+	 * @param conversion_flags a bitmask, as described in 
+	 *        convertFromImage()
 	 * @return @p true if successful, or false if the pixmap
-	 *  could not be loaded.
+	 *         could not be loaded.
 	 **/
 	bool load( const QString& fileName, const char *format,
 		int conversion_flags );
@@ -159,6 +183,11 @@ public:
 	 * This is an overloaded member function, provided for
 	 * convenience. It differs from the above function only in
 	 * what argument(s) it accepts.
+	 * @param fileName the name of the file to load the image from
+	 * @param format the format for the image
+	 * @param mode a ColorMode to apply
+	 * @return @p true if successful, or false if the pixmap
+	 *         could not be loaded.
 	 **/
 	bool load( const QString& fileName,
 		const char *format = 0,
@@ -171,6 +200,7 @@ public:
 	 * An image with one color not found in the Icon palette is
 	 * considered to be a match, since this extra color may be a
 	 * transparent background.
+	 * @param image the image to test
 	 **/
 	bool checkColorTable(const QImage &image);
 
