@@ -1,5 +1,4 @@
 /*
- * $Id: kstyle.cpp,v 1.30.2.1 2003/07/15 00:40:50 mueller Exp $
  *
  * KStyle
  * Copyright (C) 2001-2002 Karol Szwed <gallium@kde.org>
@@ -51,12 +50,18 @@
 #include <kimageeffect.h>
 #include "kstyle.h"
 
+//#if defined Q_WS_X11
+#if defined Q_WS_X11 && ! defined K_WS_QTONLY
 #include <X11/Xlib.h>
-#ifdef HAVE_XRENDER
-#include <X11/extensions/Xrender.h>
+#  ifdef HAVE_XRENDER
+#  include <X11/extensions/Xrender.h> // schroder
 extern bool qt_use_xrender;
+#  endif
+#else
+#undef HAVE_XRENDER
 #endif
 
+
 #include <limits.h>
 
 namespace
@@ -111,24 +116,8 @@ namespace
 }
 
 
-struct KStylePrivate
+namespace
 {
-	bool  highcolor                : 1;
-	bool  useFilledFrameWorkaround : 1;
-	bool  etchDisabledText         : 1;
-	bool  scrollablePopupmenus     : 1;
-	bool  menuAltKeyNavigation     : 1;
-	bool  menuDropShadow           : 1;
-	int   popupMenuDelay;
-	float menuOpacity;
-
-	TransparencyEngine   transparencyEngine;
-	KStyle::KStyleScrollBarType  scrollbarType;
-	TransparencyHandler* menuHandler;
-	KStyle::KStyleFlags flags;
-};
-
-
 class TransparencyHandler : public QObject
 {
 	public:
@@ -154,7 +143,25 @@ class TransparencyHandler : public QObject
 		KStyle* kstyle;
 		TransparencyEngine te;
 };
+} // namespace
 
+struct KStylePrivate
+{
+	bool  highcolor                : 1;
+	bool  useFilledFrameWorkaround : 1;
+	bool  etchDisabledText         : 1;
+	bool  scrollablePopupmenus     : 1;
+	bool  menuAltKeyNavigation     : 1;
+	bool  menuDropShadow           : 1;
+	bool  sloppySubMenus           : 1;
+	int   popupMenuDelay;
+	float menuOpacity;
+
+	TransparencyEngine   transparencyEngine;
+	KStyle::KStyleScrollBarType  scrollbarType;
+	TransparencyHandler* menuHandler;
+	KStyle::KStyleFlags flags;
+};
 
 // -----------------------------------------------------------------------------
 
@@ -171,6 +178,7 @@ KStyle::KStyle( KStyleFlags flags, KStyleScrollBarType sbtype )
 	// Read style settings
 	QSettings settings;
 	d->popupMenuDelay       = settings.readNumEntry ("/KStyle/Settings/PopupMenuDelay", 256);
+	d->sloppySubMenus       = settings.readBoolEntry("/KStyle/Settings/SloppySubMenus", false);
 	d->etchDisabledText     = settings.readBoolEntry("/KStyle/Settings/EtchDisabledText", true);
 	d->menuAltKeyNavigation = settings.readBoolEntry("/KStyle/Settings/MenuAltKeyNavigation", true);
 	d->scrollablePopupmenus = settings.readBoolEntry("/KStyle/Settings/ScrollablePopupMenus", false);
@@ -231,10 +239,11 @@ void KStyle::polish( QWidget* widget )
 {
 	if ( d->useFilledFrameWorkaround )
 	{
-		if (widget->inherits("QToolBar"))
-			widget->installEventFilter(this);
-		if (widget->inherits("QMenuBar"))
-			widget->installEventFilter(this);
+		if ( QFrame *frame = ::qt_cast< QFrame* >( widget ) ) {
+			QFrame::Shape shape = frame->frameShape();
+			if (shape == QFrame::ToolBarPanel || shape == QFrame::MenuBarPanel)
+				widget->installEventFilter(this);
+		} 
 	}
 }
 
@@ -243,10 +252,11 @@ void KStyle::unPolish( QWidget* widget )
 {
 	if ( d->useFilledFrameWorkaround )
 	{
-		if (widget->inherits("QMenuBar"))
-			widget->removeEventFilter(this);
-		if (widget->inherits("QToolBar"))
-			widget->removeEventFilter(this);
+		if ( QFrame *frame = ::qt_cast< QFrame* >( widget ) ) {
+			QFrame::Shape shape = frame->frameShape();
+			if (shape == QFrame::ToolBarPanel || shape == QFrame::MenuBarPanel)
+				widget->removeEventFilter(this);
+		} 
 	}
 }
 
@@ -281,7 +291,7 @@ KStyle::KStyleFlags KStyle::styleFlags() const
 void KStyle::renderMenuBlendPixmap( KPixmap &pix, const QColorGroup &cg,
 	const QPopupMenu* /* popup */ ) const
 {
-	pix.fill(cg.button());	// Just tint as the default behaviour
+	pix.fill(cg.button());	// Just tint as the default behavior
 }
 
 
@@ -305,12 +315,13 @@ void KStyle::drawKStylePrimitive( KStylePrimitive kpe,
 			bool horizontal = flags & Style_Horizontal;
 			int x,y,w,h,x2,y2;
 
-			if ((wid->width() <= 2) || (wid->height() <= 2)) {
+			r.rect( &x, &y, &w, &h );
+			if ((w <= 2) || (h <= 2)) {
 				p->fillRect(r, cg.highlight());
 				return;
 			}
 
-			r.rect( &x, &y, &w, &h );
+			
 			x2 = x + w - 1;
 			y2 = y + h - 1;
 
@@ -391,8 +402,8 @@ void KStyle::drawKStylePrimitive( KStylePrimitive kpe,
 			{
 				// make 128*1 and 1*128 bitmaps that can be used for
 				// drawing the right sort of lines.
-				verticalLine   = new QBitmap( 1, 129, TRUE );
-				horizontalLine = new QBitmap( 128, 1, TRUE );
+				verticalLine   = new QBitmap( 1, 129, true );
+				horizontalLine = new QBitmap( 128, 1, true );
 				QPointArray a( 64 );
 				QPainter p2;
 				p2.begin( verticalLine );
@@ -872,6 +883,20 @@ int KStyle::pixelMetric(PixelMetric m, const QWidget* widget) const
 		case PM_ButtonShiftVertical:		// ### Make configurable
 			return 1;
 
+		case PM_DockWindowHandleExtent:
+		{
+			QWidget* parent = 0;
+			// Check that we are not a normal toolbar or a hidden dockwidget,
+			// in which case we need to adjust the height for font size
+			if (widget && (parent = widget->parentWidget() )
+				&& !parent->inherits("QToolBar")
+				&& !parent->inherits("QMainWindow")
+				&& widget->inherits("QDockWindowHandle") )
+					return widget->fontMetrics().lineSpacing();
+			else
+				return QCommonStyle::pixelMetric(m, widget);
+		}
+
 		// TABS
 		// ------------------------------------------------------------------------
 		case PM_TabBarTabHSpace:
@@ -946,11 +971,31 @@ int KStyle::pixelMetric(PixelMetric m, const QWidget* widget) const
 		case PM_MaximumDragDistance:
 			return -1;
 
+#if QT_VERSION >= 0x030300
+		case PM_MenuBarItemSpacing:
+			return 3;
+
+		case PM_ToolBarItemSpacing:
+			return 0;
+#endif
+
 		default:
 			return QCommonStyle::pixelMetric( m, widget );
 	}
 }
 
+//Helper to find the next sibling that's not hidden
+static QListViewItem* nextVisibleSibling(QListViewItem* item)
+{
+    QListViewItem* sibling = item;
+    do
+    {
+        sibling = sibling->nextSibling();
+    }
+    while (sibling && !sibling->isVisible());
+    
+    return sibling;
+}
 
 void KStyle::drawComplexControl( ComplexControl control,
 								 QPainter* p,
@@ -1137,7 +1182,7 @@ void KStyle::drawComplexControl( ComplexControl control,
 					while ( child && y + child->height() <= 0 )
 					{
 						y += child->totalHeight();
-						child = child->nextSibling();
+						child = nextVisibleSibling(child);
 					}
 
 					int bx = r.width() / 2;
@@ -1179,10 +1224,10 @@ void KStyle::drawComplexControl( ComplexControl control,
 						}
 
 						y += child->totalHeight();
-						child = child->nextSibling();
+						child = nextVisibleSibling(child);
 					}
 
-					if ( child ) // there's a child, so move linebot to edge of rectangle
+					if ( child ) // there's a child to draw, so move linebot to edge of rectangle
 						linebot = r.height();
 
 					if ( linetop < linebot )
@@ -1511,7 +1556,7 @@ static const char * const kstyle_unshade_xpm[] = {
 "............",
 "............"};
 
-static const char * dock_window_close_xpm[] = {
+static const char * const dock_window_close_xpm[] = {
 "8 8 2 1",
 "# c #000000",
 ". c None",
@@ -1696,17 +1741,23 @@ int KStyle::styleHint( StyleHint sh, const QWidget* w,
 			return d->menuAltKeyNavigation ? 1 : 0;
 
 		case SH_PopupMenu_SubMenuPopupDelay:
-			return d->popupMenuDelay;
+			if ( styleHint( SH_PopupMenu_SloppySubMenus, w ) )
+				return QMIN( 100, d->popupMenuDelay );
+			else
+				return d->popupMenuDelay;
+
+		case SH_PopupMenu_SloppySubMenus:
+			return d->sloppySubMenus;
 
 		case SH_ItemView_ChangeHighlightOnFocus:
 		case SH_Slider_SloppyKeyEvents:
 		case SH_MainWindow_SpaceBelowMenuBar:
+		case SH_PopupMenu_AllowActiveAndDisabled:
 			return 0;
 
 		case SH_Slider_SnapToValue:
 		case SH_PrintDialog_RightAlignButtons:
 		case SH_FontDialog_SelectAssociatedText:
-		case SH_PopupMenu_AllowActiveAndDisabled:
 		case SH_MenuBar_MouseTracking:
 		case SH_PopupMenu_MouseTracking:
 		case SH_ComboBox_ListMouseTracking:
@@ -1730,43 +1781,39 @@ bool KStyle::eventFilter( QObject* object, QEvent* event )
 		// This is nasty, but I see no other way to properly repaint
 		// filled frames in all QMenuBars and QToolBars.
 		// -- Karol.
-		if (event->type() == QEvent::Paint)
+		QFrame *frame = 0;
+		if ( event->type() == QEvent::Paint
+				&& (frame = ::qt_cast<QFrame*>(object)) )
 		{
-			QMenuBar* menubar = 0;
-			QToolBar* toolbar = 0;
-			if (object->inherits("QMenuBar"))
-				menubar = static_cast<QMenuBar*>(object);
-			else if (object->inherits("QToolBar"))
-				toolbar = static_cast<QToolBar*>(object);
-			if ( menubar || toolbar )
-			{
-				bool horizontal = true;
-				QPaintEvent* pe = (QPaintEvent*)event;
-				QFrame* frame   = (QFrame*)object;
-				QRect r = pe->rect();
+			if (frame->frameShape() != QFrame::ToolBarPanel && frame->frameShape() != QFrame::MenuBarPanel)
+				return false;
+				
+			bool horizontal = true;
+			QPaintEvent* pe = (QPaintEvent*)event;
+			QToolBar *toolbar = ::qt_cast< QToolBar *>( frame );
+			QRect r = pe->rect();
 
-				if (toolbar && toolbar->orientation() == Qt::Vertical)
-					horizontal = false;
+			if (toolbar && toolbar->orientation() == Qt::Vertical)
+				horizontal = false;
 
-				if (horizontal) {
-					if ( r.height() == frame->height() )
-						return false;	// Let QFrame handle the painting now.
+			if (horizontal) {
+				if ( r.height() == frame->height() )
+					return false;	// Let QFrame handle the painting now.
 
-					// Else, send a new paint event with an updated paint rect.
-					QPaintEvent dummyPE( QRect( r.x(), 0, r.width(), frame->height()) );
-					QApplication::sendEvent( frame, &dummyPE );
-				}
-				else {	// Vertical
-					if ( r.width() == frame->width() )
-						return false;
-
-					QPaintEvent dummyPE( QRect( 0, r.y(), frame->width(), r.height()) );
-					QApplication::sendEvent( frame, &dummyPE );
-				}
-
-				// Discard this event as we sent a new paintEvent.
-				return true;
+				// Else, send a new paint event with an updated paint rect.
+				QPaintEvent dummyPE( QRect( r.x(), 0, r.width(), frame->height()) );
+				QApplication::sendEvent( frame, &dummyPE );
 			}
+			else {	// Vertical
+				if ( r.width() == frame->width() )
+					return false;
+
+				QPaintEvent dummyPE( QRect( 0, r.y(), frame->width(), r.height()) );
+				QApplication::sendEvent( frame, &dummyPE );
+			}
+
+			// Discard this event as we sent a new paintEvent.
+			return true;
 		}
 	}
 
@@ -1874,6 +1921,7 @@ void TransparencyHandler::bottomShadow(QImage& dst)
 // Create a shadow of thickness 4.
 void TransparencyHandler::createShadowWindows(const QPopupMenu* p)
 {
+#if defined Q_WS_X11 && ! defined K_WS_QTONLY
 	int x2 = p->x()+p->width();
 	int y2 = p->y()+p->height();
 	QRect shadow1(x2, p->y() + 4, 4, p->height());
@@ -1913,10 +1961,14 @@ void TransparencyHandler::createShadowWindows(const QPopupMenu* p)
 	// Don't use QWidget::show() so we don't confuse QEffects, thus causing broken focus.
 	XMapWindow(qt_xdisplay(), se.w1->winId());
 	XMapWindow(qt_xdisplay(), se.w2->winId());
+#else
+	Q_UNUSED( p )
+#endif
 }
 
 void TransparencyHandler::removeShadowWindows(const QPopupMenu* p)
 {
+#if defined Q_WS_X11 && ! defined K_WS_QTONLY
 	ShadowMap::iterator it = shadowMap().find(p);
 	if (it != shadowMap().end())
 	{
@@ -1928,10 +1980,14 @@ void TransparencyHandler::removeShadowWindows(const QPopupMenu* p)
 		delete se.w2;
 		shadowMap().erase(it);
 	}
+#else
+	Q_UNUSED( p )
+#endif
 }
 
 bool TransparencyHandler::eventFilter( QObject* object, QEvent* event )
 {
+#ifndef Q_WS_MAC
 	// Transparency idea was borrowed from KDE2's "MegaGradient" Style,
 	// Copyright (C) 2000 Daniel M. Duley <mosfet@kde.org>
 
@@ -1989,6 +2045,7 @@ bool TransparencyHandler::eventFilter( QObject* object, QEvent* event )
 			p->setErasePixmap(QPixmap());
 	}
 
+#endif
 	return false;
 }
 
