@@ -1,6 +1,6 @@
 /*
  *  This file is part of the KDE libraries
- *  Copyright (c) 2001 Michael Goffioul <goffioul@imec.be>
+ *  Copyright (c) 2001 Michael Goffioul <kdeprint@swing.be>
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Library General Public
@@ -34,7 +34,7 @@ CupsdServerMiscPage::CupsdServerMiscPage(QWidget *parent, const char *name)
 {
 	path_.append(i18n("Server"));
 	path_.append(i18n("Misc"));
-	header_ = i18n("Server miscellaneous configuration");
+	header_ = i18n("Server Miscellaneous Configuration");
 
 	for (int i=0;i<2;i++)
 		opt_[i] = new CupsdOption(this);
