@@ -1,6 +1,6 @@
 /*
  *  This file is part of the KDE libraries
- *  Copyright (c) 2001 Michael Goffioul <goffioul@imec.be>
+ *  Copyright (c) 2001 Michael Goffioul <kdeprint@swing.be>
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Library General Public
@@ -35,7 +35,7 @@ CupsdBrowsingPage::CupsdBrowsingPage(QWidget *parent, const char *name)
 	: CupsdPage(parent, name)
 {
 	setPageLabel(i18n("Browsing"));
-	setHeader(i18n("Browsing settings"));
+	setHeader(i18n("Browsing Settings"));
 	setPixmap("kdeprint_printer_remote");
 
 	browseinterval_ = new KIntNumInput(this);
