@@ -1,6 +1,6 @@
 /*
  *  This file is part of the KDE libraries
- *  Copyright (c) 2001 Michael Goffioul <goffioul@imec.be>
+ *  Copyright (c) 2001 Michael Goffioul <kdeprint@swing.be>
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Library General Public
@@ -34,7 +34,7 @@ CupsdFilterPage::CupsdFilterPage(QWidget *parent, const char *name)
 	: CupsdPage(parent, name)
 {
 	setPageLabel(i18n("Filter"));
-	setHeader(i18n("Filter settings"));
+	setHeader(i18n("Filter Settings"));
 	setPixmap("filter");
 
 	user_ = new QLineEdit(this);
