@@ -1,8 +1,6 @@
 /*
  *  This file is part of the KDE libraries
- *  Copyright (c) 2001 Michael Goffioul <goffioul@imec.be>
- *
- *  $Id: cupsinfos.cpp,v 1.11 2002/10/08 16:17:15 goffioul Exp $
+ *  Copyright (c) 2001 Michael Goffioul <kdeprint@swing.be>
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Library General Public
@@ -25,8 +23,13 @@
 #include "messagewindow.h"
 
 #include <kio/passdlg.h>
+#include <kio/authinfo.h>
 #include <klocale.h>
 #include <kconfig.h>
+#include <kapplication.h>
+#include <dcopclient.h>
+#include <kdebug.h>
+#include <kstringhandler.h>
 
 #include <cups/cups.h>
 #include <cups/ipp.h>
@@ -89,29 +92,20 @@ void CupsInfos::setPassword(const QString& s)
 	password_ = s;
 }
 
+void CupsInfos::setSavePassword( bool on )
+{
+	savepwd_ = on;
+}
+
 const char* CupsInfos::getPasswordCB()
 {
-	if (count_ == 0 && !password_.isEmpty()) return password_.latin1();
-	else
-	{
-		// to avoid focus/modality problem, message windows must be removed
-		MessageWindow::removeAll();
+	QPair<QString,QString> pwd = KMFactory::self()->requestPassword( count_, login_, host_, port_ );
 
-		QString	msg = i18n("<p>The access to the requested resource on the CUPS server running on <b>%1</b> (port <b>%2</b>) requires a password.</p>").arg(host_).arg(port_);
-		bool ok(false);;
-		KIO::PasswordDialog	dlg(msg,login_);
-		count_++;
-		KMTimer::self()->hold();
-		if (dlg.exec())
-		{
-			setLogin(dlg.username());
-			setPassword(dlg.password());
-			ok = true;
-		}
-		KMTimer::self()->release();
-		return (ok ? password_.latin1() : NULL);
-	}
-	return NULL;
+	if ( pwd.first.isEmpty() && pwd.second.isEmpty() )
+		return NULL;
+	setLogin( pwd.first );
+	setPassword( pwd.second );
+	return pwd.second.latin1();
 }
 
 void CupsInfos::load()
@@ -121,7 +115,14 @@ void CupsInfos::load()
 	host_ = conf_->readEntry("Host",QString::fromLatin1(cupsServer()));
 	port_ = conf_->readNumEntry("Port",ippPort());
 	login_ = conf_->readEntry("Login",QString::fromLatin1(cupsUser()));
-	password_ = QString::null;
+	savepwd_ = conf_->readBoolEntry( "SavePassword", false );
+	if ( savepwd_ )
+	{
+		password_ = KStringHandler::obscure( conf_->readEntry( "Password" ) );
+		KMFactory::self()->initPassword( login_, password_, host_, port_ );
+	}
+	else
+		password_ = QString::null;
 	if (login_.isEmpty()) login_ = QString::null;
 	reallogin_ = cupsUser();
 
@@ -138,7 +139,11 @@ void CupsInfos::save()
 	conf_->writeEntry("Host",host_);
 	conf_->writeEntry("Port",port_);
 	conf_->writeEntry("Login",login_);
-	// don't write password for obvious security...
+	conf_->writeEntry( "SavePassword", savepwd_ );
+	if ( savepwd_ )
+		conf_->writeEntry( "Password", KStringHandler::obscure( password_ ) );
+	else
+		conf_->deleteEntry( "Password" );
 	conf_->sync();
 }
 
