@@ -1,6 +1,6 @@
 /*
  *  This file is part of the KDE libraries
- *  Copyright (c) 2001 Michael Goffioul <goffioul@imec.be>
+ *  Copyright (c) 2001 Michael Goffioul <kdeprint@swing.be>
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Library General Public
@@ -22,18 +22,20 @@
 
 #include <klocale.h>
 #include <kconfig.h>
+#include <kdialog.h>
+
 #include <qlayout.h>
 
 KMConfigCups::KMConfigCups(QWidget *parent)
 : KMConfigPage(parent,"ConfigCups")
 {
-	setPageName(i18n("CUPS server"));
+	setPageName(i18n("CUPS Server"));
 	setPageHeader(i18n("CUPS Server Settings"));
 	setPagePixmap("gear");
 
 	m_widget = new KMCupsConfigWidget(this);
 
-	QVBoxLayout	*lay0 = new QVBoxLayout(this, 5, 10);
+	QVBoxLayout *lay0 = new QVBoxLayout(this, 0, KDialog::spacingHint());
 	lay0->addWidget(m_widget);
 	lay0->addStretch(1);
 }
