@@ -1,8 +1,6 @@
 /*
  *  This file is part of the KDE libraries
- *  Copyright (c) 2001 Michael Goffioul <goffioul@imec.be>
- *
- *  $Id: kmcupsjobmanager.h,v 1.11 2002/05/23 16:55:14 goffioul Exp $
+ *  Copyright (c) 2001 Michael Goffioul <kdeprint@swing.be>
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Library General Public
@@ -33,7 +31,7 @@ class KMCupsJobManager : public KMJobManager
 	Q_OBJECT
 
 public:
-	KMCupsJobManager(QObject *parent = 0, const char *name = 0);
+	KMCupsJobManager(QObject *parent, const char *name, const QStringList &/*args*/);
 	virtual ~KMCupsJobManager();
 
 	int actions();
