@@ -1,8 +1,6 @@
 /*
  *  This file is part of the KDE libraries
- *  Copyright (c) 2001 Michael Goffioul <goffioul@imec.be>
- *
- *  $Id: kmcupsmanager.cpp,v 1.44.2.9 2003/06/29 21:43:00 deller Exp $
+ *  Copyright (c) 2001 Michael Goffioul <kdeprint@swing.be>
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Library General Public
@@ -26,13 +24,14 @@
 #include "ipprequest.h"
 #include "cupsinfos.h"
 #include "driver.h"
-#include "matic.h"
 #include "kmfactory.h"
 #include "kmdbentry.h"
 #include "cupsaddsmb2.h"
 #include "ippreportdlg.h"
 #include "kpipeprocess.h"
 #include "util.h"
+#include "foomatic2loader.h"
+#include "ppdloader.h"
 
 #include <qfile.h>
 #include <qtextstream.h>
@@ -52,6 +51,7 @@
 #include <kdialogbase.h>
 #include <kextendedsocket.h>
 #include <kprocess.h>
+#include <kfilterdev.h>
 #include <cups/cups.h>
 #include <cups/ppd.h>
 #include <math.h>
@@ -66,7 +66,7 @@ static int trials = 5;
 
 //*****************************************************************************************************
 
-KMCupsManager::KMCupsManager(QObject *parent, const char *name)
+	KMCupsManager::KMCupsManager(QObject *parent, const char *name, const QStringList & /*args*/)
 : KMManager(parent,name)
 {
 	// be sure to create the CupsInfos object -> password
@@ -148,12 +148,12 @@ bool KMCupsManager::createPrinter(KMPrinter *p)
 		KMPrinter	*otherP = findPrinter(p->printerName());
 		if (!otherP || otherP->device() != p->device())
 		{
-			// do not encode special chars in login/passwd in SMB devices
-			// to be compatible with CUPS/smbspool
-			if (p->device().protocol() == "smb")
-				req.addURI(IPP_TAG_PRINTER, "device-uri", urlToSmb(p->device()));
-			else
-				req.addURI(IPP_TAG_PRINTER,"device-uri",p->device().url());
+			/**
+			 * As now the device is a QString instead of KURL, special encoding
+			 * required for SMB is not needed anymore. Use a unique mechanism
+			 * for all backends.
+			 */
+			req.addURI(IPP_TAG_PRINTER,"device-uri",p->device());
 		}
 		if (!p->option("kde-banners").isEmpty())
 		{
@@ -269,40 +269,44 @@ bool KMCupsManager::completePrinterShort(KMPrinter *p)
 	// change host and port for remote stuffs
 	if (!p->uri().isEmpty())
 	{
-		m_hostSuccess = false;
-		m_lookupDone = false;
-		// Give 3 seconds to connect to the printer, or abort
-		KExtendedSocket *kes = new KExtendedSocket(p->uri().host(),
-							p->uri().port());
-		connect(kes, SIGNAL(connectionSuccess()), this, SLOT(hostPingSlot()));
-		connect(kes, SIGNAL(connectionFailed(int)), this, SLOT(hostPingFailedSlot()));
-		if (kes->startAsyncConnect() != 0) {
-			delete kes;
-			m_hostSuccess = false;
-		} else {
-			QDateTime tm = QDateTime::currentDateTime().addSecs(2);
-			while (!m_lookupDone && (QDateTime::currentDateTime() < tm))
-				qApp->processEvents();
+	// THIS IS AN UGLY HACK!! FIXME
+	// This attempts a "pre-connection" to see if the host is
+	// actually reachable.  It times out after 2 seconds at most,
+	// preventing application freezes.
+	m_hostSuccess = false;
+	m_lookupDone = false;
+	// Give 2 seconds to connect to the printer, or abort
+	KExtendedSocket *kes = new KExtendedSocket(p->uri().host(),
+	p->uri().port());
+	connect(kes, SIGNAL(connectionSuccess()), this, SLOT(hostPingSlot()));
+	connect(kes, SIGNAL(connectionFailed(int)), this, SLOT(hostPingFailedSlot()));
+	if (kes->startAsyncConnect() != 0) {
+	delete kes;
+	m_hostSuccess = false;
+	} else {
+	QDateTime tm = QDateTime::currentDateTime().addSecs(2);
+	while (!m_lookupDone && (QDateTime::currentDateTime() < tm))
+	qApp->processEvents();
 
-			kes->cancelAsyncConnect();
+	kes->cancelAsyncConnect();
 
-			delete kes;
+	delete kes;
 
-			if (!m_lookupDone)
-				m_hostSuccess = false;
-		}
-
-		if (m_hostSuccess == true) {
-			req.setHost(p->uri().host());
-			req.setPort(p->uri().port());
-		}
+	if (!m_lookupDone)
+	m_hostSuccess = false;
 	}
+
+	if (m_hostSuccess == true) {
+	req.setHost(p->uri().host());
+	req.setPort(p->uri().port());
+	}
+	}
+	*/
+
 	// disable location as it has been transferred to listing (for filtering)
 	//keys.append("printer-location");
 	keys.append("printer-info");
 	keys.append("printer-make-and-model");
-	*/
-
 	keys.append("job-sheets-default");
 	keys.append("job-sheets-supported");
 	keys.append("job-quota-period");
@@ -328,24 +332,24 @@ bool KMCupsManager::completePrinterShort(KMPrinter *p)
 		if (req.text("printer-make-and-model",value)) p->setDriverInfo(value);
 		if (req.uri("device-uri",value))
 		{
-			// in case of SMB, encode special chars in login/passwd
-			if (value.startsWith("smb://"))
-				p->setDevice(smbToUrl(value));
-			else
-				p->setDevice(KURL(value));
+			/**
+			 * No specific treatment required as the device is
+			 * a normal QString instead of a KURL
+			 */
+			p->setDevice( value );
 		}
 		QStringList	values;
-/*		if (req.uri("member-uris",values))
-		{
-			QStringList	members;
-			for (QStringList::ConstIterator it=values.begin(); it!=values.end(); ++it)
-			{
+		/*		if (req.uri("member-uris",values))
+				{
+				QStringList	members;
+				for (QStringList::ConstIterator it=values.begin(); it!=values.end(); ++it)
+				{
 				int	p = (*it).findRev('/');
 				if (p != -1)
-					members.append((*it).right((*it).length()-p-1));
-			}
-			p->setMembers(members);
-		}*/
+				members.append((*it).right((*it).length()-p-1));
+				}
+				p->setMembers(members);
+				}*/
 		if (req.name("member-names",values))
 			p->setMembers(values);
 		// banners
@@ -383,27 +387,27 @@ bool KMCupsManager::testPrinter(KMPrinter *p)
 {
 	return KMManager::testPrinter(p);
 	/*
-	QString	testpage = testPage();
-	if (testpage.isEmpty())
-	{
-		setErrorMsg(i18n("Unable to locate test page."));
-		return false;
-	}
+	   QString	testpage = testPage();
+	   if (testpage.isEmpty())
+	   {
+	   setErrorMsg(i18n("Unable to locate test page."));
+	   return false;
+	   }
 
-	IppRequest	req;
-	QString		uri;
+	   IppRequest	req;
+	   QString		uri;
 
-	req.setOperation(IPP_PRINT_JOB);
-	uri = printerURI(p);
-	req.addURI(IPP_TAG_OPERATION,"printer-uri",uri);
-	req.addMime(IPP_TAG_OPERATION,"document-format","application/postscript");
-	if (!CupsInfos::self()->login().isEmpty()) req.addName(IPP_TAG_OPERATION,"requesting-user-name",CupsInfos::self()->login());
-	req.addName(IPP_TAG_OPERATION,"job-name",QString::fromLatin1("KDE Print Test"));
-	if (req.doFileRequest("/printers/",testpage))
-		return true;
-	reportIppError(&req);
-	return false;
-	*/
+	   req.setOperation(IPP_PRINT_JOB);
+	   uri = printerURI(p);
+	   req.addURI(IPP_TAG_OPERATION,"printer-uri",uri);
+	   req.addMime(IPP_TAG_OPERATION,"document-format","application/postscript");
+	   if (!CupsInfos::self()->login().isEmpty()) req.addName(IPP_TAG_OPERATION,"requesting-user-name",CupsInfos::self()->login());
+	   req.addName(IPP_TAG_OPERATION,"job-name",QString::fromLatin1("KDE Print Test"));
+	   if (req.doFileRequest("/printers/",testpage))
+	   return true;
+	   reportIppError(&req);
+	   return false;
+	   */
 }
 
 void KMCupsManager::listPrinters()
@@ -485,7 +489,7 @@ void KMCupsManager::processRequest(IppRequest* req)
 			if ((value & CUPS_PRINTER_IMPLICIT)) printer->addType(KMPrinter::Implicit);
 
 			// convert printer-type attribute
-			//printer->setPrinterCap( ( value & CUPS_PRINTER_OPTIONS ) >> 2 );
+			printer->setPrinterCap( ( value & CUPS_PRINTER_OPTIONS ) >> 2 );
 		}
 		else if (attrname == "printer-state")
 		{
@@ -554,7 +558,7 @@ DrMain* KMCupsManager::loadMaticDriver(const QString& drname)
 	if (exe.isEmpty())
 	{
 		setErrorMsg(i18n("Unable to find the executable foomatic-datafile "
-		                 "in your PATH. Check that Foomatic is correctly installed."));
+					"in your PATH. Check that Foomatic is correctly installed."));
 		return NULL;
 	}
 
@@ -586,8 +590,8 @@ DrMain* KMCupsManager::loadMaticDriver(const QString& drname)
 		}
 	}
 	setErrorMsg(i18n("Unable to create the Foomatic driver [%1,%2]. "
-	                 "Either that driver does not exist, or you don't have "
-	                 "the required permissions to perform that operation.").arg(comps[1]).arg(comps[2]));
+				"Either that driver does not exist, or you don't have "
+				"the required permissions to perform that operation.").arg(comps[1]).arg(comps[2]));
 	QFile::remove(tmpFile);
 	return NULL;
 }
@@ -596,167 +600,25 @@ DrMain* KMCupsManager::loadDriverFile(const QString& fname)
 {
 	if (QFile::exists(fname))
 	{
-		QString	unzipfname;
-		if (!uncompressFile(fname,unzipfname))
-			return NULL;
-
-		DrMain	*driver = new DrMain();
-		if (unzipfname.isEmpty()) unzipfname = fname;
-		else driver->set("temporary",unzipfname);
-
-		// at this point we can only work with unzipfname
-		ppd_file_t	*ppd = ppdOpenFile(unzipfname.local8Bit());
-		if (ppd)
+		DrMain *driver = PPDLoader::loadDriver( fname );
+		if ( driver )
 		{
-			ppdMarkDefaults(ppd);
-
-			driver->set("text",QString::fromLocal8Bit(ppd->nickname));
-			driver->set("manufacturer",QString::fromLocal8Bit(ppd->manufacturer));
-			driver->set("model",QString::fromLocal8Bit(ppd->shortnickname));
-			driver->set("description",QString::fromLocal8Bit(ppd->nickname));
-			driver->set("template",unzipfname);
-			driver->set("colordevice", QString::number(ppd->color_device));
-			for (int i=0;i<ppd->num_groups;i++)
-			{
-				ppd_group_t	*grp = ppd->groups+i;
-				DrGroup	*gr = new DrGroup();
-				gr->set("text",ppdi18n(grp->text));
-				bool 	fixed = QString::fromLocal8Bit(grp->text).contains("install",false);
-				driver->addGroup(gr);
-				for (int k=0;k<grp->num_options;k++)
-				{
-					ppd_option_t	*opt = grp->options+k;
-					if (strcmp(opt->keyword,"PageRegion") == 0) continue;
-					DrListOption	*op;
-					switch (opt->ui)
-					{
-					   case PPD_UI_BOOLEAN:
-						op = new DrBooleanOption();
-						break;
-					   case PPD_UI_PICKONE:
-						op = new DrListOption();
-						break;
-					   default:
-						continue;	// skip option
-					}
-					op->setName(QString::fromLocal8Bit(opt->keyword));
-					op->set("text",ppdi18n(opt->text));
-					op->set("default",QString::fromLocal8Bit(opt->defchoice));
-					if (fixed) op->set("fixed","1");
-					gr->addOption(op);
-					for (int n=0;n<opt->num_choices;n++)
-					{
-						ppd_choice_t	*cho = opt->choices+n;
-						DrBase	*ch = new DrBase();
-						ch->setName(QString::fromLocal8Bit(cho->choice));
-						ch->set("text",ppdi18n(cho->text));
-						op->addChoice(ch);
-					}
-					op->setValueText(QString::fromLocal8Bit(opt->defchoice));
-				}
-			}
-
-			// add constraints
-			for (int i=0; i<ppd->num_consts; i++)
-			{
-				ppd_const_t	*cst = ppd->consts+i;
-				driver->addConstraint(new DrConstraint(QString::fromLatin1(cst->option1),QString::fromLatin1(cst->option2),QString::fromLatin1(cst->choice1),QString::fromLatin1(cst->choice2)));
-			}
-
-			// add page sizes
-			for (int i=0; i<ppd->num_sizes; i++)
-			{
-				ppd_size_t	*sz = ppd->sizes+i;
-				//kdDebug( 500 ) << "PageSize " << sz->name << ", " << sz->width << ", " << sz->length << ", " << sz->left << ", "
-				//	<< sz->bottom << ", " << sz->right << ", " << sz->top << endl;
-				driver->addPageSize(new DrPageSize(QString::fromLatin1(sz->name),(int)sz->width,(int)sz->length,(int)ceil( sz->left ),(int)ceil( sz->bottom ),
-							(int)ceil( sz->width - sz->right ),(int)ceil( sz->length - sz->top )));
-			}
-
-			ppdClose(ppd);
-
-			// try to extract Matic data
-			QString	maticdata;
-			DrGroup	*adjgrp(0);
-			extractMaticData(maticdata,unzipfname);
-			if (!maticdata.isEmpty())
-			{
-				driver->set("matic","1");
-
-				MaticBlock	*blk = loadMaticData(maticdata.latin1()), *varblk(0), *argblk(0);
-				if (blk) varblk = blk->block("$VAR1");
-				if (varblk) argblk = varblk->block("args_byname");
-				for (int i=0;i<2;i++)
-				{
-					if (argblk)
-					{
-						QDictIterator<MaticBlock>	it(argblk->m_blocks);
-						for (;it.current();++it)
-						{
-							QString	type = it.current()->arg("type");
-							if (type != "float" && type != "int" && type != "string")
-								continue;	// skip it
-							if (!adjgrp)
-							{
-								adjgrp = new DrGroup();
-								adjgrp->set("text",i18n("Adjustments"));
-								driver->addGroup(adjgrp);
-							}
-							DrBase	*opt(0);
-							if (type == "float")
-								opt = new DrFloatOption();
-							else if (type == "int")
-								opt = new DrIntegerOption();
-							else
-								opt = new DrStringOption();
-							opt->setName(it.current()->arg("name"));
-							opt->set("text",i18n(it.current()->arg("comment").local8Bit()));
-							if (type == "float" || type == "int")
-							{
-								opt->set("minval",it.current()->arg("min"));
-								opt->set("maxval",it.current()->arg("max"));
-							}
-							opt->setValueText(it.current()->arg("default"));
-							opt->set("default",it.current()->arg("default"));
-							// remove any option (list-type) with the same name (compliant
-							// with the new PPD structure in Foomatic)
-							DrBase	*oldOpt(0);
-							if ((oldOpt=driver->findOption(opt->name())) && oldOpt->type() == DrBase::List)
-							{
-								DrListOption	*oldLOpt = static_cast<DrListOption*>(oldOpt);
-								QString	fixedvals;
-								QPtrListIterator<DrBase>	it(*(oldLOpt->choices()));
-								for (; it.current(); ++it)
-								{
-									fixedvals.append(it.current()->name());
-									if (!it.atLast())
-										fixedvals.append("|");
-								}
-								opt->set("fixedvals", fixedvals);
-							}
-							driver->removeOptionGlobally(opt->name());
-							// finally add the (new) numerical option
-							adjgrp->addOption(opt);
-						}
-					}
-					if (varblk) argblk = varblk->block("args");	// for new Foomatic structure
-				}
-				delete blk;
-			}
-
-			return driver;
+			driver->set( "template", fname );
+			// FIXME: should fix option in group "install"
 		}
-		else delete driver;
+		return driver;
 	}
 	return NULL;
 }
 
 void KMCupsManager::saveDriverFile(DrMain *driver, const QString& filename)
 {
-	QFile	in(driver->get("template")), out(filename);
-	if (in.exists() && in.open(IO_ReadOnly) && out.open(IO_WriteOnly))
+	kdDebug( 500 ) << "Saving PPD file with template=" << driver->get( "template" ) << endl;
+	QIODevice *in = KFilterDev::deviceForFile( driver->get( "template" ) );
+	QFile	out(filename);
+	if (in && in->open(IO_ReadOnly) && out.open(IO_WriteOnly))
 	{
-		QTextStream	tin(&in), tout(&out);
+		QTextStream	tin(in), tout(&out);
 		QString		line, keyword;
 		bool 		isnumeric(false);
 		DrBase		*opt(0);
@@ -842,6 +704,7 @@ void KMCupsManager::saveDriverFile(DrMain *driver, const QString& filename)
 				tout << line << endl;
 		}
 	}
+	delete in;
 }
 
 bool KMCupsManager::savePrinterDriver(KMPrinter *p, DrMain *d)
@@ -873,16 +736,16 @@ void* KMCupsManager::loadCupsdConfFunction(const char *name)
 {
 	if (!m_cupsdconf)
 	{
-		m_cupsdconf = KLibLoader::self()->library("libcupsdconf");
+		m_cupsdconf = KLibLoader::self()->library("cupsdconf");
 		if (!m_cupsdconf)
 		{
-			setErrorMsg(i18n("Library libcupsdconf not found. Check your installation."));
+			setErrorMsg(i18n("Library cupsdconf not found. Check your installation."));
 			return NULL;
 		}
 	}
 	void*	func = m_cupsdconf->symbol(name);
 	if (!func)
-		setErrorMsg(i18n("Symbol %1 not found in libcupsdconf library.").arg(name));
+		setErrorMsg(i18n("Symbol %1 not found in cupsdconf library.").arg(name));
 	return func;
 }
 
@@ -1043,7 +906,7 @@ void KMCupsManager::slotConnectionSuccess()
 {
 	kdDebug(500) << "Connection success, trying to send a request..." << endl;
 	m_socket->close();
-	
+
 	IppRequest req;
 	req.setOperation( CUPS_GET_PRINTERS );
 	req.addKeyword( IPP_TAG_OPERATION, "requested-attributes", QString::fromLatin1( "printer-name" ) );
