@@ -1,8 +1,6 @@
 /*
  *  This file is part of the KDE libraries
- *  Copyright (c) 2001 Michael Goffioul <goffioul@imec.be>
- *
- *  $Id: kmwipp.cpp,v 1.4 2002/07/08 21:06:46 binner Exp $
+ *  Copyright (c) 2001 Michael Goffioul <kdeprint@swing.be>
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Library General Public
@@ -26,6 +24,7 @@
 #include <qlabel.h>
 #include <qlineedit.h>
 #include <klocale.h>
+#include <qvalidator.h>
 
 #include <cups/http.h>
 
@@ -35,12 +34,14 @@ KMWIpp::KMWIpp(QWidget *parent, const char *name)
 	m_ID = KMWizard::IPP;
 	m_title = i18n("Remote IPP server");
 	m_nextpage = KMWizard::IPPSelect;
+	lineEdit( 1 )->setValidator( new QIntValidator( this ) );
 
 	setInfo(i18n("<p>Enter the information concerning the remote IPP server "
 		     "owning the targeted printer. This wizard will poll the server "
 		     "before continuing.</p>"));
 	setLabel(0,i18n("Host:"));
 	setLabel(1,i18n("Port:"));
+	setText( 1, QString::fromLatin1( "631" ) );
 }
 
 bool KMWIpp::isValid(QString& msg)
@@ -81,5 +82,5 @@ void KMWIpp::updatePrinter(KMPrinter *p)
 	url.setPort(text(1).toInt());
 	if (!p->option("kde-login").isEmpty()) url.setUser(p->option("kde-login"));
 	if (!p->option("kde-password").isEmpty()) url.setPass(p->option("kde-password"));
-	p->setDevice(url);
+	p->setDevice(url.url());
 }
