@@ -1,6 +1,6 @@
 /*
  *  This file is part of the KDE libraries
- *  Copyright (c) 2001 Michael Goffioul <goffioul@imec.be>
+ *  Copyright (c) 2001 Michael Goffioul <kdeprint@swing.be>
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Library General Public
@@ -102,7 +102,7 @@ KMWIppPrinter::~KMWIppPrinter()
 
 void KMWIppPrinter::updatePrinter(KMPrinter *p)
 {
-	p->setDevice(KURL(m_uri->text()));
+	p->setDevice(m_uri->text());
 }
 
 bool KMWIppPrinter::isValid(QString& msg)
