@@ -1,8 +1,6 @@
 /*
  *  This file is part of the KDE libraries
- *  Copyright (c) 2001 Michael Goffioul <goffioul@imec.be>
- *
- *  $Id: kmwippselect.cpp,v 1.8 2002/08/14 12:36:27 faure Exp $
+ *  Copyright (c) 2001 Michael Goffioul <kdeprint@swing.be>
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Library General Public
@@ -105,6 +103,6 @@ void KMWIppSelect::updatePrinter(KMPrinter *p)
 	QString	path = m_list->currentText();
 	path.prepend("/printers/");
 	url.setPath(path);
-	p->setDevice(url);
+	p->setDevice(url.url());
 kdDebug(500) << url.url() << endl;
 }
