@@ -1,6 +1,6 @@
 /*
  *  This file is part of the KDE libraries
- *  Copyright (c) 2001 Michael Goffioul <goffioul@imec.be>
+ *  Copyright (c) 2001 Michael Goffioul <kdeprint@swing.be>
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Library General Public
@@ -25,7 +25,7 @@
 class KExtPrinterImpl : public KPrinterImpl
 {
 public:
-	KExtPrinterImpl(QObject *parent = 0, const char *name = 0);
+	KExtPrinterImpl(QObject *parent, const char *name, const QStringList & /*args*/);
 	~KExtPrinterImpl();
 
 	void preparePrinting(KPrinter*);
