@@ -1,6 +1,6 @@
 /*
  *  This file is part of the KDE libraries
- *  Copyright (c) 2001 Michael Goffioul <goffioul@imec.be>
+ *  Copyright (c) 2001 Michael Goffioul <kdeprint@swing.be>
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Library General Public
@@ -18,10 +18,12 @@
  **/
 
 #include "kmextuimanager.h"
+#include "kpqtpage.h"
+#include "kprinterpropertydialog.h"
 
 #include <klocale.h>
 
-KMExtUiManager::KMExtUiManager(QObject *parent, const char *name)
+KMExtUiManager::KMExtUiManager(QObject *parent, const char *name, const QStringList & /*args*/)
 : KMUiManager(parent,name)
 {
 	m_printdialogflags |= KMUiManager::PrintCommand;
@@ -35,3 +37,8 @@ int KMExtUiManager::pluginPageCap()
 {
 	return KMUiManager::NoAutoCollate;
 }
+
+void KMExtUiManager::setupPrinterPropertyDialog( KPrinterPropertyDialog *dlg )
+{
+	dlg->addPage( new KPQtPage( dlg, "QtPage" ) );
+}
