@@ -1,6 +1,6 @@
 /*
  *  This file is part of the KDE libraries
- *  Copyright (c) 2001 Michael Goffioul <goffioul@imec.be>
+ *  Copyright (c) 2001 Michael Goffioul <kdeprint@swing.be>
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Library General Public
@@ -27,11 +27,16 @@
 #include <kdebug.h>
 #include <dcopclient.h>
 #include <kio/passdlg.h>
+#include <kio/authinfo.h>
 #include <qlabel.h>
 #include <kpushbutton.h>
 #include <kiconloader.h>
+#include <kstandarddirs.h>
 #include <kwin.h>
+#include <kapplication.h>
 #include <qlayout.h>
+#include <qtimer.h>
+#include <qregexp.h>
 
 #include <unistd.h>
 
@@ -43,12 +48,6 @@ extern "C"
 	}
 }
 
-static void cleanFileList(const QStringList& files)
-{
-	for (QStringList::ConstIterator it=files.begin(); it!=files.end(); ++it)
-		QFile::remove(*it);
-}
-
 class StatusWindow : public QWidget
 {
 public:
@@ -102,71 +101,62 @@ KDEPrintd::KDEPrintd(const QCString& obj)
 : KDEDModule(obj)
 {
 	m_processpool.setAutoDelete(true);
-	m_tempfiles.setAutoDelete(true);
 	m_windows.setAutoDelete(false);
+	m_requestsPending.setAutoDelete( true );
 }
 
 KDEPrintd::~KDEPrintd()
 {
-	cleanTempFiles();
 }
 
 int KDEPrintd::print(const QString& cmd, const QStringList& files, bool remflag)
 {
+	KPrintProcess *proc = new KPrintProcess;
 	QString	command(cmd);
-	if (!checkFiles(command, files))
-		return (-1);
+	QRegExp re( "\\$out\\{([^}]*)\\}" );
 
-	KPrintProcess	*proc = new KPrintProcess;
-	connect(proc,SIGNAL(processExited(KProcess*)),SLOT(slotProcessExited(KProcess*)));
-//	connect(proc,SIGNAL(passwordRequested(KProcess*,const QString&)),SLOT(slotPasswordRequested(KProcess*,const QString&)));
-	*proc << command;
-	if (remflag)
-		m_tempfiles.insert(proc,new QStringList(files));
-	if (proc->print())
+	connect(proc,SIGNAL(printTerminated(KPrintProcess*)),SLOT(slotPrintTerminated(KPrintProcess*)));
+	connect(proc,SIGNAL(printError(KPrintProcess*,const QString&)),SLOT(slotPrintError(KPrintProcess*,const QString&)));
+	proc->setCommand( command );
+	if ( re.search( command ) != -1 )
 	{
-		m_processpool.append(proc);
-		return (int)(proc->pid());
+		KURL url( re.cap( 1 ) );
+		if ( !url.isLocalFile() )
+		{
+			QString tmpFilename = locateLocal( "tmp", "kdeprint_" + kapp->randomString( 8 ) );
+			command.replace( re, KProcess::quote( tmpFilename ) );
+			proc->setOutput( re.cap( 1 ) );
+			proc->setTempOutput( tmpFilename );
+		}
+		else
+			command.replace( re, KProcess::quote( re.cap( 1 ) ) );
 	}
-	else
+
+	if ( checkFiles( command, files ) )
 	{
-		cleanTempFile(proc);
-		delete proc;
-		return (-1);
+		*proc << command;
+		if ( remflag )
+			proc->setTempFiles( files );
+		if ( proc->print() )
+		{
+			m_processpool.append( proc );
+			return ( int )proc->pid();
+		}
 	}
+
+	delete proc;
+	return -1;
 }
 
-void KDEPrintd::cleanTempFiles()
+void KDEPrintd::slotPrintTerminated( KPrintProcess *proc )
 {
-	QPtrDictIterator<QStringList>	it(m_tempfiles);
-	for (;it.current();++it)
-		cleanFileList(*(it.current()));
+	m_processpool.removeRef( proc );
 }
 
-void KDEPrintd::cleanTempFile(KProcess *p)
+void KDEPrintd::slotPrintError( KPrintProcess *proc, const QString& msg )
 {
-	QStringList	*l = m_tempfiles.find(p);
-	if (l)
-		cleanFileList(*l);
-}
-
-void KDEPrintd::slotProcessExited(KProcess *proc)
-{
-	KPrintProcess	*pproc = (KPrintProcess*)proc;
-	if (m_processpool.findRef(pproc) != -1)
-	{
-		m_processpool.take();
-		QString		msg;
-		if (!pproc->normalExit())
-			msg = i18n("Abnormal process termination (<b>%1</b>).").arg(pproc->args().first());
-		else if (pproc->exitStatus() != 0)
-			msg = i18n("<b>%1</b>: execution failed with message:<p>%2</p>").arg(pproc->args().first()).arg(pproc->errorMessage());
-		cleanTempFile(pproc);
-
-		delete pproc;
-		if (!msg.isEmpty())
-			KNotifyClient::event("printerror",i18n("<p><nobr>A print error occured. Error message received from system:</nobr></p><br>%1").arg(msg));
-	}
+	KNotifyClient::event("printerror",i18n("<p><nobr>A print error occurred. Error message received from system:</nobr></p><br>%1").arg(msg));
+	m_processpool.removeRef( proc );
 }
 
 QString KDEPrintd::openPassDlg(const QString& user)
@@ -232,4 +222,94 @@ void KDEPrintd::slotClosed()
 	}
 }
 
+//******************************************************************************************
+
+class KDEPrintd::Request
+{
+public:
+	DCOPClientTransaction *transaction;
+	QString user;
+	QString uri;
+	int seqNbr;
+};
+
+QString KDEPrintd::requestPassword( const QString& user, const QString& host, int port, int seqNbr )
+{
+	Request *req = new Request;
+	req->user = user;
+	req->uri = "print://" + user + "@" + host + ":" + QString::number(port);
+	req->seqNbr = seqNbr;
+	req->transaction = callingDcopClient()->beginTransaction();
+	m_requestsPending.append( req );
+	if ( m_requestsPending.count() == 1 )
+		QTimer::singleShot( 0, this, SLOT( processRequest() ) );
+	return "::";
+}
+
+void KDEPrintd::processRequest()
+{
+	if ( m_requestsPending.count() == 0 )
+		return;
+
+	Request *req = m_requestsPending.first();
+	KIO::AuthInfo info;
+	QByteArray params, reply;
+	QCString replyType;
+	QString authString( "::" );
+
+	info.username = req->user;
+	info.keepPassword = true;
+	info.url = req->uri;
+	info.comment = i18n( "Printing system" );
+
+	QDataStream input( params, IO_WriteOnly );
+	input << info << i18n( "Authentication failed (user name=%1)" ).arg( info.username ) << 0 << req->seqNbr;
+	if ( callingDcopClient()->call( "kded", "kpasswdserver", "queryAuthInfo(KIO::AuthInfo,QString,long int,long int)",
+				params, replyType, reply ) )
+	{
+		if ( replyType == "KIO::AuthInfo" )
+		{
+			QDataStream output( reply, IO_ReadOnly );
+			KIO::AuthInfo result;
+			int seqNbr;
+			output >> result >> seqNbr;
+
+			if ( result.isModified() )
+				authString = result.username + ":" + result.password + ":" + QString::number( seqNbr );
+		}
+		else
+			kdWarning( 500 ) << "DCOP returned type error, expected KIO::AuthInfo, received " << replyType << endl;
+	}
+	else
+		kdWarning( 500 ) << "Cannot communicate with kded_kpasswdserver" << endl;
+
+	QByteArray outputData;
+	QDataStream output( outputData, IO_WriteOnly );
+	output << authString;
+	replyType = "QString";
+	callingDcopClient()->endTransaction( req->transaction, replyType, outputData );
+
+	m_requestsPending.remove( ( unsigned int )0 );
+	if ( m_requestsPending.count() > 0 )
+		QTimer::singleShot( 0, this, SLOT( processRequest() ) );
+}
+
+void KDEPrintd::initPassword( const QString& user, const QString& passwd, const QString& host, int port )
+{
+	QByteArray params, reply;
+	QCString replyType;
+	KIO::AuthInfo info;
+
+	info.username = user;
+	info.password = passwd;
+	info.url = "print://" + user + "@" + host + ":" + QString::number(port);
+
+	QDataStream input( params, IO_WriteOnly );
+	input << info << ( long int )0;
+
+	if ( !callingDcopClient()->call( "kded", "kpasswdserver", "addAuthInfo(KIO::AuthInfo,long int)",
+			params, replyType, reply ) )
+		kdWarning( 500 ) << "Unable to initialize password, cannot communicate with kded_kpasswdserver" << endl;
+}
+
 #include "kdeprintd.moc"
