@@ -1,8 +1,6 @@
 /*
  *  This file is part of the KDE libraries
- *  Copyright (c) 2001 Michael Goffioul <goffioul@imec.be>
- *
- *  $Id: kmjob.cpp,v 1.8 2002/03/28 09:03:40 goffioul Exp $
+ *  Copyright (c) 2001 Michael Goffioul <kdeprint@swing.be>
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Library General Public
@@ -122,7 +120,7 @@ QString KMJob::stateString()
 			str = i18n("Error");
 			break;
 		case KMJob::Cancelled:
-			str = i18n("Cancelled");
+			str = i18n("Canceled");
 			break;
 		case KMJob::Aborted:
 			str = i18n("Aborted");
