@@ -1,8 +1,6 @@
 /*
  *  This file is part of the KDE libraries
- *  Copyright (c) 2001 Michael Goffioul <goffioul@imec.be>
- *
- *  $Id: kmuimanager.cpp,v 1.26 2002/09/17 16:15:49 goffioul Exp $
+ *  Copyright (c) 2001 Michael Goffioul <kdeprint@swing.be>
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Library General Public
@@ -148,12 +146,10 @@ void KMUiManager::setupPropertyDialog(KPrinterPropertyDialog *dlg)
 		if (dlg->parent() && dlg->parent()->isA("KPrintDialog"))
 			prt = static_cast<KPrintDialog*>(dlg->parent())->printer();
 
-#ifdef KDEPRINT_USE_MARGINS
 		// add margin page
 		if ( ( prt && !prt->fullPage() && prt->applicationType() == KPrinter::Dialog )
 					|| prt->applicationType() < 0 )
 			dlg->addPage(new KPMarginPage(prt, driver, dlg, "MarginPage"));
-#endif
 
 		// add driver page
 		if (driver)
@@ -201,3 +197,5 @@ void KMUiManager::setupPrintDialogPages(QPtrList<KPrintDialogPage>*)
 void KMUiManager::setupJobViewer(QListView*)
 {
 }
+
+#include "kmuimanager.moc"
