@@ -1,6 +1,6 @@
 /*
  *  This file is part of the KDE libraries
- *  Copyright (c) 2001 Michael Goffioul <goffioul@imec.be>
+ *  Copyright (c) 2001 Michael Goffioul <kdeprint@swing.be>
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Library General Public
@@ -30,7 +30,7 @@
 KPFileSelectPage::KPFileSelectPage(QWidget *parent, const char *name)
 : KPrintDialogPage(parent, name)
 {
-	setTitle(i18n("Files"));
+	setTitle(i18n("&Files"));
 	m_first = true;
 
 	m_files = new KFileList(this);
