@@ -1,8 +1,6 @@
 /*
  *  This file is part of the KDE libraries
- *  Copyright (c) 2001 Michael Goffioul <goffioul@imec.be>
- *
- *  $Id: kprintdialog.cpp,v 1.60.2.1 2003/03/13 23:42:40 waba Exp $
+ *  Copyright (c) 2001 Michael Goffioul <kdeprint@swing.be>
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Library General Public
@@ -19,6 +17,11 @@
  *  Boston, MA 02111-1307, USA.
  **/
 
+/*
+ *  "WhatsThis" help items added by Kurt Pfeifle, August 2003,
+ *  same copyright as above.
+ **/
+
 #include "kprintdialog.h"
 #include "kprinter.h"
 #include "kprinterimpl.h"
@@ -46,6 +49,7 @@
 #include <kmessagebox.h>
 #include <qdir.h>
 #include <qtooltip.h>
+#include <qwhatsthis.h>
 
 #include <klocale.h>
 #include <kiconloader.h>
@@ -68,7 +72,8 @@ class KPrintDialog::KPrintDialogPrivate
 {
 public:
 	QLabel	*m_type, *m_state, *m_comment, *m_location, *m_cmdlabel, *m_filelabel;
-	QPushButton	*m_properties, *m_default, *m_options, *m_ok, *m_wizard, *m_extbtn, *m_filter;
+	KPushButton	*m_properties, *m_default, *m_options, *m_ok, *m_extbtn;
+	QPushButton	*m_wizard, *m_filter;
 	QCheckBox	*m_preview;
 	QLineEdit	*m_cmd;
 	TreeComboBox	*m_printers;
@@ -85,6 +90,159 @@ public:
 KPrintDialog::KPrintDialog(QWidget *parent, const char *name)
 : KDialog(parent,name,true)
 {
+	//WhatsThis strings.... (added by pfeifle@kde.org)
+	QString whatsThisLocationLabel = i18n(  " <qt> The <em>Location</em> may describe where the"
+						" selected printer is located. The Location description is created"
+						" by the administrator of the print system (or may be"
+						" left empty)."
+						" </qt>" );
+	QString whatsThisPrinterType = i18n(    " <qt> The <em>Type</em> indicates your printer type."
+						" </qt>" );
+	QString whatsThisPrinterState = i18n(   " <qt> The <em>State</em> indicates the state of the"
+						" print queue on the print server (which could be your localhost). The"
+						" state may be 'Idle', 'Processing', 'Stopped', 'Paused' or similar."
+						" </qt>" );
+	QString whatsThisPrinterComment = i18n( " <qt> The <em>Comment</em> may describe the selected"
+						" printer. This comment is created by the administrator"
+						" of the print system (or may be left empty)."
+						" </qt>" );
+	QString whatsThisPrinterSelect = i18n(  " <qt>Use this combo box to select the printer to which you want to print."
+						" If you only find the <em>KDE special printers</em> -- which save"
+						" jobs to disk (as PostScript- or PDF-files), or deliver jobs via"
+						" email (as a PDF"
+						" attachment) -- but are missing a real printer, you need to..."
+						" <ul>"
+						" <li>...either create a local printer with the help of the <em>KDE Add"
+						" Printer Wizard</em> which is available for the CUPS and RLPR printing"
+						" systems (click button to the left of the 'Properties' button),</li>"
+						" <li>...or you can try to connect to a remote"
+						" CUPS print server by clicking the <em>System Options...</em> button"
+						" below. A new dialog opens: click on the <em>CUPS server</em>"
+						" icon and fill in the information required to use the remote"
+						" server.</li>"
+						" </ul>"
+						" </qt>" );
+	QString whatsThisPrintJobProperties = i18n( " <qt> This button takes you to where you can make decisions"
+						" regarding all supported print job options."
+						" </qt>" );
+	QString whatsThisPrinterFilter = i18n(  " <qt><p> This button reduces the list of visible printers"
+						" to a shorter, more convenient, pre-defined list.</p>"
+						" <p>This is particularly useful in enterprise environments"
+						" with lots of printers. The default is to show <b>all</b> printers.</p>"
+						" <p>To create a personal filter list, click on the"
+						" <em>\"System Options...</em>\" button at the bottom of this dialog."
+						" Then, in the new dialog, select <em>\"Filter\"</em> (left column in the"
+						" <em>KDE Print Configuration</em> dialog).</p>"
+						" </qt>" );
+	QString whatsThisAddPrinterWizard = i18n( "<qt> This button starts the <em>KDE Add Printer Wizard</em>."
+						" (<b>NOT</b> if you use <em>\"Generic LPD</em>\""
+						" or <em>\"LPRng\"</em>, or <em>\"Print Through an External Program</em>\".)"
+						" Use the Wizard (with <em>\"CUPS\"</em> or <em>\"RLPR\"</em>) to add locally"
+						" defined printers to your system."
+						" </qt>" );
+	QString whatsThisExternalPrintCommand = i18n( " <qt> You can enter any command that"
+						" would also print for you in a <em>konsole</em> window. <br>"
+						" <b>Example:</b> <pre>a2ps -P &lt;printername&gt; --medium=A3</pre>."
+						" </qt>" );
+	QString whatsThisExpandCollapse = i18n( " <qt>This button expands or reduces the KDE"
+						" printing dialog."
+						" KDEPrint will remember what you do here; next time, it will"
+						" start this dialog 'expanded' or 'collapsed', depending"
+						" on the mode you are using this time...."
+						" </qt>" );
+	QString whatsThisSystemOptions = i18n(  " <qt> This button starts a new dialog where you can adjust various"
+  						" settings of your printing system. Amongst them:"
+						" <ul><li> Should KDE"
+						" applications embed all fonts into the PostScript they"
+ 						" generate for printing?"
+						" <li> Should KDE use an external PostScript viewer"
+						" like <em>gv</em> for print page previews?"
+						" <li> Should KDEPrint use a local or a remote CUPS server?,"
+						" </ul> and many more.... "
+					        " </qt>" );
+
+	QString whatsThisHelpButton = i18n(     " <qt> This button takes you to the complete <em>KDEPrint"
+						" Manual</em>."
+					        " </qt>" );
+
+	QString whatsThisCancelButton = i18n(   " <qt> This button cancels your print job and quits the"
+						" kprinter dialog."
+					        " </qt>" );
+
+	QString whatsThisPrintButton = i18n(    " <qt> This button sends the job to the printing process."
+						" If you are sending non-PostScript files, you may be"
+						" asked if you want KDE to convert the files into PostScript,"
+						" or if you want your print subsystem (like CUPS) to do this."
+					        " </qt>" );
+
+	QString whatsThisKeepDialogOpenCheckbox = i18n( " <qt><p>If you enable this checkbox, the printing dialog"
+						" stays open after you hit the <em>Print</em> button.</p>"
+						" <p> This is"
+						" especially useful, if you need to test various"
+						" print settings (like color matching for an inkjet printer)"
+						" or if you want to send your job to multiple printers (one after"
+						" the other) to have it finished more quickly.</p>"
+					        " </qt>" );
+
+	QString whatsThisOutputFileLabel = i18n(" <qt> The \"Output file:\" shows you where your file will be"
+						" saved if you decide to \"Print to File\" your job, using one of the"
+						" KDE <em>Special Printers</em> named \"Print to File (PostScript)\""
+						" or \"Print to File (PDF)\". Choose a name and location that suits"
+						" you by using the button and/or editing the line on the right."
+					        " </qt>" );
+
+	QString whatsThisOutputFileLineedit = i18n(" <qt> Edit this line to create a path and filename that suits"
+						" you. (Only available if you \"Print to File\")"
+					        " </qt>" );
+
+	QString whatsThisOutputFileButton = i18n(" <qt> This button calls the \"File Open\" dialog to let you"
+						" choose a directory and file name where your \"Print-to-File\""
+						" job should be saved."
+					        " </qt>" );
+
+	QString whatsThisAddFileButton = i18n(  " <qt> This button calls the \"File Open\" dialog to allow you"
+						" to select a file for printing. Note, that "
+						" <ul><li>you can select ASCII or International Text, PDF,"
+						" PostScript, JPEG, TIFF, PNG, GIF and many other graphical"
+						" formats."
+						" <li>you can select various files from different paths"
+						" and send them as one \"multi-file job\" to the printing"
+						" system."
+						" </ul>"
+					        " </qt>" );
+
+	QString whatsThisRemoveFileButton = i18n(" <qt> This button removes the highlighted file from the"
+						" list of files to be printed."
+					        " </qt>" );
+
+	QString whatsThisOpenFileButton = i18n( " <qt> This button tries to open the highlighted file, so"
+						" you can view or edit it before you send it to the printing"
+						" system."
+					        " </qt>" );
+
+	QString whatsThisMoveFileUpButton = i18n(" <qt> This button moves the highlighted file up in the list"
+						" of files to be printed. (In effect, this changes the order"
+						" of the files' printout.)"
+					        " </qt>" );
+
+	QString whatsThisMoveFileDownButton = i18n(" <qt> This button moves the highlighted file down in the list"
+						" of files to be printed. (In effect, this changes the order"
+						" of the files' printout.)"
+					        " </qt>" );
+
+	QString whatsThisPreviewCheckBox = i18n(" <qt> Enable this checkbox if you want to see a preview of"
+						" your printout. A preview lets you check if, for instance,"
+ 						" your intended \"poster\" or \"pamphlet\" layout"
+						" looks like you intended, without wasting paper first. It"
+						" also lets you cancel the job if something looks wrong. "
+					        " </qt>" );
+
+	QString whatsThisSetDefaultPrinter = i18n(" <qt> This button sets the current printer as the user's"
+						" default. (Button is only visible if the <em>System Options...</em>"
+						" --> <em>General</em> --> <em>Miscellaneous</em>: <em>\"Defaults"
+						" to the last printer used in the application\"</em> is disabled.)"
+					        " </qt>" );
+
 	d = new KPrintDialogPrivate;
 
 	d->m_pages.setAutoDelete(false);
@@ -94,48 +252,88 @@ KPrintDialog::KPrintDialog(QWidget *parent, const char *name)
 	// widget creation
 	QGroupBox	*m_pbox = new QGroupBox(0,Qt::Vertical,i18n("Printer"), this);
 	d->m_type = new QLabel(m_pbox);
+	QWhatsThis::add(d->m_type, whatsThisPrinterType);
 	d->m_state = new QLabel(m_pbox);
+	QWhatsThis::add(d->m_state, whatsThisPrinterState);
 	d->m_comment = new QLabel(m_pbox);
+	QWhatsThis::add(d->m_comment, whatsThisPrinterComment);
 	d->m_location = new QLabel(m_pbox);
+	QWhatsThis::add(d->m_location, whatsThisLocationLabel);
+
 	d->m_printers = new TreeComboBox(m_pbox);
+	QWhatsThis::add(d->m_printers, whatsThisPrinterSelect);
 	d->m_printers->setMinimumHeight(25);
 	QLabel	*m_printerlabel = new QLabel(i18n("&Name:"), m_pbox);
+	QWhatsThis::add(m_printerlabel, whatsThisPrinterSelect);
 	QLabel	*m_statelabel = new QLabel(i18n("Status", "State:"), m_pbox);
+	QWhatsThis::add(m_statelabel, whatsThisPrinterState);
 	QLabel	*m_typelabel = new QLabel(i18n("Type:"), m_pbox);
+	QWhatsThis::add(m_typelabel, whatsThisPrinterType);
 	QLabel	*m_locationlabel = new QLabel(i18n("Location:"), m_pbox);
+	QWhatsThis::add(m_locationlabel, whatsThisLocationLabel);
 	QLabel	*m_commentlabel = new QLabel(i18n("Comment:"), m_pbox);
+	QWhatsThis::add(m_commentlabel, whatsThisPrinterComment);
 	m_printerlabel->setBuddy(d->m_printers);
 	d->m_properties = new KPushButton(KGuiItem(i18n("P&roperties..."), "edit"), m_pbox);
+	QWhatsThis::add( d->m_properties, whatsThisPrintJobProperties);
 	d->m_options = new KPushButton(KGuiItem(i18n("System Op&tions..."), "kdeprint_configmgr"), this);
+	QWhatsThis::add(d->m_options,whatsThisSystemOptions);
 	d->m_default = new KPushButton(KGuiItem(i18n("Set as &Default"), "kdeprint_defaultsoft"), m_pbox);
+	QWhatsThis::add(d->m_default,whatsThisSetDefaultPrinter);
 	d->m_filter = new QPushButton(m_pbox);
 	d->m_filter->setPixmap(SmallIcon("filter"));
 	d->m_filter->setMinimumSize(QSize(d->m_printers->minimumHeight(),d->m_printers->minimumHeight()));
 	d->m_filter->setToggleButton(true);
 	d->m_filter->setOn(KMManager::self()->isFilterEnabled());
-	QToolTip::add(d->m_filter, i18n("Toggle Printer Filtering"));
+	QToolTip::add(d->m_filter, i18n("Toggle printer filtering"));
+	QWhatsThis::add(d->m_filter, whatsThisPrinterFilter);
 	d->m_wizard = new QPushButton(m_pbox);
 	d->m_wizard->setPixmap(SmallIcon("wizard"));
 	d->m_wizard->setMinimumSize(QSize(d->m_printers->minimumHeight(),d->m_printers->minimumHeight()));
 	QToolTip::add(d->m_wizard, i18n("Add printer..."));
+	QWhatsThis::add(d->m_wizard, whatsThisAddPrinterWizard);
 	d->m_ok = new KPushButton(KGuiItem(i18n("&Print"), "fileprint"), this);
+        QWhatsThis::add( d->m_ok, whatsThisPrintButton);
 	d->m_ok->setDefault(true);
 	d->m_ok->setEnabled( false );
 	QPushButton	*m_cancel = new KPushButton(KStdGuiItem::cancel(), this);
+        QWhatsThis::add(m_cancel, whatsThisCancelButton);
 	d->m_preview = new QCheckBox(i18n("Previe&w"), m_pbox);
+	QWhatsThis::add(d->m_preview, whatsThisPreviewCheckBox);
 	d->m_filelabel = new QLabel(i18n("O&utput file:"), m_pbox);
+	QWhatsThis::add(d->m_filelabel,whatsThisOutputFileLabel);
 	d->m_file = new KURLRequester(QDir::homeDirPath()+"/print.ps", m_pbox);
+	QWhatsThis::add(d->m_file,whatsThisOutputFileLineedit);
 	d->m_file->setEnabled(false);
 	d->m_filelabel->setBuddy(d->m_file);
 	d->m_cmdlabel = new QLabel(i18n("Print co&mmand:"), m_pbox);
+        QWhatsThis::add( d->m_cmdlabel, whatsThisExternalPrintCommand);
+
 	d->m_cmd = new QLineEdit(m_pbox);
+        QWhatsThis::add( d->m_cmd, whatsThisExternalPrintCommand);
 	d->m_cmdlabel->setBuddy(d->m_cmd);
 	d->m_dummy = new QVBox(this);
 	d->m_plugin = new PluginComboBox(this);
 	d->m_extbtn = new KPushButton(this);
-	QToolTip::add(d->m_extbtn, i18n("Show/Hide Advanced Options"));
+	QToolTip::add(d->m_extbtn, i18n("Show/hide advanced options"));
+	QWhatsThis::add(d->m_extbtn, whatsThisExpandCollapse);
 	d->m_persistent = new QCheckBox(i18n("&Keep this dialog open after printing"), this);
-	QPushButton	*m_help = new KPushButton(KGuiItem(i18n("&Help"), "help"), this);
+        QWhatsThis::add( d->m_persistent, whatsThisKeepDialogOpenCheckbox);
+	QPushButton	*m_help = new KPushButton(KStdGuiItem::help(), this);
+        QWhatsThis::add( m_help, whatsThisHelpButton);
+
+	QWidget::setTabOrder( d->m_printers, d->m_filter );
+	QWidget::setTabOrder( d->m_filter, d->m_wizard );
+	QWidget::setTabOrder( d->m_wizard, d->m_properties );
+	QWidget::setTabOrder( d->m_properties, d->m_preview );
+	QWidget::setTabOrder( d->m_preview, d->m_file );
+	QWidget::setTabOrder( d->m_file, d->m_cmd );
+	QWidget::setTabOrder( d->m_plugin, d->m_persistent );
+	QWidget::setTabOrder( d->m_persistent, d->m_extbtn );
+	QWidget::setTabOrder( d->m_extbtn, d->m_options );
+	QWidget::setTabOrder( d->m_options, m_help );
+	QWidget::setTabOrder( m_help, d->m_ok );
+	QWidget::setTabOrder( d->m_ok, m_cancel );
 
 	// layout creation
 	QVBoxLayout	*l1 = new QVBoxLayout(this, 10, 10);
@@ -217,7 +415,7 @@ KPrintDialog::KPrintDialog(QWidget *parent, const char *name)
 
 		d->m_extbtn->hide();
 		m_pbox->hide();
-		
+
 		expandDialog(true);
 	}
 	else
@@ -370,9 +568,9 @@ void KPrintDialog::initialize(KPrinter *printer)
 
 	// Initialize output filename
 	if (!d->m_printer->outputFileName().isEmpty())
-		d->m_file->lineEdit()->setText(d->m_printer->outputFileName());
+		d->m_file->setURL( d->m_printer->outputFileName() );
 	else if (!d->m_printer->docFileName().isEmpty())
-		d->m_file->lineEdit()->setText(d->m_printer->docDirectory()+"/"+d->m_printer->docFileName()+".ps");
+		d->m_file->setURL( d->m_printer->docDirectory()+"/"+d->m_printer->docFileName()+".ps" );
 
 	if ( d->m_printers->count() > 0 )
 		slotPrinterSelected( d->m_printers->currentItem() );
@@ -410,6 +608,8 @@ void KPrintDialog::slotPrinterSelected(int index)
 			enableOutputFile(p->option("kde-special-file") == "1");
 			setOutputFileExtension(p->option("kde-special-extension"));
 		}
+                else
+                    enableOutputFile( ok );
 	}
 	d->m_properties->setEnabled(ok);
 	d->m_ok->setEnabled(ok);
@@ -460,7 +660,8 @@ void KPrintDialog::done(int result)
 		{
 			if (!checkOutputFile()) return;
 			d->m_printer->setOutputToFile(true);
-			d->m_printer->setOutputFileName(d->m_file->lineEdit()->text());
+			/* be sure to decode the output filename */
+			d->m_printer->setOutputFileName( KURL::decode_string( d->m_file->url() ) );
 		}
 		else
 			d->m_printer->setOutputToFile(false);
@@ -470,7 +671,7 @@ void KPrintDialog::done(int result)
 		opts["kde-preview"] = (d->m_preview->isChecked() ? "1" : "0");
 		opts["kde-isspecial"] = (prt->isSpecial() ? "1" : "0");
 		opts["kde-special-command"] = prt->option("kde-special-command");
-		
+
 		// merge options with KMPrinter object options
 		QMap<QString,QString>	popts = (prt->isEdited() ? prt->editedOptions() : prt->defaultOptions());
 		for (QMap<QString,QString>::ConstIterator it=popts.begin(); it!=popts.end(); ++it)
@@ -488,35 +689,30 @@ void KPrintDialog::done(int result)
 		KDialog::done(result);
 }
 
-static time_t qtToTime_t( const QDateTime& dt )
-{
-	struct tm t;
-	t.tm_sec = dt.time().second();
-	t.tm_min = dt.time().minute();
-	t.tm_hour = dt.time().hour();
-	t.tm_mday = dt.date().day();
-	t.tm_mon = dt.date().month();
-	t.tm_year = dt.date().year() - 1900;
-	t.tm_isdst = -1;
-	return mktime( &t );
-}
-
 bool KPrintDialog::checkOutputFile()
 {
 	bool	value(false);
-	if (d->m_file->lineEdit()->text().isEmpty())
+	if (d->m_file->url().isEmpty())
 		KMessageBox::error(this,i18n("The output filename is empty."));
 	else
 	{
-		QFileInfo	f(d->m_file->lineEdit()->text());
+		KURL url( d->m_file->url() );
+		if ( !url.isLocalFile() )
+			return true;
+
+		bool	anotherCheck;
+		do
+		{
+		anotherCheck = false;
+		QFileInfo	f(url.path());
 		if (f.exists())
 		{
 			if (f.isWritable())
 			{
 				//value = (KMessageBox::warningYesNo(this,i18n("File \"%1\" already exists. Overwrite?").arg(f.absFilePath())) == KMessageBox::Yes);
-				time_t mtimeDest = qtToTime_t( f.lastModified() );
-				KIO::RenameDlg dlg( this, i18n( "Print" ), QString::null, d->m_file->lineEdit()->text(),
-						KIO::M_OVERWRITE, ( time_t ) -1, f.size(), ( time_t ) -1, qtToTime_t( f.created() ), mtimeDest+1, mtimeDest, true );
+				time_t mtimeDest = f.lastModified().toTime_t();
+				KIO::RenameDlg dlg( this, i18n( "Print" ), QString::null, d->m_file->url(),
+						KIO::M_OVERWRITE, ( time_t ) -1, f.size(), ( time_t ) -1, f.created().toTime_t() , mtimeDest+1, mtimeDest, true );
 				int result = dlg.exec();
 				switch ( result )
 				{
@@ -528,8 +724,10 @@ bool KPrintDialog::checkOutputFile()
 						value = false;
 						break;
 					case KIO::R_RENAME:
-						d->m_file->lineEdit()->setText( dlg.newDestURL().path() );
+						url = dlg.newDestURL();
+						d->m_file->setURL( url.path() );
 						value = true;
+						anotherCheck = true;
 						break;
 				}
 			}
@@ -538,11 +736,15 @@ bool KPrintDialog::checkOutputFile()
 		}
 		else
 		{
-			if (QFileInfo(f.dirPath(true)).isWritable())
-				value = true;
-			else
+			QFileInfo d( f.dirPath( true ) );
+			if ( !d.exists() )
+				KMessageBox::error( this, i18n( "The output directory does not exist." ) );
+			else if ( !d.isWritable() )
 				KMessageBox::error(this,i18n("You don't have write permissions in that directory."));
+			else
+				value = true;
 		}
+		} while( anotherCheck );
 	}
 	return value;
 }
@@ -577,9 +779,15 @@ void KPrintDialog::setOutputFileExtension(const QString& ext)
 {
 	if (!ext.isEmpty())
 	{
-		QFileInfo	fi(d->m_file->lineEdit()->text());
-		QString		str = fi.dirPath(true)+"/"+fi.baseName()+"."+ext;
-		d->m_file->lineEdit()->setText(QDir::cleanDirPath(str));
+		KURL url( d->m_file->url() );
+		QString f( url.fileName() );
+		int p = f.findRev( '.' );
+		// change "file.ext"; don't change "file", "file." or ".file" but do change ".file.ext"
+		if ( p > 0 && p != int (f.length () - 1) )
+		{
+			url.setFileName( f.left( p ) + "." + ext );
+			d->m_file->setURL( KURL::decode_string( url.url() ) );
+		}
 	}
 }
 
@@ -636,18 +844,24 @@ void KPrintDialog::expandDialog(bool on)
 
 	if (on)
 	{
-		sz.setHeight(sz.height()+d->m_dummy->minimumSize().height()+layout()->spacing());
-		if (isVisible() || !d->m_dummy->isVisible())
+		sz.setHeight(sz.height()+d->m_dummy->minimumSize().height()+d->m_plugin->minimumSize().height()+2*layout()->spacing());
+		if (isVisible() || !d->m_dummy->isVisible() || !d->m_plugin->isVisible())
+		{
 			d->m_dummy->show();
+			d->m_plugin->show();
+		}
 		d->m_extbtn->setIconSet(SmallIconSet("up"));
 		d->m_extbtn->setText(i18n("Collaps&e"));
 		d->m_reduced = false;
 	}
 	else
 	{
-		sz.setHeight(sz.height()-d->m_dummy->height()-layout()->spacing());
-		if (!isVisible() || d->m_dummy->isVisible())
+		sz.setHeight(sz.height()-d->m_dummy->height()-d->m_plugin->height()-2*layout()->spacing());
+		if (!isVisible() || d->m_dummy->isVisible() || d->m_plugin->isVisible())
+		{
 			d->m_dummy->hide();
+			d->m_plugin->hide();
+		}
 		d->m_extbtn->setIconSet(SmallIconSet("down"));
 		d->m_extbtn->setText(i18n("&Expand"));
 		d->m_reduced = true;
@@ -684,15 +898,7 @@ void KPrintDialog::slotHelp()
 
 void KPrintDialog::slotOutputFileSelected(const QString& txt)
 {
-	KMPrinter	*prt = KMFactory::self()->manager()->findPrinter(d->m_printers->currentText());
-	if (prt && prt->isSpecial())
-	{
-		QString	ext("."+prt->option("kde-special-extension"));
-		if (ext.length() > 1 && txt.right(ext.length()) != ext)
-		{
-			d->m_file->lineEdit()->setText(txt+ext);
-		}
-	}
+	d->m_file->setURL( txt );
 }
 
 void KPrintDialog::init()
@@ -733,8 +939,28 @@ void KPrintDialog::enableDialogPage( int index, bool flag )
 
 void KPrintDialog::slotOpenFileDialog()
 {
-	d->m_file->fileDialog()->setCaption(i18n("Print to File"));
-	d->m_file->fileDialog()->setOperationMode( KFileDialog::Saving );
+	KFileDialog *dialog = d->m_file->fileDialog();
+
+	dialog->setCaption(i18n("Print to File"));
+	dialog->setMode(d->m_file->fileDialog()->mode() & ~KFile::LocalOnly);
+	dialog->setOperationMode( KFileDialog::Saving );
+
+	KMPrinter *prt = KMFactory::self()->manager()->findPrinter(d->m_printers->currentText());
+	if (prt)
+	{
+		QString	mimetype(prt->option("kde-special-mimetype"));
+		QString	ext(prt->option("kde-special-extension"));
+
+		if (!mimetype.isEmpty())
+		{
+			QStringList filter;
+			filter << mimetype;
+			filter << "all/allfiles";
+			dialog->setMimeFilter (filter, mimetype);
+		}
+		else if (!ext.isEmpty())
+			dialog->setFilter ("*." + ext + "\n*|" + i18n ("All Files"));
+	}
 }
 
 #include "kprintdialog.moc"
