@@ -1,8 +1,7 @@
 /*
  *  This file is part of the KDE libraries
- *  Copyright (c) 2001 Michael Goffioul <goffioul@imec.be>
+ *  Copyright (c) 2001 Michael Goffioul <kdeprint@swing.be>
  *
- *  $Id: kprintdialogpage.h,v 1.6 2001/08/27 13:54:50 goffioul Exp $
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Library General Public
@@ -32,12 +31,12 @@ class DrMain;
  * This class is intended to be used as base class for customized print dialog page. One of
  * the feature of the KDE print framework is to allow to customize the print dialog to
  * add some application specific print options. This is done by subclassing KPrintDialogPage
- * and reimplementing the 3 virtual functions @ref getOptions, @ref setOptions and @ref
+ * and reimplementing the 3 virtual functions getOptions, setOptions and
  * isValid(). The print options will be stored in the KPrinter object, and will be
- * accessible via @ref KPrinter::option(). The option name should follow the form
+ * accessible via KPrinter::option(). The option name should follow the form
  * "kde-appname-optionname" for internal reasons.
  *
- * <pre>
+ * \code
  * #include <kdeprint/kprintdialogpage.h>
  *
  * class MyDialogPage : public KPrintDialogPage
@@ -81,7 +80,7 @@ class DrMain;
  *   }
  *   return true;
  * }
- * </pre>
+ * \endcode
  *
  * @short Base class for customized print dialog pages.
  * @see KPrinter
@@ -125,7 +124,7 @@ public:
 	 * @param msg should contain an error message when returning false
 	 * @returns valid status
 	 */
-	virtual bool isValid(QString&);
+	virtual bool isValid(QString& msg);
 	/**
 	 * Get the ID of the page. Not used yet.
 	 * @returns the page ID
@@ -164,7 +163,7 @@ public:
 	/**
 	 * Change the page state when a non real printer is selected in the print dialog.
 	 * Usually, the default value (false) is OK in most cases and you don't need to
-	 * call this function explicitely.
+	 * call this function explicitly.
 	 * @param on if true, then the page will be disabled if a non real printer is selected
 	 * @see onlyRealPrinters()
 	 */
