@@ -1,8 +1,7 @@
 /*
  *  This file is part of the KDE libraries
- *  Copyright (c) 2001 Michael Goffioul <goffioul@imec.be>
+ *  Copyright (c) 2001 Michael Goffioul <kdeprint@swing.be>
  *
- *  $Id: kprinterimpl.cpp,v 1.42.2.2 2003/04/29 16:58:32 waba Exp $
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Library General Public
@@ -32,7 +31,7 @@
 
 #include <qfile.h>
 #include <qregexp.h>
-#include <qinputdialog.h>
+#include <kinputdialog.h>
 #include <klocale.h>
 #include <dcopclient.h>
 #include <kapplication.h>
@@ -77,6 +76,9 @@ void KPrinterImpl::preparePrinting(KPrinter *printer)
 	DrMain	*driver = mgr->loadPrinterDriver(mgr->findPrinter(printer->printerName()), false);
 	if (driver)
 	{
+		// Find the page size:
+		// 1) print option
+		// 2) default driver option
 		QString	psname = printer->option("PageSize");
 		if (psname.isEmpty())
 		{
@@ -89,11 +91,42 @@ void KPrinterImpl::preparePrinting(KPrinter *printer)
 			DrPageSize	*ps = driver->findPageSize(psname);
 			if (ps)
 			{
-				printer->setRealPageSize(ps->pageSize());
-				printer->setMargins(ps->margins());
-				printer->setRealDrawableArea( ps->pageRect() );
+				printer->setRealPageSize( ps );
 			}
 		}
+
+		// Find the numerical resolution
+		// 1) print option (Resolution)
+		// 2) default driver option (Resolution)
+		// 3) default printer resolution
+		// The resolution must have the format: XXXdpi or XXXxYYYdpi. In the second
+		// case the YYY value is used as resolution.
+		QString res = printer->option( "Resolution" );
+		if ( res.isEmpty() )
+		{
+			DrBase *opt = driver->findOption( "Resolution" );
+			if ( opt )
+				res = opt->get( "default" );
+			if ( res.isEmpty() )
+				res = driver->get( "resolution" );
+		}
+		if ( !res.isEmpty() )
+		{
+			QRegExp re( "(\\d+)(?:x(\\d+))?dpi" );
+			if ( re.search( res ) != -1 )
+			{
+				if ( !re.cap( 2 ).isEmpty() )
+					printer->setOption( "kde-resolution", re.cap( 2 ) );
+				else
+					printer->setOption( "kde-resolution", re.cap( 1 ) );
+			}
+		}
+
+		// Find the supported fonts
+		QString fonts = driver->get( "fonts" );
+		if ( !fonts.isEmpty() )
+			printer->setOption( "kde-fonts", fonts );
+
 		delete driver;
 	}
 
@@ -111,24 +144,32 @@ bool KPrinterImpl::printFiles(KPrinter *p, const QStringList& f, bool flag)
 	{
 		if (p->option("kde-special-command").isEmpty() && p->outputToFile())
 		{
-			if (f.count() > 1)
+			KURL url( p->outputFileName() );
+			if ( !url.isLocalFile() )
 			{
-				p->setErrorMessage(i18n("Cannot copy multiple files into one file."));
-				return false;
+				cmd = ( flag ? "mv" : "cp" ) + ( " %in $out{" + p->outputFileName() + "}" );
 			}
 			else
 			{
-				KProcess proc;
-				proc << (flag?"mv":"cp") << f[0] << p->outputFileName();
-				if (!proc.start(KProcess::Block) || !proc.normalExit() || proc.exitStatus() != 0)
+				if (f.count() > 1)
 				{
-					p->setErrorMessage(i18n("Cannot save print file to %1. Check that you have write access to it.").arg(p->outputFileName()));
+					p->setErrorMessage(i18n("Cannot copy multiple files into one file."));
 					return false;
 				}
+				else
+				{
+					KProcess proc;
+					proc << (flag?"mv":"cp") << f[0] << p->outputFileName();
+					if (!proc.start(KProcess::Block) || !proc.normalExit() || proc.exitStatus() != 0)
+					{
+						p->setErrorMessage(i18n("Cannot save print file to %1. Check that you have write access to it.").arg(p->outputFileName()));
+						return false;
+					}
+				}
+				return true;
 			}
-			return true;
 		}
-		if (!setupSpecialCommand(cmd,p,f))
+		else if (!setupSpecialCommand(cmd,p,f))
 			return false;
 	}
 	else if (!setupCommand(cmd,p))
@@ -217,7 +258,7 @@ bool KPrinterImpl::startPrinting(const QString& cmd, KPrinter *printer, const QS
 		if (QFile::exists(*it))
 		{
 			// quote and encode filenames
-			filestr.append(quote(QFile::encodeName(*it))).append(" ");;
+			filestr.append(quote(QFile::encodeName(*it))).append(" ");
 			printfiles.append(*it);
 		}
 		else
@@ -225,7 +266,7 @@ bool KPrinterImpl::startPrinting(const QString& cmd, KPrinter *printer, const QS
 
 	if (printfiles.count() > 0)
 	{
-		command.replace(QRegExp("%in"),filestr);
+		command.replace("%in",filestr);
 		int pid = dcopPrint(command,files,flag);
 		if (pid > 0)
 		{
@@ -309,6 +350,12 @@ int KPrinterImpl::doFilterFiles(KPrinter *printer, QStringList& files, const QSt
 	for (uint i=0;i<flist.count();i++)
 	{
 		KXmlCommand	*filter = KXmlCommandManager::self()->loadCommand(flist[i]);
+		if (!filter)
+		{
+			// TODO: better error message
+			printer->setErrorMessage(i18n("<p>Error while reading filter description for <b>%1</b>. Empty command line received.</p>").arg(flist[i]));
+			return -1; // Error
+		}
 		if (i == 0)
 			inputMimeTypes = filter->inputMimeTypes();
 
@@ -328,7 +375,7 @@ int KPrinterImpl::doFilterFiles(KPrinter *printer, QStringList& files, const QSt
 	}
 	kdDebug(500) << "kdeprint: filter command: " << filtercmd << endl;
 
-	QRegExp	rin("%in"), rout("%out"), rpsl("%psl"), rpsu("%psu");
+	QString	rin("%in"), rout("%out"), rpsl("%psl"), rpsu("%psu");
 	QString	ps = pageSizeToPageName( printer->option( "kde-printsize" ).isEmpty() ? printer->pageSize() : ( KPrinter::PageSize )printer->option( "kde-printsize" ).toInt() );
 	for (QStringList::Iterator it=files.begin(); it!=files.end(); ++it)
 	{
@@ -349,7 +396,7 @@ int KPrinterImpl::doFilterFiles(KPrinter *printer, QStringList& files, const QSt
 				while (done == 0)
 				{
 					bool	ok(false);
-					QString	targetMime = QInputDialog::getItem(
+					QString	targetMime = KInputDialog::getItem(
 						i18n("Select MIME Type"),
 						i18n("Select the target format for the conversion:"),
 						inputMimeTypes, 0, false, &ok);
@@ -407,31 +454,53 @@ int KPrinterImpl::doFilterFiles(KPrinter *printer, QStringList& files, const QSt
 
 int KPrinterImpl::autoConvertFiles(KPrinter *printer, QStringList& files, bool flag)
 {
+	QString primaryMimeType = "application/postscript";
+	QStringList mimeTypes( primaryMimeType );
+	if ( printer->option( "kde-isspecial" ) == "1" )
+	{
+		if ( !printer->option( "kde-special-command" ).isEmpty() )
+		{
+			KXmlCommand *cmd = KXmlCommandManager::self()->loadCommand( printer->option( "kde-special-command" ), true );
+			if ( cmd )
+			{
+				mimeTypes = cmd->inputMimeTypes();
+				// FIXME: the XML command description should now contain a primiary
+				// mime type as well. This is a temporary-only solution.
+				primaryMimeType = mimeTypes[ 0 ];
+			}
+		}
+	}
+	else
+	{
+		KMFactory::PluginInfo	info = KMFactory::self()->pluginInfo(KMFactory::self()->printSystem());
+		mimeTypes = info.mimeTypes;
+		primaryMimeType = info.primaryMimeType;
+	}
 	KMFactory::PluginInfo	info = KMFactory::self()->pluginInfo(KMFactory::self()->printSystem());
 	int		status(0), result;
 	for (QStringList::Iterator it=files.begin(); it!=files.end(); )
 	{
 		QString	mime = KMimeMagic::self()->findFileType(*it)->mimeType();
-		if (info.mimeTypes.findIndex(mime) == -1)
+		if (mimeTypes.findIndex(mime) == -1)
 		{
 			if ((result=KMessageBox::warningYesNoCancel(NULL,
 					       i18n("The file format %1 is not directly supported by the current print system. "
 					            "KDE can try to convert this file automatically to a supported format. But you can "
 						    "still try to send the file to the printer without any conversion. Do you want KDE "
-						    "to try to convert this file to %2?").arg(mime).arg(info.primaryMimeType),
+						    "to try to convert this file to %2?").arg(mime).arg(primaryMimeType),
 					       QString::null,
 					       i18n("Convert"),
 					       i18n("Keep"),
 					       QString::fromLatin1("kdeprintAutoConvert"))) == KMessageBox::Yes)
 			{
 				// find the filter chain
-				QStringList	flist = KXmlCommandManager::self()->autoConvert(mime, info.primaryMimeType);
+				QStringList	flist = KXmlCommandManager::self()->autoConvert(mime, primaryMimeType);
 				if (flist.count() == 0)
 				{
 					if (KMessageBox::warningYesNo(NULL,
 								      i18n("No appropriate filter was found to convert the file "
 								           "format %1 into %2. Do you want to print the "
-									   "file using its original format?").arg(mime).arg(info.primaryMimeType),
+									   "file using its original format?").arg(mime).arg(primaryMimeType),
 								      QString::null,
 								      i18n("Print"),
 								      i18n("Skip")) == KMessageBox::No)
@@ -480,9 +549,9 @@ bool KPrinterImpl::setupSpecialCommand(QString& cmd, KPrinter *p, const QStringL
 	s = KMFactory::self()->specialManager()->setupCommand(s, p->options());
 
 	QString	ps = pageSizeToPageName( p->option( "kde-printsize" ).isEmpty() ? p->pageSize() : ( KPrinter::PageSize )p->option( "kde-printsize" ).toInt() );
-	s.replace(QRegExp("%out"), quote(p->outputFileName()));
-	s.replace(QRegExp("%psl"), ps.lower());
-	s.replace(QRegExp("%psu"), ps);
+	s.replace("%psl", ps.lower());
+	s.replace("%psu", ps);
+	s.replace("%out", "$out{" + p->outputFileName() + "}"); // Replace as last
 	cmd = s;
 	return true;
 }
