@@ -1,8 +1,6 @@
 /*
  *  This file is part of the KDE libraries
- *  Copyright (c) 2001 Michael Goffioul <goffioul@imec.be>
- *
- *  $Id: kprinterpropertydialog.cpp,v 1.13 2002/04/22 19:15:11 binner Exp $
+ *  Copyright (c) 2001 Michael Goffioul <kdeprint@swing.be>
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Library General Public
@@ -32,10 +30,9 @@
 #include <klocale.h>
 #include <kpushbutton.h>
 #include <kguiitem.h>
-#include <kguiitem.h>
 
 KPrinterPropertyDialog::KPrinterPropertyDialog(KMPrinter *p, QWidget *parent, const char *name)
-: KDialogBase(parent, name, true, QString::null, KDialogBase::Ok|KDialogBase::Cancel|KDialogBase::User1, KDialogBase::Ok, false, KGuiItem(i18n("&Save"), "filesave")),
+: KDialogBase(parent, name, true, QString::null, KDialogBase::Ok|KDialogBase::Cancel|KDialogBase::User1, KDialogBase::Ok, false, KStdGuiItem::save()),
   m_printer(p), m_driver(0), m_current(0)
 {
 	m_pages.setAutoDelete(false);
