@@ -1,6 +1,6 @@
 /*
  *  This file is part of the KDE libraries
- *  Copyright (c) 2001 Michael Goffioul <goffioul@imec.be>
+ *  Copyright (c) 2001 Michael Goffioul <kdeprint@swing.be>
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Library General Public
@@ -357,6 +357,7 @@ DrBase* KXmlCommand::parseArgument(const QDomElement& e)
 	setOptionText(opt, e.attribute("description"));
 	opt->set("format", e.attribute("format"));
 	opt->set("default", e.attribute("default"));
+	opt->set( "persistent", e.attribute( "persistent" ) );
 	opt->setValueText(opt->get("default"));
 
 	return opt;
@@ -367,7 +368,7 @@ QString KXmlCommand::buildCommand(const QMap<QString,QString>& opts, bool pipein
 	check(true);
 
 	QString		str, cmd = d->m_command;
-	QRegExp re( "%value" ), quotedRe( "'%value'" );
+	QString re( "%value" ), quotedRe( "'%value'" );
 
 	if (d->m_driver)
 	{
@@ -382,7 +383,7 @@ QString KXmlCommand::buildCommand(const QMap<QString,QString>& opts, bool pipein
 			{
 				QString	format = dopt->get("format");
 				QString value = dopt->valueText();
-				if ( format.find( "'%value'" ) != -1 )
+				if ( format.find( quotedRe ) != -1 )
 				{
 					if ( ( value.right( 1 ) == "'" && value.left( 1 ) == "'" )  ||
 					     ( value.right( 1 ) == "\"" && value.left( 1 ) == "\"" ) )
@@ -397,11 +398,11 @@ QString KXmlCommand::buildCommand(const QMap<QString,QString>& opts, bool pipein
 				str.append(format).append(" ");
 			}
 		}
-		cmd.replace(QRegExp("%filterargs"), str);
+		cmd.replace("%filterargs", str);
 	}
 
-	cmd.replace(QRegExp("%filterinput"), d->m_io[0].m_format[(pipein?1:0)]);
-	cmd.replace(QRegExp("%filteroutput"), d->m_io[1].m_format[(pipeout?1:0)]);
+	cmd.replace("%filterinput", d->m_io[0].m_format[(pipein?1:0)]);
+	cmd.replace("%filteroutput", d->m_io[1].m_format[(pipeout?1:0)]);
 
 	return cmd;
 }
@@ -503,6 +504,7 @@ QDomElement KXmlCommand::createElement(QDomDocument& doc, DrBase *opt)
 	elem.setAttribute("format", opt->get("format"));
 	elem.setAttribute("description", opt->get("text"));
 	elem.setAttribute("default", opt->get("default"));
+	elem.setAttribute( "persistent", opt->get( "persistent" ) == "1" ? "1" : "0");
 
 	switch (opt->type())
 	{
@@ -606,6 +608,8 @@ void KXmlCommandManager::preload()
 		for (QStringList::Iterator it=d->m_cmdlist.begin(); it!=d->m_cmdlist.end(); ++it)
 		{
 			KXmlCommand	*xmlCmd = loadCommand(*it);
+			if (!xmlCmd) continue; // Error!
+
 			QStringList	inputMime = xmlCmd->inputMimeTypes();
 			for (QStringList::ConstIterator mime=inputMime.begin(); mime!=inputMime.end(); ++mime)
 			{
@@ -651,7 +655,7 @@ QStringList KXmlCommandManager::commandListWithDescription()
 
 QString KXmlCommandManager::selectCommand(QWidget *parent)
 {
-	KLibrary *lib = KLibLoader::self()->library( "libkdeprint_management" );
+	KLibrary *lib = KLibLoader::self()->library( "libkdeprint_management_module" );
 	if ( !lib )
 	{
 		KMessageBox::error( parent, i18n( "Unable to load KDE print management library: %1" ).arg( KLibLoader::self()->lastErrorMessage() ) );
@@ -680,13 +684,16 @@ int KXmlCommandManager::insertCommand(QStringList& list, const QString& filterna
 	preload();
 
 	int	pos(0);
-	KXmlCommand	*f1 = command(filtername), *f2(0);
+	KXmlCommand	*f1 = command(filtername), *f2 = 0;
 	if (f1 && f1->inputMimeTypes().count() > 0)
 	{
 		QString	mimetype = f1->inputMimeTypes()[0];
 		for (QStringList::Iterator it=list.begin(); it!=list.end(); ++it, pos++)
 		{
 			f2 = command(*it);
+			if (!f2)
+				return -1; // Shouldn't happen
+
 			if (f2->acceptMimeType(f1->mimeType()) && f1->acceptMimeType(mimetype))
 			{
 				list.insert(it, filtername);
