@@ -1,8 +1,6 @@
 /*
  *  This file is part of the KDE libraries
- *  Copyright (c) 2001 Michael Goffioul <goffioul@imec.be>
- *
- *  $Id: kmlpdmanager.cpp,v 1.16.4.3 2003/06/29 21:43:00 deller Exp $
+ *  Copyright (c) 2001 Michael Goffioul <kdeprint@swing.be>
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Library General Public
@@ -148,7 +146,7 @@ bool KMLpdManager::createPrinter(KMPrinter *printer)
 	{
 		// remote lpd queue
 		ent->m_args["rm"] = printer->device().host();
-		ent->m_args["rp"] = printer->device().path().replace(QRegExp("/"),QString::fromLatin1(""));
+		ent->m_args["rp"] = printer->device().path().replace("/",QString::fromLatin1(""));
 		ent->m_args["lpd_bounce"] = "true";
 		ent->m_comment = QString::fromLatin1("##PRINTTOOL3## REMOTE");
 	}
@@ -477,8 +475,8 @@ QMap<QString,QString> KMLpdManager::loadPrinttoolCfgFile(const QString& filename
 			{
 				name = line.left(p);
 				val = line.right(line.length()-p-1);
-				val.replace(QRegExp("\""),"");
-				val.replace(QRegExp("'"),"");
+				val.replace("\"","");
+				val.replace("'","");
 				if (!name.isEmpty() && !val.isEmpty())
 					map[name] = val;
 			}
@@ -602,7 +600,7 @@ bool KMLpdManager::createPrinttoolEntry(KMPrinter *printer, PrintcapEntry *entry
 			else if (prot == "ncp")
 			{
 				t << "server=" << dev.host() << endl;
-				t << "queue=" << dev.path().replace(QRegExp("/"),QString::fromLatin1("")) << endl;
+				t << "queue=" << dev.path().replace("/",QString::fromLatin1("")) << endl;
 				t << "user=" << dev.user() << endl;
 				t << "password=" << dev.pass() << endl;
 				entry->m_args["if"] = driverDirectory()+QString::fromLatin1("/ncpprint");
