@@ -1,8 +1,6 @@
 /*
  *  This file is part of the KDE libraries
- *  Copyright (c) 2001 Michael Goffioul <goffioul@imec.be>
- *
- *  $Id: kmlpdunixmanager.cpp,v 1.10.4.2 2003/02/07 17:07:18 goffioul Exp $
+ *  Copyright (c) 2001 Michael Goffioul <kdeprint@swing.be>
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Library General Public
@@ -389,7 +387,7 @@ void KMLpdUnixManager::parseSpoolInterface()
 
 //*********************************************************************************************************
 
-KMLpdUnixManager::KMLpdUnixManager(QObject *parent, const char *name)
+KMLpdUnixManager::KMLpdUnixManager(QObject *parent, const char *name, const QStringList & /*args*/)
 : KMManager(parent,name)
 {
 	m_loaded = false;
