@@ -1,8 +1,6 @@
 /*
  *  This file is part of the KDE libraries
- *  Copyright (c) 2001 Michael Goffioul <goffioul@imec.be>
- *
- *  $Id: kmlpdunixmanager.h,v 1.3 2001/05/06 18:40:09 goffioul Exp $
+ *  Copyright (c) 2001 Michael Goffioul <kdeprint@swing.be>
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Library General Public
@@ -27,7 +25,7 @@
 class KMLpdUnixManager : public KMManager
 {
 public:
-	KMLpdUnixManager(QObject *parent = 0, const char *name = 0);
+	KMLpdUnixManager(QObject *parent, const char *name, const QStringList & /*args*/);
 
 protected:
 	void listPrinters();
