@@ -1,6 +1,6 @@
 /*
  *  This file is part of the KDE libraries
- *  Copyright (c) 2001 Michael Goffioul <goffioul@imec.be>
+ *  Copyright (c) 2001 Michael Goffioul <kdeprint@swing.be>
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Library General Public
@@ -27,7 +27,7 @@
 #include <qptrlist.h>
 #include <klocale.h>
 
-KMLprJobManager::KMLprJobManager(QObject *parent, const char *name)
+KMLprJobManager::KMLprJobManager(QObject *parent, const char *name, const QStringList & /*args*/)
 : KMJobManager(parent, name)
 {
 	m_lpqhelper = new LpqHelper(this, "LpqHelper");
