@@ -1,6 +1,6 @@
 /*
  *  This file is part of the KDE libraries
- *  Copyright (c) 2001 Michael Goffioul <goffioul@imec.be>
+ *  Copyright (c) 2001 Michael Goffioul <kdeprint@swing.be>
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Library General Public
@@ -95,7 +95,7 @@ bool LprHandler::completePrinter(KMPrinter *prt, PrintcapEntry *entry, bool)
 	}
 	else
 		prt->setLocation(i18n("Unknown (unrecognized entry)"));
-	prt->setDevice(uri);
+	prt->setDevice(uri.url());
 	return true;
 }
 
@@ -122,7 +122,7 @@ PrintcapEntry* LprHandler::createEntry(KMPrinter *prt)
 	// this default handler only supports local parallel and remote lpd URIs
 	KURL	uri = prt->device();
 	QString	prot = uri.protocol();
-	if (!prot.isEmpty() && prot != "parallel" && prot != "file" && prot != "lpd")
+	if (!prot.isEmpty() && prot != "parallel" && prot != "file" && prot != "lpd" && prot != "socket")
 	{
 		manager()->setErrorMsg(i18n("Unsupported backend: %1.").arg(prot));
 		return NULL;
@@ -140,6 +140,14 @@ PrintcapEntry* LprHandler::createEntry(KMPrinter *prt)
 		// to /dev/lp0 by default)
 		entry->addField("lp", Field::String, QString::null);
 	}
+	else if ( prot == "socket" )
+	{
+		QString lp = uri.host();
+		if ( uri.port() == 0 )
+			lp.append( "%9100" );
+		else
+			lp.append( "%" ).append( QString::number( uri.port() ) );
+	}
 	else
 	{
 		entry->addField("lp", Field::String, uri.path());
