@@ -1,6 +1,6 @@
 /*
  *  This file is part of the KDE libraries
- *  Copyright (c) 2001,2002 Michael Goffioul <goffioul@imec.be>
+ *  Copyright (c) 2001,2002 Michael Goffioul <kdeprint@swing.be>
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Library General Public
@@ -20,9 +20,20 @@
 #ifndef LPRSETTINGS_H
 #define LPRSETTINGS_H
 
+#if !defined( _KDEPRINT_COMPILE ) && defined( __GNUC__ )
+#warning internal header, do not use except if you are a KDEPrint developer
+#endif
+
 #include <qobject.h>
 #include <kdeprint/kpreloadobject.h>
 
+/**
+ * @internal
+ * This class is internal to KDEPrint and is not intended to be
+ * used outside it. Please do not make use of this header, except
+ * if you're a KDEPrint developer. The API might change in the
+ * future and binary compatibility might be broken.
+ */
 class LprSettings : public QObject, public KPReloadObject
 {
 public:
