@@ -1,8 +1,6 @@
 /*
  *  This file is part of the KDE libraries
- *  Copyright (c) 2001 Michael Goffioul <goffioul@imec.be>
- *
- *  $Id: kmdriverdb.cpp,v 1.14 2002/05/13 16:44:04 goffioul Exp $
+ *  Copyright (c) 2001 Michael Goffioul <kdeprint@swing.be>
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Library General Public
@@ -29,7 +27,6 @@
 #include <qfile.h>
 #include <qtextstream.h>
 #include <qfileinfo.h>
-#include <qregexp.h>
 #include <kstandarddirs.h>
 #include <kapplication.h>
 #include <kmessagebox.h>
@@ -236,9 +233,9 @@ void KMDriverDB::loadDbFile()
 			else if (words[0] == "DESCRIPTION" && entry)
 				entry->description = words[1];
 			else if (words[0] == "RECOMMANDED" && entry && words[1].lower() == "yes")
-				entry->recommanded = true;
+				entry->recommended = true;
 			else if (words[0] == "DRIVERCOMMENT" && entry)
-				entry->drivercomment = ("<qt>"+words[1].replace(QRegExp("&lt;"), "<").replace(QRegExp("&gt;"), ">")+"</qt>");
+				entry->drivercomment = ("<qt>"+words[1].replace("&lt;", "<").replace("&gt;", ">")+"</qt>");
 		}
 		if (entry)
 			insertEntry(entry);
