@@ -1,8 +1,6 @@
 /*
  *  This file is part of the KDE libraries
- *  Copyright (c) 2001 Michael Goffioul <goffioul@imec.be>
- *
- *  $Id: kmjobviewer.cpp,v 1.38 2002/10/01 09:10:44 goffioul Exp $
+ *  Copyright (c) 2001 Michael Goffioul <kdeprint@swing.be>
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Library General Public
@@ -53,7 +51,8 @@
 #include <qlineedit.h>
 #include <kdialogbase.h>
 #include <qcheckbox.h>
-#include <qdragobject.h>
+#include <kurldrag.h>
+#include <kconfig.h>
 
 #undef m_manager
 #define	m_manager	KMFactory::self()->jobManager()
@@ -76,7 +75,7 @@ KJobListView::KJobListView( QWidget *parent, const char *name )
 
 bool KJobListView::acceptDrag( QDropEvent *e ) const
 {
-	if ( QUriDrag::canDecode( e ) )
+	if ( KURLDrag::canDecode( e ) )
 		return true;
 	else
 		return KListView::acceptDrag( e );
@@ -92,21 +91,31 @@ KMJobViewer::KMJobViewer(QWidget *parent, const char *name)
 	m_printers.setAutoDelete(false);
 	m_type = KMJobManager::ActiveJobs;
 	m_stickybox = 0;
+	m_standalone = ( parent == NULL );
 
 	setToolBarsMovable(false);
 	init();
 
-	if (!parent)
+	if (m_standalone)
 	{
 		setCaption(i18n("No Printer"));
-		resize(550,250);
+		KConfig *conf = KMFactory::self()->printConfig();
+		QSize defSize( 550, 250 );
+		conf->setGroup( "Jobs" );
+		resize( conf->readSizeEntry( "Size", &defSize ) );
 	}
 }
 
 KMJobViewer::~KMJobViewer()
 {
-	if (!parentWidget())
+	if (m_standalone)
+	{
+		kdDebug( 500 ) << "Destroying stand-alone job viewer window" << endl;
+		KConfig *conf = KMFactory::self()->printConfig();
+		conf->setGroup( "Jobs" );
+		conf->writeEntry( "Size", size() );
 		emit viewerDestroyed(this);
+	}
 	removeFromManager();
 }
 
@@ -118,7 +127,7 @@ void KMJobViewer::setPrinter(KMPrinter *p)
 void KMJobViewer::setPrinter(const QString& prname)
 {
 	// We need to trigger a refresh even if the printer
-	// has not changed, some jobs may have been cancelled
+	// has not changed, some jobs may have been canceled
 	// outside kdeprint. We can't return simply if
 	// prname == m_prname.
 	if (m_prname != prname)
@@ -133,7 +142,7 @@ void KMJobViewer::setPrinter(const QString& prname)
 
 void KMJobViewer::updateCaption()
 {
-	if (parentWidget())
+	if (!m_standalone)
 		return;
 
 	QString	pixname("fileprint");
@@ -153,7 +162,7 @@ void KMJobViewer::updateCaption()
 
 void KMJobViewer::updateStatusBar()
 {
-	if (parentWidget())
+	if (!m_standalone)
 		return;
 
 	int	limit = m_manager->limit();
@@ -170,11 +179,13 @@ void KMJobViewer::addToManager()
 		loadPrinters();
 		QPtrListIterator<KMPrinter>	it(m_printers);
 		for (; it.current(); ++it)
-			m_manager->addPrinter(it.current()->printerName(), (KMJobManager::JobType)m_type);
+			m_manager->addPrinter(it.current()->printerName(), (KMJobManager::JobType)m_type, it.current()->isSpecial());
 	}
 	else if (!m_prname.isEmpty())
 	{
-		m_manager->addPrinter(m_prname, (KMJobManager::JobType)m_type);
+		KMPrinter *prt = KMManager::self()->findPrinter( m_prname );
+		bool isSpecial = ( prt ? prt->isSpecial() : false );
+		m_manager->addPrinter(m_prname, (KMJobManager::JobType)m_type, isSpecial);
 	}
 }
 
@@ -293,7 +304,7 @@ void KMJobViewer::initActions()
 	connect(fact->popupMenu(),SIGNAL(aboutToHide()),KMTimer::self(),SLOT(release()));
 	connect(fact->popupMenu(),SIGNAL(aboutToShow()),SLOT(slotShowPrinterMenu()));
 
-	if (parentWidget())
+	if (!m_standalone)
 	{
 		KToolBar	*toolbar = toolBar();
 		hact->plug(toolbar);
@@ -332,7 +343,7 @@ void KMJobViewer::initActions()
 	slotSelectionChanged();
 }
 
-void KMJobViewer::buildPrinterMenu(QPopupMenu *menu, bool use_all)
+void KMJobViewer::buildPrinterMenu(QPopupMenu *menu, bool use_all, bool use_specials)
 {
 	loadPrinters();
 	menu->clear();
@@ -346,7 +357,8 @@ void KMJobViewer::buildPrinterMenu(QPopupMenu *menu, bool use_all)
 	}
 	for (; it.current(); ++it, i++)
 	{
-		if (!it.current()->instanceName().isEmpty())
+		if ( !it.current()->instanceName().isEmpty() ||
+				( it.current()->isSpecial() && !use_specials ) )
 			continue;
 		menu->insertItem(SmallIcon(it.current()->pixmap()), it.current()->printerName(), i);
 	}
@@ -355,13 +367,13 @@ void KMJobViewer::buildPrinterMenu(QPopupMenu *menu, bool use_all)
 void KMJobViewer::slotShowMoveMenu()
 {
 	QPopupMenu	*menu = static_cast<KActionMenu*>(actionCollection()->action("job_move"))->popupMenu();
-	buildPrinterMenu(menu, false);
+	buildPrinterMenu(menu, false, false);
 }
 
 void KMJobViewer::slotShowPrinterMenu()
 {
 	QPopupMenu	*menu = static_cast<KActionMenu*>(actionCollection()->action("filter_modify"))->popupMenu();
-	buildPrinterMenu(menu, true);
+	buildPrinterMenu(menu, true, true);
 }
 
 void KMJobViewer::updateJobs()
@@ -515,8 +527,10 @@ void KMJobViewer::loadPrinters()
 	QPtrListIterator<KMPrinter>	it(*(KMFactory::self()->manager()->printerList(false)));
 	for (;it.current();++it)
 	{
-		// keep only real printers (no instance, no implicit)
-		if ((it.current()->isPrinter() || it.current()->isClass(false)) && (it.current()->name() == it.current()->printerName()))
+		// keep only real printers (no instance, no implicit) and special printers
+		if ((it.current()->isPrinter() || it.current()->isClass(false) || 
+					( it.current()->isSpecial() && it.current()->isValid() ) )
+				&& (it.current()->name() == it.current()->printerName()))
 			m_printers.append(it.current());
 	}
 }
@@ -541,7 +555,7 @@ void KMJobViewer::triggerRefresh()
 	// to update itself. Otherwise, it's standalone
 	// kjobviewer and we need to synchronize all possible
 	// opened windows -> do the job on higher level.
-	if (parentWidget())
+	if (!m_standalone)
 		refresh(true);
 	else
 		emit refreshClicked();
@@ -562,10 +576,10 @@ void KMJobViewer::slotClose()
 
 void KMJobViewer::loadPluginActions()
 {
-	int	mpopindex(7), toolbarindex(parentWidget()?7:8), menuindex(7);
+	int	mpopindex(7), toolbarindex(!m_standalone?7:8), menuindex(7);
 	QMenuData	*menu(0);
 
-	if (!parentWidget())
+	if (m_standalone)
 	{
 		// standalone window, insert actions into main menubar
 		KAction	*act = actionCollection()->action("job_restart");
@@ -627,7 +641,7 @@ void KMJobViewer::reload()
 	// no refresh needed: view has been cleared before reloading
 	// and the actual refresh will be triggered either by the KControl
 	// module, or by KJobViewerApp using timer.
-	
+
 	// reload the columns needed: remove the old one
 	for (int c=m_view->columns()-1; c>5; c--)
 		m_view->removeColumn(c);
@@ -640,7 +654,7 @@ void KMJobViewer::reload()
 
 void KMJobViewer::closeEvent(QCloseEvent *e)
 {
-	if (!parentWidget())
+	if (m_standalone)
 	{
 		hide();
 		e->ignore();
@@ -681,7 +695,7 @@ void KMJobViewer::slotConfigure()
 {
 	KMTimer::self()->hold();
 
-	KDialogBase	dlg(this, 0, true, i18n("Print job settings"), KDialogBase::Ok|KDialogBase::Cancel);
+	KDialogBase	dlg(this, 0, true, i18n("Print Job Settings"), KDialogBase::Ok|KDialogBase::Cancel);
 	KMConfigJobs	*w = new KMConfigJobs(&dlg);
 	dlg.setMainWidget(w);
 	dlg.resize(300, 10);
@@ -704,15 +718,15 @@ bool KMJobViewer::isSticky() const
 
 void KMJobViewer::slotDropped( QDropEvent *e, QListViewItem* )
 {
-	QStrList uris;
 	QStringList files;
 	QString target;
 
-	QUriDrag::decode( e, uris );
-	for ( uint i=0; i<uris.count(); i++ )
+        KURL::List uris;
+	KURLDrag::decode( e, uris );
+	for ( KURL::List::ConstIterator it = uris.begin();
+	      it != uris.end(); ++it)
 	{
-		KURL url( uris.at( i ) );
-		if ( KIO::NetAccess::download( url, target ) )
+		if ( KIO::NetAccess::download( *it, target, 0 ) )
 			files << target;
 	}
 
