@@ -1,8 +1,6 @@
 /*
  *  This file is part of the KDE libraries
- *  Copyright (c) 2001 Michael Goffioul <goffioul@imec.be>
- *
- *  $Id: kmpropbackend.cpp,v 1.6 2002/08/13 08:03:13 binner Exp $
+ *  Copyright (c) 2001 Michael Goffioul <kdeprint@swing.be>
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Library General Public
@@ -59,8 +57,8 @@ void KMPropBackend::setPrinter(KMPrinter *p)
 {
 	if (p && p->isPrinter())
 	{
-		m_uri->setText(p->device().prettyURL());
-		QString	prot = p->device().protocol().lower();
+		m_uri->setText(p->device());
+		QString	prot = p->deviceProtocol();
 		if (prot == "ipp" || prot == "http") m_type->setText(i18n("IPP Printer"));
 		else if (prot == "usb") m_type->setText(i18n("Local USB Printer"));
 		else if (prot == "parallel") m_type->setText(i18n("Local Parallel Printer"));
