@@ -1,8 +1,6 @@
 /*
  *  This file is part of the KDE libraries
- *  Copyright (c) 2001 Michael Goffioul <goffioul@imec.be>
- *
- *  $Id: kmpropgeneral.cpp,v 1.5.2.1 2003/02/19 18:34:30 mueller Exp $
+ *  Copyright (c) 2001 Michael Goffioul <kdeprint@swing.be>
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Library General Public
@@ -37,7 +35,7 @@ KMPropGeneral::KMPropGeneral(QWidget *parent, const char *name)
 	m_description = new QLabel("",this);
 
 	QLabel	*l1 = new QLabel(i18n("Printer name:"), this);
-	QLabel	*l2 = new QLabel(i18n("Phys. Location", "Location:"), this);
+	QLabel	*l2 = new QLabel(i18n("Physical Location", "Location:"), this);
 	QLabel	*l3 = new QLabel(i18n("Description:"), this);
 
 	// layout
