@@ -1,8 +1,6 @@
 /*
  *  This file is part of the KDE libraries
- *  Copyright (c) 2001 Michael Goffioul <goffioul@imec.be>
- *
- *  $Id: kmwbackend.cpp,v 1.7 2002/08/12 19:38:47 binner Exp $
+ *  Copyright (c) 2001 Michael Goffioul <kdeprint@swing.be>
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Library General Public
@@ -31,6 +29,7 @@
 #include <kcursor.h>
 #include <klocale.h>
 #include <kseparator.h>
+#include <kdialog.h>
 
 class KRadioButton : public QRadioButton
 {
@@ -55,7 +54,7 @@ KMWBackend::KMWBackend(QWidget *parent, const char *name)
 	m_buttons = new QButtonGroup(this);
 	m_buttons->hide();
 
-	m_layout = new QVBoxLayout(this, 10, 10);
+	m_layout = new QVBoxLayout(this, 0, KDialog::spacingHint());
 	m_layout->addStretch(1);
 	m_count = 0;
 }
@@ -79,7 +78,7 @@ void KMWBackend::initPrinter(KMPrinter *p)
 		ID = s.toInt();
 	else
 	{
-		s = p->device().protocol();
+		s = p->deviceProtocol();
 		// extract protocol
 		if (s == "parallel" || s == "serial" || s == "usb") ID = KMWizard::Local;
 		else if (s == "smb") ID = KMWizard::SMB;
