@@ -1,8 +1,6 @@
 /*
  *  This file is part of the KDE libraries
- *  Copyright (c) 2001 Michael Goffioul <goffioul@imec.be>
- *
- *  $Id: kmwdrivertest.cpp,v 1.14 2002/08/12 19:38:47 binner Exp $
+ *  Copyright (c) 2001 Michael Goffioul <kdeprint@swing.be>
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Library General Public
@@ -34,6 +32,7 @@
 #include <kapplication.h>
 #include <kmessagebox.h>
 #include <kguiitem.h>
+#include <kio/netaccess.h>
 
 KMWDriverTest::KMWDriverTest(QWidget *parent, const char *name)
 : KMWizardPage(parent,name)
@@ -102,7 +101,11 @@ void KMWDriverTest::initPrinter(KMPrinter *p)
 	QString	drfile = p->option("kde-driver");
 	bool	checkDriver(true);
 	if (!drfile.isEmpty() && drfile != "raw")
+	{
 		m_driver = KMFactory::self()->manager()->loadFileDriver(drfile);
+		/* remove the temp file if it has been downloaded */
+		KIO::NetAccess::removeTempFile( drfile );
+	}
 	else if (p->dbEntry() != NULL)
 		m_driver = KMFactory::self()->manager()->loadDbDriver(p->dbEntry());
 	else
@@ -139,7 +142,7 @@ void KMWDriverTest::slotTest()
 	if (KMFactory::self()->manager()->createPrinter(m_printer))
 	{
 		if (KMFactory::self()->manager()->testPrinter(m_printer))
-			KMessageBox::information(this,i18n("Test page successfully sent to printer. Wait until printing is complete, then click the OK button."));
+			KMessageBox::information(this,"<qt>"+i18n("Test page successfully sent to printer. Wait until printing is complete, then click the OK button."));
 		else
 			KMessageBox::error(this,"<qt>"+i18n("Unable to test printer: ")+KMFactory::self()->manager()->errorMsg()+"</qt>");
 		if (!KMFactory::self()->manager()->removePrinter(m_printer))
