@@ -1,8 +1,6 @@
 /*
  *  This file is part of the KDE libraries
- *  Copyright (c) 2001 Michael Goffioul <goffioul@imec.be>
- *
- *  $Id: kmwfile.cpp,v 1.7 2002/08/12 19:38:47 binner Exp $
+ *  Copyright (c) 2001 Michael Goffioul <kdeprint@swing.be>
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Library General Public
@@ -75,5 +73,5 @@ bool KMWFile::isValid(QString& msg)
 void KMWFile::updatePrinter(KMPrinter *p)
 {
 	QString	dev = QString::fromLatin1("file:%1").arg(m_url->url());
-	p->setDevice(KURL(dev));
+	p->setDevice(dev);
 }
