@@ -1,6 +1,6 @@
 /*
  *  This file is part of the KDE libraries
- *  Copyright (c) 2001 Michael Goffioul <goffioul@imec.be>
+ *  Copyright (c) 2001 Michael Goffioul <kdeprint@swing.be>
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Library General Public
@@ -26,31 +26,32 @@ class MarginValueWidget : public KDoubleNumInput
 {
 	Q_OBJECT
 public:
-	enum Mode { Pixels = 0, IN, CM };
+	enum Mode { Pixels = 0, IN, CM, MM };
 	MarginValueWidget(KNumInput *below, double value = 18.0, QWidget *parent = 0, const char *name = 0);
 
-	int margin();
+	float margin();
 	int resolution() const;
 	void setResolution(int dpi);
 
 public slots:
 	void setMode(int);
-	void setMargin(int);
+	void setMargin(float);
 
 signals:
-	void marginChanged(int);
+	void marginChanged(float);
 
 protected slots:
 	void slotValueChanged(double);
 
 protected:
-	int toPixel(double value, int mode);
-	double toValue(int pix, int mode);
+	float toPixel(double value, int mode);
+	double toValue(float pix, int mode);
 
 private:
 	int		m_mode;
 	double	m_dpi;
 	bool	m_block;
+	float m_margin;
 };
 
 #endif
