@@ -1,6 +1,6 @@
 /* vi: ts=8 sts=4 sw=4
  *
- * $Id: kdesu_pty.cpp,v 2.12 2002/09/30 20:15:02 garbanzo Exp $
+ * $Id: kdesu_pty.cpp,v 2.14.2.1 2004/03/30 14:44:59 waba Exp $
  *
  * This file is part of the KDE project, module kdesu.
  * Copyright (C) 1999,2000 Geert Jansen <jansen@kde.org>
@@ -20,12 +20,6 @@
 #define _GNU_SOURCE   /* Needed for getpt, ptsname in glibc 2.1.x systems */
 #endif
 
-// This should probably be a check for glibc, rather than a check for 
-// every system it breaks which is most everything else
-#if !defined(_XOPEN_SOURCE) && !defined(__osf__) && !defined(__FreeBSD__) && !defined(__NetBSD__)
-#define _XOPEN_SOURCE /* Needed for grantpt, unlockpt in glibc 2.1.x      */
-#endif
-
 #include <config.h>
 
 #include <stdio.h>
@@ -109,7 +103,7 @@ int PTY::getpt()
 
 #elif defined(HAVE_OPENPTY)
     // 2: BSD interface
-    // More prefered than the linux hacks
+    // More preferred than the linux hacks
     char name[30];
     int master_fd, slave_fd;
     if (openpty(&master_fd, &slave_fd, name, 0L, 0L) != -1)  {
@@ -251,7 +245,7 @@ int PTY::grantpt()
 	// Child: exec konsole_grantpty
 	if (ptyfd != pty_fileno && dup2(ptyfd, pty_fileno) < 0)
 	    _exit(1);
-	execlp("konsole_grantpty", "konsole_grantpty", "--grant", NULL);
+	execlp("konsole_grantpty", "konsole_grantpty", "--grant", (void *)0);
 	kdError(900) << k_lineinfo << "exec(): " << perror << "\n";
 	_exit(1);
     }
