@@ -1,6 +1,6 @@
 /* vi: ts=8 sts=4 sw=4
  *
- * $Id: su.cpp,v 1.26.4.2 2003/07/09 09:27:27 waba Exp $
+ * $Id: su.cpp,v 1.32.2.3 2004/04/13 13:32:27 waba Exp $
  *
  * This file is part of the KDE project, module kdesu.
  * Copyright (C) 1999,2000 Geert Jansen <jansen@kde.org>
@@ -56,12 +56,12 @@ SuProcess::~SuProcess()
 
 int SuProcess::checkInstall(const char *password)
 {
-    return exec(password, 1);
+    return exec(password, Install);
 }
 
 int SuProcess::checkNeedPassword()
 {
-    return exec(0L, 2);
+    return exec(0L, NeedPassword);
 }
 
 /*
@@ -80,6 +80,7 @@ int SuProcess::exec(const char *password, int check)
 	args += m_User;
     args += "-c";
     args += QCString(__KDE_BINDIR) + "/kdesu_stub";
+    args += "-";
 
     QCString command = __PATH_SU;
     if (::access(__PATH_SU, X_OK) != 0)
@@ -89,26 +90,33 @@ int SuProcess::exec(const char *password, int check)
           return check ? SuNotFound : -1;
     }
 
+    // kdDebug(900) << k_lineinfo << "Call StubProcess::exec()" << endl;
     if (StubProcess::exec(command, args) < 0)
     {
 	return check ? SuNotFound : -1;
     }
+    // kdDebug(900) << k_lineinfo << "Done StubProcess::exec()" << endl;
     
-    int ret = ConverseSU(password);
-    if (ret < 0) 
+    SuErrors ret = (SuErrors) ConverseSU(password);
+    // kdDebug(900) << k_lineinfo << "Conversation returned " << ret << endl;
+
+    if (ret == error) 
     {
 	if (!check)
 	    kdError(900) << k_lineinfo << "Conversation with su failed\n";
 	return ret;
     } 
-    if (check == 2)
+    if (check == NeedPassword)
     {
-	if (ret == 1)
+	if (ret == killme)
 	{
-	    if (kill(m_Pid, SIGTERM) == 0)
-               waitForChild();
-            else
-	       close(fd());
+	    if (kill(m_Pid, SIGKILL) < 0) ret=error;
+	    else 
+	    {
+		int iret = waitForChild();
+		if (iret < 0) ret=error;
+		else /* nothing */ {} ;
+	    }
 	}
 	return ret;
     }
@@ -120,38 +128,34 @@ int SuProcess::exec(const char *password, int check)
 	    ptr[i] = '\000';
     }
 
-    if (ret == 2)
+    if (ret == notauthorized)
     {
-       if (kill(m_Pid, SIGTERM) == 0)
-          waitForChild();
-       else
-          close(fd());
-       return SuIncorrectPassword;
+	kill(m_Pid, SIGKILL);
+	waitForChild();
+	return SuIncorrectPassword;
     }
 
-    ret = ConverseStub(check);
-    if (ret < 0)
+    int iret = ConverseStub(check);
+    if (iret < 0)
     {
 	if (!check)
 	    kdError(900) << k_lineinfo << "Converstation with kdesu_stub failed\n";
-	return ret;
-    } else if (ret == 1)
+	return iret;
+    } else if (iret == 1)
     {
-        if (kill(m_Pid, SIGTERM) == 0)
-           waitForChild();
-        else
-           close(fd());
-        return SuIncorrectPassword;
+	kill(m_Pid, SIGKILL);
+	waitForChild();
+	return SuIncorrectPassword;
     }
 
-    if (check == 1)
+    if (check == Install)
     {
 	waitForChild();
 	return 0;
     }
 	
-    ret = waitForChild();
-    return ret;
+    iret = waitForChild();
+    return iret;
 }
 
 /*
@@ -161,26 +165,42 @@ int SuProcess::exec(const char *password, int check)
 
 int SuProcess::ConverseSU(const char *password)
 {	
-    int colon, state = 0;
+    enum { WaitForPrompt, CheckStar, HandleStub } state = WaitForPrompt;
+    int colon;
     unsigned i, j;
+    
+    // kdDebug(900) << k_lineinfo << "ConverseSU starting." << endl;
 
     QCString line;
     while (true)
     {
 	line = readLine(); 
 	if (line.isNull())
-	    return ( state == 2 ? 2 : -1);
-	
+	    return ( state == HandleStub ? notauthorized : error);
+	kdDebug(900) << k_lineinfo << "Read line <" << line << ">" << endl;
+
 	switch (state) 
 	{
-	case 0:
+	case WaitForPrompt:
 	    // In case no password is needed.
 	    if (line == "kdesu_stub")
 	    {
 		unreadLine(line);
-		return 0;
+		return ok;
 	    }
 
+            while(waitMS(m_Fd,100)>0)
+            {
+		// There is more output available, so the previous line
+		// couldn't have been a password prompt (the definition
+		// of prompt being that  there's a line of output followed 
+		// by a colon, and then the process waits).
+		QCString more = readLine();
+		if (more.isEmpty()) break;
+		line = more;
+		kdDebug(900) << k_lineinfo << "Read line <" << more << ">" << endl;
+            }
+            
 	    // Match "Password: " with the regex ^[^:]+:[\w]*$.
 	    for (i=0,j=0,colon=0; i<line.length(); i++) 
 	    {
@@ -195,42 +215,53 @@ int SuProcess::ConverseSU(const char *password)
 	    if ((colon == 1) && (line[j] == ':')) 
 	    {
 		if (password == 0L)
-		    return 1;
-		WaitSlave();
-		write(m_Fd, password, strlen(password));
-		write(m_Fd, "\n", 1);
-		state++;
+		    return killme;
+		if (!checkPid(m_Pid))
+		{
+		    kdError(900) << "su has exited while waiting for pwd." << endl;
+		    return error;
+		}
+		if ((WaitSlave() == 0) && checkPid(m_Pid))
+		{
+			write(m_Fd, password, strlen(password));
+			write(m_Fd, "\n", 1);
+			state=CheckStar;
+		}
+		else
+		{
+			return error;
+		}
 	    }
 	    break;
 
-	case 1: 
+	case CheckStar: 
 	{
 	    QCString s = line.stripWhiteSpace();
 	    if (s.isEmpty()) 
 	    {
-		state++;
+		state=HandleStub;
 		break;
 	    }
 	    for (i=0; i<s.length(); i++)
             {
 		if (s[i] != '*')
-		    return -1;
+		    return error;
 	    }
-	    state++;
+	    state=HandleStub;
 	    break;
 	}
 
-	case 2:
+	case HandleStub:
 	    // Read till we get "kdesu_stub"
 	    if (line == "kdesu_stub")
 	    {
 		unreadLine(line);
-		return 0;
+		return ok;
 	    }
 	    break;
 	}
     }
-    return 0;
+    return ok;
 }
 
 void SuProcess::virtual_hook( int id, void* data )
