@@ -1,6 +1,6 @@
 /* vi: ts=8 sts=4 sw=4
  *
- * $Id: su.h,v 1.14 2002/03/05 23:13:27 rich Exp $
+ * $Id: su.h,v 1.16 2003/10/14 11:09:49 waba Exp $
  *
  * This file is part of the KDE project, module kdesu.
  * Copyright (C) 1999,2000 Geert Jansen <jansen@kde.org>
@@ -32,7 +32,8 @@ public:
     /**
      * Executes the command. This will wait for the command to finish.
      */
-    int exec(const char *password, int check=0);
+    enum checkMode { NoCheck=0, Install=1, NeedPassword=2 } ;
+    int exec(const char *password, int check=NoCheck);
 
     /** 
      * Checks if the stub is installed and the password is correct.
@@ -46,6 +47,7 @@ public:
     int checkNeedPassword();
 
 private:
+    enum SuErrors { error=-1, ok=0, killme=1, notauthorized=2 } ;
     int ConverseSU(const char *password);
 
 protected:
