@@ -322,7 +322,7 @@ KAboutContainerBase::KAboutContainerBase( int layoutType, QWidget *_parent,
   if( layoutType & AbtProduct )
   {
     QWidget *productArea = new  QWidget( this, "area" );
-    mTopLayout->addWidget( productArea, 0, AlignLeft );
+    mTopLayout->addWidget( productArea, 0, QApplication::reverseLayout() ? AlignRight : AlignLeft );
 
     QHBoxLayout *hbox = new QHBoxLayout(productArea,0,KDialog::spacingHint());
     if( hbox == 0 ) { return; }
@@ -438,7 +438,10 @@ void KAboutContainerBase::fontChange( const QFont &/*oldFont*/ )
   {
     QFont f( KGlobalSettings::generalFont() );
     f.setBold( true );
-    f.setPointSize( 14 ); // Perhaps changeable ?
+    int fs = f.pointSize();
+    if (fs == -1)
+       fs = QFontInfo(f).pointSize();
+    f.setPointSize( fs+2 ); // Lets not make it too big
     mTitleLabel->setFont(f);
   }
 
@@ -502,10 +505,9 @@ QFrame *KAboutContainerBase::addLicensePage( const QString &title,
   textEdit->setFont( KGlobalSettings::fixedFont() );
   textEdit->setReadOnly( true );
   textEdit->setWordWrap( QTextEdit::NoWrap );
-  textEdit->setText( text ); 
+  textEdit->setText( text );
   textEdit->setMinimumHeight( fontMetrics().lineSpacing()*numLines );
   vbox->addWidget( textEdit );
-
   return( page );
 }
 
@@ -632,15 +634,11 @@ void KAboutContainerBase::setImage( const QString &fileName )
     return;
   }
 
-  QImage logo( fileName );
-  if( logo.isNull() == false )
-  {
-    QPixmap pix;
-    pix = logo;
-    mImageLabel->setPixmap( pix );
-  }
-  mImageFrame->layout()->activate();
+  QPixmap logo( fileName );
+  if( !logo.isNull() )
+    mImageLabel->setPixmap( logo );
 
+  mImageFrame->layout()->activate();
 }
 
 
