@@ -2,9 +2,7 @@
 #define _KACTIONSHORTCUTLIST_H
 
 #include <kshortcutlist.h>
-
-class KActionCollection;
-typedef QValueList<KAction *> KActionPtrList;
+#include <kaction.h>
 
 //---------------------------------------------------------------------
 // class KActionShortcutList
