@@ -26,16 +26,17 @@
 #include <qwhatsthis.h>
 #include <qregexp.h>
 
-#include <kapplication.h>
 #include <kaboutdata.h>
+#include <kapplication.h>
 #include <kconfig.h>
-#include <kstandarddirs.h>
+#include <kdebug.h>
 #include <klineedit.h>
 #include <klocale.h>
-#include <kurl.h>
 #include <kmessagebox.h>
-#include <kdebug.h>
 #include <kprocess.h>
+#include <kstandarddirs.h>
+#include <kstdguiitem.h>
+#include <kurl.h>
 #include <kurllabel.h>
 
 #include "kbugreport.h"
@@ -90,7 +91,7 @@ KBugReport::KBugReport( QWidget * parentw, bool modal, const KAboutData *aboutDa
   {
     // This is a core KDE application -> redirect to the web form
     d->webFormLabel = new KURLLabel( parent );
-    setButtonCancelText(i18n("&Close"));
+    setButtonCancel( KStdGuiItem::close() );
   }
 
   QLabel * tmpLabel;
@@ -130,9 +131,8 @@ KBugReport::KBugReport( QWidget * parentw, bool modal, const KAboutData *aboutDa
     glay->addWidget( tmpLabel, row, 1 );
     QWhatsThis::add( tmpLabel, qwtstr );
 
-    setButtonOKText(i18n("&Send"),
-                    i18n("Send bug report."),
-                    i18n("Send this bug report to the KDE buglist."));
+    setButtonOK( KGuiItem( i18n("&Send"), "mail_send", i18n( "Send bug report." ),
+                    i18n( "Send this bug report to %1." ).arg( m_aboutData->bugAddress() ) ) );
 
   }
   else
@@ -152,7 +152,7 @@ KBugReport::KBugReport( QWidget * parentw, bool modal, const KAboutData *aboutDa
   d->appcombo->insertStrList((const char**)packages);
   connect(d->appcombo, SIGNAL(activated(int)), SLOT(appChanged(int)));
   d->appname = QString::fromLatin1( m_aboutData
-                                    ? m_aboutData->appName()
+                                    ? m_aboutData->productName()
                                     : qApp->name() );
   glay->addWidget( d->appcombo, row, 1 );
   int index = 0;
@@ -177,9 +177,10 @@ KBugReport::KBugReport( QWidget * parentw, bool modal, const KAboutData *aboutDa
       m_strVersion = m_aboutData->version();
   else
       m_strVersion = i18n("no version set (programmer error!)");
-  d->kde_version = QString::fromLatin1(" (KDE " KDE_VERSION_STRING ") ("
-          KDE_DISTRIBUTION_TEXT ")");
-  m_strVersion += d->kde_version;
+  d->kde_version = QString::fromLatin1( KDE_VERSION_STRING );
+  d->kde_version += ", " + QString::fromLatin1( KDE_DISTRIBUTION_TEXT );
+  if ( !d->webFormLabel )
+      m_strVersion += " " + d->kde_version;
   m_version = new QLabel( m_strVersion, parent );
   //glay->addWidget( m_version, row, 1 );
   glay->addMultiCellWidget( m_version, row, row, 1, 2 );
@@ -191,7 +192,7 @@ KBugReport::KBugReport( QWidget * parentw, bool modal, const KAboutData *aboutDa
   struct utsname unameBuf;
   uname( &unameBuf );
   d->os = QString::fromLatin1( unameBuf.sysname ) +
-					" (" + QString::fromLatin1( unameBuf.machine ) + ") "
+          " (" + QString::fromLatin1( unameBuf.machine ) + ") "
           "release " + QString::fromLatin1( unameBuf.release );
 
   tmpLabel = new QLabel(d->os, parent);
@@ -231,23 +232,9 @@ KBugReport::KBugReport( QWidget * parentw, bool modal, const KAboutData *aboutDa
                         "Enter the text (in English if possible) that you wish to submit for the "
                         "bug report.\n"
                         "If you press \"Send\", a mail message will be sent to the maintainer of "
-                        "this program \n"
-                        "and to the KDE buglist.");
+                        "this program.\n");
     QLabel * label = new QLabel( parent, "label" );
 
-    /*
-      2000-01-15 Espen
-      Does not work (yet). The label has no well defined height so the
-      dialog can be resized so that the action buttons become obscured
-
-      QString text = i18n(""
-      "Enter the text (in English if possible) that you wish to submit for the "
-      "bug report. If you press \"Send\", a mail message will be sent to the "
-      "maintainer of this program and to the KDE buglist.");
-      QLabel * label = new QLabel( parent, "label" );
-      label->setTextFormat( RichText );
-    */
-
     label->setText( text );
     lay->addWidget( label );
 
@@ -270,7 +257,7 @@ KBugReport::KBugReport( QWidget * parentw, bool modal, const KAboutData *aboutDa
     lay->addSpacing(20);
 
     updateURL();
-    d->webFormLabel->setText( "http://bugs.kde.org/wizard/index.php" );
+    d->webFormLabel->setText( "http://bugs.kde.org/wizard.cgi" );
     lay->addWidget( d->webFormLabel );
     lay->addSpacing(20);
 
@@ -286,27 +273,32 @@ KBugReport::~KBugReport()
 
 void KBugReport::updateURL()
 {
-    QString url = QString::fromLatin1("http://bugs.kde.org/wizard/index.php");
-    url += "?os=";
-    url += KURL::encode_string( d->os );
-    url += "&compiler=";
-    url += KURL::encode_string( QString::fromLatin1(KDE_COMPILER_VERSION) );
-    url += "&version=";
-    url += KURL::encode_string( m_strVersion );
-    url += "&package=";
-    url += KURL::encode_string( d->appcombo->currentText() );
-    url += "&kbugreport=1";
-    d->webFormLabel->setURL( url );
+    KURL url = "http://bugs.kde.org/wizard.cgi";
+    url.addQueryItem( "os", d->os );
+    url.addQueryItem( "compiler", KDE_COMPILER_VERSION );
+    url.addQueryItem( "kdeVersion", d->kde_version );
+    url.addQueryItem( "appVersion", m_strVersion );
+    url.addQueryItem( "package", d->appcombo->currentText() );
+    url.addQueryItem( "kbugreport", "1" );
+    d->webFormLabel->setURL( url.url() );
 }
 
 void KBugReport::appChanged(int i)
 {
-    if (d->appname == d->appcombo->text(i) && m_aboutData)
+    QString appName = d->appcombo->text(i);
+    int index = appName.find( '/' );
+    if ( index > 0 )
+        appName = appName.left( index );
+    kdDebug() << "appName " << appName << endl;
+
+    if (d->appname == appName && m_aboutData)
         m_strVersion = m_aboutData->version();
     else
         m_strVersion = i18n("unknown program name", "unknown");
 
-    m_strVersion += d->kde_version;
+    if ( !d->webFormLabel )
+        m_strVersion += d->kde_version;
+
     m_version->setText(m_strVersion);
     if ( d->webFormLabel )
         updateURL();
@@ -334,7 +326,7 @@ void KBugReport::slotSetFrom()
   m_process = 0;
   m_configureEmail->setEnabled(true);
 
-  // ### KDE3: why oh why is KEmailSettings in kio?
+  // ### KDE4: why oh why is KEmailSettings in kio?
   KConfig emailConf( QString::fromLatin1("emaildefaults") );
 
   // find out the default profile
@@ -429,7 +421,7 @@ void KBugReport::slotCancel()
   {
     int rc = KMessageBox::warningYesNo( this,
              i18n( "Close and discard\nedited message?" ),
-             i18n( "Close message" ), i18n( "&Discard" ), i18n("&Continue") );
+             i18n( "Close Message" ), KStdGuiItem::discard(), KStdGuiItem::cont() );
     if( rc == KMessageBox::No )
       return;
   }
@@ -450,7 +442,7 @@ QString KBugReport::text() const
   for(int i = 0; i < m_lineedit->numLines(); i++)
   {
      QString line = m_lineedit->textLine(i);
-     if (line.right(1) != "\n")
+     if (!line.endsWith("\n"))
         line += '\n';
      bodyText += line;
   }
@@ -458,7 +450,7 @@ QString KBugReport::text() const
   if (severity == QString::fromLatin1("i18n") && KGlobal::locale()->language() != KLocale::defaultLanguage()) {
       // Case 1 : i18n bug
       QString package = QString::fromLatin1("i18n_%1").arg(KGlobal::locale()->language());
-      package = package.replace(QRegExp("_"), QString::fromLatin1("-"));
+      package = package.replace(QString::fromLatin1("_"), QString::fromLatin1("-"));
       return QString::fromLatin1("Package: %1").arg(package) +
           QString::fromLatin1("\n"
                               "Application: %1\n"
@@ -466,7 +458,7 @@ QString KBugReport::text() const
                               "Version: %2\n").arg(appname).arg(m_strVersion)+
           os+QString::fromLatin1("\n")+bodyText;
   } else {
-      appname = appname.replace(QRegExp("_"), QString::fromLatin1("-"));
+      appname = appname.replace(QString::fromLatin1("_"), QString::fromLatin1("-"));
       // Case 2 : normal bug
       return QString::fromLatin1("Package: %1\n"
                                  "Version: %2\n"
@@ -492,16 +484,17 @@ bool KBugReport::sendBugReport()
   outputfile.close();
 
   QString subject = m_subject->text();
-  subject = subject.replace(QRegExp("\""), QString::fromLatin1("\\\""));
-  command += QString::fromLatin1(" --subject \"%1\" --recipient \"%2\" > %3").
-             arg(KProcess::quote(subject)).
-             arg(KProcess::quote(recipient)).
-             arg(outputfile.name());
+  command += " --subject ";
+  command += KProcess::quote(subject);
+  command += " --recipient ";
+  command += KProcess::quote(recipient);
+  command += " > ";
+  command += KProcess::quote(outputfile.name());
 
   fflush(stdin);
   fflush(stderr);
 
-  FILE * fd = popen(command.local8Bit(), "w");
+  FILE * fd = popen(QFile::encodeName(command), "w");
   if (!fd)
   {
     kdError() << "Unable to open a pipe to " << command << endl;
