@@ -81,7 +81,7 @@ KButtonBox::KButtonBox(QWidget *parent, Orientation _orientation,
   data->orientation = _orientation;
   data->border = border;
   data->autoborder = autoborder < 0 ? border : autoborder;
-  data->buttons.setAutoDelete(TRUE);
+  data->buttons.setAutoDelete(true);
 }
 
 KButtonBox::~KButtonBox() {
@@ -96,6 +96,8 @@ QPushButton *KButtonBox::addButton(const QString& text, bool noexpand) {
   data->buttons.append(item);
   item->button->adjustSize();
 
+  this->updateGeometry();
+  
   return item->button;
 }
 
@@ -120,7 +122,7 @@ void KButtonBox::addStretch(int scale) {
   if(scale > 0) {
     Item *item = new Item;
     item->button = 0;
-    item->noexpand  = FALSE;
+    item->noexpand  = false;
     item->stretch = scale;
     data->buttons.append(item);
   }
