@@ -32,6 +32,7 @@
 #include <qkeycode.h>
 #include <qfontdatabase.h>
 #include <qstyle.h>
+#include <qtooltip.h>
 
 #include <klocale.h>
 #include <kdebug.h>
@@ -65,6 +66,8 @@ KCharSelectTable::KCharSelectTable( QWidget *parent, const char *name, const QSt
     setNumRows( 8 );
 
     repaintContents( false );
+    
+    setToolTips();
 
     setFocusPolicy( QWidget::StrongFocus );
     setBackgroundMode( QWidget::NoBackground );
@@ -75,6 +78,8 @@ void KCharSelectTable::setFont( const QString &_font )
 {
     vFont = _font;
     repaintContents( false );
+
+    setToolTips();
 }
 
 //==================================================================
@@ -91,6 +96,8 @@ void KCharSelectTable::setTableNum( int _tableNum )
 
     vTableNum = _tableNum;
     repaintContents( false );
+
+    setToolTips();
 }
 
 //==================================================================
@@ -115,6 +122,8 @@ void KCharSelectTable::resizeEvent( QResizeEvent * e )
         setCellWidth( new_w );
     if( new_h !=  cellHeight())
         setCellHeight( new_h );
+
+    setToolTips();
 }
 
 //==================================================================
@@ -155,12 +164,8 @@ void KCharSelectTable::paintCell( class QPainter* p, int row, int col )
     }
 
     if ( c == focusItem.unicode() && hasFocus() ) {
-#if QT_VERSION < 300
-	style().drawFocusRect( p, QRect( 2, 2, w - 4, h - 4 ), colorGroup() );
-#else
 	style().drawPrimitive( QStyle::PE_FocusRect, p, QRect( 2, 2, w - 4, h - 4 ), 
 			       colorGroup() );
-#endif
 	focusPos = QPoint( col, row );
     }
 
@@ -325,6 +330,20 @@ void KCharSelectTable::gotoDown()
     }
 }
 
+//==================================================================
+void KCharSelectTable::setToolTips()
+{
+    for( int i=0 ; i< numRows(); i++ )
+    {
+	for( int j=0; j< numCols(); j++ )
+	{
+	    QRect r( cellWidth()*j, cellHeight()*i, cellWidth(), cellHeight() );
+	    QToolTip::remove(this,r);
+	    QToolTip::add(this, r, i18n("Character code","UTF code: %1").arg(QString::number(vTableNum * 256 + numCols()*i + j)));
+	}
+    }
+}
+
 /******************************************************************/
 /* Class: KCharSelect						  */
 /******************************************************************/
@@ -337,7 +356,7 @@ KCharSelect::KCharSelect( QWidget *parent, const char *name, const QString &_fon
     QHBox *bar = new QHBox( this );
     bar->setSpacing( KDialog::spacingHint() );
 
-    QLabel *lFont = new QLabel( i18n( "	 Font:	" ), bar );
+    QLabel *lFont = new QLabel( i18n( "Font:" ), bar );
     lFont->resize( lFont->sizeHint() );
     lFont->setAlignment( Qt::AlignRight | Qt::AlignVCenter );
     lFont->setMaximumWidth( lFont->sizeHint().width() );
@@ -348,7 +367,7 @@ KCharSelect::KCharSelect( QWidget *parent, const char *name, const QString &_fon
 
     connect( fontCombo, SIGNAL( activated( const QString & ) ), this, SLOT( fontSelected( const QString & ) ) );
 
-    QLabel *lTable = new QLabel( i18n( "  Table:  " ), bar );
+    QLabel *lTable = new QLabel( i18n( "Table:" ), bar );
     lTable->resize( lTable->sizeHint() );
     lTable->setAlignment( Qt::AlignRight | Qt::AlignVCenter );
     lTable->setMaximumWidth( lTable->sizeHint().width() );
