@@ -38,10 +38,9 @@ public:
 KCModule::KCModule(QWidget *parent, const char *name, const QStringList &)
 	: QWidget(parent, name), _btn(Help|Default|Apply)
 {
-    kdDebug() << "KCModule " << name << endl;
+    kdDebug( 281 ) << "KCModule " << name << endl;
     d = new KCModulePrivate;
     d->_useRootOnlyMsg = true;
-    d->_instance = new KInstance(name);
 	if (name && strlen(name)) {
 		d->_instance = new KInstance(name);
 		KGlobal::locale()->insertCatalogue(name);
@@ -54,7 +53,7 @@ KCModule::KCModule(QWidget *parent, const char *name, const QStringList &)
 KCModule::KCModule(KInstance *instance, QWidget *parent, const QStringList & )
     : QWidget(parent, instance ? instance->instanceName().data() : 0), _btn(Help|Default|Apply)
 {
-    kdDebug() << "KCModule instance " << (instance ? instance->instanceName().data() : "none") << endl;
+    kdDebug( 281 ) << "KCModule instance " << (instance ? instance->instanceName().data() : "none") << endl;
     d = new KCModulePrivate;
     d->_useRootOnlyMsg = true;
     d->_instance = instance;
@@ -67,7 +66,7 @@ KCModule::~KCModule()
 {
     if (d->_hasOwnInstance)
        delete d->_instance;
-	delete d;
+    delete d;
 }
 
 void KCModule::setRootOnlyMsg(const QString& msg)
@@ -99,3 +98,5 @@ void KCModule::virtual_hook( int, void* )
 { /*BASE::virtual_hook( id, data );*/ }
 
 #include "kcmodule.moc"
+
+// vim: sw=4 et sts=4
