@@ -21,10 +21,10 @@
 //
 // 1999-09-27 Espen Sand <espensa@online.no>
 // KColorDialog is now subclassed from KDialogBase. I have also extended
-// KColorDialog::getColor() so that in contains a parent argument. This
+// KColorDialog::getColor() so that it contains a parent argument. This
 // improves centering capability.
 //
-// layout managment added Oct 1997 by Mario Weilguni
+// layout management added Oct 1997 by Mario Weilguni
 // <mweilguni@sime.com>
 //
 
@@ -64,15 +64,14 @@
 #include <config.h>
 #include <kdebug.h>
 
-#ifdef Q_WS_X11
-#include <X11/Xlib.h>
+#include "config.h"
+#if defined Q_WS_X11 && ! defined K_WS_QTONLY
+#include <X11/Xlib.h> // schroder
 
 // defined in qapplication_x11.cpp
 typedef int (*QX11EventFilter) (XEvent*);
 extern QX11EventFilter qt_set_x11_event_filter (QX11EventFilter filter);
-
-#define HSV_X 305
-#define RGB_X 385
+#endif
 
 static const char * const recentColors = "Recent_Colors";
 static const char * const customColors = "Custom_Colors";
@@ -91,7 +90,7 @@ public:
       emit valueChanged( value() );
       emit valueChanged( currentValueText() );
   }
-  
+
 };
 
 
@@ -168,7 +167,7 @@ static void createStandardPalette()
     if ( standardPalette )
 	return;
 
-    standardPalette = spd.setObject(new QColor [STANDARD_PAL_SIZE], true);
+    spd.setObject(standardPalette, new QColor [STANDARD_PAL_SIZE], true/*array*/);
 
     int i = 0;
 
@@ -189,6 +188,8 @@ static void createStandardPalette()
     standardPalette[i++] = Qt::gray;
     standardPalette[i++] = Qt::darkGray;
     standardPalette[i++] = Qt::black;
+
+    KGlobal::locale()->insertCatalogue("kdelibs_colors");
 }
 
 
@@ -360,7 +361,7 @@ void KColorCells::paintCell( QPainter *painter, int row, int col )
 	if (shade)
         {
 		qDrawShadePanel( painter, 1, 1, cellWidth()-2,
-		    cellHeight()-2, colorGroup(), TRUE, 1, &brush );
+		    cellHeight()-2, colorGroup(), true, 1, &brush );
 		w = 2;
         }
         QColor color = colors[ row * numCols() + col ];
@@ -420,7 +421,7 @@ void KColorCells::mouseMoveEvent( QMouseEvent *e )
             int cell = posToCell(mPos);
             if ((cell != -1) && colors[cell].isValid())
             {
-               KColorDrag *d = KColorDrag::makeDrag( colors[cell], this);
+               KColorDrag *d = new KColorDrag( colors[cell], this);
                d->dragCopy();
             }
         }
@@ -515,7 +516,7 @@ void KColorPatch::mouseMoveEvent( QMouseEvent *e )
 {
         // Drag color object
         if( !(e->state() && LeftButton)) return;
-	KColorDrag *d = KColorDrag::makeDrag( color, this);
+	KColorDrag *d = new KColorDrag( color, this);
 	d->dragCopy();
 }
 
@@ -647,11 +648,11 @@ KPaletteTable::readNamedColor( void )
 	//
 	QString name = line.mid(pos).stripWhiteSpace();
 	if( name.isNull() == true || name.find(' ') != -1 ||
-	    name.find( "gray" ) != -1 )
+	    name.find( "gray" ) != -1 ||  name.find( "grey" ) != -1 )
 	{
 	  continue;
 	}
-	list.append( name );
+	list.append( i18n("color", name.latin1() ) );
       }
     }
 
@@ -759,7 +760,7 @@ KPaletteTable::setPalette( const QString &_paletteName )
   // In fact, 'mPalette' and 'cells' are 0 when in this mode. The reason
   // for this is maninly that KPalette reads from and writes to files using
   // "locate()". The colors used in "i18n_namedColors" mode comes from the
-  // X11 diretory and is not writable. I dont think this fit in KPalette.
+  // X11 diretory and is not writable. I don't think this fit in KPalette.
   //
   if( mPalette == 0 || mPalette->name() != paletteName )
   {
@@ -867,6 +868,7 @@ KPaletteTable::addToRecentColors( const QColor &color)
 class KColorDialog::KColorDialogPrivate {
 public:
     KPaletteTable *table;
+    QString originalPalette;
     bool bRecursion;
     bool bEditRgb;
     bool bEditHsv;
@@ -889,7 +891,9 @@ public:
     QCheckBox *cbDefaultColor;
     KColor defaultColor;
     KColor selColor;
+#if defined Q_WS_X11 && ! defined K_WS_QTONLY
     QX11EventFilter oldfilter;
+#endif
 };
 
 
@@ -901,7 +905,9 @@ KColorDialog::KColorDialog( QWidget *parent, const char *name, bool modal )
   d = new KColorDialogPrivate;
   d->bRecursion = true;
   d->bColorPicking = false;
+#if defined Q_WS_X11 && ! defined K_WS_QTONLY
   d->oldfilter = 0;
+#endif
   d->cbDefaultColor = 0L;
   setHelp( QString::fromLatin1("kcolordialog.html"), QString::null );
   connect( this, SIGNAL(okClicked(void)),this,SLOT(slotWriteSettings(void)));
@@ -1047,6 +1053,8 @@ KColorDialog::KColorDialog( QWidget *parent, const char *name, bool modal )
     SIGNAL( colorDoubleClicked( const QColor &, const QString & ) ),
     SLOT( slotColorDoubleClicked( const QColor &, const QString & ) )
   );
+  // Store the default value for saving time.
+  d->originalPalette = d->table->palette();
 
   //
   // a little space between
@@ -1101,7 +1109,7 @@ KColorDialog::KColorDialog( QWidget *parent, const char *name, bool modal )
   d->htmlName->setFixedWidth(w);
   l_grid->addWidget(d->htmlName, 1, 2, AlignLeft);
 
-  connect( d->htmlName, SIGNAL( textChanged(const QString &) ), 
+  connect( d->htmlName, SIGNAL( textChanged(const QString &) ),
       SLOT( slotHtmlChanged() ) );
 
   d->patch = new KColorPatch( page );
@@ -1120,15 +1128,43 @@ KColorDialog::KColorDialog( QWidget *parent, const char *name, bool modal )
   d->bEditHtml = false;
 
   disableResize();
+  KColor col;
+  col.setHsv( 0, 0, 255 );
+  _setColor( col );
+
+  d->htmlName->installEventFilter(this);
+  d->hsSelector->installEventFilter(this);
+  d->hsSelector->setAcceptDrops(true);
 }
 
 KColorDialog::~KColorDialog()
 {
+#if defined Q_WS_X11 && ! defined K_WS_QTONLY
     if (d->bColorPicking)
         qt_set_x11_event_filter(d->oldfilter);
+#endif
     delete d;
 }
 
+bool
+KColorDialog::eventFilter( QObject *obj, QEvent *ev )
+{
+    if ((obj == d->htmlName) || (obj == d->hsSelector))
+    switch(ev->type())
+    {
+      case QEvent::DragEnter:
+      case QEvent::DragMove:
+      case QEvent::DragLeave:
+      case QEvent::Drop:
+      case QEvent::DragResponse:
+            qApp->sendEvent(d->patch, ev);
+            return true;
+      default:
+            break;
+    }
+    return KDialogBase::eventFilter(obj, ev);
+}
+
 void
 KColorDialog::setDefaultColor( const QColor& col )
 {
@@ -1143,6 +1179,8 @@ KColorDialog::setDefaultColor( const QColor& col )
         // and the "default color" checkbox, under all items on the right side
         //
         d->cbDefaultColor = new QCheckBox( i18n( "Default color" ), mainWidget() );
+        d->cbDefaultColor->setChecked(true);
+
         d->l_right->addWidget( d->cbDefaultColor );
 
         mainWidget()->setMaximumSize( QWIDGETSIZE_MAX, QWIDGETSIZE_MAX ); // cancel setFixedSize()
@@ -1165,25 +1203,14 @@ QColor KColorDialog::defaultColor() const
 
 void KColorDialog::slotDefaultColorClicked()
 {
-    bool enable;
     if ( d->cbDefaultColor->isChecked() )
     {
-        d->selColor = QColor();
-        showColor( d->defaultColor, i18n( "-default-" ) );
-        enable = false;
+        d->selColor = d->defaultColor;
+        showColor( d->selColor, i18n( "-default-" ) );
     } else
     {
-        d->selColor = d->defaultColor;
-        enable = true;
+        showColor( d->selColor, QString::null );
     }
-    d->hedit->setEnabled( enable );
-    d->sedit->setEnabled( enable );
-    d->vedit->setEnabled( enable );
-    d->redit->setEnabled( enable );
-    d->gedit->setEnabled( enable );
-    d->bedit->setEnabled( enable );
-    d->valuePal->setEnabled( enable );
-    d->hsSelector->setEnabled( enable );
 }
 
 void
@@ -1196,7 +1223,6 @@ KColorDialog::readSettings()
   config->setGroup("Colors");
   QString palette = config->readEntry("CurrentPalette");
   d->table->setPalette(palette);
-
   config->setGroup( oldgroup );
 }
 
@@ -1204,18 +1230,24 @@ void
 KColorDialog::slotWriteSettings()
 {
   KConfig* config = KGlobal::config();
-
-  QString oldgroup = config->group();
-
   config->setGroup("Colors");
-  config->writeEntry("CurrentPalette", d->table->palette() );
-
-  config->setGroup( oldgroup );
+  QString palette = d->table->palette();
+  if (!config->hasDefault("CurrentPalette") &&
+      (d->table->palette() == d->originalPalette))
+  {
+     config->revertToDefault("CurrentPalette");
+  }
+  else
+  {
+     config->writeEntry("CurrentPalette", d->table->palette());
+  }
 }
 
 QColor
 KColorDialog::color() const
 {
+  if ( d->cbDefaultColor && d->cbDefaultColor->isChecked() )
+     return QColor();
   if ( d->selColor.isValid() )
     d->table->addToRecentColors( d->selColor );
   return d->selColor;
@@ -1231,7 +1263,7 @@ void KColorDialog::setColor( const QColor &col )
 //
 int KColorDialog::getColor( QColor &theColor, QWidget *parent )
 {
-  KColorDialog dlg( parent, "Color Selector", TRUE );
+  KColorDialog dlg( parent, "Color Selector", true );
   if ( theColor.isValid() )
     dlg.setColor( theColor );
   int result = dlg.exec();
@@ -1249,7 +1281,7 @@ int KColorDialog::getColor( QColor &theColor, QWidget *parent )
 //
 int KColorDialog::getColor( QColor &theColor, const QColor& defaultCol, QWidget *parent )
 {
-  KColorDialog dlg( parent, "Color Selector", TRUE );
+  KColorDialog dlg( parent, "Color Selector", true );
   dlg.setDefaultColor( defaultCol );
   dlg.setColor( theColor );
   int result = dlg.exec();
@@ -1363,11 +1395,20 @@ void KColorDialog::slotColorDoubleClicked
 
 void KColorDialog::_setColor(const KColor &color, const QString &name)
 {
-  if (color == d->selColor) return;
+  if (color.isValid())
+  {
+     if (d->cbDefaultColor && d->cbDefaultColor->isChecked())
+        d->cbDefaultColor->setChecked(false);
+     d->selColor = color;
+  }
+  else
+  {
+     if (d->cbDefaultColor && d->cbDefaultColor->isChecked())
+        d->cbDefaultColor->setChecked(true);
+     d->selColor = d->defaultColor;
+  }
 
-  d->selColor = color;
-
-  showColor( color, name );
+  showColor( d->selColor, name );
 
   emit colorSelected( d->selColor );
 }
@@ -1391,34 +1432,39 @@ void KColorDialog::showColor( const KColor &color, const QString &name )
   int h, s, v;
   color.hsv( &h, &s, &v );
   d->hsSelector->setValues( h, s );
+  d->valuePal->blockSignals(true);
   d->valuePal->setHue( h );
   d->valuePal->setSaturation( s );
   d->valuePal->setValue( v );
   d->valuePal->updateContents();
-  d->valuePal->repaint( FALSE );
+  d->valuePal->blockSignals(false);
+  d->valuePal->repaint( false );
   d->bRecursion = false;
 }
 
 
 static QWidget *kde_color_dlg_widget = 0;
 
-int kde_color_dlg_handler(XEvent *event)
+#if defined Q_WS_X11 && ! defined K_WS_QTONLY
+static int kde_color_dlg_handler(XEvent *event)
 {
     if (event->type == ButtonRelease)
     {
         QMouseEvent e( QEvent::MouseButtonRelease, QPoint(),
                        QPoint(event->xmotion.x_root, event->xmotion.y_root) , 0, 0 );
         QApplication::sendEvent( kde_color_dlg_widget, &e );
-        return TRUE;
+        return true;
     }
-    return FALSE;
+    return false;
 }
-
+#endif
 void
 KColorDialog::slotColorPicker()
 {
     d->bColorPicking = true;
+#if defined Q_WS_X11 && ! defined K_WS_QTONLY
     d->oldfilter = qt_set_x11_event_filter(kde_color_dlg_handler);
+#endif
     kde_color_dlg_widget = this;
     grabMouse( crossCursor );
     grabKeyboard();
@@ -1430,8 +1476,10 @@ KColorDialog::mouseReleaseEvent( QMouseEvent *e )
   if (d->bColorPicking)
   {
      d->bColorPicking = false;
+#if defined Q_WS_X11 && ! defined K_WS_QTONLY
      qt_set_x11_event_filter(d->oldfilter);
      d->oldfilter = 0;
+#endif
      releaseMouse();
      releaseKeyboard();
      _setColor( grabColor( e->globalPos() ) );
@@ -1457,8 +1505,10 @@ KColorDialog::keyPressEvent( QKeyEvent *e )
      if (e->key() == Key_Escape)
      {
         d->bColorPicking = false;
+#if defined Q_WS_X11 && ! defined K_WS_QTONLY
         qt_set_x11_event_filter(d->oldfilter);
         d->oldfilter = 0;
+#endif
         releaseMouse();
         releaseKeyboard();
      }
@@ -1522,4 +1572,4 @@ void KColorDialog::virtual_hook( int id, void* data )
 
 
 #include "kcolordialog.moc"
-#endif
+//#endif
