@@ -19,7 +19,7 @@
 //----------------------------------------------------------------------
 // KDE color selection dialog.
 
-// layout managment added Oct 1997 by Mario Weilguni
+// layout management added Oct 1997 by Mario Weilguni
 // <mweilguni@sime.com>
 
 #ifndef __KCOLORDIALOG_H__
@@ -49,9 +49,11 @@ class KColorCells;
  * The actual values can be fetched using the inherited xValue and yValue
  * methods.
  *
+ * \image html khsselector.png "KDE Hue/Saturation Selection Widget"
+ *
  * @see KXYSelector, KValueSelector, KColorDialog
  * @author Martin Jones (mjones@kde.org)
- * @version $Id: kcolordialog.h,v 1.50 2002/08/01 14:19:52 coolo Exp $
+ * @version $Id: kcolordialog.h,v 1.61 2003/09/10 00:12:13 wheeler Exp $
 */
 class KHSSelector : public KXYSelector
 {
@@ -69,10 +71,8 @@ protected:
    * which is used for buffering.
    */
   virtual void drawPalette( QPixmap *pixmap );
-  /**
-   * @reimplemented
-   */
   virtual void resizeEvent( QResizeEvent * );
+
   /**
    * Reimplemented from KXYSelector. This drawing is
    * buffered in a pixmap here. As real drawing
@@ -98,7 +98,7 @@ class KValueSelectorPrivate;
  *
  * @see KHSSelector, KColorDialog
  * @author Martin Jones (mjones@kde.org)
- * @version $Id: kcolordialog.h,v 1.50 2002/08/01 14:19:52 coolo Exp $
+ * @version $Id: kcolordialog.h,v 1.61 2003/09/10 00:12:13 wheeler Exp $
  */
 class KValueSelector : public KSelector
 {
@@ -130,10 +130,8 @@ protected:
    * which is used for buffering.
    */
   virtual void drawPalette( QPixmap *pixmap );
-  /**
-   * @reimplemented
-   */
   virtual void resizeEvent( QResizeEvent * );
+
   /**
    * Reimplemented from KSelector. The drawing is
    * buffered in a pixmap here. As real drawing
@@ -154,6 +152,13 @@ private:
 };
 
 
+/**
+ * A color class that preserves both RGB and HSV values; this is
+ * unlike QColor which only preserves RGB values and recalculates HSV
+ * values. The QColor behavior leads to an accumulation of rounding
+ * errors when working in the HSV color space.
+ * @author Waldo Bastian <bastian@kde.org>
+ **/
 class KColor : public QColor
 {
 public:
@@ -186,7 +191,7 @@ private:
 /**
  * A color palette in table form.
  * @author Waldo Bastian <bastian@kde.org>
- * @version $Id: kcolordialog.h,v 1.50 2002/08/01 14:19:52 coolo Exp $
+ * @version $Id: kcolordialog.h,v 1.61 2003/09/10 00:12:13 wheeler Exp $
  **/
 class KPaletteTable : public QWidget
 {
@@ -239,7 +244,7 @@ private:
 /**
 * A table of editable color cells.
 * @author Martin Jones <mjones@kde.org>
-* @version $Id: kcolordialog.h,v 1.50 2002/08/01 14:19:52 coolo Exp $
+* @version $Id: kcolordialog.h,v 1.61 2003/09/10 00:12:13 wheeler Exp $
 */
 class KColorCells : public QGridView
 {
@@ -266,38 +271,13 @@ public:
   void colorDoubleClicked( int col );
 
 protected:
-  /**
-   * @reimplemented
-   */
   virtual void paintCell( QPainter *painter, int row, int col );
-  /**
-   * @reimplemented
-   */
   virtual void resizeEvent( QResizeEvent * );
-  /**
-   * @reimplemented
-   */
   virtual void mouseReleaseEvent( QMouseEvent * );
-  /**
-   * @reimplemented
-   */
   virtual void mousePressEvent( QMouseEvent * );
-  /**
-   * @reimplemented
-   */
   virtual void mouseMoveEvent( QMouseEvent * );
-  /**
-   * @reimplemented
-   */
   virtual void dragEnterEvent( QDragEnterEvent *);
-  /**
-   * @reimplemented
-   */
   virtual void dropEvent( QDropEvent *);
-
-  /**
-   * @reimplemented
-   */
   virtual void mouseDoubleClickEvent( QMouseEvent * );
 
   int posToCell(const QPoint &pos, bool ignoreBorders=false);
@@ -318,10 +298,10 @@ private:
 
 /**
  * The KColorPatch widget is a (usually small) widget showing
- * a selected color e. g. in the @ref KColorDialog. It
+ * a selected color e. g. in the KColorDialog. It
  * automatically handles drag and drop from and on the widget.
  *
- * @short A color shower.
+ * @short A color displayer.
  */
 class KColorPatch : public QFrame
 {
@@ -336,21 +316,9 @@ signals:
   void colorChanged( const QColor&);
 
 protected:
-  /**
-   * @reimplemented
-   */
   virtual void drawContents( QPainter *painter );
-  /**
-   * @reimplemented
-   */
   virtual void mouseMoveEvent( QMouseEvent * );
-  /**
-   * @reimplemented
-   */
   virtual void dragEnterEvent( QDragEnterEvent *);
-  /**
-   * @reimplemented
-   */
   virtual void dropEvent( QDropEvent *);
 
 private:
@@ -368,37 +336,37 @@ private:
 /**
  * The KColorDialog provides a dialog for color selection.
  *
- * @sect Features:
+ * <b>Features:</b>\n
  *
- * @li Colour selection from a wide range of palettes.
- * @li Colour selection from a palette of H vs S and V selectors (similar to windoze).
+ * @li Color selection from a wide range of palettes.
+ * @li Color selection from a palette of H vs S and V selectors (similar to windoze).
  * @li Direct input of HSV or RGB values.
  * @li Saving of custom colors
  *
- * In most cases, you will want to use the static method @ref KColorDialog::getColor().
+ * In most cases, you will want to use the static method KColorDialog::getColor().
  * This pops up the dialog (with an initial selection provided by you), lets the
  * user choose a color, and returns.
  *
  * Example:
  *
- * <pre>
+ * \code
  * 	QColor myColor;
  * 	int result = KColorDialog::getColor( myColor );
  *         if ( result == KColorDialog::Accepted )
  *            ...
- * </pre>
+ * \endcode
  *
  * @image html kcolordialog.png "KDE Color Dialog"
  *
  * The color dialog is really a collection of several widgets which can
  * you can also use separately: the quadratic plane in the top left of
- * the dialog is a @ref KXYSelector. Right next to it is a @ref KHSSelector
- * for chosing hue/saturation.
+ * the dialog is a KXYSelector. Right next to it is a KHSSelector
+ * for choosing hue/saturation.
  *
- * On the right side of the dialog you see a @ref KPaletteTable showing
+ * On the right side of the dialog you see a KPaletteTable showing
  * up to 40 colors with a combo box which offers several predefined
  * palettes or a palette configured by the user. The small field showing
- * the currently selected color is a @ref KColorPatch.
+ * the currently selected color is a KColorPatch.
  *
  * @short A color selection dialog.
  **/
@@ -411,7 +379,7 @@ class KColorDialog : public KDialogBase
      * Constructs a color selection dialog.
      */
     KColorDialog( QWidget *parent = 0L, const char *name = 0L,
-		  bool modal = FALSE );
+		  bool modal = false );
     /**
      * Destroys the color selection dialog.
      */
@@ -428,7 +396,7 @@ class KColorDialog : public KDialogBase
      *
      * The selected color is returned in the argument @p theColor.
      *
-     * @returns @ref QDialog::result().
+     * @returns QDialog::result().
      */
     static int getColor( QColor &theColor, QWidget *parent=0L );
 
@@ -442,7 +410,7 @@ class KColorDialog : public KDialogBase
      * selected by the "default color" checkbox. When this checkbox is checked,
      * the invalid color (QColor()) is returned into @p theColor.
      *
-     * @returns @ref QDialog::result().
+     * @returns QDialog::result().
      */
     static int getColor( QColor &theColor, const QColor& defaultColor, QWidget *parent=0L );
 
@@ -508,14 +476,9 @@ class KColorDialog : public KDialogBase
     void showColor( const KColor &color, const QString &name );
 
   protected:
-   /**
-    * @reimplemented
-    */
     virtual void mouseReleaseEvent( QMouseEvent * );
-   /**
-    * @reimplemented
-    */
     virtual void keyPressEvent( QKeyEvent * );
+    virtual bool eventFilter( QObject *obj, QEvent *ev );
 
   protected:
     virtual void virtual_hook( int id, void* data );
