@@ -20,8 +20,6 @@
    Boston, MA 02111-1307, USA.
 */
 
-#include <stdlib.h>
-
 #include <qclipboard.h>
 #include <qlistbox.h>
 #include <qpopupmenu.h>
@@ -43,6 +41,8 @@
 
 #include "kcombobox.h"
 
+#include <stdlib.h> // getenv
+
 class KComboBox::KComboBoxPrivate
 {
 public:
@@ -70,8 +70,8 @@ KComboBox::KComboBox( bool rw, QWidget *parent, const char *name )
 
     if ( rw )
     {
-        KLineEdit *edit = new KLineEdit( this, "combo lineedit" );
-        setLineEdit( edit );
+      KLineEdit *edit = new KLineEdit( this, "combo lineedit" );
+      setLineEdit( edit );
     }
 }
 
@@ -92,12 +92,7 @@ void KComboBox::init()
     setContextMenuEnabled( true );
 
     // for wheelscrolling
-    installEventFilter( this );
-    if ( lineEdit() )
-    {
-        connect( lineEdit(), SIGNAL( returnPressed() ),
-                 this, SIGNAL( returnPressed() ));
-    }
+    // installEventFilter( this );
 }
 
 
@@ -260,7 +255,7 @@ void KComboBox::setLineEdit( QLineEdit *edit )
     if ( !editable() && edit &&
          qstrcmp( edit->className(), "QLineEdit" ) == 0 )
     {
-        // uic generates code that creates a read-only QComboBox and then
+        // uic generates code that creates a read-only KComboBox and then
         // calls combo->setEditable( true ), which causes QComboBox to set up
         // a dumb QLineEdit instead of our nice KLineEdit.
         // As some KComboBox features rely on the KLineEdit, we reject
@@ -273,8 +268,8 @@ void KComboBox::setLineEdit( QLineEdit *edit )
     d->klineEdit = dynamic_cast<KLineEdit*>( edit );
     setDelegate( d->klineEdit );
 
-    // forward some signals.
-    if ( edit )
+    // Connect the returnPressed signal for both Q[K]LineEdits'
+    if (edit)
         connect( edit, SIGNAL( returnPressed() ), SIGNAL( returnPressed() ));
 
     if ( d->klineEdit )
@@ -284,7 +279,7 @@ void KComboBox::setLineEdit( QLineEdit *edit )
         // be a dangling pointer then, so prevent that. Note: only do this
         // when it is a KLineEdit!
         connect( edit, SIGNAL( destroyed() ), SLOT( lineEditDeleted() ));
-        
+
         connect( d->klineEdit, SIGNAL( returnPressed( const QString& )),
                  SIGNAL( returnPressed( const QString& ) ));
 
@@ -348,6 +343,7 @@ void KComboBox::lineEditDeleted()
         setDelegate( 0L );
 }
 
+
 // *********************************************************************
 // *********************************************************************
 
@@ -426,17 +422,21 @@ QStringList KHistoryCombo::historyItems() const
 
 void KHistoryCombo::clearHistory()
 {
+    QString temp = currentText();
     KComboBox::clear();
     if ( useCompletion() )
         completionObject()->clear();
+    setEditText( temp );
 }
 
 void KHistoryCombo::addContextMenuItems( QPopupMenu* menu )
 {
-    if ( menu &&!lineEdit()->text().isEmpty())
+    if ( menu )
     {
         menu->insertSeparator();
-        menu->insertItem( i18n("Empty Contents"), this, SLOT( slotClear()));
+        int id = menu->insertItem( SmallIcon("history_clear"), i18n("Clear &History"), this, SLOT( slotClear()));
+        if (!count())
+           menu->setItemEnabled(id, false);
     }
 }
 
@@ -526,7 +526,7 @@ void KHistoryCombo::rotateUp()
         myIterateIndex = -1;
 
         // if the typed text is the same as the first item, skip the first
-        if ( myText == text(0) )
+        if ( count() > 0 && myText == text(0) )
             myIterateIndex = 0;
 
         setEditText( myText );
@@ -558,7 +558,7 @@ void KHistoryCombo::rotateDown()
         }
         else { // bottom of history
             if ( myIterateIndex == -2 ) {
-                KNotifyClient::event( KNotifyClient::notification,
+                KNotifyClient::event( winId(), KNotifyClient::notification,
                                       i18n("No further item in the history."));
             }
 
@@ -574,15 +574,16 @@ void KHistoryCombo::rotateDown()
 
 void KHistoryCombo::keyPressEvent( QKeyEvent *e )
 {
+    KKey event_key( e );
+
     // going up in the history, rotating when reaching QListBox::count()
-    if ( KStdAccel::isEqual( e, KStdAccel::rotateUp() ) ) {
+    if ( KKey(KStdAccel::rotateUp().keyCodeQt()) == event_key )
         rotateUp();
-    }
+
     // going down in the history, no rotation possible. Last item will be
     // the text that was in the lineedit before Up was called.
-    else if ( KStdAccel::isEqual( e, KStdAccel::rotateDown() ) ) {
+    else if ( KKey(KStdAccel::rotateDown().keyCodeQt()) == event_key )
         rotateDown();
-    }
     else
         KComboBox::keyPressEvent( e );
 }
@@ -621,7 +622,7 @@ void KHistoryCombo::setPixmapProvider( KPixmapProvider *prov )
     myPixProvider = prov;
 
     // re-insert all the items with/without pixmap
-    // I would prefer to use changeItem(), but that doesn't honour the pixmap
+    // I would prefer to use changeItem(), but that doesn't honor the pixmap
     // when using an editable combobox (what we do)
     if ( count() > 0 ) {
         QStringList items( historyItems() );
