@@ -24,6 +24,7 @@
 #include <qptrlist.h>
 #include <qstring.h>
 #include <qobject.h>
+#include <kdemacros.h>
 
 class KAction;
 class KActionCollection;
@@ -70,11 +71,12 @@ protected:
 };
 
 /**
+ * @deprecated
  * A command which stores its name.
  * This class mostly exists for compatibility with KDE-2.x
  * It is more memory-efficient to use KCommand and to implement the name() method.
  */
-class KNamedCommand : public KCommand
+class KDE_DEPRECATED KNamedCommand : public KCommand
 {
 protected:
     /**
@@ -110,7 +112,7 @@ class KMacroCommand : public KNamedCommand
 {
 public:
     /**
-     * Creates a macro command. You will then need to call @ref addCommand
+     * Creates a macro command. You will then need to call addCommand
      * for each subcommand to be added to this macro command.
      * @param name the name of this command, translated, since it will appear
      * in the menus.
@@ -168,6 +170,7 @@ public:
      * using the standard names ("edit_undo" and "edit_redo").
      * @param withMenus if true, the actions will display a menu when plugged
      * into a toolbar.
+     * @param actionCollection the parent collection
      */
     KCommandHistory(KActionCollection *actionCollection, bool withMenus = true);
 
@@ -187,7 +190,7 @@ public:
      * Adds a command to the history. Call this for each @p command you create.
      * Unless you set @p execute to false, this will also execute the command.
      * This means, most of the application's code will look like
-     *    MyCommand * cmd = new MyCommand(i18n("The name"), parameters);
+     *    MyCommand * cmd = new MyCommand(i18n("Capitalized Name"), parameters);
      *    m_historyCommand.addCommand( cmd );
      */
     void addCommand(KCommand *command, bool execute=true);
@@ -232,7 +235,7 @@ public slots:
      * Remembers when you saved the document.
      * Call this right after saving the document. As soon as
      * the history reaches the current index again (via some
-     * undo/redo operations) it will emit @ref documentRestored
+     * undo/redo operations) it will emit documentRestored
      * If you implemented undo/redo properly the document is
      * the same you saved before.
      */
@@ -252,8 +255,8 @@ signals:
      */
     void commandExecuted();
     /**
-     * Emitted everytime we reach the index where you
-     * saved the document for the last time. See @ref documentSaved
+     * Emitted every time we reach the index where you
+     * saved the document for the last time. See documentSaved
      */
     void documentRestored();
 
