@@ -21,6 +21,7 @@
 #define KDATEPICKER_H
 #include <qdatetime.h>
 #include <qframe.h>
+#include <kdemacros.h>
 
 class QLineEdit;
 class QToolButton;
@@ -32,7 +33,7 @@ class KDateTable;
  *
  *     Different from the
  *     previous versions, it now emits two types of signals, either
- *     @ref dateSelected() or @ref dateEntered() (see documentation for both
+ * dateSelected() or dateEntered() (see documentation for both
  *     signals).
  *
  *     A line edit has been added in the newer versions to allow the user
@@ -41,7 +42,7 @@ class KDateTable;
  *
  * \image html kdatepicker.png "KDE Date Widget"
  *
- *     @version $Id: kdatepicker.h,v 1.30 2002/09/28 15:16:22 tjansen Exp $
+ *     @version $Id: kdatepicker.h,v 1.39 2003/10/17 22:25:17 danimo Exp $
  *     @author Tim Gilman, Mirko Boehm
  *
  * @short A date selection widget.
@@ -51,7 +52,8 @@ class KDatePicker: public QFrame
   Q_OBJECT
   Q_PROPERTY( QDate date READ date WRITE setDate)
   Q_PROPERTY( bool closeButton READ hasCloseButton WRITE setCloseButton )
-
+  Q_PROPERTY( int fontSize READ fontSize WRITE setFontSize )
+  
 public:
   /** The usual constructor.  The given date will be displayed
    * initially.
@@ -101,7 +103,7 @@ public:
    * Returns the selected date.
    * @deprecated
    **/
-  const QDate& getDate() const;
+  const QDate& getDate() const KDE_DEPRECATED;
 
   /**
    * @returns the selected date.
@@ -113,6 +115,13 @@ public:
    **/
   void setEnabled(bool);
 
+  /**
+   * @returns the KDateTable widget child of this KDatePicker
+   * widget.
+   * @since 3.2
+   */
+  KDateTable *dateTable() const { return table; };
+  
   /**
    * Sets the font size of the widgets elements.
    **/
@@ -129,14 +138,14 @@ public:
    * close-button will cause the KDatePicker's topLevelWidget()'s close()
    * method being called. This is mostly useful for toplevel datepickers
    * without a window manager decoration.
-   * @see #hasCloseButton
+   * @see hasCloseButton
    * @since 3.1
    */
   void setCloseButton( bool enable );
 
   /**
    * @returns true if a KDatePicker shows a close-button.
-   * @see #setCloseButton
+   * @see setCloseButton
    * @since 3.1
    */
   bool hasCloseButton() const;
@@ -175,12 +184,35 @@ protected slots:
   void monthBackwardClicked();
   void yearForwardClicked();
   void yearBackwardClicked();
-  /// @since 3.1
-  void selectWeekClicked();
+  /**
+   * @since 3.1
+   * @deprecated in 3.2
+   */
+  void selectWeekClicked() KDE_DEPRECATED;
+  /**
+   * @since 3.1
+   */
   void selectMonthClicked();
+  /**
+   * @since 3.1
+   */
   void selectYearClicked();
+  /**
+   * @since 3.1
+   */
   void lineEnterPressed();
+  /**
+   * @since 3.2
+   */
+  void todayButtonClicked();
+  /**
+   * @since 3.2
+   */
+  void weekSelected(int);
+
 signals:
+  // ### KDE 4.0 Make all QDate parameters const references
+
   /** This signal is emitted each time the selected date is changed.
    *  Usually, this does not mean that the date has been entered,
    *  since the date also changes, for example, when another month is
@@ -213,10 +245,9 @@ protected:
   virtual void virtual_hook( int id, void* data );
 private:
   void init( const QDate &dt );
+  void fillWeeksCombo(const QDate &date);
   class KDatePickerPrivate;
   KDatePickerPrivate *d;
-  // calculate ISO 8601 week number
-  int weekOfYear(QDate);
 };
 
 #endif //  KDATEPICKER_H
