@@ -32,21 +32,53 @@
 // When a date is selected by the user, it emits a signal:
 //      dateSelected(QDate)
 
+#include <kconfig.h>
 #include <kglobal.h>
 #include <kglobalsettings.h>
 #include <kapplication.h>
 #include <klocale.h>
 #include <kdebug.h>
 #include <knotifyclient.h>
+#include <kcalendarsystem.h>
 #include "kdatepicker.h"
 #include "kdatetbl.h"
+#include "kpopupmenu.h"
 #include <qdatetime.h>
 #include <qstring.h>
 #include <qpen.h>
 #include <qpainter.h>
 #include <qdialog.h>
+#include <qdict.h>
 #include <assert.h>
 
+
+class KDateTable::KDateTablePrivate
+{
+public:
+   KDateTablePrivate()
+   {
+      popupMenuEnabled=false;
+      useCustomColors=false;
+   }
+
+   ~KDateTablePrivate()
+   {
+   }
+
+   bool popupMenuEnabled;
+   bool useCustomColors;
+
+   struct DatePaintingMode
+   {
+     QColor fgColor;
+     QColor bgColor;
+     BackgroundMode bgMode;
+   };
+   QDict <DatePaintingMode> customPaintingModes;
+
+};
+
+
 KDateValidator::KDateValidator(QWidget* parent, const char* name)
     : QValidator(parent, name)
 {
@@ -81,6 +113,7 @@ KDateValidator::fixup( QString& ) const
 KDateTable::KDateTable(QWidget *parent, QDate date_, const char* name, WFlags f)
   : QGridView(parent, name, f)
 {
+  d = new KDateTablePrivate;
   setFontSize(10);
   if(!date_.isValid())
     {
@@ -96,32 +129,63 @@ KDateTable::KDateTable(QWidget *parent, QDate date_, const char* name, WFlags f)
   setDate(date_); // this initializes firstday, numdays, numDaysPrevMonth
 }
 
+KDateTable::~KDateTable()
+{
+  delete d;
+}
+
+int KDateTable::posFromDate( const QDate &dt )
+{
+  const KCalendarSystem * calendar = KGlobal::locale()->calendar();
+  const int firstWeekDay = KGlobal::locale()->weekStartDay();
+  int pos = calendar->day( dt );
+  int offset = (firstday - firstWeekDay + 7) % 7;
+  // make sure at least one day of the previous month is visible.
+  // adjust this <1 if more days should be forced visible:
+  if ( offset < 1 ) offset += 7;
+  return pos + offset;
+}
+
+QDate KDateTable::dateFromPos( int pos )
+{
+  QDate pCellDate;
+  const KCalendarSystem * calendar = KGlobal::locale()->calendar();
+  calendar->setYMD(pCellDate, calendar->year(date), calendar->month(date), 1);
+
+  int firstWeekDay = KGlobal::locale()->weekStartDay();
+  int offset = (firstday - firstWeekDay + 7) % 7;
+  // make sure at least one day of the previous month is visible.
+  // adjust this <1 if more days should be forced visible:
+  if ( offset < 1 ) offset += 7;
+  pCellDate = calendar->addDays( pCellDate, pos - offset );
+  return pCellDate;
+}
+
 void
 KDateTable::paintCell(QPainter *painter, int row, int col)
 {
+  const KCalendarSystem * calendar = KGlobal::locale()->calendar();
+
   QRect rect;
   QString text;
   QPen pen;
   int w=cellWidth();
   int h=cellHeight();
-  int pos;
   QBrush brushBlue(KGlobalSettings::activeTitleColor());
   QBrush brushLightblue(KGlobalSettings::baseColor());
   QFont font=KGlobalSettings::generalFont();
   // -----
-  font.setPointSize(fontsize);
-  int firstWeekDay = KGlobal::locale()->weekStartDay();
+
   if(row==0)
     { // we are drawing the headline
       font.setBold(true);
       painter->setFont(font);
       bool normalday = true;
-      QString daystr;
-      if ( col+firstWeekDay < 8 )
-          daystr = KGlobal::locale()->weekDayName(col+firstWeekDay, true);
-      else
-          daystr = KGlobal::locale()->weekDayName(col+firstWeekDay-7, true);
-      if ( daystr==i18n("Sunday", "Sun") || daystr==i18n("Saturday", "Sat") )
+      int firstWeekDay = KGlobal::locale()->weekStartDay();
+      int daynum = ( col+firstWeekDay < 8 ) ? col+firstWeekDay :
+                                              col+firstWeekDay-7;
+      if ( daynum == calendar->weekDayOfPray() ||
+         ( daynum == 6 && calendar->calendarName() == "gregorian" ) )
           normalday=false;
 
       if (!normalday)
@@ -137,36 +201,59 @@ KDateTable::paintCell(QPainter *painter, int row, int col)
           painter->setPen(KGlobalSettings::activeTextColor());
         }
       painter->drawText(0, 0, w, h-1, AlignCenter,
-                        daystr, -1, &rect);
+                        calendar->weekDayName(daynum, true), -1, &rect);
       painter->setPen(KGlobalSettings::textColor());
       painter->moveTo(0, h-1);
       painter->lineTo(w-1, h-1);
       // ----- draw the weekday:
     } else {
+      bool paintRect=true;
       painter->setFont(font);
-      pos=7*(row-1)+col;
-      if ( firstWeekDay < 4 )
-          pos += firstWeekDay;
-      else
-          pos += firstWeekDay - 7;
-      if(pos<firstday || (firstday+numdays<=pos))
+      int pos=7*(row-1)+col;
+
+      QDate pCellDate = dateFromPos( pos );
+      // First day of month
+      text = calendar->dayString(pCellDate, true);
+      if( calendar->month(pCellDate) != calendar->month(date) )
         { // we are either
           //  painting a day of the previous month or
           //  painting a day of the following month
-          if(pos<firstday)
-            { // previous month
-              text.setNum(numDaysPrevMonth+pos-firstday+1);
-            } else { // following month
-              text.setNum(pos-firstday-numdays+1);
-            }
           painter->setPen(gray);
         } else { // paint a day of the current month
-          text.setNum(pos-firstday+1);
+          if ( d->useCustomColors )
+          {
+            KDateTablePrivate::DatePaintingMode *mode=d->customPaintingModes[pCellDate.toString()];
+            if (mode)
+            {
+              if (mode->bgMode != NoBgMode)
+              {
+                QBrush oldbrush=painter->brush();
+                painter->setBrush( mode->bgColor );
+                switch(mode->bgMode)
+                {
+                  case(CircleMode) : painter->drawEllipse(0,0,w,h);break;
+                  case(RectangleMode) : painter->drawRect(0,0,w,h);break;
+                  case(NoBgMode) : // Should never be here, but just to get one
+                                   // less warning when compiling
+                  default: break;
+                }
+                painter->setBrush( oldbrush );
+                paintRect=false;
+              }
+              painter->setPen( mode->fgColor );
+            } else
+              painter->setPen(KGlobalSettings::textColor());
+          } else //if ( firstWeekDay < 4 ) // <- this doesn' make sense at all!
           painter->setPen(KGlobalSettings::textColor());
         }
 
       pen=painter->pen();
-      if(firstday+date.day()-1==pos)
+      int firstWeekDay=KGlobal::locale()->weekStartDay();
+      int offset=firstday-firstWeekDay;
+      if(offset<1)
+        offset+=7;
+      int d = calendar->day(date);
+      if( (offset+d) == (pos+1) )
         {
           if(hasFocus())
             { // draw the currently selected date
@@ -183,15 +270,12 @@ KDateTable::paintCell(QPainter *painter, int row, int col)
           painter->setPen(KGlobalSettings::baseColor());
         }
 
-      QDate cur_date = QDate::currentDate();
-      if ( (date.year()  == cur_date.year()) &&
-           (date.month() == cur_date.month()) &&
-           (firstday+cur_date.day()-1 == pos) )
+      if ( pCellDate == QDate::currentDate() )
       {
          painter->setPen(KGlobalSettings::textColor());
       }
 
-      painter->drawRect(0, 0, w, h);
+      if ( paintRect ) painter->drawRect(0, 0, w, h);
       painter->setPen(pen);
       painter->drawText(0, 0, w, h, AlignCenter, text, -1, &rect);
     }
@@ -202,51 +286,57 @@ KDateTable::paintCell(QPainter *painter, int row, int col)
 void
 KDateTable::keyPressEvent( QKeyEvent *e )
 {
-    if ( e->key() == Qt::Key_Prior ) {
-        setDate(date.addMonths(-1));
-        return;
-    }
-    if ( e->key() == Qt::Key_Next ) {
-        setDate(date.addMonths(1));
-        return;
-    }
+    const KCalendarSystem * calendar = KGlobal::locale()->calendar();
+    QDate temp = date;
 
-    if ( e->key() == Qt::Key_Up ) {
-        if ( date.day() > 7 ) {
+    switch( e->key() ) {
+    case Key_Prior:
+        temp = calendar->addMonths( date, -1 );
+        setDate(temp);
+        return;
+    case Key_Next:
+        temp = calendar->addMonths( date, 1 );
+        setDate(temp);
+        return;
+    case Key_Up:
+        if ( calendar->day(date) > 7 ) {
             setDate(date.addDays(-7));
             return;
         }
-    }
-    if ( e->key() == Qt::Key_Down ) {
-        if ( date.day() <= date.daysInMonth()-7 ) {
+        break;
+    case Key_Down:
+        if ( calendar->day(date) <= calendar->daysInMonth(date)-7 ) {
             setDate(date.addDays(7));
             return;
         }
-    }
-    if ( e->key() == Qt::Key_Left ) {
-        if ( date.day() > 1 ) {
+        break;
+    case Key_Left:
+        if ( calendar->day(date) > 1 ) {
             setDate(date.addDays(-1));
             return;
         }
-    }
-    if ( e->key() == Qt::Key_Right ) {
-        if ( date.day() < date.daysInMonth() ) {
+        break;
+    case Key_Right:
+        if ( calendar->day(date) < calendar->daysInMonth(date) ) {
             setDate(date.addDays(1));
             return;
         }
-    }
-
-    if ( e->key() == Qt::Key_Minus ) {
+        break;
+    case Key_Minus:
         setDate(date.addDays(-1));
         return;
-    }
-    if ( e->key() == Qt::Key_Plus ) {
+    case Key_Plus:
         setDate(date.addDays(1));
         return;
-    }
-    if ( e->key() == Qt::Key_N ) {
+    case Key_N:
         setDate(QDate::currentDate());
         return;
+    case Key_Return:
+    case Key_Enter:
+        emit tableClicked();
+        return;
+    default:
+        break;
     }
 
     KNotifyClient::beep();
@@ -274,7 +364,8 @@ KDateTable::setFontSize(int size)
   maxCell.setHeight(0);
   for(count=0; count<7; ++count)
     {
-      rect=metrics.boundingRect(KGlobal::locale()->weekDayName(count+1, true));
+      rect=metrics.boundingRect(KGlobal::locale()->calendar()
+                                ->weekDayName(count+1, true));
       maxCell.setWidth(QMAX(maxCell.width(), rect.width()));
       maxCell.setHeight(QMAX(maxCell.height(), rect.height()));
     }
@@ -294,6 +385,7 @@ KDateTable::wheelEvent ( QWheelEvent * e )
 void
 KDateTable::contentsMousePressEvent(QMouseEvent *e)
 {
+
   if(e->type()!=QEvent::MouseButtonPress)
     { // the KDatePicker only reacts on mouse press events:
       return;
@@ -304,8 +396,6 @@ KDateTable::contentsMousePressEvent(QMouseEvent *e)
       return;
     }
 
-  //int dayoff = KGlobal::locale()->weekStartsMonday() ? 1 : 0;
-  int dayoff = KGlobal::locale()->weekStartDay();
   // -----
   int row, col, pos, temp;
   QPoint mouseCoord;
@@ -318,36 +408,33 @@ KDateTable::contentsMousePressEvent(QMouseEvent *e)
       return;
     }
 
-  // Rows and columns are zero indexed.  The (row - 1) below is to avoid counting 
-  // the row with the days of the week in the calculation.  We however want pos
-  // to be "1 indexed", hence the "+ 1" at the end of the sum.
+  // Rows and columns are zero indexed.  The (row - 1) below is to avoid counting
+  // the row with the days of the week in the calculation.
 
-  pos = (7 * (row - 1)) + col + 1;
+  // old selected date:
+  temp = posFromDate( date );
+  // new position and date
+  pos = (7 * (row - 1)) + col; 
+  QDate clickedDate = dateFromPos( pos );
 
-  // This gets pretty nasty below.  firstday is a locale independant index for 
-  // the first day of the week.  dayoff is the day of the week that the week
-  // starts with for the selected locale.  Monday = 1 .. Sunday = 7
-  // Strangely, in some cases dayoff is in fact set to 8, hence all of the
-  // "dayoff % 7" sections below.
-  
-  if(pos + dayoff % 7 <= firstday)
-    { // this day is in the previous month
-      setDate(date.addDays(-1 * (date.day() + firstday - pos - dayoff % 7)));
-      return;
-    }
-  if(firstday + numdays < pos + dayoff % 7)
-    { // this date is in the next month
-      setDate(date.addDays(pos - firstday - date.day() + dayoff % 7));
-      return;
-    }
-  temp = firstday + date.day() - dayoff % 7 - 1;
+  // set the new date. If it is in the previous or next month, the month will
+  // automatically be changed, no need to do that manually...
+  setDate( clickedDate );
 
-  setDate(QDate(date.year(), date.month(), pos - firstday + dayoff % 7));
+  // call updateCell on the old and new selection. If setDate switched to a different 
+  // month, these cells will be painted twice, but that's no problem.
+  updateCell( temp/7+1, temp%7 );
+  updateCell( row, col );
 
-  updateCell(temp/7+1, temp%7); // Update the previously selected cell
-  updateCell(row, col); // Update the selected cell
-  // assert(QDate(date.year(), date.month(), pos-firstday+dayoff).isValid());
-  emit(tableClicked());
+  emit tableClicked();
+
+  if (  e->button() == Qt::RightButton && d->popupMenuEnabled )
+  {
+        KPopupMenu *menu = new KPopupMenu();
+        menu->insertTitle( KGlobal::locale()->formatDate(clickedDate) );
+        emit aboutToShowContextMenu( menu, clickedDate );
+        menu->popup(e->globalPos());
+  }
 }
 
 bool
@@ -363,25 +450,25 @@ KDateTable::setDate(const QDate& date_)
     }
   if(date!=date_)
     {
+      emit(dateChanged(date, date_));
       date=date_;
+      emit(dateChanged(date));
       changed=true;
     }
-  temp.setYMD(date.year(), date.month(), 1);
+  const KCalendarSystem * calendar = KGlobal::locale()->calendar();
+
+  calendar->setYMD(temp, calendar->year(date), calendar->month(date), 1);
+  //temp.setYMD(date.year(), date.month(), 1);
+  //kdDebug() << "firstDayInWeek: " << temp.toString() << endl;
   firstday=temp.dayOfWeek();
-  if(firstday==1) firstday=8;
-  numdays=date.daysInMonth();
-  if(date.month()==1)
-    { // set to december of previous year
-      temp.setYMD(date.year()-1, 12, 1);
-    } else { // set to previous month
-      temp.setYMD(date.year(), date.month()-1, 1);
-    }
-  numDaysPrevMonth=temp.daysInMonth();
+  numdays=calendar->daysInMonth(date);
+
+  temp = calendar->addMonths(temp, -1);
+  numDaysPrevMonth=calendar->daysInMonth(temp);
   if(changed)
     {
       repaintContents(false);
     }
-  emit(dateChanged(date));
   return true;
 }
 
@@ -417,8 +504,41 @@ KDateTable::sizeHint() const
     }
 }
 
+void KDateTable::setPopupMenuEnabled( bool enable )
+{
+   d->popupMenuEnabled=enable;
+}
+
+bool KDateTable::popupMenuEnabled() const
+{
+   return d->popupMenuEnabled;
+}
+
+void KDateTable::setCustomDatePainting(const QDate &date, const QColor &fgColor, BackgroundMode bgMode, const QColor &bgColor)
+{
+    if (!fgColor.isValid())
+    {
+        unsetCustomDatePainting( date );
+        return;
+    }
+
+    KDateTablePrivate::DatePaintingMode *mode=new KDateTablePrivate::DatePaintingMode;
+    mode->bgMode=bgMode;
+    mode->fgColor=fgColor;
+    mode->bgColor=bgColor;
+
+    d->customPaintingModes.replace( date.toString(), mode );
+    d->useCustomColors=true;
+    update();
+}
+
+void KDateTable::unsetCustomDatePainting( const QDate &date )
+{
+    d->customPaintingModes.remove( date.toString() );
+}
+
 KDateInternalWeekSelector::KDateInternalWeekSelector
-(int fontsize, QWidget* parent, const char* name)
+(QWidget* parent, const char* name)
   : QLineEdit(parent, name),
     val(new QIntValidator(this)),
     result(0)
@@ -426,10 +546,8 @@ KDateInternalWeekSelector::KDateInternalWeekSelector
   QFont font;
   // -----
   font=KGlobalSettings::generalFont();
-  font.setPointSize(fontsize);
   setFont(font);
   setFrameStyle(QFrame::NoFrame);
-  val->setRange(1, 53);
   setValidator(val);
   connect(this, SIGNAL(returnPressed()), SLOT(weekEnteredSlot()));
 }
@@ -465,8 +583,31 @@ KDateInternalWeekSelector::setWeek(int week)
   setText(temp);
 }
 
+void
+KDateInternalWeekSelector::setMaxWeek(int max)
+{
+  val->setRange(1, max);
+}
+
+// ### CFM To avoid binary incompatibility.
+//     In future releases, remove this and replace by  a QDate
+//     private member, needed in KDateInternalMonthPicker::paintCell
+class KDateInternalMonthPicker::KDateInternalMonthPrivate {
+public:
+        KDateInternalMonthPrivate (int y, int m, int d)
+        : year(y), month(m), day(d)
+        {};
+        int year;
+        int month;
+        int day;
+};
+
+KDateInternalMonthPicker::~KDateInternalMonthPicker() {
+   delete d;
+}
+
 KDateInternalMonthPicker::KDateInternalMonthPicker
-(int fontsize, QWidget* parent, const char* name)
+(const QDate & date, QWidget* parent, const char* name)
   : QGridView(parent, name),
     result(0) // invalid
 {
@@ -476,26 +617,29 @@ KDateInternalMonthPicker::KDateInternalMonthPicker
   activeCol = -1;
   activeRow = -1;
   font=KGlobalSettings::generalFont();
-  font.setPointSize(fontsize);
   setFont(font);
   setHScrollBarMode(AlwaysOff);
   setVScrollBarMode(AlwaysOff);
   setFrameStyle(QFrame::NoFrame);
-  setNumRows(4);
   setNumCols(3);
+  d = new KDateInternalMonthPrivate(date.year(), date.month(), date.day());
+  // For monthsInYear != 12
+  setNumRows( (KGlobal::locale()->calendar()->monthsInYear(date) + 2) / 3);
   // enable to find drawing failures:
   // setTableFlags(Tbl_clipCellPainting);
   viewport()->setEraseColor(KGlobalSettings::baseColor()); // for consistency with the datepicker
   // ----- find the preferred size
-  //       (this is slow, possibly, but unfortunatly it is needed here):
+  //       (this is slow, possibly, but unfortunately it is needed here):
   QFontMetrics metrics(font);
-  for(int i=1; i <= 12; ++i)
+  for(int i = 1; ; ++i)
     {
-      rect=metrics.boundingRect(KGlobal::locale()->monthName(i, false));
+      QString str = KGlobal::locale()->calendar()->monthName(i,
+         KGlobal::locale()->calendar()->year(date), false);
+      if (str.isNull()) break;
+      rect=metrics.boundingRect(str);
       if(max.width()<rect.width()) max.setWidth(rect.width());
       if(max.height()<rect.height()) max.setHeight(rect.height());
     }
-
 }
 
 QSize
@@ -520,8 +664,8 @@ KDateInternalMonthPicker::setupPainter(QPainter *p)
 void
 KDateInternalMonthPicker::viewportResizeEvent(QResizeEvent*)
 {
-  setCellWidth(width()/3);
-  setCellHeight(height()/4);
+  setCellWidth(width() / numCols());
+  setCellHeight(height() / numRows());
 }
 
 void
@@ -531,7 +675,9 @@ KDateInternalMonthPicker::paintCell(QPainter* painter, int row, int col)
   QString text;
   // ----- find the number of the cell:
   index=3*row+col+1;
-  text=KGlobal::locale()->monthName(index, false);
+  text=KGlobal::locale()->calendar()->monthName(index,
+    KGlobal::locale()->calendar()->year(QDate(d->year, d->month,
+    d->day)), false);
   painter->drawText(0, 0, cellWidth(), cellHeight(), AlignCenter, text);
   if ( activeCol == col && activeRow == row )
       painter->drawRect( 0, 0, cellWidth(), cellHeight() );
@@ -622,6 +768,7 @@ KDateInternalMonthPicker::contentsMouseReleaseEvent(QMouseEvent *e)
     { // the user clicked on the frame of the table
       emit(closeMe(0));
     }
+
   pos=3*row+col+1;
   result=pos;
   emit(closeMe(1));
@@ -630,7 +777,7 @@ KDateInternalMonthPicker::contentsMouseReleaseEvent(QMouseEvent *e)
 
 
 KDateInternalYearSelector::KDateInternalYearSelector
-(int fontsize, QWidget* parent, const char* name)
+(QWidget* parent, const char* name)
   : QLineEdit(parent, name),
     val(new QIntValidator(this)),
     result(0)
@@ -638,7 +785,6 @@ KDateInternalYearSelector::KDateInternalYearSelector
   QFont font;
   // -----
   font=KGlobalSettings::generalFont();
-  font.setPointSize(fontsize);
   setFont(font);
   setFrameStyle(QFrame::NoFrame);
   // we have to respect the limits of QDate here, I fear:
@@ -660,7 +806,8 @@ KDateInternalYearSelector::yearEnteredSlot()
       KNotifyClient::beep();
       return;
     }
-  date.setYMD(year, 1, 1);
+  //date.setYMD(year, 1, 1);
+  KGlobal::locale()->calendar()->setYMD(date, year, 1, 1);
   if(!date.isValid())
     {
       KNotifyClient::beep();
@@ -735,7 +882,8 @@ void
 KPopupFrame::popup(const QPoint &pos)
 {
   // Make sure the whole popup is visible.
-  QRect d = QApplication::desktop()->screenGeometry(QApplication::desktop()->screenNumber(pos));
+  QRect d = KGlobalSettings::desktopGeometry(pos);
+
   int x = pos.x();
   int y = pos.y();
   int w = width();
