@@ -24,10 +24,13 @@
 #include <qgridview.h>
 #include <qlineedit.h>
 #include <qdatetime.h>
+#include <qcolor.h>
+
+class KPopupMenu;
 
 /** Week selection widget.
 * @internal
-* @version $Id: kdatetbl.h,v 1.31 2002/07/25 13:26:37 lukas Exp $
+* @version $Id: kdatetbl.h,v 1.43 2003/10/17 22:25:17 danimo Exp $
 * @author Stephan Binner
 */
 class KDateInternalWeekSelector : public QLineEdit
@@ -38,12 +41,11 @@ protected:
   int result;
 public slots:
   void weekEnteredSlot();
+  void setMaxWeek(int max);
 signals:
   void closeMe(int);
 public:
-  KDateInternalWeekSelector(int fontsize,
-			    QWidget* parent=0,
-			    const char* name=0);
+  KDateInternalWeekSelector( QWidget* parent=0, const char* name=0);
   int getWeek();
   void setWeek(int week);
 
@@ -55,7 +57,7 @@ private:
 /**
 * A table containing month names. It is used to pick a month directly.
 * @internal
-* @version $Id: kdatetbl.h,v 1.31 2002/07/25 13:26:37 lukas Exp $
+* @version $Id: kdatetbl.h,v 1.43 2003/10/17 22:25:17 danimo Exp $
 * @author Tim Gilman, Mirko Boehm
 */
 class KDateInternalMonthPicker : public QGridView
@@ -84,7 +86,11 @@ public:
   /**
    * The constructor.
    */
-  KDateInternalMonthPicker(int fontsize, QWidget* parent, const char* name=0);
+  KDateInternalMonthPicker(const QDate& date, QWidget* parent, const char* name=0);
+  /**
+   * The destructor.
+   */
+  ~KDateInternalMonthPicker();
   /**
    * The size hint.
    */
@@ -124,7 +130,7 @@ private:
 
 /** Year selection widget.
 * @internal
-* @version $Id: kdatetbl.h,v 1.31 2002/07/25 13:26:37 lukas Exp $
+* @version $Id: kdatetbl.h,v 1.43 2003/10/17 22:25:17 danimo Exp $
 * @author Tim Gilman, Mirko Boehm
 */
 class KDateInternalYearSelector : public QLineEdit
@@ -138,21 +144,20 @@ public slots:
 signals:
   void closeMe(int);
 public:
-  KDateInternalYearSelector(int fontsize,
-			    QWidget* parent=0,
-			    const char* name=0);
+  KDateInternalYearSelector( QWidget* parent=0, const char* name=0);
   int getYear();
   void setYear(int year);
 
 private:
   class KDateInternalYearPrivate;
   KDateInternalYearPrivate *d;
+
 };
 
 /**
- * Frame with popup menu behaviour.
+ * Frame with popup menu behavior.
  * @author Tim Gilman, Mirko Boehm
- * @version $Id: kdatetbl.h,v 1.31 2002/07/25 13:26:37 lukas Exp $
+ * @version $Id: kdatetbl.h,v 1.43 2003/10/17 22:25:17 danimo Exp $
  */
 class KPopupFrame : public QFrame
 {
@@ -239,12 +244,15 @@ public:
  * dateSelected(QDate)
  *
  * @internal
- * @version $Id: kdatetbl.h,v 1.31 2002/07/25 13:26:37 lukas Exp $
+ * @version $Id: kdatetbl.h,v 1.43 2003/10/17 22:25:17 danimo Exp $
  * @author Tim Gilman, Mirko Boehm
  */
 class KDateTable : public QGridView
 {
     Q_OBJECT
+    Q_PROPERTY( QDate date READ getDate WRITE setDate )
+    Q_PROPERTY( bool popupMenu READ popupMenuEnabled WRITE setPopupMenuEnabled )
+    
 public:
     /**
      * The constructor.
@@ -252,6 +260,12 @@ public:
     KDateTable(QWidget *parent=0,
 	       QDate date=QDate::currentDate(),
 	       const char* name=0, WFlags f=0);
+	       
+    /**
+     * The destructor.
+     */
+    ~KDateTable();
+    
     /**
      * Returns a recommended size for the widget.
      * To save some time, the size of the largest used cell content is
@@ -268,10 +282,52 @@ public:
      * Select and display this date.
      */
     bool setDate(const QDate&);
+    // ### 4.0 rename to date()
     const QDate& getDate() const;
 
+    /**
+     * Enables a popup menu when right clicking on a date. 
+     * 
+     * When it's enabled, this object emits a aboutToShowContextMenu signal
+     * where you can fill in the menu items.
+     *
+     * @since 3.2
+     */
+    void setPopupMenuEnabled( bool enable );
+
+    /**
+     * Returns if the popup menu is enabled or not
+     */
+    bool popupMenuEnabled() const;
+
+    enum BackgroundMode { NoBgMode=0, RectangleMode, CircleMode };
+
+    /**
+     * Makes a given date be painted with a given foregroundColor, and background
+     * (a rectangle, or a circle/ellipse) in a given color.
+     *
+     * @since 3.2
+     */
+    void setCustomDatePainting( const QDate &date, const QColor &fgColor, BackgroundMode bgMode=NoBgMode, const QColor &bgColor=QColor());
+
+    /**
+     * Unsets the custom painting of a date so that the date is painted as usual.
+     *
+     * @since 3.2
+     */
+    void unsetCustomDatePainting( const QDate &date );
 
 protected:
+    /**
+     * calculate the position of the cell in the matrix for the given date. The result is the 0-based index.
+     */
+    int posFromDate( const QDate &date ); // KDE4: make this virtual, so subclasses can reimplement this and use a different default for the start of the matrix
+    /**
+     * calculate the date that is displayed at a given cell in the matrix. pos is the 
+     * 0-based index in the matrix. Inverse function to posForDate().
+     */
+    QDate dateFromPos( int pos ); // KDE4: make this virtual
+    
     /**
      * Paint a cell.
      */
@@ -288,6 +344,9 @@ protected:
     virtual void keyPressEvent( QKeyEvent *e );
     virtual void focusInEvent( QFocusEvent *e );
     virtual void focusOutEvent( QFocusEvent *e );
+
+    // ### KDE 4.0 make the following private and mark as members
+    
     /**
      * The font size of the displayed text.
      */
@@ -321,12 +380,29 @@ signals:
     /**
      * The selected date changed.
      */
+    // ### KDE 4.0 make parameter a const reference
     void dateChanged(QDate);
+    /**
+     * This function behaves essentially like the one above.
+     * The selected date changed.
+     * @param cur The current date
+     * @param old The date before the date was changed
+     */
+    void dateChanged(const QDate& cur, const QDate& old);
     /**
      * A date has been selected by clicking on the table.
      */
     void tableClicked();
 
+    /**
+     * A popup menu for a given date is about to be shown (as when the user
+     * right clicks on that date and the popup menu is enabled). Connect
+     * the slot where you fill the menu to this signal.
+     *
+     * @since 3.2
+     */
+    void aboutToShowContextMenu( KPopupMenu * menu, const QDate &date);
+
 protected:
   virtual void virtual_hook( int id, void* data );
 private:
