@@ -18,18 +18,24 @@
  *  Boston, MA 02111-1307, USA.
  */
 
+#include <kconfig.h>
+#include <kapplication.h>
+#include <kdialog.h>
+#include <kdebug.h>
+#include <kstaticdeleter.h>
+
 #include <qlayout.h>
 #include <qobjectlist.h>
 #include <qguardedptr.h>
 #include <qlineedit.h>
 #include <qvaluelist.h>
 #include <qtimer.h>
-#include <netwm.h>
+#include <qcursor.h>
 
-#include <kapplication.h>
-#include <kdialog.h>
-#include <kdebug.h>
-#include <kstaticdeleter.h>
+#include "config.h"
+#if defined Q_WS_X11 && ! defined K_WS_QTONLY
+#include <netwm.h> // schroder
+#endif
 
 int KDialog::mMarginSize = 11;
 int KDialog::mSpacingSize = 6;
@@ -72,10 +78,7 @@ void KDialog::keyPressEvent(QKeyEvent *e)
   else
   {
       // accept the dialog when Ctrl-Return is pressed
-      if ( e->state() == ControlButton && 
-           qApp->focusWidget()  && 
-           (qApp->focusWidget()->inherits( "QTextEdit" ) ||
-            qApp->focusWidget()->inherits( "QLineEdit" )) &&
+      if ( e->state() == ControlButton &&
            (e->key() == Key_Return || e->key() == Key_Enter) )
       {
           e->accept();
@@ -122,7 +125,8 @@ void KDialog::setPlainCaption( const QString &caption )
 {
   QDialog::setCaption( caption );
 
-#ifdef Q_WS_X11 //FIXME(E) Implement for Qt/E
+#if defined Q_WS_X11 && ! defined K_WS_QTONLY
+//#ifdef Q_WS_X11 //FIXME(E) Implement for Qt/E
   NETWinInfo info( qt_xdisplay(), winId(), qt_xrootwin(), 0 );
   info.setName( caption.utf8().data() );
 #endif
@@ -167,24 +171,91 @@ void KDialog::resizeLayout( QLayoutItem *lay, int margin, int spacing )
   }
 }
 
+static QRect screenRect( QWidget *w, int screen )
+{
+  QDesktopWidget *desktop = QApplication::desktop();
+  KConfig gc("kdeglobals", false, false);
+  gc.setGroup("Windows");
+  if (desktop->isVirtualDesktop() &&
+      gc.readBoolEntry("XineramaEnabled", true) &&
+      gc.readBoolEntry("XineramaPlacementEnabled", true)) {
+    if ( screen < 0 || screen >= desktop->numScreens() ) {
+      if ( screen == -1 ) {
+        screen = desktop->primaryScreen();
+      } else if ( screen == -3 ) {
+        screen = desktop->screenNumber( QCursor::pos() );
+      } else {
+        screen = desktop->screenNumber( w );
+      }
+    }
+    return desktop->screenGeometry(screen);
+  } else {
+    return desktop->geometry();
+  }
+}
+
 void KDialog::centerOnScreen( QWidget *w, int screen )
 {
   if ( !w )
     return;
+  QRect r = screenRect( w, screen );
 
-  QDesktopWidget *desktop = QApplication::desktop();
-  if ( screen < 0 || screen >= desktop->numScreens() )
-  {
-    screen = desktop->screenNumber( w );
-    if ( screen == -1 )
-      screen = desktop->primaryScreen();
-  }
-
-  QRect r = desktop->screenGeometry( screen );
   w->move( r.center().x() - w->width()/2,
            r.center().y() - w->height()/2 );
 }
 
+bool KDialog::avoidArea( QWidget *w, const QRect& area, int screen )
+{
+  if ( !w )
+    return false;
+  QRect fg = w->frameGeometry();
+  if ( !fg.intersects( area ) )
+    return true; // nothing to do.
+
+  QRect scr = screenRect( w, screen );
+  QRect avoid( area ); // let's add some margin
+  avoid.moveBy( -5, -5 );
+  avoid.rRight() += 10;
+  avoid.rBottom() += 10;
+
+  if ( QMAX( fg.top(), avoid.top() ) <= QMIN( fg.bottom(), avoid.bottom() ) )
+  {
+    // We need to move the widget up or down
+    int spaceAbove = QMAX(0, avoid.top() - scr.top());
+    int spaceBelow = QMAX(0, scr.bottom() - avoid.bottom());
+    if ( spaceAbove > spaceBelow ) // where's the biggest side?
+      if ( fg.height() <= spaceAbove ) // big enough?
+        fg.setY( avoid.top() - fg.height() );
+      else
+        return false;
+    else
+      if ( fg.height() <= spaceBelow ) // big enough?
+        fg.setY( avoid.bottom() );
+      else
+        return false;
+  }
+
+  if ( QMAX( fg.left(), avoid.left() ) <= QMIN( fg.right(), avoid.right() ) )
+  {
+    // We need to move the widget left or right
+    int spaceLeft = QMAX(0, avoid.left() - scr.left());
+    int spaceRight = QMAX(0, scr.right() - avoid.right());
+    if ( spaceLeft > spaceRight ) // where's the biggest side?
+      if ( fg.width() <= spaceLeft ) // big enough?
+        fg.setX( avoid.left() - fg.width() );
+      else
+        return false;
+    else
+      if ( fg.width() <= spaceRight ) // big enough?
+        fg.setX( avoid.right() );
+      else
+        return false;
+  }
+  //kdDebug() << "Moving window to " << fg.x() << "," << fg.y() << endl;
+  w->move(fg.x(), fg.y());
+  return true;
+}
+
 class KDialogQueuePrivate
 {
 public:
@@ -199,7 +270,7 @@ KDialogQueue *KDialogQueue::_self=0;
 KDialogQueue* KDialogQueue::self()
 {
    if (!_self)
-      _self = ksdkdq.setObject(new KDialogQueue);
+      _self = ksdkdq.setObject(_self, new KDialogQueue);
    return _self;
 }
 
