@@ -31,6 +31,7 @@
 #include <qvbox.h>
 #include <qwhatsthis.h>
 #include <qtimer.h>
+#include <qfocusdata.h>
 
 #include <kapplication.h>
 #include <klocale.h>
@@ -56,6 +57,8 @@ template class QPtrList<KDialogBaseButton>;
 /**
  * @internal
  */
+namespace
+{
 struct SButton : public Qt
 {
   SButton()
@@ -76,6 +79,7 @@ struct SButton : public Qt
   int style;
   QPtrList<KDialogBaseButton> list;
 };
+}// namespace
 
 class KDialogBase::KDialogBasePrivate {
 public:
@@ -105,7 +109,7 @@ KDialogBase::KDialogBase( QWidget *parent, const char *name, bool modal,
   setCaption( caption );
 
   makeRelay();
-  connect( this, SIGNAL(layoutHintChanged()), this, SLOT(updateGeometry()) );
+  connect( this, SIGNAL(layoutHintChanged()), SLOT(updateGeometry()) );
 
   enableButtonSeparator( separator );
   makeButtonBox( buttonMask, defaultButton, user1, user2, user3 );
@@ -128,7 +132,37 @@ KDialogBase::KDialogBase( int dialogFace, const QString &caption,
   setCaption( caption );
 
   makeRelay();
-  connect( this, SIGNAL(layoutHintChanged()), this, SLOT(updateGeometry()) );
+  connect( this, SIGNAL(layoutHintChanged()), SLOT(updateGeometry()) );
+
+  mJanus = new KJanusWidget( this, "janus", dialogFace );
+  connect(mJanus, SIGNAL(aboutToShowPage(QWidget *)),
+          this, SIGNAL(aboutToShowPage(QWidget *)));
+
+  if( mJanus == 0 || mJanus->isValid() == false ) { return; }
+
+  enableButtonSeparator( separator );
+  makeButtonBox( buttonMask, defaultButton, user1, user2, user3 );
+
+  mIsActivated = true;
+  setupLayout();
+}
+
+KDialogBase::KDialogBase(  KDialogBase::DialogType dialogFace, WFlags f, QWidget *parent, const char *name,
+		          bool modal,
+			  const QString &caption, int buttonMask,
+			  ButtonCode defaultButton, bool separator,
+			  const KGuiItem &user1, const KGuiItem &user2,
+			  const KGuiItem &user3 )
+  :KDialog( parent, name, modal, f ),
+   mTopLayout(0), mMainWidget(0), mUrlHelp(0), mJanus(0), mActionSep(0),
+   mIsActivated(false), mShowTile(false), mMessageBoxMode(false),
+   mButtonOrientation(Horizontal)
+{
+  d = new KDialogBasePrivate;
+  setCaption( caption );
+
+  makeRelay();
+  connect( this, SIGNAL(layoutHintChanged()), SLOT(updateGeometry()) );
 
   mJanus = new KJanusWidget( this, "janus", dialogFace );
   connect(mJanus, SIGNAL(aboutToShowPage(QWidget *)),
@@ -158,7 +192,7 @@ KDialogBase::KDialogBase( const QString &caption, int buttonMask,
   setCaption( caption );
 
   makeRelay();
-  connect( this, SIGNAL(layoutHintChanged()), this, SLOT(updateGeometry()) );
+  connect( this, SIGNAL(layoutHintChanged()), SLOT(updateGeometry()) );
 
   enableButtonSeparator( separator );
 
@@ -168,8 +202,8 @@ KDialogBase::KDialogBase( const QString &caption, int buttonMask,
                  no.text().isEmpty()  ? KStdGuiItem::no()  : no,
                  yes.text().isEmpty() ? KStdGuiItem::yes() : yes );
 
-  setButtonCancelText( cancel.text().isEmpty() ?
-                       KStdGuiItem::cancel().text() : cancel.text() );
+  setButtonCancel( cancel.text().isEmpty() ?
+                       KStdGuiItem::cancel() : cancel );
 
   mIsActivated = true;
   setupLayout();
@@ -340,14 +374,14 @@ void KDialogBase::makeRelay()
 {
   if( mTile != 0 )
   {
-    connect( mTile, SIGNAL(pixmapChanged()), this, SLOT(updateBackground()) );
+    connect( mTile, SIGNAL(pixmapChanged()), SLOT(updateBackground()) );
     return;
   }
 
   mTile = new KDialogBaseTile;
   if( mTile != 0 )
   {
-    connect( mTile, SIGNAL(pixmapChanged()), this, SLOT(updateBackground()) );
+    connect( mTile, SIGNAL(pixmapChanged()), SLOT(updateBackground()) );
     connect( qApp, SIGNAL(aboutToQuit()), mTile, SLOT(cleanup()) );
   }
 }
@@ -541,31 +575,31 @@ void KDialogBase::makeButtonBox( int buttonMask, ButtonCode defaultButton,
   {
     KPushButton *pb = d->mButton.append( Help, KStdGuiItem::help() );
 
-    connect( pb, SIGNAL(clicked()), this, SLOT(slotHelp()) );
+    connect( pb, SIGNAL(clicked()), SLOT(slotHelp()) );
   }
   if( d->mButton.mask & Default )
   {
     KPushButton *pb = d->mButton.append( Default, KStdGuiItem::defaults() );
 
-    connect( pb, SIGNAL(clicked()), this, SLOT(slotDefault()) );
+    connect( pb, SIGNAL(clicked()), SLOT(slotDefault()) );
   }
   if( d->mButton.mask & Details )
   {
     KPushButton *pb = d->mButton.append( Details, QString::null );
-    connect( pb, SIGNAL(clicked()), this, SLOT(slotDetails()) );
+    connect( pb, SIGNAL(clicked()), SLOT(slotDetails()) );
     setDetails(false);
   }
   if( d->mButton.mask & User3 )
   {
     KPushButton *pb = d->mButton.append( User3, user3 );
-    connect( pb, SIGNAL(clicked()), this, SLOT(slotUser3()) );
+    connect( pb, SIGNAL(clicked()), SLOT(slotUser3()) );
   }
   if( d->mButton.mask & User2 )
   {
     KPushButton *pb = d->mButton.append( User2, user2 );
     if( mMessageBoxMode == true )
     {
-      connect( pb, SIGNAL(clicked()), this, SLOT(slotYes()) );
+      connect( pb, SIGNAL(clicked()), SLOT(slotYes()) );
     }
     else
     {
@@ -581,41 +615,41 @@ void KDialogBase::makeButtonBox( int buttonMask, ButtonCode defaultButton,
     }
     else
     {
-      connect( pb, SIGNAL(clicked()), this, SLOT(slotUser1()) );
+      connect( pb, SIGNAL(clicked()), SLOT(slotUser1()) );
     }
   }
   if( d->mButton.mask & Ok )
   {
     KPushButton *pb = d->mButton.append( Ok, KStdGuiItem::ok() );
-    connect( pb, SIGNAL(clicked()), this, SLOT(slotOk()) );
+    connect( pb, SIGNAL(clicked()), SLOT(slotOk()) );
   }
   if( d->mButton.mask & Apply )
   {
     KPushButton *pb = d->mButton.append( Apply, KStdGuiItem::apply() );
-    connect( pb, SIGNAL(clicked()), this, SLOT(slotApply()) );
-    connect( pb, SIGNAL(clicked()), this, SLOT(applyPressed()) );
+    connect( pb, SIGNAL(clicked()), SLOT(slotApply()) );
+    connect( pb, SIGNAL(clicked()), SLOT(applyPressed()) );
   }
   if( d->mButton.mask & Try )
   {
     KPushButton *pb = d->mButton.append( Try,
                           KGuiItem( i18n( "&Try" ), "try" ) );
-    connect( pb, SIGNAL(clicked()), this, SLOT(slotTry()) );
+    connect( pb, SIGNAL(clicked()), SLOT(slotTry()) );
   }
   if( d->mButton.mask & Cancel )
   {
     KPushButton *pb = d->mButton.append( Cancel, KStdGuiItem::cancel() );
-    connect( pb, SIGNAL(clicked()), this, SLOT(slotCancel()) );
+    connect( pb, SIGNAL(clicked()), SLOT(slotCancel()) );
   }
   if( d->mButton.mask & Close )
   {
     KPushButton *pb = d->mButton.append( Close, KStdGuiItem::close() );
-    connect( pb, SIGNAL(clicked()), this, SLOT(slotClose()) );
+    connect( pb, SIGNAL(clicked()), SLOT(slotClose()) );
   }
 
   QPushButton *pb = actionButton( defaultButton );
   if( pb != 0 )
   {
-    setButtonFocus( pb, true, true );
+    setButtonFocus( pb, true, false );
   }
 
   setButtonStyle( ActionStyle0 );
@@ -650,29 +684,29 @@ void KDialogBase::setButtonStyle( int style )
   }
   else if (mButtonOrientation == Horizontal)
   {
-    static const int layoutRule[5][9] =
+    static const int layoutRule[5][10] =
     {
-      {Help,Default,Stretch,User3,User2,User1,Ok,Apply|Try,Cancel|Close},
-      {Help,Default,Stretch,User3,User2,User1,Cancel|Close,Apply|Try,Ok},
-      {Help,Default,Stretch,User3,User2,User1,Apply|Try,Cancel|Close,Ok},
-      {Ok,Apply|Try,Cancel|Close,User3,User2,User1,Stretch,Default,Help},
-      {Ok,Cancel|Close,Apply|Try,User3,User2,User1,Stretch,Default,Help}
+      {Details,Help,Default,Stretch,User3,User2,User1,Ok,Apply|Try,Cancel|Close},
+      {Details,Help,Default,Stretch,User3,User2,User1,Cancel|Close,Apply|Try,Ok},
+      {Details,Help,Default,Stretch,User3,User2,User1,Apply|Try,Cancel|Close,Ok},
+      {Ok,Apply|Try,Cancel|Close,User3,User2,User1,Stretch,Default,Help,Details},
+      {Ok,Cancel|Close,Apply|Try,User3,User2,User1,Stretch,Default,Help,Details}
     };
-    layoutMax = 9;
+    layoutMax = 10;
     layout = layoutRule[ d->mButton.style ];
   }
   else
   {
-    static const int layoutRule[5][9] =
+    static const int layoutRule[5][10] =
     {
-      {Ok,Apply|Try,User1,User2,User3,Stretch,Default,Cancel|Close,Help},
-      //{Ok,Apply|Try,Cancel|Close,User1,User2,User3,Stretch, Default,Help},
-      {Help,Default,Stretch,User3,User2,User1,Cancel|Close,Apply|Try,Ok},
-      {Help,Default,Stretch,User3,User2,User1,Apply|Try,Cancel|Close,Ok},
-      {Ok,Apply|Try,Cancel|Close,User3,User2,User1,Stretch,Default,Help},
-      {Ok,Cancel|Close,Apply|Try,User3,User2,User1,Stretch,Default,Help}
+      {Ok,Apply|Try,User1,User2,User3,Stretch,Default,Cancel|Close,Help, Details},
+      //{Ok,Apply|Try,Cancel|Close,User1,User2,User3,Stretch, Default,Help, Details},
+      {Details,Help,Default,Stretch,User3,User2,User1,Cancel|Close,Apply|Try,Ok},
+      {Details,Help,Default,Stretch,User3,User2,User1,Apply|Try,Cancel|Close,Ok},
+      {Ok,Apply|Try,Cancel|Close,User3,User2,User1,Stretch,Default,Help,Details},
+      {Ok,Cancel|Close,Apply|Try,User3,User2,User1,Stretch,Default,Help,Details}
     };
-    layoutMax = 9;
+    layoutMax = 10;
     layout = layoutRule[ d->mButton.style ];
   }
 
@@ -875,6 +909,16 @@ void KDialogBase::setButtonOKText( const QString &text,
 }
 
 
+void KDialogBase::setButtonOK( const KGuiItem &item /* = KStdGuiItem::ok() */ )
+{
+  KPushButton *pb = static_cast<KPushButton *>( actionButton( Ok ) );
+  if( pb == 0 )
+    return;
+
+  pb->setGuiItem( item );
+  d->mButton.resize( false, 0, spacingHint(), mButtonOrientation );
+}
+
 
 void KDialogBase::setButtonApplyText( const QString &text,
 				      const QString &tooltip,
@@ -900,6 +944,17 @@ void KDialogBase::setButtonApplyText( const QString &text,
 }
 
 
+void KDialogBase::setButtonApply( const KGuiItem &item /* = KStdGuiItem::apply() */ )
+{
+  KPushButton *pb = static_cast<KPushButton *>( actionButton( Apply ) );
+  if( pb == 0 )
+    return;
+
+  pb->setGuiItem( item );
+  d->mButton.resize( false, 0, spacingHint(), mButtonOrientation );
+}
+
+
 void KDialogBase::setButtonCancelText( const QString& text,
 				       const QString& tooltip,
 				       const QString& quickhelp )
@@ -918,6 +973,17 @@ void KDialogBase::setButtonCancelText( const QString& text,
 }
 
 
+void KDialogBase::setButtonCancel( const KGuiItem &item /* = KStdGuiItem::cancel() */ )
+{
+  KPushButton *pb = static_cast<KPushButton *>( actionButton( Cancel ) );
+  if( pb == 0 )
+    return;
+
+  pb->setGuiItem( item );
+  d->mButton.resize( false, 0, spacingHint(), mButtonOrientation );
+}
+
+
 void KDialogBase::setButtonText( ButtonCode id, const QString &text )
 {
   if (!d->bSettingDetails && (id == Details))
@@ -964,7 +1030,8 @@ void KDialogBase::setButtonWhatsThis( ButtonCode id, const QString &text )
 void KDialogBase::setButtonFocus( QPushButton *p,bool isDefault, bool isFocus )
 {
   p->setDefault( isDefault );
-  isFocus ? p->setFocus() : p->clearFocus();
+  if( isFocus )
+      p->setFocus();
 }
 
 
@@ -992,7 +1059,29 @@ void KDialogBase::setRootIsDecorated( bool state )
   }
 }
 
+void KDialogBase::unfoldTreeList( bool persist )
+{
+  if( mJanus != 0 )
+    mJanus->unfoldTreeList( persist );
+}
 
+void KDialogBase::addWidgetBelowList( QWidget * widget )
+{
+  if( mJanus != 0 )
+    mJanus->addWidgetBelowList( widget );
+}
+
+void KDialogBase::addButtonBelowList( const QString & text, QObject * recv, const char * slot )
+{
+  if( mJanus != 0 )
+    mJanus->addButtonBelowList( text, recv, slot );
+}
+
+void KDialogBase::addButtonBelowList( const KGuiItem & item, QObject * recv, const char * slot )
+{
+  if( mJanus != 0 )
+    mJanus->addButtonBelowList( item, recv, slot );
+}
 
 void KDialogBase::setIconListAllVisible( bool state )
 {
@@ -1359,6 +1448,19 @@ void KDialogBase::setMainWidget( QWidget *widget )
       setupLayout();
     }
   }
+  if( mMainWidget != NULL )
+  {
+    QFocusData* fd = focusData();
+    QWidget* prev = fd->last();
+    for( QPtrListIterator<KDialogBaseButton> it( d->mButton.list );
+	 it != NULL;
+	 ++it )
+    {
+      if( prev != *it )
+	setTabOrder( prev, *it );
+      prev = *it;
+    }
+  }
 }
 
 
@@ -1436,7 +1538,7 @@ QSize KDialogBase::calculateSize(int w, int h) const
 
 QString KDialogBase::helpLinkText() const
 {
-  return( mHelpLinkText==QString::null ? i18n("Get help...") : mHelpLinkText );
+  return( mHelpLinkText.isNull() ? i18n("Get help...") : mHelpLinkText );
 }
 
 
@@ -1490,9 +1592,6 @@ void KDialogBase::keyPressEvent( QKeyEvent *e )
 
   // accept the dialog when Ctrl-Return is pressed
   else if ( e->state() == ControlButton &&
-            qApp->focusWidget() &&
-            (qApp->focusWidget()->inherits( "QTextEdit" ) ||
-             qApp->focusWidget()->inherits( "QLineEdit" )) &&
             (e->key() == Key_Return || e->key() == Key_Enter) )
   {
     QPushButton *pb = actionButton( Ok );
@@ -1643,6 +1742,13 @@ void KDialogBase::showTile( bool state )
 }
 
 QSize KDialogBase::configDialogSize( const QString& groupName ) const
+{
+   return configDialogSize( *KGlobal::config(), groupName );
+}
+
+
+QSize KDialogBase::configDialogSize( KConfig& config,
+				      const QString& groupName ) const
 {
    int w, h;
    int scnum = QApplication::desktop()->screenNumber(parentWidget());
@@ -1650,34 +1756,33 @@ QSize KDialogBase::configDialogSize( const QString& groupName ) const
    w = QMIN( 530, (int) (desk.width() * 0.5)); // maximum default width = 530
    h = (int) (desk.height() * 0.4);
 
-   KConfig *kc = KGlobal::config();
+   KConfigGroupSaver cs(&config, groupName);
+   w = config.readNumEntry( QString::fromLatin1("Width %1").arg( desk.width()), w );
+   h = config.readNumEntry( QString::fromLatin1("Height %1").arg( desk.height()), h );
 
-   if( kc )
-   {
-      KConfigGroupSaver cs(kc, groupName);
-      w = kc->readNumEntry( QString::fromLatin1("Width %1").arg( desk.width()), w );
-      h = kc->readNumEntry( QString::fromLatin1("Height %1").arg( desk.height()), h );
-   }
    return( QSize( w, h ) );
 }
 
 
 void KDialogBase::saveDialogSize( const QString& groupName, bool global )
+{
+   saveDialogSize( *KGlobal::config(), groupName, global );
+}
+
+
+void KDialogBase::saveDialogSize( KConfig& config, const QString& groupName,
+				      bool global ) const
 {
    int scnum = QApplication::desktop()->screenNumber(parentWidget());
    QRect desk = QApplication::desktop()->screenGeometry(scnum);
-   KConfig *kc = KGlobal::config();
 
-   if( kc )
-   {
-      KConfigGroupSaver cs(kc, groupName);
-      QSize sizeToSave = size();
+   KConfigGroupSaver cs(&config, groupName);
+   QSize sizeToSave = size();
 
-      kc->writeEntry( QString::fromLatin1("Width %1").arg( desk.width()),
+   config.writeEntry( QString::fromLatin1("Width %1").arg( desk.width()),
 		      QString::number( sizeToSave.width()), true, global);
-      kc->writeEntry( QString::fromLatin1("Height %1").arg( desk.height()),
+   config.writeEntry( QString::fromLatin1("Height %1").arg( desk.height()),
 		      QString::number( sizeToSave.height()), true, global);
-   }
 }
 
 
