@@ -1,5 +1,6 @@
 /* This file is part of the KDE libraries
    Copyright (C) 2000 Max Judin <novaprint@mtu-net.ru>
+   Copyright (C) 2002,2003 Joseph Wenninger <jowenn@kde.org>
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
@@ -16,6 +17,7 @@
    Boston, MA 02111-1307, USA.
 */
 #include "kdockwidget.h"
+#include "kdockwidget_p.h"
 #include "kdockwidget_private.h"
 
 #include <qpainter.h>
@@ -81,6 +83,20 @@ void KDockSplitter::activate(QWidget *c0, QWidget *c1)
   	setForcedFixedWidth(((KDockWidget*)child1),((KDockWidget*)child1)->forcedFixedWidth());
 	//QTimer::singleShot(100,this,SLOT(delayedResize()));
   }
+
+  if (((KDockWidget*)child0)->forcedFixedHeight()!=-1)
+  {
+        setForcedFixedHeight(((KDockWidget*)child0),((KDockWidget*)child0)->forcedFixedHeight());
+        //QTimer::singleShot(100,this,SLOT(delayedResize()));
+  }
+  else
+  if (((KDockWidget*)child1)->forcedFixedHeight()!=-1)
+  {
+        setForcedFixedHeight(((KDockWidget*)child1),((KDockWidget*)child1)->forcedFixedHeight());
+        //QTimer::singleShot(100,this,SLOT(delayedResize()));
+  }
+
+
 }
 
 /*
@@ -179,7 +195,7 @@ void KDockSplitter::setupMinMaxSize()
 
 void KDockSplitter::deactivate()
 {
-  if (divider) delete divider;
+  delete divider;
   divider = 0L;
   initialised= false;
 }
@@ -219,9 +235,9 @@ void KDockSplitter::resizeEvent(QResizeEvent *ev)
           else
           {
 //	kdDebug()<<"!mKeepSize : "<< ((m_orientation == Horizontal) ? "Horizontal":"Vertical") <<endl;
-	if (ev && isVisible()) {
+	if (/*ev &&*/ isVisible()) {
 		if (m_orientation == Horizontal) {
-			if (ev->oldSize().height() != ev->size().height())
+		/*	if (ev->oldSize().height() != ev->size().height())*/
 			{
 			  if (fixedHeight0!=-1)
 				xpos=fixedHeight0*factor/height();
@@ -232,7 +248,7 @@ void KDockSplitter::resizeEvent(QResizeEvent *ev)
 		}
 		else
 		{
-	        	if (ev->oldSize().width() != ev->size().width())
+/*	        	if (ev->oldSize().width() != ev->size().width()) */
 			{
 			  if (fixedWidth0!=-1)
 				xpos=fixedWidth0*factor/width();
@@ -245,19 +261,96 @@ void KDockSplitter::resizeEvent(QResizeEvent *ev)
 //	else kdDebug()<<"Something else happened"<<endl;
    }
 
-    int position = checkValue( (m_orientation == Vertical ? width() : height()) * xpos/factor );
-    if (m_orientation == Horizontal){
-      child0->setGeometry(0, 0, width(), position);
-      child1->setGeometry(0, position+4, width(), height()-position-4);
-      divider->setGeometry(0, position, width(), 4);
-    } else {
-      child0->setGeometry(0, 0, position, height());
-      child1->setGeometry(position+4, 0, width()-position-4, height());
-      divider->setGeometry(position, 0, 4, height());
-    }
+    KDockContainer *dc;
+    KDockWidget *c0=(KDockWidget*)child0;
+    KDockWidget *c1=(KDockWidget*)child1;
+    bool stdHandling=false;
+    if ((fixedWidth0==-1) && (fixedWidth1==-1)) {
+	    if ((c0->getWidget()) && (dc=dynamic_cast<KDockContainer*>(c0->getWidget()))
+		 && (dc->m_overlapMode)) {
+			int position= (m_orientation == Vertical ? width() : height()) * xpos/factor;
+			position=checkValueOverlapped(position,child0);
+			child0->raise();
+			divider->raise();
+	        	      if (m_orientation == Horizontal){
+        	        	child0->setGeometry(0, 0, width(), position);
+	                	child1->setGeometry(0, dc->m_nonOverlapSize+4, width(), 
+						height()-dc->m_nonOverlapSize-4);
+	        	        divider->setGeometry(0, position, width(), 4);
+	        	      } else {
+        	        	child0->setGeometry(0, 0, position, height());
+		                child1->setGeometry(dc->m_nonOverlapSize+4, 0, 
+						width()-dc->m_nonOverlapSize-4, height());
+        		        divider->setGeometry(position, 0, 4, height());
+		              }
+	    } else {
+		 if ((c1->getWidget()) && (dc=dynamic_cast<KDockContainer*>(c1->getWidget()))
+        	 && (dc->m_overlapMode)) {
+                	int position= (m_orientation == Vertical ? width() : height()) * xpos/factor;
+			position=checkValueOverlapped(position,child1);
+	                child1->raise();
+        	        divider->raise();
+	                      if (m_orientation == Horizontal){
+        	                child0->setGeometry(0, 0, width(), height()-dc->m_nonOverlapSize-4);
+                	        child1->setGeometry(0, position+4, width(),
+	                                        height()-position-4);
+        	                divider->setGeometry(0, position, width(), 4);
+                	      } else {
+                        	child0->setGeometry(0, 0, width()-dc->m_nonOverlapSize-4, height());
+	                        child1->setGeometry(position+4, 0,
+        	                                width()-position-4, height());
+                	        divider->setGeometry(position, 0, 4, height());
+	                      }
+		}
+		else stdHandling=true;
+	      }
+            }
+	 else stdHandling=true;
+
+	if (stdHandling) {
+		      int position = checkValue( (m_orientation == Vertical ? width() : height()) * xpos/factor );
+		      if (m_orientation == Horizontal){
+        		child0->setGeometry(0, 0, width(), position);
+		        child1->setGeometry(0, position+4, width(), height()-position-4);
+        		divider->setGeometry(0, position, width(), 4);
+		      } else {
+        		child0->setGeometry(0, 0, position, height());
+	        	child1->setGeometry(position+4, 0, width()-position-4, height());
+	        	divider->setGeometry(position, 0, 4, height());
+	}
+
+	}
+	
   }
 }
 
+int KDockSplitter::checkValueOverlapped(int position, QWidget *overlappingWidget) const {
+	if (initialised) {
+		if (m_orientation == Vertical) {
+			if (child0==overlappingWidget) {
+				if (position<(child0->minimumSize().width()))
+					position=child0->minimumSize().width();
+				if (position>width()) position=width()-4;
+			} else if (position>(width()-(child1->minimumSize().width())-4)){
+				position=width()-(child1->minimumSize().width())-4;
+				if (position<0) position=0;
+			}
+		} else {// orientation  == Horizontal
+			if (child0==overlappingWidget) {
+				if (position<(child0->minimumSize().height()))
+					position=child0->minimumSize().height();
+				if (position>height()) position=height()-4;
+			} else if (position>(height()-(child1->minimumSize().height())-4)){
+				position=height()-(child1->minimumSize().height())-4;
+				if (position<0) position=0;
+
+			}
+		}
+
+	}
+	return position;
+}
+
 int KDockSplitter::checkValue( int position ) const
 {
   if (initialised){
