@@ -19,6 +19,7 @@
    Boston, MA 02111-1307, USA.
 */
 
+#include <qdragobject.h>
 #include <qpopupmenu.h>
 #include <qtextstream.h>
 #include <qtimer.h>
@@ -77,7 +78,7 @@ KEdit::~KEdit()
 void
 KEdit::insertText(QTextStream *stream)
 {
-//   setAutoUpdate(FALSE);
+//   setAutoUpdate(false);
    int line, col;
    getCursorPosition(&line, &col);
    int saveline = line;
@@ -138,7 +139,7 @@ KEdit::insertText(QTextStream *stream)
 void
 KEdit::cleanWhiteSpace()
 {
-   setAutoUpdate(FALSE);
+   setAutoUpdate(false);
    if (!hasMarkedText())
       selectAll();
    QString oldText = markedText();
@@ -190,7 +191,7 @@ KEdit::cleanWhiteSpace()
    if (oldText == newText)
    {
       deselect();
-      setAutoUpdate(TRUE);
+      setAutoUpdate(true);
       repaint();
       return;
    }
@@ -198,7 +199,7 @@ KEdit::cleanWhiteSpace()
    {
       // If wordwrap is off, we have to do some line-wrapping ourselves now
       // We use another QMultiLineEdit for this, so that we get nice undo
-      // behaviour.
+      // behavior.
       QMultiLineEdit *we = new QMultiLineEdit();
       we->setWordWrap(FixedColumnWidth);
       we->setWrapColumnOrWidth(78);
@@ -215,7 +216,7 @@ KEdit::cleanWhiteSpace()
    }
 
    insert(newText);
-   setAutoUpdate(TRUE);
+   setAutoUpdate(true);
    repaint();
 
    setModified(true);
@@ -350,7 +351,7 @@ void KEdit::computePosition()
 
 
   while(find >=0 && find <= coltemp- 1 ){
-    find = linetext.find('\t', find+start_of_line, TRUE )-start_of_line;
+    find = linetext.find('\t', find+start_of_line, true )-start_of_line;
     if( find >=0 && find <= coltemp - 1 ){
       found_one = true;
       pos = pos + find - mem;
@@ -625,7 +626,7 @@ void KEdit::setOverwriteEnabled(bool b)
 void KEdit::create( WId id, bool initializeWindow, bool destroyOldWindow )
 {
   QMultiLineEdit::create( id, initializeWindow, destroyOldWindow );
-  KCursor::setAutoHideCursor( this, true, true );
+  KCursor::setAutoHideCursor( this, true );
 }
 
 
