@@ -23,13 +23,13 @@
 
 #include <qframe.h>
 #include <qlabel.h>
-#include <qlayout.h>
 #include <qlineedit.h>
 #include <qvbuttongroup.h>
 #include <qcheckbox.h>
 #include <qlayout.h>
 #include <qpushbutton.h>
 #include <qhbox.h>
+#include <qpopupmenu.h>
 
 #include <kapplication.h>
 #include <kcombobox.h>
@@ -38,6 +38,7 @@
 #include <knotifyclient.h>
 #include <klocale.h>
 #include <kdebug.h>
+#include <kiconloader.h>
 
 #include "keditcl.h"
 
@@ -94,7 +95,7 @@ void KEdit::search_slot(){
 
 again:
   int  result = doSearch(to_find_string, srchdialog->case_sensitive(),
-			 FALSE, (!srchdialog->get_direction()),line,col);
+			 false, (!srchdialog->get_direction()),line,col);
 
   if(result == 0){
     if(!srchdialog->get_direction()){ // forward search
@@ -143,6 +144,7 @@ void KEdit::searchdone_slot(){
   last_search = NONE;
 }
 
+/* antlarr: KDE 4: make it const QString & */
 int KEdit::doSearch(QString s_pattern, bool case_sensitive,
 		    bool wildcard, bool forward, int line, int col){
 
@@ -166,13 +168,13 @@ int KEdit::doSearch(QString s_pattern, bool case_sensitive,
 
 	length = s_pattern.length();
 
-	setCursorPosition(i,pos,FALSE);
+	setCursorPosition(i,pos,false);
 
 	for(int l = 0 ; l < length; l++){
-	  cursorRight(TRUE);
+	  cursorRight(true);
 	}
 
-	setCursorPosition( i , pos + length, TRUE );
+	setCursorPosition( i , pos + length, true );
 	pattern = s_pattern;
 	last_search = FORWARD;
 
@@ -197,13 +199,13 @@ int KEdit::doSearch(QString s_pattern, bool case_sensitive,
 
 	if( ! (line == i && pos > col ) ){
 
-	  setCursorPosition(i ,pos ,FALSE );
+	  setCursorPosition(i ,pos ,false );
 
 	  for(int l = 0 ; l < length; l++){
-	    cursorRight(TRUE);
+	    cursorRight(true);
 	  }
 
-	  setCursorPosition(i ,pos + length ,TRUE );
+	  setCursorPosition(i ,pos + length ,true );
 	  pattern = s_pattern;
 	  last_search = BACKWARD;
 	  return 1;
@@ -291,14 +293,14 @@ void KEdit::replace_slot(){
 
   insertAt(string,line,col);
   setModified(true);
-  can_replace = FALSE;
+  can_replace = false;
 
   if (replace_dialog->get_direction())
   {
     // Backward
     setCursorPosition(line,col+length);
     for( int k = 0; k < length; k++){
-      cursorLeft(TRUE);
+      cursorLeft(true);
     }
   }
   else
@@ -306,7 +308,7 @@ void KEdit::replace_slot(){
     // Forward
     setCursorPosition(line,col);
     for( int k = 0; k < length; k++){
-      cursorRight(TRUE);
+      cursorRight(true);
     }
   }
 }
@@ -352,18 +354,18 @@ void KEdit::replace_all_slot(){
 
 again:
 
-  setAutoUpdate(FALSE);
+  setAutoUpdate(false);
   int result = 1;
 
   while(result){
 
     result = doReplace(to_find_string, replace_dialog->case_sensitive(),
-		       FALSE, (!replace_dialog->get_direction()),
-		       replace_all_line,replace_all_col,TRUE);
+		       false, (!replace_dialog->get_direction()),
+		       replace_all_line,replace_all_col,true);
 
   }
 
-  setAutoUpdate(TRUE);
+  setAutoUpdate(true);
   update();
 
   if(!replace_dialog->get_direction()){ // forward search
@@ -442,7 +444,7 @@ void KEdit::replace_search_slot(){
 again:
 
   int  result = doReplace(to_find_string, replace_dialog->case_sensitive(),
-			 FALSE, (!replace_dialog->get_direction()), line, col, FALSE );
+			 false, (!replace_dialog->get_direction()), line, col, false );
 
   if(result == 0){
     if(!replace_dialog->get_direction()){ // forward search
@@ -493,12 +495,13 @@ void KEdit::replacedone_slot(){
   setFocus();
 
   last_replace = NONE;
-  can_replace  = FALSE;
+  can_replace  = false;
 
 }
 
 
 
+/* antlarr: KDE 4: make it const QString & */
 int KEdit::doReplace(QString s_pattern, bool case_sensitive,
 	   bool wildcard, bool forward, int line, int col, bool replace_all){
 
@@ -556,16 +559,16 @@ int KEdit::doReplace(QString s_pattern, bool case_sensitive,
 	}
 	else{ // interactive
 
-	  setCursorPosition( line_counter , pos, FALSE );
+	  setCursorPosition( line_counter , pos, false );
 
 	  for(int l = 0 ; l < length; l++){
-	    cursorRight(TRUE);
+	    cursorRight(true);
 	  }
 
-	  setCursorPosition( line_counter , pos + length, TRUE );
+	  setCursorPosition( line_counter , pos + length, true );
 	  pattern = s_pattern;
 	  last_replace = FORWARD;
-	  can_replace = TRUE;
+	  can_replace = true;
 
 	  return 1;
 
@@ -591,7 +594,7 @@ int KEdit::doReplace(QString s_pattern, bool case_sensitive,
       else{
         if ((line == line_counter) && (col < 0))
           pos = -1;
-        else 
+        else
           pos = string.findRev(s_pattern,
 			   line == line_counter ? col : line_length , case_sensitive);
       }
@@ -640,17 +643,17 @@ int KEdit::doReplace(QString s_pattern, bool case_sensitive,
 	  //	  printf("line_counter %d pos %d col %d\n",line_counter, pos,col);
 	  if( ! (line == line_counter && pos > col ) ){
 
-	    setCursorPosition(line_counter, pos + length ,FALSE );
+	    setCursorPosition(line_counter, pos + length ,false );
 
 	    for(int l = 0 ; l < length; l++){
-	      cursorLeft(TRUE);
+	      cursorLeft(true);
 	    }
 
-	    setCursorPosition(line_counter, pos ,TRUE );
+	    setCursorPosition(line_counter, pos ,true );
 	    pattern = s_pattern;
 
 	    last_replace = BACKWARD;
-	    can_replace = TRUE;
+	    can_replace = true;
 
 	    return 1;
 	  }
@@ -689,7 +692,7 @@ public:
 
 KEdFind::KEdFind( QWidget *parent, const char *name, bool modal )
   :KDialogBase( parent, name, modal, i18n("Find"),
-		modal ? User1|Cancel : User1|Close, User1, false, i18n("&Find") )
+		modal ? User1|Cancel : User1|Close, User1, false, KGuiItem( i18n("&Find"), "find") )
 {
   setWFlags( WType_TopLevel );
 
@@ -715,13 +718,13 @@ KEdFind::KEdFind( QWidget *parent, const char *name, bool modal )
   topLayout->addWidget( group );
 
   QHBox* row1 = new QHBox( group );
-  
+
   text = i18n("Case &sensitive");
   sensitive = new QCheckBox( text, row1, "case");
   text = i18n("Find &backwards");
   direction = new QCheckBox( text, row1, "direction" );
 
-  
+
   enableButton( KDialogBase::User1, !d->combo->currentText().isEmpty() );
 
   if ( !modal )
@@ -760,6 +763,7 @@ QString KEdFind::getText() const
 }
 
 
+/* antlarr: KDE 4: make it const QString & */
 void KEdFind::setText(QString string)
 {
   d->combo->setEditText(string);
@@ -820,7 +824,7 @@ KEdReplace::KEdReplace( QWidget *parent, const char *name, bool modal )
   :KDialogBase( parent, name, modal, i18n("Replace"),
 		modal ? User3|User2|User1|Cancel : User3|User2|User1|Close,
                 User3, false,
-		i18n("Replace &All"), i18n("&Replace"), i18n("&Find") )
+		i18n("Replace &All"), i18n("&Replace"), KGuiItem( i18n("&Find"), "find") )
 {
   setWFlags( WType_TopLevel );
 
@@ -932,6 +936,7 @@ QString KEdReplace::getReplaceText()
 }
 
 
+/* antlarr: KDE 4: make it const QString & */
 void KEdReplace::setText(QString string)
 {
   d->searchCombo->setEditText(string);
@@ -962,7 +967,7 @@ KHistoryCombo * KEdReplace::replaceCombo() const
 
 
 KEdGotoLine::KEdGotoLine( QWidget *parent, const char *name, bool modal )
-  :KDialogBase( parent, name, modal, i18n("Goto Line"), modal ? Ok|Cancel : Ok|Close, Ok, false )
+  :KDialogBase( parent, name, modal, i18n("Go to Line"), modal ? Ok|Cancel : Ok|Close, Ok, false )
 {
   QWidget *page = new QWidget( this );
   setMainWidget(page);
@@ -970,7 +975,7 @@ KEdGotoLine::KEdGotoLine( QWidget *parent, const char *name, bool modal )
 
   lineNum = new KIntNumInput( 1, page);
   lineNum->setRange(1, 1000000, 1, false);
-  lineNum->setLabel(i18n("Goto line:"), AlignVCenter | AlignLeft);
+  lineNum->setLabel(i18n("Go to line:"), AlignVCenter | AlignLeft);
 //  lineNum->setMinimumWidth(fontMetrics().maxWidth()*20);
   topLayout->addWidget( lineNum );
 
@@ -1038,7 +1043,7 @@ void KEdit::corrected (const QString &originalword, const QString &newword, unsi
     insert(newword);
     setReadOnly ( true );
   }
-  else 
+  else
   {
     deselect();
   }
@@ -1061,5 +1066,52 @@ void KEdit::spellcheck_stop()
   setReadOnly ( saved_readonlystate);
 }
 
+QString KEdit::selectWordUnderCursor( )
+{
+    int parag;
+    int pos;
 
+    getCursorPosition(&parag, &pos);
 
+    QString txt = text(parag);
+
+    // Find start
+    int start = pos;
+    while( start > 0 )
+    {
+       const QChar &ch = txt[start-1];
+       if (ch.isSpace() || ch.isPunct())
+          break;
+       start--;
+    }
+
+    // Find end
+    int end = pos;
+    int len = txt.length();
+    while( end < len )
+    {
+       const QChar &ch = txt[end];
+       if (ch.isSpace() || ch.isPunct())
+          break;
+       end++;
+    }
+    setSelection(parag, start, parag, end);
+    return txt.mid(start, end-start);
+}
+
+QPopupMenu *KEdit::createPopupMenu( const QPoint& pos )
+{
+    enum { IdUndo, IdRedo, IdSep1, IdCut, IdCopy, IdPaste, IdClear, IdSep2, IdSelectAll };
+
+    QPopupMenu *menu = QMultiLineEdit::createPopupMenu( pos );
+
+    int id = menu->idAt(0);
+    menu->changeItem( id - IdUndo, SmallIcon("undo"), menu->text( id - IdUndo) );
+    menu->changeItem( id - IdRedo, SmallIcon("redo"), menu->text( id - IdRedo) );
+    menu->changeItem( id - IdCut, SmallIcon("editcut"), menu->text( id - IdCut) );
+    menu->changeItem( id - IdCopy, SmallIcon("editcopy"), menu->text( id - IdCopy) );
+    menu->changeItem( id - IdPaste, SmallIcon("editpaste"), menu->text( id - IdPaste) );
+    menu->changeItem( id - IdClear, SmallIcon("editclear"), menu->text( id - IdClear) );
+
+    return menu;
+}
