@@ -1,5 +1,5 @@
 /*
-    $Id: kfontdialog.cpp,v 1.95.2.1 2003/07/17 16:14:06 waba Exp $
+    $Id: kfontdialog.cpp,v 1.99 2003/09/03 15:33:17 schroder Exp $
 
     Requires the Qt widget libraries, available at no cost at
     http://www.troll.no
@@ -54,8 +54,9 @@
 #include <kstandarddirs.h>
 #include <kdebug.h>
 #include <knuminput.h>
-#ifdef Q_WS_X11
-#include <X11/Xlib.h>
+#if defined Q_WS_X11 && ! defined K_WS_QTONLY
+#include <X11/Xlib.h> // schroder // remove it for the X11 case too,
+		    // not needed any more...
 #endif
 
 #include "kfontdialog.moc"
@@ -245,7 +246,7 @@ KFontChooser::KFontChooser(QWidget *parent, const char *name,
   sizeOfFont->setEnabled( !diff );
   if( sizeIsRelativeState ) {
     QString sizeIsRelativeCBText =
-      i18n("relative");
+      i18n("Relative");
     QString sizeIsRelativeCBToolTipText =
       i18n("Font size<br><i>fixed</i> or <i>relative</i><br>to environment");
     QString sizeIsRelativeCBWhatsThisText =
@@ -623,6 +624,10 @@ void KFontChooser::setupDisplay()
     }
   }
 
+  // Fall back in case nothing matched. Otherwise, diff doesn't work
+  if ( i == numEntries )
+    familyListBox->setCurrentItem( 0 );
+
   styleListBox->setCurrentItem(style);
 
   numEntries = sizeListBox->count();
