@@ -42,7 +42,10 @@
 #include <kstdaccel.h>
 #include <kstdaction.h>
 
+#include "config.h"
+#if defined Q_WS_X11 && ! defined K_WS_QTONLY
 #include <qxembed.h>
+#endif
 
 class KHelpMenuPrivate
 {
@@ -111,7 +114,7 @@ KPopupMenu* KHelpMenu::menu()
     // I use hardcoded menu id's here. Reason is to stay backward
     // compatible.
     //
-    const KAboutData *aboutData = KGlobal::instance()->aboutData();
+    const KAboutData *aboutData = d->mAboutData ? d->mAboutData : KGlobal::instance()->aboutData();
     QString appName = (aboutData)? aboutData->programName() : QString::fromLatin1(qApp->name());
 
     mMenu = new KPopupMenu();
@@ -123,7 +126,7 @@ KPopupMenu* KHelpMenu::menu()
       mMenu->insertItem( BarIcon( "contents", KIcon::SizeSmall),
                      i18n( "%1 &Handbook" ).arg( appName) ,menuHelpContents );
       mMenu->connectItem( menuHelpContents, this, SLOT(appHelpActivated()) );
-      mMenu->setAccel( KStdAccel::key(KStdAccel::Help), menuHelpContents );
+      mMenu->setAccel( KStdAccel::shortcut(KStdAccel::Help), menuHelpContents );
       need_separator = true;
     }
 
@@ -137,7 +140,7 @@ KPopupMenu* KHelpMenu::menu()
       need_separator = true;
     }
 
-    if (kapp->authorizeKAction("help_report_bug"))
+    if (kapp->authorizeKAction("help_report_bug") && aboutData && !aboutData->bugAddress().isEmpty() )
     {
       if (need_separator)
         mMenu->insertSeparator();
@@ -155,7 +158,7 @@ KPopupMenu* KHelpMenu::menu()
         i18n( "&About %1" ).arg(appName), menuAboutApp );
       mMenu->connectItem( menuAboutApp, this, SLOT( aboutApplication() ) );
     }
-    
+
     if (kapp->authorizeKAction("help_about_kde"))
     {
       mMenu->insertItem( SmallIcon("about_kde"), i18n( "About &KDE" ), menuAboutKDE );
@@ -196,7 +199,7 @@ void KHelpMenu::aboutApplication()
       mAboutApp = new KDialogBase( QString::null, // Caption is defined below
 				   KDialogBase::Yes, KDialogBase::Yes,
 				   KDialogBase::Yes, mParent, "about",
-				   false, true, i18n("&OK") );
+				   false, true, KStdGuiItem::ok() );
       connect( mAboutApp, SIGNAL(finished()), this, SLOT( dialogFinished()) );
 
       QHBox *hbox = new QHBox( mAboutApp );
@@ -233,7 +236,7 @@ void KHelpMenu::reportBug()
 {
   if( mBugReport == 0 )
   {
-    mBugReport = new KBugReport( mParent, false );
+    mBugReport = new KBugReport( mParent, false, d->mAboutData );
     connect( mBugReport, SIGNAL(finished()),this,SLOT( dialogFinished()) );
   }
   mBugReport->show();
@@ -274,10 +277,10 @@ void KHelpMenu::menuDestroyed()
 void KHelpMenu::contextHelpActivated()
 {
   QWhatsThis::enterWhatsThisMode();
-  QWidget* w = QApplication::widgetAt( QCursor::pos(), TRUE );
+  QWidget* w = QApplication::widgetAt( QCursor::pos(), true );
   while ( w && !w->isTopLevel() && !w->inherits("QXEmbed")  )
       w = w->parentWidget();
-#ifndef Q_WS_QWS
+#if defined Q_WS_X11 && ! defined K_WS_QTONLY
    if ( w && w->inherits("QXEmbed") )
 	  (( QXEmbed*) w )->enterWhatsThisMode();
 #endif
