@@ -15,6 +15,9 @@
    the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
    Boston, MA 02111-1307, USA.
 */
+
+#include "config.h"
+
 #include <qtimer.h>
 #include <qpainter.h>
 #include <qpixmapcache.h>
@@ -27,13 +30,17 @@
 #include <kglobal.h>
 #include <kglobalsettings.h>
 #include <kapplication.h>
-#include <kipc.h>
+
+#if defined Q_WS_X11 && ! defined K_WS_QTONLY
+#include <kipc.h> // schroder
+#endif
+
 #include <kcursor.h>
 #include <kpixmap.h>
 #include <kpixmapeffect.h>
 
-#ifdef Q_WS_X11
-#include <X11/Xlib.h>
+#if defined Q_WS_X11 && ! defined K_WS_QTONLY
+#include <X11/Xlib.h> // schroder still stuff to be fixed...
 #endif
 
 class KIconView::KIconViewPrivate
@@ -62,7 +69,9 @@ KIconView::KIconView( QWidget *parent, const char *name, WFlags f )
     slotSettingsChanged( KApplication::SETTINGS_MOUSE );
     if ( kapp ) { // maybe null when used inside designer
         connect( kapp, SIGNAL( settingsChanged(int) ), SLOT( slotSettingsChanged(int) ) );
+#if defined Q_WS_X11 && ! defined K_WS_QTONLY
         kapp->addKipcEventMask( KIPC::SettingsChanged );
+#endif
     }
 
     m_pCurrentItem = 0L;
@@ -159,7 +168,7 @@ void KIconView::slotAutoSelect()
   if( !hasFocus() )
     setFocus();
 
-#ifdef Q_WS_X11
+#if defined Q_WS_X11 && ! defined K_WS_QTONLY
   //FIXME(E): Implement for Qt Embedded
   Window root;
   Window child;
@@ -167,14 +176,14 @@ void KIconView::slotAutoSelect()
   uint keybstate;
   XQueryPointer( qt_xdisplay(), qt_xrootwin(), &root, &child,
 		 &root_x, &root_y, &win_x, &win_y, &keybstate );
-#endif
 
   QIconViewItem* previousItem = currentItem();
+#endif
   setCurrentItem( m_pCurrentItem );
 
   if( m_pCurrentItem ) {
     //Shift pressed?
-#ifdef Q_WS_X11 //FIXME
+#if defined Q_WS_X11 && ! defined K_WS_QTONLY //FIXME
     if( (keybstate & ShiftMask) ) {
       //Temporary implementation of the selection until QIconView supports it
       bool block = signalsBlocked();
@@ -234,7 +243,8 @@ void KIconView::slotAutoSelect()
 #endif
       setSelected( m_pCurrentItem, true );
   }
-#ifndef Q_WS_QWS //FIXME: Remove #if as soon as the stuff above is implemented
+#if defined Q_WS_X11 && ! defined K_WS_QTONLY
+   //FIXME: Remove #if as soon as the stuff above is implemented
   else
     kdDebug() << "KIconView: That's not supposed to happen!!!!" << endl;
 #endif
@@ -248,7 +258,7 @@ void KIconView::emitExecute( QIconViewItem *item, const QPoint &pos )
     return;
   }
 
-#ifdef Q_WS_X11 //FIXME
+#if defined Q_WS_X11 && ! defined K_WS_QTONLY //FIXME
   Window root;
   Window child;
   int root_x, root_y, win_x, win_y;
@@ -260,7 +270,7 @@ void KIconView::emitExecute( QIconViewItem *item, const QPoint &pos )
   m_pAutoSelect->stop();
 
   //Dont emit executed if in SC mode and Shift or Ctrl are pressed
-#ifdef Q_WS_X11 //FIXME
+#if defined Q_WS_X11 && ! defined K_WS_QTONLY //FIXME
   if( !( m_bUseSingle && ((keybstate & ShiftMask) || (keybstate & ControlMask)) ) ) {
     setSelected( item, false );
     emit executed( item );
@@ -295,7 +305,7 @@ void KIconView::contentsMousePressEvent( QMouseEvent *e )
   }
 
   QIconView::contentsMousePressEvent( e );
-  d->doAutoSelect = FALSE;
+  d->doAutoSelect = false;
 }
 
 void KIconView::contentsMouseDoubleClickEvent ( QMouseEvent * e )
@@ -321,7 +331,7 @@ void KIconView::slotMouseButtonClicked( int btn, QIconViewItem *item, const QPoi
 
 void KIconView::contentsMouseReleaseEvent( QMouseEvent *e )
 {
-    d->doAutoSelect = TRUE;
+    d->doAutoSelect = true;
     QIconView::contentsMouseReleaseEvent( e );
 }
 
@@ -361,7 +371,9 @@ void KIconViewItem::init()
 {
     m_wordWrap = 0L;
     d = 0L;
-    calcRect();
+    //Only re-calculate rectangle when wrapping, otherwise Qt did what we want
+    if ( iconView() && iconView()->wordWrapIconText() )
+        calcRect();
 }
 
 KIconViewItem::~KIconViewItem()
@@ -479,8 +491,8 @@ void KIconViewItem::calcRect( const QString& text_ )
         if ( itemIconRect.height() > itemTextRect.height() ) // icon bigger than text -> center vertically
             itemIconRect = QRect( 0, ( height - itemIconRect.height() ) / 2,
                                   itemIconRect.width(), itemIconRect.height() );
-        else // icon smaller than text -> center with first line
-            itemIconRect = QRect( 0, ( fm->height() - itemIconRect.height() ) / 2,
+        else // icon smaller than text -> place in top or center with first line
+	    itemIconRect = QRect( 0, QMAX(( fm->height() - itemIconRect.height() ) / 2, 0),
                                   itemIconRect.width(), itemIconRect.height() );
     }
 #if 0
@@ -532,20 +544,32 @@ void KIconViewItem::paintItem( QPainter *p, const QColorGroup &cg )
         kdWarning() << "KIconViewItem::paintItem called but wordwrap not ready - calcRect not called, or aborted!" << endl;
         return;
     }
-    KIconView *kview = static_cast<KIconView *>(iconView());
-    int textX = textRect( FALSE ).x();
-    int textY = textRect( FALSE ).y();
-    int iconX = pixmapRect( FALSE ).x();
-    int iconY = pixmapRect( FALSE ).y();
 
     p->save();
 
+    paintPixmap(p, cg);
+    paintText(p, cg);
+
+    p->restore();
+}
+
+KWordWrap * KIconViewItem::wordWrap()
+{
+    return m_wordWrap;
+}
+
+void KIconViewItem::paintPixmap( QPainter *p, const QColorGroup &cg )
+{
+    KIconView *kview = static_cast<KIconView *>(iconView());
+    int iconX = pixmapRect( false ).x();
+    int iconY = pixmapRect( false ).y();
+
 #ifndef QT_NO_PICTURE
     if ( picture() ) {
 	QPicture *pic = picture();
 	if ( isSelected() ) {
             // TODO something as nice as selectedIconPixmap if possible ;)
-	    p->fillRect( pixmapRect( FALSE ), QBrush( cg.highlight(), QBrush::Dense4Pattern) );
+	    p->fillRect( pixmapRect( false ), QBrush( cg.highlight(), QBrush::Dense4Pattern) );
 	}
 	p->drawPicture( x()-pic->boundingRect().x(), y()-pic->boundingRect().y(), *pic );
     } else
@@ -561,20 +585,24 @@ void KIconViewItem::paintItem( QPainter *p, const QColorGroup &cg )
             p->drawPixmap( iconX, iconY, *pix );
         }
     }
+}
+
+void KIconViewItem::paintText( QPainter *p, const QColorGroup &cg )
+{
+    int textX = textRect( false ).x();
+    int textY = textRect( false ).y();
 
     if ( isSelected() ) {
-        p->fillRect( textRect( FALSE ), cg.highlight() );
+        p->fillRect( textRect( false ), cg.highlight() );
         p->setPen( QPen( cg.highlightedText() ) );
     } else {
-        if ( view->itemTextBackground() != NoBrush )
-            p->fillRect( textRect( FALSE ), view->itemTextBackground() );
-	p->setPen( cg.text() );
+        if ( iconView()->itemTextBackground() != NoBrush )
+            p->fillRect( textRect( false ), iconView()->itemTextBackground() );
+        p->setPen( cg.text() );
     }
 
-    int align = view->itemTextPos() == QIconView::Bottom ? AlignHCenter : AlignAuto;
+    int align = iconView()->itemTextPos() == QIconView::Bottom ? AlignHCenter : AlignAuto;
     m_wordWrap->drawText( p, textX, textY, align );
-
-    p->restore();
 }
 
 void KIconView::virtual_hook( int, void* )
