@@ -29,11 +29,12 @@
 #include <kglobalaccel.h>
 #include <klocale.h>
 
-#ifdef Q_WS_X11
+#include "config.h"
+#if defined Q_WS_X11 && ! defined K_WS_QTONLY
 #define XK_XKB_KEYS
 #define XK_MISCELLANY
-#include <X11/Xlib.h>	// For x11Event()
-#include <X11/keysymdef.h> // For XK_...
+#include <X11/Xlib.h>	// For x11Event() // schroder
+#include <X11/keysymdef.h> // For XK_... // schroder is this still needed ?
 
 #ifdef KeyPress
 const int XFocusOut = FocusOut;
@@ -44,10 +45,12 @@ const int XKeyRelease = KeyRelease;
 #undef KeyPress
 #undef FocusOut
 #undef FocusIn
-#endif
-#endif
+#endif // KeyPress
+#endif // Q_WS_X11 && ! K_WS_QTONLY
 
-const char* psTemp[] = { I18N_NOOP("Primary"), I18N_NOOP("Alternate"), I18N_NOOP("Multi-Key") };
+//static const char* psTemp[] = { 
+//  I18N_NOOP("Primary"), I18N_NOOP("Alternate"), I18N_NOOP("Multi-Key") 
+//};
 
 class KKeyButtonPrivate
 {
@@ -104,9 +107,13 @@ void KKeyButton::captureShortcut()
 	m_bEditing = true;
 	repaint();
 
+        {
 	KShortcutDialog dlg( m_cut, d->bQtShortcut, this );
 	if( dlg.exec() == KDialog::Accepted )
-		emit capturedShortcut( dlg.cut() );
+                cut = dlg.shortcut();
+        } // emit the signal after the dialog is destroyed, otherwise it still has grab
+        if( !cut.isNull())
+	    emit capturedShortcut( cut );
 
 	m_bEditing = false;
 	repaint();
