@@ -22,6 +22,7 @@
 
 #include <qpushbutton.h>
 #include <kshortcut.h>
+#include <kdemacros.h>
 
 /**
  * A push button that looks like a keyboard key.
@@ -52,8 +53,8 @@ class KKeyButton: public QPushButton
 	*/
 	virtual ~KKeyButton();
 
-	/** @deprecated */
-	void setShortcut( const KShortcut& cut );
+	/** @deprecated Use setShortcut( cut, false ) instead */
+	void setShortcut( const KShortcut& cut ) KDE_DEPRECATED;
 	/// @since 3.1
 	void setShortcut( const KShortcut& cut, bool bQtShortcut );
 	const KShortcut& shortcut() const
@@ -70,7 +71,7 @@ class KKeyButton: public QPushButton
  public slots:
 	/**
 	 * Call this method to capture a shortcut from the keyboard.
-	 * If it succeeds, the @ref capturedShortcut() will be emitted.
+	 * If it succeeds, the capturedShortcut() will be emitted.
 	 */
 	void captureShortcut();
 
