@@ -36,6 +36,7 @@
 
 #include <kaccel.h>
 #include <kaction.h>
+#include <kaccelaction.h>
 #include <kactionshortcutlist.h>
 #include <kapplication.h>
 #include <kconfig.h>
@@ -50,11 +51,11 @@
 #include <kaboutdata.h>
 #include <kstaticdeleter.h>
 
-#ifdef Q_WS_X11
+#if defined Q_WS_X11 && ! defined K_WS_QTONLY
 #define XK_XKB_KEYS
 #define XK_MISCELLANY
-#include <X11/Xlib.h>	// For x11Event()
-#include <X11/keysymdef.h> // For XK_...
+#include <X11/Xlib.h>	// For x11Event() // schroder
+#include <X11/keysymdef.h> // For XK_... // schroder
 
 #ifdef KeyPress
 const int XFocusOut = FocusOut;
@@ -65,8 +66,8 @@ const int XKeyRelease = KeyRelease;
 #undef KeyPress
 #undef FocusOut
 #undef FocusIn
-#endif
-#endif
+#endif // KEYPRESS
+#endif // Q_WX_X11 && ! K_WS_QTONLY
 
 //---------------------------------------------------------------------
 // KKeyChooserItem
@@ -116,7 +117,6 @@ class KKeyChooserPrivate
 
 	QMap<QString, KShortcut> mapGlobals;
 
-	bool bAllowWinKey;
 	// If this is set, then shortcuts require a modifier:
 	//  so 'A' would not be valid, whereas 'Ctrl+A' would be.
 	// Note, however, that this only applies to printable characters.
@@ -193,17 +193,17 @@ KKeyChooser::KKeyChooser( KGlobalAccel* actions, QWidget* parent,
 	insert( actions );
 }
 
-// list of all existing KKeyChooser's of type Global
-// used when checking global shortcut for a possible conflict
+// list of all existing KKeyChooser's
+// Used when checking global shortcut for a possible conflict
 // (just checking against kdeglobals isn't enough, the shortcuts
-// might have changed in KKeyChooser and not being saved yet)
-static QValueList< KKeyChooser* >* globalChoosers = NULL;
-static KStaticDeleter< QValueList< KKeyChooser* > > globalChoosersDeleter;
+// might have changed in KKeyChooser and not being saved yet).
+// Also used when reassigning a shortcut from one chooser to another.
+static QValueList< KKeyChooser* >* allChoosers = NULL;
+static KStaticDeleter< QValueList< KKeyChooser* > > allChoosersDeleter;
 
 KKeyChooser::~KKeyChooser()
 {
-        if( m_type == Global && globalChoosers != NULL )
-            globalChoosers->remove( this );
+        allChoosers->remove( this );
 	// Delete allocated KShortcutLists
 	for( uint i = 0; i < d->rgpListsAllocated.count(); i++ )
 		delete d->rgpListsAllocated[i];
@@ -276,7 +276,6 @@ void KKeyChooser::initGUI( ActionType type, bool bAllowLetterShortcuts )
   m_type = type;
   d->bAllowLetterShortcuts = bAllowLetterShortcuts;
 
-  d->bAllowWinKey = (m_type == Global || m_type == ApplicationGlobal);
   d->bPreferFourModifierKeys = KGlobalAccel::useFourModifierKeys();
 
   //
@@ -319,10 +318,11 @@ void KKeyChooser::initGUI( ActionType type, bool bAllowLetterShortcuts )
 
   connect( d->pList, SIGNAL(currentChanged(QListViewItem*)),
            SLOT(slotListItemSelected(QListViewItem*)) );
+
   // handle double clicking an item
   connect ( d->pList, SIGNAL ( doubleClicked ( QListViewItem *, const QPoint &, int ) ),
-                       SLOT ( slotListItemDoubleClicked ( QListViewItem *, const QPoint &, int ) ) );
-
+                       SLOT ( captureCurrentItem()) );
+  connect ( d->pList, SIGNAL ( spacePressed( QListViewItem* )), SLOT( captureCurrentItem()));
   //
   // CREATE CHOOSE KEY GROUP
   //
@@ -404,11 +404,9 @@ void KKeyChooser::initGUI( ActionType type, bool bAllowLetterShortcuts )
   //if (type == Application || type == ApplicationGlobal)
   //  readStdKeys();
   connect( kapp, SIGNAL( settingsChanged( int )), SLOT( slotSettingsChanged( int )));
-  if( m_type == Global ) {
-      if( globalChoosers == NULL )
-          globalChoosers = globalChoosersDeleter.setObject( new QValueList< KKeyChooser* > );
-      globalChoosers->append( this );
-  }
+  if( allChoosers == NULL )
+        allChoosers = allChoosersDeleter.setObject( allChoosers, new QValueList< KKeyChooser* > );
+  allChoosers->append( this );
 }
 
 // Add all shortcuts to the list
@@ -416,7 +414,8 @@ void KKeyChooser::buildListView( uint iList, const QString &title )
 {
 	KShortcutList* pList = d->rgpLists[iList];
 
-	//d->pList->setSorting( -1 );
+        if( m_type == Global || m_type == ApplicationGlobal )
+	    d->pList->setSorting( -1 );
 	KListViewItem *pProgramItem, *pGroupItem = 0, *pParentItem, *pItem;
 
 	QString str = (title.isEmpty() ? i18n("Shortcuts") : title);
@@ -468,7 +467,7 @@ void KKeyChooser::updateButtons()
 		m_prbDef->setEnabled( false );
 		m_prbCustom->setEnabled( false );
 		d->pbtnShortcut->setEnabled( false );
-		d->pbtnShortcut->setShortcut( KShortcut() );
+		d->pbtnShortcut->setShortcut( KShortcut(), false );
 	} else {
 		bool bConfigurable = pItem->isConfigurable();
 		bool bQtShortcut = (m_type == Application || m_type == Standard);
@@ -516,11 +515,8 @@ void KKeyChooser::slotDefaultKey()
 {
 	// return if no key is selected
 	KKeyChooserItem* pItem = dynamic_cast<KKeyChooserItem*>( d->pList->currentItem() );
-	if( pItem ) {
-		pItem->setShortcut( pItem->shortcutDefault() );
-		updateButtons();
-		emit keyChange();
-	}
+	if( pItem ) // don't set it directly, check for conflicts
+		setShortcut( pItem->shortcutDefault() );
 }
 
 void KKeyChooser::slotCustomKey()
@@ -533,10 +529,15 @@ void KKeyChooser::readGlobalKeys()
         d->mapGlobals.clear();
         if( m_type == Global )
             return; // they will be checked normally, because we're configuring them
+        readGlobalKeys( d->mapGlobals );
+}
+
+void KKeyChooser::readGlobalKeys( QMap< QString, KShortcut >& map )
+{
 	QMap<QString, QString> mapEntry = KGlobal::config()->entryMap( "Global Shortcuts" );
 	QMap<QString, QString>::Iterator it( mapEntry.begin() );
 	for( uint i = 0; it != mapEntry.end(); ++it, i++ )
-		d->mapGlobals[it.key()] = KShortcut(*it);
+		map[it.key()] = KShortcut(*it);
 }
 
 void KKeyChooser::slotSettingsChanged( int category )
@@ -554,6 +555,10 @@ void KKeyChooser::fontChange( const QFont & )
         setMinimumWidth( 20+5*(widget_width+10) );
 }
 
+// KDE4 IMHO this shouldn't be here at all - it cannot check whether the default
+// shortcut don't conflict with some already changed ones (e.g. global shortcuts).
+// Also, I personally find reseting all shortcuts to default (i.e. hardcoded in the app)
+// ones after pressing the 'Default' button rather a misfeature.
 void KKeyChooser::allDefault()
 {
 	kdDebug(125) << "KKeyChooser::allDefault()" << endl;
@@ -574,14 +579,16 @@ void KKeyChooser::slotListItemSelected( QListViewItem* )
 	updateButtons();
 }
 
-
 void KKeyChooser::slotListItemDoubleClicked ( QListViewItem *, const QPoint & , int )
-{
+{ // KDE4 dump this
+  captureCurrentItem();
+}
 
+void KKeyChooser::captureCurrentItem()
+{
   KKeyChooserItem* pItem = dynamic_cast<KKeyChooserItem*>( d->pList->currentItem() );
   if( pItem != NULL && pItem->isConfigurable())
       d->pbtnShortcut->captureShortcut ( );
-
 }
 
 void KKeyChooser::setPreferFourModifierKeys( bool bPreferFourModifierKeys )
@@ -650,11 +657,6 @@ void KKeyChooser::setShortcut( const KShortcut& cut )
 		const KKeySequence& seq = cut.seq(i);
 		const KKey& key = seq.key(0);
 
-		if( !d->bAllowWinKey && (key.modFlags() & KKey::WIN) ) {
-			QString s = i18n("The Win key is not allowed in this context.");
-			KMessageBox::sorry( this, s, i18n("Invalid Shortcut Key") );
-			return;
-		}
 		if( !d->bAllowLetterShortcuts && key.modFlags() == 0
 		    && key.sym() < 0x3000 && QChar(key.sym()).isLetterOrNumber() ) {
 			QString s = i18n( 	"In order to use the '%1' key as a shortcut, "
@@ -692,71 +694,175 @@ bool KKeyChooser::isKeyPresent( const KShortcut& cut, bool bWarnUser )
 {
 	KKeyChooserItem* pItem = dynamic_cast<KKeyChooserItem*>(d->pList->currentItem());
 
+	if (!pItem) {
+		return false;
+	}
+
+        bool has_global_chooser = false;
+        bool has_standard_chooser = false;
+        for( QValueList< KKeyChooser* >::ConstIterator it = allChoosers->begin();
+             it != allChoosers->end();
+             ++it ) {
+            has_global_chooser |= ((*it)->m_type == Global);
+            has_standard_chooser |= ((*it)->m_type == Standard);
+        }
+
 	// If editing global shortcuts, check them for conflicts with the stdaccels.
 	if( m_type == ApplicationGlobal || m_type == Global ) {
-		// For each key sequence in the shortcut,
-		for( uint i = 0; i < cut.count(); i++ ) {
-			const KKeySequence& seq = cut.seq(i);
-
-			KStdAccel::StdAccel id = KStdAccel::findStdAccel( seq );
-			if( id != KStdAccel::AccelNone
-			    && keyConflict( cut, KStdAccel::shortcut( id ) ) > -1 ) {
-				if( bWarnUser )
-					_warning( seq, KStdAccel::label(id), i18n("Conflict with Standard Application Shortcut") );
-				return true;
-			}
-		}
-	}
-
-	QMap<QString, KShortcut>::ConstIterator it;
-	for( it = d->mapGlobals.begin(); it != d->mapGlobals.end(); ++it ) {
-		int iSeq = keyConflict( cut, (*it) );
-		if( iSeq > -1 ) {
-			if( m_type != Global || it.key() != pItem->actionName() ) {
-				if( bWarnUser )
-					_warning( cut.seq(iSeq), it.key(), i18n("Conflict with Global Shortcuts") );
-				return true;
-			}
-		}
-	}
-
-        if( isKeyPresentLocally( cut, pItem, bWarnUser ? i18n("Key Conflict") : QString::null ))
-            return true;
-
-        // check also other Global KKeyChooser's
-        if( m_type == Global && globalChoosers != NULL ) {
-            for( QValueList< KKeyChooser* >::ConstIterator it = globalChoosers->begin();
-                 it != globalChoosers->end();
-                 ++it ) {
-                if( (*it) != this && (*it)->isKeyPresentLocally( cut, NULL,
-                    bWarnUser ? i18n("Key Conflict") : QString::null))
+            if( !has_standard_chooser ) {
+                if( checkStandardShortcutsConflict( cut, bWarnUser, this ))
                     return true;
             }
+	}
+
+        // only check the global keys if one of the keychoosers isn't global
+        if( !has_global_chooser ) {
+            if( checkGlobalShortcutsConflict( cut, bWarnUser, this, d->mapGlobals,
+                m_type == Global ? pItem->actionName() : QString::null ))
+                return true;
         }
+
+        if( isKeyPresentLocally( cut, pItem, bWarnUser ))
+            return true;
+
+        // check also other KKeyChooser's
+        for( QValueList< KKeyChooser* >::ConstIterator it = allChoosers->begin();
+             it != allChoosers->end();
+             ++it ) {
+            if( (*it) != this && (*it)->isKeyPresentLocally( cut, NULL, bWarnUser ))
+                    return true;
+            }
 	return false;
 }
 
+// KDE4 remove
 bool KKeyChooser::isKeyPresentLocally( const KShortcut& cut, KKeyChooserItem* ignoreItem, const QString& warnText )
 {
-    if ( cut.toString().isEmpty())
-	       return false;
+    return isKeyPresentLocally( cut, ignoreItem, !warnText.isNull());
+}
 
-		// Search for shortcut conflicts with other actions in the
+bool KKeyChooser::isKeyPresentLocally( const KShortcut& cut, KKeyChooserItem* ignoreItem, bool bWarnUser )
+{
+    if ( cut.toString().isEmpty())
+        return false;
+	// Search for shortcut conflicts with other actions in the
 	//  lists we're configuring.
 	for( QListViewItemIterator it( d->pList ); it.current(); ++it ) {
 		KKeyChooserItem* pItem2 = dynamic_cast<KKeyChooserItem*>(it.current());
 		if( pItem2 && pItem2 != ignoreItem ) {
 			int iSeq = keyConflict( cut, pItem2->shortcut() );
 			if( iSeq > -1 ) {
-				if( !warnText.isNull() )
-					_warning( cut.seq(iSeq), pItem2->text(0), warnText );
-				return true;
+				if( bWarnUser ) {
+                                        if( !promptForReassign( cut.seq(iSeq), pItem2->text(0), Application, this ))
+				                return true;
+                                        // else remove the shortcut from it
+                                        pItem2->setShortcut( KShortcut());
+                                        updateButtons();
+		                        emit keyChange();
+                                }
 			}
 		}
 	}
         return false;
 }
 
+bool KKeyChooser::checkStandardShortcutsConflict( const KShortcut& cut, bool bWarnUser, QWidget* parent )
+{
+    // For each key sequence in the shortcut,
+    for( uint i = 0; i < cut.count(); i++ ) {
+	const KKeySequence& seq = cut.seq(i);
+	KStdAccel::StdAccel id = KStdAccel::findStdAccel( seq );
+	if( id != KStdAccel::AccelNone
+	    && keyConflict( cut, KStdAccel::shortcut( id ) ) > -1 ) {
+		if( bWarnUser ) {
+			if( !promptForReassign( seq, KStdAccel::label(id), Standard, parent ))
+                                return true;
+                        removeStandardShortcut( KStdAccel::label(id), dynamic_cast< KKeyChooser* > ( parent ));
+                }
+	}
+    }
+    return false;
+}
+
+bool KKeyChooser::checkGlobalShortcutsConflict( const KShortcut& cut, bool bWarnUser, QWidget* parent )
+{
+    QMap< QString, KShortcut > map;
+    readGlobalKeys( map );
+    return checkGlobalShortcutsConflict( cut, bWarnUser, parent, map, QString::null );
+}
+
+bool KKeyChooser::checkGlobalShortcutsConflict( const KShortcut& cut, bool bWarnUser, QWidget* parent,
+    const QMap< QString, KShortcut >& map, const QString& ignoreAction )
+{
+    QMap<QString, KShortcut>::ConstIterator it;
+    for( it = map.begin(); it != map.end(); ++it ) {
+	    int iSeq = keyConflict( cut, (*it) );
+	    if( iSeq > -1 ) {
+	    	if( ignoreAction.isEmpty() || it.key() != ignoreAction ) {
+                    if( bWarnUser ) {
+			if( !promptForReassign( cut.seq(iSeq), it.key(), Global, parent ))
+                                    return true;
+                            removeGlobalShortcut( it.key(), dynamic_cast< KKeyChooser* >( parent ));
+                    }
+		}
+	}
+    }
+    return false;
+}
+
+void KKeyChooser::removeStandardShortcut( const QString& name, KKeyChooser* chooser )
+{
+    bool was_in_choosers = false;
+    if( allChoosers != NULL ) {
+        for( QValueList< KKeyChooser* >::ConstIterator it = allChoosers->begin();
+             it != allChoosers->end();
+             ++it ) {
+            if( (*it) != chooser && (*it)->m_type == Standard ) {
+                was_in_choosers |= ( (*it)->resetShortcut( name ));
+            }
+        }
+    }
+    if( !was_in_choosers ) { // not edited, needs to be changed in config file
+        KStdAccel::ShortcutList std_list;
+        std_list.setShortcut( std_list.index( name ), KShortcut());
+        std_list.save();
+    }
+}
+
+void KKeyChooser::removeGlobalShortcut( const QString& name, KKeyChooser* chooser )
+{
+    bool was_in_choosers = false;
+    if( allChoosers != NULL ) {
+        for( QValueList< KKeyChooser* >::ConstIterator it = allChoosers->begin();
+             it != allChoosers->end();
+             ++it ) {
+            if( (*it) != chooser && (*it)->m_type == Global ) {
+                was_in_choosers |= ( (*it)->resetShortcut( name ));
+            }
+        }
+    }
+    if( !was_in_choosers ) { // not edited, needs to be changed in config file
+        KAccelActions actions;
+        actions.insert( name, "", "", KShortcut(), KShortcut());
+	actions.writeActions( "Global Shortcuts", 0, true, true );
+    }
+}
+
+bool KKeyChooser::resetShortcut( const QString& name )
+{
+	for( QListViewItemIterator it( d->pList ); it.current(); ++it ) {
+		KKeyChooserItem* pItem2 = dynamic_cast<KKeyChooserItem*>(it.current());
+                    if( pItem2 && pItem2->actionName() == name ) {
+                        pItem2->setShortcut( KShortcut());
+                        updateButtons();
+		        emit keyChange();
+                        return true;
+                    }
+        }
+        return false;
+}
+
+// KDE4 remove this
 void KKeyChooser::_warning( const KKeySequence& cut, QString sAction, QString sTitle )
 {
 	sAction = sAction.stripWhiteSpace();
@@ -770,6 +876,33 @@ void KKeyChooser::_warning( const KKeySequence& cut, QString sAction, QString sT
 	KMessageBox::sorry( this, s, sTitle );
 }
 
+bool KKeyChooser::promptForReassign( const KKeySequence& cut, const QString& sAction, ActionType type, QWidget* parent )
+{
+        QString sTitle;
+        QString s;
+        if( type == Standard ) {
+                sTitle = i18n("Conflict with Standard Application Shortcut");
+		s = i18n("The '%1' key combination has already been allocated "
+		"to the standard action \"%2\".\n"
+		"Do you want to reassign it from that action to the current one?");
+        }
+        else if( type == Global ) {
+                sTitle = i18n("Conflict with Global Shortcut");
+		s = i18n("The '%1' key combination has already been allocated "
+		"to the global action \"%2\".\n"
+		"Do you want to reassign it from that action to the current one?");
+        }
+        else {
+                sTitle = i18n("Key Conflict");
+		s = i18n("The '%1' key combination has already been allocated "
+		"to the \"%2\" action.\n"
+		"Do you want to reassign it from that action to the current one?");
+        }
+	s = s.arg(cut.toString()).arg(sAction.stripWhiteSpace());
+
+	return KMessageBox::warningYesNo( parent, s, sTitle ) == KMessageBox::Yes;
+}
+
 //---------------------------------------------------
 KKeyChooserItem::KKeyChooserItem( KListView* parent, QListViewItem* after, KShortcutList* pList, uint iAction )
 :	KListViewItem( parent, after )
