@@ -18,90 +18,7 @@
 */
 
 /*************************************************************************
- * $Id: kled.cpp,v 1.23 2002/03/04 00:51:49 lunakl Exp $
- *
- * $Log: kled.cpp,v $
- * Revision 1.23  2002/03/04 00:51:49  lunakl
- * Keep BC changes (the patch is almost 100KiB of boring stuff
- * ... anybody willing to review? ;) ).
- *
- * Revision 1.22  2002/02/27 23:15:45  pfeiffer
- * kapp--
- *
- * Revision 1.21  2002/01/22 10:23:55  hausmann
- * - minor API fix (don't return a const QColor object)
- *
- * Revision 1.20  2001/10/10 17:40:39  mueller
- * CVS_SILENT: fixincludes
- *
- * Revision 1.19  2001/08/08 14:35:12  hausmann
- * - removed empty KActionCollection::childEvent
- * - added sizeHint() and minimumSizeHint() methods to KLed, as advised by
- *   Rik in the comment in kled.h
- * - removed unused mousePressEvent, mouseMoveEvent and mouseReleaseEvent
- *   handlers from KPassDlg
- * - merged KToolBar::insertSeparator() and KToolBar::insertLineSeparator()
- *   with their overloaded variants
- *
- * Revision 1.18  2001/04/16 22:08:43  pfeiffer
- * don't assume that the first item of an enum is 0
- *
- * Revision 1.17  2000/09/12 19:15:53  pfeiffer
- * Stefan Hellwig in cooperation with Joerg Habenicht:
- * Draw nicer LEDs, especially sunken ones.
- *
- * Revision 1.16  2000/08/24 12:44:48  porten
- * "friend class" patches from Thomas Kunert <kunert@physik.tu-dresden.de>
- *
- * Revision 1.15  2000/08/17 16:44:44  reggie
- * Don't crash
- *
- * Revision 1.14  2000/06/03 01:04:42  gehrmab
- * * Made drawing routines available for overriding
- * * Added a parent/name constructor
- * * Propertyfication
- *
- * Revision 1.13  2000/05/08 19:38:49  sschiman
- * Calling setColor before setting up the private data is a bad idea ;-)
- *
- * Revision 1.12  2000/05/07 09:49:57  habenich
- * provided method to set the factor to dark the LED
- * precalculated the dark color, is just retrieved at paint events
- *
- * Revision 1.11  2000/04/09 16:08:33  habenich
- * fixed nasty bug #70 disappearing led
- * reenabled flat and raised led painting
- *
- * Revision 1.10  1999/12/25 17:12:18  mirko
- * Modified Look "round" to "raised", as the others are flat and
- * sunken. All enums start with uppercase letters now to fit the overall
- * KDE style.
- * Implemented raised rectangluar look.
- * --Mirko.
- *
- * Revision 1.9  1999/11/12 21:17:09  antlarr
- * Fixed some bugs.
- * Added the possibility to draw a sunk rectangle as the "old" KLedLamp did.
- *
- * Revision 1.9  1999/11/11 16:08:15  antlarr
- * Fixed some bugs.
- * Added the possibility to draw a sunk rectangle as the "old" KLedLamp did.
- *
- * Revision 1.8  1999/11/01 22:03:15  dmuell
- * fixing all kinds of compile warnings
- * (unused var, unused argument etc)
- *
- * Revision 1.7  1999/10/10 13:34:14  mirko
- * First merge with KLedLamp that shows a rectangular LED.
- * It does not yet work reliably.
- *
- * Revision 1.6  1999/03/01 23:34:49  kulow
- * CVS_SILENT ported to Qt 2.0
- *
- * Revision 1.5  1999/02/19 08:52:42  habenich
- * ID und LOG tags included
- *
- *
+ * $Id: kled.cpp,v 1.26 2003/09/06 19:06:31 binner Exp $
  *************************************************************************/
 
 #define PAINT_BENCH
@@ -262,12 +179,12 @@ KLed::paintFlat() // paint a ROUND FLAT led lamp
    	color = ( led_state ) ? led_color : d->offcolor;
 
 	// Set the brush to SolidPattern, this fills the entire area
-	// of the ellipse which is drawn with a thin grey "border" (pen)
+	// of the ellipse which is drawn with a thin gray "border" (pen)
 	brush.setStyle( QBrush::SolidPattern );
 	brush.setColor( color );
 
 	pen.setWidth( 1 );
-	color.setRgb( 170, 170, 170 );		// This is a grey color value
+	color = colorGroup().dark();
 	pen.setColor( color );			// Set the pen accordingly
 
 	paint.setPen( pen );			// Select pen for drawing
@@ -348,12 +265,12 @@ KLed::paintRound() // paint a ROUND RAISED led lamp
 	pos++; light_width--;
     }
 
-    // Drawing of bright spot finished, now draw a thin grey border
+    // Drawing of bright spot finished, now draw a thin gray border
     // around the LED; it looks nicer that way. We do this here to
     // avoid that the border can be erased by the bright spot of the LED
 
     pen.setWidth( 1 );
-    color.setRgb( 170, 170, 170 );		// This is a grey color value
+    color = colorGroup().dark();
     pen.setColor( color );			// Set the pen accordingly
     paint.setPen( pen );			// Select pen for drawing
     brush.setStyle( QBrush::NoBrush );		// Switch off the brush
@@ -440,32 +357,22 @@ KLed::paintSunken() // paint a ROUND SUNKEN led lamp
     // around the LED which resembles a shadow with light coming
     // from the upper left.
 
-    pen.setWidth( 3 );
+    pen.setWidth( 3 ); // ### shouldn't this value be smaller for smaller LEDs?
     brush.setStyle( QBrush::NoBrush );              // Switch off the brush
     paint.setBrush( brush );                        // This avoids filling of the ellipse
 
-    // Set the initial color value to 200 (bright) and start
+    // Set the initial color value to colorGroup().light() (bright) and start
     // drawing the shadow border at 45 (45*16 = 720).
 
-    int shadow_color = 200, angle;
+    int angle = -720;
+    color = colorGroup().light();
     
-    for ( angle = 720; angle < 6480; angle += 240 ) {
-      color.setRgb( shadow_color, shadow_color, shadow_color );
+    for ( int arc = 120; arc < 2880; arc += 240 ) {
       pen.setColor( color );
       paint.setPen( pen );
-      paint.drawArc( 1, 1, width, width, angle, 240 );
-      if ( angle < 2320 ) {
-	shadow_color -= 25;			// set color to a darker value
-	if ( shadow_color < 100 ) shadow_color = 100;
-      }
-      else if ( ( angle > 2320 ) && ( angle < 5760 ) ) {
-	shadow_color += 25;			// set color to a brighter value
-	if ( shadow_color > 255 ) shadow_color = 255;
-      }
-      else {
-	shadow_color -= 25;			// set color to a darker value again
-	if ( shadow_color < 100 ) shadow_color = 100;
-      }	// end if ( angle < 2320 )
+      paint.drawArc( 1, 1, width, width, angle + arc, 240 );
+      paint.drawArc( 1, 1, width, width, angle - arc, 240 );
+      color = color.dark( 110 ); //FIXME: this should somehow use the contrast value
     }	// end for ( angle = 720; angle < 6480; angle += 160 )
 
     paint.end();
