@@ -17,6 +17,7 @@
    the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
    Boston, MA 02111-1307, USA.
 */
+#include "config.h"
 
 #include <qdragobject.h>
 #include <qtimer.h>
@@ -30,18 +31,18 @@
 #include <kconfig.h>
 #include <kcursor.h>
 #include <kapplication.h>
-#include <kipc.h>
-#include <kdebug.h>
 
-#define private public
-#include <qlistview.h>
-#undef private
+#if defined Q_WS_X11 && ! defined K_WS_QTONLY
+#include <kipc.h> // schroder
+#endif
+
+#include <kdebug.h>
 
 #include "klistview.h"
 #include "klistviewlineedit.h"
 
-#ifdef Q_WS_X11
-#include <X11/Xlib.h>
+#if defined Q_WS_X11 && ! defined K_WS_QTONLY
+#include <X11/Xlib.h> // schroder
 #endif
 
 class KListView::Tooltip : public QToolTip
@@ -86,6 +87,7 @@ public:
       validDrag (false),
       dragEnabled (false),
       autoOpen (true),
+      disableAutoSelection (false),
       dropVisualizer (true),
       dropHighlighter (false),
       createChildren (true),
@@ -120,6 +122,10 @@ public:
   QTimer autoSelect;
   int autoSelectDelay;
 
+  QTimer dragExpand;
+  QListViewItem* dragOverItem;
+  QPoint dragOverPoint;
+
   QPoint startDragPos;
   int dragDelay;
 
@@ -136,6 +142,7 @@ public:
   bool validDrag:1;
   bool dragEnabled:1;
   bool autoOpen:1;
+  bool disableAutoSelection:1;
   bool dropVisualizer:1;
   bool dropHighlighter:1;
   bool createChildren:1;
@@ -182,6 +189,11 @@ KListViewLineEdit::~KListViewLineEdit()
 {
 }
 
+QListViewItem *KListViewLineEdit::currentItem() const
+{
+	return item;
+}
+
 void KListViewLineEdit::load(QListViewItem *i, int c)
 {
         item=i;
@@ -189,7 +201,8 @@ void KListViewLineEdit::load(QListViewItem *i, int c)
 
         QRect rect(p->itemRect(i));
         setText(item->text(c));
-
+        home( true );
+        
         int fieldX = rect.x() - 1;
         int fieldW = p->columnWidth(col) + 2;
 
@@ -219,7 +232,7 @@ void KListViewLineEdit::load(QListViewItem *i, int c)
  *	tabOrderedRename functionality.
  */
 
-int nextCol (KListView *pl, QListViewItem *pi, int start, int dir)
+static int nextCol (KListView *pl, QListViewItem *pi, int start, int dir)
 {
 	if (pi)
 	{
@@ -232,7 +245,7 @@ int nextCol (KListView *pl, QListViewItem *pi, int start, int dir)
 	return -1;
 }
 
-QListViewItem *prevItem (QListViewItem *pi)
+static QListViewItem *prevItem (QListViewItem *pi)
 {
 	QListViewItem *pa = pi->itemAbove();
 
@@ -245,7 +258,7 @@ QListViewItem *prevItem (QListViewItem *pi)
 	return 0;
 }
 
-QListViewItem *lastQChild (QListViewItem *pi)
+static QListViewItem *lastQChild (QListViewItem *pi)
 {
 	if (pi)
 	{
@@ -359,7 +372,8 @@ void KListViewLineEdit::terminate(bool commit)
         col=0;
         item=0;
         hide(); // will call focusOutEvent, that's why we set item=0 before
-        emit done(i,c);
+        if (commit)
+			emit done(i,c);
     }
 }
 
@@ -369,6 +383,8 @@ void KListViewLineEdit::focusOutEvent(QFocusEvent *ev)
     // Don't let a RMB close the editor
     if (focusEv->reason() != QFocusEvent::Popup && focusEv->reason() != QFocusEvent::ActiveWindow)
         terminate(true);
+    else
+        KLineEdit::focusOutEvent(ev);
 }
 
 void KListViewLineEdit::paintEvent( QPaintEvent *e )
@@ -413,11 +429,15 @@ KListView::KListView( QWidget *parent, const char *name )
   if (kapp)
   {
     connect( kapp, SIGNAL( settingsChanged(int) ), SLOT( slotSettingsChanged(int) ) );
+#if defined Q_WS_X11 && ! defined K_WS_QTONLY
     kapp->addKipcEventMask( KIPC::SettingsChanged );
+#endif
   }
 
   connect(&d->autoSelect, SIGNAL( timeout() ),
                   this, SLOT( slotAutoSelect() ) );
+  connect(&d->dragExpand, SIGNAL( timeout() ),
+                  this, SLOT( slotDragExpand() ) );
 
   // context menu handling
   if (d->showContextMenusOnPress)
@@ -436,8 +456,6 @@ KListView::KListView( QWidget *parent, const char *name )
   d->alternateBackground = KGlobalSettings::alternateBackgroundColor();
 }
 
-
-
 KListView::~KListView()
 {
   delete d;
@@ -502,7 +520,8 @@ void KListView::slotSettingsChanged(int category)
                this, SLOT (slotMouseButtonClicked( int, QListViewItem*, const QPoint &, int)));
 
     d->bChangeCursorOverItem = KGlobalSettings::changeCursorOverIcon();
-    d->autoSelectDelay = KGlobalSettings::autoSelectDelay();
+    if ( !d->disableAutoSelection )
+      d->autoSelectDelay = KGlobalSettings::autoSelectDelay();
 
     if( !d->bUseSingle || !d->bChangeCursorOverItem )
        viewport()->unsetCursor();
@@ -550,7 +569,7 @@ void KListView::slotAutoSelect()
   if( !hasFocus() )
     setFocus();
 
-#ifdef Q_WS_X11
+#if defined Q_WS_X11 && ! defined K_WS_QTONLY
   // FIXME(E): Implement for Qt Embedded
   Window root;
   Window child;
@@ -563,7 +582,8 @@ void KListView::slotAutoSelect()
   QListViewItem* previousItem = currentItem();
   setCurrentItem( d->pCurrentItem );
 
-#ifndef Q_WS_QWS
+//#ifndef Q_WS_QWS
+#if defined Q_WS_X11 && ! defined K_WS_QTONLY
   // FIXME(E): Implement for Qt Embedded
   if( d->pCurrentItem ) {
     //Shift pressed?
@@ -643,7 +663,8 @@ void KListView::emitExecute( QListViewItem *item, const QPoint &pos, int c )
         }
         else
         {
-#ifndef Q_WS_QWS
+//#ifndef Q_WS_QWS
+#if defined Q_WS_X11 && ! defined K_WS_QTONLY
 	    // FIXME(E): Implement for Qt Embedded
             Window root;
             Window child;
@@ -867,6 +888,7 @@ void KListView::contentsDropEvent(QDropEvent* e)
 {
   cleanDropVisualizer();
   cleanItemHighlighter();
+  d->dragExpand.stop();
 
   if (acceptDrag (e))
   {
@@ -945,6 +967,17 @@ void KListView::contentsDragMoveEvent(QDragMoveEvent *event)
     //Clean up the view
 
     findDrop(event->pos(), d->parentItemDrop, d->afterItemDrop);
+    QPoint vp = contentsToViewport( event->pos() );
+    QListViewItem *item = isExecuteArea( vp ) ? itemAt( vp ) : 0L;
+
+    if ( item != d->dragOverItem )
+    {
+      d->dragExpand.stop();
+      d->dragOverItem = item;
+      d->dragOverPoint = vp;
+      if ( d->dragOverItem && d->dragOverItem->isExpandable() && !d->dragOverItem->isOpen() )
+        d->dragExpand.start( QApplication::startDragTime(), true );
+    }
     if (dropVisualizer())
     {
       QRect tmpRect = drawDropVisualizer(0, d->parentItemDrop, d->afterItemDrop);
@@ -970,8 +1003,15 @@ void KListView::contentsDragMoveEvent(QDragMoveEvent *event)
       event->ignore();
 }
 
+void KListView::slotDragExpand()
+{
+  if ( itemAt( d->dragOverPoint ) == d->dragOverItem )
+    d->dragOverItem->setOpen( true );
+}
+
 void KListView::contentsDragLeaveEvent (QDragLeaveEvent*)
 {
+  d->dragExpand.stop();
   cleanDropVisualizer();
   cleanItemHighlighter();
 }
@@ -1158,8 +1198,12 @@ void KListView::setDropVisualizer(bool b)
 QPtrList<QListViewItem> KListView::selectedItems() const
 {
   QPtrList<QListViewItem> list;
-  for (QListViewItem *i=firstChild(); i!=0; i=i->itemBelow())
-        if (i->isSelected()) list.append(i);
+
+  QListViewItemIterator it(const_cast<KListView *>(this), QListViewItemIterator::Selected);
+
+  for(; it.current(); ++it)
+      list.append(it.current());
+
   return list;
 }
 
@@ -1175,6 +1219,16 @@ void KListView::moveItem(QListViewItem *item, QListViewItem *parent, QListViewIt
       i = i->parent();
     }
 
+  if (after)
+  {
+      item->moveItem(after);
+      return;
+  }
+
+  // NOTE: This code shouldn't ever be reached if this method is used proprely,
+  // QListVIew::moveItem() handles the same cases.  However, to avoid changing the (albeit
+  // undocumented behavior) it's being left in for the moment.
+
   // Basically reimplementing the QListViewItem(QListViewItem*, QListViewItem*) constructor
   // in here, without ever deleting the item.
   if (item->parent())
@@ -1186,9 +1240,6 @@ void KListView::moveItem(QListViewItem *item, QListViewItem *parent, QListViewIt
         parent->insertItem(item);
   else
         insertItem(item);
-
-  if (after)
-        item->moveToJustAfter(after);
 }
 
 void KListView::contentsDragEnterEvent(QDragEnterEvent *event)
@@ -1258,12 +1309,8 @@ QRect KListView::drawItemHighlighter(QPainter *painter, QListViewItem *item)
     r = itemRect(item);
     r.setLeft(r.left()+(item->depth()+1)*treeStepSize());
     if (painter)
-#if QT_VERSION < 300
-      style().drawFocusRect(painter, r, colorGroup(), &colorGroup().highlight(), true);
-#else
       style().drawPrimitive(QStyle::PE_FocusRect, painter, r, colorGroup(),
                             QStyle::Style_FocusAtBorder, colorGroup().highlight());
-#endif
   }
 
   return r;
@@ -1283,8 +1330,8 @@ void KListView::rename(QListViewItem *item, int c)
 {
   if (d->renameable.contains(c))
   {
-  ensureItemVisible(item);
-  d->editor->load(item,c);
+    ensureItemVisible(item);
+    d->editor->load(item,c);
   }
 }
 
@@ -1415,10 +1462,10 @@ void KListView::fileManagerKeyPressEvent (QKeyEvent* e)
         && (e->key()!=Key_Meta) && (e->key()!=Key_Alt))
     {
        if ((e_state==ShiftButton) && (!d->wasShiftEvent) && (!d->selectedBySimpleMove))
-          selectAll(FALSE);
+          selectAll(false);
        d->selectionDirection=0;
        d->wasShiftEvent = (e_state == ShiftButton);
-    }
+    };
 
     //d->wasShiftEvent = (e_state == ShiftButton);
 
@@ -1426,8 +1473,6 @@ void KListView::fileManagerKeyPressEvent (QKeyEvent* e)
     QListViewItem* item = currentItem();
     if (item==0) return;
 
-    setUpdatesEnabled(false);
-
     QListViewItem* repaintItem1 = item;
     QListViewItem* repaintItem2 = 0L;
     QListViewItem* visItem = 0L;
@@ -1460,8 +1505,8 @@ void KListView::fileManagerKeyPressEvent (QKeyEvent* e)
     switch (e->key())
     {
     case Key_Escape:
-       selectAll(FALSE);
-       emitSelectionChanged=TRUE;
+       selectAll(false);
+       emitSelectionChanged=true;
        break;
 
     case Key_Space:
@@ -1469,7 +1514,7 @@ void KListView::fileManagerKeyPressEvent (QKeyEvent* e)
        if (d->selectedBySimpleMove)
           d->selectedBySimpleMove=false;
        item->setSelected(!item->isSelected());
-       emitSelectionChanged=TRUE;
+       emitSelectionChanged=true;
        break;
 
     case Key_Insert:
@@ -1477,7 +1522,7 @@ void KListView::fileManagerKeyPressEvent (QKeyEvent* e)
        if (d->selectedBySimpleMove)
        {
           d->selectedBySimpleMove=false;
-          if (!item->isSelected()) item->setSelected(TRUE);
+          if (!item->isSelected()) item->setSelected(true);
        }
        else
        {
@@ -1493,7 +1538,7 @@ void KListView::fileManagerKeyPressEvent (QKeyEvent* e)
           setCurrentItem(nextItem);
        };
        d->selectionDirection=1;
-       emitSelectionChanged=TRUE;
+       emitSelectionChanged=true;
        break;
 
     case Key_Down:
@@ -1509,14 +1554,14 @@ void KListView::fileManagerKeyPressEvent (QKeyEvent* e)
              if (oldSelectionDirection!=-1)
              {
                 item->setSelected(!item->isSelected());
-                emitSelectionChanged=TRUE;
+                emitSelectionChanged=true;
              };
           };
        }
        else if ((d->selectedBySimpleMove) && (nextItem!=0))
        {
           item->setSelected(false);
-          emitSelectionChanged=TRUE;
+          emitSelectionChanged=true;
        };
 
        if (nextItem!=0)
@@ -1544,14 +1589,14 @@ void KListView::fileManagerKeyPressEvent (QKeyEvent* e)
              if (oldSelectionDirection!=1)
              {
                 item->setSelected(!item->isSelected());
-                emitSelectionChanged=TRUE;
+                emitSelectionChanged=true;
              };
           }
        }
        else if ((d->selectedBySimpleMove) && (nextItem!=0))
        {
           item->setSelected(false);
-          emitSelectionChanged=TRUE;
+          emitSelectionChanged=true;
        };
 
        if (nextItem!=0)
@@ -1586,32 +1631,29 @@ void KListView::fileManagerKeyPressEvent (QKeyEvent* e)
           }
           nextItem=nextItem->itemBelow();
        }
-       emitSelectionChanged=TRUE;
+       emitSelectionChanged=true;
        break;
 
     case Key_Home:
-       //move to the last item and toggle selection of all items inbetween
-       nextItem=item;
+       // move to the first item and toggle selection of all items inbetween
+       nextItem = firstChild();
+       visItem = nextItem;
+       repaintItem2 = visItem;
        if (d->selectedBySimpleMove)
           item->setSelected(false);
        if (shiftOrCtrl)
+       {
           d->selectedBySimpleMove=false;
 
-       while(nextItem!=0)
-       {
-          if (shiftOrCtrl)
-             nextItem->setSelected(!nextItem->isSelected());
-          if (nextItem->itemAbove()==0)
+          while ( nextItem != item )
           {
-             if (d->selectedBySimpleMove)
-                nextItem->setSelected(true);
-             repaintItem2=nextItem;
-             visItem=nextItem;
-             setCurrentItem(nextItem);
+             nextItem->setSelected( !nextItem->isSelected() );
+             nextItem = nextItem->itemBelow();
           }
-          nextItem=nextItem->itemAbove();
+          item->setSelected( !item->isSelected() );
        }
-       emitSelectionChanged=TRUE;
+       setCurrentItem( firstChild() );
+       emitSelectionChanged=true;
        break;
 
     case Key_Next:
@@ -1637,7 +1679,6 @@ void KListView::fileManagerKeyPressEvent (QKeyEvent* e)
                 nextItem->setSelected(!nextItem->isSelected());
              if (d->selectedBySimpleMove)
                 nextItem->setSelected(true);
-             setUpdatesEnabled(true);
              ensureItemVisible(nextItem);
              setCurrentItem(nextItem);
              update();
@@ -1672,7 +1713,6 @@ void KListView::fileManagerKeyPressEvent (QKeyEvent* e)
           {
              if (d->selectedBySimpleMove)
                 nextItem->setSelected(true);
-             setUpdatesEnabled(true);
              ensureItemVisible(nextItem);
              setCurrentItem(nextItem);
              update();
@@ -1688,11 +1728,11 @@ void KListView::fileManagerKeyPressEvent (QKeyEvent* e)
 
     case Key_Minus:
        if ( item->isOpen() )
-          setOpen( item, FALSE );
+          setOpen( item, false );
        break;
     case Key_Plus:
        if (  !item->isOpen() && (item->isExpandable() || item->childCount()) )
-          setOpen( item, TRUE );
+          setOpen( item, true );
        break;
     default:
        bool realKey = ((e->key()!=Key_Shift) && (e->key()!=Key_Control)
@@ -1709,7 +1749,7 @@ void KListView::fileManagerKeyPressEvent (QKeyEvent* e)
        if (realKey && selectCurrentItem)
        {
           currentItem()->setSelected(true);
-          emitSelectionChanged=TRUE;
+          emitSelectionChanged=true;
        }
        repaintItem2=currentItem();
        if (realKey)
@@ -1717,7 +1757,6 @@ void KListView::fileManagerKeyPressEvent (QKeyEvent* e)
        break;
     }
 
-    setUpdatesEnabled(true);
     if (visItem)
        ensureItemVisible(visItem);
 
@@ -1731,7 +1770,7 @@ void KListView::fileManagerKeyPressEvent (QKeyEvent* e)
     {                 // rectangle to be repainted
        if ( ir.x() < 0 )
           ir.moveBy( -ir.x(), 0 );
-       viewport()->repaint( ir, FALSE );
+       viewport()->repaint( ir, false );
     }
     /*if (repaintItem1)
        repaintItem1->repaint();
@@ -1855,12 +1894,8 @@ void KListView::viewportPaintEvent(QPaintEvent *e)
       QPainter painter(viewport());
 
       // This is where we actually draw the drop-highlighter
-#if QT_VERSION < 300
-      style().drawFocusRect(&painter, d->mOldDropHighlighter, colorGroup(), 0, true);
-#else
       style().drawPrimitive(QStyle::PE_FocusRect, &painter, d->mOldDropHighlighter, colorGroup(),
                             QStyle::Style_FocusAtBorder);
-#endif
     }
   d->painting = false;
 }
@@ -1971,7 +2006,32 @@ bool KListView::ascendingSort(void) const
   return d->sortAscending;
 }
 
+void KListView::takeItem(QListViewItem *item)
+{
+  if(item && item == d->editor->currentItem())
+    d->editor->terminate();
 
+  QListView::takeItem(item);
+}
+
+void KListView::disableAutoSelection()
+{
+  if ( d->disableAutoSelection )
+    return;
+
+  d->disableAutoSelection = true;
+  d->autoSelect.stop();
+  d->autoSelectDelay = -1;
+}
+
+void KListView::resetAutoSelection()
+{
+  if ( !d->disableAutoSelection )
+    return;
+
+  d->disableAutoSelection = false;
+  d->autoSelectDelay = KGlobalSettings::autoSelectDelay();
+}
 
 
 
@@ -2037,7 +2097,7 @@ KListViewItem::~KListViewItem()
 
 void KListViewItem::init()
 {
-  m_known = m_odd = false;
+  m_odd = m_known = false;
   KListView *lv = static_cast<KListView *>(listView());
   setDragEnabled( dragEnabled() || lv->dragEnabled() );
 }
@@ -2065,7 +2125,7 @@ bool KListViewItem::isAlternate()
     // said painting is happening top to bottem -- this assumption is present
     // elsewhere in the implementation of this class, (2) itemBelow() is fast --
     // roughly constant time.
-    // 
+    //
     // Given these assumptions we can do a mixture of caching and telling the
     // next item that the when that item is the current item that the now
     // current item will be the item above it.
@@ -2076,18 +2136,12 @@ bool KListViewItem::isAlternate()
     if (lv->d->painting) {
       if (lv->d->paintCurrent != this)
       {
-        lv->d->paintAbove = lv->d->paintBelow == this ? lv->d->paintCurrent : 0;
+        lv->d->paintAbove = lv->d->paintBelow == this ? lv->d->paintCurrent : itemAbove();
         lv->d->paintCurrent = this;
         lv->d->paintBelow = itemBelow();
       }
 
       above = dynamic_cast<KListViewItem *>(lv->d->paintAbove);
-
-      if (!above && !lv->d->paintAbove)
-      {
-        above = dynamic_cast<KListViewItem *>(itemAbove());
-        lv->d->paintAbove = above;
-      }
     }
     else
     {
@@ -2134,7 +2188,8 @@ void KListViewItem::paintCell(QPainter *p, const QColorGroup &cg, int column, in
   if (pm && !pm->isNull())
   {
         _cg.setBrush(QColorGroup::Base, QBrush(backgroundColor(), *pm));
-        p->setBrushOrigin( -listView()->contentsX(), -listView()->contentsY() );
+        QPoint o = p->brushOrigin();
+        p->setBrushOrigin( o.x()-listView()->contentsX(), o.y()-listView()->contentsY() );
   }
   else if (isAlternate())
        if (listView()->viewport()->backgroundMode()==Qt::FixedColor)
