@@ -23,6 +23,7 @@
 #include <qlistview.h>
 
 #include <qptrlist.h>
+#include <kdemacros.h>
 
 class QDragObject;
 class KConfig;
@@ -33,23 +34,21 @@ class KLineEdit;
  * ChangeCursorOverLink (TM).
  *
  * There is a new signal executed(). It gets connected to either
- * @ref QListView::clicked() or  @ref QListView::doubleClicked() depending on the KDE
+ * QListView::clicked() or QListView::doubleClicked() depending on the KDE
  * wide Single Click/Double Click settings. It is strongly recommended that
  * you use this signal instead of the above mentioned. This way you dont
  * need to care about the current settings.
  * If you want to get informed when the user selects something connect to the
  * QListView::selectionChanged() signal.
  *
- * Drag-and-Drop is supported with the signal @ref #dropped(), just @ref #setAcceptDrops(true)
+ * Drag-and-Drop is supported with the signal dropped(), just setAcceptDrops(true)
  * and connect it to a suitable slot.
- * To see where you are dropping, @ref setDropVisualizer(true).
- * And also you'll need @ref acceptDrag(QDropEvent*)
+ * To see where you are dropping, setDropVisualizer(true).
+ * And also you'll need acceptDrag(QDropEvent*)
  *
  * KListView is drag-enabled, too: to benefit from that you've got derive from it.
- * Reimplement @ref dragObject() and (possibly) @ref startDrag(),
- * and @ref setDragEnabled(true).
- *
- * @version $Id: klistview.h,v 1.92.2.3 2003/09/03 11:22:19 wheeler Exp $
+ * Reimplement dragObject() and (possibly) startDrag(),
+ * and setDragEnabled(true).
  */
 class KListView : public QListView
 {
@@ -58,6 +57,15 @@ class KListView : public QListView
   Q_OBJECT
   Q_ENUMS( SelectionModeExt )
   Q_PROPERTY( bool fullWidth READ fullWidth WRITE setFullWidth )
+  Q_PROPERTY( bool itemsMovable READ itemsMovable WRITE setItemsMovable )
+  Q_PROPERTY( bool itemsRenameable READ itemsRenameable WRITE setItemsRenameable )
+  Q_PROPERTY( bool dragEnabled READ dragEnabled  WRITE  setDragEnabled )
+  Q_PROPERTY( bool autoOpen READ autoOpen WRITE setAutoOpen )
+  Q_PROPERTY( bool dropVisualizer READ dropVisualizer WRITE setDropVisualizer )
+  Q_PROPERTY( int tooltipColumn READ tooltipColumn WRITE  setTooltipColumn )
+  Q_PROPERTY( int dropVisualizerWidth READ dropVisualizerWidth WRITE  setDropVisualizerWidth )
+  Q_PROPERTY( QColor alternateBackground READ alternateBackground WRITE  setAlternateBackground )
+
   Q_OVERRIDE( SelectionModeExt selectionMode READ selectionModeExt WRITE setSelectionModeExt )
 
 public:
@@ -88,7 +96,7 @@ public:
    *    before will be deselected first.
    *
    *    Additionally the current item is always selected automatically when
-   *    navigating using the keyboard, except other items were selected explicitely.
+   *    navigating using the keyboard, except other items were selected explicitly.
    *
    *   This way e.g. SHIFT+up/PgUp then SHIFT+down/PgDn leaves no item selected
    */
@@ -104,7 +112,7 @@ public:
    * Constructor.
    *
    * The parameters @p parent and @p name are handled by
-   * @ref QListView, as usual.
+   * QListView, as usual.
    */
   KListView (QWidget *parent = 0, const char *name = 0);
 
@@ -124,9 +132,9 @@ public:
 
   /**
    * This function determines whether the given coordinates are within the
-   * execute area. The execute area is the part of a @ref QListViewItem where mouse
-   * clicks or double clicks respectively generate a @ref #executed() signal.
-   * Depending on @ref QListView::allColumnsShowFocus() this is either the
+   * execute area. The execute area is the part of a QListViewItem where mouse
+   * clicks or double clicks respectively generate a executed() signal.
+   * Depending on QListView::allColumnsShowFocus() this is either the
    * whole item or only the first column.
    * @return true if point is inside execute area of an item, false in all
    * other cases including the case that it is over the viewport.
@@ -143,12 +151,16 @@ public:
   /**
    * @return a list containing the currently selected items.
    */
-  QPtrList<QListViewItem> selectedItems() const;
+  QPtrList<QListViewItem> selectedItems() const; // ### BIC: KDE 4: use an implicitly shared class! (QValueList?)
 
   /**
    * Arbitrarily move @p item to @p parent, positioned immediately after item @p after.
+   *
+   * @see QListViewItem::moveItem()
+   *
+   * @deprecated
    */
-  void moveItem(QListViewItem *item, QListViewItem *parent, QListViewItem *after);
+  void moveItem(QListViewItem *item, QListViewItem *parent, QListViewItem *after) KDE_DEPRECATED;
 
   /**
    * @return the last item (not child!) of this listview.
@@ -159,51 +171,66 @@ public:
 
   /**
    * @return the last child of this listview.
+   *
+   * @see lastItem()
    */
   QListViewItem* lastChild () const;
 
   /**
    * @return the lineedit used for inline renaming.
-   * Use that to setup a @ref KCompletion or @ref QValidator for the lineedit
+   * Use that to setup a KCompletion or QValidator for the lineedit
+   *
+   * @since 3.2
    */
   KLineEdit* renameLineEdit() const;
 
   /**
-   * @returns if it is legal to move items in the list view
+   * @returns if it is legal to move items in the list view.  True by default.
+   *
+   * @see setDragEnabled()
+   * @see setItemsMovable()
    */
   bool itemsMovable() const;
 
   /**
-   * @return whether inplace-renaming has been enabled.
+   * @return whether inplace-renaming has been enabled.  False by default.
    *
-   * @see #setItemsRenameable()
+   * @see setItemsRenameable()
    */
   bool itemsRenameable() const;
 
   /**
-   * @return whether dragging is enabled.
+   * @return whether dragging is enabled.  False by default.
+   *
+   * @see setDragEnabled()
    */
   bool dragEnabled() const;
 
   /**
    * @return true if AutoOpen is enabled (not implemented currently).
    *
-   * @see #setAutoOpen()
+   * @see setAutoOpen()
    */
   bool autoOpen() const;
 
   /**
    * @return true if @p column is renamable.
+   *
+   * @see setRenameable()
    */
   bool isRenameable (int column) const;
 
   /**
-   * @return true if drawing of the drop-visualizer has been enabled.
+   * @return true if drawing of the drop-visualizer has been enabled.  True by default.
+   *
+   * @see setDropVisualizer()
    */
   bool dropVisualizer() const;
 
   /**
    * @return the column for which tooltips are displayed (or -1 if none set).
+   *
+   * @see setTooltipColumn()
    */
   int tooltipColumn() const;
 
@@ -213,16 +240,19 @@ public:
    * Do not use.
    * @deprecated
    */
-  bool createChildren() const;
+  bool createChildren() const KDE_DEPRECATED;
 
   /**
-   * @return true if drawing of the drop-highlighter has been enabled.
+   * @return true if drawing of the drop-highlighter has been enabled.  False by default.
+   *
+   * @see setDropHighlighter()
    */
   bool dropHighlighter() const;
 
   /**
    * The dropVisualizerWidth defaults to 4.
    *
+   * @see setDropVisualizerWidth()
    * @return the current width of the drop-visualizer.
    */
   int dropVisualizerWidth () const;
@@ -251,18 +281,28 @@ public:
 
   /**
    * @deprecated
+   * @see setFullWidth()
    */
-  void setFullWidth();
+  void setFullWidth() KDE_DEPRECATED;
+
   /**
    * Let the last column fit exactly all the available width.
+   *
+   * @see fullWidth()
    */
   void setFullWidth(bool fullWidth);
+
   /**
    * Returns whether the last column is set to fit the available width.
+   *
+   * @see setFullWidth()
    */
   bool fullWidth() const;
+
   /**
    * Reimplemented for full width support
+   *
+   * @see removeColumn()
    */
   virtual int addColumn(const QString& label, int width = -1);
   /**
@@ -271,6 +311,8 @@ public:
   virtual int addColumn(const QIconSet& iconset, const QString& label, int width = -1);
   /**
    * Reimplemented for full width support
+   *
+   * @see addColumn()
    */
   virtual void removeColumn(int index);
 
@@ -279,11 +321,15 @@ public:
    * This only has an effect if the items are KListViewItems
    *
    * @param c the color to use for every other item. Set to an invalid
-   *        colour to disable alternate colours.
-   */
+   *        color to disable alternate colors.
+   *
+   * @see alternateBackground()
+   **/
   void setAlternateBackground(const QColor &c);
   /**
    * @return the alternate background color
+   *
+   * @see setAlternateBackground()
    */
   const QColor &alternateBackground() const;
 
@@ -291,20 +337,23 @@ public:
    * Saves the list view's layout (column widtsh, column order, sort column)
    * to a KConfig group
    *
-   * @param config the @ref KConfig object to write to
+   * @param config the KConfig object to write to
    * @param group the config group to use
    */
   void saveLayout(KConfig *config, const QString &group) const;
   /**
    * Reads the list view's layout from a KConfig group as stored with
-   * @ref #saveLayout
+   * saveLayout
    *
-   * @param config the @ref KConfig object to read from
+   * @param config the KConfig object to read from
    * @param group the config group to use
    */
   void restoreLayout(KConfig *config, const QString &group);
   /**
-   * Reimplemented to remember the current sort column and order
+   * Reimplemented to remember the current sort column and order.
+   * @param column is the column to be sorted, or -1 to sort in order of
+   * insertion
+   * @param ascending whether to sort ascending (or descending)
    */
   virtual void setSorting(int column, bool ascending = true);
 
@@ -318,6 +367,11 @@ public:
    */
   bool ascendingSort(void) const;
 
+  /**
+   * Reimplemented for internal reasons.
+   */
+  virtual void takeItem(QListViewItem *i);
+
 signals:
 
   /**
@@ -326,7 +380,7 @@ signals:
    * setting the user clicked or double clicked on that item.
    * @param item is the pointer to the executed listview item.
    *
-   * Note that you may not delete any @ref QListViewItem objects in slots
+   * Note that you may not delete any QListViewItem objects in slots
    * connected to this signal.
    */
   void executed( QListViewItem *item );
@@ -339,7 +393,7 @@ signals:
    * @param pos is the position where the user has clicked
    * @param c is the column into which the user clicked.
    *
-   * Note that you may not delete any @ref QListViewItem objects in slots
+   * Note that you may not delete any QListViewItem objects in slots
    * connected to this signal.
    */
   void executed( QListViewItem *item, const QPoint &pos, int c );
@@ -351,13 +405,14 @@ signals:
    * @param pos is the position where the user has clicked, and
    * @param c is the column into which the user clicked.
    *
-   * Note that you may not delete any @ref QListViewItem objects in slots
+   * Note that you may not delete any QListViewItem objects in slots
    * connected to this signal.
    *
    * This signal is more or less here for the sake of completeness.
    * You should normally not need to use this. In most cases its better
-   * to use @ref #executed() instead.
+   * to use executed() instead.
    */
+  // KDE 4: Remove this signal...already in QListView
   void doubleClicked( QListViewItem *item, const QPoint &pos, int c );
 
   /**
@@ -365,10 +420,10 @@ signals:
    * dropped onto the listview.
    *
    * @param e is the drop event itself (it has already been accepted)
-   * @param after is the item after which the drop occured (or 0L, if
+   * @param after is the item after which the drop occurred (or 0L, if
    * the drop was above all items)
    *
-   * @see #acceptDrop()
+   * @see acceptDrop()
    */
   void dropped (QDropEvent * e, QListViewItem *after);
 
@@ -381,7 +436,7 @@ signals:
    *
    * @param list is the listview
    * @param e is the drop event itself (it has already been accepted)
-   * @param after is the item after which the drop occured (or 0L, if
+   * @param after is the item after which the drop occurred (or 0L, if
    * the drop was above all items
    */
   void dropped (KListView* list, QDropEvent* e, QListViewItem* after);
@@ -395,7 +450,7 @@ signals:
    * @param list is the listview
    * @param e is the drop event itself (it has already been accepted)
    * @param parent the item that is to be the parent of the new item
-   * @param after is the item after which the drop occured (or 0L, if
+   * @param after is the item after which the drop occurred (or 0L, if
    * the drop was above all items
    */
   void dropped (KListView* list, QDropEvent* e, QListViewItem* parent, QListViewItem* after);
@@ -408,7 +463,7 @@ signals:
    * is a tree
    * @param e is the drop event itself (it has already been accepted)
    * @param parent the item that is to be the parent of the new item
-   * @param after is the item after which the drop occured (or 0L, if
+   * @param after is the item after which the drop occurred (or 0L, if
    * the drop was above all items
    */
   void dropped (QDropEvent* e, QListViewItem* parent, QListViewItem* after);
@@ -421,28 +476,27 @@ signals:
    */
   void moved();
 
-	/**
-	 * Connect to this signal if you want to do some preprocessing before
-	 * a move is made, for example, to disable sorting
-	 *
-	 * This is sent only once per each groups of moves.  That is, for each
-	 * drop that is a move this will be emitted once, before KListView calls
-	 * @ref moveItem()
-	 **/
-	void aboutToMove();
+  /**
+   * Connect to this signal if you want to do some preprocessing before
+   * a move is made, for example, to disable sorting
+   *
+   * This is sent only once per each groups of moves.  That is, for each
+   * drop that is a move this will be emitted once, before KListView calls
+   * @see moveItem()
+   */
+  void aboutToMove();
 
   /**
    * This signal is emitted when ever the user moves an item in the list via
    * DnD.
    * If more than one item is moved at the same time, @p afterFirst and
    * @p afterNow will reflect what was true before the move.
-   * This differs from @ref moved(), so be careful. All the items will have been
-   * moved before @ref moved() is emitted, which is not true in this method. // FIXME
+   * This differs from moved(), so be careful. All the items will have been
+   * moved before moved() is emitted, which is not true in this method. // FIXME
    * @param item the item that was moved
    * @param afterFirst the item that parameter item was in before the move, in the list
    * @param afterNow the item it's currently after.
-   *
-   **/
+   */
   void moved (QListViewItem *item, QListViewItem *afterFirst, QListViewItem *afterNow);
 
 
@@ -450,7 +504,7 @@ signals:
    * This signal is emitted after all the items have been moved. It reports info for
    * each and every item moved, in order.  The first element in @p items associates
    * with the first of afterFirst and afterNow.
-   **/
+   */
   void moved(QPtrList<QListViewItem> &items, QPtrList<QListViewItem> &afterFirst, QPtrList<QListViewItem> &afterNow);
 
   /**
@@ -471,11 +525,11 @@ signals:
    * This signal is emitted when the shortcut key for popup-menus is pressed.
    *
    * Normally you should not use this, just connect a slot to signal
-   * @ref contextMenu (KListView*, QListViewItem*, const QPoint&) to correctly
+   * contextMenu (KListView*, QListViewItem*, const QPoint&) to correctly
    * handle showing context menus regardless of settings.
    *
    * @param list is this listview.
-   * @param item is the @ref currentItem() at the time the key was pressed. May be 0L.
+   * @param item is the currentItem() at the time the key was pressed. May be 0L.
    */
   void menuShortCutPressed (KListView* list, QListViewItem* item);
 
@@ -502,9 +556,9 @@ public slots:
    *
    * If you want more intelligent (dynamic) selection,
    * you'll have to derive from KListView,
-   * and override @ref rename() and call only call it
+   * and override rename() and call only call it
    * if you want the item to be renamed.
-   **/
+   */
   void setRenameable (int column, bool yesno=true);
 
   /**
@@ -569,12 +623,12 @@ public slots:
    * Do not use.
    * @deprecated
    */
-  virtual void setCreateChildren(bool b);
+  virtual void setCreateChildren(bool b) KDE_DEPRECATED;
 
   /**
    * Set the selection mode.
    *
-   * A different name was chosen to avoid API-clashes with @ref QListView::setSelectionMode().
+   * A different name was chosen to avoid API-clashes with QListView::setSelectionMode().
    */
   void setSelectionModeExt (SelectionModeExt mode);
 
@@ -625,7 +679,7 @@ protected:
   virtual bool event( QEvent * );
 
   /**
-   * Emit signal @ref #executed.
+   * Emit signal executed.
    * @internal
    */
   void emitExecute( QListViewItem *item, const QPoint &pos, int c );
@@ -771,7 +825,7 @@ protected:
   virtual QRect drawItemHighlighter(QPainter *painter, QListViewItem *item);
 
   /**
-   * This method calls @ref dragObject() and starts the drag.
+   * This method calls dragObject() and starts the drag.
    *
    * Reimplement it to do fancy stuff like setting a pixmap or
    * using a non-default DragMode
@@ -797,19 +851,19 @@ protected:
   virtual void viewportPaintEvent(QPaintEvent*);
 
   /**
-   * In FileManager selection mode: explicitely activate the mode
+   * In FileManager selection mode: explicitly activate the mode
    * in which the current item is automatically selected.
    */
   void activateAutomaticSelection();
   /**
-   * In FileManager selection mode: explicitely deactivate the mode
+   * In FileManager selection mode: explicitly deactivate the mode
    * in which the current item is automatically selected.
    */
   void deactivateAutomaticSelection();
   /**
    * In FileManager selection mode: return whether it is currently in the mode
    * where the current item is selected automatically.
-   * Returns false if items were selected explicitely, e.g. using the mouse.
+   * Returns false if items were selected explicitly, e.g. using the mouse.
    */
   bool automaticSelection() const;
 
@@ -818,6 +872,22 @@ protected:
    */
   virtual void viewportResizeEvent(QResizeEvent* e);
 
+  /**
+   * Disable AutoSelection. This overrides the system wide setting for
+   * AutoSelection. Please don't call this unless you have good reasons to
+   * override the system wide setting for AutoSelection.
+   * @see resetAutoSelection()
+   * @since 3.2
+   */
+  void disableAutoSelection();
+
+  /**
+   * Reset AutoSelection to the system wide setting.
+   * @see disableAutoSelection()
+   * @since 3.2
+   */
+  void resetAutoSelection();
+
 protected slots:
   /**
    * Update internal settings whenever the global ones change.
@@ -839,12 +909,12 @@ protected slots:
   void cleanItemHighlighter();
 
   /**
-   * Emit the @ref contextMenu signal. This slot is for mouse actions.
+   * Emit the contextMenu signal. This slot is for mouse actions.
    */
   void emitContextMenu (QListViewItem*, const QPoint&, int);
 
   /**
-   * Emit the @ref #contextMenu signal. This slot is for key presses.
+   * Emit the contextMenu signal. This slot is for key presses.
    */
   void emitContextMenu (KListView*, QListViewItem*);
 
@@ -866,6 +936,8 @@ protected slots:
    */
   void slotAutoSelect();
 
+  void slotDragExpand();
+
   /**
    * Reacts to header changes in full width mode
    * @internal
@@ -880,6 +952,9 @@ protected:
 
   /**
    * Where is the nearest QListViewItem that I'm going to drop?
+   *
+   * FIXME KDE 4.0: Make this method const so it can be called from an
+   * acceptDrag method without ugly casts
    */
   virtual void findDrop(const QPoint &pos, QListViewItem *&parent, QListViewItem *&after);
 
@@ -903,17 +978,17 @@ private:
 };
 
 /**
- * A listview item with support for alternate background colours. It is
- * a drop-in replacement for @ref QListViewItem
+ * A listview item with support for alternate background colors. It is
+ * a drop-in replacement for QListViewItem
  *
- * @short listview item with alternate background colour support
+ * @short listview item with alternate background color support
  */
 class KListViewItem : public QListViewItem
 {
 public:
   /**
-   * constructors. The semantics remain as in @ref QListViewItem.
-   * Although they accept a @ref QListViewItem as parent, please
+   * constructors. The semantics remain as in QListViewItem.
+   * Although they accept a QListViewItem as parent, please
    * don't mix KListViewItem (or subclasses) with QListViewItem
    * (or subclasses).
    */
@@ -953,7 +1028,7 @@ public:
    */
   bool isAlternate();
   /**
-   * returns the background colour for this item
+   * returns the background color for this item
    */
   const QColor &backgroundColor();
 
