@@ -21,6 +21,7 @@
      the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
      Boston, MA 02111-1307, USA.
  */
+#include "config.h"
 
 #include "kmainwindow.h"
 #include "kmainwindowiface.h"
@@ -40,11 +41,12 @@
 #include <khelpmenu.h>
 #include <kmenubar.h>
 #include <kstatusbar.h>
+#include <kwin.h>
 
 #include <klocale.h>
 #include <kstandarddirs.h>
 #include <kstaticdeleter.h>
-#ifndef Q_WS_QWS
+#if defined Q_WS_X11 && ! defined K_WS_QTONLY
 #include <netwm.h>
 #endif
 
@@ -60,11 +62,15 @@ public:
     bool settingsDirty:1;
     bool autoSaveWindowSize:1;
     bool care_about_geometry:1;
+    bool shuttingDown:1;
     QString autoSaveGroup;
     KAccel * kaccel;
     KMainWindowInterface *m_interface;
     KDEPrivate::ToolBarHandler *toolBarHandler;
     QTimer* settingsTimer;
+    KToggleAction *showStatusBarAction;
+    QRect defaultWindowSize;
+    QPtrList<QDockWindow> hiddenDockWindows;
 };
 
 QPtrList<KMainWindow>* KMainWindow::memberList = 0L;
@@ -86,35 +92,37 @@ public:
         KConfig* config = KApplication::kApplication()->sessionConfig();
         if ( KMainWindow::memberList->first() ){
             // According to Jochen Wilhelmy <digisnap@cs.tu-berlin.de>, this
-            // hook is usefull for better document orientation
+            // hook is useful for better document orientation
             KMainWindow::memberList->first()->saveGlobalProperties(config);
         }
 
         QPtrListIterator<KMainWindow> it(*KMainWindow::memberList);
         int n = 0;
-        config->setGroup(QString::fromLatin1("Number"));
-        config->writeEntry(QString::fromLatin1("NumberOfWindows"), KMainWindow::memberList->count());
         for (it.toFirst(); it.current(); ++it){
-            n++;
-            it.current()->savePropertiesInternal(config, n);
+            if( it.current()->isVisible()) {
+                n++;
+                it.current()->savePropertiesInternal(config, n);
+            }
         }
-        return TRUE;
+        config->setGroup(QString::fromLatin1("Number"));
+        config->writeEntry(QString::fromLatin1("NumberOfWindows"), n );
+        return true;
     }
 
     bool commitData( QSessionManager& sm )
     {
         // not really a fast method but the only compatible one
         if ( sm.allowsInteraction() ) {
-            bool cancelled = false;
+            bool canceled = false;
             QPtrListIterator<KMainWindow> it(*KMainWindow::memberList);
             ::no_query_exit = true;
-            for (it.toFirst(); it.current() && !cancelled;){
+            for (it.toFirst(); it.current() && !canceled;){
                 KMainWindow *window = *it;
                 ++it; // Update now, the current window might get deleted
-                if ( !window->testWState( Qt::WState_ForceHide ) ) {
+                if ( window->isVisible()) {
                     QCloseEvent e;
                     QApplication::sendEvent( window, &e );
-                    cancelled = !e.isAccepted();
+                    canceled = !e.isAccepted();
 		    /* Don't even think_about deleting widgets with
 		     Qt::WDestructiveClose flag set at this point. We
 		     are faking a close event, but we are *not*_
@@ -132,13 +140,13 @@ public:
                 }
             }
             ::no_query_exit = false;
-            if (cancelled)
+            if (canceled)
                return false;
 
             KMainWindow* last = 0;
-            for (it.toFirst(); it.current() && !cancelled; ++it){
+            for (it.toFirst(); it.current() && !canceled; ++it){
                 KMainWindow *window = *it;
-                if ( !window->testWState( Qt::WState_ForceHide ) ) {
+                if ( window->isVisible()) {
                     last = window;
                 }
             }
@@ -149,21 +157,27 @@ public:
         }
 
         // the user wants it, the user gets it
-        return TRUE;
+        return true;
     }
 };
 
-static bool beeing_first = true;
+static bool being_first = true;
 
 KMainWindow::KMainWindow( QWidget* parent, const char *name, WFlags f )
     : QMainWindow( parent, name, f ), KXMLGUIBuilder( this ), helpMenu2( 0 ), factory_( 0 )
 {
-    initKMainWindow(name);
+    initKMainWindow(name, 0);
 }
 
-void KMainWindow::initKMainWindow(const char *name)
+KMainWindow::KMainWindow( int cflags, QWidget* parent, const char *name, WFlags f )
+    : QMainWindow( parent, name, f ), KXMLGUIBuilder( this ), helpMenu2( 0 ), factory_( 0 )
 {
-    setDockMenuEnabled( FALSE );
+    initKMainWindow(name, cflags);
+}
+
+void KMainWindow::initKMainWindow(const char *name, int cflags)
+{
+    setDockMenuEnabled( false );
     mHelpMenu = 0;
     kapp->setTopWidget( this );
     actionCollection()->setWidget( this );
@@ -172,7 +186,7 @@ void KMainWindow::initKMainWindow(const char *name)
         memberList = new QPtrList<KMainWindow>;
 
     if ( !ksm )
-        ksm = ksmd.setObject(new KMWSessionManaged());
+        ksm = ksmd.setObject(ksm, new KMWSessionManaged());
     // set a unique object name. Required by session management.
     QCString objname;
     QCString s;
@@ -225,8 +239,10 @@ void KMainWindow::initKMainWindow(const char *name)
     d->kaccel = actionCollection()->kaccel();
     d->toolBarHandler = 0;
     d->settingsTimer = 0;
-    if ((d->care_about_geometry = beeing_first)) {
-        beeing_first = false;
+    d->showStatusBarAction = NULL;
+    d->shuttingDown = false;
+    if ((d->care_about_geometry = being_first)) {
+        being_first = false;
         if ( kapp->geometryArgument().isNull() ) // if there is no geometry, it doesn't mater
             d->care_about_geometry = false;
         else
@@ -234,8 +250,10 @@ void KMainWindow::initKMainWindow(const char *name)
     }
 
     setCaption( kapp->caption() );
-    // attach dcop interface
-    d->m_interface = new KMainWindowInterface(this);
+    if ( cflags & NoDCOPObject)
+        d->m_interface = 0;
+    else
+        d->m_interface = new KMainWindowInterface(this);
 
     if (!kapp->authorize("movable_toolbars"))
         setDockWindowsMovable(false);
@@ -254,7 +272,8 @@ void KMainWindow::parseGeometry(bool parsewidth)
     assert ( !kapp->geometryArgument().isNull() );
     assert ( d->care_about_geometry );
 
-#ifndef Q_WS_QWS
+#if defined Q_WS_X11 && ! defined K_WS_QTONLY
+//#ifndef Q_WS_QWS
     // FIXME: (E) Implement something similar for Qt Embedded (or decide we don't need it)
     int x, y;
     int w, h;
@@ -326,10 +345,10 @@ KPopupMenu* KMainWindow::customHelpMenu( bool showWhatsThis )
 bool KMainWindow::canBeRestored( int number )
 {
     if ( !kapp->isRestored() )
-        return FALSE;
+        return false;
     KConfig *config = kapp->sessionConfig();
     if ( !config )
-        return FALSE;
+        return false;
     config->setGroup( QString::fromLatin1("Number") );
     int n = config->readNumEntry( QString::fromLatin1("NumberOfWindows") , 1 );
     return number >= 1 && number <= n;
@@ -352,17 +371,47 @@ const QString KMainWindow::classNameOfToplevel( int number )
         return config->readEntry( QString::fromLatin1("ClassName") );
 }
 
+void KMainWindow::show()
+{
+    QMainWindow::show();
+
+    for ( QPtrListIterator<QDockWindow> it( d->hiddenDockWindows ); it.current(); ++it )
+	it.current()->show();
+
+    d->hiddenDockWindows.clear();
+}
+
+void KMainWindow::hide()
+{
+    if ( isVisible() ) {
+
+        d->hiddenDockWindows.clear();
+
+        QObjectList *list = queryList( "QDockWindow" );
+        for( QObjectListIt it( *list ); it.current(); ++it ) {
+            QDockWindow *dw = (QDockWindow*)it.current();
+            if ( dw->isTopLevel() && dw->isVisible() ) {
+                d->hiddenDockWindows.append( dw );
+                dw->hide();
+            }
+        }
+        delete list;
+    }
+
+    QWidget::hide();
+}
+
 bool KMainWindow::restore( int number, bool show )
 {
     if ( !canBeRestored( number ) )
-        return FALSE;
+        return false;
     KConfig *config = kapp->sessionConfig();
     if ( readPropertiesInternal( config, number ) ){
         if ( show )
             KMainWindow::show();
-        return FALSE;
+        return false;
     }
-    return FALSE;
+    return false;
 }
 
 KXMLGUIFactory *KMainWindow::guiFactory()
@@ -472,7 +521,8 @@ void KMainWindow::setCaption( const QString &caption, bool modified )
 void KMainWindow::setPlainCaption( const QString &caption )
 {
     QMainWindow::setCaption( caption );
-#ifndef Q_WS_QWS
+#if defined Q_WS_X11 && ! defined K_WS_QTONLY
+//#ifndef Q_WS_QWS
     NETWinInfo info( qt_xdisplay(), winId(), qt_xrootwin(), 0 );
     info.setName( caption.utf8().data() );
 #endif
@@ -529,10 +579,11 @@ void KMainWindow::closeEvent ( QCloseEvent *e )
         }
 
         if ( !no_query_exit && not_withdrawn <= 0 ) { // last window close accepted?
-            if ( queryExit() && !kapp->sessionSaving()) {            // Yes, Quit app?
-                // We saved the toolbars already
+            if ( queryExit() && !kapp->sessionSaving() && !d->shuttingDown ) { // Yes, Quit app?
+                // don't call queryExit() twice
                 disconnect(kapp, SIGNAL(shutDown()), this, SLOT(shuttingDown()));
-                kapp->deref();             // ...and quit aplication.
+                d->shuttingDown = true;
+                kapp->deref();             // ...and quit application.
             }  else {
                 // cancel closing, it's stupid to end up with no windows at all....
                 e->ignore();
@@ -543,12 +594,12 @@ void KMainWindow::closeEvent ( QCloseEvent *e )
 
 bool KMainWindow::queryExit()
 {
-    return TRUE;
+    return true;
 }
 
 bool KMainWindow::queryClose()
 {
-    return TRUE;
+    return true;
 }
 
 void KMainWindow::saveGlobalProperties( KConfig*  )
@@ -596,11 +647,13 @@ void KMainWindow::savePropertiesInternal( KConfig *config, int number )
 void KMainWindow::saveMainWindowSettings(KConfig *config, const QString &configGroup)
 {
     kdDebug(200) << "KMainWindow::saveMainWindowSettings " << configGroup << endl;
-    QString entry;
-    QStrList entryList;
+    QString oldGroup;
 
     if (!configGroup.isEmpty())
+    {
+       oldGroup = config->group();
        config->setGroup(configGroup);
+    }
 
     // Called by session management - or if we want to save the window size anyway
     if ( d->autoSaveWindowSize )
@@ -608,27 +661,23 @@ void KMainWindow::saveMainWindowSettings(KConfig *config, const QString &configG
 
     QStatusBar* sb = internalStatusBar();
     if (sb) {
-        entryList.clear();
-        if ( sb->isHidden() )
-            entryList.append("Disabled");
-        else
-            entryList.append("Enabled");
-        config->writeEntry(QString::fromLatin1("StatusBar"), entryList, ';');
+       if(!config->hasDefault("StatusBar") && !sb->isHidden() )
+           config->revertToDefault("StatusBar");
+       else
+           config->writeEntry("StatusBar", sb->isHidden() ? "Disabled" : "Enabled");
     }
 
     QMenuBar* mb = internalMenuBar();
     if (mb) {
-        entryList.clear();
-        if ( mb->isHidden() )
-            entryList.append("Disabled");
-        else
-            entryList.append("Enabled");
-        config->writeEntry(QString::fromLatin1("MenuBar"), entryList, ';');
+       QString MenuBar = QString::fromLatin1("MenuBar");
+       if(!config->hasDefault("MenuBar") && !mb->isHidden() )
+           config->revertToDefault("MenuBar");
+       else
+           config->writeEntry("MenuBar", mb->isHidden() ? "Disabled" : "Enabled");
     }
 
     int n = 1; // Toolbar counter. toolbars are counted from 1,
     KToolBar *toolbar = 0;
-    QString toolKey;
     QPtrListIterator<KToolBar> it( toolBarIterator() );
     while ( ( toolbar = it.current() ) ) {
         ++it;
@@ -644,27 +693,29 @@ void KMainWindow::saveMainWindowSettings(KConfig *config, const QString &configG
         toolbar->saveSettings(config, group);
         n++;
     }
+    if (!configGroup.isEmpty())
+       config->setGroup(oldGroup);
 }
 
 void KMainWindow::setStandardToolBarMenuEnabled( bool enable )
 {
     if ( enable ) {
-	if ( d->toolBarHandler )
-	    return;
+        if ( d->toolBarHandler )
+            return;
 
-	d->toolBarHandler = new KDEPrivate::ToolBarHandler( this );
+    d->toolBarHandler = new KDEPrivate::ToolBarHandler( this );
 
-	if ( factory() )
-	    factory()->addClient( d->toolBarHandler );
+    if ( factory() )
+        factory()->addClient( d->toolBarHandler );
     } else {
-	if ( !d->toolBarHandler )
-	    return;
+        if ( !d->toolBarHandler )
+            return;
 
-	if ( factory() )
-	    factory()->removeClient( d->toolBarHandler );
+        if ( factory() )
+            factory()->removeClient( d->toolBarHandler );
 
-	delete d->toolBarHandler;
-	d->toolBarHandler = 0;
+        delete d->toolBarHandler;
+        d->toolBarHandler = 0;
     }
 }
 
@@ -673,6 +724,15 @@ bool KMainWindow::isStandardToolBarMenuEnabled() const
     return ( d->toolBarHandler != 0 );
 }
 
+void KMainWindow::createStandardStatusBarAction(){
+  if(!d->showStatusBarAction){
+    d->showStatusBarAction = KStdAction::showStatusbar(this, SLOT(setSettingsDirty()), actionCollection());
+    KStatusBar *sb = statusBar(); // Creates statusbar if it doesn't exist already.
+    connect(d->showStatusBarAction, SIGNAL(toggled(bool)), sb, SLOT(setShown(bool)));
+    d->showStatusBarAction->setChecked(sb->isHidden());
+  }
+}
+
 bool KMainWindow::readPropertiesInternal( KConfig *config, int number )
 {
     if ( number == 1 )
@@ -700,10 +760,6 @@ bool KMainWindow::readPropertiesInternal( KConfig *config, int number )
 void KMainWindow::applyMainWindowSettings(KConfig *config, const QString &configGroup)
 {
     kdDebug(200) << "KMainWindow::applyMainWindowSettings" << endl;
-    QString entry;
-    QStrList entryList;
-    int i = 0; // Number of entries in list
-
     if (!configGroup.isEmpty())
        config->setGroup(configGroup);
 
@@ -711,29 +767,26 @@ void KMainWindow::applyMainWindowSettings(KConfig *config, const QString &config
 
     QStatusBar* sb = internalStatusBar();
     if (sb) {
-        entryList.clear();
-        i = config->readListEntry (QString::fromLatin1("StatusBar"), entryList, ';');
-        entry = entryList.first();
-        if (entry == QString::fromLatin1("Disabled"))
-            sb->hide();
+        QString entry = config->readEntry("StatusBar", "Enabled");
+        if ( entry == "Disabled" )
+           sb->hide();
         else
-            sb->show();
+           sb->show();
+        if(d->showStatusBarAction)
+            d->showStatusBarAction->setChecked(!sb->isHidden());
     }
 
     QMenuBar* mb = internalMenuBar();
     if (mb) {
-        entryList.clear();
-        i = config->readListEntry (QString::fromLatin1("MenuBar"), entryList, ';');
-        entry = entryList.first();
-        if (entry==QString::fromLatin1("Disabled"))
-            mb->hide();
+        QString entry = config->readEntry ("MenuBar", "Enabled");
+        if ( entry == "Disabled" )
+           mb->hide();
         else
-            mb->show();
+           mb->show();
     }
 
     int n = 1; // Toolbar counter. toolbars are counted from 1,
     KToolBar *toolbar;
-    QString toolKey;
     QPtrListIterator<KToolBar> it( toolBarIterator() ); // must use own iterator
 
     for ( ; it.current(); ++it) {
@@ -773,10 +826,32 @@ void KMainWindow::finalizeGUI( bool force )
 
 void KMainWindow::saveWindowSize( KConfig * config ) const
 {
-    int scnum = QApplication::desktop()->screenNumber(parentWidget());
-    QRect desk = QApplication::desktop()->screenGeometry(scnum);
-    config->writeEntry(QString::fromLatin1("Width %1").arg(desk.width()), width() );
-    config->writeEntry(QString::fromLatin1("Height %1").arg(desk.height()), height() );
+#if defined Q_WS_X11 && ! defined K_WS_QTONLY
+  int scnum = QApplication::desktop()->screenNumber(parentWidget());
+  QRect desk = QApplication::desktop()->screenGeometry(scnum);
+  // save maximalization as desktop size + 1 in that direction
+  KWin::WindowInfo info = KWin::windowInfo( winId(), NET::WMState );
+  int w = info.state() & NET::MaxHoriz ? desk.width() + 1 : width();
+  int h = info.state() & NET::MaxVert ? desk.height() + 1 : height();
+  QRect size( desk.width(), w, desk.height(), h );
+#else
+  int w = 500;
+  int h = 500;
+  QRect desk( 100, 100, 200, 200 ); // fixme
+  QRect size( 100, 100, 200, 200 ); // fixme
+#endif
+  bool defaultSize = (size == d->defaultWindowSize);
+  QString widthString = QString::fromLatin1("Width %1").arg(desk.width());
+  QString heightString = QString::fromLatin1("Height %1").arg(desk.height());
+  if (!config->hasDefault(widthString) && defaultSize)
+     config->revertToDefault(widthString);
+  else
+     config->writeEntry(widthString, w );
+
+  if (!config->hasDefault(heightString) && defaultSize)
+     config->revertToDefault(heightString);
+  else
+     config->writeEntry(heightString, h );
 }
 
 void KMainWindow::restoreWindowSize( KConfig * config )
@@ -799,8 +874,20 @@ void KMainWindow::restoreWindowSize( KConfig * config )
                 config->writeEntry( QString::fromLatin1("Height"), 0 );
             }
         }
-        if ( !size.isEmpty() )
-            resize( size );
+        if ( !size.isEmpty() ) {
+            int state = ( size.width() > desk.width() ? NET::MaxHoriz : 0 )
+                        | ( size.height() > desk.height() ? NET::MaxVert : 0 );
+            if(( state & NET::Max ) == NET::Max )
+                ; // no resize
+            else if(( state & NET::MaxHoriz ) == NET::MaxHoriz )
+                resize( width(), size.height());
+            else if(( state & NET::MaxVert ) == NET::MaxVert )
+                resize( size.width(), height());
+            else
+                resize( size );
+            // QWidget::showMaximized() is both insufficient and broken
+            KWin::setState( winId(), state );
+        }
     }
 }
 
@@ -809,6 +896,11 @@ bool KMainWindow::initialGeometrySet() const
     return d->care_about_geometry;
 }
 
+void KMainWindow::ignoreInitialGeometry()
+{
+    d->care_about_geometry = false;
+}
+
 void KMainWindow::setSettingsDirty()
 {
     //kdDebug(200) << "KMainWindow::setSettingsDirty" << endl;
@@ -844,6 +936,11 @@ void KMainWindow::setAutoSaveSettings( const QString & groupName, bool saveWindo
     // Get notified when the user moves a toolbar around
     connect( this, SIGNAL( dockWindowPositionChanged( QDockWindow * ) ),
              this, SLOT( setSettingsDirty() ) );
+
+    // Get default values
+    int scnum = QApplication::desktop()->screenNumber(parentWidget());
+    QRect desk = QApplication::desktop()->screenGeometry(scnum);
+    d->defaultWindowSize = QRect(desk.width(), width(), desk.height(), height());
     // Now read the previously saved settings
     applyMainWindowSettings( KGlobal::config(), groupName );
 }
@@ -884,7 +981,7 @@ void KMainWindow::resizeEvent( QResizeEvent * )
 
 bool KMainWindow::hasMenuBar()
 {
-	return (internalMenuBar());
+    return (internalMenuBar());
 }
 
 KMenuBar *KMainWindow::menuBar()
@@ -928,7 +1025,7 @@ void KMainWindow::shuttingDown()
 
 KMenuBar *KMainWindow::internalMenuBar()
 {
-    QObjectList *l = queryList( "KMenuBar" );
+    QObjectList *l = queryList( "KMenuBar", 0, false, false );
     if ( !l || !l->first() ) {
         delete l;
         return 0;
@@ -941,7 +1038,7 @@ KMenuBar *KMainWindow::internalMenuBar()
 
 KStatusBar *KMainWindow::internalStatusBar()
 {
-    QObjectList *l = queryList( "KStatusBar" );
+    QObjectList *l = queryList( "KStatusBar", 0, false, false );
     if ( !l || !l->first() ) {
         delete l;
         return 0;
@@ -1001,8 +1098,8 @@ void KMainWindow::paintEvent( QPaintEvent * )
 
 QSize KMainWindow::sizeForCentralWidgetSize(QSize size)
 {
-    KToolBar *tb = toolBar();
-    if (!tb->isHidden()) {
+    KToolBar *tb = (KToolBar*)child( "mainToolBar", "KToolBar" );
+    if (tb && !tb->isHidden()) {
         switch( tb->barPos() )
         {
           case KToolBar::Top:
@@ -1023,8 +1120,8 @@ QSize KMainWindow::sizeForCentralWidgetSize(QSize size)
             break;
         }
     }
-    KMenuBar *mb = menuBar();
-    if (!mb->isHidden()) {
+    KMenuBar *mb = internalMenuBar();
+    if (mb && !mb->isHidden()) {
         size += QSize(0,mb->heightForWidth(size.width()));
         if (style().styleHint(QStyle::SH_MainWindow_SpaceBelowMenuBar, this))
            size += QSize( 0, dockWindowsMovable() ? 1 : 2);
@@ -1037,6 +1134,7 @@ QSize KMainWindow::sizeForCentralWidgetSize(QSize size)
 }
 
 // why do we support old gcc versions? using KXMLGUIBuilder::finalizeGUI;
+// DF: because they compile KDE much faster :)
 void KMainWindow::finalizeGUI( KXMLGUIClient *client )
 { KXMLGUIBuilder::finalizeGUI( client ); }
 
