@@ -15,7 +15,7 @@
     the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
     Boston, MA 02111-1307, USA.
 
-    $Id: kmainwindow.h,v 1.58.2.4 2003/03/03 14:39:42 lunakl Exp $
+    $Id: kmainwindow.h,v 1.99 2004/01/05 11:05:55 bhards Exp $
 
 */
 
@@ -26,7 +26,9 @@
 #include "kxmlguiclient.h"
 #include "kxmlguibuilder.h"
 #include <qmainwindow.h>
+#include <qmetaobject.h>
 #include <ktoolbar.h>
+#include <kdemacros.h>
 
 class KPopupMenu;
 class KXMLGUIFactory;
@@ -39,7 +41,7 @@ class KMWSessionManaged;
 class KMainWindowPrivate;
 class KAccel;
 class KToolBarMenuAction;
-
+class DCOPObject;
 
 #define KDE_DEFAULT_WINDOWFLAGS WType_TopLevel | WDestructiveClose
 
@@ -71,17 +73,17 @@ class KToolBarMenuAction;
  * not supported.
 *
  * KMainWindow will set icon, mini icon and caption, which it gets
- * from @ref KApplication. It provides full session management, and
+ * from KApplication. It provides full session management, and
  * will save its position, geometry and positions of toolbars and
  * menubar on logout. If you want to save additional data, reimplement
- * @ref saveProperties() and (to read them again on next login) @ref
+ * saveProperties() and (to read them again on next login)
  * readProperties(). To save special data about your data, reimplement
- * @ref saveGlobalProperties(). To warn user that application or
- * windows have unsaved data on close or logout, reimplement @ref
- * queryClose() and/or @ref queryExit().
+ * saveGlobalProperties(). To warn user that application or
+ * windows have unsaved data on close or logout, reimplement
+ * queryClose() and/or queryExit().
  *
- * There is also a macro RESTORE which can restore all your windows
- * on next login.
+ * There are also kRestoreMainWindows convenience functions which
+ * can restore all your windows on next login.
  *
  *  Note that a KMainWindow per-default is created with the
  *  WDestructiveClose flag, i.e. it is automatically destroyed when the
@@ -124,15 +126,40 @@ public:
      * parent or not. DestructiveClose indicates that a main window is
      * automatically destroyed when its window is closed. Pass 0 if
      * you do not want this behavior.
+     * 
+     * @see http://doc.trolltech.com/3.2/qt.html#WidgetFlags-enum
      *
      * KMainWindows must be created on the heap with 'new', like:
-     *  <pre> KMainWindow *kmw = new KMainWindow (...</pre>
+     * \code 
+     * KMainWindow *kmw = new KMainWindow (...);
+     * \endcode
      **/
     KMainWindow( QWidget* parent = 0, const char *name = 0, WFlags f = WType_TopLevel | WDestructiveClose );
 
+    /**
+     * Flags that can be passed in an argument to the constructor to
+     * change the behavior.
+     *
+     * NoDCOPObject tells KMainWindow not to create a KMainWindowInterface.
+     * This can be useful in particular for inherited classes, which
+     * might want to create more specific dcop interfaces. It's a good
+     * idea to use KMainWindowInterface as the base class for such interfaces
+     * though (to provide the standard mainwindow functionality via DCOP).
+     */
+    enum CreationFlags
+    {
+        NoDCOPObject = 1
+    };
 
     /**
-     * Destructor.
+     * Overloaded constructor which allows passing some KMainWindow::CreationFlags.
+     *
+     * @since 3.2
+     */
+    KMainWindow( int cflags, QWidget* parent = 0, const char *name = 0, WFlags f = WType_TopLevel | WDestructiveClose );
+
+    /**
+     * \brief Destructor.
      *
      * Will also destroy the toolbars, and menubar if
      * needed.
@@ -148,14 +175,14 @@ public:
      * and an "About KDE" dialog box.
      *
      * Example (adding a standard help menu to your application):
-     * <pre>
+     * \code
      * KPopupMenu *help = helpMenu( <myTextString> );
      * menuBar()->insertItem( i18n("&Help"), help );
-     * </pre>
+     * \endcode
      *
      * @param aboutAppText The string that is used in the application
      *        specific dialog box. If you leave this string empty the
-     *        information in the global @ref KAboutData of the
+     *        information in the global KAboutData of the
      *        application will be used to make a standard dialog box.
      *
      * @param showWhatsThis Set this to false if you do not want to include
@@ -164,7 +191,7 @@ public:
      * @return A standard help menu.
      */
     KPopupMenu* helpMenu( const QString &aboutAppText = QString::null,
-			  bool showWhatsThis = TRUE );
+			  bool showWhatsThis = true );
 
     /**
      * Returns the help menu. Creates a standard help menu if none exists yet.
@@ -175,23 +202,23 @@ public:
      * and an "About KDE" dialog box. You must create the application
      * specific dialog box yourself. When the "About application"
      * menu entry is activated, a signal will trigger the
-     * @ref showAboutApplication slot. See @ref showAboutApplication for more
+     * showAboutApplication slot. See showAboutApplication for more
      * information.
      *
      * Example (adding a help menu to your application):
-     * <pre>
+     * \code
      * menuBar()->insertItem( i18n("&Help"), customHelpMenu() );
-     * </pre>
+     * \endcode
      *
      * @param showWhatsThis Set this to @p false if you do not want to include
      *        the "What's This" menu entry.
      *
      * @return A standard help menu.
      */
-    KPopupMenu* customHelpMenu( bool showWhatsThis = TRUE );
+    KPopupMenu* customHelpMenu( bool showWhatsThis = true );
 
     /**
-     * @sect Session Management
+     * <b>Session Management</b>
      *
      * Try to restore the toplevel widget as defined by the number (1..X).
      *
@@ -199,7 +226,7 @@ public:
      * is not changed and @p false returned.
      *
      * That means clients could simply do the following:
-     * <pre>
+     * \code
      * if (kapp->isRestored()){
      *   int n = 1;
      *   while (KMainWindow::canBeRestored(n)){
@@ -207,57 +234,88 @@ public:
      *     n++;
      *   }
      * } else {
-     * // create default application as usual
+     *   // create default application as usual
      * }
-     * </pre>
-     * Note that @ref QWidget::show() is called implicitly in restore.
+     * \endcode
+     * Note that QWidget::show() is called implicitly in restore.
      *
      * With this you can easily restore all toplevel windows of your
      * application.
      *
      * If your application uses different kinds of toplevel
-     * windows, then you can use @ref KMainWindow::classNameOfToplevel(n)
+     * windows, then you can use KMainWindow::classNameOfToplevel(n)
      * to determine the exact type before calling the childMW
      * constructor in the example from above.
      *
-     * If your client has only one kind of toplevel widgets (which should
-     * be pretty usual) then you should use the RESTORE-macro:
+     * If your client has only one kind of toplevel widgets (which
+     * should be pretty usual) then you should use the RESTORE-macro
+     * for backwards compatibility with 3.1 and 3.0 branches:
      *
-     * <pre>
+     * \code
      * if (kapp->isRestored())
      *   RESTORE(childMW)
      * else {
-     * // create default application as usual
+     *   // create default application as usual
      * }
-     * </pre>
+     * \endcode
      *
      * The macro expands to the term above but is easier to use and
      * less code to write.
      *
+     * For new code or if you have more than one kind of toplevel
+     * widget (each derived from KMainWindow, of course), you can
+     * use the templated kRestoreMainWindows global functions:
+     *
+     * \code
+     * if (kapp->isRestored())
+     *   kRestoreMainWindows< childMW1, childMW2, childMW3 >();
+     * else {
+     *   // create default application as usual
+     * }
+     * \endcode
+     *
+     * Currently, these functions are provided for up to three
+     * template arguments. If you need more, tell us. To help you in
+     * deciding whether or not you can use kRestoreMainWindows, a
+     * define KDE_RESTORE_MAIN_WINDOWS_NUM_TEMPLATE_ARGS is provided.
+     *
      * @see restore()
      * @see classNameOfToplevel()
      *
-     */
+     **/
     static bool canBeRestored( int number );
 
     /**
-     * Returns the @ref className() of the @p number of the toplevel window which
+     * Returns the className() of the @p number of the toplevel window which
      * should be restored.
      *
-     *  This is only useful if your application uses
+     * This is only useful if your application uses
      * different kinds of toplevel windows.
      */
+    // KDE 4 return QCString - QObject::className() returns const char*
     static const QString classNameOfToplevel( int number );
 
+    /**
+     * Reimplementation of QMainWindow::show()
+     */
+    // KDE4 remove this method if this has been fixed in Qt
+    virtual void show();
+
+    /**
+     * Reimplementation of QMainWindow::hide()
+     */
+    // KDE4 remove this method if this has been fixed in Qt
+    virtual void hide();
+
     /**
      * Restore the session specified by @p number.
      *
      * Returns @p false if this
      * fails, otherwise returns @p true and shows the window.
-     * You should call @ref canBeRestored() first.
+     * You should call canBeRestored() first.
      * If @p show is true (default), this widget will be shown automatically.
      */
-    bool restore( int number, bool show = TRUE );
+    bool restore( int number, bool show = true );
 
     virtual KXMLGUIFactory *guiFactory();
 
@@ -277,10 +335,10 @@ public:
      *
      * @param xmlfile The local xmlfile (relative or absolute)
      * @param _conserveMemory Specify whether createGUI() should call
-     *     @ref KXMLGuiClient::conserveMemory() to free all memory
-     *     allocated by the @ref QDomDocument and by the KXMLGUIFactory.
+     * KXMLGuiClient::conserveMemory() to free all memory
+     *     allocated by the QDomDocument and by the KXMLGUIFactory.
      */
-    void createGUI( const QString &xmlfile = QString::null, bool _conserveMemory = TRUE );
+    void createGUI( const QString &xmlfile = QString::null, bool _conserveMemory = true );
 
     /**
      * Enables the build of a standard help menu when calling createGUI().
@@ -330,7 +388,7 @@ public:
      * @param name The internal name of the toolbar. If no name is
      *             specified "mainToolBar" is assumed.
      *
-     * @return A pointer to the toolbar or @p 0L if it doesn't exist
+     * @return A pointer to the toolbar
      **/
     KToolBar *toolBar( const char *name=0 );
 
@@ -411,8 +469,8 @@ public:
 
     /**
      * Sets whether KMainWindow should provide a menu that allows showing/hiding
-     * the available toolbars ( using @ref KToggleToolBarAction ) . In case there
-     * is only one toolbar configured a simple 'Show <toolbar name here>' menu item
+     * the available toolbars ( using KToggleToolBarAction ) . In case there
+     * is only one toolbar configured a simple 'Show \<toolbar name here\>' menu item
      * is shown.
      *
      * The menu / menu item is implemented using xmlgui. It will be inserted in your
@@ -431,6 +489,30 @@ public:
     /// @since 3.1
     bool isStandardToolBarMenuEnabled() const;
 
+
+    /**
+     * Sets whether KMainWindow should provide a menu that allows showing/hiding
+     * of the statusbar ( using KToggleStatusBarAction ).
+     *
+     * The menu / menu item is implemented using xmlgui. It will be inserted
+     * in your menu structure in the 'Settings' menu.
+     *
+     * Note that you should enable this feature before calling createGUI()
+     * ( or similar ).
+     *
+     * If an application maintains the action on its own (i.e. never calls
+     * this function) a connection needs to be made to let KMainWindow
+     * know when that status (hidden/shown) of the statusbar has changed.
+     * For example:
+     * connect(action, SIGNAL(activated()),
+     *         kmainwindow, SLOT(setSettingsDirty()));
+     * Otherwise the status (hidden/show) of the statusbar might not be saved
+     * by KMainWindow.
+     * @since 3.2
+     */
+    void createStandardStatusBarAction();
+
+
     /**
      * Returns a pointer to the mainwindows action responsible for the toolbars menu
      * @since 3.1
@@ -452,11 +534,29 @@ public:
      */
     bool initialGeometrySet() const;
 
+    /**
+     * @internal
+     * Used from Konqueror when reusing the main window.
+     */
+    void ignoreInitialGeometry();
+
     /**
      * @return the size the mainwindow should have so that the central
      * widget will be of @p size.
+     *
+     * @deprecated You normally don't need this, the recommended way to achieve a
+     *   certain central widget size is as follows:
+     *     @li Override sizeHint() in the central widget so that it
+     *      returns the desired size.
+     *     @li Call updateGeometry() in the central widget whenever the
+     *      desired size changes. This ensures that the new sizeHint() is properly
+     *      propagated to any parent layout.
+     *     @li Now call adjustSize() in the mainwindow to resize the
+     *      mainwindow such that the central widget will become the desired size.
+     * 
      */
-    QSize sizeForCentralWidgetSize(QSize size);
+    // KDE4 to be removed
+    QSize sizeForCentralWidgetSize(QSize size) KDE_DEPRECATED;
 
 public slots:
     /**
@@ -491,20 +591,20 @@ public slots:
      *
      *  The application name is
      * used as a key to determine what to display and the system will attempt
-     * to open <appName>/index.html.
+     * to open \<appName\>/index.html.
      *
      * This method is intended for use by a help button in the toolbar or
-     * components outside the regular help menu. Use @ref helpMenu() when you
+     * components outside the regular help menu. Use helpMenu() when you
      * want to provide access to the help system from the help menu.
      *
      * Example (adding a help button to the first toolbar):
      *
-     * <pre>
+     * \code
      * KIconLoader &loader = *KGlobal::iconLoader();
      * QPixmap pixmap = loader.loadIcon( "help" );
      * toolBar(0)->insertButton( pixmap, 0, SIGNAL(clicked()),
      *   this, SLOT(appHelpActivated()), true, i18n("Help") );
-     * </pre>
+     * \endcode
      *
      */
     void appHelpActivated( void );
@@ -540,7 +640,11 @@ public slots:
 //                           bool reverse); // KDE 4.0: enable this
 
     /**
-     * @internal. Used for the auto-save-settings feature.
+     * Tell the main window that it should save its settings when being closed.
+     * This is part of the auto-save-settings feature.
+     * For everything related to toolbars this happens automatically,
+     * but you have to call setSettingsDirty() in the slot that toggles
+     * the visibility of the statusbar.
      */
     void setSettingsDirty();
 
@@ -551,31 +655,41 @@ protected:
     /**
      * Reimplemented to call the queryClose() and queryExit() handlers.
      *
-     * We recommend that you reimplement the handlers rather than @ref closeEvent().
+     * We recommend that you reimplement the handlers rather than closeEvent().
      * If you do it anyway, ensure to call the base implementation to keep
-     * @ref queryExit() running.
+     * queryExit() running.
      */
     virtual void closeEvent ( QCloseEvent *);
+
+    // KDE4 This seems to be flawed to me. Either the app has only one
+    // mainwindow, so queryClose() is enough, or if it can have more of them,
+    // then the windows should take care of themselves, and queryExit()
+    // would be useful only for the annoying 'really quit' dialog, which
+    // also doesn't make sense in apps with multiple mainwindows.
+    // And saving configuration in something called queryExit()? IMHO
+    // one can e.g. use KApplication::shutDown(), which if nothing else
+    // has at least better fitting name.
+    // See also KApplication::sessionSaving().
+    // This stuff should get changed somehow, so that it at least doesn't
+    // mess with session management.
     /**
        Called before the very last window is closed, either by the
        user or indirectly by the session manager.
 
        It is not recommended to do any user interaction in this
        function other than indicating severe errors. Better ask the
-       user on @ref queryClose() (see below).
+       user on queryClose() (see below).
 
-       However, @ref queryExit() is useful to do some final cleanups. A
-       typical example would be to write configuration data back.
-
-       Note that the application may continue to run after @ref queryExit()
-       (the user may have cancelled a shutdown). The purpose of
-       @ref queryExit() is purely to prepare the application (with possible
-       user interaction) so it can safely be closed later (without
+       A typical usage of queryExit() is to write configuration data back.
+       Note that the application may continue to run after queryExit()
+       (the user may have canceled a shutdown), so you should not do any cleanups
+       here. The purpose of queryExit() is purely to prepare the application
+       (with possible user interaction) so it can safely be closed later (without
        user interaction).
 
        If you need to do serious things on exit (like shutting a
        dial-up connection down), connect to the signal
-       @ref KApplication::shutDown().
+ KApplication::shutDown().
 
        Default implementation returns @p true. Returning @p false will
        cancel the exiting. In the latter case, the last window will
@@ -600,25 +714,29 @@ protected:
 
        Reimplement this function to prevent the user from losing data.
        Example:
-       <pre>
+       \code
+       switch ( KMessageBox::warningYesNoCancel( this,
+                i18n("Save changes to document foo?")) ) {
+       case KMessageBox::Yes :
+         // save document here. If saving fails, return false;
+         return true;
+       case KMessageBox::No :
+         return true;
+       default: // cancel
+         return false;
+       \endcode
 
-           switch ( KMessageBox::warningYesNoCancel( this,
-				   i18n("Save changes to Document Foo?")) ) {
-           case KMessageBox::Yes :
-             // save document here. If saving fails, return FALSE;
-             return TRUE;
-           case KMessageBox::No :
-             return TRUE;
-           default: // cancel
-             return FALSE;
-
-    </pre>
-
-   @see queryExit()
-   @see KApplication::sessionSaving()
+       Note that you should probably @em not actually close the document from
+       within this method, as it may be called by the session manager before the
+       session is saved. If the document is closed before the session save occurs,
+       its location might not be properly saved. In addition, the session shutdown
+       may be canceled, in which case the document should remain open.
 
+       @see queryExit()
+       @see KApplication::sessionSaving()
     */
     virtual bool queryClose();
+
     /**
      * Save your instance-specific properties. The function is
      * invoked when the session manager requests your application
@@ -644,7 +762,7 @@ protected:
      * invoked when the session manager requests your application
      * to save its state.
      *
-     * This function is similar to @ref saveProperties() but is only called for
+     * This function is similar to saveProperties() but is only called for
      * the very first main window, regardless how many main window are open.
 
      * Override it if you need to save other data about your documents on
@@ -658,7 +776,7 @@ protected:
     virtual void saveGlobalProperties( KConfig* sessionConfig );
 
     /**
-     * The counterpart of @ref saveGlobalProperties().
+     * The counterpart of saveGlobalProperties().
      *
      * Read the application-specific properties in again.
      */
@@ -697,10 +815,10 @@ protected slots:
     * It must be reimplemented if you want
     * to use a custom About Application dialog box. This slot is
     * connected to the About Application entry in the menu returned
-    * by @ref customHelpMenu.
+    * by customHelpMenu.
     *
     * Example:
-    * <pre>
+    * \code
     *
     * void MyMainLevel::setupInterface()
     * {
@@ -713,18 +831,41 @@ protected slots:
     * {
     *   <activate your custom dialog>
     * }
-    * </pre>
+    * \endcode
     */
     virtual void showAboutApplication();
 
+   /**
+    * This slot should only be called in case you reimplement closeEvent() and
+    * if you are using the "auto-save" feature. In all other cases,
+    * setSettingsDirty() should be called instead to benefit from the delayed
+    * saving.
+    *
+    * @see setAutoSaveSettings
+    * @see setSettingsDirty
+    *
+    * @since 3.2
+    *
+    * Example:
+    * \code
+    *
+    * void MyMainWindow::closeEvent( QCloseEvent *e )
+    * {
+    *   // Save settings if auto-save is enabled, and settings have changed
+    *   if ( settingsDirty() && autoSaveSettings() )
+    *     saveAutoSaveSettings();
+    *   ..
+    * }
+    * \endcode
+    */ 
+    void saveAutoSaveSettings();
+
 private slots:
    /**
     * Called when the app is shutting down.
     */
     void shuttingDown();
 
-    void saveAutoSaveSettings();
-
 private:
     KMenuBar *internalMenuBar();
     KStatusBar *internalStatusBar();
@@ -735,7 +876,7 @@ protected:
     virtual void virtual_hook( int id, void* data );
 private:
     KMainWindowPrivate *d;
-    void initKMainWindow(const char *name);
+    void initKMainWindow(const char *name, int cflags);
 };
 
 #define RESTORE(type) { int n = 1;\
@@ -743,5 +884,56 @@ private:
       (new type)->restore(n);\
       n++;}}
 
+#define KDE_RESTORE_MAIN_WINDOWS_NUM_TEMPLATE_ARGS 3
+
+/**
+ *  These global convenience functions (that come with a varying
+ *  number of template arguments) are a replacement for the RESTORE
+ *  macro provided in earlier versions of KDE. The old RESTORE macro
+ *  is still provided for backwards compatibility. See
+ *  KMainWindow documentation for more.
+ *
+ * \since KDE 3.2
+ *
+ **/
+template <typename T>
+inline void kRestoreMainWindows() {
+  for ( int n = 1 ; KMainWindow::canBeRestored( n ) ; ++n ) {
+    const QString className = KMainWindow::classNameOfToplevel( n );
+    if ( className == QString::fromLatin1( T::staticMetaObject()->className() ) )
+      (new T)->restore( n );
+  }
+}
+
+template <typename T0, typename T1>
+inline void kRestoreMainWindows() {
+  const char * classNames[2];
+  classNames[0] = T0::staticMetaObject()->className();
+  classNames[1] = T1::staticMetaObject()->className();
+  for ( int n = 1 ; KMainWindow::canBeRestored( n ) ; ++n ) {
+    const QString className = KMainWindow::classNameOfToplevel( n );
+    if ( className == QString::fromLatin1( classNames[0] ) )
+      (new T0)->restore( n );
+    else if ( className == QString::fromLatin1( classNames[1] ) )
+      (new T1)->restore( n );
+  }
+}
+
+template <typename T0, typename T1, typename T2>
+inline void kRestoreMainWindows() {
+  const char * classNames[3];
+  classNames[0] = T0::staticMetaObject()->className();
+  classNames[1] = T1::staticMetaObject()->className();
+  classNames[2] = T1::staticMetaObject()->className();
+  for ( int n = 1 ; KMainWindow::canBeRestored( n ) ; ++n ) {
+    const QString className = KMainWindow::classNameOfToplevel( n );
+    if ( className == QString::fromLatin1( classNames[0] ) )
+      (new T0)->restore( n );
+    else if ( className == QString::fromLatin1( classNames[1] ) )
+      (new T1)->restore( n );
+    else if ( className == QString::fromLatin1( classNames[2] ) )
+      (new T2)->restore( n );
+  }
+}
 
 #endif
