@@ -20,7 +20,7 @@
     Boston, MA 02111-1307, USA.
 */
 
-//$Id: kmenubar.h,v 1.51 2002/03/20 07:45:32 waba Exp $
+//$Id: kmenubar.h,v 1.61.2.1 2004/04/19 11:53:37 lunakl Exp $
 
 #ifndef _KMENUBAR_H
 #define _KMENUBAR_H
@@ -30,11 +30,11 @@
 /**
  * KDE Style-able menubar.
  *
- * This is required since @ref QMenuBar is currently not handled by
- * @ref QStyle.
+ * This is required since QMenuBar is currently not handled by
+ * QStyle.
  *
  * @author Daniel "Mosfet" Duley.
- * @version $Id: kmenubar.h,v 1.51 2002/03/20 07:45:32 waba Exp $
+ * @version $Id: kmenubar.h,v 1.61.2.1 2004/04/19 11:53:37 lunakl Exp $
 */
 
 class KMenuBar : public QMenuBar
@@ -72,20 +72,34 @@ public:
      */
     bool isTopLevelMenu() const;
 
-    virtual void        setGeometry( int x, int y, int w, int h );
+    // TT are overloading virtuals :(
+    virtual void setGeometry( const QRect &r );
+    virtual void setGeometry( int x, int y, int w, int h );
+    virtual void resize( int w, int h );
+    void resize( const QSize& s ) { QMenuBar::resize( s ); }
 
     virtual void show();
+    virtual void setFrameStyle( int );
+    virtual void setLineWidth( int );
+    virtual void setMargin( int );
 protected slots:
     void slotReadConfig();
-
 protected:
-    void showEvent( QShowEvent* );
-    bool eventFilter(QObject *, QEvent *);
-
+    virtual void showEvent( QShowEvent* );
+    virtual void resizeEvent( QResizeEvent* );
+    virtual bool eventFilter(QObject *, QEvent *);
+    virtual bool x11Event( XEvent* );
+    virtual void closeEvent( QCloseEvent* );
+private slots:
+    void updateFallbackSize();
+    void selectionTimeout();
+private:
+    void setTopLevelMenuInternal(bool top_level);
+    void checkSize( int& w, int& h );
+    static int block_resize;
 protected:
     virtual void virtual_hook( int id, void* data );
 private:
-    void setTopLevelMenuInternal(bool top_level = true);
     class KMenuBarPrivate;
     KMenuBarPrivate *d;
 };
