@@ -36,6 +36,7 @@
 #include <math.h>
 #include <algorithm>
 
+#include <qapplication.h>
 #include <qlabel.h>
 #include <qlineedit.h>
 #include <qsize.h>
@@ -47,6 +48,7 @@
 #include <klocale.h>
 #include <kdebug.h>
 
+#include "kdialog.h"
 #include "knumvalidator.h"
 #include "knuminput.h"
 
@@ -285,7 +287,12 @@ void KIntNumInput::init(int val, int _base)
 {
     d = new KIntNumInputPrivate( val );
     m_spin = new KIntSpinBox(INT_MIN, INT_MAX, 1, val, _base, this, "KIntNumInput::KIntSpinBox");
-    m_spin->setValidator(new KIntValidator(this, _base, "KNumInput::KIntValidtr"));
+    // the KIntValidator is broken beyond believe for
+    // spinboxes which have suffix or prefix texts, so
+    // better don't use it unless absolutely necessary
+    if (_base != 10)
+        m_spin->setValidator(new KIntValidator(this, _base, "KNumInput::KIntValidtr"));
+
     connect(m_spin, SIGNAL(valueChanged(int)), SLOT(spinValueChanged(int)));
     connect(this, SIGNAL(valueChanged(int)),
 	    SLOT(slotEmitRelativeValueChanged(int)));
@@ -446,17 +453,27 @@ void KIntNumInput::resizeEvent(QResizeEvent* e)
 
     if(m_label && (m_alignment & AlignTop)) {
         m_label->setGeometry(0, 0, e->size().width(), m_sizeLabel.height());
-        h += m_sizeLabel.height() + 4;
+        h += m_sizeLabel.height() + KDialog::spacingHint();
     }
 
     if(m_label && (m_alignment & AlignVCenter))
         m_label->setGeometry(0, 0, w, m_sizeSpin.height());
 
-    m_spin->setGeometry(w, h, m_slider ? m_colw2 : QMAX(m_colw2, e->size().width() - w), m_sizeSpin.height());
-    w += m_colw2 + 8;
+    if (qApp->reverseLayout())
+    {
+        m_spin->setGeometry(w, h, m_slider ? m_colw2 : QMAX(m_colw2, e->size().width() - w), m_sizeSpin.height());
+        w += m_colw2 + 8;
 
-    if(m_slider)
-        m_slider->setGeometry(w, h, e->size().width() - w, m_sizeSpin.height());
+        if(m_slider)
+            m_slider->setGeometry(w, h, e->size().width() - w, m_sizeSpin.height());
+    }
+    else if(m_slider) {
+        m_slider->setGeometry(w, h, e->size().width() - (w + m_colw2 + KDialog::spacingHint()), m_sizeSpin.height());
+        m_spin->setGeometry(w + m_slider->size().width() + KDialog::spacingHint(), h, m_colw2, m_sizeSpin.height());
+    }
+    else {
+        m_spin->setGeometry(w, h, QMAX(m_colw2, e->size().width() - w), m_sizeSpin.height());
+    }
 
     h += m_sizeSpin.height() + 2;
 
@@ -674,12 +691,25 @@ void KDoubleNumInput::resizeEvent(QResizeEvent* e)
     if(m_label && (m_alignment & AlignVCenter))
         m_label->setGeometry(0, 0, w, m_sizeEdit.height());
 
-    d->spin->setGeometry(w, h, m_slider ? m_colw2
-                                          : e->size().width() - w, m_sizeEdit.height());
-    w += m_colw2 + 8;
+    if (qApp->reverseLayout())
+    {
+        d->spin->setGeometry(w, h, m_slider ? m_colw2
+                                            : e->size().width() - w, m_sizeEdit.height());
+        w += m_colw2 + KDialog::spacingHint();
 
-    if(m_slider)
-        m_slider->setGeometry(w, h, e->size().width() - w, m_sizeEdit.height());
+        if(m_slider)
+            m_slider->setGeometry(w, h, e->size().width() - w, m_sizeEdit.height());
+    }
+    else if(m_slider) {
+        m_slider->setGeometry(w, h, e->size().width() -
+                                    (m_colw1 + m_colw2 + KDialog::spacingHint()),
+                              m_sizeEdit.height());
+        d->spin->setGeometry(w + m_slider->width() + KDialog::spacingHint(), h,
+                             m_colw2, m_sizeEdit.height());
+    }
+    else {
+        d->spin->setGeometry(w, h, e->size().width() - w, m_sizeEdit.height());
+    }
 
     h += m_sizeEdit.height() + 2;
 
@@ -852,6 +882,60 @@ void KDoubleNumInput::setLabel(const QString & label, int a)
 // ----------------------------------------------------------------------------
 
 
+class KDoubleSpinBoxValidator : public KDoubleValidator
+{
+public:
+    KDoubleSpinBoxValidator( double bottom, double top, int decimals, KDoubleSpinBox* sb, const char *name )
+        : KDoubleValidator( bottom, top, decimals, sb, name ), spinBox( sb ) { }
+
+    virtual State validate( QString& str, int& pos ) const;
+
+private:
+    KDoubleSpinBox *spinBox;
+};
+
+QValidator::State KDoubleSpinBoxValidator::validate( QString& str, int& pos ) const
+{
+    QString pref = spinBox->prefix();
+    QString suff = spinBox->suffix();
+    QString suffStriped = suff.stripWhiteSpace();
+    uint overhead = pref.length() + suff.length();
+    State state = Invalid;
+
+    if ( overhead == 0 ) {
+        state = KDoubleValidator::validate( str, pos );
+    } else {
+        bool stripedVersion = false;
+        if ( str.length() >= overhead && str.startsWith(pref)
+             && (str.endsWith(suff)
+                 || (stripedVersion = str.endsWith(suffStriped))) ) {
+            if ( stripedVersion )
+                overhead = pref.length() + suffStriped.length();
+            QString core = str.mid( pref.length(), str.length() - overhead );
+            int corePos = pos - pref.length();
+            state = KDoubleValidator::validate( core, corePos );
+            pos = corePos + pref.length();
+            str.replace( pref.length(), str.length() - overhead, core );
+        } else {
+            state = KDoubleValidator::validate( str, pos );
+            if ( state == Invalid ) {
+                // stripWhiteSpace(), cf. QSpinBox::interpretText()
+                QString special = spinBox->specialValueText().stripWhiteSpace();
+                QString candidate = str.stripWhiteSpace();
+
+                if ( special.startsWith(candidate) ) {
+                    if ( candidate.length() == special.length() ) {
+                        state = Acceptable;
+                    } else {
+                        state = Intermediate;
+                    }
+                }
+            }
+        }
+    }
+    return state;
+}
+
 // We use a kind of fixed-point arithmetic to represent the range of
 // doubles [mLower,mUpper] in steps of 10^(-mPrecision). Thus, the
 // following relations hold:
@@ -906,7 +990,7 @@ public:
   }
 
   int mPrecision;
-  KDoubleValidator * mValidator;
+  KDoubleSpinBoxValidator * mValidator;
 };
 
 KDoubleSpinBox::KDoubleSpinBox( QWidget * parent, const char * name )
@@ -1070,7 +1154,7 @@ void KDoubleSpinBox::slotValueChanged( int value ) {
 
 void KDoubleSpinBox::updateValidator() {
   if ( !d->mValidator ) {
-    d->mValidator =  new KDoubleValidator( minValue(), maxValue(), precision(),
+    d->mValidator =  new KDoubleSpinBoxValidator( minValue(), maxValue(), precision(),
 					   this, "d->mValidator" );
     base::setValidator( d->mValidator );
   } else
