@@ -46,10 +46,16 @@
  * Password line editor.
  */
 
+class KPasswordDialog::KPasswordDialogPrivate
+{
+public:
+  QLabel *m_MatchLabel;
+};
+
 const int KPasswordEdit::PassLen = 100;
 
 KPasswordEdit::KPasswordEdit(QWidget *parent, const char *name)
-    : QLineEdit(parent, name), m_EchoMode(OneStar)
+    : QLineEdit(parent, name)
 {
     init();
 
@@ -71,12 +77,19 @@ KPasswordEdit::KPasswordEdit(QWidget *parent, const char *name, int echoMode)
     init();
 }
 
-KPasswordEdit::KPasswordEdit(EchoMode echoMode, QWidget *parent, const char *name)
+KPasswordEdit::KPasswordEdit(EchoModes echoMode, QWidget *parent, const char *name)
     : QLineEdit(parent, name), m_EchoMode(echoMode)
 {
     init();
 }
 
+KPasswordEdit::KPasswordEdit(EchoMode echoMode, QWidget *parent, const char *name)
+    : QLineEdit(parent, name)
+    , m_EchoMode( echoMode == QLineEdit::NoEcho ? NoEcho : OneStar )
+{
+    init();
+}
+
 void KPasswordEdit::init()
 {
     setEchoMode(QLineEdit::Password); // Just in case
@@ -163,13 +176,13 @@ bool KPasswordEdit::event(QEvent *e) {
       case QEvent::MouseMove:
       case QEvent::IMStart:
       case QEvent::IMCompose:
-        return TRUE; //Ignore
+        return true; //Ignore
 
       case QEvent::IMEnd:
       {
         QIMEvent *ie = (QIMEvent*) e;
         insert( ie->text() );
-        return TRUE;
+        return true;
       }
 
       case QEvent::AccelOverride:
@@ -183,7 +196,7 @@ bool KPasswordEdit::event(QEvent *e) {
                     showPass();
                 }
         }
-        return TRUE; // stop bubbling
+        return true; // stop bubbling
       }
 
       default:
@@ -207,6 +220,7 @@ void KPasswordEdit::showPass()
 	setText(tmp);
 	break;
     case NoEcho: default:
+	emit textChanged(QString::null); //To update the password comparison if need be.
 	break;
     }
 }
@@ -219,7 +233,7 @@ void KPasswordEdit::showPass()
 KPasswordDialog::KPasswordDialog(Types type, bool enableKeep, int extraBttn,
                                  QWidget *parent, const char *name)
     : KDialogBase(parent, name, true, "", Ok|Cancel|extraBttn,
-                  Ok, true), m_Keep(enableKeep? 1 : 0), m_Type(type)
+                  Ok, true), m_Keep(enableKeep? 1 : 0), m_Type(type), d(new KPasswordDialogPrivate)
 {
     init();
 }
@@ -228,7 +242,7 @@ KPasswordDialog::KPasswordDialog(Types type, bool enableKeep, int extraBttn,
 KPasswordDialog::KPasswordDialog(int type, QString prompt, bool enableKeep,
                                  int extraBttn)
     : KDialogBase(0L, "Password Dialog", true, "", Ok|Cancel|extraBttn,
-                  Ok, true), m_Keep(enableKeep? 1 : 0), m_Type(type)
+                  Ok, true), m_Keep(enableKeep? 1 : 0), m_Type(type), d(new KPasswordDialogPrivate)
 {
     init();
     setPrompt(prompt);
@@ -246,7 +260,7 @@ void KPasswordDialog::init()
 
     m_pMain = new QWidget(this);
     setMainWidget(m_pMain);
-    m_pGrid = new QGridLayout(m_pMain, 10, 3, 10, 0);
+    m_pGrid = new QGridLayout(m_pMain, 10, 3, 0, 0);
     m_pGrid->addColSpacing(1, 10);
 
     // Row 1: pixmap + prompt
@@ -284,8 +298,7 @@ void KPasswordDialog::init()
     QSize size = m_pEdit->sizeHint();
     m_pEdit->setFixedHeight(size.height());
     m_pEdit->setMinimumWidth(size.width());
-    h_lay->addWidget(m_pEdit, 12);
-    h_lay->addStretch(4);
+    h_lay->addWidget(m_pEdit);
 
     // Row 4: Password editor #2 or keep password checkbox
 
@@ -315,8 +328,18 @@ void KPasswordDialog::init()
 	size = m_pEdit2->sizeHint();
 	m_pEdit2->setFixedHeight(size.height());
 	m_pEdit2->setMinimumWidth(size.width());
-	h_lay->addWidget(m_pEdit2, 12);
-	h_lay->addStretch(4);
+	h_lay->addWidget(m_pEdit2);
+
+        // Row 5: Label saying whether the passwords match
+        m_pGrid->addRowSpacing(10, 10);
+        m_pGrid->setRowStretch(10, 12);
+        d->m_MatchLabel = new QLabel(m_pMain);
+        d->m_MatchLabel->setAlignment(AlignLeft|AlignVCenter|WordBreak);
+        m_pGrid->addMultiCellWidget(d->m_MatchLabel, 11, 11, 0, 2);
+        d->m_MatchLabel->setText(i18n("Passwords do not match"));
+        connect( m_pEdit, SIGNAL(textChanged(const QString&)), SLOT(enableOkBtn()) );
+        connect( m_pEdit2, SIGNAL(textChanged(const QString&)), SLOT(enableOkBtn()) );
+        enableOkBtn();
     }
 
     erase();
@@ -328,6 +351,7 @@ KPasswordDialog::~KPasswordDialog()
 }
 
 
+/* KDE 4: Make it const QString & */
 void KPasswordDialog::setPrompt(QString prompt)
 {
     m_pHelpLbl->setText(prompt);
@@ -342,20 +366,19 @@ QString KPasswordDialog::prompt() const
 }
 
 
+/* KDE 4: Make them const QString & */
 void KPasswordDialog::addLine(QString key, QString value)
 {
     if (m_Row > 3)
 	return;
 
     QLabel *lbl = new QLabel(key, m_pMain);
-    lbl->setAlignment(AlignTop);
-    lbl->setIndent(5);
+    lbl->setAlignment(AlignLeft|AlignTop);
     lbl->setFixedSize(lbl->sizeHint());
     m_pGrid->addWidget(lbl, m_Row+2, 0, AlignLeft);
 
     lbl = new QLabel(value, m_pMain);
     lbl->setAlignment(AlignTop|WordBreak);
-    lbl->setIndent(5);
     lbl->setFixedSize(275, lbl->heightForWidth(275));
     m_pGrid->addWidget(lbl, m_Row+2, 2, AlignLeft);
     m_Row++;
@@ -401,7 +424,7 @@ void KPasswordDialog::slotKeep(bool keep)
 }
 
 
-// static
+// static . antlarr: KDE 4: Make it const QString & prompt
 int KPasswordDialog::getPassword(QCString &password, QString prompt,
 	int *keep)
 {
@@ -418,7 +441,7 @@ int KPasswordDialog::getPassword(QCString &password, QString prompt,
 }
 
 
-// static
+// static . antlarr: KDE 4: Make it const QString & prompt
 int KPasswordDialog::getNewPassword(QCString &password, QString prompt)
 {
     KPasswordDialog *dlg = new KPasswordDialog(NewPassword, prompt);
@@ -441,4 +464,14 @@ void KPasswordDialog::disableCoreDumps()
 void KPasswordDialog::virtual_hook( int id, void* data )
 { KDialogBase::virtual_hook( id, data ); }
 
+void KPasswordDialog::enableOkBtn()
+{
+    if (m_Type == NewPassword) {
+      bool match = ((strcmp(m_pEdit->password(), m_pEdit2->password()))==0)
+                   && (strcmp(m_pEdit->password(), "") != 0);
+      enableButtonOK( match );
+      d->m_MatchLabel->setText( match?QString(i18n("Passwords match")):QString(i18n("Passwords do not match")) );
+   }
+}
+
 #include "kpassdlg.moc"
