@@ -4,6 +4,8 @@
  *   email                : rich@kde.org
  */
 
+#include <kconfig.h>
+
 #include <qapplication.h>
 #include <qlabel.h>
 #include <qlayout.h>
@@ -15,7 +17,11 @@
 #include <kpixmap.h>
 #include <kpixmapeffect.h>
 #include <kglobalsettings.h>
-#include <netwm.h>
+
+#include "config.h"
+#if defined Q_WS_X11 && ! defined K_WS_QTONLY
+#include <netwm.h> // schroder
+#endif
 
 #include "kpassivepopup.h"
 #include "kpassivepopup.moc"
@@ -27,7 +33,7 @@ static const int POPUP_FLAGS = Qt::WStyle_Customize | Qt::WDestructiveClose | Qt
 
 KPassivePopup::KPassivePopup( QWidget *parent, const char *name, WFlags f )
     : QFrame( 0, name, f ? f : POPUP_FLAGS ),
-      window( parent ? parent->winId() : 0L ), msgView( 0 ), layout( 0 ),
+      window( parent ? parent->winId() : 0L ), msgView( 0 ), topLayout( 0 ),
       hideDelay( DEFAULT_POPUP_TIME ), hideTimer( new QTimer( this, "hide_timer" ) ), m_autoDelete( false ), d( 0 )
 {
     init();
@@ -35,7 +41,7 @@ KPassivePopup::KPassivePopup( QWidget *parent, const char *name, WFlags f )
 
 KPassivePopup::KPassivePopup( WId win, const char *name, WFlags f )
     : QFrame( 0, name, f ? f : POPUP_FLAGS ),
-      window( win ), msgView( 0 ), layout( 0 ),
+      window( win ), msgView( 0 ), topLayout( 0 ),
       hideDelay( DEFAULT_POPUP_TIME ), hideTimer( new QTimer( this, "hide_timer" ) ), m_autoDelete( false ), d( 0 )
 {
     init();
@@ -58,10 +64,10 @@ void KPassivePopup::setView( QWidget *child )
     delete msgView;
     msgView = child;
 
-    delete layout;
-    layout = new QVBoxLayout( this, KDialog::marginHint(), KDialog::spacingHint() );
-    layout->addWidget( msgView );
-    layout->activate();
+    delete topLayout;
+    topLayout = new QVBoxLayout( this, KDialog::marginHint(), KDialog::spacingHint() );
+    topLayout->addWidget( msgView );
+    topLayout->activate();
 }
 
 void KPassivePopup::setView( const QString &caption, const QString &text,
@@ -160,6 +166,7 @@ void KPassivePopup::hideEvent( QHideEvent * )
 
 QRect KPassivePopup::defaultArea() const
 {
+#if defined Q_WS_X11 && ! defined K_WS_QTONLY
     NETRootInfo info( qt_xdisplay(),
                       NET::NumberOfDesktops |
                       NET::CurrentDesktop |
@@ -169,6 +176,11 @@ QRect KPassivePopup::defaultArea() const
     NETRect workArea = info.workArea( info.currentDesktop() );
     QRect r;
     r.setRect( workArea.pos.x, workArea.pos.y, 0, 0 ); // top left
+#else
+    // FIX IT
+    QRect r;
+    r.setRect( 100, 100, 200, 200 ); // top left
+#endif
     return r;
 }
 
@@ -176,6 +188,7 @@ void KPassivePopup::positionSelf()
 {
     QRect target;
 
+#if defined Q_WS_X11 && ! defined K_WS_QTONLY
     if ( window == 0L ) {
         target = defaultArea();
     }
@@ -206,7 +219,9 @@ void KPassivePopup::positionSelf()
                 }
         }
     }
-
+#else
+        target = defaultArea();
+#endif
     moveNear( target );
 }
 
@@ -218,7 +233,7 @@ void KPassivePopup::moveNear( QRect target )
     int w = width();
     int h = height();
 
-    QRect r( qApp->desktop()->screenGeometry( QPoint(x+w/2,y+h/2) ) );
+    QRect r = KGlobalSettings::desktopGeometry(QPoint(x+w/2,y+h/2));
 
     if ( x < ( r.width() / 2 ) )
 	x = x + target.width();
@@ -232,6 +247,9 @@ void KPassivePopup::moveNear( QRect target )
     if ( y < 0 )
         y = 0;
 
+    if ( x < 0 )
+	x = 0;
+
 #ifdef OLD_BITS
     if ( (x - w) >= 0  )
 	x = x - w;
