@@ -1,6 +1,6 @@
 /* vi: ts=8 sts=4 sw=4
  *
- * $Id: kpixmapio.cpp,v 1.23.4.2 2003/02/16 14:27:18 adrian Exp $
+ * $Id: kpixmapio.cpp,v 1.29 2003/09/03 15:33:18 schroder Exp $
  *
  * This file is part of the KDE project, module kdeui.
  * Copyright (C) 2000 Geert Jansen <jansen@kde.org>.
@@ -27,12 +27,12 @@
 #include <kdebug.h>
 #include "kpixmapio.h"
 
-#ifndef Q_WS_QWS
-#include <X11/X.h>
-#include <X11/Xlib.h>
-#include <X11/Xutil.h>
+#if defined Q_WS_X11 && ! defined K_WS_QTONLY
+#include <X11/X.h> // schroder
+#include <X11/Xlib.h> // schroder
+#include <X11/Xutil.h> // schroder
 #ifdef HAVE_MITSHM
-#include <X11/extensions/XShm.h>
+#include <X11/extensions/XShm.h> // schroder
 #endif
 #ifdef __osf__
 extern "C" int XShmQueryExtension(Display *display);
@@ -50,10 +50,11 @@ struct KPixmapIOPrivate
     int threshold;
     int bpp;
     int byteorder;
-#ifndef Q_WS_QWS
+#if defined Q_WS_X11 && ! defined K_WS_QTONLY
     XImage *ximage;
 #ifdef HAVE_MITSHM
     XShmSegmentInfo *shminfo;
+    bool first_try;
 #endif
 #else
     void *ximage;
@@ -109,6 +110,7 @@ KPixmapIO::KPixmapIO()
     d->ximage = XShmCreateImage(qt_xdisplay(), (Visual *) QPaintDevice::x11AppVisual(),
 	    QPaintDevice::x11AppDepth(), ZPixmap, 0L, d->shminfo, 10, 10);
     d->bpp = d->ximage->bits_per_pixel;
+    d->first_try = true;
     int bpp = d->bpp;
     if (d->ximage->byte_order == LSBFirst)
 	bpp++;
@@ -189,7 +191,7 @@ QPixmap KPixmapIO::convertToPixmap(const QImage &img)
 	dst.convertFromImage(img);
 	return dst;
     }
-	
+
 }
 
 
@@ -222,13 +224,9 @@ void KPixmapIO::putImage(QPixmap *dst, int dx, int dy, const QImage *src)
 	if( initXImage(src->width(), src->height()))
 	{
 	    convertToXImage(*src);
-#if QT_VERSION < 300
-	    XShmPutImage(qt_xdisplay(), dst->handle(), qt_xget_temp_gc(), d->ximage,
-		    dx, dy, 0, 0, src->width(), src->height(), false);
-#else
 	    XShmPutImage(qt_xdisplay(), dst->handle(), qt_xget_temp_gc(qt_xscreen(), false), d->ximage,
 		    dx, dy, 0, 0, src->width(), src->height(), false);
-#endif
+            // coolo: do we really need this here? I see no good for it
 	    XSync(qt_xdisplay(), false);
 	    doneXImage();
 	    fallback  = false;
@@ -365,6 +363,23 @@ void KPixmapIO::destroyShmSegment()
     }
 }
 
+static bool use_xshm = true;
+static unsigned long kpixmapio_serial;
+static int (*old_errhandler)(Display *dpy, XErrorEvent *ev) = 0;
+
+static int kpixmapio_errorhandler(Display *dpy, XErrorEvent *ev)
+{
+    if(ev->serial == kpixmapio_serial) {
+        /* assuming that xshm errors mean it can't be used at all
+           (e.g. remote display) */
+        use_xshm = false;
+        kdDebug(290) << "Disabling Xshm" << endl;
+        return 0;
+    } else {
+        // another error
+        return old_errhandler(dpy, ev);
+    }
+}
 
 bool KPixmapIO::createShmSegment(int size)
 {
@@ -387,18 +402,34 @@ bool KPixmapIO::createShmSegment(int size)
     }
 
     d->shminfo->readOnly = false;
+
+    if (d->first_try) {
+        // make sure that we don't get errors of old stuff
+        XSync(qt_xdisplay(), False);
+        old_errhandler = XSetErrorHandler(kpixmapio_errorhandler);
+        kpixmapio_serial = NextRequest(qt_xdisplay());
+    }
+
     if ( !XShmAttach(qt_xdisplay(), d->shminfo))
     {
 	kdWarning() << "X-Server could not attach shared memory segment.\n";
 	m_bShm = false;
 	shmdt(d->shminfo->shmaddr);
 	shmctl(d->shminfo->shmid, IPC_RMID, 0);
-	return false;
     }
 
+    if (d->first_try) {
+        XSync(qt_xdisplay(), false);
+
+        if (!use_xshm)
+            m_bShm = false;
+
+        XSetErrorHandler(old_errhandler);
+        d->first_try = false;
+    }
     d->shmsize = size;
-    XSync(qt_xdisplay(), false);
-    return true;
+
+    return m_bShm;
 }
 
 
