@@ -1,7 +1,7 @@
 /* This file is part of the KDE libraries
     Copyright (C) 2000 Carsten Pfeiffer <pfeiffer@kde.org>
 
-    library is free software; you can redistribute it and/or
+    This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Library General Public
     License as published by the Free Software Foundation; either
     version 2 of the License, or (at your option) any later version.
@@ -28,8 +28,8 @@ class QDragObject;
 
 /**
  * This is nothing but a QPushButton with drag-support and KGuiItem support. You have to call
- * @ref setDragEnabled( true ) and override the virtual method
- * @ref dragObject() to specify the QDragObject to be used.
+ * setDragEnabled( true ) and override the virtual method
+ * dragObject() to specify the QDragObject to be used.
  *
  * @short A QPushButton with drag-support and KGuiItem support
  * @author Carsten Pfeiffer <pfeiffer@kde.org>
@@ -81,6 +81,17 @@ public:
      */
     void setGuiItem( const KGuiItem& item );
 
+    /**
+     * Sets the Icon Set for this button. It also takes into account hte
+     * KGlobalSettings::showIconsOnPushButtons() setting.
+     */
+    void setIconSet( const QIconSet &iconSet );
+
+    /**
+    * Sets the text of the button
+    */
+    void setText( const QString &text );
+
 protected:
     /**
      * Reimplement this and return the QDragObject that should be used
@@ -99,7 +110,7 @@ protected:
     virtual void mouseMoveEvent( QMouseEvent * );
 
     /**
-     * Starts a drag (dragCopy() by default) using @ref dragObject()
+     * Starts a drag (dragCopy() by default) using dragObject()
      */
     virtual void startDrag();
 
@@ -116,9 +127,8 @@ private:
      * Initialize the KPushButton instance
      */
     void init( const KGuiItem &item );
-    
+
     void readSettings();
-    bool needIcons();
     static bool s_useIcons;
 
 protected:
