@@ -1,6 +1,6 @@
 /* vi: ts=8 sts=4 sw=4
  *
- * $Id: krootpixmap.cpp,v 1.18 2002/08/04 14:07:53 coolo Exp $
+ * $Id: krootpixmap.cpp,v 1.23.2.3 2004/05/24 10:57:23 waba Exp $
  *
  * This file is part of the KDE project, module kdeui.
  * Copyright (C) 1999,2000 Geert Jansen <jansen@kde.org>
@@ -20,6 +20,7 @@
 #include <kimageeffect.h>
 #include <kpixmapio.h>
 #include <kwinmodule.h>
+#include <kwin.h>
 #include <kdebug.h>
 #include <netwm.h>
 #include <dcopclient.h>
@@ -27,6 +28,13 @@
 #include <ksharedpixmap.h>
 #include <krootpixmap.h>
 
+class KRootPixmapData
+{
+public:
+    QWidget *toplevel;
+    KWinModule *kwin;
+};
+
 
 KRootPixmap::KRootPixmap( QWidget *widget, const char *name )
     : QObject(widget, name ? name : "KRootPixmap" ), m_pWidget(widget)
@@ -42,6 +50,7 @@ KRootPixmap::KRootPixmap( QWidget *widget, QObject *parent, const char *name )
 
 void KRootPixmap::init()
 {
+    d = new KRootPixmapData;
     m_Fade = 0;
     m_pPixmap = new KSharedPixmap;
     m_pTimer = new QTimer( this );
@@ -53,15 +62,18 @@ void KRootPixmap::init()
     connect(m_pPixmap, SIGNAL(done(bool)), SLOT(slotDone(bool)));
     connect(m_pTimer, SIGNAL(timeout()), SLOT(repaint()));
 
-    QObject *obj = m_pWidget;
-    while (obj->parent())
-	obj = obj->parent();
-    obj->installEventFilter(this);
+    d->kwin = new KWinModule( this );
+    connect( d->kwin, SIGNAL(currentDesktopChanged(int)), SLOT(desktopChanged(int)) );
+
+    d->toplevel = m_pWidget->topLevelWidget();
+    d->toplevel->installEventFilter(this);
+    m_pWidget->installEventFilter(this);
 }
 
 KRootPixmap::~KRootPixmap()
 {
     delete m_pPixmap;
+    delete d;
 }
 
 
@@ -134,6 +146,12 @@ bool KRootPixmap::eventFilter(QObject *, QEvent *event)
 	m_pTimer->start(0, true);
 	break;
 
+    case QEvent::Reparent:
+        d->toplevel->removeEventFilter(this);
+        d->toplevel = m_pWidget->topLevelWidget();
+        d->toplevel->installEventFilter(this);
+        break;
+
     default:
 	break;
     }
@@ -141,6 +159,16 @@ bool KRootPixmap::eventFilter(QObject *, QEvent *event)
     return false; // always continue processing
 }
 
+void KRootPixmap::desktopChanged( int desk )
+{
+    if( !m_pWidget->isVisible())
+        return; // not visible, no need to update
+    QWidget* widget = m_pWidget->topLevelWidget();
+    if( !widget->testWFlags( WX11BypassWM )
+        && !KWin::windowInfo( widget->winId(), NET::WMDesktop ).isOnCurrentDesktop())
+        return; // not on current desktop -> not visible, no need to update
+    repaint(true);
+}
 
 void KRootPixmap::repaint()
 {
@@ -163,6 +191,7 @@ void KRootPixmap::repaint(bool force)
 	(m_pWidget->height() < m_Rect.height())
        )
     {
+        m_Rect = QRect(p1, p2);
  	updateBackground( m_pPixmap );
 	return;
     }
@@ -170,13 +199,21 @@ void KRootPixmap::repaint(bool force)
     m_Desk = currentDesktop();
 
     // KSharedPixmap will correctly generate a tile for us.
-    m_pPixmap->loadFromShared(QString("DESKTOP%1").arg(m_Desk), m_Rect);
+    m_pPixmap->loadFromShared(pixmapName(m_Desk), m_Rect);
 }
 
 bool KRootPixmap::isAvailable() const
 {
-    QString name = QString("DESKTOP%1").arg( currentDesktop() );
-    return m_pPixmap->isAvailable(name);
+    return m_pPixmap->isAvailable(pixmapName(m_Desk));
+}
+
+QString KRootPixmap::pixmapName(int desk) {
+    QString pattern = QString("DESKTOP%1");
+    int screen_number = DefaultScreen(qt_xdisplay());
+    if (screen_number) {
+        pattern = QString("SCREEN%1-DESKTOP").arg(screen_number) + "%1";
+    }
+    return pattern.arg( desk );
 }
 
 
@@ -189,7 +226,13 @@ void KRootPixmap::enableExports()
     QByteArray data;
     QDataStream args( data, IO_WriteOnly );
     args << 1;
-    client->send( "kdesktop", "KBackgroundIface", "setExport(int)", data );
+
+    QCString appname( "kdesktop" );
+    int screen_number = DefaultScreen(qt_xdisplay());
+    if ( screen_number )
+        appname.sprintf("kdesktop-screen-%d", screen_number );
+
+    client->send( appname, "KBackgroundIface", "setExport(int)", data );
 }
 
 
