@@ -96,7 +96,7 @@ void KXYSelector::paintEvent( QPaintEvent *ev )
 
 	QBrush brush;
 	qDrawShadePanel( &painter, 0, 0, width(), height(), colorGroup(),
-			TRUE, 2, &brush );
+			true, 2, &brush );
 
 	drawContents( &painter );
 	if (paintRect.contains(cursorRect))
@@ -115,18 +115,48 @@ void KXYSelector::paintEvent( QPaintEvent *ev )
 
 void KXYSelector::mousePressEvent( QMouseEvent *e )
 {
-	setPosition( e->pos().x() - 2, e->pos().y() - 2 );
+	int xVal, yVal;
+	valuesFromPosition( e->pos().x() - 2, e->pos().y() - 2, xVal, yVal );
+	setValues( xVal, yVal );
 
 	emit valueChanged( xPos, yPos );
 }
 
 void KXYSelector::mouseMoveEvent( QMouseEvent *e )
 {
-	setPosition( e->pos().x() - 2, e->pos().y() - 2 );
-
+	int xVal, yVal;
+	valuesFromPosition( e->pos().x() - 2, e->pos().y() - 2, xVal, yVal );
+	setValues( xVal, yVal );
+	
 	emit valueChanged( xPos, yPos );
 }
 
+void KXYSelector::wheelEvent( QWheelEvent *e )
+{
+	if ( e->orientation() == Qt::Horizontal )
+		setValues( xValue() + e->delta()/120, yValue() );
+	else	
+		setValues( xValue(), yValue() + e->delta()/120 );
+	
+	emit valueChanged( xPos, yPos );
+}
+
+void KXYSelector::valuesFromPosition( int x, int y, int &xVal, int &yVal ) const
+{
+	xVal = ( (maxX-minX) * (x-2) ) / ( width()-4 );
+	yVal = maxY - ( ( (maxY-minY) * (y-2) ) / ( height()-4 ) );
+	
+	if ( xVal > maxX )
+		xVal = maxX;
+	else if ( xVal < minX )
+		xVal = minX;
+	
+	if ( yVal > maxY )
+		yVal = maxY;
+	else if ( yVal < minY )
+		yVal = minY;
+}
+
 void KXYSelector::setPosition( int xp, int yp )
 {
 	if ( xp < 2 )
@@ -151,19 +181,6 @@ void KXYSelector::setPosition( int xp, int yp )
 	py = yp;
 
 	painter.end();
-
-	xPos = ( (maxX-minX) * (xp-2) ) / ( width()-4 );
-	yPos = maxY - ( ( (maxY-minY) * (yp-2) ) / ( height()-4 ) );
-	
-	if ( xPos > maxX )
-		xPos = maxX;
-	else if ( xPos < minX )
-		xPos = minX;
-	
-	if ( yPos > maxY )
-		yPos = maxY;
-	else if ( yPos < minY )
-		yPos = minY;
 }
 
 void KXYSelector::drawContents( QPainter * )
@@ -191,14 +208,14 @@ KSelector::KSelector( QWidget *parent, const char *name )
 	: QWidget( parent, name ), QRangeControl()
 {
 	_orientation = Horizontal;
-	_indent = TRUE;
+	_indent = true;
 }
 
 KSelector::KSelector( Orientation o, QWidget *parent, const char *name )
 	: QWidget( parent, name ), QRangeControl()
 {
 	_orientation = o;
-	_indent = TRUE;
+	_indent = true;
 }
 
 
@@ -228,14 +245,14 @@ void KSelector::paintEvent( QPaintEvent * )
 	{
 		if ( orientation() == Vertical )
 			qDrawShadePanel( &painter, 0, 3, width()-5, height()-6,
-				colorGroup(), TRUE, 2, &brush );
+				colorGroup(), true, 2, &brush );
 		else
 			qDrawShadePanel( &painter, 3, 0, width()-6, height()-5,
-				colorGroup(), TRUE, 2, &brush );
+				colorGroup(), true, 2, &brush );
 	}
 
 	QPoint pos = calcArrowPos( value() );
-	drawArrow( &painter, TRUE, pos );   
+	drawArrow( &painter, true, pos );   
 
 	painter.end();
 }
@@ -250,6 +267,12 @@ void KSelector::mouseMoveEvent( QMouseEvent *e )
 	moveArrow( e->pos() );
 }
 
+void KSelector::wheelEvent( QWheelEvent *e )
+{
+	int val = value() + e->delta()/120;
+	setValue( val );
+}
+
 void KSelector::valueChange()
 {
 	QPainter painter;
@@ -258,12 +281,14 @@ void KSelector::valueChange()
 	painter.begin( this );
 
 	pos = calcArrowPos( prevValue() );
-	drawArrow( &painter, FALSE, pos );   
+	drawArrow( &painter, false, pos );   
 
 	pos = calcArrowPos( value() );
-	drawArrow( &painter, TRUE, pos );   
+	drawArrow( &painter, true, pos );   
 
 	painter.end();
+
+	emit valueChanged( value() );
 }
 
 void KSelector::moveArrow( const QPoint &pos )
@@ -277,12 +302,6 @@ void KSelector::moveArrow( const QPoint &pos )
 		val = ( maxValue() - minValue() ) * (width()-pos.x()-3)
 				/ (width()-10) + minValue();
 
-	if ( val > maxValue() )
-		val = maxValue();
-	if ( val < minValue() )
-		val = minValue();
-
-	emit valueChanged( val );
 	setValue( val );
 }
 
