@@ -1,6 +1,6 @@
 /* vi: ts=8 sts=4 sw=4
  *
- * $Id: ksharedpixmap.cpp,v 1.15 2002/08/13 17:10:12 coolo Exp $
+ * $Id: ksharedpixmap.cpp,v 1.17.2.1 2004/05/24 10:30:12 waba Exp $
  *
  * This file is part of the KDE libraries.
  * Copyright (C) 1999,2000 Geert Jansen <jansen@kde.org>
@@ -21,7 +21,7 @@
 #include <qwindowdefs.h>
 #include <qwidget.h>
 
-#ifndef Q_WS_QWS
+#if defined Q_WS_X11 && ! defined K_WS_QTONLY
 
 #include <kapplication.h>
 #include <krootprop.h>
@@ -29,16 +29,16 @@
 #include <kdebug.h>
 #include <stdlib.h> // for abs
 
-#include <X11/Xlib.h>
+#include <X11/Xlib.h> // schroder
 
 // Make sure to include all this X-based shit before we clean up the mess.
 // Needed for --enable-final. Not needed by this file itself!
-#include <X11/Xutil.h>
+#include <X11/Xutil.h> // schroder
 #ifdef HAVE_MITSHM
-#include <X11/extensions/XShm.h>
+#include <X11/extensions/XShm.h> // schroder
 #endif
 
-#include <netwm.h>
+#include <netwm.h> // schroder
 
 // Clean up the mess
 
@@ -98,11 +98,11 @@ bool KSharedPixmap::isAvailable(const QString & name) const
 
 bool KSharedPixmap::loadFromShared(const QString & name, const QRect & rect)
 {
+    d->rect = rect;
     if (d->selection != None)
 	// already active
 	return false;
 
-    d->rect = rect;
     QPixmap::resize(0, 0); // invalidate
 
     QString str = QString("KDESHPIXMAP:%1").arg(name);
@@ -164,11 +164,7 @@ bool KSharedPixmap::x11Event(XEvent *event)
     if (d->rect.isEmpty())
     {
 	QPixmap::resize(width, height);
-#if QT_VERSION < 300
-	XCopyArea(qt_xdisplay(), *pixmap_id, ((KPixmap*)this)->handle(), qt_xget_temp_gc(),
-#else
 	XCopyArea(qt_xdisplay(), *pixmap_id, ((KPixmap*)this)->handle(), qt_xget_temp_gc(qt_xscreen(), false),
-#endif
 		0, 0, width, height, 0, 0);
 
         XFree(pixmap_id);
@@ -206,16 +202,6 @@ bool KSharedPixmap::x11Event(XEvent *event)
 
     QPixmap::resize( tw+origin.x(), th+origin.y() );
 
-#if QT_VERSION < 300
-    XCopyArea(qt_xdisplay(), *pixmap_id, ((KPixmap*)this)->handle(), qt_xget_temp_gc(),
-            xa, ya, t1w+origin.x(), t1h+origin.y(), origin.x(), origin.y() );
-    XCopyArea(qt_xdisplay(), *pixmap_id, ((KPixmap*)this)->handle(), qt_xget_temp_gc(),
-	    0, ya, tw-t1w, t1h, t1w, 0);
-    XCopyArea(qt_xdisplay(), *pixmap_id, ((KPixmap*)this)->handle(), qt_xget_temp_gc(),
-	    xa, 0, t1w, th-t1h, 0, t1h);
-    XCopyArea(qt_xdisplay(), *pixmap_id, ((KPixmap*)this)->handle(), qt_xget_temp_gc(),
-	    0, 0, tw-t1w, th-t1h, t1w, t1h);
-#else
     XCopyArea(qt_xdisplay(), *pixmap_id, ((KPixmap*)this)->handle(), qt_xget_temp_gc(qt_xscreen(), false),
             xa, ya, t1w+origin.x(), t1h+origin.y(), origin.x(), origin.y() );
     XCopyArea(qt_xdisplay(), *pixmap_id, ((KPixmap*)this)->handle(), qt_xget_temp_gc(qt_xscreen(), false),
@@ -224,7 +210,6 @@ bool KSharedPixmap::x11Event(XEvent *event)
 	    xa, 0, t1w, th-t1h, 0, t1h);
     XCopyArea(qt_xdisplay(), *pixmap_id, ((KPixmap*)this)->handle(), qt_xget_temp_gc(qt_xscreen(), false),
 	    0, 0, tw-t1w, th-t1h, t1w, t1h);
-#endif
 
     XFree(pixmap_id);
 
