@@ -23,7 +23,7 @@
 #include <kconfig.h>
 #include <kglobal.h>
 
-// $Id: kstatusbar.cpp,v 1.43.2.1 2003/02/26 13:18:00 davec Exp $
+// $Id: kstatusbar.cpp,v 1.46 2003/12/16 00:44:52 kniederk Exp $
 
 KStatusBarLabel::KStatusBarLabel( const QString& text, int _id,
                                  KStatusBar *parent, const char *name) :
@@ -59,6 +59,13 @@ void KStatusBarLabel::mouseReleaseEvent (QMouseEvent *)
   emit itemReleased (id);
 }
 
+class KStatusBarPrivate
+{
+  public:
+
+    // KDE 4.0: Move into KStatusBarLabel
+    QMap<int, bool> is_permanent_item;
+};
 
 KStatusBar::KStatusBar( QWidget *parent, const char *name )
   : QStatusBar( parent, name )
@@ -71,17 +78,23 @@ KStatusBar::KStatusBar( QWidget *parent, const char *name )
   bool grip_enabled = config->readBoolEntry(QString::fromLatin1("SizeGripEnabled"), false);
   setSizeGripEnabled(grip_enabled);
   config->setGroup(group);
+  d = new KStatusBarPrivate;
 }
 
 KStatusBar::~KStatusBar ()
 {
+  delete d;
 }
 
 void KStatusBar::insertItem( const QString& text, int id, int stretch, bool permanent)
 {
+  if (items[id])
+    kdDebug() << "KStatusBar::insertItem: item id " << id << " already exists." << endl;
+
   KStatusBarLabel *l = new KStatusBarLabel (text, id, this);
   l->setFixedHeight(fontMetrics().height()+2);
   items.insert(id, l);
+  d->is_permanent_item.insert(id, permanent);
   addWidget (l, stretch, permanent);
   l->show();
 }
@@ -93,18 +106,31 @@ void KStatusBar::removeItem (int id)
   {
     removeWidget (l);
     items.remove(id);
+    d->is_permanent_item.remove(id);
     delete l;
   }
   else
     kdDebug() << "KStatusBar::removeItem: bad item id: " << id << endl;
 }
 
+bool KStatusBar::hasItem( int id ) const
+{
+  KStatusBarLabel *l = items[id];
+  if (l)
+    return true;
+  else
+    return false;
+}
+
 void KStatusBar::changeItem( const QString& text, int id )
 {
   KStatusBarLabel *l = items[id];
   if (l)
   {
-    clear();
+    if(!d->is_permanent_item[id])
+    {
+      clear();
+    }
     l->setText(text);
     reformat();
   }
