@@ -42,6 +42,8 @@ KGuiItem KStdGuiItem::guiItem ( StdItem ui_enum )
   case Forward  : return forward();
   case Print    : return print();
   case Continue : return cont();
+  case Open     : return open();
+  case Quit     : return quit();
   default       : return KGuiItem();
   };
 }
@@ -65,6 +67,8 @@ QString KStdGuiItem::stdItem( StdItem ui_enum )
   case Forward  : return QString::fromLatin1("forward");
   case Print    : return QString::fromLatin1("print");
   case Continue : return QString::fromLatin1("continue");
+  case Open     : return QString::fromLatin1("open");
+  case Quit     : return QString::fromLatin1("quit");
   default       : return QString::null;
   };
 }
@@ -73,7 +77,7 @@ KGuiItem KStdGuiItem::ok()
 {
   return KGuiItem( i18n( "&OK" ), "button_ok", i18n( "Accept settings" ),
                    i18n( "If you press the <b>OK</b> button, all changes\n"
-                         "you made will be used to proceed." ) ); 
+                         "you made will be used to proceed." ) );
 }
 
 
@@ -106,7 +110,7 @@ KGuiItem KStdGuiItem::save()
 
 KGuiItem KStdGuiItem::dontSave()
 {
-  return KGuiItem( i18n( "&Don't Save" ), "filedontsave",
+  return KGuiItem( i18n( "&Don't Save" ), "",
                    i18n( "Don't save data" ) );
 }
 
@@ -173,7 +177,7 @@ QPair<KGuiItem, KGuiItem> KStdGuiItem::backAndForward()
 
 KGuiItem KStdGuiItem::print()
 {
-  return KGuiItem( i18n( "&Print" ), "fileprint",
+  return KGuiItem( i18n( "&Print..." ), "fileprint",
                    i18n( "Opens the print dialog to print "
                          "the current document" ) );
 }
@@ -184,4 +188,16 @@ KGuiItem KStdGuiItem::cont()
                    i18n( "Continue operation" ) );
 }
 
+KGuiItem KStdGuiItem::open()
+{
+  return KGuiItem( i18n( "&Open..." ), "fileopen",
+                   i18n( "Open file" ) );
+}
+
+KGuiItem KStdGuiItem::quit()
+{
+  return KGuiItem( i18n( "&Quit" ), "exit",
+                   i18n( "Quit application" ) );
+}
+
 // vim: set ts=2 sts=2 sw=2 et:
