@@ -18,19 +18,30 @@
     Boston, MA 02111-1307, USA.
 */
 
+#include "config.h"
 #include "kaction.h"
 #include "kshortcut.h"
 #include "ksystemtray.h"
 #include "kpopupmenu.h"
 #include "kapplication.h"
 #include "klocale.h"
-#include <kwin.h>
-#include <kwinmodule.h>
+
+#if defined Q_WS_X11 && ! defined K_WS_QTONLY
+#include <kwin.h> // schroder
+#include <kwinmodule.h> // schroder
+#endif
+
 #include <kiconloader.h>
+#include <kconfig.h>
+
+#if defined Q_WS_X11 && ! defined K_WS_QTONLY
+#include <qxembed.h> // schroder
+#endif
 
 #include <qapplication.h>
-#ifndef Q_WS_QWS
-#include <X11/Xlib.h>
+
+#if defined Q_WS_X11 && ! defined K_WS_QTONLY
+#include <X11/Xlib.h> // schroder
 #ifndef KDE_USE_FINAL
 const int XFocusOut = FocusOut;
 const int XFocusIn = FocusIn;
@@ -41,7 +52,7 @@ const int XFocusIn = FocusIn;
 #undef KeyRelease
 
 extern Time qt_x_time;
-#endif
+#endif // Q_WS_X11 && ! K_WS_QTONLY
 
 class KSystemTrayPrivate
 {
@@ -63,13 +74,19 @@ public:
 KSystemTray::KSystemTray( QWidget* parent, const char* name )
     : QLabel( parent, name, WType_TopLevel )
 {
+#if defined Q_WS_X11 && ! defined K_WS_QTONLY
+    QXEmbed::initialize();
+#endif
+    
     d = new KSystemTrayPrivate;
     d->actionCollection = new KActionCollection(this);
 
-#ifndef Q_WS_QWS
+#if defined Q_WS_X11 && ! defined K_WS_QTONLY
+//#ifndef Q_WS_QWS
     // FIXME(E): Talk with QWS
     KWin::setSystemTrayWindowFor( winId(), parent?parent->topLevelWidget()->winId(): qt_xrootwin() );
     setBackgroundMode(X11ParentRelative);
+    setBackgroundOrigin(WindowOrigin);
 #endif
     hasQuit = 0;
     menu = new KPopupMenu( this );
@@ -80,11 +97,13 @@ KSystemTray::KSystemTray( QWidget* parent, const char* name )
     if (parentWidget())
     {
         connect(quitAction, SIGNAL(activated()), parentWidget(), SLOT(close()));
-        new KAction(i18n("Minimize"), KShortcut(), 
+        new KAction(i18n("Minimize"), KShortcut(),
                     this, SLOT( minimizeRestoreAction() ),
                     d->actionCollection, "minimizeRestore");
-	KWin::Info info = KWin::info( parentWidget()->winId());
-	d->on_all_desktops = info.onAllDesktops;
+#if defined Q_WS_X11 && ! defined K_WS_QTONLY
+	KWin::WindowInfo info = KWin::windowInfo( parentWidget()->winId());
+	d->on_all_desktops = info.onAllDesktops();
+#endif
     }
     else
     {
@@ -125,7 +144,8 @@ void KSystemTray::showEvent( QShowEvent * )
 void KSystemTray::enterEvent( QEvent* e )
 {
 #if QT_VERSION < 0x030200
-#ifndef Q_WS_QWS
+#if defined Q_WS_X11 && ! defined K_WS_QTONLY
+//#ifndef Q_WS_QWS
     // FIXME(E): Implement for Qt Embedded
     if ( !qApp->focusWidget() ) {
 	XEvent ev;
@@ -166,9 +186,9 @@ void KSystemTray::mousePressEvent( QMouseEvent *e )
 	if ( parentWidget() ) {
             KAction* action = d->actionCollection->action("minimizeRestore");
 	    if ( parentWidget()->isVisible() )
-		action->setText( i18n("Minimize") );
+		action->setText( i18n("&Minimize") );
 	    else
-		action->setText( i18n("Restore") );
+		action->setText( i18n("&Restore") );
 	}
 	contextMenuAboutToShow( menu );
 	menu->popup( e->globalPos() );
@@ -188,7 +208,6 @@ void KSystemTray::contextMenuAboutToShow( KPopupMenu* )
 {
 }
 
-
 // called from the popup menu - always do what the menu entry says,
 // i.e. if the window is shown, no matter if active or not, the menu
 // entry is "minimize", otherwise it's "restore"
@@ -210,22 +229,43 @@ void KSystemTray::activateOrHide()
     if ( !pw )
 	return;
 
-    KWin::Info info = KWin::info( pw->winId() );
-    // mapped = not hidden by calling hide()
-    bool mapped = (info.mappingState != NET::Withdrawn);
-#if QT_VERSION >= 0x030200
-    if( mapped && !pw->isActiveWindow()) // visible not active -> activate
-#else
-    // SELI using !pw->isActiveWindow() should be enough here,
-    // but it doesn't work - e.g. with kscd, the "active" window
-    // is the widget docked in Kicker
-    if ( mapped && ( KWinModule().activeWindow() != pw->winId() ))
-#endif
+#if defined Q_WS_X11 && ! defined K_WS_QTONLY
+    KWin::WindowInfo info1 = KWin::windowInfo( pw->winId(), NET::XAWMState | NET::WMState );
+    // mapped = visible (but possibly obscured)
+    bool mapped = (info1.mappingState() == NET::Visible) && !info1.isMinimized();
+//    - not mapped -> show, raise, focus
+//    - mapped
+//        - obscured -> raise, focus
+//        - not obscured -> hide
+    if( !mapped )
+        minimizeRestore( true );
+    else
     {
-        KWin::setActiveWindow( pw->winId() );
-        return;
+        KWinModule module;
+        for( QValueList< WId >::ConstIterator it = module.stackingOrder().fromLast();
+             it != module.stackingOrder().end() && (*it) != pw->winId();
+             --it )
+        {
+            KWin::WindowInfo info2 = KWin::windowInfo( *it,
+                NET::WMGeometry | NET::XAWMState | NET::WMState | NET::WMWindowType );
+            if( info2.mappingState() != NET::Visible )
+                continue; // not visible on current desktop -> ignore
+            if( !info2.geometry().intersects( pw->geometry()))
+                continue; // not obscuring the window -> ignore
+            if( !info1.hasState( NET::KeepAbove ) && info2.hasState( NET::KeepAbove ))
+                continue; // obscured by window kept above -> ignore
+            NET::WindowType type = info2.windowType( NET::NormalMask | NET::DesktopMask
+                | NET::DockMask | NET::ToolbarMask | NET::MenuMask | NET::DialogMask
+                | NET::OverrideMask | NET::TopMenuMask | NET::UtilityMask | NET::SplashMask );
+            if( type == NET::Dock || type == NET::TopMenu )
+                continue; // obscured by dock or topmenu -> ignore
+            pw->raise();
+            KWin::activateWindow( pw->winId());
+            return;
+        }
+        minimizeRestore( false ); // hide
     }
-    minimizeRestore( !mapped );
+#endif
 }
 
 void KSystemTray::minimizeRestore( bool restore )
@@ -233,28 +273,39 @@ void KSystemTray::minimizeRestore( bool restore )
     QWidget* pw = parentWidget();
     if( !pw )
 	return;
-    KWin::Info info = KWin::info( pw->winId() );
+#if defined Q_WS_X11 && ! defined K_WS_QTONLY
+    KWin::WindowInfo info = KWin::windowInfo( pw->winId(), NET::WMGeometry | NET::WMDesktop );
     if ( restore )
     {
-#ifndef Q_WS_QWS //FIXME
+//#ifndef Q_WS_QWS //FIXME
+//#if defined Q_WS_X11 && ! defined K_WS_QTONLY
 	if( d->on_all_desktops )
 	    KWin::setOnAllDesktops( pw->winId(), true );
 	else
 	    KWin::setOnDesktop( pw->winId(), KWin::currentDesktop());
-        pw->move( info.geometry.topLeft() ); // avoid placement policies
+        pw->move( info.geometry().topLeft() ); // avoid placement policies
         pw->show();
-	KWin::setActiveWindow( pw->winId() );
-#endif
+        pw->raise();
+	KWin::activateWindow( pw->winId() );
     } else {
-	d->on_all_desktops = info.onAllDesktops;
+	d->on_all_desktops = info.onAllDesktops();
 	pw->hide();
     }
+#endif
 }
 
 KActionCollection* KSystemTray::actionCollection()
 {
     return d->actionCollection;
 }
+    
+QPixmap KSystemTray::loadIcon( const QString &icon, KInstance *instance )
+{
+    KConfig *appCfg = kapp->config();
+    KConfigGroupSaver configSaver(appCfg, "System Tray");
+    int iconWidth = appCfg->readNumEntry("systrayIconWidth", 22);
+    return instance->iconLoader()->loadIcon( icon, KIcon::Panel, iconWidth );
+}
 
 void KSystemTray::virtual_hook( int, void* )
 { /*BASE::virtual_hook( id, data );*/ }
