@@ -1,7 +1,8 @@
 /*****************************************************************
 
-Copyright (c) 2000, 2001 Matthias Hoelzer-Kluepfel
-                         Tobias Koenig <tokoe82@yahoo.de>
+Copyright (c) 2000-2003 Matthias Hoelzer-Kluepfel <mhk@kde.org>
+                        Tobias Koenig <tokoe@kde.org>
+                        Daniel Molkentin <molkentin@kde.org>
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
@@ -36,7 +37,6 @@ CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 #include <kconfig.h>
 #include <kdebug.h>
 #include <kglobal.h>
-#include <kglobalsettings.h>
 #include <kiconloader.h>
 #include <klocale.h>
 #include <kpushbutton.h>
@@ -58,15 +58,37 @@ KTipDatabase::KTipDatabase(const QString &_tipFile)
 
     loadTips(tipFile);
 
-    if (tips.count())
-	current = kapp->random() % tips.count();
+    if (!mTips.isEmpty())
+	mCurrent = kapp->random() % mTips.count();
 }
 
 
+KTipDatabase::KTipDatabase( const QStringList& tipsFiles )
+{
+   if ( tipsFiles.isEmpty() || ( ( tipsFiles.count() == 1 ) && tipsFiles.first().isEmpty() ) )
+   {
+       addTips(QString::fromLatin1(KGlobal::instance()->aboutData()->appName()) + "/tips");
+   }
+   else
+   {
+       for (QStringList::ConstIterator it = tipsFiles.begin(); it != tipsFiles.end(); ++it)
+           addTips( *it );
+   }
+    if (!mTips.isEmpty())
+	mCurrent = kapp->random() % mTips.count();
+
+}
+
+void KTipDatabase::loadTips(const QString &tipFile)
+{
+    mTips.clear();
+    addTips(tipFile);
+}
+
 // if you change something here, please update the script
 // preparetips, which depends on extracting exactly the same
 // text as done here.
-void KTipDatabase::loadTips(const QString &tipFile)
+void KTipDatabase::addTips(const QString& tipFile )
 {
     QString fileName = locate("data", tipFile);
 
@@ -83,8 +105,6 @@ void KTipDatabase::loadTips(const QString &tipFile)
 	return;
     }
 
-    tips.clear();
-
     QString content = file.readAll();
 
     int pos = -1;
@@ -93,38 +113,39 @@ void KTipDatabase::loadTips(const QString &tipFile)
 	QString tip = content.mid(pos + 6, content.find("</html>", pos, false) - pos - 6);
 	if (tip.startsWith("\n"))
 		tip = tip.mid(1);
-	tips.append(tip);
+	mTips.append(tip);
     }
 
     file.close();
+
 }
 
 void KTipDatabase::nextTip()
 {
-    if (tips.count() == 0)
+    if (mTips.isEmpty())
 	return ;
-    current += 1;
-    if (current >= (int) tips.count())
-	current = 0;
+    mCurrent += 1;
+    if (mCurrent >= (int) mTips.count())
+	mCurrent = 0;
 }
 
 
 void KTipDatabase::prevTip()
 {
-    if (tips.count() == 0)
+    if (mTips.isEmpty())
 	return ;
-    current -= 1;
-    if (current < 0)
-	current = tips.count() - 1;
+    mCurrent -= 1;
+    if (mCurrent < 0)
+	mCurrent = mTips.count() - 1;
 }
 
 
 QString KTipDatabase::tip() const
 {
-    return tips[current];
+    return mTips[mCurrent];
 }
 
-KTipDialog *KTipDialog::_instance = 0;
+KTipDialog *KTipDialog::mInstance = 0;
 
 
 KTipDialog::KTipDialog(KTipDatabase *db, QWidget *parent, const char *name)
@@ -139,28 +160,28 @@ KTipDialog::KTipDialog(KTipDatabase *db, QWidget *parent, const char *name)
     QImage img;
     int h,s,v;
 
-    _blendedColor = KGlobalSettings::activeTitleColor();
-    _blendedColor.hsv(&h,&s,&v);
-    _blendedColor.setHsv(h,s*(71/76.0),v*(67/93.0));
+    mBlendedColor = KGlobalSettings::activeTitleColor();
+    mBlendedColor.hsv(&h,&s,&v);
+    mBlendedColor.setHsv(h, int(s*(71/76.0)), int(v*(67/93.0)));
 
     if (!isTipDialog)
     {
 	img = QImage(locate("data", "kdewizard/pics/wizard_small.png"));
 	// colorize and check to figure the correct color
-	KIconEffect::colorize(img, _blendedColor, 1.0);
+	KIconEffect::colorize(img, mBlendedColor, 1.0);
 	QRgb colPixel( img.pixel(0,0) );
 
-	_blendedColor = QColor(qRed(colPixel),qGreen(colPixel),qBlue(colPixel));
+	mBlendedColor = QColor(qRed(colPixel),qGreen(colPixel),qBlue(colPixel));
     }
 
-    _baseColor = KGlobalSettings::alternateBackgroundColor();
-    _baseColor.hsv(&h,&s,&v);
-    _baseColor.setHsv(h,s*(10/6.0),v*(93/99.0));
+    mBaseColor = KGlobalSettings::alternateBackgroundColor();
+    mBaseColor.hsv(&h,&s,&v);
+    mBaseColor.setHsv(h, int(s*(10/6.0)), int(v*(93/99.0)));
 
-    _textColor = KGlobalSettings::textColor();
+    mTextColor = KGlobalSettings::textColor();
 
 
-    _database = db;
+    mDatabase = db;
 
     setCaption(i18n("Tip of the Day"));
     setIcon(KGlobal::iconLoader()->loadIcon("ktip", KIcon::Small));
@@ -190,45 +211,47 @@ KTipDialog::KTipDialog(KTipDatabase *db, QWidget *parent, const char *name)
 
     QHBox *tl = new QHBox(hbox);
     tl->setMargin(7);
-    tl->setBackgroundColor(_blendedColor);
+    tl->setBackgroundColor(mBlendedColor);
 
     QHBox *topLeft = new QHBox(tl);
     topLeft->setMargin(15);
-    topLeft->setBackgroundColor(_baseColor);
+    topLeft->setBackgroundColor(mBaseColor);
 
-    _tipText = new KTextBrowser(topLeft);
+    mTipText = new KTextBrowser(topLeft);
 
-    _tipText->setWrapPolicy( QTextEdit::AtWordOrDocumentBoundary );
-    _tipText->mimeSourceFactory()->addFilePath(
+    mTipText->setWrapPolicy( QTextEdit::AtWordOrDocumentBoundary );
+    mTipText->mimeSourceFactory()->addFilePath(
 	KGlobal::dirs()->findResourceDir("data", "kdewizard/pics")+"kdewizard/pics/");
-    _tipText->setFrameStyle(QFrame::NoFrame | QFrame::Plain);
-    _tipText->setHScrollBarMode(QScrollView::AlwaysOff);
-    _tipText->setLinkUnderline(false);
+    mTipText->setFrameStyle(QFrame::NoFrame | QFrame::Plain);
+    mTipText->setHScrollBarMode(QScrollView::AlwaysOff);
+    mTipText->setLinkUnderline(false);
 
-    QStyleSheet *sheet = _tipText->styleSheet();
+    QStyleSheet *sheet = mTipText->styleSheet();
     QStyleSheetItem *item = sheet->item("a");
     item->setFontWeight(QFont::Bold);
-    _tipText->setStyleSheet(sheet);
-    QPalette pal = _tipText->palette();
-    pal.setColor( QPalette::Active, QColorGroup::Link, _blendedColor );
-    pal.setColor( QPalette::Inactive, QColorGroup::Link, _blendedColor );
-    _tipText->setPalette(pal);
+    mTipText->setStyleSheet(sheet);
+    QPalette pal = mTipText->palette();
+    pal.setColor( QPalette::Active, QColorGroup::Link, mBlendedColor );
+    pal.setColor( QPalette::Inactive, QColorGroup::Link, mBlendedColor );
+    mTipText->setPalette(pal);
 
     QStringList icons = KGlobal::dirs()->resourceDirs("icon");
     QStringList::Iterator it;
     for (it = icons.begin(); it != icons.end(); ++it)
-        _tipText->mimeSourceFactory()->addFilePath(*it);
+        mTipText->mimeSourceFactory()->addFilePath(*it);
 
     if (!isTipDialog)
     {
 	QLabel *l = new QLabel(hbox);
 	l->setPixmap(img);
-	l->setBackgroundColor(_blendedColor);
+	l->setBackgroundColor(mBlendedColor);
 	l->setAlignment(Qt::AlignRight | Qt::AlignBottom);
 
 	resize(550, 230);
         QSize sh = size();
-	QRect rect = KApplication::desktop()->screenGeometry();
+
+        QRect rect = KGlobalSettings::splashScreenDesktopGeometry();
+
         move(rect.x() + (rect.width() - sh.width())/2,
 	rect.y() + (rect.height() - sh.height())/2);
     }
@@ -238,8 +261,8 @@ KTipDialog::KTipDialog(KTipDatabase *db, QWidget *parent, const char *name)
 
     QHBoxLayout *hbox2 = new QHBoxLayout(vbox, 4);
 
-    _tipOnStart = new QCheckBox(i18n("&Show tips on startup"), this);
-    hbox2->addWidget(_tipOnStart, 1);
+    mTipOnStart = new QCheckBox(i18n("&Show tips on startup"), this);
+    hbox2->addWidget(mTipOnStart, 1);
 
     KPushButton *prev = new KPushButton( KStdGuiItem::back(
             KStdGuiItem::UseRTL ), this );
@@ -256,23 +279,22 @@ KTipDialog::KTipDialog(KTipDatabase *db, QWidget *parent, const char *name)
     hbox2->addWidget(ok);
 
     KConfigGroup config(kapp->config(), "TipOfDay");
-    _tipOnStart->setChecked(config.readBoolEntry("RunOnStart", true));
+    mTipOnStart->setChecked(config.readBoolEntry("RunOnStart", true));
 
     connect(next, SIGNAL(clicked()), this, SLOT(nextTip()));
     connect(prev, SIGNAL(clicked()), this, SLOT(prevTip()));
     connect(ok, SIGNAL(clicked()), this, SLOT(accept()));
-    connect(_tipOnStart, SIGNAL(toggled(bool)), this, SLOT(showOnStart(bool)));
+    connect(mTipOnStart, SIGNAL(toggled(bool)), this, SLOT(showOnStart(bool)));
 
     ok->setFocus();
 
     nextTip();
 }
 
-
 KTipDialog::~KTipDialog()
 {
-    if( _instance==this )
-        _instance = 0L;
+    if( mInstance==this )
+        mInstance = 0L;
 }
 
 void KTipDialog::showTip(const QString &tipFile, bool force)
@@ -280,57 +302,64 @@ void KTipDialog::showTip(const QString &tipFile, bool force)
     showTip(kapp->mainWidget(), tipFile, force);
 }
 
-void KTipDialog::showTip(QWidget *parent,const QString &tipFile, bool force)
+void KTipDialog::showTip(QWidget *parent, const QString &tipFile, bool force)
 {
-    if (!force)
-    {
-	KConfigGroup config(kapp->config(), "TipOfDay");
-	if (!config.readBoolEntry("RunOnStart", true))
+  showMultiTip( parent, QStringList(tipFile), force );
+}
+
+void KTipDialog::showMultiTip(QWidget *parent, const QStringList &tipFiles, bool force)
+{
+    const bool runOnStart = KConfigGroup(kapp->config(), "TipOfDay")
+                                        .readBoolEntry("RunOnStart", true);
+
+    if (!force && !runOnStart)
 	    return;
-    }
 
-    if (!_instance)
-	_instance = new KTipDialog(new KTipDatabase(tipFile), parent);
+    if (!mInstance)
+	mInstance = new KTipDialog(new KTipDatabase(tipFiles), parent);
+    else
+	// The application might have changed the RunOnStart option in its own
+	// configuration dialog, so we should update the checkbox.
+      mInstance->mTipOnStart->setChecked(runOnStart);
 
-    _instance->nextTip();
-    _instance->show();
-    _instance->raise();
-}
+      mInstance->show();
+      mInstance->raise();
+  }
 
-void KTipDialog::prevTip()
-{
-    _database->prevTip();
-    _tipText->setText(QString::fromLatin1(
-	 "<qt text=\"%1\" bgcolor=\"%2\">%3</qt>")
-	 .arg(_textColor.name())
-	 .arg(_baseColor.name())
-	 .arg(i18n(_database->tip().utf8())));
-}
+  void KTipDialog::prevTip()
+  {
+      mDatabase->prevTip();
+      mTipText->setText(QString::fromLatin1(
+     "<qt text=\"%1\" bgcolor=\"%2\">%3</qt>")
+     .arg(mTextColor.name())
+     .arg(mBaseColor.name())
+     .arg(i18n(mDatabase->tip().utf8())));
+  }
 
-void KTipDialog::nextTip()
-{
-    _database->nextTip();
-    _tipText->setText(QString::fromLatin1("<qt text=\"%1\" bgcolor=\"%2\">%3</qt>")
-			.arg(_textColor.name())
-			.arg(_baseColor.name())
-			.arg(i18n(_database->tip().utf8())));
-}
+  void KTipDialog::nextTip()
+  {
+      mDatabase->nextTip();
+      mTipText->setText(QString::fromLatin1("<qt text=\"%1\" bgcolor=\"%2\">%3</qt>")
+        .arg(mTextColor.name())
+        .arg(mBaseColor.name())
+        .arg(i18n(mDatabase->tip().utf8())));
+  }
 
-void KTipDialog::showOnStart(bool on)
-{
-    setShowOnStart(on);
-}
+  void KTipDialog::showOnStart(bool on)
+  {
+      setShowOnStart(on);
+  }
 
-void KTipDialog::setShowOnStart(bool on)
-{
-    KConfigGroup config(kapp->config(), "TipOfDay");
-    config.writeEntry("RunOnStart", on);
-    config.sync();
-}
+  void KTipDialog::setShowOnStart(bool on)
+  {
+      KConfigGroup config(kapp->config(), "TipOfDay");
+      config.writeEntry("RunOnStart", on);
+      config.sync();
+  }
 
-bool KTipDialog::eventFilter(QObject *o, QEvent *e)
-{
-	if (o == _tipText && e->type()== QEvent::KeyPress &&
+  bool KTipDialog::eventFilter(QObject *o, QEvent *e)
+  {
+    if (o == mTipText && e->type()== QEvent::KeyPress &&
 		(((QKeyEvent *)e)->key() == Key_Return ||
 		((QKeyEvent *)e)->key() == Key_Space ))
 		accept();
