@@ -26,6 +26,7 @@
 #include <qdom.h>
 #include <qtextstream.h>
 #include <qregexp.h>
+#include <qguardedptr.h>
 
 #include <kinstance.h>
 #include <kstandarddirs.h>
@@ -41,7 +42,6 @@ public:
   KXMLGUIClientPrivate()
   {
     m_instance = KGlobal::instance();
-    m_factory = 0L;
     m_parent = 0L;
     m_builder = 0L;
     m_actionCollection = 0;
@@ -55,7 +55,7 @@ public:
   QDomDocument m_doc;
   KActionCollection *m_actionCollection;
   QDomDocument m_buildDocument;
-  KXMLGUIFactory *m_factory;
+  QGuardedPtr<KXMLGUIFactory> m_factory;
   KXMLGUIClient *m_parent;
   //QPtrList<KXMLGUIClient> m_supers;
   QPtrList<KXMLGUIClient> m_children;
@@ -108,7 +108,7 @@ KActionCollection *KXMLGUIClient::actionCollection() const
 {
   if ( !d->m_actionCollection )
     d->m_actionCollection = new KActionCollection( 0, 0,
-      "KXMLGUILClient-KActionCollection" );
+      "KXMLGUIClient-KActionCollection" );
   return d->m_actionCollection;
 }
 
@@ -187,6 +187,12 @@ void KXMLGUIClient::setXMLFile( const QString& _file, bool merge, bool setXMLDoc
       // this might or might not be an error.  for the time being,
       // let's treat this as if it isn't a problem and the user just
       // wants the global standards file
+
+      // however if a non-empty file gets passed and we can't find it we might
+      // inform the developer using some debug output
+      if ( !_file.isEmpty() )
+          kdWarning() << "KXMLGUIClient::setXMLFile: cannot find .rc file " << _file << endl;
+
       setXML( QString::null, true );
       return;
     }
@@ -862,6 +868,7 @@ void KXMLGUIClient::addStateActionEnabled(const QString& state,
   StateChange stateChange = getActionsToChangeForState(state);
 
   stateChange.actionsToEnable.append( action );
+  //kdDebug() << "KXMLGUIClient::addStateActionEnabled( " << state << ", " << action << ")" << endl;
 
   m_actionsStateMap.replace( state, stateChange );
 }
@@ -873,6 +880,7 @@ void KXMLGUIClient::addStateActionDisabled(const QString& state,
   StateChange stateChange = getActionsToChangeForState(state);
 
   stateChange.actionsToDisable.append( action );
+  //kdDebug() << "KXMLGUIClient::addStateActionDisabled( " << state << ", " << action << ")" << endl;
 
   m_actionsStateMap.replace( state, stateChange );
 }
