@@ -214,15 +214,19 @@ void KXMLGUIFactory::addClient( KXMLGUIClient *client )
     kdDebug(129) << "KXMLGUIFactory::addClient( " << client << " )" << endl; // ellis
     static const QString &actionPropElementName = KGlobal::staticQString( "ActionProperties" );
 
+    if ( client->factory() ) {
+        if ( client->factory() == this )
+            return;
+        else
+            client->factory()->removeClient( client ); //just in case someone does stupid things ;-)
+    }
+
     d->pushState();
 
 //    QTime dt; dt.start();
 
     d->guiClient = client;
 
-    if ( client->factory() && client->factory() != this )
-        client->factory()->removeClient( client ); //just in case someone does stupid things ;-)
-
     // add this client to our client list
     if ( d->m_clients.containsRef( client ) == 0 )
         d->m_clients.append( client );
@@ -318,7 +322,8 @@ void KXMLGUIFactory::removeClient( KXMLGUIClient *client )
     {
         const QPtrList<KXMLGUIClient> *children = client->childClients();
         QPtrListIterator<KXMLGUIClient> childIt( *children );
-        for (; childIt.current(); ++childIt )
+        childIt.toLast();
+        for (; childIt.current(); --childIt )
             removeClient( childIt.current() );
     }
 
@@ -518,7 +523,7 @@ void KXMLGUIFactory::configureAction( KAction *action, const QDomAttr &attribute
 
     QVariant::Type propertyType = action->property( attribute.name().latin1() ).type();
 
-    // If the attribute is a depricated "accel", change to "shortcut".
+    // If the attribute is a deprecated "accel", change to "shortcut".
     if ( attrName.lower() == "accel" )
         attrName = attrShortcut;
 
