@@ -4,15 +4,15 @@
 #include <libxml/parser.h>
 
 QString transform(const QString &file, const QString& stylesheet);
-QString transform(xmlParserCtxtPtr ctxt, const QString &tss);
 QString splitOut(const QString &parsed, int index);
-void fillInstance(KInstance &ins);
+void fillInstance(KInstance &ins, const QString &srcdir = QString::null );
 bool saveToCache( const QString &contents, const QString &filename );
 QString lookForCache( const QString &filename );
 QCString fromUnicode( const QString &data );
 void replaceCharsetHeader( QString &output );
 
 extern bool warnings_exist;
+extern QString *SRCDIR;
 
 /**
  * Compares two files and returns true if @param newer exists and is newer than
