@@ -1,7 +1,9 @@
 /*
  * This file is part of the DOM implementation for KDE.
  *
- * (C) 1999 Lars Knoll (knoll@kde.org)
+ * (C) 1999-2003 Lars Knoll (knoll@kde.org)
+ * (C) 2002-2003 Dirk Mueller (mueller@kde.org)
+ * Copyright (C) 2002 Apple Computer, Inc.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -18,14 +20,14 @@
  * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
  *
- * $Id: css_ruleimpl.h,v 1.18.2.2 2003/07/15 10:42:17 mueller Exp $
+ * $Id: css_ruleimpl.h,v 1.25 2003/08/16 06:38:39 pmk Exp $
  */
 #ifndef _CSS_css_ruleimpl_h_
 #define _CSS_css_ruleimpl_h_
 
 #include "dom/dom_string.h"
 #include "dom/css_rule.h"
-#include "css/cssparser.h"
+#include "css/css_base.h"
 #include "misc/loader_client.h"
 #include "misc/shared.h"
 
@@ -44,13 +46,12 @@ class MediaListImpl;
 class CSSRuleImpl : public StyleBaseImpl
 {
 public:
-    CSSRuleImpl(StyleBaseImpl *parent);
+    CSSRuleImpl(StyleBaseImpl *parent)
+        : StyleBaseImpl(parent), m_type(CSSRule::UNKNOWN_RULE) {}
 
-    virtual ~CSSRuleImpl();
+    virtual bool isRule() const { return true; }
+    unsigned short type() const { return m_type; }
 
-    virtual bool isRule() { return true; }
-
-    unsigned short type() const;
     CSSStyleSheetImpl *parentStyleSheet() const;
     CSSRuleImpl *parentRule() const;
 
@@ -66,11 +67,10 @@ protected:
 class CSSCharsetRuleImpl : public CSSRuleImpl
 {
 public:
-    CSSCharsetRuleImpl(StyleBaseImpl *parent);
+    CSSCharsetRuleImpl(StyleBaseImpl *parent)
+        : CSSRuleImpl(parent) { m_type = CSSRule::CHARSET_RULE; }
 
-    virtual ~CSSCharsetRuleImpl();
-
-    virtual bool isCharsetRule() { return true; }
+    virtual bool isCharsetRule() const { return true; }
 
     DOMString encoding() const { return m_encoding; }
     void setEncoding(DOMString _encoding) { m_encoding = _encoding; }
@@ -87,9 +87,9 @@ public:
 
     virtual ~CSSFontFaceRuleImpl();
 
-    CSSStyleDeclarationImpl *style() const;
+    CSSStyleDeclarationImpl *style() const { return m_style; }
 
-    virtual bool isFontFaceRule() { return true; }
+    virtual bool isFontFaceRule() const { return true; }
 
 protected:
     CSSStyleDeclarationImpl *m_style;
@@ -101,17 +101,20 @@ class CSSImportRuleImpl : public khtml::CachedObjectClient, public CSSRuleImpl
 public:
     CSSImportRuleImpl( StyleBaseImpl *parent, const DOM::DOMString &href,
                        const DOM::DOMString &media );
+    CSSImportRuleImpl( StyleBaseImpl *parent, const DOM::DOMString &href,
+                       MediaListImpl *media );
 
     virtual ~CSSImportRuleImpl();
 
-    DOM::DOMString href() const;
-    MediaListImpl *media() const;
-    CSSStyleSheetImpl *styleSheet() const;
+    DOM::DOMString href() const { return m_strHref; }
+    MediaListImpl *media() const { return m_lstMedia; }
+    CSSStyleSheetImpl *styleSheet() const { return m_styleSheet; }
 
-    virtual bool isImportRule() { return true; }
+    virtual bool isImportRule() const { return true; }
 
     // from CachedObjectClient
     virtual void setStyleSheet(const DOM::DOMString &url, const DOM::DOMString &sheet);
+    virtual void error(int err, const QString &text);
 
     bool isLoading();
     virtual void init();
@@ -122,33 +125,53 @@ protected:
     CSSStyleSheetImpl *m_styleSheet;
     khtml::CachedCSSStyleSheet *m_cachedSheet;
     bool m_loading;
+    bool m_done;
 };
 
-
 class MediaList;
-class CSSRuleList;
+
+class CSSRuleListImpl : public khtml::Shared<CSSRuleListImpl>
+{
+public:
+    CSSRuleListImpl() {}
+
+    ~CSSRuleListImpl();
+
+    unsigned long length() const { return m_lstCSSRules.count(); }
+    CSSRuleImpl *item ( unsigned long index ) { return m_lstCSSRules.at( index ); }
+
+
+    /* not part of the DOM */
+    unsigned long insertRule ( CSSRuleImpl *rule, unsigned long index );
+    void deleteRule ( unsigned long index );
+
+    void append( CSSRuleImpl *rule ) { m_lstCSSRules.append( rule ); }
+protected:
+    QPtrList<CSSRuleImpl> m_lstCSSRules;
+};
 
 class CSSMediaRuleImpl : public CSSRuleImpl
 {
 public:
     CSSMediaRuleImpl( StyleBaseImpl *parent );
-    CSSMediaRuleImpl( StyleBaseImpl *parent, const QChar *&curP,
-                      const QChar * endP, const DOM::DOMString &media );
+    CSSMediaRuleImpl( StyleBaseImpl *parent, const DOM::DOMString &media );
+    CSSMediaRuleImpl( StyleBaseImpl *parent, MediaListImpl *mediaList, CSSRuleListImpl *ruleList );
 
     virtual ~CSSMediaRuleImpl();
 
-    MediaListImpl *media() const;
-    CSSRuleListImpl *cssRules();
-    unsigned long insertRule ( const DOM::DOMString &rule, unsigned long index );
-    void deleteRule ( unsigned long index );
+    MediaListImpl *media() const { return m_lstMedia; }
+    CSSRuleListImpl *cssRules() { return m_lstCSSRules; }
 
-    virtual bool isMediaRule() { return true; }
+    unsigned long insertRule ( const DOM::DOMString &rule, unsigned long index );
+    void deleteRule ( unsigned long index ) { m_lstCSSRules->deleteRule( index ); }
+
+    virtual bool isMediaRule() const { return true; }
+
+    /* Not part of the DOM */
+    unsigned long append( CSSRuleImpl *rule );
 protected:
     MediaListImpl *m_lstMedia;
     CSSRuleListImpl *m_lstCSSRules;
-
-    /* Not part of the DOM */
-    unsigned long appendRule( CSSRuleImpl *rule );
 };
 
 
@@ -159,9 +182,9 @@ public:
 
     virtual ~CSSPageRuleImpl();
 
-    CSSStyleDeclarationImpl *style() const;
+    CSSStyleDeclarationImpl *style() const { return m_style; }
 
-    virtual bool isPageRule() { return true; }
+    virtual bool isPageRule() const { return true; }
 
     DOM::DOMString selectorText() const;
     void setSelectorText(DOM::DOMString str);
@@ -178,16 +201,16 @@ public:
 
     virtual ~CSSStyleRuleImpl();
 
-    CSSStyleDeclarationImpl *style() const;
+    CSSStyleDeclarationImpl *style() const { return m_style; }
 
-    virtual bool isStyleRule() { return true; }
+    virtual bool isStyleRule() const { return true; }
 
     DOM::DOMString selectorText() const;
     void setSelectorText(DOM::DOMString str);
 
     virtual bool parseString( const DOMString &string, bool = false );
 
-    void setSelector( QPtrList<CSSSelector> *selector);
+    void setSelector( QPtrList<CSSSelector> *selector) { m_selector = selector; }
     void setDeclaration( CSSStyleDeclarationImpl *style);
 
     QPtrList<CSSSelector> *selector() { return m_selector; }
@@ -201,35 +224,15 @@ protected:
 };
 
 
-
 class CSSUnknownRuleImpl : public CSSRuleImpl
 {
 public:
-    CSSUnknownRuleImpl(StyleBaseImpl *parent);
+    CSSUnknownRuleImpl(StyleBaseImpl *parent) : CSSRuleImpl(parent) {}
 
-    ~CSSUnknownRuleImpl();
-
-    virtual bool isUnknownRule() { return true; }
+    virtual bool isUnknownRule() const { return true; }
 };
 
 
-class CSSRuleListImpl : public khtml::Shared<CSSRuleListImpl>
-{
-public:
-    CSSRuleListImpl();
-    ~CSSRuleListImpl();
-
-    unsigned long length() const;
-    CSSRuleImpl *item ( unsigned long index );
-
-    /* not part of the DOM */
-    unsigned long insertRule ( CSSRuleImpl *rule, unsigned long index );
-    void deleteRule ( unsigned long index );
-
-protected:
-    QPtrList<CSSRuleImpl> m_lstCSSRules;
-};
-
 } // namespace
 
 #endif
