@@ -1,7 +1,7 @@
 /**
  * This file is part of the DOM implementation for KDE.
  *
- * (C) 1999 Lars Knoll (knoll@kde.org)
+ * (C) 1999-2003 Lars Knoll (knoll@kde.org)
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -18,7 +18,7 @@
  * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
  *
- * $Id: css_stylesheetimpl.cpp,v 1.38 2002/04/11 08:53:07 knoll Exp $
+ * $Id: css_stylesheetimpl.cpp,v 1.43 2003/04/19 16:04:33 mueller Exp $
  */
 
 //#define CSS_STYLESHEET_DEBUG
@@ -79,21 +79,6 @@ StyleSheetImpl::~StyleSheetImpl()
     }
 }
 
-bool StyleSheetImpl::disabled() const
-{
-    return m_disabled;
-}
-
-void StyleSheetImpl::setDisabled( bool disabled )
-{
-    m_disabled = disabled;
-}
-
-NodeImpl *StyleSheetImpl::ownerNode() const
-{
-    return m_parentNode;
-}
-
 StyleSheetImpl *StyleSheetImpl::parentStyleSheet() const
 {
     if( !m_parent ) return 0;
@@ -101,21 +86,6 @@ StyleSheetImpl *StyleSheetImpl::parentStyleSheet() const
     return 0;
 }
 
-DOMString StyleSheetImpl::href() const
-{
-    return m_strHref;
-}
-
-DOMString StyleSheetImpl::title() const
-{
-    return m_strTitle;
-}
-
-MediaListImpl *StyleSheetImpl::media() const
-{
-    return m_media;
-}
-
 void StyleSheetImpl::setMedia( MediaListImpl *media )
 {
     if( media )
@@ -169,6 +139,7 @@ CSSStyleSheetImpl::CSSStyleSheetImpl(DOM::NodeImpl *parentNode, CSSStyleSheetImp
 CSSStyleSheetImpl::CSSStyleSheetImpl(CSSRuleImpl *ownerRule, CSSStyleSheetImpl *orig)
     : StyleSheetImpl(ownerRule, orig->m_strHref)
 {
+    // m_lstChildren is deleted in StyleListImpl
     m_lstChildren = new QPtrList<StyleBaseImpl>;
     StyleBaseImpl *rule;
     for ( rule = orig->m_lstChildren->first(); rule != 0; rule = orig->m_lstChildren->next() )
@@ -180,11 +151,6 @@ CSSStyleSheetImpl::CSSStyleSheetImpl(CSSRuleImpl *ownerRule, CSSStyleSheetImpl *
     m_implicit = false;
 }
 
-CSSStyleSheetImpl::~CSSStyleSheetImpl()
-{
-    // m_lstChildren is deleted in StyleListImpl
-}
-
 CSSRuleImpl *CSSStyleSheetImpl::ownerRule() const
 {
     if( !m_parent ) return 0;
@@ -192,11 +158,6 @@ CSSRuleImpl *CSSStyleSheetImpl::ownerRule() const
     return 0;
 }
 
-CSSRuleList CSSStyleSheetImpl::cssRules()
-{
-    return this;
-}
-
 unsigned long CSSStyleSheetImpl::insertRule( const DOMString &rule, unsigned long index, int &exceptioncode )
 {
     exceptioncode = 0;
@@ -204,10 +165,8 @@ unsigned long CSSStyleSheetImpl::insertRule( const DOMString &rule, unsigned lon
         exceptioncode = DOMException::INDEX_SIZE_ERR;
         return 0;
     }
-    const QString preprocessed = preprocess(rule.string());
-    const QChar *curP = preprocessed.unicode();
-    const QChar *endP = preprocessed.unicode() + preprocessed.length();
-    CSSRuleImpl *r = parseRule(curP, endP);
+    CSSParser p( strictParsing );
+    CSSRuleImpl *r = p.parseRule( this, rule );
 
     if(!r) {
         exceptioncode = CSSException::SYNTAX_ERR + CSSException::_EXCEPTION_OFFSET;
@@ -220,6 +179,11 @@ unsigned long CSSStyleSheetImpl::insertRule( const DOMString &rule, unsigned lon
     return index;
 }
 
+CSSRuleList CSSStyleSheetImpl::cssRules()
+{
+    return this;
+}
+
 void CSSStyleSheetImpl::deleteRule( unsigned long index, int &exceptioncode )
 {
     exceptioncode = 0;
@@ -233,34 +197,17 @@ void CSSStyleSheetImpl::deleteRule( unsigned long index, int &exceptioncode )
 
 bool CSSStyleSheetImpl::parseString(const DOMString &string, bool strict)
 {
-    strictParsing = strict;
-    const QString preprocessed = preprocess(string.string());
-
 #ifdef CSS_STYLESHEET_DEBUG
     kdDebug( 6080 ) << "parsing sheet, len=" << string.length() << ", sheet is " << string.string() << endl;
 #endif
 
-    const QChar *curP = preprocessed.unicode();
-    const QChar *endP = preprocessed.unicode() + preprocessed.length();
-
-#ifdef CSS_STYLESHEET_DEBUG
-    kdDebug( 6080 ) << "preprocessed sheet, len=" << preprocessed.length() << ", sheet is " << preprocessed << endl;
-#endif
-
-    while (curP && (curP < endP))
-    {
-        CSSRuleImpl *rule = parseRule(curP, endP);
-        if(rule)
-        {
-           m_lstChildren->append(rule);
-           rule->setParent(this);
-//           rule->init();
-        }
-    }
+    strictParsing = strict;
+    CSSParser p( strict );
+    p.parseSheet( this, string );
     return true;
 }
 
-bool CSSStyleSheetImpl::isLoading()
+bool CSSStyleSheetImpl::isLoading() const
 {
     StyleBaseImpl *rule;
     for ( rule = m_lstChildren->first(); rule != 0; rule = m_lstChildren->next() )
@@ -283,7 +230,7 @@ bool CSSStyleSheetImpl::isLoading()
     return false;
 }
 
-void CSSStyleSheetImpl::checkLoaded()
+void CSSStyleSheetImpl::checkLoaded() const
 {
     if(isLoading()) return;
     if(m_parent) m_parent->checkLoaded();
@@ -301,20 +248,9 @@ void CSSStyleSheetImpl::setNonCSSHints()
     }
 }
 
-khtml::DocLoader *CSSStyleSheetImpl::docLoader()
-{
-    if ( !m_doc ) // doc is 0 for the user- and default-sheet!
-        return 0;
 
-    // ### remove? (clients just use sheet->doc()->docLoader())
-    return m_doc->docLoader();
-}
+// ---------------------------------------------------------------------------
 
-// ---------------------------------------------------------------------------------------------
-
-StyleSheetListImpl::StyleSheetListImpl()
-{
-}
 
 StyleSheetListImpl::~StyleSheetListImpl()
 {
@@ -364,11 +300,6 @@ StyleSheetImpl *StyleSheetListImpl::item ( unsigned long index )
 
 // --------------------------------------------------------------------------------------------
 
-MediaListImpl::MediaListImpl(CSSStyleSheetImpl *parentSheet)
-    : StyleBaseImpl(parentSheet)
-{
-}
-
 MediaListImpl::MediaListImpl( CSSStyleSheetImpl *parentSheet,
                               const DOMString &media )
     : StyleBaseImpl( parentSheet )
@@ -376,21 +307,12 @@ MediaListImpl::MediaListImpl( CSSStyleSheetImpl *parentSheet,
     setMediaText( media );
 }
 
-MediaListImpl::MediaListImpl(CSSRuleImpl *parentRule)
-    : StyleBaseImpl(parentRule)
-{
-}
-
 MediaListImpl::MediaListImpl( CSSRuleImpl *parentRule, const DOMString &media )
     : StyleBaseImpl(parentRule)
 {
     setMediaText( media );
 }
 
-MediaListImpl::~MediaListImpl()
-{
-}
-
 bool MediaListImpl::contains( const DOMString &medium ) const
 {
     return m_lstMedia.count() == 0 || m_lstMedia.contains( medium ) ||
@@ -409,16 +331,6 @@ CSSRuleImpl *MediaListImpl::parentRule() const
     return 0;
 }
 
-unsigned long MediaListImpl::length() const
-{
-    return m_lstMedia.count();
-}
-
-DOMString MediaListImpl::item( unsigned long index ) const
-{
-    return m_lstMedia[index];
-}
-
 void MediaListImpl::deleteMedium( const DOMString &oldMedium )
 {
     for ( QValueList<DOMString>::Iterator it = m_lstMedia.begin(); it != m_lstMedia.end(); ++it ) {
@@ -429,11 +341,6 @@ void MediaListImpl::deleteMedium( const DOMString &oldMedium )
     }
 }
 
-void MediaListImpl::appendMedium( const DOMString &newMedium )
-{
-    m_lstMedia.append( newMedium );
-}
-
 DOM::DOMString MediaListImpl::mediaText() const
 {
     DOMString text;
@@ -452,7 +359,7 @@ void MediaListImpl::setMediaText(const DOM::DOMString &value)
     for ( QStringList::Iterator it = list.begin(); it != list.end(); ++it )
     {
         DOMString medium = (*it).stripWhiteSpace();
-        if( medium != "" )
+        if( !medium.isEmpty() )
             m_lstMedia.append( medium );
     }
 }
