@@ -1,7 +1,8 @@
 /**
  * This file is part of the DOM implementation for KDE.
  *
- * (C) 1999 Lars Knoll (knoll@kde.org)
+ * (C) 1999-2003 Lars Knoll (knoll@kde.org)
+ * Copyright (C) 2002 Apple Computer, Inc.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -35,6 +36,7 @@
 
 #include "misc/loader.h"
 
+#include "rendering/font.h"
 #include "rendering/render_style.h"
 
 #include <kdebug.h>
@@ -45,6 +47,8 @@
 // Hack for debugging purposes
 extern DOM::DOMString getPropertyName(unsigned short id);
 
+using khtml::FontDef;
+
 using namespace DOM;
 
 CSSStyleDeclarationImpl::CSSStyleDeclarationImpl(CSSRuleImpl *parent)
@@ -87,7 +91,6 @@ CSSStyleDeclarationImpl::~CSSStyleDeclarationImpl()
 DOMString CSSStyleDeclarationImpl::getPropertyValue( int propertyID ) const
 {
     if(!m_lstValues) return DOMString();
-
     CSSValueImpl* value = getPropertyCSSValue( propertyID );
     if ( value )
         return value->cssText();
@@ -214,7 +217,7 @@ DOMString CSSStyleDeclarationImpl::getShortHandValue( const int* properties, int
     return res;
 }
 
-CSSValueImpl *CSSStyleDeclarationImpl::getPropertyCSSValue( int propertyID ) const
+ CSSValueImpl *CSSStyleDeclarationImpl::getPropertyCSSValue( int propertyID ) const
 {
     if(!m_lstValues) return 0;
 
@@ -233,13 +236,14 @@ DOMString CSSStyleDeclarationImpl::removeProperty( int propertyID, bool NonCSSHi
 
     QPtrListIterator<CSSProperty> lstValuesIt(*m_lstValues);
      CSSProperty *current;
-     for ( lstValuesIt.toLast(); (current = lstValuesIt.current()); --lstValuesIt )
+     for ( lstValuesIt.toLast(); (current = lstValuesIt.current()); --lstValuesIt )  {
          if (current->m_id == propertyID && NonCSSHint == current->nonCSSHint) {
              value = current->value()->cssText();
              m_lstValues->removeRef(current);
-            setChanged();
+             setChanged();
 	     break;
         }
+     }
 
     return value;
 }
@@ -259,6 +263,17 @@ void CSSStyleDeclarationImpl::setChanged()
         }
 }
 
+void CSSStyleDeclarationImpl::removeCSSHints()
+{
+    if (!m_lstValues)
+	return;
+
+    for (int i = (int)m_lstValues->count()-1; i >= 0; i--) {
+	if (!m_lstValues->at(i)->nonCSSHint)
+	    m_lstValues->remove(i);
+    }
+}
+
 bool CSSStyleDeclarationImpl::getPropertyPriority( int propertyID ) const
 {
     if ( m_lstValues) {
@@ -272,22 +287,21 @@ bool CSSStyleDeclarationImpl::getPropertyPriority( int propertyID ) const
     return false;
 }
 
-void CSSStyleDeclarationImpl::setProperty(int id, const DOMString &value, bool important, bool nonCSSHint)
+bool CSSStyleDeclarationImpl::setProperty(int id, const DOMString &value, bool important, bool nonCSSHint)
 {
     if(!m_lstValues) {
 	m_lstValues = new QPtrList<CSSProperty>;
 	m_lstValues->setAutoDelete(true);
     }
-    removeProperty(id, nonCSSHint );
-
-    DOMString ppValue = preprocess(value.string(),true);
-    bool success = parseValue(ppValue.unicode(), ppValue.unicode()+ppValue.length(), id, important, nonCSSHint, m_lstValues);
 
+    CSSParser parser( strictParsing );
+    bool success = parser.parseValue( this, id, value, important, nonCSSHint );
     if(!success)
 	kdDebug( 6080 ) << "CSSStyleDeclarationImpl::setProperty invalid property: [" << getPropertyName(id).string()
-					<< "] value: [" << value.string() << "]"<< endl;
+			<< "] value: [" << value.string() << "]"<< endl;
     else
         setChanged();
+    return success;
 }
 
 void CSSStyleDeclarationImpl::setProperty(int id, int value, bool important, bool nonCSSHint)
@@ -303,32 +317,6 @@ void CSSStyleDeclarationImpl::setProperty(int id, int value, bool important, boo
     setChanged();
 }
 
-void CSSStyleDeclarationImpl::setProperty ( const DOMString &propertyString)
-{
-    DOMString ppPropertyString = preprocess(propertyString.string(),true);
-    QPtrList<CSSProperty> *props = parseProperties(ppPropertyString.unicode(),
-						ppPropertyString.unicode()+ppPropertyString.length());
-    if(!props || !props->count())
-	return;
-
-    props->setAutoDelete(false);
-
-    if(!m_lstValues) {
-	m_lstValues = new QPtrList<CSSProperty>;
-	m_lstValues->setAutoDelete( true );
-    }
-
-    CSSProperty *prop = props->first();
-    while( prop ) {
-	removeProperty(prop->m_id, false);
-	m_lstValues->append(prop);
- 	prop = props->next();
-    }
-
-    delete props;
-    setChanged();
-}
-
 void CSSStyleDeclarationImpl::setLengthProperty(int id, const DOM::DOMString &value, bool important, bool nonCSSHint, bool _multiLength )
 {
     bool parseMode = strictParsing;
@@ -339,15 +327,27 @@ void CSSStyleDeclarationImpl::setLengthProperty(int id, const DOM::DOMString &va
     multiLength = false;
 }
 
+void CSSStyleDeclarationImpl::setProperty ( const DOMString &propertyString)
+{
+    if(!m_lstValues) {
+	m_lstValues = new QPtrList<CSSProperty>;
+	m_lstValues->setAutoDelete( true );
+    }
+
+    CSSParser parser( strictParsing );
+    parser.parseDeclaration( this, propertyString, false );
+    setChanged();
+}
+
 unsigned long CSSStyleDeclarationImpl::length() const
 {
     return m_lstValues ? m_lstValues->count() : 0;
 }
 
-DOMString CSSStyleDeclarationImpl::item( unsigned long /*index*/ )
+DOMString CSSStyleDeclarationImpl::item( unsigned long index ) const
 {
-    // ###
-    //return m_lstValues->at(index);
+    if(m_lstValues && index < m_lstValues->count() && m_lstValues->at(index))
+	return getPropertyName(m_lstValues->at(index)->m_id);
     return DOMString();
 }
 
@@ -359,17 +359,36 @@ CSSRuleImpl *CSSStyleDeclarationImpl::parentRule() const
 
 DOM::DOMString CSSStyleDeclarationImpl::cssText() const
 {
-    return DOM::DOMString();
-    // ###
+    DOMString result;
+
+    if ( m_lstValues) {
+	QPtrListIterator<CSSProperty> lstValuesIt(*m_lstValues);
+	CSSProperty *current;
+	for ( lstValuesIt.toFirst(); (current = lstValuesIt.current()); ++lstValuesIt ) {
+	    result += current->cssText();
+	}
+    }
+
+    return result;
 }
 
-void CSSStyleDeclarationImpl::setCssText(DOM::DOMString /*str*/)
+void CSSStyleDeclarationImpl::setCssText(DOM::DOMString text)
 {
-    // ###
+    if (m_lstValues) {
+	m_lstValues->clear();
+    } else {
+	m_lstValues = new QPtrList<CSSProperty>;
+	m_lstValues->setAutoDelete( true );
+    }
+
+    CSSParser parser( strictParsing );
+    parser.parseDeclaration( this, text, false );
+    setChanged();
 }
 
 bool CSSStyleDeclarationImpl::parseString( const DOMString &/*string*/, bool )
 {
+    kdDebug() << "WARNING: CSSStyleDeclarationImpl::parseString, unimplemented, was called" << endl;
     return false;
     // ###
 }
@@ -377,36 +396,28 @@ bool CSSStyleDeclarationImpl::parseString( const DOMString &/*string*/, bool )
 
 // --------------------------------------------------------------------------------------
 
-CSSValueImpl::CSSValueImpl()
-    : StyleBaseImpl()
+unsigned short CSSInheritedValueImpl::cssValueType() const
 {
-}
-
-CSSValueImpl::~CSSValueImpl()
-{
-}
-
-DOM::DOMString CSSValueImpl::cssText() const
-{
-    return DOM::DOMString();
-}
-
-void CSSValueImpl::setCssText(DOM::DOMString /*str*/)
-{
-    // ###
+    return CSSValue::CSS_INHERIT;
 }
 
 DOM::DOMString CSSInheritedValueImpl::cssText() const
 {
     return DOMString("inherited");
 }
-// ----------------------------------------------------------------------------------------
 
-CSSValueListImpl::CSSValueListImpl()
-    : CSSValueImpl()
+unsigned short CSSInitialValueImpl::cssValueType() const
 {
+    return CSSValue::CSS_INITIAL;
 }
 
+DOM::DOMString CSSInitialValueImpl::cssText() const
+{
+    return DOMString("initial");
+}
+
+// ----------------------------------------------------------------------------------------
+
 CSSValueListImpl::~CSSValueListImpl()
 {
     CSSValueImpl *val = m_values.first();
@@ -429,8 +440,13 @@ void CSSValueListImpl::append(CSSValueImpl *val)
 
 DOM::DOMString CSSValueListImpl::cssText() const
 {
-    // ###
-    return DOM::DOMString();
+    DOMString result = "";
+
+    for (QPtrListIterator<CSSValueImpl> iterator(m_values); iterator.current(); ++iterator) {
+	result += iterator.current()->cssText();
+    }
+
+    return result;
 }
 
 // -------------------------------------------------------------------------------------
@@ -448,7 +464,7 @@ CSSPrimitiveValueImpl::CSSPrimitiveValueImpl(int ident)
     m_type = CSSPrimitiveValue::CSS_IDENT;
 }
 
-CSSPrimitiveValueImpl::CSSPrimitiveValueImpl(float num, CSSPrimitiveValue::UnitTypes type)
+CSSPrimitiveValueImpl::CSSPrimitiveValueImpl(double num, CSSPrimitiveValue::UnitTypes type)
 {
     m_value.num = num;
     m_type = type;
@@ -510,30 +526,44 @@ void CSSPrimitiveValueImpl::cleanup()
 
 int CSSPrimitiveValueImpl::computeLength( khtml::RenderStyle *style, QPaintDeviceMetrics *devMetrics )
 {
-    return ( int ) computeLengthFloat( style, devMetrics );
+    double result = computeLengthFloat( style, devMetrics );
+    int intResult = (int)result;
+#ifdef APPLE_CHANGES
+    // This conversion is imprecise, often resulting in values of e.g., 44.99998.  We
+    // need to go ahead and round if we're really close to the next integer value.
+    double newResult = (intResult < 0) ? result-0.01 : result+0.01;
+    int secondIntResult = (int)newResult;
+    if (secondIntResult != intResult)
+        return secondIntResult;
+#endif
+    return intResult;
 }
 
-float CSSPrimitiveValueImpl::computeLengthFloat( khtml::RenderStyle *style, QPaintDeviceMetrics *devMetrics )
+double CSSPrimitiveValueImpl::computeLengthFloat( khtml::RenderStyle *style, QPaintDeviceMetrics *devMetrics )
 {
     unsigned short type = primitiveType();
 
-    float dpiY = 72.; // fallback
+    double dpiY = 72.; // fallback
     if ( devMetrics )
         dpiY = devMetrics->logicalDpiY();
     if ( !khtml::printpainter && dpiY < 96 )
         dpiY = 96.;
 
-    float factor = 1.;
+    double factor = 1.;
     switch(type)
     {
     case CSSPrimitiveValue::CSS_EMS:
-       	factor = style->font().pixelSize();
-		break;
+        factor = style->font().pixelSize();
+        break;
     case CSSPrimitiveValue::CSS_EXS:
 	{
         QFontMetrics fm = style->fontMetrics();
+#ifdef APPLE_CHANGES
+        factor = fm.xHeight();
+#else
         QRect b = fm.boundingRect('x');
         factor = b.height();
+#endif
         break;
 	}
     case CSSPrimitiveValue::CSS_PX:
@@ -557,10 +587,11 @@ float CSSPrimitiveValueImpl::computeLengthFloat( khtml::RenderStyle *style, QPai
     default:
         return -1;
     }
-    return getFloatValue(type)*factor;
+
+    return floatValue(type)*factor;
 }
 
-void CSSPrimitiveValueImpl::setFloatValue( unsigned short unitType, float floatValue, int &exceptioncode )
+void CSSPrimitiveValueImpl::setFloatValue( unsigned short unitType, double floatValue, int &exceptioncode )
 {
     exceptioncode = 0;
     cleanup();
@@ -580,7 +611,7 @@ void CSSPrimitiveValueImpl::setStringValue( unsigned short stringType, const DOM
     cleanup();
     //if(m_type < CSSPrimitiveValue::CSS_STRING) throw DOMException(DOMException::INVALID_ACCESS_ERR);
     //if(m_type > CSSPrimitiveValue::CSS_ATTR) throw DOMException(DOMException::INVALID_ACCESS_ERR);
-    if(m_type < CSSPrimitiveValue::CSS_STRING || m_type >> CSSPrimitiveValue::CSS_ATTR) {
+    if(m_type < CSSPrimitiveValue::CSS_STRING || m_type > CSSPrimitiveValue::CSS_ATTR) {
 	exceptioncode = CSSException::SYNTAX_ERR + CSSException::_EXCEPTION_OFFSET;
 	return;
     }
@@ -601,6 +632,7 @@ unsigned short CSSPrimitiveValueImpl::cssValueType() const
 bool CSSPrimitiveValueImpl::parseString( const DOMString &/*string*/, bool )
 {
     // ###
+    kdDebug() << "WARNING: CSSPrimitiveValueImpl::parseString, unimplemented, was called" << endl;
     return false;
 }
 
@@ -677,7 +709,9 @@ DOM::DOMString CSSPrimitiveValueImpl::cssText() const
 	    // ###
 	    break;
 	case CSSPrimitiveValue::CSS_URI:
-	    text = DOMString( m_value.string );
+            text  = "url(";
+	    text += DOMString( m_value.string );
+            text += ")";
 	    break;
 	case CSSPrimitiveValue::CSS_IDENT:
 	    text = getValueName(m_value.ident);
@@ -689,8 +723,15 @@ DOM::DOMString CSSPrimitiveValueImpl::cssText() const
 	    // ###
 	    break;
 	case CSSPrimitiveValue::CSS_RECT:
-	    // ###
-	    break;
+        {
+            RectImpl* rectVal = getRectValue();
+            text = "rect(";
+            text += rectVal->top()->cssText() + " ";
+            text += rectVal->right()->cssText() + " ";
+            text += rectVal->bottom()->cssText() + " ";
+            text += rectVal->left()->cssText() + ")";
+        }
+        break;
 	case CSSPrimitiveValue::CSS_RGBCOLOR:
 	    text = QColor(m_value.rgbcolor).name();
 	    break;
@@ -748,21 +789,17 @@ void RectImpl::setLeft( CSSPrimitiveValueImpl *left )
 
 // -----------------------------------------------------------------
 
-CSSImageValueImpl::CSSImageValueImpl(const DOMString &url, StyleBaseImpl *style)
+CSSImageValueImpl::CSSImageValueImpl(const DOMString &url, const StyleBaseImpl* style)
     : CSSPrimitiveValueImpl(url, CSSPrimitiveValue::CSS_URI)
 {
     khtml::DocLoader *docLoader = 0;
-    StyleBaseImpl *root = style;
+    const StyleBaseImpl *root = style;
     while (root->parent())
 	root = root->parent();
     if (root->isCSSStyleSheet())
-	docLoader = static_cast<CSSStyleSheetImpl*>(root)->docLoader();
-
-    if (docLoader)
-	m_image = docLoader->requestImage(url);
-    else
-	m_image = khtml::Cache::requestImage(0, url);
+	docLoader = static_cast<const CSSStyleSheetImpl*>(root)->docLoader();
 
+    m_image = docLoader->requestImage(url);
     if(m_image) m_image->ref(this);
 }
 
@@ -780,45 +817,107 @@ CSSImageValueImpl::~CSSImageValueImpl()
 // ------------------------------------------------------------------------
 
 FontFamilyValueImpl::FontFamilyValueImpl( const QString &string)
-    : CSSPrimitiveValueImpl( DOMString(string), CSSPrimitiveValue::CSS_STRING)
+: CSSPrimitiveValueImpl( DOMString(string), CSSPrimitiveValue::CSS_STRING)
 {
+    static const QRegExp parenReg(" \\(.*\\)$");
+    static const QRegExp braceReg(" \\[.*\\]$");
+
+    parsedFontName = string;
+    // a language tag is often added in braces at the end. Remove it.
+    parsedFontName.replace(parenReg, QString::null);
+    // remove [Xft] qualifiers
+    parsedFontName.replace(braceReg, QString::null);
+
+#ifndef APPLE_CHANGES
     const QString &available = KHTMLSettings::availableFamilies();
 
-    QString face = string.lower();
-    // a languge tag is often added in braces at the end. Remove it.
-    face = face.replace(QRegExp(" \\(.*\\)$"), "");
-    // remove [Xft] qualifiers
-    face = face.replace(QRegExp(" \\[.*\\]$"), "");
-    //kdDebug(0) << "searching for face '" << face << "'" << endl;
-    if(face == "serif" ||
-       face == "sans-serif" ||
-       face == "cursive" ||
-       face == "fantasy" ||
-       face == "monospace" ||
-       face == "konq_default") {
-	parsedFontName = face;
-    } else {
-	int pos = available.find( face, 0, false );
-	if( pos == -1 ) {
-	    QString str = face;
-	    int p = face.find(' ');
-	    // Arial Blk --> Arial
-	    // MS Sans Serif --> Sans Serif
-	    if ( p != -1 ) {
-		if(p > 0 && (int)str.length() - p > p + 1)
-		    str = str.mid( p+1 );
-		else
-		    str.truncate( p );
-		pos = available.find( str, 0, false);
-	    }
-	}
+    parsedFontName = parsedFontName.lower();
+    // kdDebug(0) << "searching for face '" << parsedFontName << "'" << endl;
 
-	if ( pos != -1 ) {
-	    int pos1 = available.findRev( ',', pos ) + 1;
-	    pos = available.find( ',', pos );
-	    if ( pos == -1 )
-		pos = available.length();
-	    parsedFontName = available.mid( pos1, pos - pos1 );
-	}
+    int pos = available.find( ',' + parsedFontName + ',', 0, false );
+    if ( pos == -1 ) {
+        // many pages add extra MSs to make sure it's windows only ;(
+        if ( parsedFontName.startsWith( "ms " ) )
+            parsedFontName = parsedFontName.mid( 3 );
+        if ( parsedFontName.endsWith( " ms" ) )
+            parsedFontName.truncate( parsedFontName.length() - 3 );
+        pos = available.find( ",ms " + parsedFontName + ',', 0, false );
+        if ( pos == -1 )
+            pos = available.find( ',' + parsedFontName + " ms,", 0, false );
     }
+
+    if ( pos != -1 ) {
+       ++pos;
+       int p = available.find(',', pos);
+       assert( p != -1 ); // available is supposed to start and end with ,
+       parsedFontName = available.mid( pos, p - pos);
+       // kdDebug(0) << "going for '" << parsedFontName << "'" << endl;
+    } else
+        parsedFontName = QString::null;
+
+#endif // !APPLE_CHANGES
+}
+
+FontValueImpl::FontValueImpl()
+    : style(0), variant(0), weight(0), size(0), lineHeight(0), family(0)
+{
+}
+
+FontValueImpl::~FontValueImpl()
+{
+    delete style;
+    delete variant;
+    delete weight;
+    delete size;
+    delete lineHeight;
+    delete family;
+}
+
+DOMString FontValueImpl::cssText() const
+{
+    // font variant weight size / line-height family
+
+    DOMString result("");
+
+    if (style) {
+	result += style->cssText();
+    }
+    if (variant) {
+	if (result.length() > 0) {
+	    result += " ";
+	}
+	result += variant->cssText();
+    }
+    if (weight) {
+	if (result.length() > 0) {
+	    result += " ";
+	}
+	result += weight->cssText();
+    }
+    if (size) {
+	if (result.length() > 0) {
+	    result += " ";
+	}
+	result += size->cssText();
+    }
+    if (lineHeight) {
+	if (!size) {
+	    result += " ";
+	}
+	result += "/";
+	result += lineHeight->cssText();
+    }
+    if (family) {
+	if (result.length() > 0) {
+	    result += " ";
+	}
+	result += family->cssText();
+    }
+
+    return result;
+}
+
+DOMString CSSProperty::cssText() const
+{
+    return getPropertyName(m_id) + DOMString(": ") + m_value->cssText() + (m_bImportant ? DOMString(" !important") : DOMString()) + DOMString("; ");
 }
