@@ -1,7 +1,8 @@
 /*
  * This file is part of the DOM implementation for KDE.
  *
- * (C) 1999 Lars Knoll (knoll@kde.org)
+ * (C) 1999-2003 Lars Knoll (knoll@kde.org)
+ * Copyright (C) 2002 Apple Computer, Inc.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -18,14 +19,14 @@
  * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
  *
- * $Id: css_valueimpl.h,v 1.41.2.2 2003/08/17 14:38:51 mueller Exp $
+ * $Id: css_valueimpl.h,v 1.53.2.1 2004/02/29 15:27:45 mueller Exp $
  */
 #ifndef _CSS_css_valueimpl_h_
 #define _CSS_css_valueimpl_h_
 
 #include "dom/css_value.h"
 #include "dom/dom_string.h"
-#include "css/cssparser.h"
+#include "css/css_base.h"
 #include "misc/loader_client.h"
 #include "misc/shared.h"
 
@@ -55,33 +56,34 @@ public:
 
     unsigned long length() const;
     CSSRuleImpl *parentRule() const;
-    DOM::DOMString removeProperty( int propertyID, bool NonCSSHints = false );
-    void setProperty ( int propertyId, const DOM::DOMString &value, bool important = false, bool nonCSSHint = false);
-    void setProperty ( int propertyId, int value, bool important = false, bool nonCSSHint = false);
+    virtual DOM::DOMString removeProperty( int propertyID, bool NonCSSHints = false );
+    virtual bool setProperty ( int propertyId, const DOM::DOMString &value, bool important = false, bool nonCSSHint = false);
+    virtual void setProperty ( int propertyId, int value, bool important = false, bool nonCSSHint = false);
     // this treats integers as pixels!
     // needed for conversion of html attributes
-    void setLengthProperty(int id, const DOM::DOMString &value, bool important, bool nonCSSHint = true, bool multiLength = false);
+    virtual void setLengthProperty(int id, const DOM::DOMString &value, bool important, bool nonCSSHint = true, bool multiLength = false);
 
     // add a whole, unparsed property
-    void setProperty ( const DOMString &propertyString);
-    DOM::DOMString item ( unsigned long index );
+    virtual void setProperty ( const DOMString &propertyString);
+    virtual DOM::DOMString item ( unsigned long index ) const;
 
     DOM::DOMString cssText() const;
     void setCssText(DOM::DOMString str);
 
-    virtual bool isStyleDeclaration() { return true; }
-
+    virtual bool isStyleDeclaration() const { return true; }
     virtual bool parseString( const DOMString &string, bool = false );
 
-    CSSValueImpl *getPropertyCSSValue( int propertyID ) const;
-    DOMString getPropertyValue( int propertyID ) const;
-    bool getPropertyPriority( int propertyID ) const;
+    virtual CSSValueImpl *getPropertyCSSValue( int propertyID ) const;
+    virtual DOMString getPropertyValue( int propertyID ) const;
+    virtual bool getPropertyPriority( int propertyID ) const;
 
-    QPtrList<CSSProperty> *values() { return m_lstValues; }
+    QPtrList<CSSProperty> *values() const { return m_lstValues; }
     void setNode(NodeImpl *_node) { m_node = _node; }
 
     void setChanged();
 
+    void removeCSSHints();
+
 protected:
     DOMString getShortHandValue( const int* properties, int number ) const;
     DOMString get4Values( const int* properties ) const;
@@ -97,16 +99,14 @@ private:
 class CSSValueImpl : public StyleBaseImpl
 {
 public:
-    CSSValueImpl();
-
-    virtual ~CSSValueImpl();
+    CSSValueImpl() : StyleBaseImpl() {}
 
     virtual unsigned short cssValueType() const = 0;
 
-    virtual DOM::DOMString cssText() const;
-    void setCssText(DOM::DOMString str);
+    virtual DOM::DOMString cssText() const = 0;
 
-    virtual bool isValue() { return true; }
+    virtual bool isValue() const { return true; }
+    virtual bool isFontValue() const { return false; }
 };
 
 class CSSInheritedValueImpl : public CSSValueImpl
@@ -115,22 +115,28 @@ public:
     CSSInheritedValueImpl() : CSSValueImpl() {}
     virtual ~CSSInheritedValueImpl() {}
 
-    virtual unsigned short cssValueType() const { return CSSValue::CSS_INHERIT; }
+    virtual unsigned short cssValueType() const;
     virtual DOM::DOMString cssText() const;
 };
 
+class CSSInitialValueImpl : public CSSValueImpl
+{
+public:
+    virtual unsigned short cssValueType() const;
+    virtual DOM::DOMString cssText() const;
+};
 
 class CSSValueListImpl : public CSSValueImpl
 {
 public:
-    CSSValueListImpl();
+    CSSValueListImpl() : CSSValueImpl() {}
 
     virtual ~CSSValueListImpl();
 
     unsigned long length() const { return m_values.count(); }
     CSSValueImpl *item ( unsigned long index ) { return m_values.at(index); }
 
-    virtual bool isValueList() { return true; }
+    virtual bool isValueList() const { return true; }
 
     virtual unsigned short cssValueType() const;
 
@@ -151,7 +157,7 @@ class CSSPrimitiveValueImpl : public CSSValueImpl
 public:
     CSSPrimitiveValueImpl();
     CSSPrimitiveValueImpl(int ident);
-    CSSPrimitiveValueImpl(float num, CSSPrimitiveValue::UnitTypes type);
+    CSSPrimitiveValueImpl(double num, CSSPrimitiveValue::UnitTypes type);
     CSSPrimitiveValueImpl(const DOMString &str, CSSPrimitiveValue::UnitTypes type);
     CSSPrimitiveValueImpl(const Counter &c);
     CSSPrimitiveValueImpl( RectImpl *r);
@@ -161,9 +167,7 @@ public:
 
     void cleanup();
 
-    unsigned short primitiveType() const {
-	    return m_type;
-    }
+    unsigned short primitiveType() const { return m_type; }
 
     /*
      * computes a length in pixels out of the given CSSValue. Need the RenderStyle to get
@@ -177,15 +181,12 @@ public:
      */
     int computeLength( khtml::RenderStyle *style, QPaintDeviceMetrics *devMetrics );
 
-    float computeLengthFloat( khtml::RenderStyle *style, QPaintDeviceMetrics *devMetrics );
-
+    double computeLengthFloat( khtml::RenderStyle *style, QPaintDeviceMetrics *devMetrics );
 
     // use with care!!!
     void setPrimitiveType(unsigned short type) { m_type = type; }
-    void setFloatValue ( unsigned short unitType, float floatValue, int &exceptioncode );
-    float getFloatValue ( unsigned short/* unitType */) const {
-	return m_value.num;
-    }
+    void setFloatValue ( unsigned short unitType, double floatValue, int &exceptioncode );
+    double floatValue ( unsigned short/* unitType */) const { return m_value.num; }
 
     void setStringValue ( unsigned short stringType, const DOM::DOMString &stringValue, int &exceptioncode );
     DOM::DOMStringImpl *getStringValue () const {
@@ -214,11 +215,13 @@ public:
     virtual bool parseString( const DOMString &string, bool = false);
     virtual DOM::DOMString cssText() const;
 
+    virtual bool isQuirkValue() const { return false; }
+
 protected:
     int m_type;
     union {
 	int ident;
-	float num;
+	double num;
 	DOM::DOMStringImpl *string;
 	CounterImpl *counter;
 	RectImpl *rect;
@@ -226,6 +229,21 @@ protected:
     } m_value;
 };
 
+// This value is used to handle quirky margins in reflow roots (body, td, and th) like WinIE.
+// The basic idea is that a stylesheet can use the value __qem (for quirky em) instead of em
+// in a stylesheet.  When the quirky value is used, if you're in quirks mode, the margin will
+// collapse away inside a table cell.
+class CSSQuirkPrimitiveValueImpl : public CSSPrimitiveValueImpl
+{
+public:
+    CSSQuirkPrimitiveValueImpl(double num, CSSPrimitiveValue::UnitTypes type)
+      :CSSPrimitiveValueImpl(num, type) {}
+
+    virtual ~CSSQuirkPrimitiveValueImpl() {}
+
+    virtual bool isQuirkValue() const { return true; }
+};
+
 class CounterImpl : public khtml::Shared<CounterImpl> {
 public:
     CounterImpl() { }
@@ -243,10 +261,10 @@ public:
     RectImpl();
     ~RectImpl();
 
-    CSSPrimitiveValueImpl *top() { return m_top; }
-    CSSPrimitiveValueImpl *right() { return m_right; }
-    CSSPrimitiveValueImpl *bottom() { return m_bottom; }
-    CSSPrimitiveValueImpl *left() { return m_left; }
+    CSSPrimitiveValueImpl *top() const { return m_top; }
+    CSSPrimitiveValueImpl *right() const { return m_right; }
+    CSSPrimitiveValueImpl *bottom() const { return m_bottom; }
+    CSSPrimitiveValueImpl *left() const { return m_left; }
 
     void setTop( CSSPrimitiveValueImpl *top );
     void setRight( CSSPrimitiveValueImpl *right );
@@ -262,7 +280,7 @@ protected:
 class CSSImageValueImpl : public CSSPrimitiveValueImpl, public khtml::CachedObjectClient
 {
 public:
-    CSSImageValueImpl(const DOMString &url, StyleBaseImpl *style);
+    CSSImageValueImpl(const DOMString &url, const StyleBaseImpl *style);
     CSSImageValueImpl();
     virtual ~CSSImageValueImpl();
 
@@ -276,8 +294,31 @@ class FontFamilyValueImpl : public CSSPrimitiveValueImpl
 public:
     FontFamilyValueImpl( const QString &string);
     const QString &fontName() const { return parsedFontName; }
+    int genericFamilyType() const { return _genericFamilyType; }
 protected:
     QString parsedFontName;
+private:
+    int _genericFamilyType;
+};
+
+class FontValueImpl : public CSSValueImpl
+{
+public:
+    FontValueImpl();
+    virtual ~FontValueImpl();
+
+    virtual unsigned short cssValueType() const { return CSSValue::CSS_CUSTOM; }
+
+    virtual DOM::DOMString cssText() const;
+
+    virtual bool isFontValue() const { return true; }
+
+    CSSPrimitiveValueImpl *style;
+    CSSPrimitiveValueImpl *variant;
+    CSSPrimitiveValueImpl *weight;
+    CSSPrimitiveValueImpl *size;
+    CSSPrimitiveValueImpl *lineHeight;
+    CSSValueListImpl *family;
 };
 
 // ------------------------------------------------------------------------------
@@ -313,7 +354,9 @@ public:
 	}
     }
 
-    CSSValueImpl *value() { return m_value; }
+    CSSValueImpl *value() const { return m_value; }
+
+    DOM::DOMString cssText() const;
 
     // make sure the following fits in 4 bytes.
     signed int  m_id 	: 29;
