@@ -18,7 +18,7 @@
  * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
  *
- * $Id: dom2_events.cpp,v 1.7.2.2 2003/01/27 13:27:55 faure Exp $
+ * $Id: dom2_events.cpp,v 1.11.2.2 2004/02/29 15:27:51 mueller Exp $
  */
 
 #include "dom/dom2_views.h"
@@ -169,14 +169,6 @@ bool Event::isNull() const
     return (impl == 0);
 }
 
-DOMString Event::eventModuleName()
-{
-    if (!impl)
-	throw DOMException(DOMException::INVALID_STATE_ERR);
-
-    return impl->eventModuleName();
-}
-
 // -----------------------------------------------------------------------------
 
 #ifndef SAVE_SPACE
@@ -256,6 +248,70 @@ long UIEvent::detail() const
     return static_cast<UIEventImpl*>(impl)->detail();
 }
 
+int UIEvent::keyCode() const
+{
+    if ( !impl ) throw DOMException( DOMException::INVALID_STATE_ERR );
+
+    if( impl->isTextEvent() )
+        return static_cast<TextEventImpl*>( impl )->keyVal();
+
+    return 0;
+}
+
+int UIEvent::pageX() const
+{
+    if (!impl)
+        throw DOMException(DOMException::INVALID_STATE_ERR);
+
+    if (impl->isMouseEvent() )
+        return static_cast<MouseEventImpl*>( impl )->pageX();
+    else
+        return 0;
+}
+
+int UIEvent::pageY() const
+{
+    if (!impl)
+        throw DOMException(DOMException::INVALID_STATE_ERR);
+
+    if ( impl->isMouseEvent() )
+        return  static_cast<MouseEventImpl*>( impl )->pageY();
+    else
+        return 0;
+}
+
+int UIEvent::layerX() const
+{
+    if( !impl )
+        throw DOMException( DOMException::INVALID_STATE_ERR );
+
+    if( impl->isMouseEvent() )
+        return static_cast<MouseEventImpl*>( impl )->layerX();
+    return 0;
+}
+
+int UIEvent::layerY() const
+{
+    if( !impl )
+        throw DOMException( DOMException::INVALID_STATE_ERR );
+
+    if( impl->isMouseEvent() )
+        return static_cast<MouseEventImpl*>( impl )->layerY();
+    return 0;
+}
+
+int UIEvent::which() const
+{
+    if( !impl ) throw DOMException( DOMException::INVALID_STATE_ERR );
+
+    if( impl->isMouseEvent() )
+        return static_cast<MouseEventImpl*>( impl )->button() + 1;
+    else if( impl->isTextEvent() )
+        return static_cast<TextEventImpl*>( impl )->keyVal();
+
+    return 0;
+}
+
 void UIEvent::initUIEvent(const DOMString &typeArg,
                                  bool canBubbleArg,
                                  bool cancelableArg,
@@ -411,7 +467,7 @@ void MouseEvent::initMouseEvent(const DOMString &typeArg,
 
     static_cast<MouseEventImpl*>(impl)->initMouseEvent(typeArg,canBubbleArg,
 	cancelableArg,viewArg,detailArg,screenXArg,screenYArg,clientXArg,
-	clientYArg,ctrlKeyArg,altKeyArg,shiftKeyArg,metaKeyArg,buttonArg,
+        clientYArg,ctrlKeyArg,altKeyArg,shiftKeyArg,metaKeyArg,buttonArg,
 	relatedTargetArg);
 }
 
