@@ -5,6 +5,7 @@
  * (C) 2000 Gunnstein Lye (gunnstein@netcom.no)
  * (C) 2000 Frederik Holljen (frederik.holljen@hig.no)
  * (C) 2001 Peter Kelly (pmk@post.com)
+ * Copyright (C) 2003 Apple Computer, Inc.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -21,7 +22,7 @@
  * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
  *
- * $Id: dom2_range.cpp,v 1.33 2002/04/08 17:53:37 knoll Exp $
+ * $Id: dom2_range.cpp,v 1.35 2003/10/04 18:56:52 mueller Exp $
  */
 #include "dom/dom_exception.h"
 #include "xml/dom_docimpl.h"
@@ -348,6 +349,18 @@ DOMString Range::toHTML(  )
     return impl->toHTML();
 }
 
+DocumentFragment Range::createContextualFragment( const DOMString &html )
+{
+    if (!impl)
+        throw DOMException(DOMException::INVALID_STATE_ERR);
+
+    int exceptioncode = 0;
+    DocumentFragment r = impl->createContextualFragment(html, exceptioncode);
+    throwException(exceptioncode);
+    return r;
+}
+
+
 void Range::detach(  )
 {
     if (!impl)
