@@ -1,4 +1,4 @@
-/**
+/*
  * This file is part of the DOM implementation for KDE.
  *
  * (C) 1999 Lars Knoll (knoll@kde.org)
@@ -18,7 +18,6 @@
  * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
  *
- * $Id: dom_doc.cpp,v 1.42.2.2 2003/06/24 21:34:00 faure Exp $
  */
 
 #include "dom/dom_exception.h"
@@ -35,7 +34,7 @@
 
 #include <kdebug.h>
 
-using namespace DOM;
+namespace DOM {
 
 DOMImplementation::DOMImplementation()
 {
@@ -229,23 +228,25 @@ Element Document::documentElement() const
 Element Document::createElement( const DOMString &tagName )
 {
     if (!impl)
-        return 0;
+        throw DOMException(DOMException::NOT_FOUND_ERR);
+
     int exceptioncode = 0;
-    Element el = ((DocumentImpl *)impl)->createElement(tagName, &exceptioncode);
-    if (exceptioncode)
-        throw DOMException(exceptioncode);
-    return el;
+    ElementImpl* r = ((DocumentImpl *)impl)->createElement(tagName, &exceptioncode);
+    if ( exceptioncode )
+        throw DOMException( exceptioncode );
+    return r;
 }
 
 Element Document::createElementNS( const DOMString &namespaceURI, const DOMString &qualifiedName )
 {
     if (!impl)
-        return 0;
+        throw DOMException(DOMException::NOT_FOUND_ERR);
+
     int exceptioncode = 0;
-    Element el = ((DocumentImpl *)impl)->createElementNS(namespaceURI, qualifiedName, &exceptioncode);
-    if (exceptioncode)
-        throw DOMException(exceptioncode);
-    return el;
+    ElementImpl* r = ((DocumentImpl *)impl)->createElementNS(namespaceURI,qualifiedName, &exceptioncode);
+    if ( exceptioncode )
+        throw DOMException( exceptioncode );
+    return r;
 }
 
 DocumentFragment Document::createDocumentFragment(  )
@@ -281,34 +282,24 @@ ProcessingInstruction Document::createProcessingInstruction( const DOMString &ta
 
 Attr Document::createAttribute( const DOMString &name )
 {
-    return createAttributeNS(DOMString(), name);
+    if (!impl) throw DOMException(DOMException::NOT_FOUND_ERR);
+    if (name.isNull()) throw DOMException(DOMException::NOT_FOUND_ERR);
+    int exceptioncode = 0;
+    AttrImpl* a = impl->getDocument()->createAttribute(name, &exceptioncode);
+    if ( exceptioncode )
+        throw DOMException( exceptioncode );
+    return a;
 }
 
 Attr Document::createAttributeNS( const DOMString &namespaceURI, const DOMString &qualifiedName )
 {
     if (!impl) throw DOMException(DOMException::NOT_FOUND_ERR);
     if (qualifiedName.isNull()) throw DOMException(DOMException::NAMESPACE_ERR);
-
-    DOMString localName(qualifiedName.copy());
-    DOMString prefix;
-    int colonpos;
-    if ((colonpos = qualifiedName.find(':')) >= 0) {
-        prefix = qualifiedName.copy();
-        prefix.truncate(colonpos);
-        localName.remove(0, colonpos+1);
-    }
-
     int exceptioncode = 0;
-    NodeImpl::Id id = static_cast<DocumentImpl*>(impl)->attrId(namespaceURI.implementation(), localName.implementation(), false /* allocate */, &exceptioncode);
-    if (exceptioncode)
-        throw DOMException(exceptioncode);
-
-    Attr r = static_cast<DocumentImpl*>(impl)->createAttribute(id);
-    if (r.handle() && prefix.implementation())
-        r.handle()->setPrefix(prefix.implementation(), exceptioncode);
-    if (exceptioncode)
-        throw DOMException(exceptioncode);
-    return r;
+    AttrImpl* a = impl->getDocument()->createAttributeNS(namespaceURI, qualifiedName, &exceptioncode);
+    if ( exceptioncode )
+        throw DOMException( exceptioncode );
+    return a;
 }
 
 EntityReference Document::createEntityReference( const DOMString &name )
@@ -326,15 +317,18 @@ Element Document::getElementById( const DOMString &elementId ) const
 NodeList Document::getElementsByTagName( const DOMString &tagName )
 {
     if (!impl) return 0;
-    return static_cast<DocumentImpl*>(impl)->
-        getElementsByTagNameNS(0, tagName.implementation());
+    NodeImpl::Id id;
+    if ( tagName == "*" )
+        id = 0;
+    else
+        id = impl->getDocument()->getId(NodeImpl::ElementId, tagName.implementation(), false, true);
+    return new TagNodeListImpl( impl, id );
 }
 
 NodeList Document::getElementsByTagNameNS( const DOMString &namespaceURI, const DOMString &localName )
 {
     if (!impl) return 0;
-    return static_cast<DocumentImpl*>(impl)->
-        getElementsByTagNameNS(namespaceURI.implementation(), localName.implementation());
+    return new TagNodeListImpl( impl, namespaceURI, localName );
 }
 
 Node Document::importNode( const Node & importedNode, bool deep )
@@ -378,8 +372,17 @@ NodeIterator Document::createNodeIterator(Node root, unsigned long whatToShow,
 TreeWalker Document::createTreeWalker(Node root, unsigned long whatToShow, NodeFilter filter,
                                 bool entityReferenceExpansion)
 {
-    if (impl) return ((DocumentImpl *)impl)->createTreeWalker(root,whatToShow,filter,entityReferenceExpansion);
-    return 0;
+    if (!impl)
+	throw DOMException(DOMException::INVALID_STATE_ERR);
+
+     int exceptioncode = 0;
+
+     TreeWalkerImpl *tw = static_cast<DocumentImpl *>(impl)->createTreeWalker(
+         root.handle(), whatToShow, filter.handle(), entityReferenceExpansion, exceptioncode);
+     if (exceptioncode)
+         throw DOMException(exceptioncode);
+
+     return tw;
 }
 
 Event Document::createEvent(const DOMString &eventType)
@@ -399,7 +402,7 @@ AbstractView Document::defaultView() const
     if (!impl)
 	throw DOMException(DOMException::INVALID_STATE_ERR);
 
-    return ((DocumentImpl *)impl)->defaultView();
+    return static_cast<DocumentImpl*>(impl)->defaultView();
 }
 
 StyleSheetList Document::styleSheets() const
@@ -407,7 +410,31 @@ StyleSheetList Document::styleSheets() const
     if (!impl)
 	throw DOMException(DOMException::INVALID_STATE_ERR);
 
-    return ((DocumentImpl *)impl)->styleSheets();
+    return static_cast<DocumentImpl*>(impl)->styleSheets();
+}
+
+DOMString Document::preferredStylesheetSet()
+{
+    if (!impl)
+        throw DOMException(DOMException::INVALID_STATE_ERR);
+
+    return static_cast<DocumentImpl*>(impl)->preferredStylesheetSet();
+}
+
+DOMString Document::selectedStylesheetSet()
+{
+    if (!impl)
+        throw DOMException(DOMException::INVALID_STATE_ERR);
+
+    return static_cast<DocumentImpl*>(impl)->selectedStylesheetSet();
+}
+
+void Document::setSelectedStylesheetSet(const DOMString& s)
+{
+    if (!impl)
+        throw DOMException(DOMException::INVALID_STATE_ERR);
+
+    static_cast<DocumentImpl*>(impl)->setSelectedStylesheetSet(s);
 }
 
 
@@ -418,12 +445,6 @@ KHTMLView *Document::view() const
     return static_cast<DocumentImpl*>(impl)->view();
 }
 
-DOMString Document::completeURL(const DOMString& url)
-{
-    if ( !impl ) return url;
-    return static_cast<DocumentImpl*>( impl )->completeURL( url.string() );
-}
-
 CSSStyleDeclaration Document::getOverrideStyle(const Element &elt, const DOMString &pseudoElt)
 {
     if (!impl)
@@ -436,6 +457,74 @@ CSSStyleDeclaration Document::getOverrideStyle(const Element &elt, const DOMStri
     return r;
 }
 
+bool Document::async() const
+{
+    if (!impl)
+        throw DOMException(DOMException::INVALID_STATE_ERR);
+
+    return static_cast<DocumentImpl*>( impl )->async(  );
+}
+
+void Document::setAsync( bool b )
+{
+    if (!impl)
+        throw DOMException(DOMException::INVALID_STATE_ERR);
+
+    static_cast<DocumentImpl*>( impl )->setAsync( b );
+}
+
+void Document::abort()
+{
+    if (!impl)
+        throw DOMException(DOMException::INVALID_STATE_ERR);
+
+
+    static_cast<DocumentImpl*>( impl )->abort(  );
+}
+
+void Document::load( const DOMString &uri )
+{
+    if (!impl)
+        throw DOMException(DOMException::INVALID_STATE_ERR);
+
+    static_cast<DocumentImpl*>( impl )->load( uri );
+}
+
+void Document::loadXML( const DOMString &source )
+{
+    if (!impl)
+        throw DOMException(DOMException::INVALID_STATE_ERR);
+
+
+    static_cast<DocumentImpl*>( impl )->loadXML( source );
+}
+
+bool Document::designMode() const {
+    if (!impl)
+        throw DOMException(DOMException::INVALID_STATE_ERR);
+
+    return static_cast<DocumentImpl*>( impl )->designMode();
+}
+
+void Document::setDesignMode(bool enable) {
+    if (!impl)
+        throw DOMException(DOMException::INVALID_STATE_ERR);
+
+    static_cast<DocumentImpl*>( impl )->setDesignMode( enable );
+}
+
+DOMString Document::completeURL(const DOMString& url)
+{
+    if ( !impl ) return url;
+    return static_cast<DocumentImpl*>( impl )->completeURL( url.string() );
+}
+
+void Document::updateRendering()
+{
+    if ( !impl ) return;
+    static_cast<DocumentImpl*>( impl )->updateRendering(  );
+}
+
 // ----------------------------------------------------------------------------
 
 DocumentFragment::DocumentFragment() : Node()
@@ -562,3 +651,4 @@ DOMString DocumentType::internalSubset() const
     return static_cast<DocumentTypeImpl*>(impl)->internalSubset();
 }
 
+} // namespace
