@@ -24,9 +24,8 @@
  * Copyright  World Wide Web Consortium , (Massachusetts Institute of
  * Technology , Institut National de Recherche en Informatique et en
  * Automatique , Keio University ). All Rights Reserved.
- *
- * $Id: dom_doc.h,v 1.28.2.1 2003/05/18 12:34:37 mueller Exp $
  */
+
 #ifndef _DOM_Document_h_
 #define _DOM_Document_h_
 
@@ -60,13 +59,15 @@ class DocumentTypeImpl;
 class Event;
 class AbstractView;
 class CSSStyleDeclaration;
+class HTMLElementImpl;
 class HTMLFrameElement;
+class HTMLElementImpl;
 class HTMLIFrameElement;
 class HTMLObjectElement;
 class HTMLDocument;
 
 /**
- * The <code> DOMImplementation </code> interface provides a number of
+ * The \c DOMImplementation interface provides a number of
  * methods for performing operations that are independent of any
  * particular instance of the document object model.
  *
@@ -93,10 +94,10 @@ public:
      * @param version This is the version number of the package name
      * to test. In Level 1, this is the string "1.0". If the version
      * is not specified, supporting any version of the feature will
-     * cause the method to return <code> true </code> .
+     * cause the method to return \c true .
      *
-     * @return <code> true </code> if the feature is implemented in
-     * the specified version, <code> false </code> otherwise.
+     * @return \c true if the feature is implemented in
+     * the specified version, \c false otherwise.
      *
      */
     bool hasFeature ( const DOMString &feature, const DOMString &version );
@@ -189,8 +190,8 @@ public:
      * @param title The advisory title. See also the Style Sheet Interfaces
      * section.
      *
-     * media The comma-separated list of media associated with the new style
-     * sheet. See also the Style Sheet Interfaces section.
+     * @param media The comma-separated list of media associated with the 
+     * new style sheet. See also the Style Sheet Interfaces section.
      *
      * @return A new CSS style sheet.
      *
@@ -228,16 +229,16 @@ protected:
 };
 
 /**
- * The <code> Document </code> interface represents the entire HTML or
+ * The \c Document interface represents the entire HTML or
  * XML document. Conceptually, it is the root of the document tree,
  * and provides the primary access to the document's data.
  *
  *  Since elements, text nodes, comments, processing instructions,
- * etc. cannot exist outside the context of a <code> Document </code>
- * , the <code> Document </code> interface also contains the factory
- * methods needed to create these objects. The <code> Node </code>
- * objects created have a <code> ownerDocument </code> attribute which
- * associates them with the <code> Document </code> within whose
+ * etc. cannot exist outside the context of a \c Document
+ * , the \c Document interface also contains the factory
+ * methods needed to create these objects. The \c Node
+ * objects created have a \c ownerDocument attribute which
+ * associates them with the \c Document within whose
  * context they were created.
  *
  */
@@ -268,18 +269,18 @@ public:
     ~Document();
 
     /**
-     * The Document Type Declaration (see <code> DocumentType </code>
+     * The Document Type Declaration (see \c DocumentType
      * ) associated with this document. For HTML documents as well as
      * XML documents without a document type declaration this returns
-     * <code> null </code> . The DOM Level 1 does not support editing
-     * the Document Type Declaration, therefore <code> docType </code>
+     * \c null . The DOM Level 1 does not support editing
+     * the Document Type Declaration, therefore \c docType
      * cannot be altered in any way.
      *
      */
     DocumentType doctype() const;
 
     /**
-     * The <code> DOMImplementation </code> object that handles this
+     * The \c DOMImplementation object that handles this
      * document. A DOM application may use objects from multiple
      * implementations.
      *
@@ -300,12 +301,12 @@ public:
      * attributes can be specified directly on the returned object.
      *
      * @param tagName The name of the element type to instantiate. For
-     * XML, this is case-sensitive. For HTML, the <code> tagName
-     * </code> parameter may be provided in any case, but it must be
+     * XML, this is case-sensitive. For HTML, the \c tagName
+     * parameter may be provided in any case, but it must be
      * mapped to the canonical uppercase form by the DOM
      * implementation.
      *
-     * @return A new <code> Element </code> object.
+     * @return A new \c Element object.
      *
      * @exception DOMException
      * INVALID_CHARACTER_ERR: Raised if the specified name contains an
@@ -336,42 +337,42 @@ public:
                              const DOMString &qualifiedName );
 
     /**
-     * Creates an empty <code> DocumentFragment </code> object.
+     * Creates an empty \c DocumentFragment object.
      *
-     * @return A new <code> DocumentFragment </code> .
+     * @return A new \c DocumentFragment .
      *
      */
     DocumentFragment createDocumentFragment (  );
 
     /**
-     * Creates a <code> Text </code> node given the specified string.
+     * Creates a \c Text node given the specified string.
      *
      * @param data The data for the node.
      *
-     * @return The new <code> Text </code> object.
+     * @return The new \c Text object.
      *
      */
     Text createTextNode ( const DOMString &data );
 
     /**
-     * Creates a <code> Comment </code> node given the specified
+     * Creates a \c Comment node given the specified
      * string.
      *
      * @param data The data for the node.
      *
-     * @return The new <code> Comment </code> object.
+     * @return The new \c Comment object.
      *
      */
     Comment createComment ( const DOMString &data );
 
     /**
-     * Creates a <code> CDATASection </code> node whose value is the
+     * Creates a \c CDATASection node whose value is the
      * specified string.
      *
-     * @param data The data for the <code> CDATASection </code>
+     * @param data The data for the \c CDATASection
      * contents.
      *
-     * @return The new <code> CDATASection </code> object.
+     * @return The new \c CDATASection object.
      *
      * @exception DOMException
      * NOT_SUPPORTED_ERR: Raised if this document is an HTML document.
@@ -380,14 +381,14 @@ public:
     CDATASection createCDATASection ( const DOMString &data );
 
     /**
-     * Creates a <code> ProcessingInstruction </code> node given the
+     * Creates a \c ProcessingInstruction node given the
      * specified name and data strings.
      *
      * @param target The target part of the processing instruction.
      *
      * @param data The data for the node.
      *
-     * @return The new <code> ProcessingInstruction </code> object.
+     * @return The new \c ProcessingInstruction object.
      *
      * @exception DOMException
      * INVALID_CHARACTER_ERR: Raised if an invalid character is
@@ -401,13 +402,13 @@ public:
                                                         const DOMString &data );
 
     /**
-     * Creates an <code> Attr </code> of the given name. Note that the
-     * <code> Attr </code> instance can then be set on an <code>
-     * Element </code> using the <code> setAttribute </code> method.
+     * Creates an \c Attr of the given name. Note that the
+     * \c Attr instance can then be set on an \c Element
+     * using the \c setAttribute method.
      *
      * @param name The name of the attribute.
      *
-     * @return A new <code> Attr </code> object.
+     * @return A new \c Attr object.
      *
      * @exception DOMException
      * INVALID_CHARACTER_ERR: Raised if the specified name contains an
@@ -452,7 +453,7 @@ public:
      *
      * @param name The name of the entity to reference.
      *
-     * @return The new <code> EntityReference </code> object.
+     * @return The new \c EntityReference object.
      *
      * @exception DOMException
      * INVALID_CHARACTER_ERR: Raised if the specified name contains an
@@ -466,12 +467,12 @@ public:
 
     /**
      * Moved from HTMLDocument in DOM Level 2
-     * Returns the Element whose <code> id </code> is given by
-     * elementId. If no such element exists, returns <code> null
-     * </code> . Behavior is not defined if more than one element has
-     * this <code> id </code> .
+     * Returns the Element whose \c id is given by
+     * elementId. If no such element exists, returns \c null
+     * . Behavior is not defined if more than one element has
+     * this \c id .
      *
-     * @param elementId The unique <code> id </code> value for an
+     * @param elementId The unique \c id value for an
      * element.
      *
      * @return The matching element.
@@ -482,16 +483,16 @@ public:
     /**
      * No Exceptions.
      *
-     * Returns a <code> NodeList </code> of all the <code> Element
-     * </code> s with a given tag name in the order in which they
-     * would be encountered in a preorder traversal of the <code>
-     * Document </code> tree.
+     * Returns a \c NodeList of all the \c Element 's
+     * with a given tag name in the order in which they
+     * would be encountered in a preorder traversal of the
+     * \c Document tree.
      *
      * @param tagname The name of the tag to match on. The special
      * value "*" matches all tags.
      *
-     * @return A new <code> NodeList </code> object containing all the
-     * matched <code> Element </code> s.
+     * @return A new \c NodeList object containing all the
+     * matched \c Element s.
      *
      */
     NodeList getElementsByTagName ( const DOMString &tagname );
@@ -645,7 +646,7 @@ public:
      * of NodeFilter for the set of possible SHOW_ values. These flags can be
      * combined using OR.
      *
-     * @param filter The NodeFilter to be used with this TreeWalker, or null to
+     * @param filter The NodeFilter to be used with this NodeIterator, or null to
      * indicate no filter.
      *
      * @param entityReferenceExpansion The value of this flag determines
@@ -743,6 +744,15 @@ public:
      */
     StyleSheetList styleSheets() const;
 
+    /**
+     * CSS3 mechanism for selecting alternate stylesheets using the DOM.
+     * Might change without further notice.
+     */
+
+    DOMString preferredStylesheetSet();
+    DOMString selectedStylesheetSet();
+    void setSelectedStylesheetSet(const DOMString& aString);
+
     /**
      * @return The KHTML view widget of this document.
      */
@@ -765,6 +775,120 @@ public:
     CSSStyleDeclaration getOverrideStyle(const Element &elt,
                                          const DOMString &pseudoElt);
 
+    /**
+     * Introduced in DOM Level 3
+     * This method is from the DocumentLS interface
+     *
+     * Indicates whether the method DocumentLS.load() should be synchronous or
+     * asynchronous. When the async attribute is set to true the load method
+     * returns control to the caller before the document has completed loading.
+     * The default value of this attribute is true.
+     */
+    bool async() const;
+
+    /**
+     * Introduced in DOM Level 3
+     * This method is from the DocumentLS interface
+     *
+     * see async
+     *
+     * @exception DOMException
+     * NOT_SUPPORTED_ERR: Raised if the implementation doesn't support the mode
+     * the attribute is being set to.
+     */
+    void setAsync( bool );
+
+
+    /**
+     * Introduced in DOM Level 3
+     * This method is from the DocumentLS interface
+     *
+     * If the document is currently being loaded as a result of the method load
+     * being invoked the loading and parsing is immediately aborted. The
+     * possibly partial result of parsing the document is discarded and the
+     * document is cleared.
+     */
+    void abort();
+
+    /**
+     * Introduced in DOM Level 3
+     * This method is from the DocumentLS interface
+     *
+     * Replaces the content of the document with the result of parsing the
+     * given URI. Invoking this method will either block the caller or return
+     * to the caller immediately depending on the value of the async attribute.
+     * Once the document is fully loaded a "load" event (as defined in
+     * [DOM Level 3 Events], except that the Event.targetNode will be the
+     * document, not an element) will be dispatched on the document. If an
+     * error occurs, an implementation dependent "error" event will be
+     * dispatched on the document. If this method is called on a document that
+     * is currently loading, the current load is interrupted and the new URI
+     * load is initiated.
+     *
+     * When invoking this method the parameters used in the DOMParser interface
+     * are assumed to have their default values with the exception that the
+     * parameters "entities", "normalize-characters",
+     * "check-character-normalization" are set to "false".
+     *
+     * The result of a call to this method is the same the result of a call to
+     * DOMParser.parseWithContext with an input stream referencing the URI that
+     * was passed to this call, the document as the context node, and the
+     * action ACTION_REPLACE_CHILDREN.
+     *
+     * @param uri of type DOMString
+     * The URI reference for the XML file to be loaded. If this is a relative
+     * URI, the base URI used by the implementation is implementation dependent.
+     *
+     * @return If async is set to true load returns true if the document load
+     * was successfully initiated. If an error occurred when initiating the
+     * document load, load returns false.
+     * If async is set to false load returns true if the document was
+     * successfully loaded and parsed. If an error occurred when either loading
+     * or parsing the URI, load returns false.
+     */
+    void load( const DOMString &uri );
+
+    /**
+     * Introduced in DOM Level 3
+     * This method is from the DocumentLS interface
+     *
+     * Replace the content of the document with the result of parsing the input
+     * string, this method is always synchronous. This method always parses
+     * from a DOMString, which means the data is always UTF-16. All other
+     * encoding information is ignored.
+     *
+     * The parameters used in the DOMParser interface are assumed to have their
+     * default values when invoking this method.
+     *
+     * The result of a call to this method is the same as the result of a call
+     * to DOMParser.parseWithContext with an input stream containing the string
+     * passed to this call, the document as the context node, and the action
+     * ACTION_REPLACE_CHILDREN.
+     *
+     * @param source A string containing an XML document.
+     */
+    void loadXML( const DOMString &source );
+
+    /**
+     * not part of the official DOM
+     *
+     * Documents are read-only by default, but they can be made editable by
+     * entering "design mode".
+     *
+     * @return whether this document is in design mode.
+     * @since 3.2
+     */
+    bool designMode() const;
+
+    /**
+     * not part of the official DOM
+     *
+     * @param enable @p true to enable design mode, @p false to disable.
+     * @see designMode
+     * @since 3.2
+     */
+    void setDesignMode(bool enable);
+
     /**
      * not part of the DOM
      *
@@ -772,6 +896,14 @@ public:
      */
     DOMString completeURL(const DOMString& url);
 
+    /**
+     * not part of the DOM
+     *
+     * Updates the rendered display after one or more changes to
+     * the DOM structure
+     */
+    void updateRendering();
+
 protected:
     Document( DocumentImpl *i);
 
@@ -781,50 +913,51 @@ protected:
 class DocumentFragmentImpl;
 
 /**
- * <code> DocumentFragment </code> is a "lightweight" or "minimal"
- * <code> Document </code> object. It is very common to want to be
+ * \c DocumentFragment is a "lightweight" or "minimal"
+ * \c Document object. It is very common to want to be
  * able to extract a portion of a document's tree or to create a new
  * fragment of a document. Imagine implementing a user command like
  * cut or rearranging a document by moving fragments around. It is
  * desirable to have an object which can hold such fragments and it is
  * quite natural to use a Node for this purpose. While it is true that
- * a <code> Document </code> object could fulfil this role, a <code>
- * Document </code> object can potentially be a heavyweight object,
+ * a \c Document object could fulfil this role, a
+ * \c Document object can potentially be a heavyweight object,
  * depending on the underlying implementation. What is really needed
- * for this is a very lightweight object. <code> DocumentFragment
- * </code> is such an object.
+ * for this is a very lightweight object. \c DocumentFragment
+ * is such an object.
  *
  *  Furthermore, various operations -- such as inserting nodes as
- * children of another <code> Node </code> -- may take <code>
- * DocumentFragment </code> objects as arguments; this results in all
- * the child nodes of the <code> DocumentFragment </code> being moved
+ * children of another \c Node -- may take
+ * \c DocumentFragment objects as arguments; this results in all
+ * the child nodes of the \c DocumentFragment being moved
  * to the child list of this node.
  *
- *  The children of a <code> DocumentFragment </code> node are zero or
+ *  The children of a \c DocumentFragment node are zero or
  * more nodes representing the tops of any sub-trees defining the
- * structure of the document. <code> DocumentFragment </code> nodes do
+ * structure of the document. \c DocumentFragment nodes do
  * not need to be well-formed XML documents (although they do need to
  * follow the rules imposed upon well-formed XML parsed entities,
- * which can have multiple top nodes). For example, a <code>
- * DocumentFragment </code> might have only one child and that child
- * node could be a <code> Text </code> node. Such a structure model
+ * which can have multiple top nodes). For example, a
+ * \c DocumentFragment might have only one child and that child
+ * node could be a \c Text node. Such a structure model
  * represents neither an HTML document nor a well-formed XML document.
  *
- *  When a <code> DocumentFragment </code> is inserted into a <code>
- * Document </code> (or indeed any other <code> Node </code> that may
- * take children) the children of the <code> DocumentFragment </code>
- * and not the <code> DocumentFragment </code> itself are inserted
- * into the <code> Node </code> . This makes the <code>
- * DocumentFragment </code> very useful when the user wishes to create
- * nodes that are siblings; the <code> DocumentFragment </code> acts
+ *  When a \c DocumentFragment is inserted into a
+ * \c Document (or indeed any other \c Node that may
+ * take children) the children of the \c DocumentFragment
+ * and not the \c DocumentFragment itself are inserted
+ * into the \c Node . This makes the
+ * \c DocumentFragment very useful when the user wishes to create
+ * nodes that are siblings; the \c DocumentFragment acts
  * as the parent of these nodes so that the user can use the standard
- * methods from the <code> Node </code> interface, such as <code>
- * insertBefore() </code> and <code> appendChild() </code> .
+ * methods from the \c Node interface, such as
+ * \c insertBefore() and \c appendChild() .
  *
  */
 class DocumentFragment : public Node
 {
     friend class Document;
+    friend class HTMLElementImpl;
     friend class Range;
 
 public:
@@ -846,16 +979,16 @@ class NamedNodeMap;
 class DOMString;
 
 /**
- * Each <code> Document </code> has a <code> doctype </code> attribute
- * whose value is either <code> null </code> or a <code> DocumentType
- * </code> object. The <code> DocumentType </code> interface in the
+ * Each \c Document has a \c doctype attribute
+ * whose value is either \c null or a \c DocumentType
+ * object. The \c DocumentType interface in the
  * DOM Level 1 Core provides an interface to the list of entities that
  * are defined for the document, and little else because the effect of
  * namespaces and the various XML scheme efforts on DTD representation
  * are not clearly understood as of this writing.
  *
- *  The DOM Level 1 doesn't support editing <code> DocumentType
- * </code> nodes.
+ *  The DOM Level 1 doesn't support editing \c DocumentType
+ * nodes.
  *
  */
 class DocumentType : public Node
@@ -875,34 +1008,34 @@ public:
 
     /**
      * The name of DTD; i.e., the name immediately following the
-     * <code> DOCTYPE </code> keyword.
+     * \c DOCTYPE keyword.
      *
      */
     DOMString name() const;
 
     /**
-     * A <code> NamedNodeMap </code> containing the general entities,
+     * A \c NamedNodeMap containing the general entities,
      * both external and internal, declared in the DTD. Duplicates are
      * discarded. For example in: &lt;!DOCTYPE ex SYSTEM "ex.dtd" [
      * &lt;!ENTITY foo "foo"> &lt;!ENTITY bar "bar"> &lt;!ENTITY % baz
-     * "baz"> ]> &lt;ex/> the interface provides access to <code> foo
-     * </code> and <code> bar </code> but not <code> baz </code> .
-     * Every node in this map also implements the <code> Entity
-     * </code> interface.
+     * "baz"> ]> &lt;ex/> the interface provides access to \c foo
+     * and \c bar but not \c baz .
+     * Every node in this map also implements the \c Entity
+     * interface.
      *
      *  The DOM Level 1 does not support editing entities, therefore
-     * <code> entities </code> cannot be altered in any way.
+     * \c entities cannot be altered in any way.
      *
      */
     NamedNodeMap entities() const;
 
     /**
-     * A <code> NamedNodeMap </code> containing the notations declared
+     * A \c NamedNodeMap containing the notations declared
      * in the DTD. Duplicates are discarded. Every node in this map
-     * also implements the <code> Notation </code> interface.
+     * also implements the \c Notation interface.
      *
      *  The DOM Level 1 does not support editing notations, therefore
-     * <code> notations </code> cannot be altered in any way.
+     * \c notations cannot be altered in any way.
      *
      */
     NamedNodeMap notations() const;
