@@ -18,14 +18,13 @@
  * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
  *
- * $Id: dom_element.cpp,v 1.30.2.7 2003/06/30 23:52:13 faure Exp $
+ * $Id: dom_element.cpp,v 1.46.2.3 2004/02/29 15:27:51 mueller Exp $
  */
 
 #include "dom/dom_exception.h"
 #include "xml/dom_docimpl.h"
 #include "xml/dom_elementimpl.h"
 #include "html/html_formimpl.h"
-#include <kdebug.h>
 
 using namespace DOM;
 
@@ -70,11 +69,9 @@ Attr::~Attr()
 DOMString Attr::name() const
 {
     if (!impl) throw DOMException(DOMException::NOT_FOUND_ERR);
-    return impl->getDocument()->attrName(
-        static_cast<AttrImpl*>(impl)->attrImpl()->id());
+    return ((AttrImpl *)impl)->name();
 }
 
-
 bool Attr::specified() const
 {
   if (impl) return ((AttrImpl *)impl)->specified();
@@ -150,69 +147,114 @@ DOMString Element::tagName() const
 
 DOMString Element::getAttribute( const DOMString &name )
 {
-    return getAttributeNS(DOMString(), name);
+    // ### getAttribute() and getAttributeNS() are supposed to return the empty string if the attribute
+    // does not exist. However, there are a number of places around khtml that expect a null string
+    // for nonexistent attributes. These need to be changed to use hasAttribute() instead.
+    if (!impl) throw DOMException(DOMException::NOT_FOUND_ERR);
+    if (!name.implementation()) throw DOMException(DOMException::NOT_FOUND_ERR);
+
+    NodeImpl::Id id = impl->getDocument()->getId(NodeImpl::AttributeId,name.implementation(),true,true);
+    if (!id) return DOMString();
+
+    ElementImpl* e = static_cast<ElementImpl*>(impl);
+    return e->getAttribute(id, false, name);
 }
 
 void Element::setAttribute( const DOMString &name, const DOMString &value )
 {
-    setAttributeNS(DOMString(), name, value);
+    if (!impl) throw DOMException(DOMException::NOT_FOUND_ERR);
+    int exceptioncode = 0;
+    NodeImpl::Id id = impl->getDocument()->getId(NodeImpl::AttributeId, name.implementation(), false /* allocate */,
+                                                 true, &exceptioncode);
+
+    static_cast<ElementImpl*>(impl)->setAttribute(id, value, name, exceptioncode);
+    if ( exceptioncode )
+        throw DOMException( exceptioncode );
 }
 
 void Element::removeAttribute( const DOMString &name )
 {
-    removeAttributeNS(DOMString(), name);
+    if (!impl) throw DOMException(DOMException::NOT_FOUND_ERR);
+    NodeImpl::Id id = impl->getDocument()->getId(NodeImpl::AttributeId, name.implementation(), true, true);
+    if (!id) return;
+
+    int exceptioncode = 0;
+    NamedNodeMapImpl *attributes = static_cast<ElementImpl*>(impl)->attributes(false);
+    attributes->removeNamedItem(id, false, name.implementation(), exceptioncode);
+    // it's allowed to remove attributes that don't exist.
+    if ( exceptioncode && exceptioncode != DOMException::NOT_FOUND_ERR )
+        throw DOMException( exceptioncode );
 }
 
 Attr Element::getAttributeNode( const DOMString &name )
 {
-    return getAttributeNodeNS(DOMString(), name);
+    if (!impl) throw DOMException(DOMException::NOT_FOUND_ERR);
+    if (!name.implementation()) throw DOMException(DOMException::NOT_FOUND_ERR);
+    NodeImpl::Id id = impl->getDocument()->getId(NodeImpl::AttributeId, name.implementation(), true, true);
+    if (!id) return 0;
+
+    ElementImpl* e = static_cast<ElementImpl*>(impl);
+    if (!e->attributes()) return 0;
+
+    return static_cast<AttrImpl*>(e->attributes()->getNamedItem(id, false, name.implementation()));
 }
 
 Attr Element::setAttributeNode( const Attr &newAttr )
 {
-    return setAttributeNodeNS(newAttr);
+    if (!impl || newAttr.isNull())
+        throw DOMException(DOMException::NOT_FOUND_ERR);
+    // WRONG_DOCUMENT_ERR and INUSE_ATTRIBUTE_ERR are already tested & thrown by setNamedItem
+
+    int exceptioncode = 0;
+    Attr r = static_cast<ElementImpl*>(impl)->attributes(false)->setNamedItem(newAttr.handle(), false,
+                               newAttr.handle()->nodeName().implementation(), exceptioncode);
+    if ( exceptioncode )
+        throw DOMException( exceptioncode );
+    static_cast<AttrImpl *>(newAttr.handle())->setOwnerElement( static_cast<ElementImpl*>(impl) );
+    return r;
 }
 
 Attr Element::removeAttributeNode( const Attr &oldAttr )
 {
     if (!impl || oldAttr.isNull() || oldAttr.ownerElement().handle() != impl)
         throw DOMException(DOMException::NOT_FOUND_ERR);
-    if (impl->getDocument() != oldAttr.handle()->getDocument())
-        throw DOMException(DOMException::WRONG_DOCUMENT_ERR);
 
-    int exceptioncode = 0;
+    if (impl->isReadOnly())
+        throw DOMException(DOMException::NO_MODIFICATION_ALLOWED_ERR);
 
-    NodeImpl::Id id = static_cast<AttrImpl *>(oldAttr.handle())->attrImpl()->id();
-    Attr r = static_cast<ElementImpl*>(impl)->attributes(true)->removeNamedItem(id, exceptioncode);
-    if ( exceptioncode )
-        throw DOMException( exceptioncode );
-    return r;
+    if (!static_cast<ElementImpl*>(impl)->attributes(true))
+        throw DOMException(DOMException::NOT_FOUND_ERR);
+
+    NamedAttrMapImpl *attributes = static_cast<ElementImpl*>(impl)->attributes(false);
+    return attributes->removeAttr(static_cast<AttrImpl*>(static_cast<AttrImpl*>(oldAttr.handle())));
 }
 
-NodeList Element::getElementsByTagName( const DOMString &name )
+NodeList Element::getElementsByTagName( const DOMString &tagName )
 {
     if (!impl) return 0;
-    return static_cast<ElementImpl*>(impl)->
-        getElementsByTagNameNS(0, name.implementation());
+    NodeImpl::Id id;
+    if ( tagName == "*" )
+        id = 0;
+    else
+        id = impl->getDocument()->getId(NodeImpl::ElementId, tagName.implementation(), false, true);
+    return new TagNodeListImpl( impl, id );
 }
 
 NodeList Element::getElementsByTagNameNS( const DOMString &namespaceURI,
                                           const DOMString &localName )
 {
     if (!impl) return 0;
-    return static_cast<ElementImpl*>(impl)->
-        getElementsByTagNameNS(namespaceURI.implementation(), localName.implementation());
+    return new TagNodeListImpl( impl, namespaceURI, localName );
 }
 
 DOMString Element::getAttributeNS( const DOMString &namespaceURI,
                                    const DOMString &localName)
 {
     if (!impl) throw DOMException(DOMException::NOT_FOUND_ERR);
-    NodeImpl::Id id = impl->getDocument()->attrId(namespaceURI.implementation(),
-                                                  localName.implementation(), true,
-                                                  0);
-    if (!id) return DOMString();
-    return static_cast<ElementImpl*>(impl)->getAttribute(id);
+    if (!localName.implementation()) throw DOMException(DOMException::NOT_FOUND_ERR);
+    NodeImpl::Id id = impl->getDocument()->getId(NodeImpl::AttributeId, namespaceURI.implementation(), 0/*prefix*/, localName.implementation(), true, true);
+    ElementImpl* e = static_cast<ElementImpl*>(impl);
+    return e->getAttribute(id, true);
 }
 
 void Element::setAttributeNS( const DOMString &namespaceURI,
@@ -221,34 +263,21 @@ void Element::setAttributeNS( const DOMString &namespaceURI,
 {
     if (!impl) throw DOMException(DOMException::NOT_FOUND_ERR);
 
-    int colonpos = qualifiedName.find(':');
-    DOMString localName = qualifiedName;
-    if (colonpos >= 0) {
-        localName.remove(0, colonpos+1);
-        // ### extract and set new prefix
-    }
     int exceptioncode = 0;
-    NodeImpl::Id id = impl->getDocument()->attrId(namespaceURI.implementation(),
-                                                  localName.implementation(), false /* allocate */,
-                                                  &exceptioncode);
-    if (exceptioncode)
-        throw DOMException(exceptioncode);
-    static_cast<ElementImpl*>(impl)->setAttribute(id, value.implementation(), exceptioncode);
-    if (exceptioncode)
-        throw DOMException(exceptioncode);
+    static_cast<ElementImpl*>(impl)->setAttributeNS(namespaceURI, qualifiedName, value, exceptioncode);
+    if ( exceptioncode )
+        throw DOMException( exceptioncode );
 }
 
 void Element::removeAttributeNS( const DOMString &namespaceURI,
                                  const DOMString &localName )
 {
     if (!impl) throw DOMException(DOMException::NOT_FOUND_ERR);
-    NodeImpl::Id id = impl->getDocument()->attrId(namespaceURI.implementation(),
-                                                  localName.implementation(), true,
-                                                  0);
-    if (!id) return;
 
     int exceptioncode = 0;
-    ((ElementImpl *)impl)->removeAttribute(id, exceptioncode);
+    NamedNodeMapImpl *attributes = static_cast<ElementImpl*>(impl)->attributes(false);
+    NodeImpl::Id id = impl->getDocument()->getId(NodeImpl::AttributeId, namespaceURI.implementation(), 0/*prefix*/, localName.implementation(), false, true);
+    attributes->removeNamedItem(id, true, 0, exceptioncode);
     if ( exceptioncode )
         throw DOMException( exceptioncode );
 }
@@ -257,24 +286,23 @@ Attr Element::getAttributeNodeNS( const DOMString &namespaceURI,
                                   const DOMString &localName )
 {
     if (!impl) throw DOMException(DOMException::NOT_FOUND_ERR);
-    NodeImpl::Id id = impl->getDocument()->attrId(namespaceURI.implementation(),
-                                                  localName.implementation(), true,
-                                                  0);
-    if (!id) return 0;
-
+    if (!localName.implementation()) throw DOMException(DOMException::NOT_FOUND_ERR);
+    NodeImpl::Id id = impl->getDocument()->getId(NodeImpl::AttributeId, namespaceURI.implementation(),
+						0/*prefix*/, localName.implementation(), true, true);
     ElementImpl* e = static_cast<ElementImpl*>(impl);
-    if (!e->attributes()) return 0; // exception ?
+    if (!e->attributes()) return 0;
 
-    return e->attributes()->getNamedItem(id);
+    return static_cast<AttrImpl*>(e->attributes()->getNamedItem(id, true));
 }
 
 Attr Element::setAttributeNodeNS( const Attr &newAttr )
 {
     if (!impl || newAttr.isNull())
         throw DOMException(DOMException::NOT_FOUND_ERR);
+    // WRONG_DOCUMENT_ERR and INUSE_ATTRIBUTE_ERR are already tested & thrown by setNamedItem
 
     int exceptioncode = 0;
-    Attr r = static_cast<ElementImpl*>(impl)->attributes(false)->setNamedItem(newAttr.handle(), exceptioncode);
+    Attr r = static_cast<ElementImpl*>(impl)->attributes(false)->setNamedItem(newAttr.handle(), true, 0, exceptioncode);
     if ( exceptioncode )
         throw DOMException( exceptioncode );
     static_cast<AttrImpl *>(newAttr.handle())->setOwnerElement( static_cast<ElementImpl*>(impl) );
@@ -284,20 +312,22 @@ Attr Element::setAttributeNodeNS( const Attr &newAttr )
 
 bool Element::hasAttribute( const DOMString& name )
 {
-    return hasAttributeNS(DOMString(), name);
+    if (!impl || !static_cast<ElementImpl*>(impl)->attributes()) return false; // ### throw ?
+    NodeImpl::Id id = impl->getDocument()->getId(NodeImpl::AttributeId, name.implementation(), true, true);
+    if (!id) return false;
+
+    if (!static_cast<ElementImpl*>(impl)->attributes(true /*readonly*/)) return false;
+    return static_cast<ElementImpl*>(impl)->attributes(true)->getValue(id, false, name.implementation()) != 0;
 }
 
 bool Element::hasAttributeNS( const DOMString &namespaceURI,
                               const DOMString &localName )
 {
     if (!impl || !static_cast<ElementImpl*>(impl)->attributes()) return false; // ### throw ?
-    NodeImpl::Id id = impl->getDocument()->attrId(namespaceURI.implementation(),
-                                                  localName.implementation(), true,
-                                                  0);
-    if (!id) return false;
-
     if (!static_cast<ElementImpl*>(impl)->attributes(true /*readonly*/)) return false;
-    return static_cast<ElementImpl*>(impl)->attributes(true)->getAttributeItem(id) != 0;
+    NodeImpl::Id id = impl->getDocument()->getId(NodeImpl::AttributeId,namespaceURI.implementation(),
+						 0/*prefix*/, localName.implementation(), true, true);
+    return static_cast<ElementImpl*>(impl)->attributes(true)->getValue(id, true) != 0;
 }
 
 bool Element::isHTMLElement() const
@@ -310,6 +340,11 @@ Element Element::form() const
 {
     if (!impl || !impl->isGenericFormElement()) return 0;
     return static_cast<HTMLGenericFormElementImpl*>(impl)->form();
+    ElementImpl* f = static_cast<HTMLGenericFormElementImpl*>( impl )->form();
+
+    if( f && f->implicitNode() )
+        return 0;
+    return f;
 }
 
 CSSStyleDeclaration Element::style()
@@ -318,6 +353,18 @@ CSSStyleDeclaration Element::style()
     return 0;
 }
 
+bool Element::contentEditable() const {
+    if(!impl) return false;
+    return static_cast<ElementImpl *>(impl)->contentEditable();
+}
+
+void Element::setContentEditable(bool enabled) {
+    if(!impl)
+        throw DOMException(DOMException::INVALID_STATE_ERR);
+
+    static_cast<ElementImpl *>(impl)->setContentEditable(enabled);
+}
+
 bool Element::khtmlValidAttrName(const DOMString &name)
 {
     // Check if name is valid
