@@ -18,7 +18,6 @@
  * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
  *
- * $Id: dom_node.cpp,v 1.44.2.2 2003/06/27 20:05:48 faure Exp $
  */
 
 #include "dom/dom_doc.h"
@@ -66,17 +65,33 @@ NamedNodeMap::~NamedNodeMap()
 
 Node NamedNodeMap::getNamedItem( const DOMString &name ) const
 {
-    return getNamedItemNS(DOMString(), name);
+    if (!impl) return 0;
+    NodeImpl::Id nid = impl->mapId(0, name.implementation(), true);
+    if (!nid) return 0;
+    return impl->getNamedItem(nid, false, name.implementation());
 }
 
 Node NamedNodeMap::setNamedItem( const Node &arg )
 {
-    return setNamedItemNS(arg);
+    if (!impl) throw DOMException(DOMException::NOT_FOUND_ERR);
+    if (!arg.impl) throw DOMException(DOMException::NOT_FOUND_ERR);
+    int exceptioncode = 0;
+    Node r = impl->setNamedItem(arg.impl, false,
+                       arg.impl->nodeName().implementation(), exceptioncode);
+    if (exceptioncode)
+        throw DOMException(exceptioncode);
+    return r;
 }
 
 Node NamedNodeMap::removeNamedItem( const DOMString &name )
 {
-    return removeNamedItemNS(DOMString(), name);
+    if (!impl) throw DOMException(DOMException::NOT_FOUND_ERR);
+    int exceptioncode = 0;
+    Node r = impl->removeNamedItem(impl->mapId(0, name.implementation(), false),
+                                   false, name.implementation(), exceptioncode);
+    if (exceptioncode)
+        throw DOMException(exceptioncode);
+    return r;
 }
 
 Node NamedNodeMap::item( unsigned long index ) const
@@ -88,14 +103,15 @@ Node NamedNodeMap::item( unsigned long index ) const
 Node NamedNodeMap::getNamedItemNS( const DOMString &namespaceURI, const DOMString &localName ) const
 {
     if (!impl) return 0;
-    return impl->getNamedItem(impl->mapId(namespaceURI, localName, true));
+    NodeImpl::Id nid = impl->mapId( namespaceURI.implementation(), localName.implementation(), true );
+    return impl->getNamedItem(nid, true);
 }
 
 Node NamedNodeMap::setNamedItemNS( const Node &arg )
 {
     if (!impl) throw DOMException(DOMException::NOT_FOUND_ERR);
     int exceptioncode = 0;
-    Node r = impl->setNamedItem(arg.impl, exceptioncode);
+    Node r = impl->setNamedItem(arg.impl, true, 0, exceptioncode);
     if (exceptioncode)
         throw DOMException(exceptioncode);
     return r;
@@ -105,12 +121,8 @@ Node NamedNodeMap::removeNamedItemNS( const DOMString &namespaceURI, const DOMSt
 {
     if (!impl) throw DOMException(DOMException::NOT_FOUND_ERR);
     int exceptioncode = 0;
-    NodeImpl::Id id = impl->mapId(namespaceURI, localName, true);
-    NodeImpl* oldItem = impl->getNamedItem(id);
-    if (!oldItem) throw DOMException(DOMException::NOT_FOUND_ERR);
-
-    Node r(oldItem->cloneNode(true));
-    impl->removeNamedItem(id, exceptioncode);
+    NodeImpl::Id nid = impl->mapId( namespaceURI.implementation(), localName.implementation(), false );
+    Node r = impl->removeNamedItem(nid, true, 0, exceptioncode);
     if (exceptioncode)
         throw DOMException(exceptioncode);
     return r;
@@ -122,23 +134,8 @@ unsigned long NamedNodeMap::length() const
     return impl->length();
 }
 
-NamedNodeMapImpl *NamedNodeMap::handle() const throw()
-{
-    return impl;
-}
-
-bool NamedNodeMap::isNull() const throw()
-{
-    return (impl == 0);
-}
-
 // ---------------------------------------------------------------------------
 
-Node::Node()
-{
-    impl = 0;
-}
-
 Node::Node(const Node &other)
 {
     impl = other.impl;
@@ -161,12 +158,12 @@ Node &Node::operator = (const Node &other)
     return *this;
 }
 
-bool Node::operator == (const Node &other)
+bool Node::operator == (const Node &other) const
 {
     return (impl == other.impl);
 }
 
-bool Node::operator != (const Node &other)
+bool Node::operator != (const Node &other) const
 {
     return !(impl == other.impl);
 }
@@ -336,7 +333,7 @@ bool Node::isSupported( const DOMString &feature,
 DOMString Node::namespaceURI(  ) const
 {
     if (!impl) return DOMString();
-    return impl->getDocument()->namespaceURI(impl->id());
+    return impl->namespaceURI();
 }
 
 DOMString Node::prefix(  ) const
@@ -382,11 +379,14 @@ bool Node::dispatchEvent(const Event &evt)
     if (!impl)
 	throw DOMException(DOMException::INVALID_STATE_ERR);
 
+    if (!evt.handle())
+        throw DOMException(DOMException::NOT_FOUND_ERR);
+
     int exceptioncode = 0;
-    bool r = impl->dispatchEvent(evt.handle(),exceptioncode);
+    impl->dispatchEvent(evt.handle(),exceptioncode);
     if (exceptioncode)
 	throw DOMException(exceptioncode);
-    return r;
+    return !evt.handle()->defaultPrevented();
 }
 
 
@@ -417,7 +417,8 @@ void Node::applyChanges()
 void Node::getCursor(int offset, int &_x, int &_y, int &height)
 {
     if (!impl) throw DOMException(DOMException::NOT_FOUND_ERR);
-    impl->getCursor(offset, _x, _y, height);
+    int dummy;
+    impl->getCaret(offset, false, _x, _y, dummy, height);
 }
 
 QRect Node::getRect()
@@ -426,16 +427,6 @@ QRect Node::getRect()
     return impl->getRect();
 }
 
-bool Node::isNull() const
-{
-    return (impl == 0);
-}
-
-NodeImpl *Node::handle() const
-{
-    return impl;
-}
-
 //-----------------------------------------------------------------------------
 
 NodeList::NodeList()
@@ -481,14 +472,3 @@ unsigned long NodeList::length() const
     if (!impl) return 0;
     return impl->length();
 }
-
-NodeListImpl *NodeList::handle() const
-{
-    return impl;
-}
-
-bool NodeList::isNull() const
-{
-    return (impl == 0);
-}
-
