@@ -18,11 +18,11 @@
  * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
  *
- * $Id: dom_string.h,v 1.27.4.1 2003/05/18 12:34:37 mueller Exp $
  */
 #ifndef _DOM_DOMString_h_
 #define _DOM_DOMString_h_
 
+#include <kdebug.h>
 #include <qstring.h>
 
 namespace DOM {
@@ -36,7 +36,7 @@ class DOMStringImpl;
  *
  * All DOMStrings are explicitly shared (they behave like pointers), meaning
  * that modifications to one instance will also modify all others. If you
- * wish to get a DOMString that is independent, use @ref copy().
+ * wish to get a DOMString that is independent, use copy().
  */
 class DOMString
 {
@@ -46,12 +46,13 @@ public:
     /**
      * default constructor. Gives an empty DOMString
      */
-    DOMString();
+    DOMString() : impl(0) {}
 
     DOMString(const QChar *str, uint len);
     DOMString(const QString &);
     DOMString(const char *str);
     DOMString(DOMStringImpl *i);
+
     virtual ~DOMString();
 
     // assign and copy
@@ -115,6 +116,16 @@ protected:
     DOMStringImpl *impl;
 };
 
+#ifndef NDEBUG
+inline kdbgstream &operator<<(kdbgstream &stream, const DOMString &string) { 
+	return (stream << string.string());
+}
+#else
+inline kndbgstream &operator<<(kndbgstream &stream, const DOMString &) {
+	return stream;
+}
+#endif
+
 bool operator==( const DOMString &a, const DOMString &b );
 bool operator==( const DOMString &a, const QString &b );
 bool operator==( const DOMString &a, const char *b );
