@@ -25,7 +25,6 @@
  * Technology , Institut National de Recherche en Informatique et en
  * Automatique , Keio University ). All Rights Reserved.
  *
- * $Id: dom_text.h,v 1.9.12.1 2003/05/18 12:34:37 mueller Exp $
  */
 #ifndef _DOM_CharacterData_h_
 #define _DOM_CharacterData_h_
@@ -38,17 +37,19 @@ class DOMString;
 class CharacterDataImpl;
 
 /**
- * The <code> CharacterData </code> interface extends Node with a set
+ * The \c CharacterData interface extends Node with a set
  * of attributes and methods for accessing character data in the DOM.
  * For clarity this set is defined here rather than on each object
  * that uses these attributes and methods. No DOM objects correspond
- * directly to <code> CharacterData </code> , though <code> Text
- * </code> and others do inherit the interface from it. All <code>
- * offset</code>s in this interface start from 0.
+ * directly to \c CharacterData , though \c Text
+ * and others do inherit the interface from it. All
+ * <code>offset</code>s in this interface start from 0.
  *
  */
 class CharacterData : public Node
 {
+    friend class CharacterDataImpl;
+
 public:
     CharacterData();
     CharacterData(const CharacterData &other);
@@ -63,23 +64,23 @@ public:
     /**
      * The character data of the node that implements this interface.
      * The DOM implementation may not put arbitrary limits on the
-     * amount of data that may be stored in a <code> CharacterData
-     * </code> node. However, implementation limits may mean that the
-     * entirety of a node's data may not fit into a single <code>
-     * DOMString </code> . In such cases, the user may call <code>
-     * substringData </code> to retrieve the data in appropriately
+     * amount of data that may be stored in a \c CharacterData
+     * node. However, implementation limits may mean that the
+     * entirety of a node's data may not fit into a single
+     * \c DOMString . In such cases, the user may call
+     * \c substringData to retrieve the data in appropriately
      * sized pieces.
      *
      * @exception DOMException
      * DOMSTRING_SIZE_ERR: Raised when it would return more characters
-     * than fit in a <code> DOMString </code> variable on the
+     * than fit in a \c DOMString variable on the
      * implementation platform.
      *
      */
     DOMString data() const;
 
     /**
-     * see @ref data
+     * see data
      * @exception DOMException
      * NO_MODIFICATION_ALLOWED_ERR: Raised when the node is readonly.
      *
@@ -87,9 +88,9 @@ public:
     void setData( const DOMString & );
 
     /**
-     * The number of characters that are available through <code> data
-     * </code> and the <code> substringData </code> method below. This
-     * may have the value zero, i.e., <code> CharacterData </code>
+     * The number of characters that are available through \c data
+     * and the \c substringData method below. This
+     * may have the value zero, i.e., \c CharacterData
      * nodes may be empty.
      *
      */
@@ -102,29 +103,29 @@ public:
      *
      * @param count The number of characters to extract.
      *
-     * @return The specified substring. If the sum of <code> offset
-     * </code> and <code> count </code> exceeds the <code> length
-     * </code> , then all characters to the end of the data are
+     * @return The specified substring. If the sum of \c offset
+     * and \c count exceeds the \c length
+     *  , then all characters to the end of the data are
      * returned.
      *
      * @exception DOMException
      * INDEX_SIZE_ERR: Raised if the specified offset is negative or
-     * greater than the number of characters in <code> data </code> ,
-     * or if the specified <code> count </code> is negative.
+     * greater than the number of characters in \c data ,
+     * or if the specified \c count is negative.
      *
      *  DOMSTRING_SIZE_ERR: Raised if the specified range of text does
-     * not fit into a <code> DOMString </code> .
+     * not fit into a \c DOMString .
      *
      */
     DOMString substringData ( const unsigned long offset, const unsigned long count );
 
     /**
      * Append the string to the end of the character data of the node.
-     * Upon success, <code> data </code> provides access to the
-     * concatenation of <code> data </code> and the <code> DOMString
-     * </code> specified.
+     * Upon success, \c data provides access to the
+     * concatenation of \c data and the \c DOMString
+     * specified.
      *
-     * @param arg The <code> DOMString </code> to append.
+     * @param arg The \c DOMString to append.
      *
      * @return
      *
@@ -139,13 +140,13 @@ public:
      *
      * @param offset The character offset at which to insert.
      *
-     * @param arg The <code> DOMString </code> to insert.
+     * @param arg The \c DOMString to insert.
      *
      * @return
      *
      * @exception DOMException
      * INDEX_SIZE_ERR: Raised if the specified offset is negative or
-     * greater than the number of characters in <code> data </code> .
+     * greater than the number of characters in \c data .
      *
      *  NO_MODIFICATION_ALLOWED_ERR: Raised if this node is readonly.
      *
@@ -154,22 +155,22 @@ public:
 
     /**
      * Remove a range of characters from the node. Upon success,
-     * <code> data </code> and <code> length </code> reflect the
+     * \c data and \c length reflect the
      * change.
      *
      * @param offset The offset from which to remove characters.
      *
      * @param count The number of characters to delete. If the sum of
-     * <code> offset </code> and <code> count </code> exceeds <code>
-     * length </code> then all characters from <code> offset </code>
+     * \c offset and \c count exceeds
+     * \c length then all characters from \c offset
      * to the end of the data are deleted.
      *
      * @return
      *
      * @exception DOMException
      * INDEX_SIZE_ERR: Raised if the specified offset is negative or
-     * greater than the number of characters in <code> data </code> ,
-     * or if the specified <code> count </code> is negative.
+     * greater than the number of characters in \c data ,
+     * or if the specified \c count is negative.
      *
      *  NO_MODIFICATION_ALLOWED_ERR: Raised if this node is readonly.
      *
@@ -183,21 +184,21 @@ public:
      * @param offset The offset from which to start replacing.
      *
      * @param count The number of characters to replace. If the sum of
-     * <code> offset </code> and <code> count </code> exceeds <code>
-     * length </code> , then all characters to the end of the data are
-     * replaced (i.e., the effect is the same as a <code> remove
-     * </code> method call with the same range, followed by an <code>
-     * append </code> method invocation).
+     * \c offset and \c count exceeds
+     * \c length , then all characters to the end of the data are
+     * replaced (i.e., the effect is the same as a \c remove
+     * method call with the same range, followed by an
+     * \c append method invocation).
      *
-     * @param arg The <code> DOMString </code> with which the range
+     * @param arg The \c DOMString with which the range
      * must be replaced.
      *
      * @return
      *
      * @exception DOMException
      * INDEX_SIZE_ERR: Raised if the specified offset is negative or
-     * greater than the number of characters in <code> data </code> ,
-     * or if the specified <code> count </code> is negative.
+     * greater than the number of characters in \c data ,
+     * or if the specified \c count is negative.
      *
      *  NO_MODIFICATION_ALLOWED_ERR: Raised if this node is readonly.
      *
@@ -213,8 +214,8 @@ class CommentImpl;
 
 /**
  * This represents the content of a comment, i.e., all the characters
- * between the starting ' <code> &lt;!-- </code> ' and ending ' <code>
- * --&gt; </code> '. Note that this is the definition of a comment in
+ * between the starting ' \c &lt;!-- ' and ending '
+ * \c --&gt; '. Note that this is the definition of a comment in
  * XML, and, in practice, HTML, although some HTML tools may implement
  * the full SGML comment structure.
  *
@@ -222,6 +223,7 @@ class CommentImpl;
 class Comment : public CharacterData
 {
     friend class Document;
+    friend class TextImpl;
 
 public:
     Comment();
@@ -241,32 +243,33 @@ protected:
 class TextImpl;
 
 /**
- * The <code> Text </code> interface represents the textual content
+ * The \c Text interface represents the textual content
  * (termed <a href="&xml-spec;#syntax"> character data </a> in XML) of
- * an <code> Element </code> or <code> Attr </code> . If there is no
+ * an \c Element or \c Attr . If there is no
  * markup inside an element's content, the text is contained in a
- * single object implementing the <code> Text </code> interface that
+ * single object implementing the \c Text interface that
  * is the only child of the element. If there is markup, it is parsed
- * into a list of elements and <code> Text </code> nodes that form the
+ * into a list of elements and \c Text nodes that form the
  * list of children of the element.
  *
  *  When a document is first made available via the DOM, there is only
- * one <code> Text </code> node for each block of text. Users may
- * create adjacent <code> Text </code> nodes that represent the
+ * one \c Text node for each block of text. Users may
+ * create adjacent \c Text nodes that represent the
  * contents of a given element without any intervening markup, but
  * should be aware that there is no way to represent the separations
  * between these nodes in XML or HTML, so they will not (in general)
- * persist between DOM editing sessions. The <code> normalize()
- * </code> method on <code> Element </code> merges any such adjacent
- * <code> Text </code> objects into a single node for each block of
+ * persist between DOM editing sessions. The \c normalize()
+ * method on \c Element merges any such adjacent
+ * \c Text objects into a single node for each block of
  * text; this is recommended before employing operations that depend
- * on a particular document structure, such as navigation with <code>
- * XPointers. </code>
+ * on a particular document structure, such as navigation with
+ * \c XPointers.
  *
  */
 class Text : public CharacterData
 {
     friend class Document;
+    friend class TextImpl;
 
 public:
     Text();
@@ -280,20 +283,20 @@ public:
     ~Text();
 
     /**
-     * Breaks this <code> Text </code> node into two Text nodes at the
+     * Breaks this \c Text node into two Text nodes at the
      * specified offset, keeping both in the tree as siblings. This
-     * node then only contains all the content up to the <code> offset
-     * </code> point. And a new <code> Text </code> node, which is
+     * node then only contains all the content up to the \c offset
+     * point. And a new \c Text node, which is
      * inserted as the next sibling of this node, contains all the
-     * content at and after the <code> offset </code> point.
+     * content at and after the \c offset point.
      *
      * @param offset The offset at which to split, starting from 0.
      *
-     * @return The new <code> Text </code> node.
+     * @return The new \c Text node.
      *
      * @exception DOMException
      * INDEX_SIZE_ERR: Raised if the specified offset is negative or
-     * greater than the number of characters in <code> data </code> .
+     * greater than the number of characters in \c data .
      *
      *  NO_MODIFICATION_ALLOWED_ERR: Raised if this node is readonly.
      *
