@@ -18,7 +18,6 @@
  * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
  *
- * $Id: html_block.cpp,v 1.13.2.1 2003/01/03 21:09:23 mueller Exp $
  */
 // --------------------------------------------------------------------------
 
@@ -74,15 +73,18 @@ void HTMLBlockquoteElement::setCite( const DOMString &value )
 
 // --------------------------------------------------------------------------
 
-HTMLDivElement::HTMLDivElement() : HTMLElement()
+HTMLDivElement::HTMLDivElement()
+    : HTMLElement()
 {
 }
 
-HTMLDivElement::HTMLDivElement(const HTMLDivElement &other) : HTMLElement(other)
+HTMLDivElement::HTMLDivElement(const HTMLDivElement &other)
+    : HTMLElement(other)
 {
 }
 
-HTMLDivElement::HTMLDivElement(HTMLDivElementImpl *impl) : HTMLElement(impl)
+HTMLDivElement::HTMLDivElement(HTMLDivElementImpl *impl)
+    : HTMLElement(impl)
 {
 }
 
@@ -115,15 +117,18 @@ void HTMLDivElement::setAlign( const DOMString &value )
 
 // --------------------------------------------------------------------------
 
-HTMLHRElement::HTMLHRElement() : HTMLElement()
+HTMLHRElement::HTMLHRElement() 
+    : HTMLElement()
 {
 }
 
-HTMLHRElement::HTMLHRElement(const HTMLHRElement &other) : HTMLElement(other)
+HTMLHRElement::HTMLHRElement(const HTMLHRElement &other) 
+    : HTMLElement(other)
 {
 }
 
-HTMLHRElement::HTMLHRElement(HTMLHRElementImpl *impl) : HTMLElement(impl)
+HTMLHRElement::HTMLHRElement(HTMLHRElementImpl *impl) 
+    : HTMLElement(impl)
 {
 }
 
@@ -195,15 +200,18 @@ void HTMLHRElement::setWidth( const DOMString &value )
 
 // --------------------------------------------------------------------------
 
-HTMLHeadingElement::HTMLHeadingElement() : HTMLElement()
+HTMLHeadingElement::HTMLHeadingElement() 
+    : HTMLElement()
 {
 }
 
-HTMLHeadingElement::HTMLHeadingElement(const HTMLHeadingElement &other) : HTMLElement(other)
+HTMLHeadingElement::HTMLHeadingElement(const HTMLHeadingElement &other) 
+    : HTMLElement(other)
 {
 }
 
-HTMLHeadingElement::HTMLHeadingElement(HTMLElementImpl *impl) : HTMLElement(impl)
+HTMLHeadingElement::HTMLHeadingElement(HTMLElementImpl *impl) 
+    : HTMLElement(impl)
 {
 }
 
@@ -251,11 +259,13 @@ HTMLParagraphElement::HTMLParagraphElement() : HTMLElement()
 {
 }
 
-HTMLParagraphElement::HTMLParagraphElement(const HTMLParagraphElement &other) : HTMLElement(other)
+HTMLParagraphElement::HTMLParagraphElement(const HTMLParagraphElement &other) 
+    : HTMLElement(other)
 {
 }
 
-HTMLParagraphElement::HTMLParagraphElement(HTMLElementImpl *impl) : HTMLElement(impl)
+HTMLParagraphElement::HTMLParagraphElement(HTMLElementImpl *impl) 
+    : HTMLElement(impl)
 {
 }
 
@@ -292,11 +302,13 @@ HTMLPreElement::HTMLPreElement() : HTMLElement()
 {
 }
 
-HTMLPreElement::HTMLPreElement(const HTMLPreElement &other) : HTMLElement(other)
+HTMLPreElement::HTMLPreElement(const HTMLPreElement &other) 
+    : HTMLElement(other)
 {
 }
 
-HTMLPreElement::HTMLPreElement(HTMLPreElementImpl *impl) : HTMLElement(impl)
+HTMLPreElement::HTMLPreElement(HTMLPreElementImpl *impl) 
+    : HTMLElement(impl)
 {
 }
 
