@@ -25,7 +25,6 @@
  * Technology , Institut National de Recherche en Informatique et en
  * Automatique , Keio University ). All Rights Reserved.
  *
- * $Id: html_element.h,v 1.19 2002/10/30 10:17:48 faure Exp $
  */
 #ifndef HTML_ELEMENT_H
 #define HTML_ELEMENT_H
@@ -43,25 +42,25 @@ class HTMLCollection;
 /**
  * All HTML element interfaces derive from this class. Elements that
  * only expose the HTML core attributes are represented by the base
- * <code> HTMLElement </code> interface. These elements are as
+ * \c HTMLElement interface. These elements are as
  * follows:
  *
- *  <ulist> <item> HEAD
+ *  \li \c HEAD
  *
- *  </item> <item> special: SUB, SUP, SPAN, BDO
+ *  \li special: <tt> SUB, SUP, SPAN, BDO </tt>
  *
- *  </item> <item> font: TT, I, B, U, S, STRIKE, BIG, SMALL
+ *  \li font: <tt> TT, I, B, U, S, STRIKE, BIG, SMALL </tt>
  *
- *  </item> <item> phrase: EM, STRONG, DFN, CODE, SAMP, KBD, VAR,
- * CITE, ACRONYM, ABBR
+ *  \li phrase: <tt> EM, STRONG, DFN, CODE, SAMP, KBD, VAR,
+ * CITE, ACRONYM, ABBR</tt>
  *
- *  </item> <item> list: DD, DT
+ *  \li list: <tt> DD, DT </tt>
  *
- *  </item> <item> NOFRAMES, NOSCRIPT
+ *  \li <tt> NOFRAMES, NOSCRIPT </tt>
  *
- *  </item> <item> ADDRESS, CENTER
+ *  \li <tt> ADDRESS, CENTER </tt>
  *
- *  </item> </ulist> Note. The <code> style </code> attribute for this
+ *  Note: The \c style attribute for this
  * interface is reserved for future usage.
  *
  */
@@ -97,7 +96,7 @@ public:
     DOMString id() const;
 
     /**
-     * see @ref id
+     * see id
      */
     void setId( const DOMString & );
 
@@ -110,7 +109,7 @@ public:
     DOMString title() const;
 
     /**
-     * see @ref title
+     * see title
      */
     void setTitle( const DOMString & );
 
@@ -123,7 +122,7 @@ public:
     DOMString lang() const;
 
     /**
-     * see @ref lang
+     * see lang
      */
     void setLang( const DOMString & );
 
@@ -137,7 +136,7 @@ public:
     DOMString dir() const;
 
     /**
-     * see @ref dir
+     * see dir
      */
     void setDir( const DOMString & );
 
@@ -152,7 +151,7 @@ public:
     DOMString className() const;
 
     /**
-     * see @ref className
+     * see className
      */
     void setClassName( const DOMString & );
 
@@ -215,6 +214,6 @@ protected:
     void assignOther( const Node &other, int elementId );
 };
 
-}; //namespace
+} //namespace
 
 #endif
