@@ -25,7 +25,7 @@
  * Technology , Institut National de Recherche en Informatique et en
  * Automatique , Keio University ). All Rights Reserved.
  *
- * $Id: html_form.h,v 1.14.2.2 2003/05/18 12:34:37 mueller Exp $
+ * $Id: html_form.h,v 1.22 2003/10/13 06:20:16 faure Exp $
  */
 #ifndef HTML_FORM_H
 #define HTML_FORM_H
@@ -34,6 +34,8 @@
 #include <dom/html_element.h>
 #include <dom/html_misc.h>
 
+#include <kdemacros.h>
+
 namespace DOM {
 
 class HTMLButtonElementImpl;
@@ -63,7 +65,7 @@ public:
     ~HTMLButtonElement();
 
     /**
-     * Returns the <code> FORM </code> element containing this
+     * Returns the \c FORM element containing this
      * control. Returns null if this control is not within the context
      * of a form.
      *
@@ -80,7 +82,7 @@ public:
     DOMString accessKey() const;
 
     /**
-     * see @ref accessKey
+     * see accessKey
      */
     void setAccessKey( const DOMString & );
 
@@ -93,7 +95,7 @@ public:
     bool disabled() const;
 
     /**
-     * see @ref disabled
+     * see disabled
      */
     void setDisabled( bool );
 
@@ -107,7 +109,7 @@ public:
     DOMString name() const;
 
     /**
-     * see @ref name
+     * see name
      */
     void setName( const DOMString & );
 
@@ -121,7 +123,7 @@ public:
     long tabIndex() const;
 
     /**
-     * see @ref tabIndex
+     * see tabIndex
      */
     void setTabIndex( long );
 
@@ -142,7 +144,7 @@ public:
     DOMString value() const;
 
     /**
-     * see @ref value
+     * see value
      */
     void setValue( const DOMString & );
 };
@@ -173,7 +175,7 @@ public:
     ~HTMLFieldSetElement();
 
     /**
-     * @deprecated. ### KDE 4.0: remove.
+     * ### KDE 4.0: remove.
      */
     HTMLFormElement form() const;
 };
@@ -182,7 +184,7 @@ public:
 
 class HTMLFormElementImpl;
 /**
- * The <code> FORM </code> element encompasses behavior similar to a
+ * The \c FORM element encompasses behavior similar to a
  * collection and an element. It provides direct access to the
  * contained input elements as well as the attributes of the form
  * element. See the <a
@@ -236,7 +238,7 @@ public:
     DOMString name() const;
 
     /**
-     * see @ref name
+     * see name
      */
     void setName( const DOMString & );
 
@@ -249,7 +251,7 @@ public:
     DOMString acceptCharset() const;
 
     /**
-     * see @ref acceptCharset
+     * see acceptCharset
      */
     void setAcceptCharset( const DOMString & );
 
@@ -262,7 +264,7 @@ public:
     DOMString action() const;
 
     /**
-     * see @ref action
+     * see action
      */
     void setAction( const DOMString & );
 
@@ -276,7 +278,7 @@ public:
     DOMString enctype() const;
 
     /**
-     * see @ref enctype
+     * see enctype
      */
     void setEnctype( const DOMString & );
 
@@ -289,7 +291,7 @@ public:
     DOMString method() const;
 
     /**
-     * see @ref method
+     * see method
      */
     void setMethod( const DOMString & );
 
@@ -302,7 +304,7 @@ public:
     DOMString target() const;
 
     /**
-     * see @ref target
+     * see target
      */
     void setTarget( const DOMString & );
 
@@ -351,31 +353,31 @@ public:
 
     /**
      * Stores the initial control value (i.e., the initial value of
-     * <code> value </code> ).
+     * \c value ).
      *
      */
     DOMString defaultValue() const;
 
     /**
-     * see @ref defaultValue
+     * see defaultValue
      */
     void setDefaultValue( const DOMString & );
 
     /**
-     * When <code> type </code> has the value "Radio" or "Checkbox",
-     * stores the initial value of the <code> checked </code>
+     * When \c type has the value "Radio" or "Checkbox",
+     * stores the initial value of the \c checked
      * attribute.
      *
      */
     bool defaultChecked() const;
 
     /**
-     * see @ref defaultChecked
+     * see defaultChecked
      */
     void setDefaultChecked( bool );
 
     /**
-     * @deprecated. ### KDE 4.0: remove.
+     * ### KDE 4.0: remove.
      */
     HTMLFormElement form() const;
 
@@ -389,7 +391,7 @@ public:
     DOMString accept() const;
 
     /**
-     * see @ref accept
+     * see accept
      */
     void setAccept( const DOMString & );
 
@@ -403,7 +405,7 @@ public:
     DOMString accessKey() const;
 
     /**
-     * see @ref accessKey
+     * see accessKey
      */
     void setAccessKey( const DOMString & );
 
@@ -418,7 +420,7 @@ public:
     DOMString align() const;
 
     /**
-     * see @ref align
+     * see align
      */
     void setAlign( const DOMString & );
 
@@ -432,14 +434,14 @@ public:
     DOMString alt() const;
 
     /**
-     * see @ref alt
+     * see alt
      */
     void setAlt( const DOMString & );
 
     /**
-     * Describes whether a radio or check box is checked, when <code>
-     * type </code> has the value "Radio" or "Checkbox". The value is
-     * TRUE if explicitly set. Represents the current state of the
+     * Describes whether a radio or check box is checked, when
+     * \c type has the value "Radio" or "Checkbox". The value is
+     * true if explicitly set. Represents the current state of the
      * checkbox or radio button. See the <a
      * href="http://www.w3.org/TR/REC-html40/interact/forms.html#adef-checked">
      * checked attribute definition </a> in HTML 4.0.
@@ -448,7 +450,7 @@ public:
     bool checked() const;
 
     /**
-     * see @ref checked
+     * see checked
      */
     void setChecked( bool );
 
@@ -461,13 +463,13 @@ public:
     bool disabled() const;
 
     /**
-     * see @ref disabled
+     * see disabled
      */
     void setDisabled( bool );
 
     /**
-     * Maximum number of characters for text fields, when <code> type
-     * </code> has the value "Text" or "Password". See the <a
+     * Maximum number of characters for text fields, when \c type
+     * has the value "Text" or "Password". See the <a
      * href="http://www.w3.org/TR/REC-html40/interact/forms.html#adef-maxlength">
      * maxlength attribute definition </a> in HTML 4.0.
      *
@@ -475,7 +477,7 @@ public:
     long maxLength() const;
 
     /**
-     * see @ref maxLength
+     * see maxLength
      */
     void setMaxLength( long );
 
@@ -489,12 +491,12 @@ public:
     DOMString name() const;
 
     /**
-     * see @ref name
+     * see name
      */
     void setName( const DOMString & );
 
     /**
-     * This control is read-only. When <code> type </code> has the
+     * This control is read-only. When \c type has the
      * value "text" or "password" only. See the <a
      * href="http://www.w3.org/TR/REC-html40/interact/forms.html#adef-readonly">
      * readonly attribute definition </a> in HTML 4.0.
@@ -504,19 +506,19 @@ public:
 
     // ### remove in 4.0
     /**
-     * see @ref readOnly
+     * see readOnly
      */
     void setReadOnly( bool );
 
     /**
      * @deprecated
      */
-    DOMString size() const;
+    DOMString size() const KDE_DEPRECATED;
 
     /**
      * @deprecated
      */
-    void setSize( const DOMString & );
+    void setSize( const DOMString & ) KDE_DEPRECATED;
 
     /**
      * Size information. The precise meaning is specific to each type
@@ -528,12 +530,12 @@ public:
     long getSize() const;
 
     /**
-     * see @ref getSize
+     * see getSize
      */
     void setSize( long );
 
     /**
-     * When the <code> type </code> attribute has the value "Image",
+     * When the \c type attribute has the value "Image",
      * this attribute specifies the location of the image to be used
      * to decorate the graphical submit button. See the <a
      * href="http://www.w3.org/TR/REC-html40/interact/forms.html#adef-src">
@@ -543,7 +545,7 @@ public:
     DOMString src() const;
 
     /**
-     * see @ref src
+     * see src
      */
     void setSrc( const DOMString & );
 
@@ -557,7 +559,7 @@ public:
     long tabIndex() const;
 
     /**
-     * see @ref tabIndex
+     * see tabIndex
      */
     void setTabIndex( long );
 
@@ -570,7 +572,7 @@ public:
     DOMString type() const;
 
     /**
-     * see @ref type
+     * see type
      */
     void setType(const DOMString&);
 
@@ -583,7 +585,7 @@ public:
     DOMString useMap() const;
 
     /**
-     * see @ref useMap
+     * see useMap
      */
     void setUseMap( const DOMString & );
 
@@ -597,7 +599,7 @@ public:
     DOMString value() const;
 
     /**
-     * see @ref value
+     * see value
      */
     void setValue( const DOMString & );
 
@@ -614,16 +616,16 @@ public:
     void focus (  );
 
     /**
-     * Select the contents of the text area. For <code> INPUT </code>
-     * elements whose <code> type </code> attribute has one of the
+     * Select the contents of the text area. For \c INPUT
+     * elements whose \c type attribute has one of the
      * following values: "Text", "File", or "Password".
      *
      */
     void select (  );
 
     /**
-     * Simulate a mouse-click. For <code> INPUT </code> elements whose
-     * <code> type </code> attribute has one of the following values:
+     * Simulate a mouse-click. For \c INPUT elements whose
+     * \c type attribute has one of the following values:
      * "Button", "Checkbox", "Radio", "Reset", or "Submit".
      */
     void click (  );
@@ -664,13 +666,13 @@ public:
     DOMString accessKey() const;
 
     /**
-     * see @ref accessKey
+     * see accessKey
      */
     void setAccessKey( const DOMString & );
 
     /**
      * This attribute links this label with another form control by
-     * <code> id </code> attribute. See the <a
+     * \c id attribute. See the <a
      * href="http://www.w3.org/TR/REC-html40/interact/forms.html#adef-for">
      * for attribute definition </a> in HTML 4.0.
      *
@@ -678,7 +680,7 @@ public:
     DOMString htmlFor() const;
 
     /**
-     * see @ref htmlFor
+     * see htmlFor
      */
     void setHtmlFor( const DOMString & );
 };
@@ -687,7 +689,7 @@ public:
 
 class HTMLLegendElementImpl;
 /**
- * Provides a caption for a <code> FIELDSET </code> grouping. See the
+ * Provides a caption for a \c FIELDSET grouping. See the
  * <a
  * href="http://www.w3.org/TR/REC-html40/interact/forms.html#edef-LEGEND">
  * LEGEND element definition </a> in HTML 4.0.
@@ -710,7 +712,7 @@ public:
     ~HTMLLegendElement();
 
     /**
-     * @deprecated. ### KDE 4.0: remove
+     * ### KDE 4.0: remove
      */
     HTMLFormElement form() const;
 
@@ -724,12 +726,12 @@ public:
     DOMString accessKey() const;
 
     /**
-     * see @ref accessKey
+     * see accessKey
      */
     void setAccessKey( const DOMString & );
 
     /**
-     * Text alignment relative to <code> FIELDSET </code> . See the <a
+     * Text alignment relative to \c FIELDSET . See the <a
      * href="http://www.w3.org/TR/REC-html40/interact/forms.html#adef-align-LEGEND">
      * align attribute definition </a> in HTML 4.0. This attribute is
      * deprecated in HTML 4.0.
@@ -738,7 +740,7 @@ public:
     DOMString align() const;
 
     /**
-     * see @ref align
+     * see align
      */
     void setAlign( const DOMString & );
 };
@@ -777,7 +779,7 @@ public:
     bool disabled() const;
 
     /**
-     * see @ref disabled
+     * see disabled
      */
     void setDisabled( bool );
 
@@ -790,7 +792,7 @@ public:
     DOMString label() const;
 
     /**
-     * see @ref label
+     * see label
      */
     void setLabel( const DOMString & );
 };
@@ -837,7 +839,7 @@ public:
     long selectedIndex() const;
 
     /**
-     * see @ref selectedIndex
+     * see selectedIndex
      */
     void setSelectedIndex( long );
 
@@ -848,23 +850,23 @@ public:
     DOMString value() const;
 
     /**
-     * see @ref value
+     * see value
      */
     void setValue( const DOMString & );
 
     /**
-     * The number of options in this <code> SELECT </code> .
+     * The number of options in this \c SELECT .
      *
      */
     long length() const;
 
     /**
-     * @deprecated. ### KDE 4.0: remove.
+     * ### KDE 4.0: remove.
      */
     HTMLFormElement form() const;
 
     /**
-     * The collection of <code> OPTION </code> elements contained by
+     * The collection of \c OPTION elements contained by
      * this element.
      *
      */
@@ -879,13 +881,13 @@ public:
     bool disabled() const;
 
     /**
-     * see @ref disabled
+     * see disabled
      */
     void setDisabled( bool );
 
     /**
-     * If true, multiple <code> OPTION </code> elements may be
-     * selected in this <code> SELECT </code> . See the <a
+     * If true, multiple \c OPTION elements may be
+     * selected in this \c SELECT . See the <a
      * href="http://www.w3.org/TR/REC-html40/interact/forms.html#adef-multiple">
      * multiple attribute definition </a> in HTML 4.0.
      *
@@ -893,7 +895,7 @@ public:
     bool multiple() const;
 
     /**
-     * see @ref multiple
+     * see multiple
      */
     void setMultiple( bool );
 
@@ -907,7 +909,7 @@ public:
     DOMString name() const;
 
     /**
-     * see @ref name
+     * see name
      */
     void setName( const DOMString & );
 
@@ -920,7 +922,7 @@ public:
     long size() const;
 
     /**
-     * see @ref size
+     * see size
      */
     void setSize( long );
 
@@ -934,13 +936,13 @@ public:
     long tabIndex() const;
 
     /**
-     * see @ref tabIndex
+     * see tabIndex
      */
     void setTabIndex( long );
 
     /**
-     * Add a new element to the collection of <code> OPTION </code>
-     * elements for this <code> SELECT </code> .
+     * Add a new element to the collection of \c OPTION
+     * elements for this \c SELECT .
      *
      * @param element The element to add.
      *
@@ -951,8 +953,8 @@ public:
     void add ( const HTMLElement &element, const HTMLElement &before );
 
     /**
-     * Remove an element from the collection of <code> OPTION </code>
-     * elements for this <code> SELECT </code> . Does nothing if no
+     * Remove an element from the collection of \c OPTION
+     * elements for this \c SELECT . Does nothing if no
      * element has the given index.
      *
      * @param index The index of the item to remove.
@@ -1000,18 +1002,18 @@ public:
 
     /**
      * Stores the initial control value (i.e., the initial value of
-     * <code> value </code> ).
+     * \c value ).
      *
      */
     DOMString defaultValue() const;
 
     /**
-     * see @ref defaultValue
+     * see defaultValue
      */
     void setDefaultValue( const DOMString & );
 
     /**
-     * @deprecated. ### KDE 4.0: remove.
+     * ### KDE 4.0: remove.
      */
     HTMLFormElement form() const;
 
@@ -1025,7 +1027,7 @@ public:
     DOMString accessKey() const;
 
     /**
-     * see @ref accessKey
+     * see accessKey
      */
     void setAccessKey( const DOMString & );
 
@@ -1038,7 +1040,7 @@ public:
     long cols() const;
 
     /**
-     * see @ref cols
+     * see cols
      */
     void setCols( long );
 
@@ -1051,7 +1053,7 @@ public:
     bool disabled() const;
 
     /**
-     * see @ref disabled
+     * see disabled
      */
     void setDisabled( bool );
 
@@ -1065,7 +1067,7 @@ public:
     DOMString name() const;
 
     /**
-     * see @ref name
+     * see name
      */
     void setName( const DOMString & );
 
@@ -1078,7 +1080,7 @@ public:
     bool readOnly() const;
 
     /**
-     * see @ref readOnly
+     * see readOnly
      */
     void setReadOnly( bool );
 
@@ -1091,7 +1093,7 @@ public:
     long rows() const;
 
     /**
-     * see @ref rows
+     * see rows
      */
     void setRows( long );
 
@@ -1105,7 +1107,7 @@ public:
     long tabIndex() const;
 
     /**
-     * see @ref tabIndex
+     * see tabIndex
      */
     void setTabIndex( long );
 
@@ -1124,7 +1126,7 @@ public:
     DOMString value() const;
 
     /**
-     * see @ref value
+     * see value
      */
     void setValue( const DOMString & );
 
@@ -1139,7 +1141,7 @@ public:
     void focus (  );
 
     /**
-     * Select the contents of the <code> TEXTAREA </code> .
+     * Select the contents of the \c TEXTAREA .
      */
     void select (  );
 };
@@ -1170,19 +1172,19 @@ public:
     ~HTMLOptionElement();
 
     /**
-     * @deprecated. ### KDE 4.0: remove
+     * ### KDE 4.0: remove
      */
     HTMLFormElement form() const;
 
     /**
-     * Stores the initial value of the <code> selected </code>
+     * Stores the initial value of the \c selected
      * attribute.
      *
      */
     bool defaultSelected() const;
 
     /**
-     * see @ref defaultSelected
+     * see defaultSelected
      */
     void setDefaultSelected( bool );
 
@@ -1193,14 +1195,14 @@ public:
     DOMString text() const;
 
     /**
-     * The index of this <code> OPTION </code> in its parent <code>
-     * SELECT </code> .
+     * The index of this \c OPTION in its parent
+     * \c SELECT .
      *
      */
     long index() const;
 
     /**
-     * see @ref index
+     * see index
      *
      * This function is obsolete - the index property is actually supposed to be read-only
      * (http://www.w3.org/DOM/updates/REC-DOM-Level-1-19981001-errata.html)
@@ -1216,7 +1218,7 @@ public:
     bool disabled() const;
 
     /**
-     * see @ref disabled
+     * see disabled
      */
     void setDisabled( bool );
 
@@ -1229,7 +1231,7 @@ public:
     DOMString label() const;
 
     /**
-     * see @ref label
+     * see label
      */
     void setLabel( const DOMString & );
 
@@ -1242,7 +1244,7 @@ public:
     bool selected() const;
 
     /**
-     * see @ref selected
+     * see selected
      */
     void setSelected( bool );
 
@@ -1255,7 +1257,7 @@ public:
     DOMString value() const;
 
     /**
-     * see @ref value
+     * see value
      */
     void setValue( const DOMString & );
 };
@@ -1290,7 +1292,7 @@ public:
     ~HTMLIsIndexElement();
 
     /**
-     * @deprecated: ### KDE 4.0: remove
+     * ### KDE 4.0: remove
      */
     HTMLFormElement form() const;
 
@@ -1304,7 +1306,7 @@ public:
     DOMString prompt() const;
 
     /**
-     * see @ref prompt
+     * see prompt
      */
     void setPrompt( const DOMString & );
 };
