@@ -18,7 +18,6 @@
  * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
  *
- * $Id: html_image.cpp,v 1.16.2.2 2003/01/23 03:56:03 mueller Exp $
  */
 // --------------------------------------------------------------------------
 
@@ -98,7 +97,7 @@ DOMString HTMLAreaElement::href() const
 {
     if(!impl) return DOMString();
     DOMString href = static_cast<ElementImpl*>(impl)->getAttribute(ATTR_HREF);
-    return href.length() ? impl->getDocument()->completeURL(href.string()) : href;
+    return !href.isNull() ? impl->getDocument()->completeURL(href.string()) : href;
 }
 
 void HTMLAreaElement::setHref( const DOMString &value )
@@ -301,10 +300,7 @@ DOMString HTMLImageElement::src() const
 {
     if(!impl) return DOMString();
     DOMString s = ((ElementImpl *)impl)->getAttribute(ATTR_SRC);
-    // ### not sure if we're supposed to do the completion
-    if ( !s.isEmpty() )
-        s = ownerDocument().completeURL( s );
-    return s;
+    return !s.isNull() ? impl->getDocument()->completeURL( s.string() ) : s;
 }
 
 void HTMLImageElement::setSrc( const DOMString &value )
@@ -346,6 +342,28 @@ void HTMLImageElement::setWidth( long value )
     if(impl) ((ElementImpl *)impl)->setAttribute(ATTR_WIDTH, QString::number(value));
 }
 
+long HTMLImageElement::x() const
+{
+    if (impl && impl->renderer()) {
+        int x = 0;
+        int y = 0;
+        impl->renderer()->absolutePosition(x,y);
+        return x;
+    }
+    return 0;
+}
+
+long HTMLImageElement::y() const
+{
+    if (impl && impl->renderer()) {
+        int x = 0;
+        int y = 0;
+        impl->renderer()->absolutePosition(x,y);
+        return y;
+    }
+    return 0;
+}
+
 // --------------------------------------------------------------------------
 
 HTMLMapElement::HTMLMapElement() : HTMLElement()
