@@ -25,13 +25,14 @@
  * Technology , Institut National de Recherche en Informatique et en
  * Automatique , Keio University ). All Rights Reserved.
  *
- * $Id: html_image.h,v 1.10.4.2 2003/05/18 12:34:37 mueller Exp $
  */
 #ifndef HTML_IMAGE_H
 #define HTML_IMAGE_H
 
 #include <dom/html_element.h>
 
+#include <kdemacros.h>
+
 namespace DOM {
 
 class HTMLAreaElementImpl;
@@ -69,7 +70,7 @@ public:
     DOMString accessKey() const;
 
     /**
-     * see @ref accessKey
+     * see accessKey
      */
     void setAccessKey( const DOMString & );
 
@@ -83,13 +84,13 @@ public:
     DOMString alt() const;
 
     /**
-     * see @ref alt
+     * see alt
      */
     void setAlt( const DOMString & );
 
     /**
      * Comma-separated list of lengths, defining an active region
-     * geometry. See also <code> shape </code> for the shape of the
+     * geometry. See also \c shape for the shape of the
      * region. See the <a
      * href="http://www.w3.org/TR/REC-html40/struct/objects.html#adef-coords">
      * coords attribute definition </a> in HTML 4.0.
@@ -98,7 +99,7 @@ public:
     DOMString coords() const;
 
     /**
-     * see @ref coords
+     * see coords
      */
     void setCoords( const DOMString & );
 
@@ -111,7 +112,7 @@ public:
     DOMString href() const;
 
     /**
-     * see @ref href
+     * see href
      */
     void setHref( const DOMString & );
 
@@ -125,13 +126,13 @@ public:
     bool noHref() const;
 
     /**
-     * see @ref noHref
+     * see noHref
      */
     void setNoHref( bool );
 
     /**
      * The shape of the active area. The coordinates are given by
-     * <code> coords </code> . See the <a
+     * \c coords . See the <a
      * href="http://www.w3.org/TR/REC-html40/struct/objects.html#adef-shape">
      * shape attribute definition </a> in HTML 4.0.
      *
@@ -139,7 +140,7 @@ public:
     DOMString shape() const;
 
     /**
-     * see @ref shape
+     * see shape
      */
     void setShape( const DOMString & );
 
@@ -153,7 +154,7 @@ public:
     long tabIndex() const;
 
     /**
-     * see @ref tabIndex
+     * see tabIndex
      */
     void setTabIndex( long );
 
@@ -166,7 +167,7 @@ public:
     DOMString target() const;
 
     /**
-     * see @ref target
+     * see target
      */
     void setTarget( const DOMString & );
 };
@@ -204,7 +205,7 @@ public:
     DOMString name() const;
 
     /**
-     * see @ref name
+     * see name
      */
     void setName( const DOMString & );
 
@@ -219,7 +220,7 @@ public:
     DOMString align() const;
 
     /**
-     * see @ref align
+     * see align
      */
     void setAlign( const DOMString & );
 
@@ -233,7 +234,7 @@ public:
     DOMString alt() const;
 
     /**
-     * see @ref alt
+     * see alt
      */
     void setAlt( const DOMString & );
 
@@ -247,7 +248,7 @@ public:
     DOMString getBorder() const;
 
      /**
-      * see @ref border
+      * see border
       */
     void setBorder( const DOMString& );
 
@@ -255,12 +256,12 @@ public:
     /**
      * @deprecated
      */
-    long border() const;
+    long border() const KDE_DEPRECATED;
 
     /**
      * @deprecated
      */
-    void setBorder( long );
+    void setBorder( long ) KDE_DEPRECATED;
 
     /**
      * Override height. See the <a
@@ -271,7 +272,7 @@ public:
     long height() const;
 
     /**
-     * see @ref height
+     * see height
      */
     void setHeight( long );
 
@@ -286,7 +287,7 @@ public:
     long hspace() const;
 
     /**
-     * see @ref hspace
+     * see hspace
      */
     void setHspace( long );
 
@@ -299,7 +300,7 @@ public:
     bool isMap() const;
 
     /**
-     * see @ref isMap
+     * see isMap
      */
     void setIsMap( bool );
 
@@ -313,7 +314,7 @@ public:
     DOMString longDesc() const;
 
     /**
-     * see @ref longDesc
+     * see longDesc
      */
     void setLongDesc( const DOMString & );
 
@@ -326,7 +327,7 @@ public:
     DOMString src() const;
 
     /**
-     * see @ref src
+     * see src
      */
     void setSrc( const DOMString & );
 
@@ -339,7 +340,7 @@ public:
     DOMString useMap() const;
 
     /**
-     * see @ref useMap
+     * see useMap
      */
     void setUseMap( const DOMString & );
 
@@ -353,7 +354,7 @@ public:
     long vspace() const;
 
     /**
-     * see @ref vspace
+     * see vspace
      */
     void setVspace( long );
 
@@ -366,10 +367,15 @@ public:
     long width() const;
 
     /**
-     * see @ref width
+     * see width
      */
     void setWidth( long );
 
+    /**
+     * Nonstandard extension to DOM::ImgElement
+     */
+    long x() const;
+    long y() const;
 };
 
 // --------------------------------------------------------------------------
@@ -407,7 +413,7 @@ public:
     HTMLCollection areas() const;
 
     /**
-     * Names the map (for use with <code> usemap </code> ). See the <a
+     * Names the map (for use with \c usemap ). See the <a
      * href="http://www.w3.org/TR/REC-html40/struct/objects.html#adef-name-MAP">
      * name attribute definition </a> in HTML 4.0.
      *
@@ -415,7 +421,7 @@ public:
     DOMString name() const;
 
     /**
-     * see @ref name
+     * see name
      */
     void setName( const DOMString & );
 };
