@@ -3,6 +3,7 @@
  *  This file is part of the KDE libraries
  *  Copyright (C) 1999-2003 Harri Porten (porten@kde.org)
  *  Copyright (C) 2001-2003 David Faure (faure@kde.org)
+ *  Copyright (C) 2003 Apple Computer, Inc.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Library General Public
@@ -40,7 +41,7 @@ using namespace KJS;
  * these may be CSS exceptions - need to check - pmk
  */
 
-Value DOMObject::get(ExecState *exec, const UString &p) const
+Value DOMObject::get(ExecState *exec, const Identifier &p) const
 {
   Value result;
   try {
@@ -62,7 +63,7 @@ Value DOMObject::get(ExecState *exec, const UString &p) const
   return result;
 }
 
-void DOMObject::put(ExecState *exec, const UString &propertyName,
+void DOMObject::put(ExecState *exec, const Identifier &propertyName,
                     const Value &value, int attr)
 {
   try {
@@ -82,7 +83,7 @@ UString DOMObject::toString(ExecState *) const
   return "[object " + className() + "]";
 }
 
-Value DOMFunction::get(ExecState *exec, const UString &propertyName) const
+Value DOMFunction::get(ExecState *exec, const Identifier &propertyName) const
 {
   Value result;
   try {
@@ -108,6 +109,8 @@ Value DOMFunction::call(ExecState *exec, Object &thisObj, const List &args)
     val = tryCall(exec, thisObj, args);
   }
   // pity there's no way to distinguish between these in JS code
+  // ### Look into setting prototypes of these & the use of instanceof so the exception
+  // type can be determined. See what other browsers do.
   catch (DOM::DOMException e) {
     Object err = Error::create(exec, GeneralError, QString("DOM Exception %1").arg(e.code).local8Bit());
     err.put(exec, "code", Number(e.code));
@@ -179,7 +182,7 @@ void ScriptInterpreter::mark()
 {
   Interpreter::mark();
 #ifdef KJS_VERBOSE
-  kdDebug(6070) << "ScriptInterpreter::mark marking " << m_domObjects.count() << " DOM objects" << endl;
+  kdDebug(6070) << "ScriptInterpreter::mark " << this << " marking " << m_domObjects.count() << " DOM objects" << endl;
 #endif
   QPtrDictIterator<DOMObject> it( m_domObjects );
   for( ; it.current(); ++it )
@@ -196,8 +199,8 @@ bool ScriptInterpreter::isWindowOpenAllowed() const
       id == DOM::EventImpl::MOUSEUP_EVENT || id == DOM::EventImpl::MOUSEDOWN_EVENT ||
       id == DOM::EventImpl::KHTML_ECMA_CLICK_EVENT || id == DOM::EventImpl::KHTML_ECMA_DBLCLICK_EVENT ||
       // keyboard events
-      id == DOM::EventImpl::KHTML_KEYDOWN_EVENT || id == DOM::EventImpl::KHTML_KEYPRESS_EVENT ||
-      id == DOM::EventImpl::KHTML_KEYUP_EVENT ||
+      id == DOM::EventImpl::KEYDOWN_EVENT || id == DOM::EventImpl::KHTML_KEYPRESS_EVENT ||
+      id == DOM::EventImpl::KEYUP_EVENT ||
       // other accepted events
       id == DOM::EventImpl::SELECT_EVENT || id == DOM::EventImpl::CHANGE_EVENT ||
       id == DOM::EventImpl::SUBMIT_EVENT );
@@ -218,7 +221,6 @@ bool ScriptInterpreter::isWindowOpenAllowed() const
   return false;
 }
 
-//////
 
 UString::UString(const QString &d)
 {
@@ -256,6 +258,16 @@ QConstString UString::qconststring() const
   return QConstString((QChar*) data(), size());
 }
 
+DOM::DOMString Identifier::string() const
+{
+  return DOM::DOMString((QChar*) data(), size());
+}
+
+QString Identifier::qstring() const
+{
+  return QString((QChar*) data(), size());
+}
+
 DOM::Node KJS::toNode(const Value& val)
 {
   Object obj = Object::dynamicCast(val);
