@@ -2,6 +2,7 @@
 /*
  *  This file is part of the KDE libraries
  *  Copyright (C) 1999-2001 Harri Porten (porten@kde.org)
+ *  Copyright (C) 2003 Apple Computer, Inc.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Library General Public
@@ -22,6 +23,8 @@
 #define _KJS_BINDING_H_
 
 #include <kjs/interpreter.h>
+#include <kjs/function_object.h> /// for FunctionPrototypeImp
+
 #include <dom/dom_node.h>
 #include <qvariant.h>
 #include <qptrdict.h>
@@ -40,13 +43,13 @@ namespace KJS {
   class DOMObject : public ObjectImp {
   public:
     DOMObject(const Object &proto) : ObjectImp(proto) {}
-    virtual Value get(ExecState *exec, const UString &propertyName) const;
-    virtual Value tryGet(ExecState *exec, const UString &propertyName) const
+    virtual Value get(ExecState *exec, const Identifier &propertyName) const;
+    virtual Value tryGet(ExecState *exec, const Identifier &propertyName) const
       { return ObjectImp::get(exec, propertyName); }
 
-    virtual void put(ExecState *exec, const UString &propertyName,
+    virtual void put(ExecState *exec, const Identifier &propertyName,
                      const Value &value, int attr = None);
-    virtual void tryPut(ExecState *exec, const UString &propertyName,
+    virtual void tryPut(ExecState *exec, const Identifier &propertyName,
                         const Value& value, int attr = None)
       { ObjectImp::put(exec,propertyName,value,attr); }
 
@@ -58,11 +61,13 @@ namespace KJS {
    * tryGet() and tryCall() respectively, and catch exceptions if they
    * occur.
    */
-  class DOMFunction : public ObjectImp {
+  class DOMFunction : public InternalFunctionImp {
   public:
-    DOMFunction() : ObjectImp( /* proto? */ ) {}
-    virtual Value get(ExecState *exec, const UString &propertyName) const;
-    virtual Value tryGet(ExecState *exec, const UString &propertyName) const
+    DOMFunction(ExecState* exec) : InternalFunctionImp(
+      static_cast<FunctionPrototypeImp*>(exec->interpreter()->builtinFunctionPrototype().imp())
+      ) {}
+    virtual Value get(ExecState *exec, const Identifier &propertyName) const;
+    virtual Value tryGet(ExecState *exec, const Identifier &propertyName) const
       { return ObjectImp::get(exec, propertyName); }
 
     virtual bool implementsCall() const { return true; }
@@ -71,8 +76,6 @@ namespace KJS {
     virtual Value tryCall(ExecState *exec, Object &thisObj, const List&args)
       { return ObjectImp::call(exec, thisObj, args); }
     virtual bool toBoolean(ExecState *) const { return true; }
-    virtual Value toPrimitive(ExecState *exec, Type) const { return String(toString(exec)); }
-    virtual UString toString(ExecState *) const { return UString("[function]"); }
   };
 
   /**
@@ -167,7 +170,7 @@ namespace KJS {
    * we call tryGet instead of get, in DOMObjects.
    */
   template <class FuncImp, class ThisImp, class ParentImp>
-  inline Value DOMObjectLookupGet(ExecState *exec, const UString &propertyName,
+  inline Value DOMObjectLookupGet(ExecState *exec, const Identifier &propertyName,
                                   const HashTable* table, const ThisImp* thisObj)
   {
     const HashEntry* entry = Lookup::findEntry(table, propertyName);
@@ -175,8 +178,9 @@ namespace KJS {
     if (!entry) // not found, forward to parent
       return thisObj->ParentImp::tryGet(exec, propertyName);
 
-    if (entry->attr & Function)
+    if (entry->attr & Function) {
       return lookupOrCreateFunction<FuncImp>(exec, propertyName, thisObj, entry->value, entry->params, entry->attr);
+    }
     return thisObj->getValueProperty(exec, entry->value);
   }
 
@@ -185,7 +189,7 @@ namespace KJS {
    * functions, only "values".
    */
   template <class ThisImp, class ParentImp>
-  inline Value DOMObjectLookupGetValue(ExecState *exec, const UString &propertyName,
+  inline Value DOMObjectLookupGetValue(ExecState *exec, const Identifier &propertyName,
                                        const HashTable* table, const ThisImp* thisObj)
   {
     const HashEntry* entry = Lookup::findEntry(table, propertyName);
@@ -203,7 +207,7 @@ namespace KJS {
    * we call tryPut instead of put, in DOMObjects.
    */
   template <class ThisImp, class ParentImp>
-  inline void DOMObjectLookupPut(ExecState *exec, const UString &propertyName,
+  inline void DOMObjectLookupPut(ExecState *exec, const Identifier &propertyName,
                                  const Value& value, int attr,
                                  const HashTable* table, ThisImp* thisObj)
   {
@@ -215,7 +219,7 @@ namespace KJS {
       thisObj->ObjectImp::put(exec, propertyName, value, attr);
     else if (entry->attr & ReadOnly) // readonly! Can't put!
 #ifdef KJS_VERBOSE
-      fprintf(stderr,"Attempt to change value of readonly property '%s'\n",propertyName.ascii());
+      fprintf(stderr,"WARNING: Attempt to change value of readonly property '%s'\n",propertyName.ascii());
 #else
     ; // do nothing
 #endif
@@ -229,9 +233,9 @@ namespace KJS {
   class ClassFunc : public DOMFunction { \
   public: \
     ClassFunc(ExecState *exec, int i, int len) \
-       : DOMFunction( /*proto? */ ), id(i) { \
+       : DOMFunction( exec ), id(i) { \
        Value protect(this); \
-       put(exec,"length",Number(len),DontDelete|ReadOnly|DontEnum); \
+       put(exec,lengthPropertyName,Number(len),DontDelete|ReadOnly|DontEnum); \
     } \
     /** You need to implement that one */ \
     virtual Value tryCall(ExecState *exec, Object &thisObj, const List &args); \
