@@ -3,6 +3,7 @@
  *  This file is part of the KDE libraries
  *  Copyright (C) 2000 Harri Porten (porten@kde.org)
  *  Copyright (C) 2001 Peter Kelly (pmk@post.com)
+ *  Copyright (C) 2003 Apple Computer, Inc.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Library General Public
@@ -24,24 +25,36 @@
 
 #include <dom/html_head.h> // for HTMLStyleElement
 
-#include <css/cssparser.h>
+#include <css/css_base.h>
 #include "kjs_dom.h"
 
 using namespace KJS;
 #include <kdebug.h>
 
-static QString jsNameToProp( const UString &p )
+static QString cssPropertyName( const Identifier &p, bool& hadPixelPrefix )
 {
     QString prop = p.qstring();
     int i = prop.length();
-    while( --i ) {
-	char c = prop[i].latin1();
-	if ( c < 'A' || c > 'Z' )
-	    continue;
-	prop.insert( i, '-' );
+    while ( --i ) {
+        char c = prop[i].latin1();
+        if ( c >= 'A' && c <= 'Z' )
+            prop.insert( i, '-' );
     }
 
-    return prop.lower();
+    prop = prop.lower();
+    hadPixelPrefix = false;
+
+    if (prop.startsWith("css-")) {
+        prop = prop.mid(4);
+    } else if (prop.startsWith("pixel-")) {
+        prop = prop.mid(6);
+        hadPixelPrefix = true;
+    } else if (prop.startsWith("pos-")) {
+        prop = prop.mid(4);
+        hadPixelPrefix = true;
+    }
+
+    return prop;
 }
 
 /*
@@ -78,17 +91,17 @@ DOMCSSStyleDeclaration::~DOMCSSStyleDeclaration()
   ScriptInterpreter::forgetDOMObject(styleDecl.handle());
 }
 
-bool DOMCSSStyleDeclaration::hasProperty(ExecState *exec, const UString &p) const
+bool DOMCSSStyleDeclaration::hasProperty(ExecState *exec, const Identifier &p) const
 {
-  DOM::DOMString cssprop = jsNameToProp(p);
-  // strip pos- / pixel- prefix here?
-  if (DOM::getPropertyID(cssprop.string().ascii(), cssprop.length()))
+  bool hadPixelPrefix;
+  QString cssprop = cssPropertyName(p, hadPixelPrefix);
+  if (DOM::getPropertyID(cssprop.latin1(), cssprop.length()))
       return true;
 
   return ObjectImp::hasProperty(exec, p);
 }
 
-Value DOMCSSStyleDeclaration::tryGet(ExecState *exec, const UString &propertyName) const
+Value DOMCSSStyleDeclaration::tryGet(ExecState *exec, const Identifier &propertyName) const
 {
 #ifdef KJS_VERBOSE
   kdDebug(6070) << "DOMCSSStyleDeclaration::tryGet " << propertyName.qstring() << endl;
@@ -116,25 +129,19 @@ Value DOMCSSStyleDeclaration::tryGet(ExecState *exec, const UString &propertyNam
   if (ok)
     return getString(DOM::CSSStyleDeclaration(styleDecl).item(u));
 
-#ifdef KJS_VERBOSE
-  kdDebug(6070) << "DOMCSSStyleDeclaration: converting to css property name: " << jsNameToProp(propertyName) << endl;
-#endif
-  DOM::DOMString p = jsNameToProp(propertyName);
-  bool asNumber = false;
   // pixelTop returns "CSS Top" as number value in unit pixels
   // posTop returns "CSS top" as number value in unit pixels _if_ its a
   // positioned element. if it is not a positioned element, return 0
   // from MSIE documentation ### IMPLEMENT THAT (Dirk)
-  {
-    QString prop = p.string();
-    if(prop.startsWith( "pixel-") || prop.startsWith( "pos-" ) ) {
-      p = prop.mid(prop.find( '-' )+1);
-      asNumber = true;
-    }
-  }
+  bool asNumber;
+  QString p = cssPropertyName(propertyName, asNumber);
+
+#ifdef KJS_VERBOSE
+  kdDebug(6070) << "DOMCSSStyleDeclaration: converting to css property name: " << p << ( asNumber ? "px" : "" ) << endl;
+#endif
 
   if (asNumber) {
-    DOM::CSSValue v = const_cast<DOM::CSSStyleDeclaration &>( styleDecl ).getPropertyCSSValue(p);
+    DOM::CSSValue v = styleDecl.getPropertyCSSValue(p);
     if ( !v.isNull() && v.cssValueType() == DOM::CSSValue::CSS_PRIMITIVE_VALUE)
       return Number(static_cast<DOM::CSSPrimitiveValue>(v).getFloatValue(DOM::CSSPrimitiveValue::CSS_PX));
   }
@@ -144,18 +151,15 @@ Value DOMCSSStyleDeclaration::tryGet(ExecState *exec, const UString &propertyNam
     return String(str);
 
   // see if we know this css property, return empty then
-  QCString prop = p.string().latin1();
-  if (DOM::getPropertyID(prop.data(), prop.length()))
+  if (DOM::getPropertyID(p.latin1(), p.length()))
       return getString(DOM::DOMString(""));
 
   return DOMObject::tryGet(exec, propertyName);
 }
 
 
-void DOMCSSStyleDeclaration::tryPut(ExecState *exec, const UString &pName, const Value& value, int attr )
+void DOMCSSStyleDeclaration::tryPut(ExecState *exec, const Identifier &propertyName, const Value& value, int attr )
 {
-  UString propertyName = pName;
-
 #ifdef KJS_VERBOSE
   kdDebug(6070) << "DOMCSSStyleDeclaration::tryPut " << propertyName.qstring() << endl;
 #endif
@@ -163,32 +167,25 @@ void DOMCSSStyleDeclaration::tryPut(ExecState *exec, const UString &pName, const
     styleDecl.setCssText(value.toString(exec).string());
   }
   else {
-    QString prop = jsNameToProp(propertyName);
+    bool pxSuffix;
+    QString prop = cssPropertyName(propertyName, pxSuffix);
     QString propvalue = value.toString(exec).qstring();
 
-    if(prop.left(4) == "css-")
-      prop = prop.mid(4);
-
-    if(prop.startsWith( "pixel-") || prop.startsWith( "pos-" ) ) {
-      prop = prop.mid(prop.find( '-' )+1);
+    if (pxSuffix)
       propvalue += "px";
-    }
 #ifdef KJS_VERBOSE
     kdDebug(6070) << "DOMCSSStyleDeclaration: prop=" << prop << " propvalue=" << propvalue << endl;
 #endif
-    styleDecl.removeProperty(prop);
-    if(!propvalue.isEmpty())
-    {
-      // Look whether the property is known. In that case add it as a CSS property.
-      QCString cprop = prop.latin1();
-      if (DOM::getPropertyID(cprop.data(), cprop.length()))
-        styleDecl.setProperty(prop,DOM::DOMString(propvalue),""); // ### is "" ok for priority?
+    // Look whether the property is known.d In that case add it as a CSS property.
+    if (DOM::getPropertyID(prop.latin1(), prop.length())) {
+      if (propvalue.isEmpty())
+        styleDecl.removeProperty(prop);
       else
-      {
-        // otherwise add it as a JS property
-        DOMObject::tryPut( exec, pName, value, attr );
-      }
+        styleDecl.setProperty(prop,DOM::DOMString(propvalue),""); // ### is "" ok for priority?
     }
+    else
+      // otherwise add it as a JS property
+      DOMObject::tryPut( exec, propertyName, value, attr );
   }
 }
 
@@ -250,7 +247,7 @@ DOMStyleSheet::~DOMStyleSheet()
   ScriptInterpreter::forgetDOMObject(styleSheet.handle());
 }
 
-Value DOMStyleSheet::tryGet(ExecState *exec, const UString &propertyName) const
+Value DOMStyleSheet::tryGet(ExecState *exec, const Identifier &propertyName) const
 {
   return DOMObjectLookupGetValue<DOMStyleSheet,DOMObject>(exec,propertyName,&DOMStyleSheetTable,this);
 }
@@ -276,7 +273,7 @@ Value DOMStyleSheet::getValueProperty(ExecState *exec, int token) const
   return Value();
 }
 
-void DOMStyleSheet::tryPut(ExecState *exec, const UString &propertyName, const Value& value, int attr)
+void DOMStyleSheet::tryPut(ExecState *exec, const Identifier &propertyName, const Value& value, int attr)
 {
   if (propertyName == "disabled") {
     styleSheet.setDisabled(value.toBoolean(exec));
@@ -328,12 +325,12 @@ DOMStyleSheetList::~DOMStyleSheetList()
   ScriptInterpreter::forgetDOMObject(styleSheetList.handle());
 }
 
-Value DOMStyleSheetList::tryGet(ExecState *exec, const UString &p) const
+Value DOMStyleSheetList::tryGet(ExecState *exec, const Identifier &p) const
 {
 #ifdef KJS_VERBOSE
   kdDebug(6070) << "DOMStyleSheetList::tryGet " << p.qstring() << endl;
 #endif
-  if (p == "length")
+  if (p == lengthPropertyName)
     return Number(styleSheetList.length());
   else if (p == "item")
     return lookupOrCreateFunction<DOMStyleSheetListFunc>(exec,p,this,DOMStyleSheetList::Item,1,DontDelete|Function);
@@ -397,7 +394,7 @@ Value DOMStyleSheetList::tryCall(ExecState *exec, Object & /*thisObj*/, const Li
 {
   if (args.size() == 1) {
     // support for styleSheets(<index>) and styleSheets(<name>)
-    return tryGet( exec, args[0].toString(exec) );
+    return tryGet( exec, Identifier(args[0].toString(exec)) );
   }
   return Undefined();
 }
@@ -454,11 +451,11 @@ DOMMediaList::~DOMMediaList()
   ScriptInterpreter::forgetDOMObject(mediaList.handle());
 }
 
-Value DOMMediaList::tryGet(ExecState *exec, const UString &p) const
+Value DOMMediaList::tryGet(ExecState *exec, const Identifier &p) const
 {
   if (p == "mediaText")
     return getString(mediaList.mediaText());
-  else if (p == "length")
+  else if (p == lengthPropertyName)
     return Number(mediaList.length());
 
   bool ok;
@@ -469,7 +466,7 @@ Value DOMMediaList::tryGet(ExecState *exec, const UString &p) const
   return DOMObject::tryGet(exec, p);
 }
 
-void DOMMediaList::tryPut(ExecState *exec, const UString &propertyName, const Value& value, int attr)
+void DOMMediaList::tryPut(ExecState *exec, const Identifier &propertyName, const Value& value, int attr)
 {
   if (propertyName == "mediaText")
     mediaList.setMediaText(value.toString(exec).string());
@@ -530,7 +527,7 @@ DOMCSSStyleSheet::~DOMCSSStyleSheet()
 {
 }
 
-Value DOMCSSStyleSheet::tryGet(ExecState *exec, const UString &p) const
+Value DOMCSSStyleSheet::tryGet(ExecState *exec, const Identifier &p) const
 {
   DOM::CSSStyleSheet cssStyleSheet = static_cast<DOM::CSSStyleSheet>(styleSheet);
   if (p == "ownerRule")
@@ -587,10 +584,10 @@ DOMCSSRuleList::~DOMCSSRuleList()
   ScriptInterpreter::forgetDOMObject(cssRuleList.handle());
 }
 
-Value DOMCSSRuleList::tryGet(ExecState *exec, const UString &p) const
+Value DOMCSSRuleList::tryGet(ExecState *exec, const Identifier &p) const
 {
   Value result;
-  if (p == "length")
+  if (p == lengthPropertyName)
     return Number(cssRuleList.length());
   else if (p == "item")
     return lookupOrCreateFunction<DOMCSSRuleListFunc>(exec,p,this,DOMCSSRuleList::Item,1,DontDelete|Function);
@@ -695,7 +692,7 @@ const ClassInfo* DOMCSSRule::classInfo() const
   encoding		DOMCSSRule::Charset_Encoding	DontDelete
 @end
 */
-Value DOMCSSRule::tryGet(ExecState *exec, const UString &propertyName) const
+Value DOMCSSRule::tryGet(ExecState *exec, const Identifier &propertyName) const
 {
 #ifdef KJS_VERBOSE
   kdDebug(6070) << "DOMCSSRule::tryGet " << propertyName.qstring() << endl;
@@ -759,12 +756,12 @@ Value DOMCSSRule::getValueProperty(ExecState *exec, int token) const
     return getString(static_cast<DOM::CSSCharsetRule>(cssRule).encoding());
 
   default:
-    kdWarning() << "DOMCSSRule::getValueProperty unhandled token " << token << endl;
+    kdDebug(6070) << "WARNING: DOMCSSRule::getValueProperty unhandled token " << token << endl;
   }
   return Undefined();
 }
 
-void DOMCSSRule::tryPut(ExecState *exec, const UString &propertyName, const Value& value, int attr)
+void DOMCSSRule::tryPut(ExecState *exec, const Identifier &propertyName, const Value& value, int attr)
 {
   const HashTable* table = classInfo()->propHashTable; // get the right hashtable
   const HashEntry* entry = Lookup::findEntry(table, propertyName);
@@ -802,7 +799,7 @@ void DOMCSSRule::putValueProperty(ExecState *exec, int token, const Value& value
     return;
 
   default:
-    kdWarning() << "DOMCSSRule::putValueProperty unhandled token " << token << endl;
+    kdDebug(6070) << "WARNING: DOMCSSRule::putValueProperty unhandled token " << token << endl;
   }
 }
 
@@ -860,7 +857,7 @@ CSSRuleConstructor::CSSRuleConstructor(ExecState *exec)
 {
 }
 
-Value CSSRuleConstructor::tryGet(ExecState *exec, const UString &p) const
+Value CSSRuleConstructor::tryGet(ExecState *exec, const Identifier &p) const
 {
   return DOMObjectLookupGetValue<CSSRuleConstructor,DOMObject>(exec,p,&CSSRuleConstructorTable,this);
 }
@@ -912,7 +909,7 @@ DOMCSSValue::~DOMCSSValue()
   ScriptInterpreter::forgetDOMObject(cssValue.handle());
 }
 
-Value DOMCSSValue::tryGet(ExecState *exec, const UString &p) const
+Value DOMCSSValue::tryGet(ExecState *exec, const Identifier &p) const
 {
   if (p == "cssText")
     return getString(cssValue.cssText());
@@ -921,7 +918,7 @@ Value DOMCSSValue::tryGet(ExecState *exec, const UString &p) const
   return DOMObject::tryGet(exec,p);
 }
 
-void DOMCSSValue::tryPut(ExecState *exec, const UString &propertyName, const Value& value, int attr)
+void DOMCSSValue::tryPut(ExecState *exec, const Identifier &propertyName, const Value& value, int attr)
 {
   if (propertyName == "cssText")
     cssValue.setCssText(value.toString(exec).string());
@@ -966,7 +963,7 @@ CSSValueConstructor::CSSValueConstructor(ExecState *exec)
 {
 }
 
-Value CSSValueConstructor::tryGet(ExecState *exec, const UString &p) const
+Value CSSValueConstructor::tryGet(ExecState *exec, const Identifier &p) const
 {
   return DOMObjectLookupGetValue<CSSValueConstructor,DOMObject>(exec,p,&CSSValueConstructorTable,this);
 }
@@ -1015,7 +1012,7 @@ IMPLEMENT_PROTOTYPE(DOMCSSPrimitiveValueProto,DOMCSSPrimitiveValueProtoFunc)
 DOMCSSPrimitiveValue::DOMCSSPrimitiveValue(ExecState *exec, const DOM::CSSPrimitiveValue& v)
   : DOMCSSValue(DOMCSSPrimitiveValueProto::self(exec), v) { }
 
-Value DOMCSSPrimitiveValue::tryGet(ExecState *exec, const UString &p) const
+Value DOMCSSPrimitiveValue::tryGet(ExecState *exec, const Identifier &p) const
 {
   if (p=="primitiveType")
     return Number(static_cast<DOM::CSSPrimitiveValue>(cssValue).primitiveType());
@@ -1083,7 +1080,7 @@ const ClassInfo CSSPrimitiveValueConstructor::info = { "CSSPrimitiveValueConstru
 @end
 */
 
-Value CSSPrimitiveValueConstructor::tryGet(ExecState *exec, const UString &p) const
+Value CSSPrimitiveValueConstructor::tryGet(ExecState *exec, const Identifier &p) const
 {
   return DOMObjectLookupGetValue<CSSPrimitiveValueConstructor,CSSValueConstructor>(exec,p,&CSSPrimitiveValueConstructorTable,this);
 }
@@ -1114,12 +1111,12 @@ IMPLEMENT_PROTOFUNC_DOM(DOMCSSValueListFunc) // not really a proto, but doesn't
 DOMCSSValueList::DOMCSSValueList(ExecState *exec, const DOM::CSSValueList& v)
   : DOMCSSValue(exec, v) { }
 
-Value DOMCSSValueList::tryGet(ExecState *exec, const UString &p) const
+Value DOMCSSValueList::tryGet(ExecState *exec, const Identifier &p) const
 {
   Value result;
   DOM::CSSValueList valueList = static_cast<DOM::CSSValueList>(cssValue);
 
-  if (p == "length")
+  if (p == lengthPropertyName)
     return Number(valueList.length());
   else if (p == "item")
     return lookupOrCreateFunction<DOMCSSValueListFunc>(exec,p,this,DOMCSSValueList::Item,1,DontDelete|Function);
@@ -1166,7 +1163,7 @@ DOMRGBColor::~DOMRGBColor()
   //rgbColors.remove(rgbColor.handle());
 }
 
-Value DOMRGBColor::tryGet(ExecState *exec, const UString &p) const
+Value DOMRGBColor::tryGet(ExecState *exec, const Identifier &p) const
 {
   return DOMObjectLookupGetValue<DOMRGBColor,DOMObject>(exec, p,
 						       &DOMRGBColorTable,
@@ -1215,7 +1212,7 @@ DOMRect::~DOMRect()
   ScriptInterpreter::forgetDOMObject(rect.handle());
 }
 
-Value DOMRect::tryGet(ExecState *exec, const UString &p) const
+Value DOMRect::tryGet(ExecState *exec, const Identifier &p) const
 {
   return DOMObjectLookupGetValue<DOMRect,DOMObject>(exec, p,
 						    &DOMRectTable, this);
@@ -1247,9 +1244,9 @@ Value KJS::getDOMRect(ExecState *exec, const DOM::Rect& r)
 const ClassInfo DOMCounter::info = { "Counter", 0, &DOMCounterTable, 0 };
 /*
 @begin DOMCounterTable 3
-  identifier	DOMCounter::_Identifier	DontDelete|ReadOnly
-  listStyle	DOMCounter::ListStyle	DontDelete|ReadOnly
-  separator	DOMCounter::Separator	DontDelete|ReadOnly
+  identifier	DOMCounter::identifier	DontDelete|ReadOnly
+  listStyle	DOMCounter::listStyle	DontDelete|ReadOnly
+  separator	DOMCounter::separator	DontDelete|ReadOnly
 @end
 */
 DOMCounter::DOMCounter(ExecState *exec, const DOM::Counter& c)
@@ -1262,7 +1259,7 @@ DOMCounter::~DOMCounter()
   ScriptInterpreter::forgetDOMObject(counter.handle());
 }
 
-Value DOMCounter::tryGet(ExecState *exec, const UString &p) const
+Value DOMCounter::tryGet(ExecState *exec, const Identifier &p) const
 {
   return DOMObjectLookupGetValue<DOMCounter,DOMObject>(exec, p,
 						       &DOMCounterTable, this);
@@ -1271,11 +1268,11 @@ Value DOMCounter::tryGet(ExecState *exec, const UString &p) const
 Value DOMCounter::getValueProperty(ExecState *, int token) const
 {
   switch (token) {
-  case _Identifier:
+  case identifier:
     return getString(counter.identifier());
-  case ListStyle:
+  case listStyle:
     return getString(counter.listStyle());
-  case Separator:
+  case separator:
     return getString(counter.separator());
   default:
     return Value();
