@@ -2,6 +2,7 @@
 /*
  *  This file is part of the KDE libraries
  *  Copyright (C) 2000 Harri Porten (porten@kde.org)
+ *  Copyright (C) 2003 Apple Computer, Inc.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Library General Public
@@ -38,10 +39,10 @@ namespace KJS {
     DOMNode(const Object& proto, const DOM::Node& n);
     ~DOMNode();
     virtual bool toBoolean(ExecState *) const;
-    virtual Value tryGet(ExecState *exec, const UString &propertyName) const;
+    virtual Value tryGet(ExecState *exec, const Identifier &propertyName) const;
     Value getValueProperty(ExecState *exec, int token) const;
 
-    virtual void tryPut(ExecState *exec, const UString &propertyName, const Value& value, int attr = None);
+    virtual void tryPut(ExecState *exec, const Identifier &propertyName, const Value& value, int attr = None);
     void putValueProperty(ExecState *exec, int token, const Value& value, int attr);
     virtual DOM::Node toNode() const { return node; }
     virtual const ClassInfo* classInfo() const { return &info; }
@@ -51,10 +52,10 @@ namespace KJS {
     virtual UString toString(ExecState *exec) const;
     void setListener(ExecState *exec, int eventId, const Value& func) const;
     Value getListener(int eventId) const;
-    virtual List eventHandlerScope(ExecState *exec) const;
+    virtual void pushEventHandlerScope(ExecState *exec, ScopeChain &scope) const;
 
     enum { NodeName, NodeValue, NodeType, ParentNode, ParentElement,
-           ChildNodes, FirstChild, LastChild, PreviousSibling, NextSibling,
+           ChildNodes, FirstChild, LastChild, PreviousSibling, NextSibling, Item,
            Attributes, NamespaceURI, Prefix, LocalName, OwnerDocument, InsertBefore,
            ReplaceChild, RemoveChild, AppendChild, HasAttributes, HasChildNodes,
            CloneNode, Normalize, IsSupported, AddEventListener, RemoveEventListener,
@@ -64,7 +65,8 @@ namespace KJS {
            OnMouseMove, OnMouseOut, OnMouseOver, OnMouseUp, OnMove, OnReset,
            OnResize, OnSelect, OnSubmit, OnUnload,
            OffsetLeft, OffsetTop, OffsetWidth, OffsetHeight, OffsetParent,
-           ClientWidth, ClientHeight, ScrollLeft, ScrollTop, SourceIndex };
+           ClientWidth, ClientHeight, ScrollLeft, ScrollTop,
+	   ScrollWidth, ScrollHeight, SourceIndex };
 
   protected:
     DOM::Node node;
@@ -74,8 +76,8 @@ namespace KJS {
   public:
     DOMNodeList(ExecState *, const DOM::NodeList& l);
     ~DOMNodeList();
-    virtual bool hasProperty(ExecState *exec, const UString &p) const;
-    virtual Value tryGet(ExecState *exec, const UString &propertyName) const;
+    virtual bool hasProperty(ExecState *exec, const Identifier &p) const;
+    virtual Value tryGet(ExecState *exec, const Identifier &propertyName) const;
     virtual Value call(ExecState *exec, Object &thisObj, const List&args);
     virtual Value tryCall(ExecState *exec, Object &thisObj, const List&args);
     virtual bool implementsCall() const { return true; }
@@ -84,20 +86,11 @@ namespace KJS {
     virtual bool toBoolean(ExecState *) const { return true; }
     static const ClassInfo info;
     DOM::NodeList nodeList() const { return list; }
+    enum { Item, NamedItem };
   private:
     DOM::NodeList list;
   };
 
-  class DOMNodeListFunc : public DOMFunction {
-    friend class DOMNodeList;
-  public:
-    DOMNodeListFunc(ExecState *exec, int id, int len);
-    virtual Value tryCall(ExecState *exec, Object &thisObj, const List &);
-    enum { Item };
-  private:
-    int id;
-  };
-
   class DOMDocument : public DOMNode {
   public:
     // Build a DOMDocument
@@ -105,8 +98,10 @@ namespace KJS {
     // Constructor for inherited classes
     DOMDocument(const Object& proto, const DOM::Document& d);
     virtual ~DOMDocument();
-    virtual Value tryGet(ExecState *exec, const UString &propertyName) const;
+    virtual Value tryGet(ExecState *exec, const Identifier &propertyName) const;
     Value getValueProperty(ExecState *exec, int token) const;
+    virtual void tryPut(ExecState *exec, const Identifier &propertyName, const Value& value, int attr = None);
+    void putValueProperty(ExecState *exec, int token, const Value& value, int attr);
     virtual const ClassInfo* classInfo() const { return &info; }
     static const ClassInfo info;
     enum { DocType, Implementation, DocumentElement,
@@ -116,14 +111,15 @@ namespace KJS {
            CreateEntityReference, GetElementsByTagName, ImportNode, CreateElementNS,
            CreateAttributeNS, GetElementsByTagNameNS, GetElementById,
            CreateRange, CreateNodeIterator, CreateTreeWalker, DefaultView,
-           CreateEvent, StyleSheets, GetOverrideStyle, ReadyState };
+           CreateEvent, StyleSheets, GetOverrideStyle, Abort, Load, LoadXML,
+           PreferredStylesheetSet, SelectedStylesheetSet, ReadyState, Async };
   };
 
   class DOMAttr : public DOMNode {
   public:
     DOMAttr(ExecState *exec, const DOM::Attr& a) : DOMNode(exec, a) { }
-    virtual Value tryGet(ExecState *exec, const UString &propertyName) const;
-    virtual void tryPut(ExecState *exec, const UString &propertyName, const Value& value, int attr = None);
+    virtual Value tryGet(ExecState *exec, const Identifier &propertyName) const;
+    virtual void tryPut(ExecState *exec, const Identifier &propertyName, const Value& value, int attr = None);
     Value getValueProperty(ExecState *exec, int token) const;
     void putValueProperty(ExecState *exec, int token, const Value& value, int attr);
     virtual const ClassInfo* classInfo() const { return &info; }
@@ -137,7 +133,7 @@ namespace KJS {
     DOMElement(ExecState *exec, const DOM::Element& e);
     // Constructor for inherited classes
     DOMElement(const Object& proto, const DOM::Element& e);
-    virtual Value tryGet(ExecState *exec, const UString &propertyName) const;
+    virtual Value tryGet(ExecState *exec, const Identifier &propertyName) const;
     // no put - all read-only
     virtual const ClassInfo* classInfo() const { return &info; }
     static const ClassInfo info;
@@ -167,7 +163,7 @@ namespace KJS {
   public:
     // Build a DOMDocumentType
     DOMDocumentType(ExecState *exec, const DOM::DocumentType& dt);
-    virtual Value tryGet(ExecState *exec, const UString &propertyName) const;
+    virtual Value tryGet(ExecState *exec, const Identifier &propertyName) const;
     Value getValueProperty(ExecState *exec, int token) const;
     // no put - all read-only
     virtual const ClassInfo* classInfo() const { return &info; }
@@ -179,8 +175,8 @@ namespace KJS {
   public:
     DOMNamedNodeMap(ExecState *, const DOM::NamedNodeMap& m);
     ~DOMNamedNodeMap();
-    virtual bool hasProperty(ExecState *exec, const UString &p) const;
-    virtual Value tryGet(ExecState *exec, const UString &propertyName) const;
+    virtual bool hasProperty(ExecState *exec, const Identifier &p) const;
+    virtual Value tryGet(ExecState *exec, const Identifier &propertyName) const;
     // no put - all read-only
     virtual const ClassInfo* classInfo() const { return &info; }
     virtual bool toBoolean(ExecState *) const { return true; }
@@ -195,9 +191,9 @@ namespace KJS {
   class DOMProcessingInstruction : public DOMNode {
   public:
     DOMProcessingInstruction(ExecState *exec, const DOM::ProcessingInstruction& pi) : DOMNode(exec, pi) { }
-    virtual Value tryGet(ExecState *exec, const UString &propertyName) const;
+    virtual Value tryGet(ExecState *exec, const Identifier &propertyName) const;
     Value getValueProperty(ExecState *exec, int token) const;
-    virtual void tryPut(ExecState *exec, const UString &propertyName, const Value& value, int attr = None);
+    virtual void tryPut(ExecState *exec, const Identifier &propertyName, const Value& value, int attr = None);
     virtual const ClassInfo* classInfo() const { return &info; }
     static const ClassInfo info;
     enum { Target, Data, Sheet };
@@ -206,7 +202,7 @@ namespace KJS {
   class DOMNotation : public DOMNode {
   public:
     DOMNotation(ExecState *exec, const DOM::Notation& n) : DOMNode(exec, n) { }
-    virtual Value tryGet(ExecState *exec, const UString &propertyName) const;
+    virtual Value tryGet(ExecState *exec, const Identifier &propertyName) const;
     Value getValueProperty(ExecState *exec, int token) const;
     // no put - all read-only
     virtual const ClassInfo* classInfo() const { return &info; }
@@ -217,7 +213,7 @@ namespace KJS {
   class DOMEntity : public DOMNode {
   public:
     DOMEntity(ExecState *exec, const DOM::Entity& e) : DOMNode(exec, e) { }
-    virtual Value tryGet(ExecState *exec, const UString &propertyName) const;
+    virtual Value tryGet(ExecState *exec, const Identifier &propertyName) const;
     Value getValueProperty(ExecState *exec, int token) const;
     // no put - all read-only
     virtual const ClassInfo* classInfo() const { return &info; }
@@ -229,7 +225,7 @@ namespace KJS {
   class NodeConstructor : public DOMObject {
   public:
     NodeConstructor(ExecState *);
-    virtual Value tryGet(ExecState *exec, const UString &propertyName) const;
+    virtual Value tryGet(ExecState *exec, const Identifier &propertyName) const;
     Value getValueProperty(ExecState *exec, int token) const;
     // no put - all read-only
     virtual const ClassInfo* classInfo() const { return &info; }
@@ -240,7 +236,7 @@ namespace KJS {
   class DOMExceptionConstructor : public DOMObject {
   public:
     DOMExceptionConstructor(ExecState *);
-    virtual Value tryGet(ExecState *exec, const UString &propertyName) const;
+    virtual Value tryGet(ExecState *exec, const Identifier &propertyName) const;
     Value getValueProperty(ExecState *exec, int token) const;
     // no put - all read-only
     virtual const ClassInfo* classInfo() const { return &info; }
@@ -260,7 +256,7 @@ namespace KJS {
   class DOMNamedNodesCollection : public DOMObject {
   public:
     DOMNamedNodesCollection(ExecState *exec, const QValueList<DOM::Node>& nodes );
-    virtual Value tryGet(ExecState *exec, const UString &propertyName) const;
+    virtual Value tryGet(ExecState *exec, const Identifier &propertyName) const;
     virtual const ClassInfo* classInfo() const { return &info; }
     static const ClassInfo info;
     const QValueList<DOM::Node>& nodes() const { return m_nodes; }
@@ -274,9 +270,9 @@ namespace KJS {
     DOMCharacterData(ExecState *exec, const DOM::CharacterData& d);
     // Constructor for inherited classes
     DOMCharacterData(const Object& proto, const DOM::CharacterData& d);
-    virtual Value tryGet(ExecState *exec,const UString &propertyName) const;
+    virtual Value tryGet(ExecState *exec,const Identifier &propertyName) const;
     Value getValueProperty(ExecState *, int token) const;
-    virtual void tryPut(ExecState *exec, const UString &propertyName, const Value& value, int attr = None);
+    virtual void tryPut(ExecState *exec, const Identifier &propertyName, const Value& value, int attr = None);
     virtual const ClassInfo* classInfo() const { return &info; }
     static const ClassInfo info;
     DOM::CharacterData toData() const { return static_cast<DOM::CharacterData>(node); }
@@ -287,7 +283,7 @@ namespace KJS {
   class DOMText : public DOMCharacterData {
   public:
     DOMText(ExecState *exec, const DOM::Text& t);
-    virtual Value tryGet(ExecState *exec,const UString &propertyName) const;
+    virtual Value tryGet(ExecState *exec,const Identifier &propertyName) const;
     Value getValueProperty(ExecState *, int token) const;
     virtual const ClassInfo* classInfo() const { return &info; }
     static const ClassInfo info;
