@@ -2,6 +2,7 @@
 /*
  *  This file is part of the KDE libraries
  *  Copyright (C) 2001 Peter Kelly (pmk@post.com)
+ *  Copyright (C) 2003 Apple Computer, Inc.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Library General Public
@@ -27,10 +28,13 @@
 #include "xml/dom_docimpl.h"
 #include "xml/dom2_eventsimpl.h"
 #include "rendering/render_object.h"
+#include "xml/dom2_eventsimpl.h"
+#include "khtml_part.h"
 
 #include <kdebug.h>
 
 using namespace KJS;
+using namespace DOM;
 
 // -------------------------------------------------------------------------
 
@@ -38,27 +42,28 @@ JSEventListener::JSEventListener(Object _listener, const Object &_win, bool _htm
   : listener( _listener ), html( _html ), win( _win )
 {
     //fprintf(stderr,"JSEventListener::JSEventListener this=%p listener=%p\n",this,listener.imp());
-    static_cast<Window*>(win.imp())->jsEventListeners.append(this);
+    static_cast<Window*>(win.imp())->jsEventListeners.insert(this, this);
 }
 
 JSEventListener::~JSEventListener()
 {
-    static_cast<Window*>(win.imp())->jsEventListeners.removeRef(this);
+    static_cast<Window*>(win.imp())->jsEventListeners.remove(this);
     //fprintf(stderr,"JSEventListener::~JSEventListener this=%p listener=%p\n",this,listener.imp());
 }
 
 void JSEventListener::handleEvent(DOM::Event &evt)
 {
 #ifdef KJS_DEBUGGER
-  if (KJSDebugWin::instance() && KJSDebugWin::instance()->inSession())
+  if (KJSDebugWin::debugWindow() && KJSDebugWin::debugWindow()->inSession())
     return;
 #endif
   KHTMLPart *part = static_cast<Window*>(win.imp())->part();
   KJSProxy *proxy = 0L;
   if (part)
-      proxy = KJSProxy::proxy( part );
+    proxy = part->jScript();
 
-  if (proxy && listener.implementsCall()) {
+  Object listenerObj = Object::dynamicCast( listener );
+  if (proxy && listenerObj.implementsCall()) {
     ref();
 
     KJS::ScriptInterpreter *interpreter = static_cast<KJS::ScriptInterpreter *>(proxy->interpreter());
@@ -67,24 +72,26 @@ void JSEventListener::handleEvent(DOM::Event &evt)
     List args;
     args.append(getDOMEvent(exec,evt));
 
-    // Add the event's target element to the scope
-    // (and the document, and the form - see KJS::HTMLElement::eventHandlerScope)
+    // Set "this" to the event's current target
     Object thisObj = Object::dynamicCast(getDOMNode(exec,evt.currentTarget()));
-    List scope;
-    List oldScope = listener.scope();
+    ScopeChain oldScope = listenerObj.scope();
     if ( thisObj.isValid() ) {
-      scope = static_cast<DOMNode*>(thisObj.imp())->eventHandlerScope(exec);
-      if ( !scope.isEmpty() ) {
-        List curScope = oldScope.copy();
-        curScope.prependList( scope );
-        listener.setScope( curScope );
-      }
+      ScopeChain scope = oldScope;
+      // Add the event's target element to the scope
+      // (and the document, and the form - see KJS::HTMLElement::eventHandlerScope)
+      static_cast<DOMNode*>(thisObj.imp())->pushEventHandlerScope(exec, scope);
+      listenerObj.setScope( scope );
     }
     else {
-      if ( m_hackThisObj.isValid() )
+      if ( m_hackThisObj.isValid() ) { // special hack for Image
         thisObj = m_hackThisObj;
+      }
       else
-        kdWarning() << "Null 'this' object! evt=" << evt.type().string() << " currentTarget==" << evt.currentTarget().handle() << endl;
+      {
+        // Window events (window.onload/window.onresize etc.) must have 'this' set to the window.
+        // DocumentImpl::defaultEventHandler sets currentTarget to 0 to mean 'window'.
+        thisObj = win;
+      }
     }
 
     Window *window = static_cast<Window*>(win.imp());
@@ -95,22 +102,20 @@ void JSEventListener::handleEvent(DOM::Event &evt)
 
     KJSCPUGuard guard;
     guard.start();
-    Value retval = listener.call(exec, thisObj, args);
+    Value retval = listenerObj.call(exec, thisObj, args);
     guard.stop();
 
-    if ( !scope.isEmpty() ) {
-      listener.setScope( oldScope );
-    }
+    listenerObj.setScope( oldScope );
 
     window->setCurrentEvent( 0 );
     interpreter->setCurrentEvent( 0 );
     if ( exec->hadException() )
-        exec->clearException();
+      exec->clearException();
     else
     {
-        QVariant ret = ValueToVariant(exec, retval);
-        if (ret.type() == QVariant::Bool && ret.toBool() == false)
-            evt.preventDefault();
+      QVariant ret = ValueToVariant(exec, retval);
+      if (ret.type() == QVariant::Bool && ret.toBool() == false)
+        evt.preventDefault();
     }
     window->afterScriptExecution();
     deref();
@@ -125,15 +130,6 @@ DOM::DOMString JSEventListener::eventListenerType()
 	return "_khtml_JSEventListener";
 }
 
-Value KJS::getNodeEventListener(DOM::Node n, int eventId)
-{
-    DOM::EventListener *listener = n.handle()->getHTMLEventListener(eventId);
-    if (listener)
-	return static_cast<JSEventListener*>(listener)->listenerObj();
-    else
-	return Null();
-}
-
 // -------------------------------------------------------------------------
 
 const ClassInfo EventConstructor::info = { "EventConstructor", 0, &EventConstructorTable, 0 };
@@ -167,7 +163,7 @@ EventConstructor::EventConstructor(ExecState *exec)
 {
 }
 
-Value EventConstructor::tryGet(ExecState *exec, const UString &p) const
+Value EventConstructor::tryGet(ExecState *exec, const Identifier &p) const
 {
   return DOMObjectLookupGetValue<EventConstructor, DOMObject>(exec,p,&EventConstructorTable,this);
 }
@@ -212,12 +208,15 @@ IMPLEMENT_PROTOTYPE(DOMEventProto, DOMEventProtoFunc)
 DOMEvent::DOMEvent(ExecState *exec, DOM::Event e)
   : DOMObject(DOMEventProto::self(exec)), event(e) { }
 
+DOMEvent::DOMEvent(const Object &proto, DOM::Event e)
+  : DOMObject(proto), event(e) { }
+
 DOMEvent::~DOMEvent()
 {
   ScriptInterpreter::forgetDOMObject(event.handle());
 }
 
-Value DOMEvent::tryGet(ExecState *exec, const UString &p) const
+Value DOMEvent::tryGet(ExecState *exec, const Identifier &p) const
 {
 #ifdef KJS_VERBOSE
   kdDebug() << "KJS::DOMEvent::tryGet " << p.qstring() << endl;
@@ -248,12 +247,21 @@ Value DOMEvent::getValueProperty(ExecState *exec, int token) const
   case CancelBubble: // MSIE extension
     return Boolean(event.handle()->propagationStopped());
   default:
-    kdWarning() << "Unhandled token in DOMEvent::getValueProperty : " << token << endl;
+    kdDebug(6070) << "WARNING: Unhandled token in DOMEvent::getValueProperty : " << token << endl;
     return Value();
   }
 }
 
-void DOMEvent::tryPut(ExecState *exec, const UString &propertyName,
+Value DOMEvent::defaultValue(ExecState *exec, KJS::Type hint) const
+{
+  if (event.handle()->id() == EventImpl::ERROR_EVENT && !event.handle()->message().isNull()) {
+    return String(event.handle()->message());
+  }
+  else
+    return DOMObject::defaultValue(exec,hint);
+}
+
+void DOMEvent::tryPut(ExecState *exec, const Identifier &propertyName,
                       const Value& value, int attr)
 {
   DOMObjectLookupPut<DOMEvent, DOMObject>(exec, propertyName, value, attr,
@@ -283,6 +291,7 @@ Value DOMEventProtoFunc::tryCall(ExecState *exec, Object & thisObj, const List &
   switch (id) {
     case DOMEvent::StopPropagation:
       event.stopPropagation();
+      return Undefined();
     case DOMEvent::PreventDefault:
       event.preventDefault();
       return Undefined();
@@ -295,26 +304,26 @@ Value DOMEventProtoFunc::tryCall(ExecState *exec, Object & thisObj, const List &
 
 Value KJS::getDOMEvent(ExecState *exec, DOM::Event e)
 {
-  DOMObject *ret;
-  if (e.isNull())
+  DOM::EventImpl *ei = e.handle();
+  if (!ei)
     return Null();
   ScriptInterpreter* interp = static_cast<ScriptInterpreter *>(exec->interpreter());
-  if ((ret = interp->getDOMObject(e.handle())))
-    return Value(ret);
+  DOMObject *ret = interp->getDOMObject(ei);
+  if (!ret) {
+    if (ei->isTextEvent())
+      ret = new DOMTextEvent(exec, e);
+    else if (ei->isMouseEvent())
+      ret = new DOMMouseEvent(exec, e);
+    else if (ei->isUIEvent())
+      ret = new DOMUIEvent(exec, e);
+    else if (ei->isMutationEvent())
+      ret = new DOMMutationEvent(exec, e);
+    else
+      ret = new DOMEvent(exec, e);
 
-  DOM::DOMString module = e.eventModuleName();
-  if (module == "UIEvents")
-    ret = new DOMUIEvent(exec, static_cast<DOM::UIEvent>(e));
-  else if (module == "MouseEvents")
-    ret = new DOMMouseEvent(exec, static_cast<DOM::MouseEvent>(e));
-  else if (module == "TextEvents")
-    ret = new DOMTextEvent(exec, static_cast<DOM::TextEvent>(e));
-  else if (module == "MutationEvents")
-    ret = new DOMMutationEvent(exec, static_cast<DOM::MutationEvent>(e));
-  else
-    ret = new DOMEvent(exec, e);
+    interp->putDOMObject(ei, ret);
+  }
 
-  interp->putDOMObject(e.handle(),ret);
   return Value(ret);
 }
 
@@ -342,7 +351,7 @@ EventExceptionConstructor::EventExceptionConstructor(ExecState *exec)
 {
 }
 
-Value EventExceptionConstructor::tryGet(ExecState *exec, const UString &p) const
+Value EventExceptionConstructor::tryGet(ExecState *exec, const Identifier &p) const
 {
   return DOMObjectLookupGetValue<EventExceptionConstructor, DOMObject>(exec,p,&EventExceptionConstructorTable,this);
 }
@@ -362,9 +371,15 @@ Value KJS::getEventExceptionConstructor(ExecState *exec)
 
 const ClassInfo DOMUIEvent::info = { "UIEvent", &DOMEvent::info, &DOMUIEventTable, 0 };
 /*
-@begin DOMUIEventTable 2
+@begin DOMUIEventTable 7
   view		DOMUIEvent::View	DontDelete|ReadOnly
   detail	DOMUIEvent::Detail	DontDelete|ReadOnly
+  keyCode	DOMUIEvent::KeyCode	DontDelete|ReadOnly
+  layerX	DOMUIEvent::LayerX	DontDelete|ReadOnly
+  layerY	DOMUIEvent::LayerY	DontDelete|ReadOnly
+  pageX		DOMUIEvent::PageX	DontDelete|ReadOnly
+  pageY		DOMUIEvent::PageY	DontDelete|ReadOnly
+  which		DOMUIEvent::Which	DontDelete|ReadOnly
 @end
 @begin DOMUIEventProtoTable 1
   initUIEvent	DOMUIEvent::InitUIEvent	DontDelete|Function 5
@@ -374,11 +389,17 @@ DEFINE_PROTOTYPE("DOMUIEvent",DOMUIEventProto)
 IMPLEMENT_PROTOFUNC_DOM(DOMUIEventProtoFunc)
 IMPLEMENT_PROTOTYPE_WITH_PARENT(DOMUIEventProto,DOMUIEventProtoFunc,DOMEventProto)
 
+DOMUIEvent::DOMUIEvent(ExecState *exec, DOM::UIEvent ue) :
+  DOMEvent(DOMUIEventProto::self(exec), ue) {}
+
+DOMUIEvent::DOMUIEvent(const Object &proto, DOM::UIEvent ue) :
+  DOMEvent(proto, ue) {}
+
 DOMUIEvent::~DOMUIEvent()
 {
 }
 
-Value DOMUIEvent::tryGet(ExecState *exec, const UString &p) const
+Value DOMUIEvent::tryGet(ExecState *exec, const Identifier &p) const
 {
   return DOMObjectLookupGetValue<DOMUIEvent,DOMEvent>(exec,p,&DOMUIEventTable,this);
 }
@@ -390,9 +411,27 @@ Value DOMUIEvent::getValueProperty(ExecState *exec, int token) const
     return getDOMAbstractView(exec,static_cast<DOM::UIEvent>(event).view());
   case Detail:
     return Number(static_cast<DOM::UIEvent>(event).detail());
+  case KeyCode:
+    // IE-compatibility
+    return Number(static_cast<DOM::UIEvent>(event).keyCode());
+  case LayerX:
+    // NS-compatibility
+    return Number(static_cast<DOM::UIEvent>(event).layerX());
+  case LayerY:
+    // NS-compatibility
+    return Number(static_cast<DOM::UIEvent>(event).layerY());
+  case PageX:
+    // NS-compatibility
+    return Number(static_cast<DOM::UIEvent>(event).pageX());
+  case PageY:
+    // NS-compatibility
+    return Number(static_cast<DOM::UIEvent>(event).pageY());
+  case Which:
+    // NS-compatibility
+    return Number(static_cast<DOM::UIEvent>(event).which());
   default:
-    kdWarning() << "Unhandled token in DOMUIEvent::getValueProperty : " << token << endl;
-    return Value();
+    kdDebug(6070) << "WARNING: Unhandled token in DOMUIEvent::getValueProperty : " << token << endl;
+    return Undefined();
   }
 }
 
@@ -445,11 +484,14 @@ DEFINE_PROTOTYPE("DOMMouseEvent",DOMMouseEventProto)
 IMPLEMENT_PROTOFUNC_DOM(DOMMouseEventProtoFunc)
 IMPLEMENT_PROTOTYPE_WITH_PARENT(DOMMouseEventProto,DOMMouseEventProtoFunc,DOMUIEventProto)
 
+DOMMouseEvent::DOMMouseEvent(ExecState *exec, DOM::MouseEvent me) :
+  DOMUIEvent(DOMMouseEventProto::self(exec), me) {}
+
 DOMMouseEvent::~DOMMouseEvent()
 {
 }
 
-Value DOMMouseEvent::tryGet(ExecState *exec, const UString &p) const
+Value DOMMouseEvent::tryGet(ExecState *exec, const Identifier &p) const
 {
 #ifdef KJS_VERBOSE
   kdDebug(6070) << "DOMMouseEvent::tryGet " << p.qstring() << endl;
@@ -518,7 +560,7 @@ Value DOMMouseEvent::getValueProperty(ExecState *exec, int token) const
   case RelatedTarget:
     return getDOMNode(exec,static_cast<DOM::MouseEvent>(event).relatedTarget());
   default:
-    kdWarning() << "Unhandled token in DOMMouseEvent::getValueProperty : " << token << endl;
+    kdDebug(6070) << "WARNING: Unhandled token in DOMMouseEvent::getValueProperty : " << token << endl;
     return Value();
   }
 }
@@ -556,8 +598,6 @@ const ClassInfo DOMTextEvent::info = { "TextEvent", &DOMUIEvent::info, &DOMTextE
 /*
 @begin DOMTextEventTable 2
   keyVal   	 DOMTextEvent::Key	     DontDelete|ReadOnly
-# IE's name for it
-  keyCode   	 DOMTextEvent::Key	     DontDelete|ReadOnly
   virtKeyVal	 DOMTextEvent::VirtKey        DontDelete|ReadOnly
   outputString	 DOMTextEvent::OutputString   DontDelete|ReadOnly
   inputGenerated DOMTextEvent::InputGenerated DontDelete|ReadOnly
@@ -572,11 +612,14 @@ DEFINE_PROTOTYPE("DOMTextEvent",DOMTextEventProto)
 IMPLEMENT_PROTOFUNC_DOM(DOMTextEventProtoFunc)
 IMPLEMENT_PROTOTYPE_WITH_PARENT(DOMTextEventProto,DOMTextEventProtoFunc,DOMUIEventProto)
 
+DOMTextEvent::DOMTextEvent(ExecState *exec, DOM::TextEvent ke) :
+  DOMUIEvent(DOMTextEventProto::self(exec), ke) {}
+
 DOMTextEvent::~DOMTextEvent()
 {
 }
 
-Value DOMTextEvent::tryGet(ExecState *exec, const UString &p) const
+Value DOMTextEvent::tryGet(ExecState *exec, const Identifier &p) const
 {
 #ifdef KJS_VERBOSE
   kdDebug(6070) << "DOMTextEvent::tryGet " << p.qstring() << endl;
@@ -598,7 +641,7 @@ Value DOMTextEvent::getValueProperty(ExecState *, int token) const
   case NumPad:
     return Boolean(static_cast<DOM::TextEvent>(event).numPad());
   default:
-    kdWarning() << "Unhandled token in DOMTextEvent::getValueProperty : " << token << endl;
+    kdDebug(6070) << "WARNING: Unhandled token in DOMTextEvent::getValueProperty : " << token << endl;
     return Value();
   }
 }
@@ -640,7 +683,7 @@ MutationEventConstructor::MutationEventConstructor(ExecState* exec)
 {
 }
 
-Value MutationEventConstructor::tryGet(ExecState *exec, const UString &p) const
+Value MutationEventConstructor::tryGet(ExecState *exec, const Identifier &p) const
 {
   return DOMObjectLookupGetValue<MutationEventConstructor,DOMObject>(exec,p,&MutationEventConstructorTable,this);
 }
@@ -675,11 +718,14 @@ DEFINE_PROTOTYPE("DOMMutationEvent",DOMMutationEventProto)
 IMPLEMENT_PROTOFUNC_DOM(DOMMutationEventProtoFunc)
 IMPLEMENT_PROTOTYPE_WITH_PARENT(DOMMutationEventProto,DOMMutationEventProtoFunc,DOMEventProto)
 
+DOMMutationEvent::DOMMutationEvent(ExecState *exec, DOM::MutationEvent me) :
+  DOMEvent(DOMMutationEventProto::self(exec), me) {}
+
 DOMMutationEvent::~DOMMutationEvent()
 {
 }
 
-Value DOMMutationEvent::tryGet(ExecState *exec, const UString &p) const
+Value DOMMutationEvent::tryGet(ExecState *exec, const Identifier &p) const
 {
   return DOMObjectLookupGetValue<DOMMutationEvent,DOMEvent>(exec,p,&DOMMutationEventTable,this);
 }
@@ -698,7 +744,7 @@ Value DOMMutationEvent::getValueProperty(ExecState *exec, int token) const
   case AttrChange:
     return Number((unsigned int)static_cast<DOM::MutationEvent>(event).attrChange());
   default:
-    kdWarning() << "Unhandled token in DOMMutationEvent::getValueProperty : " << token << endl;
+    kdDebug(6070) << "WARNING: Unhandled token in DOMMutationEvent::getValueProperty : " << token << endl;
     return Value();
   }
 }
