@@ -2,6 +2,7 @@
 /*
  *  This file is part of the KDE libraries
  *  Copyright (C) 2001 Peter Kelly (pmk@post.com)
+ *  Copyright (C) 2003 Apple Computer, Inc.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Library General Public
@@ -43,25 +44,23 @@ namespace KJS {
     virtual void handleEvent(DOM::Event &evt);
     virtual DOM::DOMString eventListenerType();
     // Return the KJS function object executed when this event is emitted
-    Object listenerObj() const { return listener; }
-    // Faster version of listenerObj()
-    ObjectImp *listenerObjImp() const { return static_cast<ObjectImp *>(listener.imp()); }
-    // for Window::clear()
-    void clear() { listener = Object(); }
+    Value listenerObj() const { return listener; }
+    // for Window::clear(). This is a bad hack though. The JSEventListener might not get deleted
+    // if it was added to a DOM node in another frame (#61467). But calling removeEventListener on
+    // all nodes we're listening to is quite difficult.
+    void clear() { listener = Null(); }
 
   protected:
-    Object listener;
+    Value listener;
     bool html;
     Object win, m_hackThisObj;
   };
 
-  Value getNodeEventListener(DOM::Node n, int eventId);
-
   // Constructor for Event - currently only used for some global vars
   class EventConstructor : public DOMObject {
   public:
     EventConstructor(ExecState *);
-    virtual Value tryGet(ExecState *exec,const UString &p) const;
+    virtual Value tryGet(ExecState *exec,const Identifier &p) const;
     Value getValueProperty(ExecState *, int token) const;
     // no put - all read-only
     virtual const ClassInfo* classInfo() const { return &info; }
@@ -72,12 +71,16 @@ namespace KJS {
 
   class DOMEvent : public DOMObject {
   public:
+    // Build a DOMEvent
     DOMEvent(ExecState *exec, DOM::Event e);
+    // Constructor for inherited classes
+    DOMEvent(const Object &proto, DOM::Event e);
     ~DOMEvent();
-    virtual Value tryGet(ExecState *exec,const UString &p) const;
+    virtual Value tryGet(ExecState *exec,const Identifier &p) const;
     Value getValueProperty(ExecState *, int token) const;
-    virtual void tryPut(ExecState *exec, const UString &propertyName,
+    virtual void tryPut(ExecState *exec, const Identifier &propertyName,
 			const Value& value, int attr = None);
+    virtual Value defaultValue(ExecState *exec, KJS::Type hint) const;
     void putValueProperty(ExecState *exec, int token, const Value& value, int);
     virtual const ClassInfo* classInfo() const { return &info; }
     static const ClassInfo info;
@@ -101,7 +104,7 @@ namespace KJS {
   class EventExceptionConstructor : public DOMObject {
   public:
     EventExceptionConstructor(ExecState *);
-    virtual Value tryGet(ExecState *exec,const UString &p) const;
+    virtual Value tryGet(ExecState *exec,const Identifier &p) const;
     Value getValueProperty(ExecState *, int token) const;
     // no put - all read-only
     virtual const ClassInfo* classInfo() const { return &info; }
@@ -112,22 +115,25 @@ namespace KJS {
 
   class DOMUIEvent : public DOMEvent {
   public:
-    DOMUIEvent(ExecState *exec, DOM::UIEvent ue) : DOMEvent(exec, ue) {}
+    // Build a DOMUIEvent
+    DOMUIEvent(ExecState *exec, DOM::UIEvent ue);
+    // Constructor for inherited classes
+    DOMUIEvent(const Object &proto, DOM::UIEvent ue);
     ~DOMUIEvent();
-    virtual Value tryGet(ExecState *exec,const UString &p) const;
+    virtual Value tryGet(ExecState *exec,const Identifier &p) const;
     Value getValueProperty(ExecState *, int token) const;
     // no put - all read-only
     virtual const ClassInfo* classInfo() const { return &info; }
     static const ClassInfo info;
-    enum { View, Detail, InitUIEvent };
+    enum { View, Detail, KeyCode, LayerX, LayerY, PageX, PageY, Which, InitUIEvent };
     DOM::UIEvent toUIEvent() const { return static_cast<DOM::UIEvent>(event); }
   };
 
   class DOMMouseEvent : public DOMUIEvent {
   public:
-    DOMMouseEvent(ExecState *exec, DOM::MouseEvent me) : DOMUIEvent(exec, me) {}
+    DOMMouseEvent(ExecState *exec, DOM::MouseEvent me);
     ~DOMMouseEvent();
-    virtual Value tryGet(ExecState *exec,const UString &p) const;
+    virtual Value tryGet(ExecState *exec,const Identifier &p) const;
     Value getValueProperty(ExecState *, int token) const;
     // no put - all read-only
     virtual const ClassInfo* classInfo() const { return &info; }
@@ -135,15 +141,16 @@ namespace KJS {
     enum { ScreenX, ScreenY, ClientX, X, ClientY, Y, OffsetX, OffsetY,
            CtrlKey, ShiftKey, AltKey,
            MetaKey, Button, RelatedTarget, FromElement, ToElement,
-           InitMouseEvent };
+           InitMouseEvent
+    };
     DOM::MouseEvent toMouseEvent() const { return static_cast<DOM::MouseEvent>(event); }
   };
 
   class DOMTextEvent : public DOMUIEvent {
   public:
-    DOMTextEvent(ExecState *exec, DOM::TextEvent ke) : DOMUIEvent(exec, ke) {}
+    DOMTextEvent(ExecState *exec, DOM::TextEvent ke);
     ~DOMTextEvent();
-    virtual Value tryGet(ExecState *exec,const UString &p) const;
+    virtual Value tryGet(ExecState *exec,const Identifier &p) const;
     Value getValueProperty(ExecState *, int token) const;
     // no put - all read-only
     virtual const ClassInfo* classInfo() const { return &info; }
@@ -156,7 +163,7 @@ namespace KJS {
   class MutationEventConstructor : public DOMObject {
   public:
     MutationEventConstructor(ExecState *);
-    virtual Value tryGet(ExecState *exec,const UString &p) const;
+    virtual Value tryGet(ExecState *exec,const Identifier &p) const;
     Value getValueProperty(ExecState *, int token) const;
     // no put - all read-only
     virtual const ClassInfo* classInfo() const { return &info; }
@@ -167,9 +174,9 @@ namespace KJS {
 
   class DOMMutationEvent : public DOMEvent {
   public:
-    DOMMutationEvent(ExecState *exec, DOM::MutationEvent me) : DOMEvent(exec, me) {}
+    DOMMutationEvent(ExecState *exec, DOM::MutationEvent me);
     ~DOMMutationEvent();
-    virtual Value tryGet(ExecState *exec,const UString &p) const;
+    virtual Value tryGet(ExecState *exec,const Identifier &p) const;
     Value getValueProperty(ExecState *, int token) const;
     // no put - all read-only
     virtual const ClassInfo* classInfo() const { return &info; }
