@@ -4,6 +4,7 @@
  *  Copyright (C) 2000 Harri Porten (porten@kde.org)
  *  Copyright (c) 2000 Daniel Molkentin (molkentin@kde.org)
  *  Copyright (c) 2000 Stefan Schimanski (schimmi@kde.org)
+ *  Copyright (C) 2003 Apple Computer, Inc.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Library General Public
@@ -27,6 +28,9 @@
 #include <kdebug.h>
 
 #include <kio/kprotocolmanager.h>
+#include <kio/kmimetype.h>
+#include <kio/kservice.h>
+#include <kio/ktrader.h>
 #include "kjs_navigator.h"
 #include "kjs/lookup.h"
 #include "kjs_binding.h"
@@ -73,7 +77,7 @@ namespace KJS {
     class Plugins : public PluginBase {
     public:
         Plugins(ExecState *exec) : PluginBase(exec) {};
-        virtual Value get(ExecState *exec, const UString &propertyName) const;
+        virtual Value get(ExecState *exec, const Identifier &propertyName) const;
         virtual const ClassInfo* classInfo() const { return &info; }
         static const ClassInfo info;
     private:
@@ -84,7 +88,7 @@ namespace KJS {
     class MimeTypes : public PluginBase {
     public:
         MimeTypes(ExecState *exec) : PluginBase(exec) { };
-        virtual Value get(ExecState *exec, const UString &propertyName) const;
+        virtual Value get(ExecState *exec, const Identifier &propertyName) const;
         virtual const ClassInfo* classInfo() const { return &info; }
         static const ClassInfo info;
     private:
@@ -97,7 +101,7 @@ namespace KJS {
         Plugin( ExecState *exec, PluginBase::PluginInfo *info )
           : PluginBase( exec )
         { m_info = info; };
-        virtual Value get(ExecState *exec, const UString &propertyName) const;
+        virtual Value get(ExecState *exec, const Identifier &propertyName) const;
         virtual const ClassInfo* classInfo() const { return &info; }
         static const ClassInfo info;
     private:
@@ -111,7 +115,7 @@ namespace KJS {
         MimeType( ExecState *exec, PluginBase::MimeClassInfo *info )
           : PluginBase( exec )
         { m_info = info; };
-        virtual Value get(ExecState *exec, const UString &propertyName) const;
+        virtual Value get(ExecState *exec, const Identifier &propertyName) const;
         virtual const ClassInfo* classInfo() const { return &info; }
         static const ClassInfo info;
     private:
@@ -149,7 +153,7 @@ IMPLEMENT_PROTOFUNC_DOM(NavigatorFunc)
 Navigator::Navigator(ExecState *exec, KHTMLPart *p)
   : ObjectImp(exec->interpreter()->builtinObjectPrototype()), m_part(p) { }
 
-Value Navigator::get(ExecState *exec, const UString &propertyName) const
+Value Navigator::get(ExecState *exec, const Identifier &propertyName) const
 {
 #ifdef KJS_VERBOSE
   kdDebug(6070) << "Navigator::get " << propertyName.ascii() << endl;
@@ -215,7 +219,7 @@ Value Navigator::getValueProperty(ExecState *exec, int token) const
   case CookieEnabled:
     return Boolean(true); /// ##### FIXME
   default:
-    kdWarning() << "Unhandled token in DOMEvent::getValueProperty : " << token << endl;
+    kdDebug(6070) << "WARNING: Unhandled token in DOMEvent::getValueProperty : " << token << endl;
     return Value();
   }
 }
@@ -231,48 +235,66 @@ PluginBase::PluginBase(ExecState *exec)
         plugins->setAutoDelete( true );
         mimes->setAutoDelete( true );
 
-        // read configuration
-        KConfig c(KGlobal::dirs()->saveLocation("data","nsplugins")+"/pluginsinfo");
-        unsigned num = (unsigned int)c.readNumEntry("number");
-        for ( unsigned n=0; n<num; n++ ) {
+        // FIXME: add domain support here
+        KConfig kc("konquerorrc", true);
+        if (!KConfigGroup(&kc, "Java/JavaScript Settings").readBoolEntry("EnablePlugins", true))
+            return; // plugins disabled
 
-            c.setGroup( QString::number(n) );
-            PluginInfo *plugin = new PluginInfo;
+        // read in using KTrader
+        KTrader::OfferList offers = KTrader::self()->query("Browser/View");
+        KTrader::OfferList::iterator it;
+        for ( it = offers.begin(); it != offers.end(); ++it ) {
 
-            plugin->name = c.readEntry("name");
-            plugin->file = c.readPathEntry("file");
-            plugin->desc = c.readEntry("description");
+            QVariant pluginsinfo = (**it).property( "X-KDE-BrowserView-PluginsInfo" );
+            if ( !pluginsinfo.isValid() ) {
+                // <backwards compatible>
+                if ((**it).library() == QString("libnsplugin"))
+                    pluginsinfo = QVariant("nsplugins/pluginsinfo");
+                else
+                // </backwards compatible>
+                    continue;
+            }
+            // read configuration
+            KConfig kc( locate ("data", pluginsinfo.toString()) );
+            unsigned num = (unsigned int) kc.readNumEntry("number");
+            for ( unsigned n = 0; n < num; n++ ) {
+                kc.setGroup( QString::number(n) );
+                PluginInfo *plugin = new PluginInfo;
 
-            //kdDebug(6070) << "plugin : " << plugin->name << " - " << plugin->desc << endl;
+                plugin->name = kc.readEntry("name");
+                plugin->file = kc.readPathEntry("file");
+                plugin->desc = kc.readEntry("description");
 
-            plugins->append( plugin );
+                plugins->append( plugin );
 
-            // get mime types from string
-            QStringList types = QStringList::split( ';', c.readEntry("mime") );
-            QStringList::Iterator type;
-            for ( type=types.begin(); type!=types.end(); ++type ) {
+                // get mime types from string
+                QStringList types = QStringList::split( ';', kc.readEntry("mime") );
+                QStringList::Iterator type;
+                for ( type=types.begin(); type!=types.end(); ++type ) {
 
-                // get mime information
-                QStringList tokens = QStringList::split(':', *type, TRUE);
-                if ( tokens.count() < 3 ) // we need 3 items
-                  continue;
+                    // get mime information
+                    QStringList tokens = QStringList::split(':', *type, true);
+                    if ( tokens.count() < 3 ) // we need 3 items
+                        continue;
 
-                MimeClassInfo *mime = new MimeClassInfo;
-                QStringList::Iterator token = tokens.begin();
-                mime->type = (*token).lower();
-                //kdDebug(6070) << "mime->type=" << mime->type << endl;
-                ++token;
+                    MimeClassInfo *mime = new MimeClassInfo;
+                    QStringList::Iterator token = tokens.begin();
+                    mime->type = (*token).lower();
+                    //kdDebug(6070) << "mime->type=" << mime->type << endl;
+                    ++token;
 
-                mime->suffixes = *token;
-                ++token;
+                    mime->suffixes = *token;
+                    ++token;
 
-                mime->desc = *token;
-                ++token;
+                    mime->desc = *token;
+                    ++token;
 
-                mime->plugin = plugin;
+                    mime->plugin = plugin;
 
-                mimes->append( mime );
-                plugin->mimes.append( mime );
+                    mimes->append( mime );
+                    plugin->mimes.append( mime );
+
+                }
             }
         }
     }
@@ -295,14 +317,14 @@ PluginBase::~PluginBase()
 /*******************************************************************/
 IMPLEMENT_PROTOFUNC_DOM(PluginsFunc)
 
-Value Plugins::get(ExecState *exec, const UString &propertyName) const
+Value Plugins::get(ExecState *exec, const Identifier &propertyName) const
 {
 #ifdef KJS_VERBOSE
   kdDebug(6070) << "Plugins::get " << propertyName.qstring() << endl;
 #endif
     if (propertyName == "refresh")
       return lookupOrCreateFunction<PluginsFunc>(exec,propertyName,this,0,0,DontDelete|Function);
-    else if ( propertyName =="length" )
+    else if ( propertyName ==lengthPropertyName )
       return Number(plugins->count());
     else {
 
@@ -324,12 +346,12 @@ Value Plugins::get(ExecState *exec, const UString &propertyName) const
 
 /*******************************************************************/
 
-Value MimeTypes::get(ExecState *exec, const UString &propertyName) const
+Value MimeTypes::get(ExecState *exec, const Identifier &propertyName) const
 {
 #ifdef KJS_VERBOSE
   kdDebug(6070) << "MimeTypes::get " << propertyName.qstring() << endl;
 #endif
-    if( propertyName=="length" )
+    if( propertyName==lengthPropertyName )
         return Number( mimes->count() );
     else {
 
@@ -354,7 +376,7 @@ Value MimeTypes::get(ExecState *exec, const UString &propertyName) const
 
 /************************************************************************/
 
-Value Plugin::get(ExecState *exec, const UString &propertyName) const
+Value Plugin::get(ExecState *exec, const Identifier &propertyName) const
 {
 #ifdef KJS_VERBOSE
   kdDebug(6070) << "Plugin::get " << propertyName.qstring() << endl;
@@ -365,7 +387,7 @@ Value Plugin::get(ExecState *exec, const UString &propertyName) const
         return String( m_info->file );
     else if ( propertyName == "description" )
         return String( m_info->desc );
-    else if ( propertyName == "length" )
+    else if ( propertyName == lengthPropertyName )
         return Number( m_info->mimes.count() );
     else {
 
@@ -394,7 +416,7 @@ Value Plugin::get(ExecState *exec, const UString &propertyName) const
 
 /*****************************************************************************/
 
-Value MimeType::get(ExecState *exec, const UString &propertyName) const
+Value MimeType::get(ExecState *exec, const Identifier &propertyName) const
 {
 #ifdef KJS_VERBOSE
   kdDebug(6070) << "MimeType::get " << propertyName.qstring() << endl;
