@@ -33,8 +33,8 @@
 #include <unistd.h>
 #include <signal.h>
 #include <sys/time.h>
-#include <kjs/collector.h>
 #include <assert.h>
+#include <kjs/function.h>
 
 using namespace KJS;
 
@@ -42,6 +42,8 @@ extern "C" {
   KJSProxy *kjs_html_init(KHTMLPart *khtmlpart);
 }
 
+namespace KJS {
+
 class KJSProxyImpl : public KJSProxy {
 public:
   KJSProxyImpl(KHTMLPart *part);
@@ -49,14 +51,14 @@ public:
   virtual QVariant evaluate(QString filename, int baseLine, const QString &, const DOM::Node &n,
 			    Completion *completion = 0);
   virtual void clear();
-  virtual DOM::EventListener *createHTMLEventHandler(QString sourceUrl, QString code);
+  virtual DOM::EventListener *createHTMLEventHandler(QString sourceUrl, QString name, QString code);
   virtual void finishedWithEvent(const DOM::Event &event);
-  virtual KJS::ScriptInterpreter *interpreter();
+  virtual KJS::Interpreter *interpreter();
 
   virtual void setDebugEnabled(bool enabled);
+  virtual void showDebugWindow(bool show=true);
   virtual bool paused() const;
-  virtual void setSourceFile(QString url, QString code);
-  virtual void appendSourceFile(QString url, QString code);
+  virtual void dataReceived();
 
   void initScript();
   void applyUserAgent();
@@ -69,6 +71,8 @@ private:
 #endif
 };
 
+} // namespace KJS
+
 #ifndef NDEBUG
 int KJSProxyImpl::s_count = 0;
 #endif
@@ -90,7 +94,7 @@ KJSProxyImpl::~KJSProxyImpl()
     // This allows to delete the global-object properties, like all the protos
     static_cast<ObjectImp*>(m_script->globalObject().imp())->deleteAllProperties( m_script->globalExec() );
     //kdDebug() << "KJSProxyImpl::~KJSProxyImpl garbage collecting" << endl;
-    while (KJS::Collector::collect())
+    while (KJS::Interpreter::collect())
         ;
     //kdDebug() << "KJSProxyImpl::~KJSProxyImpl deleting interpreter " << m_script << endl;
     delete m_script;
@@ -98,7 +102,7 @@ KJSProxyImpl::~KJSProxyImpl()
     // Garbage collect - as many times as necessary
     // (we could delete an object which was holding another object, so
     // the deref() will happen too late for deleting the impl of the 2nd object).
-    while (KJS::Collector::collect())
+    while (KJS::Interpreter::collect())
         ;
   }
 
@@ -115,7 +119,7 @@ KJSProxyImpl::~KJSProxyImpl()
 QVariant KJSProxyImpl::evaluate(QString filename, int baseLine,
                                 const QString&str, const DOM::Node &n, Completion *completion) {
   // evaluate code. Returns the JS return value or an invalid QVariant
-  // if there was none, an error occured or the type couldn't be converted.
+  // if there was none, an error occurred or the type couldn't be converted.
 
   initScript();
   // inlineCode is true for <a href="javascript:doSomething()">
@@ -128,10 +132,10 @@ QVariant KJSProxyImpl::evaluate(QString filename, int baseLine,
 #ifdef KJS_DEBUGGER
   if (inlineCode)
     filename = "(unknown file)";
-  if (KJSDebugWin::instance()) {
-    KJSDebugWin::instance()->attach(m_script);
-    KJSDebugWin::instance()->setNextSourceInfo(filename,baseLine);
-  //    KJSDebugWin::instance()->setMode(KJSDebugWin::Step);
+  if (KJSDebugWin::debugWindow()) {
+    KJSDebugWin::debugWindow()->attach(m_script);
+    KJSDebugWin::debugWindow()->setNextSourceInfo(filename,baseLine);
+  //    KJSDebugWin::debugWindow()->setMode(KJSDebugWin::Step);
   }
 #else
   Q_UNUSED(baseLine);
@@ -154,7 +158,7 @@ QVariant KJSProxyImpl::evaluate(QString filename, int baseLine,
     *completion = comp;
 
 #ifdef KJS_DEBUGGER
-    //    KJSDebugWin::instance()->setCode(QString::null);
+    //    KJSDebugWin::debugWindow()->setCode(QString::null);
 #endif
 
   window->afterScriptExecution();
@@ -167,7 +171,7 @@ QVariant KJSProxyImpl::evaluate(QString filename, int baseLine,
     if ( comp.complType() == Throw )
     {
         UString msg = comp.value().toString(m_script->globalExec());
-        kdWarning(6070) << "Script threw exception: " << msg.qstring() << endl;
+        kdDebug(6070) << "WARNING: Script threw exception: " << msg.qstring() << endl;
     }
     return QVariant();
   }
@@ -193,11 +197,14 @@ void KJSProxyImpl::clear() {
   // (we used to delete and re-create it, previously)
   if (m_script) {
 #ifdef KJS_DEBUGGER
-    //KJSDebugWin *debugWin = KJSDebugWin::instance();
-    //if (debugWin && debugWin->currentScript() == m_script) {
-    //    debugWin->setMode(KJSDebugWin::Stop);
-//        debugWin->leaveSession();
-    //}
+    // ###
+    KJSDebugWin *debugWin = KJSDebugWin::debugWindow();
+    if (debugWin) {
+      if (debugWin->getExecState() &&
+          debugWin->getExecState()->interpreter() == m_script)
+        debugWin->slotStop();
+      debugWin->clearInterpreter(m_script);
+    }
 #endif
     m_script->clear();
 
@@ -210,26 +217,42 @@ void KJSProxyImpl::clear() {
       if ( !win->part().isNull() )
         applyUserAgent();
     }
+
+    // Really delete everything that can be, so that the DOM nodes get deref'ed
+    //kdDebug() << k_funcinfo << "all done -> collecting" << endl;
+    while (KJS::Interpreter::collect())
+        ;
   }
 }
 
-DOM::EventListener *KJSProxyImpl::createHTMLEventHandler(QString sourceUrl, QString code)
+DOM::EventListener *KJSProxyImpl::createHTMLEventHandler(QString sourceUrl, QString name, QString code)
 {
+  initScript();
+
 #ifdef KJS_DEBUGGER
-  if (KJSDebugWin::instance())
-    KJSDebugWin::instance()->setNextSourceInfo(sourceUrl,m_handlerLineno);
+  if (KJSDebugWin::debugWindow()) {
+    KJSDebugWin::debugWindow()->attach(m_script);
+    KJSDebugWin::debugWindow()->setNextSourceInfo(sourceUrl,m_handlerLineno);
+  }
 #else
   Q_UNUSED(sourceUrl);
 #endif
 
-  initScript();
   //KJS::Constructor constr(KJS::Global::current().get("Function").imp());
   KJS::Object constr = m_script->builtinFunction();
   KJS::List args;
   args.append(KJS::String("event"));
   args.append(KJS::String(code));
-  Object handlerFunc = constr.construct(m_script->globalExec(), args); // ### is globalExec ok ?
 
+  Object handlerFunc = constr.construct(m_script->globalExec(), args); // ### is globalExec ok ?
+  if (m_script->globalExec()->hadException())
+    m_script->globalExec()->clearException();
+
+  if (!handlerFunc.inherits(&DeclaredFunctionImp::info))
+    return 0; // Error creating function
+
+  DeclaredFunctionImp *declFunc = static_cast<DeclaredFunctionImp*>(handlerFunc.imp());
+  declFunc->setName(Identifier(name));
   return KJS::Window::retrieveWindow(m_part)->getJSEventListener(handlerFunc,true);
 }
 
@@ -239,10 +262,10 @@ void KJSProxyImpl::finishedWithEvent(const DOM::Event &event)
   // is the case in sitations where an event has been created just for temporary usage,
   // e.g. an image load or mouse move. Once the event has been dispatched, it is forgotten
   // by the DOM implementation and so does not need to be cached still by the interpreter
-  m_script->forgetDOMObject(event.handle());
+  ScriptInterpreter::forgetDOMObject(event.handle());
 }
 
-KJS::ScriptInterpreter *KJSProxyImpl::interpreter()
+KJS::Interpreter *KJSProxyImpl::interpreter()
 {
   if (!m_script)
     initScript();
@@ -257,48 +280,43 @@ void KJSProxyImpl::setDebugEnabled(bool enabled)
   //    m_script->setDebuggingEnabled(enabled);
   // NOTE: this is consistent across all KJSProxyImpl instances, as we only
   // ever have 1 debug window
-  if (!enabled && KJSDebugWin::instance()) {
+  if (!enabled && KJSDebugWin::debugWindow()) {
     KJSDebugWin::destroyInstance();
   }
-  else if (enabled && !KJSDebugWin::instance()) {
+  else if (enabled && !KJSDebugWin::debugWindow()) {
     KJSDebugWin::createInstance();
     initScript();
-    KJSDebugWin::instance()->attach(m_script);
+    KJSDebugWin::debugWindow()->attach(m_script);
   }
 #else
   Q_UNUSED(enabled);
 #endif
 }
 
+void KJSProxyImpl::showDebugWindow(bool /*show*/)
+{
+#ifdef KJS_DEBUGGER
+  if (KJSDebugWin::debugWindow())
+    KJSDebugWin::debugWindow()->show();
+#else
+  //Q_UNUSED(show);
+#endif
+}
+
 bool KJSProxyImpl::paused() const
 {
 #ifdef KJS_DEBUGGER
-  if (KJSDebugWin::instance())
-    return KJSDebugWin::instance()->inSession();
+  if (KJSDebugWin::debugWindow())
+    return KJSDebugWin::debugWindow()->inSession();
 #endif
   return false;
 }
 
-void KJSProxyImpl::setSourceFile(QString url, QString code)
+void KJSProxyImpl::dataReceived()
 {
 #ifdef KJS_DEBUGGER
-  if (KJSDebugWin::instance())
-    KJSDebugWin::instance()->setSourceFile(url,code);
-#else
-  Q_UNUSED(url);
-  Q_UNUSED(code);
-#endif
-
-}
-
-void KJSProxyImpl::appendSourceFile(QString url, QString code)
-{
-#ifdef KJS_DEBUGGER
-  if (KJSDebugWin::instance())
-    KJSDebugWin::instance()->appendSourceFile(url,code);
-#else
-  Q_UNUSED(url);
-  Q_UNUSED(code);
+  if (KJSDebugWin::debugWindow())
+    KJSDebugWin::debugWindow()->sourceChanged(m_script,m_part->url().url());
 #endif
 }
 
@@ -326,7 +344,8 @@ void KJSProxyImpl::initScript()
 void KJSProxyImpl::applyUserAgent()
 {
   assert( m_script );
-  QString userAgent = KProtocolManager::userAgentForHost(m_part->url().host());
+  QString host = m_part->url().isLocalFile() ? "localhost" : m_part->url().host();
+  QString userAgent = KProtocolManager::userAgentForHost(host);
   if (userAgent.find(QString::fromLatin1("Microsoft")) >= 0 ||
       userAgent.find(QString::fromLatin1("MSIE")) >= 0)
   {
@@ -347,13 +366,6 @@ void KJSProxyImpl::applyUserAgent()
     }
 }
 
-// Helper method, so that all classes which need jScript() don't need to be added
-// as friend to KHTMLPart
-KJSProxy * KJSProxy::proxy( KHTMLPart *part )
-{
-    return part->jScript();
-}
-
 // initialize HTML module
 KJSProxy *kjs_html_init(KHTMLPart *khtmlpart)
 {
@@ -376,8 +388,12 @@ void KJSCPUGuard::stop()
   signal(SIGVTALRM, oldAlarmHandler);
 }
 
-void KJSCPUGuard::alarmHandler(int) {
+bool KJSCPUGuard::confirmTerminate() {
   kdDebug(6070) << "alarmhandler" << endl;
-  if (KMessageBox::warningYesNo(0L, i18n("A script on this page is causing KHTML to freeze. If it continues to run, other applications may become less responsive.\nDo you want to abort the script?"), i18n("JavaScript"), i18n("Abort"), KStdGuiItem::cont(), "kjscupguard_alarmhandler") == KMessageBox::Yes)
+  return KMessageBox::warningYesNo(0L, i18n("A script on this page is causing KHTML to freeze. If it continues to run, other applications may become less responsive.\nDo you want to abort the script?"), i18n("JavaScript"), i18n("Abort"), KStdGuiItem::cont(), "kjscupguard_alarmhandler") == KMessageBox::Yes;
+}
+
+void KJSCPUGuard::alarmHandler(int) {
     ExecState::requestTerminate();
+    ExecState::confirmTerminate = KJSCPUGuard::confirmTerminate;
 }
