@@ -2,6 +2,7 @@
 /*
  *  This file is part of the KDE libraries
  *  Copyright (C) 2001 Peter Kelly (pmk@post.com)
+ *  Copyright (C) 2003 Apple Computer, Inc.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Library General Public
@@ -37,24 +38,25 @@ const ClassInfo DOMRange::info = { "Range", 0, &DOMRangeTable, 0 };
   commonAncestorContainer DOMRange::CommonAncestorContainer	DontDelete|ReadOnly
 @end
 @begin DOMRangeProtoTable 17
-  setStart		DOMRange::SetStart		DontDelete|Function 2
-  setEnd		DOMRange::SetEnd		DontDelete|Function 2
-  setStartBefore	DOMRange::SetStartBefore	DontDelete|Function 1
-  setStartAfter		DOMRange::SetStartAfter		DontDelete|Function 1
-  setEndBefore		DOMRange::SetEndBefore		DontDelete|Function 1
-  setEndAfter		DOMRange::SetEndAfter		DontDelete|Function 1
-  collapse		DOMRange::Collapse		DontDelete|Function 1
-  selectNode		DOMRange::SelectNode		DontDelete|Function 1
-  selectNodeContents	DOMRange::SelectNodeContents	DontDelete|Function 1
-  compareBoundaryPoints	DOMRange::CompareBoundaryPoints	DontDelete|Function 2
-  deleteContents	DOMRange::DeleteContents	DontDelete|Function 0
-  extractContents	DOMRange::ExtractContents	DontDelete|Function 0
-  cloneContents		DOMRange::CloneContents		DontDelete|Function 0
-  insertNode		DOMRange::InsertNode		DontDelete|Function 1
-  surroundContents	DOMRange::SurroundContents	DontDelete|Function 1
-  cloneRange		DOMRange::CloneRange		DontDelete|Function 0
-  toString		DOMRange::ToString		DontDelete|Function 0
-  detach		DOMRange::Detach		DontDelete|Function 0
+setStart		    DOMRange::SetStart			DontDelete|Function 2
+  setEnd		    DOMRange::SetEnd			DontDelete|Function 2
+  setStartBefore	    DOMRange::SetStartBefore		DontDelete|Function 1
+  setStartAfter		    DOMRange::SetStartAfter		DontDelete|Function 1
+  setEndBefore		    DOMRange::SetEndBefore		DontDelete|Function 1
+  setEndAfter		    DOMRange::SetEndAfter		DontDelete|Function 1
+  collapse		    DOMRange::Collapse			DontDelete|Function 1
+  selectNode		    DOMRange::SelectNode		DontDelete|Function 1
+  selectNodeContents	    DOMRange::SelectNodeContents	DontDelete|Function 1
+  compareBoundaryPoints	    DOMRange::CompareBoundaryPoints	DontDelete|Function 2
+  deleteContents	    DOMRange::DeleteContents		DontDelete|Function 0
+  extractContents	    DOMRange::ExtractContents		DontDelete|Function 0
+  cloneContents		    DOMRange::CloneContents		DontDelete|Function 0
+  insertNode		    DOMRange::InsertNode		DontDelete|Function 1
+  surroundContents	    DOMRange::SurroundContents		DontDelete|Function 1
+  cloneRange		    DOMRange::CloneRange		DontDelete|Function 0
+  toString		    DOMRange::ToString			DontDelete|Function 0
+  detach		    DOMRange::Detach			DontDelete|Function 0
+  createContextualFragment  DOMRange::CreateContextualFragment  DontDelete|Function 1
 @end
 */
 DEFINE_PROTOTYPE("DOMRange",DOMRangeProto)
@@ -69,7 +71,7 @@ DOMRange::~DOMRange()
   ScriptInterpreter::forgetDOMObject(range.handle());
 }
 
-Value DOMRange::tryGet(ExecState *exec, const UString &p) const
+Value DOMRange::tryGet(ExecState *exec, const Identifier &p) const
 {
   return DOMObjectLookupGetValue<DOMRange,DOMObject>(exec,p,&DOMRangeTable,this);
 }
@@ -92,7 +94,7 @@ Value DOMRange::getValueProperty(ExecState *exec, int token) const
     return getDOMNode(exec,range2.commonAncestorContainer());
   }
   default:
-    kdWarning() << "Unhandled token in DOMRange::getValueProperty : " << token << endl;
+    kdDebug(6070) << "WARNING: Unhandled token in DOMRange::getValueProperty : " << token << endl;
     return Value();
   }
 }
@@ -171,6 +173,11 @@ Value DOMRangeProtoFunc::tryCall(ExecState *exec, Object &thisObj, const List &a
       range.detach();
       result = Undefined();
       break;
+    case DOMRange::CreateContextualFragment:
+      Value value = args[0];
+      DOM::DOMString str = value.isA(NullType) ? DOM::DOMString() : value.toString(exec).string();
+      result = getDOMNode(exec, range.createContextualFragment(str));
+      break;
   };
 
   return result;
@@ -196,7 +203,7 @@ const ClassInfo RangeConstructor::info = { "RangeConstructor", 0, &RangeConstruc
 RangeConstructor::RangeConstructor(ExecState *exec)
   : DOMObject(exec->interpreter()->builtinObjectPrototype()) { }
 
-Value RangeConstructor::tryGet(ExecState *exec, const UString &p) const
+Value RangeConstructor::tryGet(ExecState *exec, const Identifier &p) const
 {
   return DOMObjectLookupGetValue<RangeConstructor,DOMObject>(exec,p,&RangeConstructorTable,this);
 }
