@@ -2,6 +2,7 @@
 /*
  *  This file is part of the KDE libraries
  *  Copyright (C) 2000 Harri Porten (porten@kde.org)
+ *  Copyright (C) 2003 Apple Computer, Inc.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Library General Public
@@ -25,6 +26,7 @@
 #include <qguardedptr.h>
 #include <qmap.h>
 #include <qptrlist.h>
+#include <qdatetime.h>
 
 #include "kjs_binding.h"
 
@@ -38,6 +40,7 @@ namespace KJS {
   class WindowQObject;
   class Location;
   class History;
+  class External;
   class FrameArray;
   class JSEventListener;
 
@@ -48,7 +51,7 @@ namespace KJS {
       Height, Width, ColorDepth, PixelDepth, AvailLeft, AvailTop, AvailHeight,
       AvailWidth
     };
-    virtual Value get(ExecState *exec, const UString &propertyName) const;
+    virtual Value get(ExecState *exec, const Identifier &propertyName) const;
     Value getValueProperty(ExecState *exec, int token) const;
   private:
     KHTMLView *view;
@@ -83,12 +86,10 @@ namespace KJS {
     static Window *retrieveActive(ExecState *exec);
     QGuardedPtr<KHTMLPart> part() const { return m_part; }
     virtual void mark();
-    virtual bool hasProperty(ExecState *exec, const UString &p) const;
-    virtual Value get(ExecState *exec, const UString &propertyName) const;
-    virtual void put(ExecState *exec, const UString &propertyName, const Value &value, int attr = None);
+    virtual bool hasProperty(ExecState *exec, const Identifier &p) const;
+    virtual Value get(ExecState *exec, const Identifier &propertyName) const;
+    virtual void put(ExecState *exec, const Identifier &propertyName, const Value &value, int attr = None);
     virtual bool toBoolean(ExecState *exec) const;
-    int installTimeout(const UString &handler, int t, bool singleShot);
-    void clearTimeout(int timerId);
     void scheduleClose();
     void closeNow();
     void delayedGoHistory(int steps);
@@ -97,7 +98,11 @@ namespace KJS {
     Value openWindow(ExecState *exec, const List &args);
     void resizeTo(QWidget* tl, int width, int height);
     void afterScriptExecution();
-    bool isSafeScript(ExecState *exec) const;
+    bool isSafeScript(ExecState *exec) const {
+      KHTMLPart *activePart = static_cast<KJS::ScriptInterpreter *>(  exec->interpreter() )->part();
+      if ( activePart == m_part ) return true;
+      return checkIsSafeScript( activePart );
+    }
     Location *location() const;
     ObjectImp* frames( ExecState* exec ) const;
     JSEventListener *getJSEventListener(const Value &val, bool html = false);
@@ -107,24 +112,25 @@ namespace KJS {
     // Set the current "event" object
     void setCurrentEvent( DOM::Event *evt );
 
-    QPtrList<JSEventListener> jsEventListeners;
+    QPtrDict<JSEventListener> jsEventListeners;
     virtual const ClassInfo* classInfo() const { return &info; }
     static const ClassInfo info;
     enum { Closed, Crypto, DefaultStatus, Status, Document, Node, EventCtor, Range,
-           NodeFilter, DOMException, CSSRule, Frames, _History, Event, InnerHeight,
-           InnerWidth, Length, _Location, Name, _Navigator, _Konqueror, ClientInformation,
+           NodeFilter, DOMException, CSSRule, Frames, _History, _External, Event, InnerHeight,
+           InnerWidth, Length, _Location, Navigate, Name, _Navigator, _Konqueror, ClientInformation,
            OffscreenBuffering, Opener, OuterHeight, OuterWidth, PageXOffset, PageYOffset,
            Parent, Personalbar, ScreenX, ScreenY, Scrollbars, Scroll, ScrollBy,
            ScreenTop, ScreenLeft,
            ScrollTo, ScrollX, ScrollY, MoveBy, MoveTo, ResizeBy, ResizeTo, Self, _Window, Top, _Screen,
-           Image, Option, Alert, Confirm, Prompt, Open, SetTimeout, ClearTimeout,
+           Image, Option, Alert, Confirm, Prompt, Open, SetTimeout, ClearTimeout, XMLHttpRequest,
            Focus, Blur, Close, SetInterval, ClearInterval, CaptureEvents, ReleaseEvents,
-           Print, AddEventListener, RemoveEventListener,
+           Print, AddEventListener, RemoveEventListener, SideBar,
            Onabort, Onblur,
            Onchange, Onclick, Ondblclick, Ondragdrop, Onerror, Onfocus,
            Onkeydown, Onkeypress, Onkeyup, Onload, Onmousedown, Onmousemove,
            Onmouseout, Onmouseover, Onmouseup, Onmove, Onreset, Onresize,
            Onselect, Onsubmit, Onunload };
+    WindowQObject *winq;
   protected:
     enum DelayedActionId { NullAction, DelayedClose, DelayedGoHistory };
 
@@ -134,12 +140,14 @@ namespace KJS {
     struct DelayedAction;
     friend struct DelayedAction;
 
+    bool checkIsSafeScript( KHTMLPart* activePart ) const;
+
     QGuardedPtr<KHTMLPart> m_part;
     Screen *screen;
     History *history;
+    External *external;
     FrameArray *m_frames;
     Location *loc;
-    WindowQObject *winq;
     DOM::Event *m_evt;
 
     struct DelayedAction {
@@ -158,16 +166,22 @@ namespace KJS {
    */
   class ScheduledAction {
   public:
-    ScheduledAction(Object _func, List _args, bool _singleShot);
-    ScheduledAction(QString _code, bool _singleShot);
+    ScheduledAction(Object _func, List _args, QTime _nextTime, int _interval, bool _singleShot, int _timerId);
+    ScheduledAction(QString _code, QTime _nextTime, int _interval, bool _singleShot, int _timerId);
     ~ScheduledAction();
     void execute(Window *window);
+    void mark();
 
-    ObjectImp func;
+    ObjectImp *func;
     List args;
     QString code;
     bool isFunction;
     bool singleShot;
+
+    QTime nextTime;
+    int interval;
+    bool executing;
+    int timerId;
   };
 
   class WindowQObject : public QObject {
@@ -175,26 +189,30 @@ namespace KJS {
   public:
     WindowQObject(Window *w);
     ~WindowQObject();
-    int installTimeout(const UString &handler, int t, bool singleShot);
+    int installTimeout(const Identifier &handler, int t, bool singleShot);
     int installTimeout(const Value &func, List args, int t, bool singleShot);
-    void clearTimeout(int timerId, bool delAction = true);
+    void clearTimeout(int timerId);
+    void mark();
   public slots:
     void timeoutClose();
   protected slots:
     void parentDestroyed();
   protected:
     void timerEvent(QTimerEvent *e);
+    void setNextTimer();
   private:
     Window *parent;
     KHTMLPart *part;   		// not guarded, may be dangling
-    QMap<int, ScheduledAction*> scheduledActions;
+    QPtrList<ScheduledAction> scheduledActions;
+    int pausedTime;
+    int lastTimerId;
   };
 
   class Location : public ObjectImp {
   public:
     ~Location();
-    virtual Value get(ExecState *exec, const UString &propertyName) const;
-    virtual void put(ExecState *exec, const UString &propertyName, const Value &value, int attr = None);
+    virtual Value get(ExecState *exec, const Identifier &propertyName) const;
+    virtual void put(ExecState *exec, const Identifier &propertyName, const Value &value, int attr = None);
     virtual Value toPrimitive(ExecState *exec, Type preferred) const;
     virtual UString toString(ExecState *exec) const;
     enum { Hash, Href, Hostname, Host, Pathname, Port, Protocol, Search, EqualEqual,
@@ -213,8 +231,8 @@ namespace KJS {
     friend class KonquerorFunc;
   public:
     Konqueror(KHTMLPart *p) : part(p) { }
-    virtual Value get(ExecState *exec, const UString &propertyName) const;
-    virtual bool hasProperty(ExecState *exec, const UString &p) const;
+    virtual Value get(ExecState *exec, const Identifier &propertyName) const;
+    virtual bool hasProperty(ExecState *exec, const Identifier &p) const;
     virtual UString toString(ExecState *exec) const;
     virtual const ClassInfo* classInfo() const { return &info; }
     static const ClassInfo info;
