@@ -18,7 +18,7 @@
  * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
  *
- * $Id: dtd.cpp,v 1.64.2.1 2003/01/03 21:09:25 mueller Exp $
+ * $Id: dtd.cpp,v 1.71.2.4 2004/03/03 12:57:51 mueller Exp $
  */
 
 #include "html/dtd.h"
@@ -27,6 +27,7 @@
 using namespace DOM;
 
 #include <kdebug.h>
+#include <kglobal.h>
 
 // priority of tags. Closing tags of higher priority close tags of lower
 // priority.
@@ -44,9 +45,10 @@ using namespace DOM;
 // 8 table
 // 9 body frameset
 // 10 html
-const unsigned short DOM::tagPriority[] = {
+
+const unsigned short KDE_NO_EXPORT DOM::tagPriority[] = {
     0, // 0
-    5, // ID_A == 1
+    1, // ID_A == 1
     1, // ID_ABBR
     1, // ID_ACRONYM
     3, // ID_ADDRESS
@@ -77,7 +79,7 @@ const unsigned short DOM::tagPriority[] = {
     1, // ID_EM
     0, // ID_EMBED
     3, // ID_FIELDSET
-    5, // ID_FONT
+    1, // ID_FONT
     3, // ID_FORM
     0, // ID_FRAME
    10,// ID_FRAMESET
@@ -92,13 +94,13 @@ const unsigned short DOM::tagPriority[] = {
    11,// ID_HTML
     1, // ID_I
     1, // ID_IFRAME
+    1, // ID_ILAYER
     0, // ID_IMG
     0, // ID_INPUT
     1, // ID_INS
     0, // ID_ISINDEX
     1, // ID_KBD
     0, // ID_KEYGEN
-    5, // ID__KONQBLOCK
     1, // ID_LABEL
     1, // ID_LAYER
     1, // ID_LEGEND
@@ -108,7 +110,7 @@ const unsigned short DOM::tagPriority[] = {
     3, // ID_MARQUEE
     5, // ID_MENU
     0, // ID_META
-    1, // ID_NOBR
+    4, // ID_NOBR
    10,// ID_NOEMBED
    10,// ID_NOFRAMES
     3, // ID_NOSCRIPT
@@ -146,7 +148,7 @@ const unsigned short DOM::tagPriority[] = {
     1, // ID_U
     5, // ID_UL
     1, // ID_VAR
-    1, // ID_WBR
+    4, // ID_WBR
     5, // ID_XMP
     0, // ID_TEXT
 };
@@ -199,13 +201,13 @@ const tagStatus DOM::endTag[] = {
     REQUIRED,  // ID_HTML
     REQUIRED,  // ID_I
     REQUIRED,  // ID_IFRAME
+    REQUIRED,  // ID_ILAYER
     FORBIDDEN, // ID_IMG
     FORBIDDEN, // ID_INPUT
     REQUIRED,  // ID_INS
     FORBIDDEN, // ID_ISINDEX
     REQUIRED,  // ID_KBD
     REQUIRED,  // ID_KEYGEN
-    REQUIRED,  // ID__KONQBLOCK
     REQUIRED,  // ID_LABEL
     REQUIRED,  // ID_LAYER
     REQUIRED,  // ID_LEGEND
@@ -324,6 +326,7 @@ static const ushort tag_list_1[] = {
     ID_PLAINTEXT,
     ID_DL,
     ID_DIV,
+    ID_ILAYER,
     ID_LAYER,
     ID_CENTER,
     ID_NOSCRIPT,
@@ -379,7 +382,6 @@ static const ushort tag_list_1[] = {
     ID_COMMENT,
     ID_LI,
     ID_XMP,
-    ID__KONQBLOCK,
     ID_INS,
     ID_DEL,
     ID_NOBR,
@@ -409,6 +411,7 @@ static const ushort tag_list_3[] = {
     ID_PLAINTEXT,
     ID_DL,
     ID_DIV,
+    ID_ILAYER,
     ID_LAYER,
     ID_CENTER,
     ID_NOSCRIPT,
@@ -423,7 +426,6 @@ static const ushort tag_list_3[] = {
     ID_COMMENT,
     ID_LI,
     ID_XMP,
-    ID__KONQBLOCK,
     ID_MARQUEE,
     0
 };
@@ -445,6 +447,7 @@ static const ushort tag_list_4[] = {
     ID_PLAINTEXT,
     ID_DL,
     ID_DIV,
+    ID_ILAYER,
     ID_LAYER,
     ID_CENTER,
     ID_NOSCRIPT,
@@ -582,6 +585,14 @@ bool DOM::checkChild(ushort tagID, ushort childID)
     case ID_SUB:
     case ID_SUP:
     case ID_BDO:
+    case ID_FONT:
+    case ID_LEGEND:
+    case ID_Q:
+    case ID_A:
+    case ID_NOBR:
+    case ID_WBR:
+        // _1 *
+        return check_array(childID, tag_list_1);
     case ID_P:
     case ID_H1:
     case ID_H2:
@@ -589,8 +600,6 @@ bool DOM::checkChild(ushort tagID, ushort childID)
     case ID_H4:
     case ID_H5:
     case ID_H6:
-    case ID_Q:
-    case ID_LEGEND:
         // _0 *
         return check_array(childID, tag_list_0);
     case ID_BASEFONT:
@@ -613,22 +622,16 @@ bool DOM::checkChild(ushort tagID, ushort childID)
         // BODY: _1 * + _2
         if( check_array(childID, tag_list_1) ) return true;
         return check_array(childID, tag_list_2);
-    case ID_NOBR:
-    case ID_WBR:
     case ID_ADDRESS:
         // ADDRESS: ( _0 | P ) *
         if( check_array(childID, tag_list_0) ) return true;
         return (childID == ID_P);
-    case ID_FONT:
-        // special handling for FONT: (_0 | 3)
-        return check_array(childID, tag_list_3) || check_array(childID, tag_list_0);
-    case ID__KONQBLOCK:
-        if ( childID == ID__KONQBLOCK || childID == ID_TABLE ) return false;
-        // Fall through!
-    case ID_LI:
     case ID_DT:
+        if ( childID == ID_DL ) return false;
+    case ID_LI:
     case ID_DIV:
     case ID_SPAN:
+    case ID_ILAYER:
     case ID_LAYER:
     case ID_CENTER:
     case ID_BLOCKQUOTE:
@@ -644,17 +647,10 @@ bool DOM::checkChild(ushort tagID, ushort childID)
     case ID_MARQUEE:
         // DIV: _1 *
         return check_array(childID, tag_list_1);
-    case ID_A:
-        // A: _0 * - A
-        if( check_array(childID, tag_list_0) ) return true;
-        if ( childID == ID_TABLE ) return false; // no table in anchor
-        if( check_array(childID, tag_list_3) ) return true;
-        if( childID == ID_ADDRESS ) return true;
-        return false;
     case ID_MAP:
-        // MAP: ( _3 + | AREA + )
+        // MAP: ( _3 + | AREA + | SCRIPT + )
         if( check_array(childID, tag_list_3) ) return true;
-        return (childID == ID_AREA);
+        return (childID == ID_AREA || childID == ID_SCRIPT);
     case ID_OBJECT:
     case ID_EMBED:
     case ID_APPLET:
@@ -671,7 +667,6 @@ bool DOM::checkChild(ushort tagID, ushort childID)
     case ID_OL:
     case ID_UL:
         // OL: LI +
-        if(childID == ID_LI) return true;
         return check_array(childID, tag_list_1);
     case ID_DIR:
     case ID_MENU:
@@ -684,7 +679,7 @@ bool DOM::checkChild(ushort tagID, ushort childID)
     case ID_LABEL:
         // LABEL: _0 * - LABEL
         return check_array(childID, tag_list_0);
-        // KEYGEN does not really allow any childs
+        // KEYGEN does not really allow any children
         // from outside, just need this to be able
         // to add the keylengths ourself
         // Yes, consider it a hack (Dirk)
@@ -722,6 +717,7 @@ bool DOM::checkChild(ushort tagID, ushort childID)
         case ID_THEAD:
         case ID_TFOOT:
         case ID_TBODY:
+        case ID_SCRIPT:
             return true;
         default:
             return false;
@@ -730,7 +726,7 @@ bool DOM::checkChild(ushort tagID, ushort childID)
     case ID_TFOOT:
     case ID_TBODY:
         // THEAD: TR +
-        if(childID == ID_TR) return true;
+        if(childID == ID_TR || childID == ID_SCRIPT) return true;
         return false;
     case ID_COLGROUP:
         // COLGROUP: COL *
@@ -738,7 +734,7 @@ bool DOM::checkChild(ushort tagID, ushort childID)
         return false;
     case ID_TR:
         // TR: (TD, TH)
-        return (childID == ID_TH || childID == ID_TD);
+        return (childID == ID_TH || childID == ID_TD || childID == ID_SCRIPT);
     case ID_FRAMESET:
         // FRAMESET: _10
         return check_array(childID, tag_list_10);
@@ -911,174 +907,3 @@ void DOM::removeForbidden(int tagId, ushort *forbiddenTags)
     }
 }
 
-
-#if 0
-
-
-struct attr_priv {
-    attr_priv() { id = len = 0, val = 0; }
-    attr_priv(ushort i, const QChar *v, ushort l)
-    { id =i; len = l, val = v; }
-    ushort id;
-    const QChar *val;
-    ushort len;
-};
-
-DOMString find_attr(ushort id, const attr_priv *attrs)
-{
-    int i = 0;
-    while(attrs[i].id != 0)
-    {
-        if(attrs[i].id == id)
-            return DOMString(attrs[i].val, attrs[i].len);
-        i++;
-    }
-    return DOMString();
-}
-
-static const QChar value_1_0 [] = { 'N','O','N','E' };
-
-attr_priv attr_list_1[] = {
-    attr_priv(ATTR_CLEAR, value_1_0, 4)
-};
-
-static const QChar value_2_0 [] = { 'R','E','C','T' };
-
-attr_priv attr_list_2[] = {
-    attr_priv(ATTR_SHAPE, value_2_0, 4)
-};
-
-static const QChar value_3_0 [] = { 'R','E','C','T' };
-
-attr_priv attr_list_3[] = {
-    attr_priv(ATTR_SHAPE, value_3_0, 4)
-};
-
-static const QChar value_4_0 [] = { 'D','A','T','A' };
-
-attr_priv attr_list_4[] = {
-    attr_priv(ATTR_VALUETYPE, value_4_0, 4)
-};
-
-static const QChar value_5_0 [] = { 'G','E','T' };
-static const QChar value_5_1 [] = { 'A','P','P','L','I','C','A','T','I','O','N','/','X','-','W','W','W','-','F','O','R','M','-','U','R','L','E','N','C','O','D','E','D' };
-
-attr_priv attr_list_5[] = {
-    attr_priv(ATTR_METHOD, value_5_0, 3),
-    attr_priv(ATTR_ENCTYPE, value_5_1, 33)
-};
-
-static const QChar value_6_0 [] = { 'T','E','X','T' };
-
-attr_priv attr_list_6[] = {
-    attr_priv(ATTR_TYPE, value_6_0, 4)
-};
-
-static const QChar value_7_0 [] = { 'S','U','B','M','I','T' };
-
-attr_priv attr_list_7[] = {
-    attr_priv(ATTR_TYPE, value_7_0, 6)
-};
-
-static const QChar value_8_0 [] = { '1' };
-
-attr_priv attr_list_8[] = {
-    attr_priv(ATTR_SPAN, value_8_0, 1)
-};
-
-static const QChar value_9_0 [] = { '1' };
-
-attr_priv attr_list_9[] = {
-    attr_priv(ATTR_SPAN, value_9_0, 1)
-};
-
-static const QChar value_10_0 [] = { '1' };
-static const QChar value_10_1 [] = { '1' };
-
-attr_priv attr_list_10[] = {
-    attr_priv(ATTR_ROWSPAN, value_10_0, 1),
-    attr_priv(ATTR_COLSPAN, value_10_1, 1)
-};
-
-static const QChar value_11_0 [] = { '1' };
-static const QChar value_11_1 [] = { 'A','U','T','O' };
-
-attr_priv attr_list_11[] = {
-    attr_priv(ATTR_FRAMEBORDER, value_11_0, 1),
-    attr_priv(ATTR_SCROLLING, value_11_1, 4)
-};
-
-static const QChar value_12_0 [] = { '1' };
-static const QChar value_12_1 [] = { 'A','U','T','O' };
-
-attr_priv attr_list_12[] = {
-    attr_priv(ATTR_FRAMEBORDER, value_12_0, 1),
-    attr_priv(ATTR_SCROLLING, value_12_1, 4)
-};
-
-static const QChar value_13_0 [] = { '-','/','/','W','3','C','/','/','D','T','D' };
-static const QChar value_13_1 [] = { 'T','R','A','N','S','I','T','I','O','N','A','L','/','/','E','N' };
-
-attr_priv attr_list_13[] = {
-    attr_priv(ATTR_VERSION, value_13_0, 11),
-    attr_priv(ATTR_HTML, value_13_1, 16)
-};
-
-DOMString DOM::findDefAttrNone(ushort)
-{
-    return DOMString();
-};
-DOMString DOM::findDefAttrBR(ushort id)
-{
-    return find_attr(id, attr_list_1);
-}
-DOMString DOM::findDefAttrA(ushort id)
-{
-    return find_attr(id, attr_list_2);
-}
-DOMString DOM::findDefAttrAREA(ushort id)
-{
-    return find_attr(id, attr_list_3);
-}
-DOMString DOM::findDefAttrPARAM(ushort id)
-{
-    return find_attr(id, attr_list_4);
-}
-DOMString DOM::findDefAttrFORM(ushort id)
-{
-    return find_attr(id, attr_list_5);
-}
-DOMString DOM::findDefAttrINPUT(ushort id)
-{
-    return find_attr(id, attr_list_6);
-}
-DOMString DOM::findDefAttrBUTTON(ushort id)
-{
-    return find_attr(id, attr_list_7);
-}
-DOMString DOM::findDefAttrCOLGROUP(ushort id)
-{
-    return find_attr(id, attr_list_8);
-}
-DOMString DOM::findDefAttrCOL(ushort id)
-{
-    return find_attr(id, attr_list_9);
-}
-DOMString DOM::findDefAttrTH(ushort id)
-{
-    return find_attr(id, attr_list_10);
-}
-DOMString DOM::findDefAttrFRAME(ushort id)
-{
-    return find_attr(id, attr_list_11);
-}
-DOMString DOM::findDefAttrIFRAME(ushort id)
-{
-    return find_attr(id, attr_list_12);
-}
-DOMString DOM::findDefAttrHTML(ushort id)
-{
-    return find_attr(id, attr_list_13);
-}
-
-#endif
