@@ -1,7 +1,7 @@
 /**
  * This file is part of the DOM implementation for KDE.
  *
- * Copyright (C) 1999 Lars Knoll (knoll@kde.org)
+ * Copyright (C) 1999-2003 Lars Knoll (knoll@kde.org)
  *           (C) 1999 Antti Koivisto (koivisto@kde.org)
  *
  * This library is free software; you can redistribute it and/or
@@ -19,7 +19,7 @@
  * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
  *
- * $Id: html_blockimpl.cpp,v 1.63.4.2 2003/01/13 22:04:47 mueller Exp $
+ * $Id: html_blockimpl.cpp,v 1.71 2003/10/03 14:33:29 mueller Exp $
  */
 // -------------------------------------------------------------------------
 //#define DEBUG
@@ -44,7 +44,7 @@ void HTMLDivElementImpl::parseAttribute(AttributeImpl *attr)
     {
         DOMString v = attr->value();
         if ( strcasecmp( attr->value(), "center" ) == 0 )
-            v = "-konq-center";
+            v = "\\2d khtml-center";
         addCSSProperty(CSS_PROP_TEXT_ALIGN, v);
         break;
     }
@@ -108,7 +108,7 @@ void HTMLHRElementImpl::attach()
             addCSSProperty(CSS_PROP_BORDER_LEFT_STYLE, CSS_VAL_SOLID);
             addCSSProperty(CSS_PROP_BORDER_TOP_WIDTH, DOMString("0"));
             addCSSLength(CSS_PROP_BORDER_BOTTOM_WIDTH, DOMString(si));
-            addCSSProperty(CSS_PROP_BORDER_COLOR, color);
+            addHTMLColor(CSS_PROP_BORDER_COLOR, color);
         }
         else {
             if (_s > 1 && getAttribute(ATTR_NOSHADE).isNull()) {
