@@ -27,6 +27,7 @@
 #include "html/html_baseimpl.h"
 #include "html/htmltokenizer.h"
 #include "html/html_miscimpl.h"
+#include "html/html_formimpl.h"
 
 #include "khtmlview.h"
 #include "khtml_part.h"
@@ -40,6 +41,7 @@
 
 #include "khtml_factory.h"
 #include "rendering/render_object.h"
+#include "dom/dom_exception.h"
 
 #include <dcopclient.h>
 #include <kapplication.h>
@@ -68,6 +70,8 @@ HTMLDocumentImpl::HTMLDocumentImpl(DOMImplementationImpl *_implementation, KHTML
     bodyElement = 0;
     htmlElement = 0;
 
+    m_doAutoFill = false;
+
 /* dynamic history stuff to be fixed later (pfeiffer)
     connect( KHTMLFactory::vLinks(), SIGNAL( inserted( const QString& )),
              SLOT( slotHistoryChanged() ));
@@ -92,14 +96,14 @@ DOMString HTMLDocumentImpl::referrer() const
 DOMString HTMLDocumentImpl::domain() const
 {
     if ( m_domain.isEmpty() ) // not set yet (we set it on demand to save time and space)
-        m_domain = KURL(URL()).host(); // Initially set to the host
+        m_domain = URL().host(); // Initially set to the host
     return m_domain;
 }
 
 void HTMLDocumentImpl::setDomain(const DOMString &newDomain)
 {
     if ( m_domain.isEmpty() ) // not set yet (we set it on demand to save time and space)
-        m_domain = KURL(URL()).host().lower(); // Initially set to the host
+        m_domain = URL().host().lower(); // Initially set to the host
 
     if ( m_domain.isEmpty() /*&& view() && view()->part()->openedByJS()*/ )
         m_domain = newDomain.lower();
@@ -139,7 +143,7 @@ DOMString HTMLDocumentImpl::cookie() const
     QCString replyType;
     QByteArray params, reply;
     QDataStream stream(params, IO_WriteOnly);
-    stream << URL() << windowId;
+    stream << URL().url() << windowId;
     if (!kapp->dcopClient()->call("kcookiejar", "kcookiejar",
                                   "findDOMCookies(QString,long int)", params,
                                   replyType, reply))
@@ -170,10 +174,10 @@ void HTMLDocumentImpl::setCookie( const DOMString & value )
 
     QByteArray params;
     QDataStream stream(params, IO_WriteOnly);
-    QString fake_header("Set-Cookie: ");
-    fake_header.append(value.string());
+    QCString fake_header("Set-Cookie: ");
+    fake_header.append(value.string().latin1());
     fake_header.append("\n");
-    stream << URL() << fake_header.utf8() << windowId;
+    stream << URL().url() << fake_header << windowId;
     if (!kapp->dcopClient()->send("kcookiejar", "kcookiejar",
                                   "addCookies(QString,QCString,long int)", params))
     {
@@ -205,14 +209,14 @@ HTMLElementImpl *HTMLDocumentImpl::body()
     return static_cast<HTMLElementImpl *>(body);
 }
 
-void HTMLDocumentImpl::setBody(HTMLElementImpl *_body)
+void HTMLDocumentImpl::setBody(HTMLElementImpl *_body, int& exceptioncode)
 {
-    int exceptioncode = 0;
     HTMLElementImpl *b = body();
-    if ( !_body && !b ) return;
-    if ( !_body )
-        documentElement()->removeChild( b, exceptioncode );
-    else if ( !b )
+    if ( !_body ) {
+        exceptioncode = DOMException::HIERARCHY_REQUEST_ERR;
+        return;
+    }
+    if ( !b )
         documentElement()->appendChild( _body, exceptioncode );
     else
         documentElement()->replaceChild( _body, b, exceptioncode );
@@ -235,10 +239,12 @@ bool HTMLDocumentImpl::childAllowed( NodeImpl *newChild )
 
 ElementImpl *HTMLDocumentImpl::createElement( const DOMString &name, int* pExceptioncode )
 {
-    ElementImpl* e = createHTMLElement( name );
-    if ( e )
+    ElementImpl *e = createHTMLElement(name);
+    if ( e ) {
+        e->setHTMLCompat( htmlMode() != XHtml );
         return e;
-    return DOM::DocumentImpl::createElement( name, pExceptioncode );
+    }
+    return DocumentImpl::createElement(name, pExceptioncode);
 }
 
 void HTMLDocumentImpl::slotHistoryChanged()
@@ -274,7 +280,6 @@ static bool isTransitional(const QString &spec, int start)
        (spec.find("LATIN1", start, false ) != -1 ) ||
        (spec.find("SYMBOLS", start, false ) != -1 ) ||
        (spec.find("SPECIAL", start, false ) != -1 ) ) {
-        //kdDebug() << "isTransitional" << endl;
         return true;
     }
     return false;
@@ -288,6 +293,18 @@ void HTMLDocumentImpl::close()
 
     HTMLElementImpl* b = body();
     if (b && doload) {
+
+        if (title().isEmpty()) // ensure setTitle is called at least once
+            setTitle( DOMString() );
+
+        // auto fill: walk the tree and try to fill in login credentials
+        if (view() && m_doAutoFill) {
+            for (NodeImpl* n = this; n; n = n->traverseNextNode())
+                if (n->id() == ID_FORM)
+                    static_cast<HTMLFormElementImpl*>(n)->doAutoFill();
+            m_doAutoFill = false;
+        }
+
         // According to dom the load event must not bubble
         // but other browsers execute in a frameset document
         // the first(IE)/last(Moz/Konq) registered onload on a <frame> and the
@@ -305,7 +322,7 @@ void HTMLDocumentImpl::close()
         // don't update rendering if we're going to redirect anyway
         if ( view() && ( view()->part()->d->m_redirectURL.isNull() ||
                          view()->part()->d->m_delayRedirect > 1 ) )
-        updateRendering();
+            updateRendering();
     }
 }
 
@@ -378,7 +395,6 @@ void HTMLDocumentImpl::determineParseMode( const QString &str )
                         if ( tagPos != -1 ) {
                             tagPos = val.find(QRegExp("[0-9]"), tagPos );
                             int version = val.mid( tagPos, 1 ).toInt();
-                            //kdDebug() << "DocumentImpl::determineParseMode tagPos = " << tagPos << " version=" << version << endl;
                             if( version > 3 ) {
                                 hMode = Html4;
                                 publicId = isTransitional( val, tagPos ) ? Transitional : Strict;
@@ -390,29 +406,32 @@ void HTMLDocumentImpl::determineParseMode( const QString &str )
             }
         }
 
-        if( systemId == publicId )
+        if( systemId && systemId == publicId ) { // not unknown and both agree
             pMode = publicId;
+        }
         else if ( systemId == Unknown )
-            pMode = hMode == Html4 ? Compat : publicId;
-        else if ( publicId == Transitional && systemId == Strict ) {
-            pMode = hMode == Html3 ? Compat : Strict;
-        } else
+            pMode = hMode == Html3 ? Compat : publicId;
+        else if ( ( publicId == Transitional && systemId == Strict ) ||
+                  ( publicId == Strict && systemId == Transitional ) ) {
+            pMode = hMode == Html3 ? Compat : Transitional;
+        } else {
             pMode = Compat;
+        }
 
         if ( hMode == XHtml )
-            pMode = Strict;
+            pMode = publicId;
     }
-//     kdDebug() << "DocumentImpl::determineParseMode: publicId =" << publicId << " systemId = " << systemId << endl;
-//     kdDebug() << "DocumentImpl::determineParseMode: htmlMode = " << hMode<< endl;
-//     if( pMode == Strict )
-//         kdDebug(6020) << " using strict parseMode" << endl;
-//     else if (pMode == Compat )
-//         kdDebug(6020) << " using compatibility parseMode" << endl;
-//     else
-//         kdDebug(6020) << " using transitional parseMode" << endl;
+    // kdDebug() << "DocumentImpl::determineParseMode: publicId =" << publicId << " systemId = " << systemId << endl;
+    // kdDebug() << "DocumentImpl::determineParseMode: htmlMode = " << hMode<< endl;
+    if( pMode == Strict )
+        kdDebug(6020) << " using strict parseMode" << endl;
+    else if (pMode == Compat )
+        kdDebug(6020) << " using compatibility parseMode" << endl;
+    else
+        kdDebug(6020) << " using transitional parseMode" << endl;
 
     if ( pMode != oldPMode && styleSelector() )
-	recalcStyleSelector();
+        recalcStyleSelector();
 }
 
 #include "html_documentimpl.moc"
