@@ -19,14 +19,13 @@
  * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
  *
- * $Id: html_documentimpl.h,v 1.65.2.3 2003/06/25 11:56:35 faure Exp $
+ * $Id: html_documentimpl.h,v 1.70.2.1 2004/02/29 15:27:53 mueller Exp $
  */
 
 #ifndef HTML_DOCUMENTIMPL_H
 #define HTML_DOCUMENTIMPL_H
 
 #include "xml/dom_docimpl.h"
-#include "misc/loader_client.h"
 
 #include <qmap.h>
 
@@ -44,7 +43,7 @@ namespace DOM {
     class CSSStyleSheetImpl;
     class HTMLMapElementImpl;
 
-class HTMLDocumentImpl : public DOM::DocumentImpl, public khtml::CachedObjectClient
+class HTMLDocumentImpl : public DOM::DocumentImpl
 {
     Q_OBJECT
 public:
@@ -61,9 +60,9 @@ public:
     void setCookie( const DOMString &);
 
     HTMLElementImpl *body();
-    void setBody(HTMLElementImpl *_body);
+    void setBody(HTMLElementImpl *_body, int& exceptioncode);
 
-    virtual Tokenizer *createTokenizer();
+    virtual khtml::Tokenizer *createTokenizer();
 
     virtual bool childAllowed( NodeImpl *newChild );
 
@@ -74,12 +73,15 @@ public:
     virtual void determineParseMode( const QString &str );
     virtual void close();
 
+    void setAutoFill() { m_doAutoFill = true; }
+
 protected:
     HTMLElementImpl *bodyElement;
     HTMLElementImpl *htmlElement;
     friend class HTMLMapElementImpl;
     friend class HTMLImageElementImpl;
     QMap<QString,HTMLMapElementImpl*> mapMap;
+    bool m_doAutoFill;
 
 protected slots:
     /**
