@@ -19,7 +19,7 @@
  * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
  *
- * $Id: html_elementimpl.h,v 1.60.2.3 2003/05/18 12:34:39 mueller Exp $
+ * $Id: html_elementimpl.h,v 1.74 2003/10/25 16:52:06 faure Exp $
  */
 #ifndef HTML_ELEMENTIMPL_H
 #define HTML_ELEMENTIMPL_H
@@ -30,38 +30,42 @@ namespace DOM {
 
 class DOMString;
 class CSSStyleDeclarationImpl;
+class HTMLFormElementImpl;
+class DocumentFragment;
 
 class HTMLElementImpl : public ElementImpl
 {
 public:
-    HTMLElementImpl(DocumentPtr *doc)
-        : ElementImpl(doc) {}
+    HTMLElementImpl(DocumentPtr *doc);
 
     virtual ~HTMLElementImpl();
 
     virtual bool isHTMLElement() const { return true; }
 
+    virtual bool isInline() const;
+
     virtual Id id() const = 0;
+    virtual DOMString tagName() const;
+    virtual DOMString localName() const;
+    virtual DOMString namespaceURI() const;
 
     virtual void parseAttribute(AttributeImpl *token);
 
     void addCSSLength(int id, const DOMString &value, bool numOnly = false, bool multiLength = false);
     void addCSSProperty(int id, const DOMString &value);
     void addCSSProperty(int id, int value);
+    void addHTMLColor( int id, const DOMString &c );
     void removeCSSProperty(int id);
 
     virtual void recalcStyle( StyleChange );
 
     DOMString innerHTML() const;
     DOMString innerText() const;
+    DocumentFragment createContextualFragment( const DOMString &html );
     bool setInnerHTML( const DOMString &html );
     bool setInnerText( const DOMString &text );
 
-    virtual DOMString namespaceURI() const;
-
 protected:
-    bool isURLAllowed(const QString& url) const;
-
     // for IMG, OBJECT and APPLET
     void addHTMLAlignment( DOMString alignment );
 };
