@@ -20,6 +20,7 @@
  * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
  *
+ * $Id: html_formimpl.cpp,v 1.359.2.5 2004/03/20 23:47:13 mueller Exp $
  */
 
 #undef FORMS_DEBUG
@@ -51,6 +52,7 @@
 #include <kmessagebox.h>
 #include <kapplication.h>
 #include <klocale.h>
+#include <kwallet.h>
 #include <netaccess.h>
 #include <kfileitem.h>
 #include <qfile.h>
@@ -78,6 +80,7 @@ HTMLFormElementImpl::HTMLFormElementImpl(DocumentPtr *doc, bool implicit)
     m_enctype = "application/x-www-form-urlencoded";
     m_boundary = "----------" + KApplication::randomString( 42 + 13 );
     m_acceptcharset = "UNKNOWN";
+    m_malformed = false;
 }
 
 HTMLFormElementImpl::~HTMLFormElementImpl()
@@ -260,19 +263,20 @@ QByteArray HTMLFormElementImpl::formData(bool& ok)
 
                     // if the current type is FILE, then we also need to
                     // include the filename
-                    if (current->nodeType() == Node::ELEMENT_NODE && current->id() == ID_INPUT &&
-                        static_cast<HTMLInputElementImpl*>(current)->inputType() == HTMLInputElementImpl::FILE)
+                    if (current->id() == ID_INPUT &&
+                        static_cast<HTMLInputElementImpl*>(current)->inputType() == HTMLInputElementImpl::FILE &&
+                        current->renderer())
                     {
-                        QString path = static_cast<HTMLInputElementImpl*>(current)->value().string();
-                        if (path.length()) fileUploads << path;
-                        QString onlyfilename = path.mid(path.findRev('/')+1);
+                        KURL path ( static_cast<HTMLInputElementImpl*>(current)->value().string());
 
-                        hstr += fixUpfromUnicode(codec, "; filename=\"" + onlyfilename + "\"");
-                        if(!static_cast<HTMLInputElementImpl*>(current)->value().isEmpty())
-                        {
-                            hstr += "\r\nContent-Type: ";
-                            KMimeType::Ptr ptr = KMimeType::findByURL(KURL(path));
-                            hstr += ptr->name().ascii();
+                        hstr += fixUpfromUnicode(codec, "; filename=\"" + path.fileName() + "\"");
+                        if(path.isValid()) {
+                            fileUploads << path.prettyURL(0, KURL::StripFileProtocol);
+                            KMimeType::Ptr ptr = KMimeType::findByURL(path);
+                            if (!ptr->name().isEmpty()) {
+                                hstr += "\r\nContent-Type: ";
+                                hstr += ptr->name().ascii();
+                            }
                         }
                     }
 
@@ -336,9 +340,55 @@ void HTMLFormElementImpl::setEnctype( const DOMString& type )
     m_encCharset = QString::null;
 }
 
+static QString calculateAutoFillKey(const HTMLFormElementImpl& e)
+{
+    KURL k(e.getDocument()->URL());
+    k.setRef(QString::null);
+    k.setQuery(QString::null);
+    // ensure that we have the user / password inside the url
+    // otherwise we might have a potential security problem
+    // by saving passwords under wrong lookup key.
+    QString name = e.getAttribute(ATTR_NAME).string().stripWhiteSpace();
+    return k.url() + '#' + name;
+}
+
+void HTMLFormElementImpl::doAutoFill()
+{
+    QString key = calculateAutoFillKey(*this);
+
+    if (KWallet::Wallet::keyDoesNotExist(KWallet::Wallet::NetworkWallet(),
+                                         KWallet::Wallet::FormDataFolder(),
+                                         key) )
+        return;
+
+    // assert(view())
+    KWallet::Wallet* w = getDocument()->view()->part()->wallet();
+
+    if (w) {
+        if (!w->hasFolder(KWallet::Wallet::FormDataFolder())) {
+            if (!w->createFolder(KWallet::Wallet::FormDataFolder()))
+                return; // failed
+        }
+        w->setFolder(KWallet::Wallet::FormDataFolder());
+        QMap<QString, QString> map;
+        if (w->readMap(key, map))
+            return; // failed, abort
+
+        for (QPtrListIterator<HTMLGenericFormElementImpl> it(formElements); it.current(); ++it) {
+            if (it.current()->id() == ID_INPUT) {
+                HTMLInputElementImpl* current = static_cast<HTMLInputElementImpl*>(it.current());
+                if (current->inputType() == HTMLInputElementImpl::PASSWORD ||
+                    current->inputType() == HTMLInputElementImpl::TEXT &&
+                    map.contains(current->name().string()))
+                    current->setValue(map[current->name().string()]);
+            }
+        }
+    }
+}
+
 void HTMLFormElementImpl::submitFromKeyboard()
 {
-    // Activate the first submit button
+    // Activate the first nondisabled submit button
     // if there is none, do a submit anyway if not more
     // than one <input type=text> or <input type=password>
     unsigned int inputtext = 0;
@@ -402,21 +452,81 @@ void HTMLFormElementImpl::submit(  )
     kdDebug( 6030 ) << "submitting!" << endl;
 #endif
 
-    KHTMLView *view = getDocument()->view();
-    for (QPtrListIterator<HTMLGenericFormElementImpl> it(formElements); it.current(); ++it) {
-        HTMLGenericFormElementImpl* current = it.current();
-        if (current->id() == ID_INPUT &&
-            static_cast<HTMLInputElementImpl*>(current)->inputType() == HTMLInputElementImpl::TEXT &&
-            static_cast<HTMLInputElementImpl*>(current)->autoComplete() )
-        {
-            HTMLInputElementImpl *input = static_cast<HTMLInputElementImpl *>(current);
-            view->addFormCompletionItem(input->name().string(), input->value().string());
-        }
-    }
-
     bool ok;
+    KHTMLView* view = getDocument()->view();
     QByteArray form_data = formData(ok);
-    if (ok) {
+
+    if (ok && view) {
+        // check if we have any password input's
+        QMap<QString, QString> walletMap;
+        bool havePassword = false;
+        bool haveTextarea = false;
+        KURL formUrl(getDocument()->URL());
+        if (!view->nonPasswordStorableSite(formUrl.host())) {
+            for (QPtrListIterator<HTMLGenericFormElementImpl> it(formElements); it.current(); ++it)
+                if (it.current()->id() == ID_INPUT)  {
+                    HTMLInputElementImpl* c = static_cast<HTMLInputElementImpl*> (it.current());
+                    if (c->inputType() == HTMLInputElementImpl::TEXT ||
+                        c->inputType() == HTMLInputElementImpl::PASSWORD)  {
+                        walletMap.insert(c->name().string(), c->value().string());
+                        if (c->inputType() == HTMLInputElementImpl::PASSWORD &&
+                            !c->value().isEmpty())
+                            havePassword = true;
+                    }
+                }
+                else if (it.current()->id() == ID_TEXTAREA)
+                    haveTextarea = true;
+        }
+
+        int savePassword = KMessageBox::Yes;
+        QString key = calculateAutoFillKey(*this);
+        bool doesnotexist = KWallet::Wallet::keyDoesNotExist(KWallet::Wallet::NetworkWallet(),
+                                                             KWallet::Wallet::FormDataFolder(), key);
+
+        if (havePassword && !haveTextarea ) {
+            KWallet::Wallet* w = view->part()->wallet();
+            if (w)  {
+                if (!w->hasFolder(KWallet::Wallet::FormDataFolder()))
+                    w->createFolder(KWallet::Wallet::FormDataFolder());
+                w->setFolder(KWallet::Wallet::FormDataFolder());
+                bool login_changed = false;
+                if ( !doesnotexist ) {
+                    // check if the login information changed from what
+                    // we had so far.
+                    QMap<QString, QString> map;
+                    if (!w->readMap(key, map)) {
+                        for ( QMapIterator<QString, QString> it( map.begin() ); it != map.end(); ++it )
+                            if ( map[it.key()] != walletMap[it.key()] ) {
+                                login_changed = true;
+                                break;
+                            }
+                    }
+                    else
+                        login_changed = true;
+                }
+
+                if ( ( doesnotexist || login_changed ) &&
+                     (savePassword=KMessageBox::questionYesNoCancel(
+                         0,
+                         i18n("Konqueror has the ability to store the password "
+                              "in an encrypted wallet. When the wallet is unlocked, it "
+                              "can then automatically restore the login information "
+                              "next time you visit this site. Do you want to store "
+                              "the information now?"),
+                         i18n("Save Login Information"),
+                         KStdGuiItem::yes(),
+                         KGuiItem(i18n("Never for This Site")))) == KMessageBox::Yes)  {
+                    // ensure that we have the user / password inside the url
+                    // otherwise we might have a potential security problem
+                    // by saving passwords under wrong lookup key.
+
+                    w->setFolder(KWallet::Wallet::FormDataFolder());
+                    w->writeMap(key, walletMap);
+                } else if (savePassword == KMessageBox::No)
+                    view->addNonPasswordStorableSite(formUrl.host());
+            }
+        }
+
         DOMString url(khtml::parseURL(getAttribute(ATTR_ACTION)));
         if(m_post) {
             view->part()->submitForm( "post", url.string(), form_data,
@@ -445,7 +555,7 @@ void HTMLFormElementImpl::reset(  )
 #endif
 
     // ### DOM2 labels this event as not cancelable, however
-    // common browsers( sick! ) allow it be cancelled.
+    // common browsers( sick! ) allow it be canceled.
     if ( !dispatchHTMLEvent(EventImpl::RESET_EVENT,true, true) ) {
         m_inreset = false;
         return;
@@ -485,11 +595,11 @@ void HTMLFormElementImpl::parseAttribute(AttributeImpl *attr)
         break;
     case ATTR_ONSUBMIT:
         setHTMLEventListener(EventImpl::SUBMIT_EVENT,
-	    getDocument()->createHTMLEventListener(attr->value().string()));
+	    getDocument()->createHTMLEventListener(attr->value().string(),"onsubmit"));
         break;
     case ATTR_ONRESET:
         setHTMLEventListener(EventImpl::RESET_EVENT,
-	    getDocument()->createHTMLEventListener(attr->value().string()));
+	    getDocument()->createHTMLEventListener(attr->value().string(),"onreset"));
         break;
     case ATTR_ID:
     case ATTR_NAME:
@@ -505,9 +615,8 @@ void HTMLFormElementImpl::radioClicked( HTMLGenericFormElementImpl *caller )
         HTMLGenericFormElementImpl *current = it.current();
         if (current->id() == ID_INPUT &&
             static_cast<HTMLInputElementImpl*>(current)->inputType() == HTMLInputElementImpl::RADIO &&
-            current != caller && current->form() == caller->form() && current->name() == caller->name()) {
+            current != caller && current->form() == caller->form() && current->name() == caller->name())
             static_cast<HTMLInputElementImpl*>(current)->setChecked(false);
-        }
     }
 }
 
@@ -603,10 +712,16 @@ void HTMLGenericFormElementImpl::attach()
     if (m_render) {
         assert(m_render->style());
         parentNode()->renderer()->addChild(m_render, nextRenderer());
-        m_render->updateFromElement();
     }
 
     NodeBaseImpl::attach();
+
+    // The call to updateFromElement() needs to go after the call through
+    // to the base class's attach() because that can sometimes do a close
+    // on the renderer.
+    if (m_render)
+        m_render->updateFromElement();
+
 }
 
 HTMLFormElementImpl *HTMLGenericFormElementImpl::getForm() const
@@ -673,23 +788,49 @@ bool HTMLGenericFormElementImpl::isSelectable() const
         static_cast<RenderWidget*>(m_render)->widget()->focusPolicy() >= QWidget::TabFocus;
 }
 
+class FocusHandleWidget : public QWidget
+{
+public:
+    void focusNextPrev(bool n) {
+	focusNextPrevChild(n);
+    }
+};
+
 void HTMLGenericFormElementImpl::defaultEventHandler(EventImpl *evt)
 {
+    if (evt->target() == this && renderer() && renderer()->isWidget() &&
+        !static_cast<RenderWidget*>(renderer())->widget()->inherits("QScrollView")) {
+        switch(evt->id())  {
+        case EventImpl::MOUSEDOWN_EVENT:
+        case EventImpl::MOUSEUP_EVENT:
+        case EventImpl::MOUSEMOVE_EVENT:
+        case EventImpl::MOUSEOUT_EVENT:
+        case EventImpl::MOUSEOVER_EVENT:
+        case EventImpl::KEYDOWN_EVENT:
+        case EventImpl::KEYUP_EVENT:
+        case EventImpl::KHTML_KEYPRESS_EVENT:
+            if (static_cast<RenderWidget*>(renderer())->handleEvent(*evt))
+		evt->setDefaultHandled();
+        default:
+            break;
+        }
+    }
+
     if (evt->target()==this && !m_disabled)
     {
         // Report focus in/out changes to the browser extension (editable widgets only)
         KHTMLView *view = getDocument()->view();
-        if (evt->id()==EventImpl::DOMFOCUSIN_EVENT && isEditable() && m_render && m_render->isWidget()) {
+        if (view && evt->id() == EventImpl::DOMFOCUSIN_EVENT && isEditable() && m_render && m_render->isWidget()) {
             KHTMLPartBrowserExtension *ext = static_cast<KHTMLPartBrowserExtension *>(view->part()->browserExtension());
             QWidget *widget = static_cast<RenderWidget*>(m_render)->widget();
             if (ext)
                 ext->editableWidgetFocused(widget);
         }
-        if (evt->id()==EventImpl::MOUSEDOWN_EVENT || evt->id()==EventImpl::KHTML_KEYDOWN_EVENT)
+        if (evt->id()==EventImpl::MOUSEDOWN_EVENT || evt->id()==EventImpl::KEYDOWN_EVENT)
         {
             setActive();
         }
-        else if (evt->id() == EventImpl::MOUSEUP_EVENT || evt->id()==EventImpl::KHTML_KEYUP_EVENT)
+        else if (evt->id() == EventImpl::MOUSEUP_EVENT || evt->id()==EventImpl::KEYUP_EVENT)
         {
 	    if (m_active)
 	    {
@@ -701,15 +842,19 @@ void HTMLGenericFormElementImpl::defaultEventHandler(EventImpl *evt)
             }
         }
 
-	if (evt->id()==EventImpl::KHTML_KEYDOWN_EVENT ||
-	    evt->id()==EventImpl::KHTML_KEYUP_EVENT)
-	{
+	if (evt->id() == EventImpl::KHTML_KEYPRESS_EVENT) {
 	    TextEventImpl * k = static_cast<TextEventImpl *>(evt);
-	    if (k->keyVal() == QChar('\n').unicode() && m_render && m_render->isWidget() && k->qKeyEvent)
-		QApplication::sendEvent(static_cast<RenderWidget *>(m_render)->widget(), k->qKeyEvent);
+	    int key = k->qKeyEvent ? k->qKeyEvent->key() : 0;
+	    if (m_render && (key == Qt::Key_Tab || key == Qt::Key_BackTab)) {
+		QWidget *widget = static_cast<RenderWidget*>(m_render)->widget();
+		if (widget)
+		    static_cast<FocusHandleWidget *>(widget)
+			->focusNextPrev(key == Qt::Key_Tab);
+	    }
 	}
 
-	if (evt->id()==EventImpl::DOMFOCUSOUT_EVENT && isEditable() && m_render && m_render->isWidget()) {
+
+	if (view && evt->id() == EventImpl::DOMFOCUSOUT_EVENT && isEditable() && m_render && m_render->isWidget()) {
 	    KHTMLPartBrowserExtension *ext = static_cast<KHTMLPartBrowserExtension *>(view->part()->browserExtension());
 	    QWidget *widget = static_cast<RenderWidget*>(m_render)->widget();
 	    if (ext)
@@ -773,32 +918,23 @@ void HTMLButtonElementImpl::parseAttribute(AttributeImpl *attr)
         break;
     case ATTR_ACCESSKEY:
         break;
-    case ATTR_ONFOCUS:
-        setHTMLEventListener(EventImpl::FOCUS_EVENT,
-            getDocument()->createHTMLEventListener(attr->value().string()));
-        break;
-    case ATTR_ONBLUR:
-        setHTMLEventListener(EventImpl::BLUR_EVENT,
-            getDocument()->createHTMLEventListener(attr->value().string()));
-        break;
     default:
         HTMLGenericFormElementImpl::parseAttribute(attr);
     }
 }
 
-void HTMLButtonElementImpl::attach()
-{
-    // skip the generic handler
-    HTMLElementImpl::attach();
-    // doesn't work yet in the renderer ### fixme
-    if (renderer())
-        renderer()->setReplaced(true);
-}
-
 void HTMLButtonElementImpl::defaultEventHandler(EventImpl *evt)
 {
-    if (m_type != BUTTON && (evt->id() == EventImpl::DOMACTIVATE_EVENT) && !m_disabled)
-        activate();
+    if (m_type != BUTTON && !m_disabled) {
+	bool act = (evt->id() == EventImpl::DOMACTIVATE_EVENT);
+	if (!act && evt->id()==EventImpl::KEYUP_EVENT) {
+	    QKeyEvent *ke = static_cast<TextEventImpl *>(evt)->qKeyEvent;
+	    if (ke && active() && (ke->key() == Qt::Key_Return || ke->key() == Qt::Key_Enter || ke->key() == Qt::Key_Space))
+		act = true;
+	}
+	if (act)
+	    activate();
+    }
     HTMLGenericFormElementImpl::defaultEventHandler(evt);
 }
 
@@ -815,6 +951,12 @@ void HTMLButtonElementImpl::activate()
         m_form->reset();
 }
 
+void HTMLButtonElementImpl::click()
+{
+    QMouseEvent me(QEvent::MouseButtonRelease, QPoint(0,0),Qt::LeftButton, 0);
+    dispatchMouseEvent(&me,EventImpl::CLICK_EVENT, 1);
+}
+
 bool HTMLButtonElementImpl::encoding(const QTextCodec* codec, khtml::encodingList& encoding, bool /*multipart*/)
 {
     if (m_type != SUBMIT || name().isEmpty() || !m_activeSubmit)
@@ -827,6 +969,11 @@ bool HTMLButtonElementImpl::encoding(const QTextCodec* codec, khtml::encodingLis
     return true;
 }
 
+void HTMLButtonElementImpl::attach()
+{
+    // skip the generic handler
+    HTMLElementImpl::attach();
+}
 
 // -------------------------------------------------------------------------
 
@@ -849,20 +996,11 @@ void HTMLFieldSetElementImpl::attach()
     assert(!attached());
     assert(!m_render);
     assert(parentNode());
-    addCSSProperty(CSS_PROP_BORDER_TOP_STYLE, CSS_VAL_SOLID);
-    addCSSProperty(CSS_PROP_BORDER_BOTTOM_STYLE, CSS_VAL_SOLID);
-    addCSSProperty(CSS_PROP_BORDER_LEFT_STYLE, CSS_VAL_SOLID);
-    addCSSProperty(CSS_PROP_BORDER_RIGHT_STYLE, CSS_VAL_SOLID);
-    addCSSProperty(CSS_PROP_BORDER_WIDTH, "1px");
-    addCSSProperty(CSS_PROP_PADDING_LEFT, "4px");
-    addCSSProperty(CSS_PROP_PADDING_RIGHT, "4px");
-    addCSSProperty(CSS_PROP_PADDING_BOTTOM, "4px");
-
 
     RenderStyle* _style = getDocument()->styleSelector()->styleForElement(this);
     _style->ref();
     if (parentNode()->renderer() && _style->display() != NONE) {
-        m_render = new RenderFieldset(this);
+        m_render = new (getDocument()->renderArena()) RenderFieldset(this);
         m_render->setStyle(_style);
     }
     HTMLGenericFormElementImpl::attach();
@@ -948,6 +1086,10 @@ QString HTMLInputElementImpl::state( )
     case CHECKBOX:
     case RADIO:
         return QString::fromLatin1(m_checked ? "on" : "off");
+    case TEXT:
+        if (autoComplete() && value() != getAttribute(ATTR_VALUE) && getDocument()->view())
+            getDocument()->view()->addFormCompletionItem(name().string(), value().string());
+        /* nobreak */
     default:
         return value().string()+'.'; // Make sure the string is not empty!
     }
@@ -980,12 +1122,10 @@ void HTMLInputElementImpl::select(  )
         static_cast<RenderFileButton*>(m_render)->select();
 }
 
-void HTMLInputElementImpl::click(  )
+void HTMLInputElementImpl::click()
 {
-    // ###
-#ifdef FORMS_DEBUG
-    kdDebug( 6030 ) << " HTMLInputElementImpl::click(  )" << endl;
-#endif
+    QMouseEvent me(QEvent::MouseButtonRelease, QPoint(0,0),Qt::LeftButton, 0);
+    dispatchMouseEvent(&me,EventImpl::CLICK_EVENT, 1);
 }
 
 void HTMLInputElementImpl::parseAttribute(AttributeImpl *attr)
@@ -1023,11 +1163,9 @@ void HTMLInputElementImpl::parseAttribute(AttributeImpl *attr)
         if (m_render && m_type == IMAGE) m_render->updateFromElement();
         break;
     case ATTR_USEMAP:
-    case ATTR_ACCESSKEY:
         // ### ignore for the moment
         break;
-    case ATTR_ALIGN:
-        addHTMLAlignment( attr->value() );
+    case ATTR_ACCESSKEY:
         break;
     case ATTR_WIDTH:
         // ignore this attribute,  do _not_ add
@@ -1037,21 +1175,13 @@ void HTMLInputElementImpl::parseAttribute(AttributeImpl *attr)
     case ATTR_HEIGHT:
         addCSSLength(CSS_PROP_HEIGHT, attr->value() );
         break;
-    case ATTR_ONFOCUS:
-        setHTMLEventListener(EventImpl::FOCUS_EVENT,
-            getDocument()->createHTMLEventListener(attr->value().string()));
-        break;
-    case ATTR_ONBLUR:
-        setHTMLEventListener(EventImpl::BLUR_EVENT,
-            getDocument()->createHTMLEventListener(attr->value().string()));
-        break;
     case ATTR_ONSELECT:
         setHTMLEventListener(EventImpl::SELECT_EVENT,
-            getDocument()->createHTMLEventListener(attr->value().string()));
+            getDocument()->createHTMLEventListener(attr->value().string(),"onselect"));
         break;
     case ATTR_ONCHANGE:
         setHTMLEventListener(EventImpl::CHANGE_EVENT,
-            getDocument()->createHTMLEventListener(attr->value().string()));
+            getDocument()->createHTMLEventListener(attr->value().string(),"onchange"));
         break;
     default:
         HTMLGenericFormElementImpl::parseAttribute(attr);
@@ -1100,32 +1230,21 @@ void HTMLInputElementImpl::attach()
             m_value = nvalue;
         }
         m_checked = (getAttribute(ATTR_CHECKED) != 0);
+        if ( m_type == IMAGE )
+            addHTMLAlignment( getAttribute( ATTR_ALIGN ) );
         m_inited = true;
     }
 
-    // make sure we don't inherit a color to the form elements
-    // by adding a non-CSS color property. this his higher
-    // priority than inherited color, but lesser priority than
-    // any color specified by CSS for the elements.
     switch( m_type ) {
-    case TEXT:
     case PASSWORD:
-        addCSSProperty(CSS_PROP_FONT_FAMILY,  "monospace");
-        /* nobreak */
-    case ISINDEX:
-    case FILE:
-        addCSSProperty(CSS_PROP_COLOR, "text");
+        if (getDocument()->isHTMLDocument())
+            static_cast<HTMLDocumentImpl*>(getDocument())->setAutoFill();
         break;
-    case SUBMIT:
-    case RESET:
-    case BUTTON:
-    case CHECKBOX:
-    case RADIO:
-        addCSSProperty(CSS_PROP_COLOR, "buttontext" );
     case HIDDEN:
     case IMAGE:
         if (!getAttribute(ATTR_WIDTH).isNull())
             addCSSLength(CSS_PROP_WIDTH, getAttribute(ATTR_WIDTH));
+    default:
         break;
     };
 
@@ -1136,14 +1255,14 @@ void HTMLInputElementImpl::attach()
         {
         case TEXT:
         case PASSWORD:
-        case ISINDEX:      m_render = new RenderLineEdit(this);   break;
-        case CHECKBOX:  m_render = new RenderCheckBox(this); break;
-        case RADIO:        m_render = new RenderRadioButton(this); break;
-        case SUBMIT:      m_render = new RenderSubmitButton(this); break;
-        case IMAGE:       m_render =  new RenderImageButton(this); break;
-        case RESET:      m_render = new RenderResetButton(this);   break;
-        case FILE:         m_render =  new RenderFileButton(this);    break;
-        case BUTTON:  m_render = new RenderPushButton(this);
+        case ISINDEX:      m_render = new (getDocument()->renderArena()) RenderLineEdit(this);   break;
+        case CHECKBOX:  m_render = new (getDocument()->renderArena()) RenderCheckBox(this); break;
+        case RADIO:        m_render = new (getDocument()->renderArena()) RenderRadioButton(this); break;
+        case SUBMIT:      m_render = new (getDocument()->renderArena()) RenderSubmitButton(this); break;
+        case IMAGE:       m_render =  new (getDocument()->renderArena()) RenderImageButton(this); break;
+        case RESET:      m_render = new (getDocument()->renderArena()) RenderResetButton(this);   break;
+        case FILE:         m_render =  new (getDocument()->renderArena()) RenderFileButton(this);    break;
+        case BUTTON:  m_render = new (getDocument()->renderArena()) RenderPushButton(this);
         case HIDDEN:   break;
         }
     }
@@ -1245,30 +1364,30 @@ bool HTMLInputElementImpl::encoding(const QTextCodec* codec, khtml::encodingList
                 return false;
 
             QString local;
-            QCString dummy("");
-
             KURL fileurl(value().string());
             KIO::UDSEntry filestat;
 
             // can't submit file in www-url-form encoded
-            if (multipart && KIO::NetAccess::stat(fileurl, filestat)) {
-                KFileItem fileitem(filestat, fileurl, true, false);
-
-                if ( fileitem.isFile() && KIO::NetAccess::download(KURL(value().string()), local) ) {
-                    QFile file(local);
-                    if (file.open(IO_ReadOnly)) {
-                        QCString filearray(file.size()+1);
-                        int readbytes = file.readBlock( filearray.data(), file.size());
-                        if ( readbytes >= 0 )
-                            filearray[readbytes] = '\0';
-                        file.close();
-
-                        encoding += filearray;
+            QWidget* toplevel = static_cast<RenderSubmitButton*>(m_render)->widget()->topLevelWidget();
+            if (multipart) {
+                QCString filearray( "" );
+                if ( KIO::NetAccess::stat(fileurl, filestat, toplevel)) {
+                    KFileItem fileitem(filestat, fileurl, true, false);
+                    if ( fileitem.isFile() &&
+                         KIO::NetAccess::download(KURL(value().string()), local, toplevel) ) {
+                        QFile file(local);
+                        filearray.resize(file.size()+1);
+                        if ( file.open( IO_ReadOnly ) ) {
+                            int readbytes = file.readBlock( filearray.data(), file.size());
+                            if ( readbytes >= 0 )
+                                filearray[readbytes] = '\0';
+                            file.close();
+                        }
                         KIO::NetAccess::removeTempFile( local );
-
-                        return true;
                     }
                 }
+                encoding += filearray;
+                return true;
             }
             // else fall through
         }
@@ -1335,23 +1454,57 @@ void HTMLInputElementImpl::defaultEventHandler(EventImpl *evt)
 {
     if ( !m_disabled )
     {
-        if (evt->isMouseEvent() &&
-            evt->id() == EventImpl::CLICK_EVENT && m_type == IMAGE && m_render) {
-            // record the mouse position for when we get the DOMActivate event
-            MouseEventImpl *me = static_cast<MouseEventImpl*>(evt);
-            int offsetX, offsetY;
-            m_render->absolutePosition(offsetX,offsetY);
-            xPos = me->clientX()-offsetX;
-            yPos = me->clientY()-offsetY;
+
+        if (evt->isMouseEvent()) {
+	    MouseEventImpl *me = static_cast<MouseEventImpl*>(evt);
+            if ((m_type == RADIO || m_type == CHECKBOX)
+		&& me->id() == EventImpl::MOUSEUP_EVENT && me->detail() > 0) {
+		// click will follow
+		setChecked(m_type == RADIO ? true : !checked());
+	    }
+            if (evt->id() == EventImpl::CLICK_EVENT && m_type == IMAGE && m_render) {
+		// record the mouse position for when we get the DOMActivate event
+		int offsetX, offsetY;
+		m_render->absolutePosition(offsetX,offsetY);
+		xPos = me->clientX()-offsetX;
+		yPos = me->clientY()-offsetY;
+	    }
+	}
+
+        if (m_type == RADIO || m_type == CHECKBOX || m_type == SUBMIT || m_type == RESET || m_type == BUTTON ) {
+	    bool check = false;
+	    if (active() && ( evt->id() == EventImpl::KEYUP_EVENT ||
+	                      evt->id() == EventImpl::KHTML_KEYPRESS_EVENT ) ) {
+		TextEventImpl *te = static_cast<TextEventImpl *>(evt);
+		if (te->keyVal() == ' ')
+		    check = true;
+	    }
+	    if (check) {
+	        if (evt->id() == EventImpl::KEYUP_EVENT) {
+		    if (m_type == RADIO || m_type == CHECKBOX)
+		        setChecked(m_type == RADIO ? true : !checked());
+		    click();
+		}
+	        // Tell the parent that we handle this key (keyup and keydown), even though only keyup activates (#70478)
+	        evt->setDefaultHandled();
+	    }
         }
 
+
         // DOMActivate events cause the input to be "activated" - in the case of image and submit inputs, this means
         // actually submitting the form. For reset inputs, the form is reset. These events are sent when the user clicks
         // on the element, or presses enter while it is the active element. Javascript code wishing to activate the element
         // must dispatch a DOMActivate event - a click event will not do the job.
-        if ((evt->id() == EventImpl::DOMACTIVATE_EVENT) &&
-            (m_type == IMAGE || m_type == SUBMIT || m_type == RESET))
-            activate();
+        if (m_type == IMAGE || m_type == SUBMIT || m_type == RESET) {
+	    bool act = (evt->id() == EventImpl::DOMACTIVATE_EVENT);
+	    if (!act && evt->id() == EventImpl::KEYUP_EVENT) {
+		QKeyEvent *ke = static_cast<TextEventImpl *>(evt)->qKeyEvent;
+		if (ke && active() && (ke->key() == Qt::Key_Return || ke->key() == Qt::Key_Enter || ke->key() == Qt::Key_Space))
+		    act = true;
+	    }
+	    if (act)
+		activate();
+	}
     }
     HTMLGenericFormElementImpl::defaultEventHandler(evt);
 }
@@ -1396,23 +1549,6 @@ NodeImpl::Id HTMLLabelElementImpl::id() const
     return ID_LABEL;
 }
 
-void HTMLLabelElementImpl::parseAttribute(AttributeImpl *attr)
-{
-    switch(attr->id())
-    {
-    case ATTR_ONFOCUS:
-        setHTMLEventListener(EventImpl::FOCUS_EVENT,
-            getDocument()->createHTMLEventListener(attr->value().string()));
-        break;
-    case ATTR_ONBLUR:
-        setHTMLEventListener(EventImpl::BLUR_EVENT,
-            getDocument()->createHTMLEventListener(attr->value().string()));
-        break;
-    default:
-        HTMLElementImpl::parseAttribute(attr);
-    }
-}
-
 void HTMLLabelElementImpl::attach()
 {
     // skip the generic handler
@@ -1450,12 +1586,10 @@ void HTMLLegendElementImpl::attach()
     assert(!attached());
     assert(!m_render);
     assert(parentNode());
-    addCSSProperty(CSS_PROP_PADDING_LEFT, "1px");
-    addCSSProperty(CSS_PROP_PADDING_RIGHT, "1px");
     RenderStyle* _style = getDocument()->styleSelector()->styleForElement(this);
     _style->ref();
     if (parentNode()->renderer() && _style->display() != NONE) {
-        m_render = new RenderLegend(this);
+        m_render = new (getDocument()->renderArena()) RenderLegend(this);
         m_render->setStyle(_style);
     }
     HTMLGenericFormElementImpl::attach();
@@ -1467,7 +1601,6 @@ void HTMLLegendElementImpl::parseAttribute(AttributeImpl *attr)
     switch(attr->id())
     {
     case ATTR_ACCESSKEY:
-        // ### ignore for the moment
         break;
     default:
         HTMLElementImpl::parseAttribute(attr);
@@ -1680,28 +1813,19 @@ void HTMLSelectElementImpl::parseAttribute(AttributeImpl *attr)
     switch(attr->id())
     {
     case ATTR_SIZE:
-        m_size = QMAX( attr->val()->toInt(), 1 );
+        m_size = kMax( attr->val()->toInt(), 1 );
         break;
     case ATTR_WIDTH:
-        m_minwidth = QMAX( attr->val()->toInt(), 0 );
+        m_minwidth = kMax( attr->val()->toInt(), 0 );
         break;
     case ATTR_MULTIPLE:
         m_multiple = (attr->val() != 0);
         break;
     case ATTR_ACCESSKEY:
-        // ### ignore for the moment
-        break;
-    case ATTR_ONFOCUS:
-        setHTMLEventListener(EventImpl::FOCUS_EVENT,
-            getDocument()->createHTMLEventListener(attr->value().string()));
-        break;
-    case ATTR_ONBLUR:
-        setHTMLEventListener(EventImpl::BLUR_EVENT,
-            getDocument()->createHTMLEventListener(attr->value().string()));
         break;
     case ATTR_ONCHANGE:
         setHTMLEventListener(EventImpl::CHANGE_EVENT,
-            getDocument()->createHTMLEventListener(attr->value().string()));
+            getDocument()->createHTMLEventListener(attr->value().string(),"onchange"));
         break;
     default:
         HTMLGenericFormElementImpl::parseAttribute(attr);
@@ -1714,12 +1838,10 @@ void HTMLSelectElementImpl::attach()
     assert(parentNode());
     assert(!renderer());
 
-    addCSSProperty(CSS_PROP_COLOR, "text");
-
     RenderStyle* _style = getDocument()->styleSelector()->styleForElement(this);
     _style->ref();
     if (parentNode()->renderer() && _style->display() != NONE) {
-        m_render = new RenderSelect(this);
+        m_render = new (getDocument()->renderArena()) RenderSelect(this);
         m_render->setStyle(_style);
     }
 
@@ -2106,23 +2228,14 @@ void HTMLTextAreaElementImpl::parseAttribute(AttributeImpl *attr)
             m_wrap = ta_NoWrap;
         break;
     case ATTR_ACCESSKEY:
-        // ignore for the moment
-        break;
-    case ATTR_ONFOCUS:
-        setHTMLEventListener(EventImpl::FOCUS_EVENT,
-	    getDocument()->createHTMLEventListener(attr->value().string()));
-        break;
-    case ATTR_ONBLUR:
-        setHTMLEventListener(EventImpl::BLUR_EVENT,
-	    getDocument()->createHTMLEventListener(attr->value().string()));
         break;
     case ATTR_ONSELECT:
         setHTMLEventListener(EventImpl::SELECT_EVENT,
-	    getDocument()->createHTMLEventListener(attr->value().string()));
+	    getDocument()->createHTMLEventListener(attr->value().string(),"onselect"));
         break;
     case ATTR_ONCHANGE:
         setHTMLEventListener(EventImpl::CHANGE_EVENT,
-	    getDocument()->createHTMLEventListener(attr->value().string()));
+	    getDocument()->createHTMLEventListener(attr->value().string(),"onchange"));
         break;
     default:
         HTMLGenericFormElementImpl::parseAttribute(attr);
@@ -2135,13 +2248,10 @@ void HTMLTextAreaElementImpl::attach()
     assert(!m_render);
     assert(parentNode());
 
-    addCSSProperty(CSS_PROP_COLOR, "text");
-    addCSSProperty( CSS_PROP_FONT_FAMILY, "monospace" );
-
     RenderStyle* _style = getDocument()->styleSelector()->styleForElement(this);
     _style->ref();
     if (parentNode()->renderer() && _style->display() != NONE) {
-        m_render = new RenderTextArea(this);
+        m_render = new (getDocument()->renderArena()) RenderTextArea(this);
         m_render->setStyle(_style);
     }
 
@@ -2151,7 +2261,7 @@ void HTMLTextAreaElementImpl::attach()
 
 bool HTMLTextAreaElementImpl::encoding(const QTextCodec* codec, encodingList& encoding, bool)
 {
-    if (name().isEmpty() || !m_render) return false;
+    if (name().isEmpty()) return false;
 
     encoding += fixUpfromUnicode(codec, name().string());
     encoding += fixUpfromUnicode(codec, value().string());
@@ -2167,7 +2277,11 @@ void HTMLTextAreaElementImpl::reset()
 DOMString HTMLTextAreaElementImpl::value()
 {
     if ( m_dirtyvalue) {
-        if ( m_render )  m_value = static_cast<RenderTextArea*>( m_render )->text();
+        if ( m_render )
+            m_value = static_cast<RenderTextArea*>( m_render )->text();
+        else
+            m_value = defaultValue().string();
+
         m_dirtyvalue = false;
     }
 
@@ -2180,7 +2294,7 @@ void HTMLTextAreaElementImpl::setValue(DOMString _value)
 {
     // \r\n -> \n, \r -> \n
     QString str = _value.string().replace( "\r\n", "\n" );
-    m_value = str.replace( "\r", "\n" );
+    m_value = str.replace( '\r', '\n' );
     m_dirtyvalue = false;
     setChanged(true);
 }
@@ -2194,6 +2308,7 @@ DOMString HTMLTextAreaElementImpl::defaultValue()
     for (n = firstChild(); n; n = n->nextSibling())
         if (n->isTextNode())
             val += static_cast<TextImpl*>(n)->data();
+
     if (val[0] == '\r' && val[1] == '\n') {
 	val = val.copy();
 	val.remove(0,2);
