@@ -20,6 +20,7 @@
  * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
  *
+ * $Id: html_formimpl.h,v 1.148.2.2 2004/03/18 13:48:44 lunakl Exp $
  */
 #ifndef HTML_FORMIMPL_H
 #define HTML_FORMIMPL_H
@@ -33,6 +34,9 @@
 #include <qcstring.h>
 #include <qmemarray.h>
 
+#include <kdebug.h>
+
+
 class KHTMLView;
 class QTextCodec;
 
@@ -73,6 +77,7 @@ public:
     void setEnctype( const DOMString & );
 
     bool autoComplete() const { return m_autocomplete; }
+    void doAutoFill();
 
     virtual void parseAttribute(AttributeImpl *attr);
 
@@ -88,6 +93,9 @@ public:
     void submit();
     void reset();
 
+    void setMalformed(bool malformed) { m_malformed = malformed; }
+    bool isMalformed() const { return m_malformed; }
+
     friend class HTMLFormElement;
     friend class HTMLFormCollectionImpl;
 
@@ -105,6 +113,7 @@ private:
     bool m_insubmit : 1;
     bool m_doingsubmit : 1;
     bool m_inreset : 1;
+    bool m_malformed : 1;
 };
 
 // -------------------------------------------------------------------------
@@ -181,11 +190,12 @@ public:
 
     DOMString type() const;
     typeEnum buttonType() const { return m_type; }
-    virtual void attach();
     virtual void parseAttribute(AttributeImpl *attr);
     virtual void defaultEventHandler(EventImpl *evt);
     virtual bool encoding(const QTextCodec*, khtml::encodingList&, bool);
     void activate();
+    virtual void attach();
+    void click();
 
 protected:
     DOMString m_value;
@@ -310,7 +320,6 @@ public:
     virtual ~HTMLLabelElementImpl();
 
     virtual Id id() const;
-    virtual void parseAttribute(AttributeImpl *attr);
     virtual void attach();
 
  private:
