@@ -4,6 +4,7 @@
  * Copyright (C) 1999 Lars Knoll (knoll@kde.org)
  *           (C) 1999 Antti Koivisto (koivisto@kde.org)
  *           (C) 2001 Dirk Mueller (mueller@kde.org)
+ * Copyright (C) 2002 Apple Computer, Inc.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -76,7 +77,7 @@ void HTMLBaseElementImpl::removedFromDocument()
 
     // Since the document doesn't have a base element...
     // (This will break in the case of multiple base elements, but that's not valid anyway (?))
-    getDocument()->setBaseURL( QString::null );
+    getDocument()->setBaseURL( KURL() );
     getDocument()->setBaseTarget( QString::null );
 }
 
@@ -85,8 +86,8 @@ void HTMLBaseElementImpl::process()
     if (!inDocument())
 	return;
 
-    if(!m_href.isEmpty())
-	getDocument()->setBaseURL( KURL( getDocument()->view()->part()->url(), m_href.string() ).url() );
+    if(!m_href.isEmpty() && getDocument()->view())
+	getDocument()->setBaseURL( KURL( getDocument()->view()->part()->url(), m_href.string() ) );
 
     if(!m_target.isEmpty())
 	getDocument()->setBaseTarget( m_target.string() );
@@ -112,25 +113,53 @@ void HTMLLinkElementImpl::parseAttribute(AttributeImpl *attr)
 {
     switch (attr->id())
     {
-    case ATTR_REL:
-        m_rel = attr->value();
-	process();
-        break;
     case ATTR_HREF:
         m_url = getDocument()->completeURL( khtml::parseURL(attr->value()).string() );
 	process();
         break;
+    case ATTR_REL:
     case ATTR_TYPE:
-        m_type = attr->value();
 	process();
         break;
+    case ATTR_TITLE:
+        // ### when title changes we have to reconsider our alternative
+        // stylesheet choice
+        break;
     case ATTR_MEDIA:
         m_media = attr->value().string().lower();
-	process();
+        process();
         break;
-    case ATTR_DISABLED:
-        // ###
+    case ATTR_DISABLED: {
+        bool m_oldisDisabled = m_isDisabled;
+        m_isDisabled = attr->val();
+        if (m_oldisDisabled != m_isDisabled) {
+            if (isLoading()) {
+                if (m_oldisDisabled)
+                    getDocument()->addPendingSheet();
+                else if (!m_alternate)
+                    getDocument()->styleSheetLoaded();
+            }
+            if (m_oldisDisabled) {
+                // enabling: if it's an alternate sheet, pretend it's not.
+                m_alternate = false;
+            } else if (!m_alternate) {
+                // disabling: recheck alternate status
+                QString rel =  getAttribute(ATTR_REL).string().lower();
+                QString type = getAttribute(ATTR_TYPE).string().lower();
+                m_alternate = (type.contains("text/css") || rel.contains("stylesheet")) && rel.contains("alternate");
+            }
+            if (isLoading())
+                break;
+            if (!m_sheet && !m_isDisabled) {
+                process();
+                if (isLoading() && m_alternate)
+                    getDocument()->addPendingSheet();
+                m_alternate = false;
+            } else
+                getDocument()->updateStyleSelector(); // Update the style selector.
+        }
         break;
+    }
     default:
         HTMLElementImpl::parseAttribute(attr);
     }
@@ -139,37 +168,50 @@ void HTMLLinkElementImpl::parseAttribute(AttributeImpl *attr)
 void HTMLLinkElementImpl::process()
 {
     if (!inDocument())
-	return;
+        return;
 
-    QString type = m_type.string().lower();
-    QString rel = m_rel.string().lower();
+    QString type = getAttribute(ATTR_TYPE).string().lower();
+    QString rel = getAttribute(ATTR_REL).string().lower();
 
     KHTMLPart* part = getDocument()->view() ? getDocument()->view()->part() : 0;
 
     // IE extension: location of small icon for locationbar / bookmarks
     // Uses both "shortcut icon" and "icon"
-
     if ( part && rel.contains("icon") && !m_url.isEmpty() && !part->parentPart())
         part->browserExtension()->setIconURL( KURL(m_url.string()) );
 
     // Stylesheet
-    if(type.contains("text/css") || rel.contains("stylesheet")) {
+    else if (!m_isDisabled &&
+        (type.contains("text/css") || rel.contains("stylesheet"))) {
         // no need to load style sheets which aren't for the screen output
         // ### there may be in some situations e.g. for an editor or script to manipulate
         if( m_media.isNull() || m_media.contains("screen") || m_media.contains("all") || m_media.contains("print") ) {
             m_loading = true;
+            // Add ourselves as a pending sheet, but only if we aren't an alternate
+            // stylesheet.  Alternate stylesheets don't hold up render tree construction.
+            m_alternate = rel.contains("alternate");
+            if (!isAlternate())
+                getDocument()->addPendingSheet();
+
             QString chset = getAttribute( ATTR_CHARSET ).string();
             if (m_cachedSheet)
 		m_cachedSheet->deref(this);
             m_cachedSheet = getDocument()->docLoader()->requestStyleSheet(m_url, chset);
-            if (m_cachedSheet)
+            if (m_cachedSheet) {
+                m_isCSSSheet = true;
 		m_cachedSheet->ref(this);
+            }
+            else if (!isAlternate()) {
+                // Error requesting sheet; decrement pending sheet count
+                getDocument()->styleSheetLoaded();
+            }
         }
     }
     else if (m_sheet) {
 	// we no longer contain a stylesheet, e.g. perhaps rel or type was changed
 	m_sheet->deref();
 	m_sheet = 0;
+        m_isCSSSheet = false;
 	getDocument()->updateStyleSelector();
     }
 }
@@ -183,27 +225,41 @@ void HTMLLinkElementImpl::insertedIntoDocument()
 void HTMLLinkElementImpl::removedFromDocument()
 {
     HTMLElementImpl::removedFromDocument();
-    process();
+    getDocument()->updateStyleSelector();
 }
 
 void HTMLLinkElementImpl::setStyleSheet(const DOM::DOMString &url, const DOM::DOMString &sheetStr)
 {
-//    kdDebug( 6030 ) << "HTMLLinkElement::setStyleSheet()" << endl;
-//    kdDebug( 6030 ) << "**** current medium: " << m_media << endl;
-
     if (m_sheet)
-	m_sheet->deref();
+        m_sheet->deref();
     m_sheet = new CSSStyleSheetImpl(this, url);
-    kdDebug( 6030 ) << "style sheet parse mode strict = " << ( getDocument()->parseMode() == DocumentImpl::Strict ) << endl;
     m_sheet->ref();
     m_sheet->parseString( sheetStr, getDocument()->parseMode() == DocumentImpl::Strict );
 
     MediaListImpl *media = new MediaListImpl( m_sheet, m_media );
     m_sheet->setMedia( media );
 
+    finished();
+}
+
+void HTMLLinkElementImpl::finished()
+{
     m_loading = false;
 
-    getDocument()->updateStyleSelector();
+    // Tell the doc about the sheet.
+    if (!isLoading() && !isDisabled() && !isAlternate())
+        getDocument()->styleSheetLoaded();
+
+    // ### major inefficiency, but currently necessary for proper
+    // alternate styles support. don't recalc the styleselector
+    // when nothing actually changed!
+    if ( isAlternate() && m_sheet && !isDisabled())
+        getDocument()->updateStyleSelector();
+}
+
+void HTMLLinkElementImpl::error( int, const QString& )
+{
+    finished();
 }
 
 bool HTMLLinkElementImpl::isLoading() const
@@ -217,7 +273,8 @@ bool HTMLLinkElementImpl::isLoading() const
 
 void HTMLLinkElementImpl::sheetLoaded()
 {
-    getDocument()->updateStyleSelector();
+    if (!isLoading() && !isDisabled() && !isAlternate())
+        getDocument()->styleSheetLoaded();
 }
 
 // -------------------------------------------------------------------------
@@ -305,8 +362,11 @@ void HTMLStyleElementImpl::parseAttribute(AttributeImpl *attr)
     switch (attr->id())
     {
     case ATTR_TYPE:
+        m_type = attr->value().lower();
+        break;
     case ATTR_MEDIA:
-	break;
+        m_media = attr->value().string().lower();
+        break;
     default:
         HTMLElementImpl::parseAttribute(attr);
     }
@@ -315,13 +375,15 @@ void HTMLStyleElementImpl::parseAttribute(AttributeImpl *attr)
 void HTMLStyleElementImpl::insertedIntoDocument()
 {
     HTMLElementImpl::insertedIntoDocument();
-    getDocument()->updateStyleSelector();
+    if (m_sheet)
+        getDocument()->updateStyleSelector();
 }
 
 void HTMLStyleElementImpl::removedFromDocument()
 {
     HTMLElementImpl::removedFromDocument();
-    getDocument()->updateStyleSelector();
+    if (m_sheet)
+        getDocument()->updateStyleSelector();
 }
 
 void HTMLStyleElementImpl::childrenChanged()
@@ -337,23 +399,39 @@ void HTMLStyleElementImpl::childrenChanged()
 	    text += c->nodeValue();
     }
 
-    if(m_sheet)
-	m_sheet->deref();
-    m_sheet = new CSSStyleSheetImpl(this);
-    m_sheet->ref();
-    m_sheet->parseString( text, (getDocument()->parseMode() == DocumentImpl::Strict) );
-    getDocument()->updateStyleSelector();
+    if (m_sheet) {
+        m_sheet->deref();
+        m_sheet = 0;
+    }
+
+    m_loading = false;
+    if ((m_type.isEmpty() || m_type == "text/css") // Type must be empty or CSS
+         && (m_media.isNull() || m_media.contains("screen") || m_media.contains("all") || m_media.contains("print"))) {
+        getDocument()->addPendingSheet();
+        m_loading = true;
+        m_sheet = new CSSStyleSheetImpl(this);
+        m_sheet->ref();
+        m_sheet->parseString( text, (getDocument()->parseMode() == DocumentImpl::Strict) );
+        MediaListImpl* media = new MediaListImpl( m_sheet, DOMString( m_media ) );
+        m_sheet->setMedia( media );
+        m_loading = false;
+    }
+
+    if (!isLoading() && m_sheet)
+        getDocument()->styleSheetLoaded();
 }
 
 bool HTMLStyleElementImpl::isLoading() const
 {
+    if (m_loading) return true;
     if(!m_sheet) return false;
     return static_cast<CSSStyleSheetImpl *>(m_sheet)->isLoading();
 }
 
 void HTMLStyleElementImpl::sheetLoaded()
 {
-    getDocument()->updateStyleSelector();
+    if (!isLoading())
+        getDocument()->styleSheetLoaded();
 }
 
 // -------------------------------------------------------------------------
@@ -363,20 +441,6 @@ NodeImpl::Id HTMLTitleElementImpl::id() const
     return ID_TITLE;
 }
 
-void HTMLTitleElementImpl::insertedIntoDocument()
-{
-    HTMLElementImpl::insertedIntoDocument();
-    getDocument()->setTitle(m_title);
-}
-
-void HTMLTitleElementImpl::removedFromDocument()
-{
-    HTMLElementImpl::removedFromDocument();
-    // Title element removed, so we have no title... we ignore the case of multiple title elements, as it's invalid
-    // anyway (?)
-    getDocument()->setTitle(DOMString());
-}
-
 void HTMLTitleElementImpl::childrenChanged()
 {
     HTMLElementImpl::childrenChanged();
@@ -386,8 +450,8 @@ void HTMLTitleElementImpl::childrenChanged()
 	if ((c->nodeType() == Node::TEXT_NODE) || (c->nodeType() == Node::CDATA_SECTION_NODE))
 	    m_title += c->nodeValue();
     }
-    if (inDocument())
-	getDocument()->setTitle(m_title);
+    if ( !m_title.isEmpty() && inDocument())
+        getDocument()->setTitle(m_title);
 }
 
 DOMString HTMLTitleElementImpl::text()
