@@ -123,15 +123,15 @@ void HTMLImageElementImpl::parseAttribute(AttributeImpl *attr)
         break;
     case ATTR_ONABORT: // ### add support for this
         setHTMLEventListener(EventImpl::ABORT_EVENT,
-	    getDocument()->createHTMLEventListener(attr->value().string()));
+	    getDocument()->createHTMLEventListener(attr->value().string(),"onabort"));
         break;
-    case ATTR_ONERROR: // ### add support for this
+    case ATTR_ONERROR:
         setHTMLEventListener(EventImpl::ERROR_EVENT,
-	    getDocument()->createHTMLEventListener(attr->value().string()));
+	    getDocument()->createHTMLEventListener(attr->value().string(),"onerror"));
         break;
     case ATTR_ONLOAD:
         setHTMLEventListener(EventImpl::LOAD_EVENT,
-	    getDocument()->createHTMLEventListener(attr->value().string()));
+	    getDocument()->createHTMLEventListener(attr->value().string(),"onload"));
         break;
     case ATTR_NAME:
     case ATTR_NOSAVE:
@@ -171,8 +171,10 @@ void HTMLImageElementImpl::attach()
 
     RenderStyle* _style = getDocument()->styleSelector()->styleForElement(this);
     _style->ref();
-    if (parentNode()->renderer() && _style->display() != NONE) {
-        m_render = new RenderImage(this);
+    if (parentNode()->renderer() && parentNode()->renderer()->childAllowed() &&
+        _style->display() != NONE)
+    {
+        m_render = new (getDocument()->renderArena()) RenderImage(this);
         m_render->setStyle(getDocument()->styleSelector()->styleForElement(this));
         parentNode()->renderer()->addChild(m_render, nextRenderer());
         m_render->updateFromElement();
@@ -325,7 +327,7 @@ HTMLAreaElementImpl::HTMLAreaElementImpl(DocumentPtr *doc)
 
 HTMLAreaElementImpl::~HTMLAreaElementImpl()
 {
-    if (m_coords) delete [] m_coords;
+    delete [] m_coords;
 }
 
 NodeImpl::Id HTMLAreaElementImpl::id() const
@@ -342,13 +344,13 @@ void HTMLAreaElementImpl::parseAttribute(AttributeImpl *attr)
             shape = Default;
         else if ( strcasecmp( attr->value(), "circle" ) == 0 )
             shape = Circle;
-        else if ( strcasecmp( attr->value(), "poly" ) == 0 )
+        else if ( strcasecmp( attr->value(), "poly" ) == 0 || strcasecmp( attr->value(),  "polygon" ) == 0 )
             shape = Poly;
         else if ( strcasecmp( attr->value(), "rect" ) == 0 )
             shape = Rect;
         break;
     case ATTR_COORDS:
-        if (m_coords) delete [] m_coords;
+        delete [] m_coords;
         m_coords = attr->val()->toLengthArray(m_coordsLen);
         break;
     case ATTR_NOHREF:
