@@ -1,9 +1,11 @@
 /**
  * This file is part of the DOM implementation for KDE.
  *
- * Copyright (C) 1999 Lars Knoll (knoll@kde.org)
+ * Copyright (C) 1999-2003 Lars Knoll (knoll@kde.org)
  *           (C) 1999 Antti Koivisto (koivisto@kde.org)
  *           (C) 2000 Simon Hausmann <hausmann@kde.org>
+ *           (C) 2001-2003 Dirk Mueller (mueller@kde.org)
+ *           (C) 2002 Apple Computer, Inc.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -49,13 +51,12 @@ NodeImpl::Id HTMLAnchorElementImpl::id() const
 
 void HTMLAnchorElementImpl::defaultEventHandler(EventImpl *evt)
 {
-    bool keydown = evt->id() == EventImpl::KHTML_KEYPRESS_EVENT ||
-                   evt->id() == EventImpl::KHTML_KEYDOWN_EVENT;
+    bool keydown = evt->id() == EventImpl::KEYDOWN_EVENT;
 
     // React on clicks and on keypresses.
     // Don't make this KEYUP_EVENT again, it makes khtml follow links
     // it shouldn't, when pressing Enter in the combo.
-    if ( ( (evt->id() == EventImpl::CLICK_EVENT && static_cast<MouseEventImpl*>(evt)->detail() == 1) ||
+    if ( ( (evt->id() == EventImpl::CLICK_EVENT && !static_cast<MouseEventImpl*>(evt)->isDoubleClick()) ||
          ( keydown && m_focused)) && m_hasAnchor) {
 
         MouseEventImpl *e = 0;
@@ -97,10 +98,17 @@ void HTMLAnchorElementImpl::defaultEventHandler(EventImpl *evt)
                 khtml::RenderImage *r = static_cast<khtml::RenderImage *>(img->renderer());
                 if(r && e)
                 {
-                    int absx, absy;
+                    KHTMLView* v = getDocument()->view();
+                    int x = e->clientX();
+                    int y = e->clientY();
+                    int absx = 0;
+                    int absy = 0;
+                    if ( v ) {
+                        x += v->contentsX();
+                        y += v->contentsY();
+                    }
                     r->absolutePosition(absx, absy);
-                    int x(e->clientX() - absx), y(e->clientY() - absy);
-                    url += QString("?%1,%2").arg( x ).arg( y );
+                    url += QString("?%1,%2").arg( x - absx ).arg( y - absy );
                 }
                 else {
                     evt->setDefaultHandled();
@@ -120,7 +128,7 @@ void HTMLAnchorElementImpl::defaultEventHandler(EventImpl *evt)
                     state |= Qt::ShiftButton;
                 if ( e->altKey() )
                     state |= Qt::AltButton;
-                if (  e->metaKey() )
+                if ( e->metaKey() )
                     state |= Qt::MetaButton;
 
                 if ( e->button() == 0 )
@@ -140,8 +148,11 @@ void HTMLAnchorElementImpl::defaultEventHandler(EventImpl *evt)
                 state |= Qt::ControlButton;
 	    }
 
-            getDocument()->view()->part()->
-                urlSelected( url, button, state, utarget );
+	    // ### also check if focused node is editable if not in designmode,
+	    // and prevent link loading then (LS)
+	    if (getDocument()->view() && !getDocument()->designMode())
+		getDocument()->view()->part()->
+		    urlSelected( url, button, state, utarget );
         }
         evt->setDefaultHandled();
     }
@@ -149,6 +160,12 @@ void HTMLAnchorElementImpl::defaultEventHandler(EventImpl *evt)
 }
 
 
+void HTMLAnchorElementImpl::click()
+{
+    QMouseEvent me(QEvent::MouseButtonRelease, QPoint(0,0),Qt::LeftButton, 0);
+    dispatchMouseEvent(&me,EventImpl::CLICK_EVENT, 1);
+}
+
 void HTMLAnchorElementImpl::parseAttribute(AttributeImpl *attr)
 {
     switch(attr->id())
@@ -163,6 +180,8 @@ void HTMLAnchorElementImpl::parseAttribute(AttributeImpl *attr)
     case ATTR_TITLE:
     case ATTR_REL:
 	break;
+    case ATTR_ACCESSKEY:
+        break;
     default:
         HTMLElementImpl::parseAttribute(attr);
     }
@@ -199,12 +218,12 @@ void HTMLBRElementImpl::attach()
     assert(parentNode());
 
     if (parentNode()->renderer()) {
-        RenderStyle* style = getDocument()->styleSelector()->styleForElement(  this );
+        RenderStyle* style = getDocument()->styleSelector()->styleForElement( this );
         style->ref();
-        if(  style->display() != NONE ) {
-          m_render = new RenderBR( this );
-          m_render->setStyle( style );
-          parentNode()->renderer()->addChild( m_render, nextRenderer() );
+        if( style->display() != NONE ) {
+          m_render = new (getDocument()->renderArena()) RenderBR(this);
+          m_render->setStyle(style);
+          parentNode()->renderer()->addChild(m_render, nextRenderer());
         }
         style->deref();
     }
@@ -224,36 +243,42 @@ void HTMLFontElementImpl::parseAttribute(AttributeImpl *attr)
     {
     case ATTR_SIZE:
     {
-        DOMString s = attr->value();
-        if(!s.isNull()) {
-            int num = s.toInt();
-            if ( *s.unicode() == '+' || *s.unicode() == '-' ) {
+        DOMStringImpl* v = attr->val();
+        if(v) {
+            const QChar* s = v->s;
+            int num = v->toInt();
+            int len = v->l;
+            while( len && s->isSpace() )
+              len--,s++;
+            if ( len && *s == '+' )
                 num += 3;
-            }
-            int size = 0;
+            int size;
             switch (num)
             {
-            case 1: size = CSS_VAL_X_SMALL; break;
-            case 2: size = CSS_VAL_SMALL;   break;
-            case 3: size = CSS_VAL_MEDIUM;  break;
-            case 4: size = CSS_VAL_LARGE;   break;
-            case 5: size = CSS_VAL_X_LARGE; break;
-            case 6: size = CSS_VAL_XX_LARGE;break;
+            case -2:
+            case  1: size = CSS_VAL_X_SMALL;  break;
+            case -1:
+            case  2: size = CSS_VAL_SMALL;    break;
+            case  0: // treat 0 the same as 3, because people
+                     // expect it to be between -1 and +1
+            case  3: size = CSS_VAL_MEDIUM;   break;
+            case  4: size = CSS_VAL_LARGE;    break;
+            case  5: size = CSS_VAL_X_LARGE;  break;
+            case  6: size = CSS_VAL_XX_LARGE; break;
             default:
-                if (num >= 6)
-                    size = CSS_VAL__KONQ_XXX_LARGE;
-                else if (num < 1)
+                if (num > 6)
+                    size = CSS_VAL__KHTML_XXX_LARGE;
+                else
                     size = CSS_VAL_XX_SMALL;
             }
-            if ( size )
-                addCSSProperty(CSS_PROP_FONT_SIZE, size);
+            addCSSProperty(CSS_PROP_FONT_SIZE, size);
         }
         break;
     }
     case ATTR_COLOR:
-        addCSSProperty(CSS_PROP_COLOR, attr->value());
+        addHTMLColor(CSS_PROP_COLOR, attr->value());
         // HTML4 compatibility hack
-        addCSSProperty(CSS_PROP_TEXT_DECORATION_COLOR, attr->value());
+        addHTMLColor(CSS_PROP__KHTML_TEXT_DECORATION_COLOR, attr->value());
         break;
     case ATTR_FACE:
         addCSSProperty(CSS_PROP_FONT_FAMILY, attr->value());
