@@ -19,20 +19,17 @@
  * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
  *
- * $Id: html_objectimpl.h,v 1.46.2.3 2003/05/18 12:34:39 mueller Exp $
+ * $Id: html_objectimpl.h,v 1.55.2.2 2004/02/29 15:27:56 mueller Exp $
  */
 #ifndef HTML_OBJECTIMPL_H
 #define HTML_OBJECTIMPL_H
 
 #include "html_elementimpl.h"
 #include "xml/dom_stringimpl.h"
-#include "java/kjavaapplet.h"
 #include <kparts/browserextension.h>
 #include <qstringlist.h>
 
 class KHTMLView;
-class QTimer;
-class KJavaApplet;
 
 // -------------------------------------------------------------------------
 namespace DOM {
@@ -40,11 +37,19 @@ namespace DOM {
 class HTMLFormElementImpl;
 class DOMStringImpl;
 
-class LiveConnectElementImpl : public QObject, public HTMLElementImpl
+class HTMLObjectBaseElementImpl : public QObject, public HTMLElementImpl
 {
     Q_OBJECT
 public:
-    LiveConnectElementImpl(DocumentPtr *doc);
+    HTMLObjectBaseElementImpl(DocumentPtr *doc);
+
+    virtual void parseAttribute(AttributeImpl *attr);
+    virtual void attach();
+    virtual void detach();
+
+    virtual void recalcStyle( StyleChange ch );
+
+    void renderAlternative();
 
     bool get(const unsigned long, const QString &, KParts::LiveConnectExtension::Type &, unsigned long &, QString &);
     bool put(const unsigned long, const QString &, const QString &);
@@ -52,24 +57,25 @@ public:
     void unregister(const unsigned long);
 
     void setLiveConnect(KParts::LiveConnectExtension * lc);
+    void setServiceType(const QString &);
 
-    virtual void detach();
+    QString url;
+    QString classId;
+    QString serviceType;
+    bool needWidgetUpdate;
+    bool m_renderAlternative;
 
 protected slots:
     void liveConnectEvent(const unsigned long, const QString&, const KParts::LiveConnectExtension::ArgList&);
-
-private slots:
-    void timerDone();
+    void slotRenderAlternative();
 
 private:
     KParts::LiveConnectExtension *liveconnect;
-    QTimer *timer;
-    QString script;
 };
 
 // -------------------------------------------------------------------------
 
-class HTMLAppletElementImpl : public LiveConnectElementImpl
+class HTMLAppletElementImpl : public HTMLObjectBaseElementImpl
 {
 public:
     HTMLAppletElementImpl(DocumentPtr *doc);
@@ -81,14 +87,13 @@ public:
     virtual void parseAttribute(AttributeImpl *token);
     virtual void attach();
 
-    KJavaApplet* applet() const;
 protected:
     khtml::VAlign valign;
 };
 
 // -------------------------------------------------------------------------
 
-class HTMLEmbedElementImpl : public LiveConnectElementImpl
+class HTMLEmbedElementImpl : public HTMLObjectBaseElementImpl
 {
 public:
     HTMLEmbedElementImpl(DocumentPtr *doc);
@@ -99,15 +104,13 @@ public:
     virtual void parseAttribute(AttributeImpl *attr);
     virtual void attach();
 
-    QString url;
     QString pluginPage;
-    QString serviceType;
     bool hidden;
 };
 
 // -------------------------------------------------------------------------
 
-class HTMLObjectElementImpl : public LiveConnectElementImpl
+class HTMLObjectElementImpl : public HTMLObjectBaseElementImpl
 {
 public:
     HTMLObjectElementImpl(DocumentPtr *doc);
@@ -121,19 +124,8 @@ public:
     virtual void parseAttribute(AttributeImpl *token);
 
     virtual void attach();
-    virtual void detach();
-
-    virtual void recalcStyle( StyleChange ch );
 
     DocumentImpl* contentDocument() const;
-
-    void renderAlternative();
-
-    QString serviceType;
-    QString url;
-    QString classId;
-    bool needWidgetUpdate;
-    bool m_renderAlternative;
 };
 
 // -------------------------------------------------------------------------
@@ -156,5 +148,5 @@ public:
     QString m_value;
 };
 
-}
+} // namespace
 #endif
