@@ -4,7 +4,7 @@
  * Copyright (C) 1997 Martin Jones (mjones@kde.org)
  *           (C) 1997 Torben Weis (weis@kde.org)
  *           (C) 1998 Waldo Bastian (bastian@kde.org)
- *           (C) 1999 Lars Knoll (knoll@kde.org)
+ *           (C) 1999-2003 Lars Knoll (knoll@kde.org)
  *           (C) 1999 Antti Koivisto (koivisto@kde.org)
  *
  * This library is free software; you can redistribute it and/or
@@ -66,10 +66,9 @@ HTMLTableElementImpl::HTMLTableElementImpl(DocumentPtr *doc)
     // reset font color and sizes here, if we don't have strict parse mode.
     // this is 90% compatible to ie and mozilla, and the by way easiest solution...
     // only difference to 100% correct is that in strict mode <font> elements are propagated into tables.
-    if ( getDocument()->parseMode() != DocumentImpl::Strict ) {
+    if ( getDocument()->parseMode() < DocumentImpl::Transitional ) {
         addCSSProperty( CSS_PROP_FONT_SIZE, CSS_VAL_MEDIUM );
-        addCSSProperty( CSS_PROP_COLOR, "-konq-text" );
-        addCSSProperty( CSS_PROP_FONT_FAMILY, "konq_default" );
+        addCSSProperty( CSS_PROP_COLOR, CSS_VAL__KHTML_TEXT );
     }
 }
 
@@ -122,8 +121,7 @@ NodeImpl* HTMLTableElementImpl::setTFoot( HTMLTableSectionElementImpl *s )
     if(foot) {
         replaceChild ( s, foot, exceptioncode );
         r = s;
-    }
-    else if( firstBody )
+    } else if( firstBody )
         r = insertBefore( s, firstBody, exceptioncode );
     else
         r = appendChild( s, exceptioncode );
@@ -136,13 +134,12 @@ NodeImpl* HTMLTableElementImpl::setTBody( HTMLTableSectionElementImpl *s )
     int exceptioncode = 0;
     NodeImpl* r;
 
-    if(!firstBody)
-        firstBody = s;
-
-    if( foot )
-        r = insertBefore( s, foot, exceptioncode );
-    else
+    if(firstBody) {
+        replaceChild ( s, firstBody, exceptioncode );
+        r = s;
+    } else
         r = appendChild( s, exceptioncode );
+    firstBody = s;
 
     return r;
 }
@@ -155,7 +152,7 @@ HTMLElementImpl *HTMLTableElementImpl::createTHead(  )
         head = new HTMLTableSectionElementImpl(docPtr(), ID_THEAD, true /* implicit */);
         if(foot)
             insertBefore( head, foot, exceptioncode );
-        if(firstBody)
+        else if(firstBody)
             insertBefore( head, firstBody, exceptioncode);
         else
             appendChild(head, exceptioncode);
@@ -224,57 +221,47 @@ HTMLElementImpl *HTMLTableElementImpl::insertRow( long index, int &exceptioncode
     if(!firstBody && !head && !foot && !hasChildNodes())
         setTBody( new HTMLTableSectionElementImpl(docPtr(), ID_TBODY, true /* implicit */) );
 
-    //kdDebug(6030) << k_funcinfo << index << endl;
-    // IE treats index=-1 as default value meaning 'append after last'
-    // This isn't in the DOM. So, not implemented yet.
+    kdDebug(6030) << k_funcinfo << index << endl;
     HTMLTableSectionElementImpl* section = 0L;
     HTMLTableSectionElementImpl* lastSection = 0L;
     NodeImpl *node = firstChild();
-    bool append = index == -1;
+    // The DOM requires that index=-1 means 'append after last'
+    bool append = (index == -1);
+    bool found = false;
     for ( ; node && (index>=0 || append) ; node = node->nextSibling() )
     {
-        if ( node->id() == ID_THEAD || node->id() == ID_TFOOT || node->id() == ID_TBODY )
+	// there could be 2 tfoot elements in the table. Only the first one is the "foot", that's why we have the more
+	// complicated if statement below.
+        if ( node != foot && (node->id() == ID_THEAD || node->id() == ID_TFOOT || node->id() == ID_TBODY) )
         {
             section = static_cast<HTMLTableSectionElementImpl *>(node);
             lastSection = section;
-            //kdDebug(6030) << k_funcinfo << "section id=" << node->id() << " rows:" << section->numRows() << endl;
+            //kdDebug(6030) << k_funcinfo << "section=" << section->tagName() << " rows:" << section->numRows() << endl;
             if ( !append )
             {
                 int rows = section->numRows();
-                if ( rows > index )
+                if ( rows > index ) {
+		    found = true;
                     break;
-                else
+                } else
                     index -= rows;
                 //kdDebug(6030) << "       index is now " << index << endl;
             }
         }
-        // Note: we now can have both TR and sections (THEAD/TBODY/TFOOT) as children of a TABLE
-        else if ( node->id() == ID_TR )
-        {
-            section = 0L;
-            //kdDebug(6030) << k_funcinfo << "row" << endl;
-            if (!append && !index)
-            {
-                // Insert row right here, before "node"
-                HTMLTableRowElementImpl* row = new HTMLTableRowElementImpl(docPtr());
-                insertBefore(row, node, exceptioncode );
-                return row;
-            }
-            if ( !append )
-                index--;
-            //kdDebug(6030) << "       index is now " << index << endl;
-        }
-        section = 0L;
     }
-    // Index == 0 means "insert before first row in current section"
+    // insertRow(numRows) appends to TFOOT. insertRow(-1) appends to TBODY, hence the !append.
+    if ( !found && !append )
+        section = static_cast<HTMLTableSectionElementImpl *>(foot);
+
+    // If index has decreased to 0, it means "insert before first row in current section"
     // or "append after last row" (if there's no current section anymore)
     if ( !section && ( index == 0 || append ) )
     {
         section = lastSection;
         index = section ? section->numRows() : 0;
     }
-    if ( section && ( index >= 0 || append ) ) {
-        //kdDebug(6030) << "Inserting row into section " << section << " at index " << index << endl;
+    if ( section && (index >= 0 || append) ) {
+        //kdDebug(6030) << "Inserting row into section " << section << "(" << section->tagName().string() << ") at index " << index << endl;
         return section->insertRow( index, exceptioncode );
     } else {
         // No more sections => index is too big
@@ -289,23 +276,28 @@ void HTMLTableElementImpl::deleteRow( long index, int &exceptioncode )
     NodeImpl *node = firstChild();
     bool lastRow = index == -1;
     HTMLTableSectionElementImpl* lastSection = 0L;
+    bool found = false;
     for ( ; node ; node = node->nextSibling() )
     {
-        if ( node->id() == ID_THEAD || node->id() == ID_TFOOT || node->id() == ID_TBODY )
+        if ( node != foot && (node->id() == ID_THEAD || node->id() == ID_TFOOT || node->id() == ID_TBODY) )
         {
             section = static_cast<HTMLTableSectionElementImpl *>(node);
             lastSection = section;
             int rows = section->numRows();
             if ( !lastRow )
             {
-                if ( rows > index )
+                if ( rows > index ) {
+                    found = true;
                     break;
-                else
+                } else
                     index -= rows;
             }
         }
         section = 0L;
     }
+    if ( !found && foot )
+        section = static_cast<HTMLTableSectionElementImpl *>(foot);
+
     if ( lastRow )
         lastSection->deleteRow( -1, exceptioncode );
     else if ( section && index >= 0 && index < section->numRows() )
@@ -320,30 +312,32 @@ NodeImpl *HTMLTableElementImpl::addChild(NodeImpl *child)
     kdDebug( 6030 ) << nodeName().string() << "(Table)::addChild( " << child->nodeName().string() << " )" << endl;
 #endif
 
-    switch(child->id())
-    {
-    case ID_CAPTION:
-        return setCaption(static_cast<HTMLTableCaptionElementImpl *>(child));
-        break;
-    case ID_COL:
-    case ID_COLGROUP:
-        if(head || foot || firstBody)
-            return 0;
-	{
-	    int exceptioncode = 0;
-	    return appendChild(child, exceptioncode);
+    int exceptioncode = 0;
+    NodeImpl *retval = appendChild( child, exceptioncode );
+    if ( retval ) {
+	switch(child->id()) {
+	case ID_CAPTION:
+	    if ( !tCaption )
+		tCaption = static_cast<HTMLTableCaptionElementImpl *>(child);
+	    break;
+	case ID_COL:
+	case ID_COLGROUP:
+	    break;
+	case ID_THEAD:
+	    if ( !head )
+		head = static_cast<HTMLTableSectionElementImpl *>(child);
+	    break;
+	case ID_TFOOT:
+	    if ( !foot )
+		foot = static_cast<HTMLTableSectionElementImpl *>(child);
+	    break;
+	case ID_TBODY:
+	    if ( !firstBody )
+		firstBody = static_cast<HTMLTableSectionElementImpl *>(child);
+	    break;
 	}
-    case ID_THEAD:
-        return setTHead(static_cast<HTMLTableSectionElementImpl *>(child));
-        break;
-    case ID_TFOOT:
-        return setTFoot(static_cast<HTMLTableSectionElementImpl *>(child));
-        break;
-    case ID_TBODY:
-        return setTBody(static_cast<HTMLTableSectionElementImpl *>(child));
-        break;
     }
-    return 0;
+    return retval;
 }
 
 void HTMLTableElementImpl::parseAttribute(AttributeImpl *attr)
@@ -391,13 +385,13 @@ void HTMLTableElementImpl::parseAttribute(AttributeImpl *attr)
     }
     case ATTR_BGCOLOR:
         if (!attr->value().isEmpty())
-            addCSSProperty(CSS_PROP_BACKGROUND_COLOR, attr->value());
+            addHTMLColor(CSS_PROP_BACKGROUND_COLOR, attr->value());
         else
             removeCSSProperty(CSS_PROP_BACKGROUND_COLOR);
         break;
     case ATTR_BORDERCOLOR:
         if(!attr->value().isEmpty()) {
-            addCSSProperty(CSS_PROP_BORDER_COLOR, attr->value());
+            addHTMLColor(CSS_PROP_BORDER_COLOR, attr->value());
             addCSSProperty(CSS_PROP_BORDER_TOP_STYLE, CSS_VAL_SOLID);
             addCSSProperty(CSS_PROP_BORDER_BOTTOM_STYLE, CSS_VAL_SOLID);
             addCSSProperty(CSS_PROP_BORDER_LEFT_STYLE, CSS_VAL_SOLID);
@@ -526,7 +520,7 @@ void HTMLTablePartElementImpl::parseAttribute(AttributeImpl *attr)
     {
     case ATTR_BGCOLOR:
         if (attr->val())
-            addCSSProperty(CSS_PROP_BACKGROUND_COLOR, attr->value() );
+            addHTMLColor(CSS_PROP_BACKGROUND_COLOR, attr->value() );
         else
             removeCSSProperty(CSS_PROP_BACKGROUND_COLOR);
         break;
@@ -544,15 +538,22 @@ void HTMLTablePartElementImpl::parseAttribute(AttributeImpl *attr)
     case ATTR_BORDERCOLOR:
     {
         if(!attr->value().isEmpty()) {
-            addCSSProperty(CSS_PROP_BORDER_COLOR, attr->value());
+            addHTMLColor(CSS_PROP_BORDER_COLOR, attr->value());
             addCSSProperty(CSS_PROP_BORDER_TOP_STYLE, CSS_VAL_SOLID);
             addCSSProperty(CSS_PROP_BORDER_BOTTOM_STYLE, CSS_VAL_SOLID);
             addCSSProperty(CSS_PROP_BORDER_LEFT_STYLE, CSS_VAL_SOLID);
             addCSSProperty(CSS_PROP_BORDER_RIGHT_STYLE, CSS_VAL_SOLID);
-            m_solid = true;
         }
         break;
     }
+    case ATTR_ALIGN:
+    {
+        DOMString v = attr->value();
+        if ( strcasecmp( attr->value(), "center" ) == 0 )
+            v = "\\2d khtml-center";
+        addCSSProperty(CSS_PROP_TEXT_ALIGN, v);
+        break;
+    }
     case ATTR_VALIGN:
     {
         if (!attr->value().isEmpty())
@@ -651,15 +652,6 @@ int HTMLTableSectionElementImpl::numRows() const
 
 // -------------------------------------------------------------------------
 
-HTMLTableRowElementImpl::HTMLTableRowElementImpl(DocumentPtr *doc)
-  : HTMLTablePartElementImpl(doc)
-{
-}
-
-HTMLTableRowElementImpl::~HTMLTableRowElementImpl()
-{
-}
-
 NodeImpl::Id HTMLTableRowElementImpl::id() const
 {
     return ID_TR;
@@ -667,26 +659,39 @@ NodeImpl::Id HTMLTableRowElementImpl::id() const
 
 long HTMLTableRowElementImpl::rowIndex() const
 {
-    // some complex traversal stuff here to take into account that some rows may be in different sections
     int rIndex = 0;
-    const NodeImpl *n = this;
-    do {
-        while (!n->previousSibling() && !(n->isElementNode() && n->id() == ID_TABLE))
-            n = n->parentNode();
-        if (n->isElementNode() && n->id() == ID_TABLE)
-            n = 0;
-        if (n) {
-            n = n->previousSibling();
-            while (!(n->isElementNode() && n->id() == ID_TR) && n->lastChild())
-                n = n->lastChild();
-        }
 
-        if (n && n->isElementNode() && n->id() == ID_TR)
-            rIndex++;
+    NodeImpl *table = parentNode();
+    if ( !table )
+	return -1;
+    table = table->parentNode();
+    if ( !table || table->id() != ID_TABLE )
+	return -1;
+
+    HTMLTableSectionElementImpl *foot = static_cast<HTMLTableElementImpl *>(table)->tFoot();
+    NodeImpl *node = table->firstChild();
+    while ( node ) {
+        if ( node != foot && (node->id() == ID_THEAD || node->id() == ID_TFOOT || node->id() == ID_TBODY) ) {
+	    HTMLTableSectionElementImpl* section = static_cast<HTMLTableSectionElementImpl *>(node);
+	    const NodeImpl *row = section->firstChild();
+	    while ( row ) {
+		if ( row == this )
+		    return rIndex;
+		rIndex++;
+		row = row->nextSibling();
+	    }
+	}
+	node = node->nextSibling();
     }
-    while (n && n->isElementNode() && n->id() == ID_TR);
-
-    return rIndex;
+    const NodeImpl *row = foot->firstChild();
+    while ( row ) {
+	if ( row == this )
+	    return rIndex;
+	rIndex++;
+	row = row->nextSibling();
+    }
+    // should never happen
+    return -1;
 }
 
 long HTMLTableRowElementImpl::sectionRowIndex() const
@@ -748,9 +753,9 @@ HTMLTableCellElementImpl::HTMLTableCellElementImpl(DocumentPtr *doc, int tag)
   _col = -1;
   _row = -1;
   cSpan = rSpan = 1;
-  m_nowrap = false;
   _id = tag;
   rowHeight = 0;
+  m_solid = false;
 }
 
 HTMLTableCellElementImpl::~HTMLTableCellElementImpl()
@@ -761,6 +766,17 @@ void HTMLTableCellElementImpl::parseAttribute(AttributeImpl *attr)
 {
     switch(attr->id())
     {
+    case ATTR_ALIGN:
+        if (attr->val()) {
+            if ( strcasecmp(attr->value(), "middle" ) == 0 ||
+                strcasecmp(attr->value(), "center" ) == 0 )
+                addCSSProperty( CSS_PROP_TEXT_ALIGN, CSS_VAL__KHTML_CENTER );
+            else
+                addCSSProperty(CSS_PROP_TEXT_ALIGN, attr->value());
+        }
+        else
+            removeCSSProperty(CSS_PROP_TEXT_ALIGN);
+        break;
     case ATTR_BORDER:
         // euhm? not supported by other browsers as far as I can see (Dirk)
         //addCSSLength(CSS_PROP_BORDER_WIDTH, attr->value());
@@ -778,7 +794,10 @@ void HTMLTableCellElementImpl::parseAttribute(AttributeImpl *attr)
         if(cSpan < 1 || cSpan > 1024) cSpan = 1;
         break;
     case ATTR_NOWRAP:
-        m_nowrap = (attr->val() != 0);
+        if (attr->val() != 0)
+	    addCSSProperty(CSS_PROP_WHITE_SPACE, CSS_VAL__KHTML_NOWRAP);
+        else
+	    removeCSSProperty(CSS_PROP_WHITE_SPACE);
         break;
     case ATTR_WIDTH:
         if (!attr->value().isEmpty())
@@ -813,7 +832,7 @@ void HTMLTableCellElementImpl::attach()
             addCSSProperty(CSS_PROP_BORDER_RIGHT_STYLE, v);
 
             if (!m_solid)
-                addCSSProperty(CSS_PROP_BORDER_COLOR, "inherit");
+                addCSSProperty(CSS_PROP_BORDER_COLOR, CSS_VAL_INHERIT);
         }
     }
 
@@ -829,10 +848,6 @@ HTMLTableColElementImpl::HTMLTableColElementImpl(DocumentPtr *doc, ushort i)
     _span = (_id == ID_COLGROUP ? 0 : 1);
 }
 
-HTMLTableColElementImpl::~HTMLTableColElementImpl()
-{
-}
-
 NodeImpl::Id HTMLTableColElementImpl::id() const
 {
     return _id;
@@ -866,15 +881,6 @@ void HTMLTableColElementImpl::parseAttribute(AttributeImpl *attr)
 
 // -------------------------------------------------------------------------
 
-HTMLTableCaptionElementImpl::HTMLTableCaptionElementImpl(DocumentPtr *doc)
-  : HTMLTablePartElementImpl(doc)
-{
-}
-
-HTMLTableCaptionElementImpl::~HTMLTableCaptionElementImpl()
-{
-}
-
 NodeImpl::Id HTMLTableCaptionElementImpl::id() const
 {
     return ID_CAPTION;
