@@ -22,7 +22,7 @@
  * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
  *
- * $Id: html_tableimpl.h,v 1.65.2.1 2003/05/18 12:34:39 mueller Exp $
+ * $Id: html_tableimpl.h,v 1.70 2003/02/11 01:49:54 mueller Exp $
  */
 #ifndef HTML_TABLEIMPL_H
 #define HTML_TABLEIMPL_H
@@ -124,13 +124,10 @@ class HTMLTablePartElementImpl : public HTMLElementImpl
 {
 public:
     HTMLTablePartElementImpl(DocumentPtr *doc)
-        : HTMLElementImpl(doc), m_solid(false)
+        : HTMLElementImpl(doc)
         { }
 
     virtual void parseAttribute(AttributeImpl *attr);
-
-protected:
-    bool m_solid : 1;
 };
 
 // -------------------------------------------------------------------------
@@ -158,9 +155,8 @@ protected:
 class HTMLTableRowElementImpl : public HTMLTablePartElementImpl
 {
 public:
-    HTMLTableRowElementImpl(DocumentPtr *doc);
-
-    ~HTMLTableRowElementImpl();
+    HTMLTableRowElementImpl(DocumentPtr *doc)
+        : HTMLTablePartElementImpl(doc) {}
 
     virtual Id id() const;
 
@@ -192,8 +188,7 @@ public:
 
     int colSpan() const { return cSpan; }
     int rowSpan() const { return rSpan; }
-    bool noWrap() const { return m_nowrap; }
-
+    
     virtual Id id() const { return _id; }
     virtual void parseAttribute(AttributeImpl *attr);
     virtual void attach();
@@ -205,7 +200,7 @@ protected:
     int cSpan;
     int _id;
     int rowHeight;
-
+    bool m_solid        : 1;
     bool m_nowrap : 1;
 };
 
@@ -216,8 +211,6 @@ class HTMLTableColElementImpl : public HTMLTablePartElementImpl
 public:
     HTMLTableColElementImpl(DocumentPtr *doc, ushort i);
 
-    ~HTMLTableColElementImpl();
-
     virtual Id id() const;
 
     void setTable(HTMLTableElementImpl *t) { table = t; }
@@ -241,12 +234,10 @@ protected:
 class HTMLTableCaptionElementImpl : public HTMLTablePartElementImpl
 {
 public:
-    HTMLTableCaptionElementImpl(DocumentPtr *doc);
-
-    ~HTMLTableCaptionElementImpl();
+    HTMLTableCaptionElementImpl(DocumentPtr *doc)
+        : HTMLTablePartElementImpl(doc) {}
 
     virtual Id id() const;
-
     virtual void parseAttribute(AttributeImpl *attr);
 };
 
