@@ -70,15 +70,17 @@ public:
                    int _level,
                    DOM::NodeImpl *_node,
 		   bool _inline,
-                   HTMLStackElem * _next
-        )
+                   HTMLStackElem * _next )
         :
         id(_id),
         level(_level),
 	m_inline(_inline),
         node(_node),
         next(_next)
-        { }
+        { node->ref(); }
+
+    ~HTMLStackElem()
+        { node->deref(); }
 
     int       id;
     int       level;
@@ -110,8 +112,8 @@ public:
  *    element or ignore the tag.
  *
  */
-KHTMLParser::KHTMLParser
-( KHTMLView *_parent, DocumentPtr *doc)
+
+KHTMLParser::KHTMLParser( KHTMLView *_parent, DocumentPtr *doc)
 {
     //kdDebug( 6035 ) << "parser constructor" << endl;
 #if SPEED_DEBUG > 0
@@ -123,6 +125,7 @@ KHTMLParser::KHTMLParser
     document->ref();
 
     blockStack = 0;
+    current = 0;
 
     // ID_CLOSE_TAG == Num of tags
     forbiddenTag = new ushort[ID_CLOSE_TAG+1];
@@ -139,9 +142,12 @@ KHTMLParser::KHTMLParser( DOM::DocumentFragmentImpl *i, DocumentPtr *doc )
     forbiddenTag = new ushort[ID_CLOSE_TAG+1];
 
     blockStack = 0;
+    current = 0;
 
     reset();
-    current = i;
+
+    setCurrent(i);
+
     inBody = true;
 }
 
@@ -153,6 +159,8 @@ KHTMLParser::~KHTMLParser()
 
     freeBlock();
 
+    if (current) current->deref();
+
     document->deref();
 
     delete [] forbiddenTag;
@@ -161,7 +169,7 @@ KHTMLParser::~KHTMLParser()
 
 void KHTMLParser::reset()
 {
-    current = document->document();
+    setCurrent ( document->document() );
 
     freeBlock();
 
@@ -172,6 +180,8 @@ void KHTMLParser::reset()
     haveFrameSet = false;
     haveContent = false;
     haveBody = false;
+    haveTitle = false;
+    haveMalformedTable = false;
     inSelect = false;
     m_inline = false;
 
@@ -181,46 +191,45 @@ void KHTMLParser::reset()
     head = 0;
     end = false;
     isindex = 0;
-    haveKonqBlock = false;
 
     discard_until = 0;
 }
 
 void KHTMLParser::parseToken(Token *t)
 {
-    if (t->id > 2*ID_CLOSE_TAG)
+    if (t->tid > 2*ID_CLOSE_TAG)
     {
-      kdDebug( 6035 ) << "Unknown tag!! tagID = " << t->id << endl;
+      kdDebug( 6035 ) << "Unknown tag!! tagID = " << t->tid << endl;
       return;
     }
     if(discard_until) {
-        if(t->id == discard_until)
+        if(t->tid == discard_until)
             discard_until = 0;
 
         // do not skip </iframe>
-        if ( discard_until || current->id() + ID_CLOSE_TAG != t->id )
+        if ( discard_until || current->id() + ID_CLOSE_TAG != t->tid )
             return;
     }
 
 #ifdef PARSER_DEBUG
-    kdDebug( 6035 ) << "\n\n==> parser: processing token " << getTagName(t->id).string() << "(" << t->id << ")"
-                    << " current = " << getTagName(current->id()).string() << "(" << current->id() << ")" << endl;
+    kdDebug( 6035 ) << "\n\n==> parser: processing token " << getTagName(t->tid) << "(" << t->tid << ")"
+                    << " current = " << getTagName(current->id()) << "(" << current->id() << ")" << endl;
     kdDebug(6035) << "inline=" << m_inline << " inBody=" << inBody << " haveFrameSet=" << haveFrameSet << " haveContent=" << haveContent << endl;
 #endif
 
     // holy shit. apparently some sites use </br> instead of <br>
     // be compatible with IE and NS
-    if(t->id == ID_BR+ID_CLOSE_TAG && document->document()->parseMode() != DocumentImpl::Strict)
-        t->id -= ID_CLOSE_TAG;
+    if(t->tid == ID_BR+ID_CLOSE_TAG && document->document()->parseMode() != DocumentImpl::Strict)
+        t->tid -= ID_CLOSE_TAG;
 
-    if(t->id > ID_CLOSE_TAG)
+    if(t->tid > ID_CLOSE_TAG)
     {
         processCloseTag(t);
         return;
     }
 
     // ignore spaces, if we're not inside a paragraph or other inline code
-    if( t->id == ID_TEXT && t->text ) {
+    if( t->tid == ID_TEXT && t->text ) {
         if(inBody && !skipMode() &&
            current->id() != ID_STYLE && current->id() != ID_TITLE &&
            current->id() != ID_SCRIPT &&
@@ -236,33 +245,34 @@ void KHTMLParser::parseToken(Token *t)
         return;
 
     // set attributes
-    if(n->isElementNode() && t->id != ID_ISINDEX)
+    if(n->isElementNode() && t->tid != ID_ISINDEX)
     {
         ElementImpl *e = static_cast<ElementImpl *>(n);
         e->setAttributeMap(t->attrs);
 
         // take care of optional close tags
         if(endTag[e->id()] == DOM::OPTIONAL)
-            popBlock(t->id);
+            popBlock(t->tid);
     }
 
     // if this tag is forbidden inside the current context, pop
     // blocks until we are allowed to add it...
-    while(forbiddenTag[t->id]) {
+    while(forbiddenTag[t->tid]) {
 #ifdef PARSER_DEBUG
-        kdDebug( 6035 ) << "t->id: " << t->id << " is forbidden :-( " << endl;
+        kdDebug( 6035 ) << "t->id: " << t->tid << " is forbidden :-( " << endl;
 #endif
         popOneBlock();
     }
 
     // sometimes flat doesn't make sense
-    switch(t->id) {
+    switch(t->tid) {
+    case ID_SELECT:
     case ID_OPTION:
         t->flat = false;
     }
 
     // the tokenizer needs the feedback for space discarding
-    if ( tagPriority[t->id] == 0 )
+    if ( tagPriority[t->tid] == 0 )
 	t->flat = true;
 
     if ( !insertNode(n, t->flat) ) {
@@ -303,13 +313,6 @@ bool KHTMLParser::insertNode(NodeImpl *n, bool flat)
         kdDebug( 6035 ) << "added " << n->nodeName().string() << " to " << tmp->nodeName().string() << ", new current=" << newNode->nodeName().string() << endl;
 #endif
 
-	// have to do this here (and not when creating the node, as we don't know before where we add the LI element to.
-	if ( id == ID_LI && n->isElementNode() ) {
-	    int cid = current->id();
-	    if ( cid != ID_UL && cid != ID_OL )
-	    static_cast<HTMLElementImpl*>(n)->addCSSProperty(CSS_PROP_LIST_STYLE_POSITION, CSS_VAL_INSIDE);
-	}
-
 	// don't push elements without end tag on the stack
         if(tagPriority[id] != 0 && !flat) {
 #if SPEED_DEBUG < 2
@@ -318,7 +321,7 @@ bool KHTMLParser::insertNode(NodeImpl *n, bool flat)
 #endif
 	    if(n->isInline()) m_inline = true;
             pushBlock(id, tagPriority[id]);
-            current = newNode;
+            setCurrent( newNode );
         } else {
 #if SPEED_DEBUG < 2
             if(!n->attached() && HTMLWidget)
@@ -328,8 +331,7 @@ bool KHTMLParser::insertNode(NodeImpl *n, bool flat)
                 QString state(document->document()->nextState());
                 if (!state.isNull()) n->restoreState(state);
             }
-            if(n->renderer())
-                n->renderer()->close();
+            n->closeRenderer();
 #endif
 	    if(n->isInline()) m_inline = true;
         }
@@ -390,9 +392,10 @@ bool KHTMLParser::insertNode(NodeImpl *n, bool flat)
 		    NamedAttrMapImpl *bmap = static_cast<ElementImpl*>(doc()->firstChild())->attributes(false);
 		    bool changed = false;
 		    for (unsigned long l = 0; map && l < map->length(); ++l) {
-			AttributeImpl* it = map->attributeItem(l);
-			changed = !bmap->getAttributeItem(it->id());
-			bmap->insertAttribute(new AttributeImpl(it->id(), it->val()));
+			NodeImpl::Id attrId = map->idAt(l);
+			DOMStringImpl *attrValue = map->valueAt(l);
+			changed = !bmap->getValue(attrId);
+			bmap->setValue(attrId,attrValue);
 		    }
 		    if ( changed )
 			doc()->recalcStyle( NodeImpl::Inherit );
@@ -408,7 +411,7 @@ bool KHTMLParser::insertNode(NodeImpl *n, bool flat)
                 DOM::NodeImpl *newNode = head->addChild(n);
                 if ( newNode ) {
                     pushBlock(id, tagPriority[id]);
-                    current = newNode;
+                    setCurrent ( newNode );
 #if SPEED_DEBUG < 2
 		    if(!n->attached() && HTMLWidget)
                         n->attach();
@@ -439,9 +442,12 @@ bool KHTMLParser::insertNode(NodeImpl *n, bool flat)
                 NamedAttrMapImpl *bmap = doc()->body()->attributes(false);
                 bool changed = false;
                 for (unsigned long l = 0; map && l < map->length(); ++l) {
-                    AttributeImpl* it = map->attributeItem(l);
-                    changed = !bmap->getAttributeItem(it->id());
-                    bmap->insertAttribute(new AttributeImpl(it->id(), it->val()));
+                    NodeImpl::Id attrId = map->idAt(l);
+                    DOMStringImpl *attrValue = map->valueAt(l);
+                    if ( !bmap->getValue(attrId) ) {
+                        bmap->setValue(attrId,attrValue);
+                        changed = true;
+                    }
                 }
                 if ( changed )
                     doc()->recalcStyle( NodeImpl::Inherit );
@@ -473,12 +479,22 @@ bool KHTMLParser::insertNode(NodeImpl *n, bool flat)
                 // fall through!!
             };
             break;
+         case ID_DL:
+             popBlock( ID_DT );
+             if ( current->id() == ID_DL ) {
+                 e = new HTMLGenericElementImpl( document, ID_DD );
+                 insertNode( e );
+                 handled = true;
+             }
+             break;
         case ID_DD:
+            popBlock( ID_DD );
+            /* nobreak */
         case ID_DT:
-            e = new HTMLDListElementImpl(document);
-            if ( insertNode(e) ) {
-                insertNode(n);
-                return true;
+            if ( checkChild( current->id(), ID_DL ) ) {
+                e = new HTMLDListElementImpl( document );
+                insertNode( e );
+                handled = true;
             }
             break;
         case ID_AREA:
@@ -491,18 +507,22 @@ bool KHTMLParser::insertNode(NodeImpl *n, bool flat)
 		    n->attach();
 #endif
                 handled = true;
+                return true;
             }
-            else
-                return false;
-            return true;
+            return false;
         }
+        case ID_THEAD:
+        case ID_TBODY:
+        case ID_TFOOT:
+        case ID_TR:
         case ID_TD:
         case ID_TH:
-            // lets try to close the konqblock
-            if ( haveKonqBlock ) {
-                popBlock( ID__KONQBLOCK );
-                haveKonqBlock = false;
-                return insertNode( n );
+            // lets try to close the stray table content
+            if ( haveMalformedTable ) {
+                while (haveMalformedTable)
+                    popOneBlock();
+                handled = true;
+                break;
             }
         default:
             break;
@@ -555,25 +575,6 @@ bool KHTMLParser::insertNode(NodeImpl *n, bool flat)
             break;
         case ID_BODY:
             break;
-        case ID__KONQBLOCK:
-            switch( id ) {
-            case ID_THEAD:
-            case ID_TFOOT:
-            case ID_TBODY:
-            case ID_TR:
-            case ID_TD:
-            case ID_TH:
-                // now the actual table contents starts
-                // lets close our anonymous block before the table
-                // and go ahead!
-                popBlock( ID__KONQBLOCK );
-                haveKonqBlock = false;
-                handled = checkChild( current->id(), id );
-                break;
-            default:
-                break;
-            }
-            break;
         case ID_TABLE:
         case ID_THEAD:
         case ID_TFOOT:
@@ -612,21 +613,21 @@ bool KHTMLParser::insertNode(NodeImpl *n, bool flat)
                     node = ( node->id() == ID_TR ) ? parentparent : parent;
                     NodeImpl *parent = node->parentNode();
                     int exceptioncode = 0;
-                    NodeImpl *container = new HTMLGenericElementImpl( document, ID__KONQBLOCK );
-                    parent->insertBefore( container, node, exceptioncode );
-                    if ( exceptioncode ) {
+                    parent->insertBefore(n, node, exceptioncode);
+                    if (exceptioncode) {
 #ifdef PARSER_DEBUG
-                        kdDebug( 6035 ) << "adding anonymous container before table failed!" << endl;
+                        kdDebug(6035) << "adding content before table failed.." << endl;
 #endif
                         break;
                     }
-                    if ( !container->attached() && HTMLWidget )
-			container->attach();
-                    pushBlock( ID__KONQBLOCK, tagPriority[ID__KONQBLOCK] );
-                    haveKonqBlock = true;
-                    current = container;
-                    handled = true;
-                    break;
+                    if ( n->isElementNode() && tagPriority[id] != 0 &&
+                         !flat && endTag[id] != DOM::FORBIDDEN ) {
+
+                        pushBlock(id, tagPriority[id]);
+                        setCurrent ( n );
+                        haveMalformedTable = true;
+                    }
+                    return true;
                 }
 
                 if ( current->id() == ID_TR )
@@ -654,6 +655,11 @@ bool KHTMLParser::insertNode(NodeImpl *n, bool flat)
             insertNode(e);
             handled = true;
             break;
+        case ID_DL:
+            e = new HTMLGenericElementImpl( document, ID_DT );
+            insertNode( e );
+            handled = true;
+            break;
         case ID_SELECT:
             if( n->isInline() )
                 return false;
@@ -686,8 +692,6 @@ bool KHTMLParser::insertNode(NodeImpl *n, bool flat)
             break;
             // head elements in the body should be ignored.
 
-        case ID_DL:
-        case ID_DT:
         case ID_ADDRESS:
         case ID_COLGROUP:
         case ID_FONT:
@@ -727,7 +731,7 @@ NodeImpl *KHTMLParser::getElement(Token* t)
 {
     NodeImpl *n = 0;
 
-    switch(t->id)
+    switch(t->tid)
     {
     case ID_HTML:
         n = new HTMLHtmlElementImpl(document);
@@ -761,7 +765,13 @@ NodeImpl *KHTMLParser::getElement(Token* t)
         n = new HTMLStyleElementImpl(document);
         break;
     case ID_TITLE:
+        // only one non-empty <title> allowed
+        if (haveTitle) {
+            discard_until = ID_TITLE+ID_CLOSE_TAG;
+            break;
+        }
         n = new HTMLTitleElementImpl(document);
+        // we'll set haveTitle when closing the tag
         break;
 
 // frames
@@ -773,12 +783,12 @@ NodeImpl *KHTMLParser::getElement(Token* t)
         if ( inBody && !haveFrameSet && !haveContent && !haveBody) {
             popBlock( ID_BODY );
             // ### actually for IE document.body returns the now hidden "body" element
-            // we can't implement that behaviour now because it could cause too many
+            // we can't implement that behavior now because it could cause too many
             // regressions and the headaches are not worth the work as long as there is
             // no site actually relying on that detail (Dirk)
             if (static_cast<HTMLDocumentImpl*>(document->document())->body())
                 static_cast<HTMLDocumentImpl*>(document->document())->body()
-                    ->addCSSProperty(CSS_PROP_DISPLAY, "none");
+                    ->addCSSProperty(CSS_PROP_DISPLAY, CSS_VAL_NONE);
             inBody = false;
         }
         if ( (haveBody || haveContent || haveFrameSet) && current->id() == ID_HTML)
@@ -844,12 +854,12 @@ NodeImpl *KHTMLParser::getElement(Token* t)
         n = new HTMLDListElementImpl(document);
         break;
     case ID_DD:
-        n = new HTMLGenericElementImpl(document, t->id);
+        n = new HTMLGenericElementImpl(document, t->tid);
         popBlock(ID_DT);
         popBlock(ID_DD);
         break;
     case ID_DT:
-        n = new HTMLGenericElementImpl(document, t->id);
+        n = new HTMLGenericElementImpl(document, t->tid);
         popBlock(ID_DD);
         popBlock(ID_DT);
         break;
@@ -879,11 +889,12 @@ NodeImpl *KHTMLParser::getElement(Token* t)
 // formatting elements (block)
     case ID_BLOCKQUOTE:
     case ID_LAYER:
-        n = new HTMLGenericElementImpl(document, t->id);
+    case ID_ILAYER:
+        n = new HTMLGenericElementImpl(document, t->tid);
         break;
     case ID_P:
     case ID_DIV:
-        n = new HTMLDivElementImpl(document, t->id);
+        n = new HTMLDivElementImpl(document, t->tid);
         break;
     case ID_H1:
     case ID_H2:
@@ -891,7 +902,7 @@ NodeImpl *KHTMLParser::getElement(Token* t)
     case ID_H4:
     case ID_H5:
     case ID_H6:
-        n = new HTMLGenericElementImpl(document, t->id);
+        n = new HTMLGenericElementImpl(document, t->tid);
         break;
     case ID_HR:
         n = new HTMLHRElementImpl(document);
@@ -900,7 +911,7 @@ NodeImpl *KHTMLParser::getElement(Token* t)
         ++inPre;
     case ID_XMP:
     case ID_PLAINTEXT:
-        n = new HTMLPreElementImpl(document, t->id);
+        n = new HTMLPreElementImpl(document, t->tid);
         break;
 
 // font stuff
@@ -914,7 +925,7 @@ NodeImpl *KHTMLParser::getElement(Token* t)
 // ins/del
     case ID_DEL:
     case ID_INS:
-        n = new HTMLGenericElementImpl(document, t->id);
+        n = new HTMLGenericElementImpl(document, t->tid);
         break;
 
 // anchor
@@ -963,7 +974,7 @@ NodeImpl *KHTMLParser::getElement(Token* t)
         break;
     case ID_COLGROUP:
     case ID_COL:
-        n = new HTMLTableColElementImpl(document, t->id);
+        n = new HTMLTableColElementImpl(document, t->tid);
         break;
     case ID_TR:
         popBlock(ID_TR);
@@ -973,7 +984,7 @@ NodeImpl *KHTMLParser::getElement(Token* t)
     case ID_TH:
         popBlock(ID_TH);
         popBlock(ID_TD);
-        n = new HTMLTableCellElementImpl(document, t->id);
+        n = new HTMLTableCellElementImpl(document, t->tid);
         break;
     case ID_TBODY:
     case ID_THEAD:
@@ -981,7 +992,7 @@ NodeImpl *KHTMLParser::getElement(Token* t)
         popBlock( ID_THEAD );
         popBlock( ID_TBODY );
         popBlock( ID_TFOOT );
-        n = new HTMLTableSectionElementImpl(document, t->id, false);
+        n = new HTMLTableSectionElementImpl(document, t->tid, false);
         break;
 
 // inline elements
@@ -989,7 +1000,7 @@ NodeImpl *KHTMLParser::getElement(Token* t)
         n = new HTMLBRElementImpl(document);
         break;
     case ID_Q:
-        n = new HTMLGenericElementImpl(document, t->id);
+        n = new HTMLGenericElementImpl(document, t->tid);
         break;
 
 // elements with no special representation in the DOM
@@ -997,7 +1008,7 @@ NodeImpl *KHTMLParser::getElement(Token* t)
 // block:
     case ID_ADDRESS:
     case ID_CENTER:
-        n = new HTMLGenericElementImpl(document, t->id);
+        n = new HTMLGenericElementImpl(document, t->tid);
         break;
 // inline
         // %fontstyle
@@ -1026,28 +1037,30 @@ NodeImpl *KHTMLParser::getElement(Token* t)
     case ID_SUB:
     case ID_SUP:
     case ID_SPAN:
-    case ID_NOBR:
     case ID_WBR:
+    case ID_NOBR:
+        if ( t->tid == ID_NOBR || t->tid == ID_WBR )
+            popBlock( t->tid );
     case ID_BDO:
-        n = new HTMLGenericElementImpl(document, t->id);
+        n = new HTMLGenericElementImpl(document, t->tid);
         break;
 
         // these are special, and normally not rendered
     case ID_NOEMBED:
         if (!t->flat) {
-            n = new HTMLGenericElementImpl(document, t->id);
+            n = new HTMLGenericElementImpl(document, t->tid);
             discard_until = ID_NOEMBED + ID_CLOSE_TAG;
         }
         return n;
     case ID_NOFRAMES:
         if (!t->flat) {
-            n = new HTMLGenericElementImpl(document, t->id);
+            n = new HTMLGenericElementImpl(document, t->tid);
             discard_until = ID_NOFRAMES + ID_CLOSE_TAG;
         }
         return n;
     case ID_NOSCRIPT:
         if (!t->flat) {
-            n = new HTMLGenericElementImpl(document, t->id);
+            n = new HTMLGenericElementImpl(document, t->tid);
             if(HTMLWidget && HTMLWidget->part()->jScriptEnabled())
                 discard_until = ID_NOSCRIPT + ID_CLOSE_TAG;
         }
@@ -1057,10 +1070,11 @@ NodeImpl *KHTMLParser::getElement(Token* t)
         return 0;
         break;
     case ID_MARQUEE:
-        n = new HTMLGenericElementImpl(document, t->id);
+        n = new HTMLGenericElementImpl(document, t->tid);
         break;
 // text
     case ID_TEXT:
+//        kdDebug(6035) << "ID_TEXT: \"" << DOMString(t->text).string() << "\"" << endl;
         n = new TextImpl(document, t->text);
         break;
     case ID_COMMENT:
@@ -1069,7 +1083,7 @@ NodeImpl *KHTMLParser::getElement(Token* t)
 #endif
         break;
     default:
-        kdDebug( 6035 ) << "Unknown tag " << t->id << "!" << endl;
+        kdDebug( 6035 ) << "Unknown tag " << t->tid << "!" << endl;
     }
     return n;
 }
@@ -1077,12 +1091,14 @@ NodeImpl *KHTMLParser::getElement(Token* t)
 void KHTMLParser::processCloseTag(Token *t)
 {
     // support for really broken html. Can't believe I'm supporting such crap (lars)
-    switch(t->id)
+    switch(t->tid)
     {
     case ID_HTML+ID_CLOSE_TAG:
     case ID_BODY+ID_CLOSE_TAG:
-        // we never close the body tag, since some stupid web pages close it before the actual end of the doc.
-        // let's rely on the end() call to close things.
+    case ID_DT+ID_CLOSE_TAG:
+    case ID_DD+ID_CLOSE_TAG:
+        // we never trust those close tags, since stupid webpages close
+        // them prematurely
         return;
     case ID_FORM+ID_CLOSE_TAG:
         form = 0;
@@ -1094,12 +1110,17 @@ void KHTMLParser::processCloseTag(Token *t)
     case ID_SELECT+ID_CLOSE_TAG:
         inSelect = false;
         break;
+    case ID_TITLE+ID_CLOSE_TAG:
+        // Set haveTitle only if <title> isn't empty
+        if ( current->firstChild() )
+            haveTitle = true;
+        break;
     default:
         break;
     }
 
 #ifdef PARSER_DEBUG
-    kdDebug( 6035 ) << "added the following childs to " << current->nodeName().string() << endl;
+    kdDebug( 6035 ) << "added the following children to " << current->nodeName().string() << endl;
     NodeImpl *child = current->firstChild();
     while(child != 0)
     {
@@ -1107,7 +1128,7 @@ void KHTMLParser::processCloseTag(Token *t)
         child = child->nextSibling();
     }
 #endif
-    popBlock(t->id-ID_CLOSE_TAG);
+    popBlock(t->tid-ID_CLOSE_TAG);
 #ifdef PARSER_DEBUG
     kdDebug( 6035 ) << "closeTag --> current = " << current->nodeName().string() << endl;
 #endif
@@ -1128,9 +1149,9 @@ void KHTMLParser::popBlock( int _id )
     int maxLevel = 0;
 
 #ifdef PARSER_DEBUG
-    kdDebug( 6035 ) << "popBlock(" << getTagName(_id).string() << ")" << endl;
+    kdDebug( 6035 ) << "popBlock(" << getTagName(_id) << ")" << endl;
     while(Elem) {
-        kdDebug( 6035) << "   > " << getTagName(Elem->id).string() << endl;
+        kdDebug( 6035) << "   > " << getTagName(Elem->id) << endl;
         Elem = Elem->next;
     }
     Elem = blockStack;
@@ -1169,10 +1190,9 @@ void KHTMLParser::popOneBlock()
     HTMLStackElem *Elem = blockStack;
 
     // we should never get here, but some bad html might cause it.
-#ifndef PARSER_DEBUG
     if(!Elem) return;
-#else
-    kdDebug( 6035 ) << "popping block: " << getTagName(Elem->id).string() << "(" << Elem->id << ")" << endl;
+#ifdef PARSER_DEBUG
+    kdDebug( 6035 ) << "popping block: " << getTagName(Elem->id) << "(" << Elem->id << ")" << endl;
 #endif
 
 #if SPEED_DEBUG < 1
@@ -1182,8 +1202,7 @@ void KHTMLParser::popOneBlock()
             QString state(document->document()->nextState());
             if (!state.isNull()) current->restoreState(state);
         }
-        if (current->renderer())
-            current->renderer()->close();
+        current->closeRenderer();
     }
 #endif
 
@@ -1196,14 +1215,26 @@ void KHTMLParser::popOneBlock()
         --inPre;
 
     m_inline = Elem->m_inline;
-    current = Elem->node;
+
+    if (current->id() == ID_FORM && form && haveMalformedTable)
+        form->setMalformed(true);
+
+    setCurrent( Elem->node );
+
+    if (haveMalformedTable && (!current ||
+                               current->id() == ID_TR ||
+                               current->id() == ID_THEAD ||
+                               current->id() == ID_TBODY ||
+                               current->id() == ID_TFOOT ||
+                               current->id() == ID_TABLE ) )
+        haveMalformedTable = false;
 
     delete Elem;
 }
 
 void KHTMLParser::popInlineBlocks()
 {
-    while(current->isInline() && current->id() != ID_FONT)
+    while(blockStack && current->isInline() && current->id() != ID_FONT)
         popOneBlock();
 }
 
@@ -1243,10 +1274,10 @@ NodeImpl *KHTMLParser::handleIsindex( Token *t )
         n = new HTMLDivElementImpl( document, ID_DIV );
     NodeImpl *child = new HTMLHRElementImpl( document );
     n->addChild( child );
-    AttributeImpl* a = t->attrs ? t->attrs->getAttributeItem(ATTR_PROMPT) : 0;
+    DOMStringImpl* a = t->attrs ? t->attrs->getValue(ATTR_PROMPT) : 0;
     DOMString text = i18n("This is a searchable index. Enter search keywords: ");
     if (a)
-        text = a->value();
+        text = a;
     child = new TextImpl(document, text.implementation());
     n->addChild( child );
     child = new HTMLIsIndexElementImpl(document, myform);
