@@ -24,7 +24,7 @@
 //----------------------------------------------------------------------------
 //
 // KDE HTML Widget -- HTML Parser
-// $Id: htmlparser.h,v 1.59.2.3 2003/05/18 12:34:39 mueller Exp $
+// $Id: htmlparser.h,v 1.65 2003/11/24 00:22:39 faure Exp $
 
 #ifndef HTMLPARSER_H
 #define HTMLPARSER_H
@@ -107,6 +107,14 @@ protected:
     /*
      * The currently active element (the one new elements will be added to)
      */
+    void setCurrent( DOM::NodeImpl* newNode )
+    {
+        if ( newNode ) newNode->ref();
+        if ( current ) current->deref();
+        current = newNode;
+    }
+
+private:
     DOM::NodeImpl *current;
 
     HTMLStackElem *blockStack;
@@ -154,9 +162,10 @@ protected:
     bool haveContent;
     bool haveBody;
     bool haveFrameSet;
+    bool haveTitle;
     bool m_inline;
     bool end;
-    bool haveKonqBlock;
+    bool haveMalformedTable;
     bool inSelect;
 
 
