@@ -27,7 +27,7 @@
 //----------------------------------------------------------------------------
 //
 // KDE HTML Widget - Tokenizers
-// $Id: htmltokenizer.cpp,v 1.248.2.7 2003/08/29 11:41:51 mueller Exp $
+// $Id: htmltokenizer.cpp,v 1.286 2004/01/16 12:52:03 coolo Exp $
 
 //#define TOKEN_DEBUG 1
 //#define TOKEN_DEBUG 2
@@ -36,7 +36,6 @@
 #include "config.h"
 #endif
 
-//#include <string.h>
 #include "html/htmltokenizer.h"
 #include "html/html_documentimpl.h"
 #include "html/htmlparser.h"
@@ -90,43 +89,37 @@ static const char titleEnd [] = "</title";
 #define fixUpChar(x)
 #else
 #define fixUpChar(x) \
-            if (!(x).row() ) { \
-                switch ((x).cell()) \
-                { \
-                /* ALL of these should be changed to Unicode SOON */ \
-                case 0x80: (x) = 0x20ac; break; \
-                case 0x82: (x) = ',';    break; \
-                case 0x83: (x) = 0x0192; break; \
-                case 0x84: (x) = '"';    break; \
-                case 0x85: (x) = 0x2026; break; \
-                case 0x86: (x) = 0x2020; break; \
-                case 0x87: (x) = 0x2021; break; \
-                case 0x88: (x) = 0x02C6; break; \
-                case 0x89: (x) = 0x2030; break; \
-                case 0x8A: (x) = 0x0160; break; \
-                case 0x8b: (x) = '<';    break; \
-                case 0x8C: (x) = 0x0152; break; \
-\
-                case 0x8E: (x) = 0x017D; break; \
-\
-\
-                case 0x91: (x) = '\'';   break; \
-                case 0x92: (x) = '\'';   break; \
-                case 0x93: (x) = '"';    break; \
-                case 0x94: (x) = '"';    break; \
-                case 0x95: (x) = '*';    break; \
-                case 0x96: (x) = '-';    break; \
-                case 0x97: (x) = '-';    break; \
-                case 0x98: (x) = '~';    break; \
-                case 0x99: (x) = 0x2122; break; \
-                case 0x9A: (x) = 0x0161; break; \
-                case 0x9b: (x) = '>';    break; \
-                case 0x9C: (x) = 0x0153; break; \
-\
-                case 0x9E: (x) = 0x017E; break; \
-                case 0x9F: (x) = 0x0178; break; \
-                default: break; \
-                } \
+            switch ((x).unicode()) \
+            { \
+            /* ALL of these should be changed to Unicode SOON */ \
+            case 0x80: (x) = 0x20ac; break; \
+            case 0x82: (x) = ',';    break; \
+            case 0x83: (x) = 0x0192; break; \
+            case 0x84: (x) = '"';    break; \
+            case 0x85: (x) = 0x2026; break; \
+            case 0x86: (x) = 0x2020; break; \
+            case 0x87: (x) = 0x2021; break; \
+            case 0x88: (x) = 0x02C6; break; \
+            case 0x89: (x) = 0x2030; break; \
+            case 0x8A: (x) = 0x0160; break; \
+            case 0x8b: (x) = '<';    break; \
+            case 0x8C: (x) = 0x0152; break; \
+            case 0x8E: (x) = 0x017D; break; \
+            case 0x91: (x) = '\'';   break; \
+            case 0x92: (x) = '\'';   break; \
+            case 0x93: (x) = '"';    break; \
+            case 0x94: (x) = '"';    break; \
+            case 0x95: (x) = '*';    break; \
+            case 0x96: (x) = '-';    break; \
+            case 0x97: (x) = '-';    break; \
+            case 0x98: (x) = '~';    break; \
+            case 0x99: (x) = 0x2122; break; \
+            case 0x9A: (x) = 0x0161; break; \
+            case 0x9b: (x) = '>';    break; \
+            case 0x9C: (x) = 0x0153; break; \
+            case 0x9E: (x) = 0x017E; break; \
+            case 0x9F: (x) = 0x0178; break; \
+            default: break; \
             }
 #endif
 
@@ -255,7 +248,14 @@ void HTMLTokenizer::processListing(DOMStringIt list)
                 // Process this LF
                 if (pending)
                     addPending();
-                pending = LFPending;
+
+                // we used to do it not at all and we want to have
+                // it fixed for textarea. So here we are
+                if ( textarea ) {
+                    prePos++;
+                    *dest++ = *list;
+                } else
+                    pending = LFPending;
             }
             /* Check for MS-DOS CRLF sequence */
             if (*list == '\r')
@@ -325,10 +325,10 @@ void HTMLTokenizer::parseSpecial(DOMStringIt &src)
             else {
                 processListing(DOMStringIt(scriptCode, scriptCodeSize));
                 processToken();
-                if ( style )         { currToken.id = ID_STYLE + ID_CLOSE_TAG; }
-                else if ( textarea ) { currToken.id = ID_TEXTAREA + ID_CLOSE_TAG; }
-                else if ( title ) { currToken.id = ID_TITLE + ID_CLOSE_TAG; }
-                else if ( xmp )  { currToken.id = ID_XMP + ID_CLOSE_TAG; }
+                if ( style )         { currToken.tid = ID_STYLE + ID_CLOSE_TAG; }
+                else if ( textarea ) { currToken.tid = ID_TEXTAREA + ID_CLOSE_TAG; }
+                else if ( title ) { currToken.tid = ID_TITLE + ID_CLOSE_TAG; }
+                else if ( xmp )  { currToken.tid = ID_XMP + ID_CLOSE_TAG; }
                 processToken();
                 style = script = style = textarea = title = xmp = false;
                 tquote = NoQuote;
@@ -375,7 +375,7 @@ void HTMLTokenizer::scriptHandler()
     QString exScript( buffer, dest-buffer );
 
     processToken();
-    currToken.id = ID_SCRIPT + ID_CLOSE_TAG;
+    currToken.tid = ID_SCRIPT + ID_CLOSE_TAG;
     processToken();
 
     QString prependingSrc;
@@ -421,19 +421,20 @@ void HTMLTokenizer::scriptHandler()
         write( prependingSrc, false );
 }
 
-void HTMLTokenizer::scriptExecution( const QString& str, QString scriptURL,
+void HTMLTokenizer::scriptExecution( const QString& str, const QString& scriptURL,
                                      int baseLine)
 {
     bool oldscript = script;
     m_executingScript++;
     script = false;
     QString url;
-    if (scriptURL.isNull())
-      url = static_cast<DocumentImpl*>(view->part()->document().handle())->URL();
+    if (scriptURL.isNull() && view)
+      url = static_cast<DocumentImpl*>(view->part()->document().handle())->URL().url();
     else
       url = scriptURL;
 
-    view->part()->executeScript(url,baseLine,Node(),str);
+    if (view)
+	view->part()->executeScript(url,baseLine+1,Node(),str);
     m_executingScript--;
     script = oldscript;
 }
@@ -445,7 +446,7 @@ void HTMLTokenizer::parseComment(DOMStringIt &src)
         scriptCode[ scriptCodeSize++ ] = *src;
 #if defined(TOKEN_DEBUG) && TOKEN_DEBUG > 1
         qDebug("comment is now: *%s*",
-               QConstString((QChar*)src.current(), QMIN(16, src.length())).string().latin1());
+               QConstString((QChar*)src.current(), kMin(16U, src.length())).string().latin1());
 #endif
         if (src->unicode() == '>' &&
             ( ( brokenComments && !( script || style ) ) ||
@@ -457,10 +458,10 @@ void HTMLTokenizer::parseComment(DOMStringIt &src)
                 checkScriptBuffer();
                 scriptCode[ scriptCodeSize ] = 0;
                 scriptCode[ scriptCodeSize + 1 ] = 0;
-                currToken.id = ID_COMMENT;
+                currToken.tid = ID_COMMENT;
                 processListing(DOMStringIt(scriptCode, scriptCodeSize - 2));
                 processToken();
-                currToken.id = ID_COMMENT + ID_CLOSE_TAG;
+                currToken.tid = ID_COMMENT + ID_CLOSE_TAG;
                 processToken();
 #endif
                 scriptCodeSize = 0;
@@ -647,34 +648,39 @@ void HTMLTokenizer::parseEntity(DOMStringIt &src, QChar *&dest, bool start)
 
                 cBuffer[cBufferPos++] = cc;
                 ++src;
+
+                // be IE compatible and interpret even unterminated entities
+                // outside tags. like "foo &nbspstuff bla".
+                if ( tag == NoTag ) {
+                    const entity* e = kde_findEntity(cBuffer, cBufferPos);
+                    if ( e && e->code < 256 ) {
+                        Entity = SearchSemicolon;
+                        break;
+                    }
+                }
             }
             if(cBufferPos == 9) Entity = SearchSemicolon;
             if(Entity == SearchSemicolon) {
                 if(cBufferPos > 1) {
-                    const entity *e = findEntity(cBuffer, cBufferPos);
-                    if(e)
+                    const entity *e = kde_findEntity(cBuffer, cBufferPos);
+                    if(e && ( e->code < 256 || *src == ';' ))
                         EntityChar = e->code;
-
-                    // be IE compatible
-                    if(tag && EntityChar.unicode() > 255 && *src != ';')
-                        EntityChar = QChar::null;
                 }
             }
-            else
-                break;
+            break;
         }
         case SearchSemicolon:
-
-            //kdDebug( 6036 ) << "ENTITY " << EntityChar.unicode() << ", " << res << endl;
-
+#ifdef TOKEN_DEBUG
+            kdDebug( 6036 ) << "ENTITY " << EntityChar.unicode() << endl;
+#endif
             fixUpChar(EntityChar);
 
-            if ( EntityChar != QChar::null ) {
-                checkBuffer();
-                // Just insert it
-                if (*src == ';')
+            if (*src == ';')
                     ++src;
 
+            if ( !EntityChar.isNull() ) {
+                checkBuffer();
+                // Just insert it
                 src.push( EntityChar );
             } else {
 #ifdef TOKEN_DEBUG
@@ -701,6 +707,7 @@ void HTMLTokenizer::parseEntity(DOMStringIt &src, QChar *&dest, bool start)
 void HTMLTokenizer::parseTag(DOMStringIt &src)
 {
     assert(!Entity );
+    checkScriptBuffer( src.length() );
 
     while ( src.length() )
     {
@@ -714,9 +721,7 @@ void HTMLTokenizer::parseTag(DOMStringIt &src)
 #endif
         switch(tag) {
         case NoTag:
-        {
             return;
-        }
         case TagName:
         {
 #if defined(TOKEN_DEBUG) &&  TOKEN_DEBUG > 1
@@ -809,7 +814,7 @@ void HTMLTokenizer::parseTag(DOMStringIt &src)
                     QCString tmp(ptr, len+1);
                     kdDebug( 6036 ) << "found tag id=" << tagID << ": " << tmp.data() << endl;
 #endif
-                    currToken.id = beginTag ? tagID : tagID + ID_CLOSE_TAG;
+                    currToken.tid = beginTag ? tagID : tagID + ID_CLOSE_TAG;
                     dest = buffer;
                 }
                 tag = SearchAttribute;
@@ -827,7 +832,7 @@ void HTMLTokenizer::parseTag(DOMStringIt &src)
             while(src.length()) {
                 curchar = *src;
                 if(curchar > ' ') {
-                    if(curchar == '>')
+                    if(curchar == '<' || curchar == '>')
                         tag = SearchEnd;
                     else if(atespace && (curchar == '\'' || curchar == '"'))
                     {
@@ -880,7 +885,8 @@ void HTMLTokenizer::parseTag(DOMStringIt &src)
                         break;
                     }
                 }
-                cBuffer[cBufferPos++] = (char) curchar | 0x20;
+                cBuffer[cBufferPos++] =
+                     (  curchar >= 'A' && curchar <= 'Z' ) ? curchar | 0x20 : curchar;
                 ++src;
             }
             if ( cBufferPos == CBUFLEN ) {
@@ -1026,7 +1032,7 @@ void HTMLTokenizer::parseTag(DOMStringIt &src)
                 qDebug("SearchEnd");
 #endif
             while(src.length()) {
-                if(*src == '>')
+                if(*src == '<' || *src == '>')
                     break;
 
                 if (*src == '/')
@@ -1034,17 +1040,18 @@ void HTMLTokenizer::parseTag(DOMStringIt &src)
 
                 ++src;
             }
-            if(!src.length() && *src != '>') break;
+            if(!src.length() && *src != '<' && *src != '>') break;
 
             searchCount = 0; // Stop looking for '<!--' sequence
             tag = NoTag;
             tquote = NoQuote;
-            ++src;
+            if ( *src == '>' )
+                ++src;
 
-            if ( !currToken.id ) //stop if tag is unknown
+            if ( !currToken.tid ) //stop if tag is unknown
                 return;
 
-            uint tagID = currToken.id;
+            uint tagID = currToken.tid;
 #if defined(TOKEN_DEBUG) && TOKEN_DEBUG > 0
             kdDebug( 6036 ) << "appending Tag: " << tagID << endl;
 #endif
@@ -1053,24 +1060,24 @@ void HTMLTokenizer::parseTag(DOMStringIt &src)
             if(tagID >= ID_CLOSE_TAG)
                 tagID -= ID_CLOSE_TAG;
             else if ( beginTag && !brokenScript && tagID == ID_SCRIPT ) {
-                AttributeImpl* a = 0;
+                DOMStringImpl* a = 0;
                 scriptSrc = scriptSrcCharset = QString::null;
                 if ( currToken.attrs && /* potentially have a ATTR_SRC ? */
-                     parser->doc()->view()->part()->jScriptEnabled() && /* jscript allowed at all? */
-                     view /* are we a regular tokenizer or just for innerHTML ? */
+                     view &&  /* are we a regular tokenizer or just for innerHTML ? */
+                     parser->doc()->view()->part()->jScriptEnabled() /* jscript allowed at all? */
                     ) {
-                    if ( ( a = currToken.attrs->getAttributeItem( ATTR_SRC ) ) )
-                        scriptSrc = parser->doc()->completeURL(khtml::parseURL( a->value() ).string() );
-                    if ( ( a = currToken.attrs->getAttributeItem( ATTR_CHARSET ) ) )
-                        scriptSrcCharset = a->value().string().stripWhiteSpace();
-                    if ( scriptSrcCharset.isEmpty() )
+                    if ( ( a = currToken.attrs->getValue( ATTR_SRC ) ) )
+                        scriptSrc = parser->doc()->completeURL(khtml::parseURL( DOMString(a) ).string() );
+                    if ( ( a = currToken.attrs->getValue( ATTR_CHARSET ) ) )
+                        scriptSrcCharset = DOMString(a).string().stripWhiteSpace();
+                    if ( scriptSrcCharset.isEmpty() && view)
                         scriptSrcCharset = parser->doc()->view()->part()->encoding();
-                    if (!(a = currToken.attrs->getAttributeItem( ATTR_LANGUAGE )))
-                        a = currToken.attrs->getAttributeItem(ATTR_TYPE);
+                    if (!(a = currToken.attrs->getValue( ATTR_LANGUAGE )))
+                        a = currToken.attrs->getValue(ATTR_TYPE);
                 }
                 javascript = true;
                 if( a ) {
-                    QString lang = a->value().string();
+                    QString lang = DOMString(a).string();
                     lang = lang.lower();
                     if( !lang.contains("javascript") &&
                         !lang.contains("ecmascript") &&
@@ -1087,6 +1094,7 @@ void HTMLTokenizer::parseTag(DOMStringIt &src)
 
             switch( tagID ) {
             case ID_PRE:
+                discard = LFDiscard;
                 prePos = 0;
                 break;
             case ID_SCRIPT:
@@ -1110,7 +1118,7 @@ void HTMLTokenizer::parseTag(DOMStringIt &src)
                     searchStopper = textareaEnd;
                     searchStopperLen = 10;
                     textarea = true;
-                    discard = AllDiscard;
+                    discard = NoneDiscard;
                     parseSpecial(src);
                 }
                 break;
@@ -1475,15 +1483,15 @@ void HTMLTokenizer::finish()
         else if (server)
             brokenServer = true;
         else if (script)
-            brokenServer = true;
+            brokenScript = true;
+
         checkScriptBuffer();
         scriptCode[ scriptCodeSize ] = 0;
         scriptCode[ scriptCodeSize + 1 ] = 0;
         int pos;
         QString food;
-        if (title || script || style) {
+        if (title || style || script)
             food.setUnicode(scriptCode, scriptCodeSize);
-        }
         else if (server) {
             food = "<";
             food += QString(scriptCode, scriptCodeSize);
@@ -1495,13 +1503,14 @@ void HTMLTokenizer::finish()
         KHTML_DELETE_QCHAR_VEC(scriptCode);
         scriptCode = 0;
         scriptCodeSize = scriptCodeMaxSize = scriptCodeResync = 0;
-        if ( script )
+        if (script)
             scriptHandler();
-        comment = script = title = server = false;
+
+        comment = title = server = script = false;
         if ( !food.isEmpty() )
             write(food, true);
     }
-    // this indicates we will not recieve any more data... but if we are waiting on
+    // this indicates we will not receive any more data... but if we are waiting on
     // an external script to load, we can't finish parsing until that is done
     noMoreData = true;
     if (cachedScript.isEmpty() && !m_executingScript && !onHold)
@@ -1512,36 +1521,36 @@ void HTMLTokenizer::processToken()
 {
     KJSProxy *jsProxy = view ? view->part()->jScript() : 0L;
     if (jsProxy)
-        jsProxy->setEventHandlerLineno(tagStartLineno);
+        jsProxy->setEventHandlerLineno(tagStartLineno+1);
     if ( dest > buffer )
     {
-#ifdef TOKEN_DEBUG
-        if(currToken.id) {
-            qDebug( "unexpected token id: %d, str: *%s*", currToken.id,QConstString( buffer,dest-buffer ).string().latin1() );
+#if 0
+        if(currToken.tid) {
+            qDebug( "unexpected token id: %d, str: *%s*", currToken.tid,QConstString( buffer,dest-buffer ).string().latin1() );
             assert(0);
         }
 
 #endif
         currToken.text = new DOMStringImpl( buffer, dest - buffer );
         currToken.text->ref();
-        currToken.id = ID_TEXT;
+        currToken.tid = ID_TEXT;
     }
-    else if(!currToken.id) {
+    else if(!currToken.tid) {
         currToken.reset();
         if (jsProxy)
-            jsProxy->setEventHandlerLineno(lineno+src.lineCount());
+            jsProxy->setEventHandlerLineno(lineno+src.lineCount()+1);
         return;
     }
 
     dest = buffer;
 
 #ifdef TOKEN_DEBUG
-    QString name = getTagName(currToken.id).string();
+    QString name = QString( getTagName(currToken.tid) );
     QString text;
     if(currToken.text)
         text = QConstString(currToken.text->s, currToken.text->l).string();
 
-    kdDebug( 6036 ) << "Token --> " << name << "   id = " << currToken.id << endl;
+    kdDebug( 6036 ) << "Token --> " << name << "   id = " << currToken.tid << endl;
     if (currToken.flat)
         kdDebug( 6036 ) << "Token is FLAT!" << endl;
     if(!text.isNull())
@@ -1550,24 +1559,33 @@ void HTMLTokenizer::processToken()
     if(l) {
         kdDebug( 6036 ) << "Attributes: " << l << endl;
         for (unsigned long i = 0; i < l; ++i) {
-            AttributeImpl* c = currToken.attrs->attributeItem(i);
-            kdDebug( 6036 ) << "    " << c->id() << " " << parser->doc()->getDocument()->attrName(c->id()).string()
-                            << "=\"" << c->value().string() << "\"" << endl;
+            NodeImpl::Id tid = currToken.attrs->idAt(i);
+            DOMString value = currToken.attrs->valueAt(i);
+            kdDebug( 6036 ) << "    " << tid << " " << parser->doc()->getDocument()->getName(NodeImpl::AttributeId, tid).string()
+                            << "=\"" << value.string() << "\"" << endl;
         }
     }
     kdDebug( 6036 ) << endl;
 #endif
+
+    // In some cases, parseToken() can cause javascript code to be executed
+    // (for example, when setting an attribute that causes an event handler
+    // to be created). So we need to protect against re-entrancy into the parser
+    m_executingScript++;
+
     // pass the token over to the parser, the parser DOES NOT delete the token
     parser->parseToken(&currToken);
 
-    if ( currToken.flat && currToken.id != ID_TEXT && !parser->noSpaces() )
+    m_executingScript--;
+
+    if ( currToken.flat && currToken.tid != ID_TEXT && !parser->noSpaces() )
 	discard = NoneDiscard;
     else if ( parser->selectMode() )
         discard = AllDiscard;
 
     currToken.reset();
     if (jsProxy)
-        jsProxy->setEventHandlerLineno(0);
+        jsProxy->setEventHandlerLineno(1);
 }
 
 
@@ -1604,7 +1622,6 @@ void HTMLTokenizer::notifyFinished(CachedObject* /*finishedObj*/)
         kdDebug( 6036 ) << "Finished loading an external script" << endl;
 #endif
         CachedScript* cs = cachedScript.dequeue();
-        done = cachedScript.isEmpty();
         DOMString scriptSource = cs->script();
 #ifdef TOKEN_DEBUG
         kdDebug( 6036 ) << "External script is:" << endl << scriptSource.string() << endl;
@@ -1618,6 +1635,8 @@ void HTMLTokenizer::notifyFinished(CachedObject* /*finishedObj*/)
 
 	scriptExecution( scriptSource.string(), cachedScriptUrl );
 
+        done = cachedScript.isEmpty();
+
         // 'script' is true when we are called synchronously from
         // parseScript(). In that case parseScript() will take care
         // of 'scriptOutput'.
