@@ -24,7 +24,7 @@
 //----------------------------------------------------------------------------
 //
 // KDE HTML Widget -- Tokenizers
-// $Id: htmltokenizer.h,v 1.62.2.5 2003/08/29 11:43:18 mueller Exp $
+// $Id: htmltokenizer.h,v 1.78.2.1 2004/02/29 15:27:57 mueller Exp $
 
 #ifndef HTMLTOKENIZER_H
 #define HTMLTOKENIZER_H
@@ -63,7 +63,7 @@ namespace khtml {
     {
     public:
         Token() {
-            id = 0;
+            tid = 0;
             attrs = 0;
             text = 0;
             flat = false;
@@ -75,19 +75,24 @@ namespace khtml {
         }
         void addAttribute(DocumentImpl* doc, QChar* buffer, const QString& attrName, const DOMString& v)
         {
-            AttributeImpl* a = 0;
-            if(buffer->unicode())
-                a = new AttributeImpl(buffer->unicode(), v.implementation());
-            else if ( !attrName.isEmpty() && attrName != "/" )
-                a = new AttributeImpl(doc->attrId(0, DOMString(attrName).implementation(), false, 0),
-                                      v.implementation());
+            DOMStringImpl *value = 0;
+            NodeImpl::Id tid = 0;
+            if(buffer->unicode()) {
+		tid = buffer->unicode();
+		value = v.implementation();
+            }
+            else if ( !attrName.isEmpty() && attrName != "/" ) {
+		tid = doc->getId(NodeImpl::AttributeId, DOMString(attrName).implementation(), false, true);
+		value = v.implementation();
+            }
 
-            if (a) {
+            if (value && tid) {
                 if(!attrs) {
                     attrs = new DOM::NamedAttrMapImpl(0);
                     attrs->ref();
                 }
-                attrs->insertAttribute(a);
+                if (!attrs->getValue(tid))
+		    attrs->setValue(tid,value);
             }
         }
         void reset()
@@ -96,7 +101,7 @@ namespace khtml {
                 attrs->deref();
                 attrs = 0;
             }
-            id = 0;
+            tid = 0;
             if(text) {
                 text->deref();
                 text = 0;
@@ -105,7 +110,7 @@ namespace khtml {
         }
         DOM::NamedAttrMapImpl* attrs;
         DOMStringImpl* text;
-        ushort id;
+        ushort tid;
         bool flat;
     };
 
@@ -116,6 +121,7 @@ namespace khtml {
 
 class HTMLTokenizer : public Tokenizer, public CachedObjectClient
 {
+    friend class KHTMLParser;
 public:
     HTMLTokenizer(DOM::DocumentPtr *, KHTMLView * = 0);
     HTMLTokenizer(DOM::DocumentPtr *, DOM::DocumentFragmentImpl *frag);
@@ -142,8 +148,7 @@ protected:
     void parseEntity(khtml::DOMStringIt &str, QChar *&dest, bool start = false);
     void parseProcessingInstruction(khtml::DOMStringIt &str);
     void scriptHandler();
-    void scriptExecution(const QString& script, QString scriptURL = QString(),
-                         int baseLine = 0);
+    void scriptExecution(const QString& script, const QString& scriptURL = QString::null, int baseLine = 0);
     void setSrc(const QString& source);
 
     // check if we have enough space in the buffer.
@@ -234,7 +239,7 @@ protected:
         SearchSemicolon
     } Entity;
 
-    // are we in a <script> ... </script block
+    // are we in a <script> ... </script> block
     bool script;
 
     QChar EntityChar;
@@ -284,9 +289,9 @@ protected:
 
     // Used to store the code of a srcipting sequence
     QChar *scriptCode;
-    // Size of the script sequenze stored in @ref #scriptCode
+    // Size of the script sequenze stored in scriptCode
     int scriptCodeSize;
-    // Maximal size that can be stored in @ref #scriptCode
+    // Maximal size that can be stored in scriptCode
     int scriptCodeMaxSize;
     // resync point of script code size
     int scriptCodeResync;
@@ -318,7 +323,7 @@ protected:
     bool onHold;
 
     // if we found one broken comment, there are most likely others as well
-    // store a flag to get rid of the O(n^2) behaviour in such a case.
+    // store a flag to get rid of the O(n^2) behavior in such a case.
     bool brokenComments;
     // current line number
     int lineno;
@@ -326,7 +331,7 @@ protected:
     int scriptStartLineno;
     int tagStartLineno;
 
-#define CBUFLEN 14
+#define CBUFLEN 20
     char cBuffer[CBUFLEN+2];
     unsigned int cBufferPos;
 
