@@ -157,7 +157,7 @@ void KJavaAppletContext::received( const QString& cmd, const QStringList& arg )
         emit showDocument( arg[0], "_top" );
     }
     else if ( cmd == QString::fromLatin1( "resizeapplet" )
-              && arg.count() > 0 )
+              && arg.count() > 2 )
     {
         //arg[1] should be appletID
         //arg[2] should be new width
@@ -186,24 +186,15 @@ void KJavaAppletContext::received( const QString& cmd, const QStringList& arg )
     {
         bool ok;
         int appletID = arg[0].toInt(&ok);
-        unsigned long objid = arg[1].toInt(&ok);
-        if (ok)
+        KJavaApplet * applet;
+        if (ok && (applet = d->applets[appletID]))
         {
-            KJavaApplet * applet = d->applets[appletID];
-            if (applet)
-            {
-                KParts::LiveConnectExtension::ArgList arglist;
-                for (unsigned i = 3; i < arg.count(); i += 2)
-                    // take a deep breath here
-                    arglist.push_back(KParts::LiveConnectExtension::ArgList::value_type((KParts::LiveConnectExtension::Type) arg[i].toInt(), arg[i+1]));
-
-                emit static_cast<KJavaLiveConnect*>(applet->getLiveConnectExtension())->sendEvent(objid, arg[2], arglist);
-            }
-            else
-                kdError(DEBUGAREA) << "could find applet for JS event" << endl;
+            QStringList js_args(arg);
+            js_args.pop_front();
+            applet->jsData(js_args);
         }
         else
-            kdError(DEBUGAREA) << "could not parse applet ID for JS event " << arg[0] << " " << arg[1] << endl;
+            kdError(DEBUGAREA) << "parse JS event " << arg[0] << " " << arg[1] << endl;
     }
     else if ( cmd == QString::fromLatin1( "AppletStateNotification" ) )
     {
@@ -259,20 +250,24 @@ bool KJavaAppletContext::appletsLoaded() const {
     return true;
 }
 
-bool KJavaAppletContext::getMember(KJavaApplet * applet, const unsigned long objid, const QString & name, int & type, unsigned long & rid, QString & value) {
-    return server->getMember(id, applet->appletId(), objid, name, type, rid, value);
+bool KJavaAppletContext::getMember(QStringList & args, QStringList & ret_args) {
+    args.push_front( QString::number(id) );
+    return server->getMember( args, ret_args );
 }
 
-bool KJavaAppletContext::putMember(KJavaApplet * applet, const unsigned long objid, const QString & name, const QString & value) {
-    return server->putMember(id, applet->appletId(), objid, name, value);
+bool KJavaAppletContext::putMember( QStringList & args ) {
+    args.push_front( QString::number(id) );
+    return server->putMember( args );
 }
 
-bool KJavaAppletContext::callMember(KJavaApplet * applet, const unsigned long objid, const QString & name, const QStringList & args, int & type, unsigned long & retobjid, QString & value) {
-    return server->callMember(id, applet->appletId(), objid, name, args, type, retobjid, value);
+bool KJavaAppletContext::callMember(QStringList & args, QStringList &ret_args) {
+    args.push_front( QString::number(id) );
+    return server->callMember( args, ret_args );
 }
 
-void KJavaAppletContext::derefObject(KJavaApplet * applet, const unsigned long jid) {
-    server->derefObject(id, applet->appletId(), jid);
+void KJavaAppletContext::derefObject( QStringList & args ) {
+    args.push_front( QString::number(id) );
+    server->derefObject( args );
 }
 
 #include <kjavaappletcontext.moc>
