@@ -37,7 +37,9 @@
 
 #include <qtimer.h>
 #include <qguardedptr.h>
+#include <qvaluelist.h>
 #include <qdir.h>
+#include <qeventloop.h>
 
 #include <stdlib.h>
 #include <assert.h>
@@ -66,27 +68,49 @@
 #define KJAS_AUDIOCLIP_STOP    (char)22
 #define KJAS_APPLET_STATE      (char)23
 #define KJAS_APPLET_FAILED     (char)24
+#define KJAS_DATA_COMMAND      (char)25
+#define KJAS_PUT_URLDATA       (char)26
+#define KJAS_PUT_DATA          (char)27
 
 
-class JSStackNode {
+class JSStackFrame;
+
+typedef QMap< int, KJavaKIOJob* > KIOJobMap;
+typedef QMap< int, JSStackFrame* > JSStack;
+
+class JSStackFrame {
 public:
-    JSStackNode() : ready(false) {}
-    bool ready;
-    QStringList args;
-    int size;
+    JSStackFrame(JSStack & stack, QStringList & a)
+    : jsstack(stack), args(a), ticket(counter++), ready(false), exit (false) {
+        jsstack.insert( ticket, this );
+    }
+    ~JSStackFrame() {
+        jsstack.erase( ticket );
+    }
+    JSStack & jsstack;
+    QStringList & args;
+    int ticket;
+    bool ready : 1;
+    bool exit : 1;
+    static int counter;
 };
 
-// For future expansion
+int JSStackFrame::counter = 0;
+
 class KJavaAppletServerPrivate
 {
 friend class KJavaAppletServer;
 private:
+   //KJavaAppletServerPrivate() : locked_context(-1) {}
    int counter;
-   int ticketcounter;
    QMap< int, QGuardedPtr<KJavaAppletContext> > contexts;
    QString appletLabel;
-   QMap< int, JSStackNode* > jsstack;
+   JSStack jsstack;
+   KIOJobMap kiojobs;
    bool javaProcessFailed;
+   bool useKIO;
+   //int locked_context;
+   //QValueList<QByteArray> java_requests;
 };
 
 static KJavaAppletServer* self = 0;
@@ -94,7 +118,6 @@ static KJavaAppletServer* self = 0;
 KJavaAppletServer::KJavaAppletServer()
 {
     d = new KJavaAppletServerPrivate;
-    d->ticketcounter = 0;
     process = new KJavaProcess();
 
     connect( process, SIGNAL(received(const QByteArray&)),
@@ -247,10 +270,16 @@ void KJavaAppletServer::setupJava( KJavaProcess *p )
                               "org.kde.kjas.server.KJASSecurityManager" );
     }
 
+    d->useKIO = config.readBoolEntry( "UseKio", false);
+    if( d->useKIO )
+    {
+        p->setSystemProperty( "kjas.useKio", QString::null );
+    }
+
     //check for http proxies...
     if( KProtocolManager::useProxy() )
     {
-        // only proxyForURL honours automatic proxy scripts
+        // only proxyForURL honors automatic proxy scripts
         // we do not know the applet url here so we just use a dummy url
         // this is a workaround for now
         // FIXME
@@ -387,16 +416,22 @@ void KJavaAppletServer::stopApplet( int contextId, int appletId )
     process->send( KJAS_STOP_APPLET, args );
 }
 
-void KJavaAppletServer::sendURLData( const QString& loaderID,
-                                     const QString& url,
-                                     const QByteArray& data )
+void KJavaAppletServer::sendURLData( int loaderID, int code, const QByteArray& data )
 {
     QStringList args;
-    args.append( loaderID );
-    args.append( url );
+    args.append( QString::number(loaderID) );
+    args.append( QString::number(code) );
 
     process->send( KJAS_URLDATA, args, data );
+}
 
+void KJavaAppletServer::removeDataJob( int loaderID )
+{
+    KIOJobMap::iterator it = d->kiojobs.find( loaderID );
+    if (it != d->kiojobs.end()) {
+        it.data()->deleteLater();
+        d->kiojobs.erase( it );
+    }
 }
 
 void KJavaAppletServer::quit()
@@ -404,6 +439,8 @@ void KJavaAppletServer::quit()
     QStringList args;
 
     process->send( KJAS_SHUTDOWN_SERVER, args );
+    process->flushBuffers();
+    process->wait( 10 );
 }
 
 void KJavaAppletServer::slotJavaRequest( const QByteArray& qb )
@@ -425,19 +462,48 @@ void KJavaAppletServer::slotJavaRequest( const QByteArray& qb )
     {
         contextID += qb[ index++ ];
     }
+    bool ok;
+    int ID_num = contextID.toInt( &ok ); // context id or kio job id
+    /*if (d->locked_context > -1 && 
+        ID_num != d->locked_context &&
+        (cmd_code == KJAS_JAVASCRIPT_EVENT ||
+         cmd_code == KJAS_APPLET_STATE ||
+         cmd_code == KJAS_APPLET_FAILED))
+    {
+        / * Don't allow requests from other contexts if we're waiting
+         * on a return value that can trigger JavaScript events
+         * /
+        d->java_requests.push_back(qb);
+        return;
+    }*/
     ++index; //skip the sep
 
+    if (cmd_code == KJAS_PUT_DATA) {
+        // rest of the data is for kio put
+        if (ok) {
+            KIOJobMap::iterator it = d->kiojobs.find( ID_num );
+            if (ok && it != d->kiojobs.end()) {
+                QByteArray qba;
+                qba.setRawData(qb.data() + index, qb.size() - index - 1);
+                it.data()->data(qba);
+                qba.resetRawData(qb.data() + index, qb.size() - index - 1);
+            }
+            kdDebug(6100) << "PutData(" << ID_num << ") size=" << qb.size() - index << endl;
+        } else
+            kdError(6100) << "PutData error " << ok << endl;
+        return;
+    }
     //now parse out the arguments
     while( index < qb_size )
     {
-        QString tmp;
-        while( qb[index] != 0 )
-            tmp += qb[ index++ ];
-
-        kdDebug(6100) << "KJavaAppletServer::slotJavaRequest: "<< tmp << endl;
-        args.append( tmp );
-
-        ++index; //skip the sep
+        int sep_pos = qb.find( 0, index );
+        if (sep_pos < 0) {
+            kdError(6100) << "Missing separation byte" << endl;
+            sep_pos = qb_size;
+        }
+        //kdDebug(6100) << "KJavaAppletServer::slotJavaRequest: "<< QString::fromLocal8Bit( qb.data() + index, sep_pos - index ) << endl;
+        args.append( QString::fromLocal8Bit( qb.data() + index, sep_pos - index ) );
+        index = sep_pos + 1; //skip the sep
     }
     //here I should find the context and call the method directly
     //instead of emitting signals
@@ -460,10 +526,31 @@ void KJavaAppletServer::slotJavaRequest( const QByteArray& qb )
             break;
 
         case KJAS_GET_URLDATA:
-            //here we need to get some data for a class loader and send it back...
-            kdDebug(6100) << "GetURLData from classloader: "<< contextID
-                          << " for url: " << args[0] << endl;
-            break;
+            if (ok && args.size () > 0) {
+                d->kiojobs.insert(ID_num, new KJavaDownloader(ID_num, args[0]));
+                kdDebug(6100) << "GetURLData(" << ID_num << ") url=" << args[0] << endl;
+            } else
+                kdError(6100) << "GetURLData error " << ok << " args:" << args.size() << endl;
+            return;
+        case KJAS_PUT_URLDATA:
+            if (ok && args.size () > 0) {
+                KJavaUploader *job = new KJavaUploader(ID_num, args[0]);
+                d->kiojobs.insert(ID_num, job);
+                job->start();
+                kdDebug(6100) << "PutURLData(" << ID_num << ") url=" << args[0] << endl;
+            } else
+                kdError(6100) << "PutURLData error " << ok << " args:" << args.size() << endl;
+            return;
+        case KJAS_DATA_COMMAND:
+            if (ok && args.size () > 0) {
+                int cmd = args[0].toInt( &ok );
+                KIOJobMap::iterator it = d->kiojobs.find( ID_num );
+                if (ok && it != d->kiojobs.end())
+                    it.data()->jobCommand( cmd );
+                kdDebug(6100) << "KIO Data command: " << ID_num << " " << args[0] << endl;
+            } else
+                kdError(6100) << "KIO Data command error " << ok << " args:" << args.size() << endl;
+            return;
         case KJAS_JAVASCRIPT_EVENT:
             cmd = QString::fromLatin1( "JS_Event" );
             kdDebug(6100) << "Javascript request: "<< contextID
@@ -473,13 +560,13 @@ void KJavaAppletServer::slotJavaRequest( const QByteArray& qb )
         case KJAS_PUT_MEMBER:
         case KJAS_CALL_MEMBER: {
             int ticket = args[0].toInt();
-            QMap<int, JSStackNode*>::iterator it = d->jsstack.find(ticket);
+            JSStack::iterator it = d->jsstack.find(ticket);
             if (it != d->jsstack.end()) {
                 kdDebug(6100) << "slotJavaRequest: " << ticket << endl; 
                 args.pop_front();
-                it.data()->args = args;
+                it.data()->args.operator=(args); // just in case ..
                 it.data()->ready = true;
-                process->syncCommandReceived(ticket);
+                it.data()->exit = true;
             } else
                 kdDebug(6100) << "Error: Missed return member data" << endl;
             return;
@@ -509,132 +596,81 @@ void KJavaAppletServer::slotJavaRequest( const QByteArray& qb )
             break;
     }
 
-    if( cmd_code == KJAS_GET_URLDATA )
+
+    if( !ok )
     {
-        new KJavaDownloader( contextID, args[0] );
+        kdError(6100) << "could not parse out contextID to call command on" << endl;
+        return;
     }
-    else
-    {
-        bool ok;
-        int contextID_num = contextID.toInt( &ok );
 
-        if( !ok )
-        {
-            kdError(6100) << "could not parse out contextID to call command on" << endl;
-            return;
-        }
-
-        KJavaAppletContext* context = d->contexts[ contextID_num ];
-        if( context )
-            context->processCmd( cmd, args );
-        else if (cmd != "AppletStateNotification") 
-            kdError(6100) << "no context object for this id" << endl;
-    }
+    KJavaAppletContext* context = d->contexts[ ID_num ];
+    if( context )
+        context->processCmd( cmd, args );
+    else if (cmd != "AppletStateNotification") 
+        kdError(6100) << "no context object for this id" << endl;
 }
 
-bool KJavaAppletServer::getMember(int contextId, int appletId, const unsigned long objid, const QString & name, int & type, unsigned long & rid, QString & value) {
-    QStringList args;
-    int ticket = d->ticketcounter++;
-    args.append( QString::number(contextId) );
-    args.append( QString::number(appletId) );
-    args.append( QString::number(ticket) );
-    args.append( QString::number(objid) );
-    args.append( name );
-
-    JSStackNode * frame = new JSStackNode;
-    d->jsstack.insert(ticket, frame); 
-    kdDebug(6100) << "KJavaAppletServer::getMember " << name << " " << ticket << endl;
-    process->sendSync( ticket, KJAS_GET_MEMBER, args );
-
-    bool retval = frame->ready;
-    if (retval && frame->args.count() == 3) {
-        type = frame->args[0].toInt(&retval);
-        if (retval && type >= 0) {
-            rid = frame->args[1].toInt(&retval);
-            value = frame->args[2];
-        } else
-            retval = false;
-    } else {
-        kdError(6100) << "getMember: " << (retval ? "args ":"timeout ") << ticket << endl;
-    }
-
-    d->jsstack.erase(ticket);
-    delete frame;
-
-    return retval;
+void KJavaAppletServer::endWaitForReturnData() {
+    kdDebug(6100) << "KJavaAppletServer::endWaitForReturnData" << endl;
+    killTimers();
+    JSStack::iterator it = d->jsstack.begin();
+    for (; it != d->jsstack.end(); ++it)
+        it.data()->exit = true;
 }
 
-bool KJavaAppletServer::putMember(int contextId, int appletId, const unsigned long objid, const QString & name, const QString & value) {
-    QStringList args;
-    int ticket = d->ticketcounter++;
-    args.append( QString::number(contextId) );
-    args.append( QString::number(appletId) );
-    args.append( QString::number(ticket) );
-    args.append( QString::number(objid) );
-    args.append( name );
-    args.append( value );
-
-    JSStackNode * frame = new JSStackNode;
-    d->jsstack.insert(ticket, frame); 
-    kdDebug(6100) << "KJavaAppletServer::putMember " << name << " " << ticket << endl;
-
-    process->sendSync( ticket, KJAS_PUT_MEMBER, args );
-
-    bool retval = frame->ready;
-    if (retval) {
-        retval = frame->args[0].toInt(&retval);
-    } else {
-        kdError(6100) << "putMember: timeout" << endl;
-    }
-
-    d->jsstack.erase(ticket);
-    delete frame;
-
-    return retval;
+void KJavaAppletServer::timerEvent(QTimerEvent *) {
+    endWaitForReturnData();
+    kdDebug(6100) << "KJavaAppletServer::timerEvent timeout" << endl;
 }
 
-bool KJavaAppletServer::callMember(int contextId, int appletId, const unsigned long objid, const QString & name, const QStringList & fargs, int & type, unsigned long & rid, QString & value) {
-    QStringList args;
-    int ticket = d->ticketcounter++;
-    args.append( QString::number(contextId) );
-    args.append( QString::number(appletId) );
-    args.append( QString::number(ticket) );
-    args.append( QString::number(objid) );
-    args.append( name );
-    for (QStringList::const_iterator it = fargs.begin(); it != fargs.end(); it++)
-        args.append(*it);
-
-    JSStackNode * frame = new JSStackNode;
-    d->jsstack.insert(ticket, frame); 
-
-    kdDebug(6100) << "KJavaAppletServer::callMember " << name << " " << ticket << endl;
-    process->sendSync( ticket, KJAS_CALL_MEMBER, args );
-
-    bool retval = frame->ready;
-    if (retval) {
-        type = frame->args[0].toInt(&retval);
-        if (retval && type > -1) {
-            rid = frame->args[1].toInt(&retval);
-            if (retval)
-                value = frame->args[2];
-        } else
-            retval = false;
-    } else {
-        kdError(6100) << "callMember: timeout return data" << endl;
-    }
-
-    d->jsstack.erase(ticket);
-    delete frame;
-
-    return retval;
+void KJavaAppletServer::waitForReturnData(JSStackFrame * frame) {
+    kdDebug(6100) << ">KJavaAppletServer::waitForReturnData" << endl;
+    killTimers();
+    startTimer(15000);
+    while (!frame->exit)
+        kapp->eventLoop()->processEvents (QEventLoop::AllEvents | QEventLoop::WaitForMore);
+    if (d->jsstack.size() <= 1)
+        killTimers();
+    kdDebug(6100) << "<KJavaAppletServer::waitForReturnData stacksize:" << d->jsstack.size() << endl;
 }
 
-void KJavaAppletServer::derefObject(int contextId, int appletId, const unsigned long objid) {
-    QStringList args;
-    args.append( QString::number(contextId) );
-    args.append( QString::number(appletId) );
-    args.append( QString::number(objid) );
+bool KJavaAppletServer::getMember(QStringList & args, QStringList & ret_args) {
+    JSStackFrame frame( d->jsstack, ret_args );
+    args.push_front( QString::number(frame.ticket) );
 
+    process->send( KJAS_GET_MEMBER, args );
+    waitForReturnData( &frame );
+
+    return frame.ready;
+}
+
+bool KJavaAppletServer::putMember( QStringList & args ) {
+    QStringList ret_args;
+    JSStackFrame frame( d->jsstack, ret_args );
+    args.push_front( QString::number(frame.ticket) );
+
+    process->send( KJAS_PUT_MEMBER, args );
+    waitForReturnData( &frame );
+
+    return frame.ready && ret_args.count() > 0 && ret_args[0].toInt();
+}
+
+bool KJavaAppletServer::callMember(QStringList & args, QStringList & ret_args) {
+    JSStackFrame frame( d->jsstack, ret_args );
+    args.push_front( QString::number(frame.ticket) );
+
+    process->send( KJAS_CALL_MEMBER, args );
+    waitForReturnData( &frame );
+
+    return frame.ready;
+}
+
+void KJavaAppletServer::derefObject( QStringList & args ) {
     process->send( KJAS_DEREF_OBJECT, args );
 }
+
+bool KJavaAppletServer::usingKIO() {
+    return d->useKIO;
+}
+
 #include "kjavaappletserver.moc"
