@@ -32,12 +32,13 @@
 /**
  * @short Communicates with a KJAS server to display and control Java applets.
  *
- * @version $Id: kjavaappletserver.h,v 1.21.2.1 2003/05/06 20:56:39 vriezen Exp $
+ * @version $Id: kjavaappletserver.h,v 1.28 2003/08/16 14:46:42 vriezen Exp $
  * @author Richard J. Moore, rich@kde.org
  */
 
 class KJavaAppletContext;
 class KJavaAppletServerPrivate;
+class JSStackFrame;
 
 class KJavaAppletServer : public QObject
 {
@@ -116,10 +117,12 @@ public:
     /**
      * Send data we got back from a KJavaDownloader back to the appropriate
      * class loader.
-     * (This is currently unimplemented on the java side.
      */
-    void sendURLData( const QString& loaderID, const QString& url,
-                      const QByteArray& data );
+    void sendURLData( int loaderID, int code, const QByteArray& data );
+    /**
+     * Removes KJavaDownloader from the list (deletes it too).
+     */
+    void removeDataJob( int loaderID );
 
     /**
      * Shut down the KJAS server.
@@ -128,10 +131,15 @@ public:
 
     QString appletLabel();
 
-    bool getMember(int contextId, int appletId, const unsigned long, const QString &, int &, unsigned long &, QString &);
-    bool putMember(int contextId, int appletId, const unsigned long, const QString &, const QString &);
-    bool callMember(int contextId, int appletId, const unsigned long, const QString &, const QStringList &, int &, unsigned long &, QString &);
-    void derefObject(int contextId, int appletId, const unsigned long id);
+    void waitForReturnData(JSStackFrame *);
+    void endWaitForReturnData();
+
+    bool getMember(QStringList & args, QStringList & ret_args);
+    bool putMember(QStringList & args);
+    bool callMember(QStringList & args, QStringList & ret_args);
+    void derefObject(QStringList & args);
+
+    bool usingKIO();
 protected:
     void setupJava( KJavaProcess* p );
 
@@ -140,6 +148,7 @@ protected:
 protected slots:
     void slotJavaRequest( const QByteArray& qb );
     void checkShutdown();
+    void timerEvent(QTimerEvent *);
 
 private:
     KJavaAppletServerPrivate* d;
