@@ -25,13 +25,17 @@
 #endif
 #include <qtable.h>
 #include <qpair.h>
+#include <qguardedptr.h>
 
 #include <klibloader.h>
 #include <kaboutdata.h>
 #include <kstaticdeleter.h>
 #include <klocale.h>
+#include <kapplication.h>
 #include <kdebug.h>
 #include <kconfig.h>
+#include <kio/authinfo.h>
+#include <dcopclient.h>
 
 #include "kjavaappletwidget.h"
 #include "kjavaappletviewer.h"
@@ -64,15 +68,22 @@ static KJavaServerMaintainer * serverMaintainer = 0;
 class KJavaServerMaintainer {
 public:
     KJavaServerMaintainer () { }
+    ~KJavaServerMaintainer ();
 
     KJavaAppletContext * getContext (QObject*, const QString &);
     void releaseContext (QObject*, const QString &);
-public:
+    void setServer (KJavaAppletServer * s);
+private:
     typedef QMap <QPair <QObject*, QString>, QPair <KJavaAppletContext*, int> >
             ContextMap;
     ContextMap m_contextmap;
+    QGuardedPtr <KJavaAppletServer> server;
 };
 
+KJavaServerMaintainer::~KJavaServerMaintainer () {
+    delete server;
+}
+
 KJavaAppletContext * KJavaServerMaintainer::getContext (QObject * w, const QString & doc) {
     ContextMap::key_type key = qMakePair (w, doc);
     ContextMap::iterator it = m_contextmap.find (key);
@@ -94,13 +105,17 @@ void KJavaServerMaintainer::releaseContext (QObject * w, const QString & doc) {
     }
 }
 
+inline void KJavaServerMaintainer::setServer (KJavaAppletServer * s) {
+    if (!server)
+        server = s;
+}
+
 static KStaticDeleter <KJavaServerMaintainer> serverMaintainerDeleter;
 
 //-----------------------------------------------------------------------------
 
 AppletParameterDialog::AppletParameterDialog (KJavaAppletWidget * parent)
-    //: KDialogBase (parent, "paramdialog", true, i18n ("Applet Parameters"), 
-    : KDialogBase (parent, "paramdialog", true, "Applet Parameters", 
+    : KDialogBase (parent, "paramdialog", true, i18n ("Applet Parameters"),
                    KDialogBase::Close, KDialogBase::Close, true),
       m_appletWidget (parent) {
     KJavaApplet * applet = parent->applet ();
@@ -109,22 +124,17 @@ AppletParameterDialog::AppletParameterDialog (KJavaAppletWidget * parent)
     table->setColumnWidth (0, 200);
     table->setColumnWidth (1, 340);
     QHeader *header = table->horizontalHeader();
-    //header->setLabel (0, i18n ("Parameter"));
-    header->setLabel (0, "Parameter");
-    //header->setLabel (1, i18n ("Value"));
-    header->setLabel (1, "Value");
-    //QTableItem * tit = new QTableItem (table, QTableItem::Never, i18n("Class"));
-    QTableItem * tit = new QTableItem (table, QTableItem::Never, "Class");
+    header->setLabel (0, i18n ("Parameter"));
+    header->setLabel (1, i18n ("Value"));
+    QTableItem * tit = new QTableItem (table, QTableItem::Never, i18n("Class"));
     table->setItem (0, 0, tit);
     tit = new QTableItem(table, QTableItem::Always, applet->appletClass());
     table->setItem (0, 1, tit);
-    //tit = new QTableItem (table, QTableItem::Never, i18n ("Base URL"));
-    tit = new QTableItem (table, QTableItem::Never, "Base URL");
+    tit = new QTableItem (table, QTableItem::Never, i18n ("Base URL"));
     table->setItem (1, 0, tit);
     tit = new QTableItem(table, QTableItem::Always, applet->baseURL());
     table->setItem (1, 1, tit);
-    //tit = new QTableItem (table, QTableItem::Never, i18n ("Archives"));
-    tit = new QTableItem (table, QTableItem::Never, "Archives");
+    tit = new QTableItem (table, QTableItem::Never, i18n ("Archives"));
     table->setItem (2, 0, tit);
     tit = new QTableItem(table, QTableItem::Always, applet->archives());
     table->setItem (2, 1, tit);
@@ -153,21 +163,48 @@ void AppletParameterDialog::slotClose () {
 }
 //-----------------------------------------------------------------------------
 
+class CoverWidget : public QWidget {
+    KJavaAppletWidget * m_appletwidget;
+public:
+    CoverWidget (QWidget *);
+    ~CoverWidget () {}
+    KJavaAppletWidget * appletWidget () const;
+protected:
+    void resizeEvent (QResizeEvent * e);
+};
+
+inline CoverWidget::CoverWidget (QWidget * parent) : QWidget (parent) {
+    m_appletwidget = new KJavaAppletWidget (this);
+    setFocusProxy (m_appletwidget);
+    hide ();
+}
+
+inline KJavaAppletWidget * CoverWidget::appletWidget () const {
+    return m_appletwidget;
+}
+
+void CoverWidget::resizeEvent (QResizeEvent * e) {
+    m_appletwidget->resize (e->size().width(), e->size().height());
+}
+
+//-----------------------------------------------------------------------------
+
 KJavaAppletViewer::KJavaAppletViewer (QWidget * wparent, const char *,
                  QObject * parent, const char * name, const QStringList & args)
  : KParts::ReadOnlyPart (parent, name),
-   m_browserextension (new KJavaAppletViewerBrowserExtension (this))
+   m_browserextension (new KJavaAppletViewerBrowserExtension (this)),
+   m_liveconnect (new KJavaAppletViewerLiveConnectExtension (this)),
+   m_closed (true)
 {
     if (!serverMaintainer) {
         serverMaintainerDeleter.setObject (serverMaintainer,
                                            new KJavaServerMaintainer);
     }
-    m_view = new KJavaAppletViewerWidget (wparent);
-
+    m_view = new CoverWidget (wparent);
     QString classname, classid, codebase, khtml_codebase;
     int width = -1;
     int height = -1;
-    KJavaApplet * applet = m_view->applet ();
+    KJavaApplet * applet = m_view->appletWidget()->applet ();
     QStringList::const_iterator it = args.begin ();
     for ( ; it != args.end (); ++it) {
         int equalPos = (*it).find("=");
@@ -186,10 +223,10 @@ KJavaAppletViewer::KJavaAppletViewer (QWidget * wparent, const char *,
                     QString fn = url.fileName (false);
                     baseurl = fn.isEmpty () ?
                         value : value.left (value.length ()-fn.length ());
-                } else if (name == "__KHTML__CODEBASE") {
+                } else if (name == "__KHTML__CODEBASE")
                     khtml_codebase = value;
-                } else if (name_lower == QString::fromLatin1("codebase") ||
-                        name_lower == QString::fromLatin1("java_codebase")) {
+                else if (name_lower == QString::fromLatin1("codebase") ||
+                         name_lower == QString::fromLatin1("java_codebase")) {
                     if (!value.isEmpty ())
                         codebase = value;
                 } else if (name == "__KHTML__CLASSID")
@@ -200,7 +237,8 @@ KJavaAppletViewer::KJavaAppletViewer (QWidget * wparent, const char *,
                          name_lower == QString::fromLatin1("src"))
                     classname = value;
                 else if (name_lower == QString::fromLatin1("archive") ||
-                         name_lower == QString::fromLatin1("java_archieve"))
+                         name_lower == QString::fromLatin1("java_archive") ||
+                         name_lower.startsWith ("cache_archive"))
                     applet->setArchives (value);
                 else if (name_lower == QString::fromLatin1("name"))
                     applet->setAppletName (value);
@@ -226,31 +264,88 @@ KJavaAppletViewer::KJavaAppletViewer (QWidget * wparent, const char *,
     if (codebase.isEmpty ())
         codebase = khtml_codebase;
 
-    if (width > 0 && height > 0)
-        applet->setSize (QSize(width, height));
+    if (width > 0 && height > 0) {
+        m_view->resize (width, height);
+        applet->setSize( QSize( width, height ) );
+    }
     applet->setBaseURL (baseurl);
-    applet->setCodeBase (codebase);
+    // check codebase first
+    KURL kbaseURL( baseurl );
+    KURL newURL(kbaseURL, codebase);
+    if (kapp->authorizeURLAction("redirect", KURL(baseurl), newURL))
+        applet->setCodeBase (newURL.url());
     applet->setAppletClass (classname);
-    applet->setAppletContext (serverMaintainer->getContext (parent, baseurl));
+    KJavaAppletContext * cxt = serverMaintainer->getContext (parent, baseurl);
+    applet->setAppletContext (cxt);
+
+    KJavaAppletServer * server = cxt->getServer ();
+
+    serverMaintainer->setServer (server);
+
+    if (!server->usingKIO ()) {
+        /* if this page needs authentication */
+        KIO::AuthInfo info;
+        QString errorMsg;
+        QCString replyType;
+        QByteArray params;
+        QByteArray reply;
+        KIO::AuthInfo authResult;
+
+        //(void) dcopClient(); // Make sure to have a dcop client.
+        info.url = baseurl;
+        info.verifyPath = true;
+
+        QDataStream stream(params, IO_WriteOnly);
+        stream << info << m_view->topLevelWidget()->winId();
+
+        if (!kapp->dcopClient ()->call( "kded", "kpasswdserver", "checkAuthInfo(KIO::AuthInfo, long int)", params, replyType, reply ) ) {
+            kdWarning() << "Can't communicate with kded_kpasswdserver!" << endl;
+        } else if ( replyType == "KIO::AuthInfo" ) {
+            QDataStream stream2( reply, IO_ReadOnly );
+            stream2 >> authResult;
+            applet->setUser (authResult.username);
+            applet->setPassword (authResult.password);
+            applet->setAuthName (authResult.realmValue);
+        }
+    }
+
+    /* install event filter for close events */
+    if (wparent)
+        wparent->topLevelWidget ()->installEventFilter (this);
 
     setInstance (KJavaAppletViewerFactory::instance ());
     KParts::Part::setWidget (m_view);
-    insertChild (applet->getLiveConnectExtension ()); // hack
 
     connect (applet->getContext(), SIGNAL(appletLoaded()), this, SLOT(appletLoaded()));
     connect (applet->getContext(), SIGNAL(showDocument(const QString&, const QString&)), m_browserextension, SLOT(showDocument(const QString&, const QString&)));
     connect (applet->getContext(), SIGNAL(showStatus(const QString &)), this, SLOT(infoMessage(const QString &)));
+    connect (applet, SIGNAL(jsEvent (const QStringList &)), m_liveconnect, SLOT(jsEvent (const QStringList &)));
+}
+
+bool KJavaAppletViewer::eventFilter (QObject *o, QEvent *e) {
+    if (m_liveconnect->jsSessions () > 0) {
+        switch (e->type()) {
+            case QEvent::Destroy:
+            case QEvent::Close:
+            case QEvent::Quit:
+                return true;
+            default:
+                break;
+        }
+    }
+    return KParts::ReadOnlyPart::eventFilter(o,e);
 }
 
 KJavaAppletViewer::~KJavaAppletViewer () {
-    delete m_view;
+    m_view = 0L;
     serverMaintainer->releaseContext (parent(), baseurl);
-    delete m_browserextension;
 }
 
 bool KJavaAppletViewer::openURL (const KURL & url) {
     if (!m_view) return false;
-    KJavaApplet * applet = m_view->applet ();
+    m_closed = false;
+    KJavaAppletWidget * w = m_view->appletWidget ();
+    KJavaApplet * applet = w->applet ();
     if (applet->isCreated ())
         applet->stop ();
     if (applet->appletClass ().isEmpty ()) {
@@ -260,22 +355,40 @@ bool KJavaAppletViewer::openURL (const KURL & url) {
             applet->setBaseURL (url.upURL ().url ());
         } else
             applet->setAppletClass (url.url ());
-        AppletParameterDialog (m_view).exec ();
-        applet->setSize (m_view->sizeHint());
+        AppletParameterDialog (w).exec ();
+        applet->setSize (w->sizeHint());
     }
     // delay showApplet if size is unknown and m_view not shown
     if (applet->size().width() > 0 || m_view->isVisible())
-        m_view->showApplet ();
-    emit started (0L);
+        w->showApplet ();
+    if (!applet->failed ())
+        emit started (0L);
     return url.isValid ();
 }
 
+bool KJavaAppletViewer::closeURL () {
+    kdDebug(6100) << "closeURL" << endl;
+    m_closed = true;
+    KJavaApplet * applet = m_view->appletWidget ()->applet ();
+    if (applet->isCreated ())
+        applet->stop ();
+    applet->getContext()->getServer()->endWaitForReturnData();
+    return true;
+}
+
+bool KJavaAppletViewer::appletAlive () const {
+    return !m_closed && m_view &&
+           m_view->appletWidget ()->applet () &&
+           m_view->appletWidget ()->applet ()->isAlive ();
+}
+
 bool KJavaAppletViewer::openFile () {
     return false;
 }
 
 void KJavaAppletViewer::appletLoaded () {
-    KJavaApplet * applet = m_view->applet ();
+    if (!m_view) return;
+    KJavaApplet * applet = m_view->appletWidget ()->applet ();
     if (applet->isAlive() || applet->failed())
         emit completed();
 }
@@ -285,8 +398,7 @@ void KJavaAppletViewer::infoMessage (const QString & msg) {
 }
 
 KAboutData* KJavaAppletViewer::createAboutData () {
-    //return new KAboutData("KJavaAppletViewer", I18N_NOOP("KDE Java Applet Plugin"), "1.0");
-    return new KAboutData("KJavaAppletViewer", "KDE Java Applet Plugin", "1.0");
+    return new KAboutData("KJavaAppletViewer", I18N_NOOP("KDE Java Applet Plugin"), "1.0");
 }
 
 //---------------------------------------------------------------------
@@ -307,7 +419,7 @@ void KJavaAppletViewerBrowserExtension::setURLArgs (const KParts::URLArgs & /*ar
 }
 
 void KJavaAppletViewerBrowserExtension::saveState (QDataStream & stream) {
-    KJavaApplet * applet = static_cast<KJavaAppletViewer*>(parent())->view()->applet ();
+    KJavaApplet * applet = static_cast<KJavaAppletViewer*>(parent())->view()->appletWidget ()->applet ();
     stream << applet->appletClass();
     stream << applet->baseURL();
     stream << applet->archives();
@@ -320,8 +432,8 @@ void KJavaAppletViewerBrowserExtension::saveState (QDataStream & stream) {
 }
 
 void KJavaAppletViewerBrowserExtension::restoreState (QDataStream & stream) {
-    KJavaAppletViewer * viewer = static_cast<KJavaAppletViewer*>(parent());
-    KJavaApplet * applet = viewer->view()->applet ();
+    KJavaAppletWidget * w = static_cast<KJavaAppletViewer*>(parent())->view()->appletWidget();
+    KJavaApplet * applet = w->applet ();
     QString key, val;
     int paramcount;
     stream >> val;
@@ -337,9 +449,9 @@ void KJavaAppletViewerBrowserExtension::restoreState (QDataStream & stream) {
         applet->setParameter (key, val);
         kdDebug(6100) << "restoreState key:" << key << " val:" << val << endl;
     }
-    applet->setSize (viewer->view ()->sizeHint ());
-    if (viewer->view ()->isVisible())
-        viewer->view ()->showApplet ();
+    applet->setSize (w->sizeHint ());
+    if (w->isVisible())
+        w->showApplet ();
 }
 
 void KJavaAppletViewerBrowserExtension::showDocument (const QString & doc,
@@ -349,21 +461,111 @@ void KJavaAppletViewerBrowserExtension::showDocument (const QString & doc,
     args.frameName = frame;
     emit openURLRequest (url, args);
 }
-        
+
 //-----------------------------------------------------------------------------
-// TODO move this to kjavaappletwidget
 
-KJavaAppletViewerWidget::KJavaAppletViewerWidget(QWidget* parent, const char* name)
-  : KJavaAppletWidget(parent, name) {}
-
-void KJavaAppletViewerWidget::showEvent (QShowEvent * e) {
-    KJavaAppletWidget::showEvent(e);
-    if (!applet()->isCreated() && !applet()->appletClass().isEmpty()) {
-        // delayed showApplet
-        if (applet()->size().width() <= 0)
-            applet()->setSize (sizeHint());
-        showApplet();
-    }
+KJavaAppletViewerLiveConnectExtension::KJavaAppletViewerLiveConnectExtension(KJavaAppletViewer * parent)
+    : KParts::LiveConnectExtension (parent), m_viewer (parent) {
 }
 
+bool KJavaAppletViewerLiveConnectExtension::get (
+        const unsigned long objid, const QString & name,
+        KParts::LiveConnectExtension::Type & type,
+        unsigned long & rid, QString & value)
+{
+    if (!m_viewer->appletAlive ())
+        return false;
+    QStringList args, ret_args;
+    KJavaApplet * applet = m_viewer->view ()->appletWidget ()->applet ();
+    args.append (QString::number (applet->appletId ()));
+    args.append (QString::number ((int) objid));
+    args.append (name);
+    m_jssessions++;
+    bool ret = applet->getContext()->getMember (args, ret_args);
+    m_jssessions--;
+    if (!ret || ret_args.count() != 3) return false;
+    bool ok;
+    int itype = ret_args[0].toInt (&ok);
+    if (!ok || itype < 0) return false;
+    type = (KParts::LiveConnectExtension::Type) itype;
+    rid = ret_args[1].toInt (&ok);
+    if (!ok) return false;
+    value = ret_args[2];
+    return true;
+}
+
+bool KJavaAppletViewerLiveConnectExtension::put(const unsigned long objid, const QString & name, const QString & value)
+{
+    if (!m_viewer->appletAlive ())
+        return false;
+    QStringList args;
+    KJavaApplet * applet = m_viewer->view ()->appletWidget ()->applet ();
+    args.append (QString::number (applet->appletId ()));
+    args.append (QString::number ((int) objid));
+    args.append (name);
+    args.append (value);
+    m_jssessions++;
+    bool ret = applet->getContext()->putMember (args);
+    m_jssessions--;
+    return ret;
+}
+
+bool KJavaAppletViewerLiveConnectExtension::call( const unsigned long objid, const QString & func, const QStringList & fargs, KParts::LiveConnectExtension::Type & type, unsigned long & retobjid, QString & value )
+{
+    if (!m_viewer->appletAlive ())
+        return false;
+    KJavaApplet * applet = m_viewer->view ()->appletWidget ()->applet ();
+    QStringList args, ret_args;
+    args.append (QString::number (applet->appletId ()));
+    args.append (QString::number ((int) objid));
+    args.append (func);
+    for (QStringList::const_iterator it=fargs.begin(); it != fargs.end(); ++it)
+        args.append(*it);
+    m_jssessions++;
+    bool ret = applet->getContext()->callMember (args, ret_args);
+    m_jssessions--;
+    if (!ret || ret_args.count () != 3) return false;
+    bool ok;
+    int itype = ret_args[0].toInt (&ok);
+    if (!ok || itype < 0) return false;
+    type = (KParts::LiveConnectExtension::Type) itype;
+    retobjid = ret_args[1].toInt (&ok);
+    if (!ok) return false;
+    value = ret_args[2];
+    return true;
+}
+
+void KJavaAppletViewerLiveConnectExtension::unregister(const unsigned long objid)
+{
+    if (!m_viewer->view () || !m_viewer->view ())
+        return;
+    KJavaApplet * applet = m_viewer->view ()->appletWidget ()->applet ();
+    if (!applet || objid == 0) {
+        // typically a gc after a function call on the applet,
+        // no need to send to the jvm
+        return;
+    }
+    QStringList args;
+    args.append (QString::number (applet->appletId ()));
+    args.append (QString::number ((int) objid));
+    applet->getContext()->derefObject (args);
+}
+
+void KJavaAppletViewerLiveConnectExtension::jsEvent (const QStringList & args) {
+    if (args.count () < 2 || !m_viewer->appletAlive ())
+        return;
+    bool ok;
+    unsigned long objid = args[0].toInt(&ok);
+    QString event = args[1];
+    KParts::LiveConnectExtension::ArgList arglist;
+    for (unsigned i = 2; i < args.count(); i += 2)
+        // take a deep breath here
+        arglist.push_back(KParts::LiveConnectExtension::ArgList::value_type((KParts::LiveConnectExtension::Type) args[i].toInt(), args[i+1]));
+    emit partEvent (objid, event, arglist);
+}
+
+int KJavaAppletViewerLiveConnectExtension::m_jssessions = 0;
+
+//-----------------------------------------------------------------------------
+
 #include "kjavaappletviewer.moc"
