@@ -40,14 +40,7 @@ class KJavaAppletViewer;
 class KAboutData;
 class KInstance;
 class KConfig;
-
-/* FIXME: merge with KJavaAppletWidget */
-class KJavaAppletViewerWidget : public KJavaAppletWidget {
-public:
-    KJavaAppletViewerWidget (QWidget* parent=0, const char* name=0);
-protected:
-    void showEvent (QShowEvent *);
-};
+class CoverWidget;
 
 class KJavaAppletViewerBrowserExtension : public KParts::BrowserExtension {
     Q_OBJECT
@@ -63,29 +56,59 @@ public slots:
     void showDocument (const QString & doc, const QString & frame);
 };
 
+class KJavaAppletViewerLiveConnectExtension : public KParts::LiveConnectExtension {
+    Q_OBJECT
+public:
+    KJavaAppletViewerLiveConnectExtension(KJavaAppletViewer * parent);
+
+    bool get (const unsigned long objid, const QString & field, KParts::LiveConnectExtension::Type & type, unsigned long & retobjid, QString & value);
+    bool put(const unsigned long, const QString & field, const QString & value);
+    bool call (const unsigned long , const QString & func, const QStringList & args, KParts::LiveConnectExtension::Type & type, unsigned long & retobjid, QString & value);
+    void unregister (const unsigned long objid);
+
+    int jsSessions () const { return m_jssessions; }
+public slots:
+    void jsEvent (const QStringList & args);
+signals:
+    virtual void partEvent (const unsigned long objid, const QString & event, const KParts::LiveConnectExtension::ArgList & args);
+
+private:
+    KJavaAppletViewer * m_viewer;
+    static int m_jssessions;
+};
+
 class KJavaAppletViewer : public KParts::ReadOnlyPart {
     Q_OBJECT
 public: 
     KJavaAppletViewer (QWidget * wparent, const char * wname,
               QObject * parent, const char * name, const QStringList &args);
     ~KJavaAppletViewer ();
-    KJavaAppletWidget * view () const { return m_view; }
+    CoverWidget * view () const { return m_view; }
     static KAboutData* createAboutData ();
 
     KJavaAppletViewerBrowserExtension * browserextension() const
         { return m_browserextension; }
+    KParts::LiveConnectExtension * liveConnectExtension () const
+        { return m_liveconnect; }
+
+    bool eventFilter (QObject *o, QEvent *e);
+
+    bool appletAlive () const;
 public slots:
     virtual bool openURL (const KURL & url);
+    virtual bool closeURL ();
     void appletLoaded ();
     void infoMessage (const QString &);
 protected:
     bool openFile();
 private:
-    QGuardedPtr <KJavaAppletWidget> m_view;
+    QGuardedPtr <CoverWidget> m_view;
     KConfig * m_config;
     KJavaProcess * process;
     KJavaAppletViewerBrowserExtension * m_browserextension;
+    KJavaAppletViewerLiveConnectExtension * m_liveconnect;
     QString baseurl;
+    bool m_closed;
 };
 
 class KJavaAppletViewerFactory : public KParts::Factory {
