@@ -33,38 +33,52 @@ public class JSObject extends netscape.javascript.JSObject {
         return id;
     }
 
-    private Object evaluate(String _script, boolean global) throws netscape.javascript.JSException {
-        Main.info("evaluate (\"" + _script + "\")");
-        
-        // the following line only works from java 1.4 on
-        //String script = _script.replaceAll("\\\\", "\\\\\\\\\\\\\\\\").replaceAll("\"", "\\\\\\\\\\\\\\\"");
-        
-        // and this is the replacement for older versions:
+    private String escapeString(String string) {
         StringBuffer sb = new StringBuffer();
-        int idx;
-        int off;
-        String script = _script;
-        
-        for (off = 0; (idx = script.indexOf("\\", off)) >= 0; off = idx + 1) {
-            sb.append(script.substring(off, idx));
-            sb.append("\\\\\\\\");
+        int idx = 0;
+        boolean cr = false;
+        char [] chars = string.toCharArray();
+        while (idx < chars.length) {
+            if (cr && chars[idx] != '\n') {
+                cr = false;
+                sb.append("\\n");
+            }
+            switch (chars[idx]) {
+                case '\\':
+                    sb.append("\\\\");
+                    break;
+                case '"':
+                    sb.append("\\\"");
+                    break;
+                case '\n':
+                    cr = false;
+                    sb.append("\\n");
+                    break;
+                case '\r':
+                    cr = true;
+                    break;
+                default:
+                    sb.append(chars[idx]);
+            }
+            idx++;
         }
-        sb.append(script.substring(off, script.length()));
-        script = sb.toString();
-        sb = new StringBuffer();
-        for (off = 0; (idx = script.indexOf("\"", off)) >= 0; off = idx + 1) {
-            sb.append(script.substring(off, idx));
-            sb.append("\\\\\\\"");
+        if (cr)
+            sb.append("\\n");
+        return sb.toString();
+    }
+
+    private Object evaluate(String script, boolean global) throws netscape.javascript.JSException {
+        Main.info("evaluate (\"" + script + "\")");
+        if (!applet.isActive()) {
+            Main.debug("evaluate on not active applet");
+            return null;
         }
-        sb.append(script.substring(off, script.length()));
-        script = sb.toString();
-        // end of replacement
          
         KJASAppletContext kc = (KJASAppletContext) applet.getAppletContext();
         //String appletname = kc.getAppletName(appletID);
         thread = Thread.currentThread();
 
-        kc.evaluateJavaScript("window.__lc[1](" + id + ",\\\"" + script + "\\\",this" + (global ? ",true)" : ")"), appletID, this);
+        kc.evaluateJavaScript("window.__lc[1](" + id + ",\"" + escapeString(script) + "\",this" + (global ? ",true)" : ")"), appletID, this);
         boolean timedout = true;
         try {
             Thread.currentThread().sleep(30000);
@@ -93,42 +107,39 @@ public class JSObject extends netscape.javascript.JSObject {
         /* Is it an applet? */
         if (value.startsWith("[object APPLET ref=")) {
             int p1 = value.indexOf('=');
-            int p2 = value.indexOf(',', p1+1);
-            int p3 = value.indexOf(']', p2+1);
-            String appletid = value.substring(p2+1, p3);
-            //FIXME: get the correct context, not our own
-            //int contextid = Integer.parseInt(value.substring(p1+1, p2));
-            return kc.getAppletById(appletid);
+            int p2 = value.indexOf(']', p1+1);
+            int applethashcode = Integer.parseInt(value.substring(p1+1, p2));
+            java.util.Enumeration e = kc.getApplets();
+            while (e.hasMoreElements()) {
+                Applet applet = (Applet) e.nextElement();
+                if (applet.hashCode() == applethashcode)
+                    return applet;
+            }
+            return null;
         }
         /* Is it a Java object then? */
-        if (value.startsWith("[[embed ")) {
+        if (value.startsWith("[object ") && value.indexOf("ref=") > 0) {
             int p1 = value.indexOf("ref=");
-            int p2 = value.indexOf(',', p1+1);
-            int p3 = value.indexOf(',', p2+1);
-            int p4 = value.indexOf(']', p3+1);
-            //int contextid = Integer.parseInt(value.substring(p1+1, p2));
-            String appletid = value.substring(p2+1, p3);
-            int objindex = Integer.parseInt(value.substring(p3+1, p4));
-            return kc.getJSReferencedObject(objindex);
+            int p2 = value.indexOf(']', p1+4);
+            int objecthashcode = Integer.parseInt(value.substring(p1+4, p2));
+            return kc.getJSReferencedObject(objecthashcode);
         }
         /* Ok, make it a JSObject */
         return new JSObject(applet, value, Integer.parseInt(type));
     }
     private String convertValueJ2JS(Object o) {
-        /* FIXME: escape strings: ' -> \'
-         */
         if (o == null)
             return new String("null");
         if (o instanceof java.lang.Number || o instanceof java.lang.Boolean)
             return o.toString();
         if (o instanceof netscape.javascript.JSObject)
             return new String("window.__lc[0][" + ((JSObject)o).getId() + "]");
-        return new String("'" + o.toString() + "'");
+        return new String("\"" + escapeString(o.toString()) + "\"");
     }
     public Object call(String func, Object [] args) throws netscape.javascript.JSException {
         Main.info("JSObject.call: " + jsobject + "." + func);
         String script = new String("." + func + "(");
-        for (int i = 0; i < args.length; i++)
+        for (int i = 0; args != null && i < args.length; i++)
             script += (i > 0 ? "," : "") + convertValueJ2JS(args[i]);
         script += ")";
         return evaluate(script, false);
