@@ -15,7 +15,7 @@ public final class KJASAppletClassLoader
     extends URLClassLoader
 {
     private static Hashtable loaders = new Hashtable();
-    public static synchronized KJASAppletClassLoader getLoader( String docBase, String codeBase )
+    public static synchronized KJASAppletClassLoader getLoader( String docBase, String codeBase, String archives )
     {
         SecurityManager security = System.getSecurityManager();
         if (security != null) {
@@ -28,14 +28,18 @@ public final class KJASAppletClassLoader
             docBaseURL = new URL( docBase );
         
             URL codeBaseURL = getCodeBaseURL( docBaseURL, codeBase );
-            Main.debug( "CL: getLoader: key = " + codeBaseURL );
+            String key = codeBaseURL.toString();
+            if (archives != null)
+                key += archives;
 
-            loader = (KJASAppletClassLoader) loaders.get( codeBaseURL.toString() );
+            Main.debug( "CL: getLoader: key = " + key );
+
+            loader = (KJASAppletClassLoader) loaders.get( key );
             if( loader == null )
             {
                 URL [] urlList = {};
                 loader = new KJASAppletClassLoader( urlList, docBaseURL, codeBaseURL);
-                loaders.put( codeBaseURL.toString(), loader );
+                loaders.put( key, loader );
             }
             else
             {
@@ -234,6 +238,9 @@ public final class KJASAppletClassLoader
                 } catch (ClassFormatError cfe) {
                     Main.debug(name + ": Catched " + cfe + ". Trying to repair...");
                     rval = loadFixedClass( name );
+                } catch (Exception ex)
+                {
+                    Main.debug("findClass " + name + " " + ex.getMessage());
                 }
             }
         }
@@ -246,7 +253,7 @@ public final class KJASAppletClassLoader
     private Hashtable loadedClasses = new Hashtable();
     public synchronized Class loadClass( String name ) throws ClassNotFoundException
     {
-        Main.debug( dbgID + "loadClass, class name = " + name );
+        //Main.debug( dbgID + "loadClass, class name = " + name );
         //We need to be able to handle foo.class, so strip off the suffix
         String fixed_name = name;
         /*
@@ -273,7 +280,7 @@ public final class KJASAppletClassLoader
         }
         Class cl = findClass(fixed_name);
         //Class cl = super.loadClass(fixed_name);
-        Main.debug(dbgID + " returns class " + cl.getName() + " Classloader=" + cl.getClassLoader());
+        //Main.debug(dbgID + " returns class " + cl.getName() + " Classloader=" + cl.getClassLoader());
         return cl;
     }
 
