@@ -50,7 +50,6 @@ public class KJASAppletContext implements AppletContext
     private String myID;
     private KJASAppletClassLoader loader;
     private boolean active;
-    private int refcounter = 0;
     // a mapping JS referenced Java objects
     private Hashtable jsReferencedObjects;
     private final static KJASAuthenticator authenticator = new KJASAuthenticator();
@@ -167,17 +166,22 @@ public class KJASAppletContext implements AppletContext
         }
         try
         {
-            KJASAppletClassLoader loader =
-                KJASAppletClassLoader.getLoader( docBase, codeBase );
+            String sorted_archives = "";
+            TreeSet archive_set = new TreeSet();
             if( archives != null )
             {
                 StringTokenizer parser = new StringTokenizer( archives, ",", false );
                 while( parser.hasMoreTokens() )
-                {
-                    String jar = parser.nextToken().trim();
-                    loader.addArchiveName( jar );
-                }
+                    archive_set.add ( parser.nextToken().trim() );
             }
+            Iterator it = archive_set.iterator();
+            while (it.hasNext())
+                sorted_archives += (String) it.next();
+            KJASAppletClassLoader loader =
+                KJASAppletClassLoader.getLoader( docBase, codeBase, sorted_archives );
+            it = archive_set.iterator();
+            while (it.hasNext())
+                loader.addArchiveName( (String) it.next() );
             loader.paramsDone();
 
             KJASAppletStub stub = new KJASAppletStub
@@ -221,9 +225,9 @@ public class KJASAppletContext implements AppletContext
         else
         {
             //Main.debug( "stopping applet: " + appletID );
-            stub.destroyApplet();
-
             stubs.remove( appletID );
+
+            stub.destroyApplet();
         }
     }
 
@@ -430,8 +434,8 @@ public class KJASAppletContext implements AppletContext
             rettype[0] = JString;
         else {
             rettype[0] = JObject;
-            rettype[1] = ++refcounter;
-            jsReferencedObjects.put(new Integer(refcounter), obj);
+            rettype[1] = obj.hashCode();
+            jsReferencedObjects.put(new Integer(rettype[1]), obj);
         }
         value.insert(0, val);
         return rettype;
