@@ -5,7 +5,6 @@ import java.util.*;
 import java.net.*;
 import java.awt.*;
 import java.awt.event.*;
-
 import javax.swing.JFrame;
 
 
@@ -67,6 +66,7 @@ public final class KJASAppletStub
     private Applet                app;
     private Thread                runThread;
     private Thread                appletThread = null;
+    private Thread                destroyThread = null;
     KJASAppletStub                me;
 
     /**
@@ -110,6 +110,20 @@ public final class KJASAppletStub
         
     }
 
+    /**
+     * Helper function for ending the runThread and appletThread
+     **/
+    private void tryToStopThread(Thread thread) {
+        try {
+            thread.interrupt();
+        } catch (SecurityException se) {}
+        if (thread.isAlive()) {
+            try {
+                thread.join(5000);
+            } catch (InterruptedException ie) {}
+        }
+    }
+
     private void stateChange(int newState) {
         if (failed)
             return;
@@ -122,12 +136,14 @@ public final class KJASAppletStub
     
     private void setFailed(String why) {
         failed = true;
+        loader.removeStatusListener(panel);
         panel.stopAnimation();
+        panel.showFailed();
         Main.protocol.sendAppletFailed(context.getID(), appletID, why); 
     }
     
     void createApplet() {
-        panel = new KJASAppletPanel( appletSize );
+        panel = new KJASAppletPanel();
         frame = new JFrame(windowName);
         // under certain circumstances, it may happen that the
         // applet is not embedded but shown in a separate window.
@@ -142,7 +158,10 @@ public final class KJASAppletStub
                 }
             }
         );
-        frame.getContentPane().add( "Center", panel );
+        frame.getContentPane().add( panel, BorderLayout.CENTER );
+        if (Main.java_version > 1.399)
+            frame.setUndecorated(true);
+        frame.setLocation( 0, 0 );
         frame.pack();
         frame.setVisible(true);
         loader.addStatusListener(panel);
@@ -164,37 +183,54 @@ public final class KJASAppletStub
                     } catch (Exception e) {
                         Main.kjas_err("Class could not be loaded: " + className, e);
                         setFailed(e.toString());
-                        panel.showFailed();
                         return;
                     }
-                    stateChange(CLASS_LOADED);                
+                    if (Thread.interrupted())
+                        return;
+                    stateChange(CLASS_LOADED);
+                    Object object = null;
                     try {
                         synchronized (appletClass) {
-                           app = (Applet) appletClass.newInstance();
+                           object = appletClass.newInstance();
+                           app = (Applet) object;
                         }
                     }
                     catch( InstantiationException e ) {
                         Main.kjas_err( "Could not instantiate applet", e );
-                        panel.showFailed();
                         setFailed(e.toString());
                         return;
                     }
                     catch( IllegalAccessException e ) {
                         Main.kjas_err( "Could not instantiate applet", e );
-                        panel.showFailed();
+                        setFailed(e.toString());
+                        return;
+                    }
+                    catch ( ClassCastException e ) {
+                        if ( object != null && object instanceof java.awt.Component) {
+                            app = new Applet();
+                            app.setLayout(new BorderLayout());
+                            app.add( (Component) object, BorderLayout.CENTER);
+                        } else {
+                            setFailed(e.toString());
+                            return;
+                        }
+                    }
+                    catch ( Throwable e ) {
                         setFailed(e.toString());
                         return;
                     }
                 //} // synchronized
+                if (Thread.interrupted())
+                    return;
                 app.setStub( me );
                 stateChange(INSTANCIATED);
                 app.setVisible(false);
                 panel.setApplet( app );
-                
-                app.setSize(appletSize);
-                                
-                context.showStatus("Initializing Applet " + appletName + " ...");
-                
+                if (appletSize.getWidth() > 0)
+                    app.setBounds( 0, 0, appletSize.width, appletSize.height );
+                else
+                    app.setBounds( 0, 0, panel.getSize().width, panel.getSize().height );
+
                 try {
                     app.init();
                 } catch (Error er) {
@@ -209,13 +245,13 @@ public final class KJASAppletStub
                     return;
                 }
 
+                if (Thread.interrupted())
+                     return;
                 stateChange(INITIALIZED);
                 loader.removeStatusListener(panel);
                 app.setVisible(true);
-                app.setSize(appletSize);                
-                app.validate();
+                //panel.validate();
                
-                context.showStatus("Starting Applet " + appletName + " ...");
                 // stop the loading... animation 
                 panel.stopAnimation();
                 // create a new thread, so we know, when the applet was started
@@ -239,21 +275,23 @@ public final class KJASAppletStub
     */
     void startApplet()
     {
-        if( app != null ) {
-            active = true;                    
-            if (appletThread != null) {
+        if( app != null && state == INITIALIZED) {
+            active = true;
+            if (appletThread == null) {
                 appletThread = new Thread("KJAS-Applet-" + appletID + "-" + appletName) {
                     public void run() {
+                        frame.validate();
                         app.start();
-                        context.showStatus("Applet " + appletName + " started.");
                         app.repaint();
+                        appletThread = null;
                     }
                 };
                 appletThread.start();
-            } else {
+            } /*else {
+                frame.validate();
                 app.start();
                 app.repaint();
-            }
+            }*/
             stateChange(STARTED);
        }
     }
@@ -268,9 +306,21 @@ public final class KJASAppletStub
     void stopApplet()
     {
         if( app != null ) {
+            if( appletThread != null && appletThread.isAlive() ) {
+                Main.debug( "appletThread is active when stop is called" );
+                tryToStopThread(appletThread);
+            }
+            appletThread = null;
             active = false;
             app.stop();
             stateChange(STOPPED);
+            if (Main.java_version > 1.399) {
+                // kill the windowClosing listener(s)
+                WindowListener[] wl = frame.getWindowListeners();
+                for (int i = 0; wl != null && i < wl.length; i++)
+                    frame.removeWindowListener(wl[i]);
+            }
+            frame.hide();
         }
     }
 
@@ -291,30 +341,56 @@ public final class KJASAppletStub
     * Also marks the the applet as inactive.
     * @see java.applet.Applet#init()
     */
-    void destroyApplet()
+    synchronized void destroyApplet()
     {
-        if( app != null ) {
-            synchronized (app) {
-                if (active) {
-                    try {
-                        stopApplet();
-                    } catch (Exception e) {
+        if (state >= DESTROYED || destroyThread != null)
+            return;
+        destroyThread = new Thread("applet destroy thread") {
+            public void run() {
+                if( runThread != null && runThread.isAlive() ) {
+                    Main.debug( "runThread is active when stub is dying" );
+                    tryToStopThread(runThread);
+                    runThread = null;
+                    panel.stopAnimation();
+                    loader.removeStatusListener(panel);
+                }
+                if( app != null ) {
+                    synchronized (app) {
+                        if (active) {
+                            try {
+                                stopApplet();
+                            } catch (Exception e) {
+                            }
+                        }
+                        try {
+                            app.destroy();
+                        } catch (Exception e) {
+                        }
                     }
+                    frame.dispose();
+                    app = null;
+                    stateChange(DESTROYED);
                 }
-                try {
-                    app.destroy();
-                } catch (Exception e) {
-                }
-            }
-            stateChange(DESTROYED);
-        }
 
-        if( runThread != null && runThread.isAlive() )
-            Main.debug( "runThread is active when stub is dying" );
-        loader = null;
-        context = null;
-        app = null;
-        frame.dispose();
+                loader = null;
+                context = null;
+                destroyThread = null;
+            }
+        };
+        destroyThread.start();
+    }
+
+    static void waitForDestroyThreads()
+    {
+        Thread [] ts = new Thread[Thread.activeCount() + 5];
+        int len = Thread.enumerate(ts);
+        for (int i = 0; i < len; i++)
+            if (ts[i].getName() != null && 
+                ts[i].getName().equals("applet destroy thread")) {
+                try {
+                    ts[i].join(10000);
+                } catch (InterruptedException ie) {}
+            }
     }
 
     /**
@@ -327,10 +403,6 @@ public final class KJASAppletStub
             return app;
     }
 
-    Dimension getAppletSize()
-    {
-        return appletSize;
-    }
     /**
     * get a parameter value given in the &lt;APPLET&gt; tag 
     * @param name the name of the parameter
@@ -370,7 +442,6 @@ public final class KJASAppletStub
                 Main.debug( "Applet #" + appletID + ": appletResize to : (" + width + ", " + height + ")" );
                 Main.protocol.sendResizeAppletCmd( context.getID(), appletID, width, height );
                 appletSize = new Dimension( width, height );
-                panel.setAppletSize( appletSize );
                 //pack();
             }
         }
