@@ -1,3 +1,29 @@
+// -*- c-basic-offset: 2 -*-
+/* This file is part of the KDE project
+ *
+ * Copyright (C) 2000-2003 Simon Hausmann <hausmann@kde.org>
+ *               2001-2003 George Staikos <staikos@kde.org>
+ *               2001-2003 Laurent Montel <montel@kde.org>
+ *               2001-2003 Dirk Mueller <mueller@kde.org>
+ *               2001-2003 Waldo Bastian <bastian@kde.org>
+ *               2001-2003 David Faure <faure@kde.org>
+ *               2001-2003 Daniel Naber <dnaber@kde.org>
+ *
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Library General Public
+ * License as published by the Free Software Foundation; either
+ * version 2 of the License, or (at your option) any later version.
+ *
+ * This library is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Library General Public License for more details.
+ *
+ * You should have received a copy of the GNU Library General Public License
+ * along with this library; see the file COPYING.LIB.  If not, write to
+ * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+ * Boston, MA 02111-1307, USA.
+ */
 
 #include <assert.h>
 #include "khtml_ext.h"
@@ -81,13 +107,14 @@ void KHTMLPartBrowserExtension::editableWidgetFocused( QWidget *widget )
 
         m_connectedToClipboard = true;
     }
+    editableWidgetFocused();
 }
 
 void KHTMLPartBrowserExtension::editableWidgetBlurred( QWidget *widget )
 {
     QWidget *oldWidget = m_editableFormWidget;
 
-    m_editableFormWidget = widget;
+    m_editableFormWidget = 0;
     enableAction( "cut", false );
     enableAction( "paste", false );
     m_part->emitSelectionChanged();
@@ -106,13 +133,23 @@ void KHTMLPartBrowserExtension::editableWidgetBlurred( QWidget *widget )
 
         m_connectedToClipboard = false;
     }
+    editableWidgetBlurred();
 }
 
 void KHTMLPartBrowserExtension::setExtensionProxy( KParts::BrowserExtension *proxy )
 {
     if ( m_extensionProxy )
+    {
         disconnect( m_extensionProxy, SIGNAL( enableAction( const char *, bool ) ),
                     this, SLOT( extensionProxyActionEnabled( const char *, bool ) ) );
+        if ( m_extensionProxy->inherits( "KHTMLPartBrowserExtension" ) )
+        {
+            disconnect( m_extensionProxy, SIGNAL( editableWidgetFocused() ),
+                        this, SLOT( extensionProxyEditableWidgetFocused() ) );
+            disconnect( m_extensionProxy, SIGNAL( editableWidgetBlurred() ),
+                        this, SLOT( extensionProxyEditableWidgetBlurred() ) );
+        }
+    }
 
     m_extensionProxy = proxy;
 
@@ -120,6 +157,13 @@ void KHTMLPartBrowserExtension::setExtensionProxy( KParts::BrowserExtension *pro
     {
         connect( m_extensionProxy, SIGNAL( enableAction( const char *, bool ) ),
                  this, SLOT( extensionProxyActionEnabled( const char *, bool ) ) );
+        if ( m_extensionProxy->inherits( "KHTMLPartBrowserExtension" ) )
+        {
+            connect( m_extensionProxy, SIGNAL( editableWidgetFocused() ),
+                     this, SLOT( extensionProxyEditableWidgetFocused() ) );
+            connect( m_extensionProxy, SIGNAL( editableWidgetBlurred() ),
+                     this, SLOT( extensionProxyEditableWidgetBlurred() ) );
+        }
 
         enableAction( "cut", m_extensionProxy->isActionEnabled( "cut" ) );
         enableAction( "copy", m_extensionProxy->isActionEnabled( "copy" ) );
@@ -141,7 +185,7 @@ void KHTMLPartBrowserExtension::cut()
     }
 
     if ( !m_editableFormWidget )
-        return; // shouldn't happen
+        return;
 
     if ( m_editableFormWidget->inherits( "QLineEdit" ) )
         static_cast<QLineEdit *>( &(*m_editableFormWidget) )->cut();
@@ -162,6 +206,7 @@ void KHTMLPartBrowserExtension::copy()
     {
         // get selected text and paste to the clipboard
         QString text = m_part->selectedText();
+	text.replace( QChar( 0xa0 ), ' ' );
         QClipboard *cb = QApplication::clipboard();
         disconnect( cb, SIGNAL( selectionChanged() ), m_part, SLOT( slotClearSelection() ) );
         cb->setText(text);
@@ -185,7 +230,7 @@ void KHTMLPartBrowserExtension::paste()
     }
 
     if ( !m_editableFormWidget )
-        return; // shouldn't happen
+        return;
 
     if ( m_editableFormWidget->inherits( "QLineEdit" ) )
         static_cast<QLineEdit *>( &(*m_editableFormWidget) )->paste();
@@ -238,9 +283,17 @@ void KHTMLPartBrowserExtension::updateEditActions()
     enableAction( "cut", hasSelection );
 }
 
+void KHTMLPartBrowserExtension::extensionProxyEditableWidgetFocused() {
+	editableWidgetFocused();
+}
+
+void KHTMLPartBrowserExtension::extensionProxyEditableWidgetBlurred() {
+	editableWidgetBlurred();
+}
+
 void KHTMLPartBrowserExtension::extensionProxyActionEnabled( const char *action, bool enable )
 {
-    // only forward enableAction calls for actions we actually do foward
+    // only forward enableAction calls for actions we actually do forward
     if ( strcmp( action, "cut" ) == 0 ||
          strcmp( action, "copy" ) == 0 ||
          strcmp( action, "paste" ) == 0 ) {
@@ -264,77 +317,88 @@ public:
   KHTMLPart *m_khtml;
   KURL m_url;
   KURL m_imageURL;
-  KAction *m_paPrintFrame;
-  KAction *m_paSaveLinkAs;
-  KAction *m_paSaveImageAs;
-  KAction *m_paCopyLinkLocation;
-  KAction *m_paStopAnimations;
-  KAction *m_paCopyImageLocation;
-  KAction *m_paViewImage;
-  KAction *m_paReloadFrame;
-  KAction *m_paViewFrameSource;
-  KAction *m_paViewFrameInfo;
 };
 
 
 KHTMLPopupGUIClient::KHTMLPopupGUIClient( KHTMLPart *khtml, const QString &doc, const KURL &url )
+  : QObject( khtml )
 {
   d = new KHTMLPopupGUIClientPrivate;
   d->m_khtml = khtml;
   d->m_url = url;
-
+  bool isImage = false;
   setInstance( khtml->instance() );
 
-  actionCollection()->insert( khtml->actionCollection()->action( "selectAll" ) );
-  actionCollection()->insert( khtml->actionCollection()->action( "security" ) );
-  actionCollection()->insert( khtml->actionCollection()->action( "viewDocumentSource" ) );
-
-  // frameset? -> add "Reload Frame" etc.
-  if ( khtml->parentPart() )
-  {
-    d->m_paReloadFrame = new KAction( i18n( "Reload Frame" ), 0, this, SLOT( slotReloadFrame() ),
-                                      actionCollection(), "reloadframe" );
-    d->m_paViewFrameSource = new KAction( i18n( "View Frame Source" ), 0, d->m_khtml, SLOT( slotViewDocumentSource() ),
-                                          actionCollection(), "viewFrameSource" );
-    d->m_paViewFrameInfo = new KAction( i18n( "View Frame Information" ), 0, d->m_khtml, SLOT( slotViewPageInfo() ), actionCollection(), "viewPageInfo" );
-    // This one isn't in khtml_popupmenu.rc anymore, because Print isn't either,
-    // and because print frame is already in the toolbar and the menu.
-    // But leave this here, so that it's easy to readd it.
-    d->m_paPrintFrame = new KAction( i18n( "Print Frame..." ), "fileprint", 0, d->m_khtml->browserExtension(), SLOT( print() ), actionCollection(), "printFrame" );
-  } else {
-    actionCollection()->insert( khtml->actionCollection()->action( "viewPageInfo" ) );
-  }
-
-  actionCollection()->insert( khtml->actionCollection()->action( "setEncoding" ) );
-
-  if ( !url.isEmpty() )
-  {
-    d->m_paSaveLinkAs = new KAction( i18n( "&Save Link As..." ), 0, this, SLOT( slotSaveLinkAs() ),
-                                     actionCollection(), "savelinkas" );
-    d->m_paCopyLinkLocation = new KAction( i18n( "Copy Link Location" ), 0, this, SLOT( slotCopyLinkLocation() ),
-                                           actionCollection(), "copylinklocation" );
-  }
-
-  d->m_paStopAnimations = new KAction( i18n( "Stop Animations" ), 0, this, SLOT( slotStopAnimations() ),
-                                       actionCollection(), "stopanimations" );
-
   DOM::Element e;
   e = khtml->nodeUnderMouse();
 
   if ( !e.isNull() && (e.elementId() == ID_IMG ||
                        (e.elementId() == ID_INPUT && !static_cast<DOM::HTMLInputElement>(e).src().isEmpty())))
   {
-      if ( e.elementId() == ID_IMG )
-          d->m_imageURL = KURL( static_cast<DOM::HTMLImageElement>( e ).src().string() );
-      else
-          d->m_imageURL = KURL( static_cast<DOM::HTMLInputElement>( e ).src().string() );
-    d->m_paSaveImageAs = new KAction( i18n( "Save Image As..." ), 0, this, SLOT( slotSaveImageAs() ),
-                                      actionCollection(), "saveimageas" );
-    d->m_paCopyImageLocation = new KAction( i18n( "Copy Image Location" ), 0, this, SLOT( slotCopyImageLocation() ),
-                                            actionCollection(), "copyimagelocation" );
+    isImage=true;
+  }
+
+  if ( url.isEmpty() && !isImage )
+  {
+    KAction* copyAction = KStdAction::copy( d->m_khtml->browserExtension(), SLOT( copy() ), actionCollection(), "copy" );
+    copyAction->setText(i18n("&Copy Text"));
+    copyAction->setEnabled(d->m_khtml->browserExtension()->isActionEnabled( "copy" ));
+    actionCollection()->insert( khtml->actionCollection()->action( "selectAll" ) );
+    actionCollection()->insert( khtml->actionCollection()->action( "security" ) );
+    actionCollection()->insert( khtml->actionCollection()->action( "setEncoding" ) );
+    new KAction( i18n( "Stop Animations" ), 0, this, SLOT( slotStopAnimations() ),
+                 actionCollection(), "stopanimations" );
+  }
+
+  if ( !url.isEmpty() )
+  {
+    new KAction( i18n( "&Save Link As..." ), 0, this, SLOT( slotSaveLinkAs() ),
+                 actionCollection(), "savelinkas" );
+    new KAction( i18n( "Copy Link Location" ), 0, this, SLOT( slotCopyLinkLocation() ),
+                 actionCollection(), "copylinklocation" );
+  }
+
+  // frameset? -> add "Reload Frame" etc.
+  if ( khtml->parentPart() )
+  {
+    new KAction( i18n( "Open in New &Window" ), "window_new", 0, this, SLOT( slotFrameInWindow() ),
+                                        actionCollection(), "frameinwindow" );
+    new KAction( i18n( "Open in &New Tab" ), "tab_new", 0, this, SLOT( slotFrameInTab() ),
+                                     actionCollection(), "frameintab" );
+    new KAction( i18n( "Reload Frame" ), 0, this, SLOT( slotReloadFrame() ),
+                                      actionCollection(), "reloadframe" );
+    new KAction( i18n( "View Frame Source" ), 0, d->m_khtml, SLOT( slotViewDocumentSource() ),
+                                          actionCollection(), "viewFrameSource" );
+    new KAction( i18n( "View Frame Information" ), 0, d->m_khtml, SLOT( slotViewPageInfo() ), actionCollection(), "viewFrameInfo" );
+    // This one isn't in khtml_popupmenu.rc anymore, because Print isn't either,
+    // and because print frame is already in the toolbar and the menu.
+    // But leave this here, so that it's easy to readd it.
+    new KAction( i18n( "Print Frame..." ), "frameprint", 0, d->m_khtml->browserExtension(), SLOT( print() ), actionCollection(), "printFrame" );
+
+    actionCollection()->insert( khtml->parentPart()->actionCollection()->action( "viewDocumentSource" ) );
+    actionCollection()->insert( khtml->parentPart()->actionCollection()->action( "viewPageInfo" ) );
+  } else {
+    actionCollection()->insert( khtml->actionCollection()->action( "viewDocumentSource" ) );
+    actionCollection()->insert( khtml->actionCollection()->action( "viewPageInfo" ) );
+  }
+
+  if (isImage)
+  {
+    if ( e.elementId() == ID_IMG )
+      d->m_imageURL = KURL( static_cast<DOM::HTMLImageElement>( e ).src().string() );
+    else
+      d->m_imageURL = KURL( static_cast<DOM::HTMLInputElement>( e ).src().string() );
+    new KAction( i18n( "Save Image As..." ), 0, this, SLOT( slotSaveImageAs() ),
+                 actionCollection(), "saveimageas" );
+    new KAction( i18n( "Send Image" ), 0, this, SLOT( slotSendImage() ),
+                 actionCollection(), "sendimage" );
+
+
+    new KAction( i18n( "Copy Image Location" ), 0, this, SLOT( slotCopyImageLocation() ),
+                 actionCollection(), "copyimagelocation" );
     QString name = KStringHandler::csqueeze(d->m_imageURL.fileName()+d->m_imageURL.query(), 25);
-    d->m_paViewImage = new KAction( i18n( "View Image (%1)" ).arg(name.replace(QRegExp("&"), "&&")), 0, this, SLOT( slotViewImage() ),
-                                            actionCollection(), "viewimage" );
+    new KAction( i18n( "View Image (%1)" ).arg(name.replace("&", "&&")), 0, this, SLOT( slotViewImage() ),
+                 actionCollection(), "viewimage" );
   }
 
   setXML( doc );
@@ -358,6 +422,19 @@ void KHTMLPopupGUIClient::slotSaveLinkAs()
   saveURL( d->m_khtml->widget(), i18n( "Save Link As" ), d->m_url, metaData );
 }
 
+void KHTMLPopupGUIClient::slotSendImage()
+{
+    QStringList urls;
+    urls.append( d->m_imageURL.url());
+    QString subject = d->m_imageURL.url();
+    kapp->invokeMailer(QString::null, QString::null, QString::null, subject,
+                       QString::null, //body
+                       QString::null,
+                       urls); // attachments
+
+
+}
+
 void KHTMLPopupGUIClient::slotSaveImageAs()
 {
   KIO::MetaData metaData;
@@ -367,14 +444,14 @@ void KHTMLPopupGUIClient::slotSaveImageAs()
 
 void KHTMLPopupGUIClient::slotCopyLinkLocation()
 {
-  KURL::List lst;
-  lst.append( d->m_url );
 #ifndef QT_NO_MIMECLIPBOARD
   // Set it in both the mouse selection and in the clipboard
+  KURL::List lst;
+  lst.append( d->m_url );
   QApplication::clipboard()->setSelectionMode(true);
-  QApplication::clipboard()->setData( KURLDrag::newDrag( lst ) );
+  QApplication::clipboard()->setData( new KURLDrag( lst ) );
   QApplication::clipboard()->setSelectionMode(false);
-  QApplication::clipboard()->setData( KURLDrag::newDrag( lst ) );
+  QApplication::clipboard()->setData( new KURLDrag( lst ) );
 #else
   QApplication::clipboard()->setText( d->m_url.url() ); //FIXME(E): Handle multiple entries
 #endif
@@ -387,14 +464,14 @@ void KHTMLPopupGUIClient::slotStopAnimations()
 
 void KHTMLPopupGUIClient::slotCopyImageLocation()
 {
-  KURL::List lst;
-  lst.append( d->m_imageURL );
 #ifndef QT_NO_MIMECLIPBOARD
   // Set it in both the mouse selection and in the clipboard
+  KURL::List lst;
+  lst.append( d->m_imageURL);
   QApplication::clipboard()->setSelectionMode(true);
-  QApplication::clipboard()->setData( KURLDrag::newDrag( lst ) );
+  QApplication::clipboard()->setData( new KURLDrag( lst ) );
   QApplication::clipboard()->setSelectionMode(false);
-  QApplication::clipboard()->setData( KURLDrag::newDrag( lst ) );
+  QApplication::clipboard()->setData( new KURLDrag( lst ) );
 #else
   QApplication::clipboard()->setText(d->m_imageURL.url()); //FIXME(E): Handle multiple entries
 #endif
@@ -402,19 +479,36 @@ void KHTMLPopupGUIClient::slotCopyImageLocation()
 
 void KHTMLPopupGUIClient::slotViewImage()
 {
-  d->m_khtml->browserExtension()->createNewWindow(d->m_imageURL.url());
+  d->m_khtml->browserExtension()->createNewWindow(d->m_imageURL);
 }
 
 void KHTMLPopupGUIClient::slotReloadFrame()
 {
   KParts::URLArgs args( d->m_khtml->browserExtension()->urlArgs() );
   args.reload = true;
+  args.metaData()["referrer"] = d->m_khtml->pageReferrer();
   // reload document
   d->m_khtml->closeURL();
   d->m_khtml->browserExtension()->setURLArgs( args );
   d->m_khtml->openURL( d->m_khtml->url() );
 }
 
+void KHTMLPopupGUIClient::slotFrameInWindow()
+{
+  KParts::URLArgs args( d->m_khtml->browserExtension()->urlArgs() );
+  args.metaData()["referrer"] = d->m_khtml->pageReferrer();
+  args.metaData()["forcenewwindow"] = "true";
+  emit d->m_khtml->browserExtension()->createNewWindow( d->m_khtml->url(), args );
+}
+
+void KHTMLPopupGUIClient::slotFrameInTab()
+{
+  KParts::URLArgs args( d->m_khtml->browserExtension()->urlArgs() );
+  args.metaData()["referrer"] = d->m_khtml->pageReferrer();
+  args.setNewTab(true);
+  emit d->m_khtml->browserExtension()->createNewWindow( d->m_khtml->url(), args );
+}
+
 void KHTMLPopupGUIClient::saveURL( QWidget *parent, const QString &caption,
                                    const KURL &url,
                                    const QMap<QString, QString> &metadata,
@@ -436,11 +530,11 @@ void KHTMLPopupGUIClient::saveURL( QWidget *parent, const QString &caption,
       {
         QFileInfo info( destURL.path() );
         if( info.exists() )
-          query = KMessageBox::warningContinueCancel( parent, i18n( "A file named \"%1\" already exists. " "Are you sure you want to overwrite it?" ).arg( info.fileName() ), i18n( "Overwrite File?" ), i18n( "Overwrite" ) );
+          query = KMessageBox::warningYesNo( parent, i18n( "A file named \"%1\" already exists. " "Are you sure you want to overwrite it?" ).arg( info.fileName() ), i18n( "Overwrite File?" ), i18n( "Overwrite" ), KStdGuiItem::cancel() );
        }
-   } while ( query == KMessageBox::Cancel );
+   } while ( query == KMessageBox::No );
 
-  if ( !destURL.isMalformed() )
+  if ( destURL.isValid() )
     saveURL(url, destURL, metadata, cacheId);
 }
 
@@ -448,10 +542,10 @@ void KHTMLPopupGUIClient::saveURL( const KURL &url, const KURL &destURL,
                                    const QMap<QString, QString> &metadata,
                                    long cacheId )
 {
-    if ( !destURL.isMalformed() )
+    if ( destURL.isValid() )
     {
         bool saved = false;
-        if (KHTMLPageCache::self()->isValid(cacheId))
+        if (KHTMLPageCache::self()->isComplete(cacheId))
         {
             if (destURL.isLocalFile())
             {
@@ -479,27 +573,32 @@ void KHTMLPopupGUIClient::saveURL( const KURL &url, const KURL &destURL,
         }
         if(!saved)
         {
-            // DownloadManager <-> konqueror integration
-            // find if the integration is enabled
-            // the empty key  means no integration
-            KConfig *cfg = new KConfig("konquerorrc", false, false);
-            cfg->setGroup("HTML Settings");
-            QString downloadManger=cfg->readPathEntry("DownloadManager");
+          // DownloadManager <-> konqueror integration
+          // find if the integration is enabled
+          // the empty key  means no integration
+          // only use download manager for non-local urls!
+          bool downloadViaKIO = true;
+          if ( !url.isLocalFile() )
+          {
+            KConfig cfg("konquerorrc", false, false);
+            cfg.setGroup("HTML Settings");
+            QString downloadManger = cfg.readPathEntry("DownloadManager");
             if (!downloadManger.isEmpty())
             {
                 // then find the download manager location
                 kdDebug(1000) << "Using: "<<downloadManger <<" as Download Manager" <<endl;
-                QString cmd=KStandardDirs::findExe(downloadManger);
+                QString cmd = KStandardDirs::findExe(downloadManger);
                 if (cmd.isEmpty())
                 {
                     QString errMsg=i18n("The Download Manager (%1) could not be found in your $PATH ").arg(downloadManger);
                     QString errMsgEx= i18n("Try to reinstall it  \n\nThe integration with Konqueror will be disabled!");
                     KMessageBox::detailedSorry(0,errMsg,errMsgEx);
-                    cfg->writeEntry("DownloadManager",QString::null);
-                    cfg->sync ();
+                    cfg.writePathEntry("DownloadManager",QString::null);
+                    cfg.sync ();
                 }
                 else
                 {
+                    downloadViaKIO = false;
                     KURL cleanDest = destURL;
                     cleanDest.setPass( QString::null ); // don't put password into commandline
                     cmd += " " + KProcess::quote(url.url()) + " " +
@@ -508,15 +607,16 @@ void KHTMLPopupGUIClient::saveURL( const KURL &url, const KURL &destURL,
                     KRun::runCommand(cmd);
                 }
             }
-            else
-            {
-                KIO::Job *job = KIO::copy( url, destURL );
-                job->setMetaData(metadata);
-                job->addMetaData("MaxCacheSize", "0"); // Don't store in http cache.
-                job->addMetaData("cache", "cache"); // Use entry from cache if available.
-                job->setAutoErrorHandlingEnabled( true );
-            }
-            delete cfg;
+          }
+          
+          if ( downloadViaKIO )
+          {
+              KIO::Job *job = KIO::copy( url, destURL );
+              job->setMetaData(metadata);
+              job->addMetaData("MaxCacheSize", "0"); // Don't store in http cache.
+              job->addMetaData("cache", "cache"); // Use entry from cache if available.
+              job->setAutoErrorHandlingEnabled( true );
+          }
         } //end if(!saved)
     }
 }
@@ -546,6 +646,7 @@ bool KHTMLPartBrowserHostExtension::openURLInFrame( const KURL &url, const KPart
   return m_part->openURLInFrame( url, urlArgs );
 }
 
+// BCI: remove in KDE 4
 KHTMLZoomFactorAction::KHTMLZoomFactorAction( KHTMLPart *part, bool direction, const QString &text, const QString &icon, const QObject *receiver, const char *slot, QObject *parent, const char *name )
     : KAction( text, icon, 0, receiver, slot, parent, name )
 {
@@ -569,6 +670,29 @@ KHTMLZoomFactorAction::KHTMLZoomFactorAction( KHTMLPart *part, bool direction, c
     connect( m_popup, SIGNAL( activated( int ) ), this, SLOT( slotActivated( int ) ) );
 }
 
+KHTMLZoomFactorAction::KHTMLZoomFactorAction( KHTMLPart *part, bool direction, const QString &text, const QString &icon, const KShortcut &cut, const QObject *receiver, const char *slot, QObject *parent, const char *name )
+    : KAction( text, icon, cut, receiver, slot, parent, name )
+{
+    m_direction = direction;
+    m_part = part;
+
+    m_popup = new QPopupMenu;
+    m_popup->insertItem( i18n( "Default Font Size" ) );
+
+    int m = m_direction ? 1 : -1;
+
+    for ( int i = 1; i < 5; ++i )
+    {
+        int num = i * m;
+        QString numStr = QString::number( num );
+        if ( num > 0 ) numStr.prepend( '+' );
+
+        m_popup->insertItem( i18n( "Font Size %1" ).arg( numStr ) );
+    }
+
+    connect( m_popup, SIGNAL( activated( int ) ), this, SLOT( slotActivated( int ) ) );
+}
+
 KHTMLZoomFactorAction::~KHTMLZoomFactorAction()
 {
     delete m_popup;
@@ -580,7 +704,7 @@ int KHTMLZoomFactorAction::plug( QWidget *w, int index )
     if ( containerId == -1 || !w->inherits( "KToolBar" ) )
         return containerId;
 
-    KToolBarButton *button = static_cast<KToolBar *>( w )->getButton( menuId( containerId ) );
+    KToolBarButton *button = static_cast<KToolBar *>( w )->getButton( itemId( containerId ) );
     if ( !button )
         return containerId;
 
