@@ -1,14 +1,41 @@
+/* This file is part of the KDE project
+ *
+ * Copyright (C) 2000-2003 Simon Hausmann <hausmann@kde.org>
+ *               2001-2003 George Staikos <staikos@kde.org>
+ *               2001-2003 Laurent Montel <montel@kde.org>
+ *               2001-2003 Dirk Mueller <mueller@kde.org>
+ *               2001-2003 Waldo Bastian <bastian@kde.org>
+ *               2001-2003 David Faure <faure@kde.org>
+ *               2001-2003 Daniel Naber <dnaber@kde.org>
+ *
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Library General Public
+ * License as published by the Free Software Foundation; either
+ * version 2 of the License, or (at your option) any later version.
+ *
+ * This library is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Library General Public License for more details.
+ *
+ * You should have received a copy of the GNU Library General Public License
+ * along with this library; see the file COPYING.LIB.  If not, write to
+ * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+ * Boston, MA 02111-1307, USA.
+ */
+
 #ifndef __khtml_ext_h__
 #define __khtml_ext_h__
 
 #include "khtml_part.h"
 
+#include <qguardedptr.h>
 
 #include <kaction.h>
 #include <kio/global.h>
 
 /**
- * This is the BrowserExtension for a @ref KHTMLPart document. Please see the KParts documentation for
+ * This is the BrowserExtension for a KHTMLPart document. Please see the KParts documentation for
  * more information about the BrowserExtension.
  */
 class KHTMLPartBrowserExtension : public KParts::BrowserExtension
@@ -45,7 +72,12 @@ public slots:
 private slots:
     // connected to a frame's browserextensions enableAction signal
     void extensionProxyActionEnabled( const char *action, bool enable );
+    void extensionProxyEditableWidgetFocused();
+    void extensionProxyEditableWidgetBlurred();
 
+signals:
+    void editableWidgetFocused();
+    void editableWidgetBlurred();
 private:
     void callExtensionProxyMethod( const char *method );
 
@@ -93,10 +125,13 @@ private slots:
   void slotSaveLinkAs();
   void slotSaveImageAs();
   void slotCopyLinkLocation();
+  void slotSendImage();
   void slotStopAnimations();
   void slotCopyImageLocation();
   void slotViewImage();
   void slotReloadFrame();
+  void slotFrameInWindow();
+  void slotFrameInTab();
 private:
   class KHTMLPopupGUIClientPrivate;
   KHTMLPopupGUIClientPrivate *d;
@@ -106,7 +141,11 @@ class KHTMLZoomFactorAction : public KAction
 {
     Q_OBJECT
 public:
+    //BCI: remove in KDE 4
     KHTMLZoomFactorAction( KHTMLPart *part, bool direction, const QString &text, const QString &icon, const QObject *receiver, const char *slot, QObject *parent, const char *name );
+    KHTMLZoomFactorAction( KHTMLPart *part, bool direction, const QString &text,
+            const QString &icon, const KShortcut& cut, const QObject *receiver,
+            const char *slot, QObject *parent, const char *name );
     virtual ~KHTMLZoomFactorAction();
 
     virtual int plug( QWidget *w, int index );
