@@ -31,6 +31,7 @@
 
 #include <kparts/part.h>
 #include <kparts/browserextension.h>
+#include <kdemacros.h>
 
 #include <qregexp.h>
 
@@ -40,6 +41,7 @@ class KJSProxy;
 class KHTMLView;
 class KHTMLSettings;
 class KJavaAppletContext;
+class KJSErrorDlg;
 
 namespace DOM
 {
@@ -60,6 +62,11 @@ namespace DOM
   class EventListener;
 }
 
+namespace KJS
+{
+  class Interpreter;
+}
+
 namespace khtml
 {
   class DocLoader;
@@ -76,13 +83,19 @@ namespace khtml
   class RenderWidget;
   class CSSStyleSelector;
   class HTMLTokenizer;
+  class Decoder;
+  class XMLTokenizer;
 }
 
 namespace KJS {
     class Window;
     class WindowFunc;
+    class ExternalFunc;
     class JSEventListener;
+    class JSNodeFilter;
     class DOMDocument;
+    class SourceFile;
+    class ScheduledAction;
 }
 
 namespace KParts
@@ -91,57 +104,82 @@ namespace KParts
   class LiveConnectExtension;
 }
 
+namespace KWallet
+{
+  class Wallet;
+}
+
 /**
  * This class is khtml's main class. It features an almost complete
  * web browser, and html renderer.
  *
- * The easiest way to use this class (if you just want to display a an HTML
+ * The easiest way to use this class (if you just want to display an HTML
  * page at some URL) is the following:
  *
- * <pre>
+ * \code
  * KURL url = "http://www.kde.org";
  * KHTMLPart *w = new KHTMLPart();
  * w->openURL(url);
  * w->view()->resize(500, 400);
  * w->show();
- * </pre>
+ * \endcode
  *
- * If Java and JavaScript are enabled by default depends on the user's
- * settings. If you do not need them, and escpecially if you display
- * unfiltered data from untrusted sources, it is strongly recommended to 
- * turn them off. In that case, you should also turn off the automatic 
+ * Java and JavaScript are enabled by default depending on the user's
+ * settings. If you do not need them, and especially if you display
+ * unfiltered data from untrusted sources, it is strongly recommended to
+ * turn them off. In that case, you should also turn off the automatic
  * redirect and plugins:
  *
- * <pre>
+ * \code
  * w->setJScriptEnabled(false);
  * w->setJavaEnabled(false);
  * w->setMetaRefreshEnabled(false);
  * w->setPluginsEnabled(false);
- * </pre>
+ * \endcode
+ *
+ * You may also wish to disable external references.  This will prevent KHTML
+ * from loading images, frames, etc,  or redirecting to external sites.
+ *
+ * \code
+ * w->setOnlyLocalReferences(true);
+ * \endcode
  *
  * Some apps want to write their HTML code directly into the widget instead of
- * it opening an url. You can also do that in the following way:
+ * opening an url. You can do this in the following way:
  *
- * <pre>
+ * \code
  * QString myHTMLCode = ...;
  * KHTMLPart *w = new KHTMLPart();
  * w->begin();
  * w->write(myHTMLCode);
  * ...
  * w->end();
- * </pre>
+ * \endcode
  *
- * You can do as many calls to write as you want. But there are two
- * @ref write() methods, one accepting a @ref QString one accepting a
+ * You can do as many calls to write() as you wish.  There are two
+ * write() methods, one accepting a QString and one accepting a
  * @p char @p * argument. You should use one or the other
  * (but not both) since the method using
  * the @p char @p * argument does an additional decoding step to convert the
  * written data to Unicode.
  *
+ * It is also possible to write content to the HTML part using the
+ * standard streaming API from KParts::ReadOnlyPart. The usage of
+ * the API is similar to that of the begin(), write(), end() process
+ * described above as the following example shows:
+ *
+ * \code
+ * KHTMLPart *doc = new KHTMLPart();
+ * doc->openStream( "text/html", KURL() );
+ * doc->writeStream( QCString( "<html><body><p>KHTML Rocks!</p></body></html>" ) );
+ * doc->closeStream();
+ * \endcode
+ *
  * @short HTML Browser Widget
  * @author Lars Knoll (knoll@kde.org)
- * @version $Id: khtml_part.h,v 1.197.2.6 2003/07/10 12:13:14 waba Exp $
- * */
+ * @version $Id: khtml_part.h,v 1.248.2.4 2004/03/23 00:50:48 mueller Exp $
+ *
+ */
 class KHTMLPart : public KParts::ReadOnlyPart
 {
   Q_OBJECT
@@ -157,16 +195,20 @@ class KHTMLPart : public KParts::ReadOnlyPart
   friend class DOM::HTMLFormElementImpl;
   friend class khtml::RenderPartObject;
   friend class KJS::Window;
+  friend class KJS::ScheduledAction;
+  friend class KJS::JSNodeFilter;
   friend class KJS::WindowFunc;
+  friend class KJS::ExternalFunc;
   friend class KJS::JSEventListener;
   friend class KJS::DOMDocument;
+  friend class KJS::SourceFile;
   friend class KJSProxy;
   friend class KHTMLPartBrowserExtension;
   friend class DOM::DocumentImpl;
   friend class DOM::HTMLDocumentImpl;
   friend class KHTMLPartBrowserHostExtension;
   friend class khtml::HTMLTokenizer;
-  friend class XMLTokenizer;
+  friend class khtml::XMLTokenizer;
   friend class khtml::RenderWidget;
   friend class khtml::CSSStyleSelector;
   friend class KHTMLPartIface;
@@ -187,11 +229,11 @@ public:
    * Constructs a new KHTMLPart.
    *
    * KHTML basically consists of two objects: The KHTMLPart itself,
-   * holding the document data (DOM document), and the @ref KHTMLView,
-   * derived from @ref QScrollView, in which the document content is
+   * holding the document data (DOM document), and the KHTMLView,
+   * derived from QScrollView, in which the document content is
    * rendered in. You can specify two different parent objects for a
    * KHTMLPart, one parent for the KHTMLPart document and on parent
-   * for the @ref KHTMLView. If the second @p parent argument is 0L, then
+   * for the KHTMLView. If the second @p parent argument is 0L, then
    * @p parentWidget is used as parent for both objects, the part and
    * the view.
    */
@@ -208,22 +250,22 @@ public:
   /**
    * Opens the specified URL @p url.
    *
-   * Reimplemented from @ref KParts::ReadOnlyPart::openURL .
+   * Reimplemented from KParts::ReadOnlyPart::openURL .
    */
   virtual bool openURL( const KURL &url );
 
   /**
-   * Stops loading the document and kill all data requests (for images, etc.)
+   * Stops loading the document and kills all data requests (for images, etc.)
    */
   virtual bool closeURL();
 
   /**
-   * is called when a certain error situation (i.e. connection timed out) occured.
-   * default implementation either shows a KIO error dialog or loads a more verbose
-   * error description a as page, depending on the users configuration.
+   * Called when a certain error situation (i.e. connection timed out) occurred.
+   * The default implementation either shows a KIO error dialog or loads a more
+   * verbose error description a as page, depending on the users configuration.
    * @p job is the job that signaled the error situation
    */
-  virtual void showError(KIO::Job* job);
+  virtual void showError( KIO::Job* job );
 
   /**
    * Returns a reference to the DOM HTML document (for non-HTML documents, returns null)
@@ -241,10 +283,11 @@ public:
   DOM::Node activeNode() const;
 
   /**
-   * Returns a pointer to the @ref KParts::BrowserExtension.
+   * Returns a pointer to the KParts::BrowserExtension.
    */
   KParts::BrowserExtension *browserExtension() const;
   KParts::LiveConnectExtension *liveConnectExtension( const khtml::RenderPart *) const;
+
   /**
    * Returns a pointer to the HTML document's view.
    */
@@ -265,7 +308,36 @@ public:
   bool jScriptEnabled() const;
 
   /**
-   * Enable/disable the automatic forwarding by &lt;meta http-equiv="refresh" ....&gt;
+   * Returns the JavaScript interpreter the part is using. This method is
+   * mainly intended for applications which embed and extend the part and
+   * provides a mechanism for adding additional native objects to the
+   * interpreter (or removing the built-ins).
+   *
+   * One thing people using this method to add things to the interpreter must
+   * consider, is that when you start writing new content to the part, the
+   * interpreter is cleared. This includes both use of the
+   * begin( const KURL &, int, int ) method, and the openURL( const KURL & )
+   * method. If you want your objects to have a longer lifespan, then you must
+   * retain a KJS::Object yourself to ensure that the reference count of your
+   * custom objects never reaches 0. You will also need to re-add your
+   * bindings everytime this happens - one way to detect the need for this is
+   * to connect to the docCreated() signal, another is to reimplement the
+   * begin() method.
+   */
+  KJS::Interpreter *jScriptInterpreter();
+
+  /**
+   * Enable/disable statusbar messages
+   */
+  void setStatusMessagesEnabled( bool enable );
+
+  /**
+   * Returns @p true if status messages are enabled.
+   */
+  bool statusMessagesEnabled() const;
+
+  /**
+   * Enable/disable automatic forwarding by &lt;meta http-equiv="refresh" ....&gt;
    */
   void setMetaRefreshEnabled( bool enable );
 
@@ -275,15 +347,8 @@ public:
   bool metaRefreshEnabled() const;
 
   /**
-   * Execute the specified snippet of JavaScript code.
-   *
-   * Returns @p true if JavaScript was enabled, no error occured
-   * and the code returned true itself or @p false otherwise.
-   * @deprecated, use the one below.
-   */
-  QVariant executeScript( const QString &script );
-  /**
-   * Same as above except the Node parameter specifying the 'this' value.
+   * Same as executeScript( const QString & ) except with the Node parameter
+   * specifying the 'this' value.
    */
   QVariant executeScript( const DOM::Node &n, const QString &script );
 
@@ -292,6 +357,7 @@ public:
    * the users drags a link.
    */
   void setDNDEnabled( bool b );
+
   /**
    * Returns whether Dragn'n'Drop support is enabled or not.
    */
@@ -306,7 +372,7 @@ public:
   void setJavaEnabled( bool enable );
 
   /**
-   * Return if Java applet support is enabled/disabled.
+   * Return true if Java applet support is enabled, false if disabled
    */
   bool javaEnabled() const;
 
@@ -316,18 +382,18 @@ public:
   KJavaAppletContext *javaContext();
 
   /**
-   * Returns the java context of the applets. If no context exists yet, a new one is
-   * created.
+   * Returns the java context of the applets. If no context exists yet, a
+   * new one is created.
    */
   KJavaAppletContext *createJavaContext();
 
   /**
-   * Enables or disables plugins via, default is enabled
+   * Enables or disables plugins, default is enabled
    */
   void setPluginsEnabled( bool enable );
 
   /**
-   * Returns trie if plugins are enabled/disabled.
+   * Returns true if plugins are enabled/disabled.
    */
   bool pluginsEnabled() const;
 
@@ -335,7 +401,7 @@ public:
    * Specifies whether images contained in the document should be loaded
    * automatically or not.
    *
-   * @note Request will be ignored if called before @ref begin().
+   * @note Request will be ignored if called before begin().
    */
   void setAutoloadImages( bool enable );
   /**
@@ -349,23 +415,88 @@ public:
   /**
    * Security option.
    *
-   * Specify whether only local references ( stylesheets, images, scripts, subdocuments )
-   * should be loaded. ( default false - everything is loaded, if the more specific
-   * options allow )
+   * Specify whether only file:/ or data:/ urls are allowed to be loaded without
+   * user confirmation by KHTML.
+   * ( for example referenced by stylesheets, images, scripts, subdocuments, embedded elements ).
+   *
+   * This option is mainly intended for enabling the "mail reader mode", where you load untrusted
+   * content with a file:/ url.
+   *
+   * Please note that enabling this option currently automatically disables Javascript, 
+   * Java and Plugins support. This might change in the future if the security model
+   * is becoming more sophisticated, so don't rely on this behaviour.
+   *
+   * ( default false - everything is loaded unless forbidden by KApplication::authorizeURLAction).
    */
-  void setOnlyLocalReferences(bool enable);
+  void setOnlyLocalReferences( bool enable );
 
   /**
-   * Returnd whether references should be loaded ( default false )
+   * Returns whether only file:/ or data:/ references are allowed 
+   * to be loaded ( default false ). @see setOnlyLocalReferences.
    **/
   bool onlyLocalReferences() const;
 
+  /** Returns whether caret mode is on/off.
+   * @since 3.2
+   */
+  bool isCaretMode() const;
+
+  /**
+   * Returns @p true if the document is editable, @p false otherwise.
+   * @since 3.2
+   */
+  bool isEditable() const;
+
+  /**
+   * Sets the caret to the given position.
+   *
+   * If the given location is invalid, it will snap to the nearest valid
+   * location. Immediately afterwards a @p caretPositionChanged signal
+   * containing the effective position is emitted
+   * @param node node to set to
+   * @param offset zero-based offset within the node
+   * @param extendSelection If @p true, a selection will be spanned from the
+   *	last caret position to the given one. Otherwise, any existing selection
+   *	will be deselected.
+   * @since 3.2
+   */
+  void setCaretPosition(DOM::Node node, long offset, bool extendSelection = false);
+
+  /**
+   * Enumeration for displaying the caret.
+   * @param Visible caret is displayed
+   * @param Invisible caret is not displayed
+   * @param Blink caret toggles between visible and invisible
+   * @since 3.2
+   */
+  enum CaretDisplayPolicy {
+	CaretVisible, CaretInvisible, CaretBlink
+  };
+
+  /**
+   * Returns the current caret policy when the view is not focused.
+   * @since 3.2
+   */
+  CaretDisplayPolicy caretDisplayPolicyNonFocused() const;
+
+  /**
+   * Sets the caret display policy when the view is not focused.
+   *
+   * Whenever the caret is in use, this property determines how the
+   * caret should be displayed when the document view is not focused.
+   *
+   * The default policy is CaretInvisible.
+   * @param policy new display policy
+   * @since 3.2
+   */
+  void setCaretDisplayPolicyNonFocused(CaretDisplayPolicy policy);
+
 #ifndef KDE_NO_COMPAT
-  void enableJScript(bool e) { setJScriptEnabled(e); }
-  void enableJava(bool e) { setJavaEnabled(e); }
-  void enablePlugins(bool e) { setPluginsEnabled(e); }
-  void autoloadImages(bool e) { setAutoloadImages(e); }
-  void enableMetaRefresh(bool e) { setMetaRefreshEnabled(e); }
+  void enableJScript( bool e ) { setJScriptEnabled(e); }
+  void enableJava( bool e ) { setJavaEnabled(e); }
+  void enablePlugins( bool e ) { setPluginsEnabled(e); }
+  void autoloadImages( bool e ) { setAutoloadImages(e); }
+  void enableMetaRefresh( bool e ) { setMetaRefreshEnabled(e); }
   bool setCharset( const QString &, bool ) { return true; }
 
   KURL baseURL() const;
@@ -385,11 +516,11 @@ public:
   /**
    * Clears the widget and prepares it for new content.
    *
-   * If you want @ref url() to return
+   * If you want url() to return
    * for example "file:/tmp/test.html", you can use the following code:
-   * <PRE>
+   * \code
    * view->begin( KURL("file:/tmp/test.html" ) );
-   * </PRE>
+   * \endcode
    *
    * @param url is the url of the document to be displayed.  Even if you
    * are generating the HTML on the fly, it may be useful to specify
@@ -417,12 +548,12 @@ public:
    * Unicode.
    *
    * The @p len parameter is needed for streams encoded in utf-16,
-   * since these can have \0 chars in them. In case the encoding
+   * since these can have \\0 chars in them. In case the encoding
    * you're using isn't utf-16, you can safely leave out the length
    * parameter.
    *
-   * Attention: Don't mix calls to @ref write( const char *) with calls
-   * to @ref write( const QString & ).
+   * Attention: Don't mix calls to write( const char *) with calls
+   * to write( const QString & ).
    *
    * The result might not be what you want.
    */
@@ -438,21 +569,21 @@ public:
   virtual void write( const QString &str );
 
   /**
-   * Call this after your last call to @ref write().
+   * Call this after your last call to write().
    */
   virtual void end();
 
   /*
-   * Prints the current HTML page layed out for the printer.
+   * Prints the current HTML page laid out for the printer.
    *
    * (not implemented at the moment)
    */
   //    void print(QPainter *, int pageHeight, int pageWidth);
 
   /**
-   * Paints the HTML page to a QPainter. See @ref KHTMLView::paint for details
+   * Paints the HTML page to a QPainter. See KHTMLView::paint for details
    */
-  void paint(QPainter *, const QRect &, int = 0, bool * = 0);
+  void paint( QPainter *, const QRect &, int = 0, bool * = 0 );
 
   /**
    * Sets the encoding the page uses.
@@ -476,7 +607,7 @@ public:
    * This gives a wide range of possibilities to
    * change the layout of the page.
    */
-  void setUserStyleSheet(const KURL &url);
+  void setUserStyleSheet( const KURL &url );
 
   /**
    * Sets a user defined style sheet to be used on top of the HTML 4
@@ -485,7 +616,7 @@ public:
    * This gives a wide range of possibilities to
    * change the layout of the page.
    */
-  void setUserStyleSheet(const QString &styleSheet);
+  void setUserStyleSheet( const QString &styleSheet );
 
 public:
 
@@ -513,6 +644,20 @@ public:
    */
   bool gotoAnchor( const QString &name );
 
+  /**
+   * Go to the next anchor
+   *
+   * This is useful to navigate from outside the navigator
+   * @since 3.2
+   */
+  bool nextAnchor();
+
+  /**
+   * Go to previous anchor
+   * @since 3.2
+   */
+  bool prevAnchor();
+
   /**
    * Sets the cursor to use when the cursor is on a link.
    */
@@ -529,7 +674,7 @@ public:
   void findTextBegin();
 
   /**
-   * Finds the next occurence of the string or expression.
+   * Finds the next occurrence of the string or expression.
    * If isRegExp is true then str is converted to a QRegExp, and caseSensitive is ignored.
    */
   bool findTextNext( const QString &str, bool forward, bool caseSensitive, bool isRegExp );
@@ -561,6 +706,20 @@ public:
    */
   DOM::Range selection() const;
 
+  /**
+   * Returns the selected part of the HTML by returning the starting and end
+   * position.
+   *
+   * If there is no selection, both nodes and offsets are equal.
+   * @param startNode returns node selection starts in
+   * @param startOffset returns offset within starting node
+   * @param endNode returns node selection ends in
+   * @param endOffset returns offset within end node.
+   * @since 3.2
+   */
+  void selection(DOM::Node &startNode, long &startOffset,
+  		DOM::Node &endNode, long &endOffset) const;
+
   /**
    * Sets the current selection.
    */
@@ -569,7 +728,7 @@ public:
   /**
    * Has the user selected anything?
    *
-   *  Call @ref selectedText() to
+   *  Call selectedText() to
    * retrieve the selected text.
    *
    * @return @p true if there is text selected.
@@ -603,10 +762,10 @@ public:
 
   /**
    * Saves the KHTMLPart's complete state (including child frame
-   * objects) to the provided @ref QDataStream.
+   * objects) to the provided QDataStream.
    *
-   * This is called from the @ref saveState() method of the
-   *  @ref browserExtension().
+   * This is called from the saveState() method of the
+   * browserExtension().
    */
   virtual void saveState( QDataStream &stream );
   /**
@@ -615,8 +774,8 @@ public:
    *
    * @see saveState()
    *
-   * This is called from the @ref restoreState() method of the
-   * @ref browserExtension() .
+   * This is called from the restoreState() method of the
+   * browserExtension() .
    **/
   virtual void restoreState( QDataStream &stream );
 
@@ -661,7 +820,7 @@ public:
 
   /**
    * Returns whether a frame with the specified name is exists or not.
-   * In contrary to the @ref findFrame method this one also returns true
+   * In contrary to the findFrame method this one also returns true
    * if the frame is defined but no displaying component has been
    * found/loaded, yet.
    */
@@ -714,18 +873,46 @@ public:
   /**
    * Loads a style sheet into the stylesheet cache.
    */
-  void preloadStyleSheet(const QString &url, const QString &stylesheet);
+  void preloadStyleSheet( const QString &url, const QString &stylesheet );
 
   /**
    * Loads a script into the script cache.
    */
-  void preloadScript(const QString &url, const QString &script);
+  void preloadScript( const QString &url, const QString &script );
 
   /**
    * @internal
    */
   bool restored() const;
 
+  // ### KDE4 remove me
+  enum FormNotification { NoNotification = 0, Before, Only, Unused=255 };
+  /**
+   * Determine if signal should be emitted before, instead or never when a
+   * submitForm() happens.
+   * @since 3.2
+   * ### KDE4 remove me
+   */
+  void setFormNotification(FormNotification fn);
+
+  /**
+   * Determine if signal should be emitted before, instead or never when a
+   * submitForm() happens.
+   * ### KDE4 remove me
+   * @since 3.2
+   */
+  FormNotification formNotification() const;
+
+  /**
+   * Returns the toplevel (origin) URL of this document, even if this
+   * part is a frame or an iframe.
+   *
+   * @return the actual orginal url.
+   * @since 3.2
+   */
+  KURL toplevelURL();
+
+
 signals:
   /**
    * Emitted if the cursor is moved over an URL.
@@ -735,7 +922,7 @@ signals:
   /**
    * Emitted when the user clicks the right mouse button on the document.
    */
-  void popupMenu(const QString &url, const QPoint &point);
+  void popupMenu( const QString &url, const QPoint &point );
 
   /**
    * This signal is emitted when the selection changes.
@@ -749,12 +936,37 @@ signals:
    * has explicitly been deactivated without a new one
    * becoming active.
    */
-  void nodeActivated(const DOM::Node &);
+  void nodeActivated( const DOM::Node & );
 
   /**
    * @internal */
   void docCreated();
 
+  /**
+   * This signal is emitted whenever the caret position has been changed.
+   *
+   * The signal transmits the position the DOM::Range way, the node and
+   * the zero-based offset within this node.
+   * @param node node which the caret is in. This can be null if the caret
+   *	has been deactivated.
+   * @param offset offset within the node. If the node is null, the offset
+   *	is meaningless.
+   * @since 3.2
+   */
+  void caretPositionChanged(const DOM::Node &node, long offset);
+
+
+  /**
+   * If form notification is on, this will be emitted either for a form
+   * submit or before the form submit according to the setting.
+   * @since 3.2
+   * ### KDE4 remove me
+   */
+  void formSubmitNotification(const char *action, const QString& url,
+                  const QByteArray& formData, const QString& target,
+                  const QString& contentType, const QString& boundary);
+
+
 protected:
 
   /**
@@ -769,7 +981,7 @@ protected:
    * @p text kio additional information text.
    * @p url the url that triggered the error.
    */
-  void htmlError(int errorCode, const QString& text, const KURL& reqUrl);
+  void htmlError( int errorCode, const QString& text, const KURL& reqUrl );
 
   virtual void customEvent( QCustomEvent *event );
 
@@ -800,7 +1012,7 @@ protected:
   virtual void guiActivateEvent( KParts::GUIActivateEvent *event );
 
   /**
-   * Internal empty reimplementation of @ref KParts::ReadOnlyPart::openFile .
+   * Internal empty reimplementation of KParts::ReadOnlyPart::openFile .
    */
   virtual bool openFile();
 
@@ -822,21 +1034,39 @@ protected:
 
   // This is for RenderPartObject. We want to ask the 'download plugin?'
   // question only once per mimetype
-  bool pluginPageQuestionAsked(const QString& mimetype) const;
-  void setPluginPageQuestionAsked(const QString& mimetype);
+  bool pluginPageQuestionAsked( const QString& mimetype ) const;
+  void setPluginPageQuestionAsked( const QString& mimetype );
+
+  enum PageSecurity { NotCrypted, Encrypted, Mixed };
+  void setPageSecurity( PageSecurity sec );
+
+  /**
+   * Implements the streaming API of KParts::ReadOnlyPart.
+   */
+  virtual bool doOpenStream( const QString& mimeType );
+
+  /**
+   * Implements the streaming API of KParts::ReadOnlyPart.
+   */
+  virtual bool doWriteStream( const QByteArray& data );
+
+  /**
+   * Implements the streaming API of KParts::ReadOnlyPart.
+   */
+  virtual bool doCloseStream();
 
 public slots:
 
   /**
-   * Sets the focussed node of the document to the specified node. If the node is a form control, the control will
+   * Sets the focused node of the document to the specified node. If the node is a form control, the control will
    * receive focus in the same way that it would if the user had clicked on it or tabbed to it with the keyboard. For
-   * most other types of elements, there is no visul indiction of whether or not they are focussed.
+   * most other types of elements, there is no visual indication of whether or not they are focused.
    *
-   * See @ref activeNode
+   * See activeNode
    *
    * @param node The node to focus
    */
-  void setActiveNode(const DOM::Node &node);
+  void setActiveNode( const DOM::Node &node );
 
   /**
    * Stops all animated images on the current and child pages
@@ -845,6 +1075,61 @@ public slots:
 
   QCString dcopObjectId() const;
 
+  /**
+   * Execute the specified snippet of JavaScript code.
+   *
+   * Returns @p true if JavaScript was enabled, no error occurred
+   * and the code returned true itself or @p false otherwise.
+   * @deprecated, use executeString( DOM::Node(), script)
+   */
+  QVariant executeScript( const QString &script );
+
+  /**
+   * Enables/disables caret mode.
+   *
+   * Enabling caret mode displays a caret which can be used to navigate
+   * the document using the keyboard only. Caret mode is switched off by
+   * default.
+   *
+   * @param enable @p true to enable, @p false to disable caret mode.
+   * @since 3.2
+   */
+  void setCaretMode(bool enable);
+
+  /**
+   * do not use
+   * @internal
+   */
+  void setEditable(bool enable);
+
+  /**
+   * Sets the visibility of the caret.
+   *
+   * This methods displays or hides the caret regardless of the current
+   * caret display policy (see setCaretDisplayNonFocused), and regardless
+   * of focus.
+   *
+   * The caret will be shown/hidden only under at least one of
+   * the following conditions:
+   * @li the document is editable
+   * @li the document is in caret mode
+   * @li the document's currently focused element is editable
+   *
+   * @param show @p true to make visible, @p false to hide.
+   * @since 3.2
+   */
+  void setCaretVisible(bool show);
+
+  // ### KDE4 FIXME:
+  //          Remove this and make the one below protected+virtual slot.
+  //          Warning: this is effectively "internal".  Be careful.
+  // @since 3.2
+  void submitFormProxy( const char *action, const QString &url,
+                        const QByteArray &formData,
+                        const QString &target,
+                        const QString& contentType = QString::null,
+                        const QString& boundary = QString::null );
+
 private slots:
 
   /**
@@ -879,7 +1164,11 @@ private slots:
   /**
    * @internal
    */
-  void slotRedirection(KIO::Job*, const KURL&);
+  void slotRedirection( KIO::Job*, const KURL& );
+  /**
+   * @internal
+   */
+  void slotDebugScript();
   /**
    * @internal
    */
@@ -888,6 +1177,10 @@ private slots:
    * @internal
    */
   void slotDebugRenderTree();
+  /**
+   * @internal
+   */
+  void slotStopAnimations();
   /**
    * @internal
    */
@@ -935,6 +1228,9 @@ private slots:
   void slotDecZoom();
 
   void slotLoadImages();
+  void slotWalletClosed();
+  void launchWalletManager();
+  void walletMenu();
 
   /**
    * @internal
@@ -982,11 +1278,6 @@ private slots:
   void slotLoaderRequestDone( khtml::DocLoader*, khtml::CachedObject *obj );
   void checkCompleted();
 
-  /**
-   * @internal
-   */
-  void slotShowDocument( const QString &url, const QString &target );
-
   /**
    * @internal
    */
@@ -1004,17 +1295,22 @@ private slots:
   /*
    * @internal
    */
-  void slotJobPercent(KIO::Job*, unsigned long);
+  void slotJobPercent( KIO::Job*, unsigned long );
 
   /*
    * @internal
    */
-  void slotJobDone(KIO::Job*);
+  void slotJobDone( KIO::Job* );
 
   /*
    * @internal
    */
-  void slotJobSpeed(KIO::Job*, unsigned long);
+  void slotUserSheetStatDone( KIO::Job* );
+
+  /*
+   * @internal
+   */
+  void slotJobSpeed( KIO::Job*, unsigned long );
 
   /**
    * @internal
@@ -1031,53 +1327,80 @@ private slots:
    */
   void slotHighlight( const QString &, int index, int length );
 
+  /**
+   * @internal
+   */
+  void slotAutomaticDetectionLanguage( int _id );
+
+  /**
+   * @internal
+   */
+  void slotToggleCaretMode();
+
+  /**
+   * @internal
+   */
+  void launchJSErrorDialog();
+
+  /**
+   * @internal
+   */
+  void removeJSErrorExtension();
+
+  /**
+   * @internal
+   */
+  void disableJSErrorExtension();
+
+  /**
+   * @internal
+   */
+  void jsErrorDialogContextMenu();
+
+
+
 private:
 
+  KJSErrorDlg *jsErrorExtension();
+
   enum StatusBarPriority { BarDefaultText, BarHoverText, BarOverrideText };
   void setStatusBarText( const QString& text, StatusBarPriority p);
 
-  /**
-   * @internal
-   */
   bool restoreURL( const KURL &url );
-
-  /**
-   * @internal
-   */
   void emitSelectionChanged();
-
-  /**
-   * @internal
-   */
   bool openURLInFrame( const KURL &url, const KParts::URLArgs &urlArgs );
-
   void startAutoScroll();
   void stopAutoScroll();
   void overURL( const QString &url, const QString &target, bool shiftPressed = false );
 
-  /**
-   * @internal
-   */
   bool processObjectRequest( khtml::ChildFrame *child, const KURL &url, const QString &mimetype );
 
+  KWallet::Wallet* wallet();
+
   /**
    * @internal
    */
+  // ### KDE4 FIXME:
+  //          It is desirable to be able to filter form submissions as well.
+  //          For instance, forms can have a target and an inheriting class
+  //          might want to filter based on the target.  Make this protected
+  //          and virtual, or provide a better solution.
+  //          See the web_module for the sidebar for an example where this is
+  //          necessary.
   void submitForm( const char *action, const QString &url, const QByteArray &formData,
                    const QString &target, const QString& contentType = QString::null,
                    const QString& boundary = QString::null );
 
-  /**
-   * @internal
-   */
   void popupMenu( const QString &url );
 
   void init( KHTMLView *view, GUIProfile prof );
 
+
   void clear();
 
   bool scheduleScript( const DOM::Node &n, const QString& script);
 
+  QVariant crossFrameExecuteScript(const QString& target, const QString& script);
   QVariant executeScheduledScript();
 
   bool requestFrame( khtml::RenderPart *frame, const QString &url, const QString &frameName,
@@ -1097,7 +1420,7 @@ private:
 
   bool requestObject( khtml::ChildFrame *child, const KURL &url, const KParts::URLArgs &args = KParts::URLArgs() );
 
-  DOM::EventListener *createHTMLEventListener( QString code );
+  DOM::EventListener *createHTMLEventListener( QString code, QString name );
 
   DOM::HTMLDocumentImpl *docImpl() const;
   DOM::DocumentImpl *xmlDocImpl() const;
@@ -1105,25 +1428,53 @@ private:
 
   khtml::ChildFrame *recursiveFrameRequest( const KURL &url, const KParts::URLArgs &args, bool callParent = true );
 
-  bool checkLinkSecurity(const KURL &linkURL,const QString &message = QString::null, const QString &button = QString::null);
-  QVariant executeScript(QString filename, int baseLine, const DOM::Node &n, const QString &script);
+  bool checkLinkSecurity( const KURL &linkURL,const QString &message = QString::null, const QString &button = QString::null );
+  QVariant executeScript( const QString& filename, int baseLine, const DOM::Node &n, const QString& script );
 
   KJSProxy *jScript();
 
   KHTMLPart *opener();
   long cacheId() const;
-  void setOpener(KHTMLPart *_opener);
+  void setOpener( KHTMLPart *_opener );
   bool openedByJS();
-  void setOpenedByJS(bool _openedByJS);
+  void setOpenedByJS( bool _openedByJS );
 
   void checkEmitLoadEvent();
   void emitLoadEvent();
 
-  bool initFindNode( bool selection, bool reverse );
+  bool initFindNode( bool selection, bool reverse, bool fromCursor );
   void findText();
   void findTextNext();
   void extendSelection( DOM::NodeImpl* node, long offset, DOM::Node& selectionNode, long& selectionOffset, bool right, bool paragraph );
+  /** extends the current selection to the given content-coordinates @p x, @p y
+   * @param x content x-coordinate
+   * @param y content y-coordinate
+   * @param absX absolute x-coordinate of @p innerNode
+   * @param absY absolute y-coordinate of @p innerNode
+   * @param innerNode node from which to start extending the selection. The
+   *	caller has to ensure that the node has a renderer.
+   * @internal
+   */
+  void extendSelectionTo(int x, int y, int absX, int absY, const DOM::Node &innerNode);
+  /** checks whether a selection is extended.
+   * @return @p true if a selection is extended by the mouse.
+   */
+  bool isExtendingSelection() const;
+  khtml::Decoder *createDecoder();
 
+  /** @internal
+   * generic zoom in
+   */
+  void zoomIn(const int stepping[], int count);
+  /** @internal
+   * generic zoom out
+   */
+  void zoomOut(const int stepping[], int count);
+
+  void emitCaretPositionChanged(const DOM::Node &node, long offset);
+
+  void setDebugScript( bool enable );
+  
   KHTMLPartPrivate *d;
   friend class KHTMLPartPrivate;
 };
