@@ -18,7 +18,6 @@
 */
 
 #include <qfontdatabase.h>
-#include <qregexp.h>
 
 #include "khtml_settings.h"
 #include "khtmldefaults.h"
@@ -29,8 +28,37 @@
 #include <kdebug.h>
 #include <qregexp.h>
 
+/**
+ * @internal
+ * Contains all settings which are both available globally and per-domain
+ */
+struct KPerDomainSettings {
+    bool m_bEnableJava : 1;
+    bool m_bEnableJavaScript : 1;
+    bool m_bEnablePlugins : 1;
+    // don't forget to maintain the bitfields as the enums grow
+    KHTMLSettings::KJSWindowOpenPolicy m_windowOpenPolicy : 2;
+    KHTMLSettings::KJSWindowStatusPolicy m_windowStatusPolicy : 1;
+    KHTMLSettings::KJSWindowFocusPolicy m_windowFocusPolicy : 1;
+    KHTMLSettings::KJSWindowMovePolicy m_windowMovePolicy : 1;
+    KHTMLSettings::KJSWindowResizePolicy m_windowResizePolicy : 1;
 
-typedef QMap<QString,KHTMLSettings::KJavaScriptAdvice> PolicyMap;
+#ifdef DEBUG_SETTINGS
+    void dump(const QString &infix = QString::null) const {
+      kdDebug() << "KPerDomainSettings " << infix << " @" << this << ":" << endl;
+      kdDebug() << "  m_bEnableJava: " << m_bEnableJava << endl;
+      kdDebug() << "  m_bEnableJavaScript: " << m_bEnableJavaScript << endl;
+      kdDebug() << "  m_bEnablePlugins: " << m_bEnablePlugins << endl;
+      kdDebug() << "  m_windowOpenPolicy: " << m_windowOpenPolicy << endl;
+      kdDebug() << "  m_windowStatusPolicy: " << m_windowStatusPolicy << endl;
+      kdDebug() << "  m_windowFocusPolicy: " << m_windowFocusPolicy << endl;
+      kdDebug() << "  m_windowMovePolicy: " << m_windowMovePolicy << endl;
+      kdDebug() << "  m_windowResizePolicy: " << m_windowResizePolicy << endl;
+    }
+#endif
+};
+
+typedef QMap<QString,KPerDomainSettings> PolicyMap;
 
 class KHTMLSettingsPrivate
 {
@@ -39,14 +67,16 @@ public:
     bool m_bBackRightClick : 1;
     bool m_underlineLink : 1;
     bool m_hoverLink : 1;
+    bool m_bEnableJavaScriptDebug : 1;
+    bool m_bEnableJavaScriptErrorReporting : 1;
     bool enforceCharset : 1;
     bool m_bAutoLoadImages : 1;
-    bool m_bEnableJava : 1;
-    bool m_bEnableJavaScript : 1;
-    bool m_bEnableJavaScriptDebug : 1;
-    bool m_bEnablePlugins : 1;
     bool m_formCompletionEnabled : 1;
-    bool m_autoDelayedActionsEnabled: 1;
+    bool m_autoDelayedActionsEnabled : 1;
+    bool m_jsErrorsEnabled : 1;
+
+    // the virtual global "domain"
+    KPerDomainSettings global;
 
     int m_fontSize;
     int m_minFontSize;
@@ -60,13 +90,32 @@ public:
     QColor m_linkColor;
     QColor m_vLinkColor;
 
-    QMap<QString,KHTMLSettings::KJavaScriptAdvice> javaDomainPolicy;
-    QMap<QString,KHTMLSettings::KJavaScriptAdvice> javaScriptDomainPolicy;
+    PolicyMap domainPolicy;
     QStringList fonts;
     QStringList defaultFonts;
 };
 
 
+/** Returns a writeable per-domains settings instance for the given domain
+  * or a deep copy of the global settings if not existent.
+  */
+static KPerDomainSettings &setup_per_domain_policy(
+				KHTMLSettingsPrivate *d,
+				const QString &domain) {
+  if (domain.isEmpty()) {
+    kdWarning() << "setup_per_domain_policy: domain is empty" << endl;
+  }
+  QString ldomain = domain.lower();
+  PolicyMap::iterator it = d->domainPolicy.find(ldomain);
+  if (it == d->domainPolicy.end()) {
+    // simply copy global domain settings (they should have been initialized
+    // by this time)
+    it = d->domainPolicy.insert(ldomain,d->global);
+  }
+  return *it;
+}
+
+
 KHTMLSettings::KJavaScriptAdvice KHTMLSettings::strToAdvice(const QString& _str)
 {
   KJavaScriptAdvice ret = KJavaScriptDunno;
@@ -122,6 +171,74 @@ void KHTMLSettings::splitDomainAdvice(const QString& configStr, QString &domain,
     }
 }
 
+void KHTMLSettings::readDomainSettings(KConfig *config, bool reset,
+	bool global, KPerDomainSettings &pd_settings) {
+  QString jsPrefix = global ? QString::null
+  				: QString::fromLatin1("javascript.");
+  QString javaPrefix = global ? QString::null
+  				: QString::fromLatin1("java.");
+  QString pluginsPrefix = global ? QString::null
+  				: QString::fromLatin1("plugins.");
+
+  // The setting for Java
+  QString key = javaPrefix + QString::fromLatin1("EnableJava");
+  if ( (global && reset) || config->hasKey( key ) )
+    pd_settings.m_bEnableJava = config->readBoolEntry( key, false );
+  else if ( !global )
+    pd_settings.m_bEnableJava = d->global.m_bEnableJava;
+
+  // The setting for Plugins
+  key = pluginsPrefix + QString::fromLatin1("EnablePlugins");
+  if ( (global && reset) || config->hasKey( key ) )
+    pd_settings.m_bEnablePlugins = config->readBoolEntry( key, true );
+  else if ( !global )
+    pd_settings.m_bEnablePlugins = d->global.m_bEnablePlugins;
+
+  // The setting for JavaScript
+  key = jsPrefix + QString::fromLatin1("EnableJavaScript");
+  if ( (global && reset) || config->hasKey( key ) )
+    pd_settings.m_bEnableJavaScript = config->readBoolEntry( key, true );
+  else if ( !global )
+    pd_settings.m_bEnableJavaScript = d->global.m_bEnableJavaScript;
+
+  // window property policies
+  key = jsPrefix + QString::fromLatin1("WindowOpenPolicy");
+  if ( (global && reset) || config->hasKey( key ) )
+    pd_settings.m_windowOpenPolicy = (KJSWindowOpenPolicy)
+    		config->readUnsignedNumEntry( key, KJSWindowOpenAllow );
+  else if ( !global )
+    pd_settings.m_windowOpenPolicy = d->global.m_windowOpenPolicy;
+
+  key = jsPrefix + QString::fromLatin1("WindowMovePolicy");
+  if ( (global && reset) || config->hasKey( key ) )
+    pd_settings.m_windowMovePolicy = (KJSWindowMovePolicy)
+    		config->readUnsignedNumEntry( key, KJSWindowMoveAllow );
+  else if ( !global )
+    pd_settings.m_windowMovePolicy = d->global.m_windowMovePolicy;
+
+  key = jsPrefix + QString::fromLatin1("WindowResizePolicy");
+  if ( (global && reset) || config->hasKey( key ) )
+    pd_settings.m_windowResizePolicy = (KJSWindowResizePolicy)
+    		config->readUnsignedNumEntry( key, KJSWindowResizeAllow );
+  else if ( !global )
+    pd_settings.m_windowResizePolicy = d->global.m_windowResizePolicy;
+
+  key = jsPrefix + QString::fromLatin1("WindowStatusPolicy");
+  if ( (global && reset) || config->hasKey( key ) )
+    pd_settings.m_windowStatusPolicy = (KJSWindowStatusPolicy)
+    		config->readUnsignedNumEntry( key, KJSWindowStatusAllow );
+  else if ( !global )
+    pd_settings.m_windowStatusPolicy = d->global.m_windowStatusPolicy;
+
+  key = jsPrefix + QString::fromLatin1("WindowFocusPolicy");
+  if ( (global && reset) || config->hasKey( key ) )
+    pd_settings.m_windowFocusPolicy = (KJSWindowFocusPolicy)
+    		config->readUnsignedNumEntry( key, KJSWindowFocusAllow );
+  else if ( !global )
+    pd_settings.m_windowFocusPolicy = d->global.m_windowFocusPolicy;
+
+}
+
 
 KHTMLSettings::KHTMLSettings()
 {
@@ -176,7 +293,7 @@ void KHTMLSettings::init( KConfig * config, bool reset )
     if ( reset || config->hasKey( "BackRightClick" ) )
         d->m_bBackRightClick = config->readBoolEntry( "BackRightClick", false );
   }
-  
+
   if (reset || config->hasGroup("HTML Settings"))
   {
     config->setGroup( "HTML Settings" );
@@ -209,7 +326,7 @@ void KHTMLSettings::init( KConfig * config, bool reset )
     if ( reset || config->hasKey( "EnforceDefaultCharset" ) )
         d->enforceCharset = config->readBoolEntry( "EnforceDefaultCharset", false );
 
-    // Behaviour
+    // Behavior
     if ( reset || config->hasKey( "ChangeCursor" ) )
         d->m_bChangeCursor = config->readBoolEntry( "ChangeCursor", KDE_DEFAULT_CHANGECURSOR );
 
@@ -245,6 +362,7 @@ void KHTMLSettings::init( KConfig * config, bool reset )
     d->m_formCompletionEnabled = config->readBoolEntry("FormCompletion", true);
     d->m_maxFormCompletionItems = config->readNumEntry("MaxFormCompletionItems", 10);
     d->m_autoDelayedActionsEnabled = config->readBoolEntry ("AutoDelayedActions", true);
+    d->m_jsErrorsEnabled = config->readBoolEntry("ReportJSErrors", true);
   }
 
   // Colors
@@ -266,25 +384,64 @@ void KHTMLSettings::init( KConfig * config, bool reset )
   {
     config->setGroup( "Java/JavaScript Settings" );
 
-    // The global setting for Java
-    if ( reset || config->hasKey( "EnableJava" ) )
-      d->m_bEnableJava = config->readBoolEntry( "EnableJava", false );
-
-    // The global setting for JavaScript
-    if ( reset || config->hasKey( "EnableJavaScript" ) )
-      d->m_bEnableJavaScript = config->readBoolEntry( "EnableJavaScript", true );
-
     // The global setting for JavaScript debugging
+    // This is currently always enabled by default
     if ( reset || config->hasKey( "EnableJavaScriptDebug" ) )
       d->m_bEnableJavaScriptDebug = config->readBoolEntry( "EnableJavaScriptDebug", false );
 
-    // The global setting for Plugins (there's no local setting yet)
-    if ( reset || config->hasKey( "EnablePlugins" ) )
-      d->m_bEnablePlugins = config->readBoolEntry( "EnablePlugins", true );
+    // The global setting for JavaScript error reporting
+    if ( reset || config->hasKey( "ReportJavaScriptErrors" ) )
+      d->m_bEnableJavaScriptErrorReporting = config->readBoolEntry( "ReportJavaScriptErrors", false );
+
+    // Read options from the global "domain"
+    readDomainSettings(config,reset,true,d->global);
+#ifdef DEBUG_SETTINGS
+    d->global.dump("init global");
+#endif
 
     // The domain-specific settings.
+
+    static const char *const domain_keys[] = {	// always keep order of keys
+    	"ECMADomains", "JavaDomains", "PluginDomains"
+    };
+    bool check_old_ecma_settings = true;
+    bool check_old_java_settings = true;
+    // merge all domains into one list
+    QMap<QString,int> domainList;	// why can't Qt have a QSet?
+    for (unsigned i = 0; i < sizeof domain_keys/sizeof domain_keys[0]; i++) {
+      if ( reset || config->hasKey(domain_keys[i]) ) {
+        if (i == 0) check_old_ecma_settings = false;
+	else if (i == 1) check_old_java_settings = false;
+        QStringList dl = config->readListEntry( domain_keys[i] );
+	QMap<QString,int>::Iterator notfound = domainList.end();
+	QStringList::ConstIterator it;
+	for (it = dl.begin(); it != dl.end(); ++it) {
+	  QString domain = (*it).lower();
+	  QMap<QString,int>::Iterator pos = domainList.find(domain);
+	  if (pos == notfound) domainList.insert(domain,0);
+	}/*next it*/
+      }
+    }/*next i*/
+
+    if (reset)
+      d->domainPolicy.clear();
+
+    QString js_group_save = config->group();
+    for ( QMap<QString,int>::ConstIterator it = domainList.begin();
+                it != domainList.end(); ++it)
+    {
+      QString domain = it.key();
+      config->setGroup(domain);
+      readDomainSettings(config,reset,false,d->domainPolicy[domain]);
+#ifdef DEBUG_SETTINGS
+      d->domainPolicy[domain].dump("init "+domain);
+#endif
+    }
+    config->setGroup(js_group_save);
+
     bool check_old_java = true;
-    if( reset || config->hasKey( "JavaDomainSettings" ) )
+    if( ( reset || config->hasKey( "JavaDomainSettings" ) )
+    	&& check_old_java_settings )
     {
       check_old_java = false;
       QStringList domainList = config->readListEntry( "JavaDomainSettings" );
@@ -295,12 +452,17 @@ void KHTMLSettings::init( KConfig * config, bool reset )
         KJavaScriptAdvice javaAdvice;
         KJavaScriptAdvice javaScriptAdvice;
         splitDomainAdvice(*it, domain, javaAdvice, javaScriptAdvice);
-        d->javaDomainPolicy[domain] = javaAdvice;
+        setup_per_domain_policy(d,domain).m_bEnableJava =
+		javaAdvice == KJavaScriptAccept;
+#ifdef DEBUG_SETTINGS
+	setup_per_domain_policy(d,domain).dump("JavaDomainSettings 4 "+domain);
+#endif
       }
     }
 
     bool check_old_ecma = true;
-    if( reset || config->hasKey( "ECMADomainSettings" ) )
+    if( ( reset || config->hasKey( "ECMADomainSettings" ) )
+	&& check_old_ecma_settings )
     {
       check_old_ecma = false;
       QStringList domainList = config->readListEntry( "ECMADomainSettings" );
@@ -311,12 +473,17 @@ void KHTMLSettings::init( KConfig * config, bool reset )
         KJavaScriptAdvice javaAdvice;
         KJavaScriptAdvice javaScriptAdvice;
         splitDomainAdvice(*it, domain, javaAdvice, javaScriptAdvice);
-        d->javaScriptDomainPolicy[domain] = javaScriptAdvice;
+        setup_per_domain_policy(d,domain).m_bEnableJavaScript =
+			javaScriptAdvice == KJavaScriptAccept;
+#ifdef DEBUG_SETTINGS
+	setup_per_domain_policy(d,domain).dump("ECMADomainSettings 4 "+domain);
+#endif
       }
     }
 
-    if( reset || config->hasKey( "JavaScriptDomainAdvice" )
-             && ( check_old_java || check_old_ecma ) )
+    if( ( reset || config->hasKey( "JavaScriptDomainAdvice" ) )
+             && ( check_old_java || check_old_ecma )
+	     && ( check_old_ecma_settings || check_old_java_settings ) )
     {
       QStringList domainList = config->readListEntry( "JavaScriptDomainAdvice" );
       for ( QStringList::ConstIterator it = domainList.begin();
@@ -327,12 +494,18 @@ void KHTMLSettings::init( KConfig * config, bool reset )
         KJavaScriptAdvice javaScriptAdvice;
         splitDomainAdvice(*it, domain, javaAdvice, javaScriptAdvice);
         if( check_old_java )
-          d->javaDomainPolicy[domain] = javaAdvice;
+          setup_per_domain_policy(d,domain).m_bEnableJava =
+	  		javaAdvice == KJavaScriptAccept;
         if( check_old_ecma )
-          d->javaScriptDomainPolicy[domain] = javaScriptAdvice;
+          setup_per_domain_policy(d,domain).m_bEnableJavaScript =
+	  		javaScriptAdvice == KJavaScriptAccept;
+#ifdef DEBUG_SETTINGS
+	setup_per_domain_policy(d,domain).dump("JavaScriptDomainAdvice 4 "+domain);
+#endif
       }
 
       //save all the settings into the new keywords if they don't exist
+#if 0
       if( check_old_java )
       {
         QStringList domainConfig;
@@ -358,30 +531,42 @@ void KHTMLSettings::init( KConfig * config, bool reset )
         }
         config->writeEntry( "ECMADomainSettings", domainConfig );
       }
+#endif
     }
   }
   config->setGroup(group_save);
 }
 
 
-// Local helper for isJavaEnabled & isJavaScriptEnabled.
-static bool lookup_hostname_policy(const QString& hostname,
-                                   const PolicyMap& policy,
-                                   const bool default_retval)
+/** Local helper for retrieving per-domain settings.
+  *
+  * In case of doubt, the global domain is returned.
+  */
+static const KPerDomainSettings &lookup_hostname_policy(
+			const KHTMLSettingsPrivate *d,
+			const QString& hostname)
 {
+#ifdef DEBUG_SETTINGS
+  kdDebug() << "lookup_hostname_policy(" << hostname << ")" << endl;
+#endif
   if (hostname.isEmpty()) {
-    return default_retval;
+#ifdef DEBUG_SETTINGS
+    d->global.dump("global");
+#endif
+    return d->global;
   }
 
+  PolicyMap::const_iterator notfound = d->domainPolicy.end();
+
   // First check whether there is a perfect match.
-  if( policy.contains( hostname ) ) {
+  PolicyMap::const_iterator it = d->domainPolicy.find(hostname);
+  if( it != notfound ) {
+#ifdef DEBUG_SETTINGS
+    kdDebug() << "perfect match" << endl;
+    (*it).dump(hostname);
+#endif
     // yes, use it (unless dunno)
-    KHTMLSettings::KJavaScriptAdvice adv = policy[ hostname ];
-    if( adv == KHTMLSettings::KJavaScriptReject ) {
-      return false;
-    } else if( adv == KHTMLSettings::KJavaScriptAccept ) {
-      return true;
-    }
+    return *it;
   }
 
   // Now, check for partial match.  Chop host from the left until
@@ -390,20 +575,25 @@ static bool lookup_hostname_policy(const QString& hostname,
   int dot_idx = -1;
   while( (dot_idx = host_part.find(QChar('.'))) >= 0 ) {
     host_part.remove(0,dot_idx);
-    if( policy.contains( host_part ) ) {
-      KHTMLSettings::KJavaScriptAdvice adv = policy[ host_part ];
-      if( adv == KHTMLSettings::KJavaScriptReject ) {
-        return false;
-      } else if( adv == KHTMLSettings::KJavaScriptAccept ) {
-        return true;
-      }
+    it = d->domainPolicy.find(host_part);
+    Q_ASSERT(notfound == d->domainPolicy.end());
+    if( it != notfound ) {
+#ifdef DEBUG_SETTINGS
+      kdDebug() << "partial match" << endl;
+      (*it).dump(host_part);
+#endif
+      return *it;
     }
     // assert(host_part[0] == QChar('.'));
     host_part.remove(0,1); // Chop off the dot.
   }
 
-  // No domain-specific entry, or was dunno: use global setting
-  return default_retval;
+  // No domain-specific entry: use global domain
+#ifdef DEBUG_SETTINGS
+  kdDebug() << "no match" << endl;
+  d->global.dump("global");
+#endif
+  return d->global;
 }
 
 bool KHTMLSettings::isBackRightClickEnabled()
@@ -413,12 +603,12 @@ bool KHTMLSettings::isBackRightClickEnabled()
 
 bool KHTMLSettings::isJavaEnabled( const QString& hostname )
 {
-  return lookup_hostname_policy(hostname.lower(), d->javaDomainPolicy, d->m_bEnableJava);
+  return lookup_hostname_policy(d,hostname.lower()).m_bEnableJava;
 }
 
 bool KHTMLSettings::isJavaScriptEnabled( const QString& hostname )
 {
-  return lookup_hostname_policy(hostname.lower(), d->javaScriptDomainPolicy, d->m_bEnableJavaScript);
+  return lookup_hostname_policy(d,hostname.lower()).m_bEnableJavaScript;
 }
 
 bool KHTMLSettings::isJavaScriptDebugEnabled( const QString& /*hostname*/ )
@@ -427,10 +617,40 @@ bool KHTMLSettings::isJavaScriptDebugEnabled( const QString& /*hostname*/ )
   return d->m_bEnableJavaScriptDebug;
 }
 
-bool KHTMLSettings::isPluginsEnabled( const QString& /* hostname */ )
+bool KHTMLSettings::isJavaScriptErrorReportingEnabled( const QString& /*hostname*/ ) const
 {
-  // FIXME: hostname is ignored (dnaber, 2001-01-03)
-  return d->m_bEnablePlugins;
+  // error reporting setting is global for now, but could change in the future
+  return d->m_bEnableJavaScriptErrorReporting;
+}
+
+bool KHTMLSettings::isPluginsEnabled( const QString& hostname )
+{
+  return lookup_hostname_policy(d,hostname.lower()).m_bEnablePlugins;
+}
+
+KHTMLSettings::KJSWindowOpenPolicy KHTMLSettings::windowOpenPolicy(
+				const QString& hostname) const {
+  return lookup_hostname_policy(d,hostname.lower()).m_windowOpenPolicy;
+}
+
+KHTMLSettings::KJSWindowMovePolicy KHTMLSettings::windowMovePolicy(
+				const QString& hostname) const {
+  return lookup_hostname_policy(d,hostname.lower()).m_windowMovePolicy;
+}
+
+KHTMLSettings::KJSWindowResizePolicy KHTMLSettings::windowResizePolicy(
+				const QString& hostname) const {
+  return lookup_hostname_policy(d,hostname.lower()).m_windowResizePolicy;
+}
+
+KHTMLSettings::KJSWindowStatusPolicy KHTMLSettings::windowStatusPolicy(
+				const QString& hostname) const {
+  return lookup_hostname_policy(d,hostname.lower()).m_windowStatusPolicy;
+}
+
+KHTMLSettings::KJSWindowFocusPolicy KHTMLSettings::windowFocusPolicy(
+				const QString& hostname) const {
+  return lookup_hostname_policy(d,hostname.lower()).m_windowFocusPolicy;
 }
 
 int KHTMLSettings::mediumFontSize() const
@@ -491,7 +711,7 @@ const QString &KHTMLSettings::availableFamilies()
         }
         s.sort();
 
-        *avFamilies = s.join(",");
+        *avFamilies = ',' + s.join(",") + ',';
     }
 
   return *avFamilies;
@@ -600,3 +820,19 @@ bool KHTMLSettings::isAutoDelayedActionsEnabled() const
 {
   return d->m_autoDelayedActionsEnabled;
 }
+
+bool KHTMLSettings::jsErrorsEnabled() const
+{
+  return d->m_jsErrorsEnabled;
+}
+
+void KHTMLSettings::setJSErrorsEnabled(bool enabled)
+{
+  d->m_jsErrorsEnabled = enabled;
+  // save it
+  KConfig *config = KGlobal::config();
+  config->setGroup("HTML Settings");
+  config->writeEntry("ReportJSErrors", enabled);
+  config->sync();
+}
+
