@@ -24,6 +24,7 @@
 #include "html/html_documentimpl.h"
 #include "html/html_elementimpl.h"
 #include "rendering/render_image.h"
+#include "misc/loader.h"
 
 #include <qvbox.h>
 #include <qtimer.h>
@@ -33,7 +34,7 @@
 #include <kmimetype.h>
 #include <klocale.h>
 
-K_EXPORT_COMPONENT_FACTORY( khtmlimagepart, KHTMLImageFactory )
+K_EXPORT_COMPONENT_FACTORY( khtmlimagefactory /*NOT the part name, see Makefile.am*/, KHTMLImageFactory )
 
 KInstance *KHTMLImageFactory::s_instance = 0;
 
@@ -56,14 +57,15 @@ KParts::Part *KHTMLImageFactory::createPartObject( QWidget *parentWidget, const
 
 KHTMLImage::KHTMLImage( QWidget *parentWidget, const char *widgetName,
                         QObject *parent, const char *name )
-    : KParts::ReadOnlyPart( parent, name )
+    : KParts::ReadOnlyPart( parent, name ), m_image( 0 )
 {
     setInstance( KHTMLImageFactory::instance() );
 
     QVBox *box = new QVBox( parentWidget, widgetName );
 
-    m_khtml = new KHTMLPart( box, widgetName, this, "htmlimagepart" );
+    m_khtml = new KHTMLPart( box, widgetName, this, "htmlimagepart", KHTMLPart::BrowserViewGUI );
     m_khtml->setAutoloadImages( true );
+    m_khtml->widget()->installEventFilter(this);
 
     setWidget( box );
 
@@ -77,7 +79,7 @@ KHTMLImage::KHTMLImage( QWidget *parentWidget, const char *widgetName,
     if ( encodingAction )
     {
         encodingAction->unplugAll();
-        delete encodingAction;
+        delete encodingAction;      
     }
     KAction *viewSourceAction= actionCollection()->action( "viewDocumentSource" );
     if ( viewSourceAction )
@@ -93,8 +95,17 @@ KHTMLImage::KHTMLImage( QWidget *parentWidget, const char *widgetName,
         delete selectAllAction;
     }
 
-   connect( m_khtml->browserExtension(), SIGNAL( popupMenu( KXMLGUIClient *, const QPoint &, const KURL &, const QString &, mode_t ) ),
-             m_ext, SIGNAL( popupMenu( KXMLGUIClient *, const QPoint &, const KURL &, const QString &, mode_t ) ) );
+    // forward important signals from the khtml part
+    
+    // forward opening requests to parent frame (if existing)
+    KHTMLPart *p = ::qt_cast<KHTMLPart *>(parent);
+    KParts::BrowserExtension *be = p ? p->browserExtension() : m_ext;
+    connect(m_khtml->browserExtension(), SIGNAL(openURLRequestDelayed(const KURL &, const KParts::URLArgs &)),
+    		be, SIGNAL(openURLRequestDelayed(const KURL &, const KParts::URLArgs &)));
+
+    connect( m_khtml->browserExtension(), SIGNAL( popupMenu( KXMLGUIClient *, const QPoint &, const KURL &,
+             const KParts::URLArgs &, KParts::BrowserExtension::PopupFlags, mode_t) ), m_ext, SIGNAL( popupMenu( KXMLGUIClient *, const QPoint &, const KURL &,
+             const KParts::URLArgs &, KParts::BrowserExtension::PopupFlags, mode_t) ) );
 
     connect( m_khtml->browserExtension(), SIGNAL( enableAction( const char *, bool ) ),
              m_ext, SIGNAL( enableAction( const char *, bool ) ) );
@@ -104,6 +115,8 @@ KHTMLImage::KHTMLImage( QWidget *parentWidget, const char *widgetName,
 
 KHTMLImage::~KHTMLImage()
 {
+    disposeImage();
+
     // important: delete the html part before the part or qobject destructor runs.
     // we now delete the htmlpart which deletes the part's widget which makes
     // _OUR_ m_widget 0 which in turn avoids our part destructor to delete the
@@ -119,6 +132,8 @@ bool KHTMLImage::openURL( const KURL &url )
 {
     static const QString &html = KGlobal::staticQString( "<html><body><img src=\"%1\"></body></html>" );
 
+    disposeImage();
+
     m_url = url;
 
     emit started( 0 );
@@ -126,6 +141,8 @@ bool KHTMLImage::openURL( const KURL &url )
     KParts::URLArgs args = m_ext->urlArgs();
     m_mimeType = args.serviceType;
 
+    emit setWindowCaption( url.prettyURL() );
+
     m_khtml->begin( m_url, args.xOffset, args.yOffset );
     m_khtml->setAutoloadImages( true );
 
@@ -133,33 +150,61 @@ bool KHTMLImage::openURL( const KURL &url )
     if ( impl && m_ext->urlArgs().reload )
         impl->docLoader()->setCachePolicy( KIO::CC_Refresh );
 
+    khtml::DocLoader *dl = impl ? impl->docLoader() : 0;
+    m_image = dl->requestImage( m_url.url() );
+    if ( m_image )
+        m_image->ref( this );
+
     m_khtml->write( html.arg( m_url.url() ) );
     m_khtml->end();
 
-    emit setWindowCaption( url.prettyURL() );
-
+    /*
     connect( khtml::Cache::loader(), SIGNAL( requestDone( khtml::DocLoader*, khtml::CachedObject *) ),
             this, SLOT( updateWindowCaption() ) );
+            */
     return true;
 }
 
 bool KHTMLImage::closeURL()
 {
+    disposeImage();
     return m_khtml->closeURL();
 }
 
+void KHTMLImage::notifyFinished( khtml::CachedObject *o )
+{
+    if ( !m_image || o != m_image )
+        return;
+
+    const QPixmap &pix = m_image->pixmap();
+    QString caption;
+
+    KMimeType::Ptr mimeType;
+    if ( !m_mimeType.isEmpty() )
+        mimeType = KMimeType::mimeType( m_mimeType );
+
+    if ( mimeType )
+        caption = i18n( "%1 - %2x%3 Pixels" ).arg( mimeType->comment() )
+                  .arg( pix.width() ).arg( pix.height() );
+    else
+        caption = i18n( "Image - %1x%2 Pixels" ).arg( pix.width() ).arg( pix.height() );
+
+    emit setWindowCaption( caption );
+    emit completed();
+    emit setStatusBarText(i18n("Done."));
+}
+
 void KHTMLImage::guiActivateEvent( KParts::GUIActivateEvent *e )
 {
+    // prevent the base implementation from emitting setWindowCaption with
+    // our url. It destroys our pretty, previously caption. Konq saves/restores
+    // the caption for us anyway.
     if ( e->activated() )
-        emit setWindowCaption( m_url.prettyURL() );
-}
-
-void KHTMLImage::slotPopupMenu( KXMLGUIClient *cl, const QPoint &pos, const KURL &u,
-                                const QString &, mode_t mode )
-{
-    emit m_ext->popupMenu( cl, pos, u, m_mimeType, mode );
+        return;
+    KParts::ReadOnlyPart::guiActivateEvent(e);
 }
 
+/*
 void KHTMLImage::slotImageJobFinished( KIO::Job *job )
 {
     if ( job->error() )
@@ -214,12 +259,42 @@ void KHTMLImage::updateWindowCaption()
         caption = i18n( "%1 - %2x%3 Pixels" ).arg( mimeType->comment() )
                   .arg( pix.width() ).arg( pix.height() );
     else
-        caption = i18n( "Image - %2x%3 Pixels" ).arg( pix.width() ).arg( pix.height() );
+        caption = i18n( "Image - %1x%2 Pixels" ).arg( pix.width() ).arg( pix.height() );
 
     emit setWindowCaption( caption );
     emit completed();
     emit setStatusBarText(i18n("Done."));
 }
+*/
+
+void KHTMLImage::disposeImage()
+{
+    if ( !m_image )
+        return;
+
+    m_image->deref( this );
+    m_image = 0;
+}
+
+bool KHTMLImage::eventFilter(QObject *, QEvent *e) {
+    switch (e->type()) {
+      case QEvent::DragEnter:
+      case QEvent::DragMove:
+      case QEvent::DragLeave:
+      case QEvent::Drop: {
+        // find out if this part is embedded in a frame, and send the
+	// event to its outside widget
+	KHTMLPart *p = ::qt_cast<KHTMLPart *>(parent());
+	if (p)
+	    return QApplication::sendEvent(p->widget(), e);
+        // otherwise simply forward all dnd events to the part widget,
+	// konqueror will handle them properly there
+        return QApplication::sendEvent(widget(), e);
+      }
+      default: ;
+    }
+    return false;
+}
 
 KHTMLImageBrowserExtension::KHTMLImageBrowserExtension( KHTMLImage *parent, const char *name )
     : KParts::BrowserExtension( parent, name )
