@@ -9,6 +9,7 @@
  *                     2000-2001 Simon Hausmann <hausmann@kde.org>
  *                     2000-2001 Dirk Mueller <mueller@kde.org>
  *                     2000 Stefan Schimanski <1Stein@gmx.de>
+ *                     2001-2003 George Stiakos <staikos@kde.org>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -26,24 +27,40 @@
  * Boston, MA 02111-1307, USA.
  */
 #include <kcursor.h>
+#include <klibloader.h>
 #include <kxmlguifactory.h>
+#include <kaction.h>
+#include <kparts/partmanager.h>
+#include <kparts/statusbarextension.h>
+#include <kwallet.h>
+#include <qtimer.h>
 
 #include "khtml_run.h"
-#include "../kutils/kfind.h"
 #include "khtml_factory.h"
 #include "khtml_events.h"
 #include "khtml_ext.h"
 #include "khtml_iface.h"
 #include "khtml_settings.h"
 #include "misc/decoder.h"
-#include "java/kjavaappletcontext.h"
 #include "ecma/kjs_proxy.h"
 
+class KFind;
+class KFindDialog;
+class KPopupMenu;
+class KSelectAction;
+class KURLLabel;
+class KJavaAppletContext;
+class KJSErrorDlg;
+
 namespace KIO
 {
   class Job;
   class TransferJob;
 }
+namespace KParts
+{
+  class StatusBarExtension;
+}
 
 namespace khtml
 {
@@ -78,14 +95,13 @@ namespace khtml
 
 }
 
-class FrameList : public QValueList<khtml::ChildFrame>
+struct KHTMLFrameList : public QValueList<khtml::ChildFrame>
 {
-public:
-    Iterator find( const QString &name );
+    Iterator find( const QString &name ) KDE_NO_EXPORT;
 };
 
-typedef FrameList::ConstIterator ConstFrameIt;
-typedef FrameList::Iterator FrameIt;
+typedef KHTMLFrameList::ConstIterator ConstFrameIt;
+typedef KHTMLFrameList::Iterator FrameIt;
 
 static int khtml_part_dcop_counter = 0;
 
@@ -97,6 +113,8 @@ public:
     m_doc = 0L;
     m_decoder = 0L;
     m_jscript = 0L;
+    m_wallet = 0L;
+    m_bWalletOpened = false;
     m_runningScripts = 0;
     m_kjs_lib = 0;
     m_job = 0L;
@@ -111,6 +129,7 @@ public:
     m_bDnd = true;
     m_startOffset = m_endOffset = 0;
     m_startBeforeEnd = true;
+    m_extendAtEnd = true;
     m_linkCursor = KCursor::handCursor();
     m_loadedObjects = 0;
     m_totalObjectCount = 0;
@@ -118,7 +137,11 @@ public:
     m_haveEncoding = false;
     m_activeFrame = 0L;
     m_find = 0;
+    m_findDialog = 0;
     m_ssl_in_use = false;
+    m_jsedlg = 0;
+    m_formNotification = KHTMLPart::NoNotification;
+
 #ifndef Q_WS_QWS
     m_javaContext = 0;
 #endif
@@ -138,12 +161,16 @@ public:
     m_bPluginsOverride = false;
     m_onlyLocalReferences = false;
 
+    m_caretMode = false;
+    m_designMode = false;
+
     m_metaRefreshEnabled = true;
-    m_bHTTPRefresh = false;
+    m_statusMessagesEnabled = true;
 
     m_bFirstData = true;
     m_submitForm = 0;
     m_delayRedirect = 0;
+    m_autoDetectLanguage = khtml::Decoder::SemiautomaticDetection;
 
     // inherit settings from parent
     if(parent && parent->inherits("KHTMLPart"))
@@ -160,7 +187,12 @@ public:
             // Same for SSL settings
             m_ssl_in_use = part->d->m_ssl_in_use;
             m_onlyLocalReferences = part->d->m_onlyLocalReferences;
+            m_caretMode = part->d->m_caretMode;
+            m_designMode = part->d->m_designMode;
             m_zoomFactor = part->d->m_zoomFactor;
+            m_autoDetectLanguage = part->d->m_autoDetectLanguage;
+            m_encoding = part->d->m_encoding;
+            m_haveEncoding = part->d->m_haveEncoding;
         }
     }
 
@@ -172,26 +204,35 @@ public:
     m_dcopobject = 0;
     m_jobspeed = 0;
     m_dcop_counter = ++khtml_part_dcop_counter;
+    m_statusBarWalletLabel = 0L;
+    m_statusBarJSErrorLabel = 0L;
+    m_userStyleSheetLastModified = 0;
   }
   ~KHTMLPartPrivate()
   {
     delete m_dcopobject;
+    delete m_statusBarExtension;
     delete m_extension;
     delete m_settings;
     delete m_jscript;
+    delete m_wallet;
     if ( m_kjs_lib)
        m_kjs_lib->unload();
 #ifndef Q_WS_QWS
-    delete m_javaContext;
+    //delete m_javaContext;
 #endif
   }
 
-  FrameList m_frames;
+  KHTMLFrameList m_frames;
   QValueList<khtml::ChildFrame> m_objects;
 
   QGuardedPtr<KHTMLView> m_view;
   KHTMLPartBrowserExtension *m_extension;
+  KParts::StatusBarExtension *m_statusBarExtension;
   KHTMLPartBrowserHostExtension *m_hostExtension;
+  KURLLabel* m_statusBarIconLabel;
+  KURLLabel* m_statusBarWalletLabel;
+  KURLLabel* m_statusBarJSErrorLabel;
   DOM::DocumentImpl *m_doc;
   khtml::Decoder *m_decoder;
   QString m_encoding;
@@ -201,6 +242,7 @@ public:
   DOM::Node scheduledScriptNode;
 
   KJSProxy *m_jscript;
+  KWallet::Wallet* m_wallet;
   KLibrary *m_kjs_lib;
   int m_runningScripts;
   bool m_bBackRightClick :1;
@@ -216,6 +258,8 @@ public:
   bool m_metaRefreshEnabled :1;
   bool m_bPluginsOverride :1;
   bool m_restored :1;
+  bool m_statusMessagesEnabled :1;
+  bool m_bWalletOpened :1;
   int m_frameNameId;
   int m_dcop_counter;
   DCOPObject *m_dcopobject;
@@ -252,7 +296,6 @@ public:
   bool m_bComplete:1;
   bool m_bLoadEventEmitted:1;
   bool m_haveEncoding:1;
-  bool m_bHTTPRefresh:1;
   bool m_onlyLocalReferences :1;
   bool m_redirectLockHistory:1;
 
@@ -271,7 +314,7 @@ public:
   KAction *m_paSaveDocument;
   KAction *m_paSaveFrame;
   KAction *m_paSecurity;
-  KSelectAction *m_paSetEncoding;
+  KActionMenu *m_paSetEncoding;
   KSelectAction *m_paUseStylesheet;
   KHTMLZoomFactorAction *m_paIncZoomFactor;
   KHTMLZoomFactorAction *m_paDecZoomFactor;
@@ -280,8 +323,11 @@ public:
   KAction *m_paFindNext;
   KAction *m_paPrintFrame;
   KAction *m_paSelectAll;
+  KAction *m_paDebugScript;
   KAction *m_paDebugDOMTree;
   KAction *m_paDebugRenderTree;
+  KAction *m_paStopAnimations;
+  KToggleAction *m_paToggleCaretMode;
 
   KParts::PartManager *m_manager;
 
@@ -311,6 +357,14 @@ public:
   bool m_bRightMousePressed;
   DOM::Node m_mousePressNode; //node under the mouse when the mouse was pressed (set in the mouse handler)
 
+  // simply using the selection limits for the caret position does not suffice
+  // as we need to know on which side to extend the selection
+//  DOM::Node m_caretNode;	// node containing the caret
+//  long m_caretOffset;		// offset within this node (0-based)
+
+  // the caret uses the selection variables for its position. If m_extendAtEnd
+  // is true, m_selectionEnd and m_endOffset contain the mandatory caret
+  // position, otherwise it's m_selectionStart and m_startOffset.
   DOM::Node m_selectionStart;
   long m_startOffset;
   DOM::Node m_selectionEnd;
@@ -319,6 +373,7 @@ public:
   QString m_overURLTarget;
 
   bool m_startBeforeEnd:1;
+  bool m_extendAtEnd:1;		// true if selection is to be extended at its end
   bool m_bDnd:1;
   bool m_bFirstData:1;
   bool m_bClearing:1;
@@ -333,6 +388,9 @@ public:
   QPoint m_dragLastPos;
 #endif
 
+  bool m_designMode;
+  bool m_caretMode;
+
   QCursor m_linkCursor;
   QTimer m_scrollTimer;
 
@@ -340,6 +398,7 @@ public:
   unsigned long m_totalObjectCount;
   unsigned int m_jobPercent;
 
+  KHTMLPart::FormNotification m_formNotification;
   QTimer m_progressUpdateTimer;
 
   QStringList m_pluginPageQuestionAsked;
@@ -366,6 +425,7 @@ public:
   QValueList<StringPortion> m_stringPortions;
 
   KFind *m_find;
+  KFindDialog *m_findDialog;
 
   struct findState
   {
@@ -378,6 +438,8 @@ public:
 
   findState m_lastFindState;
 
+  KJSErrorDlg *m_jsedlg;
+
   DOM::NodeImpl *m_findNode; // current node
   DOM::NodeImpl *m_findNodeEnd; // end node
   int m_findPos; // current pos in current node
@@ -389,6 +451,22 @@ public:
   QGuardedPtr<KHTMLPart> m_opener;
   bool m_openedByJS;
   bool m_newJSInterpreterExists; // set to 1 by setOpenedByJS, for window.open
+
+  khtml::Decoder::AutoDetectLanguage m_autoDetectLanguage;
+  KPopupMenu *m_automaticDetection;
+  KSelectAction *m_manualDetection;
+
+  void setFlagRecursively(bool KHTMLPartPrivate::*flag, bool value);
+  /** returns the caret node */
+  DOM::Node &caretNode() {
+    return m_extendAtEnd ? m_selectionEnd : m_selectionStart;
+  }
+  /** returns the caret offset */
+  long &caretOffset() {
+    return m_extendAtEnd ? m_endOffset : m_startOffset;
+  }
+
+  time_t m_userStyleSheetLastModified;
 };
 
 #endif
