@@ -96,6 +96,8 @@ KMultiPart::KMultiPart( QWidget *parentWidget, const char *widgetName,
                         QObject *parent, const char *name, const QStringList& )
     : KParts::ReadOnlyPart( parent, name )
 {
+    m_filter = 0L;
+
     setInstance( KMultiPartFactory::instance() );
 
     QVBox *box = new QVBox( parentWidget, widgetName );
@@ -125,14 +127,27 @@ KMultiPart::~KMultiPart()
     delete m_job;
     delete m_lineParser;
     delete m_tempFile;
+    delete m_filter;
+    m_filter = 0L;
 }
 
+
+void KMultiPart::startHeader()
+{
+    m_bParsingHeader = true; // we expect a header to come first
+    m_bGotAnyHeader = false;
+    m_gzip = false;
+    // just to be sure for now
+    delete m_filter;
+    m_filter = 0L;
+}
+
+
 bool KMultiPart::openURL( const KURL &url )
 {
     m_url = url;
     m_lineParser->reset();
-    m_bParsingHeader = true; // we expect a header to come first
-    m_bGotAnyHeader = false;
+    startHeader();
 
     KParts::URLArgs args = m_extension->urlArgs();
     //m_mimeType = args.serviceType;
@@ -200,6 +215,15 @@ void KMultiPart::slotData( KIO::Job *job, const QByteArray &data )
                         m_boundaryLength = m_boundary.length();
                     }
                 }
+                else if ( !qstrnicmp( line.data(), "Content-Encoding:", 17 ) )
+                {
+                    QString encoding = QString::fromLatin1(line.data()+17).stripWhiteSpace().lower();
+                    if (encoding == "gzip" || encoding == "x-gzip") {
+                        m_gzip = true;
+                    } else {
+                        kdDebug() << "FIXME: unhandled encoding type in KMultiPart: " << encoding << endl;
+                    }
+                }
                 // parse Content-Type
                 else if ( !qstrnicmp( line.data(), "Content-Type:", 13 ) )
                 {
@@ -244,8 +268,7 @@ void KMultiPart::slotData( KIO::Job *job, const QByteArray &data )
 #endif
                         if ( nextChar == '\n' || nextChar == '\r' ) {
                             endOfData();
-                            m_bParsingHeader = true;
-                            m_bGotAnyHeader = false;
+                            startHeader();
                         }
                         else {
                             // otherwise, false hit, it has trailing stuff
@@ -303,14 +326,20 @@ void KMultiPart::setPart( const QString& mimeType )
         connect( childExtension, SIGNAL( createNewWindow( const KURL &, const KParts::URLArgs &, const KParts::WindowArgs &, KParts::ReadOnlyPart *& ) ),
                  m_extension, SIGNAL( createNewWindow( const KURL &, const KParts::URLArgs & , const KParts::WindowArgs &, KParts::ReadOnlyPart *&) ) );
 
+        // Keep in sync with khtml_part.cpp
         connect( childExtension, SIGNAL( popupMenu( const QPoint &, const KFileItemList & ) ),
                  m_extension, SIGNAL( popupMenu( const QPoint &, const KFileItemList & ) ) );
         connect( childExtension, SIGNAL( popupMenu( KXMLGUIClient *, const QPoint &, const KFileItemList & ) ),
                  m_extension, SIGNAL( popupMenu( KXMLGUIClient *, const QPoint &, const KFileItemList & ) ) );
+        connect( childExtension, SIGNAL( popupMenu( KXMLGUIClient *, const QPoint &, const KFileItemList &, const KParts::URLArgs &, KParts::BrowserExtension::PopupFlags ) ),
+                 m_extension, SIGNAL( popupMenu( KXMLGUIClient *, const QPoint &, const KFileItemList &, const KParts::URLArgs &, KParts::BrowserExtension::PopupFlags ) ) );
         connect( childExtension, SIGNAL( popupMenu( const QPoint &, const KURL &, const QString &, mode_t ) ),
                  m_extension, SIGNAL( popupMenu( const QPoint &, const KURL &, const QString &, mode_t ) ) );
         connect( childExtension, SIGNAL( popupMenu( KXMLGUIClient *, const QPoint &, const KURL &, const QString &, mode_t ) ),
                  m_extension, SIGNAL( popupMenu( KXMLGUIClient *, const QPoint &, const KURL &, const QString &, mode_t ) ) );
+        connect( childExtension, SIGNAL( popupMenu( KXMLGUIClient *, const QPoint &, const KURL &, const KParts::URLArgs &, KParts::BrowserExtension::PopupFlags, mode_t ) ),
+                 m_extension, SIGNAL( popupMenu( KXMLGUIClient *, const QPoint &, const KURL &, const KParts::URLArgs &, KParts::BrowserExtension::PopupFlags, mode_t ) ) );
+
 
         connect( childExtension, SIGNAL( infoMessage( const QString & ) ),
                  m_extension, SIGNAL( infoMessage( const QString & ) ) );
@@ -335,9 +364,14 @@ void KMultiPart::setPart( const QString& mimeType )
                  m_extension, SIGNAL( selectionInfo( const KURL::List& ) ) );
         connect( childExtension, SIGNAL( mouseOverInfo( const KFileItem* ) ),
                  m_extension, SIGNAL( mouseOverInfo( const KFileItem* ) ) );
+        connect( childExtension, SIGNAL( moveTopLevelWidget( int, int ) ),
+                 m_extension, SIGNAL( moveTopLevelWidget( int, int ) ) );
+        connect( childExtension, SIGNAL( resizeTopLevelWidget( int, int ) ),
+                 m_extension, SIGNAL( resizeTopLevelWidget( int, int ) ) );
     }
 
     m_isHTMLPart = ( mimeType == "text/html" );
+    m_partIsLoading = false;
     // Load the part's plugins too.
     // ###### This is a hack. The bug is that KHTMLPart doesn't load its plugins
     // if className != "Browser/View".
@@ -353,6 +387,13 @@ void KMultiPart::startOfData()
     Q_ASSERT( !m_nextMimeType.isNull() );
     if( m_nextMimeType.isNull() )
         return;
+
+    if ( m_gzip )
+    {
+        m_filter = new HTTPFilterGZip;
+        connect( m_filter, SIGNAL( output( const QByteArray& ) ), this, SLOT( reallySendData( const QByteArray& ) ) );
+    }
+
     if ( m_mimeType != m_nextMimeType )
     {
         // Need to switch parts (or create the initial one)
@@ -374,10 +415,25 @@ void KMultiPart::startOfData()
         htmlPart->begin( url() );
     }
     else
+    {
+        // ###### TODO use a QByteArray and a data: URL instead
         m_tempFile = new KTempFile;
+    }
 }
 
 void KMultiPart::sendData( const QByteArray& line )
+{
+    if ( m_filter )
+    {
+        m_filter->slotInput( line );
+    }
+    else
+    {
+        reallySendData( line );
+    }
+}
+
+void KMultiPart::reallySendData( const QByteArray& line )
 {
     if ( m_isHTMLPart )
     {
@@ -400,10 +456,21 @@ void KMultiPart::endOfData()
     } else if ( m_tempFile )
     {
         m_tempFile->close();
-        kdDebug() << "KMultiPart::endOfData opening " << m_tempFile->name() << endl;
-        KURL url;
-        url.setPath( m_tempFile->name() );
-        (void) m_part->openURL( url );
+        if ( m_partIsLoading )
+        {
+            // The part is still loading the last data! Let it proceed then
+            // Otherwise we'd keep cancelling it, and nothing would ever show up...
+            kdDebug() << "KMultiPart::endOfData part isn't ready, skipping frame" << endl;
+            m_tempFile->setAutoDelete( true );
+        }
+        else
+        {
+            kdDebug() << "KMultiPart::endOfData opening " << m_tempFile->name() << endl;
+            KURL url;
+            url.setPath( m_tempFile->name() );
+            m_partIsLoading = true;
+            (void) m_part->openURL( url );
+        }
         delete m_tempFile;
         m_tempFile = 0L;
     }
@@ -418,6 +485,7 @@ void KMultiPart::slotPartCompleted()
         Q_ASSERT( m_part->url().isLocalFile() );
 	kdDebug() << "slotPartCompleted deleting " << m_part->url().path() << endl;
         (void) unlink( QFile::encodeName( m_part->url().path() ) );
+        m_partIsLoading = false;
         // Do not emit completed from here.
     }
 }
