@@ -2,7 +2,8 @@
     This file is part of the KDE libraries
 
     Copyright (C) 1998 Lars Knoll (knoll@mpi-hd.mpg.de)
-    Copyright (C) 2001 Dirk Mueller <mueller@kde.org>
+    Copyright (C) 2001-2003 Dirk Mueller <mueller@kde.org>
+    Copyright (C) 2003 Apple Computer, Inc
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Library General Public
@@ -45,6 +46,7 @@
 #include <qbuffer.h>
 #include <qstringlist.h>
 #include <qtextcodec.h>
+#include <qtimer.h>
 
 #include <kurl.h>
 #include <kio/global.h>
@@ -90,32 +92,27 @@ namespace khtml
 	};
 
 	enum Status {
-	    NotCached,    // this URL is not cached
 	    Unknown,      // let imagecache decide what to do with it
 	    New,          // inserting new image
             Pending,      // only partially loaded
 	    Persistent,   // never delete this pixmap
-	    Cached,       // regular case
-	    Uncacheable   // to big to be cached,
-	};  	          // will be destroyed as soon as possible
+	    Cached        // regular case
+	};
 
-	CachedObject(const DOM::DOMString &url, Type type, KIO::CacheControl _cachePolicy, time_t _expireDate)
+	CachedObject(const DOM::DOMString &url, Type type, KIO::CacheControl _cachePolicy, int size)
+            : m_url(url), m_type(type), m_cachePolicy(_cachePolicy),
+              m_expireDate(0), m_size(size)
 	{
-	    m_url = url;
-	    m_type = type;
 	    m_status = Pending;
-	    m_size = 0;
+            m_accessCount = 0;
 	    m_cachePolicy = _cachePolicy;
 	    m_request = 0;
-	    m_expireDate = _expireDate;
             m_deleted = false;
-            m_expireDateChanged = false;
             m_free = false;
+            m_hadError = false;
+            m_prev = m_next = 0;
 	}
-	virtual ~CachedObject() {
-            if(m_deleted) abort();
-            m_deleted = true;
-        }
+        virtual ~CachedObject();
 
 	virtual void data( QBuffer &buffer, bool eof) = 0;
 	virtual void error( int err, const char *text ) = 0;
@@ -123,24 +120,20 @@ namespace khtml
 	const DOM::DOMString &url() const { return m_url; }
 	Type type() const { return m_type; }
 
-	virtual void ref(CachedObjectClient *consumer) = 0;
-	virtual void deref(CachedObjectClient *consumer) = 0;
+	virtual void ref(CachedObjectClient *consumer);
+	virtual void deref(CachedObjectClient *consumer);
 
 	int count() const { return m_clients.count(); }
+        int accessCount() const { return m_accessCount; }
 
 	void setStatus(Status s) { m_status = s; }
 	Status status() const { return m_status; }
 
+        QTextCodec* codecForBuffer( const QString& charset, const QByteArray& buffer ) const;
+
 	int size() const { return m_size; }
 
-	/**
-	 * computes the status of an object after loading.
-	 * the result depends on the objects size and the size of the cache
-	 * also updates the expire date on the cache entry file
-	 */
-	virtual void finish();
-
-        void setFree();
+        bool free() const { return m_free; }
 
         KIO::CacheControl cachePolicy() const { return m_cachePolicy; }
 
@@ -148,62 +141,74 @@ namespace khtml
 
         bool canDelete() const { return (m_clients.count() == 0 && !m_request); }
 
-	void setExpireDate(time_t _expireDate, bool changeHttpCache);
+	void setExpireDate(time_t _expireDate) {  m_expireDate = _expireDate; }
 
 	bool isExpired() const;
 
         virtual bool schedule() const { return false; }
+	virtual void finish();
 
         /**
-         * List of acceptable mimetypes seperated by ",". A mimetype may contain a wildcard.
+         * List of acceptable mimetypes separated by ",". A mimetype may contain a wildcard.
          */
         // e.g. "text/*"
         QString accept() const { return m_accept; }
         void setAccept(const QString &_accept) { m_accept = _accept; }
 
     protected:
-        QPtrList<CachedObjectClient> m_clients;
-
+        void setSize(int size);
+        QPtrDict<CachedObjectClient> m_clients;
 	DOM::DOMString m_url;
         QString m_accept;
         Request *m_request;
 	Type m_type;
 	Status m_status;
-	int m_size;
-	time_t m_expireDate;
+        int m_accessCount;
 	KIO::CacheControl m_cachePolicy;
+	time_t m_expireDate;
+	int m_size;
         bool m_deleted : 1;
         bool m_loading : 1;
-        bool m_expireDateChanged : 1;
         bool m_free : 1;
+	bool m_hadError : 1;
+
+    private:
+        bool allowInLRUList() const { return canDelete() && !m_free && status() != Persistent; }
+        CachedObject* m_next;
+        CachedObject* m_prev;
+        friend class Cache;
     };
 
 
     /**
-     * a cached style sheet
+     * a cached style sheet. also used for loading xml documents.
+     *
+     * ### rename to CachedTextDoc or something since it's more generic than just for css
      */
     class CachedCSSStyleSheet : public CachedObject
     {
     public:
-	CachedCSSStyleSheet(DocLoader* dl, const DOM::DOMString &url, KIO::CacheControl cachePolicy, time_t _expireDate, const QString& charset);
+	CachedCSSStyleSheet(DocLoader* dl, const DOM::DOMString &url, KIO::CacheControl cachePolicy,
+			    const char *accept);
 	CachedCSSStyleSheet(const DOM::DOMString &url, const QString &stylesheet_data);
-	virtual ~CachedCSSStyleSheet();
 
 	const DOM::DOMString &sheet() const { return m_sheet; }
 
 	virtual void ref(CachedObjectClient *consumer);
-	virtual void deref(CachedObjectClient *consumer);
 
 	virtual void data( QBuffer &buffer, bool eof );
 	virtual void error( int err, const char *text );
 
         virtual bool schedule() const { return true; }
-
-	void checkNotify();
+        void setCharset( const QString& charset ) { m_charset = charset; }
 
     protected:
+        void checkNotify();
+
 	DOM::DOMString m_sheet;
-        QTextCodec* m_codec;
+        QString m_charset;
+	int m_err;
+	QString m_errText;
     };
 
     /**
@@ -212,14 +217,12 @@ namespace khtml
     class CachedScript : public CachedObject
     {
     public:
-	CachedScript(DocLoader* dl, const DOM::DOMString &url, KIO::CacheControl cachePolicy, time_t _expireDate, const QString& charset);
+	CachedScript(DocLoader* dl, const DOM::DOMString &url, KIO::CacheControl cachePolicy, const char* accept );
 	CachedScript(const DOM::DOMString &url, const QString &script_data);
-	virtual ~CachedScript();
 
 	const DOM::DOMString &script() const { return m_script; }
 
 	virtual void ref(CachedObjectClient *consumer);
-	virtual void deref(CachedObjectClient *consumer);
 
 	virtual void data( QBuffer &buffer, bool eof );
 	virtual void error( int err, const char *text );
@@ -229,10 +232,11 @@ namespace khtml
 	void checkNotify();
 
         bool isLoaded() const { return !m_loading; }
+        void setCharset( const QString& charset ) { m_charset = charset; }
 
     protected:
+        QString m_charset;
 	DOM::DOMString m_script;
-        QTextCodec* m_codec;
     };
 
     class ImageSource;
@@ -244,7 +248,7 @@ namespace khtml
     {
 	Q_OBJECT
     public:
-	CachedImage(DocLoader* dl, const DOM::DOMString &url, KIO::CacheControl cachePolicy, time_t _expireDate);
+	CachedImage(DocLoader* dl, const DOM::DOMString &url, KIO::CacheControl cachePolicy, const char* accept);
 	virtual ~CachedImage();
 
 	const QPixmap &pixmap() const;
@@ -260,7 +264,7 @@ namespace khtml
 	virtual void error( int err, const char *text );
 
         bool isTransparent() const { return isFullyTransparent; }
-        bool isErrorImage() const { return errorOccured; }
+        bool isErrorImage() const { return m_hadError; }
 
         void setShowAnimations( KHTMLSettings::KAnimationAdvice );
 
@@ -298,7 +302,6 @@ namespace khtml
 	// Is set if movie format type ( incremental/animation) was checked
 	bool typeChecked : 1;
         bool isFullyTransparent : 1;
-        bool errorOccured : 1;
         bool monochrome : 1;
         KHTMLSettings::KAnimationAdvice m_showAnimations : 2;
 
@@ -317,7 +320,8 @@ namespace khtml
  	~DocLoader();
 
 	CachedImage *requestImage( const DOM::DOMString &url);
-	CachedCSSStyleSheet *requestStyleSheet( const DOM::DOMString &url, const QString& charset);
+	CachedCSSStyleSheet *requestStyleSheet( const DOM::DOMString &url, const QString& charset,
+						const char *accept = "text/css", bool userSheet = false );
         CachedScript *requestScript( const DOM::DOMString &url, const QString& charset);
 
 	bool autoloadImages() const { return m_bautoloadImages; }
@@ -330,9 +334,10 @@ namespace khtml
         void setCacheCreationDate( time_t );
         void setExpireDate( time_t, bool relative );
         void setAutoloadImages( bool );
-        void setCachePolicy( KIO::CacheControl cachePolicy );
+        void setCachePolicy( KIO::CacheControl cachePolicy ) { m_cachePolicy = cachePolicy; }
         void setShowAnimations( KHTMLSettings::KAnimationAdvice );
-        void removeCachedObject( CachedObject*) const;
+        void insertCachedObject( CachedObject* o ) const;
+        void removeCachedObject( CachedObject* o) const { m_docObjects.remove( o ); }
 
     private:
         bool needReload(const KURL &fullUrl);
@@ -341,7 +346,7 @@ namespace khtml
         friend class DOM::DocumentImpl;
 
         QStringList m_reloadedURLs;
-        mutable QPtrList<CachedObject> m_docObjects;
+        mutable QPtrDict<CachedObject> m_docObjects;
 	time_t m_expireDate;
 	time_t m_creationDate;
 	KIO::CacheControl m_cachePolicy;
@@ -374,7 +379,6 @@ namespace khtml
 
     public:
 	Loader();
-	~Loader();
 
 	void load(DocLoader* dl, CachedObject *object, bool incremental = true);
 
@@ -400,6 +404,7 @@ namespace khtml
 #ifdef HAVE_LIBJPEG
         KJPEGFormatType m_jpegloader;
 #endif
+        QTimer m_timer;
     };
 
         /**
@@ -411,6 +416,10 @@ namespace khtml
     class Cache
     {
 	friend class DocLoader;
+
+        template<typename CachedObjectType, enum CachedObject::Type CachedType>
+        static CachedObjectType* requestObject( DocLoader* dl, const KURL& kurl, const char* accept );
+
     public:
 	/**
 	 * init the cache in case it's not already. This needs to get called once
@@ -424,48 +433,22 @@ namespace khtml
          * if the DocLoader is zero, the url must be full-qualified.
          * Otherwise, it is automatically base-url expanded
 	 */
-	static CachedImage *requestImage( DocLoader* l, const DOM::DOMString &url, bool reload=false, time_t _expireDate=0);
-
-	/**
-	 * Ask the cache for some url. Will return a cachedObject, and
-	 * load the requested data in case it's not cached
-	 */
-	static CachedCSSStyleSheet *requestStyleSheet( DocLoader* l, const DOM::DOMString &url, bool reload=false, time_t _expireDate=0, const QString& charset = QString::null);
+// 	static CachedImage *requestImage(const KURL& url)
+//         { return Cache::requestObject<CachedImage, CachedObject::Image>( 0, url, 0 ); }
 
         /**
          * Pre-loads a stylesheet into the cache.
          */
         static void preloadStyleSheet(const QString &url, const QString &stylesheet_data);
 
-	/**
-	 * Ask the cache for some url. Will return a cachedObject, and
-	 * load the requested data in case it's not cached
-	 */
-	static CachedScript *requestScript( DocLoader* l, const DOM::DOMString &url, bool reload=false, time_t _expireDate=0, const QString& charset=QString::null);
-
         /**
          * Pre-loads a script into the cache.
          */
         static void preloadScript(const QString &url, const QString &script_data);
 
-	/**
-	 * sets the size of the cache. This will only hod approximately, since the size some
-	 * cached objects (like stylesheets) take up in memory is not exaclty known.
-	 */
 	static void setSize( int bytes );
-	/**
-	 * returns the size of the cache
-	 */
 	static int size() { return maxSize; };
-
-	/**
-	 * prints some statistics to stdout
-	 */
 	static void statistics();
-
-	/**
-	 * clean up cache
-	 */
 	static void flush(bool force=false);
 
 	/**
@@ -483,46 +466,24 @@ namespace khtml
 
         static void removeCacheEntry( CachedObject *object );
 
-        protected:
-	/*
-	 * @internal
-	 */
-	class LRUList : public QStringList
-	{
-	public:
-	    /**
-	     * implements the LRU list
-	     * The least recently used item is at the beginning of the list.
-	     */
-	    void touch( const QString &url )
-	    {
-		remove( url );
-		prepend( url );
-	    }
-	};
-        private:
-        friend class CachedObject;
-        static void flushFreeList();
-        static QPtrList<CachedObject> *freeList;
-	static QDict<CachedObject> *cache;
-	static LRUList *lru;
-        static QPtrList<DocLoader>* docloader;
+    private:
 
+        static void checkLRUAndUncacheableListIntegrity();
+
+        friend class CachedObject;
+
+	static QDict<CachedObject> *cache;
+        static QPtrList<DocLoader>* docloader;
+        static QPtrList<CachedObject> *freeList;
+        static void insertInLRUList(CachedObject*);
+        static void removeFromLRUList(CachedObject*);
+
+        static int totalSizeOfLRU;
 	static int maxSize;
-	static int flushCount;
 
 	static Loader *m_loader;
-
-        static unsigned long s_ulRefCnt;
     };
 
-    inline void CachedObject::setFree() {
-        if ( !m_free ) {
-            Cache::freeList->append(this);
-            m_free = true;
-        }
-    }
-
 } // namespace
 
 #endif
