@@ -18,7 +18,7 @@
  * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
  *
- * $Id: bidi.h,v 1.14.2.1 2003/05/18 12:34:40 mueller Exp $
+ * $Id: bidi.h,v 1.18 2003/10/03 14:33:29 mueller Exp $
  */
 #ifndef BIDI_H
 #define BIDI_H
@@ -28,6 +28,8 @@
 namespace khtml {
     class RenderFlow;
     class RenderObject;
+    class RenderArena;
+    class InlineBox;
 
     class BidiContext {
     public:
@@ -51,8 +53,8 @@ namespace khtml {
 
     struct BidiRun {
 	BidiRun(int _start, int _stop, RenderObject *_obj, BidiContext *context, QChar::Direction dir)
-	    :  vertical( 0 ), baseline( 0 ), height( 0 ), width( 0 ),
-	       start( _start ), stop( _stop ), obj( _obj )
+	    :  vertical( 0 ),
+	       start( _start ), stop( _stop ), obj( _obj ), box(0)
 	{
 	    if(dir == QChar::DirON) dir = context->dir;
 
@@ -71,13 +73,11 @@ namespace khtml {
 	}
 
 	int vertical;
-	short baseline;
-	short height;
-	int width;
 
 	int start;
 	int stop;
 	RenderObject *obj;
+        InlineBox* box;
 
 	// explicit + implicit levels here
 	uchar level;
@@ -105,6 +105,17 @@ namespace khtml {
 	const QChar &current() const;
 	QChar::Direction direction() const;
 
+	void detach(khtml::RenderArena* renderArena);
+
+	void* operator new(size_t sz, khtml::RenderArena* renderArena) throw();
+	void operator delete(void* ptr, size_t sz);
+
+    private:
+	// The normal operator new is disallowed.
+	void* operator new(size_t sz) throw();
+
+    public:
+
 	RenderFlow *par;
 	RenderObject *obj;
 	bool isText : 1;
