@@ -18,7 +18,7 @@
  * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
  *
- * $Id: render_applet.cpp,v 1.39 2002/10/23 17:03:31 mueller Exp $
+ * $Id: render_applet.cpp,v 1.40 2003/10/03 14:33:29 mueller Exp $
  */
 #include <config.h>
 #include <klocale.h>
@@ -26,7 +26,7 @@
 #include <kdebug.h>
 
 #include "rendering/render_applet.h"
-#include "rendering/render_root.h"
+#include "rendering/render_canvas.h"
 #include "xml/dom_docimpl.h"
 #include "khtmlview.h"
 #include "khtml_part.h"
