@@ -1,7 +1,7 @@
 /**
  * This file is part of the html renderer for KDE.
  *
- * Copyright (C) 2000 Lars Knoll (knoll@kde.org)
+ * Copyright (C) 2000-2003 Lars Knoll (knoll@kde.org)
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -18,10 +18,10 @@
  * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
  *
- * $Id: render_body.cpp,v 1.20.2.3 2003/02/04 08:15:04 mueller Exp $
+ * $Id: render_body.cpp,v 1.30.2.1 2004/02/29 15:27:59 mueller Exp $
  */
-#include "render_body.h"
-#include "render_root.h"
+#include "rendering/render_body.h"
+#include "rendering/render_canvas.h"
 #include "html/html_baseimpl.h"
 #include "xml/dom_docimpl.h"
 #include "khtmlview.h"
@@ -33,7 +33,7 @@ using namespace khtml;
 using namespace DOM;
 
 RenderBody::RenderBody(HTMLBodyElementImpl* element)
-    : RenderFlow(element)
+    : RenderBlock(element)
 {
     scrollbarsStyled = false;
 }
@@ -49,13 +49,12 @@ void RenderBody::setStyle(RenderStyle* style)
     if (style->htmlHacks() && style->position() != STATIC)
         style->setPosition(STATIC);
 
-    RenderFlow::setStyle(style);
+    RenderBlock::setStyle(style);
     element()->getDocument()->setTextColor( style->color() );
     scrollbarsStyled = false;
 }
 
-void RenderBody::paintBoxDecorations(QPainter *p,int, int _y,
-				       int, int _h, int _tx, int _ty)
+void RenderBody::paintBoxDecorations(PaintInfo& paintInfo, int _tx, int _ty)
 {
     //kdDebug( 6040 ) << renderName() << "::paintDecorations()" << endl;
     QColor c;
@@ -69,34 +68,35 @@ void RenderBody::paintBoxDecorations(QPainter *p,int, int _y,
     int h = height() + borderTopExtra() + borderBottomExtra();
     _ty -= borderTopExtra();
 
-    int my = QMAX(_ty,_y);
-    int end = QMIN( _y + _h,  _ty + h );
+    int my = kMax(_ty, paintInfo.r.y());
+    int end = kMin( paintInfo.r.y()+paintInfo.r.height(), _ty + h );
     int mh = end - my;
 
-    paintBackground(p, c, bg, my, mh, _tx, _ty, w, h);
+    paintBackground(paintInfo.p, c, bg, my, mh, _tx, _ty, w, h);
 
     if(style()->hasBorder())
-	paintBorder( p, _tx, _ty, w, h, style() );
+	paintBorder( paintInfo.p, _tx, _ty, w, h, style() );
 
 }
 
-void RenderBody::repaint()
+void RenderBody::repaint(bool immediate)
 {
     RenderObject *cb = containingBlock();
-    if(cb != this)
-	cb->repaint();
+    if(cb)
+	cb->repaint(immediate);
 }
 
 void RenderBody::layout()
 {
-    RenderFlow::layout();
+    RenderBlock::layout();
 
     if (!scrollbarsStyled)
     {
-        if (root()->view())
+	RenderCanvas* canvas = this->canvas();
+        if (canvas->view())
         {
-            root()->view()->horizontalScrollBar()->setPalette(style()->palette());
-            root()->view()->verticalScrollBar()->setPalette(style()->palette());
+            canvas->view()->horizontalScrollBar()->setPalette(style()->palette());
+            canvas->view()->verticalScrollBar()->setPalette(style()->palette());
         }
         scrollbarsStyled=true;
     }
@@ -104,7 +104,7 @@ void RenderBody::layout()
 
 int RenderBody::availableHeight() const
 {
-    int h = RenderFlow::availableHeight();
+    int h = RenderBlock::availableHeight();
 
     if( style()->marginTop().isFixed() )
         h  -= style()->marginTop().value();
