@@ -1,8 +1,9 @@
 /**
  * This file is part of the DOM implementation for KDE.
  *
- * Copyright (C) 1999 Lars Knoll (knoll@kde.org)
+ * Copyright (C) 1999-2003 Lars Knoll (knoll@kde.org)
  *           (C) 1999 Antti Koivisto (koivisto@kde.org)
+ *           (C) 2002 Apple Computer, Inc.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -19,7 +20,6 @@
  * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
  *
- * $Id: render_box.cpp,v 1.183.2.7 2003/02/08 19:59:45 mueller Exp $
  */
 // -------------------------------------------------------------------------
 //#define DEBUG_LAYOUT
@@ -28,11 +28,15 @@
 
 #include <qpainter.h>
 
-#include "rendering/render_box.h"
+#include "misc/loader.h"
 #include "rendering/render_replaced.h"
-#include "rendering/render_root.h"
+#include "rendering/render_canvas.h"
+#include "rendering/render_table.h"
+#include "rendering/render_block.h"
+#include "render_layer.h"
 #include "misc/htmlhashes.h"
 #include "xml/dom_nodeimpl.h"
+#include "render_line.h"
 
 #include <khtmlview.h>
 #include <kdebug.h>
@@ -57,14 +61,31 @@ RenderBox::RenderBox(DOM::NodeImpl* node)
     m_marginBottom = 0;
     m_marginLeft = 0;
     m_marginRight = 0;
+    m_staticX = 0;
+    m_staticY = 0;
+
+    m_layer = 0;
 }
 
 void RenderBox::setStyle(RenderStyle *_style)
 {
-    bool oldpos = isPositioned();
-                    
     RenderObject::setStyle(_style);
 
+    // The root always paints its background/border.
+    if (isRoot())
+        setShouldPaintBackgroundOrBorder(true);
+
+    switch(_style->display())
+    {
+    case INLINE:
+    case INLINE_BLOCK:
+    case INLINE_TABLE:
+        setInline(true);
+        break;
+    default:
+        setInline(false);
+    }
+        
     switch(_style->position())
     {
     case ABSOLUTE:
@@ -72,17 +93,34 @@ void RenderBox::setStyle(RenderStyle *_style)
         setPositioned(true);
         break;
     default:
-        if (oldpos)
-            {
-            setPositioned(true);
-            removeFromSpecialObjects();
-            }
         setPositioned(false);
-        if(!isTableCell() && _style->isFloating()) {
+        if( !isTableCell() && _style->isFloating() )
             setFloating(true);
-        } else {
-            if(_style->position() == RELATIVE)
-                setRelPositioned(true);
+
+        if( _style->position() == RELATIVE )
+            setRelPositioned(true);
+    }
+
+    if (requiresLayer()) {
+        if (!m_layer) {
+            m_layer = new (renderArena()) RenderLayer(this);
+            m_layer->insertOnlyThisLayer();
+        }
+    }
+    else if (m_layer && !isRoot() && !isCanvas()) {
+        m_layer->removeOnlyThisLayer();
+        m_layer = 0;
+    }
+
+    if (m_layer) {
+        // Make sure our z-index values are only applied if we're positioned or
+        // relpositioned.
+        if (!isPositioned() && !isRelPositioned()) {
+            // Set the auto z-index flag.
+            if (isRoot())
+                style()->setZIndex(0);
+            else
+                style()->setHasAutoZIndex();
         }
     }
 }
@@ -92,6 +130,17 @@ RenderBox::~RenderBox()
     //kdDebug( 6040 ) << "Element destructor: this=" << nodeName().string() << endl;
 }
 
+void RenderBox::detach()
+{
+    RenderLayer* layer = m_layer;
+    RenderArena* arena = renderArena();
+
+    RenderContainer::detach();
+
+    if (layer)
+        layer->detach(arena);
+}
+
 short RenderBox::contentWidth() const
 {
     short w = m_width - style()->borderLeftWidth() - style()->borderRightWidth();
@@ -124,57 +173,117 @@ int RenderBox::height() const
     return m_height;
 }
 
+void RenderBox::setWidth( int width )
+{
+    m_width = width;
+}
 
+void RenderBox::setHeight( int height )
+{
+    m_height = height;
+}
 // --------------------- painting stuff -------------------------------
 
-void RenderBox::paint(QPainter *p, int _x, int _y, int _w, int _h,
-                                  int _tx, int _ty)
+void RenderBox::paint(PaintInfo& i, int _tx, int _ty)
 {
     _tx += m_x;
     _ty += m_y;
 
+    if (style()->hidesOverflow() && m_layer)
+        m_layer->subtractScrollOffset(_tx, _ty);
+
     // default implementation. Just pass things through to the children
-    RenderObject *child = firstChild();
-    while(child != 0)
-    {
-        child->paint(p, _x, _y, _w, _h, _tx, _ty);
-        child = child->nextSibling();
-    }
+    for(RenderObject* child = firstChild(); child; child = child->nextSibling())
+        child->paint(i, _tx, _ty);
 }
 
-void RenderBox::setPixmap(const QPixmap &, const QRect&, CachedImage *image)
+void RenderBox::paintRootBoxDecorations(PaintInfo& paintInfo, int _tx, int _ty)
 {
-    if(image && image->pixmap_size() == image->valid_rect().size() && parent())
-        repaint();      //repaint bg when it finished loading
+    //kdDebug( 6040 ) << renderName() << "::paintRootBoxDecorations()" << _tx << "/" << _ty << endl;
+    QColor c = style()->backgroundColor();
+    CachedImage *bg = style()->backgroundImage();
+
+    if (!c.isValid() && !bg && firstChild()) {
+        if (!c.isValid())
+            c = firstChild()->style()->backgroundColor();
+        if (!bg)
+            bg = firstChild()->style()->backgroundImage();
+    }
+
+    if( !c.isValid() && canvas()->view())
+        c = canvas()->view()->palette().active().color(QColorGroup::Base);
+
+    int w = width();
+    int h = height();
+
+    //    kdDebug(0) << "width = " << w <<endl;
+
+    int rw, rh;
+    if (canvas()->view()) {
+        rw = canvas()->view()->contentsWidth();
+        rh = canvas()->view()->contentsHeight();
+    } else {
+        rw = canvas()->docWidth();
+        rh = canvas()->docHeight();
+    }
+
+    //    kdDebug(0) << "rw = " << rw <<endl;
+
+    int bx = _tx - marginLeft();
+    int by = _ty - marginTop();
+    int bw = QMAX(w + marginLeft() + marginRight() + borderLeft() + borderRight(), rw);
+    int bh = QMAX(h + marginTop() + marginBottom() + borderTop() + borderBottom(), rh);
+
+    // CSS2 14.2:
+    // " The background of the box generated by the root element covers the entire canvas."
+    // hence, paint the background even in the margin areas (unlike for every other element!)
+    // I just love these little inconsistencies .. :-( (Dirk)
+    int my = kMax(by, paintInfo.r.y());
+
+    paintBackground(paintInfo.p, c, bg, my, paintInfo.r.height(), bx, by, bw, bh);
+
+    if(style()->hasBorder())
+        paintBorder( paintInfo.p, _tx, _ty, w, h, style() );
 }
 
-
-void RenderBox::paintBoxDecorations(QPainter *p,int, int _y,
-                                       int, int _h, int _tx, int _ty)
+void RenderBox::paintBoxDecorations(PaintInfo& paintInfo, int _tx, int _ty)
 {
     //kdDebug( 6040 ) << renderName() << "::paintDecorations()" << endl;
 
+    if(isRoot())
+        return paintRootBoxDecorations(paintInfo, _tx, _ty);
+
     int w = width();
     int h = height() + borderTopExtra() + borderBottomExtra();
     _ty -= borderTopExtra();
 
-    int my = QMAX(_ty,_y);
-    int end = QMIN( _y + _h,  _ty + h );
+    int my = kMax(_ty,paintInfo.r.y());
+    int end = kMin( paintInfo.r.y() + paintInfo.r.height(), _ty + h );
     int mh = end - my;
 
-    paintBackground(p, style()->backgroundColor(), style()->backgroundImage(), my, mh, _tx, _ty, w, h);
+    paintBackground(paintInfo.p, style()->backgroundColor(), style()->backgroundImage(), my, mh, _tx, _ty, w, h);
 
-    if(style()->hasBorder())
-        paintBorder(p, _tx, _ty, w, h, style());
+    if(style()->hasBorder()) {
+        paintBorder(paintInfo.p, _tx, _ty, w, h, style());
+    }
 }
 
-void RenderBox::paintBackground(QPainter *p, const QColor &c, CachedImage *bg, int clipy, int cliph, int _tx, int _ty, int w, int h)
+void RenderBox::paintBackground(QPainter *p, const QColor &c, CachedImage *bg, int clipy, int cliph, int _tx, int _ty, int w, int height)
+{
+    paintBackgroundExtended(p, c, bg, clipy, cliph, _tx, _ty, w, height,
+                            borderLeft(), borderRight());
+}
+
+void RenderBox::paintBackgroundExtended(QPainter *p, const QColor &c, CachedImage *bg, int clipy, int cliph,
+                                        int _tx, int _ty, int w, int h,
+                                        int bleft, int bright)
 {
     if ( cliph < 0 )
 	return;
 
     if(c.isValid())
         p->fillRect(_tx, clipy, w, cliph, c);
+
     // no progressive loading of the background image
     if(bg && bg->pixmap_size() == bg->valid_rect().size() && !bg->isTransparent() && !bg->isErrorImage()) {
         //kdDebug( 6040 ) << "painting bgimage at " << _tx << "/" << _ty << endl;
@@ -183,14 +292,14 @@ void RenderBox::paintBackground(QPainter *p, const QColor &c, CachedImage *bg, i
 
         //hacky stuff
         RenderStyle* sptr = style();
-        if ( isHtml() && firstChild() && !style()->backgroundImage() )
+        if ( isRoot() && firstChild() && !style()->backgroundImage() )
             sptr = firstChild()->style();
 
         int sx = 0;
         int sy = 0;
-	    int cw,ch;
+        int cw,ch;
         int cx,cy;
-        int vpab = borderRight() + borderLeft();
+        int vpab = bleft + bright;
         int hpab = borderTop() + borderBottom();
 
         // CSS2 chapter 14.2.1
@@ -200,23 +309,37 @@ void RenderBox::paintBackground(QPainter *p, const QColor &c, CachedImage *bg, i
         if (sptr->backgroundAttachment())
         {
             //scroll
-            int pw = m_width - vpab;
-            int ph = m_height - hpab;
+            int pw = w - vpab;
+            int ph = h - hpab;
             EBackgroundRepeat bgr = sptr->backgroundRepeat();
-            if( (bgr == NO_REPEAT || bgr == REPEAT_Y) && w > pixw ) {
+            if( (bgr == NO_REPEAT || bgr == REPEAT_Y) && pw > pixw ) {
                 cw = pixw;
-                cx = _tx + sptr->backgroundXPosition().minWidth(pw-pixw);
+                int xp = sptr->backgroundXPosition().minWidth(pw-pixw);
+                if ( xp >= 0 )
+                    cx = _tx + xp;
+                else {
+                    cx = _tx;
+                    sx = -xp;
+                    cw += xp;
+                }
             } else {
                 cw = w-vpab;
                 cx = _tx;
-                sx =  pixw - ((sptr->backgroundXPosition().minWidth(pw-pixw)) % pixw );
+                sx =  pixw ? pixw - ((sptr->backgroundXPosition().minWidth(pw-pixw)) % pixw ) : 0;
             }
 
-            cx += borderLeft();
+            cx += bleft;
 
-            if( (bgr == NO_REPEAT || bgr == REPEAT_X) && h > pixh ) {
+            if( (bgr == NO_REPEAT || bgr == REPEAT_X) && ph > pixh ) {
                 ch = pixh;
-                cy = _ty + sptr->backgroundYPosition().minWidth(ph-pixh);
+                int yp = sptr->backgroundYPosition().minWidth(ph-pixh);
+                if ( yp >= 0 )
+                    cy = _ty + yp;
+                else {
+                    cy = _ty;
+                    sy = -yp;
+                    ch += yp;
+                }
             } else {
                 ch = h-hpab;
                 cy = _ty;
@@ -224,6 +347,8 @@ void RenderBox::paintBackground(QPainter *p, const QColor &c, CachedImage *bg, i
             }
 
             cy += borderTop();
+	    if (layer())
+		layer()->scrollOffset(sx, sy);
         }
         else
         {
@@ -233,27 +358,29 @@ void RenderBox::paintBackground(QPainter *p, const QColor &c, CachedImage *bg, i
             int ph = vr.height();
 
             EBackgroundRepeat bgr = sptr->backgroundRepeat();
-            if( (bgr == NO_REPEAT || bgr == REPEAT_Y) && w > pixw ) {
+            if( (bgr == NO_REPEAT || bgr == REPEAT_Y) && pw > pixw ) {
                 cw = pixw;
                 cx = vr.x() + sptr->backgroundXPosition().minWidth(pw-pixw);
             } else {
                 cw = pw;
                 cx = vr.x();
-                sx =  pixw - ((sptr->backgroundXPosition().minWidth(pw-pixw)) % pixw );
+                sx =  pixw ? pixw - ((sptr->backgroundXPosition().minWidth(pw-pixw)) % pixw ) : 0;
             }
 
-            if( (bgr == NO_REPEAT || bgr == REPEAT_X) && h > pixh ) {
+            if( (bgr == NO_REPEAT || bgr == REPEAT_X) && ph > pixh ) {
                 ch = pixh;
                 cy = vr.y() + sptr->backgroundYPosition().minWidth(ph-pixh);
             } else {
                 ch = ph;
                 cy = vr.y();
-                sy = pixh - ((sptr->backgroundYPosition().minWidth(ph-pixh)) % pixh );
+                sy = pixh ? pixh - ((sptr->backgroundYPosition().minWidth(ph-pixh)) % pixh ) : 0;
             }
 
             QRect fix(cx,cy,cw,ch);
-            QRect ele(_tx+borderLeft(),_ty+borderTop(),w-vpab,h-hpab);
+            QRect ele(_tx+bleft,_ty+borderTop(),w-vpab,h-hpab);
             QRect b = fix.intersect(ele);
+
+            //kdDebug() <<" ele is " << ele << " b is " << b << " fix is " << fix << endl;
             sx+=b.x()-cx;
             sy+=b.y()-cy;
             cx=b.x();cy=b.y();cw=b.width();ch=b.height();
@@ -268,7 +395,7 @@ void RenderBox::paintBackground(QPainter *p, const QColor &c, CachedImage *bg, i
 	    sy %= pixh;
 	    ch -= diff;
 	}
-	ch = QMIN( ch, clipy + cliph - cy );
+	ch = kMin( ch, clipy + cliph - cy );
  	//kdDebug() << "clip="<<cx << " cy="<<cy<< " cw="<<cw << " ch="<<ch << " sx="<<sx << " sy="<<sy << endl;
 
         if (cw>0 && ch>0)
@@ -285,11 +412,31 @@ void RenderBox::outlineBox(QPainter *p, int _tx, int _ty, const char *color)
     p->drawRect(_tx, _ty, m_width, m_height);
 }
 
-
-void RenderBox::calcClip(QPainter* p, int tx, int ty)
+QRect RenderBox::getOverflowClipRect(int tx, int ty)
 {
-    int clipw = m_width;
-    int cliph = m_height;
+    // XXX When overflow-clip (CSS3) is implemented, we'll obtain the property
+    // here.
+    int bl=borderLeft(),bt=borderTop(),bb=borderBottom(),br=borderRight();
+    int clipx = tx+bl;
+    int clipy = ty+bt;
+    int clipw = m_width-bl-br;
+    int cliph = m_height-bt-bb;
+
+    // Substract out scrollbars if we have them.
+    if (m_layer) {
+        clipw -= m_layer->verticalScrollbarWidth();
+        cliph -= m_layer->horizontalScrollbarHeight();
+    }
+
+    return QRect(clipx,clipy,clipw,cliph);
+}
+
+QRect RenderBox::getClipRect(int tx, int ty)
+{
+    int bl=borderLeft(),bt=borderTop(),bb=borderBottom(),br=borderRight();
+    // ### what about apddings?
+    int clipw = m_width-bl-br;
+    int cliph = m_height-bt-bb;
 
     bool rtl = (style()->direction() == RTL);
 
@@ -298,7 +445,7 @@ void RenderBox::calcClip(QPainter* p, int tx, int ty)
     int cliptop = 0;
     int clipbottom = cliph;
 
-    if ( style()->clipSpecified() && style()->position() == ABSOLUTE ) {
+    if ( style()->hasClip() && style()->position() == ABSOLUTE ) {
 	// the only case we use the clip property according to CSS 2.1
 	if (!style()->clipLeft().isVariable()) {
 	    int c = style()->clipLeft().width(clipw);
@@ -325,23 +472,9 @@ void RenderBox::calcClip(QPainter* p, int tx, int ty)
     clipw = clipright-clipleft;
     cliph = clipbottom-cliptop;
 
+    //kdDebug( 6040 ) << "setting clip("<<clipx<<","<<clipy<<","<<clipw<<","<<cliph<<")"<<endl;
 
-    QRect cr(clipx,clipy,clipw,cliph);
-    cr = p->xForm(cr);
-    QRegion creg(cr);
-    QRegion old = p->clipRegion();
-    if (!old.isNull())
-        creg = old.intersect(creg);
-
-#ifdef CLIP_DEBUG
-    kdDebug( 6040 ) << renderName() << ":" << this << ": setting clip("<<clipx<<","<<clipy<<","<<clipw<<","<<cliph<<") tx="<<tx<<" ty="<<ty<<endl;
-    p->setPen(QPen(Qt::red, 1, Qt::DotLine));
-    p->setBrush( Qt::NoBrush );
-    p->drawRect(clipx, clipy, clipw, cliph);
-#endif
-
-    p->save();
-    p->setClipRegion(creg);
+    return QRect(clipx,clipy,clipw,cliph);
 }
 
 void RenderBox::close()
@@ -352,11 +485,15 @@ void RenderBox::close()
 
 short RenderBox::containingBlockWidth() const
 {
-    if ( ( style()->htmlHacks() || isTable() ) && style()->flowAroundFloats() && containingBlock()->isFlow()
+    if (isRoot() && canvas()->view())
+        return canvas()->view()->visibleWidth();
+
+    RenderBlock* cb = containingBlock();
+    if ((style()->hidesOverflow() || ((style()->htmlHacks() || isTable()) && style()->flowAroundFloats()))
             && style()->width().isVariable())
-        return static_cast<RenderFlow*>(containingBlock())->lineWidth(m_y);
+        return cb->lineWidth(m_y);
     else
-        return containingBlock()->contentWidth();
+        return cb->contentWidth();
 }
 
 bool RenderBox::absolutePosition(int &xPos, int &yPos, bool f)
@@ -366,6 +503,9 @@ bool RenderBox::absolutePosition(int &xPos, int &yPos, bool f)
     RenderObject *o = container();
     if( o && o->absolutePosition(xPos, yPos, f))
     {
+        if ( o->style()->hidesOverflow() && o->layer() )
+            o->layer()->subtractScrollOffset( xPos, yPos );
+
         if(!isInline() || isReplaced())
             xPos += m_x, yPos += m_y;
 
@@ -380,34 +520,73 @@ bool RenderBox::absolutePosition(int &xPos, int &yPos, bool f)
     }
 }
 
-void RenderBox::position(int x, int y, int, int, int, bool, bool, int)
+void RenderBox::position(InlineBox* box, int /*from*/, int /*len*/, bool /*reverse*/)
 {
-    m_x = x + marginLeft();
-    m_y = y;
-    // ### paddings
-    //m_width = width;
+#ifdef APPLE_CHANGES
+    if (isPositioned()) {
+        // Cache the x position only if we were an INLINE type originally.
+        bool wasInline = style()->originalDisplay() == INLINE ||
+                         style()->originalDisplay() == INLINE_TABLE;
+        if (wasInline && hasStaticX()) {
+            // The value is cached in the xPos of the box.  We only need this value if
+            // our object was inline originally, since otherwise it would have ended up underneath
+            // the inlines.
+            m_staticX = box->xPos();
+        }
+        else if (!wasInline && hasStaticY()) {
+            // Our object was a block originally, so we make our normal flow position be
+            // just below the line box (as though all the inlines that came before us got
+            // wrapped in an anonymous block, which is what would have happened had we been
+            // in flow).  This value was cached in the yPos() of the box.
+            m_staticY = box->yPos();
+        }
+
+        box->detach(renderArena());
+    }
+    else
+#endif
+    if (isReplaced()) {
+        setPos( box->xPos(), box->yPos() );
+        box->detach(renderArena());
+    }
 }
 
-void RenderBox::repaint()
+void RenderBox::repaint(bool immediate)
 {
-    //kdDebug( 6040 ) << "repaint!" << endl;
     int ow = style() ? style()->outlineWidth() : 0;
-    repaintRectangle(-ow, -ow, m_width+ow*2, m_height+ow*2);
+    if( isInline() && !isReplaced() )
+    {
+	RenderObject* p = parent();
+	while( p && p->isInline() && !p->isReplaced() )
+	    p = p->parent();
+        p->repaintRectangle( -ow, -ow, p->overflowWidth()+ow*2, p->overflowHeight()+ow*2, immediate);
+    }
+    else
+    {
+        repaintRectangle( -ow, -ow, overflowWidth()+ow*2, overflowHeight()+ow*2, immediate);
+    }
 }
 
-void RenderBox::repaintRectangle(int x, int y, int w, int h, bool f)
+void RenderBox::repaintRectangle(int x, int y, int w, int h, bool immediate, bool f)
 {
     x += m_x;
     y += m_y;
 
+    // Apply the relative position offset when invalidating a
+    // rectangle.  The layer is translated, but the render box isn't,
+    // so we need to do this to get the right dirty rect.
+    if (isRelPositioned())
+        relativePositionOffset(x,y);
+
     if (style()->position()==FIXED) f=true;
 
-    // kdDebug( 6040 ) << "RenderBox(" << renderName() << ")::repaintRectangle (" << x << "/" << y << ") (" << w << "/" << h << ")" << endl;
+
+    // kdDebug( 6040 ) << "RenderBox(" <<this << ", " << renderName() << ")::repaintRectangle (" << x << "/" << y << ") (" << w << "/" << h << ")" << endl;
     RenderObject *o = container();
-    if( o ) o->repaintRectangle(x, y, w, h, f);
+    if( o ) o->repaintRectangle(x, y, w, h, immediate, f);
 }
 
-void RenderBox::relativePositionOffset(int &tx, int &ty)
+void RenderBox::relativePositionOffset(int &tx, int &ty) const
 {
     if(!style()->left().isVariable())
         tx += style()->left().width(containingBlockWidth());
@@ -439,7 +618,7 @@ void RenderBox::calcWidth()
     else
     {
         Length w;
-        if ( isReplaced () )
+        if ( isInline() && isReplaced () )
             w = Length( calcReplacedWidth(), Fixed );
         else
             w = style()->width();
@@ -448,10 +627,10 @@ void RenderBox::calcWidth()
         Length mr = style()->marginRight();
 
         int cw;
-	RenderObject *cb = containingBlock();
-	if ( style()->flowAroundFloats() && cb->isFlow() )
-	    cw = static_cast<RenderFlow *>(cb)->lineWidth( m_y );
-	else
+	RenderBlock *cb = containingBlock();
+ 	if ( style()->flowAroundFloats() || style()->hidesOverflow() )
+ 	    cw = cb->lineWidth( m_y );
+ 	else
 	    cw = cb->contentWidth();
 
         if (cw<0) cw = 0;
@@ -474,28 +653,39 @@ void RenderBox::calcWidth()
 
             return;
         }
-        else if (w.isVariable())
-        {
-//          kdDebug( 6040 ) << "variable" << endl;
-            m_marginLeft = ml.minWidth(cw);
-            m_marginRight = mr.minWidth(cw);
-            if (cw) m_width = cw - m_marginLeft - m_marginRight;
-
-//          kdDebug( 6040 ) <<  m_width <<"," << cw <<"," <<
-//              m_marginLeft <<"," <<  m_marginRight << endl;
-
-            if (isFloating()) {
-                if(m_width < m_minWidth) m_width = m_minWidth;
-                if(m_width > m_maxWidth) m_width = m_maxWidth;
-            }
-        }
         else
         {
-//          kdDebug( 6040 ) << "non-variable " << w.type << ","<< w.value << endl;
-            m_width = w.width(cw);
-            m_width += paddingLeft() + paddingRight() + style()->borderLeftWidth() + style()->borderRightWidth();
+            LengthType widthType, minWidthType, maxWidthType;
+            if (isInline() && isReplaced()) {
+                m_width = w.width(cw);
+                m_width += paddingLeft() + paddingRight() + borderLeft() + borderRight();
+                widthType = w.type();
+            } else {
+                m_width = calcWidthUsing(Width, cw, widthType);
+                int minW = calcWidthUsing(MinWidth, cw, minWidthType);
+                int maxW = style()->maxWidth().value() == UNDEFINED ?
+                             m_width : calcWidthUsing(MaxWidth, cw, maxWidthType);
 
-            calcHorizontalMargins(ml,mr,cw);
+                if (m_width > maxW) {
+                    m_width = maxW;
+                    widthType = maxWidthType;
+                }
+                if (m_width < minW) {
+                    m_width = minW;
+                    widthType = minWidthType;
+                }
+            }
+
+            if (widthType == Variable) {
+    //          kdDebug( 6040 ) << "variable" << endl;
+                m_marginLeft = ml.minWidth(cw);
+                m_marginRight = mr.minWidth(cw);
+            }
+            else
+            {
+//              kdDebug( 6040 ) << "non-variable " << w.type << ","<< w.value << endl;
+                calcHorizontalMargins(ml,mr,cw);
+            }
         }
 
         if (cw && cw != m_width + m_marginLeft + m_marginRight && !isFloating() && !isInline())
@@ -513,6 +703,43 @@ void RenderBox::calcWidth()
 #endif
 }
 
+int RenderBox::calcWidthUsing(WidthType widthType, int cw, LengthType& lengthType)
+{
+    int width = m_width;
+    Length w;
+    if (widthType == Width)
+        w = style()->width();
+    else if (widthType == MinWidth)
+        w = style()->minWidth();
+    else
+        w = style()->maxWidth();
+
+    lengthType = w.type();
+
+    if (lengthType == Variable) {
+        int marginLeft = style()->marginLeft().minWidth(cw);
+        int marginRight = style()->marginRight().minWidth(cw);
+        if (cw) width = cw - marginLeft - marginRight;
+
+        // size to max width?
+        if (isFloating() ||
+            style()->display() == COMPACT ||
+            ( isReplaced() && !isInline() ) ) {
+            if (width < m_minWidth)
+                width = m_minWidth;
+            if (width > m_maxWidth)
+                width = m_maxWidth;
+        }
+    }
+    else
+    {
+        width = w.width(cw);
+        width += paddingLeft() + paddingRight() + borderLeft() + borderRight();
+    }
+
+    return width;
+}
+
 void RenderBox::calcHorizontalMargins(const Length& ml, const Length& mr, int cw)
 {
     if (isFloating())
@@ -563,7 +790,7 @@ void RenderBox::calcHeight()
     else
     {
         Length h;
-        if ( isReplaced() && !isFlow() )
+        if ( isReplaced() && !isBlockFlow() )
             h = Length( calcReplacedHeight(), Fixed );
         else
             h = style()->height();
@@ -580,8 +807,26 @@ void RenderBox::calcHeight()
             if (h.isFixed())
                 fh = h.value() + borderTop() + paddingTop() + borderBottom() + paddingBottom();
             else if (h.isPercent()) {
+                // Handle a common case: nested 100% height <div>s.
+                // This is kind of a one-off hack rather than doing it right.
+                // Makes dbaron's z-index root bg testcases work. Bad dave. - dwh
+                RenderBlock* cb = containingBlock();
                 Length ch = containingBlock()->style()->height();
-                if (ch.isFixed())
+                while (cb && !cb->isTableCell() && ch.isPercent() && ch.value() == 100) {
+                    cb = cb->containingBlock();
+                    ch = cb->style()->height();
+                }
+
+                if (cb->isCanvas()) {
+                    // Don't allow this to affect the canvas' m_height member variable, since this
+                    // can get called while the canvas is still laying out its kids.
+                    // e.g., <html style="height:100%">etc. -dwh
+                    int oldHeight = cb->height();
+                    static_cast<RenderCanvas*>(cb)->calcHeight();
+                    fh = h.width(cb->height()) + borderTop() + paddingTop() + borderBottom() + paddingBottom();
+                    cb->setHeight(oldHeight);
+                }
+                else if (ch.isFixed())
                     fh = h.width(ch.value()) + borderTop() + paddingTop() + borderBottom() + paddingBottom();
             }
             if (fh!=-1)
@@ -634,8 +879,17 @@ int RenderBox::availableHeight() const
     if (h.isFixed())
         return h.value();
 
-    if (isRoot())
-        return static_cast<const RenderRoot*>(this)->viewportHeight();
+    if (isCanvas())
+        return static_cast<const RenderCanvas*>(this)->viewportHeight();
+
+    // We need to stop here, since we don't want to increase the height of the table
+    // artificially.  We're going to rely on this cell getting expanded to some new
+    // height, and then when we lay out again we'll use the calculation below.
+    if (isTableCell() && (h.isVariable() || h.isPercent())) {
+        const RenderTableCell* tableCell = static_cast<const RenderTableCell*>(this);
+        return tableCell->cellPercentageHeight() -
+	    (borderTop()+borderBottom()+paddingTop()+paddingBottom());
+    }
 
     if (h.isPercent())
         return h.width(containingBlock()->availableHeight());
@@ -663,6 +917,16 @@ void RenderBox::calcVerticalMargins()
     m_marginBottom = bm.minWidth(cw);
 }
 
+void RenderBox::setStaticX(short staticX)
+{
+    m_staticX = staticX;
+}
+
+void RenderBox::setStaticY(int staticY)
+{
+    m_staticY = staticY;
+}
+
 void RenderBox::calcAbsoluteHorizontal()
 {
     const int AUTO = -666666;
@@ -801,7 +1065,10 @@ void RenderBox::calcAbsoluteHorizontal()
     m_width = w + pab;
     m_marginLeft = ml;
     m_marginRight = mr;
+    short old_m_x = m_x;
     m_x = l + ml;
+    if ( old_m_x != m_x && m_layer)
+        m_layer->updateLayerPosition();
 
     //qDebug("h: w=%d, l=%d, r=%d, ml=%d, mr=%d",w,l,r,ml,mr);
 }
@@ -822,26 +1089,35 @@ void RenderBox::calcAbsoluteVertical()
     t=b=h=mt=mb=AUTO;
 
     int pab = borderTop()+borderBottom()+paddingTop()+paddingBottom();
+
     RenderObject* cb = containingBlock();
 
     Length hl = cb->style()->height();
     if (hl.isFixed())
         ch = hl.value() + cb->paddingTop() + cb->paddingBottom()
 	     + cb->borderTop() + cb->borderBottom();
-    else if (cb->isHtml())
+    else if (cb->isCanvas() || cb->isRoot())
         ch = cb->availableHeight();
     else
         ch = cb->height();
 
+
     if(!style()->top().isVariable())
         t = style()->top().width(ch) + cb->borderTop();
     if(!style()->bottom().isVariable())
         b = style()->bottom().width(ch) + cb->borderBottom();
-    if(!style()->height().isVariable())
+    // for tables "auto" means shrink-to-fit
+    if ( isTable() && style()->height().isVariable() )
+        h = m_height - pab;
+    else if(!style()->height().isVariable())
     {
         h = style()->height().width(ch);
-        if (m_height-pab>h)
-            h=m_height-pab;
+
+        if (m_height-pab > h) {
+            if ( isRenderBlock() )
+              static_cast<RenderBlock*>( this )->setOverflowHeight( m_height + pab - ( paddingBottom() + borderBottom() ) );
+        }
+	m_height = h + pab;
     }
     else if (isReplaced())
         h = intrinsicHeight();
@@ -946,27 +1222,75 @@ void RenderBox::calcAbsoluteVertical()
             b = ch - ( h+t+mt+mb+pab);
     }
 
-
     if (m_height<h+pab) //content must still fit
         m_height = h+pab;
 
+    if (style()->hidesOverflow() && m_height > h+pab)
+        m_height = h+pab;
+
     m_marginTop = mt;
     m_marginBottom = mb;
     m_y = t + mt;
 
-    //paintf("v: h=%d, t=%d, b=%d, mt=%d, mb=%d, m_y=%d\n",h,t,b,mt,mb,m_y);
-
+    //qDebug("v: m_height = %d, h=%d, t=%d, b=%d, mt=%d, mb=%d, m_y=%d",m_height,h,t,b,mt,mb,m_y);
 }
 
 
 int RenderBox::lowestPosition() const
 {
-    return m_height + marginBottom();
+    int bottom = m_height + marginBottom();
+    if ( m_layer )
+	bottom = kMax( bottom, m_layer->height() );
+    return bottom;
 }
 
-int RenderBox::rightmostPosition() const
+short RenderBox::rightmostPosition() const
 {
+    short right = m_width;
+    if ( m_layer )
+	right = kMax( right, m_layer->width() );
     return m_width;
 }
 
+void RenderBox::caretPos(int offset, bool override, int &_x, int &_y, int &width, int &height)
+{
+    _x = -1;
+
+    // propagate it downwards to its children, someone will feel responsible
+    RenderObject *child = firstChild();
+//    if (child) kdDebug(6040) << "delegating caretPos to " << child->renderName() << endl;
+    if (child) child->caretPos(offset, override, _x, _y, width, height);
+
+    // if not, use the extents of this box. offset 0 means left, offset 1 means
+    // right
+    if (_x == -1) {
+        //kdDebug(6040) << "no delegation" << endl;
+        _x = xPos() + (offset == 0 ? 0 : m_width);
+	_y = yPos();
+	height = m_height;
+	width = override && offset == 0 ? m_width : 1;
+
+	// If height of box is smaller than font height, use the latter one,
+	// otherwise the caret might become invisible.
+	// FIXME: ignoring :first-line, missing good reason to take care of
+	int fontHeight = style()->fontMetrics().height();
+	if (fontHeight > height)
+	  height = fontHeight;
+
+        int absx, absy;
+
+        RenderObject *cb = containingBlock();
+
+        if (cb && cb != this && cb->absolutePosition(absx,absy)) {
+            //kdDebug(6040) << "absx=" << absx << " absy=" << absy << endl;
+            _x += absx;
+            _y += absy;
+        } else {
+            // we don't know our absolute position, and there is no point returning
+            // just a relative one
+            _x = _y = -1;
+        }
+    }
+}
+
 #undef DEBUG_LAYOUT
