@@ -1,8 +1,9 @@
 /*
  * This file is part of the DOM implementation for KDE.
  *
- * Copyright (C) 1999 Lars Knoll (knoll@kde.org)
+ * Copyright (C) 1999-2003 Lars Knoll (knoll@kde.org)
  *           (C) 1999 Antti Koivisto (koivisto@kde.org)
+ *           (C) 2002 Apple Computer, Inc.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -19,16 +20,16 @@
  * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
  *
- * $Id: render_box.h,v 1.57.2.3 2003/05/18 12:34:40 mueller Exp $
+ * $Id: render_box.h,v 1.72.2.1 2004/02/29 15:27:59 mueller Exp $
  */
 #ifndef RENDER_BOX_H
 #define RENDER_BOX_H
 
 #include "render_container.h"
-#include "misc/loader.h"
 
 namespace khtml {
-    class CachedObject;
+
+enum WidthType { Width, MinWidth, MaxWidth };
 
 class RenderBox : public RenderContainer
 {
@@ -42,13 +43,15 @@ public:
     virtual ~RenderBox();
 
     virtual const char *renderName() const { return "RenderBox"; }
+    virtual bool isBox() const { return true; }
 
     virtual void setStyle(RenderStyle *style);
-    virtual void paint(QPainter *p, int _x, int _y, int _w, int _h,
-                       int _tx, int _ty);
+    virtual void paint(PaintInfo& i, int _tx, int _ty);
 
     virtual void close();
 
+    virtual void detach();
+
     virtual short minWidth() const { return m_minWidth; }
     virtual short maxWidth() const { return m_maxWidth; }
 
@@ -69,21 +72,20 @@ public:
     virtual short marginLeft() const { return m_marginLeft; }
     virtual short marginRight() const { return m_marginRight; }
 
-    virtual void setWidth( int width ) { m_width = width; }
-    virtual void setHeight( int height ) { m_height = height; }
+    virtual void setWidth( int width );
+    virtual void setHeight( int height );
 
-    virtual void position(int x, int y, int from, int len, int width, bool reverse, bool firstLine, int);
+    virtual void position(InlineBox* box, int from, int len, bool reverse);
 
     virtual int lowestPosition() const;
-    virtual int rightmostPosition() const;
+    virtual short rightmostPosition() const;
 
-    virtual void repaint();
+    virtual void repaint(bool immediate=false);
 
-    virtual void repaintRectangle(int x, int y, int w, int h, bool f=false);
-
-    virtual void setPixmap(const QPixmap &, const QRect&, CachedImage *);
+    virtual void repaintRectangle(int x, int y, int w, int h, bool immediate=false, bool f=false);
 
     virtual short containingBlockWidth() const;
+    void relativePositionOffset(int &tx, int &ty) const;
 
     virtual void calcWidth();
     virtual void calcHeight();
@@ -95,12 +97,28 @@ public:
 
     void calcVerticalMargins();
 
-    void relativePositionOffset(int &tx, int &ty);
+    virtual RenderLayer* layer() const { return m_layer; }
+
+    void setStaticX(short staticX);
+    void setStaticY(int staticY);
+
+    virtual void caretPos(int offset, bool override, int &_x, int &_y, int &width, int &height);
+
+    void calcHorizontalMargins(const Length& ml, const Length& mr, int cw);
+
+private:
+
+    int calcWidthUsing(WidthType widthType, int cw, LengthType& lengthType);
 
 protected:
-    virtual void paintBoxDecorations(QPainter *p,int _x, int _y,
-                                       int _w, int _h, int _tx, int _ty);
+    virtual void paintBoxDecorations(PaintInfo& paintInfo, int _tx, int _ty);
     void paintBackground(QPainter *p, const QColor &c, CachedImage *bg, int clipy, int cliph, int _tx, int _ty, int w, int h);
+    virtual void paintBackgroundExtended(QPainter* /*p*/, const QColor& /*c*/, CachedImage* /*bg*/,
+                                         int /*clipy*/, int /*cliph*/, int /*_tx*/, int /*_ty*/,
+                                         int /*w*/, int /*height*/, int /*bleft*/, int /*bright*/ );
+
+    void paintRootBoxDecorations( PaintInfo& paintInfo, int _tx, int _ty);
+
     void outlineBox(QPainter *p, int _tx, int _ty, const char *color = "red");
 
     virtual int borderTopExtra() { return 0; }
@@ -109,9 +127,9 @@ protected:
     void calcAbsoluteHorizontal();
     void calcAbsoluteVertical();
 
-    void calcHorizontalMargins(const Length& ml, const Length& mr, int cw);
+    QRect getOverflowClipRect(int tx, int ty);
+    QRect getClipRect(int tx, int ty);
 
-    void calcClip(QPainter* p, int tx, int ty);
 
     // the actual height of the contents + borders + padding
     int m_height;
@@ -136,6 +154,12 @@ protected:
      * ( = the width of the element with line breaking disabled)
      */
     short m_maxWidth;
+
+    // Cached normal flow values for absolute positioned elements with static left/top values.
+    short m_staticX;
+    int m_staticY;
+
+    RenderLayer *m_layer;
 };
 
 
