@@ -18,7 +18,7 @@
  * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
  *
- * $Id: render_br.cpp,v 1.15 2002/01/14 19:56:45 mueller Exp $
+ * $Id: render_br.cpp,v 1.18 2003/09/12 14:07:02 savernik Exp $
  */
 #include "render_br.h"
 
@@ -35,21 +35,25 @@ RenderBR::~RenderBR()
 {
 }
 
-void RenderBR::cursorPos(int /*offset*/, int &_x, int &_y, int &height)
+void RenderBR::caretPos(int offset, bool override, int &_x, int &_y, int &width, int &height)
 {
+  RenderText::caretPos(offset,override,_x,_y,width,height);
+  return;
+#if 0
     if (previousSibling() && !previousSibling()->isBR() && !previousSibling()->isFloating()) {
         int offset = 0;
         if (previousSibling()->isText())
-            offset = static_cast<RenderText*>(previousSibling())->length();
+            offset = static_cast<RenderText*>(previousSibling())->maxOffset();
 
-        previousSibling()->cursorPos(offset,_x,_y,height);
+	// FIXME: this won't return a big width in override mode (LS)
+        previousSibling()->caretPos(offset,override,_x,_y,width,height);
         return;
     }
 
     int absx, absy;
     absolutePosition(absx,absy);
     if (absx == -1) {
-        // we don't know out absoluate position, and there is not point returning
+        // we don't know out absolute position, and there is no point returning
         // just a relative one
         _x = _y = -1;
     }
@@ -58,6 +62,17 @@ void RenderBR::cursorPos(int /*offset*/, int &_x, int &_y, int &height)
         _y += absy;
     }
     height = RenderText::verticalPositionHint( false );
-
+    width = override ? height / 2 : 1;
+#endif
 }
 
+FindSelectionResult RenderBR::checkSelectionPoint(int _x, int _y, int _tx, int _ty, DOM::NodeImpl*& node, int &offset, SelPointState &state)
+{
+  // Simply take result of previous one
+  RenderText *prev = static_cast<RenderText *>(previousSibling());
+  if (!prev || !prev->isText() || !prev->inlineTextBoxCount() || prev->isBR())
+    prev = this;
+
+  //kdDebug(6040) << "delegated to " << prev->renderName() << "@" << prev << endl;
+  return prev->RenderText::checkSelectionPoint(_x, _y, _tx, _ty, node, offset, state);
+}
