@@ -18,7 +18,7 @@
  * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
  *
- * $Id: render_br.h,v 1.15.4.2 2003/05/18 12:34:40 mueller Exp $
+ * $Id: render_br.h,v 1.23.2.1 2004/02/29 15:27:59 mueller Exp $
  */
 #ifndef RENDER_BR_H
 #define RENDER_BR_H
@@ -39,10 +39,8 @@ public:
 
     virtual const char *renderName() const { return "RenderBR"; }
 
-    virtual void paint( QPainter *, int, int, int, int, int, int) {}
-    virtual void paintObject( QPainter *, int, int, int, int, int, int) {}
+    virtual void paint( PaintInfo&, int, int) {}
 
-    virtual void position(int, int, int, int, int, bool, bool, int) {}
     virtual unsigned int width(unsigned int, unsigned int, const Font *) const { return 0; }
     virtual unsigned int width( unsigned int, unsigned int, bool) const { return 0; }
     virtual short width() const { return RenderText::width(); }
@@ -54,8 +52,24 @@ public:
     virtual short minWidth() const { return 0; }
     virtual short maxWidth() const { return 0; }
 
+    virtual FindSelectionResult checkSelectionPoint( int _x, int _y, int _tx, int _ty,
+                                                     DOM::NodeImpl*& node, int & offset,
+						     SelPointState & );
+
     virtual bool isBR() const { return true; }
-    virtual void cursorPos(int offset, int &_x, int &_y, int &height);
+    virtual void caretPos(int offset, bool override, int &_x, int &_y, int &width, int &height);
+    /** returns the lowest possible value the caret offset may have to
+     * still point to a valid position.
+     *
+     * Returns 0.
+     */
+    virtual long minOffset() const { return 0; }
+    /** returns the highest possible value the caret offset may have to
+     * still point to a valid position.
+     *
+     * Returns also 0, as BRs have no width.
+     */
+    virtual long maxOffset() const { return 0; }
 };
 
 }
