@@ -20,14 +20,22 @@
  * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
  *
- * $Id: render_form.cpp,v 1.198.2.10 2003/08/28 23:53:15 mueller Exp $
+ * $Id: render_form.cpp,v 1.256.2.3 2004/02/29 15:28:00 mueller Exp $
  */
 
-#include <kdebug.h>
-#include <klocale.h>
-#include <kfiledialog.h>
 #include <kcompletionbox.h>
 #include <kcursor.h>
+#include <kdebug.h>
+#include <kfiledialog.h>
+#include <kfind.h>
+#include <kfinddialog.h>
+#include <kiconloader.h>
+#include <klocale.h>
+#include <kmessagebox.h>
+#include <kreplace.h>
+#include <kreplacedialog.h>
+#include <kspell.h>
+#include <kwin.h>
 
 #include <qstyle.h>
 
@@ -44,6 +52,7 @@
 #include "xml/dom_docimpl.h"
 
 #include <qpopupmenu.h>
+#include <qbitmap.h>
 
 using namespace khtml;
 
@@ -54,7 +63,6 @@ RenderFormElement::RenderFormElement(HTMLGenericFormElementImpl *element)
     setInline(true);   // our object is Inline
 
     m_state = 0;
-    m_isDoubleClick = false;
 }
 
 RenderFormElement::~RenderFormElement()
@@ -92,32 +100,6 @@ void RenderFormElement::layout()
         setLayouted();
 }
 
-void RenderFormElement::slotClicked()
-{
-    ref();
-    QMouseEvent e2( QEvent::MouseButtonRelease, m_mousePos, 1, m_state);
-
-    element()->dispatchMouseEvent(&e2, EventImpl::CLICK_EVENT, m_isDoubleClick + 1);
-    m_isDoubleClick = false;
-    deref();
-}
-
-void RenderFormElement::slotPressed()
-{
-    ref();
-    QMouseEvent e2( QEvent::MouseButtonPress, m_mousePos, 1, m_state);
-    element()->dispatchMouseEvent(&e2, EventImpl::MOUSEDOWN_EVENT, 1);
-    deref();
-}
-
-void RenderFormElement::slotReleased()
-{
-    ref();
-    QMouseEvent e2( QEvent::MouseButtonRelease, m_mousePos, 1, m_state);
-    element()->dispatchMouseEvent(&e2, EventImpl::MOUSEUP_EVENT, 1);
-    deref();
-}
-
 // -------------------------------------------------------------------------
 
 RenderButton::RenderButton(HTMLGenericFormElementImpl *element)
@@ -132,17 +114,15 @@ short RenderButton::baselinePosition( bool f ) const
 
 // -------------------------------------------------------------------------------
 
+
 RenderCheckBox::RenderCheckBox(HTMLInputElementImpl *element)
     : RenderButton(element)
 {
-    QCheckBox* b = new QCheckBox(view()->viewport());
+    QCheckBox* b = new QCheckBox(view()->viewport(), "__khtml");
     b->setAutoMask(true);
     b->setMouseTracking(true);
     setQWidget(b);
     connect(b,SIGNAL(stateChanged(int)),this,SLOT(slotStateChanged(int)));
-    connect(b, SIGNAL(clicked()), this, SLOT(slotClicked()));
-    connect(b, SIGNAL(pressed()), this, SLOT(slotPressed()));
-    connect(b, SIGNAL(released()), this, SLOT(slotReleased()));
 }
 
 
@@ -176,13 +156,9 @@ void RenderCheckBox::slotStateChanged(int state)
 RenderRadioButton::RenderRadioButton(HTMLInputElementImpl *element)
     : RenderButton(element)
 {
-    QRadioButton* b = new QRadioButton(view()->viewport());
-    b->setAutoMask(true);
+    QRadioButton* b = new QRadioButton(view()->viewport(), "__khtml");
     b->setMouseTracking(true);
     setQWidget(b);
-    connect(b, SIGNAL(clicked()), this, SLOT(slotClicked()));
-    connect(b, SIGNAL(pressed()), this, SLOT(slotPressed()));
-    connect(b, SIGNAL(released()), this, SLOT(slotReleased()));
 }
 
 void RenderRadioButton::updateFromElement()
@@ -192,14 +168,6 @@ void RenderRadioButton::updateFromElement()
     RenderButton::updateFromElement();
 }
 
-void RenderRadioButton::slotClicked()
-{
-    element()->setChecked(true);
-
-    // emit mouseClick event etc
-    RenderButton::slotClicked();
-}
-
 void RenderRadioButton::calcMinMaxWidth()
 {
     KHTMLAssert( !minMaxKnown() );
@@ -219,13 +187,10 @@ void RenderRadioButton::calcMinMaxWidth()
 RenderSubmitButton::RenderSubmitButton(HTMLInputElementImpl *element)
     : RenderButton(element)
 {
-    QPushButton* p = new QPushButton(view()->viewport());
+    QPushButton* p = new QPushButton(view()->viewport(), "__khtml");
     setQWidget(p);
     p->setAutoMask(true);
     p->setMouseTracking(true);
-    connect(p, SIGNAL(clicked()), this, SLOT(slotClicked()));
-    connect(p, SIGNAL(pressed()), this, SLOT(slotPressed()));
-    connect(p, SIGNAL(released()), this, SLOT(slotReleased()));
 }
 
 QString RenderSubmitButton::rawText()
@@ -296,15 +261,6 @@ short RenderSubmitButton::baselinePosition( bool f ) const
     return RenderFormElement::baselinePosition( f );
 }
 
-// -------------------------------------------------------------------------------
-
-RenderImageButton::RenderImageButton(HTMLInputElementImpl *element)
-    : RenderImage(element)
-{
-    // ### support DOMActivate event when clicked
-}
-
-
 // -------------------------------------------------------------------------------
 
 RenderResetButton::RenderResetButton(HTMLInputElementImpl *element)
@@ -319,11 +275,6 @@ QString RenderResetButton::defaultLabel() {
 
 // -------------------------------------------------------------------------------
 
-RenderPushButton::RenderPushButton(HTMLInputElementImpl *element)
-    : RenderSubmitButton(element)
-{
-}
-
 QString RenderPushButton::defaultLabel()
 {
     return QString::null;
@@ -331,42 +282,119 @@ QString RenderPushButton::defaultLabel()
 
 // -------------------------------------------------------------------------------
 
-LineEditWidget::LineEditWidget(QWidget *parent)
-        : KLineEdit(parent)
+LineEditWidget::LineEditWidget(DOM::HTMLInputElementImpl* input, KHTMLView* view, QWidget* parent)
+    : KLineEdit(parent, "__khtml"), m_input(input), m_view(view), m_spell(0)
 {
     setMouseTracking(true);
+    KActionCollection *ac = new KActionCollection(this);
+    m_spellAction = KStdAction::spelling( this, SLOT( slotCheckSpelling() ), ac );
 }
 
+LineEditWidget::~LineEditWidget()
+{
+    delete m_spell;
+    m_spell = 0L;
+}
+
+void LineEditWidget::slotCheckSpelling()
+{
+    if ( text().isEmpty() ) {
+        return;
+    }
+
+    delete m_spell;
+    m_spell = new KSpell( this, i18n( "Spell Checking" ), this, SLOT( slotSpellCheckReady( KSpell *) ), 0, true, true);
+
+    connect( m_spell, SIGNAL( death() ),this, SLOT( spellCheckerFinished() ) );
+    connect( m_spell, SIGNAL( misspelling( const QString &, const QStringList &, unsigned int ) ),this, SLOT( spellCheckerMisspelling( const QString &, const QStringList &, unsigned int ) ) );
+    connect( m_spell, SIGNAL( corrected( const QString &, const QString &, unsigned int ) ),this, SLOT( spellCheckerCorrected( const QString &, const QString &, unsigned int ) ) );
+}
+
+void LineEditWidget::spellCheckerMisspelling( const QString &_text, const QStringList &, unsigned int pos)
+{
+    highLightWord( _text.length(),pos );
+}
+
+void LineEditWidget::highLightWord( unsigned int length, unsigned int pos )
+{
+    setSelection ( pos, length );
+}
+
+void LineEditWidget::spellCheckerCorrected( const QString &old, const QString &corr, unsigned int pos )
+{
+    if( old!= corr )
+    {
+        setSelection ( pos, old.length() );
+        insert( corr );
+        setSelection ( pos, corr.length() );
+    }
+}
+
+void LineEditWidget::spellCheckerFinished()
+{
+}
+
+void LineEditWidget::slotSpellCheckReady( KSpell *s )
+{
+    s->check( text() );
+    connect( s, SIGNAL( done( const QString & ) ), this, SLOT( slotSpellCheckDone( const QString & ) ) );
+}
+
+void LineEditWidget::slotSpellCheckDone( const QString &s )
+{
+    if( s != text() )
+        setText( s );
+}
+
+
 QPopupMenu *LineEditWidget::createPopupMenu()
 {
     QPopupMenu *popup = KLineEdit::createPopupMenu();
-    if ( !popup )
+
+    if ( !popup ) {
         return 0L;
+    }
+
     connect( popup, SIGNAL( activated( int ) ),
              this, SLOT( extendedMenuActivated( int ) ) );
+
+    if (m_input->autoComplete()) {
+        popup->insertSeparator();
+        int id = popup->insertItem( i18n("Clear &History"), ClearHistory );
+        if (!completionObject()) {
+            popup->setItemEnabled( id, false );
+        }
+    }
+
+    if (echoMode() == QLineEdit::Normal &&
+        !isReadOnly()) {
+        popup->insertSeparator();
+
+        m_spellAction->plug(popup);
+        m_spellAction->setEnabled( !text().isEmpty() );
+    }
+
     return popup;
 }
 
+
 void LineEditWidget::extendedMenuActivated( int id)
 {
     switch ( id )
     {
     case ClearHistory:
-        clearMenuHistory();
-        break;
+        m_view->clearCompletionHistory(m_input->name().string());
+        completionObject()->clear();
     default:
         break;
     }
 }
 
-void LineEditWidget::clearMenuHistory()
-{
-    emit clearCompletionHistory();
-}
-
-
 bool LineEditWidget::event( QEvent *e )
 {
+    if (KLineEdit::event(e))
+	return true;
+
     if ( e->type() == QEvent::AccelAvailable && isReadOnly() ) {
         QKeyEvent* ke = (QKeyEvent*) e;
         if ( ke->state() & ControlButton ) {
@@ -383,24 +411,26 @@ bool LineEditWidget::event( QEvent *e )
             }
         }
     }
-    else if ( e->type() == QEvent::MouseButtonPress )
-        emit pressed();
-    else if ( e->type() == QEvent::MouseButtonRelease )
-        emit released();
-    return KLineEdit::event( e );
+    return false;
 }
 
+void LineEditWidget::mouseMoveEvent(QMouseEvent *e)
+{
+    // hack to prevent Qt from calling setCursor on the widget
+    setDragEnabled(false);
+    KLineEdit::mouseMoveEvent(e);
+    setDragEnabled(true);
+}
+
+
 // -----------------------------------------------------------------------------
 
 RenderLineEdit::RenderLineEdit(HTMLInputElementImpl *element)
     : RenderFormElement(element)
 {
-    LineEditWidget *edit = new LineEditWidget(view()->viewport());
+    LineEditWidget *edit = new LineEditWidget(element, view(), view()->viewport());
     connect(edit,SIGNAL(returnPressed()), this, SLOT(slotReturnPressed()));
     connect(edit,SIGNAL(textChanged(const QString &)),this,SLOT(slotTextChanged(const QString &)));
-    connect(edit,SIGNAL(pressed()), this, SLOT(slotPressed()));
-    connect(edit,SIGNAL(released()), this, SLOT(slotReleased()));
-    connect(edit, SIGNAL(clearCompletionHistory()), this, SLOT( slotClearCompletionHistory()));
     if(element->inputType() == HTMLInputElementImpl::PASSWORD)
         edit->setEchoMode( QLineEdit::Password );
 
@@ -409,26 +439,44 @@ RenderLineEdit::RenderLineEdit(HTMLInputElementImpl *element)
         if (completions.count()) {
             edit->completionObject()->setItems(completions);
             edit->setContextMenuEnabled(true);
+            edit->completionBox()->setTabHandling( false );
         }
     }
 
     setQWidget(edit);
 }
 
-void RenderLineEdit::slotClearCompletionHistory()
+void RenderLineEdit::setStyle(RenderStyle* _style)
 {
-    if ( element()->autoComplete() ) {
-        view()->clearCompletionHistory(element()->name().string());
-        static_cast<LineEditWidget*>(m_widget)->completionObject()->clear();
-    }
+    RenderFormElement::setStyle( _style );
+
+    if ( style()->textAlign() == LEFT )
+        widget()->setAlignment( Qt::AlignLeft );
+    else if ( style()->textAlign() == RIGHT )
+        widget()->setAlignment( Qt::AlignRight );
+    else if ( style()->textAlign() == CENTER )
+        widget()->setAlignment( Qt::AlignHCenter );
+    else
+        widget()->setAlignment(style()->direction() == RTL
+                               ? Qt::AlignRight : Qt::AlignLeft);
 }
 
+void RenderLineEdit::highLightWord( unsigned int length, unsigned int pos )
+{
+    LineEditWidget* w = static_cast<LineEditWidget*>(m_widget);
+    if ( w )
+        w->highLightWord( length, pos );
+}
+
+
 void RenderLineEdit::slotReturnPressed()
 {
     // don't submit the form when return was pressed in a completion-popup
     KCompletionBox *box = widget()->completionBox(false);
-    if ( box && box->isVisible() && box->currentItem() != -1 )
-	return;
+    if ( box && box->isVisible() && box->currentItem() != -1 ) {
+      box->hide();
+      return;
+    }
 
     // Emit onChange if necessary
     // Works but might not be enough, dirk said he had another solution at
@@ -460,7 +508,7 @@ void RenderLineEdit::calcMinMaxWidth()
     int h = fm.lineSpacing();
     int w = fm.width( 'x' ) * (size > 0 ? size+1 : 17); // "some"
     s = QSize(w + 2 + 2*widget()->frameWidth(),
-              QMAX(h, 14) + 2 + 2*widget()->frameWidth())
+              kMax(h, 14) + 2 + 2*widget()->frameWidth())
         .expandedTo(QApplication::globalStrut());
 
     setIntrinsicWidth( s.width() );
@@ -474,11 +522,17 @@ void RenderLineEdit::updateFromElement()
     int ml = element()->maxLength();
     if ( ml < 0 || ml > 1024 )
         ml = 1024;
-    if ( widget()->maxLength() != ml ) {
-        widget()->blockSignals( true );
-        widget()->setMaxLength( ml );
-        widget()->blockSignals( false );
-    }
+
+     if ( widget()->maxLength() != ml )  {
+#if QT_VERSION == 0x030201 || QT_VERSION == 0x030202
+         // workaround QT 3.2.1/2 bug
+         widget()->blockSignals(true);
+#endif
+         widget()->setMaxLength( ml );
+#if QT_VERSION == 0x030201 || QT_VERSION == 0x030202
+         widget()->blockSignals(false);
+#endif
+     }
 
     if (element()->value().string() != widget()->text()) {
         widget()->blockSignals(true);
@@ -509,7 +563,7 @@ void RenderLineEdit::select()
 // ---------------------------------------------------------------------------
 
 RenderFieldset::RenderFieldset(HTMLGenericFormElementImpl *element)
-    : RenderFlow(element)
+    : RenderBlock(element)
 {
 }
 
@@ -522,7 +576,7 @@ bool RenderFieldset::findLegend( int &lx, int &ly, int &lw, int &lh)
             r = r->firstChild();
     else
         return false;
-    if(!r->firstChild() || r->isSpecial())
+    if(!r->firstChild() || r->isFloatingOrPositioned())
         return false;
     ly = r->yPos();
     minx = r->width();
@@ -544,7 +598,7 @@ bool RenderFieldset::findLegend( int &lx, int &ly, int &lw, int &lh)
             }
             r = next;
         }
-        if(r->isSpecial())
+        if(r->isFloatingOrPositioned())
             continue;
         curx += r->xPos();
         if(r->width() && curx<minx)
@@ -563,8 +617,7 @@ bool RenderFieldset::findLegend( int &lx, int &ly, int &lw, int &lh)
         return !!maxw;
 }
 
-void RenderFieldset::paintBoxDecorations(QPainter *p,int, int _y,
-                                       int, int _h, int _tx, int _ty)
+void RenderFieldset::paintBoxDecorations(PaintInfo& pI, int _tx, int _ty)
 {
     //kdDebug( 6040 ) << renderName() << "::paintDecorations()" << endl;
 
@@ -580,17 +633,17 @@ void RenderFieldset::paintBoxDecorations(QPainter *p,int, int _y,
     }
     _ty -= borderTopExtra();
 
-    int my = QMAX(_ty,_y);
-    int end = QMIN( _y + _h,  _ty + h );
+    int my = kMax(_ty,pI.r.y());
+    int end = kMin( pI.r.y() + pI.r.height(),  _ty + h );
     int mh = end - my;
 
-    paintBackground(p, style()->backgroundColor(), style()->backgroundImage(), my, mh, _tx, _ty, w, h);
+    paintBackground(pI.p, style()->backgroundColor(), style()->backgroundImage(), my, mh, _tx, _ty, w, h);
 
     if ( style()->hasBorder() ) {
 	if ( legend )
-	    paintBorderMinusLegend(p, _tx, _ty, w, h, style(), lx, lw);
+	    paintBorderMinusLegend(pI.p, _tx, _ty, w, h, style(), lx, lw);
 	else
-	    paintBorder(p, _tx, _ty, w, h, style());
+	    paintBorder(pI.p, _tx, _ty, w, h, style());
     }
 }
 
@@ -667,27 +720,17 @@ void RenderFieldset::paintBorderMinusLegend(QPainter *p, int _tx, int _ty, int w
 RenderFileButton::RenderFileButton(HTMLInputElementImpl *element)
     : RenderFormElement(element)
 {
-    // this sucks, it creates a grey background
-    QHBox *w = new QHBox(view()->viewport());
+    KURLRequester* w = new KURLRequester( view()->viewport(), "__khtml" );
 
-    m_edit = new LineEditWidget(w);
-
-    connect(m_edit, SIGNAL(returnPressed()), this, SLOT(slotReturnPressed()));
-    connect(m_edit, SIGNAL(textChanged(const QString &)),this,SLOT(slotTextChanged(const QString &)));
-
-    m_button = new QPushButton(i18n("Browse..."), w);
-    m_button->setFocusPolicy(QWidget::ClickFocus);
-    connect(m_button,SIGNAL(clicked()), this, SLOT(slotClicked()));
-    connect(m_button, SIGNAL(pressed()), this, SLOT(slotPressed()));
-    connect(m_button, SIGNAL(released()), this, SLOT(slotReleased()));
-
-    w->setStretchFactor(m_edit, 2);
-    w->setFocusProxy(m_edit);
+    connect(w->lineEdit(), SIGNAL(returnPressed()), this, SLOT(slotReturnPressed()));
+    connect(w->lineEdit(), SIGNAL(textChanged(const QString &)),this,SLOT(slotTextChanged(const QString &)));
 
     setQWidget(w);
     m_haveFocus = false;
 }
 
+
+
 void RenderFileButton::calcMinMaxWidth()
 {
     KHTMLAssert( !minMaxKnown() );
@@ -697,10 +740,12 @@ void RenderFileButton::calcMinMaxWidth()
 
     int h = fm.lineSpacing();
     int w = fm.width( 'x' ) * (size > 0 ? size : 17); // "some"
-    QSize s = m_edit->style().sizeFromContents(QStyle::CT_LineEdit, m_edit,
-          QSize(w + 2 + 2*m_edit->frameWidth(), kMax(h, 14) + 2 + 2*m_edit->frameWidth()))
+    KLineEdit* edit = static_cast<KURLRequester*>( m_widget )->lineEdit();
+    QSize s = edit->style().sizeFromContents(QStyle::CT_LineEdit,
+                                             edit,
+          QSize(w + 2 + 2*edit->frameWidth(), kMax(h, 14) + 2 + 2*edit->frameWidth()))
         .expandedTo(QApplication::globalStrut());
-    QSize bs = m_button->sizeHint();
+    QSize bs = static_cast<KURLRequester*>( m_widget )->sizeHint();
 
     setIntrinsicWidth( s.width() + bs.width() );
     setIntrinsicHeight( kMax(s.height(), bs.height()) );
@@ -710,49 +755,43 @@ void RenderFileButton::calcMinMaxWidth()
 
 void RenderFileButton::handleFocusOut()
 {
-    if ( m_edit && m_edit->edited() ) {
+    if ( widget()->lineEdit() && widget()->lineEdit()->edited() ) {
         element()->onChange();
-        m_edit->setEdited( false );
-    }
-}
-
-void RenderFileButton::slotClicked()
-{
-    QString file_name = KFileDialog::getOpenFileName(QString::null, QString::null, 0, i18n("Browse"));
-    if (!file_name.isNull()) {
-        element()->m_value = DOMString(file_name);
-        m_edit->setText(file_name);
+        widget()->lineEdit()->setEdited( false );
     }
 }
 
 void RenderFileButton::updateFromElement()
 {
-    m_edit->blockSignals(true);
-    m_edit->setText(element()->value().string());
-    m_edit->blockSignals(false);
+    KLineEdit* edit = widget()->lineEdit();
+    edit->blockSignals(true);
+    edit->setText(element()->value().string());
+    edit->blockSignals(false);
     int ml = element()->maxLength();
     if ( ml < 0 || ml > 1024 )
         ml = 1024;
-    m_edit->setMaxLength( ml );
-    m_edit->setEdited( false );
+    edit->setMaxLength( ml );
+    edit->setEdited( false );
 
     RenderFormElement::updateFromElement();
 }
 
 void RenderFileButton::slotReturnPressed()
 {
+    handleFocusOut();
+
     if (element()->form())
 	element()->form()->submitFromKeyboard();
 }
 
-void RenderFileButton::slotTextChanged(const QString &string)
+void RenderFileButton::slotTextChanged(const QString &/*string*/)
 {
-   element()->m_value = DOMString(string);
+   element()->m_value = KURL( widget()->url() ).prettyURL( 0, KURL::StripFileProtocol );
 }
 
 void RenderFileButton::select()
 {
-    m_edit->selectAll();
+    widget()->lineEdit()->selectAll();
 }
 
 // -------------------------------------------------------------------------
@@ -766,7 +805,7 @@ RenderLabel::RenderLabel(HTMLGenericFormElementImpl *element)
 // -------------------------------------------------------------------------
 
 RenderLegend::RenderLegend(HTMLGenericFormElementImpl *element)
-    : RenderFlow(element)
+    : RenderBlock(element)
 {
     setInline(false);
 }
@@ -774,7 +813,7 @@ RenderLegend::RenderLegend(HTMLGenericFormElementImpl *element)
 // -------------------------------------------------------------------------------
 
 ComboBoxWidget::ComboBoxWidget(QWidget *parent)
-    : KComboBox(false, parent)
+    : KComboBox(false, parent, "__khtml")
 {
     setAutoMask(true);
     if (listBox()) listBox()->installEventFilter(this);
@@ -783,6 +822,8 @@ ComboBoxWidget::ComboBoxWidget(QWidget *parent)
 
 bool ComboBoxWidget::event(QEvent *e)
 {
+    if (KComboBox::event(e))
+	return true;
     if (e->type()==QEvent::KeyPress)
     {
 	QKeyEvent *ke = static_cast<QKeyEvent *>(e);
@@ -794,10 +835,10 @@ bool ComboBoxWidget::event(QEvent *e)
 	    ke->accept();
 	    return true;
 	default:
-	    return KComboBox::event(e);
+	    return false;
 	}
     }
-    return KComboBox::event(e);
+    return false;
 }
 
 bool ComboBoxWidget::eventFilter(QObject *dest, QEvent *e)
@@ -898,9 +939,11 @@ void RenderSelect::updateFromElement()
                         ->insertItem(item, listIndex);
                     item->setSelectable(false);
                 }
-                else
+                else {
                     static_cast<KComboBox*>(m_widget)
                         ->insertItem(QString(text.implementation()->s, text.implementation()->l), listIndex);
+		    static_cast<KComboBox*>(m_widget)->listBox()->item(listIndex)->setSelectable(false);
+		}
             }
             else if (listItems[listIndex]->id() == ID_OPTION) {
                 HTMLOptionElementImpl* optElem = static_cast<HTMLOptionElementImpl*>(listItems[listIndex]);
@@ -914,15 +957,26 @@ void RenderSelect::updateFromElement()
                     text = QString::fromLatin1("    ")+text;
                 }
 
-                if(m_useListBox)
-                    static_cast<KListBox*>(m_widget)->insertItem(text, listIndex);
-                else
+                if(m_useListBox) {
+                    KListBox *l = static_cast<KListBox*>(m_widget);
+                    l->insertItem(text, listIndex);
+                    DOMString disabled = optElem->getAttribute(ATTR_DISABLED);
+                    if (!disabled.isNull() && l->item( listIndex )) {
+                        l->item( listIndex )->setSelectable( false );
+                    }
+                }  else
                     static_cast<KComboBox*>(m_widget)->insertItem(text, listIndex);
             }
             else
                 KHTMLAssert(false);
             m_selectionChanged = true;
         }
+
+        // QComboBox caches the size hint unless you call setFont (ref: TT docu)
+        if(!m_useListBox) {
+            KComboBox *that = static_cast<KComboBox*>(m_widget);
+            that->setFont( that->font() );
+        }
         setMinMaxKnown(false);
         setLayouted(false);
         m_optionsChanged = false;
@@ -974,19 +1028,19 @@ void RenderSelect::layout( )
         int width = 0;
         int height = 0;
         while(p) {
-            width = QMAX(width, p->width(p->listBox()));
-            height = QMAX(height, p->height(p->listBox()));
+            width = kMax(width, p->width(p->listBox()));
+            height = kMax(height, p->height(p->listBox()));
             p = p->next();
         }
 
         int size = m_size;
         // check if multiple and size was not given or invalid
-        // Internet Exploder sets size to QMIN(number of elements, 4)
+        // Internet Exploder sets size to kMin(number of elements, 4)
         // Netscape seems to simply set it to "number of elements"
-        // the average of that is IMHO QMIN(number of elements, 10)
+        // the average of that is IMHO kMin(number of elements, 10)
         // so I did that ;-)
         if(size < 1)
-            size = QMIN(static_cast<KListBox*>(m_widget)->count(), 10);
+            size = kMin(static_cast<KListBox*>(m_widget)->count(), 10u);
 
         width += 2*w->frameWidth() + w->verticalScrollBar()->sizeHint().width();
         height = size*height + 2*w->frameWidth();
@@ -1068,7 +1122,7 @@ void RenderSelect::slotSelected(int index) // emitted by the combobox only
             // shouldn't emit onChange. Hence this bool, the if above doesn't do it.
             if ( changed )
             {
-                ref();
+		ref();
                 element()->onChange();
                 deref();
             }
@@ -1108,7 +1162,7 @@ KListBox* RenderSelect::createListBox()
     // ### looks broken
     //lb->setAutoMask(true);
     connect( lb, SIGNAL( selectionChanged() ), this, SLOT( slotSelectionChanged() ) );
-    connect( lb, SIGNAL( clicked( QListBoxItem * ) ), this, SLOT( slotClicked() ) );
+//     connect( lb, SIGNAL( clicked( QListBoxItem * ) ), this, SLOT( slotClicked() ) );
     m_ignoreSelectEvents = false;
     lb->setMouseTracking(true);
 
@@ -1158,8 +1212,10 @@ void RenderSelect::updateSelection()
 // -------------------------------------------------------------------------
 
 TextAreaWidget::TextAreaWidget(int wrap, QWidget* parent)
-    : KTextEdit(parent)
+    : KTextEdit(parent), m_findDlg(0), m_find(0), m_repDlg(0), m_replace(0)
 {
+    setCheckSpellingEnabled( true );
+
     if(wrap != DOM::HTMLTextAreaElementImpl::ta_NoWrap) {
         setWordWrap(QTextEdit::WidgetWidth);
         setHScrollBarMode( AlwaysOff );
@@ -1174,8 +1230,297 @@ TextAreaWidget::TextAreaWidget(int wrap, QWidget* parent)
     setTextFormat(QTextEdit::PlainText);
     setAutoMask(true);
     setMouseTracking(true);
+
+    KActionCollection *ac = new KActionCollection(this);
+    m_findAction = KStdAction::find( this, SLOT( slotFind() ), ac );
+    m_findNextAction = KStdAction::findNext( this, SLOT( slotFindNext() ), ac );
+    m_replaceAction = KStdAction::replace( this, SLOT( slotReplace() ), ac );
 }
 
+
+TextAreaWidget::~TextAreaWidget()
+{
+    delete m_replace;
+    m_replace = 0L;
+    delete m_find;
+    m_find = 0L;
+    delete m_repDlg;
+    m_repDlg = 0L;
+    delete m_findDlg;
+    m_findDlg = 0L;
+}
+
+
+QPopupMenu *TextAreaWidget::createPopupMenu(const QPoint& pos)
+{
+    QPopupMenu *popup = KTextEdit::createPopupMenu(pos);
+
+    if ( !popup ) {
+        return 0L;
+    }
+
+    if (!isReadOnly()) {
+        popup->insertSeparator();
+
+        m_findAction->plug(popup);
+        m_findAction->setEnabled( !text().isEmpty() );
+
+        m_findNextAction->plug(popup);
+        m_findNextAction->setEnabled( m_find != 0 );
+
+        m_replaceAction->plug(popup);
+        m_replaceAction->setEnabled( !text().isEmpty() );
+    }
+
+    return popup;
+}
+
+
+void TextAreaWidget::slotFindHighlight(const QString& text, int matchingIndex, int matchingLength)
+{
+    Q_UNUSED(text)
+    //kdDebug() << "Highlight: [" << text << "] mi:" << matchingIndex << " ml:" << matchingLength << endl;
+    if (sender() == m_replace) {
+        setSelection(m_repPara, matchingIndex, m_repPara, matchingIndex + matchingLength);
+        setCursorPosition(m_repPara, matchingIndex);
+    } else {
+        setSelection(m_findPara, matchingIndex, m_findPara, matchingIndex + matchingLength);
+        setCursorPosition(m_findPara, matchingIndex);
+    }
+    ensureCursorVisible();
+}
+
+
+void TextAreaWidget::slotReplaceText(const QString &text, int replacementIndex, int /*replacedLength*/, int matchedLength) {
+    Q_UNUSED(text)
+    //kdDebug() << "Replace: [" << text << "] ri:" << replacementIndex << " rl:" << replacedLength << " ml:" << matchedLength << endl;
+    setSelection(m_repPara, replacementIndex, m_repPara, replacementIndex + matchedLength);
+    removeSelectedText();
+    insertAt(m_repDlg->replacement(), m_repPara, replacementIndex);
+    if (m_replace->options() & KReplaceDialog::PromptOnReplace) {
+        ensureCursorVisible();
+    }
+}
+
+
+void TextAreaWidget::slotDoReplace()
+{
+    if (!m_repDlg) {
+        // Should really assert()
+        return;
+    }
+
+    delete m_replace;
+    m_replace = new KReplace(m_repDlg->pattern(), m_repDlg->replacement(), m_repDlg->options(), this);
+    if (m_replace->options() & KFindDialog::FromCursor) {
+        getCursorPosition(&m_repPara, &m_repIndex);
+    } else if (m_replace->options() & KFindDialog::FindBackwards) {
+        m_repPara = paragraphs() - 1;
+        m_repIndex = paragraphLength(m_repPara) - 1;
+    } else {
+        m_repPara = 0;
+        m_repIndex = 0;
+    }
+
+    // Connect highlight signal to code which handles highlighting
+    // of found text.
+    connect(m_replace, SIGNAL(highlight(const QString &, int, int)),
+            this, SLOT(slotFindHighlight(const QString &, int, int)));
+    connect(m_replace, SIGNAL(findNext()), this, SLOT(slotReplaceNext()));
+    connect(m_replace, SIGNAL(replace(const QString &, int, int, int)),
+            this, SLOT(slotReplaceText(const QString &, int, int, int)));
+
+    m_repDlg->close();
+    slotReplaceNext();
+}
+
+
+void TextAreaWidget::slotReplaceNext()
+{
+    if (!m_replace) {
+        // assert?
+        return;
+    }
+
+    if (!(m_replace->options() & KReplaceDialog::PromptOnReplace)) {
+        viewport()->setUpdatesEnabled(false);
+    }
+
+    KFind::Result res = KFind::NoMatch;
+    while (res == KFind::NoMatch) {
+        // If we're done.....
+        if (m_replace->options() & KFindDialog::FindBackwards) {
+            if (m_repIndex == 0 && m_repPara == 0) {
+                break;
+            }
+        } else {
+            if (m_repPara == paragraphs() - 1 &&
+                m_repIndex == paragraphLength(m_repPara) - 1) {
+                break;
+            }
+        }
+
+        if (m_replace->needData()) {
+            m_replace->setData(text(m_repPara), m_repIndex);
+        }
+
+        res = m_replace->replace();
+
+        if (res == KFind::NoMatch) {
+            if (m_replace->options() & KFindDialog::FindBackwards) {
+                if (m_repPara == 0) {
+                    m_repIndex = 0;
+                } else {
+                    m_repPara--;
+                    m_repIndex = paragraphLength(m_repPara) - 1;
+                }
+            } else {
+                if (m_repPara == paragraphs() - 1) {
+                    m_repIndex = paragraphLength(m_repPara) - 1;
+                } else {
+                    m_repPara++;
+                    m_repIndex = 0;
+                }
+            }
+        }
+    }
+
+    if (!(m_replace->options() & KReplaceDialog::PromptOnReplace)) {
+        viewport()->setUpdatesEnabled(true);
+        repaintChanged();
+    }
+
+    if (res == KFind::NoMatch) { // at end
+        m_replace->displayFinalDialog();
+        delete m_replace;
+        m_replace = 0;
+        ensureCursorVisible();
+        //or           if ( m_replace->shouldRestart() ) { reinit (w/o FromCursor) and call slotReplaceNext(); }
+    } else {
+        //m_replace->closeReplaceNextDialog();
+    }
+}
+
+
+void TextAreaWidget::slotDoFind()
+{
+    if (!m_findDlg) {
+        // Should really assert()
+        return;
+    }
+
+    delete m_find;
+    m_find = new KFind(m_findDlg->pattern(), m_findDlg->options(), this);
+    if (m_find->options() & KFindDialog::FromCursor) {
+        getCursorPosition(&m_findPara, &m_findIndex);
+    } else if (m_find->options() & KFindDialog::FindBackwards) {
+        m_findPara = paragraphs() - 1;
+        m_findIndex = paragraphLength(m_findPara) - 1;
+    } else {
+        m_findPara = 0;
+        m_findIndex = 0;
+    }
+
+    // Connect highlight signal to code which handles highlighting
+    // of found text.
+    connect(m_find, SIGNAL(highlight(const QString &, int, int)),
+            this, SLOT(slotFindHighlight(const QString &, int, int)));
+    connect(m_find, SIGNAL(findNext()), this, SLOT(slotFindNext()));
+
+    m_findDlg->close();
+    m_find->closeFindNextDialog();
+    slotFindNext();
+}
+
+
+void TextAreaWidget::slotFindNext()
+{
+    if (!m_find) {
+        // assert?
+        return;
+    }
+
+    KFind::Result res = KFind::NoMatch;
+    while (res == KFind::NoMatch) {
+        // If we're done.....
+        if (m_find->options() & KFindDialog::FindBackwards) {
+            if (m_findIndex == 0 && m_findPara == 0) {
+                break;
+            }
+        } else {
+            if (m_findPara == paragraphs() - 1 &&
+                m_findIndex == paragraphLength(m_findPara) - 1) {
+                break;
+            }
+        }
+
+        if (m_find->needData()) {
+            m_find->setData(text(m_findPara), m_findIndex);
+        }
+
+        res = m_find->find();
+
+        if (res == KFind::NoMatch) {
+            if (m_find->options() & KFindDialog::FindBackwards) {
+                if (m_findPara == 0) {
+                    m_findIndex = 0;
+                } else {
+                    m_findPara--;
+                    m_findIndex = paragraphLength(m_findPara) - 1;
+                }
+            } else {
+                if (m_findPara == paragraphs() - 1) {
+                    m_findIndex = paragraphLength(m_findPara) - 1;
+                } else {
+                    m_findPara++;
+                    m_findIndex = 0;
+                }
+            }
+        }
+    }
+
+    if (res == KFind::NoMatch) { // at end
+        m_find->displayFinalDialog();
+        delete m_find;
+        m_find = 0;
+        //or           if ( m_find->shouldRestart() ) { reinit (w/o FromCursor) and call slotFindNext(); }
+    } else {
+        //m_find->closeFindNextDialog();
+    }
+}
+
+
+void TextAreaWidget::slotFind()
+{
+    if( text().isEmpty() )  // saves having to track the text changes
+        return;
+
+    if ( m_findDlg ) {
+      KWin::activateWindow( m_findDlg->winId() );
+    } else {
+      m_findDlg = new KFindDialog(false, this, "KHTML Text Area Find Dialog");
+      connect( m_findDlg, SIGNAL(okClicked()), this, SLOT(slotDoFind()) );
+    }
+    m_findDlg->show();
+}
+
+
+void TextAreaWidget::slotReplace()
+{
+    if( text().isEmpty() )  // saves having to track the text changes
+        return;
+
+    if ( m_repDlg ) {
+      KWin::activateWindow( m_repDlg->winId() );
+    } else {
+      m_repDlg = new KReplaceDialog(this, "KHTMLText Area Replace Dialog", 0,
+                                    QStringList(), QStringList(), false);
+      connect( m_repDlg, SIGNAL(okClicked()), this, SLOT(slotDoReplace()) );
+    }
+    m_repDlg->show();
+}
+
+
 bool TextAreaWidget::event( QEvent *e )
 {
     if ( e->type() == QEvent::AccelAvailable && isReadOnly() ) {
@@ -1202,6 +1547,8 @@ bool TextAreaWidget::event( QEvent *e )
 RenderTextArea::RenderTextArea(HTMLTextAreaElementImpl *element)
     : RenderFormElement(element)
 {
+    scrollbarsStyled = false;
+
     TextAreaWidget *edit = new TextAreaWidget(element->wrap(), view());
     setQWidget(edit);
 
@@ -1233,9 +1580,9 @@ void RenderTextArea::calcMinMaxWidth()
     TextAreaWidget* w = static_cast<TextAreaWidget*>(m_widget);
     const QFontMetrics &m = style()->fontMetrics();
     w->setTabStopWidth(8 * m.width(" "));
-    QSize size( QMAX(element()->cols(), 1)*m.width('x') + w->frameWidth() +
+    QSize size( kMax(element()->cols(), 1L)*m.width('x') + w->frameWidth() +
                 w->verticalScrollBar()->sizeHint().width(),
-                QMAX(element()->rows(), 1)*m.lineSpacing() + w->frameWidth()*4 +
+                kMax(element()->rows(), 1L)*m.lineSpacing() + w->frameWidth()*4 +
                 (w->wordWrap() == QTextEdit::NoWrap ?
                  w->horizontalScrollBar()->sizeHint().height() : 0)
         );
@@ -1246,6 +1593,31 @@ void RenderTextArea::calcMinMaxWidth()
     RenderFormElement::calcMinMaxWidth();
 }
 
+void RenderTextArea::setStyle(RenderStyle* _style)
+{
+    RenderFormElement::setStyle(_style);
+
+    widget()->setAlignment( _style->direction() == RTL ?
+                            Qt::AlignRight : Qt::AlignLeft );
+
+    scrollbarsStyled = false;
+}
+
+void RenderTextArea::layout()
+{
+    KHTMLAssert( !layouted() );
+
+    RenderFormElement::layout();
+
+    TextAreaWidget* w = static_cast<TextAreaWidget*>(m_widget);
+
+    if (!scrollbarsStyled) {
+        w->horizontalScrollBar()->setPalette(style()->palette());
+        w->verticalScrollBar()->setPalette(style()->palette());
+        scrollbarsStyled=true;
+    }
+}
+
 void RenderTextArea::updateFromElement()
 {
     TextAreaWidget* w = static_cast<TextAreaWidget*>(m_widget);
@@ -1301,6 +1673,15 @@ QString RenderTextArea::text()
     return txt;
 }
 
+
+void RenderTextArea::highLightWord( unsigned int length, unsigned int pos )
+{
+    TextAreaWidget* w = static_cast<TextAreaWidget*>(m_widget);
+    if ( w )
+        w->highLightWord( length, pos );
+}
+
+
 void RenderTextArea::slotTextChanged()
 {
     element()->m_dirtyvalue = true;
