@@ -1,9 +1,9 @@
 /*
  * This file is part of the DOM implementation for KDE.
  *
- * Copyright (C) 1999 Lars Knoll (knoll@kde.org)
+ * Copyright (C) 1999-2003 Lars Knoll (knoll@kde.org)
  *           (C) 1999 Antti Koivisto (koivisto@kde.org)
- *           (C) 2000 Dirk Mueller (mueller@kde.org)
+ *           (C) 2000-2003 Dirk Mueller (mueller@kde.org)
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -20,7 +20,7 @@
  * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
  *
- * $Id: render_form.h,v 1.74.2.5 2003/05/18 12:34:40 mueller Exp $
+ * $Id: render_form.h,v 1.107.2.2 2004/02/29 18:28:57 mueller Exp $
  */
 #ifndef RENDER_FORM_H
 #define RENDER_FORM_H
@@ -36,6 +36,7 @@ class QLineEdit;
 class QListboxItem;
 
 #include <ktextedit.h>
+#include <kurlrequester.h>
 #include <klineedit.h>
 #include <qcheckbox.h>
 #include <qradiobutton.h>
@@ -46,6 +47,13 @@ class QListboxItem;
 #include "dom/dom_misc.h"
 
 class KHTMLPartBrowserExtension;
+class KSpell;
+class KFindDialog;
+class KReplaceDialog;
+class KFind;
+class KReplace;
+class KAction;
+class KURLRequester;
 
 namespace DOM {
     class HTMLFormElementImpl;
@@ -63,16 +71,24 @@ class DocLoader;
 
 class RenderFormElement : public khtml::RenderWidget
 {
-    Q_OBJECT
 public:
     RenderFormElement(DOM::HTMLGenericFormElementImpl* node);
     virtual ~RenderFormElement();
 
     virtual const char *renderName() const { return "RenderForm"; }
 
-    virtual bool isRendered() const  { return true; }
     virtual bool isFormElement() const { return true; }
 
+    // form elements never have border/padding
+    virtual int borderTop() const { return 0; }
+    virtual int borderBottom() const { return 0; }
+    virtual int borderLeft() const { return 0; }
+    virtual int borderRight() const { return 0; }
+    virtual int paddingTop() const { return 0; }
+    virtual int paddingBottom() const { return 0; }
+    virtual int paddingLeft() const { return 0; }
+    virtual int paddingRight() const { return 0; }
+
     virtual void updateFromElement();
 
     virtual void layout();
@@ -81,18 +97,12 @@ public:
     DOM::HTMLGenericFormElementImpl *element() const
     { return static_cast<DOM::HTMLGenericFormElementImpl*>(RenderObject::element()); }
 
-public slots:
-    virtual void slotClicked();
-    void slotPressed();
-    void slotReleased();
-
 protected:
     virtual bool isRenderButton() const { return false; }
     virtual bool isEditable() const { return false; }
 
     QPoint m_mousePos;
     int m_state;
-    bool m_isDoubleClick;
 };
 
 // -------------------------------------------------------------------------
@@ -127,6 +137,8 @@ public:
     virtual void updateFromElement();
     virtual void calcMinMaxWidth();
 
+    virtual bool handleEvent(const DOM::EventImpl&) { return false; }
+
     QCheckBox *widget() const { return static_cast<QCheckBox*>(m_widget); }
 
 public slots:
@@ -137,7 +149,6 @@ public slots:
 
 class RenderRadioButton : public RenderButton
 {
-    Q_OBJECT
 public:
     RenderRadioButton(DOM::HTMLInputElementImpl* node);
 
@@ -146,10 +157,9 @@ public:
     virtual void calcMinMaxWidth();
     virtual void updateFromElement();
 
-    QRadioButton *widget() const { return static_cast<QRadioButton*>(m_widget); }
+    virtual bool handleEvent(const DOM::EventImpl&) { return false; }
 
-public slots:
-    void slotClicked();
+    QRadioButton *widget() const { return static_cast<QRadioButton*>(m_widget); }
 };
 
 // -------------------------------------------------------------------------
@@ -175,7 +185,8 @@ private:
 class RenderImageButton : public RenderImage
 {
 public:
-    RenderImageButton(DOM::HTMLInputElementImpl *element);
+    RenderImageButton(DOM::HTMLInputElementImpl *element)
+        : RenderImage(element) {}
 
     virtual const char *renderName() const { return "RenderImageButton"; }
 };
@@ -198,7 +209,8 @@ public:
 class RenderPushButton : public RenderSubmitButton
 {
 public:
-    RenderPushButton(DOM::HTMLInputElementImpl *element);
+    RenderPushButton(DOM::HTMLInputElementImpl *element)
+        : RenderSubmitButton(element) {}
 
     virtual QString defaultLabel();
 };
@@ -215,17 +227,18 @@ public:
 
     virtual const char *renderName() const { return "RenderLineEdit"; }
     virtual void updateFromElement();
+    virtual void setStyle(RenderStyle *style);
 
     void select();
 
     KLineEdit *widget() const { return static_cast<KLineEdit*>(m_widget); }
     DOM::HTMLInputElementImpl* element() const
     { return static_cast<DOM::HTMLInputElementImpl*>(RenderObject::element()); }
+    void highLightWord( unsigned int length, unsigned int pos );
 
 public slots:
     void slotReturnPressed();
     void slotTextChanged(const QString &string);
-    void slotClearCompletionHistory();
 protected:
     virtual void handleFocusOut();
 
@@ -239,35 +252,44 @@ class LineEditWidget : public KLineEdit
 {
     Q_OBJECT
 public:
-    LineEditWidget(QWidget *parent);
+    LineEditWidget(DOM::HTMLInputElementImpl* input,
+                   KHTMLView* view, QWidget* parent);
+    ~LineEditWidget();
+    void highLightWord( unsigned int length, unsigned int pos );
 
 protected:
     virtual bool event( QEvent *e );
-    void clearMenuHistory();
+    virtual void mouseMoveEvent(QMouseEvent *e);
     virtual QPopupMenu *createPopupMenu();
-signals:
-    void pressed();
-    void released();
-    void clearCompletionHistory();
 private slots:
     void extendedMenuActivated( int id);
+    void slotCheckSpelling();
+    void slotSpellCheckReady( KSpell *s );
+    void slotSpellCheckDone( const QString &s );
+    void spellCheckerMisspelling( const QString &text, const QStringList &, unsigned int pos);
+    void spellCheckerCorrected( const QString &, const QString &, unsigned int );
+    void spellCheckerFinished();
+
 private:
     enum LineEditMenuID {
         ClearHistory
     };
+    DOM::HTMLInputElementImpl* m_input;
+    KHTMLView* m_view;
+    KSpell *m_spell;
+    KAction *m_spellAction;
 };
 
 // -------------------------------------------------------------------------
 
-class RenderFieldset : public RenderFlow
+class RenderFieldset : public RenderBlock
 {
 public:
     RenderFieldset(DOM::HTMLGenericFormElementImpl *element);
 
     virtual const char *renderName() const { return "RenderFieldSet"; }
 protected:
-    virtual void paintBoxDecorations(QPainter *p,int, int _y,
-                                       int, int _h, int _tx, int _ty);
+    virtual void paintBoxDecorations(PaintInfo& pI, int _tx, int _ty);
     void paintBorderMinusLegend(QPainter *p, int _tx, int _ty, int w,
                                   int h, const RenderStyle *style, int lx, int lw);
     bool findLegend( int &lx, int &ly, int &lw, int &lh);
@@ -286,15 +308,14 @@ public:
     virtual void updateFromElement();
     void select();
 
+    KURLRequester *widget() const { return static_cast<KURLRequester*>(m_widget); }
+
     DOM::HTMLInputElementImpl *element() const
     { return static_cast<DOM::HTMLInputElementImpl*>(RenderObject::element()); }
 
-    KLineEdit* lineEdit() const { return m_edit; }
-
 public slots:
-    virtual void slotClicked();
-    virtual void slotReturnPressed();
-    virtual void slotTextChanged(const QString &string);
+    void slotReturnPressed();
+    void slotTextChanged(const QString &string);
 
 protected:
     virtual void handleFocusOut();
@@ -303,8 +324,6 @@ protected:
 
     bool m_clicked;
     bool m_haveFocus;
-    KLineEdit   *m_edit;
-    QPushButton *m_button;
 };
 
 
@@ -321,7 +340,7 @@ public:
 
 // -------------------------------------------------------------------------
 
-class RenderLegend : public RenderFlow
+class RenderLegend : public RenderBlock
 {
 public:
     RenderLegend(DOM::HTMLGenericFormElementImpl *element);
@@ -382,14 +401,36 @@ protected slots:
 };
 
 // -------------------------------------------------------------------------
-
 class TextAreaWidget : public KTextEdit
 {
+    Q_OBJECT
 public:
     TextAreaWidget(int wrap, QWidget* parent);
+    virtual ~TextAreaWidget();
 
 protected:
     virtual bool event (QEvent *e );
+    virtual QPopupMenu *createPopupMenu(const QPoint& pos);
+    virtual QPopupMenu* createPopupMenu() { return KTextEdit::createPopupMenu(); }
+private slots:
+    void slotFind();
+    void slotDoFind();
+    void slotFindNext();
+    void slotReplace();
+    void slotDoReplace();
+    void slotReplaceNext();
+    void slotReplaceText(const QString&, int, int, int);
+    void slotFindHighlight(const QString&, int, int);
+private:
+    KFindDialog *m_findDlg;
+    KFind *m_find;
+    KReplaceDialog *m_repDlg;
+    KReplace *m_replace;
+    KAction *m_findAction;
+    KAction *m_findNextAction;
+    KAction *m_replaceAction;
+    int m_findIndex, m_findPara;
+    int m_repIndex, m_repPara;
 };
 
 
@@ -404,14 +445,19 @@ public:
 
     virtual const char *renderName() const { return "RenderTextArea"; }
     virtual void calcMinMaxWidth();
+    virtual void layout();
+    virtual void setStyle(RenderStyle *style);
+
     virtual void close ( );
     virtual void updateFromElement();
 
     // don't even think about making this method virtual!
+    TextAreaWidget *widget() const { return static_cast<TextAreaWidget*>(m_widget); }
     DOM::HTMLTextAreaElementImpl* element() const
     { return static_cast<DOM::HTMLTextAreaElementImpl*>(RenderObject::element()); }
 
     QString text();
+    void highLightWord( unsigned int length, unsigned int pos );
 
     void select();
 
@@ -422,6 +468,8 @@ protected:
     virtual void handleFocusOut();
 
     virtual bool isEditable() const { return true; }
+
+    bool scrollbarsStyled;
 };
 
 // -------------------------------------------------------------------------
