@@ -20,12 +20,12 @@
  * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
  *
- * $Id: render_frames.cpp,v 1.144.2.5 2003/03/06 04:17:01 staikos Exp $
+ * $Id: render_frames.cpp,v 1.175.2.3 2004/03/23 00:51:17 mueller Exp $
  */
 //#define DEBUG_LAYOUT
 
 #include "rendering/render_frames.h"
-#include "rendering/render_root.h"
+#include "rendering/render_canvas.h"
 #include "html/html_baseimpl.h"
 #include "html/html_objectimpl.h"
 #include "html/htmltokenizer.h"
@@ -76,23 +76,23 @@ RenderFrameSet::RenderFrameSet( HTMLFrameSetElementImpl *frameSet)
 RenderFrameSet::~RenderFrameSet()
 {
     for (int k = 0; k < 2; ++k) {
-        if (m_gridLayout[k]) delete [] m_gridLayout[k];
-        if (m_gridDelta[k]) delete [] m_gridDelta[k];
+        delete [] m_gridLayout[k];
+        delete [] m_gridDelta[k];
     }
-  if (m_hSplitVar)
-      delete [] m_hSplitVar;
-  if (m_vSplitVar)
-      delete [] m_vSplitVar;
+    delete [] m_hSplitVar;
+    delete [] m_vSplitVar;
 }
 
-bool RenderFrameSet::nodeAtPoint(NodeInfo& info, int _x, int _y, int _tx, int _ty)
+bool RenderFrameSet::nodeAtPoint(NodeInfo& info, int _x, int _y, int _tx, int _ty, HitTestAction hitTestAction,  bool inBox)
 {
-    RenderBox::nodeAtPoint(info, _x, _y, _tx, _ty);
+    RenderBox::nodeAtPoint(info, _x, _y, _tx, _ty, hitTestAction, inBox);
 
     bool inside = m_resizing || canResize(_x, _y);
 
-    if ( inside && element() && !element()->noResize() && !info.readonly())
+    if ( inside && element() && !element()->noResize() && !info.readonly()) {
         info.setInnerNode(element());
+        info.setInnerNonSharedNode(element());
+    }
 
     return inside || m_clientresizing;
 }
@@ -103,9 +103,9 @@ void RenderFrameSet::layout( )
     KHTMLAssert( minMaxKnown() );
 
     if ( !parent()->isFrameSet() ) {
-        KHTMLView* view = root()->view();
-        m_width = view->visibleWidth();
-        m_height = view->visibleHeight();
+        KHTMLView* view = canvas()->view();
+        m_width = view ? view->visibleWidth() : 0;
+        m_height = view ? view->visibleHeight() : 0;
     }
 
 #ifdef DEBUG_LAYOUT
@@ -128,9 +128,9 @@ void RenderFrameSet::layout( )
         m_gridLen[0] = element()->totalRows();
         m_gridLen[1] = element()->totalCols();
         for (int k = 0; k < 2; ++k) {
-            if (m_gridDelta[k]) delete [] m_gridDelta[k];
+            delete [] m_gridDelta[k];
             m_gridDelta[k] = new int[m_gridLen[k]];
-            if (m_gridLayout[k]) delete [] m_gridLayout[k];
+            delete [] m_gridLayout[k];
             m_gridLayout[k] = new int[m_gridLen[k]];
             for (int i = 0; i < m_gridLen[k]; ++i)
                 m_gridDelta[k][i] = 0;
@@ -430,8 +430,9 @@ bool RenderFrameSet::userResize( MouseEventImpl *evt )
     setLayouted(false);
   }
 
-  if (m_resizing || evt->id() == EventImpl::MOUSEUP_EVENT) {
-      QPainter paint( root()->view() );
+  KHTMLView *view = canvas()->view();
+  if ((m_resizing || evt->id() == EventImpl::MOUSEUP_EVENT) && view) {
+      QPainter paint( view );
       paint.setPen( Qt::gray );
       paint.setBrush( Qt::gray );
       paint.setRasterOp( Qt::XorROP );
@@ -484,20 +485,21 @@ bool RenderFrameSet::canResize( int _x, int _y )
     return false;
 }
 
-#ifndef NDEBUG
-void RenderFrameSet::dump(QTextStream *stream, QString ind) const
+#ifdef ENABLE_DUMP
+void RenderFrameSet::dump(QTextStream &stream, const QString &ind) const
 {
-  *stream << " totalrows=" << element()->totalRows();
-  *stream << " totalcols=" << element()->totalCols();
+    RenderBox::dump(stream,ind);
+    stream << " totalrows=" << element()->totalRows();
+    stream << " totalcols=" << element()->totalCols();
 
-  uint i;
-  for (i = 0; i < (uint)element()->totalRows(); i++)
-    *stream << " hSplitvar(" << i << ")=" << m_hSplitVar[i];
+    if ( m_hSplitVar )
+        for (uint i = 0; i < (uint)element()->totalRows(); i++) {
+            stream << " hSplitvar(" << i << ")=" << m_hSplitVar[i];
+        }
 
-  for (i = 0; i < (uint)element()->totalCols(); i++)
-    *stream << " vSplitvar(" << i << ")=" << m_vSplitVar[i];
-
-  RenderBox::dump(stream,ind);
+    if ( m_vSplitVar )
+        for (uint i = 0; i < (uint)element()->totalCols(); i++)
+            stream << " vSplitvar(" << i << ")=" << m_vSplitVar[i];
 }
 #endif
 
@@ -589,15 +591,24 @@ RenderPartObject::RenderPartObject( DOM::HTMLElementImpl* element )
 void RenderPartObject::updateWidget()
 {
   QString url;
-  QString serviceType;
-  QStringList params;
   KHTMLPart *part = m_view->part();
 
   setMinMaxKnown(false);
   setLayouted(false);
 
+  if (element()->id() == ID_IFRAME) {
+
+      HTMLIFrameElementImpl *o = static_cast<HTMLIFrameElementImpl *>(element());
+      url = o->url.string();
+      if( url.isEmpty() || !o->getDocument()->isURLAllowed(url) ) return;
+      part->requestFrame( this, url, o->name.string(), QStringList(), true );
   // ### this should be constant true - move iframe to somewhere else
-  if (element()->id() == ID_OBJECT || element()->id() == ID_EMBED) {
+  } else {
+
+      QString serviceType;
+      QStringList params;
+      HTMLObjectBaseElementImpl * objbase = static_cast<HTMLObjectBaseElementImpl *>(element());
+      url = objbase->url;
 
       for (NodeImpl* child = element()->firstChild(); child; child=child->nextSibling()) {
           if ( child->id() == ID_PARAM ) {
@@ -607,144 +618,114 @@ void RenderPartObject::updateWidget()
               aStr += QString::fromLatin1("=\"");
               aStr += p->value();
               aStr += QString::fromLatin1("\"");
+              QString name_lower = p->name().lower();
+              if (name_lower == QString::fromLatin1("type") && objbase->id() != ID_APPLET) {
+                  objbase->setServiceType(p->value());
+              } else if (url.isEmpty() &&
+                         (name_lower == QString::fromLatin1("src") ||
+                          name_lower == QString::fromLatin1("movie") ||
+                          name_lower == QString::fromLatin1("code"))) {
+                  url = p->value();
+              }
               params.append(aStr);
           }
       }
-      params.append( QString::fromLatin1("__KHTML__PLUGINEMBED=\"YES\"") );
-      params.append( QString::fromLatin1("__KHTML__PLUGINBASEURL=\"%1\"").arg( part->url().url() ) );
-  }
-
-  if(element()->id() == ID_OBJECT) {
-
-      // check for embed child object
-      HTMLObjectElementImpl *o = static_cast<HTMLObjectElementImpl *>(element());
-      HTMLEmbedElementImpl *embed = 0;
-      for (NodeImpl *child = o->firstChild(); child; child = child->nextSibling())
-          if ( child->id() == ID_EMBED ) {
-              embed = static_cast<HTMLEmbedElementImpl *>( child );
-              break;
+      if (element()->id() != ID_OBJECT) {
+          // add all attributes set on the embed object
+          NamedAttrMapImpl* a = objbase->attributes();
+          if (a) {
+              for (unsigned long i = 0; i < a->length(); ++i) {
+                  NodeImpl::Id id = a->idAt(i);
+                  DOMString value = a->valueAt(i);
+              params.append(objbase->getDocument()->getName(NodeImpl::AttributeId, id).string() + "=\"" + value.string() + "\"");
+              }
           }
+      }
+      params.append( QString::fromLatin1("__KHTML__PLUGINEMBED=\"YES\"") );
+      params.append( QString::fromLatin1("__KHTML__PLUGINBASEURL=\"%1\"").arg(element()->getDocument()->baseURL().url()));
 
-      params.append( QString::fromLatin1("__KHTML__CLASSID=\"%1\"").arg( o->classId ) );
-      params.append( QString::fromLatin1("__KHTML__CODEBASE=\"%1\"").arg( o->getAttribute(ATTR_CODEBASE).string() ) );
-      if (!o->getAttribute(ATTR_WIDTH).isEmpty())
-          params.append( QString::fromLatin1("WIDTH=\"%1\"").arg( o->getAttribute(ATTR_WIDTH).string() ) );
-      if (!o->getAttribute(ATTR_HEIGHT).isEmpty())
-          params.append( QString::fromLatin1("HEIGHT=\"%1\"").arg( o->getAttribute(ATTR_HEIGHT).string() ) );
+      if ( element()->id() == ID_EMBED ) {
+
+          serviceType = objbase->serviceType;
+
+      }
+      else { // if(element()->id() == ID_OBJECT || element()->id() == ID_APPLET)
+
+          // check for embed child object
+          HTMLEmbedElementImpl *embed = 0;
+          for (NodeImpl *child = objbase->firstChild(); child; child = child->nextSibling())
+              if ( child->id() == ID_EMBED ) {
+                  embed = static_cast<HTMLEmbedElementImpl *>( child );
+                  break;
+              }
+
+          params.append( QString::fromLatin1("__KHTML__CLASSID=\"%1\"").arg( objbase->classId ) );
+          params.append( QString::fromLatin1("__KHTML__CODEBASE=\"%1\"").arg( objbase->getAttribute(ATTR_CODEBASE).string() ) );
+          if (!objbase->getAttribute(ATTR_WIDTH).isEmpty())
+              params.append( QString::fromLatin1("WIDTH=\"%1\"").arg( objbase->getAttribute(ATTR_WIDTH).string() ) );
+          if (!objbase->getAttribute(ATTR_HEIGHT).isEmpty())
+              params.append( QString::fromLatin1("HEIGHT=\"%1\"").arg( objbase->getAttribute(ATTR_HEIGHT).string() ) );
+
+          if (url.isEmpty() && objbase->classId.startsWith("java:")) {
+              serviceType = "application/x-java-applet";
+              url = objbase->classId.mid(5);
+          }
+          if ( embed ) {
+              // render embed object
+              url = embed->url;
+              serviceType = embed->serviceType;
+          }
+          else {
+              serviceType = objbase->serviceType;
+              if(serviceType.isEmpty() && !objbase->classId.isEmpty()) {
 
-      if ( !embed )
-      {
-          url = o->url;
-          serviceType = o->serviceType;
-          if(serviceType.isEmpty() || serviceType.isNull()) {
-              if(!o->classId.isEmpty()) {
                   // We have a clsid, means this is activex (Niko)
                   serviceType = "application/x-activex-handler";
-                  url = "dummy"; // Not needed, but KHTMLPart aborts the request if empty
-              }
 
-              if(o->classId.contains(QString::fromLatin1("D27CDB6E-AE6D-11cf-96B8-444553540000"))) {
-                  // It is ActiveX, but the nsplugin system handling
-                  // should also work, that's why we don't override the
-                  // serviceType with application/x-activex-handler
-                  // but let the KTrader in khtmlpart::createPart() detect
-                  // the user's preference: launch with activex viewer or
-                  // with nspluginviewer (Niko)
-                  serviceType = "application/x-shockwave-flash";
-              }
-              else if(o->classId.contains(QString::fromLatin1("CFCDAA03-8BE4-11cf-B84B-0020AFBBCCFA")))
-                  serviceType = "audio/x-pn-realaudio-plugin";
-
-              // TODO: add more plugins here
-          }
-
-          if((url.isEmpty() || url.isNull())) {
-              // look for a SRC attribute in the params
-              NodeImpl *child = o->firstChild();
-              while ( child ) {
-                  if ( child->id() == ID_PARAM ) {
-                      HTMLParamElementImpl *p = static_cast<HTMLParamElementImpl *>( child );
-
-                      if ( p->name().lower()==QString::fromLatin1("src") ||
-                           p->name().lower()==QString::fromLatin1("movie") ||
-                           p->name().lower()==QString::fromLatin1("code") )
-                      {
-                          url = p->value();
-                          break;
-                      }
+                  if(objbase->classId.contains(QString::fromLatin1("D27CDB6E-AE6D-11cf-96B8-444553540000"))) {
+                      // It is ActiveX, but the nsplugin system handling
+                      // should also work, that's why we don't override the
+                      // serviceType with application/x-activex-handler
+                      // but let the KTrader in khtmlpart::createPart() detect
+                      // the user's preference: launch with activex viewer or
+                      // with nspluginviewer (Niko)
+                      serviceType = "application/x-shockwave-flash";
                   }
-                  child = child->nextSibling();
+                  else if(objbase->classId.contains(QString::fromLatin1("CFCDAA03-8BE4-11cf-B84B-0020AFBBCCFA")))
+                      serviceType = "audio/x-pn-realaudio-plugin";
+                  else if(objbase->classId.contains(QString::fromLatin1("8AD9C840-044E-11D1-B3E9-00805F499D93")) ||
+                          objbase->classId.contains(QString::fromLatin1("CAFEEFAC-0014-0000-0000-ABCDEFFEDCBA")))
+                      serviceType = "application/x-java-applet";
+
+                  else
+                      kdDebug(6031) << "ActiveX classId " << objbase->classId << endl;
+
+                  // TODO: add more plugins here
               }
           }
-
-
-          if ( url.isEmpty() && serviceType.isEmpty() ) {
-#ifdef DEBUG_LAYOUT
-              kdDebug() << "RenderPartObject::close - empty url and serverType" << endl;
-#endif
-              return;
-          }
-          part->requestObject( this, url, serviceType, params );
       }
-      else {
-          // render embed object
-          url = embed->url;
-          serviceType = embed->serviceType;
-
-          if ( url.isEmpty() && serviceType.isEmpty() ) {
-#ifdef DEBUG_LAYOUT
-              kdDebug() << "RenderPartObject::close - empty url and serverType" << endl;
-#endif
-              return;
-          }
-          part->requestObject( this, url, serviceType, params );
-      }
-      o->setLiveConnect(part->liveConnectExtension(this));
-  }
-  else if ( element()->id() == ID_EMBED ) {
-
-      HTMLEmbedElementImpl *o = static_cast<HTMLEmbedElementImpl *>(element());
-      url = o->url;
-      serviceType = o->serviceType;
-
-      if ( url.isEmpty() && serviceType.isEmpty() ) {
-#ifdef DEBUG_LAYOUT
-          kdDebug() << "RenderPartObject::close - empty url and serverType" << endl;
-#endif
-          return;
-      }
-      // add all attributes set on the embed object
-      NamedAttrMapImpl* a = o->attributes();
-      if (a) {
-          for (unsigned long i = 0; i < a->length(); ++i) {
-              AttributeImpl* it = a->attributeItem(i);
-              params.append(o->getDocument()->attrName(it->id()).string() + "=\"" + it->value().string() + "\"");
-          }
-      }
-      part->requestObject( this, url, serviceType, params );
-  } else {
-      assert(element()->id() == ID_IFRAME);
-      HTMLIFrameElementImpl *o = static_cast<HTMLIFrameElementImpl *>(element());
-      url = o->url.string();
-      if( url.isEmpty()) return;
-      KHTMLView *v = static_cast<KHTMLView *>(m_view);
-      v->part()->requestFrame( this, url, o->name.string(), QStringList(), true );
+      if (url.isEmpty() || !document()->isURLAllowed(url) || !part->requestObject( this, url, serviceType, params ))
+          objbase->renderAlternative();
+      else
+          objbase->setLiveConnect(part->liveConnectExtension(this));
   }
 }
 
 // ugly..
 void RenderPartObject::close()
 {
-    if ( element()->id() == ID_OBJECT )
-        updateWidget();
     RenderPart::close();
+
+    if ( element()->id() != ID_IFRAME )
+        updateWidget();
+    // deleted here
 }
 
 
 bool RenderPartObject::partLoadingErrorNotify( khtml::ChildFrame *childFrame, const KURL& url, const QString& serviceType )
 {
     KHTMLPart *part = static_cast<KHTMLView *>(m_view)->part();
-    //kdDebug() << "RenderPartObject::partLoadingErrorNotify serviceType=" << serviceType << endl;
+    kdDebug(6031) << "RenderPartObject::partLoadingErrorNotify serviceType=" << serviceType << endl;
     // Check if we just tried with e.g. nsplugin
     // and fallback to the activexhandler if there is a classid
     // and a codebase, where we may download the ocx if it's missing
@@ -765,8 +746,11 @@ bool RenderPartObject::partLoadingErrorNotify( khtml::ChildFrame *childFrame, co
         {
             KParts::URLArgs args;
             args.serviceType = "application/x-activex-handler";
+            kdDebug(6031) << "set to activex" << endl;
             if (part->requestObject( childFrame, url, args ))
                 return true; // success
+
+            return false;
         }
     }
     // Dissociate ourselves from the current event loop (to prevent crashes
@@ -805,6 +789,9 @@ void RenderPartObject::slotPartLoadingErrorNotify()
     if ( embed )
 	serviceType = embed->serviceType;
 
+    // prepare for the local eventloop in KMessageBox
+    ref();
+
     KHTMLPart *part = static_cast<KHTMLView *>(m_view)->part();
     KParts::BrowserExtension *ext = part->browserExtension();
     if( embed && !embed->pluginPage.isEmpty() && ext ) {
@@ -834,8 +821,11 @@ void RenderPartObject::slotPartLoadingErrorNotify()
     }
 
     // didn't work, render alternative content.
- //   if ( element() && element()->id() == ID_OBJECT )
-   //     static_cast<HTMLObjectElementImpl*>( element() )->renderAlternative();
+    if ( element() && (
+         element()->id() == ID_OBJECT || element()->id() == ID_EMBED || element()->id() == ID_APPLET))
+        static_cast<HTMLObjectBaseElementImpl*>( element() )->renderAlternative();
+
+    deref();
 }
 
 void RenderPartObject::layout( )
@@ -860,8 +850,8 @@ void RenderPartObject::slotViewCleared()
       QScrollView *view = static_cast<QScrollView *>(m_widget);
       int frameStyle = QFrame::NoFrame;
       QScrollView::ScrollBarMode scroll = QScrollView::Auto;
-      int marginw = 0;
-      int marginh = 0;
+      int marginw = -1;
+      int marginh = -1;
       if ( element()->id() == ID_IFRAME) {
 	  HTMLIFrameElementImpl *frame = static_cast<HTMLIFrameElementImpl *>(element());
 	  if(frame->frameBorder)
