@@ -19,7 +19,7 @@
  * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
  *
- * $Id: render_frames.h,v 1.47.2.2 2003/05/18 12:34:40 mueller Exp $
+ * $Id: render_frames.h,v 1.53.2.1 2004/02/29 15:28:00 mueller Exp $
  */
 #ifndef __render_frames_h__
 #define __render_frames_h__
@@ -38,39 +38,39 @@ namespace DOM
 
 namespace khtml
 {
-  struct ChildFrame;
+    struct ChildFrame;
 
 class RenderFrameSet : public RenderBox
 {
-  friend class DOM::HTMLFrameSetElementImpl;
+    friend class DOM::HTMLFrameSetElementImpl;
 public:
-  RenderFrameSet( DOM::HTMLFrameSetElementImpl *frameSet );
+    RenderFrameSet( DOM::HTMLFrameSetElementImpl *frameSet );
 
-  virtual ~RenderFrameSet();
+    virtual ~RenderFrameSet();
 
-  virtual const char *renderName() const { return "RenderFrameSet"; }
-  virtual bool isFrameSet() const { return true; }
+    virtual const char *renderName() const { return "RenderFrameSet"; }
+    virtual bool isFrameSet() const { return true; }
 
-  virtual void layout();
+    virtual void layout();
 
-  void positionFrames( );
+    void positionFrames( );
 
-  bool resizing() const { return m_resizing; }
-  bool noResize() const { return element()->noResize(); }
+    bool resizing() const { return m_resizing; }
+    bool noResize() const { return element()->noResize(); }
 
-  bool userResize( DOM::MouseEventImpl *evt );
-  bool canResize( int _x, int _y);
-  void setResizing(bool e);
+    bool userResize( DOM::MouseEventImpl *evt );
+    bool canResize( int _x, int _y);
+    void setResizing(bool e);
 
-  Qt::CursorShape cursorShape() const { return m_cursor; }
+    Qt::CursorShape cursorShape() const { return m_cursor; }
 
-  bool nodeAtPoint(NodeInfo& info, int x, int y, int tx, int ty);
+    bool nodeAtPoint(NodeInfo& info, int x, int y, int tx, int ty, HitTestAction hitTestAction, bool inside);
 
     DOM::HTMLFrameSetElementImpl *element() const
     { return static_cast<DOM::HTMLFrameSetElementImpl*>(RenderObject::element()); }
 
-#ifndef NDEBUG
-  virtual void dump(QTextStream *stream, QString ind = "") const;
+#ifdef ENABLE_DUMP
+    virtual void dump(QTextStream &stream, const QString &ind) const;
 #endif
 
 private:
