@@ -1,9 +1,9 @@
 /**
  * This file is part of the DOM implementation for KDE.
  *
- * Copyright (C) 1999 Lars Knoll (knoll@kde.org)
+ * Copyright (C) 1999-2003 Lars Knoll (knoll@kde.org)
  *           (C) 1999 Antti Koivisto (koivisto@kde.org)
- *           (C) 2000 Dirk Mueller (mueller@kde.org)
+ *           (C) 2000-2003 Dirk Mueller (mueller@kde.org)
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -20,22 +20,24 @@
  * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
  *
- * $Id: render_image.cpp,v 1.100.2.5 2003/04/19 16:53:04 mueller Exp $
+ * $Id: render_image.cpp,v 1.128.2.3 2004/02/29 15:28:00 mueller Exp $
  */
 //#define DEBUG_LAYOUT
 
 #include "render_image.h"
-#include "render_root.h"
+#include "render_canvas.h"
 
 #include <qdrawutil.h>
 #include <qpainter.h>
 
 #include <kapplication.h>
 #include <kdebug.h>
+#include <kglobalsettings.h>
 
 #include "css/csshelper.h"
 #include "misc/helper.h"
 #include "misc/htmlattrs.h"
+#include "misc/loader.h"
 #include "misc/htmltags.h"
 #include "html/html_formimpl.h"
 #include "html/html_imageimpl.h"
@@ -50,10 +52,12 @@ using namespace khtml;
 
 // -------------------------------------------------------------------------
 
-RenderImage::RenderImage(HTMLElementImpl *_element)
+RenderImage::RenderImage(NodeImpl *_element)
     : RenderReplaced(_element)
 {
-    image = 0;
+    oimage = image = 0;
+
+    m_selectionState = SelectionNone;
     berrorPic = false;
     loadEventSent = false;
 
@@ -64,26 +68,33 @@ RenderImage::RenderImage(HTMLElementImpl *_element)
 RenderImage::~RenderImage()
 {
     if(image) image->deref(this);
+    if (oimage) oimage->deref( this );
+}
+
+QPixmap RenderImage::pixmap() const
+{
+    return image ? image->pixmap() : QPixmap();
 }
 
 void RenderImage::setStyle(RenderStyle* _style)
 {
     RenderReplaced::setStyle(_style);
     // init RenderObject attributes
-    setInline( style()->display()==INLINE );
     //setOverhangingContents(style()->height().isPercent());
-    setSpecialObjects(true);
+    setShouldPaintBackgroundOrBorder(true);
+}
 
-    CachedObject* co = style()->contentObject();
-    if (co && image != co ) {
-        if (image) image->deref(this);
-        image = static_cast<CachedImage*>(style()->contentObject());
-        if (image) image->ref(this);
-    }
+void RenderImage::setContentObject(CachedObject* co )
+{
+    if (co && image != co)
+        updateImage( static_cast<CachedImage*>( co ) );
 }
 
 void RenderImage::setPixmap( const QPixmap &p, const QRect& r, CachedImage *o)
 {
+    if ( o == oimage )
+        return;
+
     if(o != image) {
         RenderReplaced::setPixmap(p, r, o);
         return;
@@ -126,8 +137,8 @@ void RenderImage::setPixmap( const QPixmap &p, const QRect& r, CachedImage *o)
     if( o->pixmap_size().width() != intrinsicWidth() ||
        o->pixmap_size().height() != intrinsicHeight() || iwchanged )
     {
-//          qDebug("image dimensions have been changed, old: %d/%d  new: %d/%d",
-//                 intrinsicWidth(), intrinsicHeight(),
+//           qDebug("image dimensions have been changed, old: %d/%d  new: %d/%d",
+//                  intrinsicWidth(), intrinsicHeight(),
 //               o->pixmap_size().width(), o->pixmap_size().height());
 
         if(!o->isErrorImage()) {
@@ -135,26 +146,33 @@ void RenderImage::setPixmap( const QPixmap &p, const QRect& r, CachedImage *o)
             setIntrinsicHeight( o->pixmap_size().height() );
         }
 
-        // lets see if we need to relayout at all..
-        int oldwidth = m_width;
-        int oldheight = m_height;
-        calcWidth();
-        calcHeight();
+         // lets see if we need to relayout at all..
+         int oldwidth = m_width;
+         int oldheight = m_height;
+         int oldminwidth = m_minWidth;
+         m_minWidth = 0;
 
-        if(iwchanged || m_width != oldwidth || m_height != oldheight)
-            needlayout = true;
+         if ( parent() ) {
+             calcWidth();
+             calcHeight();
+         }
 
-        m_width = oldwidth;
-        m_height = oldheight;
+         if(iwchanged || m_width != oldwidth || m_height != oldheight)
+             needlayout = true;
+
+         m_minWidth = oldminwidth;
+         m_width = oldwidth;
+         m_height = oldheight;
     }
 
-    pix = p;
+    // we're not fully integrated in the tree yet.. we'll come back.
+    if ( !parent() )
+        return;
 
     if(needlayout)
     {
-        setLayouted(false);
         setMinMaxKnown(false);
-
+        setLayouted(false);
 //         kdDebug( 6040 ) << "m_width: : " << m_width << " height: " << m_height << endl;
 //         kdDebug( 6040 ) << "Image: size " << m_width << "/" << m_height << endl;
     }
@@ -170,6 +188,7 @@ void RenderImage::setPixmap( const QPixmap &p, const QRect& r, CachedImage *o)
             return;
 
         resizeCache = QPixmap(); // for resized animations
+
         if(completeRepaint)
             repaintRectangle(borderLeft()+paddingLeft(), borderTop()+paddingTop(), contentWidth(), contentHeight());
         else
@@ -180,11 +199,21 @@ void RenderImage::setPixmap( const QPixmap &p, const QRect& r, CachedImage *o)
     }
 }
 
-void RenderImage::paintObject(QPainter *p, int /*_x*/, int /*_y*/, int /*_w*/, int /*_h*/, int _tx, int _ty)
+void RenderImage::paint(PaintInfo& paintInfo, int _tx, int _ty) 
 {
-    // add offset for relative positioning
-    if(isRelPositioned())
-        relativePositionOffset(_tx, _ty);
+     if (paintInfo.phase != PaintActionForeground)
+         return;
+
+    // not visible or not even once layouted?
+    if (style()->visibility() != VISIBLE || m_y <=  -500000)  return;
+
+    _tx += m_x;
+    _ty += m_y;
+
+    if((_ty > paintInfo.r.bottom()) || (_ty + m_height <= paintInfo.r.top())) return;
+
+    if(shouldPaintBackgroundOrBorder())
+        paintBoxDecorations(paintInfo, _tx, _ty);
 
     int cWidth = contentWidth();
     int cHeight = contentHeight();
@@ -193,45 +222,50 @@ void RenderImage::paintObject(QPainter *p, int /*_x*/, int /*_y*/, int /*_w*/, i
     int leftPad = paddingLeft();
     int topPad = paddingTop();
 
-    if (khtml::printpainter && !root()->paintImages())
+    if (khtml::printpainter && !canvas()->printImages())
         return;
 
+    CachedImage* i = oimage && oimage->valid_rect().size() == oimage->pixmap_size()
+                     ? oimage : image;
+
     //kdDebug( 6040 ) << "    contents (" << contentWidth << "/" << contentHeight << ") border=" << borderLeft() << " padding=" << paddingLeft() << endl;
-    if ( pix.isNull() || berrorPic)
+    if ( !i || berrorPic)
     {
         if(cWidth > 2 && cHeight > 2)
         {
             if ( !berrorPic ) {
                 //qDebug("qDrawShadePanel %d/%d/%d/%d", _tx + leftBorder, _ty + topBorder, cWidth, cHeight);
-                qDrawShadePanel( p, _tx + leftBorder + leftPad, _ty + topBorder + topPad, cWidth, cHeight,
+                qDrawShadePanel( paintInfo.p, _tx + leftBorder + leftPad, _ty + topBorder + topPad, cWidth, cHeight,
                                  KApplication::palette().inactive(), true, 1 );
             }
-            if(berrorPic && !pix.isNull() && (cWidth >= pix.width()+4) && (cHeight >= pix.height()+4) )
+            QPixmap const* pix = i ? &i->pixmap() : 0;
+            if(berrorPic && pix && (cWidth >= pix->width()+4) && (cHeight >= pix->height()+4) )
             {
-                QRect r(pix.rect());
+                QRect r(pix->rect());
                 r = r.intersect(QRect(0, 0, cWidth-4, cHeight-4));
-                p->drawPixmap( QPoint( _tx + leftBorder + leftPad+2, _ty + topBorder + topPad+2), pix, r );
+                paintInfo.p->drawPixmap( QPoint( _tx + leftBorder + leftPad+2, _ty + topBorder + topPad+2), *pix, r );
             }
             if(!alt.isEmpty()) {
                 QString text = alt.string();
-                p->setFont(style()->font());
-                p->setPen( style()->color() );
+                paintInfo.p->setFont(style()->font());
+                paintInfo.p->setPen( style()->color() );
                 int ax = _tx + leftBorder + leftPad + 2;
                 int ay = _ty + topBorder + topPad + 2;
                 const QFontMetrics &fm = style()->fontMetrics();
                 if (cWidth>5 && cHeight>=fm.height())
-                    p->drawText(ax, ay+1, cWidth - 4, cHeight - 4, Qt::WordBreak, text );
+                    paintInfo.p->drawText(ax, ay+1, cWidth - 4, cHeight - 4, Qt::WordBreak, text );
             }
         }
     }
-    else if (image && !image->isTransparent())
+    else if (i && !i->isTransparent())
     {
+        const QPixmap& pix = i->pixmap();
         if ( (cWidth != intrinsicWidth() ||  cHeight != intrinsicHeight()) &&
-             pix.width() > 0 && pix.height() > 0 && image->valid_rect().isValid())
+             pix.width() > 0 && pix.height() > 0 && i->valid_rect().isValid())
         {
-            if (resizeCache.isNull() && cWidth && cHeight)
+            if (resizeCache.isNull() && cWidth && cHeight && intrinsicWidth() && intrinsicHeight())
             {
-                QRect scaledrect(image->valid_rect());
+                QRect scaledrect(i->valid_rect());
 //                 kdDebug(6040) << "time elapsed: " << dt->elapsed() << endl;
 //                  kdDebug( 6040 ) << "have to scale: " << endl;
 //                  qDebug("cw=%d ch=%d  pw=%d ph=%d  rcw=%d, rch=%d",
@@ -244,33 +278,30 @@ void RenderImage::paintObject(QPainter *p, int /*_x*/, int /*_y*/, int /*_w*/, i
                 scaledrect.setHeight( ( cHeight*scaledrect.height() ) / intrinsicHeight() );
 //                   qDebug("resizeCache size: %d/%d", resizeCache.width(), resizeCache.height());
 //                   qDebug("valid: %d/%d, scaled: %d/%d",
-//                          image->valid_rect().width(), image->valid_rect().height(),
+//                          i->valid_rect().width(), i->valid_rect().height(),
 //                          scaledrect.width(), scaledrect.height());
 
                 // sometimes scaledrect.width/height are off by one because
-                // of rounding errors. if the image is fully loaded, we
+                // of rounding errors. if the i is fully loaded, we
                 // make sure that we don't do unnecessary resizes during painting
                 QSize s(scaledrect.size());
-                if(image->valid_rect().size() == QSize( intrinsicWidth(), intrinsicHeight() )) // fully loaded
+                if(i->valid_rect().size() == QSize( intrinsicWidth(), intrinsicHeight() )) // fully loaded
                     s = QSize(cWidth, cHeight);
-                if(QABS(s.width() - cWidth) < 2) // rounding errors
+                if(kAbs(s.width() - cWidth) < 2) // rounding errors
                     s.setWidth(cWidth);
                 if(resizeCache.size() != s)
                     resizeCache.resize(s);
 
-                p->drawPixmap( QPoint( _tx + leftBorder + leftPad, _ty + topBorder + topPad),
+                paintInfo.p->drawPixmap( QPoint( _tx + leftBorder + leftPad, _ty + topBorder + topPad),
                                resizeCache, scaledrect );
             }
             else
-                p->drawPixmap( QPoint( _tx + leftBorder + leftPad, _ty + topBorder + topPad), resizeCache );
+                paintInfo.p->drawPixmap( QPoint( _tx + leftBorder + leftPad, _ty + topBorder + topPad), resizeCache );
         }
         else
         {
             // we might be just about switching images
-            // so pix contains the old one (we want to paint), but image->valid_rect is still invalid
-            // so use intrinsic Size instead.
-            // ### maybe no progressive loading for the second image ?
-            QRect rect(image->valid_rect().isValid() ? image->valid_rect()
+            QRect rect(i->valid_rect().isValid() ? i->valid_rect()
                        : QRect(0, 0, intrinsicWidth(), intrinsicHeight()));
 
             QPoint offs( _tx + leftBorder + leftPad, _ty + topBorder + topPad);
@@ -285,12 +316,35 @@ void RenderImage::paintObject(QPainter *p, int /*_x*/, int /*_y*/, int /*_w*/, i
 
 
 //             p->drawPixmap( offs.x(), y, pix, rect.x(), rect.y(), rect.width(), rect.height() );
-             p->drawPixmap(offs, pix, rect);
+             paintInfo.p->drawPixmap(offs, pix, rect);
 
         }
     }
+    if (m_selectionState != SelectionNone) {
+//    kdDebug(6040) << "_tx " << _tx << " _ty " << _ty << " _x " << _x << " _y " << _y << endl;
+        // Draw in any case if inside selection. For selection borders, the
+	// offset will decide whether to draw selection or not
+	bool draw = true;
+	if (m_selectionState != SelectionInside) {
+	    int startPos, endPos;
+            selectionStartEnd(startPos, endPos);
+            if(selectionState() == SelectionStart)
+                endPos = 1;
+            else if(selectionState() == SelectionEnd)
+                startPos = 0;
+	    draw = endPos - startPos > 0;
+	}
+	if (draw) {
+    	    // setting the brush origin is important for compatibility,
+	    // don't touch it unless you know what you're doing
+    	    paintInfo.p->setBrushOrigin(_tx, _ty - paintInfo.r.y());
+            paintInfo.p->fillRect(_tx, _ty, width(), height(),
+		    QBrush(style()->palette().active().highlight(),
+		    Qt::Dense4Pattern));
+	}
+    }
     if(style()->outlineWidth())
-        paintOutline(p, _tx, _ty, width(), height(), style());
+        paintOutline(paintInfo.p, _tx, _ty, width(), height(), style());
 }
 
 void RenderImage::layout()
@@ -309,13 +363,15 @@ void RenderImage::layout()
 
     // if they are variable width and we calculate a huge height or width, we assume they
     // actually wanted the intrinsic width.
-    if ( m_width > 2048 && !style()->width().isFixed() )
-	m_width = intrinsicWidth();
+    if ( m_width > 4096 && !style()->width().isFixed() )
+	m_width = intrinsicWidth() + paddingLeft() + paddingRight() + borderLeft() + borderRight();
     if ( m_height > 2048 && !style()->height().isFixed() )
-	m_height = intrinsicHeight();
+	m_height = intrinsicHeight() + paddingTop() + paddingBottom() + borderTop() + borderBottom();
+
     // limit total size to not run out of memory when doing the xform call.
-    if ( m_width * m_height > 2048*2048 ) {
-	float scale = sqrt( m_width*m_height / ( 2048.*2048. ) );
+    if ( ( m_width * m_height > 4096*2048 ) &&
+         ( contentWidth() > intrinsicWidth() || contentHeight() > intrinsicHeight() ) ) {
+	float scale = sqrt( m_width*m_height / ( 4096.*2048. ) );
 	m_width = (int) (m_width/scale);
 	m_height = (int) (m_height/scale);
     }
@@ -334,11 +390,19 @@ void RenderImage::notifyFinished(CachedObject *finishedObj)
             image->isErrorImage() ? EventImpl::ERROR_EVENT : EventImpl::LOAD_EVENT,
             false,false);
     }
+
+    if ( ( image == finishedObj || oimage == finishedObj ) && oimage ) {
+        oimage->deref( this );
+        oimage = 0;
+        repaint();
+    }
+
+    RenderReplaced::notifyFinished(finishedObj);
 }
 
-bool RenderImage::nodeAtPoint(NodeInfo& info, int _x, int _y, int _tx, int _ty)
+bool RenderImage::nodeAtPoint(NodeInfo& info, int _x, int _y, int _tx, int _ty, HitTestAction hitTestAction, bool inside)
 {
-    bool inside = RenderReplaced::nodeAtPoint(info, _x, _y, _tx, _ty);
+    inside |= RenderReplaced::nodeAtPoint(info, _x, _y, _tx, _ty, hitTestAction, inside);
 
     if (inside && element()) {
         int tx = _tx + m_x;
@@ -352,42 +416,60 @@ bool RenderImage::nodeAtPoint(NodeInfo& info, int _x, int _y, int _tx, int _ty)
             (map = static_cast<HTMLDocumentImpl*>(i->getDocument())->getMap(i->imageMap()))) {
             // we're a client side image map
             inside = map->mapMouseEvent(_x - tx, _y - ty, contentWidth(), contentHeight(), info);
+            info.setInnerNonSharedNode(element());
         }
     }
 
     return inside;
 }
 
+void RenderImage::updateImage(CachedImage* new_image)
+{
+    CachedImage* tempimage = oimage;
+    oimage = image;
+    image = new_image;
+    assert( image != oimage );
+
+    if ( image != tempimage && image != oimage )
+        image->ref(this);
+
+    if (tempimage && image != tempimage && oimage != tempimage )
+        tempimage->deref(this);
+
+    berrorPic = image->isErrorImage();
+}
+
 void RenderImage::updateFromElement()
 {
-    DOMString u = element()->id() == ID_OBJECT ?
-                  element()->getAttribute(ATTR_DATA) : element()->getAttribute(ATTR_SRC);
-
-    if (!u.isEmpty()) {
-        CachedImage *new_image = element()->getDocument()->docLoader()->
-                                 requestImage(khtml::parseURL(u));
-
-        if(new_image && new_image != image && (!style() || !style()->contentObject())) {
-            loadEventSent = false;
-            CachedImage* oldimage = image;
-            image = new_image;
-            image->ref(this);
-            if(oldimage) oldimage->deref(this);
-            berrorPic = image->isErrorImage();
-        }
-    }
-
     if (element()->id() == ID_INPUT)
         alt = static_cast<HTMLInputElementImpl*>(element())->altText();
     else if (element()->id() == ID_IMG)
         alt = static_cast<HTMLImageElementImpl*>(element())->altText();
+
+    DOMString u = element()->id() == ID_OBJECT ?
+                  element()->getAttribute(ATTR_DATA) : element()->getAttribute(ATTR_SRC);
+
+    if (!u.isEmpty() &&
+        ( !image || image->url() != u ) ) {
+        CachedImage *new_image = element()->getDocument()->docLoader()->
+                                 requestImage(khtml::parseURL(u));
+
+        if(new_image && new_image != image
+           // check appears redundant, as we only care about this when we're anonymous
+           // which can never happen here.
+           /*&& (!style() || !style()->contentObject())*/
+            ) {
+            loadEventSent = false;
+            updateImage( new_image );
+        }
+    }
 }
 
 bool RenderImage::complete() const
 {
-    // "complete" means that the image has been loaded
-    // but also that its width/height (contentWidth(),contentHeight()) have been calculated.
-    return !pix.isNull() && layouted();
+     // "complete" means that the image has been loaded
+     // but also that its width/height (contentWidth(),contentHeight()) have been calculated.
+     return image && image->valid_rect().size() == image->pixmap_size() && layouted();
 }
 
 short RenderImage::calcReplacedWidth() const
@@ -416,3 +498,30 @@ int RenderImage::calcReplacedHeight() const
 
     return RenderReplaced::calcReplacedHeight();
 }
+
+
+void RenderImage::caretPos(int offset, bool override, int &_x, int &_y, int &width, int &height)
+{
+    RenderReplaced::caretPos(offset, override, _x, _y, width, height);
+
+#if 0	// doesn't work reliably
+    height = intrinsicHeight();
+    width = override && offset == 0 ? intrinsicWidth() : 0;
+    _x = xPos();
+    _y = yPos();
+    if (offset > 0) _x += intrinsicWidth();
+
+    RenderObject *cb = containingBlock();
+
+    int absx, absy;
+    if (cb && cb != this && cb->absolutePosition(absx,absy))
+    {
+        _x += absx;
+        _y += absy;
+    } else {
+        // we don't know our absolute position, and there is no point returning
+        // just a relative one
+        _x = _y = -1;
+    }
+#endif
+}
