@@ -1,7 +1,7 @@
 /*
  * This file is part of the DOM implementation for KDE.
  *
- * Copyright (C) 1999 Lars Knoll (knoll@kde.org)
+ * Copyright (C) 1999-2003 Lars Knoll (knoll@kde.org)
  *           (C) 1999 Antti Koivisto (koivisto@kde.org)
  *
  * This library is free software; you can redistribute it and/or
@@ -19,7 +19,7 @@
  * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
  *
- * $Id: render_image.h,v 1.36.2.3 2003/05/18 12:34:40 mueller Exp $
+ * $Id: render_image.h,v 1.50.2.1 2004/02/29 15:28:00 mueller Exp $
  */
 #ifndef RENDER_IMAGE_H
 #define RENDER_IMAGE_H
@@ -35,50 +35,46 @@
 namespace khtml {
 
 class DocLoader;
+class CachedObject;
 
 class RenderImage : public RenderReplaced
 {
 public:
-    RenderImage(DOM::HTMLElementImpl *_element);
+    RenderImage(DOM::NodeImpl* _element);
     virtual ~RenderImage();
 
     virtual const char *renderName() const { return "RenderImage"; }
-
-    virtual bool isRendered() const { return true; }
-
-    virtual void paintObject( QPainter *p, int /*x*/, int /*y*/, int /*w*/, int /*h*/, int tx, int ty);
+    virtual void paint( PaintInfo& i, int tx, int ty );
 
     virtual void layout();
 
     virtual void setPixmap( const QPixmap &, const QRect&, CachedImage *);
 
-    QPixmap pixmap() const { return pix; }
-    // don't even think about making this method virtual!
+    // don't even think about making these methods virtual!
+    QPixmap pixmap() const;
     DOM::HTMLElementImpl* element() const
     { return static_cast<DOM::HTMLElementImpl*>(RenderObject::element()); }
 
     bool complete() const;
 
+    void setContentObject( CachedObject* );
+
     // hook to keep RendeObject::m_inline() up to date
     virtual void setStyle(RenderStyle *style);
     virtual void updateFromElement();
 
     virtual void notifyFinished(CachedObject *finishedObj);
-    virtual bool nodeAtPoint(NodeInfo& info, int x, int y, int tx, int ty);
+    virtual bool nodeAtPoint(NodeInfo& info, int x, int y, int tx, int ty, HitTestAction hitTestAction, bool inside);
 
     virtual short calcReplacedWidth() const;
     virtual int   calcReplacedHeight() const;
 
+    virtual SelectionState selectionState() const {return m_selectionState;}
+    virtual void setSelectionState(SelectionState s) {m_selectionState = s; }
+    virtual void caretPos(int offset, bool override, int &_x, int &_y, int &width, int &height);
+
 private:
-    /*
-     * Pointer to the image
-     * If this pointer is 0L, that means that the picture could not be loaded
-     * for some strange reason or that the image is waiting to be downloaded
-     * from the internet for example.
-     */
-
-    QPixmap pix;
-
+    void updateImage(CachedImage* new_image);
     /*
      * Cache for images that need resizing
      */
@@ -88,8 +84,11 @@ private:
     DOM::DOMString alt;
 
     CachedImage *image;
+    CachedImage *oimage;
+
     bool berrorPic : 1;
     bool loadEventSent : 1;
+    SelectionState m_selectionState : 3; // FIXME: don't forget to enlarge this as the enum grows
 };
 
 
