@@ -1,7 +1,7 @@
 /*
  * This file is part of the DOM implementation for KDE.
  *
- * Copyright (C) 1999 Lars Knoll (knoll@kde.org)
+ * Copyright (C) 1999-2003 Lars Knoll (knoll@kde.org)
  *           (C) 1999 Antti Koivisto (koivisto@kde.org)
  *
  * This library is free software; you can redistribute it and/or
@@ -19,27 +19,65 @@
  * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
  *
- * $Id: render_list.h,v 1.25.2.2 2003/05/18 12:34:40 mueller Exp $
+ * $Id: render_list.h,v 1.34.2.1 2004/02/29 15:28:01 mueller Exp $
  */
 #ifndef RENDER_LIST_H
 #define RENDER_LIST_H
 
-#include "render_flow.h"
+#include "rendering/render_block.h"
 
 // ### list-style-position, list-style-image is still missing
 
+namespace DOM
+{
+    class DocumentImpl;
+}
+
 namespace khtml
 {
 
 class RenderListItem;
+class RenderListMarker;
+
+class RenderListItem : public RenderBlock
+{
+    friend class RenderListMarker;
+
+public:
+    RenderListItem(DOM::NodeImpl*);
+
+    virtual const char *renderName() const { return "RenderListItem"; }
+
+    virtual void setStyle(RenderStyle *style);
+
+    virtual bool isListItem() const { return true; }
+
+//    long value() const { return m_marker->m_value; }
+    void setValue( long v ) { predefVal = v; }
+    void calcListValue();
+
+    virtual void layout( );
+    virtual void detach( );
+    virtual void calcMinMaxWidth();
+
+    void setInsideList(bool b ) { m_insideList = b; }
+
+protected:
+
+    void updateMarkerLocation();
+
+    RenderListMarker *m_marker;
+    signed long predefVal : 30;
+    bool m_insideList  : 1;
+    bool m_deleteMarker: 1;
+};
+
+// -----------------------------------------------------------------------------
 
-/* used to render the lists marker.
-     This class always has to be a direct child of a RenderListItem!
-*/
 class RenderListMarker : public RenderBox
 {
 public:
-    RenderListMarker();
+    RenderListMarker(DOM::DocumentImpl* document);
     ~RenderListMarker();
 
     virtual void setStyle(RenderStyle *style);
@@ -48,49 +86,30 @@ public:
     // so the marker gets to layout itself. Only needed for
     // list-style-position: inside
 
-    virtual void paint(QPainter *p, int x, int y, int w, int h,
-                           int xoff, int yoff);
-    virtual void paintObject(QPainter *p, int x, int y, int w, int h,
-                             int xoff, int yoff);
+    virtual void paint(PaintInfo& i, int xoff, int yoff);
     virtual void layout( );
     virtual void calcMinMaxWidth();
 
-    virtual short verticalPositionHint( bool firstLine ) const;
+    virtual short lineHeight( bool firstLine ) const;
+    virtual short baselinePosition( bool firstLine ) const;
 
     virtual void setPixmap( const QPixmap &, const QRect&, CachedImage *);
 
     virtual void calcWidth();
 
+    RenderListItem* listItem() const { return m_listItem; }
+    void setListItem(RenderListItem* listItem) { m_listItem = listItem; }
+
 protected:
     friend class RenderListItem;
 
+    bool listPositionInside() const
+    { return !m_listItem->m_insideList || style()->listStylePosition() == INSIDE; }
+
     QString m_item;
     CachedImage *m_listImage;
     long m_value;
-};
-
-class RenderListItem : public RenderFlow
-{
-public:
-    RenderListItem(DOM::NodeImpl*);
-    virtual ~RenderListItem();
-
-    virtual const char *renderName() const { return "RenderListItem"; }
-
-    virtual void setStyle(RenderStyle *style);
-
-    virtual bool isListItem() const { return true; }
-    virtual bool containsSpecial() { return (specialObjects != 0 && specialObjects->count() > 1 ); }
-
-    long value() const { return m_marker->m_value; }
-    void setValue( long v ) { predefVal = v; }
-    void calcListValue();
-
-    virtual void layout( );
-
-protected:
-    long int predefVal;
-    RenderListMarker *m_marker;
+    RenderListItem* m_listItem;
 };
 
 } //namespace
