@@ -1,9 +1,10 @@
 /*
  * This file is part of the html renderer for KDE.
  *
- * Copyright (C) 2000 Lars Knoll (knoll@kde.org)
+ * Copyright (C) 2000-2003 Lars Knoll (knoll@kde.org)
  *           (C) 2000 Antti Koivisto (koivisto@kde.org)
- *           (C) 2000 Dirk Mueller (mueller@kde.org)
+ *           (C) 2000-2003 Dirk Mueller (mueller@kde.org)
+ *           (C) 2002 Apple Computer, Inc.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -20,7 +21,7 @@
  * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
  *
- * $Id: render_object.h,v 1.137.2.3 2003/05/18 12:34:40 mueller Exp $
+ * $Id: render_object.h,v 1.175.2.3 2004/05/07 11:29:14 savernik Exp $
  */
 #ifndef render_object_h
 #define render_object_h
@@ -29,6 +30,9 @@
 #include <qrect.h>
 #include <assert.h>
 
+#include <kdebug.h>
+#include <kglobal.h>
+
 #include "misc/khtmllayout.h"
 #include "misc/loader_client.h"
 #include "misc/helper.h"
@@ -43,17 +47,42 @@ class KHTMLView;
 #define KHTMLAssert( x ) if( !(x) ) { \
     const RenderObject *o = this; while( o->parent() ) o = o->parent(); \
     o->printTree(); \
-    qDebug(" this object = %p", (void*) this ); \
+    qDebug(" this object = %p, %s", (void*) this, kdBacktrace().latin1() ); \
     assert( false ); \
 }
 #else
 #define KHTMLAssert( x )
 #endif
 
+/*
+ *	The painting of a layer occurs in three distinct phases.  Each phase involves
+ *	a recursive descent into the layer's render objects. The first phase is the background phase.
+ *	The backgrounds and borders of all blocks are painted.  Inlines are not painted at all.
+ *	Floats must paint above block backgrounds but entirely below inline content that can overlap them.
+ *	In the foreground phase, all inlines are fully painted.  Inline replaced elements will get all
+ *	three phases invoked on them during this phase.
+ */
+
+typedef enum {
+    PaintActionElementBackground = 0,
+    PaintActionChildBackground,
+    PaintActionChildBackgrounds,
+    PaintActionFloat,
+    PaintActionForeground,
+    PaintActionSelection
+} PaintAction;
+
+typedef enum {
+    HitTestAll = 0,
+    HitTestSelfOnly = 1,
+    HitTestChildrenOnly = 2
+} HitTestAction;
+
 namespace DOM {
     class HTMLAreaElementImpl;
     class DOMString;
     class NodeImpl;
+    class DocumentImpl;
     class ElementImpl;
     class EventImpl;
 }
@@ -63,15 +92,23 @@ namespace khtml {
     class RenderStyle;
     class RenderTable;
     class CachedObject;
-    class RenderRoot;
+    class RenderObject;
+    class RenderCanvas;
     class RenderText;
     class RenderFrameSet;
+    class RenderArena;
+    class RenderLayer;
+    class RenderBlock;
+    class InlineBox;
+    class InlineFlowBox;
 
 /**
  * Base Class for all rendering tree objects.
  */
 class RenderObject : public CachedObjectClient
 {
+    RenderObject(const RenderObject&);
+    RenderObject& operator=(const RenderObject&);
 public:
 
     RenderObject(DOM::NodeImpl* node);
@@ -85,6 +122,35 @@ public:
     virtual RenderObject *firstChild() const { return 0; }
     virtual RenderObject *lastChild() const { return 0; }
 
+    virtual bool childAllowed() const { return false; }
+
+    virtual RenderLayer* layer() const { return 0; }
+    RenderLayer* enclosingLayer() const;
+    void addLayers(RenderLayer* parentLayer, RenderLayer* beforeChild=0);
+    void removeLayers(RenderLayer* parentLayer);
+    void moveLayers(RenderLayer* oldParent, RenderLayer* newParent);
+    RenderLayer* findNextLayer(RenderLayer* parentLayer, RenderObject* startPoint,
+                               bool checkParent=true);
+    virtual void positionChildLayers() { }
+    virtual bool requiresLayer() const {
+        return isRoot() || (!isTableCell() && (isPositioned() || isRelPositioned()));
+    }
+
+    // ### rename to overflowClipRect and clipRect
+    virtual QRect getOverflowClipRect(int /*tx*/, int /*ty*/)
+	{ return QRect(0,0,0,0); }
+    virtual QRect getClipRect(int /*tx*/, int /*ty*/) { return QRect(0,0,0,0); }
+    bool hasClip() { return isPositioned() &&  style()->hasClip(); }
+    bool hasOverflowClip() { return style()->hidesOverflow(); }
+
+    virtual int getBaselineOfFirstLineBox() { return -1; } // Tables and blocks implement this.
+    virtual InlineFlowBox* getFirstLineBox() { return 0; } // Tables and blocks implement this.
+
+    // Whether or not a positioned element requires normal flow x/y to be computed
+    // to determine its position.
+    bool hasStaticX() const;
+    bool hasStaticY() const;
+
     // Linear tree traversal
     RenderObject *objectBelow() const;
     RenderObject *objectAbove() const;
@@ -100,7 +166,6 @@ public:
     virtual void insertChildNode(RenderObject* child, RenderObject* before);
     //////////////////////////////////////////
 
-private:
     //////////////////////////////////////////
     // Helper functions. Dangerous to use!
     void setPreviousSibling(RenderObject *previous) { m_previous = previous; }
@@ -110,23 +175,44 @@ private:
 
 public:
     virtual const char *renderName() const { return "RenderObject"; }
-#ifndef NDEBUG
+#ifdef ENABLE_DUMP
     QString information() const;
     virtual void printTree(int indent=0) const;
-    virtual void dump(QTextStream *stream, QString ind = "") const;
+    virtual void dump(QTextStream &stream, const QString &ind = QString::null) const;
 #endif
 
     static RenderObject *createObject(DOM::NodeImpl* node, RenderStyle* style);
 
+    // Overloaded new operator.  Derived classes must override operator new
+    // in order to allocate out of the RenderArena.
+    void* operator new(size_t sz, RenderArena* renderArena) throw();
+
+    // Overridden to prevent the normal delete from being called.
+    void operator delete(void* ptr, size_t sz);
+
+private:
+    // The normal operator new is disallowed on all render objects.
+    void* operator new(size_t sz);
+
+public:
+    RenderArena* renderArena() const;
+    virtual RenderFlow* continuation() const { return 0; }
+    virtual bool isInlineContinuation() const { return false; }
+
+
+    bool isRoot() const;
     // some helper functions...
+    virtual bool isRenderBlock() const { return false; }
+    virtual bool isRenderInline() const { return false; }
+    virtual bool isInlineFlow() const { return false; }
+    virtual bool isBlockFlow() const { return false; }
+    virtual bool isInlineBlockOrInlineTable() const { return false; }
     virtual bool childrenInline() const { return false; }
-    virtual bool isRendered() const { return false; }
-    virtual bool isFlow() const { return false; }
+    virtual bool isBox() const { return false; }
 
     virtual bool isListItem() const { return false; }
-    virtual bool isRoot() const { return false; }
+    virtual bool isCanvas() const { return false; }
     virtual bool isBR() const { return false; }
-    virtual bool isHtml() const { return false; }
     virtual bool isTableCell() const { return false; }
     virtual bool isTableRow() const { return false; }
     virtual bool isTableSection() const { return false; }
@@ -138,17 +224,18 @@ public:
     virtual bool isFrameSet() const { return false; }
     virtual bool isApplet() const { return false; }
 
-    bool isAnonymousBox() const { return m_isAnonymous; }
-    void setIsAnonymousBox(bool b) { m_isAnonymous = b; }
+    bool isAnonymous() const { return m_isAnonymous; }
+    void setIsAnonymous(bool b) { m_isAnonymous = b; }
 
     bool isFloating() const { return m_floating; }
-    bool isPositioned() const { return m_positioned; } // absolute or fixed positioning
-    bool isRelPositioned() const { return m_relPositioned; } // relative positioning
-    bool isText() const  { return m_isText; }
-    bool isInline() const { return m_inline; }  // inline object
-    bool mouseInside() const { return m_mouseInside; }
-    bool isReplaced() const { return m_replaced; } // a "replaced" element (see CSS)
-    bool hasSpecialObjects() const { return m_paintSpecial; }
+    bool isPositioned() const { return m_positioned; }
+    bool isRelPositioned() const { return m_relPositioned; }
+    bool isText() const { return m_isText; }
+    bool isInline() const { return m_inline; }
+    bool mouseInside() const;
+    bool isReplaced() const { return m_replaced; }
+    bool isReplacedBlock() const { return isInline() && isReplaced() && isRenderBlock(); }
+    bool shouldPaintBackgroundOrBorder() const { return m_paintBackground; }
     bool layouted() const   { return m_layouted; }
     bool minMaxKnown() const{ return m_minMaxKnown; }
     bool overhangingContents() const { return m_overhangingContents; }
@@ -156,9 +243,10 @@ public:
     bool isSelectionBorder() const { return m_isSelectionBorder; }
     bool recalcMinMax() const { return m_recalcMinMax; }
 
-    RenderRoot* root() const;
+    RenderCanvas* canvas() const;
     // don't even think about making this method virtual!
-    DOM::NodeImpl* element() const { return m_node; }
+    DOM::DocumentImpl* document() const;
+    DOM::NodeImpl* element() const { return isAnonymous() ? 0L : m_node; }
 
    /**
      * returns the object containing this one. can be different from parent for
@@ -167,22 +255,7 @@ public:
     RenderObject *container() const;
 
     void setOverhangingContents(bool p=true);
-    void setLayouted() {
-	m_layouted = true;
-    }
-    void setLayouted(bool b) {
-	m_layouted = b;
-	if(!b) {
-	    RenderObject *o = m_parent;
-	    RenderObject *root = this;
-	    while( o ) {
-		o->m_layouted = false;
-		root = o;
-		o = o->m_parent;
-	    }
-	    root->scheduleRelayout();
-	}
-    }
+    void setLayouted(bool b = true);
     void setLayoutedLocal(bool b) {
 	m_layouted = b;
     }
@@ -203,32 +276,41 @@ public:
     void setFloating(bool b=true) { m_floating = b; }
     void setInline(bool b=true) { m_inline = b; }
     void setMouseInside(bool b=true) { m_mouseInside = b; }
-    void setSpecialObjects(bool b=true) { m_paintSpecial = b; }
+    void setShouldPaintBackgroundOrBorder(bool b=true) { m_paintBackground = b; }
     void setRenderText() { m_isText = true; }
     void setReplaced(bool b=true) { m_replaced = b; }
     void setIsSelectionBorder(bool b=true) { m_isSelectionBorder = b; }
 
-    void scheduleRelayout();
+    void scheduleRelayout(RenderObject *clippedObj = 0);
+
+    virtual InlineBox* createInlineBox(bool makePlaceHolderBox, bool isRootLineBox);
 
-    // for discussion of lineHeight see CSS2 spec
     virtual short lineHeight( bool firstLine ) const;
-    // for the vertical-align property of inline elements
-    // the difference between this objects baseline position and the lines baseline position.
     virtual short verticalPositionHint( bool firstLine ) const;
-    // the offset of baseline from the top of the object.
     virtual short baselinePosition( bool firstLine ) const;
 
     /*
      * Print the object and its children, clipped by (x|y|w|h).
      * (tx|ty) is the calculated position of the parent
      */
-    virtual void paint( QPainter *p, int x, int y, int w, int h, int tx, int ty);
+    struct PaintInfo {
+       PaintInfo(QPainter* _p, const QRect& _r, PaintAction _phase)
+           : p(_p), r(_r), phase(_phase) {}
+       QPainter* p;
+       QRect     r;
+       PaintAction phase;
+    };
+    virtual void paint( PaintInfo& i, int tx, int ty);
 
-    virtual void paintObject( QPainter */*p*/, int /*x*/, int /*y*/,
-                        int /*w*/, int /*h*/, int /*tx*/, int /*ty*/) {}
     void paintBorder(QPainter *p, int _tx, int _ty, int w, int h, const RenderStyle* style, bool begin=true, bool end=true);
     void paintOutline(QPainter *p, int _tx, int _ty, int w, int h, const RenderStyle* style);
 
+    virtual void paintBoxDecorations(PaintInfo&, int /*_tx*/, int /*_ty*/) {}
+
+    virtual void paintBackgroundExtended(QPainter* /*p*/, const QColor& /*c*/, CachedImage* /*bg*/,
+                                         int /*clipy*/, int /*cliph*/, int /*_tx*/, int /*_ty*/,
+                                         int /*w*/, int /*height*/, int /*bleft*/, int /*bright*/ ) {}
+
 
     /*
      * This function calculates the minimum & maximum width that the object
@@ -270,7 +352,7 @@ public:
 
     // used for element state updates that can not be fixed with a
     // repaint and do not need a relayout
-    virtual void updateFromElement() {};
+    virtual void updateFromElement() {}
 
     // The corresponding closing element has been parsed. ### remove me
     virtual void close() { }
@@ -280,43 +362,63 @@ public:
     // does a query on the rendertree and finds the innernode
     // and overURL for the given position
     // if readonly == false, it will recalc hover styles accordingly
-    class NodeInfo
+   class NodeInfo
     {
         friend class RenderImage;
         friend class RenderFlow;
+        friend class RenderInline;
         friend class RenderText;
         friend class RenderObject;
         friend class RenderFrameSet;
+	friend class RenderLayer;
         friend class DOM::HTMLAreaElementImpl;
     public:
         NodeInfo(bool readonly, bool active)
-            : m_innerNode(0), m_innerURLElement(0), m_readonly(readonly), m_active(active)
+            : m_innerNode(0), m_innerNonSharedNode(0), m_innerURLElement(0), m_readonly(readonly), m_active(active)
             { }
 
         DOM::NodeImpl* innerNode() const { return m_innerNode; }
+        DOM::NodeImpl* innerNonSharedNode() const { return m_innerNonSharedNode; }
         DOM::NodeImpl* URLElement() const { return m_innerURLElement; }
         bool readonly() const { return m_readonly; }
         bool active() const { return m_active; }
 
     private:
         void setInnerNode(DOM::NodeImpl* n) { m_innerNode = n; }
+        void setInnerNonSharedNode(DOM::NodeImpl* n) { m_innerNonSharedNode = n; }
         void setURLElement(DOM::NodeImpl* n) { m_innerURLElement = n; }
 
         DOM::NodeImpl* m_innerNode;
+        DOM::NodeImpl* m_innerNonSharedNode;
         DOM::NodeImpl* m_innerURLElement;
         bool m_readonly;
         bool m_active;
     };
 
+    /** contains stateful information for a checkSelectionPoint call
+     */
+    struct SelPointState {
+        /** last node that was before the current position */
+        DOM::NodeImpl *m_lastNode;
+	/** offset of last node */
+	long m_lastOffset;
+	/** true when the last node had the result SelectionAfterInLine */
+	bool m_afterInLine;
+
+	SelPointState() : m_lastNode(0), m_lastOffset(0), m_afterInLine(false)
+	{}
+    };
+
     virtual FindSelectionResult checkSelectionPoint( int _x, int _y, int _tx, int _ty,
-                                                     DOM::NodeImpl*&, int & offset );
-    virtual bool nodeAtPoint(NodeInfo& info, int x, int y, int tx, int ty);
+                                                     DOM::NodeImpl*&, int & offset,
+						     SelPointState & );
+    virtual bool nodeAtPoint(NodeInfo& info, int x, int y, int tx, int ty, HitTestAction, bool inside = false);
 
     // set the style of the object.
     virtual void setStyle(RenderStyle *style);
 
     // returns the containing block level element for this element.
-    RenderObject *containingBlock() const;
+    RenderBlock *containingBlock() const;
 
     // return just the width of the containing block
     virtual short containingBlockWidth() const;
@@ -339,6 +441,15 @@ public:
     virtual int xPos() const { return 0; }
     virtual int yPos() const { return 0; }
 
+    /** Leftmost coordinate of this inline element relative to containing
+     * block. Always zero for non-inline elements.
+     */
+    virtual int inlineXPos() const { return 0; }
+    /** Topmost coordinate of this inline element relative to containing
+     * block. Always zero for non-inline elements.
+     */
+    virtual int inlineYPos() const { return 0; }
+
     // calculate client position of box
     virtual bool absolutePosition(int &/*xPos*/, int &/*yPos*/, bool fixed = false);
 
@@ -346,20 +457,49 @@ public:
     virtual short width() const { return 0; }
     virtual int height() const { return 0; }
 
+    // The height of a block when you include overflow spillage out of
+    // the bottom of the block (e.g., a <div style="height:25px"> that
+    // has a 100px tall image inside it would have an overflow height
+    // of borderTop() + paddingTop() + 100px.
+    virtual int overflowHeight() const { return height(); }
+    virtual int overflowWidth() const { return width(); }
+
+    // IE extensions, heavily used in ECMA
+    virtual short offsetWidth() const { return width(); }
+    virtual int offsetHeight() const { return height(); }
+    virtual int offsetLeft() const;
+    virtual int offsetTop() const;
+    virtual RenderObject* offsetParent() const;
+    short clientWidth() const;
+    int clientHeight() const;
+    short scrollWidth() const;
+    int scrollHeight() const;
+
+    virtual bool isSelfCollapsingBlock() const { return false; }
+    short collapsedMarginTop() const { return kMax( int( marginTop() ), 0 ) + kMin( int( marginTop() ), 0 );  }
+    short collapsedMarginBottom() const { return kMax( int( marginBottom() ), 0 ) + kMin( int( marginBottom() ), 0 );  }
+
+    virtual bool isTopMarginQuirk() const { return false; }
+    virtual bool isBottomMarginQuirk() const { return false; }
+    virtual short maxTopMargin(bool positive) const
+    { return positive ? kMax( int( marginTop() ), 0 ) : - kMin( int( marginTop() ), 0 ); }
+    virtual short maxBottomMargin(bool positive) const
+    { return positive ? kMax( int( marginBottom() ), 0 ) : - kMin( int( marginBottom() ), 0 ); }
+
     virtual short marginTop() const { return 0; }
     virtual short marginBottom() const { return 0; }
     virtual short marginLeft() const { return 0; }
     virtual short marginRight() const { return 0; }
 
-    int paddingTop() const;
-    int paddingBottom() const;
-    int paddingLeft() const;
-    int paddingRight() const;
+    virtual int paddingTop() const;
+    virtual int paddingBottom() const;
+    virtual int paddingLeft() const;
+    virtual int paddingRight() const;
 
-    int borderTop() const { return style()->borderTopWidth(); }
-    int borderBottom() const { return style()->borderBottomWidth(); }
-    int borderLeft() const { return style()->borderLeftWidth(); }
-    int borderRight() const { return style()->borderRightWidth(); }
+    virtual int borderTop() const { return style()->borderTopWidth(); }
+    virtual int borderBottom() const { return style()->borderBottomWidth(); }
+    virtual int borderLeft() const { return style()->borderLeftWidth(); }
+    virtual int borderRight() const { return style()->borderRightWidth(); }
 
     virtual short minWidth() const { return 0; }
     virtual short maxWidth() const { return 0; }
@@ -375,6 +515,9 @@ public:
 	return s;
     }
 
+    void getTextDecorationColors(int decorations, QColor& underline, QColor& overline,
+                                 QColor& linethrough, bool quirksMode=false);
+
     enum BorderSide {
         BSTop, BSBottom, BSLeft, BSRight
     };
@@ -383,20 +526,23 @@ public:
                     int adjbw1, int adjbw2, bool invalidisInvert = false);
 
     // force a complete repaint
-    virtual void repaint() { if(m_parent) m_parent->repaint(); }
-    virtual void repaintRectangle(int x, int y, int w, int h, bool f=false);
+    virtual void repaint(bool immediate = false) { if(m_parent) m_parent->repaint(immediate); }
+    virtual void repaintRectangle(int x, int y, int w, int h, bool immediate = false, bool f=false);
 
     virtual unsigned int length() const { return 1; }
 
     virtual bool isHidden() const { return isFloating() || isPositioned(); }
 
     // Special objects are objects that are neither really inline nor blocklevel
-    bool isSpecial() const { return (isFloating() || isPositioned()); };
-    virtual bool containsSpecial() { return false; }
-    virtual bool hasOverhangingFloats() { return false; }
+    bool isFloatingOrPositioned() const { return (isFloating() || isPositioned()); };
+    virtual bool hasOverhangingFloats() const { return false; }
+    virtual bool hasFloats() const { return false; }
+    virtual bool containsFloat(RenderObject* /*o*/) const { return false; }
+    virtual void markAllDescendantsWithFloatsForLayout(RenderObject* /*floatToRemove*/ = 0) {}
 
-    // positioning of inline childs (bidi)
-    virtual void position(int, int, int, int, int, bool, bool, int) {}
+    // positioning of inline children (bidi)
+    virtual void position(InlineBox*, int, int, bool) {}
+//    virtual void position(int, int, int, int, int, bool, bool, int) {}
 
     enum SelectionState {
         SelectionNone,
@@ -409,22 +555,33 @@ public:
     virtual SelectionState selectionState() const { return SelectionNone;}
     virtual void setSelectionState(SelectionState) {}
 
-    virtual void cursorPos(int /*offset*/, int &/*_x*/, int &/*_y*/, int &/*height*/);
+    /**
+     * Returns the content coordinates of the caret within this render object.
+     * @param offset zero-based offset determining position within the render object.
+     * @param override @p true if input overrides existing characters,
+     *		@p false if it inserts them. The width of the caret depends on
+     *		this one.
+     * @param _x returns the left coordinate
+     * @param _y returns the top coordinate
+     * @param width returns the caret's width
+     * @param height returns the caret's height
+     */
+    virtual void caretPos(int offset, bool override, int &_x, int &_y, int &width, int &height);
 
     // returns the lowest position of the lowest object in that particular object.
     // This 'height' is relative to the topleft of the margin box of the object.
     // Implemented in RenderFlow.
     virtual int lowestPosition() const {return 0;}
 
-    virtual int rightmostPosition() const {return 0;}
+    virtual short rightmostPosition() const {return 0;}
 
     // recursively invalidate current layout
     // unused: void invalidateLayout();
 
     virtual void calcVerticalMargins() {}
-    void removeFromSpecialObjects();
+    void removeFromObjectLists();
 
-    virtual void detach();
+    virtual void detach( );
 
     const QFont &font(bool firstLine) const {
 	return style( firstLine )->font();
@@ -433,26 +590,35 @@ public:
     const QFontMetrics &fontMetrics(bool firstLine) const {
 	return style( firstLine )->fontMetrics();
     }
+
+    /** returns the lowest possible value the caret offset may have to
+     * still point to a valid position.
+     *
+     * Returns 0 by default.
+     */
+    virtual long minOffset() const { return 0; }
+    /** returns the highest possible value the caret offset may have to
+     * still point to a valid position.
+     *
+     * Returns 0 by default, as generic elements are considered to have no
+     * width.
+     */
+    virtual long maxOffset() const { return 0; }
+
+    virtual void setPixmap(const QPixmap &, const QRect&, CachedImage *);
+
 protected:
     virtual void selectionStartEnd(int& spos, int& epos);
 
-    virtual void paintBoxDecorations(QPainter* /*p*/, int /*_x*/, int /*_y*/,
-                                     int /*_w*/, int /*_h*/, int /*_tx*/, int /*_ty*/) {}
-
     virtual QRect viewRect() const;
-    void remove() {
-        removeFromSpecialObjects();
-
-        if ( parent() )
-            //have parent, take care of the tree integrity
-            parent()->removeChild(this);
-    }
-
+    void remove();
     void invalidateVerticalPositions();
     short getVerticalPosition( bool firstLine ) const;
 
     virtual void removeLeftoverAnonymousBoxes();
 
+    void arenaDelete(RenderArena *arena);
+
 private:
     RenderStyle* m_style;
     DOM::NodeImpl* m_node;
@@ -463,14 +629,15 @@ private:
     short m_verticalPosition;
 
     bool m_layouted                  : 1;
-    bool m_unused                   : 1;
+    bool m_unused                    : 1;
     bool m_minMaxKnown               : 1;
     bool m_floating                  : 1;
 
     bool m_positioned                : 1;
     bool m_overhangingContents : 1;
     bool m_relPositioned             : 1;
-    bool m_paintSpecial              : 1; // if the box has something special to print (background, border, etc)
+    bool m_paintBackground           : 1; // if the box has something to paint in the
+                                          // background painting phase (background, border, etc)
 
     bool m_isAnonymous               : 1;
     bool m_recalcMinMax 	     : 1;
@@ -481,11 +648,15 @@ private:
     bool m_mouseInside : 1;
     bool m_hasFirstLine              : 1;
     bool m_isSelectionBorder          : 1;
-
     // note: do not add unnecessary bitflags, we have 32 bit already!
+
+
+    void arenaDelete(RenderArena *arena, void *objectBase);
+
+    friend class RenderLayer;
     friend class RenderListItem;
     friend class RenderContainer;
-    friend class RenderRoot;
+    friend class RenderCanvas;
 };
 
 
