@@ -18,16 +18,22 @@
  * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
  *
- * $Id: render_replaced.h,v 1.39.2.4 2003/05/18 12:34:40 mueller Exp $
+ * $Id: render_replaced.h,v 1.67.2.3 2004/02/29 15:28:01 mueller Exp $
  */
 #ifndef render_replaced_h
 #define render_replaced_h
 
-#include "render_box.h"
+#include "rendering/render_block.h"
 #include <qobject.h>
+
 class KHTMLView;
 class QWidget;
 
+namespace DOM
+{
+    class MouseEventImpl;
+}
+
 namespace khtml {
 
 class RenderReplaced : public RenderBox
@@ -37,24 +43,37 @@ public:
 
     virtual const char *renderName() const { return "RenderReplaced"; }
 
-    virtual bool isRendered() const { return true; }
-
-    virtual short lineHeight( bool firstLine) const;
-    virtual short baselinePosition( bool firstLine ) const;
+    virtual bool childAllowed() const { return false; }
 
     virtual void calcMinMaxWidth();
 
-    virtual void paint( QPainter *, int x, int y, int w, int h,
-                        int tx, int ty);
-    virtual void paintObject(QPainter *p, int x, int y, int w, int h, int tx, int ty) = 0;
-
     virtual short intrinsicWidth() const { return m_intrinsicWidth; }
     virtual int intrinsicHeight() const { return m_intrinsicHeight; }
 
     void setIntrinsicWidth(int w) {  m_intrinsicWidth = w; }
     void setIntrinsicHeight(int h) { m_intrinsicHeight = h; }
 
-    virtual void position(int x, int y, int from, int len, int width, bool reverse, bool firstLine, int);
+    virtual void position(InlineBox*, int, int, bool);
+
+    // Return before, after (offset set to max), or inside the replaced element,
+    // at @p offset
+    virtual FindSelectionResult checkSelectionPoint( int _x, int _y, int _tx, int _ty,
+                                                     DOM::NodeImpl*& node, int & offset,
+						     SelPointState & );
+
+    /** returns the lowest possible value the caret offset may have to
+     * still point to a valid position.
+     *
+     * Returns 0.
+     */
+    virtual long minOffset() const { return 0; }
+    /** returns the highest possible value the caret offset may have to
+     * still point to a valid position.
+     *
+     * Returns 1 as replaced elements are considered to be an entity that
+     * can have the caret positioned at their beginning (0), and at their end (1).
+     */
+    virtual long maxOffset() const { return 1; }
 
 protected:
     short m_intrinsicWidth;
@@ -70,12 +89,10 @@ public:
     virtual ~RenderWidget();
 
     virtual void setStyle(RenderStyle *style);
-
-    virtual void paintObject(QPainter *p, int x, int y, int w, int h, int tx, int ty);
-
+    virtual void paint( PaintInfo& i, int tx, int ty );
     virtual bool isWidget() const { return true; };
 
-    virtual void detach();
+    virtual void detach( );
     virtual void layout( );
 
     virtual void updateFromElement();
@@ -83,6 +100,17 @@ public:
     QWidget *widget() const { return m_widget; }
     KHTMLView* view() const { return m_view; }
 
+    void deref();
+
+    static void paintWidget(PaintInfo& pI, QWidget *widget, int tx, int ty);
+    virtual bool handleEvent(const DOM::EventImpl& ev);
+
+#ifdef ENABLE_DUMP
+    virtual void dump(QTextStream &stream, const QString &ind) const;
+#endif
+
+    // for ECMA to flush all pending resizes
+    static void flushWidgetResizes();
 
 public slots:
     void slotWidgetDestructed();
@@ -97,8 +125,18 @@ protected:
 
     QWidget *m_widget;
     KHTMLView* m_view;
+
+    bool m_resizePending;
+
+public:
+    class EventPropagator : public QWidget {
+    public:
+        void sendEvent(QEvent *e);
+    };
 };
 
+extern bool allowWidgetPaintEvents;
+
 }
 
 #endif
