@@ -2,6 +2,9 @@
  * This file is part of the DOM implementation for KDE.
  *
  * Copyright (C) 1999 Antti Koivisto (koivisto@kde.org)
+ * Copyright (C) 1999-2003 Lars Knoll (knoll@kde.org)
+ * Copyright (C) 2002-2003 Dirk Mueller (mueller@kde.org)
+ * Copyright (C) 2002 Apple Computer, Inc.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -18,16 +21,16 @@
  * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
  *
- * $Id: render_style.cpp,v 1.53.2.1 2003/02/04 21:35:54 mueller Exp $
  */
 
 #include "xml/dom_stringimpl.h"
-
+#include "css/cssstyleselector.h"
 #include "render_style.h"
 
 #include "kdebug.h"
 
 using namespace khtml;
+using namespace DOM;
 
 /* CSS says Fixde for the default padding value, but we treat variable as 0 padding anyways, and like
  * this is works fine for table paddings aswell
@@ -51,8 +54,10 @@ bool StyleSurroundData::operator==(const StyleSurroundData& o) const
 }
 
 StyleBoxData::StyleBoxData()
-    : z_index( ZAUTO )
+    : z_index( 0 ), z_auto( true )
 {
+    min_width = min_height = RenderStyle::initialMinSize();
+    max_width = max_height = RenderStyle::initialMaxSize();
 }
 
 StyleBoxData::StyleBoxData(const StyleBoxData& o )
@@ -60,7 +65,7 @@ StyleBoxData::StyleBoxData(const StyleBoxData& o )
       width( o.width ), height( o.height ),
       min_width( o.min_width ), max_width( o.max_width ),
       min_height ( o.min_height ), max_height( o.max_height ),
-      z_index( o.z_index )
+      z_index( o.z_index ), z_auto( o.z_auto )
 {
 }
 
@@ -73,11 +78,13 @@ bool StyleBoxData::operator==(const StyleBoxData& o) const
 	    max_width == o.max_width &&
 	    min_height == o.min_height &&
 	    max_height == o.max_height &&
-	    z_index == o.z_index;
+	    z_index == o.z_index &&
+            z_auto == o.z_auto;
 }
 
 StyleVisualData::StyleVisualData()
-    : colspan( 1 ), counter_increment( 0 ), counter_reset( 0 ),
+     : textDecoration(RenderStyle::initialTextDecoration()),
+       counter_increment( 0 ), counter_reset( 0 ),
       palette( QApplication::palette() )
 {
 }
@@ -87,7 +94,7 @@ StyleVisualData::~StyleVisualData() {
 
 StyleVisualData::StyleVisualData(const StyleVisualData& o )
     : Shared<StyleVisualData>(),
-      clip( o.clip ), colspan( o.colspan ),
+      clip( o.clip ), textDecoration(o.textDecoration),
       counter_increment( o.counter_increment ), counter_reset( o.counter_reset ),
       palette( o.palette )
 {
@@ -96,7 +103,7 @@ StyleVisualData::StyleVisualData(const StyleVisualData& o )
 
 
 StyleBackgroundData::StyleBackgroundData()
-    : image( 0 )
+    : image( RenderStyle::initialBackgroundImage() )
 {
 }
 
@@ -121,8 +128,11 @@ bool StyleBackgroundData::operator==(const StyleBackgroundData& o) const
 
 
 StyleInheritedData::StyleInheritedData()
-    : indent( Fixed ), line_height( -100, Percent ), style_image( 0 ),
-      font(), color( Qt::black ), decoration_color( Qt::black ), border_spacing( 0 )
+    : indent( RenderStyle::initialTextIndent() ), line_height( RenderStyle::initialLineHeight() ),
+      style_image( RenderStyle::initialListStyleImage() ),
+      font(), color( RenderStyle::initialColor() ),
+      border_hspacing( RenderStyle::initialBorderHorizontalSpacing() ),
+      border_vspacing( RenderStyle::initialBorderVerticalSpacing() )
 {
 }
 
@@ -134,7 +144,8 @@ StyleInheritedData::StyleInheritedData(const StyleInheritedData& o )
     : Shared<StyleInheritedData>(),
       indent( o.indent ), line_height( o.line_height ), style_image( o.style_image ),
       font( o.font ), color( o.color ), decoration_color( o.decoration_color ),
-      border_spacing( o.border_spacing )
+      border_hspacing( o.border_hspacing ),
+      border_vspacing( o.border_vspacing )
 {
 }
 
@@ -143,7 +154,8 @@ bool StyleInheritedData::operator==(const StyleInheritedData& o) const
     return
 	indent == o.indent &&
 	line_height == o.line_height &&
-	border_spacing == o.border_spacing &&
+        border_hspacing == o.border_hspacing &&
+        border_vspacing == o.border_vspacing &&
 	style_image == o.style_image &&
 	font == o.font &&
 	color == o.color &&
@@ -246,7 +258,15 @@ RenderStyle* RenderStyle::addPseudoStyle(PseudoId pid)
 
     if (!ps)
     {
+        switch (pid) {
+          case FIRST_LETTER:             // pseudo-elements (FIRST_LINE has a special handling)
+          case BEFORE:
+          case AFTER:
+            ps = new RenderStyle();
+            break;
+          default:
             ps = new RenderStyle(*this); // use the real copy constructor to get an identical copy
+        }
         ps->ref();
         ps->noninherited_flags.f._styleType = pid;
         ps->pseudoStyle = pseudoStyle;
@@ -300,13 +320,12 @@ bool RenderStyle::inheritedNotEqual( RenderStyle *other ) const
 */
 RenderStyle::Diff RenderStyle::diff( const RenderStyle *other ) const
 {
-    Diff d = Equal;
-
     // we anyway assume they are the same
 // 	EDisplay _display : 5;
 
     // NonVisible:
 // 	ECursor _cursor_style : 4;
+//	EUserInput _user_input : 2;	as long as :enabled is not impl'd
 
 // ### this needs work to know more exactly if we need a relayout
 //     or just a repaint
@@ -320,14 +339,17 @@ RenderStyle::Diff RenderStyle::diff( const RenderStyle *other ) const
 //     DataRef<StyleInheritedData> inherited;
 
     if ( *box.get() != *other->box.get() ||
-	 *visual.get() != *other->visual.get() ||
-	 *surround.get() != *other->surround.get() ||
-	 *inherited.get() != *other->inherited.get()
-	)
-	d = CbLayout;
-
-    if ( d == CbLayout )
-	return d;
+         *visual.get() != *other->visual.get() ||
+         *surround.get() != *other->surround.get() ||
+         !(inherited->indent == other->inherited->indent) ||
+         !(inherited->line_height == other->inherited->line_height) ||
+         !(inherited->style_image == other->inherited->style_image) ||
+         !(inherited->font == other->inherited->font) ||
+         !(inherited->border_hspacing == other->inherited->border_hspacing) ||
+         !(inherited->border_vspacing == other->inherited->border_vspacing) ||
+         !(inherited_flags.f._visuallyOrdered == other->inherited_flags.f._visuallyOrdered) ||
+         !(inherited_flags.f._htmlHacks == other->inherited_flags.f._htmlHacks) )
+        return CbLayout;
 
     // changes causing Layout changes:
 
@@ -339,106 +361,79 @@ RenderStyle::Diff RenderStyle::diff( const RenderStyle *other ) const
 //     EPosition _position : 2;
 //     EFloat _floating : 2;
     if ( ((int)noninherited_flags.f._display) >= TABLE ) {
-	// Stupid gcc gives a compile error on
-	// a != other->b if a and b are bitflags. Using
-	// !(a== other->b) instead.
-	if ( !(inherited_flags.f._border_collapse == other->inherited_flags.f._border_collapse) ||
-	     !(inherited_flags.f._empty_cells == other->inherited_flags.f._empty_cells) ||
-	     !(inherited_flags.f._caption_side == other->inherited_flags.f._caption_side) ||
-	     !(noninherited_flags.f._table_layout == other->noninherited_flags.f._table_layout) ||
-	     !(noninherited_flags.f._position == other->noninherited_flags.f._position) ||
-	     !(noninherited_flags.f._floating == other->noninherited_flags.f._floating) )
-
-	    d = CbLayout;
+        if ( !(inherited_flags.f._empty_cells == other->inherited_flags.f._empty_cells) ||
+             !(inherited_flags.f._caption_side == other->inherited_flags.f._caption_side) ||
+             !(inherited_flags.f._border_collapse == other->inherited_flags.f._border_collapse) ||
+             !(noninherited_flags.f._table_layout == other->noninherited_flags.f._table_layout) ||
+             !(noninherited_flags.f._position == other->noninherited_flags.f._position) ||
+             !(noninherited_flags.f._floating == other->noninherited_flags.f._floating) ||
+             !(noninherited_flags.f._flowAroundFloats == other->noninherited_flags.f._flowAroundFloats) ||
+             !(noninherited_flags.f._unicodeBidi == other->noninherited_flags.f._unicodeBidi) )
+            return CbLayout;
     }
 
 // only for lists:
 // 	EListStyleType _list_style_type : 5 ;
 // 	EListStylePosition _list_style_position :1;
     if (noninherited_flags.f._display == LIST_ITEM ) {
-	if ( !(inherited_flags.f._list_style_type == other->inherited_flags.f._list_style_type) ||
-	     !(inherited_flags.f._list_style_position == other->inherited_flags.f._list_style_position) )
-	    d = Layout;
+        if ( !(inherited_flags.f._list_style_type == other->inherited_flags.f._list_style_type) ||
+             !(inherited_flags.f._list_style_position == other->inherited_flags.f._list_style_position) )
+            return Layout;
     }
 
-    if ( d == Layout )
-	return d;
-
 // ### These could be better optimised
 // 	ETextAlign _text_align : 3;
 // 	ETextTransform _text_transform : 4;
 // 	EDirection _direction : 1;
 // 	EWhiteSpace _white_space : 2;
-// 	EFontVariant _font_variant : 1;
 //     EClear _clear : 2;
     if ( !(inherited_flags.f._text_align == other->inherited_flags.f._text_align) ||
 	 !(inherited_flags.f._text_transform == other->inherited_flags.f._text_transform) ||
 	 !(inherited_flags.f._direction == other->inherited_flags.f._direction) ||
 	 !(inherited_flags.f._white_space == other->inherited_flags.f._white_space) ||
-	 !(inherited_flags.f._font_variant == other->inherited_flags.f._font_variant) ||
 	 !(noninherited_flags.f._clear == other->noninherited_flags.f._clear)
 	)
-	d = Layout;
+        return Layout;
 
 // only for inline:
 //     EVerticalAlign _vertical_align : 4;
 
-    if ( !(noninherited_flags.f._display == INLINE) ) {
-	if ( !(noninherited_flags.f._vertical_align == other->noninherited_flags.f._vertical_align))
-	    d = Layout;
-    }
-
-    if ( d == Layout )
-	return d;
-
+    if ( !(noninherited_flags.f._display == INLINE) &&
+         !(noninherited_flags.f._vertical_align == other->noninherited_flags.f._vertical_align) )
+	    return Layout;
 
     // Visible:
 // 	EVisibility _visibility : 2;
 //     EOverflow _overflow : 4 ;
 //     EBackgroundRepeat _bg_repeat : 2;
 //     bool _bg_attachment : 1;
-// 	int _text_decoration : 4;
+// 	int _text_decorations : 4;
 //     DataRef<StyleBackgroundData> background;
-    if ( !(inherited_flags.f._visibility == other->inherited_flags.f._visibility) ||
-	 !(noninherited_flags.f._overflow == other->noninherited_flags.f._overflow) ||
-	 !(noninherited_flags.f._bg_repeat == other->noninherited_flags.f._bg_repeat) ||
-	 !(noninherited_flags.f._bg_attachment == other->noninherited_flags.f._bg_attachment) ||
-	 !(noninherited_flags.f._clipSpecified == other->noninherited_flags.f._clipSpecified) ||
-	 !(inherited_flags.f._text_decoration == other->inherited_flags.f._text_decoration) ||
-	 *background.get() != *other->background.get()
+    if (inherited->color != other->inherited->color ||
+        inherited->decoration_color != other->inherited->decoration_color ||
+        !(inherited_flags.f._visibility == other->inherited_flags.f._visibility) ||
+        !(noninherited_flags.f._overflow == other->noninherited_flags.f._overflow) ||
+        !(noninherited_flags.f._bg_repeat == other->noninherited_flags.f._bg_repeat) ||
+        !(noninherited_flags.f._bg_attachment == other->noninherited_flags.f._bg_attachment) ||
+        !(inherited_flags.f._text_decorations == other->inherited_flags.f._text_decorations) ||
+        !(noninherited_flags.f._hasClip == other->noninherited_flags.f._hasClip) ||
+        visual->textDecoration != other->visual->textDecoration ||
+        *background.get() != *other->background.get()
 	)
-	d = Visible;
+        return Visible;
 
-    // Position:
-
-    return d;
+    return Equal;
 }
 
 
 RenderStyle* RenderStyle::_default = 0;
-//int RenderStyle::counter = 0;
-//int SharedData::counter = 0;
 
 void RenderStyle::cleanup()
 {
     delete _default;
     _default = 0;
-//    counter = 0;
-//    SharedData::counter = 0;
 }
 
-void RenderStyle::setContent(CachedObject* o)
-{
-    if ( !content )
-	content = new ContentData;
-    else
-	content->clearContent();
-
-    content->_content.object = o;
-    content->_contentType = o ? CONTENT_OBJECT : CONTENT_NONE;
-}
-
-
 void RenderStyle::setPaletteColor(QPalette::ColorGroup g, QColorGroup::ColorRole r, const QColor& c)
 {
     visual.access()->palette.setColor(g,r,c);
@@ -453,21 +448,103 @@ void RenderStyle::setClip( Length top, Length right, Length bottom, Length left
     data->clip.left = left;
 }
 
-void RenderStyle::setContent(DOM::DOMStringImpl* s)
+bool RenderStyle::contentDataEquivalent(RenderStyle* otherStyle)
 {
-    if ( !content )
-	content = new ContentData;
-    else
-	content->clearContent();
+    ContentData* c1 = content;
+    ContentData* c2 = otherStyle->content;
 
-    content->_content.text = s;
+    while (c1 && c2) {
+        if (c1->_contentType != c2->_contentType)
+            return false;
+        if (c1->_contentType == CONTENT_TEXT) {
+            DOM::DOMString c1Str(c1->_content.text);
+            DOM::DOMString c2Str(c2->_content.text);
+            if (c1Str != c2Str)
+                return false;
+        }
+        else if (c1->_contentType == CONTENT_OBJECT) {
+            if (c1->_content.object != c2->_content.object)
+                return false;
+        }
 
-    if (s) {
-        content->_content.text->ref();
-        content->_contentType = CONTENT_TEXT;
+        c1 = c1->_nextContent;
+        c2 = c2->_nextContent;
+    }
+
+    return !c1 && !c2;
+}
+
+void RenderStyle::setContent(CachedObject* o, bool add)
+{
+    if (!o)
+        return; // The object is null. Nothing to do. Just bail.
+
+    ContentData* lastContent = content;
+    while (lastContent && lastContent->_nextContent)
+        lastContent = lastContent->_nextContent;
+
+    bool reuseContent = !add;
+    ContentData* newContentData = 0;
+    if (reuseContent && content) {
+        content->clearContent();
+        newContentData = content;
     }
     else
-        content->_contentType = CONTENT_NONE;
+        newContentData = new ContentData;
+
+    if (lastContent && !reuseContent)
+        lastContent->_nextContent = newContentData;
+    else
+        content = newContentData;
+
+    //    o->ref();
+    newContentData->_content.object = o;
+    newContentData->_contentType = CONTENT_OBJECT;
+}
+
+void RenderStyle::setContent(DOM::DOMStringImpl* s, bool add)
+{
+    if (!s)
+        return; // The string is null. Nothing to do. Just bail.
+
+    ContentData* lastContent = content;
+    while (lastContent && lastContent->_nextContent)
+        lastContent = lastContent->_nextContent;
+
+    bool reuseContent = !add;
+    if (add) {
+        if (!lastContent)
+            return; // Something's wrong.  We had no previous content, and we should have.
+
+        if (lastContent->_contentType == CONTENT_TEXT) {
+            // We can augment the existing string and share this ContentData node.
+            DOMStringImpl* oldStr = lastContent->_content.text;
+            DOMStringImpl* newStr = oldStr->copy();
+            newStr->ref();
+            oldStr->deref();
+            newStr->append(s);
+            lastContent->_content.text = newStr;
+            return;
+        }
+    }
+
+    ContentData* newContentData = 0;
+    if (reuseContent && content) {
+        content->clearContent();
+        newContentData = content;
+    }
+    else
+        newContentData = new ContentData;
+
+    if (lastContent && !reuseContent)
+        lastContent->_nextContent = newContentData;
+    else
+        content = newContentData;
+
+    newContentData->_content.text = s;
+    newContentData->_content.text->ref();
+    newContentData->_contentType = CONTENT_TEXT;
+
 }
 
 ContentData::~ContentData()
@@ -477,10 +554,12 @@ ContentData::~ContentData()
 
 void ContentData::clearContent()
 {
+    delete _nextContent;
+    _nextContent = 0;
+
     switch (_contentType)
     {
         case CONTENT_OBJECT:
-//            _content.object->deref();
             _content.object = 0;
             break;
         case CONTENT_TEXT:
@@ -490,3 +569,42 @@ void ContentData::clearContent()
             ;
     }
 }
+
+#ifdef ENABLE_DUMP
+
+static QString describeFont( const QFont &f)
+{
+    QString res = "'" + f.family() + "' ";
+
+    if ( f.pointSize() > 0)
+        res += QString::number( f.pointSize() ) + "pt";
+    else
+        res += QString::number( f.pixelSize() ) + "px";
+
+    if ( f.bold() )
+        res += " bold";
+    if ( f.italic() )
+        res += " italic";
+    if ( f.underline() )
+        res += " underline";
+    if ( f.overline() )
+        res += " overline";
+    if ( f.strikeOut() )
+        res += " strikeout";
+    return res;
+}
+
+QString RenderStyle::createDiff( const RenderStyle &parent ) const
+{
+    QString res;
+      if ( parent.color() != color() )
+        res += " [color=" + color().name() + "]";
+    if ( parent.backgroundColor() != backgroundColor() )
+        res += " [bgcolor=" + backgroundColor().name() + "]";
+    if ( parent.font() != font() )
+        res += " [font=" + describeFont( font() ) + "]";
+
+    return res;
+}
+
+#endif
