@@ -4,7 +4,7 @@
  * Copyright (C) 1997 Martin Jones (mjones@kde.org)
  *           (C) 1997 Torben Weis (weis@kde.org)
  *           (C) 1998 Waldo Bastian (bastian@kde.org)
- *           (C) 1999 Lars Knoll (knoll@kde.org)
+ *           (C) 1999-2003 Lars Knoll (knoll@kde.org)
  *           (C) 1999 Antti Koivisto (koivisto@kde.org)
  *
  * This library is free software; you can redistribute it and/or
@@ -22,7 +22,7 @@
  * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
  *
- * $Id: render_table.h,v 1.79.2.3 2003/08/17 14:38:51 mueller Exp $
+ * $Id: render_table.h,v 1.100.2.1 2004/02/29 15:28:03 mueller Exp $
  */
 #ifndef RENDER_TABLE_H
 #define RENDER_TABLE_H
@@ -30,9 +30,10 @@
 #include <qcolor.h>
 #include <qptrvector.h>
 
-#include "render_box.h"
-#include "render_flow.h"
-#include "render_style.h"
+#include "rendering/render_box.h"
+#include "rendering/render_block.h"
+#include "rendering/render_style.h"
+
 #include "misc/khtmllayout.h"
 
 namespace DOM {
@@ -48,7 +49,7 @@ class RenderTableCell;
 class RenderTableCol;
 class TableLayout;
 
-class RenderTable : public RenderFlow
+class RenderTable : public RenderBlock
 {
 public:
     enum Rules {
@@ -78,13 +79,13 @@ public:
 
     virtual void setStyle(RenderStyle *style);
 
-    virtual bool isRendered() const { return true; }
     virtual bool isTable() const { return true; }
 
     int getColumnPos(int col) const
         { return columnPos[col]; }
 
-    int cellSpacing() const { return spacing; }
+    int borderHSpacing() const { return hspacing; }
+    int borderVSpacing() const { return vspacing; }
 
     Rules getRules() const { return rules; }
 
@@ -95,23 +96,29 @@ public:
 
     // overrides
     virtual void addChild(RenderObject *child, RenderObject *beforeChild = 0);
-    virtual void paint( QPainter *, int x, int y, int w, int h,
-                        int tx, int ty);
+    virtual void paint( PaintInfo&, int tx, int ty);
     virtual void layout();
     virtual void calcMinMaxWidth();
     virtual void close();
 
+    virtual short lineHeight(bool b) const;
+    virtual short baselinePosition(bool b) const;
+
     virtual void setCellWidths( );
 
-    virtual void position(int x, int y, int from, int len, int width, bool reverse, bool firstLine, int);
+    virtual void position(InlineBox *, int, int, bool) {}
 
     virtual void calcWidth();
 
     virtual int borderTopExtra();
     virtual int borderBottomExtra();
 
-#ifndef NDEBUG
-    virtual void dump(QTextStream *stream, QString ind = "") const;
+    virtual FindSelectionResult checkSelectionPoint( int _x, int _y, int _tx, int _ty,
+                                                     DOM::NodeImpl*& node, int & offset,
+						     SelPointState & );
+
+#ifdef ENABLE_DUMP
+    virtual void dump(QTextStream &stream, const QString &ind) const;
 #endif
     struct ColumnStruct {
 	enum {
@@ -149,7 +156,7 @@ public:
     }
 
     int bordersAndSpacing() const {
-	return borderLeft() + borderRight() + (numEffCols()+1) * cellSpacing();
+	return borderLeft() + borderRight() + (numEffCols()+1) * borderHSpacing();
     }
 
     RenderTableCol *colElement( int col );
@@ -158,6 +165,8 @@ public:
 
     virtual RenderObject* removeChildNode(RenderObject* child);
 
+    RenderTableSection *firstBodySection() const { return firstBody; }
+
 protected:
 
     void recalcSections();
@@ -176,9 +185,13 @@ protected:
     Rules rules                 : 4;
 
     bool has_col_elems		: 1;
-    uint spacing                : 11;
-    uint padding		: 11;
     uint needSectionRecalc	: 1;
+    uint padding		: 22;
+
+    ushort hspacing;
+    ushort vspacing;
+
+    friend class TableSectionIterator;
 };
 
 // -------------------------------------------------------------------------
@@ -199,10 +212,14 @@ public:
     virtual bool isTableSection() const { return true; }
 
     virtual short lineHeight(bool) const { return 0; }
-    virtual void position(int, int, int, int, int, bool, bool, int) {}
+    virtual void position(InlineBox*, int, int, bool) {}
 
-#ifndef NDEBUG
-    virtual void dump(QTextStream *stream, QString ind = "") const;
+    virtual FindSelectionResult checkSelectionPoint( int _x, int _y, int _tx, int _ty,
+                                                     DOM::NodeImpl*& node, int & offset,
+						     SelPointState & );
+
+#ifdef ENABLE_DUMP
+    virtual void dump(QTextStream &stream, const QString &ind) const;
 #endif
 
     void addCell( RenderTableCell *cell );
@@ -227,8 +244,7 @@ public:
 	return (*(grid[row].row))[col];
     }
 
-    virtual void paint( QPainter *, int x, int y, int w, int h,
-                        int tx, int ty);
+    virtual void paint( PaintInfo& i, int tx, int ty);
 
     int numRows() const { return grid.size(); }
     int getBaseline(int row) {return grid[row].baseLine;}
@@ -253,6 +269,8 @@ public:
 protected:
     void ensureRows( int numRows );
     void clearGrid();
+
+    friend class TableSectionIterator;
 };
 
 // -------------------------------------------------------------------------
@@ -274,25 +292,26 @@ public:
     virtual RenderObject* removeChildNode(RenderObject* child);
 
     virtual short lineHeight( bool ) const { return 0; }
-    virtual void position(int, int, int, int, int, bool, bool, int) {}
+    virtual void position(InlineBox*, int, int, bool) {}
 
     virtual void layout();
 
     RenderTable *table() const { return static_cast<RenderTable *>(parent()->parent()); }
     RenderTableSection *section() const { return static_cast<RenderTableSection *>(parent()); }
 
-#ifndef NDEBUG
-    virtual void dump(QTextStream *stream, QString ind = "") const;
+#ifdef ENABLE_DUMP
+    virtual void dump(QTextStream &stream, const QString& ind) const;
 #endif
 };
 
 // -------------------------------------------------------------------------
 
-class RenderTableCell : public RenderFlow
+class RenderTableCell : public RenderBlock
 {
 public:
     RenderTableCell(DOM::NodeImpl* node);
 
+    virtual void layout();
     virtual void detach();
 
     virtual const char *renderName() const { return "RenderTableCell"; }
@@ -308,9 +327,6 @@ public:
     unsigned short rowSpan() const { return rSpan; }
     void setRowSpan( unsigned short r ) { rSpan = r; }
 
-    bool noWrap() const { return nWrap; }
-    void setNoWrap(bool nw) { nWrap = nw; }
-
     int col() const { return _col; }
     void setCol(int col) { _col = col; }
     int row() const { return _row; }
@@ -327,15 +343,14 @@ public:
     void setCellTopExtra(int p) { _topExtra = p; }
     void setCellBottomExtra(int p) { _bottomExtra = p; }
 
-    virtual void paint( QPainter* p, int x, int y,
-                        int w, int h, int tx, int ty);
+    virtual void paint( PaintInfo& i, int tx, int ty);
 
     virtual void close();
 
     // lie position to outside observers
     virtual int yPos() const { return m_y + _topExtra; }
 
-    virtual void repaintRectangle(int x, int y, int w, int h, bool f=false);
+    virtual void repaintRectangle(int x, int y, int w, int h, bool immediate=false, bool f=false);
     virtual bool absolutePosition(int &xPos, int &yPos, bool f = false);
 
     virtual short baselinePosition( bool = false ) const;
@@ -343,8 +358,8 @@ public:
     RenderTable *table() const { return static_cast<RenderTable *>(parent()->parent()->parent()); }
     RenderTableSection *section() const { return static_cast<RenderTableSection *>(parent()->parent()); }
 
-#ifndef NDEBUG
-    virtual void dump(QTextStream *stream, QString ind = "") const;
+#ifdef ENABLE_DUMP
+    virtual void dump(QTextStream &stream, const QString &ind) const;
 #endif
 
     bool widthChanged() {
@@ -353,9 +368,13 @@ public:
 	return retval;
     }
 
+    int cellPercentageHeight() const
+	{ return m_percentageHeight; }
+    void setCellPercentageHeight(int h)
+	{ m_percentageHeight = h; }
+
 protected:
-    virtual void paintBoxDecorations(QPainter *p,int _x, int _y,
-                                     int _w, int _h, int _tx, int _ty);
+    virtual void paintBoxDecorations(PaintInfo& p, int _tx, int _ty);
     virtual int borderTopExtra() { return _topExtra; }
     virtual int borderBottomExtra() { return _bottomExtra; }
 
@@ -363,10 +382,10 @@ protected:
     short _col;
     ushort rSpan;
     ushort cSpan;
-    signed int _topExtra : 31;
-    bool nWrap : 1;
+    int _topExtra;
     signed int _bottomExtra : 31;
     bool m_widthChanged : 1;
+    int m_percentageHeight;
 };
 
 
@@ -387,19 +406,67 @@ public:
     virtual bool isTableCol() const { return true; }
 
     virtual short lineHeight( bool ) const { return 0; }
-    virtual void position(int, int, int, int, int, bool, bool, int) {}
+    virtual void position(InlineBox*, int, int, bool) {}
     virtual void layout() {}
 
     virtual void updateFromElement();
 
-#ifndef NDEBUG
-    virtual void dump(QTextStream *stream, QString ind = "") const;
+#ifdef ENABLE_DUMP
+    virtual void dump(QTextStream &stream, const QString& ind) const;
 #endif
 
 protected:
     short _span;
 };
 
+// -------------------------------------------------------------------------
+
+/** This class provides an iterator to iterate through the sections of a
+ * render table in their visual order.
+ *
+ * In HTML, sections are specified in the order of THEAD, TFOOT, and TBODY.
+ * Visually, TBODY sections appear between THEAD and TFOOT, which this iterator
+ * takes into regard.
+ * @author Leo Savernik
+ * @internal
+ * @since 3.2
+ */
+class TableSectionIterator {
+public:
+
+  /**
+   * Initializes a new iterator
+   * @param table table whose sections to iterate
+   * @param fromEnd @p true, begin with last section, @p false, begin with
+   *	first section.
+   */
+  TableSectionIterator(RenderTable *table, bool fromEnd = false);
+
+  /**
+   * Initializes a new iterator
+   * @param start table section to start with.
+   */
+  TableSectionIterator(RenderTableSection *start) : sec(start) {}
+
+  /**
+   * Uninitialized iterator.
+   */
+  TableSectionIterator() {}
+
+  /** Returns the current section, or @p 0 if the end has been reached.
+   */
+  RenderTableSection *operator *() const { return sec; }
+
+  /** Moves to the next section in visual order. */
+  TableSectionIterator &operator ++();
+
+  /** Moves to the previous section in visual order. */
+  TableSectionIterator &operator --();
+
+private:
+  RenderTableSection *sec;
+};
+
 }
 #endif
 
