@@ -19,7 +19,7 @@
  * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
  *
- * $Id: table_layout.cpp,v 1.24.2.3 2003/07/18 16:06:39 staikos Exp $
+ * $Id: table_layout.cpp,v 1.36 2003/10/19 22:09:25 mueller Exp $
  */
 #include "table_layout.h"
 #include "render_table.h"
@@ -73,7 +73,7 @@ using namespace khtml;
 
 _____________________________________________________
 
-  This is not quite true when comparing to IE. IE always honours
+  This is not quite true when comparing to IE. IE always honors
   table-layout:fixed and treats a variable table width as 100%. Makes
   a lot of sense, and is implemented here the same way.
 
@@ -88,7 +88,7 @@ FixedTableLayout::~FixedTableLayout()
 {
 }
 
-int FixedTableLayout::calcWidthArray( int tableWidth )
+int FixedTableLayout::calcWidthArray( int /*tableWidth*/ )
 {
     int usedWidth = 0;
 
@@ -234,15 +234,12 @@ void FixedTableLayout::calcMinMaxWidth()
     // unlimited.
 
     int bs = table->bordersAndSpacing();
-    table->m_minWidth = 0;
-    table->m_maxWidth = 0;
-    short tableWidth = table->style()->width().isFixed() ? table->style()->width().value() - bs : 0;
+    int tableWidth = table->style()->width().isFixed() ? table->style()->width().value() - bs : 0;
 
-    table->m_minWidth = calcWidthArray( tableWidth );
-    table->m_minWidth += bs;
-
-    table->m_minWidth = kMax( table->m_minWidth, tableWidth );
+    int mw = calcWidthArray( tableWidth ) + bs;
+    table->m_minWidth = kMin( kMax( mw, tableWidth ), 0x7fff );
     table->m_maxWidth = table->m_minWidth;
+
     if ( !tableWidth ) {
 	bool haveNonFixed = false;
 	for ( unsigned int i = 0; i < width.size(); i++ ) {
@@ -269,18 +266,10 @@ void FixedTableLayout::layout()
 #endif
 
 
-    QMemArray<short> calcWidth;
+    QMemArray<int> calcWidth;
     calcWidth.resize( nEffCols );
     calcWidth.fill( -1 );
 
-    // first assign  fixed width
-    for ( int i = 0; i < nEffCols; i++ ) {
-	if ( width[i].isFixed() ) {
-	    calcWidth[i] = width[i].value();
-	    available -= width[i].value();
-	}
-    }
-
     // assign  percent width
     if ( available > 0 ) {
 	int totalPercent = 0;
@@ -307,6 +296,14 @@ void FixedTableLayout::layout()
         }
     }
 
+    // next assign  fixed width
+    for ( int i = 0; i < nEffCols; i++ ) {
+	if ( width[i].isFixed() ) {
+	    calcWidth[i] = width[i].value();
+	    available -= width[i].value();
+	}
+    }
+
     // assign  variable width
     if ( available > 0 ) {
 	int totalVariable = 0;
@@ -328,14 +325,27 @@ void FixedTableLayout::layout()
 	if ( calcWidth[i] <= 0 )
 	    calcWidth[i] = 0; // IE gives min 1 px...
 
+    // spread extra space over columns
+    if ( available > 0 ) {
+        int total = nEffCols;
+        // still have some width to spread
+        int i = nEffCols;
+        while (  i-- ) {
+            int w = available / total;
+            available -= w;
+            total--;
+            calcWidth[i] += w;
+        }
+    }
+
     int pos = 0;
-    int spacing = table->cellSpacing();
+    int hspacing = table->borderHSpacing();
     for ( int i = 0; i < nEffCols; i++ ) {
 #ifdef DEBUG_LAYOUT
 	qDebug("col %d: %d (width %d)", i, pos, calcWidth[i] );
 #endif
 	table->columnPos[i] = pos;
-	pos += calcWidth[i] + spacing;
+	pos += calcWidth[i] + hspacing;
     }
     table->columnPos[table->columnPos.size()-1] = pos;
 }
@@ -381,6 +391,11 @@ void AutoTableLayout::recalcColumn( int effCol )
 		if ( cell == (RenderTableCell *)-1 )
 		    continue;
 		if ( cell && cell->colSpan() == 1 ) {
+                    // A cell originates in this column.  Ensure we have
+                    // a min/max width of at least 1px for this column now.
+                    l.minWidth = kMax(int( l.minWidth ), 1);
+                    l.maxWidth = kMax(int( l.maxWidth ), 1);
+
 		    if ( !cell->minMaxKnown() )
 			cell->calcMinMaxWidth();
 		    if ( cell->minWidth() > l.minWidth )
@@ -397,7 +412,7 @@ void AutoTableLayout::recalcColumn( int effCol )
 			// ignore width=0
 			if ( w.value() > 0 && !l.width.isPercent() ) {
                             // ### we should use box'es paddings here I guess.
-                            int wval = w.value() + table->cellPadding() * 2;
+                            int wval = w.value() + cell->paddingLeft() + cell->paddingRight();
 			    if ( l.width.isFixed() ) {
                                 // Nav/IE weirdness
 				if ((wval > l.width.value()) ||
@@ -423,8 +438,13 @@ void AutoTableLayout::recalcColumn( int effCol )
 			break;
 		    }
 		} else {
-		    if ( !effCol || section->cellAt( i, effCol-1 ) != cell )
+		    if ( cell && (!effCol || section->cellAt( i, effCol-1 ) != cell) ) {
+                        // This spanning cell originates in this column.  Ensure we have
+                        // a min/max width of at least 1px for this column now.
+                        l.minWidth = kMax(int( l.minWidth ), 1);
+                        l.maxWidth = kMax(int( l.maxWidth ), 1);
 			insertSpanCell( cell );
+		    }
 		    last = cell;
 		}
 	    }
@@ -509,6 +529,38 @@ void AutoTableLayout::fullRecalc()
 	recalcColumn( i );
 }
 
+static bool shouldScaleColumns(RenderTable* table)
+{
+    // A special case.  If this table is not fixed width and contained inside
+    // a cell, then don't bloat the maxwidth by examining percentage growth.
+    bool scale = true;
+    while (table) {
+        Length tw = table->style()->width();
+        if ((tw.isVariable() || tw.isPercent()) && !table->isPositioned()) {
+            RenderObject* cb = table->containingBlock();
+            while (cb && !cb->isCanvas() && !cb->isTableCell() &&
+                cb->style()->width().isVariable() && !cb->isPositioned())
+                cb = cb->containingBlock();
+
+            table = 0;
+            if (cb && cb->isTableCell() &&
+                (cb->style()->width().isVariable() || cb->style()->width().isPercent())) {
+                if (tw.isPercent())
+                    scale = false;
+                else {
+                    RenderTableCell* cell = static_cast<RenderTableCell*>(cb);
+                    if (cell->colSpan() > 1 || cell->table()->style()->width().isVariable())
+                        scale = false;
+                    else
+                        table = cell->table();
+                }
+            }
+        }
+        else
+            table = 0;
+    }
+    return scale;
+}
 
 void AutoTableLayout::calcMinMaxWidth()
 {
@@ -523,23 +575,26 @@ void AutoTableLayout::calcMinMaxWidth()
     int maxPercent = 0;
     int maxNonPercent = 0;
 
+    int remainingPercent = 100;
     for ( unsigned int i = 0; i < layoutStruct.size(); i++ ) {
 	minWidth += layoutStruct[i].effMinWidth;
 	maxWidth += layoutStruct[i].effMaxWidth;
 	if ( layoutStruct[i].effWidth.isPercent() ) {
-	    int pw = ( layoutStruct[i].effMaxWidth * 100) / layoutStruct[i].effWidth.value();
+            int percent = kMin(layoutStruct[i].effWidth.value(), remainingPercent);
+            int pw = ( layoutStruct[i].effMaxWidth * 100) / kMax(percent, 1);
+            remainingPercent -= percent;
 	    maxPercent = kMax( pw,  maxPercent );
 	} else {
 	    maxNonPercent += layoutStruct[i].effMaxWidth;
 	}
     }
 
-    int totalpct = totalPercent();
-    if ( totalpct < 100 ) {
-	maxNonPercent = (maxNonPercent * 100 + 50) / (100-totalpct);
-	maxWidth = kMax( maxNonPercent,  maxWidth );
+    if (false && shouldScaleColumns(table)) {
+        maxNonPercent = (maxNonPercent * 100 + 50) / kMax(remainingPercent, 1);
+	maxWidth = kMax( maxNonPercent, maxWidth );
+	maxWidth = kMax( maxWidth, maxPercent );
     }
-    maxWidth = kMax( maxWidth, maxPercent );
+
     maxWidth = kMax( maxWidth, spanMaxWidth );
 
     int bs = table->bordersAndSpacing();
@@ -552,8 +607,8 @@ void AutoTableLayout::calcMinMaxWidth()
 	maxWidth = minWidth;
     }
 
-    table->m_maxWidth = maxWidth;
-    table->m_minWidth = minWidth;
+    table->m_maxWidth = kMin(maxWidth, 0x7fff);
+    table->m_minWidth = kMin(minWidth, 0x7fff);
 #ifdef DEBUG_LAYOUT
     qDebug("    minWidth=%d, maxWidth=%d", table->m_minWidth, table->m_maxWidth );
 #endif
@@ -568,7 +623,7 @@ int AutoTableLayout::calcEffectiveWidth()
     int tMaxWidth = 0;
 
     unsigned int nEffCols = layoutStruct.size();
-    int spacing = table->cellSpacing();
+    int hspacing = table->borderHSpacing();
 #ifdef DEBUG_LAYOUT
     qDebug("AutoTableLayout::calcEffectiveWidth for %d cols", nEffCols );
 #endif
@@ -590,8 +645,8 @@ int AutoTableLayout::calcEffectiveWidth()
 
 	int col = table->colToEffCol( cell->col() );
 	unsigned int lastCol = col;
-	int cMinWidth = cell->minWidth() + spacing;
-	int cMaxWidth = cell->maxWidth() + spacing;
+	int cMinWidth = cell->minWidth() + hspacing;
+	int cMaxWidth = cell->maxWidth() + hspacing;
 	int totalPercent = 0;
 	int minWidth = 0;
 	int maxWidth = 0;
@@ -613,7 +668,7 @@ int AutoTableLayout::calcEffectiveWidth()
                     fixedWidth += layoutStruct[lastCol].width.value();
                     allColsArePercent = false;
                     // IE resets effWidth to Variable here, but this breaks the konqueror about page and seems to be some bad
-                    // legacy behaviour anyway. mozilla doesn't do this so I decided we don't neither.
+                    // legacy behavior anyway. mozilla doesn't do this so I decided we don't neither.
                     break;
                 }
                 // fall through
@@ -629,8 +684,8 @@ int AutoTableLayout::calcEffectiveWidth()
 	    minWidth += layoutStruct[lastCol].effMinWidth;
 	    maxWidth += layoutStruct[lastCol].effMaxWidth;
 	    lastCol++;
-	    cMinWidth -= spacing;
-	    cMaxWidth -= spacing;
+	    cMinWidth -= hspacing;
+	    cMaxWidth -= hspacing;
 	}
 #ifdef DEBUG_LAYOUT
 	qDebug("    colspan cell %p at effCol %d, span %d, type %d, value %d cmin=%d min=%d fixedwidth=%d", cell, col, cSpan, w.type, w.value, cMinWidth, minWidth, fixedWidth );
@@ -642,11 +697,11 @@ int AutoTableLayout::calcEffectiveWidth()
 		// can't satify this condition, treat as variable
 		w = Length();
 	    } else {
-		int spanMax = QMAX( maxWidth, cMaxWidth );
+		int spanMax = kMax( maxWidth, cMaxWidth );
 #ifdef DEBUG_LAYOUT
 		qDebug("    adjusting tMaxWidth (%d): spanMax=%d, value=%d, totalPercent=%d", tMaxWidth, spanMax, w.value(), totalPercent );
 #endif
-		tMaxWidth = QMAX( tMaxWidth, spanMax * 100 / w.value() );
+		tMaxWidth = kMax( tMaxWidth, spanMax * 100 / w.value() );
 
 		// all non percent columns in the span get percent vlaues to sum up correctly.
 		int percentMissing = w.value() - totalPercent;
@@ -681,7 +736,7 @@ int AutoTableLayout::calcEffectiveWidth()
 		qDebug("extending minWidth of cols %d-%d to %dpx currentMin=%d accroding to fixed sum %d", col, lastCol-1, cMinWidth, minWidth, fixedWidth );
 #endif
 		for ( unsigned int pos = col; fixedWidth > 0 && pos < lastCol; pos++ ) {
-		    int w = QMAX( layoutStruct[pos].effMinWidth, cMinWidth * layoutStruct[pos].width.value() / fixedWidth );
+		    int w = kMax( int( layoutStruct[pos].effMinWidth ), cMinWidth * layoutStruct[pos].width.value() / fixedWidth );
 #ifdef DEBUG_LAYOUT
 		    qDebug("   col %d: min=%d, effMin=%d, new=%d", pos, layoutStruct[pos].effMinWidth, layoutStruct[pos].effMinWidth, w );
 #endif
@@ -695,21 +750,34 @@ int AutoTableLayout::calcEffectiveWidth()
 		qDebug("extending minWidth of cols %d-%d to %dpx currentMin=%d", col, lastCol-1, cMinWidth, minWidth );
 #endif
 		int maxw = maxWidth;
-		for ( unsigned int pos = col; minWidth > 0 && pos < lastCol; pos++ ) {
-
-		    int w;
+		int minw = minWidth;
+		for ( unsigned int pos = col; maxw > 0 && pos < lastCol; pos++ ) {
 		    if ( layoutStruct[pos].width.isFixed() && haveVariable && fixedWidth <= cMinWidth ) {
-			w = QMAX( layoutStruct[pos].effMinWidth, layoutStruct[pos].width.value() );
+			int w = kMax( int( layoutStruct[pos].effMinWidth ), layoutStruct[pos].width.value() );
 			fixedWidth -= layoutStruct[pos].width.value();
-		    } else {
-			w = QMAX( layoutStruct[pos].effMinWidth, cMinWidth * layoutStruct[pos].effMaxWidth / maxw );
-		    }
+                        minw -= layoutStruct[pos].effMinWidth;
 #ifdef DEBUG_LAYOUT
-		    qDebug("   col %d: min=%d, effMin=%d, new=%d", pos, layoutStruct[pos].effMinWidth, layoutStruct[pos].effMinWidth, w );
+			qDebug("   col %d: min=%d, effMin=%d, new=%d", pos, layoutStruct[pos].effMinWidth, layoutStruct[pos].effMinWidth, w );
 #endif
-		    maxw -= layoutStruct[pos].effMaxWidth;
-		    cMinWidth -= w;
-		    layoutStruct[pos].effMinWidth = w;
+			maxw -= layoutStruct[pos].effMaxWidth;
+			cMinWidth -= w;
+			layoutStruct[pos].effMinWidth = w;
+		    }
+		}
+
+		for ( unsigned int pos = col; maxw > 0 && pos < lastCol && minw < cMinWidth; pos++ ) {
+		    if ( !(layoutStruct[pos].width.isFixed() && haveVariable && fixedWidth <= cMinWidth) ) {
+			int w = kMax( int( layoutStruct[pos].effMinWidth ), cMinWidth * layoutStruct[pos].effMaxWidth / maxw );
+                        w = kMin(layoutStruct[pos].effMinWidth+(cMinWidth-minw), w);
+
+#ifdef DEBUG_LAYOUT
+			qDebug("   col %d: min=%d, effMin=%d, new=%d", pos, layoutStruct[pos].effMinWidth, layoutStruct[pos].effMinWidth, w );
+#endif
+			maxw -= layoutStruct[pos].effMaxWidth;
+                        minw -= layoutStruct[pos].effMinWidth;
+			cMinWidth -= w;
+			layoutStruct[pos].effMinWidth = w;
+		    }
 		}
 	    }
 	}
@@ -719,7 +787,7 @@ int AutoTableLayout::calcEffectiveWidth()
 		qDebug("extending maxWidth of cols %d-%d to %dpx", col, lastCol-1, cMaxWidth );
 #endif
 		for ( unsigned int pos = col; maxWidth > 0 && pos < lastCol; pos++ ) {
-		    int w = QMAX( layoutStruct[pos].effMaxWidth, cMaxWidth * layoutStruct[pos].effMaxWidth / maxWidth );
+		    int w = kMax( int( layoutStruct[pos].effMaxWidth ), cMaxWidth * layoutStruct[pos].effMaxWidth / maxWidth );
 #ifdef DEBUG_LAYOUT
 		    qDebug("   col %d: max=%d, effMax=%d, new=%d", pos, layoutStruct[pos].effMaxWidth, layoutStruct[pos].effMaxWidth, w );
 #endif
@@ -730,7 +798,7 @@ int AutoTableLayout::calcEffectiveWidth()
 	    }
 	} else {
 	    for ( unsigned int pos = col; pos < lastCol; pos++ )
-		layoutStruct[pos].maxWidth = QMAX(layoutStruct[pos].maxWidth, layoutStruct[pos].minWidth );
+		layoutStruct[pos].maxWidth = kMax(layoutStruct[pos].maxWidth, layoutStruct[pos].minWidth );
 	}
     }
     effWidthDirty = false;
@@ -835,24 +903,10 @@ void AutoTableLayout::layout()
         }
     }
 
-    // then allocate width to fixed cols
-    if ( available > 0 ) {
-	for ( int i = 0; i < nEffCols; ++i ) {
-	    Length w = layoutStruct[i].effWidth;
-	    if ( w.isFixed() && w.value() > layoutStruct[i].calcWidth ) {
-		available += layoutStruct[i].calcWidth - w.value();
-		layoutStruct[i].calcWidth = w.value();
-            }
-	}
-    }
-#ifdef DEBUG_LAYOUT
-    qDebug("fixed satisfied: available is %d", available);
-#endif
-
-    // then allocate width to percent cols
+    // allocate width to percent cols
     if ( available > 0 && havePercent ) {
 	for ( int i = 0; i < nEffCols; i++ ) {
-	    Length width = layoutStruct[i].effWidth;
+	    const Length &width = layoutStruct[i].effWidth;
 	    if ( width.isPercent() ) {
                 int w = kMax ( int( layoutStruct[i].effMinWidth ), width.minWidth( tableWidth ) );
 		available += layoutStruct[i].calcWidth - w;
@@ -880,10 +934,24 @@ void AutoTableLayout::layout()
     qDebug("percent satisfied: available is %d", available);
 #endif
 
+    // then allocate width to fixed cols
+    if ( available > 0 ) {
+	for ( int i = 0; i < nEffCols; ++i ) {
+	    const Length &width = layoutStruct[i].effWidth;
+	    if ( width.isFixed() && width.value() > layoutStruct[i].calcWidth ) {
+		available += layoutStruct[i].calcWidth - width.value();
+		layoutStruct[i].calcWidth = width.value();
+            }
+	}
+    }
+#ifdef DEBUG_LAYOUT
+    qDebug("fixed satisfied: available is %d", available);
+#endif
+
     // now satisfy relative
     if ( available > 0 ) {
 	for ( int i = 0; i < nEffCols; i++ ) {
-	    Length &width = layoutStruct[i].effWidth;
+	    const Length &width = layoutStruct[i].effWidth;
 	    if ( width.isRelative() && width.value() ) {
 		// width=0* gets effMinWidth.
 		int w = width.value()*tableWidth/totalRelative;
@@ -898,7 +966,7 @@ void AutoTableLayout::layout()
 	available += allocVariable; // this gets redistributed
  	//qDebug("redistributing %dpx to %d variable columns. totalVariable=%d",  available,  numVariable,  totalVariable );
 	for ( int i = 0; i < nEffCols; i++ ) {
-	    Length &width = layoutStruct[i].effWidth;
+	    const Length &width = layoutStruct[i].effWidth;
 	    if ( width.isVariable() && totalVariable != 0 ) {
 		int w = kMax( int ( layoutStruct[i].calcWidth ),
                               available * layoutStruct[i].effMaxWidth / totalVariable );
@@ -912,29 +980,11 @@ void AutoTableLayout::layout()
     qDebug("variable satisfied: available is %d",  available );
 #endif
 
-    // spread over fixed colums
-    if ( available > 0 && numFixed) {
-        // still have some width to spread, distribute to fixed columns
-        for ( int i = 0; i < nEffCols; i++ ) {
-            Length &width = layoutStruct[i].effWidth;
-            if ( width.isFixed() ) {
-                int w = available * layoutStruct[i].effMaxWidth / totalFixed;
-                available -= w;
-                totalFixed -= layoutStruct[i].effMaxWidth;
-                layoutStruct[i].calcWidth += w;
-            }
-        }
-    }
-
-#ifdef DEBUG_LAYOUT
-    qDebug("after fixed distribution: available=%d",  available );
-#endif
-
     // spread over percent colums
     if ( available > 0 && hasPercent && totalPercent < 100) {
         // still have some width to spread, distribute weighted to percent columns
         for ( int i = 0; i < nEffCols; i++ ) {
-            Length &width = layoutStruct[i].effWidth;
+            const Length &width = layoutStruct[i].effWidth;
             if ( width.isPercent() ) {
                 int w = available * width.value() / totalPercent;
                 available -= w;
@@ -949,6 +999,24 @@ void AutoTableLayout::layout()
     qDebug("after percent distribution: available=%d",  available );
 #endif
 
+    // spread over fixed colums
+    if ( available > 0 && numFixed) {
+        // still have some width to spread, distribute to fixed columns
+        for ( int i = 0; i < nEffCols; i++ ) {
+            const Length &width = layoutStruct[i].effWidth;
+            if ( width.isFixed() ) {
+                int w = available * layoutStruct[i].effMaxWidth / totalFixed;
+                available -= w;
+                totalFixed -= layoutStruct[i].effMaxWidth;
+                layoutStruct[i].calcWidth += w;
+            }
+        }
+    }
+
+#ifdef DEBUG_LAYOUT
+    qDebug("after fixed distribution: available=%d",  available );
+#endif
+
     // spread over the rest
     if ( available > 0 ) {
         int total = nEffCols;
@@ -968,18 +1036,99 @@ void AutoTableLayout::layout()
     // if we have overallocated, reduce every cell according to the difference between desired width and minwidth
     // this seems to produce to the pixel exaxt results with IE. Wonder is some of this also holds for width distributing.
     if ( available < 0 ) {
-	int mw = 0;
-	for ( int i = nEffCols-1; i >= 0; i-- )
-	    mw += layoutStruct[i].calcWidth - layoutStruct[i].effMinWidth;
-	for ( int i = nEffCols-1; i >= 0 && mw > 0; i-- ) {
-	    int minMaxDiff = layoutStruct[i].calcWidth-layoutStruct[i].effMinWidth;
-	    int reduce = available * minMaxDiff / mw;
-	    layoutStruct[i].calcWidth += reduce;
-	    available -= reduce;
-	    mw -= minMaxDiff;
-	    if ( available >= 0 )
-		break;
-	}
+        // Need to reduce cells with the following prioritization:
+        // (1) Variable
+        // (2) Relative
+        // (3) Fixed
+        // (4) Percent
+        // This is basically the reverse of how we grew the cells.
+        if (available < 0) {
+            int mw = 0;
+            for ( int i = nEffCols-1; i >= 0; i-- ) {
+                Length &width = layoutStruct[i].effWidth;
+                if (width.isVariable())
+                    mw += layoutStruct[i].calcWidth - layoutStruct[i].effMinWidth;
+            }
+
+            for ( int i = nEffCols-1; i >= 0 && mw > 0; i-- ) {
+                Length &width = layoutStruct[i].effWidth;
+                if (width.isVariable()) {
+                    int minMaxDiff = layoutStruct[i].calcWidth-layoutStruct[i].effMinWidth;
+                    int reduce = available * minMaxDiff / mw;
+                    layoutStruct[i].calcWidth += reduce;
+                    available -= reduce;
+                    mw -= minMaxDiff;
+                    if ( available >= 0 )
+                        break;
+                }
+            }
+        }
+
+        if (available < 0) {
+            int mw = 0;
+            for ( int i = nEffCols-1; i >= 0; i-- ) {
+                Length &width = layoutStruct[i].effWidth;
+                if (width.isRelative())
+                    mw += layoutStruct[i].calcWidth - layoutStruct[i].effMinWidth;
+            }
+
+            for ( int i = nEffCols-1; i >= 0 && mw > 0; i-- ) {
+                Length &width = layoutStruct[i].effWidth;
+                if (width.isRelative()) {
+                    int minMaxDiff = layoutStruct[i].calcWidth-layoutStruct[i].effMinWidth;
+                    int reduce = available * minMaxDiff / mw;
+                    layoutStruct[i].calcWidth += reduce;
+                    available -= reduce;
+                    mw -= minMaxDiff;
+                    if ( available >= 0 )
+                        break;
+                }
+            }
+        }
+
+        if (available < 0) {
+            int mw = 0;
+            for ( int i = nEffCols-1; i >= 0; i-- ) {
+                Length &width = layoutStruct[i].effWidth;
+                if (width.isFixed())
+                    mw += layoutStruct[i].calcWidth - layoutStruct[i].effMinWidth;
+            }
+
+            for ( int i = nEffCols-1; i >= 0 && mw > 0; i-- ) {
+                Length &width = layoutStruct[i].effWidth;
+                if (width.isFixed()) {
+                    int minMaxDiff = layoutStruct[i].calcWidth-layoutStruct[i].effMinWidth;
+                    int reduce = available * minMaxDiff / mw;
+                    layoutStruct[i].calcWidth += reduce;
+                    available -= reduce;
+                    mw -= minMaxDiff;
+                    if ( available >= 0 )
+                        break;
+                }
+            }
+        }
+
+        if (available < 0) {
+            int mw = 0;
+            for ( int i = nEffCols-1; i >= 0; i-- ) {
+                Length &width = layoutStruct[i].effWidth;
+                if (width.isPercent())
+                    mw += layoutStruct[i].calcWidth - layoutStruct[i].effMinWidth;
+            }
+
+            for ( int i = nEffCols-1; i >= 0 && mw > 0; i-- ) {
+                Length &width = layoutStruct[i].effWidth;
+                if (width.isPercent()) {
+                    int minMaxDiff = layoutStruct[i].calcWidth-layoutStruct[i].effMinWidth;
+                    int reduce = available * minMaxDiff / mw;
+                    layoutStruct[i].calcWidth += reduce;
+                    available -= reduce;
+                    mw -= minMaxDiff;
+                    if ( available >= 0 )
+                        break;
+                }
+            }
+        }
     }
 
     //qDebug( "    final available=%d", available );
@@ -990,7 +1139,7 @@ void AutoTableLayout::layout()
 	qDebug("col %d: %d (width %d)", i, pos, layoutStruct[i].calcWidth );
 #endif
 	table->columnPos[i] = pos;
-	pos += layoutStruct[i].calcWidth + table->cellSpacing();
+	pos += layoutStruct[i].calcWidth + table->borderHSpacing();
     }
     table->columnPos[table->columnPos.size()-1] = pos;
 
