@@ -19,7 +19,7 @@
  * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
  *
- * $Id: table_layout.h,v 1.4.2.1 2003/05/18 12:34:40 mueller Exp $
+ * $Id: table_layout.h,v 1.5.2.1 2004/02/29 15:28:04 mueller Exp $
  */
 #ifndef TABLE_LAYOUT_H
 #define TABLE_LAYOUT_H
@@ -38,7 +38,7 @@ class TableLayout
 {
 public:
     TableLayout( RenderTable *t ) : table( t ) {}
-    virtual ~TableLayout() {};
+    virtual ~TableLayout() {}
 
     virtual void calcMinMaxWidth() = 0;
     virtual void layout() = 0;
