@@ -1,4 +1,4 @@
-// programm to test the new khtml implementation
+// program to test the new khtml implementation
 
 #include <stdlib.h>
 #include "decoder.h"
@@ -19,21 +19,24 @@
 #include "dom/html_document.h"
 #include "dom/dom_exception.h"
 #include <stdio.h>
-#include <khtml_factory.h>
-#include <css/cssstyleselector.h>
-#include <html/html_imageimpl.h>
-#include <rendering/render_style.h>
+#define protected public
+#include "khtml_factory.h"
+#undef protected
+#include "css/cssstyleselector.h"
+#include "html/html_imageimpl.h"
+#include "rendering/render_style.h"
 #include <kmainwindow.h>
 #include <kcmdlineargs.h>
 #include <kaction.h>
 #include "domtreeview.h"
 
-static KCmdLineOptions options[] = { { "+file", "url to open", 0 } , {0, 0, 0} };
+static KCmdLineOptions options[] = { { "+file", "url to open", 0 } , KCmdLineLastOption };
 
 int main(int argc, char *argv[])
 {
 
-    KCmdLineArgs::init(argc, argv, "Testkhtml", "a basic web browser using the KHTML library", "1.0");
+    KCmdLineArgs::init(argc, argv, "testkhtml", "Testkhtml",
+            "a basic web browser using the KHTML library", "1.0");
     KCmdLineArgs::addCmdLineOptions(options);
 
     KApplication a;
@@ -43,7 +46,7 @@ int main(int argc, char *argv[])
 	::exit( 1 );
     }
 
-    KHTMLFactory *fac = new KHTMLFactory();
+    KHTMLFactory *fac = new KHTMLFactory(true);
 
     KMainWindow *toplevel = new KMainWindow();
     KHTMLPart *doc = new KHTMLPart( toplevel, 0, toplevel, 0, KHTMLPart::BrowserViewGUI );
@@ -64,7 +67,7 @@ int main(int argc, char *argv[])
 //     dtv->show();
 
     toplevel->setCentralWidget( doc->widget() );
-    toplevel->resize( 640, 800);
+    toplevel->resize( 800, 600);
 
 //     dtv->resize(toplevel->width()/2, toplevel->height());
 
@@ -78,6 +81,12 @@ int main(int argc, char *argv[])
     viewMenu.appendChild( e );
     QDomElement toolBar = d.documentElement().firstChild().nextSibling().toElement();
     e = d.createElement( "action" );
+    e.setAttribute( "name", "editable" );
+    toolBar.insertBefore( e, toolBar.firstChild() );
+    e = d.createElement( "action" );
+    e.setAttribute( "name", "navigable" );
+    toolBar.insertBefore( e, toolBar.firstChild() );
+    e = d.createElement( "action" );
     e.setAttribute( "name", "reload" );
     toolBar.insertBefore( e, toolBar.firstChild() );
     e = d.createElement( "action" );
@@ -87,30 +96,28 @@ int main(int argc, char *argv[])
     (void)new KAction( "Reload", "reload", Qt::Key_F5, dummy, SLOT( reload() ), doc->actionCollection(), "reload" );
     KAction* kprint = new KAction( "Print", "print", 0, doc->browserExtension(), SLOT( print() ), doc->actionCollection(), "print" );
     kprint->setEnabled(true);
-
+    KToggleAction *ta = new KToggleAction( "Navigable", "editclear", 0, doc->actionCollection(), "navigable" );
+    ta->setChecked(doc->isCaretMode());
+    QWidget::connect(ta, SIGNAL(toggled(bool)), dummy, SLOT( toggleNavigable(bool) ));
+    ta = new KToggleAction( "Editable", "edit", 0, doc->actionCollection(), "editable" );
+    ta->setChecked(doc->isEditable());
+    QWidget::connect(ta, SIGNAL(toggled(bool)), dummy, SLOT( toggleEditable(bool) ));
     toplevel->guiFactory()->addClient( doc );
 
     doc->setJScriptEnabled(true);
     doc->setJavaEnabled(true);
+    doc->setPluginsEnabled( true );
     doc->setURLCursor(QCursor(Qt::PointingHandCursor));
     a.setTopWidget(doc->widget());
     QWidget::connect(doc, SIGNAL(setWindowCaption(const QString &)),
-		     doc->widget(), SLOT(setCaption(const QString &)));
+		     doc->widget()->topLevelWidget(), SLOT(setCaption(const QString &)));
     doc->widget()->show();
     toplevel->show();
     ((QScrollView *)doc->widget())->viewport()->show();
 
 
     int ret = a.exec();
-    
-    //delete doc;
-    //delete dtv;
-
-    khtml::Cache::clear();
-    khtml::CSSStyleSelector::clear();
-    khtml::RenderStyle::cleanup();
-
-    delete fac;
+    fac->deref();
     return ret;
 }
 
