@@ -1,10 +1,12 @@
 /**
  * This file is part of the DOM implementation for KDE.
  *
- * (C) 1999 Lars Knoll (knoll@kde.org)
+ * (C) 1999-2003 Lars Knoll (knoll@kde.org)
+ * (C) 2001-2003 Dirk Mueller (mueller@kde.org)
  * (C) 2000 Gunnstein Lye (gunnstein@netcom.no)
  * (C) 2000 Frederik Holljen (frederik.holljen@hig.no)
  * (C) 2001 Peter Kelly (pmk@post.com)
+ * Copyright (C) 2003 Apple Computer, Inc.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -27,6 +29,7 @@
 #include "dom2_rangeimpl.h"
 #include "dom_textimpl.h"
 #include "dom_xmlimpl.h"
+#include "html/html_elementimpl.h"
 
 using namespace DOM;
 
@@ -344,6 +347,7 @@ short RangeImpl::compareBoundaryPoints( NodeImpl *containerA, long offsetA, Node
     // case 4: containers A & B are siblings, or children of siblings
     // ### we need to do a traversal here instead
     NodeImpl *cmnRoot = commonAncestorContainer(containerA,containerB);
+    if (!cmnRoot) return -1; // Whatever...
     NodeImpl *childA = containerA;
     while (childA->parentNode() != cmnRoot)
         childA = childA->parentNode();
@@ -847,6 +851,29 @@ DOMString RangeImpl::toHTML(  )
     return DOMString();
 }
 
+DocumentFragment RangeImpl::createContextualFragment ( const DOMString &html, int &exceptioncode )
+{
+   if (m_detached) {
+        exceptioncode = DOMException::INVALID_STATE_ERR;
+        return DocumentFragment();
+    }
+
+    if (! m_startContainer->isHTMLElement()) {
+	exceptioncode = DOMException::NOT_SUPPORTED_ERR;
+	return DocumentFragment();
+    }
+
+    HTMLElementImpl *e = static_cast<HTMLElementImpl *>(m_startContainer);
+    DocumentFragment fragment = e->createContextualFragment(html);
+    if (fragment.isNull()) {
+	exceptioncode = DOMException::NOT_SUPPORTED_ERR;
+	return DocumentFragment();
+    }
+
+    return fragment;
+}
+
+
 void RangeImpl::detach( int &exceptioncode )
 {
     if (m_detached) {
