@@ -4,6 +4,7 @@
  * Copyright (C) 1999 Lars Knoll (knoll@kde.org)
  *           (C) 1999 Antti Koivisto (koivisto@kde.org)
  *           (C) 2001 Dirk Mueller (mueller@kde.org)
+ * Copyright (C) 2002 Apple Computer, Inc.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -20,8 +21,8 @@
  * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
  *
- * $Id: dom_docimpl.h,v 1.99.2.7 2003/06/27 09:55:33 faure Exp $
  */
+
 #ifndef _DOM_DocumentImpl_h_
 #define _DOM_DocumentImpl_h_
 
@@ -29,10 +30,12 @@
 #include "xml/dom_textimpl.h"
 #include "xml/dom2_traversalimpl.h"
 #include "misc/shared.h"
+#include "misc/loader.h"
 
 #include <qstringlist.h>
 #include <qptrlist.h>
 #include <qobject.h>
+#include <qintdict.h>
 #include <qdict.h>
 #include <qmap.h>
 
@@ -42,12 +45,15 @@ class QPaintDevice;
 class QTextCodec;
 class QPaintDeviceMetrics;
 class KHTMLView;
-class Tokenizer;
 
 namespace khtml {
+    class Tokenizer;
     class CSSStyleSelector;
     class DocLoader;
     class CSSStyleSelectorList;
+    class RenderArena;
+    class CachedObject;
+    class CachedCSSStyleSheet;
 }
 
 namespace DOM {
@@ -117,7 +123,7 @@ protected:
 /**
  * @internal
  */
-class DocumentImpl : public QObject, public NodeBaseImpl
+class DocumentImpl : public QObject, private khtml::CachedObjectClient, public NodeBaseImpl
 {
     Q_OBJECT
 public:
@@ -130,7 +136,8 @@ public:
 
     DOMImplementationImpl *implementation() const;
     ElementImpl *documentElement() const;
-    virtual ElementImpl *createElement ( const DOMString &tagName, int* exceptioncode );
+    virtual ElementImpl *createElement ( const DOMString &tagName, int* pExceptioncode = 0 );
+    virtual AttrImpl *createAttribute( const DOMString &tagName, int* pExceptioncode = 0 );
     DocumentFragmentImpl *createDocumentFragment ();
     TextImpl *createTextNode ( DOMStringImpl* data ) { return new TextImpl( docPtr(), data); }
     TextImpl *createTextNode ( const QString& data )
@@ -138,15 +145,17 @@ public:
     CommentImpl *createComment ( DOMStringImpl* data );
     CDATASectionImpl *createCDATASection ( DOMStringImpl* data );
     ProcessingInstructionImpl *createProcessingInstruction ( const DOMString &target, DOMStringImpl* data );
-    Attr createAttribute(NodeImpl::Id id);
     EntityReferenceImpl *createEntityReference ( const DOMString &name );
     NodeImpl *importNode( NodeImpl *importedNode, bool deep, int &exceptioncode );
-    virtual ElementImpl *createElementNS ( const DOMString &_namespaceURI, const DOMString &_qualifiedName, int *exceptioncode );
+    virtual ElementImpl *createElementNS ( const DOMString &_namespaceURI, const DOMString &_qualifiedName,
+                                           int* pExceptioncode = 0 );
+    virtual AttrImpl *createAttributeNS( const DOMString &_namespaceURI, const DOMString &_qualifiedName,
+                                           int* pExceptioncode = 0 );
     ElementImpl *getElementById ( const DOMString &elementId ) const;
 
     // Actually part of HTMLDocument, but used for giving XML documents a window title as well
     DOMString title() const { return m_title; }
-    void setTitle(DOMString _title);
+    void setTitle(const DOMString& _title);
 
     // DOM methods overridden from  parent classes
 
@@ -161,6 +170,23 @@ public:
 
     khtml::CSSStyleSelector *styleSelector() { return m_styleSelector; }
 
+     /**
+     * Updates the pending sheet count and then calls updateStyleSelector.
+     */
+    void styleSheetLoaded();
+
+    /**
+     * This method returns true if all top-level stylesheets have loaded (including
+     * any @imports that they may be loading).
+     */
+    bool haveStylesheetsLoaded() { return m_pendingStylesheets <= 0 || m_ignorePendingStylesheets; }
+
+    /**
+     * Increments the number of pending sheets.  The <link> elements
+     * invoke this to add themselves to the loading list.
+     */
+    void addPendingSheet() { m_pendingStylesheets++; }
+
     /**
      * Called when one or more stylesheets in the document may have been added, removed or changed.
      *
@@ -174,6 +200,9 @@ public:
 
     void recalcStyleSelector();
 
+    bool usesDescendantRules() { return m_usesDescendantRules; }
+    void setUsesDescendantRules(bool b) { m_usesDescendantRules = b; }
+
     QString nextState();
 
     // Query all registered elements for their state
@@ -191,18 +220,21 @@ public:
     NodeIteratorImpl *createNodeIterator(NodeImpl *root, unsigned long whatToShow,
                                     NodeFilter &filter, bool entityReferenceExpansion, int &exceptioncode);
 
-    TreeWalkerImpl *createTreeWalker(Node root, unsigned long whatToShow, NodeFilter &filter,
-                            bool entityReferenceExpansion);
+    TreeWalkerImpl *createTreeWalker(NodeImpl *root, unsigned long whatToShow, NodeFilterImpl *filter,
+                            bool entityReferenceExpansion, int &exceptioncode);
 
     virtual void recalcStyle( StyleChange = NoChange );
     static QPtrList<DocumentImpl> * changedDocuments;
     virtual void updateRendering();
+    void updateLayout();
     static void updateDocumentsRendering();
     khtml::DocLoader *docLoader() { return m_docLoader; }
 
     virtual void attach();
     virtual void detach();
 
+    khtml::RenderArena* renderArena() { return m_renderArena; }
+
     // to get visually ordered hebrew and arabic pages right
     void setVisuallyOrdered();
     // to get URL decoding right
@@ -211,34 +243,33 @@ public:
     void setSelection(NodeImpl* s, int sp, NodeImpl* e, int ep);
     void clearSelection();
 
-    void open (  );
+    void open ( bool clearEventListeners = true );
     virtual void close (  );
     void write ( const DOMString &text );
     void write ( const QString &text );
     void writeln ( const DOMString &text );
     void finishParsing (  );
 
-    QString URL() const { return m_url; }
-    void setURL(QString url) { m_url = url; }
+    KURL URL() const { return m_url; }
+    void setURL(const QString& url) { m_url = url; }
 
-    QString baseURL() const { return m_baseURL.isEmpty() ? m_url : m_baseURL; }
-    void setBaseURL(const QString& baseURL) { m_baseURL = baseURL; }
+    KURL baseURL() const { return m_baseURL.isEmpty() ? m_url : m_baseURL; }
+    void setBaseURL(const KURL& baseURL) { m_baseURL = baseURL; }
 
     QString baseTarget() const { return m_baseTarget; }
     void setBaseTarget(const QString& baseTarget) { m_baseTarget = baseTarget; }
 
     QString completeURL(const QString& url) const { return KURL(baseURL(),url,m_decoderMibEnum).url(); };
+    DOMString canonURL(const DOMString& url) const { return url.isEmpty() ? url : completeURL(url.string()); }
 
-    // from cachedObjectClient
-    virtual void setStyleSheet(const DOM::DOMString &url, const DOM::DOMString &sheetStr);
     void setUserStyleSheet(const QString& sheet);
     QString userStyleSheet() const { return m_usersheet; }
     void setPrintStyleSheet(const QString& sheet) { m_printSheet = sheet; }
     QString printStyleSheet() const { return m_printSheet; }
 
     CSSStyleSheetImpl* elementSheet();
-    virtual Tokenizer *createTokenizer();
-    Tokenizer *tokenizer() { return m_tokenizer; }
+    virtual khtml::Tokenizer *createTokenizer();
+    khtml::Tokenizer *tokenizer() { return m_tokenizer; }
 
     QPaintDeviceMetrics *paintDeviceMetrics() { return m_paintDeviceMetrics; }
     QPaintDevice *paintDevice() const { return m_paintDevice; }
@@ -260,6 +291,10 @@ public:
     void setParseMode( ParseMode m ) { pMode = m; }
     ParseMode parseMode() const { return pMode; }
 
+    bool inCompatMode() const { return pMode == Compat; }
+    bool inTransitionalMode() const { return pMode == Transitional; }
+    bool inStrictMode() const { return pMode == Strict; }
+
     void setHTMLMode( HTMLMode m ) { hMode = m; }
     HTMLMode htmlMode() const { return hMode; }
 
@@ -269,6 +304,9 @@ public:
     void setTextColor( QColor color ) { m_textColor = color; }
     QColor textColor() const { return m_textColor; }
 
+    void setDesignMode(bool b);
+    bool designMode() const;
+
     // internal
     NodeImpl *findElement( Id id );
 
@@ -278,22 +316,23 @@ public:
     virtual bool childTypeAllowed( unsigned short nodeType );
     virtual NodeImpl *cloneNode ( bool deep );
 
-    // ### think about implementing ref'counting for the id's
-    // in order to be able to reassign those that are no longer in use
-    // (could make problems when it is still kept somewhere around, i.e. styleselector)
-    NodeImpl::Id tagId(DOMStringImpl* _namespaceURI, DOMStringImpl *_name, bool readonly, int *exceptioncode);
-    DOMString tagName(NodeImpl::Id _id) const;
+    NodeImpl::Id getId( NodeImpl::IdType _type, DOMStringImpl* _nsURI, DOMStringImpl *_localName,
+                        DOMStringImpl *_prefix, bool readonly, bool lookupHTML, int *pExceptioncode = 0);
+    NodeImpl::Id getId( NodeImpl::IdType _type, DOMStringImpl *_nodeName, bool readonly, bool lookupHTML,
+                                      int *pExceptioncode = 0);
+    DOMString getName( NodeImpl::IdType _type, NodeImpl::Id _id ) const;
 
-    NodeImpl::Id attrId(DOMStringImpl* _namespaceURI, DOMStringImpl *_name, bool readonly, int *exceptioncode);
-    DOMString attrName(NodeImpl::Id _id) const;
+    StyleSheetListImpl* styleSheets() { return m_styleSheets; };
 
-    // the namespace uri is mapped to the same id for both
-    // tagnames as well as attributes.
-    DOMStringImpl* namespaceURI(NodeImpl::Id _id) const;
+    DOMString preferredStylesheetSet() const { return m_preferredStylesheetSet; }
+    DOMString selectedStylesheetSet() const;
+    void setSelectedStylesheetSet(const DOMString&);
+    void setPreferredStylesheetSet(const DOMString& s) { m_preferredStylesheetSet = s; }
 
-    StyleSheetListImpl* styleSheets();
+    QStringList availableStyleSheets() const { return m_availableSheets; }
 
-    QStringList availableStyleSheets() const;
+    NodeImpl* hoverNode() const { return m_hoverNode; }
+    void setHoverNode(NodeImpl *newHoverNode);
     NodeImpl *focusNode() const { return m_focusNode; }
     void setFocusNode(NodeImpl *newFocusNode);
 
@@ -302,7 +341,7 @@ public:
     void attachNodeIterator(NodeIteratorImpl *ni);
     void detachNodeIterator(NodeIteratorImpl *ni);
     void notifyBeforeNodeRemoval(NodeImpl *n);
-    AbstractViewImpl *defaultView() const;
+    AbstractViewImpl *defaultView() const { return m_defaultView; }
     EventImpl *createEvent(const DOMString &eventType, int &exceptioncode);
 
     // keep track of what types of event listeners are registered, so we don't
@@ -322,6 +361,15 @@ public:
 
     CSSStyleDeclarationImpl *getOverrideStyle(ElementImpl *elt, DOMStringImpl *pseudoElt);
 
+    bool async() const { return m_async; }
+    void setAsync(bool b) { m_async = b; }
+    void abort();
+    void load(const DOMString &uri);
+    void loadXML(const DOMString &source);
+    // from cachedObjectClient
+    void setStyleSheet(const DOM::DOMString &url, const DOM::DOMString &sheet);
+    void error(int err, const QString &text);
+
     typedef QMap<QString, ProcessingInstructionImpl*> LocalStyleRefs;
     LocalStyleRefs* localStyleRefs() { return &m_localStyleRefs; }
 
@@ -329,7 +377,7 @@ public:
     virtual void setWindowEventListener(int id, EventListener *listener);
     EventListener *getWindowEventListener(int id);
     virtual void removeWindowEventListener(int id);
-    EventListener *createHTMLEventListener(QString code);
+    EventListener *createHTMLEventListener(QString code, QString name);
 
     /**
      * Searches through the document, starting from fromNode, for the next selectable element that comes after fromNode.
@@ -357,6 +405,8 @@ public:
      */
     NodeImpl *previousFocusNode(NodeImpl *fromNode);
 
+    ElementImpl* findAccessKeyElement(QChar c);
+
     int nodeAbsIndex(NodeImpl *node);
     NodeImpl *nodeWithAbsIndex(int absIndex);
 
@@ -370,6 +420,7 @@ public:
      * @param content The header value (value of the meta tag's "content" attribute)
      */
     void processHttpEquiv(const DOMString &equiv, const DOMString &content);
+
     bool isURLAllowed(const QString& url) const;
 
 signals:
@@ -381,19 +432,25 @@ protected:
     QStringList m_state;
 
     khtml::DocLoader *m_docLoader;
-    Tokenizer *m_tokenizer;
-    QString m_url;
-    QString m_baseURL;
+    khtml::Tokenizer *m_tokenizer;
+    KURL m_url;
+    KURL m_baseURL;
     QString m_baseTarget;
 
     DocumentTypeImpl *m_doctype;
     DOMImplementationImpl *m_implementation;
 
-    StyleSheetImpl *m_sheet;
     QString m_usersheet;
     QString m_printSheet;
     QStringList m_availableSheets;
 
+    // Track the number of currently loading top-level stylesheets.  Sheets
+    // loaded using the @import directive are not included in this count.
+    // We use this count of pending sheets to detect when we can begin attaching
+    // elements.
+    int m_pendingStylesheets;
+    bool m_ignorePendingStylesheets;
+
     CSSStyleSheetImpl *m_elemSheet;
 
     QPaintDevice *m_paintDevice;
@@ -402,21 +459,26 @@ protected:
     HTMLMode hMode;
 
     QColor m_textColor;
+    NodeImpl *m_hoverNode;
     NodeImpl *m_focusNode;
 
-    // ### replace me with something more efficient
-    // in lookup and insertion.
-    DOMStringImpl **m_elementNames;
-    unsigned short m_elementNameAlloc;
-    unsigned short m_elementNameCount;
+    struct IdNameMapping {
+        IdNameMapping(unsigned short _start)
+            : idStart(_start), count(0) {}
+        ~IdNameMapping() {
+            QIntDictIterator<DOM::DOMStringImpl> it(names);
+            for (; it.current() ; ++it)
+                it.current()->deref();
+        }
+        unsigned short idStart;
+        unsigned short count;
+        QIntDict<DOM::DOMStringImpl> names;
+        QDict<void> ids;
+    };
 
-    DOMStringImpl **m_attrNames;
-    unsigned short m_attrNameAlloc;
-    unsigned short m_attrNameCount;
-
-    DOMStringImpl** m_namespaceURIs;
-    unsigned short m_namespaceURIAlloc;
-    unsigned short m_namespaceURICount;
+    IdNameMapping *m_attrMap;
+    IdNameMapping *m_elementMap;
+    IdNameMapping *m_namespaceMap;
 
     QPtrList<NodeIteratorImpl> m_nodeIterators;
     AbstractViewImpl *m_defaultView;
@@ -427,15 +489,24 @@ protected:
     QPtrList<RegisteredEventListener> m_windowEventListeners;
     QPtrList<NodeImpl> m_maintainsState;
 
-    bool m_loadingSheet;
     bool visuallyOrdered;
     bool m_bParsing;
     bool m_docChanged;
     bool m_styleSelectorDirty;
     bool m_inStyleRecalc;
+    bool m_usesDescendantRules;
+    bool m_async;
+    bool m_hadLoadError;
+    bool m_docLoading;
+    bool m_inSyncLoad;
 
     DOMString m_title;
+    DOMString m_preferredStylesheetSet;
+    khtml::CachedCSSStyleSheet *m_loadingXMLDoc;
+
     int m_decoderMibEnum;
+
+    khtml::RenderArena* m_renderArena;
 };
 
 class DocumentFragmentImpl : public NodeBaseImpl
@@ -463,8 +534,8 @@ public:
     ~DocumentTypeImpl();
 
     // DOM methods & attributes for DocumentType
-    NamedNodeMapImpl *entities() const { return m_entities; }
-    NamedNodeMapImpl *notations() const { return m_notations; }
+    NamedNodeMapImpl *entities() const;
+    NamedNodeMapImpl *notations() const;
 
     DOMString name() const { return m_qualifiedName; }
     DOMString publicId() const { return m_publicId; }
@@ -484,8 +555,8 @@ public:
 
 protected:
     DOMImplementationImpl *m_implementation;
-    NamedNodeMapImpl* m_entities;
-    NamedNodeMapImpl* m_notations;
+    mutable NamedNodeMapImpl* m_entities;
+    mutable NamedNodeMapImpl* m_notations;
 
     DOMString m_qualifiedName;
     DOMString m_publicId;
