@@ -38,6 +38,7 @@
 #include "ecma/kjs_proxy.h"
 #include "khtmlview.h"
 #include "khtml_part.h"
+#include "dom_nodeimpl.h"
 
 
 using namespace DOM;
@@ -52,7 +53,6 @@ NodeImpl::NodeImpl(DocumentPtr *doc)
       m_tabIndex( 0 ),
       m_hasId( false ),
       m_hasStyle( false ),
-      m_pressed( false ),
       m_attached(false),
       m_changed( false ),
       m_hasChangedChild( false ),
@@ -62,7 +62,9 @@ NodeImpl::NodeImpl(DocumentPtr *doc)
       m_focused( false ),
       m_active( false ),
       m_styleElement( false ),
-      m_implicit( false )
+      m_implicit( false ),
+      m_rendererNeedsClose( false ),
+      m_htmlCompat( false )
 {
     if (document)
         document->ref();
@@ -86,15 +88,12 @@ DOMString NodeImpl::nodeValue() const
   return DOMString();
 }
 
-void NodeImpl::setNodeValue( const DOMString &/*_nodeValue*/, int &exceptioncode )
+void NodeImpl::setNodeValue( const DOMString &/*_nodeValue*/, int &/*exceptioncode*/ )
 {
-    // NO_MODIFICATION_ALLOWED_ERR: Raised when the node is readonly
-    if (isReadOnly()) {
-        exceptioncode = DOMException::NO_MODIFICATION_ALLOWED_ERR;
-        return;
-    }
-
-    // be default nodeValue is null, so setting it has no effect
+    // by default nodeValue is null, so setting it has no effect
+    // don't throw NO_MODIFICATION_ALLOWED_ERR from here, DOMTS-Core-Level1's hc_nodevalue03
+    // (createEntityReference().setNodeValue())) says it would be wrong.
+    // This must be done by subclasses instead.
 }
 
 DOMString NodeImpl::nodeName() const
@@ -188,6 +187,11 @@ DOMString NodeImpl::prefix() const
     return DOMString();
 }
 
+DOMString NodeImpl::namespaceURI() const
+{
+    return DOMString();
+}
+
 void NodeImpl::setPrefix(const DOMString &/*_prefix*/, int &exceptioncode )
 {
     // The spec says that for nodes other than elements and attributes, prefix is always null.
@@ -216,7 +220,7 @@ NodeImpl *NodeImpl::addChild(NodeImpl *)
 
 QString NodeImpl::toHTML() const
 {
-    qDebug("NodeImpl::toHTML");
+    kdDebug(6020) << "NodeImpl::toHTML" << endl;
     NodeImpl* fc = firstChild();
     if ( fc )
         return fc->recursive_toHTML(true);
@@ -311,17 +315,18 @@ QString NodeImpl::recursive_toHTML(bool start) const
     return me;
 }
 
-void NodeImpl::getCursor(int offset, int &_x, int &_y, int &height)
+void NodeImpl::getCaret(int offset, bool override, int &_x, int &_y, int &width, int &height)
 {
-    if(m_render) m_render->cursorPos(offset, _x, _y, height);
-    else _x = _y = height = -1;
+    if(m_render) m_render->caretPos(offset, override, _x, _y, width, height);
+    else _x = _y = height = -1, width = 1;
 }
 
 QRect NodeImpl::getRect() const
 {
     int _x, _y;
     if(m_render && m_render->absolutePosition(_x, _y))
-        return QRect( _x, _y, m_render->width(), m_render->height() );
+        return QRect( _x + m_render->inlineXPos(), _y + m_render->inlineYPos(),
+        		m_render->width(), m_render->height() );
 
     return QRect();
 }
@@ -455,31 +460,23 @@ EventListener *NodeImpl::getHTMLEventListener(int id)
 }
 
 
-bool NodeImpl::dispatchEvent(EventImpl *evt, int &exceptioncode, bool tempEvent)
+void NodeImpl::dispatchEvent(EventImpl *evt, int &exceptioncode, bool tempEvent)
 {
-    // Not mentioned in spec: throw NOT_FOUND_ERR if evt is null
-    if (!evt) {
-        exceptioncode = DOMException::NOT_FOUND_ERR;
-        return false;
-    }
-
     evt->setTarget(this);
 
     // Since event handling code could cause this object to be deleted, grab a reference to the view now
     KHTMLView *view = document->document()->view();
 
-    bool ret = dispatchGenericEvent( evt, exceptioncode );
+    dispatchGenericEvent( evt, exceptioncode );
 
     // If tempEvent is true, this means that the DOM implementation will not be storing a reference to the event, i.e.
     // there is no way to retrieve it from javascript if a script does not already have a reference to it in a variable.
     // So there is no need for the interpreter to keep the event in its cache
     if (tempEvent && view && view->part() && view->part()->jScript())
         view->part()->jScript()->finishedWithEvent(evt);
-
-    return ret;
 }
 
-bool NodeImpl::dispatchGenericEvent( EventImpl *evt, int &/*exceptioncode */)
+void NodeImpl::dispatchGenericEvent( EventImpl *evt, int &/*exceptioncode */)
 {
     // ### check that type specified
 
@@ -506,7 +503,7 @@ bool NodeImpl::dispatchGenericEvent( EventImpl *evt, int &/*exceptioncode */)
         evt->setCurrentTarget(it.current());
         it.current()->handleLocalEvents(evt, true);
         if (!evt->propagationStopped())
-        it.current()->handleLocalEvents(evt,false);
+            it.current()->handleLocalEvents(evt,false);
     }
     --it;
 
@@ -529,7 +526,7 @@ bool NodeImpl::dispatchGenericEvent( EventImpl *evt, int &/*exceptioncode */)
             it.current()->defaultEventHandler(evt);
 
         if (evt->id() == EventImpl::CLICK_EVENT && !evt->defaultPrevented() &&
-			static_cast<MouseEventImpl*>(evt)->button() == 0)
+             static_cast<MouseEventImpl*>( evt )->button() == 0) // LMB click
             dispatchUIEvent(EventImpl::DOMACTIVATE_EVENT, static_cast<UIEventImpl*>(evt)->detail());
     }
 
@@ -544,8 +541,6 @@ bool NodeImpl::dispatchGenericEvent( EventImpl *evt, int &/*exceptioncode */)
 
     DocumentImpl::updateDocumentsRendering();
     doc->deref();
-
-    return !evt->defaultPrevented(); // ### what if defaultPrevented was called before dispatchEvent?
 }
 
 bool NodeImpl::dispatchHTMLEvent(int _id, bool canBubbleArg, bool cancelableArg)
@@ -553,12 +548,13 @@ bool NodeImpl::dispatchHTMLEvent(int _id, bool canBubbleArg, bool cancelableArg)
     int exceptioncode = 0;
     EventImpl *evt = new EventImpl(static_cast<EventImpl::EventId>(_id),canBubbleArg,cancelableArg);
     evt->ref();
-    bool r = dispatchEvent(evt,exceptioncode,true);
+    dispatchEvent(evt,exceptioncode,true);
+    bool ret = !evt->defaultPrevented();
     evt->deref();
-    return r;
+    return ret;
 }
 
-bool NodeImpl::dispatchWindowEvent(int _id, bool canBubbleArg, bool cancelableArg)
+void NodeImpl::dispatchWindowEvent(int _id, bool canBubbleArg, bool cancelableArg)
 {
     int exceptioncode = 0;
     EventImpl *evt = new EventImpl(static_cast<EventImpl::EventId>(_id),canBubbleArg,cancelableArg);
@@ -566,15 +562,14 @@ bool NodeImpl::dispatchWindowEvent(int _id, bool canBubbleArg, bool cancelableAr
     evt->ref();
     DocumentPtr *doc = document;
     doc->ref();
-    bool r = dispatchGenericEvent( evt, exceptioncode );
-    if (!evt->defaultPrevented())
+    dispatchGenericEvent( evt, exceptioncode );
+    if (!evt->defaultPrevented() && doc->document())
 	doc->document()->defaultEventHandler(evt);
     doc->deref();
     evt->deref();
-    return r;
 }
 
-bool NodeImpl::dispatchMouseEvent(QMouseEvent *_mouse, int overrideId, int overrideDetail)
+void NodeImpl::dispatchMouseEvent(QMouseEvent *_mouse, int overrideId, int overrideDetail)
 {
     bool cancelable = true;
     int detail = overrideDetail; // defaults to 0
@@ -603,15 +598,16 @@ bool NodeImpl::dispatchMouseEvent(QMouseEvent *_mouse, int overrideId, int overr
         }
     }
     if (evtId == EventImpl::UNKNOWN_EVENT)
-        return false; // shouldn't happen
+        return; // shouldn't happen
 
 
     int exceptioncode = 0;
-
-//    int clientX, clientY;
-//    viewportToContents(_mouse->x(), _mouse->y(), clientX, clientY);
-    int clientX = _mouse->x(); // ### adjust to be relative to view
-    int clientY = _mouse->y(); // ### adjust to be relative to view
+    int pageX = _mouse->x();
+    int pageY = _mouse->y();
+    int clientX = pageX;
+    int clientY = pageY;
+    if ( getDocument()->view() )
+        getDocument()->view()->viewportToContents( clientX, clientY, pageX, pageY );
 
     int screenX = _mouse->globalX();
     int screenY = _mouse->globalY();
@@ -630,22 +626,20 @@ bool NodeImpl::dispatchMouseEvent(QMouseEvent *_mouse, int overrideId, int overr
         default:
             break;
     }
-    bool ctrlKey = (_mouse->state() & Qt::ControlButton) == Qt::ControlButton;
-    bool altKey = (_mouse->state() & Qt::AltButton) == Qt::AltButton;
-    bool shiftKey = (_mouse->state() & Qt::ShiftButton) == Qt::ShiftButton;
+    bool ctrlKey = (_mouse->state() & Qt::ControlButton);
+    bool altKey = (_mouse->state() & Qt::AltButton);
+    bool shiftKey = (_mouse->state() & Qt::ShiftButton);
     bool metaKey = false; // ### qt support?
 
     EventImpl *evt = new MouseEventImpl(evtId,true,cancelable,getDocument()->defaultView(),
-                   detail,screenX,screenY,clientX,clientY,ctrlKey,altKey,shiftKey,metaKey,
+                   detail,screenX,screenY,clientX,clientY,pageX,pageY,ctrlKey,altKey,shiftKey,metaKey,
                    button,0);
     evt->ref();
-    bool r = dispatchEvent(evt,exceptioncode,true);
+    dispatchEvent(evt,exceptioncode,true);
     evt->deref();
-    return r;
-
 }
 
-bool NodeImpl::dispatchUIEvent(int _id, int detail)
+void NodeImpl::dispatchUIEvent(int _id, int detail)
 {
     assert (!( (_id != EventImpl::DOMFOCUSIN_EVENT &&
         _id != EventImpl::DOMFOCUSOUT_EVENT &&
@@ -659,32 +653,28 @@ bool NodeImpl::dispatchUIEvent(int _id, int detail)
     UIEventImpl *evt = new UIEventImpl(static_cast<EventImpl::EventId>(_id),true,
                                        cancelable,getDocument()->defaultView(),detail);
     evt->ref();
-    bool r = dispatchEvent(evt,exceptioncode,true);
+    dispatchEvent(evt,exceptioncode,true);
     evt->deref();
-    return r;
 }
 
-bool NodeImpl::dispatchSubtreeModifiedEvent()
+void NodeImpl::dispatchSubtreeModifiedEvent()
 {
     childrenChanged();
     if (!getDocument()->hasListenerType(DocumentImpl::DOMSUBTREEMODIFIED_LISTENER))
-	return false;
+	return;
     int exceptioncode = 0;
-    return dispatchEvent(new MutationEventImpl(EventImpl::DOMSUBTREEMODIFIED_EVENT,
+    dispatchEvent(new MutationEventImpl(EventImpl::DOMSUBTREEMODIFIED_EVENT,
 			 true,false,0,DOMString(),DOMString(),DOMString(),0),exceptioncode,true);
 }
 
-bool NodeImpl::dispatchKeyEvent(QKeyEvent *key)
+bool NodeImpl::dispatchKeyEvent(QKeyEvent *key, bool keypress)
 {
     int exceptioncode = 0;
     //kdDebug(6010) << "DOM::NodeImpl: dispatching keyboard event" << endl;
-    TextEventImpl *keyEventImpl = new TextEventImpl(key, getDocument()->defaultView());
+    TextEventImpl *keyEventImpl = new TextEventImpl(key, keypress, getDocument()->defaultView());
     keyEventImpl->ref();
-    bool r = dispatchEvent(keyEventImpl,exceptioncode,true);
-    // the default event handler should accept() the internal QKeyEvent
-    // to prevent the view from further evaluating it.
-    if (!keyEventImpl->defaultPrevented() && !keyEventImpl->qKeyEvent->isAccepted())
-      r = false;
+    dispatchEvent(keyEventImpl,exceptioncode,true);
+    bool r = keyEventImpl->defaultHandled() || keyEventImpl->defaultPrevented();
     keyEventImpl->deref();
     return r;
 }
@@ -695,15 +685,27 @@ void NodeImpl::handleLocalEvents(EventImpl *evt, bool useCapture)
         return;
 
     Event ev = evt;
-    for (QPtrListIterator<RegisteredEventListener> it(*m_regdListeners); it.current();) {
+    // removeEventListener (e.g. called from a JS event listener) might
+    // invalidate the item after the current iterator (which "it" is pointing to).
+    // So we make a copy of the list.
+    QPtrList<RegisteredEventListener> listeners = *m_regdListeners;
+    for (QPtrListIterator<RegisteredEventListener> it(listeners); it.current();) {
         RegisteredEventListener* current = it();
         if (current->id == evt->id() && current->useCapture == useCapture)
             current->listener->handleEvent(ev);
+
         // ECMA legacy hack
-        if (current->useCapture == useCapture && evt->id() == EventImpl::CLICK_EVENT &&
-            ( ( static_cast<MouseEventImpl*>(evt)->detail() == 1 && current->id == EventImpl::KHTML_ECMA_CLICK_EVENT) ||
-              ( static_cast<MouseEventImpl*>(evt)->detail() > 1 && current->id == EventImpl::KHTML_ECMA_DBLCLICK_EVENT) ) )
-            current->listener->handleEvent(ev);
+        if (current->useCapture == useCapture && evt->id() == EventImpl::CLICK_EVENT) {
+            MouseEventImpl* me = static_cast<MouseEventImpl*>(evt);
+            // To find whether to call onclick or ondblclick, we can't
+            // * use me->detail(), it's 2 when clicking twice w/o moving, even very slowly
+            // * use me->qEvent(), it's not available when using initMouseEvent/dispatchEvent
+            // So we currently store a bool in MouseEventImpl. If anyone needs to trigger
+            // dblclicks from the DOM API, we'll need a timer here (well in the doc).
+            if ( ( !me->isDoubleClick() && current->id == EventImpl::KHTML_ECMA_CLICK_EVENT) ||
+              ( me->isDoubleClick() && current->id == EventImpl::KHTML_ECMA_DBLCLICK_EVENT) )
+                current->listener->handleEvent(ev);
+        }
     }
 }
 
@@ -778,7 +780,7 @@ void NodeImpl::checkSetPrefix(const DOMString &_prefix, int &exceptioncode)
     //   the namespaceURI of this node is different from "http://www.w3.org/2000/xmlns/",
     // - or if this node is an attribute and the qualifiedName of this node is "xmlns" [Namespaces].
     if (Element::khtmlMalformedPrefix(_prefix) || (!(id() & NodeImpl_IdNSMask) && id() > ID_LAST_TAG) ||
-        (_prefix == "xml" && DOMString(getDocument()->namespaceURI(id())) != "http://www.w3.org/XML/1998/namespace")) {
+        (_prefix == "xml" && namespaceURI() != "http://www.w3.org/XML/1998/namespace")) {
         exceptioncode = DOMException::NAMESPACE_ERR;
         return;
     }
@@ -867,38 +869,28 @@ NodeImpl::StyleChange NodeImpl::diff( khtml::RenderStyle *s1, khtml::RenderStyle
     return ch;
 }
 
-#ifndef NDEBUG
-void NodeImpl::dump(QTextStream *stream, QString ind) const
+void NodeImpl::closeRenderer()
 {
-    // ### implement dump() for all appropriate subclasses
-
-    if (m_hasId) { *stream << " hasId"; }
-    if (m_hasStyle) { *stream << " hasStyle"; }
-    if (m_specified) { *stream << " specified"; }
-    if (m_focused) { *stream << " focused"; }
-    if (m_active) { *stream << " active"; }
-    if (m_styleElement) { *stream << " styleElement"; }
-    if (m_implicit) { *stream << " implicit"; }
-
-    *stream << " tabIndex=" << m_tabIndex;
-    if (m_regdListeners)
-	*stream << " #regdListeners=" << m_regdListeners->count(); // ### more detail
-    *stream << endl;
-
-    NodeImpl *child = firstChild();
-    while( child != 0 )
-    {
-	*stream << ind << child->nodeName().string().ascii() << ": ";
-        child->dump(stream,ind+"  ");
-        child = child->nextSibling();
-    }
+    // It's important that we close the renderer, even if it hasn't been
+    // created yet. This happens even more because of the FOUC fixes we did
+    // at Apple, which prevent renderers from being created until the stylesheets
+    // are all loaded. If the renderer is not here to be closed, we set a flag,
+    // then close it later when it's attached.
+    assert(!m_rendererNeedsClose);
+    if (m_render)
+        m_render->close();
+    else
+        m_rendererNeedsClose = true;
 }
-#endif
 
 void NodeImpl::attach()
 {
     assert(!attached());
     assert(!m_render || (m_render->style() && m_render->parent()));
+    if (m_render && m_rendererNeedsClose) {
+        m_render->close();
+        m_rendererNeedsClose = false;
+    }
     m_attached = true;
 }
 
@@ -965,6 +957,85 @@ RenderObject * NodeImpl::nextRenderer()
     return 0;
 }
 
+bool NodeImpl::contentEditable() const
+{
+    RenderObject *r = renderer();
+    if (!r) return false;
+    return r->style()->userInput() == UI_ENABLED;
+}
+
+NodeImpl *NodeImpl::prevLeafNode() const
+{
+  const NodeImpl *r = this;
+  const NodeImpl *n = firstChild();
+  if (n) {
+    while (n) { r = n; n = n->firstChild(); }
+    return const_cast<NodeImpl *>(r);
+  }/*end if*/
+  n = r->previousSibling();
+  if (n) {
+    r = n;
+    while (n) { r = n; n = n->firstChild(); }
+    return const_cast<NodeImpl *>(r);
+  }/*end if*/
+
+  n = r->parentNode();
+  while (n) {
+    r = n;
+    n = r->previousSibling();
+    if (n) {
+      r = n;
+      n = r->lastChild();
+      while (n) { r = n; n = n->lastChild(); }
+      return const_cast<NodeImpl *>(r);
+    }/*end if*/
+    n = r->parentNode();
+  }/*wend*/
+  return 0;
+}
+
+NodeImpl *NodeImpl::nextLeafNode() const
+{
+  const NodeImpl *r = this;
+  const NodeImpl *n = firstChild();
+  if (n) {
+    while (n) { r = n; n = n->firstChild(); }
+    return const_cast<NodeImpl *>(r);
+  }/*end if*/
+  n = r->nextSibling();
+  if (n) {
+    r = n;
+    while (n) { r = n; n = n->firstChild(); }
+    return const_cast<NodeImpl *>(r);
+  }/*end if*/
+
+  n = r->parentNode();
+  while (n) {
+    r = n;
+    n = r->nextSibling();
+    if (n) {
+      r = n;
+      n = r->firstChild();
+      while (n) { r = n; n = n->firstChild(); }
+      return const_cast<NodeImpl *>(r);
+    }/*end if*/
+    n = r->parentNode();
+  }/*wend*/
+  return 0;
+}
+
+long NodeImpl::minOffset() const
+{
+  // Arrgh! You'd think *every* offset starts at zero, but loo,
+  // therefore we need this method
+  return renderer() ? renderer()->minOffset() : 0;
+}
+
+long NodeImpl::maxOffset() const
+{
+  return renderer() ? renderer()->maxOffset() : 1;
+}
+
 //-------------------------------------------------------------------------
 
 NodeBaseImpl::~NodeBaseImpl()
@@ -1152,15 +1223,6 @@ NodeImpl *NodeBaseImpl::removeChild ( NodeImpl *oldChild, int &exceptioncode )
         return 0;
     }
 
-    // Dispatch pre-removal mutation events
-    getDocument()->notifyBeforeNodeRemoval(oldChild); // ### use events instead
-    if (getDocument()->hasListenerType(DocumentImpl::DOMNODEREMOVED_LISTENER)) {
-	oldChild->dispatchEvent(new MutationEventImpl(EventImpl::DOMNODEREMOVED_EVENT,
-			     true,false,this,DOMString(),DOMString(),DOMString(),0),exceptioncode,true);
-	if (exceptioncode)
-	    return 0;
-    }
-
     dispatchChildRemovalEvents(oldChild,exceptioncode);
     if (exceptioncode)
         return 0;
@@ -1391,27 +1453,6 @@ void NodeBaseImpl::cloneChildNodes(NodeImpl *clone)
     }
 }
 
-NodeListImpl* NodeBaseImpl::getElementsByTagNameNS ( DOMStringImpl* namespaceURI,
-                                                     DOMStringImpl* localName )
-{
-    if (!localName) return 0;
-
-    NodeImpl::Id idMask = NodeImpl_IdNSMask | NodeImpl_IdLocalMask;
-    if (localName->l && localName->s[0] == '*')
-        idMask &= ~NodeImpl_IdLocalMask;
-    if (namespaceURI && namespaceURI->l && namespaceURI->s[0] == '*')
-        idMask &= ~NodeImpl_IdNSMask;
-
-    Id id = 0; // 0 means "all items"
-    if ( (idMask & NodeImpl_IdLocalMask) || namespaceURI ) // not getElementsByTagName("*")
-    {
-        id = getDocument()->tagId( namespaceURI, localName, true /*readonly*/, 0 );
-        if ( !id ) // not found -> we want to return an empty list, not "all items"
-            id = (Id)-1;
-    }
-
-    return new TagNodeListImpl( this, id, idMask );
-}
 
 // I don't like this way of implementing the method, but I didn't find any
 // other way. Lars
@@ -1496,35 +1537,6 @@ bool NodeBaseImpl::getLowerRightCorner(int &xPos, int &yPos) const
     return true;
 }
 
-QRect NodeBaseImpl::getRect() const
-{
-    int xPos, yPos;
-    if (!getUpperLeftCorner(xPos,yPos))
-    {
-        xPos=0;
-        yPos=0;
-    }
-    int xEnd, yEnd;
-    if (!getLowerRightCorner(xEnd,yEnd))
-    {
-        if (xPos)
-            xEnd = xPos;
-        if (yPos)
-            yEnd = yPos;
-    }
-    else
-    {
-        if (xPos==0)
-            xPos = xEnd;
-        if (yPos==0)
-            yPos = yEnd;
-    }
-    if ( xEnd <= xPos || yEnd <= yPos )
-        return QRect( QPoint( xPos, yPos ), QSize() );
-
-    return QRect(xPos, yPos, xEnd - xPos, yEnd - yPos);
-}
-
 void NodeBaseImpl::setFocus(bool received)
 {
     if (m_focused == received) return;
@@ -1544,7 +1556,7 @@ void NodeBaseImpl::setActive(bool down)
     NodeImpl::setActive(down);
 
     // note that we need to recalc the style
-    if (m_render && m_render->style()->hasActive())
+    if (m_render && m_render->style()->affectedByActiveRules())
         setChanged();
 }
 
@@ -1575,7 +1587,7 @@ void NodeBaseImpl::dispatchChildInsertedEvents( NodeImpl *child, int &exceptionc
             return;
     }
 
-    // dispatch the DOMNOdeInsertedInfoDocument event to all descendants
+    // dispatch the DOMNodeInsertedIntoDocument event to all descendants
     bool hasInsertedListeners = getDocument()->hasListenerType(DocumentImpl::DOMNODEINSERTEDINTODOCUMENT_LISTENER);
     NodeImpl *p = this;
     while (p->parentNode())
@@ -1607,7 +1619,7 @@ void NodeBaseImpl::dispatchChildRemovalEvents( NodeImpl *child, int &exceptionco
 
     bool hasRemovalListeners = getDocument()->hasListenerType(DocumentImpl::DOMNODEREMOVEDFROMDOCUMENT_LISTENER);
 
-    // dispatch the DOMNOdeRemovedFromDocument event to all descendants
+    // dispatch the DOMNodeRemovedFromDocument event to all descendants
     NodeImpl *p = this;
     while (p->parentNode())
 	p = p->parentNode();
@@ -1641,7 +1653,7 @@ unsigned long NodeListImpl::recursiveLength(NodeImpl *start) const
 
     for(NodeImpl *n = start->firstChild(); n != 0; n = n->nextSibling()) {
         if ( n->nodeType() == Node::ELEMENT_NODE ) {
-            if (nodeMatches(n))
+            if (elementMatches(n))
                 len++;
             len+= recursiveLength(n);
         }
@@ -1654,7 +1666,7 @@ NodeImpl *NodeListImpl::recursiveItem ( NodeImpl *start, unsigned long &offset )
 {
     for(NodeImpl *n = start->firstChild(); n != 0; n = n->nextSibling()) {
         if ( n->nodeType() == Node::ELEMENT_NODE ) {
-            if (nodeMatches(n))
+            if (elementMatches(n))
                 if (!offset--)
                     return n;
 
@@ -1702,36 +1714,56 @@ NodeImpl *ChildNodeListImpl::item ( unsigned long index ) const
     return n;
 }
 
-bool ChildNodeListImpl::nodeMatches( NodeImpl */*testNode*/ ) const
+bool ChildNodeListImpl::elementMatches( NodeImpl* /*testNode*/ ) const
 {
     return true;
 }
 
-TagNodeListImpl::TagNodeListImpl(NodeImpl *n, NodeImpl::Id _id, NodeImpl::Id _idMask )
-    : refNode(n), m_id(_id & _idMask), m_idMask(_idMask)
+TagNodeListImpl::TagNodeListImpl( NodeImpl *n, NodeImpl::Id id )
+  : m_refNode(n),
+    m_id(id),
+    m_namespaceAware(false)
 {
-    refNode->ref();
+    m_refNode->ref();
+    // An id of 0 here means "*" (match all nodes)
+    m_matchAllNames = (id == 0);
+    m_matchAllNamespaces = false;
+}
+
+TagNodeListImpl::TagNodeListImpl( NodeImpl *n, const DOMString &namespaceURI, const DOMString &localName )
+  : m_refNode(n),
+    m_id(0),
+    m_namespaceURI(namespaceURI),
+    m_localName(localName),
+    m_namespaceAware(true)
+{
+    m_refNode->ref();
+    m_matchAllNames = (localName == "*");
+    m_matchAllNamespaces = (namespaceURI == "*");
 }
 
 TagNodeListImpl::~TagNodeListImpl()
 {
-    refNode->deref();
+    m_refNode->deref();
 }
 
 unsigned long TagNodeListImpl::length() const
 {
-    return recursiveLength( refNode );
+    return recursiveLength( m_refNode );
 }
 
 NodeImpl *TagNodeListImpl::item ( unsigned long index ) const
 {
-    return recursiveItem( refNode, index );
+    return recursiveItem( m_refNode, index );
 }
 
-bool TagNodeListImpl::nodeMatches( NodeImpl *testNode ) const
+bool TagNodeListImpl::elementMatches( NodeImpl *testNode ) const
 {
-    return testNode->isElementNode() &&
-           ((testNode->id() & m_idMask) == m_id);
+    if (m_namespaceAware)
+	return (m_matchAllNamespaces || testNode->namespaceURI() == m_namespaceURI) &&
+	       (m_matchAllNames || testNode->localName() == m_localName);
+    else
+	return (m_id == 0 || m_id == testNode->id());
 }
 
 NameNodeListImpl::NameNodeListImpl(NodeImpl *n, const DOMString &t )
@@ -1756,19 +1788,19 @@ NodeImpl *NameNodeListImpl::item ( unsigned long index ) const
     return recursiveItem( refNode, index );
 }
 
-bool NameNodeListImpl::nodeMatches( NodeImpl *testNode ) const
+bool NameNodeListImpl::elementMatches( NodeImpl *testNode ) const
 {
     return static_cast<ElementImpl *>(testNode)->getAttribute(ATTR_NAME) == nodeName;
 }
 
-NamedTagNodeListImpl::NamedTagNodeListImpl( NodeImpl *n, NodeImpl::Id tagId, const DOMString& name, NodeImpl::Id tagIdMask )
-    : TagNodeListImpl( n, tagId, tagIdMask ), nodeName( name )
+NamedTagNodeListImpl::NamedTagNodeListImpl( NodeImpl *n, NodeImpl::Id tagId, const DOMString& name )
+    : TagNodeListImpl( n, tagId ), nodeName( name )
 {
 }
 
-bool NamedTagNodeListImpl::nodeMatches( NodeImpl *testNode ) const
+bool NamedTagNodeListImpl::elementMatches( NodeImpl *testNode ) const
 {
-    return TagNodeListImpl::nodeMatches( testNode )
+    return TagNodeListImpl::elementMatches( testNode )
         && static_cast<ElementImpl *>(testNode)->getAttribute(ATTR_NAME) == nodeName;
 }
 
@@ -1785,8 +1817,6 @@ NamedNodeMapImpl::~NamedNodeMapImpl()
 
 // ----------------------------------------------------------------------------
 
-// ### unused
-#if 0
 GenericRONamedNodeMapImpl::GenericRONamedNodeMapImpl(DocumentPtr* doc)
     : NamedNodeMapImpl()
 {
@@ -1796,22 +1826,23 @@ GenericRONamedNodeMapImpl::GenericRONamedNodeMapImpl(DocumentPtr* doc)
 
 GenericRONamedNodeMapImpl::~GenericRONamedNodeMapImpl()
 {
-    while (m_contents->count() > 0)
+    while (!m_contents->isEmpty())
         m_contents->take(0)->deref();
 
     delete m_contents;
 }
 
-NodeImpl *GenericRONamedNodeMapImpl::getNamedItem ( const DOMString &name, int &/*exceptioncode*/ ) const
+NodeImpl *GenericRONamedNodeMapImpl::getNamedItem ( NodeImpl::Id id, bool /*nsAware*/, DOMStringImpl* /*qName*/ ) const
 {
+    // ## do we need namespace support in this class?
     QPtrListIterator<NodeImpl> it(*m_contents);
     for (; it.current(); ++it)
-        if (it.current()->nodeName() == name)
+        if (it.current()->id() == id)
             return it.current();
     return 0;
 }
 
-Node GenericRONamedNodeMapImpl::setNamedItem ( const Node &/*arg*/, int &exceptioncode )
+Node GenericRONamedNodeMapImpl::setNamedItem ( NodeImpl* /*arg*/, bool /*nsAware*/, DOMStringImpl* /*qName*/, int &exceptioncode )
 {
     // can't modify this list through standard DOM functions
     // NO_MODIFICATION_ALLOWED_ERR: Raised if this map is readonly
@@ -1819,7 +1850,7 @@ Node GenericRONamedNodeMapImpl::setNamedItem ( const Node &/*arg*/, int &excepti
     return 0;
 }
 
-Node GenericRONamedNodeMapImpl::removeNamedItem ( const DOMString &/*name*/, int &exceptioncode )
+Node GenericRONamedNodeMapImpl::removeNamedItem ( NodeImpl::Id /*id*/, bool /*nsAware*/, DOMStringImpl* /*qName*/, int &exceptioncode )
 {
     // can't modify this list through standard DOM functions
     // NO_MODIFICATION_ALLOWED_ERR: Raised if this map is readonly
@@ -1829,8 +1860,6 @@ Node GenericRONamedNodeMapImpl::removeNamedItem ( const DOMString &/*name*/, int
 
 NodeImpl *GenericRONamedNodeMapImpl::item ( unsigned long index ) const
 {
-    // ### check this when calling from javascript using -1 = 2^sizeof(int)-1
-    // (also for other similar methods)
     if (index >= m_contents->count())
         return 0;
 
@@ -1842,47 +1871,23 @@ unsigned long GenericRONamedNodeMapImpl::length(  ) const
     return m_contents->count();
 }
 
-NodeImpl *GenericRONamedNodeMapImpl::getNamedItemNS( const DOMString &namespaceURI,
-                                                     const DOMString &localName,
-                                                     int &/*exceptioncode*/ ) const
-{
-    NodeImpl::Id searchId = m_doc->tagId(namespaceURI.implementation(),
-                                         localName.implementation(), true, 0);
-
-    QPtrListIterator<NodeImpl> it(*m_contents);
-    for (; it.current(); ++it)
-        if (it.current()->id() == searchId)
-            return it.current();
-
-    return 0;
-}
-
-NodeImpl *GenericRONamedNodeMapImpl::setNamedItemNS( NodeImpl */*arg*/, int &exceptioncode )
-{
-    // can't modify this list through standard DOM functions
-    // NO_MODIFICATION_ALLOWED_ERR: Raised if this map is readonly
-    exceptioncode = DOMException::NO_MODIFICATION_ALLOWED_ERR;
-    return 0;
-}
-
-NodeImpl *GenericRONamedNodeMapImpl::removeNamedItemNS( const DOMString &/*namespaceURI*/,
-                                                        const DOMString &/*localName*/,
-                                                        int &exceptioncode )
-{
-    // can't modify this list through standard DOM functions
-    exceptioncode = DOMException::NO_MODIFICATION_ALLOWED_ERR;
-    return 0;
-}
-
 void GenericRONamedNodeMapImpl::addNode(NodeImpl *n)
 {
     // The spec says that in the case of duplicates we only keep the first one
-    int exceptioncode = 0;
-    if (getNamedItem(n->nodeName(),exceptioncode))
+    if (getNamedItem(n->id(), false, 0))
         return;
 
     n->ref();
     m_contents->append(n);
 }
 
-#endif
+NodeImpl::Id GenericRONamedNodeMapImpl::mapId(DOMStringImpl* namespaceURI,
+                                              DOMStringImpl* localName, bool readonly)
+{
+    if (!m_doc)
+	return 0;
+
+    return m_doc->getId(NodeImpl::ElementId,
+                        namespaceURI, 0, localName, readonly,
+                        false /*don't lookupHTML*/);
+}
