@@ -1,8 +1,10 @@
 /**
  * This file is part of the DOM implementation for KDE.
  *
- * Copyright (C) 1999 Lars Knoll (knoll@kde.org)
+ * Copyright (C) 1999-2003 Lars Knoll (knoll@kde.org)
+ *           (C) 2001-2003 Dirk Mueller (mueller@kde.org)
  *           (C) 1999 Antti Koivisto (koivisto@kde.org)
+ *           (C) 2002 Apple Computer, Inc.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -28,6 +30,7 @@
 
 #include "misc/htmlhashes.h"
 #include "rendering/render_text.h"
+#include "rendering/render_flow.h"
 
 #include <kdebug.h>
 
@@ -38,7 +41,7 @@ using namespace khtml;
 CharacterDataImpl::CharacterDataImpl(DocumentPtr *doc, DOMStringImpl* _text)
     : NodeImpl(doc)
 {
-    str = _text ? _text : new DOMStringImpl( 0, 0 );
+    str = _text ? _text : new DOMStringImpl(0, 0);
     str->ref();
 }
 
@@ -182,6 +185,11 @@ DOMString CharacterDataImpl::nodeValue() const
     return str;
 }
 
+bool CharacterDataImpl::containsOnlyWhitespace() const
+{
+    return str->containsOnlyWhitespace();
+}
+
 void CharacterDataImpl::setNodeValue( const DOMString &_nodeValue, int &exceptioncode )
 {
     // NO_MODIFICATION_ALLOWED_ERR: taken care of by setData()
@@ -222,14 +230,34 @@ void CharacterDataImpl::checkCharDataOperation( const unsigned long offset, int
     }
 }
 
-#ifndef NDEBUG
-void CharacterDataImpl::dump(QTextStream *stream, QString ind) const
+long CharacterDataImpl::minOffset() const
 {
-    *stream << " str=\"" << DOMString(str).string().ascii() << "\"";
+  RenderText *r = static_cast<RenderText *>(renderer());
+  if (!r || !r->isText()) return 0;
 
-    NodeImpl::dump(stream,ind);
-}
+  // take :first-letter into consideration
+#ifdef __GNUC__
+#warning FIXME
 #endif
+#if 0
+  if (r->forcedMinOffset()) {
+    RenderFlow *firstLetter = static_cast<RenderFlow *>(r->previousSibling());
+    if (firstLetter && firstLetter->isFlow() && firstLetter->isFirstLetter()) {
+      RenderText *letterText = static_cast<RenderText *>(firstLetter->firstChild());
+      return letterText->minOffset();
+    }
+  }
+#endif
+
+  return r->minOffset();
+}
+
+long CharacterDataImpl::maxOffset() const
+{
+  RenderText *r = static_cast<RenderText *>(renderer());
+  if (!r || !r->isText()) return (long)length();
+  return r->maxOffset();
+}
 
 // ---------------------------------------------------------------------------
 
@@ -324,9 +352,9 @@ void TextImpl::attach()
     assert(parentNode() && parentNode()->isElementNode());
 
     ElementImpl* element = static_cast<ElementImpl*>(parentNode());
-    if (!m_render && element->renderer()) {
+    if (element->renderer() && element->renderer()->childAllowed()) {
         khtml::RenderStyle* _style = element->renderer()->style();
-        m_render = new RenderText(this, str);
+        m_render = new (getDocument()->renderArena()) RenderText(this, str);
         m_render->setStyle(_style);
         parentNode()->renderer()->addChild(m_render, nextRenderer());
     }
