@@ -2,6 +2,7 @@
  * This file is part of the DOM implementation for KDE.
  *
  * Copyright (C) 2000 Peter Kelly (pmk@post.com)
+ * Copyright (C) 2003 Apple Computer, Inc.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -39,7 +40,58 @@
 using namespace DOM;
 using namespace khtml;
 
+XMLIncrementalSource::XMLIncrementalSource()
+    : QXmlInputSource(), m_pos( 0 ), m_unicode( 0 ),
+      m_finished( false )
+{
+}
+
+void XMLIncrementalSource::fetchData()
+{
+    //just a dummy to overwrite default behavior
+}
+
+QChar XMLIncrementalSource::next()
+{
+    if ( m_finished )
+        return QXmlInputSource::EndOfDocument;
+    else if ( m_data.length() <= m_pos )
+        return QXmlInputSource::EndOfData;
+    else
+        return m_unicode[m_pos++];
+}
+
+void XMLIncrementalSource::setData( const QString& str )
+{
+    m_data = str;
+    m_unicode = m_data.unicode();
+    m_pos = 0;
+    if ( !str.isEmpty() )
+        m_finished = false;
+}
+void XMLIncrementalSource::setData( const QByteArray& data )
+{
+    setData( fromRawData( data, true ) );
+}
+
+void XMLIncrementalSource::appendXML( const QString& str )
+{
+    m_data += str;
+    m_unicode = m_data.unicode();
+}
+
+QString XMLIncrementalSource::data()
+{
+    return m_data;
+}
+
+void XMLIncrementalSource::setFinished( bool finished )
+{
+    m_finished = finished;
+}
+
 XMLHandler::XMLHandler(DocumentPtr *_doc, KHTMLView *_view)
+    : errorLine(0)
 {
     m_doc = _doc;
     if ( m_doc ) m_doc->ref();
@@ -76,18 +128,15 @@ bool XMLHandler::startElement( const QString& namespaceURI, const QString& /*loc
         exitText();
 
     ElementImpl *newElement;
-    newElement = m_doc->document()->createElementNS(namespaceURI, qName, 0);
+    newElement = m_doc->document()->createElementNS(namespaceURI,qName);
 
     int i;
     for (i = 0; i < atts.length(); i++) {
         int exceptioncode = 0;
         DOMString uri(atts.uri(i));
-        DOMString ln(atts.localName(i));
+        DOMString qn(atts.qName(i));
         DOMString val(atts.value(i));
-        NodeImpl::Id id = m_doc->document()->attrId(uri.implementation(),
-                                                    ln.implementation(),
-                                                    false /* allocate */, 0);
-        newElement->setAttribute(id, val.implementation(), exceptioncode);
+        newElement->setAttributeNS(uri, qn, val, exceptioncode);
         if (exceptioncode) // exception setting attributes
             return false;
     }
@@ -113,8 +162,7 @@ bool XMLHandler::endElement( const QString& /*namespaceURI*/, const QString& /*l
     if (m_currentNode->nodeType() == Node::TEXT_NODE)
         exitText();
     if (m_currentNode->parentNode() != 0) {
-        if (m_currentNode->renderer())
-            m_currentNode->renderer()->close();
+        m_currentNode->closeRenderer();
         m_currentNode = m_currentNode->parentNode();
     }
 // ###  else error
@@ -151,21 +199,26 @@ bool XMLHandler::endCDATA()
 
 bool XMLHandler::characters( const QString& ch )
 {
+#if 1 // SAFARI_MERGE remove
     if (ch.stripWhiteSpace().isEmpty())
         return true;
+#endif
 
     if (m_currentNode->nodeType() == Node::TEXT_NODE ||
         m_currentNode->nodeType() == Node::CDATA_SECTION_NODE ||
         enterText()) {
 
-        unsigned short parentId = m_currentNode->parentNode() ? m_currentNode->parentNode()->id() : 0;
+#if 1 // SAFARI_MERGE
+        NodeImpl::Id parentId = m_currentNode->parentNode() ? m_currentNode->parentNode()->id() : 0;
         if (parentId == ID_SCRIPT || parentId == ID_STYLE || parentId == ID_XMP || parentId == ID_TEXTAREA) {
             // ### hack.. preserve whitespace for script, style, xmp and textarea... is this the correct
             // way of doing this?
+#endif
             int exceptioncode = 0;
             static_cast<TextImpl*>(m_currentNode)->appendData(ch,exceptioncode);
             if (exceptioncode)
                 return false;
+#if 1 // SAFARI_MERGE
         }
         else {
             // for all others, simplify the whitespace
@@ -174,6 +227,7 @@ bool XMLHandler::characters( const QString& ch )
             if (exceptioncode)
                 return false;
         }
+#endif
         return true;
     }
     else
@@ -262,9 +316,8 @@ bool XMLHandler::internalEntityDecl(const QString &name, const QString &value)
     EntityImpl *e = new EntityImpl(m_doc,name);
     // ### further parse entities inside the value and add them as separate nodes (or entityreferences)?
     e->addChild(m_doc->document()->createTextNode(new DOMStringImpl(value.unicode(), value.length())));
-// ### FIXME
-//     if (m_doc->document()->doctype())
-//         static_cast<GenericRONamedNodeMapImpl*>(m_doc->document()->doctype()->entities())->addNode(e);
+     if (m_doc->document()->doctype())
+         static_cast<GenericRONamedNodeMapImpl*>(m_doc->document()->doctype()->entities())->addNode(e);
     return true;
 }
 
@@ -289,18 +342,24 @@ bool XMLHandler::unparsedEntityDecl(const QString &/*name*/, const QString &/*pu
 //------------------------------------------------------------------------------
 
 XMLTokenizer::XMLTokenizer(DOM::DocumentPtr *_doc, KHTMLView *_view)
+    : m_handler(_doc,_view)
 {
     m_doc = _doc;
     if ( m_doc ) m_doc->ref();
     m_view = _view;
-    m_xmlCode = "";
     m_scriptsIt = 0;
     m_cachedScript = 0;
+    m_noErrors = true;
+    m_reader.setContentHandler( &m_handler );
+    m_reader.setLexicalHandler( &m_handler );
+    m_reader.setErrorHandler( &m_handler );
+    m_reader.setDeclHandler( &m_handler );
+    m_reader.setDTDHandler( &m_handler );
 }
 
 XMLTokenizer::~XMLTokenizer()
 {
-    if ( m_doc ) m_doc->deref();
+    if (m_doc) m_doc->deref();
     if (m_scriptsIt)
         delete m_scriptsIt;
     if (m_cachedScript)
@@ -310,33 +369,31 @@ XMLTokenizer::~XMLTokenizer()
 
 void XMLTokenizer::begin()
 {
+    // parse xml file
+    m_reader.parse( &m_source, true );
 }
 
-void XMLTokenizer::write( const QString &str, bool /*appendData*/ )
+void XMLTokenizer::write( const QString &str, bool appendData )
 {
-    m_xmlCode += str;
+    if ( !m_noErrors && appendData )
+        return;
+    if ( appendData ) {
+        m_source.appendXML( str );
+        m_noErrors = m_reader.parseContinue();
+    }
 }
 
 void XMLTokenizer::end()
 {
+    m_source.setFinished( true );
+    //if ( m_noErrors )
+    //m_noErrors = m_reader.parseContinue();
     emit finishedParsing();
 }
 
 void XMLTokenizer::finish()
 {
-    // parse xml file
-    XMLHandler handler(m_doc,m_view);
-    QXmlInputSource source;
-    source.setData(m_xmlCode);
-    QXmlSimpleReader reader;
-    reader.setContentHandler( &handler );
-    reader.setLexicalHandler( &handler );
-    reader.setErrorHandler( &handler );
-    reader.setDeclHandler( &handler );
-    reader.setDTDHandler( &handler );
-    bool ok = reader.parse( source );
-
-    if (!ok) {
+    if (!m_noErrors) {
         // An error occurred during parsing of the code. Display an error page to the user (the DOM
         // tree is created manually and includes an excerpt from the code where the error is located)
 
@@ -347,47 +404,57 @@ void XMLTokenizer::finish()
         while (m_doc->document()->hasChildNodes())
             static_cast<NodeImpl*>(m_doc->document())->removeChild(m_doc->document()->firstChild(),exceptioncode);
 
-        QTextIStream stream(&m_xmlCode);
-        unsigned long lineno;
-        for (lineno = 0; lineno < handler.errorLine-1; lineno++)
-          stream.readLine();
-        QString line = stream.readLine();
+        QString line, errorLocPtr;
+        if ( m_handler.errorLine ) {
+            QString xmlCode = m_source.data();
+            QTextIStream stream(&xmlCode);
+            for (unsigned long lineno = 0; lineno < m_handler.errorLine-1; lineno++)
+                stream.readLine();
+            line = stream.readLine();
 
-        unsigned long colno;
-        QString errorLocPtr = "";
-        for (colno = 0; colno < handler.errorCol-1; colno++)
-            errorLocPtr += " ";
-        errorLocPtr += "^";
+            for (unsigned long colno = 0; colno < m_handler.errorCol-1; colno++)
+                errorLocPtr += " ";
+            errorLocPtr += "^";
+        }
 
         // Create elements for display
         DocumentImpl *doc = m_doc->document();
-        NodeImpl *html = doc->createElementNS(XHTML_NAMESPACE,"html", 0);
-        NodeImpl   *body = doc->createElementNS(XHTML_NAMESPACE,"body", 0);
-        NodeImpl     *h1 = doc->createElementNS(XHTML_NAMESPACE,"h1", 0);
+        NodeImpl *html = doc->createElementNS(XHTML_NAMESPACE,"html");
+        NodeImpl   *body = doc->createElementNS(XHTML_NAMESPACE,"body");
+        NodeImpl     *h1 = doc->createElementNS(XHTML_NAMESPACE,"h1");
         NodeImpl       *headingText = doc->createTextNode(i18n("XML parsing error"));
-        NodeImpl     *errorText = doc->createTextNode(handler.errorProtocol());
-        NodeImpl     *hr = doc->createElementNS(XHTML_NAMESPACE,"hr", 0);
-        NodeImpl     *pre = doc->createElementNS(XHTML_NAMESPACE,"pre", 0);
-        NodeImpl       *lineText = doc->createTextNode(line+"\n");
-        NodeImpl       *errorLocText = doc->createTextNode(errorLocPtr);
+        NodeImpl     *errorText = doc->createTextNode(m_handler.errorProtocol());
+        NodeImpl     *hr = 0;
+        NodeImpl     *pre = 0;
+        NodeImpl     *lineText = 0;
+        NodeImpl     *errorLocText = 0;
+        if ( !line.isNull() ) {
+            hr = doc->createElementNS(XHTML_NAMESPACE,"hr");
+            pre = doc->createElementNS(XHTML_NAMESPACE,"pre");
+            lineText = doc->createTextNode(line+"\n");
+            errorLocText = doc->createTextNode(errorLocPtr);
+        }
 
         // Construct DOM tree. We ignore exceptions as we assume they will not be thrown here (due to the
         // fact we are using a known tag set)
         doc->appendChild(html,exceptioncode);
         html->appendChild(body,exceptioncode);
-        body->appendChild(h1,exceptioncode);
+        if ( body )
+            body->appendChild(h1,exceptioncode);
         h1->appendChild(headingText,exceptioncode);
         body->appendChild(errorText,exceptioncode);
         body->appendChild(hr,exceptioncode);
         body->appendChild(pre,exceptioncode);
-        pre->appendChild(lineText,exceptioncode);
-        pre->appendChild(errorLocText,exceptioncode);
+        if ( pre ) {
+            pre->appendChild(lineText,exceptioncode);
+            pre->appendChild(errorLocText,exceptioncode);
+        }
 
         // Close the renderers so that they update their display correctly
         // ### this should not be necessary, but requires changes in the rendering code...
-        h1->renderer()->close();
-        pre->renderer()->close();
-        body->renderer()->close();
+        h1->closeRenderer();
+        if ( pre ) pre->closeRenderer();
+        body->closeRenderer();
 
         m_doc->document()->recalcStyle( NodeImpl::Inherit );
         m_doc->document()->updateRendering();
@@ -428,7 +495,7 @@ void XMLTokenizer::executeScripts()
         DOMString scriptSrc = m_scriptsIt->current()->getAttribute(ATTR_SRC);
         QString charset = m_scriptsIt->current()->getAttribute(ATTR_CHARSET).string();
 
-        if (scriptSrc != "") {
+        if (!scriptSrc.isEmpty()) {
             // we have a src attribute
             m_cachedScript = m_doc->document()->docLoader()->requestScript(scriptSrc, charset);
             ++(*m_scriptsIt);
@@ -450,7 +517,7 @@ void XMLTokenizer::executeScripts()
             // another page, etc. (also in notifyFinished())
             // ### the script may add another script node after this one which should be executed
             if (m_view) {
-                m_view->part()->executeScript(scriptCode);
+                m_view->part()->executeScript(DOM::Node(), scriptCode);
             }
             ++(*m_scriptsIt);
         }
@@ -473,7 +540,7 @@ void XMLTokenizer::notifyFinished(CachedObject *finishedObj)
         DOMString scriptSource = m_cachedScript->script();
         m_cachedScript->deref(this);
         m_cachedScript = 0;
-        m_view->part()->executeScript(scriptSource.string());
+        m_view->part()->executeScript(DOM::Node(), scriptSource.string());
         executeScripts();
     }
 }
