@@ -18,7 +18,7 @@
  * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
  *
- * $Id: xml_tokenizer.h,v 1.17.2.1 2003/05/18 12:34:41 mueller Exp $
+ * $Id: xml_tokenizer.h,v 1.18.2.1 2004/02/29 15:28:07 mueller Exp $
  */
 
 #ifndef _XML_Tokenizer_h_
@@ -44,6 +44,8 @@ namespace DOM {
     class HTMLScriptElementImpl;
 }
 
+namespace khtml {
+
 class XMLHandler : public QXmlDefaultHandler
 {
 public:
@@ -120,6 +122,26 @@ signals:
 
 };
 
+class XMLIncrementalSource : public QXmlInputSource
+{
+public:
+    XMLIncrementalSource();
+    virtual void fetchData();
+    virtual QChar next();
+    virtual void setData( const QString& str );
+    virtual void setData( const QByteArray& data );
+    virtual QString data();
+
+    void appendXML( const QString& str );
+    void setFinished( bool );
+
+private:
+    QString      m_data;
+    uint         m_pos;
+    const QChar *m_unicode;
+    bool         m_finished;
+};
+
 class XMLTokenizer : public Tokenizer, public khtml::CachedObjectClient
 {
 public:
@@ -140,10 +162,16 @@ protected:
     void executeScripts();
     void addScripts(DOM::NodeImpl *n);
 
-    QString m_xmlCode;
     QPtrList<DOM::HTMLScriptElementImpl> m_scripts;
     QPtrListIterator<DOM::HTMLScriptElementImpl> *m_scriptsIt;
     khtml::CachedScript *m_cachedScript;
+
+    XMLHandler m_handler;
+    QXmlSimpleReader m_reader;
+    XMLIncrementalSource m_source;
+    bool m_noErrors;
 };
 
+} // end namespace
+
 #endif
