@@ -4,7 +4,7 @@
  *           (c) 1999 Mario Weilguni <mweilguni@sime.com>
  *           (c) 2001 Lubos Lunak <l.lunak@kde.org>
  *
- * $Id: kinit.cpp,v 1.105.2.8 2003/08/13 08:54:43 waba Exp $
+ * $Id: kinit.cpp,v 1.136.2.1 2004/03/05 14:42:33 waba Exp $
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -21,9 +21,8 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#ifdef HAVE_CONFIG_H
+#include "config.h"
 #include <config.h>
-#endif
 
 #include <sys/types.h>
 #include <sys/time.h>
@@ -57,12 +56,19 @@
 #include <klibloader.h>
 #include <kapplication.h>
 #include <klocale.h>
-#include <kstartupinfo.h>
+
+#if defined Q_WS_X11 && ! defined K_WS_QTONLY
+#include <kstartupinfo.h> // schroder
+#endif
+
+#include <kdeversion.h>
 
 #include "ltdl.h"
 #include "klauncher_cmds.h"
 
+//#if defined Q_WS_X11 && ! defined K_WS_QTONLY
 #ifdef Q_WS_X11
+//#undef K_WS_QTONLY
 #include <X11/Xlib.h>
 #include <X11/Xatom.h>
 #endif
@@ -81,14 +87,10 @@
 # endif
 #endif
 
-#if defined(HAVE_NSGETENVIRON) && defined(HAVE_CRT_EXTERNS_H)
-# include <crt_externs.h>
-# define environ (*_NSGetEnviron())
-#endif
-
 extern char **environ;
 
 extern int lt_dlopen_flag;
+//#if defined Q_WS_X11 && ! defined K_WS_QTONLY
 #ifdef Q_WS_X11
 static int X11fd = -1;
 static Display *X11display = 0;
@@ -98,11 +100,15 @@ static Display *X11_startup_notify_display = 0;
 static const KInstance *s_instance = 0;
 #define MAX_SOCK_FILE 255
 static char sock_file[MAX_SOCK_FILE];
+static char sock_file_old[MAX_SOCK_FILE];
 
+//#if defined Q_WS_X11 && ! defined K_WS_QTONLY
 #ifdef Q_WS_X11
 #define DISPLAY "DISPLAY"
 #elif defined(Q_WS_QWS)
 #define DISPLAY "QWS_DISPLAY"
+#elif defined(K_WS_QTONLY)
+#define DISPLAY "QT_DISPLAY"
 #else
 #error Use QT/X11 or QT/Embedded
 #endif
@@ -115,6 +121,7 @@ static struct {
   int deadpipe[2]; /* pipe used to detect dead children */
   int initpipe[2];
   int wrapper; /* socket for wrapper communication */
+  int wrapper_old; /* old socket for wrapper communication */
   char result;
   int exit_status;
   pid_t fork;
@@ -133,10 +140,13 @@ static struct {
   bool suicide;
 } d;
 
+//#if defined Q_WS_X11 && ! defined K_WS_QTONLY
+#ifdef Q_WS_X11
 extern "C" {
 int kdeinit_xio_errhandler( Display * );
 int kdeinit_x_errhandler( Display *, XErrorEvent *err );
 }
+#endif
 
 /*
  * Close fd's which are only useful for the parent process.
@@ -178,7 +188,13 @@ static void close_fds()
       close(d.wrapper);
       d.wrapper = 0;
    }
-#ifdef Q_WS_X11
+   if (d.wrapper_old)
+   {
+      close(d.wrapper_old);
+      d.wrapper_old = 0;
+   }
+#if defined Q_WS_X11 && ! defined K_WS_QTONLY
+//#ifdef Q_WS_X11
    if (X11fd >= 0)
    {
       close(X11fd);
@@ -233,23 +249,34 @@ static void setup_tty( const char* tty )
 }
 
 // from kdecore/netwm.cpp
-static int get_current_desktop( Display* disp )
+static void get_current_desktop_and_timestamp( Display* disp,
+    int& desktop, long& timestamp )
 {
-#ifdef Q_WS_X11 // Only X11 supports multiple desktops
+#if defined Q_WS_X11 && ! defined K_WS_QTONLY
+//#ifdef Q_WS_X11 // Only X11 supports multiple desktops
     Atom net_current_desktop = XInternAtom( disp, "_NET_CURRENT_DESKTOP", False );
+    unsigned char data[ 1 ];
+    XSelectInput( disp, DefaultRootWindow( disp ), PropertyChangeMask );
+    // only touch the property to get PropertyNotify with timestamp
+    XChangeProperty( disp, DefaultRootWindow( disp ), net_current_desktop, XA_CARDINAL,
+        32, PropModeAppend, data, 0 );
     Atom type_ret;
     int format_ret;
     unsigned char *data_ret;
     unsigned long nitems_ret, unused;
+    desktop = 0; // no desktop by default
     if( XGetWindowProperty( disp, DefaultRootWindow( disp ), net_current_desktop,
         0l, 1l, False, XA_CARDINAL, &type_ret, &format_ret, &nitems_ret, &unused, &data_ret )
 	    == Success)
     {
 	if (type_ret == XA_CARDINAL && format_ret == 32 && nitems_ret == 1)
-	    return *((long *) data_ret) + 1;
+	    desktop = *((long *) data_ret) + 1;
     }
+    XEvent ev;
+    XWindowEvent( disp, DefaultRootWindow( disp ), PropertyChangeMask, &ev );
+    timestamp = ev.xproperty.time;
+    XSelectInput( disp, DefaultRootWindow( disp ), NoEventMask );
 #endif
-    return 0;
 }
 
 // var has to be e.g. "DISPLAY=", i.e. with =
@@ -270,7 +297,8 @@ const char* get_env_var( const char* var, int envc, const char* envs )
     return NULL;
 }
 
-#ifdef Q_WS_X11 // FIXME(E): Implement for Qt/Embedded
+#if defined Q_WS_X11 && ! defined K_WS_QTONLY
+//#ifdef Q_WS_X11 // FIXME(E): Implement for Qt/Embedded
 static void init_startup_info( KStartupInfoId& id, const char* bin,
     int envc, const char* envs )
 {
@@ -282,9 +310,13 @@ static void init_startup_info( KStartupInfoId& id, const char* bin,
         return;
     X11_startup_notify_fd = XConnectionNumber( X11_startup_notify_display );
     KStartupInfoData data;
-    int desktop = get_current_desktop( X11_startup_notify_display );
+    int desktop;
+    long timestamp;
+    get_current_desktop_and_timestamp( X11_startup_notify_display,
+        desktop, timestamp );
     data.setDesktop( desktop );
     data.setBin( bin );
+    data.setTimestamp( timestamp );
     KStartupInfo::sendStartupX( X11_startup_notify_display, id, data );
     XFlush( X11_startup_notify_display );
 }
@@ -352,9 +384,7 @@ static pid_t launch(int argc, const char *_name, const char *args,
 
   if (strcmp(_name, "klauncher") == 0) {
      /* klauncher is launched in a special way:
-      * instead of calling 'main(argc, argv)',
-      * we call 'start_launcher(comm_socket)'.
-      * The other end of the socket is d.launcher[0].
+      * It has a communication socket on LAUNCHER_FD
       */
      if (0 > socketpair(AF_UNIX, SOCK_STREAM, 0, d.launcher))
      {
@@ -397,7 +427,8 @@ static pid_t launch(int argc, const char *_name, const char *args,
      exit(255);
   }
 
-#ifdef Q_WS_X11
+#if defined Q_WS_X11 && ! defined K_WS_QTONLY
+//#ifdef Q_WS_X11
   KStartupInfoId startup_id;
   startup_id.initId( startup_id_str );
   if( !startup_id.none())
@@ -415,6 +446,19 @@ static pid_t launch(int argc, const char *_name, const char *args,
      /** Child **/
      close(d.fd[0]);
      close_fds();
+     if (launcher)
+     {
+        if (d.fd[1] == LAUNCHER_FD)
+        {
+          d.fd[1] = dup(d.fd[1]); // Evacuate from LAUNCHER_FD
+        }
+        if (d.launcher[1] != LAUNCHER_FD)
+        {
+          dup2( d.launcher[1], LAUNCHER_FD); // Make sure the socket has fd LAUNCHER_FD
+          close( d.launcher[1] );
+        }
+        close( d.launcher[0] );
+     }
 
      if (cwd && *cwd)
         chdir(cwd);
@@ -445,7 +489,8 @@ static pid_t launch(int argc, const char *_name, const char *args,
         envs++;
      }
 
-#ifdef Q_WS_X11
+#if defined Q_WS_X11 && ! defined K_WS_QTONLY
+//#ifdef Q_WS_X11
       if( startup_id.none())
           KStartupInfo::resetStartupEnv();
       else
@@ -476,7 +521,10 @@ static pid_t launch(int argc, const char *_name, const char *args,
         exitWithErrorMsg(errorMsg);
      }
 
-     if ( !libpath.isEmpty())
+     if ( getenv("KDE_IS_PRELINKED") && !execpath.isEmpty() && !launcher)
+         libpath.truncate(0);
+
+     if ( !libpath.isEmpty() )
      {
        d.handle = lt_dlopen( QFile::encodeName(libpath) );
        if (!d.handle )
@@ -485,14 +533,14 @@ static pid_t launch(int argc, const char *_name, const char *args,
           if (execpath.isEmpty())
           {
              // Error
-             QString errorMsg = i18n("Could not dlopen library '%1'.\n%2").arg(QFile::decodeName(libpath))
+             QString errorMsg = i18n("Could not open library '%1'.\n%2").arg(QFile::decodeName(libpath))
 		.arg(ltdlError ? QFile::decodeName(ltdlError) : i18n("Unknown error"));
              exitWithErrorMsg(errorMsg);
           }
           else
           {
              // Print warning
-             fprintf(stderr, "Could not dlopen library %s: %s\n", lib.data(), ltdlError != 0 ? ltdlError : "(null)" );
+             fprintf(stderr, "Could not open library %s: %s\n", lib.data(), ltdlError != 0 ? ltdlError : "(null)" );
           }
        }
      }
@@ -515,63 +563,46 @@ static pid_t launch(int argc, const char *_name, const char *args,
         exit(255);
      }
 
-     if (!launcher)
+     d.sym = lt_dlsym( d.handle, "kdeinitmain");
+     if (!d.sym )
      {
-        d.sym = lt_dlsym( d.handle, "kdemain");
-        if (!d.sym )
-        {
+        d.sym = lt_dlsym( d.handle, "kdemain" );
+        if ( !d.sym ) 
+        { 
+#if ! KDE_IS_VERSION( 3, 90, 0 )
            d.sym = lt_dlsym( d.handle, "main");
+#endif
            if (!d.sym )
            {
               const char * ltdlError = lt_dlerror();
-              fprintf(stderr, "Could not find main: %s\n", ltdlError != 0 ? ltdlError : "(null)" );
-              QString errorMsg = i18n("Could not find 'main' in '%1'.\n%2").arg(libpath)
-		.arg(ltdlError ? QFile::decodeName(ltdlError) : i18n("Unknown error"));
+              fprintf(stderr, "Could not find kdemain: %s\n", ltdlError != 0 ? ltdlError : "(null)" );
+              QString errorMsg = i18n("Could not find 'kdemain' in '%1'.\n%2").arg(libpath)
+                 .arg(ltdlError ? QFile::decodeName(ltdlError) : i18n("Unknown error"));
               exitWithErrorMsg(errorMsg);
            }
         }
+     }
 
-        d.result = 0; // Success
-        write(d.fd[1], &d.result, 1);
-        close(d.fd[1]);
+     d.result = 0; // Success
+     write(d.fd[1], &d.result, 1);
+     close(d.fd[1]);
 
-        d.func = (int (*)(int, char *[])) d.sym;
-        if (d.debug_wait)
-        {
-           fprintf(stderr, "kdeinit: Suspending process\n"
-                           "kdeinit: 'gdb kdeinit %d' to debug\n"
-                           "kdeinit: 'kill -SIGCONT %d' to continue\n",
-                           getpid(), getpid());
-           kill(getpid(), SIGSTOP);
-        }
-        else
-            setup_tty( tty );
-
-	exit( d.func(argc, d.argv)); /* Launch! */
+     d.func = (int (*)(int, char *[])) d.sym;
+     if (d.debug_wait)
+     {
+        fprintf(stderr, "kdeinit: Suspending process\n"
+                        "kdeinit: 'gdb kdeinit %d' to debug\n"
+                        "kdeinit: 'kill -SIGCONT %d' to continue\n",
+                        getpid(), getpid());
+        kill(getpid(), SIGSTOP);
      }
      else
      {
-        d.sym = lt_dlsym( d.handle, "start_launcher");
-        if (!d.sym )
-        {
-           const char * ltdlError = lt_dlerror();
-           fprintf(stderr, "Could not find start_launcher: %s\n", ltdlError != 0 ? ltdlError : "(null)" );
-           d.result = 1; // Error
-           write(d.fd[1], &d.result, 1);
-           close(d.fd[1]);
-           exit(255);
-        }
-
-        d.result = 0; // Success
-        write(d.fd[1], &d.result, 1);
-        close(d.fd[1]);
-
-        d.launcher_func = (int (*)(int)) d.sym;
-        close(d.launcher[0]); // Close non-used socket.
-
-        exit( d.launcher_func( d.launcher[1] )); /* Launch! */
+        setup_tty( tty );
      }
 
+     exit( d.func(argc, d.argv)); /* Launch! */
+
      break;
   default:
      /** Parent **/
@@ -639,14 +670,15 @@ static pid_t launch(int argc, const char *_name, const char *args,
      close(d.fd[0]);
      if (launcher && (d.result == 0))
      {
-        // Trader launched successfull
+        // Trader launched successful
         d.launcher_pid = d.fork;
      }
   }
-#ifdef Q_WS_X11
+#if defined Q_WS_X11 && ! defined K_WS_QTONLY
+//#ifdef Q_WS_X11
   if( !startup_id.none())
   {
-     if( d.fork && d.result == 0 ) // launched succesfully    
+     if( d.fork && d.result == 0 ) // launched successfully
         complete_startup_info( startup_id, d.fork );
      else // failure, cancel ASN
         complete_startup_info( startup_id, 0 );
@@ -723,6 +755,7 @@ static void init_signals()
 static void init_kdeinit_socket()
 {
   struct sockaddr_un sa;
+  struct sockaddr_un sa_old;
   socklen_t socklen;
   long options;
   const char *home_dir = getenv("HOME");
@@ -800,6 +833,7 @@ static void init_kdeinit_socket()
 
   /** Delete any stale socket file (and symlink) **/
   unlink(sock_file);
+  unlink(sock_file_old);
 
   /** create socket **/
   d.wrapper = socket(PF_UNIX, SOCK_STREAM, 0);
@@ -860,6 +894,70 @@ static void init_kdeinit_socket()
      close(d.wrapper);
      exit(255);
   }
+
+  /** create compatibility socket **/
+  d.wrapper_old = socket(PF_UNIX, SOCK_STREAM, 0);
+  if (d.wrapper_old < 0)
+  {
+     // perror("kdeinit: Aborting. socket() failed: ");
+     return;
+  }
+
+  options = fcntl(d.wrapper_old, F_GETFL);
+  if (options == -1)
+  {
+     // perror("kdeinit: Aborting. Can't make socket non-blocking: ");
+     close(d.wrapper_old);
+     d.wrapper_old = 0;
+     return;
+  }
+
+  if (fcntl(d.wrapper_old, F_SETFL, options | O_NONBLOCK) == -1)
+  {
+     // perror("kdeinit: Aborting. Can't make socket non-blocking: ");
+     close(d.wrapper_old);
+     d.wrapper_old = 0;
+     return;
+  }
+
+  max_tries = 10;
+  while (1) {
+      /** bind it **/
+      socklen = sizeof(sa_old);
+      memset(&sa_old, 0, socklen);
+      sa_old.sun_family = AF_UNIX;
+      strcpy(sa_old.sun_path, sock_file_old);
+      if(bind(d.wrapper_old, (struct sockaddr *)&sa_old, socklen) != 0)
+      {
+          if (max_tries == 0) {
+	      // perror("kdeinit: Aborting. bind() failed: ");
+	      fprintf(stderr, "Could not bind to socket '%s'\n", sock_file_old);
+	      close(d.wrapper_old);
+	      d.wrapper_old = 0;
+	      return;
+	  }
+	  max_tries--;
+      } else
+          break;
+  }
+
+  /** set permissions **/
+  if (chmod(sock_file_old, 0600) != 0)
+  {
+     fprintf(stderr, "Wrong permissions of socket '%s'\n", sock_file);
+     unlink(sock_file_old);
+     close(d.wrapper_old);
+     d.wrapper_old = 0;
+     return;
+  }
+
+  if(listen(d.wrapper_old, SOMAXCONN) < 0)
+  {
+     // perror("kdeinit: Aborting. listen() failed: ");
+     unlink(sock_file_old);
+     close(d.wrapper_old);
+     d.wrapper_old = 0;
+  }
 }
 
 /*
@@ -911,6 +1009,14 @@ static void launcher_died()
 #ifndef NDEBUG
    fprintf(stderr, "kdeinit: KLauncher died unexpectedly.\n");
 #endif
+   // Make sure it's really dead.
+   if (d.launcher_pid)
+   {
+      kill(d.launcher_pid, SIGKILL);
+      sleep(1); // Give it some time
+   }
+
+   d.launcher_ok = false;
    d.launcher_pid = 0;
    close(d.launcher[0]);
    d.launcher[0] = -1;
@@ -919,7 +1025,6 @@ static void launcher_died()
 #ifndef NDEBUG
    fprintf(stderr, "kdeinit: Relaunching KLauncher, pid = %ld result = %d\n", (long) pid, d.result);
 #endif
-   WaitPid(pid);
 }
 
 static void handle_launcher_request(int sock = -1)
@@ -954,10 +1059,12 @@ static void handle_launcher_request(int sock = -1)
            return;
        }
    }
-   
-   d.launcher_ok = true;
 
-   if ((request_header.cmd == LAUNCHER_EXEC) ||
+   if (request_header.cmd == LAUNCHER_OK)
+   {
+      d.launcher_ok = true;
+   }
+   else if ((request_header.cmd == LAUNCHER_EXEC) ||
        (request_header.cmd == LAUNCHER_EXT_EXEC) ||
        (request_header.cmd == LAUNCHER_SHELL ) ||
        (request_header.cmd == LAUNCHER_KWRAPPER) ||
@@ -1030,7 +1137,7 @@ static void handle_launcher_request(int sock = -1)
          startup_id_str = arg_n;
          arg_n += strlen( startup_id_str ) + 1;
      }
-     
+
      if ((request_header.arg_length > (arg_n - request_data)) &&
          (request_header.cmd == LAUNCHER_EXT_EXEC || request_header.cmd == LAUNCHER_EXEC_NEW ))
      {
@@ -1143,9 +1250,12 @@ static void handle_launcher_request(int sock = -1)
 static void handle_requests(pid_t waitForPid)
 {
    int max_sock = d.wrapper;
+   if (d.wrapper_old > max_sock)
+      max_sock = d.wrapper_old;
    if (d.launcher_pid && (d.launcher[0] > max_sock))
       max_sock = d.launcher[0];
-#ifdef _WS_X11
+#if defined Q_WS_X11 && ! defined K_WS_QTONLY
+//#ifdef _WS_X11
    if (X11fd > max_sock)
       max_sock = X11fd;
 #endif
@@ -1173,12 +1283,8 @@ static void handle_requests(pid_t waitForPid)
 #endif
            if (waitForPid && (exit_pid == waitForPid))
               return;
-              
-           if (exit_pid == d.launcher_pid)
-           {
-             launcher_died();
-           }
-           else if (d.launcher_pid)
+
+           if (d.launcher_pid)
            {
            // TODO send process died message
               klauncher_header request_header;
@@ -1203,8 +1309,13 @@ static void handle_requests(pid_t waitForPid)
          FD_SET(d.launcher[0], &rd_set);
       }
       FD_SET(d.wrapper, &rd_set);
+      if (d.wrapper_old)
+      {
+         FD_SET(d.wrapper_old, &rd_set);
+      }
       FD_SET(d.deadpipe[0], &rd_set);
-#ifdef Q_WS_X11
+#if defined Q_WS_X11 && ! defined K_WS_QTONLY
+//#ifdef Q_WS_X11
       if(X11fd >= 0) FD_SET(X11fd, &rd_set);
 #endif
 
@@ -1227,13 +1338,32 @@ static void handle_requests(pid_t waitForPid)
             close(sock);
          }
       }
+      if ((result > 0) && (FD_ISSET(d.wrapper_old, &rd_set)))
+      {
+         struct sockaddr_un client;
+         socklen_t sClient = sizeof(client);
+         int sock = accept(d.wrapper_old, (struct sockaddr *)&client, &sClient);
+         if (sock >= 0)
+         {
+            if (fork() == 0)
+            {
+                close_fds();
+                handle_launcher_request(sock);
+                exit(255); /* Terminate process. */
+            }
+            close(sock);
+         }
+      }
 
       /* Handle launcher request */
       if ((result > 0) && (d.launcher_pid) && (FD_ISSET(d.launcher[0], &rd_set)))
       {
          handle_launcher_request();
+         if (waitForPid == d.launcher_pid)
+            return;
       }
 
+//#if defined Q_WS_X11 && ! defined K_WS_QTONLY
 #ifdef Q_WS_X11
       /* Look for incoming X11 events */
       if((result > 0) && (X11fd >= 0))
@@ -1315,6 +1445,16 @@ static void kdeinit_library_path()
      fprintf(stderr, "         '%s'\n", socketName.data());
      exit(255);
    }
+   strcpy(sock_file_old, socketName.data());
+
+   display.replace(":","_");
+   socketName = QFile::encodeName(locateLocal("socket", QString("kdeinit_%1").arg(display), s_instance));
+   if (socketName.length() >= MAX_SOCK_FILE)
+   {
+     fprintf(stderr, "kdeinit: Aborting. Socket name will be too long:\n");
+     fprintf(stderr, "         '%s'\n", socketName.data());
+     exit(255);
+   }
    strcpy(sock_file, socketName.data());
 }
 
@@ -1323,29 +1463,18 @@ int kdeinit_xio_errhandler( Display *disp )
     // disp is 0L when KDE shuts down. We don't want those warnings then.
 
     if ( disp )
-        qWarning( "kdeinit: Fatal IO error: client killed" );
+    qWarning( "kdeinit: Fatal IO error: client killed" );
 
     if (sock_file[0])
     {
       /** Delete any stale socket file **/
       unlink(sock_file);
     }
-
-    if ( disp )
-        qWarning( "kdeinit: sending SIGHUP to children." );
-
-    /* this should remove all children we started */
-    signal(SIGHUP, SIG_IGN);
-    kill(0, SIGHUP);
-
-    sleep(2);
-
-    if ( disp )
-        qWarning( "kdeinit: sending SIGTERM to children." );
-
-    /* and if they don't listen to us, this should work */
-    signal(SIGTERM, SIG_IGN);
-    kill(0, SIGTERM);
+    if (sock_file_old[0])
+    {
+      /** Delete any stale socket file **/
+      unlink(sock_file_old);
+    }
 
     // Don't kill our children in suicide mode, they may still be in use
     if (d.suicide)
@@ -1356,7 +1485,23 @@ int kdeinit_xio_errhandler( Display *disp )
     }
 
     if ( disp )
-        qWarning( "kdeinit: Exit." );
+    qWarning( "kdeinit: sending SIGHUP to children." );
+
+    /* this should remove all children we started */
+    signal(SIGHUP, SIG_IGN);
+    kill(0, SIGHUP);
+
+    sleep(2);
+
+    if ( disp )
+    qWarning( "kdeinit: sending SIGTERM to children." );
+
+    /* and if they don't listen to us, this should work */
+    signal(SIGTERM, SIG_IGN);
+    kill(0, SIGTERM);
+
+    if ( disp )
+    qWarning( "kdeinit: Exit." );
 
     exit( 0 );
     return 0;
@@ -1366,12 +1511,13 @@ int kdeinit_x_errhandler( Display *dpy, XErrorEvent *err )
 {
 #ifndef NDEBUG
     char errstr[256];
+    // kdeinit almost doesn't use X, and therefore there shouldn't be any X error
     XGetErrorText( dpy, err->error_code, errstr, 256 );
-    if ( err->error_code != BadWindow )
-    {
-        fprintf(stderr, "kdeinit: KDE detected X Error: %s %d\n", errstr, err->error_code );
-        fprintf(stderr, "  Major opcode:  %d\n", err->request_code);
-    }
+    fprintf(stderr, "kdeinit: KDE detected X Error: %s %d\n"
+                    "         Major opcode: %d\n"
+                    "         Minor opcode: %d\n"
+                    "         Resource id:  0x%lx\n",
+            errstr, err->error_code, err->request_code, err->minor_code, err->resourceid );
 #else
     Q_UNUSED(dpy);
     Q_UNUSED(err);
@@ -1380,15 +1526,21 @@ int kdeinit_x_errhandler( Display *dpy, XErrorEvent *err )
 }
 
 
+//#if defined Q_WS_X11 && ! defined K_WS_QTONLY
 #ifdef Q_WS_X11
+// needs to be done sooner than initXconnection() because of also opening
+// another X connection for startup notification purposes
+static void setupX()
+{
+    XSetIOErrorHandler(kdeinit_xio_errhandler);
+    XSetErrorHandler(kdeinit_x_errhandler);
+}
+
 // Borrowed from kdebase/kaudio/kaudioserver.cpp
 static int initXconnection()
 {
   X11display = XOpenDisplay(NULL);
   if ( X11display != 0 ) {
-    XSetIOErrorHandler(kdeinit_xio_errhandler);
-    XSetErrorHandler(kdeinit_x_errhandler);
-     
     XCreateSimpleWindow(X11display, DefaultRootWindow(X11display), 0,0,1,1, \
         0,
         BlackPixelOfScreen(DefaultScreenOfDisplay(X11display)),
@@ -1422,6 +1574,12 @@ static int initXconnection()
    which is difficult.  Currently (KAI 4.0f) it only calls __call_ctors(void)
    (a C++ function), but if that changes we need to change our's too.
    (matz) */
+/*
+ Those 'unknown reasons' are C++ standard forbidding recursive calls to main()
+ or any means that would possibly allow that (e.g. taking address of main()).
+ The correct solution is not using main() as entry point for kdeinit modules,
+ but only kdemain().
+*/
 extern "C" void _main(void);
 extern "C" void __call_ctors__Fv(void);
 static int main_called = 0;
@@ -1511,17 +1669,23 @@ int main(int argc, char **argv, char **envp)
    kdeinit_initsetproctitle(argc, argv, envp);
    kdeinit_setproctitle("Starting up...");
    kdeinit_library_path();
+   // don't change envvars before kdeinit_initsetproctitle()
    unsetenv("LD_BIND_NOW");
+   unsetenv("DYLD_BIND_AT_LAUNCH");
+   if (!d.suicide)
+      setenv("KDE_FULL_SESSION", "true", true);
    KApplication::loadedByKdeinit = true;
 
    d.maxname = strlen(argv[0]);
    d.launcher_pid = 0;
    d.wrapper = 0;
+   d.wrapper_old = 0;
    d.debug_wait = false;
    d.launcher_ok = false;
    d.lt_dlopen_flag = lt_dlopen_flag;
    lt_dlopen_flag |= LTDL_GLOBAL;
    init_signals();
+   setupX();
 
    if (keep_running)
    {
@@ -1549,13 +1713,11 @@ int main(int argc, char **argv, char **envp)
       }
    }
 
-   if (!d.suicide)
+   if (!d.suicide && !getenv("KDE_IS_PRELINKED"))
    {
       QString konq = locate("lib", "libkonq.la", s_instance);
       if (!konq.isEmpty())
-      {
-	  lt_dlhandle result = lt_dlopen(QFile::encodeName(konq).data());
-      }
+	  (void) lt_dlopen(QFile::encodeName(konq).data());
    }
 
    if (launch_klauncher)
@@ -1564,7 +1726,19 @@ int main(int argc, char **argv, char **envp)
 #ifndef NDEBUG
       fprintf(stderr, "kdeinit: Launched KLauncher, pid = %ld result = %d\n", (long) pid, d.result);
 #endif
-      WaitPid(pid);
+      handle_requests(pid); // Wait for klauncher to be ready
+   }
+
+   if ( !d.suicide )
+   {
+      pid = launch( 1, "krandrinithack", 0 );
+      if( pid > 0 && d.result == 0 )
+      {
+#ifndef NDEBUG
+         fprintf(stderr, "kdeinit: Launched KRandrInitHack, pid = %ld result = %d\n", (long) pid, d.result);
+#endif
+         WaitPid(pid);
+      }
    }
 
    if (launch_kded)
@@ -1576,7 +1750,8 @@ int main(int argc, char **argv, char **envp)
       handle_requests(pid);
    }
 
-#ifdef Q_WS_X11
+#if defined Q_WS_X11 && ! defined K_WS_QTONLY
+//#ifdef Q_WS_X11
    X11fd = initXconnection();
 #endif
 
