@@ -36,7 +36,11 @@
 #include <kprotocolinfo.h>
 #include <krun.h>
 #include <kstandarddirs.h>
-#include <kstartupinfo.h>
+
+#if defined Q_WS_X11 && ! defined K_WS_QTONLY
+#include <kstartupinfo.h> // schroder
+#endif
+
 #include <ktempfile.h>
 #include <kconfig.h>
 
@@ -47,8 +51,10 @@
 #include "klauncher.h"
 #include "klauncher_cmds.h"
 
+//#if defined Q_WS_X11 && ! defined K_WS_QTONLY
 #ifdef Q_WS_X11
-#include <X11/Xlib.h>
+//#undef K_WS_QTONLY
+#include <X11/Xlib.h> // schroder
 #endif
 
 // Dispose slaves after being idle for SLAVE_MAX_IDLE seconds
@@ -156,7 +162,8 @@ IdleSlave::age(time_t now)
 }
 
 KLauncher::KLauncher(int _kdeinitSocket)
-  : KUniqueApplication( false, false ), // No Styles, No GUI
+  : KApplication( false, false ), // No Styles, No GUI
+    DCOPObject("klauncher"), 
     kdeinitSocket(_kdeinitSocket), dontBlockReading(false)
 {
 #ifdef Q_WS_X11
@@ -198,6 +205,10 @@ KLauncher::KLauncher(int _kdeinitSocket)
    {
       qWarning("Klauncher running in slave-debug mode for slaves of protocol '%s'", mSlaveDebug.data());
    }
+   klauncher_header request_header;
+   request_header.cmd = LAUNCHER_OK;
+   request_header.arg_length = 0;
+   write(kdeinitSocket, &request_header, sizeof(request_header));
 }
 
 KLauncher::~KLauncher()
@@ -207,7 +218,8 @@ KLauncher::~KLauncher()
       QCString filename = QFile::encodeName(mPoolSocketName);
       unlink(filename.data());
    }
-#ifdef Q_WS_X11
+#if defined Q_WS_X11 && ! defined K_WS_QTONLY
+//#ifdef Q_WS_X11
    if( mCached_dpy != NULL )
        XCloseDisplay( mCached_dpy );
 #endif
@@ -252,7 +264,9 @@ KLauncher::process(const QCString &fun, const QByteArray &data,
        (fun == "start_service_by_desktop_path(QString,QStringList,QValueList<QCString>,QCString,bool)")||
        (fun == "start_service_by_desktop_name(QString,QStringList,QValueList<QCString>,QCString,bool)") ||
        (fun == "kdeinit_exec(QString,QStringList,QValueList<QCString>)") ||
-       (fun == "kdeinit_exec_wait(QString,QStringList,QValueList<QCString>)"))
+       (fun == "kdeinit_exec_wait(QString,QStringList,QValueList<QCString>)") ||
+       (fun == "kdeinit_exec(QString,QStringList,QValueList<QCString>,QCString)") ||
+       (fun == "kdeinit_exec_wait(QString,QStringList,QValueList<QCString>,QCString)"))
    {
       QDataStream stream(data, IO_ReadOnly);
       bool bNoWait = false;
@@ -276,6 +290,9 @@ KLauncher::process(const QCString &fun, const QByteArray &data,
       else if ((fun == "kdeinit_exec(QString,QStringList,QValueList<QCString>)") ||
           (fun == "kdeinit_exec_wait(QString,QStringList,QValueList<QCString>)"))
          stream >> envs;
+      else if ((fun == "kdeinit_exec(QString,QStringList,QValueList<QCString>,QCString)") ||
+          (fun == "kdeinit_exec_wait(QString,QStringList,QValueList<QCString>,QCString)"))
+         stream >> envs >> startup_id;
       bool finished;
       if (strncmp(fun, "start_service_by_name(", 22) == 0)
       {
@@ -293,15 +310,16 @@ KLauncher::process(const QCString &fun, const QByteArray &data,
          finished = start_service_by_desktop_name(serviceName, urls, envs, startup_id, bNoWait );
       }
       else if ((fun == "kdeinit_exec(QString,QStringList)")
-              || (fun == "kdeinit_exec(QString,QStringList,QValueList<QCString>)"))
+              || (fun == "kdeinit_exec(QString,QStringList,QValueList<QCString>)")
+              || (fun == "kdeinit_exec(QString,QStringList,QValueList<QCString>,QCString)"))
       {
          kdDebug(7016) << "KLauncher: Got kdeinit_exec('" << serviceName << "', ...)" << endl;
-         finished = kdeinit_exec(serviceName, urls, envs, false);
+         finished = kdeinit_exec(serviceName, urls, envs, startup_id, false);
       }
       else
       {
          kdDebug(7016) << "KLauncher: Got kdeinit_exec_wait('" << serviceName << "', ...)" << endl;
-         finished = kdeinit_exec(serviceName, urls, envs, true);
+         finished = kdeinit_exec(serviceName, urls, envs, startup_id, true);
       }
       if (!finished)
       {
@@ -397,7 +415,7 @@ KLauncher::process(const QCString &fun, const QByteArray &data,
       return true;
    }
 
-   if (KUniqueApplication::process(fun, data, replyType, replyData))
+   if (DCOPObject::process(fun, data, replyType, replyData))
    {
       return true;
    }
@@ -408,7 +426,7 @@ KLauncher::process(const QCString &fun, const QByteArray &data,
 QCStringList
 KLauncher::interfaces()
 {
-    QCStringList ifaces = KUniqueApplication::interfaces();
+    QCStringList ifaces = DCOPObject::interfaces();
     ifaces += "KLauncher";
     return ifaces;
 }
@@ -416,7 +434,7 @@ KLauncher::interfaces()
 QCStringList
 KLauncher::functions()
 {
-    QCStringList funcs = KUniqueApplication::functions();
+    QCStringList funcs = DCOPObject::functions();
     funcs << "void exec_blind(QCString,QValueList<QCString>)";
     funcs << "void exec_blind(QCString,QValueList<QCString>,QValueList<QCString>,QCString)";
     funcs << "serviceResult start_service_by_name(QString,QStringList)";
@@ -601,20 +619,34 @@ KLauncher::slotAppRegistered(const QCString &appId)
    if (!cAppId) return;
 
    KLaunchRequest *request = requestList.first();
-   for(; request; request = requestList.next())
+   KLaunchRequest *nextRequest;
+   for(; request; request = nextRequest)
    {
+      nextRequest = requestList.next();
+      if (request->status != KLaunchRequest::Launching)
+         continue;
+
+      // For unique services check the requested service name first
+      if ((request->dcop_service_type == KService::DCOP_Unique) &&
+          ((appId == request->dcop_name) || 
+           dcopClient()->isApplicationRegistered(request->dcop_name)))
+      {	  
+         request->status = KLaunchRequest::Running;
+         requestDone(request);
+         continue;
+      }
+
       const char *rAppId = request->dcop_name.data();
       if (!rAppId) continue;
 
       int l = strlen(rAppId);
-      if ((request->status == KLaunchRequest::Launching) &&
-          (strncmp(rAppId, cAppId, l) == 0) &&
+      if ((strncmp(rAppId, cAppId, l) == 0) &&
           ((cAppId[l] == '\0') || (cAppId[l] == '-')))
       {
          request->dcop_name = appId;
          request->status = KLaunchRequest::Running;
          requestDone(request);
-         return;
+         continue;
       }
    }
 }
@@ -640,12 +672,16 @@ KLauncher::slotAutoStart()
       if (service.isEmpty())
       {
          // Done
-         // Emit signal
-	 QCString autoStartSignal( "autoStartDone()" );
-	 int phase = mAutoStart.phase();
-	 if ( phase > 1 )
-	     autoStartSignal.sprintf( "autoStart%dDone()", phase ); 
-         emitDCOPSignal(autoStartSignal, QByteArray());
+	 if( !mAutoStart.phaseDone())
+	 {
+	    mAutoStart.setPhaseDone();
+	    // Emit signal
+	    QCString autoStartSignal( "autoStartDone()" );
+	    int phase = mAutoStart.phase();
+	    if ( phase > 1 )
+	        autoStartSignal.sprintf( "autoStart%dDone()", phase ); 
+            emitDCOPSignal(autoStartSignal, QByteArray());
+	 }
          return;
       }
       s = new KService(service);
@@ -674,7 +710,8 @@ KLauncher::requestDone(KLaunchRequest *request)
          DCOPresult.error += ":\n" + request->errorMsg;
       DCOPresult.pid = 0;
 
-#ifdef Q_WS_X11
+#if defined Q_WS_X11 && ! defined K_WS_QTONLY
+//#ifdef Q_WS_X11
       if (!request->startup_dpy.isEmpty())
       {
          Display* dpy = NULL;
@@ -811,12 +848,8 @@ KLauncher::exec_blind( const QCString &name, const QValueList<QCString> &arg_lis
    request->status = KLaunchRequest::Launching;
    request->transaction = 0; // No confirmation is send
    request->envs = envs;
-   // Find service, if any
-   KService::Ptr service = 0;
-   if (name[0] == '/') // Full path
-      service = new KService(name);
-   else
-      service = KService::serviceByDesktopName(name);
+   // Find service, if any - strip path if needed
+   KService::Ptr service = KService::serviceByDesktopName( name.mid( name.findRev( '/' ) + 1 ));
    if (service != NULL)
        send_service_startup_info( request,  service,
            startup_id, QValueList< QCString >());
@@ -971,12 +1004,22 @@ void
 KLauncher::send_service_startup_info( KLaunchRequest *request, KService::Ptr service, const QCString& startup_id,
     const QValueList<QCString> &envs )
 {
-#ifdef Q_WS_X11 // KStartup* isn't implemented for Qt/Embedded yet
+#if defined Q_WS_X11 && ! defined K_WS_QTONLY
+//#ifdef Q_WS_X11 // KStartup* isn't implemented for Qt/Embedded yet
     request->startup_id = "0";
     if( startup_id == "0" )
         return;
     QCString wmclass;
-    if( service->property( "X-KDE-StartupNotify" ).isValid())
+    if( service->property( "StartupNotify" ).isValid())
+    {
+        if( !service->property( "StartupNotify" ).toBool())
+        {
+            cancel_service_startup_info( request, startup_id, envs );
+            return;
+        }
+        wmclass = service->property( "StartupWMClass" ).toString().latin1();
+    }
+    else if( service->property( "X-KDE-StartupNotify" ).isValid())
     {
         if( !service->property( "X-KDE-StartupNotify" ).toBool())
         {
@@ -1021,6 +1064,7 @@ KLauncher::send_service_startup_info( KLaunchRequest *request, KService::Ptr ser
     KStartupInfoData data;
     data.setName( service->name());
     data.setIcon( service->icon());
+    data.setDescription( i18n( "Launching %1" ).arg( service->name()));
     if( !wmclass.isEmpty())
         data.setWMClass( wmclass );
     // the rest will be sent by kdeinit
@@ -1038,7 +1082,8 @@ void
 KLauncher::cancel_service_startup_info( KLaunchRequest* request, const QCString& startup_id,
     const QValueList<QCString> &envs )
 {
-#ifdef Q_WS_X11 // KStartup* isn't implemented for Qt/Embedded yet
+#if defined Q_WS_X11 && ! defined K_WS_QTONLY
+//#ifdef Q_WS_X11 // KStartup* isn't implemented for Qt/Embedded yet
     if( request != NULL )
         request->startup_id = "0";
     if( !startup_id.isEmpty() && startup_id != "0" )
@@ -1069,7 +1114,7 @@ KLauncher::cancel_service_startup_info( KLaunchRequest* request, const QCString&
 
 bool
 KLauncher::kdeinit_exec(const QString &app, const QStringList &args,
-   const QValueList<QCString> &envs,  bool wait)
+   const QValueList<QCString> &envs, QCString startup_id, bool wait)
 {
    KLaunchRequest *request = new KLaunchRequest;
    request->autoStart = false;
@@ -1082,6 +1127,9 @@ KLauncher::kdeinit_exec(const QString &app, const QStringList &args,
        request->arg_list.append(arg.local8Bit());
    }
 
+   if( startup_id == "" )
+       startup_id = "0"; // no ASN with kdeinit_exec unless requested
+
    request->name = app.local8Bit();
 
    if (wait)
@@ -1090,7 +1138,7 @@ KLauncher::kdeinit_exec(const QString &app, const QStringList &args,
       request->dcop_service_type = KService::DCOP_None;
    request->dcop_name = 0;
    request->pid = 0;
-   request->startup_id = "0";
+   request->startup_id = startup_id;
    request->envs = envs;
    request->transaction = dcopClient()->beginTransaction();
    queueRequest(request);
