@@ -23,7 +23,6 @@
 #include <sys/types.h>
 #include <unistd.h>
 #include <time.h>
-
 #include <qstring.h>
 #include <qvaluelist.h>
 #include <qsocketnotifier.h>
@@ -55,7 +54,7 @@ public:
 
 signals:
    void statusUpdate(IdleSlave *);
-   
+
 protected slots:
    void gotInput();
 
@@ -106,7 +105,7 @@ struct serviceResult
   pid_t pid;
 };
 
-class KLauncher : public KUniqueApplication
+class KLauncher : public KApplication, public DCOPObject
 {
    Q_OBJECT
 
@@ -141,7 +140,8 @@ protected:
        const QValueList<QCString> &envs, const QCString& startup_id, bool blind);
    bool start_service_by_desktop_name(const QString &serviceName, const QStringList &urls,
        const QValueList<QCString> &envs, const QCString& startup_id, bool blind);
-   bool kdeinit_exec(const QString &app, const QStringList &args, const QValueList<QCString> &envs, bool wait);
+   bool kdeinit_exec(const QString &app, const QStringList &args,
+       const QValueList<QCString> &envs, QCString startup_id, bool wait);
 
    void waitForSlave(pid_t pid);
 
@@ -156,7 +156,7 @@ protected:
 
 
    void queueRequest(KLaunchRequest *);
-   
+
    void send_service_startup_info( KLaunchRequest *request, KService::Ptr service, const QCString& startup_id,
        const QValueList<QCString> &envs );
    void cancel_service_startup_info( KLaunchRequest *request, const QCString& startup_id,
