@@ -161,6 +161,7 @@ int build_link(const char *tmp_prefix, const char *kde_prefix)
      perror("Aborting. Could not determine hostname: ");
      exit(255);
   }
+  kde_tmp_dir[sizeof(kde_tmp_dir)-1] = '\0';
 
   result = lstat(kde_tmp_dir, &stat_buf);
   if ((result == 0) && (S_ISDIR(stat_buf.st_mode)))
@@ -222,9 +223,11 @@ int main(int argc, char **argv)
   int res = 0;
 
   if ((argc != 2) || 
-      ((strcmp(argv[1], "tmp")!=0) && (strcmp(argv[1], "socket")!=0)))
+      ((strcmp(argv[1], "tmp")!=0) && 
+       (strcmp(argv[1], "socket")!=0) && 
+       (strcmp(argv[1], "cache")!=0)))
   {
-     fprintf(stderr, "Usage: lnusertemp tmp|socket\n");
+     fprintf(stderr, "Usage: lnusertemp tmp|socket|cache\n");
      return 1;
   }
 
@@ -240,7 +243,7 @@ int main(int argc, char **argv)
     
     kde_prefix = "/tmp-"; 
   }
-  else
+  else if (strcmp(argv[1], "socket") == 0)
   {
     tmp_prefix = (char *)malloc(strlen(tmp)+strlen("/ksocket-")+1);
     strcpy(tmp_prefix, tmp );
@@ -248,6 +251,18 @@ int main(int argc, char **argv)
 
     kde_prefix = "/socket-"; 
   }
+  else if (strcmp(argv[1], "cache") == 0)
+  {
+    tmp = getenv("KDEVARTMP");
+    if (!tmp || !tmp[0]) 
+      tmp = "/var/tmp";
+
+    tmp_prefix = (char *)malloc(strlen(tmp)+strlen("/kdecache-")+1);
+    strcpy(tmp_prefix, tmp );
+    strcat(tmp_prefix, "/kdecache-" );
+
+    kde_prefix = "/cache-"; 
+  }
 
   res = build_link(tmp_prefix, kde_prefix); 
     
