@@ -1,5 +1,8 @@
+// -*- c-basic-offset:4; indent-tabs-mode:nil -*-
+// vim: set ts=4 sts=4 sw=4 et:
 /* This file is part of the KDE libraries
    Copyright (C) 2000 David Faure <faure@kde.org>
+   Copyright (C) 2003 Alexander Kellett <lypanov@kde.org>
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
@@ -17,10 +20,11 @@
 */
 
 #include "kbookmark.h"
+#include <qvaluestack.h>
 #include <kdebug.h>
 #include <kmimetype.h>
 #include <kstringhandler.h>
-#include <klineeditdlg.h>
+#include <kinputdialog.h>
 #include <kglobal.h>
 #include <klocale.h>
 #include <assert.h>
@@ -89,15 +93,14 @@ KBookmarkGroup KBookmarkGroup::createNewFolder( KBookmarkManager* mgr, const QSt
     QString txt( text );
     if ( text.isEmpty() )
     {
-        KLineEditDlg l( i18n("New folder:"), "", 0L );
-        l.setCaption( parentGroup().text().isEmpty() ?
-                      i18n("Create New Bookmark Folder") :
-                      i18n("Create New Bookmark Folder in %1").arg( parentGroup().text() ) );
-        //text is empty so disable ok button.
-        l.enableButtonOK( false );
-        if ( l.exec() )
-            txt = l.text();
-        else
+        bool ok;
+        QString caption = parentGroup().fullText().isEmpty() ?
+                      i18n( "Create New Bookmark Folder" ) :
+                      i18n( "Create New Bookmark Folder in %1" )
+                      .arg( parentGroup().text() );
+        txt = KInputDialog::getText( caption, i18n( "New folder:" ),
+                      QString::null, &ok );
+        if ( !ok )
             return KBookmarkGroup();
     }
 
@@ -111,9 +114,10 @@ KBookmarkGroup KBookmarkGroup::createNewFolder( KBookmarkManager* mgr, const QSt
 
     KBookmarkGroup grp(groupElem);
 
-    if (emitSignal) emit mgr->notifier().createdNewFolder(
-        mgr->path(),
-        grp.fullText(), grp.address() );
+    if (emitSignal) 
+        emit mgr->notifier().createdNewFolder(
+                                mgr->path(), grp.fullText(), 
+                                grp.address() );
 
     return grp;
 
@@ -154,7 +158,7 @@ bool KBookmarkGroup::moveItem( const KBookmark & item, const KBookmark & after )
 
 KBookmark KBookmarkGroup::addBookmark( KBookmarkManager* mgr, const QString & text, const KURL & url, const QString & icon, bool emitSignal )
 {
-    //kdDebug(1203) << "KBookmarkGroup::addBookmark " << text << " into " << m_address << endl;
+    //kdDebug(7043) << "KBookmarkGroup::addBookmark " << text << " into " << m_address << endl;
     QDomDocument doc = element.ownerDocument();
     QDomElement elem = doc.createElement( "bookmark" );
     element.appendChild( elem );
@@ -170,10 +174,11 @@ KBookmark KBookmarkGroup::addBookmark( KBookmarkManager* mgr, const QString & te
 
     KBookmark bk(elem);
 
-    if (emitSignal) emit mgr->notifier().addedBookmark(
-        mgr->path(),
-        url.url(), text, bk.address(), icon );
-    
+    if (emitSignal)
+        emit mgr->notifier().addedBookmark(
+                                 mgr->path(), url.url(),
+                                 text, bk.address(), icon );
+
     return bk;
 }
 
@@ -210,6 +215,18 @@ QDomElement KBookmarkGroup::findToolbar() const
     return QDomElement();
 }
 
+QValueList<KURL> KBookmarkGroup::groupUrlList() const
+{
+    QValueList<KURL> urlList;
+    for ( KBookmark bm = first(); !bm.isNull(); bm = next(bm) )
+    {
+        if ( bm.isSeparator() || bm.isGroup() )
+           continue;
+        urlList << bm.url();
+    }
+    return urlList;
+}
+
 //////
 
 bool KBookmark::isGroup() const
@@ -224,6 +241,12 @@ bool KBookmark::isSeparator() const
     return (element.tagName() == "separator");
 }
 
+bool KBookmark::hasParent() const
+{
+    QDomElement parent = element.parentNode().toElement();
+    return !parent.isNull();
+}
+
 QString KBookmark::text() const
 {
     return KStringHandler::csqueeze( fullText() );
@@ -276,13 +299,12 @@ QString KBookmark::address() const
     else
     {
         // Use keditbookmarks's DEBUG_ADDRESSES flag to debug this code :)
-        QDomElement parent = element.parentNode().toElement();
-        if(parent.isNull())
+        if (!hasParent())
         {
-          Q_ASSERT(!parent.isNull());
-          return "ERROR"; // Avoid an infinite loop
+            Q_ASSERT(hasParent());
+            return "ERROR"; // Avoid an infinite loop
         }
-        KBookmarkGroup group( parent );
+        KBookmarkGroup group = parentGroup();
         QString parentAddress = group.address();
         uint counter = 0;
         // Implementation note: we don't use QDomNode's childNode list because we
@@ -306,3 +328,98 @@ KBookmark KBookmark::standaloneBookmark( const QString & text, const KURL & url,
     grp.addBookmark( 0L, text, url, icon, false );
     return grp.first();
 }
+
+static QDomNode cd_or_create(QDomNode node, QString name)
+{
+    QDomNode subnode = node.namedItem(name);
+    if (subnode.isNull()) 
+    {
+        subnode = node.ownerDocument().createElement(name);
+        node.appendChild(subnode);
+    }
+    return subnode;
+}
+
+static QDomText get_or_create_text(QDomNode node)
+{
+    QDomNode subnode = node.firstChild();
+    if (subnode.isNull()) 
+    {
+        subnode = node.ownerDocument().createTextNode("");
+        node.appendChild(subnode);
+    }
+    return subnode.toText();
+}
+
+void KBookmark::updateAccessMetadata()
+{
+    kdDebug(7043) << "KBookmark::updateAccessMetadata " << address() << " " << url().prettyURL() << endl;
+
+    QDomNode subnode = cd_or_create(internalElement(), "info");
+    subnode = cd_or_create(subnode, "metadata");
+
+    uint timet = QDateTime::currentDateTime().toTime_t();
+
+    QDomNode item = cd_or_create(subnode, "time_added");
+    QDomText domtext = get_or_create_text(item);
+    if (domtext.data().isEmpty()) 
+        domtext.setData(QString::number(timet));
+
+    item = cd_or_create(subnode, "time_visited");
+    domtext = get_or_create_text(item);
+    domtext.setData(QString::number(timet));
+
+    item = cd_or_create(subnode, "visit_count"); // TODO use spec'ed name
+    domtext = get_or_create_text(item);
+    QString countStr = domtext.data();
+    bool ok;
+    int currentCount = countStr.toInt(&ok);
+    if (!ok)
+        currentCount = 0;
+    currentCount++;
+    domtext.setData(QString::number(currentCount));
+
+    // TODO - for 3.3 - time_modified
+}
+
+void KBookmarkGroupTraverser::traverse(const KBookmarkGroup &root)
+{
+    // non-recursive bookmark iterator
+    QValueStack<KBookmarkGroup> stack;
+    stack.push(root);
+    KBookmark bk = stack.top().first();
+    for (;;) {
+        if (bk.isNull())
+        {
+            if (stack.isEmpty()) 
+                return;
+            if (stack.count() > 1)
+                visitLeave(stack.top());
+            bk = stack.pop();
+            bk = stack.top().next(bk);
+            if (bk.isNull())
+                continue;
+        } 
+
+        if (bk.isGroup()) 
+        {
+            KBookmarkGroup gp = bk.toGroup();
+            visitEnter(gp);
+            if (!gp.first().isNull()) 
+            {
+                stack.push(gp);
+                bk = gp.first();
+                continue;
+            }
+            // empty group
+            visitLeave(gp);
+        } 
+        else 
+            visit(bk);
+
+        bk = stack.top().next(bk);
+    }
+
+    // never reached
+}
+
