@@ -3,7 +3,7 @@
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Library General Public
-    License as published by the Free Software Foundation; version 2 
+    License as published by the Free Software Foundation; version 2
     of the License.
 
     This library is distributed in the hope that it will be useful,
@@ -28,6 +28,7 @@
 #include <kiconloader.h>
 #include <klistview.h>
 #include <kservice.h>
+#include <kstandarddirs.h>
 #include <kconfigbase.h>
 #include <kopenwith.h>
 
@@ -56,7 +57,7 @@ public:
 
       // item names may contain ampersands. To avoid them being converted
       // to accelators, replace them with two ampersands.
-      serviceName.replace(QRegExp("&"), "&&");
+      serviceName.replace("&", "&&");
 
       QPixmap normal = KGlobal::instance()->iconLoader()->loadIcon(s->icon(), KIcon::Small,
                               0, KIcon::DefaultState, 0L, true);
@@ -74,10 +75,19 @@ public:
    KService::Ptr s;
 };
 
+class KCustomMenuEditor::KCustomMenuEditorPrivate
+{
+public:
+    QPushButton * pbRemove;
+    QPushButton * pbMoveUp;
+    QPushButton * pbMoveDown;
+};
+
 KCustomMenuEditor::KCustomMenuEditor(QWidget *parent)
   : KDialogBase(parent, "custommenueditor", true, i18n("Menu Editor"), Ok|Cancel, Ok, true),
     m_listView(0)
 {
+    d = new KCustomMenuEditorPrivate;
    QHBox *page = makeHBoxMainWidget();
    m_listView = new KListView(page);
    m_listView->addColumn(i18n("Menu"));
@@ -85,13 +95,31 @@ KCustomMenuEditor::KCustomMenuEditor(QWidget *parent)
    m_listView->setSorting(-1);
    KButtonBox *buttonBox = new KButtonBox(page, Vertical);
    buttonBox->addButton(i18n("New..."), this, SLOT(slotNewItem()));
-   buttonBox->addButton(i18n("Remove"), this, SLOT(slotRemoveItem()));
-   buttonBox->addButton(i18n("Move Up"), this, SLOT(slotMoveUp()));
-   buttonBox->addButton(i18n("Move Down"), this, SLOT(slotMoveDown()));
+   d->pbRemove=buttonBox->addButton(i18n("Remove"), this, SLOT(slotRemoveItem()));
+   d->pbMoveUp=buttonBox->addButton(i18n("Move Up"), this, SLOT(slotMoveUp()));
+   d->pbMoveDown=buttonBox->addButton(i18n("Move Down"), this, SLOT(slotMoveDown()));
    buttonBox->layout();
+   connect( m_listView, SIGNAL( selectionChanged () ), this, SLOT( refreshButton() ) );
+   refreshButton();
 }
 
-void 
+KCustomMenuEditor::~KCustomMenuEditor()
+{
+    delete d;
+    d=0;
+}
+
+void KCustomMenuEditor::refreshButton()
+{
+    QListViewItem *item = m_listView->currentItem();
+    bool selected = ( item!= 0 );
+    int pos = m_listView->itemPos( item );
+    d->pbRemove->setEnabled( selected );
+    d->pbMoveUp->setEnabled( selected && pos>0);
+    d->pbMoveDown->setEnabled( selected && pos < ( m_listView->childCount() ));
+}
+
+void
 KCustomMenuEditor::load(KConfigBase *cfg)
 {
    cfg->setGroup(QString::null);
@@ -119,7 +147,7 @@ KCustomMenuEditor::load(KConfigBase *cfg)
    }
 }
 
-void 
+void
 KCustomMenuEditor::save(KConfigBase *cfg)
 {
    // First clear the whole config file.
@@ -136,7 +164,10 @@ KCustomMenuEditor::save(KConfigBase *cfg)
    while(item)
    {
       i++;
-      cfg->writeEntry(QString("Item%1").arg(i), item->s->desktopEntryPath());
+      QString path = item->s->desktopEntryPath();
+      if (!path.startsWith("/") || !KGlobal::dirs()->relativeLocation("xdgdata-apps", path).startsWith("/"))
+         path = item->s->desktopEntryName();
+      cfg->writePathEntry(QString("Item%1").arg(i), path);
       item = (Item *) item->nextSibling();
    }
    cfg->writeEntry("NrOfItems", i);
@@ -146,9 +177,10 @@ void
 KCustomMenuEditor::slotNewItem()
 {
    QListViewItem *item = m_listView->currentItem();
-   
+
    KOpenWithDlg dlg(this);
-   
+   dlg.setSaveNewApplications(true);
+
    if (dlg.exec())
    {
       KService::Ptr s = dlg.service();
@@ -157,6 +189,7 @@ KCustomMenuEditor::slotNewItem()
          Item *newItem = new Item(m_listView, s);
          newItem->moveItem(item);
       }
+      refreshButton();
    }
 }
 
@@ -166,8 +199,9 @@ KCustomMenuEditor::slotRemoveItem()
    QListViewItem *item = m_listView->currentItem();
    if (!item)
       return;
-      
+
    delete item;
+   refreshButton();
 }
 
 void
@@ -176,7 +210,7 @@ KCustomMenuEditor::slotMoveUp()
    QListViewItem *item = m_listView->currentItem();
    if (!item)
       return;
-      
+
    QListViewItem *searchItem = m_listView->firstChild();
    while(searchItem)
    {
@@ -187,7 +221,8 @@ KCustomMenuEditor::slotMoveUp()
          break;
       }
       searchItem = next;
-   }      
+   }
+   refreshButton();
 }
 
 void
@@ -200,8 +235,9 @@ KCustomMenuEditor::slotMoveDown()
    QListViewItem *after = item->nextSibling();
    if (!after)
       return;
-      
+
    item->moveItem( after );
+   refreshButton();
 }
 
 #include "kcustommenueditor.moc"
