@@ -1,6 +1,6 @@
 /* This file is part of the KDE libraries
     Copyright (C) 1999,2000 Stephan Kulow <coolo@kde.org>
-                  1999,2000,2001 Carsten Pfeiffer <pfeiffer@kde.org>
+                  1999,2000,2001,2002,2003 Carsten Pfeiffer <pfeiffer@kde.org>
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Library General Public
@@ -37,7 +37,7 @@
 #include <kdialog.h>
 #include <kdialogbase.h>
 #include <kdirlister.h>
-#include <klineeditdlg.h>
+#include <kinputdialog.h>
 #include <klocale.h>
 #include <kmessagebox.h>
 #include <kpopupmenu.h>
@@ -59,7 +59,7 @@
 #include "kfilepreview.h"
 #include "kfileview.h"
 #include "kfileitem.h"
-#include "kimagefilepreview.h"
+#include "kfilemetapreview.h"
 
 
 template class QPtrStack<KURL>;
@@ -74,6 +74,7 @@ public:
         progressDelayTimer = 0L;
         dirHighlighting = false;
         config = 0L;
+        dropOptions = 0;
     }
 
     ~KDirOperatorPrivate() {
@@ -85,6 +86,7 @@ public:
     bool onlyDoubleClickSelectsFiles;
     QTimer *progressDelayTimer;
     KActionSeparator *viewActionSeparator;
+    int dropOptions;
 
     KConfig *config;
     QString configGroup;
@@ -150,11 +152,11 @@ KDirOperator::~KDirOperator()
     {
         if ( d->config )
             m_fileView->writeConfig( d->config, d->configGroup );
-        
-    delete m_fileView;
-    m_fileView = 0L;
+
+        delete m_fileView;
+        m_fileView = 0L;
     }
-    
+
     delete myPreview;
     delete dir;
     delete d;
@@ -171,18 +173,18 @@ void KDirOperator::setSorting( QDir::SortSpec spec )
 
 void KDirOperator::resetCursor()
 {
-    QApplication::restoreOverrideCursor();
-    progress->hide();
+   QApplication::restoreOverrideCursor();
+   progress->hide();
 }
 
 void KDirOperator::insertViewDependentActions()
 {
    // If we have a new view actionCollection(), insert its actions
    // into viewActionMenu.
-  
+
    if( !m_fileView )
       return;
-       
+
    if ( (viewActionMenu->popupMenu()->count() == 0) || 			// Not yet initialized or...
         (viewActionCollection != m_fileView->actionCollection()) )	// ...changed since.
    {
@@ -193,7 +195,7 @@ void KDirOperator::insertViewDependentActions()
          disconnect( viewActionCollection, SIGNAL( removed( KAction * )),
                this, SLOT( slotViewActionRemoved( KAction * )));
       }
-    
+
       viewActionMenu->popupMenu()->clear();
 //      viewActionMenu->insert( shortAction );
 //      viewActionMenu->insert( detailedAction );
@@ -211,12 +213,26 @@ void KDirOperator::insertViewDependentActions()
       if (!viewActionCollection)
          return;
 
-      if ( !viewActionCollection->isEmpty() ) 
+      if ( !viewActionCollection->isEmpty() )
       {
          viewActionMenu->insert( d->viewActionSeparator );
 
-         for ( uint i = 0; i < viewActionCollection->count(); i++ )
-            viewActionMenu->insert( viewActionCollection->action( i ));
+         // first insert the normal actions, then the grouped ones
+         QStringList groups = viewActionCollection->groups();
+         groups.prepend( QString::null ); // actions without group
+         QStringList::ConstIterator git = groups.begin();
+         KActionPtrList list;
+         KAction *sep = actionCollection()->action("separator");
+         for ( ; git != groups.end(); ++git )
+         {
+            if ( git != groups.begin() )
+               viewActionMenu->insert( sep );
+
+            list = viewActionCollection->actions( *git );
+            KActionPtrList::ConstIterator it = list.begin();
+            for ( ; it != list.end(); ++it )
+               viewActionMenu->insert( *it );
+         }
       }
 
       connect( viewActionCollection, SIGNAL( inserted( KAction * )),
@@ -225,7 +241,7 @@ void KDirOperator::insertViewDependentActions()
                SLOT( slotViewActionRemoved( KAction * )));
    }
 }
-  
+
 void KDirOperator::activatedMenu( const KFileItem *, const QPoint& pos )
 {
     updateSelectionDependentActions();
@@ -305,7 +321,7 @@ void KDirOperator::slotDefaultPreview()
 {
     m_viewKind = m_viewKind | KFile::PreviewContents;
     if ( !myPreview ) {
-        myPreview = new KImageFilePreview( this );
+        myPreview = new KFileMetaPreview( this );
         (static_cast<KToggleAction*>( myActionCollection->action("preview") ))->setChecked(true);
     }
 
@@ -364,36 +380,47 @@ void KDirOperator::slotToggleIgnoreCase()
 
 void KDirOperator::mkdir()
 {
-    KLineEditDlg dlg(i18n("Create new directory in:") +
-                     QString::fromLatin1( "\n" ) + /* don't break i18n now*/
-                     url().prettyURL(), i18n("New Directory"), this);
-    dlg.setCaption(i18n("New Directory"));
-    if (dlg.exec()) {
-      mkdir( dlg.text(), true );
-    }
+    bool ok;
+    QString where = url().isLocalFile() ? url().path(+1) : url().prettyURL();
+    QString dir = KInputDialog::getText( i18n( "New Folder" ),
+                                         i18n( "Create new folder in:\n%1" ).arg( where ),
+                                         i18n("New Folder"), &ok, this);
+    if (ok)
+      mkdir( dir, true );
 }
 
 bool KDirOperator::mkdir( const QString& directory, bool enterDirectory )
 {
+    // Creates "directory", relative to the current directory (currUrl).
+    // The given path may contain any number directories, existant or not.
+    // They will all be created, if possible.
+
     bool writeOk = false;
+    bool exists = false;
     KURL url( currUrl );
-    url.addPath(directory);
 
-    if ( url.isLocalFile() ) {
-        // check if we are allowed to create local directories
-        writeOk = checkAccess( url.path(), W_OK );
-        if ( writeOk )
-            writeOk = QDir().mkdir( url.path() );
+    QStringList dirs = QStringList::split( QDir::separator(), directory );
+    QStringList::ConstIterator it = dirs.begin();
+
+    for ( ; it != dirs.end(); ++it )
+    {
+        url.addPath( *it );
+        exists = KIO::NetAccess::exists( url, false, 0 );
+        writeOk = !exists && KIO::NetAccess::mkdir( url, topLevelWidget() );
     }
-    else
-        writeOk = KIO::NetAccess::mkdir( url );
 
-    if ( !writeOk )
+    if ( exists ) // url was already existant
+    {
+        QString which = url.isLocalFile() ? url.path() : url.prettyURL();
+        KMessageBox::sorry(viewWidget(), i18n("A file or folder named %1 already exists.").arg(which));
+        enterDirectory = false;
+    }
+    else if ( !writeOk ) {
         KMessageBox::sorry(viewWidget(), i18n("You don't have permission to "
-                                              "create that directory." ));
-    else {
-        if ( enterDirectory )
-            setURL( url, true );
+                                              "create that folder." ));
+    }
+    else if ( enterDirectory ) {
+        setURL( url, true );
     }
 
     return writeOk;
@@ -412,7 +439,7 @@ KIO::DeleteJob * KDirOperator::del( const KFileItemList& items,
     if ( items.isEmpty() ) {
         KMessageBox::information( parent,
                                 i18n("You didn't select a file to delete."),
-                                i18n("Nothing to delete") );
+                                i18n("Nothing to Delete") );
         return 0L;
     }
 
@@ -437,19 +464,20 @@ KIO::DeleteJob * KDirOperator::del( const KFileItemList& items,
                 i18n( "<qt>Do you really want to delete\n <b>'%1'</b>?</qt>" )
                 .arg( files.first() ),
                                                       i18n("Delete File"),
-                                                      i18n("Delete") );
+                                                      KGuiItem(i18n("Delete"),"editdelete"), "AskForDelete" );
         }
         else
             ret = KMessageBox::warningContinueCancelList( parent,
                 i18n("translators: not called for n == 1", "Do you really want to delete these %n items?", items.count() ),
                                                     files,
                                                     i18n("Delete Files"),
-                                                    i18n("Delete") );
+                                                    KGuiItem(i18n("Delete"), "editdelete"), "AskForDelete" );
         doIt = (ret == KMessageBox::Continue);
     }
 
     if ( doIt ) {
         KIO::DeleteJob *job = KIO::del( urls, false, showProgress );
+        job->setWindow (topLevelWidget());
         job->setAutoErrorHandlingEnabled( true, parent );
         return job;
     }
@@ -534,7 +562,7 @@ void KDirOperator::setURL(const KURL& _newurl, bool clearforward)
 {
     KURL newurl;
 
-    if ( _newurl.isMalformed() )
+    if ( !_newurl.isValid() )
 	newurl.setPath( QDir::homeDirPath() );
     else
 	newurl = _newurl;
@@ -543,7 +571,7 @@ void KDirOperator::setURL(const KURL& _newurl, bool clearforward)
     newurl.setPath(pathstr);
 
     // already set
-    if ( newurl.cmp( currUrl, true ) )
+    if ( newurl.equals( currUrl, true ) )
         return;
 
     if ( !isReadable( newurl ) ) {
@@ -552,7 +580,7 @@ void KDirOperator::setURL(const KURL& _newurl, bool clearforward)
         if ( !isReadable( newurl ) ) {
             resetCursor();
             KMessageBox::error(viewWidget(),
-                               i18n("The specified directory does not exist "
+                               i18n("The specified folder does not exist "
                                     "or was not readable."));
             return;
         }
@@ -610,7 +638,7 @@ void KDirOperator::pathChanged()
 
     if ( !isReadable( currUrl )) {
         KMessageBox::error(viewWidget(),
-                           i18n("The specified directory does not exist "
+                           i18n("The specified folder does not exist "
                                 "or was not readable."));
         if (backStack.isEmpty())
             home();
@@ -805,13 +833,29 @@ KFileView* KDirOperator::createView( QWidget* parent, KFile::FileView view )
         new_view->setViewName( i18n("Detailed View") );
     }
     else /* if ( KFile::isSimpleView( view ) && !preview ) */ {
-        new_view = new KFileIconView( parent, "simple view");
+        KFileIconView *iconView =  new KFileIconView( parent, "simple view");
+        new_view = iconView;
         new_view->setViewName( i18n("Short View") );
     }
 
+    new_view->widget()->setAcceptDrops(acceptDrops());
     return new_view;
 }
 
+void KDirOperator::setAcceptDrops(bool b)
+{
+    if (m_fileView)
+       m_fileView->widget()->setAcceptDrops(b);
+    QWidget::setAcceptDrops(b);
+}
+
+void KDirOperator::setDropOptions(int options)
+{
+    d->dropOptions = options;
+    if (m_fileView)
+       m_fileView->setDropOptions(options);
+}
+
 void KDirOperator::setView( KFile::FileView view )
 {
     bool separateDirs = KFile::isSeparateDirs( view );
@@ -828,8 +872,8 @@ void KDirOperator::setView( KFile::FileView view )
                     KFile::isPreviewContents( static_cast<KFile::FileView>(defaultView) ) )
                   && myActionCollection->action("preview")->isEnabled();
 
-        if ( preview ) { // instantiates KImageFilePreview and calls setView()
-            m_viewKind = view;
+        if ( preview ) { // instantiates KFileMetaPreview and calls setView()
+            m_viewKind = defaultView;
             slotDefaultPreview();
             return;
         }
@@ -874,8 +918,9 @@ void KDirOperator::connectView(KFileView *view)
     else
         view->setSelectionMode( KFile::Single );
 
-    if (m_fileView) {
-        if ( d->config ) // save and restore coniguration the views' config
+    if (m_fileView)
+    {
+        if ( d->config ) // save and restore the views' configuration
         {
             m_fileView->writeConfig( d->config, d->configGroup );
             view->readConfig( d->config, d->configGroup );
@@ -914,6 +959,7 @@ void KDirOperator::connectView(KFileView *view)
     }
 
     m_fileView = view;
+    m_fileView->setDropOptions(d->dropOptions);
     viewActionCollection = 0L;
     KFileViewSignaler *sig = view->signaler();
 
@@ -927,6 +973,8 @@ void KDirOperator::connectView(KFileView *view)
             this, SLOT( highlightFile(const KFileItem*) ));
     connect(sig, SIGNAL( sortingChanged( QDir::SortSpec ) ),
             this, SLOT( slotViewSortingChanged( QDir::SortSpec )));
+    connect(sig, SIGNAL( dropped(const KFileItem *, QDropEvent*, const KURL::List&) ),
+            this, SIGNAL( dropped(const KFileItem *, QDropEvent*, const KURL::List&)) );
 
     if ( reverseAction->isChecked() != m_fileView->isReversed() )
         slotSortReversed();
@@ -985,6 +1033,10 @@ void KDirOperator::setDirLister( KDirLister *lister )
 
     dir->setAutoUpdate( true );
 
+    QWidget* mainWidget = topLevelWidget();
+    dir->setMainWindow (mainWidget);
+    kdDebug (kfile_area) << "mainWidget=" << mainWidget << endl;
+
     connect( dir, SIGNAL( percent( int )),
              SLOT( slotProgress( int ) ));
     connect( dir, SIGNAL(started( const KURL& )), SLOT(slotStarted()));
@@ -1035,8 +1087,11 @@ void KDirOperator::selectDir(const KFileItem *item)
 void KDirOperator::itemDeleted(KFileItem *item)
 {
     pendingMimeTypes.removeRef( item );
-    m_fileView->removeItem( static_cast<KFileItem *>( item ));
-    emit updateInformation(m_fileView->numDirs(), m_fileView->numFiles());
+    if ( m_fileView )
+    {
+        m_fileView->removeItem( static_cast<KFileItem *>( item ));
+        emit updateInformation(m_fileView->numDirs(), m_fileView->numFiles());
+    }
 }
 
 void KDirOperator::selectFile(const KFileItem *item)
@@ -1114,16 +1169,16 @@ void KDirOperator::setupActions()
     myActionCollection = new KActionCollection( this, "KDirOperator::myActionCollection" );
     actionMenu = new KActionMenu( i18n("Menu"), myActionCollection, "popupMenu" );
     upAction = KStdAction::up( this, SLOT( cdUp() ), myActionCollection, "up" );
-    upAction->setText( i18n("Parent Directory") );
+    upAction->setText( i18n("Parent Folder") );
     backAction = KStdAction::back( this, SLOT( back() ), myActionCollection, "back" );
     forwardAction = KStdAction::forward( this, SLOT(forward()), myActionCollection, "forward" );
     homeAction = KStdAction::home( this, SLOT( home() ), myActionCollection, "home" );
-    homeAction->setText(i18n("Home Directory"));
+    homeAction->setText(i18n("Home Folder"));
     reloadAction = KStdAction::redisplay( this, SLOT(rereadDir()), myActionCollection, "reload" );
     actionSeparator = new KActionSeparator( myActionCollection, "separator" );
     d->viewActionSeparator = new KActionSeparator( myActionCollection,
                                                    "viewActionSeparator" );
-    mkdirAction = new KAction( i18n("New Directory..."), 0,
+    mkdirAction = new KAction( i18n("New Folder..."), 0,
                                  this, SLOT( mkdir() ), myActionCollection, "mkdir" );
     new KAction( i18n( "Delete" ), "editdelete", Key_Delete, this,
                   SLOT( deleteSelected() ), myActionCollection, "delete" );
@@ -1153,7 +1208,7 @@ void KDirOperator::setupActions()
     bySizeAction->setExclusiveGroup( sortGroup );
 
 
-    dirsFirstAction = new KToggleAction( i18n("Directories First"), 0,
+    dirsFirstAction = new KToggleAction( i18n("Folders First"), 0,
                                          myActionCollection, "dirs first");
     caseInsensitiveAction = new KToggleAction(i18n("Case Insensitive"), 0,
                                               myActionCollection, "case insensitive" );
@@ -1166,7 +1221,7 @@ void KDirOperator::setupActions()
 
 
     // the view menu actions
-    viewActionMenu = new KActionMenu( i18n("View"), myActionCollection, "view menu" );
+    viewActionMenu = new KActionMenu( i18n("&View"), myActionCollection, "view menu" );
     connect( viewActionMenu->popupMenu(), SIGNAL( aboutToShow() ),
              SLOT( insertViewDependentActions() ));
 
@@ -1177,7 +1232,7 @@ void KDirOperator::setupActions()
 
     showHiddenAction = new KToggleAction( i18n("Show Hidden Files"), KShortcut(),
                                           myActionCollection, "show hidden" );
-    separateDirsAction = new KToggleAction( i18n("Separate Directories"), KShortcut(),
+    separateDirsAction = new KToggleAction( i18n("Separate Folders"), KShortcut(),
                                             this,
                                             SLOT(slotSeparateDirs()),
                                             myActionCollection, "separate dirs" );
@@ -1305,10 +1360,8 @@ void KDirOperator::readConfig( KConfig *kc, const QString& group )
     if ( kc->readBoolEntry( QString::fromLatin1("Separate Directories"),
                             DefaultMixDirsAndFiles ) )
         defaultView |= KFile::SeparateDirs;
-    else {
-        if ( kc->readBoolEntry(QString::fromLatin1("Show Preview"), false))
-            defaultView |= KFile::PreviewContents;
-    }
+    if ( kc->readBoolEntry(QString::fromLatin1("Show Preview"), false))
+        defaultView |= KFile::PreviewContents;
 
     if ( kc->readBoolEntry( QString::fromLatin1("Sort case insensitively"),
                             DefaultCaseInsensitive ) )
@@ -1372,7 +1425,7 @@ void KDirOperator::writeConfig( KConfig *kc, const QString& group )
     bool appSpecificPreview = false;
     if ( myPreview ) {
         QWidget *preview = const_cast<QWidget*>( myPreview ); // grmbl
-        KImageFilePreview *tmp = dynamic_cast<KImageFilePreview*>( preview );
+        KFileMetaPreview *tmp = dynamic_cast<KFileMetaPreview*>( preview );
         appSpecificPreview = (tmp == 0L);
     }
 
@@ -1441,7 +1494,7 @@ void KDirOperator::slotShowProgress()
 void KDirOperator::slotProgress( int percent )
 {
     progress->setProgress( percent );
-    // we have to redraw this in as fast as possible
+    // we have to redraw this as fast as possible
     if ( progress->isVisible() )
 	QApplication::flushX();
 }
