@@ -49,17 +49,17 @@ namespace KIO {
 
 /**
  * This widget works as a network transparent filebrowser. You specify a URL
- * to display and this url will be loaded via @ref KDirLister. The user can
+ * to display and this url will be loaded via KDirLister. The user can
  * browse through directories, highlight and select files, delete or rename
  * files.
  *
- * It supports different views, e.g. a detailed view (see @ref KFileDetailView),
- * a simple icon view (see @ref KFileIconView), a combination of two views,
- * separating directories and files (@ref KCombiView).
+ * It supports different views, e.g. a detailed view (see KFileDetailView),
+ * a simple icon view (see KFileIconView), a combination of two views,
+ * separating directories and files ( KCombiView).
  *
- * Additionally, a preview view is available (see @ref KFilePreview), which can
+ * Additionally, a preview view is available (see KFilePreview), which can
  * show either a simple or detailed view and additionally a preview widget
- * (see @ref setPreviewWidget()). @ref KImageFilePreview is one implementation
+ * (see setPreviewWidget()). KImageFilePreview is one implementation
  * of a preview widget, that displays previews for all supported filetypes
  * utilizing KIO::PreviewJob.
  *
@@ -71,7 +71,7 @@ namespace KIO {
  * This widget is the one used in the KFileDialog.
  *
  * Basic usage is like this:
- * <pre>
+ * \code
  *   KDirOperator *op = new KDirOperator( KURL( "file:/home/gis" ), this );
  *   // some signals you might be interested in
  *   connect(op, SIGNAL(urlEntered(const KURL&)),
@@ -85,7 +85,7 @@ namespace KIO {
  *
  *   op->readConfig( KGlobal::config(), "Your KDiroperator ConfigGroup" );
  *   op->setView(KFile::Default);
- * </pre>
+ * \endcode
  *
  * This will create a childwidget of 'this' showing the directory contents
  * of /home/gis in the default-view. The view is determined by the readConfig()
@@ -111,10 +111,10 @@ class KDirOperator : public QWidget
                        AllActions = 15 };
     /**
      * Constructs the KDirOperator with no initial view. As the views are
-     * configurable, call @ref readConfig() to load the user's configuration
-     * and then @ref setView to explicitly set a view.
+     * configurable, call readConfig() to load the user's configuration
+     * and then setView to explicitly set a view.
      *
-     * This constructor doesn't start loading the url, @ref setView will do it.
+     * This constructor doesn't start loading the url, setView will do it.
      */
     KDirOperator(const KURL& urlName = KURL(),
 		 QWidget *parent = 0, const char* name = 0);
@@ -126,6 +126,7 @@ class KDirOperator : public QWidget
     /**
      * Enables/disables showing hidden files.
      */
+	// ### KDE4: make virtual
     void setShowHiddenFiles ( bool s ) { showHiddenAction->setChecked( s ); }
 
     /**
@@ -140,16 +141,16 @@ class KDirOperator : public QWidget
 
     /**
      * Sets a filter like "*.cpp *.h *.o". Only files matching that filter
-     * will be shown. Call @ref updateDir() to apply it.
+     * will be shown. Call updateDir() to apply it.
      *
      * @see KDirLister::setNameFilter
-     * @see #nameFilter
+     * @see nameFilter
      */
     void setNameFilter(const QString& filter);
 
     /**
      * @returns the current namefilter.
-     * @see #setNameFilter
+     * @see setNameFilter
      */
     const QString& nameFilter() const { return dir->nameFilter(); }
 
@@ -158,17 +159,17 @@ class KDirOperator : public QWidget
      * will be shown.
      *
      * Example:
-     * <pre>
+     * \code
      * QStringList filter;
      * filter << "text/html" << "image/png" << "inode/directory";
      * dirOperator->setMimefilter( filter );
-     * </pre>
+     * \endcode
      *
      * Node: Without the mimetype inode/directory, only files would be shown.
-     * Call @ref updateDir() to apply it.
+     * Call updateDir() to apply it.
      *
      * @see KDirLister::setMimeFilter
-     * @see #mimeFilter
+     * @see mimeFilter
      */
     void setMimeFilter( const QStringList& mimetypes );
 
@@ -179,10 +180,10 @@ class KDirOperator : public QWidget
 
     /**
      * Clears both the namefilter and mimetype filter, so that all files and
-     * directories will be shown. Call @ref updateDir() to apply it.
+     * directories will be shown. Call updateDir() to apply it.
      *
-     * @see #setMimeFilter
-     * @see #setNameFilter
+     * @see setMimeFilter
+     * @see setNameFilter
      */
     void clearFilter();
 
@@ -193,9 +194,10 @@ class KDirOperator : public QWidget
 
     /**
      * Sets a new url to list.
-     * @param clearforward specifies whether the "forward" history should be
-     * cleared.
+     * @param clearforward specifies whether the "forward" history should be cleared.
+     * @param url the URL to set
      */
+	// ### KDE4: make virtual
     void setURL(const KURL& url, bool clearforward);
 
     /**
@@ -206,19 +208,20 @@ class KDirOperator : public QWidget
 
     /**
      * Sets a new KFileView to be used for showing and browsing files.
-     * Note: this will read the current @ref url() to fill the view.
+     * Note: this will read the current url() to fill the view.
      *
      * @see KFileView
      * @see KFileIconView
      * @see KFileDetailView
      * @see KFileCombiView
-     * @see #view
+     * @see view
      */
+	// ### KDE4: make virtual
     void setView(KFileView *view);
 
     /**
      * @returns the currently used view.
-     * @see #setView
+     * @see setView
      */
     KFileView * view() const { return m_fileView; }
 
@@ -232,6 +235,7 @@ class KDirOperator : public QWidget
      * Sets one of the predefined fileviews
      * @see KFile::FileView
      */
+	// ### KDE4: make virtual
     void setView(KFile::FileView view);
 
     /**
@@ -271,6 +275,7 @@ class KDirOperator : public QWidget
      * You cannot mix File and Files of course, as the former means
      * single-selection mode, the latter multi-selection.
      */
+	// ### KDE4: make virtual
     void setMode( KFile::Mode m );
     /**
      * @returns the listing/selection mode.
@@ -282,6 +287,7 @@ class KDirOperator : public QWidget
      * The ownership of @p w is transferred to KDirOperator, so don't
      * delete it yourself!
      */
+	// ### KDE4: make virtual
     void setPreviewWidget(const QWidget *w);
 
     /**
@@ -314,10 +320,10 @@ class KDirOperator : public QWidget
     /**
      * @returns a KCompletion object, containing all filenames and
      * directories of the current directory/URL.
-     * You can use it to insert it into a @ref KLineEdit or @ref KComboBox
-     * Note: it will only contain files, after @ref prepareCompletionObjects()
-     * has been called. It will be implicitly called from @ref makeCompletion()
-     * or @ref makeDirCompletion()
+     * You can use it to insert it into a KLineEdit or KComboBox
+     * Note: it will only contain files, after prepareCompletionObjects()
+     * has been called. It will be implicitly called from makeCompletion()
+     * or makeDirCompletion()
      */
     KCompletion * completionObject() const {
 	return const_cast<KCompletion *>( &myCompletion );
@@ -326,10 +332,10 @@ class KDirOperator : public QWidget
     /**
      * @returns a KCompletion object, containing only all directories of the
      * current directory/URL.
-     * You can use it to insert it into a @ref KLineEdit or @ref KComboBox
+     * You can use it to insert it into a KLineEdit or KComboBox
      * Note: it will only contain directories, after
-     * @ref prepareCompletionObjects() has been called. It will be implicitly
-     * called from @ref makeCompletion() or @ref makeDirCompletion()
+     * prepareCompletionObjects() has been called. It will be implicitly
+     * called from makeCompletion() or makeDirCompletion()
      */
     KCompletion *dirCompletionObject() const {
 	return const_cast<KCompletion *>( &myDirCompletion );
@@ -372,7 +378,9 @@ class KDirOperator : public QWidget
      * "preview" and "single" actions are in an exclusive group.
      *
      * You can e.g. use
-     * <pre>actionCollection()->action( "up" )->plug( someToolBar );</pre>
+     * \code
+     * actionCollection()->action( "up" )->plug( someToolBar );
+     * \endcode
      * to add a button into a toolbar, which makes the dirOperator change to
      * its parent directory.
      *
@@ -382,9 +390,9 @@ class KDirOperator : public QWidget
 
     /**
      * Sets the config object and the to be used group in KDirOperator. This
-     * will be used to store the view's configuration via 
-     * @ref KFileView::writeConfig() (and for @ref KFileView::readConfig()).
-     * If you don't set this, the views cannot save and restore their 
+     * will be used to store the view's configuration via
+     * KFileView::writeConfig() (and for KFileView::readConfig()).
+     * If you don't set this, the views cannot save and restore their
      * configuration.
      *
      * Usually you call this right after KDirOperator creation so that the view
@@ -393,12 +401,13 @@ class KDirOperator : public QWidget
      * Note that KDirOperator does NOT take ownership of that object (typically
      * it's KGlobal::config() anyway.
      *
-     * @see #viewConfig
-     * @see #viewConfigGroup
+     * @see viewConfig
+     * @see viewConfigGroup
      * @since 3.1
      */
+	// ### KDE4: make virtual
     void setViewConfig( KConfig *config, const QString& group );
-    
+
     /**
      * Returns the KConfig object used for saving and restoring view's
      * configuration.
@@ -416,27 +425,27 @@ class KDirOperator : public QWidget
      * @since 3.1
      */
     QString viewConfigGroup() const;
-    
+
     /**
      * Reads the default settings for a view, i.e. the default KFile::FileView.
      * Also reads the sorting and whether hidden files should be shown.
      * Note: the default view will not be set - you have to call
-     * <pre>
+     * \code
      * setView( KFile::Default )
-     * </pre>
+     * \endcode
      * to apply it.
      *
      * @see setView
-     * @see #setViewConfig
-     * @see #writeConfig
+     * @see setViewConfig
+     * @see writeConfig
      */
     virtual void readConfig( KConfig *, const QString& group = QString::null );
 
     /**
      * Saves the current settings like sorting, simple or detailed view.
      *
-     * @see #readConfig
-     * @see #setViewConfig
+     * @see readConfig
+     * @see setViewConfig
      */
     virtual void writeConfig( KConfig *, const QString& group = QString::null );
 
@@ -456,7 +465,7 @@ class KDirOperator : public QWidget
     /**
      * @returns whether files (not directories) should only be select()ed by
      * double-clicks.
-     * @see #setOnlyDoubleClickSelectsFiles
+     * @see setOnlyDoubleClickSelectsFiles
      */
     bool onlyDoubleClickSelectsFiles() const;
 
@@ -468,25 +477,29 @@ class KDirOperator : public QWidget
      * to the user.
      * @returns true if the directory could be created.
      */
+	// ### KDE4: make virtual
     bool mkdir( const QString& directory, bool enterDirectory = true );
 
     /**
-     * Starts and returns a @ref KIO::DeleteJob to delete the given @p items.
+     * Starts and returns a KIO::DeleteJob to delete the given @p items.
      *
+     * @param items the list of items to be deleted
      * @param ask specifies whether a confirmation dialog should be shown
      * @param showProgress passed to the DeleteJob to show a progress dialog
      */
+	// ### KDE4: make virtual
     KIO::DeleteJob * del( const KFileItemList& items,
                           bool ask = true, bool showProgress = true );
 
     /**
-     * Starts and returns a @ref KIO::DeleteJob to delete the given @p items.
+     * Starts and returns a KIO::DeleteJob to delete the given @p items.
      *
      * @param parent the parent widget used for the confirmation dialog
      * @param ask specifies whether a confirmation dialog should be shown
      * @param showProgress passed to the DeleteJob to show a progress dialog
      * @since 3.1
      */
+	// ### KDE4: make virtual
     KIO::DeleteJob * del( const KFileItemList& items, QWidget *parent,
                           bool ask = true, bool showProgress = true );
 
@@ -504,6 +517,7 @@ class KDirOperator : public QWidget
      *
      * Default is off.
      */
+	// ### KDE4: make virtual
     void setEnableDirHighlighting( bool enable );
 
     /**
@@ -527,35 +541,47 @@ class KDirOperator : public QWidget
 
     /**
      * Sets up the action menu.
-     * @param actionTypes is an value of OR'd @ref ActionTypes that controls which actions to show in the action menu
+     * @param whichActions is an value of OR'd ActionTypes that controls which actions to show in the action menu
      */
     void setupMenu(int whichActions);
 
+    /**
+     * Reimplemented - allow dropping of files if @p b is true
+     * @param b true if the widget should allow dropping of files
+     */
+    virtual void setAcceptDrops(bool b);
+
+    /**
+     * Sets the options for dropping files.
+     * @see KFileView::DropOptions
+     * @since 3.2
+     */
+     // ### KDE4: make virtual
+    void setDropOptions(int options);
+
 protected:
     /**
-     * A view factory for creating predefined fileviews. Called internally by @ref #setView
+     * A view factory for creating predefined fileviews. Called internally by setView
 	 * , but you can also call it directly. Reimplement this if you depend on self defined fileviews.
-	 * @param parent is the  @ref QWidget to be set as parent
+	 * @param parent is the QWidget to be set as parent
 	 * @param view is the predefined view to be set, note: this can be several ones OR:ed together.
-	 * @returns the created @ref KFileView
+	 * @returns the created KFileView
      * @see KFileView
      * @see KCombiView
      * @see KFileDetailView
      * @see KFileIconView
      * @see KFilePreview
      * @see KFile::FileView
-     * @see #setView
+     * @see setView
      */
      virtual KFileView* createView( QWidget* parent, KFile::FileView view );
      /**
-     * Sets a custom KFileReader to list directories.
+     * Sets a custom KDirLister to list directories.
      */
+	// ### KDE4: make virtual
     void setDirLister( KDirLister *lister );
 
-    /**
-     * @reimplemented
-     */
-    void resizeEvent( QResizeEvent * );
+    virtual void resizeEvent( QResizeEvent * );
 
     /**
      * Sets up all the actions. Called from the constructor, you usually
@@ -565,7 +591,7 @@ protected:
 
     /**
      * Updates the sorting-related actions to comply with the current sorting
-     * @see #sorting
+     * @see sorting
      */
     void updateSortActions();
 
@@ -586,14 +612,14 @@ protected:
      * Synchronizes the completion objects with the entries of the
      * currently listed url.
      *
-     * Automatically called from @ref makeCompletion() and
-     * @ref makeDirCompletion()
+     * Automatically called from makeCompletion() and
+     * makeDirCompletion()
      */
     void prepareCompletionObjects();
 
     /**
-     * Checks if there support from @ref KIO::PreviewJob for the currently
-     * shown files, taking @ref mimeFilter() and @ref nameFilter() into account
+     * Checks if there support from KIO::PreviewJob for the currently
+     * shown files, taking mimeFilter() and nameFilter() into account
      * Enables/disables the preview-action accordingly.
      */
     bool checkPreviewSupport();
@@ -602,21 +628,25 @@ public slots:
     /**
      * Goes one step back in the history and opens that url.
      */
+	// ### KDE4: make virtual
     void back();
 
     /**
      * Goes one step forward in the history and opens that url.
      */
+	// ### KDE4: make virtual
     void forward();
 
     /**
      * Enters the home directory.
      */
+	// ### KDE4: make virtual
     void home();
 
     /**
      * Goes one directory up from the current url.
      */
+	// ### KDE4: make virtual
     void cdUp();
 
     /**
@@ -627,16 +657,19 @@ public slots:
     /**
      * Re-reads the current url.
      */
+	// ### KDE4: make virtual
     void rereadDir();
 
     /**
      * Opens a dialog to create a new directory.
      */
+	// ### KDE4: make virtual
     void mkdir();
 
     /**
      * Deletes the currently selected files/directories.
      */
+	// ### KDE4: make virtual
     void deleteSelected();
 
     /**
@@ -664,7 +697,7 @@ protected slots:
     void resetCursor();
 
     /**
-     * Called after @ref setURL() to load the directory, update the history,
+     * Called after setURL() to load the directory, update the history,
      * etc.
      */
     void pathChanged();
@@ -684,6 +717,7 @@ protected slots:
     /**
      * Enters the directory specified by the given @p item.
      */
+	// ### KDE4: make virtual
     void selectDir(const KFileItem *item );
 
     /**
@@ -745,15 +779,28 @@ signals:
 
     /**
      * Emitted whenever the current fileview is changed, either by an explicit
-     * call to @ref setView() or by the user selecting a different view thru
+     * call to setView() or by the user selecting a different view thru
      * the GUI.
      */
     void viewChanged( KFileView * newView );
 
-    void fileHighlighted(const KFileItem*);
-    void dirActivated(const KFileItem*);
-    void fileSelected(const KFileItem*);
-
+    /**
+     * Emitted when a file is highlighted or generally the selection changes in
+     * multiselection mode. In the latter case, @p item is 0L. You can access
+     * the selected items with selectedItems().
+     */
+    void fileHighlighted( const KFileItem *item );
+    void dirActivated( const KFileItem *item );
+    void fileSelected( const KFileItem *item );
+    /**
+     * Emitted when files are dropped. Dropping files is disabled by
+     * default. You need to enable it with setAcceptDrops()
+     * @param item the item on which the drop occurred or 0.
+     * @param event the drop event itself.
+     * @param urls the urls that where dropped.
+     * @since 3.2
+     */
+    void dropped(const KFileItem *item, QDropEvent*event, const KURL::List&urls);
 private:
     /**
      * Contains all URLs you can reach with the back button.
@@ -862,9 +909,9 @@ private slots:
     void slotRefreshItems( const KFileItemList& items );
 
     void slotProperties();
-    
+
     void insertViewDependentActions();
-    
+
 private:
     static bool isReadable( const KURL& url );
 
