@@ -32,13 +32,14 @@
 #include <kapplication.h>
 #include <klocale.h>
 #include <kfileitem.h>
+#include <kiconeffect.h>
 #include <kglobalsettings.h>
+#include <kurldrag.h>
 #include <kio/previewjob.h>
 
 #include "kfileiconview.h"
 #include "config-kfile.h"
 
-
 KFileIconViewItem::~KFileIconViewItem()
 {
     fileInfo()->removeExtraData( iconView() );
@@ -49,7 +50,8 @@ class KFileIconView::KFileIconViewPrivate
 public:
     KFileIconViewPrivate( KFileIconView *parent ) {
         previewIconSize = 60;
-        job = 0L;
+        job = 0;
+        dropItem = 0;
 
         noArrangement = false;
 	smallColumns = new KRadioAction( i18n("Small Icons"), 0, parent,
@@ -61,31 +63,43 @@ public:
 				      SLOT( slotLargeRows() ),
 				      parent->actionCollection(),
 				      "large rows" );
-	
+
 	smallColumns->setExclusiveGroup(QString::fromLatin1("IconView mode"));
 	largeRows->setExclusiveGroup(QString::fromLatin1("IconView mode"));
 
         previews = new KToggleAction( i18n("Thumbnail Previews"), 0,
                                       parent->actionCollection(),
                                       "show previews" );
+        zoomIn = KStdAction::zoomIn( parent, SLOT( zoomIn() ),
+                                     parent->actionCollection(), "zoomIn" );
+        zoomOut = KStdAction::zoomOut( parent, SLOT( zoomOut() ),
+                                     parent->actionCollection(), "zoomOut" );
+
+        previews->setGroup("previews");
+        zoomIn->setGroup("previews");
+        zoomOut->setGroup("previews");
+
         connect( previews, SIGNAL( toggled( bool )),
                  parent, SLOT( slotPreviewsToggled( bool )));
 
-        previewTimer = new QTimer;
-        connect( previewTimer, SIGNAL( timeout() ),
+        connect( &previewTimer, SIGNAL( timeout() ),
                  parent, SLOT( showPreviews() ));
+        connect( &autoOpenTimer, SIGNAL( timeout() ),
+                 parent, SLOT( slotAutoOpen() ));
     }
 
     ~KFileIconViewPrivate() {
-        delete previewTimer;
         if ( job )
             job->kill();
     }
 
     KRadioAction *smallColumns, *largeRows;
+    KAction *zoomIn, *zoomOut;
     KToggleAction *previews;
     KIO::PreviewJob *job;
-    QTimer *previewTimer;
+    KFileIconViewItem *dropItem;
+    QTimer previewTimer;
+    QTimer autoOpenTimer;
     QStringList previewMimeTypes;
     int previewIconSize;
     bool noArrangement :1;
@@ -232,7 +246,7 @@ void KFileIconView::showToolTip( QIconViewItem *item )
 		toolTip->move(toolTip->x(), screen.bottom()-toolTip->y()-toolTip->height()+toolTip->y());
 	}
 	toolTip->setFont( QToolTip::font() );
-	toolTip->setPalette( QToolTip::palette(), TRUE );
+	toolTip->setPalette( QToolTip::palette(), true );
 	toolTip->show();
     }
 }
@@ -446,9 +460,12 @@ void KFileIconView::setIconSize( int size )
 
 void KFileIconView::setPreviewSize( int size )
 {
+    if ( size < 30 )
+        size = 30; // minimum
+
     d->previewIconSize = size;
     if ( d->previews->isChecked() )
-        updateIcons();
+        showPreviews();
 }
 
 void KFileIconView::updateIcons()
@@ -562,7 +579,15 @@ void KFileIconView::gotPreview( const KFileItem *item, const QPixmap& pix )
 {
     KFileIconViewItem *it = viewItem( item );
     if ( it )
-        it->setPixmap( pix );
+        if( item->overlays() & KIcon::HiddenOverlay )
+        {
+            QPixmap p( pix );
+
+            KIconEffect::semiTransparent( p );
+            it->setPixmap( p );
+        }
+        else
+            it->setPixmap( pix );
 }
 
 bool KFileIconView::canPreview( const KFileItem *item ) const
@@ -624,7 +649,8 @@ void KFileIconView::setSorting( QDir::SortSpec spec )
 
     if ( spec & QDir::Time ) {
         for ( ; (item = it.current()); ++it )
-            viewItem(item)->setKey( sortingKey( item->time( KIO::UDS_MODIFICATION_TIME ), item->isDir(), spec ));
+            // warning, time_t is often signed -> cast it
+            viewItem(item)->setKey( sortingKey( (unsigned long)item->time( KIO::UDS_MODIFICATION_TIME ), item->isDir(), spec ));
     }
 
     else if ( spec & QDir::Size ) {
@@ -658,6 +684,8 @@ void KFileIconView::determineIcon( KFileIconViewItem *item )
 
 void KFileIconView::listingCompleted()
 {
+    arrangeItemsInGrid();
+
     // QIconView doesn't set the current item automatically, so we have to do
     // that. We don't want to emit selectionChanged() tho.
     if ( !currentItem() ) {
@@ -669,7 +697,6 @@ void KFileIconView::listingCompleted()
         blockSignals( block );
     }
 
-    arrangeItemsInGrid();
     m_resolver->start( d->previews->isChecked() ? 0 : 10 );
 }
 
@@ -692,9 +719,6 @@ bool KFileIconView::eventFilter( QObject *o, QEvent *e )
 void KFileIconView::showEvent( QShowEvent *e )
 {
     KIconView::showEvent( e );
-#if QT_VERSION <= 302
-    sort( !isReversed() );
-#endif
 }
 
 
@@ -706,7 +730,7 @@ void KFileIconView::initItem( KFileIconViewItem *item, const KFileItem *i,
         // this causes a repaint of the item, which we want to avoid during
         // directory listing, when all items are created. We want to paint all
         // items at once, not every single item in that case.
-    item->setText( i->text() , false, false );
+        item->setText( i->text() , false, false );
         item->setPixmap( i->pixmap( iconSizeFor( i ) ) );
     }
 
@@ -714,7 +738,8 @@ void KFileIconView::initItem( KFileIconViewItem *item, const KFileItem *i,
     QDir::SortSpec spec = KFileView::sorting();
 
     if ( spec & QDir::Time )
-        item->setKey( sortingKey( i->time( KIO::UDS_MODIFICATION_TIME ),
+        // warning, time_t is often signed -> cast it
+        item->setKey( sortingKey( (unsigned long) i->time( KIO::UDS_MODIFICATION_TIME ),
                                   i->isDir(), spec ));
     else if ( spec & QDir::Size )
         item->setKey( sortingKey( i->size(), i->isDir(), spec ));
@@ -725,7 +750,7 @@ void KFileIconView::initItem( KFileIconViewItem *item, const KFileItem *i,
     //qDebug("** key for: %s: %s", i->text().latin1(), item->key().latin1());
 
     if ( d->previews->isChecked() )
-        d->previewTimer->start( 10, true );
+        d->previewTimer.start( 10, true );
 }
 
 void KFileIconView::arrangeItemsInGrid( bool update )
@@ -743,6 +768,145 @@ int KFileIconView::iconSizeFor( const KFileItem *item ) const
     return myIconSize;
 }
 
+void KFileIconView::zoomIn()
+{
+    setPreviewSize( d->previewIconSize + 30 );
+}
+
+void KFileIconView::zoomOut()
+{
+    setPreviewSize( d->previewIconSize - 30 );
+}
+
+QDragObject *KFileIconView::dragObject()
+{
+    // create a list of the URL:s that we want to drag
+    KURL::List urls;
+    KFileItemListIterator it( * KFileView::selectedItems() );
+    for ( ; it.current(); ++it ){
+        urls.append( (*it)->url() );
+    }
+    QPixmap pixmap;
+    if( urls.count() > 1 )
+        pixmap = DesktopIcon( "kmultiple", iconSize() );
+    if( pixmap.isNull() )
+        pixmap = currentFileItem()->pixmap( iconSize() );
+
+    QPoint hotspot;
+    hotspot.setX( pixmap.width() / 2 );
+    hotspot.setY( pixmap.height() / 2 );
+    QDragObject* myDragObject = new KURLDrag( urls, widget() );
+    myDragObject->setPixmap( pixmap, hotspot );
+    return myDragObject;
+}
+
+void KFileIconView::slotAutoOpen()
+{
+    d->autoOpenTimer.stop();
+    if( !d->dropItem )
+        return;
+
+    KFileItem *fileItem = d->dropItem->fileInfo();
+    if (!fileItem)
+        return;
+
+    if( fileItem->isFile() )
+        return;
+
+    if ( fileItem->isDir() || fileItem->isLink())
+        sig->activate( fileItem );
+}
+
+bool KFileIconView::acceptDrag(QDropEvent* e) const
+{
+   return KURLDrag::canDecode( e ) &&
+       (e->source()!=const_cast<KFileIconView*>(this)) &&
+       ( e->action() == QDropEvent::Copy
+      || e->action() == QDropEvent::Move
+      || e->action() == QDropEvent::Link );
+}
+
+void KFileIconView::contentsDragEnterEvent( QDragEnterEvent *e )
+{
+    if ( ! acceptDrag( e ) ) { // can we decode this ?
+        e->ignore();            // No
+        return;
+    }
+    e->acceptAction();     // Yes
+
+    if ((dropOptions() & AutoOpenDirs) == 0)
+       return;
+
+    KFileIconViewItem *item = dynamic_cast<KFileIconViewItem*>(findItem( contentsToViewport( e->pos() ) ));
+    if ( item ) {  // are we over an item ?
+       d->dropItem = item;
+       d->autoOpenTimer.start( autoOpenDelay() ); // restart timer
+    }
+    else
+    {
+       d->dropItem = 0;
+       d->autoOpenTimer.stop();
+    }
+}
+
+void KFileIconView::contentsDragMoveEvent( QDragMoveEvent *e )
+{
+    if ( ! acceptDrag( e ) ) { // can we decode this ?
+        e->ignore();            // No
+        return;
+    }
+    e->acceptAction();     // Yes
+
+    if ((dropOptions() & AutoOpenDirs) == 0)
+       return;
+
+    KFileIconViewItem *item = dynamic_cast<KFileIconViewItem*>(findItem( contentsToViewport( e->pos() ) ));
+    if ( item ) {  // are we over an item ?
+       if (d->dropItem != item)
+       {
+           d->dropItem = item;
+           d->autoOpenTimer.start( autoOpenDelay() ); // restart timer
+       }
+    }
+    else
+    {
+       d->dropItem = 0;
+       d->autoOpenTimer.stop();
+    }
+}
+
+void KFileIconView::contentsDragLeaveEvent( QDragLeaveEvent * )
+{
+    d->dropItem = 0;
+    d->autoOpenTimer.stop();
+}
+
+void KFileIconView::contentsDropEvent( QDropEvent *e )
+{
+    d->dropItem = 0;
+    d->autoOpenTimer.stop();
+
+    if ( ! acceptDrag( e ) ) { // can we decode this ?
+        e->ignore();            // No
+        return;
+    }
+    e->acceptAction();     // Yes
+
+    KFileIconViewItem *item = dynamic_cast<KFileIconViewItem*>(findItem( contentsToViewport( e->pos() ) ));
+    KFileItem * fileItem = 0;
+    if (item)
+        fileItem = item->fileInfo();
+
+    emit dropped(e, fileItem);
+
+    KURL::List urls;
+    if (KURLDrag::decode( e, urls ) && !urls.isEmpty())
+    {
+        emit dropped(e, urls, fileItem ? fileItem->url() : KURL());
+        sig->dropURLs(fileItem, e, urls);
+    }
+}
+
 void KFileIconView::virtual_hook( int id, void* data )
 { KIconView::virtual_hook( id, data );
   KFileView::virtual_hook( id, data ); }
