@@ -17,19 +17,18 @@
    Boston, MA 02111-1307, USA.
 */
 
+#include <qapplication.h>
 #include <qheader.h>
 #include <qtimer.h>
 #include <kdebug.h>
 #include <kdirnotify_stub.h>
 #include <kglobalsettings.h>
 #include <kfileitem.h>
-#include <kio/global.h>
+#include <kfileview.h>
 #include <kmimetype.h>
 #include <kstandarddirs.h>
-#include <kurldrag.h>
 #include <stdlib.h>
 #include <assert.h>
-#include <kfiletreeview.h>
 #include <kio/job.h>
 #include <kio/global.h>
 #include <kurldrag.h>
@@ -40,9 +39,6 @@
 #include "kfiletreebranch.h"
 #include "kfiletreeviewitem.h"
 
-static const int autoOpenTimeout = 750;
-
-
 KFileTreeView::KFileTreeView( QWidget *parent, const char *name )
     : KListView( parent, name ),
       m_wantOpenFolderPixmaps( true ),
@@ -68,8 +64,8 @@ KFileTreeView::KFileTreeView( QWidget *parent, const char *name )
     	     this, SLOT( slotExpanded( QListViewItem *) ));
     connect( this, SIGNAL( collapsed( QListViewItem *) ),
 	     this, SLOT( slotCollapsed( QListViewItem* )));
-    
-    
+
+
     /* connections from the konqtree widget */
     connect( this, SIGNAL( selectionChanged() ),
              this, SLOT( slotSelectionChanged() ) );
@@ -78,7 +74,7 @@ KFileTreeView::KFileTreeView( QWidget *parent, const char *name )
     connect( this, SIGNAL(itemRenamed(QListViewItem*, const QString &, int)),
              this, SLOT(slotItemRenamed(QListViewItem*, const QString &, int)));
 
-	     
+
     m_bDrag = false;
     m_branches.setAutoDelete( true );
 
@@ -88,7 +84,7 @@ KFileTreeView::KFileTreeView( QWidget *parent, const char *name )
 KFileTreeView::~KFileTreeView()
 {
    // we must make sure that the KFileTreeViewItems are deleted _before_ the
-   // branches are deleted. Otherwise, the KFileItems would be destroyed 
+   // branches are deleted. Otherwise, the KFileItems would be destroyed
    // and the KFileTreeViewItems had dangling pointers to them.
    hide();
    clear();
@@ -110,7 +106,7 @@ void KFileTreeView::contentsDragEnterEvent( QDragEnterEvent *ev )
    if( item )
    {
       m_dropItem = item;
-      m_autoOpenTimer->start( autoOpenTimeout );
+      m_autoOpenTimer->start( KFileView::autoOpenDelay() );
    }
    else
    {
@@ -128,18 +124,21 @@ void KFileTreeView::contentsDragMoveEvent( QDragMoveEvent *e )
    e->acceptAction();
 
 
-   QListViewItem *item; // itemAt( contentsToViewport( e->pos() ) );
-   QListViewItem *par;
+   QListViewItem *afterme;
+   QListViewItem *parent;
+
+   findDrop( e->pos(), parent, afterme );
+
+   // "afterme" is 0 when aiming at a directory itself
+   QListViewItem *item = afterme ? afterme : parent;
 
-   findDrop( e->pos(), par, item );
-   
    if( item && item->isSelectable() )
    {
       setSelected( item, true );
       if( item != m_dropItem ) {
 	 m_autoOpenTimer->stop();
 	 m_dropItem = item;
-	 m_autoOpenTimer->start( autoOpenTimeout );
+	 m_autoOpenTimer->start( KFileView::autoOpenDelay() );
       }
    }
    else
@@ -151,7 +150,6 @@ void KFileTreeView::contentsDragMoveEvent( QDragMoveEvent *e )
 
 void KFileTreeView::contentsDragLeaveEvent( QDragLeaveEvent * )
 {
-
    // Restore the current item to what it was before the dragging (#17070)
    if ( m_currentBeforeDropItem )
    {
@@ -170,7 +168,7 @@ void KFileTreeView::contentsDropEvent( QDropEvent *e )
 
     m_autoOpenTimer->stop();
     m_dropItem = 0;
-    
+
     kdDebug(250) << "contentsDropEvent !" << endl;
     if( ! acceptDrag( e ) ) {
        e->ignore();
@@ -182,6 +180,9 @@ void KFileTreeView::contentsDropEvent( QDropEvent *e )
     QListViewItem *parent;
     findDrop(e->pos(), parent, afterme);
 
+    //kdDebug(250) << " parent=" << (parent?parent->text(0):QString::null)
+    //             << " afterme=" << (afterme?afterme->text(0):QString::null) << endl;
+
     if (e->source() == viewport() && itemsMovable())
         movableDropEvent(parent, afterme);
     else
@@ -191,20 +192,20 @@ void KFileTreeView::contentsDropEvent( QDropEvent *e )
        emit dropped(e, parent, afterme);
        emit dropped(this, e, parent, afterme);
 
-       
+       KURL::List urls;
+       KURLDrag::decode( e, urls );
+       emit dropped( this, e, urls );
+
        KURL parentURL;
-       if( afterme )
-       {
-	  if(  static_cast<KFileTreeViewItem*>(afterme)->isDir() )
-	     parentURL = static_cast<KFileTreeViewItem*>(afterme)->url();
-	  else
-	     parentURL = static_cast<KFileTreeViewItem*>(parent)->url();
-       }
+       if( parent )
+           parentURL = static_cast<KFileTreeViewItem*>(parent)->url();
+       else
+           // can happen when dropping above the root item
+           // Should we choose the first branch in such a case ??
+           return;
 
-    KURL::List urls;
-    KURLDrag::decode( e, urls );
-    emit dropped( this, e, urls );
        emit dropped( urls, parentURL );
+       emit dropped( this , e, urls, parentURL );
     }
 }
 
@@ -212,9 +213,11 @@ bool KFileTreeView::acceptDrag(QDropEvent* e ) const
 {
 
    bool ancestOK= acceptDrops();
-   // kdDebug(250) << "Do accept drops: " << parentOK << endl;
+   // kdDebug(250) << "Do accept drops: " << ancestOK << endl;
    ancestOK = ancestOK && itemsMovable();
-   // kdDebug(250) << "ismovable: " << parentOK << endl;
+   // kdDebug(250) << "acceptDrag: " << ancestOK << endl;
+   // kdDebug(250) << "canDecode: " << KURLDrag::canDecode(e) << endl;
+   // kdDebug(250) << "action: " << e->action() << endl;
 
    /*  KListView::acceptDrag(e);  */
    /* this is what KListView does:
@@ -222,6 +225,7 @@ bool KFileTreeView::acceptDrag(QDropEvent* e ) const
     * ask acceptDrops and itemsMovable, but not the third
     */
    return ancestOK && KURLDrag::canDecode( e ) &&
+       // Why this test? All DnDs are one of those AFAIK (DF)
       ( e->action() == QDropEvent::Copy
 	|| e->action() == QDropEvent::Move
 	|| e->action() == QDropEvent::Link );
@@ -248,7 +252,7 @@ QDragObject * KFileTreeView::dragObject()
       pixmap = currentKFileTreeViewItem()->fileItem()->pixmap( 16 );
    hotspot.setX( pixmap.width() / 2 );
    hotspot.setY( pixmap.height() / 2 );
-   QDragObject* dragObject = KURLDrag::newDrag( urls, this );
+   QDragObject* dragObject = new KURLDrag( urls, this );
    if( dragObject )
       dragObject->setPixmap( pixmap, hotspot );
    return dragObject;
@@ -264,7 +268,6 @@ void KFileTreeView::slotCollapsed( QListViewItem *item )
    {
       item->setPixmap( 0, itemIcon(kftvi));
    }
-   
 }
 
 void KFileTreeView::slotExpanded( QListViewItem *item )
@@ -272,7 +275,7 @@ void KFileTreeView::slotExpanded( QListViewItem *item )
    kdDebug(250) << "slotExpanded here !" << endl;
 
    if( ! item ) return;
-   
+
    KFileTreeViewItem *it = static_cast<KFileTreeViewItem*>(item);
    KFileTreeBranch *branch = it->branch();
 
@@ -425,7 +428,7 @@ void KFileTreeView::slotNewTreeViewItems( KFileTreeBranch* branch, const KFileTr
 {
    if( ! branch ) return;
    kdDebug(250) << "hitting slotNewTreeViewItems" << endl;
-   
+
    /* Sometimes it happens that new items should become selected, i.e. if the user
     * creates a new dir, he probably wants it to be selected. This can not be done
     * right after creating the directory or file, because it takes some time until
@@ -436,13 +439,13 @@ void KFileTreeView::slotNewTreeViewItems( KFileTreeBranch* branch, const KFileTr
    if( ! m_nextUrlToSelect.isEmpty() )
    {
       KFileTreeViewItemListIterator it( itemList );
-      
+
       bool end = false;
       for( ; !end && it.current(); ++it )
       {
 	 KURL url = (*it)->url();
-	 
-	 if( m_nextUrlToSelect.cmp(url, true ))   // ignore trailing / on dirs
+
+	 if( m_nextUrlToSelect.equals(url, true ))   // ignore trailing / on dirs
 	 {
 	    setCurrentItem( static_cast<QListViewItem*>(*it) );
 	    m_nextUrlToSelect = KURL();
@@ -456,7 +459,7 @@ QPixmap KFileTreeView::itemIcon( KFileTreeViewItem *item, int gap ) const
 {
    QPixmap pix;
    kdDebug(250) << "Setting icon for column " << gap << endl;
-   
+
    if( item )
    {
       /* Check if it is a branch root */
@@ -483,7 +486,7 @@ QPixmap KFileTreeView::itemIcon( KFileTreeViewItem *item, int gap ) const
       }
    }
    }
-   
+
    return pix;
 }
 
@@ -526,13 +529,13 @@ void KFileTreeView::startAnimation( KFileTreeViewItem * item, const char * iconB
 void KFileTreeView::stopAnimation( KFileTreeViewItem * item )
 {
    if( ! item ) return;
-   
+
    kdDebug(250) << "Stoping Animation !" << endl;
-   
+
    MapCurrentOpeningFolders::Iterator it = m_mapCurrentOpeningFolders.find(item);
    if ( it != m_mapCurrentOpeningFolders.end() )
    {
-      if( item->isDir() && isOpen(static_cast<QListViewItem*>(item)))
+      if( item->isDir() && isOpen( item) )
       {
 	 kdDebug(250) << "Setting folder open pixmap !" << endl;
 	 item->setPixmap( 0, itemIcon( item ));
@@ -573,11 +576,11 @@ void KFileTreeView::slotOnItem( QListViewItem *item )
     KFileTreeViewItem *i = static_cast<KFileTreeViewItem *>( item );
     if( i )
     {
-       const KURL& url = i->url();
+       const KURL url = i->url();
        if ( url.isLocalFile() )
-	  emit( onItem( url.path() ));
+	  emit onItem( url.path() );
        else
-	  emit( onItem( url.prettyURL()));
+	  emit onItem( url.prettyURL() );
     }
 }
 
@@ -611,7 +614,7 @@ KFileTreeViewItem *KFileTreeView::findItem( KFileTreeBranch* brnch, const QStrin
 
          kdDebug(250) << "assembled complete dir string " << url.prettyURL() << endl;
 
-         KFileItem *fi = brnch->find( url );
+         KFileItem *fi = brnch->findByURL( url );
          if( fi )
          {
             ret = static_cast<KFileTreeViewItem*>( fi->extraData( brnch ));
