@@ -1,6 +1,6 @@
 /* vi: ts=8 sts=4 sw=4
  *
- * $Id: kicondialog.cpp,v 1.30 2002/05/06 02:16:06 nhasan Exp $
+ * $Id: kicondialog.cpp,v 1.32.2.1 2004/02/01 19:10:48 lukas Exp $
  *
  * This file is part of the KDE project, module kfile.
  * Copyright (C) 2000 Geert Jansen <jansen@kde.org>
@@ -181,7 +181,7 @@ void KIconCanvas::slotCurrentChanged(QIconViewItem *item)
 class KIconDialog::KIconDialogPrivate
 {
   public:
-    KIconDialogPrivate() { 
+    KIconDialogPrivate() {
         m_bStrictIconSize = true;
     }
     ~KIconDialogPrivate() {}
@@ -261,7 +261,7 @@ void KIconDialog::init()
     mpCombo->insertItem(i18n("Filesystems"));
     mpCombo->insertItem(i18n("Mimetypes"));
     mpCombo->setFixedSize(mpCombo->sizeHint());
-    mpBrowseBut->setFixedSize(QSize(mpCombo->width(), mpCombo->height()+6));
+    mpBrowseBut->setFixedWidth(mpCombo->width());
 
     // Make the dialog a little taller
     incInitialSize(QSize(0,100));
@@ -276,7 +276,7 @@ KIconDialog::~KIconDialog()
 void KIconDialog::slotAcceptIcons()
 {
   d->custom=QString::null;
-  accept();
+  slotOk();
 }
 
 void KIconDialog::showIcons()
@@ -284,13 +284,13 @@ void KIconDialog::showIcons()
     mpCanvas->clear();
     QStringList filelist;
     if (mType == 0)
-	if (d->m_bStrictIconSize) 
+	if (d->m_bStrictIconSize)
             filelist=mpLoader->queryIcons(mGroupOrSize, mContext);
         else
             filelist=mpLoader->queryIconsByContext(mGroupOrSize, mContext);
-    else if ( !d->customLocation.isNull() ) 
+    else if ( !d->customLocation.isNull() )
 	filelist=mpLoader->queryIconsByDir( d->customLocation );
-    else 
+    else
 	filelist=mFileList;
 
     QSortedList <IconPath>iconlist;
@@ -341,7 +341,7 @@ QString KIconDialog::selectIcon(KIcon::Group group, KIcon::Context context, bool
 }
 #endif
 
-void KIconDialog::setup(KIcon::Group group, KIcon::Context context, 
+void KIconDialog::setup(KIcon::Group group, KIcon::Context context,
                         bool strictIconSize, int iconSize, bool user )
 {
     d->m_bStrictIconSize = strictIconSize;
@@ -377,6 +377,34 @@ QString KIconDialog::openDialog()
     return QString::null;
 }
 
+void KIconDialog::showDialog()
+{
+    setModal(false);
+    showIcons();
+    show();
+}
+
+void KIconDialog::slotOk()
+{
+    QString name;
+    if (!d->custom.isNull())
+    {
+        name = d->custom;
+    }
+    else
+    {
+        name = mpCanvas->getCurrent();
+        if (!name.isEmpty() && (mType != 1))
+        {
+            QFileInfo fi(name);
+            name = fi.baseName();
+        }
+    }
+
+    emit newIconName(name);
+    KDialogBase::slotOk();
+}
+
 QString KIconDialog::getIcon(KIcon::Group group, KIcon::Context context,
                              bool strictIconSize, int iconSize, bool user,
                              QWidget *parent, const QString &caption)
@@ -385,7 +413,7 @@ QString KIconDialog::getIcon(KIcon::Group group, KIcon::Context context,
     dlg.setup( group, context, strictIconSize, iconSize, user );
     if (!caption.isNull())
         dlg.setCaption(caption);
-    
+
     return dlg.openDialog();
 }
 
@@ -416,13 +444,13 @@ void KIconDialog::slotButtonClicked(int id)
         break;
     case 2:
 	file = KFileDialog::getOpenFileName(QString::null,
-		i18n("*.png *.xpm|Icon Files (*.png *.xpm)"), this);
+		i18n("*.png *.xpm *.svg *.svgz|Icon Files (*.png *.xpm *.svg *.svgz)"), this);
 	if (!file.isEmpty())
         {
             d->custom = file;
 	    if ( mType == 1 )
 	      d->customLocation = QFileInfo( file ).dirPath( true );
-            accept();
+            slotOk();
 	}
 	break;
     }
@@ -460,7 +488,7 @@ void KIconDialog::slotFinished()
 class KIconButton::KIconButtonPrivate
 {
   public:
-    KIconButtonPrivate() { 
+    KIconButtonPrivate() {
         m_bStrictIconSize = false;
         iconSize = 0; // let KIconLoader choose the default
     }
@@ -539,8 +567,13 @@ void KIconButton::setIcon(const QString& icon)
 {
     mIcon = icon;
     setPixmap(mpLoader->loadIcon(mIcon, mGroup, d->iconSize));
+
     if (!mpDialog)
+    {
         mpDialog = new KIconDialog(mpLoader, this);
+        connect(mpDialog, SIGNAL(newIconName(const QString&)), SLOT(newIconName(const QString&)));
+    }
+
     if ( mbUser )
       mpDialog->setCustomLocation( QFileInfo( mpLoader->iconPath(mIcon, mGroup, true) ).dirPath( true ) );
 }
@@ -554,19 +587,27 @@ void KIconButton::resetIcon()
 void KIconButton::slotChangeIcon()
 {
     if (!mpDialog)
+    {
         mpDialog = new KIconDialog(mpLoader, this);
+        connect(mpDialog, SIGNAL(newIconName(const QString&)), SLOT(newIconName(const QString&)));
+    }
 
-    mpDialog->setup( mGroup, mContext, d->m_bStrictIconSize, d->iconSize, 
-                     mbUser );
-    
-    QString name = mpDialog->openDialog();
+    mpDialog->setup( mGroup, mContext, d->m_bStrictIconSize, d->iconSize, mbUser );
+    mpDialog->showDialog();
+}
+
+void KIconButton::newIconName(const QString& name)
+{
     if (name.isEmpty())
-	return;
+        return;
+
     QPixmap pm = mpLoader->loadIcon(name, mGroup, d->iconSize);
     setPixmap(pm);
     mIcon = name;
+
     if ( mbUser )
       mpDialog->setCustomLocation( QFileInfo( mpLoader->iconPath(mIcon, mGroup, true) ).dirPath( true ) );
+
     emit iconChanged(name);
 }
 
