@@ -1,6 +1,6 @@
 /* vi: ts=8 sts=4 sw=4
  *
- * $Id: kicondialog.h,v 1.18 2002/09/28 14:00:59 tjansen Exp $
+ * $Id: kicondialog.h,v 1.22 2003/11/25 08:07:05 aseigo Exp $
  *
  * This file is part of the KDE project, module kfile.
  * Copyright (C) 2000 Geert Jansen <jansen@kde.org>
@@ -62,6 +62,7 @@ signals:
      * Emitted when the current icon has changed.
      */
     void nameChanged(QString);
+    /* KDE 4: Make it const QString & */
 
     void startLoading(int);
     void progress(int);
@@ -110,7 +111,7 @@ public:
 
     /**
      * Sets a strict icon size policy for allowed icons. When true,
-     * only icons of the specified group's size in @ref getIcon() are shown.
+     * only icons of the specified group's size in getIcon() are shown.
      * When false, icons not available at the desired group's size will
      * also be selectable.
      */
@@ -128,12 +129,12 @@ public:
     /**
      * Sets the size of the icons to be shown / selected.
      * @see KIcon::StdSizes
-     * @see #iconSize
+     * @see iconSize
      */
     void setIconSize(int size);
 
     /**
-     * Returns the iconsize set via @ref setIconSize() or 0, if the default
+     * Returns the iconsize set via setIconSize() or 0, if the default
      * iconsize will be used.
      */
     int iconSize() const;
@@ -148,7 +149,7 @@ public:
 
     /**
      * Allows you to set the same parameters as in the class method
-     * @ref getIcon().
+     * getIcon().
      */
     void setup( KIcon::Group group,
                 KIcon::Context context = KIcon::Application,
@@ -159,10 +160,16 @@ public:
      * exec()utes this modal dialog and returns the name of the selected icon,
      * or QString::null if the dialog was aborted.
      * @returns the name of the icon, suitable for loading with KIconLoader.
-     * @see #getIcon
+     * @see getIcon
      */
     QString openDialog();
 
+    /**
+     * show()es this dialog and emits a newIcon(const QString&) signal when
+     * successful. QString::null will be emitted if the dialog was aborted.
+     */
+    void showDialog();
+
     /**
      * Pops up the dialog an lets the user select an icon.
      *
@@ -186,6 +193,12 @@ public:
                            bool user=false, QWidget *parent=0,
                            const QString &caption=QString::null);
 
+signals:
+    void newIconName(const QString&);
+
+protected slots:
+    void slotOk();
+
 private slots:
     void slotButtonClicked(int);
     void slotContext(int);
@@ -278,12 +291,12 @@ public:
     /**
      * Sets the size of the icon to be shown / selected.
      * @see KIcon::StdSizes
-     * @see #iconSize
+     * @see iconSize
      */
     void setIconSize( int size );
 
     /**
-     * Returns the iconsize set via @ref setIconSize() or 0, if the default
+     * Returns the iconsize set via setIconSize() or 0, if the default
      * iconsize will be used.
      */
     int iconSize() const;
@@ -293,9 +306,11 @@ signals:
      * Emitted when the icon has changed.
      */
     void iconChanged(QString icon);
+    /* KDE 4: Make it const QString & */
 
 private slots:
     void slotChangeIcon();
+    void newIconName(const QString& name);
 
 private:
     bool mbUser;
