@@ -23,10 +23,7 @@ class QTimer;
 
 class KFileDialog;
 class KFileItem;
-namespace KIO { 
-    class Job;
-    class PreviewJob; 
-}
+namespace KIO { class Job; class PreviewJob; }
 
 /**
  * Image preview widget for the file dialog.
@@ -46,7 +43,7 @@ class KImageFilePreview : public KPreviewWidgetBase
 		virtual void clearPreview();
 
 	protected slots:
-		void showPreview() { showPreview( currentURL, true ); }
+		void showPreview();
 		void showPreview( const KURL& url, bool force );
 
 		void toggleAuto(bool);
