@@ -15,16 +15,18 @@
     the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
     Boston, MA 02111-1307, USA.
   
-    $Id: kmetaprops.cpp,v 1.26 2002/06/18 08:25:21 ramagnus Exp $
+    $Id: kmetaprops.cpp,v 1.31.2.1 2004/03/30 11:22:50 waba Exp $
  */
 
 #include "kmetaprops.h"
 
-#include <kfilemetainfo.h>
-#include <kglobal.h>
-#include <klocale.h>
 #include <kdebug.h>
 #include <kfilemetainfowidget.h>
+#include <kfilemetainfo.h>
+#include <kglobal.h>
+#include <kglobalsettings.h>
+#include <klocale.h>
+#include <kprotocolinfo.h>
 
 #include <qvalidator.h>
 #include <qlayout.h>
@@ -125,7 +127,7 @@ void KFileMetaPropsPlugin::createLayout()
     }
 
     // let the dialog create the page frame
-    QFrame* topframe = properties->dialog()->addPage(i18n("&Meta Info"));
+    QFrame* topframe = properties->addPage(i18n("&Meta Info"));
     topframe->setFrameStyle(QFrame::NoFrame);
     QVBoxLayout* tmp = new QVBoxLayout(topframe);
 
@@ -243,8 +245,11 @@ bool KFileMetaPropsPlugin::supports( KFileItemList _items )
 #ifdef _GNUC
 #warning TODO: Add support for more than one item
 #endif
-  if (_items.count()!=1) return false;
-  return true;
+  if (KExecPropsPlugin::supports(_items) || KURLPropsPlugin::supports(_items))
+     return false; // Having both is redundant.
+
+  bool metaDataEnabled = KGlobalSettings::showFilePreview(_items.first()->url());
+  return _items.count() == 1 && metaDataEnabled;
 }
 
 void KFileMetaPropsPlugin::applyChanges()
