@@ -48,9 +48,21 @@
 #include <qtooltip.h>
 #include <qtimer.h>
 #include <qvbox.h>
+#include <qwhatsthis.h>
 
 using namespace KNotify;
 
+enum
+{
+    COL_EXECUTE = 0,
+    COL_STDERR  = 1,
+    COL_MESSAGE = 2,
+    COL_LOGFILE = 3,
+    COL_SOUND   = 4,
+    COL_TASKBAR = 5,
+    COL_EVENT   = 6
+};
+
 //
 // I don't feel like subclassing KComboBox and find ways to insert that into
 // the .ui file...
@@ -71,6 +83,7 @@ namespace KNotify
             combo->insertItem( i18n("Message Windows") );
             combo->insertItem( i18n("Passive Windows") );
             combo->insertItem( i18n("Standard Error Output") );
+            combo->insertItem( i18n("Taskbar") );
         }
 
         static int type( KComboBox *combo )
@@ -89,11 +102,51 @@ namespace KNotify
                     return KNotifyClient::PassivePopup;
                 case 5:
                     return KNotifyClient::Stderr;
+                case 6:
+                    return KNotifyClient::Taskbar;
             }
 
             return KNotifyClient::None;
         }
     };
+
+    // Needed for displaying tooltips in the listview's QHeader
+    class KNotifyToolTip : public QToolTip
+    {
+    public:
+        KNotifyToolTip( QHeader *header )
+            : QToolTip( header )
+        {
+            m_tips[COL_EXECUTE] = i18n("Execute a program");
+            m_tips[COL_STDERR]  = i18n("Print to Standard error output");
+            m_tips[COL_MESSAGE] = i18n("Display a messagebox");
+            m_tips[COL_LOGFILE] = i18n("Log to a file");
+            m_tips[COL_SOUND]   = i18n("Play a sound");
+            m_tips[COL_TASKBAR] = i18n("Flash the taskbar entry");
+        }
+        virtual ~KNotifyToolTip() {}
+
+    protected:
+        virtual void maybeTip ( const QPoint& p )
+        {
+            QHeader *header = static_cast<QHeader*>( parentWidget() );
+            int section = 0;
+
+            if ( header->orientation() == Horizontal )
+                section= header->sectionAt( p.x() );
+            else
+                section= header->sectionAt( p.y() );
+
+            if ( ( section < 0 ) || ( static_cast<uint>( section ) >= (sizeof(m_tips) / sizeof(QString)) ) )
+                return;
+
+            tip( header->sectionRect( section ), m_tips[section] );
+        }
+
+    private:
+        QString m_tips[6];
+    };
+
 }
 
 
@@ -155,17 +208,11 @@ void KNotifyDialog::slotDefault()
 //////////////////////////////////////////////////////////////////////
 
 
-#define COL_EXECUTE 0
-#define COL_STDERR  1
-#define COL_MESSAGE 2
-#define COL_LOGFILE 3
-#define COL_SOUND   4
-#define COL_EVENT   5
-
 class KNotifyWidget::Private
 {
 public:
-    QPixmap pixmaps[5];
+    QPixmap pixmaps[6];
+    KNotifyToolTip *toolTip;
 };
 
 // simple access to all knotify-handled applications
@@ -177,9 +224,6 @@ KNotifyWidget::KNotifyWidget( QWidget *parent, const char *name,
 
     m_allApps.setAutoDelete( true );
 
-    layout()->setMargin( 0 );
-    layout()->setSpacing( KDialogBase::spacingHint() );
-
     if ( !handleAllApps )
     {
         m_affectAllApps->hide();
@@ -197,12 +241,14 @@ KNotifyWidget::KNotifyWidget( QWidget *parent, const char *name,
     QPixmap pmessage = SmallIcon("info");
     QPixmap plogfile = SmallIcon("log");
     QPixmap psound = SmallIcon("sound");
+    QPixmap ptaskbar = SmallIcon("kicker");
 
     d->pixmaps[COL_EXECUTE] = pexec;
     d->pixmaps[COL_STDERR]  = pstderr;
     d->pixmaps[COL_MESSAGE] = pmessage;
     d->pixmaps[COL_LOGFILE] = plogfile;
     d->pixmaps[COL_SOUND]   = psound;
+    d->pixmaps[COL_TASKBAR] = ptaskbar;
 
     int w = KIcon::SizeSmall + 6;
 
@@ -212,8 +258,11 @@ KNotifyWidget::KNotifyWidget( QWidget *parent, const char *name,
     header->setLabel( COL_MESSAGE, pmessage, QString::null, w );
     header->setLabel( COL_LOGFILE, plogfile, QString::null, w );
     header->setLabel( COL_SOUND,   psound,   QString::null, w );
+    header->setLabel( COL_TASKBAR, ptaskbar, QString::null, w );
 
-    m_playButton->setPixmap( SmallIcon( "1rightarrow" ) );
+    d->toolTip = new KNotifyToolTip( header );
+
+    m_playButton->setPixmap( SmallIcon( "player_play" ) );
     connect( m_playButton, SIGNAL( clicked() ), SLOT( playSound() ));
 
     connect( m_listview, SIGNAL( currentChanged( QListViewItem * ) ),
@@ -233,6 +282,8 @@ KNotifyWidget::KNotifyWidget( QWidget *parent, const char *name,
              SLOT( messageBoxChanged() ) );
     connect( m_stderr, SIGNAL( toggled( bool )),
              SLOT( stderrToggled( bool ) ) );
+    connect( m_taskbar, SIGNAL( toggled( bool )),
+             SLOT( taskbarToggled( bool ) ) );
 
     connect( m_soundPath, SIGNAL( textChanged( const QString& )),
              SLOT( soundFileChanged( const QString& )));
@@ -254,13 +305,24 @@ KNotifyWidget::KNotifyWidget( QWidget *parent, const char *name,
     connect( m_buttonEnable, SIGNAL( clicked() ), SLOT( enableAll() ));
     connect( m_buttonDisable, SIGNAL( clicked() ), SLOT( enableAll() ));
 
+    QString whatsThis = i18n("<qt>You may use the following macros<br>"
+        "in the commandline:<br>"
+        "<b>%e</b>: for the event name,<br>"
+        "<b>%a</b>: for the name of the application that sent the event,<br>"
+        "<b>%s</b>: for the notification message,<br>"
+        "<b>%w</b>: for the numeric window ID where the event originated,<br>"
+        "<b>%i</b>: for the numeric event ID.");
+    QWhatsThis::add( m_execute, whatsThis );
+    QWhatsThis::add( m_executePath, whatsThis );
+
     showAdvanced( false );
-    
+
     slotEventChanged( 0L ); // disable widgets by default
 }
 
 KNotifyWidget::~KNotifyWidget()
 {
+    delete d->toolTip;
     delete d;
 }
 
@@ -283,7 +345,7 @@ void KNotifyWidget::showAdvanced( bool show )
         m_messageBox->show();
         m_passivePopup->show();
         m_stderr->show();
-        m_controlsBox->show();
+        m_taskbar->show();
 
         m_actionsBoxLayout->setSpacing( KDialog::spacingHint() );
     }
@@ -299,7 +361,7 @@ void KNotifyWidget::showAdvanced( bool show )
         m_messageBox->hide();
         m_passivePopup->hide();
         m_stderr->hide();
-        m_controlsBox->hide();
+        m_taskbar->hide();
 
         m_actionsBoxLayout->setSpacing( 0 );
     }
@@ -398,8 +460,20 @@ void KNotifyWidget::updateWidgets( ListViewItem *item )
 
     // other settings
     m_messageBox->setChecked(event.presentation & (KNotifyClient::Messagebox | KNotifyClient::PassivePopup));
+    enable = (event.dontShow & KNotifyClient::Messagebox) == 0;
+    m_messageBox->setEnabled( enable );
+
     m_passivePopup->setChecked(event.presentation & KNotifyClient::PassivePopup);
+    enable = (event.dontShow & KNotifyClient::PassivePopup) == 0;
+    m_passivePopup->setEnabled( enable );
+
     m_stderr->setChecked( event.presentation & KNotifyClient::Stderr );
+    enable = (event.dontShow & KNotifyClient::Stderr) == 0;
+    m_stderr->setEnabled( enable );
+
+    m_taskbar->setChecked(event.presentation & KNotifyClient::Taskbar);
+    enable = (event.dontShow & KNotifyClient::Taskbar) == 0;
+    m_taskbar->setEnabled( enable );
 
     updatePixmaps( item );
 
@@ -415,7 +489,7 @@ void KNotifyWidget::updatePixmaps( ListViewItem *item )
                 !event.commandline.isEmpty();
     item->setPixmap( COL_EXECUTE, doIt ? d->pixmaps[COL_EXECUTE] : emptyPix );
 
-    doIt = (event.presentation & KNotifyClient::Sound) && 
+    doIt = (event.presentation & KNotifyClient::Sound) &&
            !event.soundfile.isEmpty();
     item->setPixmap( COL_SOUND, doIt ? d->pixmaps[COL_SOUND] : emptyPix );
 
@@ -431,6 +505,9 @@ void KNotifyWidget::updatePixmaps( ListViewItem *item )
     item->setPixmap( COL_STDERR,
                      (event.presentation & KNotifyClient::Stderr) ?
                      d->pixmaps[COL_STDERR] : emptyPix );
+    item->setPixmap( COL_TASKBAR,
+                     (event.presentation & KNotifyClient::Taskbar) ?
+                     d->pixmaps[COL_TASKBAR] : emptyPix );
 }
 
 void KNotifyWidget::addVisibleApp( Application *app )
@@ -472,6 +549,8 @@ void KNotifyWidget::addToView( const EventList& events )
             item->setPixmap( COL_MESSAGE, d->pixmaps[COL_MESSAGE] );
         if ( event->presentation & KNotifyClient::Stderr )
             item->setPixmap( COL_STDERR, d->pixmaps[COL_STDERR] );
+        if ( event->presentation & KNotifyClient::Taskbar )
+            item->setPixmap( COL_TASKBAR, d->pixmaps[COL_TASKBAR] );
     }
 }
 
@@ -570,6 +649,15 @@ void KNotifyWidget::stderrToggled( bool on )
     widgetChanged( item, KNotifyClient::Stderr, on );
 }
 
+void KNotifyWidget::taskbarToggled( bool on )
+{
+    QListViewItem *item = m_listview->currentItem();
+    if ( !item )
+        return;
+    item->setPixmap( COL_TASKBAR, on ? d->pixmaps[COL_TASKBAR] : QPixmap() );
+    widgetChanged( item, KNotifyClient::Taskbar, on );
+}
+
 void KNotifyWidget::soundFileChanged( const QString& text )
 {
     if ( signalsBlocked() )
@@ -600,7 +688,7 @@ void KNotifyWidget::logfileChanged( const QString& text )
     currentEvent()->logfile = text;
     bool ok = !text.isEmpty() && m_logToFile->isChecked();
     item->setPixmap( COL_LOGFILE, ok ? d->pixmaps[COL_LOGFILE] : QPixmap() );
-    
+
     emit changed( true );
 }
 
@@ -620,7 +708,7 @@ void KNotifyWidget::commandlineChanged( const QString& text )
     emit changed( true );
 }
 
-void KNotifyWidget::slotItemClicked( QListViewItem *item, const QPoint&, 
+void KNotifyWidget::slotItemClicked( QListViewItem *item, const QPoint&,
                                      int col )
 {
     if ( !item || !item->isSelected() )
@@ -629,9 +717,9 @@ void KNotifyWidget::slotItemClicked( QListViewItem *item, const QPoint&,
     Event *event = currentEvent();
     if ( !event )
         return; // very unlikely, but safety first
-    
+
     bool doShowAdvanced = false;
-    
+
     switch( col )
     {
         case COL_EXECUTE:
@@ -642,6 +730,9 @@ void KNotifyWidget::slotItemClicked( QListViewItem *item, const QPoint&,
         case COL_STDERR:
             m_stderr->toggle();
             break;
+        case COL_TASKBAR:
+            m_taskbar->toggle();
+            break;
         case COL_MESSAGE:
             m_passivePopup->setChecked( true ); // default to passive popups
             m_messageBox->toggle();
@@ -657,7 +748,7 @@ void KNotifyWidget::slotItemClicked( QListViewItem *item, const QPoint&,
         default: // do nothing
             break;
     }
-    
+
     if ( doShowAdvanced && !m_logToFile->isVisible() )
     {
         showAdvanced( true );
@@ -688,8 +779,8 @@ void KNotifyWidget::resetDefaults( bool ask )
         if ( KMessageBox::warningContinueCancel(this,
                                    i18n("This will cause the notifications "
                                         "to be reset to their defaults!"),
-                                                i18n("Are you sure?"),
-                                                i18n("Continue"))
+                                                i18n("Are You Sure?"),
+                                                KStdGuiItem::cont())
              != KMessageBox::Continue)
             return;
     }
@@ -764,7 +855,7 @@ void KNotifyWidget::openSoundDialog( KURLRequester *requester )
     KFileDialog *fileDialog = requester->fileDialog();
     fileDialog->setCaption( i18n("Select Sound File") );
     QStringList filters;
-    filters << "audio/x-wav" << "audio/x-mp3" << "application/x-ogg"
+    filters << "audio/x-wav" << "audio/x-mp3" << "application/ogg"
             << "audio/x-adpcm";
     fileDialog->setMimeFilter( filters );
 
@@ -926,9 +1017,9 @@ void Application::save()
     while ( (e = it.current()) ) {
         config->setGroup( e->configGroup );
         config->writeEntry( "presentation", e->presentation );
-        config->writeEntry( "soundfile", e->soundfile );
-        config->writeEntry( "logfile", e->logfile );
-        config->writeEntry( "commandline", e->commandline );
+        config->writePathEntry( "soundfile", e->soundfile );
+        config->writePathEntry( "logfile", e->logfile );
+        config->writePathEntry( "commandline", e->commandline );
 
         ++it;
     }
@@ -963,6 +1054,7 @@ void Application::reloadEvents( bool revertToDefaults )
             e = new Event( this );
             e->name = kc->readEntry( name );
             e->description = kc->readEntry( comment );
+            e->dontShow = kc->readNumEntry("nopresentation", 0 );
             e->configGroup = *it;
 
             if ( e->name.isEmpty() || e->description.isEmpty() )
@@ -970,13 +1062,13 @@ void Application::reloadEvents( bool revertToDefaults )
 
             else { // load the event
                 // default to passive popups over plain messageboxes
-                int default_rep = kc->readNumEntry("default_presentation", 
+                int default_rep = kc->readNumEntry("default_presentation",
                                                    0 | KNotifyClient::PassivePopup);
                 QString default_logfile = kc->readPathEntry("default_logfile");
                 QString default_soundfile = kc->readPathEntry("default_sound");
                 QString default_commandline = kc->readPathEntry("default_commandline");
+
                 config->setGroup(*it);
-                e->dontShow = config->readNumEntry("nopresentation", 0 );
 
                 if ( revertToDefaults )
                 {
@@ -1046,6 +1138,9 @@ int ListViewItem::compare ( QListViewItem * i, int col, bool ascending ) const
         case COL_STDERR:
             action = KNotifyClient::Stderr;
             break;
+        case COL_TASKBAR:
+            action = KNotifyClient::Taskbar;
+            break;
     }
 
     if ( (myPres & action) == (otherPres & action) )
