@@ -20,8 +20,6 @@
     Boston, MA 02111-1307, USA.
 */
 
-// $Id: kopenwith.cpp,v 1.66.2.5 2003/06/29 21:43:01 deller Exp $
-
 #include <qfile.h>
 #include <qdir.h>
 #include <qdialog.h>
@@ -34,6 +32,7 @@
 #include <qcheckbox.h>
 #include <qtooltip.h>
 #include <qstyle.h>
+#include <qwhatsthis.h>
 
 #include <kapplication.h>
 #include <kbuttonbox.h>
@@ -72,7 +71,7 @@ template class QPtrList<QString>;
 // ----------------------------------------------------------------------
 
 KAppTreeListItem::KAppTreeListItem( KListView* parent, const QString & name,
-                                    const QPixmap& pixmap, bool parse, bool dir, QString p, QString c )
+                                    const QPixmap& pixmap, bool parse, bool dir, const QString &p, const QString &c )
     : QListViewItem( parent, name )
 {
     init(pixmap, parse, dir, p, c);
@@ -82,7 +81,7 @@ KAppTreeListItem::KAppTreeListItem( KListView* parent, const QString & name,
 // ----------------------------------------------------------------------
 
 KAppTreeListItem::KAppTreeListItem( QListViewItem* parent, const QString & name,
-                                    const QPixmap& pixmap, bool parse, bool dir, QString p, QString c )
+                                    const QPixmap& pixmap, bool parse, bool dir, const QString &p, const QString &c )
     : QListViewItem( parent, name )
 {
     init(pixmap, parse, dir, p, c);
@@ -91,7 +90,7 @@ KAppTreeListItem::KAppTreeListItem( QListViewItem* parent, const QString & name,
 
 // ----------------------------------------------------------------------
 
-void KAppTreeListItem::init(const QPixmap& pixmap, bool parse, bool dir, QString _path, QString _exec)
+void KAppTreeListItem::init(const QPixmap& pixmap, bool parse, bool dir, const QString &_path, const QString &_exec)
 {
     setPixmap(0, pixmap);
     parsed = parse;
@@ -162,7 +161,7 @@ static QPixmap appIcon(const QString &iconName)
 {
     QPixmap normal = KGlobal::iconLoader()->loadIcon(iconName, KIcon::Small, 0, KIcon::DefaultState, 0L, true);
     // make sure they are not larger than 20x20
-    if (normal.width() > 20 || normal.height() > 20) 
+    if (normal.width() > 20 || normal.height() > 20)
     {
        QImage tmp = normal.convertToImage();
        tmp = tmp.smoothScale(20, 20);
@@ -171,7 +170,7 @@ static QPixmap appIcon(const QString &iconName)
     return normal;
 }
 
-void KApplicationTree::addDesktopGroup( QString relPath, KAppTreeListItem *item)
+void KApplicationTree::addDesktopGroup( const QString &relPath, KAppTreeListItem *item)
 {
    KServiceGroup::Ptr root = KServiceGroup::group(relPath);
    KServiceGroup::List list = root->entries();
@@ -279,6 +278,14 @@ void KApplicationTree::resizeEvent( QResizeEvent * e)
  * KOpenWithDlg
  *
  ***************************************************************/
+class KOpenWithDlgPrivate
+{
+public:
+    KOpenWithDlgPrivate() : saveNewApps(false) { };
+    QPushButton* ok;
+    bool saveNewApps;
+    KService::Ptr curService;
+};
 
 KOpenWithDlg::KOpenWithDlg( const KURL::List& _urls, QWidget* parent )
              :QDialog( parent, 0L, true )
@@ -314,17 +321,14 @@ KOpenWithDlg::KOpenWithDlg( const QString &serviceType, const QString& value,
                             QWidget *parent)
              :QDialog( parent, 0L, true )
 {
-  setCaption(i18n("Choose Application for %1").arg(serviceType));
+    setCaption(i18n("Choose Application for %1").arg(serviceType));
   QString text = i18n("<qt>Select the program for the file type: <b>%1</b>. "
                       "If the program is not listed, enter the name or click "
                       "the browse button.</qt>").arg(serviceType);
   qServiceType = serviceType;
   init( text, value );
   if (remember)
-  {
-      remember->setChecked( true );
       remember->hide();
-  }
 }
 
 KOpenWithDlg::KOpenWithDlg( QWidget *parent)
@@ -352,10 +356,12 @@ void KOpenWithDlg::setServiceType( const KURL::List& _urls )
 
 void KOpenWithDlg::init( const QString& _text, const QString& _value )
 {
+  d = new KOpenWithDlgPrivate;
   bool bReadOnly = kapp && !kapp->authorize("shell_access");
   m_terminaldirty = false;
   m_pTree = 0L;
   m_pService = 0L;
+  d->curService = 0L;
 
   QBoxLayout *topLayout = new QVBoxLayout( this, KDialog::marginHint(),
           KDialog::spacingHint() );
@@ -367,7 +373,7 @@ void KOpenWithDlg::init( const QString& _text, const QString& _value )
   QToolButton *clearButton = new QToolButton( this );
   clearButton->setIconSet( BarIcon( "locationbar_erase" ) );
   clearButton->setFixedSize( clearButton->sizeHint() );
-  connect( clearButton, SIGNAL( pressed() ), SLOT( slotClear() ) );
+  connect( clearButton, SIGNAL( clicked() ), SLOT( slotClear() ) );
   QToolTip::add( clearButton, i18n( "Clear input field" ) );
 
   hbox->addWidget( clearButton );
@@ -397,12 +403,25 @@ void KOpenWithDlg::init( const QString& _text, const QString& _value )
   }
 
   edit->setURL( _value );
+  QWhatsThis::add(edit,i18n(
+    "Following the command, you can have several place holders which will be replaced "
+    "with the actual values when the actual program is run:\n"
+    "%f - a single file name\n"
+    "%F - a list of files; use for applications that can open several local files at once\n"
+    "%u - a single URL\n"
+    "%U - a list of URLs\n"
+    "%d - the directory of the file to open\n"
+    "%D - a list of directories\n"
+    "%i - the icon\n"
+    "%m - the mini-icon\n"
+    "%c - the comment"));
 
   hbox->addWidget(edit);
 
   if ( edit->comboBox() ) {
     KURLCompletion *comp = new KURLCompletion( KURLCompletion::ExeCompletion );
     edit->comboBox()->setCompletionObject( comp );
+    edit->comboBox()->setAutoDeleteCompletionObject( true );
   }
 
   connect ( edit, SIGNAL(returnPressed()), SLOT(slotOK()) );
@@ -425,6 +444,25 @@ void KOpenWithDlg::init( const QString& _text, const QString& _value )
 
   topLayout->addWidget(terminal);
 
+  QBoxLayout* nocloseonexitLayout = new QHBoxLayout( 0, 0, KDialog::spacingHint() );
+  QSpacerItem* spacer = new QSpacerItem( 20, 0, QSizePolicy::Fixed, QSizePolicy::Minimum );
+  nocloseonexitLayout->addItem( spacer );
+
+  nocloseonexit = new QCheckBox( i18n("&Do not close when command exits"), this );
+  nocloseonexit->setChecked( false );
+  nocloseonexit->setDisabled( true );
+
+  // check to see if we use konsole if not disable the nocloseonexit
+  // because we don't know how to do this on other terminal applications
+  KConfigGroup confGroup( KGlobal::config(), QString::fromLatin1("General") );
+  QString preferredTerminal = confGroup.readPathEntry(QString::fromLatin1("TerminalApplication"), QString::fromLatin1("konsole"));
+
+  if (bReadOnly || preferredTerminal != "konsole")
+     nocloseonexit->hide();
+
+  nocloseonexitLayout->addWidget( nocloseonexit );
+  topLayout->addLayout( nocloseonexitLayout );
+
   if (!qServiceType.isNull())
   {
     remember = new QCheckBox(i18n("&Remember application association for this type of file"), this);
@@ -438,12 +476,16 @@ void KOpenWithDlg::init( const QString& _text, const QString& _value )
   KButtonBox* b = new KButtonBox( this );
   b->addStretch( 2 );
 
-  QPushButton* ok = b->addButton(  i18n ( "&OK" ) );
-  ok->setDefault( true );
-  connect( ok, SIGNAL( clicked() ), SLOT( slotOK() ) );
+  d->ok = b->addButton(  i18n ( "&OK" ) );
+  d->ok->setDefault( true );
+  if (KGlobalSettings::showIconsOnPushButtons())
+    d->ok->setIconSet( SmallIconSet("button_ok") );
+  connect(  d->ok, SIGNAL( clicked() ), SLOT( slotOK() ) );
 
   QPushButton* cancel = b->addButton(  i18n( "&Cancel" ) );
-  connect( cancel, SIGNAL( clicked() ), SLOT( reject() ) );
+  if (KGlobalSettings::showIconsOnPushButtons())
+    cancel->setIconSet( SmallIconSet("button_cancel") );
+  connect(  cancel, SIGNAL( clicked() ), SLOT( reject() ) );
 
   b->layout();
   topLayout->addWidget( b );
@@ -453,6 +495,7 @@ void KOpenWithDlg::init( const QString& _text, const QString& _value )
   // Probably due to the resizeEvent handler using width().
   //resize( minimumWidth(), sizeHint().height() );
   edit->setFocus();
+  slotTextChanged();
 }
 
 
@@ -460,6 +503,8 @@ void KOpenWithDlg::init( const QString& _text, const QString& _value )
 
 KOpenWithDlg::~KOpenWithDlg()
 {
+    delete d;
+    d = 0;
 }
 
 // ----------------------------------------------------------------------
@@ -476,9 +521,9 @@ void KOpenWithDlg::slotClear()
 void KOpenWithDlg::slotSelected( const QString& /*_name*/, const QString& _exec )
 {
     kdDebug(250)<<"KOpenWithDlg::slotSelected"<<endl;
-    KService::Ptr pService = m_pService;
+    KService::Ptr pService = d->curService;
     edit->setURL( _exec ); // calls slotTextChanged :(
-    m_pService = pService;
+    d->curService = pService;
 }
 
 
@@ -488,11 +533,13 @@ void KOpenWithDlg::slotHighlighted( const QString& _name, const QString& )
 {
     kdDebug(250)<<"KOpenWithDlg::slotHighlighted"<<endl;
     qName = _name;
-    m_pService = KService::serviceByName( qName );
+    d->curService = KService::serviceByName( qName );
     if (!m_terminaldirty)
     {
         // ### indicate that default value was restored
-        terminal->setChecked(m_pService->terminal());
+        terminal->setChecked(d->curService->terminal());
+        QString terminalOptions = d->curService->terminalOptions();
+        nocloseonexit->setChecked( (terminalOptions.contains( "--noclose" ) > 0) );
         m_terminaldirty = false; // slotTerminalToggled changed it
     }
 }
@@ -503,7 +550,8 @@ void KOpenWithDlg::slotTextChanged()
 {
     kdDebug(250)<<"KOpenWithDlg::slotTextChanged"<<endl;
     // Forget about the service
-    m_pService = 0L;
+    d->curService = 0L;
+    d->ok->setEnabled( !edit->url().isEmpty());
 }
 
 // ----------------------------------------------------------------------
@@ -512,6 +560,7 @@ void KOpenWithDlg::slotTerminalToggled(bool)
 {
     // ### indicate that default value was overridden
     m_terminaldirty = true;
+    nocloseonexit->setDisabled( ! terminal->isChecked() );
 }
 
 // ----------------------------------------------------------------------
@@ -522,13 +571,19 @@ void KOpenWithDlg::slotDbClick()
    slotOK();
 }
 
+void KOpenWithDlg::setSaveNewApplications(bool b)
+{
+  d->saveNewApps = b;
+}
+
 void KOpenWithDlg::slotOK()
 {
   QString fullExec(edit->url());
 
   QString serviceName;
-  QString pathName;
   QString initialServiceName;
+  QString preferredTerminal;
+  m_pService = d->curService;
   if (!m_pService) {
     // No service selected - check the command line
 
@@ -540,24 +595,31 @@ void KOpenWithDlg::slotOK()
       return;
     }
     initialServiceName = serviceName;
+    kdDebug(250) << "initialServiceName=" << initialServiceName << endl;
     int i = 1; // We have app, app-2, app-3... Looks better for the user.
-    // Check if there's already a service by that name, with the same Exec line
     bool ok = false;
+    // Check if there's already a service by that name, with the same Exec line
     do {
+        kdDebug(250) << "looking for service " << serviceName << endl;
         KService::Ptr serv = KService::serviceByDesktopName( serviceName );
         ok = !serv; // ok if no such service yet
         // also ok if we find the exact same service (well, "kwrite" == "kwrite %U"
-        if ( serv &&
-             serv->type() == "Application" && ( // only apps
-                 serv->exec() == fullExec ||
-                 serv->exec() == fullExec + " %u" ||
-                 serv->exec() == fullExec + " %U" ||
-                 serv->exec() == fullExec + " %f" ||
-                 serv->exec() == fullExec + " %F"
-                 ) )
+        if ( serv && serv->type() == "Application")
         {
-            ok = true;
-            m_pService = serv;
+            QString exec = serv->exec();
+            exec.replace("%u", "", false);
+            exec.replace("%f", "", false);
+            exec.replace("-caption %c", "");
+            exec.replace("-caption \"%c\"", "");
+            exec.replace("%i", "");
+            exec.replace("%m", "");
+            exec = exec.simplifyWhiteSpace();
+            if (exec == fullExec)
+            {
+                ok = true;
+                m_pService = serv;
+                kdDebug(250) << k_funcinfo << "OK, found identical service: " << serv->desktopEntryPath() << endl;
+            }
         }
         if (!ok) // service was found, but it was different -> keep looking
         {
@@ -566,26 +628,22 @@ void KOpenWithDlg::slotOK()
         }
     }
     while (!ok);
-    if ( !m_pService )
-    {
-        // Let's hide it to avoid cluttering the K menu.
-        pathName = ".hidden/";
-        pathName += serviceName;
-    }
   }
   if ( m_pService )
   {
     // Existing service selected
     serviceName = m_pService->name();
     initialServiceName = serviceName;
-    pathName = m_pService->desktopEntryPath();
   }
 
-  if (terminal->isChecked()) {
-    KSimpleConfig conf(QString::fromLatin1("konquerorrc"), true);
-    conf.setGroup(QString::fromLatin1("Misc Defaults"));
-    m_command = conf.readPathEntry(QString::fromLatin1("Terminal"), QString::fromLatin1("konsole"));
-
+  if (terminal->isChecked())
+  {
+    KConfigGroup confGroup( KGlobal::config(), QString::fromLatin1("General") );
+    preferredTerminal = confGroup.readPathEntry(QString::fromLatin1("TerminalApplication"), QString::fromLatin1("konsole"));
+    m_command = preferredTerminal;
+    // only add --noclose when we are sure it is konsole we're using
+    if (preferredTerminal == "konsole" && nocloseonexit->isChecked())
+      m_command += QString::fromLatin1(" --noclose");
     m_command += QString::fromLatin1(" -e ");
     m_command += edit->url();
     kdDebug(250) << "Setting m_command to " << m_command << endl;
@@ -593,7 +651,25 @@ void KOpenWithDlg::slotOK()
   if ( m_pService && terminal->isChecked() != m_pService->terminal() )
       m_pService = 0L; // It's not exactly this service we're running
 
-  if ( m_pService && ( !remember || !remember->isChecked() ) ) {
+  bool bRemember = remember && remember->isChecked();
+
+  if ( !bRemember && m_pService)
+  {
+    accept();
+    return;
+  }
+
+  if (!bRemember && !d->saveNewApps)
+  {
+    // Create temp service
+    m_pService = new KService(initialServiceName, fullExec, QString::null);
+    if (terminal->isChecked())
+    {
+      m_pService->setTerminal(true);
+      // only add --noclose when we are sure it is konsole we're using
+      if (preferredTerminal == "konsole" && nocloseonexit->isChecked())
+         m_pService->setTerminalOptions("--noclose");
+    }
     accept();
     return;
   }
@@ -602,10 +678,21 @@ void KOpenWithDlg::slotOK()
   // wanted.  The other possibility is that they have asked for the
   // association to be remembered.  Create/update service.
 
-  kdDebug(250) << "service pathName=" << pathName << endl;
-  if (pathName.right(8) != QString::fromLatin1(".desktop"))
-    pathName += QString::fromLatin1(".desktop");
-  QString path(locateLocal("apps", pathName));
+  QString newPath;
+  QString oldPath;
+  QString menuId;
+  if (m_pService)
+  {
+    oldPath = m_pService->desktopEntryPath();
+    newPath = m_pService->locateLocal();
+    menuId = m_pService->menuId();
+    kdDebug(250) << "Updating exitsing service " << m_pService->desktopEntryPath() << " ( " << newPath << " ) " << endl;
+  }
+  else
+  {
+    newPath = KService::newServicePath(false /* hidden */, serviceName, &menuId);
+    kdDebug(250) << "Creating new service " << serviceName << " ( " << newPath << " ) " << endl;
+  }
 
   int maxPreference = 1;
   if (!qServiceType.isEmpty())
@@ -615,55 +702,56 @@ void KOpenWithDlg::slotOK()
       maxPreference = offerList.first().preference();
   }
 
-  KDesktopFile desktop(path);
-  desktop.writeEntry(QString::fromLatin1("Type"), QString::fromLatin1("Application"));
-  desktop.writeEntry(QString::fromLatin1("Name"), initialServiceName);
-  desktop.writeEntry(QString::fromLatin1("Exec"), fullExec);
-  desktop.writeEntry(QString::fromLatin1("InitialPreference"), maxPreference + 1);
-  if (remember)
-    if (remember->isChecked()) {
-      QStringList mimeList;
-      KDesktopFile oldDesktop(locate("apps", pathName), true);
-      mimeList = oldDesktop.readListEntry(QString::fromLatin1("MimeType"), ';');
-      if (!qServiceType.isEmpty() && !mimeList.contains(qServiceType))
-        mimeList.append(qServiceType);
-      desktop.writeEntry(QString::fromLatin1("MimeType"), mimeList, ';');
-      if (terminal->isChecked())
-        desktop.writeEntry(QString::fromLatin1("Terminal"), true);
-      else
-        desktop.writeEntry(QString::fromLatin1("Terminal"), false);
+  KDesktopFile *desktop = 0;
+  if (!oldPath.isEmpty() && (oldPath != newPath))
+  {
+     KDesktopFile orig(oldPath, true);
+     desktop = orig.copyTo(newPath);
+  }
+  else
+  {
+     desktop = new KDesktopFile(newPath);
+  }
+  desktop->writeEntry("Type", QString::fromLatin1("Application"));
+  desktop->writeEntry("Name", initialServiceName);
+  desktop->writePathEntry("Exec", fullExec);
+  if (terminal->isChecked())
+  {
+    desktop->writeEntry("Terminal", true);
+    // only add --noclose when we are sure it is konsole we're using
+    if (preferredTerminal == "konsole" && nocloseonexit->isChecked())
+      desktop->writeEntry("TerminalOptions", "--noclose");
+  }
+  else
+  {
+    desktop->writeEntry("Terminal", false);
+  }
+  desktop->writeEntry("InitialPreference", maxPreference + 1);
 
-      if ( !qServiceType.isEmpty() )
-      {
-        // Also make sure the "auto embed" setting for this mimetype is off
-        KDesktopFile mimeDesktop( locateLocal( "mime", qServiceType + ".desktop" ) );
-        mimeDesktop.writeEntry( QString::fromLatin1( "X-KDE-AutoEmbed" ), false );
-        mimeDesktop.sync();
-      }
+
+  if (bRemember)
+  {
+    QStringList mimeList = desktop->readListEntry("MimeType", ';');
+    if (!qServiceType.isEmpty() && !mimeList.contains(qServiceType))
+      mimeList.append(qServiceType);
+    desktop->writeEntry("MimeType", mimeList, ';');
+
+    if ( !qServiceType.isEmpty() )
+    {
+      // Also make sure the "auto embed" setting for this mimetype is off
+      KDesktopFile mimeDesktop( locateLocal( "mime", qServiceType + ".desktop" ) );
+      mimeDesktop.writeEntry( "X-KDE-AutoEmbed", false );
+      mimeDesktop.sync();
     }
-
+  }
 
   // write it all out to the file
-  desktop.sync();
+  desktop->sync();
+  delete desktop;
 
-  QApplication::setOverrideCursor( waitCursor );
+  KService::rebuildKSycoca(this);
 
-  // rebuild the database
-  QStringList args;
-  args.append("--incremental");
-  KApplication::kdeinitExecWait( "kbuildsycoca", args );
-
-  // get the new service pointer
-  kdDebug(250) << "kbuildsycoca finished, looking for service " << pathName << endl;
-  // We need to read in the new database. It seems the databaseChanged()
-  // signal hasn't been processed in this process yet, since we haven't been
-  // to the event loop yet.
-  QStringList lst;
-  lst << QString::fromLatin1("apps");
-  KSycoca::self()->notifyDatabaseChanged( lst );
-
-  m_pService = KService::serviceByDesktopPath( pathName );
-  QApplication::restoreOverrideCursor();
+  m_pService = KService::serviceByMenuId( menuId );
 
   Q_ASSERT( m_pService );
 
@@ -678,6 +766,19 @@ QString KOpenWithDlg::text() const
         return edit->url();
 }
 
+void KOpenWithDlg::hideNoCloseOnExit()
+{
+    // uncheck the checkbox because the value could be used when "Run in Terminal" is selected
+    nocloseonexit->setChecked( false );
+    nocloseonexit->hide();
+}
+
+void KOpenWithDlg::hideRunInTerminal()
+{
+    terminal->hide();
+    hideNoCloseOnExit();
+}
+
 void KOpenWithDlg::accept()
 {
     KHistoryCombo *combo = static_cast<KHistoryCombo*>( edit->comboBox() );
