@@ -31,6 +31,7 @@
 #include <qstring.h>
 #include <qptrlist.h>
 
+#include <kdemacros.h>
 #include <kurl.h>
 #include <kfileitem.h>
 #include <kdialogbase.h>
@@ -118,7 +119,7 @@ public:
    * @param autoShow tells the dialog whethr it should show itself automatically.  */
   KPropertiesDialog( const KURL& _url, mode_t _mode,
                      QWidget* parent = 0L, const char* name = 0L,
-                     bool modal = false, bool autoShow = true);
+                     bool modal = false, bool autoShow = true) KDE_DEPRECATED;
 #endif
 
   /**
@@ -146,7 +147,7 @@ public:
    * is not known yet), based on a template. Special constructor for
    * "File / New" in file-manager applications.
    *
-   * @param _templUrl template used for reading only
+   * @param _tempUrl template used for reading only
    * @param _currentDir directory where the file will be written to
    * @param _defaultName something to put in the name field,
    * like mimetype.desktop
@@ -216,10 +217,10 @@ public:
 
   /**
    * @return a pointer to the dialog
-   * @deprecated
+   * @deprecated the class directly inherits from KDialogBase, so use this instead
    */
-  KDialogBase* dialog() { return this; }
-  const KDialogBase* dialog() const { return this; }
+  KDE_DEPRECATED KDialogBase* dialog() { return this; }
+  KDE_DEPRECATED const KDialogBase* dialog() const { return this; }
 
   /**
    * If we are building this dialog from a template,
@@ -239,7 +240,7 @@ public:
    * Updates the item url (either called by rename or because
    * a global apps/mimelnk desktop file is being saved)
    * Can only be called if the dialog applies to a single file/URL.
-   * @param _name new URL
+   * @param _newUrl the new URL
    */
   void updateUrl( const KURL& _newUrl );
 
@@ -260,6 +261,13 @@ public:
    */
   void showFileSharingPage();
 
+   /**
+    * Call this to make the filename lineedit readonly, to prevent the user
+    * from renaming the file.
+    * @since 3.2
+    */
+  void setFileNameReadOnly( bool ro );
+
 public slots:
   /**
    * Called when the user presses 'Ok'.
@@ -275,6 +283,13 @@ signals:
   void applied();
   void canceled();
 
+  /**
+   * Emitted before changes to @p oldUrl are saved as @p newUrl.
+   * The receiver may change @p newUrl to point to an alternative
+   * save location.
+   */
+  void saveAs(const KURL &oldUrl, KURL &newUrl);
+
 private:
 
   /**
@@ -309,7 +324,7 @@ private:
   QPtrList<KPropsDlgPlugin> m_pageList;
 
 private slots:
-  void slotStatResult( KIO::Job * job );
+  void slotStatResult( KIO::Job * ); // No longer used
 protected:
   virtual void virtual_hook( int id, void* data );
 private:
@@ -365,7 +380,7 @@ public slots:
 signals:
   /**
    * Emit this signal when the user changed anything in the plugin's tabs.
-   * The hosting PropertiesDialog will call @ref applyChanges only if the
+   * The hosting PropertiesDialog will call applyChanges only if the
    * PropsPlugin has emitted this signal before.
    */
   void changed();
@@ -416,21 +431,29 @@ public:
    */
   void postApplyChanges();
 
+  void setFileNameReadOnly( bool ro );
+
 protected slots:
+  void slotEditFileType();
   void slotCopyFinished( KIO::Job * );
   void slotFileRenamed( KIO::Job *, const KURL &, const KURL & );
+  void slotDirSizeUpdate();
   void slotDirSizeFinished( KIO::Job * );
-  void slotFoundMountPoint( const QString& mp, unsigned long kBSize, unsigned long kBUsed, unsigned long kBAvail );
+  void slotFoundMountPoint( const QString& mp, unsigned long kBSize,
+			    unsigned long kBUsed, unsigned long kBAvail );
   void slotSizeStop();
   void slotSizeDetermine();
 
 private slots:
   // workaround for compiler bug
-void slotFoundMountPoint( const unsigned long& kBSize, const unsigned long& kBUsed, const unsigned long& kBAvail, const QString& mp );
- void nameFileChanged(const QString &text );
-private:
+  void slotFoundMountPoint( const unsigned long& kBSize, const unsigned long&
+			  kBUsed, const unsigned long& kBAvail, const QString& mp );
+  void nameFileChanged(const QString &text );
+  void slotIconChanged();
 
+private:
   void determineRelativePath( const QString & path );
+  void applyIconChanges();
 
   QWidget *iconArea;
   QWidget *nameArea;
@@ -461,6 +484,19 @@ class KFilePermissionsPropsPlugin : public KPropsDlgPlugin
 {
   Q_OBJECT
 public:
+  enum PermissionsMode {
+    PermissionsOnlyFiles = 0,
+    PermissionsOnlyDirs = 1,
+    PermissionsOnlyLinks = 2,
+    PermissionsMixed = 3
+  };
+
+  enum PermissionsTarget {
+    PermissionsOwner  = 0,
+    PermissionsGroup  = 1,
+    PermissionsOthers = 2
+  };
+
   /**
    * Constructor
    */
@@ -477,12 +513,28 @@ public:
 private slots:
 
   void slotChmodResult( KIO::Job * );
-  void slotRecursiveClicked();
+  void slotShowAdvancedPermissions();
 
 private:
+  void setComboContent(QComboBox *combo, PermissionsTarget target,
+		       mode_t permissions, mode_t partial);
+  bool isIrregular(mode_t permissions, bool isDir, bool isLink);
+  void enableAccessControls(bool enable);
+  void updateAccessControls();
+  void getPermissionMasks(mode_t &andFilePermissions,
+			  mode_t &andDirPermissions,
+			  mode_t &orFilePermissions,
+			  mode_t &orDirPermissions);
+
+  static const mode_t permissionsMasks[3];
+  static const mode_t standardPermissions[4];
+  static const char *permissionsTexts[4][4];
+
+  // unused, for binary compatibility!
   QCheckBox *permBox[3][4];
 
   QComboBox *grpCombo;
+
   KLineEdit *usrEdit, *grpEdit;
 
   /**
@@ -498,75 +550,20 @@ private:
    */
   QString strOwner;
 
-  /**
-   * Changeable Permissions
-   */
+  // unused, for compatibility
   static mode_t fperm[3][4];
 
   class KFilePermissionsPropsPluginPrivate;
   KFilePermissionsPropsPluginPrivate *d;
 };
 
-/**
- * Used to edit the files containing
- * [Desktop Entry]
- * Type=Application
- *
- * Such files are used to represent a program in kpanel and kfm.
- * @internal
- */
-class KExecPropsPlugin : public KPropsDlgPlugin
-{
-  Q_OBJECT
-public:
-  /**
-   * Constructor
-   */
-  KExecPropsPlugin( KPropertiesDialog *_props );
-  virtual ~KExecPropsPlugin();
-
-  virtual void applyChanges();
-
-  static bool supports( KFileItemList _items );
-
-public slots:
-  void slotBrowseExec();
-
-private slots:
-  void enableCheckedEdit();
-  void enableSuidEdit();
-
-private:
-
-    QLabel *terminalLabel;
-    QLabel *suidLabel;
-    KLineEdit *execEdit;
-    QCheckBox *terminalCheck;
-    QCheckBox *suidCheck;
-    KLineEdit *terminalEdit;
-    KLineEdit *suidEdit;
-    KLineEdit *swallowExecEdit;
-    KLineEdit *swallowTitleEdit;
-    QButton *execBrowse;
-
-    QString execStr;
-    QString swallowExecStr;
-    QString swallowTitleStr;
-    QString termOptionsStr;
-    bool termBool;
-    bool suidBool;
-    QString suidUserStr;
-
-    class KExecPropsPluginPrivate;
-    KExecPropsPluginPrivate *d;
-};
 
 /**
  * Used to edit the files containing
  * [Desktop Entry]
  * URL=....
  *
- * Such files are used to represent a program in kpanel and kfm.
+ * Such files are used to represent a program in kicker and konqueror.
  * @internal
  */
 class KURLPropsPlugin : public KPropsDlgPlugin
@@ -597,49 +594,6 @@ private:
   KURLPropsPluginPrivate *d;
 };
 
-/**
- * Used to edit the files containing
- * [Desktop Entry]
- * Type=Application
- *
- * Such files are used to represent a program in kpanel and kfm.
- * @internal
- */
-class KApplicationPropsPlugin : public KPropsDlgPlugin
-{
-  Q_OBJECT
-public:
-  /**
-   * Constructor
-   */
-  KApplicationPropsPlugin( KPropertiesDialog *_props );
-  virtual ~KApplicationPropsPlugin();
-
-  virtual void applyChanges();
-
-  static bool supports( KFileItemList _items );
-
-public slots:
-  void slotDelExtension();
-  void slotAddExtension();
-
-private slots:
-  void updateButton();
-
-private:
-  void addMimeType( const QString & name );
-
-  QLineEdit *commentEdit;
-  QLineEdit *genNameEdit;
-  QLineEdit *nameEdit;
-  QListBox  *extensionsList;
-  QListBox  *availableExtensionsList;
-  QPushButton *addExtensionButton;
-  QPushButton *delExtensionButton;
-
-  class KApplicationPropsPluginPrivate;
-  KApplicationPropsPluginPrivate *d;
-};
 
 /**
  * Used to edit the files containing
@@ -690,6 +644,7 @@ public:
 
 private slots:
   void slotActivated( int );
+  void slotDeviceChanged();
 
 private:
   QComboBox* device;
@@ -711,5 +666,157 @@ private:
   KDevicePropsPluginPrivate *d;
 };
 
+class KPropertiesDesktopBase;
+
+/**
+ * Used to edit the files containing
+ * [Desktop Entry]
+ * Type=Application
+ *
+ * Such files are used to represent a program in kicker and konqueror.
+ * @internal
+ */
+class KDesktopPropsPlugin : public KPropsDlgPlugin
+{
+  Q_OBJECT
+public:
+  /**
+   * Constructor
+   */
+  KDesktopPropsPlugin( KPropertiesDialog *_props );
+  virtual ~KDesktopPropsPlugin();
+
+  virtual void applyChanges();
+
+  static bool supports( KFileItemList _items );
+
+public slots:
+  void slotAddFiletype();
+  void slotDelFiletype();
+  void slotBrowseExec();
+  void slotAdvanced();
+  void slotSelectMimetype();
+
+private:
+  void checkCommandChanged();
+
+private:
+  KPropertiesDesktopBase* w;
+
+  QString m_origCommandStr;
+  QString m_terminalOptionStr;
+  QString m_suidUserStr;
+  QString m_dcopServiceType;
+  bool m_terminalBool;
+  bool m_terminalCloseBool;
+  bool m_suidBool;
+  bool m_startupBool;
+
+  class KDesktopPropsPluginPrivate;
+  KDesktopPropsPluginPrivate *d;
+};
+
+/**
+ * Used to edit the files containing
+ * [Desktop Entry]
+ * Type=Application
+ *
+ * Such files are used to represent a program in kicker and konqueror.
+ * @internal
+ * @deprecated replaced with KDesktopPropsPlugin
+ */
+ /// Remove in KDE4
+class KDE_DEPRECATED KExecPropsPlugin : public KPropsDlgPlugin
+{
+  Q_OBJECT
+public:
+  /**
+   * Constructor
+   */
+  KExecPropsPlugin( KPropertiesDialog *_props );
+  virtual ~KExecPropsPlugin();
+
+  virtual void applyChanges();
+
+  static bool supports( KFileItemList _items );
+
+public slots:
+  void slotBrowseExec();
+
+private slots:
+  void enableCheckedEdit();
+  void enableSuidEdit();
+
+private:
+
+    QLabel *terminalLabel;
+    QLabel *suidLabel;
+    KLineEdit *execEdit;
+    QCheckBox *terminalCheck;
+    QCheckBox *suidCheck;
+    KLineEdit *terminalEdit;
+    KLineEdit *suidEdit;
+    KLineEdit *swallowExecEdit;
+    KLineEdit *swallowTitleEdit;
+    QButton *execBrowse;
+
+    QString execStr;
+    QString swallowExecStr;
+    QString swallowTitleStr;
+    QString termOptionsStr;
+    bool termBool;
+    bool suidBool;
+    QString suidUserStr;
+
+    class KExecPropsPluginPrivate;
+    KExecPropsPluginPrivate *d;
+};
+
+/**
+ * Used to edit the files containing
+ * [Desktop Entry]
+ * Type=Application
+ *
+ * Such files are used to represent a program in kicker and konqueror.
+ * @internal
+ * @deprecated replaced with KDesktopPropsPlugin
+ */
+ /// Remove in KDE4
+class KDE_DEPRECATED KApplicationPropsPlugin : public KPropsDlgPlugin
+{
+  Q_OBJECT
+public:
+  /**
+   * Constructor
+   */
+  KApplicationPropsPlugin( KPropertiesDialog *_props );
+  virtual ~KApplicationPropsPlugin();
+
+  virtual void applyChanges();
+
+  static bool supports( KFileItemList _items );
+
+public slots:
+  void slotDelExtension();
+  void slotAddExtension();
+
+private slots:
+  void updateButton();
+
+private:
+  void addMimeType( const QString & name );
+
+  QLineEdit *commentEdit;
+  QLineEdit *genNameEdit;
+  QLineEdit *nameEdit;
+  QListBox  *extensionsList;
+  QListBox  *availableExtensionsList;
+  QPushButton *addExtensionButton;
+  QPushButton *delExtensionButton;
+
+  class KApplicationPropsPluginPrivate;
+  KApplicationPropsPluginPrivate *d;
+};
+
 #endif
 
