@@ -1,5 +1,5 @@
 /* This file is part of the KDE libraries
-    Copyright (C) 2001,2002 Carsten Pfeiffer <pfeiffer@kde.org>
+    Copyright (C) 2001,2002,2003 Carsten Pfeiffer <pfeiffer@kde.org>
 
     library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Library General Public
@@ -30,6 +30,7 @@
 
 #include <kaboutdata.h>
 #include <kconfig.h>
+#include <kdebug.h>
 #include <kglobal.h>
 #include <kicondialog.h>
 #include <kiconloader.h>
@@ -70,6 +71,28 @@ private:
 ///////////////////////////////////////////////////////////////////
 ///////////////////////////////////////////////////////////////////
 
+class KURLBarItem::KURLBarItemPrivate
+{
+public:
+    KURLBarItemPrivate()
+    {
+        isPersistent = true;
+    }
+
+    bool isPersistent;
+};
+
+KURLBarItem::KURLBarItem( KURLBar *parent,
+                          const KURL& url, bool persistent, const QString& description,
+                          const QString& icon, KIcon::Group group )
+    : QListBoxPixmap( KIconLoader::unknown() /*, parent->listBox()*/ ),
+      m_url( url ),
+      m_pixmap( 0L ),
+      m_parent( parent ),
+      m_appLocal( true )
+{
+    init( icon, group, description, persistent );
+}
 
 KURLBarItem::KURLBarItem( KURLBar *parent,
                           const KURL& url, const QString& description,
@@ -80,13 +103,23 @@ KURLBarItem::KURLBarItem( KURLBar *parent,
       m_parent( parent ),
       m_appLocal( true )
 {
+    init( icon, group, description, true /*persistent*/ );
+}
+
+void KURLBarItem::init( const QString& icon, KIcon::Group group,
+                        const QString& description, bool persistent )
+{
+    d = new KURLBarItemPrivate;
+    d->isPersistent = persistent;
+
     setCustomHighlighting( true );
-    setDescription( description );
     setIcon( icon, group );
+    setDescription( description );
 }
 
 KURLBarItem::~KURLBarItem()
 {
+    delete d;
 }
 
 void KURLBarItem::setURL( const KURL& url )
@@ -114,6 +147,17 @@ void KURLBarItem::setDescription( const QString& desc )
     setText( desc.isEmpty() ? m_url.fileName() : desc );
 }
 
+void KURLBarItem::setApplicationLocal( bool local )
+{
+    if ( !local && !isPersistent() )
+    {
+        kdWarning() << "KURLBar: dynamic (non-persistent) items can not be global." << endl;
+        return;
+    }
+
+    m_appLocal = local;
+}
+
 void KURLBarItem::setToolTip( const QString& tip )
 {
     m_toolTip = tip;
@@ -221,6 +265,11 @@ int KURLBarItem::height( const QListBox *lb ) const
         return QMAX( sizeHint().height(), lb->viewport()->height() );
 }
 
+bool KURLBarItem::isPersistent() const
+{
+    return d->isPersistent;
+}
+
 ///////////////////////////////////////////////////////////////////
 ///////////////////////////////////////////////////////////////////
 
@@ -274,6 +323,14 @@ KURLBarItem * KURLBar::insertItem(const KURL& url, const QString& description,
     return item;
 }
 
+KURLBarItem * KURLBar::insertDynamicItem(const KURL& url, const QString& description,
+                                         const QString& icon, KIcon::Group group )
+{
+    KURLBarItem *item = new KURLBarItem(this, url, false, description, icon, group);
+    m_listBox->insertItem( item );
+    return item;
+}
+
 void KURLBar::setOrientation( Qt::Orientation orient )
 {
     m_listBox->setOrientation( orient );
@@ -316,6 +373,7 @@ void KURLBar::setListBox( KURLBarListBox *view )
 
     setPalette( pal );
     m_listBox->viewport()->setBackgroundMode( PaletteMid );
+    m_listBox->setFocusPolicy( TabFocus );
 
     connect( m_listBox, SIGNAL( mouseButtonClicked( int, QListBoxItem *, const QPoint & ) ),
              SLOT( slotSelected( int, QListBoxItem * )));
@@ -324,6 +382,8 @@ void KURLBar::setListBox( KURLBarListBox *view )
     connect( m_listBox, SIGNAL( contextMenuRequested( QListBoxItem *,
                                                       const QPoint& )),
              SLOT( slotContextMenuRequested( QListBoxItem *, const QPoint& )));
+    connect( m_listBox, SIGNAL( returnPressed( QListBoxItem * ) ),
+             SLOT( slotSelected( QListBoxItem * ) ));
 }
 
 void KURLBar::setIconSize( int size )
@@ -362,7 +422,7 @@ QSize KURLBar::sizeHint() const
 #if 0
     // this code causes vertical and or horizontal scrollbars appearing
     // depending on the text, font, moonphase and earth rotation. Just using
-    // m_listBox->sizeHint() fixes this (although the widget can then be 
+    // m_listBox->sizeHint() fixes this (although the widget can then be
     // resized to a smaller size so that scrollbars appear).
     int w = 0;
     int h = 0;
@@ -488,32 +548,11 @@ void KURLBar::readConfig( KConfig *appConfig, const QString& itemGroup )
     }
 }
 
-static KURL kurlbar_readURLEntry( KConfig *config, const QString& key )
-{
-    // This looks a bit complex to just read a URL, but we have our reasons:
-    // Until KDE 3.1.1, the url was written as prettyURL(), so it included the file:/
-    // prefix. Now we use KConfig::writePathEntry( url.path() ) in case it's a local
-    // file, so readEntry() may return e.g. either file:/foo, /foo or $HOME
-
-    QString urlEntry = config->readEntry( key );
-    bool isPath = !urlEntry.isEmpty() && (urlEntry[0] == '/' || urlEntry[0] == '$');
-    // if it's local, try dollar expansion
-    if ( !urlEntry.isEmpty() && (isPath || urlEntry.startsWith("file:/") ))
-        urlEntry = config->readPathEntry( key );
-    KURL url;
-    if ( isPath )
-        url.setPath( urlEntry );
-    else
-        url = urlEntry;
-
-    return url;
-}
-
 void KURLBar::readItem( int i, KConfig *config, bool applicationLocal )
 {
     QString number = QString::number( i );
-    KURL url = kurlbar_readURLEntry( config, QString("URL_") + number );
-    if ( url.isMalformed() || !KProtocolInfo::isKnownProtocol( url ))
+    KURL url = KURL::fromPathOrURL( config->readPathEntry( QString("URL_") + number ));
+    if ( !url.isValid() || !KProtocolInfo::isKnownProtocol( url ))
         return; // nothing we could do.
 
     insertItem( url,
@@ -536,13 +575,18 @@ void KURLBar::writeConfig( KConfig *config, const QString& itemGroup )
     int numLocal = 0;
     KURLBarItem *item = static_cast<KURLBarItem*>( m_listBox->firstItem() );
 
-    while ( item ) {
-        if ( item->applicationLocal() ) {
-            writeItem( item, numLocal, config, false );
-            numLocal++;
-        }
+    while ( item )
+    {
+        if ( item->isPersistent() ) // we only save persistent items
+        {
+            if ( item->applicationLocal() )
+            {
+                writeItem( item, numLocal, config, false );
+                numLocal++;
+            }
 
-        i++;
+            i++;
+        }
         item = static_cast<KURLBarItem*>( item->next() );
     }
     config->writeEntry("Number of Entries", numLocal);
@@ -556,10 +600,15 @@ void KURLBar::writeConfig( KConfig *config, const QString& itemGroup )
         int numGlobals = 0;
         item = static_cast<KURLBarItem*>( m_listBox->firstItem() );
 
-        while ( item ) {
-            if ( !item->applicationLocal() ) {
-                writeItem( item, numGlobals, config, true );
-                numGlobals++;
+        while ( item )
+        {
+            if ( item->isPersistent() ) // we only save persistent items
+            {
+                if ( !item->applicationLocal() )
+                {
+                    writeItem( item, numGlobals, config, true );
+                    numGlobals++;
+                }
             }
 
             item = static_cast<KURLBarItem*>( item->next() );
@@ -573,16 +622,16 @@ void KURLBar::writeConfig( KConfig *config, const QString& itemGroup )
 void KURLBar::writeItem( KURLBarItem *item, int i, KConfig *config,
                          bool global )
 {
+    if ( !item->isPersistent() )
+        return;
+
     QString Description = "Description_";
     QString URL = "URL_";
     QString Icon = "Icon_";
     QString IconGroup = "IconGroup_";
 
     QString number = QString::number( i );
-    if ( item->url().isLocalFile() )
-        config->writePathEntry( URL + number, item->url().path(), true, global );
-    else
-    config->writeEntry( URL + number, item->url().prettyURL(), true, global );
+    config->writePathEntry( URL + number, item->url().prettyURL(), true, global );
 
     config->writeEntry( Description + number, item->description(),true,global);
     config->writeEntry( Icon + number, item->icon(), true, global );
@@ -614,10 +663,13 @@ void KURLBar::slotDropped( QDropEvent *e )
     }
 }
 
-void KURLBar::slotContextMenuRequested( QListBoxItem *item, const QPoint& pos )
+void KURLBar::slotContextMenuRequested( QListBoxItem *_item, const QPoint& pos )
 {
-    if (m_isImmutable) return;
+    if (m_isImmutable)
+        return;
 
+    KURLBarItem *item = dynamic_cast<KURLBarItem*>( _item );
+    
     static const int IconSize   = 10;
     static const int AddItem    = 20;
     static const int EditItem   = 30;
@@ -631,15 +683,16 @@ void KURLBar::slotContextMenuRequested( QListBoxItem *item, const QPoint& pos )
                        i18n("&Large Icons") : i18n("&Small Icons"),
                        IconSize );
     popup->insertSeparator();
-    popup->insertItem(SmallIcon("filenew"), i18n("&Add Entry..."), AddItem);
-    popup->insertItem(SmallIcon("edit"), i18n("&Edit Entry..."), EditItem);
-    popup->insertSeparator();
-    popup->insertItem( SmallIcon("editdelete"), i18n("&Remove Entry"),
-                       RemoveItem );
-
-    popup->setItemEnabled( EditItem, item != 0L );
-    popup->setItemEnabled( RemoveItem, item != 0L );
-
+    
+    if (item != 0L && item->isPersistent())
+    {
+        popup->insertItem(SmallIconSet("edit"), i18n("&Edit Entry..."), EditItem);
+        popup->insertSeparator();
+        popup->insertItem( SmallIconSet("editdelete"), i18n("&Remove Entry"),
+                          RemoveItem );
+    }
+    popup->insertItem(SmallIconSet("filenew"), i18n("&Add Entry..."), AddItem);
+    
     int result = popup->exec( pos );
     switch ( result ) {
         case IconSize:
@@ -680,6 +733,9 @@ bool KURLBar::addNewItem()
 
 bool KURLBar::editItem( KURLBarItem *item )
 {
+    if ( !item || !item->isPersistent() ) // should never happen tho
+        return false;
+
     KURL url            = item->url();
     QString description = item->description();
     QString icon        = item->icon();
@@ -732,7 +788,7 @@ QDragObject * KURLBarListBox::dragObject()
     }
 
     if ( !urls.isEmpty() ) // ### use custom drag-object with description etc.?
-        return KURLDrag::newDrag( urls, this, "urlbar drag" );
+        return new KURLDrag( urls, this, "urlbar drag" );
 
     return 0L;
 }
@@ -747,6 +803,15 @@ void KURLBarListBox::contentsDropEvent( QDropEvent *e )
     emit dropped( e );
 }
 
+void KURLBarListBox::contextMenuEvent( QContextMenuEvent *e )
+{
+    if (e)
+    {
+        emit contextMenuRequested( itemAt( e->globalPos() ), e->globalPos() );
+        e->consume(); // Consume the event to avoid multiple contextMenuEvent calls...
+    }
+}
+
 void KURLBarListBox::setOrientation( Qt::Orientation orient )
 {
     if ( orient == Vertical ) {
