@@ -55,7 +55,7 @@ public:
 
     /* not needed so far
     void setURLs( const KURL::List& urls ) {
-	m_urls = urls;	
+	m_urls = urls;
     }
     const KURL::List& urls() const { return m_urls; }
     */
@@ -65,7 +65,7 @@ protected:
 	if ( m_urls.isEmpty() )
 	    return 0L;
 
-	QDragObject *drag = KURLDrag::newDrag( m_urls, this, "url drag" );
+	QDragObject *drag = new KURLDrag( m_urls, this, "url drag" );
 	return drag;
     }
 
@@ -90,9 +90,21 @@ public:
 
     void setText( const QString& text ) {
 	if ( combo )
-	    combo->setEditText( text );
+	{
+	    if (combo->editable())
+	    {
+               combo->setEditText( text );
+            }
+            else
+            {
+               combo->insertItem( text );
+               combo->setCurrentItem( combo->count()-1 );
+            }
+        }
 	else
+	{
 	    edit->setText( text );
+	}
     }
 
     void connectSignals( QObject *receiver ) {
@@ -116,7 +128,7 @@ public:
 	else
 	    edit->setCompletionObject( comp );
     }
-	
+
     /**
      * replaces ~user or $FOO, if necessary
      */
@@ -196,7 +208,7 @@ void KURLRequester::init()
     QPixmap pixMap = iconSet.pixmap( QIconSet::Small, QIconSet::Normal );
     myButton->setIconSet( iconSet );
     myButton->setFixedSize( pixMap.width()+8, pixMap.height()+8 );
-    QToolTip::add(myButton, i18n("Open File Dialog"));
+    QToolTip::add(myButton, i18n("Open file dialog"));
 
     connect( myButton, SIGNAL( pressed() ), SLOT( slotUpdateURL() ));
 
@@ -229,8 +241,9 @@ void KURLRequester::setURL( const QString& url )
 
 void KURLRequester::setCaption( const QString& caption )
 {
-	fileDialog()->setCaption( caption );
-	QWidget::setCaption( caption );
+   QWidget::setCaption( caption );
+   if (myFileDialog)
+      myFileDialog->setCaption( caption );
 }
 
 QString KURLRequester::url() const
@@ -247,13 +260,20 @@ void KURLRequester::slotOpenDialog()
     if ( !d->url().isEmpty() ) {
         KURL u( url() );
         // If we won't be able to list it (e.g. http), then don't try :)
-        if ( KProtocolInfo::supportsListing( u.protocol() ) )
+        if ( KProtocolInfo::supportsListing( u ) )
 	    dlg->setSelection( u.url() );
     }
 
     if ( dlg->exec() == QDialog::Accepted )
     {
-	setURL( dlg->selectedURL().prettyURL() );
+        if ( dlg->selectedURL().isLocalFile() )
+        {
+            setURL( dlg->selectedURL().path() );
+        }
+        else
+        {
+            setURL( dlg->selectedURL().prettyURL() );
+        }
         emit urlSelected( d->url() );
     }
 }
@@ -264,7 +284,7 @@ void KURLRequester::setMode(unsigned int mode)
     d->fileDialogMode = mode;
     if ( (mode & KFile::Directory) && !(mode & KFile::File) )
         myCompletion->setMode( KURLCompletion::DirCompletion );
-    
+
     if (myFileDialog)
        myFileDialog->setMode( d->fileDialogMode );
 }
@@ -282,9 +302,10 @@ KFileDialog * KURLRequester::fileDialog() const
 	QWidget *p = parentWidget();
 	myFileDialog = new KFileDialog( QString::null, QString::null, p,
 					"file dialog", true );
-	
+
 	myFileDialog->setMode( d->fileDialogMode );
         myFileDialog->setFilter( d->fileDialogFilter );
+        myFileDialog->setCaption( caption() );
     }
 
     return myFileDialog;
@@ -348,4 +369,10 @@ KEditListBox::CustomEditor KURLRequester::customEditor()
 void KURLRequester::virtual_hook( int, void* )
 { /*BASE::virtual_hook( id, data );*/ }
 
+KURLComboRequester::KURLComboRequester( QWidget *parent,
+			      const char *name )
+  : KURLRequester( new KComboBox(false), parent, name)
+{
+}
+
 #include "kurlrequester.moc"
