@@ -46,14 +46,17 @@ class QTimer;
  * the current working directory, or the last directory where a file has been
  * selected.
  *
- * You can change this behavior by using @ref setMode() or @ref setFilter().
- *
+ * You can change this behavior by using setMode() or setFilter().
+ * 
+ * \image html kurlrequester.png "KDE URL Requester"
+ * 
  * @short A widget to request a filename/url from the user
  * @author Carsten Pfeiffer <pfeiffer@kde.org>
  */
 class KURLRequester : public QHBox
 {
     Q_OBJECT
+    Q_PROPERTY( QString url READ url WRITE setURL )
 
 public:
     /**
@@ -65,6 +68,7 @@ public:
      * Constructs a KURLRequester widget with the initial URL @p url.
      */
     KURLRequester( const QString& url, QWidget *parent=0, const char *name=0 );
+
     /**
      * Special constructor, which creates a KURLRequester widget with a custom
      * edit-widget. The edit-widget can be either a KComboBox or a KLineEdit
@@ -88,9 +92,9 @@ public:
 
     /**
      * Enables/disables showing file:/ in the lineedit, when a local file has
-     * been selected in the filedialog or was set via @ref setURL().
+     * been selected in the filedialog or was set via setURL().
      * Default is false, not showing file:/
-     * @see #showLocalProtocol
+     * @see showLocalProtocol
      */
     void setShowLocalProtocol( bool b );
 
@@ -111,7 +115,7 @@ public:
     /**
      * @returns whether local files will be prefixed with file:/ in the
      * lineedit
-     * @see #setShowLocalProtocol
+     * @see setShowLocalProtocol
      */
     bool showLocalProtocol() const { return myShowLocalProt; }
 
@@ -127,7 +131,7 @@ public:
      * special one, if you used the special constructor.
      *
      * It is provided so that you can e.g. set an own completion object
-     * (e.g. @ref KShellCompletion) into it.
+     * (e.g. KShellCompletion) into it.
      */
     KLineEdit * lineEdit() const;
 
@@ -152,11 +156,11 @@ public:
      * @returns an object, suitable for use with KEditListBox. It allows you
      * to put this KURLRequester into a KEditListBox.
      * Basically, do it like this:
-     * <pre>
+     * \code
      * KURLRequester *req = new KURLRequester( someWidget );
      * [...]
      * KEditListBox *editListBox = new KEditListBox( i18n("Some Title"), req->customEditor(), someWidget );
-     * </pre>
+     * \endcode
      * @since 3.1
      */
     KEditListBox::CustomEditor customEditor();
@@ -164,13 +168,12 @@ public:
 public slots:
     /**
      * Sets the url in the lineedit to @p url. Depending on the state of
-     * @ref showLocalProtocol(), file:/ on local files will be shown or not.
+     * showLocalProtocol(), file:/ on local files will be shown or not.
      * @since 3.1
      */
     void setURL( const QString& url );
 
     /**
-     * @reimp
      * Sets the caption of the file dialog.
      * @since 3.1
      */
@@ -235,7 +238,7 @@ private:
 protected slots:
     /**
      * Called when the button is pressed to open the filedialog.
-     * Also called when @ref KStdAccel::Open (default is Ctrl-O) is pressed.
+     * Also called when KStdAccel::Open (default is Ctrl-O) is pressed.
      */
     void slotOpenDialog();
 
@@ -249,4 +252,15 @@ private:
     KURLRequesterPrivate *d;
 };
 
+class KURLComboRequester : public KURLRequester // For use in Qt Designer
+{
+    Q_OBJECT
+public:    
+    /**
+     * Constructs a KURLRequester widget with a combobox.
+     */
+    KURLComboRequester( QWidget *parent=0, const char *name=0 );
+};
+
+
 #endif // KURLREQUESTER_H
