@@ -40,8 +40,8 @@ namespace KIO {
  * prompt for password you only need to optionally set the prompt,
  * username (if already supplied), comment and commentLabel fields.
  *
- * <u>SPECIAL NOTE:</u> If you extend this class to add additional
- * paramters do not forget to overload the stream insertion and
+ * <em>SPECIAL NOTE:</em> If you extend this class to add additional
+ * parameters do not forget to overload the stream insertion and
  * extraction operators ("<<" and ">>") so that the added data can
  * be correctly serialzed.
  *
@@ -71,11 +71,13 @@ public:
 
    /**
     * Use this method to check if the object was modified.
+    * @return true if the object has been modified
     */
    bool isModified() const { return modified; }
 
    /**
     * Use this method to indicate that this object has been modified.
+    * @param flag true to mark the object as modified, false to clear
     */
    void setModified( bool flag ) { modified = flag; }
 
@@ -92,12 +94,12 @@ public:
    KURL url;
 
    /**
-    * This setting is @em required for caching.
+    * This is @em required for caching.
     */
    QString username;
 
    /**
-    * This setting is @em required for caching.
+    * This is @em required for caching.
     */
    QString password;
 
@@ -105,8 +107,8 @@ public:
     * Information to be displayed when prompting
     * the user for authentication information.
     *
-    * <u>NOTE:</u>If this field is not set, the authentication
-    * dialog simply displays the preset default prompt.
+    * @note If this field is not set, the authentication
+    *    dialog simply displays the preset default prompt.
     *
     * This setting is @em optional and empty by default.
     */
@@ -116,8 +118,8 @@ public:
     * The text to displayed in the title bar of
     * the password prompting dialog.
     *
-    * <u>NOTE:</u>If this field is not set, the authentication
-    * dialog simply displays the preset default caption.
+    * @note If this field is not set, the authentication
+    *    dialog simply displays the preset default caption.
     *
     * This setting is @em optional and empty by default.
     */
@@ -130,12 +132,12 @@ public:
     * This field allows you to display a short (no more than
     * 80 characters) extra description in the password prompt
     * dialog.  For example, this field along with the
-    * @ref commentLabel can be used to describe the server that
+    * commentLabel can be used to describe the server that
     * requested the authentication:
     *
-    *  <pre>
+    *  \code
     *  Server:   Squid Proxy @ foo.com
-    *  </pre>
+    *  \endcode
     *
     * where "Server:" is the commentLabel and the rest is the
     * actual comment.  Note that it is always better to use
@@ -152,7 +154,7 @@ public:
     * comment when prompting the user for password.
     *
     * This setting is @em optional and only applicable when
-    * the @ref #comment field is also set.
+    * the comment field is also set.
     */
    QString commentLabel;
 
@@ -228,10 +230,11 @@ QDataStream& operator>> (QDataStream& s, AuthInfo& a);
 
 /**
  * A Singleton class that provides access to passwords
- * stored in .netrc and kionetrc for automatic login
- * purposes.
+ * stored in .netrc files for automatic login purposes.
+ * This is only meant to address backward compatability
+ * with old automated ftp client style logins...
  *
- * @short An interface to kionetrc and the ftp .netrc files
+ * @short An interface to the ftp .netrc files
  * @author Dawit Alemayehu <adawit@kde.org>
  */
 class NetRC
@@ -242,9 +245,9 @@ public:
    * Specifies the mode to be used when searching for a
    * matching automatic login info for a given site :
    *
-   * exactOnly        search entries with exact host name matches.
-   * defaultOnly      search entries that are specified as "default".
-   * presetOnly       search entries that are specified as "preset".
+   * @li exactOnly        search entries with exact host name matches.
+   * @li defaultOnly      search entries that are specified as "default".
+   * @li presetOnly       search entries that are specified as "preset".
    *
    * @see lookup
    */
@@ -256,7 +259,8 @@ public:
   };
 
   /**
-   *
+   * Contains auto login information.
+   * @see lookup()
    */
   struct AutoLogin
   {
@@ -268,25 +272,34 @@ public:
   };
 
   /**
-   *
+   * A reference to the instance of the class.
+   * @return the class
    */
   static NetRC* self();
 
   /**
+   * Looks up the @p login information for the given @p url.
    *
+   * @param url the url whose login information will be checked
+   * @param login the login information will be writte here
+   * @param userealnetrc if true, use $HOME/.netrc fle
+   * @param type the type of the login. If null, the @p url's protocol
+   *        will be taken
+   * @param mode the LookUpMode flags (ORed) for the query
    */
-  bool lookup( const KURL&, AutoLogin&, bool userealnetrc = false,
+  bool lookup( const KURL& url, AutoLogin& login,
+               bool userealnetrc = false,
                QString type = QString::null,
                int mode = (exactOnly|defaultOnly) );
   /**
-   *
+   * Reloads the auto login information.
    */
   void reload() { isDirty = true; }
 
 protected:
   QString extract( const char*, const char*, int& );
   int openf( const QString& );
-  void parse( int );
+  bool parse( int );
 
 private:
   NetRC();
@@ -303,5 +316,5 @@ private:
   class NetRCPrivate;
   NetRCPrivate* d;
 };
-};
+}
 #endif
