@@ -184,7 +184,7 @@ void ChmodJob::chmodNextFile()
             QString path = info.url.path();
             if ( chown( QFile::encodeName(path), m_newOwner, m_newGroup ) != 0 )
             {
-                int answer = KMessageBox::warningContinueCancel( 0, i18n( "<qt>Could not modify the ownership of file <b>%1</b>.You have insufficient access to the file to perform the change.</qt>" ).arg(path), QString::null, i18n("Continue") );
+                int answer = KMessageBox::warningContinueCancel( 0, i18n( "<qt>Could not modify the ownership of file <b>%1</b>.You have insufficient access to the file to perform the change.</qt>" ).arg(path), QString::null, KStdGuiItem::cont() );
                 if (answer == KMessageBox::Cancel)
                 {
                     m_error = ERR_USER_CANCELED;
@@ -233,6 +233,7 @@ void ChmodJob::slotResult( KIO::Job * job )
     }
 }
 
+// antlarr: KDE 4: Make owner and group be const QString &
 ChmodJob *KIO::chmod( const KFileItemList& lstItems, int permissions, int mask,
                       QString owner, QString group,
                       bool recursive, bool showProgressInfo )
