@@ -25,6 +25,7 @@
 #include <qstring.h>
 
 #include <kio/global.h>
+#include <kio/job.h>
 #include <kfileitem.h>
 
 /*
@@ -42,11 +43,15 @@ namespace KIO {
     /**
      * This job changes permissions on a list of files or directories,
      * optionally in a recursive manner.
+     * @see KIO::chmod()
      */
     class ChmodJob : public KIO::Job
     {
         Q_OBJECT
     public:
+	/**
+	 * Create new ChmodJobs using the KIO::chmod() function.
+	 */
         ChmodJob( const KFileItemList & lstItems,  int permissions, int mask,
                   int newOwner, int newGroup,
                   bool recursive, bool showProgressInfo );
@@ -87,17 +92,17 @@ namespace KIO {
      *
      * Inside directories, the "x" bits will only be changed for files that had
      * at least one "x" bit before, and for directories.
-     * This emulates the behaviour of chmod +X.
+     * This emulates the behavior of chmod +X.
      *
      * @param lstItems The file items representing several files or directories.
      * @param permissions the permissions we want to set
-     * @param mask the bits we are allowed to change
+     * @param mask the bits we are allowed to change.
      * For instance, if mask is 0077, we don't change
      * the "user" bits, only "group" and "others".
      * @param newOwner If non-empty, the new owner for the files
      * @param newGroup If non-empty, the new group for the files
      * @param recursive whether to open directories recursively
-     *
+     * @param showProgressInfo true to show progess information
      * @return The job handling the operation.
      */
     ChmodJob * chmod( const KFileItemList& lstItems, int permissions, int mask,
