@@ -33,6 +33,7 @@
 #include <kpushbutton.h>
 #include <kstandarddirs.h>
 #include <kstdguiitem.h>
+#include <klineedit.h>
 #include <kwin.h>
 
 #include "jobclasses.h"
@@ -50,6 +51,7 @@ public:
   KPushButton *openLocation;
   QCheckBox   *keepOpen;
   KURL        location;
+  QTime       startTime;
 };
 
 DefaultProgress::DefaultProgress( bool showNow )
@@ -72,6 +74,11 @@ DefaultProgress::DefaultProgress( QWidget* parent, const char* /*name*/ )
     init();
 }
 
+bool DefaultProgress::keepOpen() const
+{
+    return d->keepOpenChecked;
+}
+
 void DefaultProgress::init()
 {
   d = new DefaultProgressPrivate;
@@ -93,14 +100,16 @@ void DefaultProgress::init()
   // filenames or action name
   grid->addWidget(new QLabel(i18n("Source:"), this), 0, 0);
 
-  sourceLabel = new KSqueezedTextLabel(this);
-  grid->addWidget(sourceLabel, 0, 2);
+  sourceEdit = new KLineEdit(this);
+  sourceEdit->setReadOnly (true);
+  grid->addWidget(sourceEdit, 0, 2);
 
   destInvite = new QLabel(i18n("Destination:"), this);
   grid->addWidget(destInvite, 1, 0);
 
-  destLabel = new KSqueezedTextLabel(this);
-  grid->addWidget(destLabel, 1, 2);
+  destEdit = new KLineEdit(this);
+  destEdit->setReadOnly (true);
+  grid->addWidget(destEdit, 1, 2);
 
   m_pProgressBar = new KProgress(this);
   topLayout->addWidget( m_pProgressBar );
@@ -132,7 +141,7 @@ void DefaultProgress::init()
   line->setFrameShadow( QFrame::Sunken );
   topLayout->addWidget( line );
 
-  d->keepOpen = new QCheckBox( i18n("&Keep this window open after download is complete"), this);
+  d->keepOpen = new QCheckBox( i18n("&Keep this window open after transfer is complete"), this);
   connect( d->keepOpen, SIGNAL( toggled(bool) ), SLOT( slotKeepOpenToggled(bool) ) );
   topLayout->addWidget(d->keepOpen);
   d->keepOpen->hide();
@@ -173,6 +182,8 @@ DefaultProgress::~DefaultProgress()
 void DefaultProgress::slotTotalSize( KIO::Job*, KIO::filesize_t bytes )
 {
   m_iTotalSize = bytes;
+  if (d->startTime.isNull())
+    d->startTime.start();
 }
 
 
@@ -199,7 +210,7 @@ void DefaultProgress::showTotals()
     QString tmps;
     if ( m_iTotalDirs > 1 )
       // that we have a singular to translate looks weired but is only logical
-      tmps = i18n("%n directory", "%n directories", m_iTotalDirs) + "   ";
+      tmps = i18n("%n folder", "%n folders", m_iTotalDirs) + "   ";
     tmps += i18n("%n file", "%n files", m_iTotalFiles);
     progressLabel->setText( tmps );
   }
@@ -250,7 +261,7 @@ void DefaultProgress::slotProcessedDirs( KIO::Job*, unsigned long dirs )
   m_iProcessedDirs = dirs;
 
   QString tmps;
-  tmps = i18n("%1 / %n directory", "%1 / %n directories", m_iTotalDirs).arg( m_iProcessedDirs );
+  tmps = i18n("%1 / %n folder", "%1 / %n folders", m_iTotalDirs).arg( m_iProcessedDirs );
   tmps += "   ";
   tmps += i18n("%1 / %n file", "%1 / %n files", m_iTotalFiles).arg( m_iProcessedFiles );
   progressLabel->setText( tmps );
@@ -263,7 +274,7 @@ void DefaultProgress::slotProcessedFiles( KIO::Job*, unsigned long files )
 
   QString tmps;
   if ( m_iTotalDirs > 1 ) {
-    tmps = i18n("%1 / %n directory", "%1 / %n directories", m_iTotalDirs).arg( m_iProcessedDirs );
+    tmps = i18n("%1 / %n folder", "%1 / %n folders", m_iTotalDirs).arg( m_iProcessedDirs );
     tmps += "   ";
   }
   tmps += i18n("%1 / %n file", "%1 / %n files", m_iTotalFiles).arg( m_iProcessedFiles );
@@ -284,15 +295,17 @@ void DefaultProgress::slotSpeed( KIO::Job*, unsigned long bytes_per_second )
 
 void DefaultProgress::slotCopying( KIO::Job*, const KURL& from, const KURL& to )
 {
-  if ( d->noCaptionYet ) {
+    if ( d->noCaptionYet ) {
     setCaption(i18n("Copy File(s) Progress"));
     d->noCaptionYet = false;
   }
   mode = Copy;
-  sourceLabel->setText(from.prettyURL());
+  sourceEdit->setSqueezedText(from.prettyURL());
+  sourceEdit->home (false);
   setDestVisible( true );
   checkDestination( to );
-  destLabel->setText(to.prettyURL());
+  destEdit->setSqueezedText(to.prettyURL());
+  destEdit->home (false);
 }
 
 
@@ -303,21 +316,24 @@ void DefaultProgress::slotMoving( KIO::Job*, const KURL& from, const KURL& to )
     d->noCaptionYet = false;
   }
   mode = Move;
-  sourceLabel->setText(from.prettyURL());
+  sourceEdit->setSqueezedText(from.prettyURL());
+  sourceEdit->home (false);
   setDestVisible( true );
   checkDestination( to );
-  destLabel->setText(to.prettyURL());
+  destEdit->setSqueezedText(to.prettyURL());
+  destEdit->home (false);
 }
 
 
 void DefaultProgress::slotCreatingDir( KIO::Job*, const KURL& dir )
 {
   if ( d->noCaptionYet ) {
-    setCaption(i18n("Creating Directory"));
+    setCaption(i18n("Creating Folder"));
     d->noCaptionYet = false;
   }
   mode = Create;
-  sourceLabel->setText(dir.prettyURL());
+  sourceEdit->setSqueezedText(dir.prettyURL());
+  sourceEdit->home (false);
   setDestVisible( false );
 }
 
@@ -329,7 +345,8 @@ void DefaultProgress::slotDeleting( KIO::Job*, const KURL& url )
     d->noCaptionYet = false;
   }
   mode = Delete;
-  sourceLabel->setText(url.prettyURL());
+  sourceEdit->setSqueezedText(url.prettyURL());
+  sourceEdit->home (false);
   setDestVisible( false );
 }
 
@@ -339,28 +356,32 @@ void DefaultProgress::slotTransferring( KIO::Job*, const KURL& url )
     setCaption(i18n("Loading Progress"));
     d->noCaptionYet = false;
   }
-  sourceLabel->setText(url.prettyURL());
+  sourceEdit->setSqueezedText(url.prettyURL());
+  sourceEdit->home (false);
   setDestVisible( false );
 }
 
 void DefaultProgress::slotStating( KIO::Job*, const KURL& url )
 {
   setCaption(i18n("Examining File Progress"));
-  sourceLabel->setText(url.prettyURL());
+  sourceEdit->setSqueezedText(url.prettyURL());
+  sourceEdit->home (false);
   setDestVisible( false );
 }
 
 void DefaultProgress::slotMounting( KIO::Job*, const QString & dev, const QString & point )
 {
   setCaption(i18n("Mounting %1").arg(dev));
-  sourceLabel->setText(point);
+  sourceEdit->setSqueezedText(point);
+  sourceEdit->home (false);
   setDestVisible( false );
 }
 
 void DefaultProgress::slotUnmounting( KIO::Job*, const QString & point )
 {
   setCaption(i18n("Unmounting"));
-  sourceLabel->setText(point);
+  sourceEdit->setSqueezedText(point);
+  sourceEdit->home (false);
   setDestVisible( false );
 }
 
@@ -375,24 +396,37 @@ void DefaultProgress::slotCanResume( KIO::Job*, KIO::filesize_t resume )
 
 void DefaultProgress::setDestVisible( bool visible )
 {
-  // We can't hide the destInvite/destLabel labels,
+  // We can't hide the destInvite/destEdit labels,
   // because it screws up the QGridLayout.
   if (visible)
   {
+      destInvite->show();
+      destEdit->show();
+
     destInvite->setText( i18n("Destination:") );
   }
   else
   {
+      destInvite->hide();
+      destEdit->hide();
     destInvite->setText( QString::null );
-    destLabel->setText( QString::null );
+    destEdit->setText( QString::null );
   }
 }
 
 void DefaultProgress::slotClean() {
   if (d->keepOpenChecked) {
     slotPercent(0, 100);
-    d->cancelClose->setText( KStdGuiItem::close().text() );
+    d->cancelClose->setGuiItem( KStdGuiItem::close() );
     d->openFile->setEnabled(true);
+    slotProcessedSize(0, m_iTotalSize);
+    d->keepOpen->setEnabled(false);
+    if (!d->startTime.isNull()) {
+      int s = d->startTime.elapsed();
+      if (!s)
+	s = 1;
+      speedLabel->setText(i18n("%1/s (done)").arg(KIO::convertSize(1000 * m_iTotalSize / s)));
+    }
     setOnlyClean(false);
   }
   else
