@@ -1,7 +1,7 @@
 /* This file is part of the KDE libraries
    Copyright (C) 2000 David Faure <faure@kde.org>
 
-   $Id: global.cpp,v 1.101.2.6 2003/01/27 22:20:49 brade Exp $
+   $Id: global.cpp,v 1.123.2.1 2004/03/21 18:33:55 faure Exp $
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
@@ -123,7 +123,7 @@ QString KIO::itemsSummaryString(uint items, uint files, uint dirs, KIO::filesize
         text += i18n("(%1 Total)").arg(KIO::convertSize( size ) );
     }
     text += " - ";
-    text += i18n("One Directory", "%n Directories", dirs);
+    text += i18n("One Folder", "%n Folders", dirs);
     return text;
 }
 
@@ -207,22 +207,22 @@ QString KIO::buildErrorString(int errorCode, const QString &errorText)
 //       result = i18n( "Unsupported action %1" ).arg( errorText );
       break;
     case  KIO::ERR_IS_DIRECTORY:
-      result = i18n( "%1 is a directory, but a file was expected." ).arg( errorText );
+      result = i18n( "%1 is a folder, but a file was expected." ).arg( errorText );
       break;
     case  KIO::ERR_IS_FILE:
-      result = i18n( "%1 is a file, but a directory was expected." ).arg( errorText );
+      result = i18n( "%1 is a file, but a folder was expected." ).arg( errorText );
       break;
     case  KIO::ERR_DOES_NOT_EXIST:
-      result = i18n( "The file or directory %1 does not exist." ).arg( errorText );
+      result = i18n( "The file or folder %1 does not exist." ).arg( errorText );
       break;
     case  KIO::ERR_FILE_ALREADY_EXIST:
       result = i18n( "A file named %1 already exists." ).arg( errorText );
       break;
     case  KIO::ERR_DIR_ALREADY_EXIST:
-      result = i18n( "A directory named %1 already exists." ).arg( errorText );
+      result = i18n( "A folder named %1 already exists." ).arg( errorText );
       break;
     case  KIO::ERR_UNKNOWN_HOST:
-      result = i18n( "Unknown host %1" ).arg( errorText );
+      result = errorText.isEmpty() ? i18n( "No hostname specified." ) : i18n( "Unknown host %1" ).arg( errorText );
       break;
     case  KIO::ERR_ACCESS_DENIED:
       result = i18n( "Access denied to %1" ).arg( errorText );
@@ -231,10 +231,10 @@ QString KIO::buildErrorString(int errorCode, const QString &errorText)
       result = i18n( "Access denied\nCould not write to %1" ).arg( errorText );
       break;
     case  KIO::ERR_CANNOT_ENTER_DIRECTORY:
-      result = i18n( "Could not enter directory %1" ).arg( errorText );
+      result = i18n( "Could not enter folder %1" ).arg( errorText );
       break;
     case  KIO::ERR_PROTOCOL_IS_NOT_A_FILESYSTEM:
-      result = i18n( "The protocol %1 does not implement a directory service." ).arg( errorText );
+      result = i18n( "The protocol %1 does not implement a folder service." ).arg( errorText );
       break;
     case  KIO::ERR_CYCLIC_LINK:
       result = i18n( "Found a cyclic link in %1" ).arg( errorText );
@@ -288,10 +288,10 @@ QString KIO::buildErrorString(int errorCode, const QString &errorText)
       result = i18n( "Could not terminate listing %1" ).arg( errorText );
       break;
     case  KIO::ERR_COULD_NOT_MKDIR:
-      result = i18n( "Could not make directory %1" ).arg( errorText );
+      result = i18n( "Could not make folder %1" ).arg( errorText );
       break;
     case  KIO::ERR_COULD_NOT_RMDIR:
-      result = i18n( "Could not remove directory %1" ).arg( errorText );
+      result = i18n( "Could not remove folder %1" ).arg( errorText );
       break;
     case  KIO::ERR_CANNOT_RESUME:
       result = i18n( "Could not resume file %1" ).arg( errorText );
@@ -380,6 +380,45 @@ QString KIO::buildErrorString(int errorCode, const QString &errorText)
   return result;
 }
 
+QString KIO::unsupportedActionErrorString(const QString &protocol, int cmd) {
+  switch (cmd) {
+    case CMD_CONNECT:
+      return i18n("Opening connections is not supported with the protocol %1" ).arg(protocol);
+    case CMD_DISCONNECT:
+      return i18n("Closing connections is not supported with the protocol %1" ).arg(protocol);
+    case CMD_STAT:
+      return i18n("Accessing files is not supported with the protocol %1").arg(protocol);
+    case CMD_PUT:
+      return i18n("Writing to %1 is not supported").arg(protocol);
+    case CMD_SPECIAL:
+      return i18n("There are no special actions available for protocol %1").arg(protocol);
+    case CMD_LISTDIR:
+      return i18n("Listing folders is not supported for protocol %1").arg(protocol);
+    case CMD_GET:
+      return i18n("Retrieving data from %1 is not supported").arg(protocol);
+    case CMD_MIMETYPE:
+      return i18n("Retrieving mime type information from %1 is not supported").arg(protocol);
+    case CMD_RENAME:
+      return i18n("Renaming or moving files within %1 is not supported").arg(protocol);
+    case CMD_SYMLINK:
+      return i18n("Creating symlinks is not supported with protocol %1").arg(protocol);
+    case CMD_COPY:
+      return i18n("Copying files within %1 is not supported").arg(protocol);
+    case CMD_DEL:
+      return i18n("Deleting files from %1 is not supported").arg(protocol);
+    case CMD_MKDIR:
+      return i18n("Creating folders is not supported with protocol %1").arg(protocol);
+    case CMD_CHMOD:
+      return i18n("Changing the attributes of files is not supported with protocol %1").arg(protocol);
+    case CMD_SUBURL:
+      return i18n("Using sub-URLs with %1 is not supported").arg(protocol);
+    case CMD_MULTI_GET:
+      return i18n("Multiple get is not supported with protocol %1").arg(protocol);
+    default:
+      return i18n("Protocol %1 does not support action %2").arg(protocol).arg(cmd);
+  }/*end switch*/
+}
+
 QStringList KIO::Job::detailedErrorStrings( const KURL *reqUrl /*= 0L*/,
                                             int method /*= -1*/ ) const
 {
@@ -407,22 +446,22 @@ QStringList KIO::Job::detailedErrorStrings( const KURL *reqUrl /*= 0L*/,
          QString::fromLatin1( "</b></p><p>" ) + description +
          QString::fromLatin1( "</p>" );
   ret2 = QString::fromLatin1( "<qt><p>" );
-  if ( techName != QString::null )
-    ret2 += i18n( "<b>Technical Reason</b>: " ) + techName + QString::fromLatin1( "</p>" );
-  ret2 += i18n( "</p><p><b>Details of the Request</b>:" );
+  if ( !techName.isNull() )
+    ret2 += i18n( "<b>Technical reason</b>: " ) + techName + QString::fromLatin1( "</p>" );
+  ret2 += i18n( "</p><p><b>Details of the request</b>:" );
   ret2 += i18n( "</p><ul><li>URL: %1</li>" ).arg( url );
-  if ( protocol != QString::null ) {
+  if ( !protocol.isNull() ) {
     ret2 += i18n( "<li>Protocol: %1</li>" ).arg( protocol );
   }
-  ret2 += i18n( "<li>Date and Time: %1</li>" ).arg( datetime );
-  ret2 += i18n( "<li>Additional Information: %1</li></ul>" ).arg( m_errorText );
+  ret2 += i18n( "<li>Date and time: %1</li>" ).arg( datetime );
+  ret2 += i18n( "<li>Additional information: %1</li></ul>" ).arg( m_errorText );
   if ( causes.count() ) {
-    ret2 += i18n( "<p><b>Possible Causes</b>:</p><ul><li>" );
+    ret2 += i18n( "<p><b>Possible causes</b>:</p><ul><li>" );
     ret2 += causes.join( "</li><li>" );
     ret2 += QString::fromLatin1( "</li></ul>" );
   }
   if ( solutions.count() ) {
-    ret2 += i18n( "<p><b>Possible Solutions</b>:</p><ul><li>" );
+    ret2 += i18n( "<p><b>Possible solutions</b>:</p><ul><li>" );
     ret2 += solutions.join( "</li><li>" );
     ret2 += QString::fromLatin1( "</li></ul>" );
   }
@@ -446,9 +485,9 @@ QByteArray KIO::rawErrorDetail(int errorCode, const QString &errorText,
       domain = host;
 
     path = reqUrl->path(1);
-    filename = reqUrl->filename();
+    filename = reqUrl->fileName();
     dir =  path + filename;
-    
+
     // detect if protocol is a network protocol...
     // add your hacks here...
     if ( protocol == "http" ||
@@ -537,10 +576,10 @@ QByteArray KIO::rawErrorDetail(int errorCode, const QString &errorText,
     case  KIO::ERR_CANNOT_OPEN_FOR_READING:
       errorName = i18n( "Cannot Open Resource For Reading" );
       description = i18n( "This means that the contents of the requested file "
-        "or directory <strong>%1</strong> could not be retrieved, as read "
+        "or folder <strong>%1</strong> could not be retrieved, as read "
         "access could not be obtained." ).arg( dir );
       causes << i18n( "You may not have permissions to read the file or open "
-        "the directory.") << cLocked << cHardware;
+        "the folder.") << cLocked << cHardware;
       solutions << sAccess << sQuerylock << sSysadmin;
       break;
 
@@ -576,11 +615,11 @@ QByteArray KIO::rawErrorDetail(int errorCode, const QString &errorText,
       break;
 
     case  KIO::ERR_MALFORMED_URL:
-      errorName = i18n( "Inproperly Formatted URL" );
+      errorName = i18n( "Improperly Formatted URL" );
       description = i18n( "The <strong>U</strong>niform <strong>R</strong>esource "
         "<strong>L</strong>ocator (URL) that you entered was not properly "
         "formatted. The format of a URL is generally as follows:"
-        "<blockquote><strong>protocol://user:password@www.example.org:port/directory/"
+        "<blockquote><strong>protocol://user:password@www.example.org:port/folder/"
         "filename.extension?query=value</strong></blockquote>" );
       solutions << sTypo;
       break;
@@ -629,22 +668,22 @@ QByteArray KIO::rawErrorDetail(int errorCode, const QString &errorText,
     case  KIO::ERR_IS_DIRECTORY:
       errorName = i18n( "File Expected" );
       description = i18n( "The request expected a file, however the "
-        "directory <strong>%1</strong> was found instead." ).arg( dir );
+        "folder <strong>%1</strong> was found instead." ).arg( dir );
       causes << i18n( "This may be an error on the server side." ) << cBug;
       solutions << sUpdate << sSysadmin;
       break;
 
     case  KIO::ERR_IS_FILE:
-      errorName = i18n( "Directory Expected" );
-      description = i18n( "The request expected a directory, however "
+      errorName = i18n( "Folder Expected" );
+      description = i18n( "The request expected a folder, however "
         "the file <strong>%1</strong> was found instead." ).arg( filename );
       causes << cBug;
       solutions << sUpdate << sSysadmin;
       break;
 
     case  KIO::ERR_DOES_NOT_EXIST:
-      errorName = i18n( "File or Directory Does Not Exist" );
-      description = i18n( "The specified file or directory <strong>%1</strong> "
+      errorName = i18n( "File or Folder Does Not Exist" );
+      description = i18n( "The specified file or folder <strong>%1</strong> "
         "does not exist." ).arg( dir );
       causes << cBug;
       solutions << sUpdate << sSysadmin;
@@ -661,13 +700,13 @@ QByteArray KIO::rawErrorDetail(int errorCode, const QString &errorText,
       break;
 
     case  KIO::ERR_DIR_ALREADY_EXIST:
-      errorName = i18n( "Directory Already Exists" );
-      description = i18n( "The requested directory could not be created because "
-        "a directory with the same name already exists." );
-      solutions << i18n( "Try moving the current directory out of the way first, "
+      errorName = i18n( "Folder Already Exists" );
+      description = i18n( "The requested folder could not be created because "
+        "a folder with the same name already exists." );
+      solutions << i18n( "Try moving the current folder out of the way first, "
         "and then try again." )
-        << i18n( "Delete the current directory and try again." )
-        << i18n( "Choose an alternate name for the new directory." );
+        << i18n( "Delete the current folder and try again." )
+        << i18n( "Choose an alternate name for the new folder." );
       break;
 
     case  KIO::ERR_UNKNOWN_HOST:
@@ -703,19 +742,19 @@ QByteArray KIO::rawErrorDetail(int errorCode, const QString &errorText,
       break;
 
     case  KIO::ERR_CANNOT_ENTER_DIRECTORY:
-      errorName = i18n( "Unable to Enter Directory" );
+      errorName = i18n( "Unable to Enter Folder" );
       description = i18n( "This means that an attempt to enter (in other words, "
-        "to open) the requested directory <strong>%1</strong> was rejected." )
+        "to open) the requested folder <strong>%1</strong> was rejected." )
         .arg( dir );
       causes << cAccess << cLocked;
       solutions << sAccess << sQuerylock << sSysadmin;
       break;
 
     case  KIO::ERR_PROTOCOL_IS_NOT_A_FILESYSTEM:
-      errorName = i18n( "Directory Listing Unavailable" );
+      errorName = i18n( "Folder Listing Unavailable" );
       techName = i18n( "Protocol %1 is not a Filesystem" ).arg( protocol );
       description = i18n( "This means that a request was made which requires "
-        "determining the contents of the directory, and the KDE program supporting "
+        "determining the contents of the folder, and the KDE program supporting "
         "this protocol is unable to do so." );
       causes << cBug;
       solutions << sUpdate << sBugreport;
@@ -724,7 +763,7 @@ QByteArray KIO::rawErrorDetail(int errorCode, const QString &errorText,
     case  KIO::ERR_CYCLIC_LINK:
       errorName = i18n( "Cyclic Link Detected" );
       description = i18n( "UNIX environments are commonly able to link a file or "
-        "directory to a separate name and/or location. KDE detected a link or "
+        "folder to a separate name and/or location. KDE detected a link or "
         "series of links that results in an infinite loop - i.e. the file was "
         "(perhaps in a roundabout way) linked to itself." );
       solutions << i18n( "Delete one part of the loop in order that it does not "
@@ -745,7 +784,7 @@ QByteArray KIO::rawErrorDetail(int errorCode, const QString &errorText,
     case  KIO::ERR_CYCLIC_COPY:
       errorName = i18n( "Cyclic Link Detected During Copy" );
       description = i18n( "UNIX environments are commonly able to link a file or "
-        "directory to a separate name and/or location. During the requested copy "
+        "folder to a separate name and/or location. During the requested copy "
         "operation, KDE detected a link or series of links that results in an "
         "infinite loop - i.e. the file was (perhaps in a roundabout way) linked "
         "to itself." );
@@ -788,13 +827,13 @@ QByteArray KIO::rawErrorDetail(int errorCode, const QString &errorText,
       break;
 
     case  KIO::ERR_NOT_FILTER_PROTOCOL:
-      errorName = i18n( "URL Resource Invalid." );
+      errorName = i18n( "URL Resource Invalid" );
       techName = i18n( "Protocol %1 is not a Filter Protocol" ).arg( protocol );
       description = i18n( "The <strong>U</strong>niform <strong>R</strong>esource "
         "<strong>L</strong>ocator (URL) that you entered did not refer to "
         "a valid mechanism of accessing the specific resource, "
         "<strong>%1%2</strong>." )
-        .arg( host != QString::null ? host + '/' : QString::null ).arg( dir );
+        .arg( !host.isNull() ? host + '/' : QString::null ).arg( dir );
       causes << i18n( "KDE is able to communicate through a protocol within a "
         "protocol. This request specified a protocol be used as such, however "
         "this protocol is not capable of such an action. This is a rare event, "
@@ -928,23 +967,23 @@ QByteArray KIO::rawErrorDetail(int errorCode, const QString &errorText,
       break;
 
     case  KIO::ERR_COULD_NOT_MKDIR:
-      errorName = i18n( "Could Not Create Directory" );
-      description = i18n( "An attempt to create the requested directory failed." );
-      causes << cAccess << i18n( "The location where the directory was to be created "
+      errorName = i18n( "Could Not Create Folder" );
+      description = i18n( "An attempt to create the requested folder failed." );
+      causes << cAccess << i18n( "The location where the folder was to be created "
         "may not exist." );
       if ( !isSlaveNetwork ) causes << cProtocol;
       solutions << i18n( "Retry the request." ) << sAccess;
       break;
 
     case  KIO::ERR_COULD_NOT_RMDIR:
-      errorName = i18n( "Could Not Remove Directory" );
-      description = i18n( "An attempt to remove the specified directory, "
+      errorName = i18n( "Could Not Remove Folder" );
+      description = i18n( "An attempt to remove the specified folder, "
         "<strong>%1</strong>, failed." ).arg( dir );
-      causes << i18n( "The specified directory may not exist." )
-        << i18n( "The specified directory may not be empty." )
+      causes << i18n( "The specified folder may not exist." )
+        << i18n( "The specified folder may not be empty." )
         << cAccess;
       if ( !isSlaveNetwork ) causes << cProtocol;
-      solutions << i18n( "Ensure that the directory exists and is empty, and try "
+      solutions << i18n( "Ensure that the folder exists and is empty, and try "
         "again." ) << sAccess;
       break;
 
@@ -994,7 +1033,7 @@ QByteArray KIO::rawErrorDetail(int errorCode, const QString &errorText,
       break;
 
     case  KIO::ERR_OUT_OF_MEMORY:
-      errorName = i18n( "Out Of Memory" );
+      errorName = i18n( "Out of Memory" );
       description = i18n( "The program on your computer which provides access "
         "to the <strong>%1</strong> protocol could not obtain the memory "
         "required to continue." ).arg( protocol );
@@ -1188,6 +1227,8 @@ QByteArray KIO::rawErrorDetail(int errorCode, const QString &errorText,
 #endif
 #ifdef HAVE_MNTENT_H
 #include <mntent.h>
+#elif HAVE_SYS_MNTENT_H
+#include <sys/mntent.h>
 #endif
 #ifdef HAVE_SYS_UCRED_H
 #include <sys/ucred.h>
@@ -1202,6 +1243,7 @@ QByteArray KIO::rawErrorDetail(int errorCode, const QString &errorText,
 #include <sys/mntctl.h>
 #include <sys/vmount.h>
 #include <sys/vfs.h>
+
 /* AIX does not prototype mntctl anywhere that I can find */
 #ifndef mntctl
 extern "C" {
@@ -1234,6 +1276,14 @@ extern "C" void endvfsent( );
 # endif
 #endif
 
+#ifndef FSTAB
+# ifdef _PATH_FSTAB
+#  define FSTAB _PATH_FSTAB
+# else
+#  define FSTAB "/etc/fstab"
+# endif
+#endif
+
 // There are (at least) four kind of APIs:
 // setmntent + getmntent + struct mntent (linux...)
 //             getmntent + struct mnttab
@@ -1325,7 +1375,6 @@ QString KIO::findDeviceMountPoint( const QString& filename )
 		}
 	}
 	fclose( mnttab );
-	devname.~QCString();
 #else
 
     char    realpath_buffer[MAXPATHLEN];
@@ -1337,6 +1386,8 @@ QString KIO::findDeviceMountPoint( const QString& filename )
       // succes, use result from realpath
       realname = realpath_buffer;
 
+    //kdDebug(7007) << "findDeviceMountPoint realname=" << realname << endl;
+
 #ifdef HAVE_GETMNTINFO
 
     struct statfs *mounted;
@@ -1433,8 +1484,6 @@ QString KIO::findDeviceMountPoint( const QString& filename )
 
     STRUCT_SETMNTENT mtab;
 
-    //kdDebug( 7007 ) << "realname " << realname << endl;
-
     /* Get the list of mounted file systems */
 
     if ((mtab = SETMNTENT(MNTTAB, "r")) == 0) {
@@ -1459,6 +1508,8 @@ QString KIO::findDeviceMountPoint( const QString& filename )
       // There may be symbolic links into the /etc/mnttab
       // So we have to find the real device name here as well!
       QCString device_name = FSNAME(me);
+      if (device_name.isEmpty() || (device_name == "none"))
+         continue;
 
       //kdDebug( 7007 ) << "device_name=" << device_name << endl;
 
@@ -1527,16 +1578,66 @@ static void check_mount_point(const char *mounttype,
 
 // returns the mount point, checks the mount state.
 // if ismanual == Wrong this function does not check the manual mount state
-static QString get_mount_info(const QString& filename, 
-    MountState& isautofs, MountState& isslow, MountState& ismanual)
+static QString get_mount_info(const QString& filename,
+    MountState& isautofs, MountState& isslow, MountState& ismanual,
+    QString& fstype)
 {
+    static bool gotRoot = false;
+    static dev_t rootDevice;
+
+    struct cachedDevice_t
+    {
+       dev_t device;
+       QString mountPoint;
+       MountState isautofs;
+       MountState isslow;
+       MountState ismanual;
+       QString fstype;
+    };
+    static struct cachedDevice_t *cachedDevice = 0;
+
+    if (!gotRoot)
+    {
+       struct stat stat_buf;
+       stat("/", &stat_buf);
+       gotRoot = true;
+       rootDevice = stat_buf.st_dev;
+    }
+
+    bool gotDevice = false;
+    struct stat stat_buf;
+    if (stat(QFile::encodeName(filename), &stat_buf) == 0)
+    {
+       gotDevice = true;
+       if (stat_buf.st_dev == rootDevice)
+       {
+          static const QString &root = KGlobal::staticQString("/");
+          isautofs = Wrong;
+          isslow = Wrong;
+          ismanual = Wrong;
+          fstype = QString::null; // ### do we need it?
+          return root;
+       }
+       if (cachedDevice && (stat_buf.st_dev == cachedDevice->device))
+       {
+          bool interestedInIsManual = ismanual != Wrong;
+          isautofs = cachedDevice->isautofs;
+          isslow = cachedDevice->isslow;
+          ismanual = cachedDevice->ismanual;
+          fstype = cachedDevice->fstype;
+          // Don't use the cache if it doesn't have the information we're looking for
+          if ( !interestedInIsManual || ismanual != Unseen )
+              return cachedDevice->mountPoint;
+       }
+    }
+
     char realname[MAXPATHLEN];
 
     memset(realname, 0, MAXPATHLEN);
 
     /* If the path contains symlinks, get the real name */
     if (realpath(QFile::encodeName(filename), realname) == 0) {
-        if( strlcpy(realname, QFile::encodeName(filename),MAXPATHLEN)>=MAXPATHLEN)
+        if( strlcpy(realname, QFile::encodeName(filename), MAXPATHLEN)>=MAXPATHLEN)
             return QString::null;
     }
 
@@ -1577,6 +1678,7 @@ static QString get_mount_info(const QString& filename,
         if ( is_my_mountpoint( mounted[i].f_mntonname, realname, max ) )
         {
             mountPoint = QFile::decodeName(mounted[i].f_mntonname);
+            fstype = QString::fromLatin1(mounttype);
             check_mount_point( mounttype, mounted[i].f_mntfromname,
                                isautofs, isslow );
             // keep going, looking for a potentially better one
@@ -1643,13 +1745,14 @@ static QString get_mount_info(const QString& filename,
             if ( is_my_mountpoint( mountedto, realname, max ) )
             {
                 mountPoint = QFile::decodeName(mountedto);
+                fstype = QString::fromLatin1(ent->vfsent_name);
                 check_mount_point(ent->vfsent_name, device_name, isautofs, isslow);
 
                 if (ismanual == Unseen)
                 {
                     // TODO: add check for ismanual, I couldn't find any way
                     // how to get the mount attribute from /etc/filesystems
-                    ismanual == Wrong;                
+                    ismanual == Wrong;
                 }
             }
 
@@ -1684,6 +1787,7 @@ static QString get_mount_info(const QString& filename,
         if ( is_my_mountpoint( MOUNTPOINT(me), realname, max ) )
         {
             mountPoint = QFile::decodeName( MOUNTPOINT(me) );
+            fstype = MOUNTTYPE(me);
             check_mount_point(MOUNTTYPE(me), FSNAME(me), isautofs, isslow);
             // we don't check if ismanual is Right, if /a/b is manually
             // mounted /a/b/c can't be automounted. At least IMO.
@@ -1693,11 +1797,11 @@ static QString get_mount_info(const QString& filename,
                 // Copy out the info that we need
                 QCString fsname_me = FSNAME(me);
                 QCString mounttype_me = MOUNTTYPE(me);
-            
+
                 STRUCT_SETMNTENT fstab;
-                // TODO: #define FSTAB (FSTAB_FILE?), important for Solaris
-                if ((fstab = SETMNTENT("/etc/fstab", "r")) == 0)
+                if ((fstab = SETMNTENT(FSTAB, "r")) == 0) {
                     continue;
+                }
 
                 bool found = false;
                 STRUCT_MNTENT fe;
@@ -1706,7 +1810,7 @@ static QString get_mount_info(const QString& filename,
                     if (fsname_me == FSNAME(fe))
                     {
                         found = true;
-                        if (HASMNTOPT(fe, "noauto") || 
+                        if (HASMNTOPT(fe, "noauto") ||
                             !strcmp(MOUNTTYPE(fe), "supermount"))
                             ismanual = Right;
                         break;
@@ -1727,27 +1831,64 @@ static QString get_mount_info(const QString& filename,
     if (isautofs == Right && isslow == Unseen)
         isslow = Right;
 
+    if (gotDevice)
+    {
+       if (!cachedDevice)
+          cachedDevice = new cachedDevice_t;
+
+       cachedDevice->device = stat_buf.st_dev;
+       cachedDevice->mountPoint = mountPoint;
+       cachedDevice->isautofs = isautofs;
+       cachedDevice->isslow = isslow;
+       cachedDevice->ismanual = ismanual;
+       cachedDevice->fstype = fstype;
+    }
+
     return mountPoint;
 }
 
 QString KIO::findPathMountPoint(const QString& filename)
 {
   MountState isautofs = Unseen, isslow = Unseen, ismanual = Wrong;
-  return get_mount_info(filename, isautofs, isslow, ismanual);
+  QString fstype;
+  return get_mount_info(filename, isautofs, isslow, ismanual, fstype);
 }
 
 bool KIO::manually_mounted(const QString& filename)
 {
   MountState isautofs = Unseen, isslow = Unseen, ismanual = Unseen;
-  QString mountPoint = get_mount_info(filename, isautofs, isslow, ismanual);
-  return (mountPoint != QString::null) && (ismanual == Right);
+  QString fstype;
+  QString mountPoint = get_mount_info(filename, isautofs, isslow, ismanual, fstype);
+  return !mountPoint.isNull() && (ismanual == Right);
 }
 
 bool KIO::probably_slow_mounted(const QString& filename)
 {
   MountState isautofs = Unseen, isslow = Unseen, ismanual = Wrong;
-  QString mountPoint = get_mount_info(filename, isautofs, isslow, ismanual);
-  return (mountPoint != QString::null) && (isslow == Right);
+  QString fstype;
+  QString mountPoint = get_mount_info(filename, isautofs, isslow, ismanual, fstype);
+  return !mountPoint.isNull() && (isslow == Right);
+}
+
+bool KIO::testFileSystemFlag(const QString& filename, FileSystemFlag flag)
+{
+  MountState isautofs = Unseen, isslow = Unseen, ismanual = Wrong;
+  QString fstype;
+  QString mountPoint = get_mount_info(filename, isautofs, isslow, ismanual, fstype);
+    kdDebug() << "testFileSystemFlag: fstype=" << fstype << endl;
+  if (mountPoint.isNull())
+      return false;
+  bool isMsDos = ( fstype == "msdos" || fstype == "fat" || fstype == "vfat" );
+  switch (flag)  {
+  case SupportsChmod:
+  case SupportsChown:
+  case SupportsUTime:
+  case SupportsSymlinks:
+      return !isMsDos; // it's amazing the number of things FAT doesn't support :)
+  case CaseInsensitive:
+      return isMsDos;
+  }
+  return false;
 }
 
 KIO::CacheControl KIO::parseCacheControl(const QString &cacheControl)
