@@ -33,7 +33,6 @@
 #include <kglobal.h>
 #include <kglobalsettings.h>
 #include <kstaticdeleter.h>
-#include <kdebugclasses.h>
 
 #include "kdirlister_p.h"
 
@@ -53,6 +52,8 @@ static KStaticDeleter<KDirListerCache> sd_KDirListerCache;
 KDirListerCache::KDirListerCache( int maxCount )
   : itemsCached( maxCount )
 {
+  kdDebug(7004) << "+KDirListerCache" << endl;
+
   itemsInUse.setAutoDelete( false );
   itemsCached.setAutoDelete( true );
   urlsCurrentlyListed.setAutoDelete( true );
@@ -69,6 +70,8 @@ KDirListerCache::KDirListerCache( int maxCount )
 
 KDirListerCache::~KDirListerCache()
 {
+  kdDebug(7004) << "-KDirListerCache" << endl;
+
   itemsInUse.setAutoDelete( true );
   itemsInUse.clear();
   itemsCached.clear();
@@ -93,6 +96,8 @@ void KDirListerCache::listDir( KDirLister* lister, const KURL& _u,
 #ifdef DEBUG_CACHE
   printDebug();
 #endif
+  kdDebug(7004) << k_funcinfo << lister << " url=" << _url
+                << " keep=" << _keep << " reload=" << _reload << endl;
 
   if ( !_keep )
   {
@@ -104,11 +109,16 @@ void KDirListerCache::listDir( KDirLister* lister, const KURL& _u,
 
     lister->d->rootFileItem = 0;
   }
-  else if ( lister->d->lstDirs.contains( _url ) )
+  else if ( lister->d->lstDirs.find( _url ) != lister->d->lstDirs.end() )
   {
     // stop the job listing _url for this lister
     stop( lister, _url );
 
+    // remove the _url as well, it will be added in a couple of lines again!
+    // forgetDirs with three args does not do this
+    // TODO: think about moving this into forgetDirs
+    lister->d->lstDirs.remove( lister->d->lstDirs.find( _url ) );
+
     // clear _url for lister
     forgetDirs( lister, _url, true );
 
@@ -288,6 +298,7 @@ void KDirListerCache::stop( KDirLister *lister )
 #ifdef DEBUG_CACHE
   printDebug();
 #endif
+  kdDebug(7004) << k_funcinfo << "lister: " << lister << endl;
   bool stopped = false;
 
   QDictIterator< QPtrList<KDirLister> > it( urlsCurrentlyListed );
@@ -400,6 +411,9 @@ void KDirListerCache::setAutoUpdate( KDirLister *lister, bool enable )
 
 void KDirListerCache::forgetDirs( KDirLister *lister )
 {
+  kdDebug(7004) << k_funcinfo << lister << endl;
+
+  emit lister->clear();
   // clear lister->d->lstDirs before calling forgetDirs(), so that
   // it doesn't contain things that itemsInUse doesn't. When emitting
   // the canceled signals, lstDirs must not contain anything that
@@ -412,13 +426,15 @@ void KDirListerCache::forgetDirs( KDirLister *lister )
   {
     forgetDirs( lister, *it, false );
   }
-
-  emit lister->clear();
 }
 
-void KDirListerCache::forgetDirs( KDirLister *lister, const KURL& url, bool notify )
+void KDirListerCache::forgetDirs( KDirLister *lister, const KURL& _url, bool notify )
 {
-  QString urlStr = url.url(-1);
+  kdDebug(7004) << k_funcinfo << lister << " _url: " << _url << endl;
+
+  KURL url( _url );
+  url.adjustPath( -1 );
+  QString urlStr = url.url();
   QPtrList<KDirLister> *holders = urlsCurrentlyHeld[urlStr];
   Q_ASSERT( holders );
   holders->removeRef( lister );
@@ -452,7 +468,7 @@ void KDirListerCache::forgetDirs( KDirLister *lister, const KURL& url, bool noti
 
       if ( notify )
       {
-        lister->d->lstDirs.remove( urlStr );
+        lister->d->lstDirs.remove( url );
         emit lister->clear( url );
       }
 
@@ -480,6 +496,8 @@ void KDirListerCache::forgetDirs( KDirLister *lister, const KURL& url, bool noti
 
 void KDirListerCache::updateDirectory( const KURL& _dir )
 {
+  kdDebug(7004) << k_funcinfo << _dir << endl;
+
   QString urlStr = _dir.url(-1);
   if ( !checkUpdate( urlStr ) )
     return;
@@ -560,10 +578,11 @@ bool KDirListerCache::checkUpdate( const QString& _dir )
     {
       item->complete = false;
       item->decAutoUpdate();
-      kdDebug(7004) << k_funcinfo << "directory " << _dir << " not in use, marked dirty." << endl;
+      // Hmm, this debug output might include login/password from the _dir URL.
+      //kdDebug(7004) << k_funcinfo << "directory " << _dir << " not in use, marked dirty." << endl;
     }
-    else
-      kdDebug(7004) << k_funcinfo << "aborted, directory " << _dir << " not in cache." << endl;
+    //else
+      //kdDebug(7004) << k_funcinfo << "aborted, directory " << _dir << " not in cache." << endl;
 
     return false;
   }
@@ -621,11 +640,13 @@ KFileItem* KDirListerCache::findByURL( const KDirLister *lister, const KURL& _u
 
 void KDirListerCache::FilesAdded( const KURL &dir )
 {
+  kdDebug(7004) << k_funcinfo << dir << endl;
   updateDirectory( dir );
 }
 
 void KDirListerCache::FilesRemoved( const KURL::List &fileList )
 {
+  kdDebug(7004) << k_funcinfo << endl;
   KURL::List::ConstIterator it = fileList.begin();
   for ( ; it != fileList.end() ; ++it )
   {
@@ -706,6 +727,7 @@ void KDirListerCache::FilesChanged( const KURL::List &fileList )
 
 void KDirListerCache::FileRenamed( const KURL &src, const KURL &dst )
 {
+  kdDebug(7004) << k_funcinfo << src.prettyURL() << " -> " << dst.prettyURL() << endl;
 #ifdef DEBUG_CACHE
   printDebug();
 #endif
@@ -714,9 +736,10 @@ void KDirListerCache::FileRenamed( const KURL &src, const KURL &dst )
   // (Note that looking into itemsInUse isn't good enough. One could rename a subdir in a view.)
   renameDir( src, dst );
 
-  QString oldUrl = src.url(-1);
   // Now update the KFileItem representing that file or dir (not exclusive with the above!)
-  KFileItem* fileitem = findByURL( 0, oldUrl );
+  KURL oldurl( src );
+  oldurl.adjustPath( -1 );
+  KFileItem* fileitem = findByURL( 0, oldurl );
   if ( fileitem )
   {
     fileitem->setURL( dst );
@@ -766,6 +789,8 @@ KDirListerCache* KDirListerCache::self()
 // _file can also be a directory being currently held!
 void KDirListerCache::slotFileDirty( const QString& _file )
 {
+  kdDebug(7004) << k_funcinfo << _file << endl;
+
   if ( !pendingUpdates[_file] )
   {
     KURL dir = KURL( _file );
@@ -790,6 +815,8 @@ void KDirListerCache::slotFileDirtyDelayed()
 {
   QString file = QString::fromUtf8( sender()->name() );
 
+  kdDebug(7004) << k_funcinfo << file << endl;
+
   // TODO: do it better: don't always create/delete the QTimer but reuse it.
   // Delete the timer after the parent directory is removed from the cache.
   pendingUpdates.remove( file );
@@ -807,6 +834,7 @@ void KDirListerCache::slotFileDirtyDelayed()
 
 void KDirListerCache::slotFileCreated( const QString& _file )
 {
+  kdDebug(7004) << k_funcinfo << _file << endl;
   // XXX: how to avoid a complete rescan here?
   KURL u;
   u.setPath( _file );
@@ -816,6 +844,7 @@ void KDirListerCache::slotFileCreated( const QString& _file )
 
 void KDirListerCache::slotFileDeleted( const QString& _file )
 {
+  kdDebug(7004) << k_funcinfo << _file << endl;
   KURL u;
   u.setPath( _file );
   FilesRemoved( u );
@@ -827,6 +856,8 @@ void KDirListerCache::slotEntries( KIO::Job *job, const KIO::UDSEntryList &entri
   url.adjustPath(-1);
   QString urlStr = url.url();
 
+  kdDebug(7004) << k_funcinfo << "new entries for " << url << endl;
+
   DirItem *dir = itemsInUse[urlStr];
   Q_ASSERT( dir );
 
@@ -908,7 +939,7 @@ void KDirListerCache::slotResult( KIO::Job* j )
   Q_ASSERT( listers );
 
   // move the directory to the held directories, do it before emitting
-  // the signals to make sure it exists in KDirListerCache in case someone 
+  // the signals to make sure it exists in KDirListerCache in case someone
   // calls listDir during the signal emission
   Q_ASSERT( !urlsCurrentlyHeld[jobUrlStr] );
   urlsCurrentlyHeld.insert( jobUrlStr, listers );
@@ -981,7 +1012,7 @@ void KDirListerCache::slotRedirection( KIO::Job *job, const KURL &url )
 
   for ( KDirLister *kdl = listers->first(); kdl; kdl = listers->next() )
   {
-    if ( kdl->d->url.cmp( oldUrl, true ) )
+    if ( kdl->d->url.equals( oldUrl, true ) )
     {
       kdl->d->rootFileItem = 0;
       kdl->d->url = newUrl;
@@ -1026,7 +1057,7 @@ void KDirListerCache::renameDir( const KURL &oldUrl, const KURL &newUrl )
   {
     goNext = true;
     DirItem* dir = itu.current();
-    KURL oldDirUrl = itu.currentKey();
+    KURL oldDirUrl ( itu.currentKey() );
     //kdDebug(7004) << "itemInUse: " << oldDirUrl.prettyURL() << endl;
     // Check if this dir is oldUrl, or a subfolder of it
     if ( oldUrl.isParentOf( oldDirUrl ) )
@@ -1092,7 +1123,7 @@ void KDirListerCache::emitRedirections( const KURL &oldUrl, const KURL &url )
        kdl->jobDone(job);
        emit kdl->canceled( oldUrl );
     }
-        
+
     urlsCurrentlyListed.insert( urlStr, listers );
   }
 
@@ -1107,11 +1138,11 @@ void KDirListerCache::emitRedirections( const KURL &oldUrl, const KURL &url )
     }
     urlsCurrentlyHeld.insert( urlStr, holders );
   }
-  
+
   if (listers)
   {
     updateDirectory( url );
-    
+
     // Tell the world about the new url
     for ( KDirLister *kdl = listers->first(); kdl; kdl = listers->next() )
     {
@@ -1389,7 +1420,7 @@ void KDirListerCache::deleteDir( const KURL& dirUrl )
   QDictIterator<DirItem> itu( itemsInUse );
   while ( itu.current() )
   {
-    KURL deletedUrl = itu.currentKey();
+    KURL deletedUrl( itu.currentKey() );
     if ( dirUrl.isParentOf( deletedUrl ) )
     {
       // stop all jobs for deletedUrl
@@ -1428,12 +1459,15 @@ void KDirListerCache::deleteDir( const KURL& dirUrl )
           else
           {
             bool treeview = kdl->d->lstDirs.count() > 1;
-            forgetDirs( kdl, deletedUrl, treeview );
             if ( !treeview )
             {
-              kdl->d->lstDirs.clear();
               emit kdl->clear();
+              kdl->d->lstDirs.clear();
             }
+            else
+              kdl->d->lstDirs.remove( kdl->d->lstDirs.find( deletedUrl ) );
+
+            forgetDirs( kdl, deletedUrl, treeview );
           }
         }
 
@@ -1445,6 +1479,8 @@ void KDirListerCache::deleteDir( const KURL& dirUrl )
 
       DirItem *dir = itemsInUse.take( deletedUrl.url() );
       Q_ASSERT( !dir );
+      if ( !dir ) // take didn't find it - move on
+          ++itu;
     }
     else
       ++itu;
@@ -1466,7 +1502,7 @@ void KDirListerCache::printDebug()
   QDictIterator<DirItem> itu( itemsInUse );
   for ( ; itu.current() ; ++itu ) {
       kdDebug(7004) << "   " << itu.currentKey() << "  URL: " << itu.current()->url
-                    << " rootItem: " << ( itu.current()->rootItem ? itu.current()->rootItem->url() : QString("NULL") )
+                    << " rootItem: " << ( itu.current()->rootItem ? itu.current()->rootItem->url() : KURL() )
                     << " autoUpdates refcount: " << itu.current()->autoUpdates
                     << " complete: " << itu.current()->complete
                   << ( itu.current()->lstItems ? QString(" with %1 items.").arg(itu.current()->lstItems->count()) : QString(" lstItems=NULL") ) << endl;
@@ -1511,6 +1547,8 @@ void KDirListerCache::printDebug()
 
 KDirLister::KDirLister( bool _delayedMimeTypes )
 {
+  kdDebug(7003) << "+KDirLister" << endl;
+
   d = new KDirListerPrivate;
 
   d->complete = true;
@@ -1525,6 +1563,8 @@ KDirLister::KDirLister( bool _delayedMimeTypes )
 
 KDirLister::~KDirLister()
 {
+  kdDebug(7003) << "-KDirLister" << endl;
+
   // Stop all running jobs
   stop();
   s_pCache->forgetDirs( this );
@@ -1643,9 +1683,9 @@ void KDirLister::emitChanges()
       if ( d->changes & MIME_FILTER )
       {
         oldMime = doMimeFilter( (*kit)->mimetype(), d->oldMimeFilter )
-		 && doMimeExcludeFilter( (*kit)->mimetype(), d->oldMimeExcludeFilter );
+                && doMimeExcludeFilter( (*kit)->mimetype(), d->oldMimeExcludeFilter );
         newMime = doMimeFilter( (*kit)->mimetype(), d->mimeFilter )
-		&& doMimeExcludeFilter( (*kit)->mimetype(), d->mimeExcludeFilter );
+                && doMimeExcludeFilter( (*kit)->mimetype(), d->mimeExcludeFilter );
 
         if ( oldMime && !newMime )
         {
@@ -1861,9 +1901,10 @@ bool KDirLister::doMimeFilter( const QString& mime, const QStringList& filters )
   if ( filters.isEmpty() )
     return true;
 
+  KMimeType::Ptr mimeptr = KMimeType::mimeType(mime);
   QStringList::ConstIterator it = filters.begin();
   for ( ; it != filters.end(); ++it )
-    if ( (*it) == mime )
+    if ( mimeptr->is(*it) )
       return true;
 
   return false;
@@ -1885,7 +1926,7 @@ bool KDirLister::doMimeExcludeFilter( const QString& mime, const QStringList& fi
 
 bool KDirLister::validURL( const KURL& _url ) const
 {
-  if ( _url.isMalformed() )
+  if ( !_url.isValid() )
   {
     if ( d->autoErrorHandling )
     {
@@ -1951,6 +1992,11 @@ void KDirLister::addRefreshItem( const KFileItem *item )
       d->lstRefreshItems = new KFileItemList;
 
     d->lstRefreshItems->append( item );
+  } else {
+    if ( !d->lstRemoveItems )
+      d->lstRemoveItems = new KFileItemList;
+
+      d->lstRemoveItems->append( item );//notify the user that the mimetype of a file changed, which doesn't match a filter or does match an exclude  filter
   }
 }
 
@@ -1965,6 +2011,9 @@ void KDirLister::emitItems()
   KFileItemList *tmpRefresh = d->lstRefreshItems;
   d->lstRefreshItems = 0;
 
+  KFileItemList *tmpRemove = d->lstRemoveItems;
+  d->lstRemoveItems = 0;
+
   if ( tmpNew )
   {
     emit newItems( *tmpNew );
@@ -1982,6 +2031,13 @@ void KDirLister::emitItems()
     emit refreshItems( *tmpRefresh );
     delete tmpRefresh;
   }
+
+  if ( tmpRemove )
+  {
+    for ( KFileItem *tmp = tmpRemove->first(); tmp; tmp = tmpRemove->next() )
+      emit deleteItem( tmp );
+    delete tmpRemove;
+  }
 }
 
 void KDirLister::emitDeleteItem( KFileItem *item )
@@ -2087,7 +2143,7 @@ void KDirLister::jobStarted(KIO::ListJob *job)
   jobData.percent = 0;
   jobData.processedSize = 0;
   jobData.totalSize = 0;
-  
+
   d->jobData.insert(job, jobData);
 }
 
