@@ -25,6 +25,10 @@ int KDirNotify::s_serial = 0;
 KDirNotify::KDirNotify()
   :  DCOPObject( QCString().sprintf("KDirNotify-%d", ++s_serial) )
 {
+   connectDCOPSignal(0, "KDirNotify", "FilesAdded(KURL)", "FilesAdded(KURL)", false);
+   connectDCOPSignal(0, "KDirNotify", "FilesRemoved(KURL::List)", "FilesRemoved(KURL::List)", false);
+   connectDCOPSignal(0, "KDirNotify", "FilesChanged(KURL::List)", "FilesChanged(KURL::List)", false);
+   connectDCOPSignal(0, "KDirNotify", "FileRenamed(KURL,KURL)", "FileRenamed(KURL,KURL)", false);
 }
 
 void KDirNotify::FileRenamed( const KURL &, const KURL & )
