@@ -34,14 +34,14 @@
 #include <config.h>
 
 #ifdef HAVE_DNOTIFY
-//#include <sys/stat.h>
 #include <unistd.h>
-//#include <time.h>
+#include <time.h>
 #include <fcntl.h>
 #include <signal.h>
 #include <errno.h>
 #endif
 
+#include <sys/stat.h>
 #include <assert.h>
 #include <qdir.h>
 #include <qfile.h>
@@ -279,7 +279,7 @@ void KDirWatchPrivate::slotActivated()
 }
 
 /* In DNOTIFY mode, only entries which are marked dirty are scanned.
- * We first need to mark all yet nonexistant, but possible created
+ * We first need to mark all yet nonexistent, but possible created
  * entries as dirty...
  */
 void KDirWatchPrivate::Entry::propagate_dirty()
@@ -451,6 +451,24 @@ bool KDirWatchPrivate::useDNotify(Entry* e)
     e->dn_dirty = false;
     if (e->m_status == Normal) {
       int fd = open(QFile::encodeName(e->path).data(), O_RDONLY);
+      // Migrate fd to somewhere above 128. Some libraries have 
+      // constructs like:
+      //    fd = socket(...)
+      //    if (fd > ARBITRARY_LIMIT)
+      //       return error;
+      //
+      // Since programs might end up using a lot of KDirWatch objects 
+      // for a rather long time the above braindamage could get
+      // triggered.
+      // 
+      // By moving the kdirwatch fd's to > 128, calls like socket() will keep
+      // returning fd's < ARBITRARY_LIMIT for a bit longer.
+      int fd2 = fcntl(fd, F_DUPFD, 128);
+      if (fd2 >= 0)
+      {
+        close(fd);
+        fd = fd2;
+      }
       if (fd<0) {
 	e->m_mode = UnknownMode;
 	return false;
@@ -568,28 +586,31 @@ void KDirWatchPrivate::addEntry(KDirWatch* instance, const QString& _path,
 
   // we have a new path to watch
 
-  QFileInfo info(path);
+  struct stat stat_buf;
+  bool exists = (stat(QFile::encodeName(path), &stat_buf) == 0);
 
   Entry newEntry;
   m_mapEntries.insert( path, newEntry );
   // the insert does a copy, so we have to use <e> now
   Entry* e = &(m_mapEntries[path]);
 
-  if (info.exists()) {
-    e->isDir = info.isDir();
+  if (exists) {
+    e->isDir = S_ISDIR(stat_buf.st_mode);
 
     if (e->isDir && !isDir)
       qWarning("KDirWatch: %s is a directory. Use addDir!", path.ascii());
     else if (!e->isDir && isDir)
       qWarning("KDirWatch: %s is a file. Use addFile!", path.ascii());
 
-    e->m_ctime = info.lastModified();
+    e->m_ctime = stat_buf.st_ctime;
     e->m_status = Normal;
+    e->m_nlink = stat_buf.st_nlink;
   }
   else {
     e->isDir = isDir;
-    e->m_ctime = QDateTime(); // invalid
+    e->m_ctime = invalid_ctime;
     e->m_status = NonExistent;
+    e->m_nlink = 0;
   }
 
   e->path = path;
@@ -754,7 +775,7 @@ bool KDirWatchPrivate::stopEntryScan( KDirWatch* instance, Entry* e)
 
   if (stillWatching == 0) {
     // if nobody is interested, we don't watch
-    e->m_ctime = QDateTime(); // invalid
+    e->m_ctime = invalid_ctime; // invalid
     //    e->m_status = Normal;
   }
   return true;
@@ -785,14 +806,17 @@ bool KDirWatchPrivate::restartEntryScan( KDirWatch* instance, Entry* e,
   int ev = NoChange;
   if (wasWatching == 0) {
     if (!notify) {
-      QFileInfo info(e->path);
-      if (info.exists()) {
-	e->m_ctime = info.lastModified();
+      struct stat stat_buf;
+      bool exists = (stat(QFile::encodeName(e->path), &stat_buf) == 0);
+      if (exists) {
+	e->m_ctime = stat_buf.st_ctime;
 	e->m_status = Normal;
+        e->m_nlink = stat_buf.st_nlink;
       }
       else {
-	e->m_ctime = QDateTime(); // invalid
+	e->m_ctime = invalid_ctime;
 	e->m_status = NonExistent;
+        e->m_nlink = 0;
       }
     }
     e->msecLeft = 0;
@@ -870,18 +894,22 @@ int KDirWatchPrivate::scanEntry(Entry* e)
     e->msecLeft += e->freq;
   }
 
-  QFileInfo info(e->path);
-  if (info.exists()) {
+  struct stat stat_buf;
+  bool exists = (stat(QFile::encodeName(e->path), &stat_buf) == 0);
+  if (exists) {
 
     if (e->m_status == NonExistent) {
-      e->m_ctime = info.lastModified();
+      e->m_ctime = stat_buf.st_ctime;
       e->m_status = Normal;
+      e->m_nlink = stat_buf.st_nlink;
       return Created;
     }
 
-    if ( e->m_ctime.isValid() &&
-	 (info.lastModified() != e->m_ctime) ) {
-      e->m_ctime = info.lastModified();
+    if ( (e->m_ctime != invalid_ctime) && 
+	 ((stat_buf.st_ctime != e->m_ctime) || 
+	  (stat_buf.st_nlink != (nlink_t) e->m_nlink)) ) {
+      e->m_ctime = stat_buf.st_ctime;
+      e->m_nlink = stat_buf.st_nlink;
       return Changed;
     }
 
@@ -890,10 +918,11 @@ int KDirWatchPrivate::scanEntry(Entry* e)
 
   // dir/file doesn't exist
 
-  if (!e->m_ctime.isValid())
+  if (e->m_ctime == invalid_ctime)
     return NoChange;
 
-  e->m_ctime = QDateTime(); // invalid
+  e->m_ctime = invalid_ctime;
+  e->m_nlink = 0;
   e->m_status = NonExistent;
 
   return Deleted;
@@ -1096,6 +1125,9 @@ void KDirWatchPrivate::checkFAMEvent(FAMEvent* fe)
   if ( *(fe->filename) == '.') {
     if (strncmp(fe->filename, ".X.err", 6) == 0) return;
     if (strncmp(fe->filename, ".xsession-errors", 16) == 0) return;
+    // fontconfig updates the cache on every KDE app start 
+    // (inclusive kio_thumbnail slaves)
+    if (strncmp(fe->filename, ".fonts.cache", 12) == 0) return;
   }
 
   Entry* e = 0;
@@ -1313,8 +1345,10 @@ QDateTime KDirWatch::ctime( const QString &_path )
 
   if (!e)
     return QDateTime();
-  else
-    return e->m_ctime;
+
+  QDateTime result;
+  result.setTime_t(e->m_ctime);
+  return result;
 }
 
 void KDirWatch::removeDir( const QString& _path )
@@ -1401,6 +1435,20 @@ void KDirWatch::setDeleted( const QString & _file )
   emit deleted( _file );
 }
 
+KDirWatch::Method KDirWatch::internalMethod()
+{
+#ifdef HAVE_FAM
+  if (d->use_fam)
+     return KDirWatch::FAM;
+#endif
+#ifdef HAVE_DNOTIFY
+  if (d->supports_dnotify)
+     return KDirWatch::DNotify;
+#endif      
+  return KDirWatch::Stat;
+}
+
+
 #include "kdirwatch.moc"
 #include "kdirwatch_p.moc"
 
