@@ -11,6 +11,10 @@
 #include <fam.h>
 #endif
 
+#include <ctime>
+
+#define invalid_ctime ((time_t)-1)
+
 /* KDirWatchPrivate is a singleton and does the watching
  * for every KDirWatch instance in the application.
  */
@@ -36,7 +40,9 @@ public:
   {
   public:
     // the last observed modification time
-    QDateTime m_ctime;
+    time_t m_ctime;
+    // the last observed link count
+    int m_nlink; 
     entryStatus m_status;
     entryMode m_mode;
     bool isDir;
@@ -99,7 +105,6 @@ public:
   QTimer *timer;
   EntryMap m_mapEntries;
 
-private:
   int freq;
   int statEntries;
   int m_nfsPollInterval, m_PollInterval;
