@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- *	$Id: kemailsettings.cpp,v 1.11 2002/06/17 17:19:59 garbanzo Exp $
+ *	$Id: kemailsettings.cpp,v 1.14 2003/11/14 12:34:09 waba Exp $
  */
 
 #include "kemailsettings.h"
@@ -130,7 +130,7 @@ void KEMailSettings::setSetting(KEMailSettings::Setting s, const QString  &v)
 	p->m_pConfig->setGroup(QString("PROFILE_")+p->m_sCurrentProfile);
 	switch (s) {
 		case ClientProgram: {
-			p->m_pConfig->writeEntry("EmailClient", v);
+			p->m_pConfig->writePathEntry("EmailClient", v);
 			break;
 		}
 		case ClientTerminal: {
@@ -207,7 +207,6 @@ void KEMailSettings::setSetting(KEMailSettings::Setting s, const QString  &v)
 
 void KEMailSettings::setDefault(const QString &s)
 {
-	kdDebug() << "setDefault called with " << s << endl;
 	p->m_pConfig->setGroup("Defaults");
 	p->m_pConfig->writeEntry("Profile", s);
 	p->m_pConfig->sync();
@@ -253,14 +252,13 @@ KEMailSettings::KEMailSettings()
 
 	p->m_pConfig->setGroup("Defaults");
 	p->m_sDefaultProfile=p->m_pConfig->readEntry("Profile", i18n("Default"));
-	if (p->m_sDefaultProfile != QString::null) {
+	if (!p->m_sDefaultProfile.isNull()) {
 		if (!p->m_pConfig->hasGroup(QString("PROFILE_")+p->m_sDefaultProfile))
 			setDefault(i18n("Default"));
 		else
 			setDefault(p->m_sDefaultProfile);
 	} else {
 			if (p->profiles.count()) {
-				kdDebug() << "WE ALREADY HAVE PROFILES DAMNIT" << endl;
 				setDefault(p->profiles[0]);
 			} else
 				setDefault(i18n("Default"));
