@@ -17,7 +17,7 @@
    the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
    Boston, MA 02111-1307, USA.
 */
-// $Id: kfileitem.cpp,v 1.140.2.3 2003/07/24 11:06:11 waba Exp $
+// $Id: kfileitem.cpp,v 1.152 2004/01/12 19:04:11 mueller Exp $
 
 #include <sys/time.h>
 #include <pwd.h>
@@ -34,15 +34,21 @@
 #include <qmap.h>
 #include <qstylesheet.h>
 
-#include <klargefile.h>
-#include <kiconloader.h>
-#include <kglobal.h>
 #include <kdebug.h>
+#include <kfilemetainfo.h>
+#include <kfileshare.h>
+#include <kglobal.h>
+#include <kglobalsettings.h>
+#include <kiconloader.h>
+#include <klargefile.h>
 #include <klocale.h>
 #include <kmimetype.h>
 #include <krun.h>
-#include <kfilemetainfo.h>
-#include <kfileshare.h>
+
+class KFileItem::KFileItemPrivate {
+	public:
+		QString iconName;
+};
 
 KFileItem::KFileItem( const KIO::UDSEntry& _entry, const KURL& _url,
                       bool _determineMimeTypeOnDemand, bool _urlIsDirectory ) :
@@ -101,6 +107,10 @@ KFileItem::KFileItem( const KIO::UDSEntry& _entry, const KURL& _url,
         case KIO::UDS_LINK_DEST:
           m_bLink = !(*it).m_str.isEmpty(); // we don't store the link dest
           break;
+        case KIO::UDS_ICON_NAME:
+	  d=new KFileItemPrivate();
+	  d->iconName=(*it).m_str;
+	  break;
     }
   }
   // avoid creating these QStrings again and again
@@ -131,15 +141,18 @@ KFileItem::KFileItem( const KURL &url, const QString &mimeType, mode_t mode )
 :  m_url( url ),
   m_strName( url.fileName() ),
   m_strText( KIO::decodeFileName( m_strName ) ),
+  m_pMimeType( 0 ),
   m_fileMode( mode ),
   m_permissions( KFileItem::Unknown ),
   m_bMarked( false ),
   m_bLink( false ),
   m_bIsLocalURL( url.isLocalFile() ),
-  m_bMimeTypeKnown( false ),
+  m_bMimeTypeKnown( !mimeType.isEmpty() ),
   d(0L)
 {
-  m_pMimeType = KMimeType::mimeType( mimeType );
+  if (m_bMimeTypeKnown)
+    m_pMimeType = KMimeType::mimeType( mimeType );
+
   init( false );
 }
 
@@ -151,6 +164,7 @@ KFileItem::KFileItem( const KFileItem & item ) :
 
 KFileItem::~KFileItem()
 {
+  delete d;
 }
 
 void KFileItem::init( bool _determineMimeTypeOnDemand )
@@ -190,23 +204,24 @@ void KFileItem::init( bool _determineMimeTypeOnDemand )
         // While we're at it, store the times
         m_time[ Modification ] = buf.st_mtime;
         m_time[ Access ] = buf.st_atime;
+        if ( m_fileMode == KFileItem::Unknown )
+          m_fileMode = mode & S_IFMT; // extract file type
+        if ( m_permissions == KFileItem::Unknown )
+          m_permissions = mode & 07777; // extract permissions
       }
     }
-    if ( m_fileMode == KFileItem::Unknown )
-      m_fileMode = mode & S_IFMT; // extract file type
-    if ( m_permissions == KFileItem::Unknown )
-      m_permissions = mode & 07777; // extract permissions
   }
 
   // determine the mimetype
   if (!m_pMimeType )
   {
+      bool accurate = false;
       m_pMimeType = KMimeType::findByURL( m_url, m_fileMode, m_bIsLocalURL,
                                           // use fast mode if not mimetype on demand
-                                          _determineMimeTypeOnDemand );
+                                          _determineMimeTypeOnDemand, &accurate );
       // if we didn't use fast mode, or if we got a result, then this is the mimetype
       // otherwise, determineMimeType will be able to do better.
-      m_bMimeTypeKnown = (!_determineMimeTypeOnDemand) || (m_pMimeType->name() != KMimeType::defaultMimeType());
+      m_bMimeTypeKnown = (!_determineMimeTypeOnDemand) || accurate;
   }
 
 }
@@ -240,7 +255,12 @@ void KFileItem::refreshMimeType()
 void KFileItem::setURL( const KURL &url )
 {
   m_url = url;
-  m_strName = url.fileName();
+  setName( url.fileName() );
+}
+
+void KFileItem::setName( const QString& name )
+{
+  m_strName = name;
   m_strText = KIO::decodeFileName( m_strName );
 }
 
@@ -403,6 +423,7 @@ QString KFileItem::mimeComment()
 
 QString KFileItem::iconName()
 {
+  if (d && (!d->iconName.isEmpty())) return d->iconName;
   return determineMimeType()->icon(m_url, m_bIsLocalURL);
 }
 
@@ -423,7 +444,7 @@ int KFileItem::overlays() const
   {
     if (KFileShare::isDirectoryShared( m_url.path() ))
     {
-      kdDebug()<<"KFileShare::isDirectoryShared : "<<m_url.path()<<endl;
+      //kdDebug()<<"KFileShare::isDirectoryShared : "<<m_url.path()<<endl;
       _state |= KIcon::ShareOverlay;
     }
   }
@@ -435,10 +456,14 @@ int KFileItem::overlays() const
 
 QPixmap KFileItem::pixmap( int _size, int _state ) const
 {
+  if (d && (!d->iconName.isEmpty()))
+     return DesktopIcon(d->iconName,_size,_state);
+
   if ( !m_pMimeType )
   {
     static const QString & defaultFolderIcon =
        KGlobal::staticQString(KMimeType::mimeType( "inode/directory" )->KServiceType::icon());
+
     if ( S_ISDIR( m_fileMode ) )
      return DesktopIcon( defaultFolderIcon, _size, _state );
 
@@ -458,9 +483,10 @@ QPixmap KFileItem::pixmap( int _size, int _state ) const
   // See also the relevant code in overlays, which adds the zip overlay.
   if ( mime->name() == "application/x-gzip" && m_url.fileName().right(3) == ".gz" )
   {
-      QString subFileName = m_url.path().left( m_url.path().length() - 3 );
+      KURL sf;
+      sf.setPath( m_url.path().left( m_url.path().length() - 3 ) );
       //kdDebug() << "KFileItem::pixmap subFileName=" << subFileName << endl;
-      mime = KMimeType::findByURL( subFileName, 0, m_bIsLocalURL );
+      mime = KMimeType::findByURL( sf, 0, m_bIsLocalURL );
   }
 
   QPixmap p = mime->pixmap( m_url, KIcon::Desktop, _size, _state );
@@ -493,7 +519,7 @@ bool KFileItem::isReadable() const
 
 bool KFileItem::isDir() const
 {
-  if ( !m_bMimeTypeKnown && !m_guessedMimeType.isEmpty() )
+  if ( m_fileMode == KFileItem::Unknown )
   {
     kdDebug() << " KFileItem::isDir can't say -> false " << endl;
     return false; // can't say for sure, so no
@@ -683,6 +709,10 @@ void KFileItem::assign( const KFileItem & item )
 
     // We had a mimetype previously (probably), so we need to re-determine it
     determineMimeType();
+    if (item.d) {
+	d=new KFileItemPrivate;
+	d->iconName=item.d->iconName;
+    }
 }
 
 void KFileItem::setExtraData( const void *key, void *value )
@@ -770,9 +800,10 @@ void KFileItem::setMetaInfo( const KFileMetaInfo & info )
 
 const KFileMetaInfo & KFileItem::metaInfo(bool autoget, int) const
 {
-    if ( autoget && m_url.isLocalFile() && !m_metaInfo.isValid() )
+    if ( autoget && !m_metaInfo.isValid() &&
+         KGlobalSettings::showFilePreview(m_url) )
     {
-        m_metaInfo = KFileMetaInfo( m_url.path(), mimetype() );
+        m_metaInfo = KFileMetaInfo( m_url, mimetype() );
     }
 
     return m_metaInfo;
@@ -781,3 +812,23 @@ const KFileMetaInfo & KFileItem::metaInfo(bool autoget, int) const
 void KFileItem::virtual_hook( int, void* )
 { /*BASE::virtual_hook( id, data );*/ }
 
+QDataStream & operator<< ( QDataStream & s, const KFileItem & a )
+{
+    // We don't need to save/restore anything that refresh() invalidates,
+    // since that means we can re-determine those by ourselves.
+    s << a.m_url;
+    s << a.m_strName;
+    s << a.m_strText;
+    return s;
+}
+
+QDataStream & operator>> ( QDataStream & s, KFileItem & a )
+{
+    s >> a.m_url;
+    s >> a.m_strName;
+    s >> a.m_strText;
+    a.m_bIsLocalURL = a.m_url.isLocalFile();
+    a.m_bMimeTypeKnown = false;
+    a.refresh();
+    return s;
+}
