@@ -17,7 +17,7 @@
  *  the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  *  Boston, MA 02111-1307, USA.
  *
- *  $Id: kfilemetainfo.cpp,v 1.54.2.6 2003/05/17 11:19:23 mueller Exp $
+ *  $Id: kfilemetainfo.cpp,v 1.78 2004/01/12 19:15:09 mueller Exp $
  */
 
 #include <qshared.h>
@@ -54,10 +54,10 @@ public:
     Data() : mimeTypeInfo( 0L )
     {}
 
-    ~Data() 
+    ~Data()
     {
-        if ( this == null )
-            delete const_cast<KFileMimeTypeInfo::ItemInfo*>( mimeTypeInfo );
+        if ( this == null ) // only the null item owns its mimeTypeInfo
+            delete mimeTypeInfo;
     }
 
     const KFileMimeTypeInfo::ItemInfo*  mimeTypeInfo;
@@ -73,7 +73,7 @@ public:
 };
 
 //this is our null data
-KFileMetaInfoItem::Data* KFileMetaInfoItem::Data::null = 0L;
+KFileMetaInfoItem::Data* KFileMetaInfoItem::Data::null;
 static KStaticDeleter<KFileMetaInfoItem::Data> sd_KFileMetaInfoItemData;
 
 KFileMetaInfoItem::Data* KFileMetaInfoItem::Data::makeNull()
@@ -99,7 +99,7 @@ KFileMetaInfoItem::KFileMetaInfoItem( const KFileMimeTypeInfo::ItemInfo* mti,
 
 KFileMetaInfoItem::KFileMetaInfoItem( const KFileMetaInfoItem& item )
 {
-    // operator= does everything that's neccessary
+    // operator= does everything that's necessary
     d = Data::makeNull();
     *this = item;
 }
@@ -192,6 +192,11 @@ QVariant::Type KFileMetaInfoItem::type() const
     return d->mimeTypeInfo->type();
 }
 
+uint KFileMetaInfoItem::unit() const
+{
+    return d->mimeTypeInfo->unit();
+}
+
 bool KFileMetaInfoItem::isModified() const
 {
     return d->dirty;
@@ -265,16 +270,17 @@ void KFileMetaInfoItem::deref()
 class KFileMetaInfo::Data : public QShared
 {
 public:
-    Data(const QString& _path, uint _what)
+    Data(const KURL& _url, uint _what)
         : QShared(),
-          path(_path),
-          what(_what)
+          url(_url),
+          what(_what),
+          mimeTypeInfo( 0L )
     {}
 
     // wee use this one for the streaming operators
     Data() {};
 
-    QString                           path;
+    KURL                              url;
     uint                              what;
     QMap<QString, KFileMetaInfoGroup> groups;
     const KFileMimeTypeInfo*          mimeTypeInfo;
@@ -287,13 +293,27 @@ public:
 
 KFileMetaInfo::KFileMetaInfo( const QString& path, const QString& mimeType,
                               uint what )
-    : d(new Data( path, what ) )
 {
-//    kdDebug(7033) << "KFileMetaInfo( " << path << ", " << mimeType << ", what )\n";
+    KURL u;
+
+    u.setPath(path);
+    init(u, mimeType, what);
+}
+
+KFileMetaInfo::KFileMetaInfo( const KURL& url, const QString& mimeType,
+                              uint what )
+{
+    init(url, mimeType, what);
+}
+
+void KFileMetaInfo::init( const KURL& url, const QString& mimeType,
+                          uint what )
+{
+    d = new Data( url, what );
 
     QString mT;
     if (mimeType.isEmpty())
-        mT = KMimeType::findByPath(path)->name();
+        mT = KMimeType::findByURL(url)->name();
     else
         mT = mimeType;
 
@@ -317,7 +337,7 @@ KFileMetaInfo::KFileMetaInfo( const QString& path, const QString& mimeType,
 
 KFileMetaInfo::KFileMetaInfo( const KFileMetaInfo& original )
 {
-    // operator= does everything that's neccessary
+    // operator= does everything that's necessary
     d = Data::makeNull();
     *this = original;
 }
@@ -509,18 +529,18 @@ bool KFileMetaInfo::applyChanges()
             doit = true;
 
         else
-    {
-        QStringList keys = it.data().keys();
-            for (QStringList::Iterator it2 = keys.begin(); it2!=keys.end(); ++it2)
         {
-                if ( (*it)[*it2].isModified() )
+            QStringList keys = it.data().keys();
+            for (QStringList::Iterator it2 = keys.begin(); it2!=keys.end(); ++it2)
             {
-                doit = true;
-                break;
+                if ( (*it)[*it2].isModified() )
+                {
+                    doit = true;
+                    break;
+                }
             }
         }
     }
-    }
 
     if (!doit)
     {
@@ -682,7 +702,12 @@ KFileMetaInfoGroup KFileMetaInfo::appendGroup(const QString& name)
 
 QString KFileMetaInfo::path() const
 {
-    return d->path;
+    return d->url.isLocalFile() ? d->url.path() : QString::null;
+}
+
+KURL KFileMetaInfo::url() const
+{
+    return d->url;
 }
 
 void KFileMetaInfo::ref()
@@ -698,7 +723,7 @@ void KFileMetaInfo::deref()
     // null is 0L when it hasn't been initialized and d is never 0L.
     if ((d != Data::null) && d->deref())
     {
-//        kdDebug(7033) << "metainfo object for " << d->path << " is finally deleted\n";
+//        kdDebug(7033) << "metainfo object for " << d->url.path << " is finally deleted\n";
         delete d;
     }
 
@@ -714,7 +739,7 @@ KFileMetaInfo::Data* KFileMetaInfo::Data::makeNull()
         // We deliberately do not reset "null" after it has been destroyed!
         // Otherwise we will run into problems later in ~KFileMetaInfoItem
         // where the d-pointer is compared against null.
-	null = sd_KFileMetaInfoData.setObject( new KFileMetaInfo::Data(QString::null, 0) );
+	null = sd_KFileMetaInfoData.setObject( new KFileMetaInfo::Data(KURL(), 0) );
     return null;
 }
 
@@ -848,13 +873,13 @@ void KFilePlugin::appendItem(KFileMetaInfoGroup& group, const QString& key, QVar
 ///////////////////////////////////////////////////////////////////
 
 
-KFileMetaInfoProvider * KFileMetaInfoProvider::s_self = 0;
-KStaticDeleter<KFileMetaInfoProvider> sd;
+KFileMetaInfoProvider * KFileMetaInfoProvider::s_self;
+static KStaticDeleter<KFileMetaInfoProvider> sd;
 
 KFileMetaInfoProvider * KFileMetaInfoProvider::self()
 {
     if ( !s_self )
-        s_self = sd.setObject( new KFileMetaInfoProvider() );
+        s_self = sd.setObject( s_self, new KFileMetaInfoProvider() );
 
     return s_self;
 }
@@ -867,7 +892,6 @@ KFileMetaInfoProvider::KFileMetaInfoProvider()
 
 KFileMetaInfoProvider::~KFileMetaInfoProvider()
 {
-    s_self = 0;
     sd.setObject( 0 );
 }
 
@@ -1036,7 +1060,7 @@ KFileMetaInfoGroup::KFileMetaInfoGroup( const QString& name,
 
 KFileMetaInfoGroup::KFileMetaInfoGroup( const KFileMetaInfoGroup& original )
 {
-    // operator= does everything that's neccessary
+    // operator= does everything that's necessary
     d = Data::makeNull();
     *this = original;
 }
@@ -1124,6 +1148,11 @@ QStringList KFileMetaInfoGroup::keys() const
     return list;
 }
 
+QString KFileMetaInfoGroup::translatedName() const
+{
+    return d->mimeTypeInfo->groupInfo(d->name)->translatedName();
+}
+
 QStringList KFileMetaInfoGroup::supportedKeys() const
 {
       return d->mimeTypeInfo->groupInfo(d->name)->supportedKeys();
@@ -1475,6 +1504,20 @@ QString KFileMimeTypeInfo::ItemInfo::string(const QVariant& value, bool mangle)
                 s = KGlobal::locale()->formatNumber( value.toInt() , 0);
             break;
 
+#if QT_VERSION >= 0x030200
+        case QVariant::LongLong :
+            s = KGlobal::locale()->formatNumber( value.toLongLong(), 0 );
+            break;
+
+	case QVariant::ULongLong :
+            if ( unit() == KFileMimeTypeInfo::Bytes )
+                return KIO::convertSize( value.toULongLong() );
+            else if ( unit() == KFileMimeTypeInfo::KiloBytes )
+                return KIO::convertSizeFromKB( value.toULongLong() );
+            else
+                s = KGlobal::locale()->formatNumber( value.toULongLong(), 0 );
+            break;
+#endif
         case QVariant::UInt :
             s = KGlobal::locale()->formatNumber( value.toUInt() , 0);
             break;
@@ -1648,7 +1691,7 @@ QDataStream& operator <<(QDataStream& s, const KFileMetaInfo& info )
     s << isValid;
     if (isValid)
     {
-        s << d->path
+        s << d->url
           << d->what
           << d->groups
           << d->mimeTypeInfo->mimeType();
@@ -1674,7 +1717,7 @@ QDataStream& operator >>(QDataStream& s, KFileMetaInfo& info )
     info.d = new KFileMetaInfo::Data();
     info.ref();
 
-    s >> info.d->path
+    s >> info.d->url
       >> info.d->what
       >> info.d->groups
       >> mimeType;
