@@ -60,7 +60,7 @@ static KStaticDeleter<KFileSharePrivate> kstFileShare;
 KFileSharePrivate* KFileSharePrivate::self()
 {
    if (!_self)
-      _self = kstFileShare.setObject(new KFileSharePrivate());
+      _self = kstFileShare.setObject(_self, new KFileSharePrivate());
    return _self;
 }
 
@@ -97,13 +97,13 @@ void KFileShare::readConfig() // static
     QString line;
     int length;
     do {
-        length = proc.fgets(line, true);
+        length = proc.readln(line, true);
 	if ( length > 0 )
 	{
             if ( line[length-1] != '/' )
                 line += '/';
             s_shareList->append(line);
-            kdDebug() << "Shared dir:" << line << endl;
+            kdDebug(7000) << "Shared dir:" << line << endl;
         }
     } while (length > -1);
 
