@@ -31,18 +31,19 @@
 #define KMIMEMAGIC_H
 
 #include <qstring.h>
+#include <kdemacros.h>
 
 class KMimeMagic; // see below (read this one first)
 
 /**
  * @deprecated Use KMimeType::findByContent() instead
- * May be removed in KDE 4.0
- * Returned by @ref KMimeMagic @p find...Type methods.
+ * May be removed in KDE 4.0.
+ * Returned by KMimeMagic @p find...Type methods.
  *
  * It contains the mimetype and the encoding of
  * the file or buffer read.
  */
-class KMimeMagicResult
+class KDE_DEPRECATED KMimeMagicResult
 {
 public:
   KMimeMagicResult() { m_iAccuracy = 100; }
@@ -75,7 +76,7 @@ protected:
 
 /**
  * @deprecated Use KMimeType::findByContent() instead
- * May be removed in KDE 4.0
+ * May be removed in KDE 4.0.
  * Determine auto-magically the type of file,
  * not only by using its extension, but also by reading its contents.
  *
@@ -91,13 +92,13 @@ protected:
  * See kmimemagic.cpp for a full discussion on this issue.
  *
  * The basic usage of KMimeMagic is :
- * @li Get a pointer to it, using @ref KMimeMagic::self().
+ * @li Get a pointer to it, using KMimeMagic::self().
  * @li Use it for any file or buffer you want, using one of the three
  * @p find...Type() methods.
  *
- * The result is contained in the class @ref KMimeMagicResult.
+ * The result is contained in the class KMimeMagicResult.
  */
-class KMimeMagic
+class KDE_DEPRECATED KMimeMagic
 {
 public:
   /**
@@ -169,7 +170,7 @@ public:
   KMimeMagicResult* findBufferType( const QByteArray &p );
 
   /**
-   * Same functionality as @ref findBufferType() but with
+   * Same functionality as findBufferType() but with
    * additional capability of distinguishing between
    * C-headers and C-Source.
    *
@@ -201,30 +202,17 @@ protected:
 private:
   void init( const QString& configFile );
 
-  /**
-   * If true, follow symlinks.
-   */
-  bool followLinks;
-
-  /**
-   * The current result buffer during inspection.
-   */
-  QString resultBuf;
+  bool bunused;
+  QString sunused;
 
   int finishResult();
-  void process(const QString &);
-  void tryit(unsigned char *, int);
-  int fsmagic(const char *, struct stat *);
-  int match(unsigned char *, int);
   int parse_line(char *line, int *rule, int lineno);
   int parse(char *, int);
   int buff_apprentice(char*buff);
   int apprentice(const QString &configFile);
-  int ascmagic(unsigned char *, int);
-  int textmagic(unsigned char *, int);
 
   struct config_rec *conf; // this is also our "d pointer"
-  int accuracy;
+  int iunused;
 };
 
 #endif
