@@ -16,7 +16,7 @@
  *  the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  *  Boston, MA 02111-1307, USA.
  **/
-// $Id: kmimetype.cpp,v 1.157.2.2 2003/06/29 21:43:01 deller Exp $
+// $Id: kmimetype.cpp,v 1.183 2004/01/12 19:11:14 mueller Exp $
 
 #include <config.h>
 
@@ -45,6 +45,7 @@
 #include <kmessageboxwrapper.h>
 
 #include <dcopclient.h>
+#include <dcopref.h>
 #include <kapplication.h>
 #include <kprocess.h>
 #include <kdebug.h>
@@ -55,6 +56,7 @@
 #include <ksimpleconfig.h>
 #include <kstandarddirs.h>
 #include <kurl.h>
+#include <ksycoca.h>
 
 template class KSharedPtr<KMimeType>;
 template class QValueList<KMimeType::Ptr>;
@@ -83,6 +85,13 @@ void KMimeType::buildDefaultType()
   }
 }
 
+KMimeType::Ptr KMimeType::defaultMimeTypePtr()
+{
+  if ( !s_pDefaultType ) // we need a default type first
+    buildDefaultType();
+  return s_pDefaultType;
+}
+
 // Check for essential mimetypes
 void KMimeType::checkEssentialMimeTypes()
 {
@@ -134,7 +143,10 @@ KMimeType::Ptr KMimeType::mimeType( const QString& _name )
 
   if ( !mime || !mime->isType( KST_KMimeType ) )
   {
-    delete mime;
+    // When building ksycoca, findServiceTypeByName doesn't create an object
+    // but returns one from a dict.
+    if ( !KSycoca::self()->isBuilding() )
+        delete mime;
     if ( !s_pDefaultType )
       buildDefaultType();
     return s_pDefaultType;
@@ -199,9 +211,22 @@ KMimeType::Ptr KMimeType::findByURL( const KURL& _url, mode_t _mode,
       if ( mime )
       {
         // Found something - can we trust it ? (e.g. don't trust *.pl over HTTP, could be anything)
-        if ( _is_local_file ||
+        if ( _is_local_file || _url.hasSubURL() || // Explicitly trust suburls
 	     KProtocolInfo::determineMimetypeFromExtension( _url.protocol() ) )
-          return mime;
+	{
+    	    if ( _is_local_file && !_fast_mode ) {
+    	    	if ( mime->patternsAccuracy()<100 )
+    	    	{
+    	    	    KMimeMagicResult* result =
+    	    	    	    KMimeMagic::self()->findFileType( path );
+
+    	    	    if ( result && result->isValid() )
+    	    	    	return mimeType( result->mimeType() );
+    	    	}
+    	    }
+
+	    return mime;
+  	}
       }
 
       static const QString& dotdesktop = KGlobal::staticQString(".desktop");
@@ -223,20 +248,24 @@ KMimeType::Ptr KMimeType::findByURL( const KURL& _url, mode_t _mode,
 
   if ( !_is_local_file || _fast_mode )
   {
+    QString def = KProtocolInfo::defaultMimetype( _url );
     if ( path.endsWith( slash ) || path.isEmpty() )
     {
       // We have no filename at all. Maybe the protocol has a setting for
       // which mimetype this means. For HTTP we set unknown now, because
       // of redirections (e.g. freshmeat downloads).
       // Assume inode/directory otherwise.
-      QString def = KProtocolInfo::defaultMimetype( _url );
       return mimeType( def.isEmpty() ? QString::fromLatin1("inode/directory") : def );
     }
-  }
+    if ( !def.isEmpty() && def != defaultMimeType() )
+    {
+       // The protocol says it always returns a given mimetype (e.g. text/html for "man:")
+       return mimeType( def );
+    }
 
-  // No more chances for non local URLs
-  if ( !_is_local_file || _fast_mode )
-    return mimeType( defaultMimeType() );
+    // No more chances for non local URLs
+    return defaultMimeTypePtr();
+  }
 
   // Do some magic for local files
   //kdDebug(7009) << QString("Mime Type finding for '%1'").arg(path) << endl;
@@ -244,12 +273,26 @@ KMimeType::Ptr KMimeType::findByURL( const KURL& _url, mode_t _mode,
 
   // If we still did not find it, we must assume the default mime type
   if ( !result || !result->isValid() )
-    return mimeType( defaultMimeType() );
+    return defaultMimeTypePtr();
 
   // The mimemagic stuff was successful
   return mimeType( result->mimeType() );
 }
 
+KMimeType::Ptr KMimeType::findByURL( const KURL& _url, mode_t _mode,
+				     bool _is_local_file, bool _fast_mode,
+	    	    	    	     bool *accurate)
+{
+    KMimeType::Ptr mime = findByURL(_url, _mode, _is_local_file, _fast_mode);
+    if (accurate) *accurate = !(_fast_mode) || ((mime->patternsAccuracy() == 100) && mime != defaultMimeTypePtr());
+    return mime;
+}
+
+KMimeType::Ptr KMimeType::diagnoseFileName(const QString &fileName, QString &pattern)
+{
+  return KServiceTypeFactory::self()->findFromPattern( fileName, &pattern );
+}
+
 KMimeType::Ptr KMimeType::findByPath( const QString& path, mode_t mode, bool fast_mode )
 {
     KURL u;
@@ -277,6 +320,36 @@ KMimeType::Ptr KMimeType::findByFileContent( const QString &fileName, int *accur
   return mimeType( result->mimeType() );
 }
 
+#define GZIP_MAGIC1	0x1f
+#define GZIP_MAGIC2	0x8b
+
+KMimeType::Format KMimeType::findFormatByFileContent( const QString &fileName )
+{
+  KMimeType::Format result;
+  result.compression = Format::NoCompression;
+  KMimeType::Ptr mime = findByPath(fileName);
+  if (mime->name() == "application/octet-stream")
+     mime =  findByFileContent(fileName);
+
+  result.text = mime->name().startsWith("text/");
+  QVariant v = mime->property("X-KDE-text");
+  if (v.isValid())
+     result.text = v.toBool();
+
+  if (mime->name().startsWith("inode/"))
+     return result;
+
+  QFile f(fileName);
+  if (f.open(IO_ReadOnly))
+  {
+     unsigned char buf[10+1];
+     int l = f.readBlock((char *)buf, 10);
+     if ((l > 2) && (buf[0] == GZIP_MAGIC1) && (buf[1] == GZIP_MAGIC2))
+        result.compression = Format::GZipCompression;
+  }
+  return result;
+}
+
 KMimeType::KMimeType( const QString & _fullpath, const QString& _type, const QString& _icon,
                       const QString& _comment, const QStringList& _patterns )
   : KServiceType( _fullpath, _type, _icon, _comment )
@@ -311,6 +384,18 @@ void KMimeType::init( KDesktopFile * config )
   if ( config->hasKey( XKDEAutoEmbed ) )
     m_mapProps.insert( XKDEAutoEmbed, QVariant( config->readBoolEntry( XKDEAutoEmbed ), 0 ) );
 
+  QString XKDEText = QString::fromLatin1("X-KDE-text");
+  if ( config->hasKey( XKDEText ) )
+    m_mapProps.insert( XKDEText, config->readBoolEntry( XKDEText ) );
+
+  QString XKDEIsAlso = QString::fromLatin1("X-KDE-IsAlso");
+  if ( config->hasKey( XKDEIsAlso ) )
+    m_mapProps.insert( XKDEIsAlso, config->readEntry( XKDEIsAlso ) );
+
+  QString XKDEPatternsAccuracy = QString::fromLatin1("X-KDE-PatternsAccuracy");
+  if ( config->hasKey( XKDEPatternsAccuracy ) )
+    m_mapProps.insert( XKDEPatternsAccuracy, config->readEntry( XKDEPatternsAccuracy ) );
+
 }
 
 KMimeType::KMimeType( QDataStream& _str, int offset ) : KServiceType( _str, offset )
@@ -405,7 +490,7 @@ QPixmap KMimeType::pixmapForURL( const KURL & _url, mode_t _mode, KIcon::Group _
   }
 
   return iconLoader->loadIcon( iconName , _group, _force_size, _state, _path, false );
-  
+
 }
 
 QString KMimeType::iconForURL( const KURL & _url, mode_t _mode )
@@ -416,7 +501,7 @@ QString KMimeType::iconForURL( const KURL & _url, mode_t _mode )
     QString i( mt->icon( _url, _url.isLocalFile() ));
 
     // if we don't find an icon, maybe we can use the one for the protocol
-    if ( i == unknown || i.isEmpty() || mt->name() == defaultMimeType()) {
+    if ( i == unknown || i.isEmpty() || mt == defaultMimeTypePtr()) {
         i = favIconForURL( _url ); // maybe there is a favicon?
 
         if ( i.isEmpty() )
@@ -442,24 +527,42 @@ QString KMimeType::favIconForURL( const KURL& url )
          || !useFavIcons )
         return QString::null;
 
-    QByteArray data;
-    QDataStream str(data, IO_WriteOnly);
-    str << url;
-    QCString replyType;
-    QByteArray reply;
-    kapp->dcopClient()->call("kded", "favicons", "iconForURL(KURL)", data,
-                             replyType, reply);
-    if (replyType == "QString")
-    {
-        QDataStream replyStr(reply, IO_ReadOnly);
-        QString result;
-        replyStr >> result;
+    DCOPRef kded( "kded", "favicons" );
+    DCOPReply result = kded.call( "iconForURL(KURL)", url );
+    if ( result.isValid() )
         return result;
-    }
 
     return QString::null;
 }
 
+QString KMimeType::parentMimeType() const
+{
+  QVariant v = property("X-KDE-IsAlso");
+  return v.toString();
+}
+
+bool KMimeType::is( const QString& mimeTypeName ) const
+{
+  if ( name() == mimeTypeName )
+      return true;
+  QString st = parentMimeType();
+  while ( !st.isEmpty() )
+  {
+      KMimeType::Ptr ptr = KMimeType::mimeType( st );
+      if (!ptr) return false; //error
+      if ( ptr->name() == mimeTypeName )
+          return true;
+      st = ptr->parentMimeType();
+  }
+  return false;
+}
+
+int KMimeType::patternsAccuracy() const {
+  QVariant v = property("X-KDE-PatternsAccuracy");
+  if (!v.isValid()) return 100;
+  else
+      return v.toInt();
+}
 
 
 /*******************************************************
@@ -485,7 +588,7 @@ QString KFolderType::icon( const KURL& _url, bool _is_local ) const
   u.addPath( ".directory" );
 
   QString icon;
-  // using KStandardDirs as this one checks for path beeing
+  // using KStandardDirs as this one checks for path being
   // a file instead of a directory
   if ( KStandardDirs::exists( u.path() ) )
   {
@@ -693,6 +796,8 @@ pid_t KDEDesktopMimeType::runFSDevice( const KURL& _url, const KSimpleConfig &cf
   {
     bool ro = cfg.readBoolEntry( "ReadOnly", false );
     QString fstype = cfg.readEntry( "FSType" );
+    if ( fstype == "Default" ) // KDE-1 thing
+      fstype = QString::null;
     QString point = cfg.readEntry( "MountPoint" );
     (void) new KAutoMount( ro, fstype, dev, point, _url.path() );
     retval = -1; // we don't want to return 0, but we don't want to return a pid
@@ -714,15 +819,16 @@ pid_t KDEDesktopMimeType::runApplication( const KURL& , const QString & _service
 
 pid_t KDEDesktopMimeType::runLink( const KURL& _url, const KSimpleConfig &cfg )
 {
-  QString url = cfg.readPathEntry( "URL" );
-  if ( url.isEmpty() )
+  QString u = cfg.readPathEntry( "URL" );
+  if ( u.isEmpty() )
   {
-    QString tmp = i18n("The desktop entry file\n%1\nis of type Link but has no URL=... entry.").arg( _url.url() );
+    QString tmp = i18n("The desktop entry file\n%1\nis of type Link but has no URL=... entry.").arg( _url.prettyURL() );
     KMessageBoxWrapper::error( 0, tmp );
     return 0;
   }
 
-  KRun* run = new KRun( url );
+  KURL url ( u );
+  KRun* run = new KRun(url);
 
   // X-KDE-LastOpenedWith holds the service desktop entry name that
   // was should be preferred for opening this URL if possible.
@@ -750,7 +856,7 @@ pid_t KDEDesktopMimeType::runMimeType( const KURL& url , const KSimpleConfig & )
   KProcess p;
   p << "kfmclient" << args;
   p.start(KProcess::DontCare);
-  return p.getPid();
+  return p.pid();
 }
 
 QValueList<KDEDesktopMimeType::Service> KDEDesktopMimeType::builtinServices( const KURL& _url )
@@ -838,6 +944,14 @@ QValueList<KDEDesktopMimeType::Service> KDEDesktopMimeType::userDefinedServices(
     //kdDebug(7009) << "CURRENT KEY = " << (*it) << endl;
 
     QString group = *it;
+
+    if (group == "_SEPARATOR_")
+    {
+        Service s;
+        result.append(s);
+        continue;
+    }
+
     group.prepend( "Desktop Action " );
 
     bool bInvalidMenu = false;
@@ -881,7 +995,7 @@ void KDEDesktopMimeType::executeService( const QString& _url, KDEDesktopMimeType
     KURL u;
     u.setPath(_url);
     KURL::List lst;
-    lst.append( _url );
+    lst.append( u );
     executeService( lst, _service );
 }
 
