@@ -21,11 +21,11 @@
 #include <string.h>
 #include <sys/utsname.h>
 
+#include <dcopref.h>
 #include <kdebug.h>
 #include <kglobal.h>
 #include <klocale.h>
 #include <kconfig.h>
-#include <kio/kpac.h>
 #include <kstandarddirs.h>
 #include <klibloader.h>
 #include <kstringhandler.h>
@@ -46,7 +46,6 @@ public:
 
    KConfig *config;
    KConfig *http_config;
-   KPAC *pac;
    bool init_busy;
    KURL url;
    QString protocol;
@@ -55,33 +54,30 @@ public:
    QString useragent;
 };
 
+static KProtocolManagerPrivate* d = 0;
 static KStaticDeleter<KProtocolManagerPrivate> kpmpksd;
 
 KProtocolManagerPrivate::KProtocolManagerPrivate()
-                        :config(0), http_config(0), pac(0), init_busy(false)
+                        :config(0), http_config(0), init_busy(false)
 {
-   kpmpksd.setObject(this);
+   kpmpksd.setObject(d, this);
 }
 
 KProtocolManagerPrivate::~KProtocolManagerPrivate()
 {
    delete config;
    delete http_config;
-   delete pac;
-   kpmpksd.setObject(0);
 }
 
-static KProtocolManagerPrivate* d = 0;
 
-// DEFUALT USERAGENT STRING
+// DEFAULT USERAGENT STRING
 #define CFG_DEFAULT_UAGENT(X) \
-QString("Mozilla/5.0 (compatible; Konqueror/%1.%2%3)") \
+QString("Mozilla/5.0 (compatible; Konqueror/%1.%2%3) (KHTML, like Gecko)") \
         .arg(KDE_VERSION_MAJOR).arg(KDE_VERSION_MINOR).arg(X)
 
 void KProtocolManager::reparseConfiguration()
 {
-  delete d;
-  d = 0;
+  kpmpksd.destructObject();
 
   // Force the slave config to re-read its config...
   KIO::SlaveConfig::self()->reset ();
@@ -111,42 +107,6 @@ KConfig *KProtocolManager::http_config()
   return d->http_config;
 }
 
-KPAC *KProtocolManager::pac()
-{
-  ProxyType type = proxyType();
-  if (type < PACProxy)
-    return 0;
-
-  if (!d->pac)
-  {
-    if (d->init_busy) return 0;
-    d->init_busy = true;
-
-    KLibrary *lib = KLibLoader::self()->library("libkpac");
-    if (lib)
-    {
-      KPAC *(*create_pac)() = (KPAC *(*)())(lib->symbol("create_pac"));
-      if (create_pac)
-      {
-        KPAC *newPAC = create_pac();
-        switch (type)
-        {
-          case PACProxy:
-            newPAC->init( proxyConfigScript() );
-            break;
-          case WPADProxy:
-            newPAC->discover();
-          default:
-            break;
-        }
-        d->pac = newPAC;
-      }
-    }
-    d->init_busy = false;
-  }
-  return d->pac;
-}
-
 /*=============================== TIMEOUT SETTINGS ==========================*/
 
 int KProtocolManager::readTimeout()
@@ -248,7 +208,12 @@ QString KProtocolManager::noProxyFor()
 {
   KConfig *cfg = config();
   cfg->setGroup( "Proxy Settings" );
-  return cfg->readEntry( "NoProxyFor" );
+
+  QString noProxy = cfg->readEntry( "NoProxyFor" );
+  if (proxyType() == EnvVarProxy)
+    noProxy = QString::fromLocal8Bit(getenv(noProxy.local8Bit()));
+
+  return noProxy;
 }
 
 QString KProtocolManager::proxyFor( const QString& protocol )
@@ -274,8 +239,12 @@ QString KProtocolManager::proxyForURL( const KURL &url )
   {
       case PACProxy:
       case WPADProxy:
-          if (!url.host().isEmpty() && pac())
-              proxy = pac()->proxyForURL( url ).stripWhiteSpace();
+          if (!url.host().isEmpty())
+          {
+            QString p = url.protocol();
+            if ( p.startsWith( "http" ) || p == "ftp" || p == "gopher" )
+              DCOPRef( "kded", "proxyscout" ).call( "proxyForURL", url ).get( proxy );
+          }
           break;
       case EnvVarProxy:
           proxy = QString::fromLocal8Bit(getenv(proxyFor(url.protocol()).local8Bit())).stripWhiteSpace();
@@ -293,8 +262,7 @@ QString KProtocolManager::proxyForURL( const KURL &url )
 
 void KProtocolManager::badProxy( const QString &proxy )
 {
-  if ( d && d->pac ) // don't load KPAC here if it isn't already
-    d->pac->badProxy( proxy );
+    DCOPRef( "kded", "proxyscout" ).send( "blackListProxy", proxy );
 }
 
 /*
@@ -327,6 +295,8 @@ static bool revmatch(const char *host, const char *nplist)
     {
       if ( nptr==nplist || nptr[-1]==',' || nptr[-1]==' ')
         return true;
+      if ( hptr == host ) // e.g. revmatch("bugs.kde.org","mybugs.kde.org")
+        return false;
 
       hptr--;
       nptr--;
@@ -338,6 +308,13 @@ static bool revmatch(const char *host, const char *nplist)
 
 QString KProtocolManager::slaveProtocol(const KURL &url, QString &proxy)
 {
+  if (url.hasSubURL()) // We don't want the suburl's protocol
+  {
+     KURL::List list = KURL::split(url);
+     KURL::List::Iterator it = list.fromLast();
+     return slaveProtocol(*it, proxy);
+  }
+
   if (!d)
     d = new KProtocolManagerPrivate;
 
@@ -352,11 +329,10 @@ QString KProtocolManager::slaveProtocol(const KURL &url, QString &proxy)
      proxy = proxyForURL(url);
      if ((proxy != "DIRECT") && (!proxy.isEmpty()))
      {
-        QString noProxy = noProxyFor();
-        ProxyType type = proxyType();
-        bool useRevProxy = ( (type == ManualProxy || type == EnvVarProxy) &&
-                            useReverseProxy() );
         bool isRevMatch = false;
+        bool useRevProxy = ((proxyType() == ManualProxy) && useReverseProxy());
+
+        QString noProxy = noProxyFor();
 
         if (!noProxy.isEmpty())
         {
@@ -388,12 +364,15 @@ QString KProtocolManager::slaveProtocol(const KURL &url, QString &proxy)
            if ( d->url.isValid() )
            {
               // The idea behind slave protocols is not applicable to http
-              // or any of its extensions like webdav.
+              // and webdav protocols.
               QString protocol = url.protocol().lower();
               if (protocol.startsWith("http") || protocol.startsWith("webdav"))
                 d->protocol = protocol;
               else
+              {
                 d->protocol = d->url.protocol();
+                kdDebug () << "slaveProtocol: " << d->protocol << endl;
+              }
 
               d->url = url;
               d->proxy = proxy;
