@@ -49,25 +49,45 @@
 #include <kwin.h>
 #include <kdesktopfile.h>
 #include <kstartupinfo.h>
+#include <kmacroexpander.h>
+#include <kshell.h>
 #include <typeinfo>
+#include <qwidget.h>
+#include <qguardedptr.h>
+
+#ifdef Q_WS_X11
+#include <X11/Xlib.h>
+#include <fixx11h.h>
+extern Time qt_x_user_time;
+#endif
 
 class KRun::KRunPrivate
 {
 public:
     KRunPrivate() { m_showingError = false; }
+
     bool m_showingError;
+    bool m_runExecutables;
+
     QString m_preferredService;
+    QGuardedPtr <QWidget> m_window;
 };
 
 pid_t KRun::runURL( const KURL& u, const QString& _mimetype )
 {
-    return runURL( u, _mimetype, false );
+    return runURL( u, _mimetype, false, true );
+}
+
+pid_t KRun::runURL( const KURL& u, const QString& _mimetype, bool tempFile )
+{
+    return runURL( u, _mimetype, tempFile, true );
 }
 
 // This is called by foundMimeType, since it knows the mimetype of the URL
-pid_t KRun::runURL( const KURL& u, const QString& _mimetype, bool tempFile )
+pid_t KRun::runURL( const KURL& u, const QString& _mimetype, bool tempFile, bool runExecutables )
 {
-
+  bool noRun = false;
+  bool noAuth = false;
   if ( _mimetype == "inode/directory-locked" )
   {
     KMessageBoxWrapper::error( 0L,
@@ -76,25 +96,50 @@ pid_t KRun::runURL( const KURL& u, const QString& _mimetype, bool tempFile )
   }
   else if ( _mimetype == "application/x-desktop" )
   {
-    if ( u.isLocalFile() )
+    if ( u.isLocalFile() && runExecutables)
       return KDEDesktopMimeType::run( u, true );
   }
   else if ( _mimetype == "application/x-executable"  ||
             _mimetype == "application/x-shellscript")
   {
+    if ( u.isLocalFile() && runExecutables)
+    {
+      if (kapp->authorize("shell_access"))
+      {
+        QString path = u.path();
+        shellQuote( path );
+        return (KRun::runCommand(path)); // just execute the url as a command
+        // ## TODO implement deleting the file if tempFile==true
+      }
+      else
+      {
+        noAuth = true;
+      }
+    }
+    else if (_mimetype == "application/x-executable")
+      noRun = true;
+  }
+  else if ( isExecutable(_mimetype) )
+  {
+    if (!runExecutables)
+      noRun = true;
+
     if (!kapp->authorize("shell_access"))
-    {
-      KMessageBoxWrapper::error( 0L,
-            i18n("<qt>You do not have permission to run <b>%1</b>.</qt>").arg(u.htmlURL()) );
-      return 0;
-    }
-    if ( u.isLocalFile() )
-    {
-      QString path = u.path();
-      shellQuote( path );
-      return (KRun::runCommand(path)); // just execute the url as a command
-      // ## TODO implement deleting the file if tempFile==true
-    }
+      noAuth = true;
+  }
+
+  if ( noRun )
+  {
+    KMessageBox::sorry( 0L,
+        i18n("<qt>The file <b>%1</b> is an executable program. "
+             "For safety it will not be started.</qt>").arg(u.htmlURL()));
+    return 0;
+  }
+  if ( noAuth )
+  {
+    KMessageBoxWrapper::error( 0L,
+        i18n("<qt>You do not have permission to run <b>%1</b>.</qt>").arg(u.htmlURL()) );
+    return 0;
   }
 
   KURL::List lst;
@@ -122,6 +167,13 @@ bool KRun::displayOpenWithDialog( const KURL::List& lst )
 
 bool KRun::displayOpenWithDialog( const KURL::List& lst, bool tempFiles )
 {
+    if (kapp && !kapp->authorizeKAction("openwith"))
+    {
+       // TODO: Better message, i18n freeze :-(
+       KMessageBox::sorry(0L, i18n("You are not authorized to execute this file."));
+       return false;
+    }
+
     KOpenWithDlg l( lst, i18n("Open with:"), QString::null, 0L );
     if ( l.exec() )
     {
@@ -140,273 +192,132 @@ void KRun::shellQuote( QString &_str )
     // Credits to Walter, says Bernd G. :)
     if (_str.isEmpty()) // Don't create an explicit empty parameter
         return;
-    QString res = "'";
-    res += _str.replace(QRegExp("'"), "'\"'\"'");
-    res += "'";
-    _str = res;
+    QChar q('\'');
+    _str.replace(q, "'\\''").prepend(q).append(q);
 }
 
-static QStringList breakup(const QString &exec, bool *need_shell = 0)
-{
-  QStringList result;
-  // This small state machine is used to parse "exec" in order
-  // to cut arguments at spaces, but also treat "..." and '...'
-  // as a single argument even if they contain spaces. Those simple
-  // and double quotes are also removed.
-  enum { PARSE_ANY, PARSE_QUOTED, PARSE_DBLQUOTED } state = PARSE_ANY;
-  QString arg;
-  for ( uint pos = 0; pos < exec.length() ; ++pos )
-  {
-      QChar ch = exec[pos];
-      switch (state) {
-          case PARSE_ANY:
-              if ( ch == '\'' && arg.isEmpty() )
-                  state = PARSE_QUOTED;
-              else if ( ch == '"' && arg.isEmpty() )
-                  state = PARSE_DBLQUOTED;
-              else if ( ch == ' ' ) /// TODO and last char wasn't a '\'
-              {
-                  if (!arg.isEmpty())
-                      result.append(arg);
-                  arg = QString::null;
-                  state = PARSE_ANY;
-              }
-              else if (( ch == ';' ) || (ch == '|') || (ch == '<'))
-              {
-                  if (!arg.isEmpty())
-                      result.append(arg);
-                  result.append(QString(ch));
-                  arg = QString::null;
-                  state = PARSE_ANY;
-                  if (need_shell)
-                     *need_shell = true;
-              }
-              else
-                  arg += ch;
-              break;
-          case PARSE_QUOTED:
-              if ( ch == '\'' )
-              {
-                  result.append(arg);
-                  arg = QString::null;
-                  state = PARSE_ANY;
-              }
-              else
-                  arg += ch;
-              break;
-          case PARSE_DBLQUOTED:
-              if ( ch == '"' )
-              {
-                  result.append(arg);
-                  arg = QString::null;
-                  state = PARSE_ANY;
-              }
-              else
-                  arg += ch;
-              break;
-      }
-  }
-  if (!arg.isEmpty())
-          result.append(arg);
-  if (need_shell && !result.isEmpty())
-  {
-     if (result[0].contains('='))
-        *need_shell = true;
-  }
-  return result;
-}
 
-static QString conditionalQuote(const QString &s, bool quote)
-{
-   if (!quote) return s;
-   QString r = s;
-   KRun::shellQuote(r);
-   return r;
-}
+class KRunMX1 : public KMacroExpanderBase {
+public:
+    KRunMX1( const KService &_service ) :
+        KMacroExpanderBase( '%' ), hasUrls( false ), hasSpec( false ), service( _service ) {}
+    bool hasUrls:1, hasSpec:1;
 
-static QString substitution(int option, const KURL &_url, bool quote)
+protected:
+    virtual int expandEscapedMacro( const QString &str, uint pos, QStringList &ret );
+
+private:
+    const KService &service;
+};
+
+int
+KRunMX1::expandEscapedMacro( const QString &str, uint pos, QStringList &ret )
 {
-   if (option == 'u')
-      return conditionalQuote(_url.isLocalFile() ? _url.path() : _url.url(), quote);
-   if (option == 'd')
-      return conditionalQuote(_url.directory(), quote);
-   if (option == 'f')
-      return conditionalQuote(_url.path(), quote);
-   if (option == 'n')
-      return conditionalQuote(_url.fileName(), quote);
-   if (option == 'v')
-   {
-      if ( _url.isLocalFile() && QFile::exists( _url.path() ) )
-      {
-          KDesktopFile desktopFile(_url.path(), true);
-          return conditionalQuote(desktopFile.readEntry( "Dev" ), quote);
-      }
+   uint option = str[pos + 1];
+   switch( option ) {
+   case 'c':
+      ret << service.name().replace( '%', "%%" );
+      break;
+   case 'k':
+      ret << service.desktopEntryPath().replace( '%', "%%" );
+      break;
+   case 'i':
+      ret << "-icon" << service.icon().replace( '%', "%%" );
+      break;
+   case 'm':
+      ret << "-miniicon" << service.icon().replace( '%', "%%" );
+      break;
+   case 'u':
+   case 'U':
+      hasUrls = true;
+      /* fallthrough */
+   case 'f':
+   case 'F':
+   case 'n':
+   case 'N':
+   case 'd':
+   case 'D':
+   case 'v':
+      hasSpec = true;
+      /* fallthrough */
+   default:
+      return -2; // subst with same and skip
    }
-	return QString::null;
+   return 2;
 }
 
-static QStringList substitution(int option, const KService &_service, bool quote)
-{
-   QStringList result;
-   if (option == 'c')
-      result << conditionalQuote(_service.name(), quote);
-   else if (option == 'i')
-      result << "-icon" << conditionalQuote(_service.icon(), quote);
-   else if (option == 'm')
-      result << "-miniicon" << conditionalQuote(_service.icon(), quote);
-   else if (option == 'k')
-      result << conditionalQuote(_service.desktopEntryPath(), quote);
+class KRunMX2 : public KMacroExpanderBase {
+public:
+    KRunMX2( const KURL::List &_urls ) :
+        KMacroExpanderBase( '%' ), ignFile( false ), urls( _urls ) {}
+    bool ignFile:1;
 
-   if (result.isEmpty())
-      result << QString::null;
-   return result;
-}
+protected:
+    virtual int expandEscapedMacro( const QString &str, uint pos, QStringList &ret );
 
-static QStringList substitution(int option, const KURL::List &_urls, bool quote)
+private:
+    void subst( int option, const KURL &url, QStringList &ret );
+
+    const KURL::List &urls;
+};
+
+void
+KRunMX2::subst( int option, const KURL &url, QStringList &ret )
 {
-   QStringList result;
-   option = option - 'A' + 'a'; // To lower
-   for(KURL::List::ConstIterator it = _urls.begin();
-       it != _urls.end(); ++it)
-   {
-       result.append(substitution(option, *it, quote));
+   switch( option ) {
+   case 'u':
+      ret << (url.isLocalFile() ? url.path() : url.url());
+      break;
+   case 'd':
+      ret << url.directory();
+      break;
+   case 'f':
+      ret << url.path();
+      break;
+   case 'n':
+      ret << url.fileName();
+      break;
+   case 'v':
+      if (url.isLocalFile() && QFile::exists( url.path() ) )
+          ret << KDesktopFile( url.path(), true ).readEntry( "Dev" );
+      break;
    }
-   return result;
+   return;
 }
 
-static void substitute(QStringList &_list, QStringList::Iterator &it, const KService &_service, const KURL::List &_urls, bool quote, bool service_only=false)
+int
+KRunMX2::expandEscapedMacro( const QString &str, uint pos, QStringList &ret )
 {
-  QString &arg = *it;
-  if ((arg.length() == 2) && (arg[0] == '%'))
-  {
-     int option = arg[1].unicode();
-     QStringList subs;
-     switch(option)
-     {
-        case 'U':
-        case 'F':
-        case 'D':
-        case 'N':
-          if (service_only)
-             return;
-          subs = substitution(option, _urls, quote);
-          break;
-
-        case 'u':
-        case 'f':
-        case 'd':
-        case 'n':
-        case 'v':
-          if (service_only)
-             return;
-          if (_urls.count())
-             subs.append(substitution(option, _urls.first(), quote));
-          break;
-
-        case 'c':
-        case 'i':
-        case 'm':
-        case 'k':
-          subs = substitution(option, _service, quote);
-          break;
-
-        case '%':
-          subs.append("%");
-          break;
-     }
-
-     if (subs.count() == 1)
-     {
-        arg = subs[0];
-     }
-     else
-     {
-        for(QStringList::Iterator it_subs = subs.begin();
-            it_subs != subs.end(); ++it_subs)
-        {
-           _list.insert(it, *it_subs);
-        }
-        QStringList::Iterator delete_it = it;
-        --it;
-        _list.remove(delete_it);
-     }
-     return;
-  }
-
-  QStringList args = breakup(arg);
-  if (args.isEmpty())
-  {
-     arg = QString::null;
-     return;
-  }
-  else if (args.count() != 1)
-  {
-     arg = QString::null;
-     for(QStringList::Iterator it = args.begin();
-         it != args.end(); ++it)
-     {
-        substitute(args, it, _service, _urls, true, service_only);
-     }
-     arg = QString::null;
-     for(QStringList::Iterator it = args.begin();
-         it != args.end(); ++it)
-     {
-        if (!arg.isEmpty())
-           arg += " ";
-        arg += *it;
-     }
-     if (quote)
-        KRun::shellQuote(arg);
-     return;
-  }
-  arg = args[0];
-
-  bool need_quote = false;
-  int l = arg.length();
-  int p = 0;
-  while (p < l-1)
-  {
-     if (arg[p] == '%')
-     {
-        need_quote = true;
-        int option = arg[++p].unicode();
-        if (service_only &&
-            ((option == 'u') || (option == 'f') || (option == 'd') || (option == 'n')))
-           continue;
-
-        QString sub;
-        QStringList subs;
-        switch(option)
-        {
-          case 'u':
-          case 'f':
-          case 'd':
-          case 'n':
-          case 'v':
-            sub = substitution(option, _urls.first(), false);
-            break;
-
-          case 'c':
-          case 'k':
-            subs = substitution(option, _service, false);
-            if (!subs.isEmpty())
-               sub = subs[0];
-            break;
-          case '%':
-            sub = "%";
-            break;
-        }
-
-        arg.replace(p-1, 2, sub);
-        p += sub.length()-2;
-        l = arg.length();
-     }
-     p++;
-  }
-  if (quote && need_quote)
-  {
-     KRun::shellQuote(arg);
-  }
+   uint option = str[pos + 1];
+   switch( option ) {
+   case 'f':
+   case 'u':
+   case 'n':
+   case 'd':
+   case 'v':
+      if( urls.isEmpty() ) {
+         if (!ignFile)
+            kdWarning() << "KRun::processDesktopExec: No URLs supplied to single-URL service " << str << endl;
+      } else if( urls.count() > 1 )
+          kdWarning() << "KRun::processDesktopExec: " << urls.count() << " URLs supplied to single-URL service " << str << endl;
+      else
+         subst( option, urls.first(), ret );
+      break;
+   case 'F':
+   case 'U':
+   case 'N':
+   case 'D':
+      option += 'a' - 'A';
+      for( KURL::List::ConstIterator it = urls.begin(); it != urls.end(); ++it )
+         subst( option, *it, ret );
+      break;
+   case '%':
+      ret = "%";
+      break;
+   default:
+      return -2; // subst with same and skip
+   }
+   return 2;
 }
 
 // BIC: merge with method below
@@ -414,155 +325,188 @@ QStringList KRun::processDesktopExec(const KService &_service, const KURL::List&
     return processDesktopExec( _service, _urls, has_shell, false );
 }
 
-QStringList KRun::processDesktopExec(const KService &_service, const KURL::List& _urls, bool has_shell, bool tempFiles)
+QStringList KRun::processDesktopExec(const KService &_service, const KURL::List& _urls, bool has_shell /* KDE4: remove */, bool tempFiles)
 {
   QString exec = _service.exec();
-  QString user = _service.username();
-  // Did the user forget to append something like '%f' ?
+  QStringList result;
+
+  KRunMX1 mx1( _service );
+  KRunMX2 mx2( _urls );
+
+  /// compatibility hack -- KDE 4: remove
+  QRegExp re("^\\s*(?:/bin/)?sh\\s+-c\\s+(.*)$");
+  if (!re.search( exec )) {
+    exec = re.cap( 1 ).stripWhiteSpace();
+    for (uint pos = 0; pos < exec.length(); ) {
+      QChar c = exec.unicode()[pos];
+      if (c != '\'' && c != '"')
+        goto synerr; // what else can we do? after normal parsing the substs would be insecure
+      int pos2 = exec.find( c, pos + 1 ) - 1;
+      if (pos2 < 0)
+        goto synerr; // quoting error
+      memcpy( (void *)(exec.unicode() + pos), exec.unicode() + pos + 1, (pos2 - pos) * sizeof(QChar));
+      pos = pos2;
+      exec.remove( pos, 2 );
+    }
+  }
+
+  if( !mx1.expandMacrosShellQuote( exec ) )
+    goto synerr; // error in shell syntax
+
+  // FIXME: the current way of invoking kioexec disables term and su use
+
+  // Check if we need "tempexec" (kioexec in fact)
+  if( tempFiles ) {
+    result << "kioexec" << "--tempfiles" << exec;
+    result += _urls.toStringList();
+    if (has_shell)
+      result = KShell::joinArgs( result );
+    return result;
+  }
+
+  // Check if we need kioexec
+  if( !mx1.hasUrls ) {
+    for( KURL::List::ConstIterator it = _urls.begin(); it != _urls.end(); ++it )
+      if ( !(*it).isLocalFile() ) {
+        // We need to run the app through kioexec
+        result << "kioexec" << exec;
+        result += _urls.toStringList();
+        if (has_shell)
+          result = KShell::joinArgs( result );
+        return result;
+      }
+  }
+
+  // Did the user forget to append something like '%f'?
   // If so, then assume that '%f' is the right choice => the application
   // accepts only local files.
-  if ( exec.find( "%f" ) == -1 && exec.find( "%u" ) == -1 && exec.find( "%n" ) == -1 &&
-       exec.find( "%d" ) == -1 && exec.find( "%F" ) == -1 && exec.find( "%U" ) == -1 &&
-       exec.find( "%N" ) == -1 && exec.find( "%D" ) == -1 && exec.find( "%v" ) == -1 )
+  if( !mx1.hasSpec ) {
     exec += " %f";
-
-  bool terminal_su = false;
-  bool terminal_sh = false;
-  bool kdesu = false;
-
-  if (_service.substituteUid() && !user.isEmpty())
-  {
-    if (_service.terminal())
-      terminal_su = true;
-    else
-      kdesu = true;
-  }
-  else if (_service.terminal())
-  {
-    terminal_sh = true;
+    mx2.ignFile = true;
   }
 
-  // Check if we need kfmexec.
-  bool b_local_app = ( exec.find( "%u" ) == -1 && exec.find( "%U" ) == -1 );
-  bool b_local_files = true;
-  KURL::List::ConstIterator it = _urls.begin();
-  for( ; it != _urls.end(); ++it )
-    if ( !(*it).isLocalFile() )
-      b_local_files = false;
+  mx2.expandMacrosShellQuote( exec ); // syntax was already checked, so don't check return value
 
-  if ( (b_local_app && !b_local_files) || tempFiles )
-  {
-     // We need to run the app through kfmexec
-     QStringList result = breakup(exec);
+/*
+ 1 = need_shell, 2 = terminal, 4 = su, 8 = has_shell
 
-     // Substitute everything that isn't file-related.
-     for(QStringList::Iterator it = result.begin();
-         it != result.end(); ++it)
-     {
-         substitute(result, it, _service, _urls, true, true);
-     }
-     QString cmd = result.join(" ");
-     if (has_shell)
-        shellQuote(cmd);
-     result.clear();
-     result << "kfmexec" << cmd;
-     KURL::List::ConstIterator it = _urls.begin();
-     for( ; it != _urls.end(); ++it )
-     {
-        QString url = (*it).url();
-        if (has_shell)
-           shellQuote(url);
-        result << url;
-     }
-     return result;
-  }
+ 0                                                           << split(cmd)
+ 1                                                           << "sh" << "-c" << cmd
+ 2 << split(term) << "-e"                                    << split(cmd)
+ 3 << split(term) << "-e"                                    << "sh" << "-c" << cmd
 
-  // Move args to result
-  bool need_shell = false;
-  QStringList result = breakup(exec, &need_shell);
+ 4                        << "kdesu" << "-u" << user << "-c" << cmd
+ 5                        << "kdesu" << "-u" << user << "-c" << ("sh -c " + quote(cmd))
+ 6 << split(term) << "-e" << "su"            << user << "-c" << cmd
+ 7 << split(term) << "-e" << "su"            << user << "-c" << ("sh -c " + quote(cmd))
 
-  for(QStringList::Iterator it = result.begin();
-      it != result.end(); ++it)
-  {
-      substitute(result, it, _service, _urls, has_shell || need_shell);
-  }
+ 8                                                           << cmd
+ 9                                                           << cmd
+ a << term        << "-e"                                    << cmd
+ b << term        << "-e"                                    << ("sh -c " + quote(cmd))
 
-  if (need_shell && !terminal_su && !kdesu &&
-      (!has_shell || terminal_sh))
-  {
-     QString cmd = result.join(" ");
-     result.clear();
-     result << "/bin/sh" << "-c" << cmd;
-  }
+ c                        << "kdesu" << "-u" << user << "-c" << quote(cmd)
+ d                        << "kdesu" << "-u" << user << "-c" << quote("sh -c " + quote(cmd))
+ e << term        << "-e" << "su"            << user << "-c" << quote(cmd)
+ f << term        << "-e" << "su"            << user << "-c" << quote("sh -c " + quote(cmd))
 
-  KConfigGroupSaver gs(KGlobal::config(), "General");
-  QString terminal = KGlobal::config()->readPathEntry("TerminalApplication", "konsole");
-  
-  if (terminal == "konsole")
-    terminal += " -caption=%c %i %m";
+ "sh -c" is needed in the "su" case, too, as su uses the user's login shell, not sh.
+ this could be optimized with the -s switch of some su versions (e.g., debian linux).
+*/
 
-  if (terminal_su)
-  {
-    QString cmd = result.join(" ");
-    result = breakup(QString("%1 %2 -e su %3 -c").arg(terminal).arg(_service.terminalOptions()).arg(user));
-    for(QStringList::Iterator it = result.begin();
-        it != result.end(); ++it)
-    {
-        substitute(result, it, _service, _urls, has_shell);
+  if (_service.terminal()) {
+    KConfigGroupSaver gs(KGlobal::config(), "General");
+    QString terminal = KGlobal::config()->readPathEntry("TerminalApplication", "konsole");
+    if (terminal == "konsole")
+      terminal += " -caption=%c %i %m";
+    terminal += " ";
+    terminal += _service.terminalOptions();
+    if( !mx1.expandMacrosShellQuote( terminal ) ) {
+      kdWarning() << "KRun: syntax error in command `" << terminal << "', service `" << _service.name() << "'" << endl;
+      return QStringList();
     }
-    result.append(cmd);
+    mx2.expandMacrosShellQuote( terminal );
+    if (has_shell)
+      result << terminal;
+    else
+      result = KShell::splitArgs( terminal ); // assuming that the term spec never needs a shell!
+    result << "-e";
   }
-  else if (terminal_sh)
-  {
-     QStringList cmd = result;
-     result = breakup(QString("%1 %2 -e").arg(terminal).arg(_service.terminalOptions()));
-     for(QStringList::Iterator it = result.begin();
-         it != result.end(); ++it)
-     {
-         substitute(result, it, _service, _urls, has_shell);
-     }
-      result += cmd;
-  }
-  else if (kdesu)
-  {
-     result = breakup(QString("kdesu -u %1 --").arg(user))+result;
+
+  int err;
+  if (_service.substituteUid()) {
+    if (_service.terminal())
+      result << "su";
+    else
+      result << "kdesu" << "-u";
+    result << _service.username() << "-c";
+    KShell::splitArgs(exec, KShell::AbortOnMeta, &err);
+    if (err == KShell::FoundMeta) {
+      shellQuote( exec );
+      exec.prepend( "/bin/sh -c " );
+    } else if (err != KShell::NoError)
+      goto synerr;
+    if (has_shell)
+      shellQuote( exec );
+    result << exec;
+  } else {
+    if (has_shell) {
+      if (_service.terminal()) {
+        KShell::splitArgs(exec, KShell::AbortOnMeta, &err);
+        if (err == KShell::FoundMeta) {
+          shellQuote( exec );
+          exec.prepend( "/bin/sh -c " );
+        } else if (err != KShell::NoError)
+          goto synerr;
+      }
+      result << exec;
+    } else {
+      result += KShell::splitArgs(exec, KShell::AbortOnMeta, &err);
+      if (err == KShell::FoundMeta)
+        result << "/bin/sh" << "-c" << exec;
+      else if (err != KShell::NoError)
+        goto synerr;
+    }
   }
 
   return result;
+
+ synerr:
+  kdWarning() << "KRun: syntax error in command `" << _service.exec() << "', service `" << _service.name() << "'" << endl;
+  return QStringList();
 }
 
 //static
 QString KRun::binaryName( const QString & execLine, bool removePath )
 {
   // Remove parameters and/or trailing spaces.
-  QStringList args = breakup( execLine );
-  QString _bin_name;
-  do {
-      if ( args.isEmpty() )
-         return QString::null;
-      _bin_name = args.first();
-      args.pop_front();
-  } while (_bin_name.contains('='));
-  // Remove path if wanted
-  return removePath ? _bin_name.mid(_bin_name.findRev('/') + 1) : _bin_name;
+  QStringList args = KShell::splitArgs( execLine );
+  for (QStringList::ConstIterator it = args.begin(); it != args.end(); ++it)
+    if (!(*it).contains('='))
+      // Remove path if wanted
+      return removePath ? (*it).mid((*it).findRev('/') + 1) : *it;
+  return QString::null;
 }
 
 static pid_t runCommandInternal( KProcess* proc, const KService* service, const QString& binName,
-    const QString &execName_P, const QString & iconName_P )
+    const QString &execName, const QString & iconName )
 {
-  QString bin = KRun::binaryName( binName, false );
-  QString execName = execName_P;
-  QString iconName = iconName_P;
   if ( service && !KDesktopFile::isAuthorizedDesktopFile( service->desktopEntryPath() ))
   {
      KMessageBox::sorry(0, i18n("You are not authorized to execute this file."));
      return 0;
   }
+  QString bin = KRun::binaryName( binName, true );
 #ifdef Q_WS_X11 // Startup notification doesn't work with QT/E, service isn't needed without Startup notification
   bool startup_notify = false;
   QCString wmclass;
   KStartupInfoId id;
-  if( service && service->property( "X-KDE-StartupNotify" ).isValid())
+  if( service && service->property( "StartupNotify" ).isValid())
+  {
+      startup_notify = service->property( "StartupNotify" ).toBool();
+      wmclass = service->property( "StartupWMClass" ).toString().latin1();
+  }
+  else if( service && service->property( "X-KDE-StartupNotify" ).isValid())
   {
       startup_notify = service->property( "X-KDE-StartupNotify" ).toBool();
       wmclass = service->property( "X-KDE-WMClass" ).toString().latin1();
@@ -579,38 +523,33 @@ static pid_t runCommandInternal( KProcess* proc, const KService* service, const
   {
       id.initId();
       id.setupStartupEnv();
-      if( execName.isEmpty())
-          execName = service->name();
-      if( iconName.isEmpty())
-          iconName = service->icon();
       KStartupInfoData data;
       data.setHostname();
-      data.setBin( KRun::binaryName( binName, true ));
-      data.setName( execName );
-      data.setIcon( iconName );
+      data.setBin( bin );
+      data.setName( execName.isEmpty() ? service->name() : execName );
+      data.setDescription( i18n( "Launching %1" ).arg( data.name()));
+      data.setIcon( iconName.isEmpty() ? service->icon() : iconName );
+#ifdef Q_WS_X11
+      data.setTimestamp( qt_x_user_time );
+#endif
       if( !wmclass.isEmpty())
           data.setWMClass( wmclass );
       data.setDesktop( KWin::currentDesktop());
       KStartupInfo::sendStartup( id, data );
   }
-  pid_t pid = KProcessRunner::run( proc, KRun::binaryName( binName, true ), id );
-  if( startup_notify )
+  pid_t pid = KProcessRunner::run( proc, binName, id );
+  if( startup_notify && pid )
   {
       KStartupInfoData data;
-      if ( pid ) // successfully started
-      {
-          data.addPid( pid );
-          KStartupInfo::sendChange( id, data );
-      } else // not started (e.g. executable not found)
-      {
-          data.setHostname();
-          KStartupInfo::sendFinish( id, data );
-      }
+      data.addPid( pid );
+      KStartupInfo::sendChange( id, data );
       KStartupInfo::resetStartupEnv();
   }
   return pid;
 #else
-  return KProcessRunner::run( proc, KRun::binaryName( binName, true ) );
+  Q_UNUSED( execName );
+  Q_UNUSED( iconName );
+  return KProcessRunner::run( proc, bin );
 #endif
 }
 
@@ -629,7 +568,7 @@ static pid_t runTempService( const KService& _service, const KURL::List& _urls,
       // usual way. The reported result is based on this
       // application.
       KURL::List::ConstIterator it = _urls.begin();
-      for(++it; it != _urls.end(); ++it)
+      while(++it != _urls.end())
       {
          KURL::List singleUrl;
          singleUrl.append(*it);
@@ -643,15 +582,14 @@ static pid_t runTempService( const KService& _service, const KURL::List& _urls,
   {
       args = KRun::processDesktopExec(_service, _urls, false, tempFiles);
   }
-  //kdDebug(7010) << "runTempService: KProcess args=" << args << endl;
+  kdDebug(7010) << "runTempService: KProcess args=" << args << endl;
 
   KProcess * proc = new KProcess;
-  for(QStringList::Iterator it = args.begin();
-      it != args.end(); ++it)
-  {
-     QString arg = *it;
-     *proc << arg;
-  }
+  *proc << args;
+
+  if (!_service.path().isEmpty())
+     proc->setWorkingDirectory(_service.path());
+
   return runCommandInternal( proc, &_service, _service.exec(), _service.name(), _service.icon() );
 }
 
@@ -672,12 +610,12 @@ pid_t KRun::run( const KService& _service, const KURL::List& _urls, bool tempFil
 
   if ( !tempFiles )
   {
-  // Remember we opened those urls, for the "recent documents" menu in kicker
-  KURL::List::ConstIterator it = _urls.begin();
-  for(; it != _urls.end(); ++it) {
-     //kdDebug(7010) << "KRecentDocument::adding " << (*it).url() << endl;
-     KRecentDocument::add( *it, _service.desktopEntryName() );
-  }
+      // Remember we opened those urls, for the "recent documents" menu in kicker
+      KURL::List::ConstIterator it = _urls.begin();
+      for(; it != _urls.end(); ++it) {
+          //kdDebug(7010) << "KRecentDocument::adding " << (*it).url() << endl;
+          KRecentDocument::add( *it, _service.desktopEntryName() );
+      }
   }
 
   if ( tempFiles || _service.desktopEntryPath().isEmpty())
@@ -729,25 +667,39 @@ pid_t KRun::runCommand( const QString& cmd, const QString &execName, const QStri
   KProcess * proc = new KProcess;
   proc->setUseShell(true);
   *proc << cmd;
-  QString bin = binaryName( cmd, false );
-  KService::Ptr service = KService::serviceByDesktopName( bin );
-  return runCommandInternal( proc, service.data(), bin, execName, iconName );
+  KService::Ptr service = KService::serviceByDesktopName( binaryName( cmd, true ));
+  return runCommandInternal( proc, service.data(), binaryName( cmd, false ), execName, iconName );
 }
 
-KRun::KRun( const KURL& _url, mode_t _mode, bool _is_local_file, bool _showProgressInfo )
-  : m_timer(0,"KRun::timer")
+KRun::KRun( const KURL& url, mode_t mode, bool isLocalFile, bool showProgressInfo )
+     :m_timer(0,"KRun::timer")
+{
+  init (url, 0, mode, isLocalFile, showProgressInfo);
+}
+
+KRun::KRun( const KURL& url, QWidget* window, mode_t mode, bool isLocalFile,
+            bool showProgressInfo )
+     :m_timer(0,"KRun::timer")
+{
+  init (url, window, mode, isLocalFile, showProgressInfo);
+}
+
+void KRun::init ( const KURL& url, QWidget* window, mode_t mode, bool isLocalFile,
+                  bool showProgressInfo )
 {
   m_bFault = false;
   m_bAutoDelete = true;
-  m_bProgressInfo = _showProgressInfo;
+  m_bProgressInfo = showProgressInfo;
   m_bFinished = false;
   m_job = 0L;
-  m_strURL = _url;
+  m_strURL = url;
   m_bScanFile = false;
   m_bIsDirectory = false;
-  m_bIsLocalFile = _is_local_file;
-  m_mode = _mode;
+  m_bIsLocalFile = isLocalFile;
+  m_mode = mode;
   d = new KRunPrivate;
+  d->m_runExecutables = true;
+  d->m_window = window;
 
   // Start the timer. This means we will return to the event
   // loop and do initialization afterwards.
@@ -755,7 +707,7 @@ KRun::KRun( const KURL& _url, mode_t _mode, bool _is_local_file, bool _showProgr
   m_bInit = true;
   connect( &m_timer, SIGNAL( timeout() ), this, SLOT( slotTimeout() ) );
   m_timer.start( 0, true );
-  kdDebug(7010) << " new KRun " << this << " " << _url.prettyURL() << " timer=" << &m_timer << endl;
+  kdDebug(7010) << " new KRun " << this << " " << url.prettyURL() << " timer=" << &m_timer << endl;
 
   kapp->ref();
 }
@@ -763,10 +715,10 @@ KRun::KRun( const KURL& _url, mode_t _mode, bool _is_local_file, bool _showProgr
 void KRun::init()
 {
   kdDebug(7010) << "INIT called" << endl;
-  if ( m_strURL.isMalformed() )
+  if ( !m_strURL.isValid() )
   {
     d->m_showingError = true;
-    KMessageBoxWrapper::error( 0L, i18n( "Malformed URL\n%1" ).arg( m_strURL.url() ) );
+    KMessageBoxWrapper::error( d->m_window, i18n( "Malformed URL\n%1" ).arg( m_strURL.url() ) );
     d->m_showingError = false;
     m_bFault = true;
     m_bFinished = true;
@@ -777,7 +729,7 @@ void KRun::init()
   {
     QString msg = KIO::buildErrorString(KIO::ERR_ACCESS_DENIED, m_strURL.prettyURL());
     d->m_showingError = true;
-    KMessageBoxWrapper::error( 0L, msg );
+    KMessageBoxWrapper::error( d->m_window, msg );
     d->m_showingError = false;
     m_bFault = true;
     m_bFinished = true;
@@ -797,7 +749,7 @@ void KRun::init()
       if ( stat( QFile::encodeName(m_strURL.path()), &buff ) == -1 )
       {
         d->m_showingError = true;
-        KMessageBoxWrapper::error( 0L, i18n( "<qt>Unable to run the command specified. The file or directory <b>%1</b> does not exist.</qt>" ).arg( m_strURL.htmlURL() ) );
+        KMessageBoxWrapper::error( d->m_window, i18n( "<qt>Unable to run the command specified. The file or folder <b>%1</b> does not exist.</qt>" ).arg( m_strURL.htmlURL() ) );
         d->m_showingError = false;
         m_bFault = true;
         m_bFinished = true;
@@ -848,10 +800,11 @@ void KRun::init()
 
   // It may be a directory or a file, let's stat
   KIO::StatJob *job = KIO::stat( m_strURL, true, 0 /* no details */, m_bProgressInfo );
+  job->setWindow (d->m_window);
   connect( job, SIGNAL( result( KIO::Job * ) ),
            this, SLOT( slotStatResult( KIO::Job * ) ) );
   m_job = job;
-  kdDebug() << " Job " << job << " is about stating " << m_strURL.url() << endl;
+  kdDebug(7010) << " Job " << job << " is about stating " << m_strURL.url() << endl;
 }
 
 KRun::~KRun()
@@ -896,12 +849,13 @@ void KRun::scanFile()
   kdDebug(7010) << this << " Scanning file " << m_strURL.url() << endl;
 
   KIO::TransferJob *job = KIO::get( m_strURL, false /*reload*/, m_bProgressInfo );
+  job->setWindow (d->m_window);
   connect(job, SIGNAL( result(KIO::Job *)),
           this, SLOT( slotScanFinished(KIO::Job *)));
   connect(job, SIGNAL( mimetype(KIO::Job *, const QString &)),
           this, SLOT( slotScanMimeType(KIO::Job *, const QString &)));
   m_job = job;
-  kdDebug() << " Job " << job << " is about getting from " << m_strURL.url() << endl;
+  kdDebug(7010) << " Job " << job << " is about getting from " << m_strURL.url() << endl;
 }
 
 void KRun::slotTimeout()
@@ -1095,12 +1049,12 @@ void KRun::foundMimeType( const QString& type )
       }
   }
 
-  if (!m_bFinished && KRun::runURL( m_strURL, type )){
+  if (!m_bFinished && KRun::runURL( m_strURL, type, false, d->m_runExecutables )){
     m_bFinished = true;
   }
   else{
     m_bFinished = true;
-    m_bFault = true;
+     m_bFault = true;
   }
 
   m_timer.start( 0, true );
@@ -1118,7 +1072,7 @@ void KRun::killJob()
 
 void KRun::abort()
 {
-  kdDebug() << "KRun::abort " << this << " m_showingError=" << d->m_showingError << endl;
+  kdDebug(7010) << "KRun::abort " << this << " m_showingError=" << d->m_showingError << endl;
   killJob();
   // If we're showing an error message box, the rest will be done
   // after closing the msgbox -> don't autodelete nor emit signals now.
@@ -1126,6 +1080,8 @@ void KRun::abort()
     return;
   m_bFault = true;
   m_bFinished = true;
+  m_bInit = false;
+  m_bScanFile = false;
 
   // will emit the error and autodelete this
   m_timer.start( 0, true );
@@ -1136,6 +1092,19 @@ void KRun::setPreferredService( const QString& desktopEntryName )
     d->m_preferredService = desktopEntryName;
 }
 
+void KRun::setRunExecutables(bool b)
+{
+    d->m_runExecutables = b;
+}
+
+bool KRun::isExecutable( const QString& serviceType )
+{
+    return ( serviceType == "application/x-desktop" ||
+             serviceType == "application/x-executable" ||
+             serviceType == "application/x-msdos-program" ||
+             serviceType == "application/x-shellscript" );
+}
+
 /****************/
 
 pid_t
@@ -1162,6 +1131,8 @@ KProcessRunner::KProcessRunner(KProcess * p, const QString & _binName )
       this,     SLOT(slotProcessExited(KProcess *)));
 
   process_->start();
+  if ( !process_->pid() )
+      slotProcessExited( process_ );
 }
 
 #ifdef Q_WS_X11
@@ -1176,6 +1147,8 @@ KProcessRunner::KProcessRunner(KProcess * p, const QString & _binName, const KSt
       this,     SLOT(slotProcessExited(KProcess *)));
 
   process_->start();
+  if ( !process_->pid() )
+      slotProcessExited( process_ );
 }
 #endif
 
@@ -1199,8 +1172,9 @@ KProcessRunner::slotProcessExited(KProcess * p)
   kdDebug(7010) << "slotProcessExited " << binName << endl;
   kdDebug(7010) << "normalExit " << process_->normalExit() << endl;
   kdDebug(7010) << "exitStatus " << process_->exitStatus() << endl;
-  if ( !binName.isEmpty() && process_->normalExit()
-          && ( process_->exitStatus() == 127 || process_->exitStatus() == 1 ) )
+  bool showErr = process_->normalExit()
+                 && ( process_->exitStatus() == 127 || process_->exitStatus() == 1 );
+  if ( !binName.isEmpty() && ( showErr || process_->pid() == 0 ) )
   {
     // Often we get 1 (zsh, csh) or 127 (ksh, bash) because the binary doesn't exist.
     // We can't just rely on that, but it's a good hint.
