@@ -17,11 +17,13 @@
  *  Boston, MA 02111-1307, USA.
  **/
 
-// $Id: kservice.cpp,v 1.125.2.2 2003/07/09 16:11:28 waba Exp $
+// $Id: kservice.cpp,v 1.154.2.1 2004/01/20 19:11:41 waba Exp $
 
 #include <config.h>
 
 #include "kservice.h"
+#include "kservice_p.h"
+
 #include <sys/types.h>
 #include <sys/stat.h>
 
@@ -41,6 +43,7 @@
 #include <kiconloader.h>
 #include <klocale.h>
 #include <kconfigbase.h>
+#include <kstandarddirs.h>
 #include <dcopclient.h>
 
 #include "kservicefactory.h"
@@ -53,6 +56,7 @@ class KService::KServicePrivate
 {
 public:
   QStringList categories;
+  QString menuId;
 };
 
 KService::KService( const QString & _name, const QString &_exec, const QString &_icon)
@@ -80,7 +84,7 @@ KService::KService( const QString & _fullpath )
 }
 
 KService::KService( KDesktopFile *config )
- : KSycocaEntry( config->filename())
+ : KSycocaEntry( config->fileName())
 {
   init(config);
 }
@@ -94,12 +98,7 @@ KService::init( KDesktopFile *config )
   bool absPath = (entryPath()[0] == '/');
 
   config->setDesktopGroup();
-  config->setDollarExpansion( true ); // mainly for Exec and Path
-  if(absPath && access(QFile::encodeName(entryPath()), R_OK))
-  {
-    m_bValid = false;
-    return;
-  }
+
   QMap<QString, QString> entryMap = config->entryMap(config->group());
 
   entryMap.remove("Encoding"); // reserved as part of Desktop Entry Standard
@@ -112,6 +111,25 @@ KService::init( KDesktopFile *config )
     m_bValid = false;
     return;
   }
+
+  m_strName = config->readEntry( "Name" );
+  entryMap.remove("Name");
+  if ( m_strName.isEmpty() )
+  {
+    if (config->readEntry( "Exec" ).isEmpty())
+    {
+      m_bValid = false;
+      return;
+    }
+    // Try to make up a name.
+    m_strName = entryPath();
+    int i = m_strName.findRev('/');
+    m_strName = m_strName.mid(i+1);
+    i = m_strName.findRev('.');
+    if (i != -1)
+       m_strName = m_strName.left(i);
+  }
+
   m_strType = config->readEntry( "Type" );
   entryMap.remove("Type");
   if ( m_strType.isEmpty() )
@@ -174,14 +192,6 @@ KService::init( KDesktopFile *config )
 
   m_strExec = config->readPathEntry( "Exec" );
   entryMap.remove("Exec");
-  m_strName = config->readEntry( "Name" );
-  //kdDebug() << "parsing " << entryPath() << " Name=" << m_strName << endl;
-  entryMap.remove("Name");
-  if ( m_strName.isEmpty() )
-  {
-    m_bValid = false;
-    return;
-  }
 
   m_strIcon = config->readEntry( "Icon", "unknown" );
   entryMap.remove("Icon");
@@ -195,6 +205,9 @@ KService::init( KDesktopFile *config )
   entryMap.remove("Comment");
   m_strGenName = config->readEntry( "GenericName" );
   entryMap.remove("GenericName");
+  QString untranslatedGenericName = config->readEntryUntranslated( "GenericName" );
+  entryMap.insert("UntranslatedGenericName", untranslatedGenericName);
+
   m_lstKeywords = config->readListEntry("Keywords");
   entryMap.remove("Keywords");
   d->categories = config->readListEntry("Categories", ';');
@@ -267,7 +280,7 @@ QPixmap KService::pixmap( KIcon::Group _group, int _force_size, int _state, QStr
 
       iconLoader->addExtraDesktopThemes();
   }
-  
+
   return iconLoader->loadIcon( m_strIcon, _group, _force_size, _state, _path );
 }
 
@@ -293,8 +306,8 @@ void KService::load( QDataStream& s )
     >> dst
     >> m_strDesktopEntryName
     >> dummy1 >> dummyStr1 >> initpref >> dummyStr2 >> dummy2
-    >> m_lstKeywords >> m_strInit >> dummyUI32 >> m_strGenName 
-    >> d->categories;
+    >> m_lstKeywords >> m_strInit >> dummyUI32 >> m_strGenName
+    >> d->categories >> d->menuId;
 
   m_bAllowAsDefault = def;
   m_bTerminal = term;
@@ -326,8 +339,8 @@ void KService::save( QDataStream& s )
     << dst
     << m_strDesktopEntryName
     << dummy1 << dummyStr1 << initpref << dummyStr2 << dummy2
-    << m_lstKeywords << m_strInit << dummyUI32 << m_strGenName 
-    << d->categories;
+    << m_lstKeywords << m_strInit << dummyUI32 << m_strGenName
+    << d->categories << d->menuId;
 }
 
 bool KService::hasServiceType( const QString& _servicetype ) const
@@ -336,19 +349,93 @@ bool KService::hasServiceType( const QString& _servicetype ) const
 
   //kdDebug(7012) << "Testing " << m_strDesktopEntryName << " for " << _servicetype << endl;
 
+  KMimeType::Ptr mimePtr = KMimeType::mimeType( _servicetype );
+  if ( mimePtr && mimePtr == KMimeType::defaultMimeTypePtr() )
+      mimePtr = 0;
+
+  bool isNumber;
   // For each service type we are associated with, if it doesn't
   // match then we try its parent service types.
   QStringList::ConstIterator it = m_lstServiceTypes.begin();
   for( ; it != m_lstServiceTypes.end(); ++it )
   {
+      (*it).toInt(&isNumber);
+      if (isNumber)
+         continue;
       //kdDebug(7012) << "    has " << (*it) << endl;
       KServiceType::Ptr ptr = KServiceType::serviceType( *it );
       if ( ptr && ptr->inherits( _servicetype ) )
           return true;
+
+      // The mimetype inheritance ("is also") works the other way.
+      // e.g. if we're looking for a handler for mimePtr==smb-workgroup
+      // then a handler for inode/directory is ok.
+      if ( mimePtr && mimePtr->is( *it ) )
+          return true;
   }
   return false;
 }
 
+int KService::initialPreferenceForMimeType( const QString& mimeType ) const
+{
+  if (!m_bValid) return 0; // safety test
+
+  bool isNumber;
+
+  // For each service type we are associated with
+  QStringList::ConstIterator it = m_lstServiceTypes.begin();
+  for( ; it != m_lstServiceTypes.end(); ++it )
+  {
+      (*it).toInt(&isNumber);
+      if (isNumber)
+         continue;
+      //kdDebug(7012) << "    has " << (*it) << endl;
+      KServiceType::Ptr ptr = KServiceType::serviceType( *it );
+      if ( !ptr || !ptr->inherits( mimeType ) )
+          continue;
+
+      int initalPreference = m_initialPreference;
+      ++it;
+      if (it != m_lstServiceTypes.end())
+      {
+         int i = (*it).toInt(&isNumber);
+         if (isNumber)
+            initalPreference = i;
+      }
+      return initalPreference;
+  }
+
+  KMimeType::Ptr mimePtr = KMimeType::mimeType( mimeType );
+  if ( mimePtr && mimePtr == KMimeType::defaultMimeTypePtr() )
+      mimePtr = 0;
+
+  // Try its parent service types.
+  it = m_lstServiceTypes.begin();
+  for( ; it != m_lstServiceTypes.end(); ++it )
+  {
+      (*it).toInt(&isNumber);
+      if (isNumber)
+         continue;
+
+      // The mimetype inheritance ("is also") works the other way.
+      // e.g. if we're looking for a handler for mimePtr==smb-workgroup
+      // then a handler for inode/directory is ok.
+      if ( !mimePtr || !mimePtr->is( *it ) )
+          continue;
+
+      int initalPreference = m_initialPreference;
+      ++it;
+      if (it != m_lstServiceTypes.end())
+      {
+         int i = (*it).toInt(&isNumber);
+         if (isNumber)
+            initalPreference = i;
+      }
+      return initalPreference;
+  }
+  return 0;
+}
+
 class KServiceReadProperty : public KConfigBase
 {
 public:
@@ -377,26 +464,41 @@ protected:
    QCString value;
 };
 
-QVariant KService::property( const QString& _name ) const
+QVariant KService::property( const QString& _name) const
+{
+   return property( _name, QVariant::Invalid);
+}
+
+// Return a string QVariant if string isn't null, and invalid variant otherwise
+// (the variant must be invalid if the field isn't in the .desktop file)
+// This allows trader queries like "exist Library" to work.
+static QVariant makeStringVariant( const QString& string )
+{
+    // Using isEmpty here would be wrong.
+    // Empty is "specified but empty", null is "not specified" (in the .desktop file)
+    return string.isNull() ? QVariant() : QVariant( string );
+}
+
+QVariant KService::property( const QString& _name, QVariant::Type t ) const
 {
   if ( _name == "Type" )
-    return QVariant( m_strType );
+    return QVariant( m_strType ); // can't be null
   else if ( _name == "Name" )
-    return QVariant( m_strName );
+    return QVariant( m_strName ); // can't be null
   else if ( _name == "Exec" )
-    return QVariant( m_strExec );
+    return makeStringVariant( m_strExec );
   else if ( _name == "Icon" )
-    return QVariant( m_strIcon );
+    return makeStringVariant( m_strIcon );
   else if ( _name == "Terminal" )
     return QVariant( static_cast<int>(m_bTerminal) );
   else if ( _name == "TerminalOptions" )
-    return QVariant( m_strTerminalOptions );
+    return makeStringVariant( m_strTerminalOptions );
   else if ( _name == "Path" )
-    return QVariant( m_strPath );
+    return makeStringVariant( m_strPath );
   else if ( _name == "Comment" )
-    return QVariant( m_strComment );
+    return makeStringVariant( m_strComment );
   else if ( _name == "GenericName" )
-    return QVariant( m_strGenName );
+    return makeStringVariant( m_strGenName );
   else if ( _name == "ServiceTypes" )
     return QVariant( m_lstServiceTypes );
   else if ( _name == "AllowAsDefault" )
@@ -404,29 +506,32 @@ QVariant KService::property( const QString& _name ) const
   else if ( _name == "InitialPreference" )
     return QVariant( m_initialPreference );
   else if ( _name == "Library" )
-    return QVariant( m_strLibrary );
-  else if ( _name == "DesktopEntryPath" )
+    return makeStringVariant( m_strLibrary );
+  else if ( _name == "DesktopEntryPath" ) // can't be null
     return QVariant( entryPath() );
   else if ( _name == "DesktopEntryName")
-    return QVariant( m_strDesktopEntryName );
+    return QVariant( m_strDesktopEntryName ); // can't be null
   else if ( _name == "Categories")
     return QVariant( d->categories );
   else if ( _name == "Keywords")
     return QVariant( m_lstKeywords );
-  
-  // Ok we need to convert the property from a QString to its real type.
-  // First we need to ask KServiceTypeFactory what the type of this property
-  // is supposed to be.
-  // Then we use a homebuild class based on KBaseConfig to convert the QString.
-  // For some often used property types we do the conversion ourselves.
 
-  QVariant::Type t = KServiceTypeFactory::self()->findPropertyTypeByName(_name);
+  // Ok we need to convert the property from a QString to its real type.
+  // Maybe the caller helped us.
   if (t == QVariant::Invalid)
   {
-    kdDebug(7012) << "Request for unknown property '" << _name << "'\n";
-    return QVariant(); // Unknown property: Invalid variant.
+    // No luck, let's ask KServiceTypeFactory what the type of this property
+    // is supposed to be.
+    t = KServiceTypeFactory::self()->findPropertyTypeByName(_name);
+    if (t == QVariant::Invalid)
+    {
+      kdDebug(7012) << "Request for unknown property '" << _name << "'\n";
+      return QVariant(); // Unknown property: Invalid variant.
+    }
   }
 
+  // Then we use a homebuild class based on KConfigBase to convert the QString.
+  // For some often used property types we do the conversion ourselves.
   QMap<QString,QVariant>::ConstIterator it = m_mapProps.find( _name );
   if ( (it == m_mapProps.end()) || (!it.data().isValid()))
   {
@@ -454,10 +559,7 @@ QVariant KService::property( const QString& _name ) const
            }
            if (t == QVariant::Bool)
            {
-              if (val)
-                 return QVariant(true);
-              else
-                 return QVariant(false);
+               return QVariant((bool)val, 1);
            }
            return QVariant(val);
         }
@@ -516,24 +618,59 @@ KService::Ptr KService::serviceByDesktopPath( const QString& _name )
 
 KService::Ptr KService::serviceByDesktopName( const QString& _name )
 {
-  KService * s = KServiceFactory::self()->findServiceByDesktopName( _name );
+  KService * s = KServiceFactory::self()->findServiceByDesktopName( _name.lower() );
+  if (!s && !_name.startsWith("kde-"))
+     s = KServiceFactory::self()->findServiceByDesktopName( "kde-"+_name.lower() );
   return KService::Ptr( s );
 }
 
+KService::Ptr KService::serviceByMenuId( const QString& _name )
+{
+  KService * s = KServiceFactory::self()->findServiceByMenuId( _name );
+  return KService::Ptr( s );
+}
+
+KService::Ptr KService::serviceByStorageId( const QString& _storageId )
+{
+  KService::Ptr service = KService::serviceByMenuId( _storageId );
+  if (service)
+     return service;
+
+  service = KService::serviceByDesktopPath(_storageId);
+  if (service)
+     return service;
+
+  if (_storageId.startsWith("/") && QFile::exists(_storageId))
+     return new KService(_storageId);
+
+  QString tmp = _storageId;
+  tmp = tmp.mid(tmp.findRev('/')+1); // Strip dir
+
+  if (tmp.endsWith(".desktop"))
+     tmp.truncate(tmp.length()-8);
+
+  if (tmp.endsWith(".kdelnk"))
+     tmp.truncate(tmp.length()-7);
+
+  service = KService::serviceByDesktopName(tmp);
+
+  return service;
+}
+
 KService::List KService::allInitServices()
 {
   return KServiceFactory::self()->allInitServices();
 }
 
 bool KService::substituteUid() const {
-  QVariant v = property("X-KDE-SubstituteUID");
+  QVariant v = property("X-KDE-SubstituteUID", QVariant::Bool);
   return v.isValid() && v.toBool();
 }
 
 QString KService::username() const {
   // See also KDesktopFile::tryExec()
   QString user;
-  QVariant v = property("X-KDE-Username");
+  QVariant v = property("X-KDE-Username", QVariant::String);
   user = v.isValid() ? v.toString() : QString::null;
   if (user.isEmpty())
      user = ::getenv("ADMIN_ACCOUNT");
@@ -544,16 +681,36 @@ QString KService::username() const {
 
 bool KService::noDisplay() const {
   QMap<QString,QVariant>::ConstIterator it = m_mapProps.find( "NoDisplay" );
-  if ( (it == m_mapProps.end()) || (!it.data().isValid()))
+  if ( (it != m_mapProps.end()) && (it.data().isValid()))
   {
-     return false;
+     QString aValue = it.data().toString().lower();
+     if (aValue == "true" || aValue == "on" || aValue == "yes")
+        return true;
   }
 
-  QString aValue = it.data().toString();
-  if (aValue == "true" || aValue == "on" || aValue == "yes")
-     return true;
-  else
-     return false;
+  it = m_mapProps.find( "OnlyShowIn" );
+  if ( (it != m_mapProps.end()) && (it.data().isValid()))
+  {
+     QString aValue = it.data().toString();
+     QStringList aList = QStringList::split(';', aValue);
+     if (!aList.contains("KDE"))
+        return true;
+  }
+
+  it = m_mapProps.find( "NotShowIn" );
+  if ( (it != m_mapProps.end()) && (it.data().isValid()))
+  {
+     QString aValue = it.data().toString();
+     QStringList aList = QStringList::split(';', aValue);
+     if (aList.contains("KDE"))
+        return true;
+  }
+  return false;
+}
+
+QString KService::untranslatedGenericName() const {
+  QVariant v = property("UntranslatedGenericName", QVariant::String);
+  return v.isValid() ? v.toString() : QString::null;
 }
 
 QString KService::parentApp() const {
@@ -580,7 +737,140 @@ QStringList KService::categories() const
   return d->categories;
 }
 
+QString KService::menuId() const
+{
+  return d->menuId;
+}
+
+void KService::setMenuId(const QString &menuId)
+{
+  d->menuId = menuId;
+}
+
+QString KService::storageId() const
+{
+  if (!d->menuId.isEmpty())
+     return d->menuId;
+  return entryPath();
+}
+
+QString KService::locateLocal()
+{
+  if (d->menuId.isEmpty() || desktopEntryPath().startsWith(".hidden") ||
+      (!desktopEntryPath().startsWith("/") && d->categories.isEmpty()))
+     return KDesktopFile::locateLocal(desktopEntryPath());
+
+  return ::locateLocal("xdgdata-apps", d->menuId);
+}
+
+QString KService::newServicePath(bool showInMenu, const QString &suggestedName,
+                                QString *menuId, const QStringList *reservedMenuIds)
+{
+   QString base = suggestedName;
+   if (!showInMenu)
+     base.prepend("kde-");
+
+   QString result;
+   for(int i = 1; true; i++)
+   {
+      if (i == 1)
+         result = base + ".desktop";
+      else
+         result = base + QString("-%1.desktop").arg(i);
+
+      if (reservedMenuIds && reservedMenuIds->contains(result))
+         continue;
+
+      // Lookup service by menu-id
+      KService::Ptr s = serviceByMenuId(result);
+      if (s)
+         continue;
+
+      if (showInMenu)
+      {
+         if (!locate("xdgdata-apps", result).isEmpty())
+            continue;
+      }
+      else
+      {
+         QString file = result.mid(4); // Strip "kde-"
+         if (!locate("apps", ".hidden/"+file).isEmpty())
+            continue;
+      }
+
+      break;
+   }
+   if (menuId)
+      *menuId = result;
+
+   if (showInMenu)
+   {
+       return ::locateLocal("xdgdata-apps", result);
+   }
+   else
+   {
+       QString file = result.mid(4); // Strip "kde-"
+       return ::locateLocal("apps", ".hidden/"+file);
+   }
+}
+
 
 void KService::virtual_hook( int id, void* data )
 { KSycocaEntry::virtual_hook( id, data ); }
 
+
+void KService::rebuildKSycoca(QWidget *parent)
+{
+  KServiceProgressDialog dlg(parent, "ksycoca_progress",
+                      i18n("Updating System Configuration"),
+                      i18n("Updating system configuration."));
+
+  QByteArray data;
+  DCOPClient *client = kapp->dcopClient();
+
+  int result = client->callAsync("kded", "kbuildsycoca", "recreate()",
+               data, &dlg, SLOT(slotFinished()));
+
+  if (result)
+  {
+     dlg.exec();
+  }
+}
+
+KServiceProgressDialog::KServiceProgressDialog(QWidget *parent, const char *name,
+                          const QString &caption, const QString &text)
+ : KProgressDialog(parent, name, caption, text, true)
+{
+  connect(&m_timer, SIGNAL(timeout()), this, SLOT(slotProgress()));
+  progressBar()->setTotalSteps(20);
+  m_timeStep = 700;
+  m_timer.start(m_timeStep);
+  setAutoClose(false);
+}
+
+void
+KServiceProgressDialog::slotProgress()
+{
+  int p = progressBar()->progress();
+  if (p == 18)
+  {
+     progressBar()->reset();
+     progressBar()->setProgress(1);
+     m_timeStep = m_timeStep * 2;
+     m_timer.start(m_timeStep);
+  }
+  else
+  {
+     progressBar()->setProgress(p+1);
+  }
+}
+
+void
+KServiceProgressDialog::slotFinished()
+{
+  progressBar()->setProgress(20);
+  m_timer.stop();
+  QTimer::singleShot(1000, this, SLOT(close()));
+}
+
+#include "kservice_p.moc"
