@@ -16,7 +16,7 @@
  *  Boston, MA 02111-1307, USA.
  **/
 
-// $Id: kservicegroup.cpp,v 1.25 2002/11/19 21:07:31 bausi Exp $
+// $Id: kservicegroup.cpp,v 1.31 2003/12/23 07:36:00 binner Exp $
 
 #include <kiconloader.h>
 #include <kglobal.h>
@@ -35,6 +35,9 @@ class KServiceGroup::Private
 public:
   Private() { m_bNoDisplay = false; }
   bool m_bNoDisplay;
+  QStringList suppressGenericNames;
+  QString directoryEntryPath;
+  QStringList sortOrder;
 };
 
 KServiceGroup::KServiceGroup( const QString & name )
@@ -50,19 +53,37 @@ KServiceGroup::KServiceGroup( const QString &configFile, const QString & _relpat
   d = new KServiceGroup::Private;
   m_bDeleted = false;
 
-  if (!configFile.isEmpty())
+  QString cfg = configFile;
+  if (cfg.isEmpty())
+     cfg = _relpath+".directory";
+
+  d->directoryEntryPath = cfg;
+
+  KConfig config( cfg, true, false, "apps" );
+
+  config.setDesktopGroup();
+
+  m_strCaption = config.readEntry( "Name" );
+  m_strIcon = config.readEntry( "Icon" );
+  m_strComment = config.readEntry( "Comment" );
+  m_bDeleted = config.readBoolEntry( "Hidden", false );
+  d->m_bNoDisplay = config.readBoolEntry( "NoDisplay", false );
+  QStringList tmpList;
+  if (config.hasKey("OnlyShowIn"))
   {
-     KConfig config( _relpath+".directory", true, false, "apps" );
-
-     config.setDesktopGroup();
-
-     m_strCaption = config.readEntry( "Name" );
-     m_strIcon = config.readEntry( "Icon" );
-     m_strComment = config.readEntry( "Comment" );
-     m_bDeleted = config.readBoolEntry( "Hidden", false );
-     d->m_bNoDisplay = config.readBoolEntry( "NoDisplay", false );
-     m_strBaseGroupName = config.readEntry( "X-KDE-BaseGroup" );
+     if (!config.readListEntry("OnlyShowIn", ';').contains("KDE"))
+        d->m_bNoDisplay = true;
   }
+  if (config.hasKey("NotShowIn"))
+  {
+     if (config.readListEntry("NotShowIn", ';').contains("KDE"))
+        d->m_bNoDisplay = true;
+  }
+  
+  m_strBaseGroupName = config.readEntry( "X-KDE-BaseGroup" );
+  d->suppressGenericNames = config.readListEntry( "X-KDE-SuppressGenericNames" );
+//  d->sortOrder = config.readListEntry("SortOrder");
+
   // Fill in defaults.
   if (m_strCaption.isEmpty())
   {
@@ -119,7 +140,12 @@ int KServiceGroup::childCount()
 
 bool KServiceGroup::noDisplay() const
 {
-    return d->m_bNoDisplay;
+  return d->m_bNoDisplay || m_strCaption.startsWith(".");
+}
+
+QStringList KServiceGroup::suppressGenericNames() const
+{
+  return d->suppressGenericNames;
 }
 
 void KServiceGroup::load( QDataStream& s )
@@ -128,7 +154,8 @@ void KServiceGroup::load( QDataStream& s )
   Q_INT8 noDisplay;
   s >> m_strCaption >> m_strIcon >>
       m_strComment >> groupList >> m_strBaseGroupName >> m_childCount >>
-      noDisplay;
+      noDisplay >> d->suppressGenericNames >> d->directoryEntryPath >>
+      d->sortOrder;
 
   d->m_bNoDisplay = (noDisplay != 0);
 
@@ -142,13 +169,15 @@ void KServiceGroup::load( QDataStream& s )
         {
            KServiceGroup *serviceGroup;
            serviceGroup = KServiceGroupFactory::self()->findGroupByDesktopPath(path, false);
-           m_serviceList.append( SPtr(serviceGroup) );
+           if (serviceGroup)
+              m_serviceList.append( SPtr(serviceGroup) );
         }
         else
         {
            KService *service;
            service = KServiceFactory::self()->findServiceByDesktopPath(path);
-           m_serviceList.append( SPtr(service) );
+           if (service)
+              m_serviceList.append( SPtr(service) );
         }
      }
   }
@@ -189,7 +218,8 @@ void KServiceGroup::save( QDataStream& s )
   Q_INT8 noDisplay = d->m_bNoDisplay ? 1 : 0;
   s << m_strCaption << m_strIcon <<
       m_strComment << groupList << m_strBaseGroupName << m_childCount <<
-      noDisplay;
+      noDisplay << d->suppressGenericNames << d->directoryEntryPath <<
+      d->sortOrder;
 }
 
 KServiceGroup::List
@@ -200,6 +230,20 @@ KServiceGroup::entries(bool sort)
 
 KServiceGroup::List
 KServiceGroup::entries(bool sort, bool excludeNoDisplay)
+{
+   return entries(sort, excludeNoDisplay, false);
+}
+
+static void addItem(KServiceGroup::List &sorted, const KSycocaEntry::Ptr &p, bool &addSeparator)
+{
+   if (addSeparator && !sorted.isEmpty())
+      sorted.append(new KServiceSeparator());
+   sorted.append(p);
+   addSeparator = false;
+}
+
+KServiceGroup::List
+KServiceGroup::entries(bool sort, bool excludeNoDisplay, bool allowSeparators, bool sortByGenericName)
 {
     KServiceGroup *group = this;
 
@@ -219,7 +263,6 @@ KServiceGroup::entries(bool sort, bool excludeNoDisplay)
     if (!sort)
         return group->m_serviceList;
 
-
     // Sort the list alphabetically, according to locale.
     // Groups come first, then services.
 
@@ -228,11 +271,21 @@ KServiceGroup::entries(bool sort, bool excludeNoDisplay)
     for (List::ConstIterator it(group->m_serviceList.begin()); it != group->m_serviceList.end(); ++it)
     {
         KSycocaEntry *p = (*it);
+	bool noDisplay = p->isType(KST_KServiceGroup) ?
+                                   static_cast<KServiceGroup *>(p)->noDisplay() :
+                                   static_cast<KService *>(p)->noDisplay();
+        if (excludeNoDisplay && noDisplay)
+           continue;
         // Choose the right list
         KSortableValueList<SPtr,QCString> & list = p->isType(KST_KServiceGroup) ? glist : slist;
-        QString name = p->isType(KST_KServiceGroup) ? 
-                                   static_cast<KServiceGroup *>(p)->caption() :
-                                   p->name();
+        QString name;
+        if (p->isType(KST_KServiceGroup))
+          name = static_cast<KServiceGroup *>(p)->caption();
+        else if (sortByGenericName)
+          name = static_cast<KService *>(p)->genericName() + " " + p->name();
+        else
+          name = p->name() + " " + static_cast<KService *>(p)->genericName();
+                                                                      
         QCString key( name.length() * 4 + 1 );
         // strxfrm() crashes on Solaris
 #ifndef USE_SOLARIS
@@ -258,75 +311,165 @@ KServiceGroup::entries(bool sort, bool excludeNoDisplay)
     slist.sort();
     glist.sort();
 
-    List lsort;
-    for(KSortableValueList<SPtr,QCString>::ConstIterator it = glist.begin(); it != glist.end(); ++it)
+    if (d->sortOrder.isEmpty())
     {
-        if (excludeNoDisplay)
-        {
-           KServiceGroup *serviceGroup = (KServiceGroup *)((KSycocaEntry *)((*it).value()));
-           if (serviceGroup->noDisplay())
-              continue;
-        }
-        lsort.append((*it).value());
+       d->sortOrder << ":M";
+       d->sortOrder << ":F";
     }
-    for(KSortableValueList<SPtr,QCString>::ConstIterator it = slist.begin(); it != slist.end(); ++it)
-    {
-        if (excludeNoDisplay)
-        {
-           KService *service = (KService *)((KSycocaEntry *)((*it).value()));
-           if (service->noDisplay())
-              continue;
-        }
-        lsort.append((*it).value());
-    }
-
-    // honor the SortOrder Key
 
     QString rp = relPath();
     if(rp == "/") rp = QString::null;
 
-    QStringList order =
-        KDesktopFile(rp + QString::fromUtf8(".directory")).sortOrder();
-
-    if (order.isEmpty())
-        return lsort;
-
-    //kdDebug() << "Honouring sort order " << order.join(",") << endl;
-
-    // Iterate through the sort spec list. If we find an entry that matches one
-    // in the original list, take it out of the original list and add it to the
-    // sorted list. Finally, add all entries that are still in the original list
-    // to the end of the sorted list.
-
-    List sorted;
-    List orig = lsort;
-
-    for (QStringList::ConstIterator it(order.begin()); it != order.end(); ++it)
+    // Iterate through the sort spec list.
+    // If an entry gets mentioned explicitly, we remove it from the sorted list
+    for (QStringList::ConstIterator it(d->sortOrder.begin()); it != d->sortOrder.end(); ++it)
     {
-        //kdDebug() << "order has : " << *it << endl;
-        for (List::Iterator sit(orig.begin()); sit != orig.end(); ++sit)
-            {
-                QString entry = (*sit)->entryPath();
-                // Groups have a trailing slash, we need to remove it first
-                if ( entry[entry.length()-1] == '/' )
-                   entry.truncate(entry.length()-1);
-                //kdDebug() << "Comparing to : " << entry.mid(entry.findRev('/')+1) << endl;
-                if (*it == entry.mid(entry.findRev('/')+1))
-                    {
-                        //kdDebug() << "Appending to sorted : " << *it << endl;
-                        sorted.append(*sit);
-                        orig.remove(sit);
-                        break;
-                    }
-            }
+        const QString &item = *it;
+        if (item.isEmpty()) continue;
+        if (item[0] == '/')
+        {
+          QString groupPath = rp + item.mid(1) + "/";
+           // Remove entry from sorted list of services.
+          for(KSortableValueList<SPtr,QCString>::Iterator it2 = glist.begin(); it2 != glist.end(); ++it2)
+          {
+             KServiceGroup *group = (KServiceGroup *)((KSycocaEntry *)((*it2).value()));
+             if (group->relPath() == groupPath)
+             {
+                glist.remove(it2);
+                break;
+             }
+          }
+        }
+        else if (item[0] != ':')
+        {
+           // Remove entry from sorted list of services.
+           // TODO: Remove item from sortOrder-list if not found
+           // TODO: This prevents duplicates
+          for(KSortableValueList<SPtr,QCString>::Iterator it2 = slist.begin(); it2 != slist.end(); ++it2)
+          {
+             KService *service = (KService *)((KSycocaEntry *)((*it2).value()));
+             if (service->menuId() == item)
+             {
+                slist.remove(it2);
+                break;
+             }
+          }
+        }
     }
 
-    for (List::Iterator sit(orig.begin()); sit != orig.end(); ++sit)
-        sorted.append(*sit);
+    List sorted;
+
+    bool needSeparator = false;
+    // Iterate through the sort spec list.
+    // Add the entries to the list according to the sort spec.
+    for (QStringList::ConstIterator it(d->sortOrder.begin()); it != d->sortOrder.end(); ++it)
+    {
+        const QString &item = *it;
+        if (item.isEmpty()) continue;
+        if (item[0] == ':')
+        {
+          // Special condition...
+          if (item == ":S")
+          {
+             if (allowSeparators)
+                needSeparator = true;
+          }
+          else if (item == ":M")
+          {
+            // Add sorted list of sub-menus
+            for(KSortableValueList<SPtr,QCString>::Iterator it2 = glist.begin(); it2 != glist.end(); ++it2)
+            {
+              addItem(sorted, (*it2).value(), needSeparator);
+            }
+          }
+          else if (item == ":F")
+          {
+            // Add sorted list of services
+            for(KSortableValueList<SPtr,QCString>::Iterator it2 = slist.begin(); it2 != slist.end(); ++it2)
+            {
+              addItem(sorted, (*it2).value(), needSeparator);
+            }
+          }
+          else if (item == ":A")
+          {
+            // Add sorted lists of services and submenus
+            KSortableValueList<SPtr,QCString>::Iterator it_s = slist.begin();
+            KSortableValueList<SPtr,QCString>::Iterator it_g = glist.begin();
+            
+            while(true)
+            {
+               if (it_s == slist.end())
+               {
+                  if (it_g == glist.end())
+                     break; // Done
+                     
+                  // Insert remaining sub-menu
+                  addItem(sorted, (*it_g).value(), needSeparator);
+                  it_g++;
+               }
+               else if (it_g == glist.end())
+               {
+                  // Insert remaining service
+                  addItem(sorted, (*it_s).value(), needSeparator);
+                  it_s++;
+               }
+               else if ((*it_g).index() < (*it_s).index())
+               {
+                  // Insert sub-menu first
+                  addItem(sorted, (*it_g).value(), needSeparator);
+                  it_g++;
+               }
+               else
+               {
+                  // Insert service first
+                  addItem(sorted, (*it_s).value(), needSeparator);
+                  it_s++;
+               }
+            }
+          }
+        }
+        else if (item[0] == '/')
+        {
+          QString groupPath = rp + item.mid(1) + "/";
+
+          for (List::ConstIterator it2(group->m_serviceList.begin()); it2 != group->m_serviceList.end(); ++it2)
+          {
+            if (!(*it2)->isType(KST_KServiceGroup))
+               continue;
+            KServiceGroup *group = (KServiceGroup *)((KSycocaEntry *)(*it2));
+            if (group->relPath() == groupPath)
+            {
+               if (!excludeNoDisplay || !group->noDisplay())
+                  addItem(sorted, (*it2), needSeparator);
+               break;
+            }
+          }
+        }
+        else
+        {
+          for (List::ConstIterator it2(group->m_serviceList.begin()); it2 != group->m_serviceList.end(); ++it2)
+          {
+            if (!(*it2)->isType(KST_KService))
+               continue;
+            KService *service = (KService *)((KSycocaEntry *)(*it2));
+            if (service->menuId() == item)
+            {
+               if (!excludeNoDisplay || !service->noDisplay())
+                  addItem(sorted, (*it2), needSeparator);
+               break;
+            }
+          }
+        }
+    }
 
     return sorted;
 }
 
+void KServiceGroup::setLayoutInfo(const QStringList &layout)
+{
+    d->sortOrder = layout;
+}
+
 KServiceGroup::Ptr
 KServiceGroup::baseGroup( const QString & _baseGroupName )
 {
@@ -352,7 +495,18 @@ KServiceGroup::childGroup(const QString &parent)
    return KServiceGroupFactory::self()->findGroupByDesktopPath("#parent#"+parent, true);
 }
 
+QString
+KServiceGroup::directoryEntryPath() const
+{
+   return d->directoryEntryPath;
+}
 
 
 void KServiceGroup::virtual_hook( int id, void* data )
 { KSycocaEntry::virtual_hook( id, data ); }
+
+
+KServiceSeparator::KServiceSeparator( )
+ : KSycocaEntry("separator")
+{
+}
