@@ -30,8 +30,18 @@
 template QDataStream& operator>> <QString, QVariant>(QDataStream&, QMap<QString, QVariant>&);
 template QDataStream& operator<< <QString, QVariant>(QDataStream&, const QMap<QString, QVariant>&);
 
+class KServiceType::KServiceTypePrivate
+{
+public:
+  KServiceTypePrivate() : parentTypeLoaded(false) { }
+
+  KServiceType::Ptr parentType;
+  KService::List services;
+  bool parentTypeLoaded;
+};
+
 KServiceType::KServiceType( const QString & _fullpath)
- : KSycocaEntry(_fullpath)
+ : KSycocaEntry(_fullpath), d(0)
 {
   KDesktopFile config( _fullpath );
 
@@ -39,7 +49,7 @@ KServiceType::KServiceType( const QString & _fullpath)
 }
 
 KServiceType::KServiceType( KDesktopFile *config )
- : KSycocaEntry(config->filename())
+ : KSycocaEntry(config->fileName()), d(0)
 {
   init(config);
 }
@@ -98,7 +108,7 @@ KServiceType::init( KDesktopFile *config)
 
 KServiceType::KServiceType( const QString & _fullpath, const QString& _type,
                             const QString& _icon, const QString& _comment )
- : KSycocaEntry(_fullpath)
+ : KSycocaEntry(_fullpath), d(0)
 {
   m_strName = _type;
   m_strIcon = _icon;
@@ -107,7 +117,7 @@ KServiceType::KServiceType( const QString & _fullpath, const QString& _type,
 }
 
 KServiceType::KServiceType( QDataStream& _str, int offset )
- : KSycocaEntry( _str, offset )
+ : KSycocaEntry( _str, offset ), d(0)
 {
   load( _str);
 }
@@ -135,6 +145,7 @@ KServiceType::save( QDataStream& _str )
 
 KServiceType::~KServiceType()
 {
+  delete d;
 }
 
 QString KServiceType::parentServiceType() const
@@ -222,18 +233,51 @@ KServiceType::Ptr KServiceType::serviceType( const QString& _name )
   return KServiceType::Ptr( p );
 }
 
+static void addUnique(KService::List &lst, QDict<KService> &dict, const KService::List &newLst, bool lowPrio)
+{
+  QValueListConstIterator<KService::Ptr> it = newLst.begin();
+  for( ; it != newLst.end(); ++it )
+  {
+     KService *service = static_cast<KService*>(*it);
+     if (dict.find(service->desktopEntryPath()))
+        continue;
+     dict.insert(service->desktopEntryPath(), service);
+     lst.append(service);
+     if (lowPrio)
+        service->setInitialPreference( 0 );
+  }
+}
+
 KService::List KServiceType::offers( const QString& _servicetype )
 {
+  QDict<KService> dict(53);
   KService::List lst;
 
   // Services associated directly with this servicetype (the normal case)
-  KServiceType * serv = KServiceTypeFactory::self()->findServiceTypeByName( _servicetype );
+  KServiceType::Ptr serv = KServiceTypeFactory::self()->findServiceTypeByName( _servicetype );
   if ( serv )
-    lst += KServiceFactory::self()->offers( serv->offset() );
+    addUnique(lst, dict, KServiceFactory::self()->offers( serv->offset() ), false);
   else
     kdWarning(7009) << "KServiceType::offers : servicetype " << _servicetype << " not found" << endl;
-  bool isAMimeType = serv ? serv->isType( KST_KMimeType ) : false;
-  delete serv;
+
+  // Find services associated with any mimetype parents. e.g. text/x-java -> text/plain    
+  KMimeType::Ptr mime = dynamic_cast<KMimeType*>(static_cast<KServiceType *>(serv));
+  bool isAMimeType = (mime != 0);
+  if (mime)
+  {
+     while(true)
+     {
+        QString parent = mime->parentMimeType();
+        if (parent.isEmpty())
+           break;
+        mime = dynamic_cast<KMimeType *>(KServiceTypeFactory::self()->findServiceTypeByName( parent ));
+        if (!mime)
+           break;
+        
+        addUnique(lst, dict, KServiceFactory::self()->offers( mime->offset() ), false);
+     }
+  }
+  serv = mime = 0;
 
   //QValueListIterator<KService::Ptr> it = lst.begin();
   //for( ; it != lst.end(); ++it )
@@ -250,22 +294,7 @@ KService::List KServiceType::offers( const QString& _servicetype )
     KServiceType * servAll = KServiceTypeFactory::self()->findServiceTypeByName( "all/all" );
     if ( servAll )
     {
-        KService::List newOffers = KServiceFactory::self()->offers( servAll->offset() );
-        // Look if we already have those services from the initial query, to avoid duplicates
-        QValueListIterator<KService::Ptr> it = newOffers.begin();
-        for( ; it != newOffers.end(); ++it )
-        {
-            bool found = false;
-            QValueListIterator<KService::Ptr> it2 = lst.begin();
-            for( ; it2 != lst.end() && !found; ++it2 )
-                found = (*it)->desktopEntryPath() == (*it2)->desktopEntryPath();
-            if ( !found )
-            {
-                (*it)->setInitialPreference( 0 ); // all/* associations are less prioritary
-                lst += *it;
-            }
-        }
-        //kdDebug(7009) << "all/all found, got " << newOffers.count() << " more offers" << endl;
+        addUnique(lst, dict, KServiceFactory::self()->offers( servAll->offset() ), true);
     }
     else
       kdWarning(7009) << "KServiceType::offers : servicetype all/all not found" << endl;
@@ -277,22 +306,7 @@ KService::List KServiceType::offers( const QString& _servicetype )
       KServiceType * servAllFiles = KServiceTypeFactory::self()->findServiceTypeByName( "all/allfiles" );
       if ( servAllFiles )
       {
-        KService::List newOffers = KServiceFactory::self()->offers( servAllFiles->offset() );
-        // Look if we already have those services from the initial query, to avoid duplicates
-        QValueListIterator<KService::Ptr> it = newOffers.begin();
-        for( ; it != newOffers.end(); ++it )
-        {
-            bool found = false;
-            QValueListIterator<KService::Ptr> it2 = lst.begin();
-            for( ; it2 != lst.end() && !found; ++it2 )
-                found = (*it)->desktopEntryPath() == (*it2)->desktopEntryPath();
-            if ( !found )
-            {
-                (*it)->setInitialPreference( 0 ); // all/* associations are less prioritary
-                lst += *it;
-            }
-        }
-        //kdDebug(7009) << "all/allfiles found, got " << newOffers.count() << " more offers" << endl;
+        addUnique(lst, dict, KServiceFactory::self()->offers( servAllFiles->offset() ), true);
       }
       else
         kdWarning(7009) << "KServiceType::offers : servicetype all/allfiles not found" << endl;
@@ -308,5 +322,45 @@ KServiceType::List KServiceType::allServiceTypes()
   return KServiceTypeFactory::self()->allServiceTypes();
 }
 
+KServiceType::Ptr KServiceType::parentType()
+{
+  if (d && d->parentTypeLoaded)
+     return d->parentType;
+  
+  if (!d)
+     d = new KServiceTypePrivate;
+     
+  QString parentSt = parentServiceType();
+  if (!parentSt.isEmpty())
+  {
+    d->parentType = KServiceTypeFactory::self()->findServiceTypeByName( parentSt );
+    if (!d->parentType)
+      kdDebug(7009) << "'" << desktopEntryPath() << "' specifies undefined mimetype/servicetype '"<< parentSt << "'" << endl;
+  }
+  
+  d->parentTypeLoaded = true;
+
+  return d->parentType;
+}
+
+void KServiceType::addService(KService::Ptr service)
+{
+  if (!d)
+     d = new KServiceTypePrivate;
+  
+  if (d->services.count() && d->services.last() == service)
+     return;
+     
+  d->services.append(service);
+}
+
+KService::List KServiceType::services()
+{
+  if (d)
+     return d->services;
+
+  return KService::List();
+}
+
 void KServiceType::virtual_hook( int id, void* data )
 { KSycocaEntry::virtual_hook( id, data ); }
