@@ -1,9 +1,9 @@
 /* This file is part of the KDE libraries
     Copyright (C) 2000 David Smith  <dsmith@algonet.se>
 
-	This class was inspired by a previous KURLCompletion by
-	Henner Zeller <zeller@think.de>
-	
+    This class was inspired by a previous KURLCompletion by
+    Henner Zeller <zeller@think.de>
+
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Library General Public
     License as published by the Free Software Foundation; either
@@ -37,7 +37,7 @@ class KURLCompletionPrivate;
  *
  * @short Completion of a single URL
  * @author David Smith <dsmith@algonet.se>
- * @version $Id: kurlcompletion.h,v 1.28 2002/06/19 17:46:01 domi Exp $
+ * @version $Id: kurlcompletion.h,v 1.35 2004/01/02 02:27:15 adawit Exp $
  */
 class KURLCompletion : public KCompletion
 {
@@ -49,7 +49,7 @@ public:
 	 * @li ExeCompletion - executables in $PATH or with full path.
 	 * @li FileCompletion - all files with full path or in dir(), URLs
 	 * are listed using KIO.
-	 * @li DirCompletion - as FileCompletion but only returns directories.
+	 * @li DirCompletion - Same as FileCompletion but only returns directories.
 	 */
 	enum Mode { ExeCompletion=1, FileCompletion, DirCompletion };
 
@@ -59,7 +59,7 @@ public:
 	KURLCompletion();
 	/**
 	 * This overloaded constructor allows you to set the Mode to ExeCompletion
-	 * or FileCompletion without using @ref setMode. Default is FileCompletion
+	 * or FileCompletion without using setMode. Default is FileCompletion.
 	 */
 	KURLCompletion(Mode);
 	/**
@@ -75,51 +75,61 @@ public:
 	 * The completion is done asyncronously if KIO is used.
 	 *
 	 * Returns the first match for user, environment, and local dir completion
-	 * and QString::null for asyncronous completion (KIO).
+	 * and QString::null for asynchronous completion (KIO).
+	 *
+	 * @param text the text to complete
+	 * @return the first match, or QString::null if not found
 	 */
-	virtual QString makeCompletion(const QString&);
+	virtual QString makeCompletion(const QString &text);
 
 	/**
-	 * Sets the current directory (used as base for completion)
+	 * Sets the current directory (used as base for completion).
 	 * Default = $HOME.
+	 * @param dir the current directory
 	 */
 	virtual void setDir(const QString &dir);
-	
+
 	/**
 	 * Returns the current directory.
+	 * @return the current directory
 	 */
 	virtual QString dir() const;
 
 	/**
-	 * Returns true if asyncronous completion is in progress.
+	 * Check whether asynchronous completion is in progress.
+	 * @return true if asynchronous completion is in progress
 	 */
 	virtual bool isRunning() const;
 
 	/**
-	 * Stops asyncronous completion.
+	 * Stops asynchronous completion.
 	 */
 	virtual void stop();
 
 	/**
-	 * Returns the completion mode: exe or file completion (default FileCompletion)
+	 * Returns the completion mode: exe or file completion (default FileCompletion).
+	 * @return the completion mode
 	 */
 	virtual Mode mode() const;
 
 	/**
 	 * Changes the completion mode: exe or file completion
+	 * @param mode the new completion mode
 	 */
 	virtual void setMode( Mode mode );
 
 	/**
-	 * Returns whether environment variables are completed and
+	 * Checks whether environment variables are completed and
 	 * whether they are replaced internally while finding completions.
 	 * Default is enabled.
+	 * @return true if environment vvariables will be replaced
 	 */
 	virtual bool replaceEnv() const;
-	
+
 	/**
 	 * Enables/disables completion and replacement (internally) of
 	 * environment variables in URLs. Default is enabled.
+	 * @param replace true to replace environment variables
 	 */
 	virtual void setReplaceEnv( bool replace );
 
@@ -127,13 +137,15 @@ public:
 	 * Returns whether ~username is completed and whether ~username
 	 * is replaced internally with the user's home directory while
 	 * finding completions. Default is enabled.
+	 * @return true to replace tilde with the home directory
 	 */
 	virtual bool replaceHome() const;
-	
+
 	/**
 	 * Enables/disables completion of ~username and replacement
 	 * (internally) of ~username with the user's home directory.
 	 * Default is enabled.
+	 * @param replace true to replace tilde with the home directory
 	 */
 	virtual void setReplaceHome( bool replace );
 
@@ -142,9 +154,18 @@ public:
 	 * current settings and returns the filtered url. Only works with
 	 * local files, i.e. returns back the original string for non-local
 	 * urls.
+	 * @param text the text to process
+	 * @return the result of the operation
 	 */
 	QString replacedPath( const QString& text );
 
+	/**
+	 * @internal I'll let ossi add a real one to KShell :)
+	 * @since 3.2
+	*/
+	static QString replacedPath( const QString& text,
+                                     bool replaceHome, bool replaceEnv = true );
+
 	class MyURL;
 	class DirLister;
 protected:
@@ -163,7 +184,7 @@ private slots:
 private:
 
 	bool isAutoCompletion();
-	
+
 	bool userCompletion(const MyURL &url, QString *match);
 	bool envCompletion(const MyURL &url, QString *match);
 	bool exeCompletion(const MyURL &url, QString *match);
@@ -189,11 +210,11 @@ private:
 	               const QString &filter = QString::null,
 	               bool only_exe = false,
 	               bool no_hidden = false );
-	
+
 	void addMatches( QStringList * );
 	QString finished();
-	
-	void init(Mode mode);
+
+	void init();
 
 	void setListedURL(int compl_type /* enum ComplType */,
 	                  QString dir = QString::null,
