@@ -229,8 +229,10 @@ void PreviewJob::determineNextFile()
     {
         if (!d->succeeded)
             emitFailed();
-        if (d->deleteItems)
+        if (d->deleteItems) {
             delete d->currentItem.item;
+            d->currentItem.item = 0L;
+        }
     }
     // No more items ?
     if ( d->items.isEmpty() )
@@ -246,6 +248,7 @@ void PreviewJob::determineNextFile()
         d->succeeded = false;
         d->items.remove(d->items.begin());
         KIO::Job *job = KIO::stat( d->currentItem.item->url(), false );
+        job->addMetaData( "no-auth-prompt", "true" );
         addSubjob(job);
     }
 }
@@ -374,6 +377,7 @@ void PreviewJob::getOrCreateThumbnail()
     }
 }
 
+// KDE 4: Make it const QString &
 void PreviewJob::createThumbnail( QString pixPath )
 {
     d->state = PreviewJobPrivate::STATE_CREATETHUMB;
