@@ -28,6 +28,7 @@
 #include <qlabel.h>
 #include <qlayout.h>
 #include <qlineedit.h>
+#include <qdir.h>
 
 #include <kmessagebox.h>
 #include <kpushbutton.h>
@@ -42,9 +43,11 @@
 #include <kurl.h>
 #include <kmimetype.h>
 #include <kwin.h>
+#include <kseparator.h>
 #include <kstringhandler.h>
 #include <kstdguiitem.h>
 #include <kguiitem.h>
+#include <ksqueezedtextlabel.h>
 
 using namespace KIO;
 
@@ -111,7 +114,7 @@ RenameDlg::RenameDlg(QWidget *parent, const QString & _caption,
     if ( ! (_mode & M_NORENAME ) ) {
         d->b1 = new QPushButton( i18n( "&Rename" ), this );
         d->b1->setEnabled(false);
-        d->b8 = new QPushButton( i18n( "&Propose" ), this );
+        d->b8 = new QPushButton( i18n( "Suggest New &Name" ), this );
         connect(d->b8, SIGNAL(clicked()), this, SLOT(b8Pressed()));
         connect(d->b1, SIGNAL(clicked()), this, SLOT(b1Pressed()));
     }
@@ -152,7 +155,8 @@ RenameDlg::RenameDlg(QWidget *parent, const QString & _caption,
     // User tries to overwrite a file with itself ?
     if ( _mode & M_OVERWRITE_ITSELF ) {
         QLabel *lb = new QLabel( i18n( "This action would overwrite '%1' with itself.\n"
-                                       "Do you want to rename it instead?" ).arg( KStringHandler::csqueeze( d->src,100 ) ), this );
+                                       "Please enter a new file name:" ).arg( KStringHandler::csqueeze( d->src,100 ) ), this );
+        d->b1->setText(i18n("C&ontinue"));
         d->m_pLayout->addWidget( lb );
     }
     else if ( _mode & M_OVERWRITE ) {
@@ -226,7 +230,8 @@ RenameDlg::RenameDlg(QWidget *parent, const QString & _caption,
                 sentence1 = i18n("A similar file named '%1' already exists.");
             else
                 sentence1 = i18n("A newer item named '%1' already exists.");
-            QLabel * lb1 = new QLabel( sentence1.arg(KStringHandler::csqueeze(d->dest,100)), this );
+
+            QLabel * lb1 = new KSqueezedTextLabel( sentence1.arg(d->dest), this );
             gridLayout->addMultiCellWidget( lb1, 0, 0, 0, 1 ); // takes the complete first line
 
             lb1 = new QLabel( this );
@@ -261,7 +266,7 @@ RenameDlg::RenameDlg(QWidget *parent, const QString & _caption,
                 // rows 1 to 3 are the details (size/ctime/mtime), row 4 is empty
                 gridLayout->addRowSpacing( 4, 20 );
 
-                QLabel * lb2 = new QLabel( i18n("The source file is '%1'").arg(d->src), this );
+                QLabel * lb2 = new KSqueezedTextLabel( i18n("The source file is '%1'").arg(d->src), this );
                 gridLayout->addMultiCellWidget( lb2, 5, 5, 0, 1 ); // takes the complete first line
 
                 lb2 = new QLabel( this );
@@ -295,29 +300,37 @@ RenameDlg::RenameDlg(QWidget *parent, const QString & _caption,
     }
     else
     {
-        // I wonder when this happens. And 'dest' isn't shown at all here...
-        // Sounds like the case where we don't want to allow overwriting, the existing
-        // file must be preserved. This doesn't happen in KIO though. (David)
+        // This is the case where we don't want to allow overwriting, the existing
+        // file must be preserved (e.g. when renaming).
         QString sentence1;
         if (mtimeDest < mtimeSrc)
-            sentence1 = i18n("An older item than '%1' already exists.\n").arg(d->src);
+            sentence1 = i18n("An older item named '%1' already exists.");
         else if (mtimeDest == mtimeSrc)
-            sentence1 = i18n("A similar file named '%1' already exists.").arg(d->src);
+            sentence1 = i18n("A similar file named '%1' already exists.");
         else
-            sentence1 = i18n("A newer item than '%1' already exists.\n").arg(d->src);
+            sentence1 = i18n("A newer item named '%1' already exists.");
 
-        QLabel *lb = new QLabel( sentence1 + i18n("Do you want to use another file name?"), this );
+        QLabel *lb = new KSqueezedTextLabel ( sentence1.arg(d->dest), this );
         d->m_pLayout->addWidget(lb);
     }
+    QHBoxLayout* layout2 = new QHBoxLayout();
+    d->m_pLayout->addLayout( layout2 );
+
     d->m_pLineEdit = new QLineEdit( this );
-    d->m_pLayout->addWidget( d->m_pLineEdit );
+    layout2->addWidget( d->m_pLineEdit );
     QString fileName = KURL(d->dest).fileName();
     d->m_pLineEdit->setText( KIO::decodeFileName( fileName ) );
     if (d->b1)
         connect(d->m_pLineEdit, SIGNAL(textChanged(const QString &)),
                 SLOT(enableRenameButton(const QString &)));
+    if ( d->b8 )
+    {
+        layout2->addWidget( d->b8 );
+        setTabOrder( d->m_pLineEdit, d->b8 );
+    }
 
-    d->m_pLayout->addSpacing( 10 );
+    KSeparator* separator = new KSeparator( this );
+    d->m_pLayout->addWidget( separator );
 
     QHBoxLayout* layout = new QHBoxLayout();
     d->m_pLayout->addLayout( layout );
@@ -325,22 +338,40 @@ RenameDlg::RenameDlg(QWidget *parent, const QString & _caption,
     layout->addStretch(1);
 
     if ( d->b1 )
+    {
         layout->addWidget( d->b1 );
-    if( d->b8 )
-        layout->addWidget( d->b8 );
+        setTabOrder( d->b1, d->b0 );
+    }
     if ( d->b2 )
+    {
         layout->addWidget( d->b2 );
+        setTabOrder( d->b2, d->b0 );
+    }
     if ( d->b3 )
+    {
         layout->addWidget( d->b3 );
+        setTabOrder( d->b3, d->b0 );
+    }
     if ( d->b4 )
+    {
         layout->addWidget( d->b4 );
+        setTabOrder( d->b4, d->b0 );
+    }
     if ( d->b5 )
+    {
         layout->addWidget( d->b5 );
+        setTabOrder( d->b5, d->b0 );
+    }
     if ( d->b6 )
+    {
         layout->addWidget( d->b6 );
+        setTabOrder( d->b6, d->b0 );
+    }
     if ( d->b7 )
+    {
         layout->addWidget( d->b7 );
-
+        setTabOrder( d->b7, d->b0 );
+    }
 
     d->b0->setDefault( true );
     layout->addWidget( d->b0 );
@@ -381,11 +412,11 @@ void RenameDlg::b0Pressed()
 // Rename
 void RenameDlg::b1Pressed()
 {
-  if ( d->m_pLineEdit->text()  == "" )
+  if ( d->m_pLineEdit->text().isEmpty() )
     return;
 
   KURL u = newDestURL();
-  if ( u.isMalformed() )
+  if ( !u.isValid() )
   {
     KMessageBox::error( this, i18n( "Malformed URL\n%1" ).arg( u.prettyURL() ) );
     return;
@@ -393,48 +424,59 @@ void RenameDlg::b1Pressed()
 
   done( 1 );
 }
+
+static QString suggestName(const KURL& baseURL, const QString& oldName)
+{
+  QString dotSuffix, suggestedName;
+  QString basename = oldName;
+
+  int index = basename.find( '.' );
+  if ( index != -1 ) {
+    dotSuffix = basename.mid( index );
+    basename.truncate( index );
+  }
+
+  int pos = basename.findRev( '_' );
+  if(pos != -1 ){
+    QString tmp = basename.mid( pos+1 );
+    bool ok;
+    int number = tmp.toInt( &ok );
+    if ( !ok ) {// ok there is no number
+      suggestedName = basename + "1" + dotSuffix;
+    }
+    else {
+     // yes there's already a number behind the _ so increment it by one
+      basename.replace( pos+1, tmp.length(), QString::number(number+1) );
+      suggestedName = basename + dotSuffix;
+    }
+  }
+  else // no underscore yet
+    suggestedName = basename + "_1" + dotSuffix ;
+
+  // Check if suggested name already exists
+  bool exists = false;
+  // TODO: network transparency. However, using NetAccess from a modal dialog
+  // could be a problem, no? (given that it uses a modal widget itself....)
+  if ( baseURL.isLocalFile() )
+     exists = QFileInfo( baseURL.path(+1) + suggestedName ).exists();
+
+  if ( !exists )
+    return suggestedName;
+  else // already exists -> recurse
+    return suggestName( baseURL, suggestedName );
+}
+
 // Propose button clicked
 void RenameDlg::b8Pressed()
 {
-  int pos;
-
   /* no name to play with */
   if ( d->m_pLineEdit->text().isEmpty() )
     return;
 
-  QString basename, dotSuffix, tmp;
-  QFileInfo info ( d->m_pLineEdit->text() );
-  basename = info.baseName();
-  dotSuffix = info.extension();
-
-  if ( !dotSuffix.isEmpty() )
-    dotSuffix = '.' + dotSuffix;
-
-  pos = basename.findRev('_' );
-  if(pos != -1 ){
-    bool ok;
-    tmp = basename.right( basename.length() - (pos + 1) );
-    int number = tmp.toInt( &ok, 10 );
-    if ( !ok ) {// ok there is no number
-      basename.append("_1" );
-      d->m_pLineEdit->setText(basename + dotSuffix );
-      return;
-    }
-    else {
-     // yes there's allready a number behind the _ so increment it by one
-      QString tmp2 = QString::number ( number + 1 );
-      basename.replace( pos+1, tmp.length() ,tmp2);
-      d->m_pLineEdit->setText( basename + dotSuffix );
-      return;
-    }
-  }
-  else // no underscore yet
-  {
-    d->m_pLineEdit->setText( basename + "_1" + dotSuffix );
-    return;
-
-  }
-  return; // we should never return from here jic
+  KURL destDirectory( d->dest );
+  destDirectory.setPath( destDirectory.directory() );
+  d->m_pLineEdit->setText( suggestName( destDirectory, d->m_pLineEdit->text() ) );
+  return;
 }
 
 void RenameDlg::b2Pressed()
