@@ -124,8 +124,8 @@ KIO::Scheduler::ProtocolInfoDict::get(const QString &protocol)
 
 
 Scheduler::Scheduler()
-          :QObject(kapp, "scheduler"),
-           DCOPObject( "KIO::Scheduler" ),
+          : DCOPObject( "KIO::Scheduler" ),
+           QObject(kapp, "scheduler"),
            slaveTimer(0, "Scheduler::slaveTimer"),
            coSlaveTimer(0, "Scheduler::coSlaveTimer"),
            cleanupTimer(0, "Scheduler::cleanupTimer")
@@ -183,7 +183,7 @@ bool Scheduler::process(const QCString &fun, const QByteArray &data, QCString &r
     for (; slave; slave = slaveList->next() )
     if ( slave->slaveProtocol() == proto || proto.isEmpty() )
     {
-      slave->connection()->send( CMD_REPARSECONFIGURATION );
+      slave->send( CMD_REPARSECONFIGURATION );
       slave->resetHost();
     }
     return true;
@@ -208,6 +208,10 @@ void Scheduler::_doJob(SimpleJob *job) {
     extraJobData->replace(job, jobData);
     newJobs.append(job);
     slaveTimer.start(0, true);
+#ifndef NDEBUG
+    if (newJobs.count() > 150)
+	kdDebug() << "WARNING - KIO::Scheduler got more than 150 jobs! This shows a misuse in your app (yes, a job is a QObject)." << endl;
+#endif
 }
 
 void Scheduler::_scheduleJob(SimpleJob *job) {
@@ -377,6 +381,8 @@ bool Scheduler::startJobScheduled(ProtocolInfo *protInfo)
        {
           newSlave = true;
           slave = createSlave(protInfo, job, job->url());
+          if (!slave)
+             slaveTimer.start(0, true);
        }
     }
 
@@ -590,7 +596,7 @@ if (!jobData)
           idleSlaves->append(slave);
           slave->setIdle();
           _scheduleCleanup();
-//          slave->connection()->send( CMD_SLAVE_STATUS );
+//          slave->send( CMD_SLAVE_STATUS );
        }
     }
     if (protInfo->joblist.count())
@@ -703,7 +709,7 @@ Scheduler::_getConnectedSlave(const KURL &url, const KIO::MetaData &config )
 
     setupSlave(slave, url, protocol, proxy, true, &config);
 
-    slave->connection()->send( CMD_CONNECT );
+    slave->send( CMD_CONNECT );
     connect(slave, SIGNAL(connected()),
                 SLOT(slotSlaveConnected()));
     connect(slave, SIGNAL(error(int, const QString &)),
@@ -835,7 +841,7 @@ Scheduler::_disconnectSlave(KIO::Slave *slave)
     if (slave->isAlive())
     {
        idleSlaves->append(slave);
-       slave->connection()->send( CMD_DISCONNECT );
+       slave->send( CMD_DISCONNECT );
        slave->setIdle();
        slave->setConnected(false);
        _scheduleCleanup();
@@ -884,6 +890,9 @@ Scheduler::slotUnregisterWindow(QObject *obj)
    if (it == m_windowList.end())
       return;
    long windowId = it.data();
+   disconnect( it.key(), SIGNAL(destroyed(QObject *)),
+              this, SLOT(slotUnregisterWindow(QObject*)));
+   m_windowList.remove( it );
    if (kapp)
    {
       QByteArray params;
