@@ -5,7 +5,7 @@
  *  Copyright (c) 2000 David Faure <faure@kde.org>
  *  Copyright (c) 2000 Stephan Kulow <coolo@kde.org>
  *
- *  $Id: slavebase.cpp,v 1.138.2.5 2003/06/11 13:04:15 waba Exp $
+ *  $Id: slavebase.cpp,v 1.157 2004/01/11 23:14:55 waba Exp $
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Library General Public
@@ -57,6 +57,12 @@
 #include "kio/ioslave_defaults.h"
 #include "kio/slaveinterface.h"
 
+#ifndef NDEBUG
+#ifdef HAVE_BACKTRACE
+#include <execinfo.h>
+#endif
+#endif
+
 using namespace KIO;
 
 template class QPtrList<QValueList<UDSAtom> >;
@@ -107,7 +113,6 @@ public:
     QString slaveid;
     bool resume:1;
     bool needSendCanResume:1;
-    bool multipleAuthCaching:1;
     bool onHold:1;
     bool wasKilled:1;
     MetaData configData;
@@ -124,16 +129,19 @@ public:
 
 }
 
-SlaveBase *globalSlave=0;
-long SlaveBase::s_seqNr = 0;
+static SlaveBase *globalSlave;
+long SlaveBase::s_seqNr;
 
 static volatile bool slaveWriteError = false;
 
-void genericsig_handler(int sigNumber)
+static const char *s_protocol;
+
+static void genericsig_handler(int sigNumber)
 {
    signal(sigNumber,SIG_IGN);
-   //I don't think we can have the same problem here as in the sigsegv handler
-   kdDebug()<<"kioslave : exiting due to signal "<<sigNumber<<endl;
+   //WABA: Don't do anything that requires malloc, we can deadlock on it since
+   //a SIGTERM signal can come in while we are in malloc/free.
+   //kdDebug()<<"kioslave : exiting due to signal "<<sigNumber<<endl;
    //set the flag which will be checked in dispatchLoop() and which *should* be checked
    //in lengthy operations in the various slaves
    if (globalSlave!=0)
@@ -151,6 +159,7 @@ SlaveBase::SlaveBase( const QCString &protocol,
       mPoolSocket( QFile::decodeName(pool_socket)),
       mAppSocket( QFile::decodeName(app_socket))
 {
+    s_protocol = protocol.data();
     if (!getenv("KDE_DEBUG"))
     {
         KCrash::setCrashHandler( sigsegv_handler );
@@ -176,7 +185,7 @@ SlaveBase::SlaveBase( const QCString &protocol,
         signal(SIGXFSZ, &sigsegv_handler);
 #endif
     }
-        
+
     struct sigaction act;
     act.sa_handler = sigpipe_handler;
     sigemptyset( &act.sa_mask );
@@ -203,7 +212,6 @@ SlaveBase::SlaveBase( const QCString &protocol,
     d->slaveid += QString::number(getpid());
     d->resume = false;
     d->needSendCanResume = false;
-    d->multipleAuthCaching = false;
     d->config = new SlaveBaseConfig(this);
     d->onHold = false;
     d->wasKilled=false;
@@ -214,13 +222,14 @@ SlaveBase::SlaveBase( const QCString &protocol,
     d->sentListEntries=0;
     d->timeout = 0;
     connectSlave(mAppSocket);
-    
+
     d->dcopClient = 0;
 }
 
 SlaveBase::~SlaveBase()
 {
     delete d;
+    s_protocol = "";
 }
 
 DCOPClient *SlaveBase::dcopClient()
@@ -238,61 +247,69 @@ void SlaveBase::dispatchLoop()
     fd_set rfds;
     int retval;
 
-    while (true) {
-    if (d->timeout && (d->timeout < time(0)))
+    while (true)
     {
-       QByteArray data = d->timeoutData;
-       d->timeout = 0;
-       d->timeoutData = QByteArray();
-       special(data);
-    }
-    FD_ZERO(&rfds);
+       if (d->timeout && (d->timeout < time(0)))
+       {
+          QByteArray data = d->timeoutData;
+          d->timeout = 0;
+          d->timeoutData = QByteArray();
+          special(data);
+       }
+       FD_ZERO(&rfds);
 
-    assert(appconn->inited());
-    FD_SET(appconn->fd_from(), &rfds);
+       assert(appconn->inited());
+       FD_SET(appconn->fd_from(), &rfds);
 
-    struct timeval tv;
-    tv.tv_sec = 1;
-    tv.tv_usec = 0; // 1 second timeout
-    retval = select(appconn->fd_from()+ 1, &rfds, NULL, NULL, &tv);
-    if ((retval>0) && FD_ISSET(appconn->fd_from(), &rfds))
-    { // dispatch application messages
-        int cmd;
-        QByteArray data;
-        if ( appconn->read(&cmd, data) != -1 )
-        {
-          dispatch(cmd, data);
-        }
-        else // some error occured, perhaps no more application
-        {
-          // When the app exits, should the slave be put back in the pool ?
-          if (mConnectedToApp && !mPoolSocket.isEmpty())
+       if (!d->timeout) // we can wait forever
+       {
+          retval = select(appconn->fd_from()+ 1, &rfds, NULL, NULL, NULL);
+       }
+       else
+       {
+          struct timeval tv;
+          tv.tv_sec = kMax(d->timeout-time(0),(time_t) 1);
+          tv.tv_usec = 0;
+          retval = select(appconn->fd_from()+ 1, &rfds, NULL, NULL, &tv);
+       }
+       if ((retval>0) && FD_ISSET(appconn->fd_from(), &rfds))
+       { // dispatch application messages
+          int cmd;
+          QByteArray data;
+          if ( appconn->read(&cmd, data) != -1 )
           {
-            disconnectSlave();
-            mConnectedToApp = false;
-            closeConnection();
-            connectSlave(mPoolSocket);
+             dispatch(cmd, data);
           }
-          else
+          else // some error occurred, perhaps no more application
           {
-            return;
+             // When the app exits, should the slave be put back in the pool ?
+             if (mConnectedToApp && !mPoolSocket.isEmpty())
+             {
+                disconnectSlave();
+                mConnectedToApp = false;
+                closeConnection();
+                connectSlave(mPoolSocket);
+             }
+             else
+             {
+                return;
+             }
           }
-        }
+       }
+       else if ((retval<0) && (errno != EINTR))
+       {
+          kdDebug(7019) << "dispatchLoop(): select returned " << retval << " "
+            << (errno==EBADF?"EBADF":errno==EINTR?"EINTR":errno==EINVAL?"EINVAL":errno==ENOMEM?"ENOMEM":"unknown")
+            << " (" << errno << ")" << endl;
+          return;
+       }
+       //I think we get here when we were killed in dispatch() and not in select()
+       if (wasKilled())
+       {
+          kdDebug(7019)<<" dispatchLoop() slave was killed, returning"<<endl;
+          return;
+       }
     }
-    else if ((retval<0) && (errno != EINTR))
-    {
-       kdDebug(7019) << "dispatchLoop(): select returned " << retval << " "
-          << (errno==EBADF?"EBADF":errno==EINTR?"EINTR":errno==EINVAL?"EINVAL":errno==ENOMEM?"ENOMEM":"unknown")
-          << " (" << errno << ")" << endl;
-       return;
-    }
-    //I think we get here when we were killed in dispatch() and not in select()
-    if (wasKilled())
-    {
-       kdDebug(7019)<<" dispatchLoop() slave was killed, returning"<<endl;
-       return;
-    }
-  }
 }
 
 void SlaveBase::connectSlave(const QString& path)
@@ -317,7 +334,7 @@ void SlaveBase::setMetaData(const QString &key, const QString &value)
    mOutgoingMetaData.replace(key, value);
 }
 
-QString SlaveBase::metaData(const QString &key)
+QString SlaveBase::metaData(const QString &key) const
 {
    if (mIncomingMetaData.contains(key))
       return mIncomingMetaData[key];
@@ -326,7 +343,7 @@ QString SlaveBase::metaData(const QString &key)
    return QString::null;
 }
 
-bool SlaveBase::hasMetaData(const QString &key)
+bool SlaveBase::hasMetaData(const QString &key) const
 {
    if (mIncomingMetaData.contains(key))
       return true;
@@ -335,6 +352,14 @@ bool SlaveBase::hasMetaData(const QString &key)
    return false;
 }
 
+// ### remove the next two methods for KDE4 (they miss the const)
+QString SlaveBase::metaData(const QString &key) {
+   return const_cast<const SlaveBase*>(this)->metaData( key );
+}
+bool SlaveBase::hasMetaData(const QString &key) {
+   return const_cast<const SlaveBase*>(this)->hasMetaData( key );
+}
+
 KConfigBase *SlaveBase::config()
 {
    return d->config;
@@ -667,9 +692,17 @@ void SlaveBase::sigsegv_handler(int sig)
 
     // Debug and printf should be avoided because they might
     // call malloc.. and get in a nice recursive malloc loop
-    char buffer[80];
-    snprintf(buffer, sizeof(buffer), "kioslave: ####### CRASH ###### pid = %d signal = %d\n", getpid(), sig);
+    char buffer[120];
+    snprintf(buffer, sizeof(buffer), "kioslave: ####### CRASH ###### protocol = %s pid = %d signal = %d\n", s_protocol, getpid(), sig);
     write(2, buffer, strlen(buffer));
+#ifndef NDEBUG
+#ifdef HAVE_BACKTRACE
+    void* trace[256];
+    int n = backtrace(trace, 256);
+    if (n)
+      backtrace_symbols_fd(trace, n, 2);
+#endif
+#endif
     ::exit(1);
 }
 
@@ -680,7 +713,7 @@ void SlaveBase::sigpipe_handler (int)
     // 1) Communication error with application.
     // 2) Communication error with network.
     slaveWriteError = true;
-    
+
     // Don't add anything else here, especially no debug output
 }
 
@@ -689,37 +722,37 @@ void SlaveBase::setHost(QString const &, int, QString const &, QString const &)
 }
 
 void SlaveBase::openConnection(void)
-{ error(  ERR_UNSUPPORTED_ACTION, i18n("Opening connections is not supported with the protocol %1" ).arg(mProtocol)); }
+{ error(  ERR_UNSUPPORTED_ACTION, unsupportedActionErrorString(mProtocol, CMD_CONNECT)); }
 void SlaveBase::closeConnection(void)
 { } // No response!
 void SlaveBase::stat(KURL const &)
-{ error(  ERR_UNSUPPORTED_ACTION, i18n("Accessing files is not supported with the protocol %1").arg(mProtocol) ); }
+{ error(  ERR_UNSUPPORTED_ACTION, unsupportedActionErrorString(mProtocol, CMD_STAT)); }
 void SlaveBase::put(KURL const &, int, bool, bool)
-{ error(  ERR_UNSUPPORTED_ACTION, i18n("Writing to %1 is not supported").arg(mProtocol) ); }
+{ error(  ERR_UNSUPPORTED_ACTION, unsupportedActionErrorString(mProtocol, CMD_PUT)); }
 void SlaveBase::special(const QByteArray &)
-{ error(  ERR_UNSUPPORTED_ACTION, i18n("There are no special actions available for protocol %1").arg(mProtocol) ); }
+{ error(  ERR_UNSUPPORTED_ACTION, unsupportedActionErrorString(mProtocol, CMD_SPECIAL)); }
 void SlaveBase::listDir(KURL const &)
-{ error(  ERR_UNSUPPORTED_ACTION, i18n("Listing directories is not supported for protocol %1").arg(mProtocol) ); }
+{ error(  ERR_UNSUPPORTED_ACTION, unsupportedActionErrorString(mProtocol, CMD_LISTDIR)); }
 void SlaveBase::get(KURL const & )
-{ error(  ERR_UNSUPPORTED_ACTION, i18n("Retrieving data from %1 is not supported").arg(mProtocol) ); }
+{ error(  ERR_UNSUPPORTED_ACTION, unsupportedActionErrorString(mProtocol, CMD_GET)); }
 void SlaveBase::mimetype(KURL const &url)
 { get(url); }
 void SlaveBase::rename(KURL const &, KURL const &, bool)
-{ error(  ERR_UNSUPPORTED_ACTION, i18n("Renaming or moving files within %1 is not supported").arg(mProtocol) ); }
+{ error(  ERR_UNSUPPORTED_ACTION, unsupportedActionErrorString(mProtocol, CMD_RENAME)); }
 void SlaveBase::symlink(QString const &, KURL const &, bool)
-{ error(  ERR_UNSUPPORTED_ACTION, i18n("Creating symlinks is not supported with protocol %1").arg(mProtocol) ); }
+{ error(  ERR_UNSUPPORTED_ACTION, unsupportedActionErrorString(mProtocol, CMD_SYMLINK)); }
 void SlaveBase::copy(KURL const &, KURL const &, int, bool)
-{ error(  ERR_UNSUPPORTED_ACTION, i18n("Copying files within %1 is not supported").arg(mProtocol) ); }
+{ error(  ERR_UNSUPPORTED_ACTION, unsupportedActionErrorString(mProtocol, CMD_COPY)); }
 void SlaveBase::del(KURL const &, bool)
-{ error(  ERR_UNSUPPORTED_ACTION, i18n("Deleting files from %1 is not supported").arg(mProtocol) ); }
+{ error(  ERR_UNSUPPORTED_ACTION, unsupportedActionErrorString(mProtocol, CMD_DEL)); }
 void SlaveBase::mkdir(KURL const &, int)
-{ error(  ERR_UNSUPPORTED_ACTION, i18n("Creating directories is not supported with protocol %1").arg(mProtocol) ); }
+{ error(  ERR_UNSUPPORTED_ACTION, unsupportedActionErrorString(mProtocol, CMD_MKDIR)); }
 void SlaveBase::chmod(KURL const &, int)
-{ error(  ERR_UNSUPPORTED_ACTION, i18n("Changing the attributes of files is not supported with protocol %1").arg(mProtocol) ); }
+{ error(  ERR_UNSUPPORTED_ACTION, unsupportedActionErrorString(mProtocol, CMD_CHMOD)); }
 void SlaveBase::setSubURL(KURL const &)
-{ error(  ERR_UNSUPPORTED_ACTION, i18n("Using sub-URLs with %1 is not supported").arg(mProtocol) ); }
+{ error(  ERR_UNSUPPORTED_ACTION, unsupportedActionErrorString(mProtocol, CMD_SUBURL)); }
 void SlaveBase::multiGet(const QByteArray &)
-{ error(  ERR_UNSUPPORTED_ACTION, i18n("Multiple get is not supported with protocol %1").arg(mProtocol) ); }
+{ error(  ERR_UNSUPPORTED_ACTION, unsupportedActionErrorString(mProtocol, CMD_MULTI_GET)); }
 
 
 void SlaveBase::slave_status()
@@ -752,30 +785,30 @@ bool SlaveBase::openPassDlg( AuthInfo& info )
 
 bool SlaveBase::openPassDlg( AuthInfo& info, const QString &errorMsg )
 {
-    kdDebug(7019) << "SlaveBase::OpenPassDlg User= " << info.username << endl;
-
     QCString replyType;
     QByteArray params;
     QByteArray reply;
     AuthInfo authResult;
     long windowId = metaData("window-id").toLong();
 
+    kdDebug(7019) << "SlaveBase::openPassDlg window-id=" << windowId << endl;
+
     (void) dcopClient(); // Make sure to have a dcop client.
-            
+
     QDataStream stream(params, IO_WriteOnly);
-    
+
     if (metaData("no-auth-prompt").lower() == "true")
        stream << info << QString("<NoAuthPrompt>") << windowId << s_seqNr;
     else
        stream << info << errorMsg << windowId << s_seqNr;
-            
+
     if (!d->dcopClient->call( "kded", "kpasswdserver", "queryAuthInfo(KIO::AuthInfo, QString, long int, long int)",
                                params, replyType, reply ) )
     {
-       kdWarning(7019) << "Can't communicate with kded!" << endl;
+       kdWarning(7019) << "Can't communicate with kded_kpasswdserver!" << endl;
        return false;
     }
-    
+
     if ( replyType == "KIO::AuthInfo" )
     {
        QDataStream stream2( reply, IO_ReadOnly );
@@ -792,6 +825,10 @@ bool SlaveBase::openPassDlg( AuthInfo& info, const QString &errorMsg )
        return false;
 
     info = authResult;
+
+    kdDebug(7019) << "SlaveBase::openPassDlg: username=" << info.username << endl;
+    kdDebug(7019) << "SlaveBase::openPassDlg: password=[hidden]" << endl;
+
     return true;
 }
 
@@ -877,7 +914,7 @@ void SlaveBase::setTimeoutSpecialCommand(int timeout, const QByteArray &data)
       d->timeout = 1; // Immediate timeout
    else
       d->timeout = 0; // Canceled
-      
+
    d->timeoutData = data;
 }
 
@@ -1034,7 +1071,7 @@ void SlaveBase::dispatch( int command, const QByteArray &data )
 
 QString SlaveBase::createAuthCacheKey( const KURL& url )
 {
-    if( url.isMalformed() )
+    if( !url.isValid() )
         return QString::null;
 
     // Generate the basic key sequence.
@@ -1055,11 +1092,11 @@ bool SlaveBase::pingCacheDaemon() const
 {
     // TODO: Ping kded / kpasswdserver
     KDEsuClient client;
-    int sucess = client.ping();
-    if( sucess == -1 )
+    int success = client.ping();
+    if( success == -1 )
     {
-        sucess = client.startServer();
-        if( sucess == -1 )
+        success = client.startServer();
+        if( success == -1 )
         {
             kdDebug(7019) << "Cannot start a new deamon!!" << endl;
             return false;
@@ -1077,7 +1114,7 @@ bool SlaveBase::checkCachedAuthentication( AuthInfo& info )
     AuthInfo authResult;
     long windowId = metaData("window-id").toLong();
 
-    kdDebug(7019) << "SlaveBase::checkCachedAuthInfo window = " << windowId << " url = " << info.url.url() << endl; 
+    kdDebug(7019) << "SlaveBase::checkCachedAuthInfo window = " << windowId << " url = " << info.url.url() << endl;
 
     (void) dcopClient(); // Make sure to have a dcop client.
 
@@ -1087,7 +1124,7 @@ bool SlaveBase::checkCachedAuthentication( AuthInfo& info )
     if ( !d->dcopClient->call( "kded", "kpasswdserver", "checkAuthInfo(KIO::AuthInfo, long int)",
                                params, replyType, reply ) )
     {
-       kdWarning(7019) << "Can't communicate with kded!" << endl;
+       kdWarning(7019) << "Can't communicate with kded_kpasswdserver!" << endl;
        return false;
     }
 
@@ -1111,13 +1148,6 @@ bool SlaveBase::checkCachedAuthentication( AuthInfo& info )
     return true;
 }
 
-bool SlaveBase::storeAuthInfo( const QCString&, const QCString&,
-                               const AuthInfo& )
-{
-    // Obsolete
-    return false;
-}
-
 bool SlaveBase::cacheAuthentication( const AuthInfo& info )
 {
     QByteArray params;
@@ -1133,16 +1163,6 @@ bool SlaveBase::cacheAuthentication( const AuthInfo& info )
     return true;
 }
 
-void SlaveBase::setMultipleAuthCaching( bool enable )
-{
-    d->multipleAuthCaching = enable;
-}
-
-bool SlaveBase::multipleAuthCaching() const
-{
-    return d->multipleAuthCaching;
-}
-
 int SlaveBase::connectTimeout()
 {
     bool ok;
