@@ -3,7 +3,7 @@
  *  This file is part of the KDE libraries
  *  Copyright (c) 2001 Waldo Bastian <bastian@kde.org>
  *
- * $Id: slaveconfig.cpp,v 1.20 2002/09/25 11:18:25 knoll Exp $
+ * $Id: slaveconfig.cpp,v 1.21 2003/08/31 11:15:17 raabe Exp $
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Library General Public
@@ -139,7 +139,7 @@ KStaticDeleter<SlaveConfig> slaveconfigsd;
 SlaveConfig *SlaveConfig::self()
 {
    if (!_self)
-      _self = slaveconfigsd.setObject(new SlaveConfig);
+      _self = slaveconfigsd.setObject(_self, new SlaveConfig);
    return _self;
 }
 
