@@ -1,8 +1,8 @@
 /*
- * $Id: tcpslavebase.cpp,v 1.116.2.4 2003/09/04 22:11:22 rompf Exp $
+ * $Id: tcpslavebase.cpp,v 1.132 2003/10/23 09:04:34 rompf Exp $
  *
  * Copyright (C) 2000 Alex Zepeda <zipzippy@sonic.net
- * Copyright (C) 2001 George Staikos <staikos@kde.org>
+ * Copyright (C) 2001-2003 George Staikos <staikos@kde.org>
  * Copyright (C) 2001 Dawit Alemayehu <adawit@kde.org>
  *
  * This file is part of the KDE project
@@ -41,13 +41,8 @@
 
 #include <ksocks.h>
 #include <kdebug.h>
-#include <kssl.h>
-#include <ksslcertificate.h>
-#include <ksslcertificatecache.h>
-#include <ksslcertificatehome.h>
+#include <ksslall.h>
 #include <ksslcertdlg.h>
-#include <ksslpkcs12.h>
-#include <ksslx509v3.h>
 #include <kmessagebox.h>
 
 #include <klocale.h>
@@ -122,7 +117,7 @@ TCPSlaveBase::TCPSlaveBase(unsigned short int defaultPort,
 {
     doConstructorStuff();
     if (useSSL)
-        m_bIsSSL = InitializeSSL();
+        m_bIsSSL = initializeSSL();
 }
 
 // The constructor procedures go here now.
@@ -143,7 +138,7 @@ void TCPSlaveBase::doConstructorStuff()
 
 TCPSlaveBase::~TCPSlaveBase()
 {
-    CleanSSL();
+    cleanSSL();
     if (d->usingTLS) delete d->kssl;
     if (d->dcc) delete d->dcc;
     if (d->pkcs) delete d->pkcs;
@@ -173,11 +168,11 @@ ssize_t TCPSlaveBase::read(void *data, ssize_t len)
 }
 
 
-void TCPSlaveBase::setBlockSize(int sz) 
+void TCPSlaveBase::setBlockSize(int sz)
 {
   if (sz <= 0)
     sz = 1;
-  
+
   d->rblockSz = sz;
 }
 
@@ -190,7 +185,7 @@ ssize_t TCPSlaveBase::readLine(char *data, ssize_t len)
 //   so that the while loop is as small as possible.  (GS)
 
   // let's not segfault!
-  if (!data) 
+  if (!data)
     return -1;
 
   char tmpbuf[1024];   // 1kb temporary buffer for peeking
@@ -298,8 +293,8 @@ bool TCPSlaveBase::connectToHost( const QString &host,
     d->userAborted = false;
 
     //  - leaving SSL - warn before we even connect
-    if (metaData("main_frame_request") == "TRUE" &&
-               metaData("ssl_activate_warnings") == "TRUE" &&
+    if (metaData("main_frame_request") == "TRUE" && 
+        metaData("ssl_activate_warnings") == "TRUE" &&
                metaData("ssl_was_in_use") == "TRUE" &&
         !m_bIsSSL) {
        KSSLSettings kss;
@@ -439,6 +434,13 @@ int TCPSlaveBase::startTLS()
       d->kssl->setPeerHost(d->host);
     }
 
+    if (hasMetaData("ssl_session_id")) {
+        KSSLSession *s = KSSLSession::fromString(metaData("ssl_session_id"));
+        if (s) {
+            d->kssl->setSession(s);
+            delete s;
+        }
+    }
     certificatePrompt();
 
     int rc = d->kssl->connect(m_iSock);
@@ -447,14 +449,19 @@ int TCPSlaveBase::startTLS()
         return -2;
     }
 
+    setMetaData("ssl_session_id", d->kssl->session()->toString());
+
     d->usingTLS = true;
     setMetaData("ssl_in_use", "TRUE");
-    rc = verifyCertificate();
-    if (rc != 1) {
-        setMetaData("ssl_in_use", "FALSE");
-        d->usingTLS = false;
-        delete d->kssl;
-        return -3;
+
+    if (!d->kssl->reusingSession()) {
+        rc = verifyCertificate();
+        if (rc != 1) {
+            setMetaData("ssl_in_use", "FALSE");
+            d->usingTLS = false;
+            delete d->kssl;
+            return -3;
+        }
     }
 
     d->savedMetaData = mOutgoingMetaData;
@@ -518,7 +525,7 @@ KSSLCertificateHome::KSSLAuthAction aa;
          break;
         case KSSLCertificateHome::AuthDont:
           send = false; prompt = false;
-          certname = "";
+          certname = QString::null;
          break;
         case KSSLCertificateHome::AuthPrompt:
           send = false; prompt = true;
@@ -529,24 +536,29 @@ KSSLCertificateHome::KSSLAuthAction aa;
   }
 
   QString ourHost;
-  if (!d->realHost.isEmpty())
+  if (!d->realHost.isEmpty()) {
      ourHost = d->realHost;
-  else ourHost = d->host;
+  } else {
+     ourHost = d->host;
+  }
 
   // Look for a certificate on a per-host basis as an override
   QString tmpcn = KSSLCertificateHome::getDefaultCertificateName(ourHost, &aa);
   if (aa != KSSLCertificateHome::AuthNone) {   // we must override
     switch (aa) {
         case KSSLCertificateHome::AuthSend:
-          send = true; prompt = false;
+          send = true;
+          prompt = false;
           certname = tmpcn;
          break;
         case KSSLCertificateHome::AuthDont:
-          send = false; prompt = false;
-          certname = "";
+          send = false;
+          prompt = false;
+          certname = QString::null;
          break;
         case KSSLCertificateHome::AuthPrompt:
-          send = false; prompt = true;
+          send = false;
+          prompt = true;
           certname = tmpcn;
          break;
         default:
@@ -568,30 +580,26 @@ KSSLCertificateHome::KSSLAuthAction aa;
 
   // Ok, we're supposed to prompt the user....
   if (prompt || forcePrompt) {
-     QStringList certs = KSSLCertificateHome::getCertificateList();
+    QStringList certs = KSSLCertificateHome::getCertificateList();
 
-  for (QStringList::Iterator it = certs.begin();
-           it != certs.end();
-           ++it) {
-    KSSLPKCS12 *pkcs =
-      KSSLCertificateHome::getCertificateByName(*it);
-    if (pkcs)
-    if (!pkcs->getCertificate() ||
-        !pkcs->getCertificate()->x509V3Extensions().certTypeSSLClient()) {
-      certs.remove(*it);
+    for (QStringList::Iterator it = certs.begin(); it != certs.end(); ++it) {
+      KSSLPKCS12 *pkcs = KSSLCertificateHome::getCertificateByName(*it);
+      if (pkcs && (!pkcs->getCertificate() ||
+          !pkcs->getCertificate()->x509V3Extensions().certTypeSSLClient())) {
+        certs.remove(*it);
+      }
     }
-  }
 
-        if (certs.isEmpty()) return;  // we had nothing else, and prompt failed
+    if (certs.isEmpty()) return;  // we had nothing else, and prompt failed
 
-     if (!d->dcc) {
+    if (!d->dcc) {
         d->dcc = new DCOPClient;
         d->dcc->attach();
         if (!d->dcc->isApplicationRegistered("kio_uiserver")) {
            KApplication::startServiceByDesktopPath("kio_uiserver.desktop",
                                                    QStringList() );
         }
-     }
+    }
 
      QByteArray data, retval;
      QCString rettype;
@@ -614,12 +622,12 @@ KSSLCertificateHome::KSSLAuthAction aa;
      }
   }
 
-    // The user may have said to not send the certificate,
-    // but to save the choice
+  // The user may have said to not send the certificate,
+  // but to save the choice
   if (!send) {
      if (save) {
-            KSSLCertificateHome::setDefaultCertificate(certname, ourHost,
-                                                       false, false);
+       KSSLCertificateHome::setDefaultCertificate(certname, ourHost,
+                                                  false, false);
      }
      return;
   }
@@ -654,15 +662,21 @@ KSSLCertificateHome::KSSLAuthAction aa;
            bool rc = d->dcc->call("kio_uiserver", "UIServer",
                                    "openPassDlg(KIO::AuthInfo)",
                                    authdata, rettype, authval);
-           if (!rc) break;
-           if (rettype != "QByteArray") continue;
+           if (!rc) {
+             break;
+           }
+           if (rettype != "QByteArray") {
+             continue;
+           }
 
            QDataStream qdret(authval, IO_ReadOnly);
            QByteArray authdecode;
            qdret >> authdecode;
            QDataStream qdtoo(authdecode, IO_ReadOnly);
            qdtoo >> ai;
-           if (!ai.isModified()) break;
+           if (!ai.isModified()) {
+             break;
+           }
         }
         pass = ai.password;
         pkcs = KSSLCertificateHome::getCertificateByName(certname, pass);
@@ -672,11 +686,15 @@ KSSLCertificateHome::KSSLAuthAction aa;
                                                      "certificate. Try a "
                                                      "new password?"),
                                                 i18n("SSL"));
-              if (rc == KMessageBox::No) break;
+              if (rc == KMessageBox::No) {
+                break;
+              }
               showprompt = true;
         }
      } while (!pkcs);
-     if (pkcs) cacheAuthentication(ai);
+     if (pkcs) {
+       cacheAuthentication(ai);
+     }
   }
 
    // If we could open the certificate, let's send it
@@ -686,6 +704,7 @@ KSSLCertificateHome::KSSLAuthAction aa;
                                          "client certificate for the session "
                                          "failed."), i18n("SSL"));
          delete pkcs;  // we don't need this anymore
+         pkcs = 0L;
       } else {
          kdDebug(7029) << "Client SSL certificate is being used." << endl;
          setMetaData("ssl_using_client_cert", "TRUE");
@@ -696,12 +715,16 @@ KSSLCertificateHome::KSSLAuthAction aa;
       }
       d->pkcs = pkcs;
    }
-
 }
 
 
 
+bool TCPSlaveBase::usingTLS() const
+{
+    return d->usingTLS;
+}
 
+// ### remove this for KDE4 (misses const):
 bool TCPSlaveBase::usingTLS()
 {
     return d->usingTLS;
@@ -734,7 +757,22 @@ int TCPSlaveBase::verifyCertificate()
 
     KSSLCertificate& pc = d->kssl->peerInfo().getPeerCertificate();
 
-    KSSLCertificate::KSSLValidation ksv = pc.validate();
+    KSSLCertificate::KSSLValidationList ksvl = pc.validateVerbose(KSSLCertificate::SSLServer);
+
+   _IPmatchesCN = d->kssl->peerInfo().certMatchesAddress();
+   if (!_IPmatchesCN && !d->militantSSL) {  // force this if the user wants it
+      if (d->cc->getHostList(pc).contains(ourHost))
+         _IPmatchesCN = true;
+   }
+
+   if (!_IPmatchesCN)
+   {
+      ksvl << KSSLCertificate::InvalidHost;
+   }
+
+   KSSLCertificate::KSSLValidation ksv = KSSLCertificate::Ok;
+   if (!ksvl.isEmpty())
+      ksv = ksvl.first();
 
     /* Setting the various bits of meta-info that will be needed. */
     setMetaData("ssl_cipher", d->kssl->connectionInfo().getCipher());
@@ -747,7 +785,14 @@ int TCPSlaveBase::verifyCertificate()
     setMetaData("ssl_cipher_bits",
                   QString::number(d->kssl->connectionInfo().getCipherBits()));
     setMetaData("ssl_peer_ip", d->ip);
-    setMetaData("ssl_cert_state", QString::number(ksv));
+    
+    QString errorStr;
+    for(KSSLCertificate::KSSLValidationList::ConstIterator it = ksvl.begin();
+        it != ksvl.end(); ++it)
+    {
+       errorStr += QString::number(*it)+":";
+    }
+    setMetaData("ssl_cert_errors", errorStr);
     setMetaData("ssl_peer_certificate", pc.toString());
 
     if (pc.chain().isValid() && pc.chain().depth() > 1) {
@@ -760,18 +805,13 @@ int TCPSlaveBase::verifyCertificate()
        setMetaData("ssl_peer_chain", theChain);
     } else setMetaData("ssl_peer_chain", "");
 
+   setMetaData("ssl_cert_state", QString::number(ksv));
 
    if (ksv == KSSLCertificate::Ok) {
       rc = 1;
       setMetaData("ssl_action", "accept");
    }
 
-   _IPmatchesCN = d->kssl->peerInfo().certMatchesAddress();
-   if (!_IPmatchesCN && !d->militantSSL) {  // force this if the user wants it
-      if (d->cc->getHostList(pc).contains(ourHost))
-         _IPmatchesCN = true;
-   }
-
    kdDebug(7029) << "SSL HTTP frame the parent? " << metaData("main_frame_request") << endl;
    if (!hasMetaData("main_frame_request") || metaData("main_frame_request") == "TRUE") {
       // Since we're the parent, we need to teach the child.
@@ -782,22 +822,22 @@ int TCPSlaveBase::verifyCertificate()
                                          d->cc->getPolicyByCertificate(pc);
 
       //  - validation code
-      if (ksv != KSSLCertificate::Ok || !_IPmatchesCN) {
-   if (d->militantSSL) {
-         return -1;
-   }
+      if (ksv != KSSLCertificate::Ok) {
+         if (d->militantSSL) {
+            return -1;
+         }
 
          if (cp == KSSLCertificateCache::Unknown ||
              cp == KSSLCertificateCache::Ambiguous) {
             cp = KSSLCertificateCache::Prompt;
          } else {
-            // A policy was already set so let's honour that.
+            // A policy was already set so let's honor that.
             permacache = d->cc->isPermanent(pc);
          }
 
          if (!_IPmatchesCN && cp == KSSLCertificateCache::Accept) {
             cp = KSSLCertificateCache::Prompt;
-            ksv = KSSLCertificate::Ok;
+//            ksv = KSSLCertificate::Ok;
          }
 
          // Precondition: cp is one of Reject, Accept or Prompt
@@ -813,14 +853,14 @@ int TCPSlaveBase::verifyCertificate()
          case KSSLCertificateCache::Prompt:
            {
              do {
-                if (ksv == KSSLCertificate::Ok && !_IPmatchesCN) {
+                if (ksv == KSSLCertificate::InvalidHost) {
                         QString msg = i18n("The IP address of the host %1 "
                                            "does not match the one the "
                                            "certificate was issued to.");
                    result = messageBox( WarningYesNoCancel,
                               msg.arg(ourHost),
                               i18n("Server Authentication"),
-                              i18n("&Details..."),
+                              i18n("&Details"),
                               i18n("Co&ntinue") );
                 } else {
                    QString msg = i18n("The server certificate failed the "
@@ -828,7 +868,7 @@ int TCPSlaveBase::verifyCertificate()
                    result = messageBox( WarningYesNoCancel,
                               msg.arg(ourHost),
                               i18n("Server Authentication"),
-                              i18n("&Details..."),
+                              i18n("&Details"),
                               i18n("Co&ntinue") );
                 }
 
@@ -898,7 +938,7 @@ int TCPSlaveBase::verifyCertificate()
       bool certAndIPTheSame = (d->ip == metaData("ssl_parent_ip") &&
                                pc.toString() == metaData("ssl_parent_cert"));
 
-      if (ksv == KSSLCertificate::Ok && _IPmatchesCN) {
+      if (ksv == KSSLCertificate::Ok) {
         if (certAndIPTheSame) {       // success
           rc = 1;
           setMetaData("ssl_action", "accept");
@@ -957,7 +997,7 @@ int TCPSlaveBase::verifyCertificate()
              result = messageBox(WarningYesNoCancel,
                                  msg.arg(ourHost),
                                  i18n("Server Authentication"),
-                                 i18n("&Details..."),
+                                 i18n("&Details"),
                                  i18n("Co&ntinue"));
                 if (result == KMessageBox::Yes) {
                    if (!d->dcc) {
@@ -982,13 +1022,13 @@ int TCPSlaveBase::verifyCertificate()
              setMetaData("ssl_action", "accept");
              rc = 1;
              cp = KSSLCertificateCache::Accept;
-             result = messageBox( WarningYesNo,
-                               i18n("Would you like to accept this "
-                                    "certificate forever without "
-                                    "being prompted?"),
-                               i18n("Server Authentication"),
-                               i18n("&Forever"),
-                               i18n("&Current Sessions Only"));
+             result = messageBox(WarningYesNo,
+                                 i18n("Would you like to accept this "
+                                      "certificate forever without "
+                                      "being prompted?"),
+                                 i18n("Server Authentication"),
+                                 i18n("&Forever"),
+                                 i18n("&Current Sessions Only"));
              permacache = (result == KMessageBox::Yes);
              d->cc->addCertificate(pc, cp, permacache);
              d->cc->addHost(pc, ourHost);
@@ -1003,7 +1043,9 @@ int TCPSlaveBase::verifyCertificate()
     }
 
 
-   if (rc == -1) return rc;
+   if (rc == -1) {
+      return rc;
+   }
 
    if (metaData("ssl_activate_warnings") == "TRUE") {
    //  - entering SSL
@@ -1085,8 +1127,10 @@ bool TCPSlaveBase::isConnectionValid()
     int retval;
     do {
        retval = KSocks::self()->select(m_iSock+1, &rdfs, NULL, NULL, &tv);
+       if (wasKilled())
+          return false; // Beam us out of here
     } while ((retval == -1) && (errno == EAGAIN));
-    // retval == -1 ==> Error  
+    // retval == -1 ==> Error
     // retval ==  0 ==> Connection Idle
     // retval >=  1 ==> Connection Active
     //kdDebug(7029) << "TCPSlaveBase::isConnectionValid: select returned: "
@@ -1094,21 +1138,21 @@ bool TCPSlaveBase::isConnectionValid()
 
     if (retval == -1)
        return false;
-    
+
     if (retval == 0)
        return true;
-       
+
     // Connection is active, check if it has closed.
     char buffer[100];
     do {
        retval = KSocks::self()->recv(m_iSock, buffer, 80, MSG_PEEK);
-                     
+
     } while ((retval == -1) && (errno == EAGAIN));
     //kdDebug(7029) << "TCPSlaveBase::isConnectionValid: recv returned: "
     //                 << retval << endl;
     if (retval <= 0)
        return false; // Error or connection closed.
-       
+
     return true; // Connection still valid.
 }
 
@@ -1135,6 +1179,8 @@ bool TCPSlaveBase::waitForResponse( int t )
 reSelect:
   startTime = time(NULL);
   rc = KSocks::self()->select(m_iSock+1, &rd, NULL, NULL, &timeout);
+  if (wasKilled())
+    return false; // We're dead.
 
   if (rc == -1)
     return false;
@@ -1146,7 +1192,7 @@ reSelect:
   // returned too early (perhaps from an errant signal) and
   // start over with the remaining time
   int timeDone = time(NULL) - startTime;
-  if (timeDone < n) 
+  if (timeDone < n)
   {
     n -= timeDone;
     timeout.tv_sec = n;
@@ -1193,6 +1239,13 @@ bool TCPSlaveBase::doSSLHandShake( bool sendError )
 
     d->kssl->reInitialize();
 
+    if (hasMetaData("ssl_session_id")) {
+        KSSLSession *s = KSSLSession::fromString(metaData("ssl_session_id"));
+        if (s) {
+            d->kssl->setSession(s);
+            delete s;
+	}    
+    }
     certificatePrompt();
 
     if ( !d->realHost.isEmpty() )
@@ -1204,7 +1257,7 @@ bool TCPSlaveBase::doSSLHandShake( bool sendError )
     d->kssl->setPeerHost(msgHost);
 
     d->status = d->kssl->connect(m_iSock);
-    if (d->status < 0) 
+    if (d->status < 0)
     {
         closeDescriptor();
         if ( sendError )
@@ -1212,16 +1265,18 @@ bool TCPSlaveBase::doSSLHandShake( bool sendError )
         return false;
     }
 
+    setMetaData("ssl_session_id", d->kssl->session()->toString());
     setMetaData("ssl_in_use", "TRUE");
 
-    int rc = verifyCertificate();
-    if ( rc != 1 ) 
-    {
-        d->status = -1;
-        closeDescriptor();
-        if ( sendError )
-            error( ERR_COULD_NOT_CONNECT, msgHost);
-        return false;
+    if (!d->kssl->reusingSession()) {
+        int rc = verifyCertificate();
+        if ( rc != 1 ) {
+            d->status = -1;
+            closeDescriptor();
+            if ( sendError )
+                error( ERR_COULD_NOT_CONNECT, msgHost);
+            return false;
+        }
     }
 
     d->needSSLHandShake = false;
@@ -1231,7 +1286,7 @@ bool TCPSlaveBase::doSSLHandShake( bool sendError )
 }
 
 
-bool TCPSlaveBase::userAborted() const 
+bool TCPSlaveBase::userAborted() const
 {
    return d->userAborted;
 }
