@@ -1,6 +1,6 @@
 /* This file is part of the KDE project
  *
- * Copyright (C) 2000,2001 George Staikos <staikos@kde.org>
+ * Copyright (C) 2000-2003 George Staikos <staikos@kde.org>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -27,6 +27,7 @@
 
 #include <unistd.h>
 #include <qstring.h>
+#include <qstringlist.h>
 #include <qfile.h>
 
 #include "kssldefs.h"
@@ -218,6 +219,41 @@ return rc;
 }
 
 
+void KSSLCertificate::getEmails(QStringList &to) const {
+	to.clear();
+#ifdef KSSL_HAVE_SSL
+	if (!d->m_cert)
+		return;
+	
+	STACK *s = d->kossl->X509_get1_email(d->m_cert);
+	if (s) {
+		for(int n=0; n < s->num; n++) {
+			to.append(d->kossl->sk_value(s,n));
+		}
+		d->kossl->X509_email_free(s);
+	}
+#endif	
+}	
+
+
+QString KSSLCertificate::getKDEKey() const {
+	return getSubject() + " (" + getMD5DigestText() + ")";
+}
+
+
+QString KSSLCertificate::getMD5DigestFromKDEKey(const QString &k) {
+	QString rc;
+	int pos = k.findRev('(');
+	if (pos != -1) {
+		unsigned int len = k.length();
+		if (k.at(len-1) == ')') {
+			rc = k.mid(pos+1, len-pos-2);
+		}
+	}
+	return rc;
+}
+
+
 QString KSSLCertificate::getMD5DigestText() const {
 QString rc = "";
 
@@ -426,57 +462,55 @@ if (c) {
 		      << endl;
 	if (c->ex_flags & EXFLAG_KUSAGE)
 		kdDebug(7029) << "     --- Key Usage extensions found" << endl;
-        else
-		kdDebug(7029) << "     --- Key Usage extensions NOT found" << endl;
+        else kdDebug(7029) << "     --- Key Usage extensions NOT found" << endl;
+
 	if (c->ex_flags & EXFLAG_XKUSAGE)
 		kdDebug(7029) << "     --- Extended key usage extensions found" << endl;
-        else
-		kdDebug(7029) << "     --- Extended key usage extensions NOT found" << endl;
+        else kdDebug(7029) << "     --- Extended key usage extensions NOT found" << endl;
+
 	if (c->ex_flags & EXFLAG_NSCERT)
 		kdDebug(7029) << "     --- NS extensions found" << endl;
-        else
-		kdDebug(7029) << "     --- NS extensions NOT found" << endl;
+        else kdDebug(7029) << "     --- NS extensions NOT found" << endl;
 
         if (d->_extensions.certTypeSSLCA())
                 kdDebug(7029) << "NOTE: this is an SSL CA file." << endl;
-        else
-                kdDebug(7029) << "NOTE: this is NOT an SSL CA file." << endl;
+        else kdDebug(7029) << "NOTE: this is NOT an SSL CA file." << endl;
+
         if (d->_extensions.certTypeEmailCA())
                 kdDebug(7029) << "NOTE: this is an EMAIL CA file." << endl;
-        else
-                kdDebug(7029) << "NOTE: this is NOT an EMAIL CA file." << endl;
+        else kdDebug(7029) << "NOTE: this is NOT an EMAIL CA file." << endl;
+
         if (d->_extensions.certTypeCodeCA())
                 kdDebug(7029) << "NOTE: this is a CODE CA file." << endl;
-        else
-                kdDebug(7029) << "NOTE: this is NOT a CODE CA file." << endl;
+        else kdDebug(7029) << "NOTE: this is NOT a CODE CA file." << endl;
+
         if (d->_extensions.certTypeSSLClient())
                 kdDebug(7029) << "NOTE: this is an SSL client." << endl;
-        else
-                kdDebug(7029) << "NOTE: this is NOT an SSL client." << endl;
+        else kdDebug(7029) << "NOTE: this is NOT an SSL client." << endl;
+
         if (d->_extensions.certTypeSSLServer())
                 kdDebug(7029) << "NOTE: this is an SSL server." << endl;
-        else
-                kdDebug(7029) << "NOTE: this is NOT an SSL server." << endl;
+        else kdDebug(7029) << "NOTE: this is NOT an SSL server." << endl;
+
         if (d->_extensions.certTypeNSSSLServer())
                 kdDebug(7029) << "NOTE: this is a NETSCAPE SSL server." << endl;
-        else
-                kdDebug(7029) << "NOTE: this is NOT a NETSCAPE SSL server." << endl;
+        else kdDebug(7029) << "NOTE: this is NOT a NETSCAPE SSL server." << endl;
+
         if (d->_extensions.certTypeSMIME())
                 kdDebug(7029) << "NOTE: this is an SMIME certificate." << endl;
-        else
-                kdDebug(7029) << "NOTE: this is NOT an SMIME certificate." << endl;
+        else kdDebug(7029) << "NOTE: this is NOT an SMIME certificate." << endl;
+
         if (d->_extensions.certTypeSMIMEEncrypt())
                 kdDebug(7029) << "NOTE: this is an SMIME encrypt cert." << endl;
-        else
-                kdDebug(7029) << "NOTE: this is NOT an SMIME encrypt cert." << endl;
+        else kdDebug(7029) << "NOTE: this is NOT an SMIME encrypt cert." << endl;
+
         if (d->_extensions.certTypeSMIMESign())
                 kdDebug(7029) << "NOTE: this is an SMIME sign cert." << endl;
-        else
-                kdDebug(7029) << "NOTE: this is NOT an SMIME sign cert." << endl;
+        else kdDebug(7029) << "NOTE: this is NOT an SMIME sign cert." << endl;
+
         if (d->_extensions.certTypeCRLSign())
                 kdDebug(7029) << "NOTE: this is a CRL signer." << endl;
-        else
-                kdDebug(7029) << "NOTE: this is NOT a CRL signer." << endl;
+        else kdDebug(7029) << "NOTE: this is NOT a CRL signer." << endl;
 
 	kdDebug(7029) << "-----------------------------------------------" 
 		      << endl;
@@ -534,15 +568,23 @@ KSSLCertificate::KSSLValidation KSSLCertificate::validate() {
 	return validate(KSSLCertificate::SSLServer);
 }
 
+KSSLCertificate::KSSLValidation KSSLCertificate::validate(KSSLCertificate::KSSLPurpose purpose)
+{
+	KSSLValidationList result = validateVerbose(purpose);
+	if (result.isEmpty())
+		return KSSLCertificate::Ok;
+	else
+		return result.first();
+} 
 
 //
 // See apps/verify.c in OpenSSL for the source of most of this logic.
 //
 
 // CRL files?  we don't do that yet
-
-KSSLCertificate::KSSLValidation KSSLCertificate::validate(KSSLCertificate::KSSLPurpose purpose) {
-
+KSSLCertificate::KSSLValidationList KSSLCertificate::validateVerbose(KSSLCertificate::KSSLPurpose purpose) 
+{
+	KSSLValidationList errors;
 	if (d->_lastPurpose != purpose && d->m_stateCached) {
 		d->m_stateCached = false;
 	}
@@ -551,22 +593,27 @@ KSSLCertificate::KSSLValidation KSSLCertificate::validate(KSSLCertificate::KSSLP
 		d->_lastPurpose = purpose;
 
 #ifdef KSSL_HAVE_SSL
-X509_STORE *certStore;
-X509_LOOKUP *certLookup;
-X509_STORE_CTX *certStoreCTX;
-int rc = 0;
+	X509_STORE *certStore;
+	X509_LOOKUP *certLookup;
+	X509_STORE_CTX *certStoreCTX;
+	int rc = 0;
 
 	if (!d->m_cert)
-		return KSSLCertificate::Unknown;
+	{
+		errors << KSSLCertificate::Unknown;
+		return errors;
+	}
 
 	if (d->m_stateCached) {
-		return d->m_stateCache;
+		errors << d->m_stateCache;
+		return errors;
 	}
 
 	QStringList qsl = KGlobal::dirs()->resourceDirs("kssl");
 
 	if (qsl.isEmpty()) {
-		return KSSLCertificate::NoCARoot;
+		errors << KSSLCertificate::NoCARoot;
+		return errors;
 	}
 
 	KSSLCertificate::KSSLValidation ksslv = Unknown;
@@ -578,7 +625,10 @@ int rc = 0;
 
 		certStore = d->kossl->X509_STORE_new();
 		if (!certStore)
-			return KSSLCertificate::Unknown;
+		{
+			errors << KSSLCertificate::Unknown;
+			return errors;
+		}
 
 		X509_STORE_set_verify_cb_func(certStore, X509Callback);
 
@@ -651,11 +701,13 @@ int rc = 0;
 			break;
 		}
 	}
-
-return (d->m_stateCache);
-
+	
+	if (ksslv != KSSLCertificate::Ok)
+		errors << ksslv;
+#else
+	errors << KSSLCertificate::NoSSL;
 #endif
-return NoSSL;
+	return errors;
 }
 
 
@@ -849,6 +901,8 @@ case KSSLCertificate::InvalidPurpose:
 	return i18n("Rejected, possibly due to an invalid purpose.");
 case KSSLCertificate::PrivateKeyFailed:
 	return i18n("Private key test failed.");
+case KSSLCertificate::InvalidHost:
+	return i18n("The certificate has not been issued for this host.");
 default:
 break;
 }
@@ -959,7 +1013,7 @@ KTempFile ktf;
 return text;
 }
 
-
+// KDE 4: Make it const QString &
 bool KSSLCertificate::setCert(QString& cert) {
 #ifdef KSSL_HAVE_SSL
 QByteArray qba, qbb = cert.local8Bit().copy();
@@ -1008,7 +1062,7 @@ QString cert;
 s >> cert >> qsl;
 
 	if (r.setCert(cert) && !qsl.isEmpty())
-		r.chain().setChain(qsl);
+		r.chain().setCertChain(qsl);
 
 return s;
 }
