@@ -24,7 +24,6 @@
 #include <kssl.h>
 
 #include <qlayout.h>
-#include <qpushbutton.h>
 #include <qframe.h>
 #include <qlabel.h>
 #include <qscrollview.h>
@@ -36,10 +35,14 @@
 #include <kprocess.h>
 #include <kiconloader.h>
 #include <kglobalsettings.h>
+#include <ksqueezedtextlabel.h>
 #include <kurllabel.h>
 //#include <kstandarddirs.h>
 //#include <krun.h>
 #include <kcombobox.h>
+#include <kpushbutton.h>
+#include <kstdguiitem.h>
+
 #include "ksslcertificate.h"
 #include "ksslcertchain.h"
 #include "ksslsigners.h"
@@ -52,6 +55,8 @@ private:
     QGridLayout *m_layout;
     KComboBox *_chain;
     KSSLCertificate *_cert;
+    KSSLCertificate::KSSLValidationList _cert_ksvl;
+
     bool inQuestion;
 
     QLabel *_serialNum;
@@ -101,13 +106,14 @@ KSSLInfoDlg::KSSLInfoDlg(bool secureConnection, QWidget *parent, const char *nam
 
     QPushButton *button;
 
+    bool buttonicon =  KGlobalSettings::showIconsOnPushButtons();
     if (KSSL::doesSSLWork()) {
-      button = new QPushButton(i18n("C&ryptography Configuration..."), this);
+      button = new QPushButton(buttonicon ? SmallIcon("configure") : QPixmap(0), i18n("C&ryptography Configuration..."), this);
       connect(button, SIGNAL(clicked()), SLOT(launchConfig()));
       buttonLayout->addWidget( button );
     }
 
-    button = new QPushButton(i18n("&Close"), this);
+    button = new KPushButton(KStdGuiItem::close(), this);
     connect(button, SIGNAL(clicked()), SLOT(close()));
     buttonLayout->addWidget( button );
 
@@ -203,9 +209,9 @@ void KSSLInfoDlg::setup(KSSLCertificate *cert,
     } else d->_chain->setEnabled(false);
 
     layout->addWidget(new QLabel(i18n("Peer certificate:"), this), 2, 0);
-    layout->addWidget(d->_subject = dynamic_cast<KSSLCertBox*>(buildCertInfo(cert->getSubject())), 3, 0);
+    layout->addWidget(d->_subject = static_cast<KSSLCertBox*>(buildCertInfo(cert->getSubject())), 3, 0);
     layout->addWidget(new QLabel(i18n("Issuer:"), this), 2, 1);
-    layout->addWidget(d->_issuer = dynamic_cast<KSSLCertBox*>(buildCertInfo(cert->getIssuer())), 3, 1);
+    layout->addWidget(d->_issuer = static_cast<KSSLCertBox*>(buildCertInfo(cert->getIssuer())), 3, 1);
     d->m_layout->addMultiCell(layout, 1, 1, 0, 2);
 
     layout = new QGridLayout(11, 2, KDialog::spacingHint());
@@ -213,20 +219,14 @@ void KSSLInfoDlg::setup(KSSLCertificate *cert,
     layout->addWidget(new QLabel(i18n("IP address:"), this), 0, 0);
     layout->addWidget(new QLabel(ip, this), 0, 1);
     layout->addWidget(new QLabel(i18n("URL:"), this), 1, 0);
-    // truncate the label if it will be too long
-    QString urllabel;
-    if (url.length() > 80) {
-      urllabel = url.left(80) + " ...";
-    } else urllabel = url;
-    KURLLabel *urlLabel = new KURLLabel(url, urllabel, this);
+    KSqueezedTextLabel *urlLabel = new KSqueezedTextLabel(url, this);
     layout->addWidget(urlLabel, 1, 1);
-    connect(urlLabel, SIGNAL(leftClickedURL(const QString &)), SLOT(urlClicked(const QString &)));
     layout->addWidget(new QLabel(i18n("Certificate state:"), this), 2, 0);
 
     layout->addWidget(d->_csl = new QLabel("", this), 2, 1);
 
     update();
-    
+
     layout->addWidget(new QLabel(i18n("Valid from:"), this), 3, 0);
     layout->addWidget(d->_validFrom = new QLabel("", this), 3, 1);
     layout->addWidget(new QLabel(i18n("Valid until:"), this), 4, 0);
@@ -250,6 +250,16 @@ void KSSLInfoDlg::setup(KSSLCertificate *cert,
     displayCert(cert);
 }
 
+void KSSLInfoDlg::setCertState(const QString &errorNrs)
+{
+    d->_cert_ksvl.clear();
+    QStringList errors = QStringList::split(':', errorNrs);
+    for(QStringList::ConstIterator it = errors.begin();
+        it != errors.end(); ++it)
+    {
+       d->_cert_ksvl << (KSSLCertificate::KSSLValidation) (*it).toInt();
+    }
+}
 
 void KSSLInfoDlg::displayCert(KSSLCertificate *x) {
 QPalette cspl;
@@ -257,37 +267,57 @@ QPalette cspl;
    d->_serialNum->setText(x->getSerialNumber());
 
    cspl = d->_validFrom->palette();
-   if (x->getQDTNotBefore() > QDateTime::currentDateTime())
+   if (x->getQDTNotBefore() > QDateTime::currentDateTime(Qt::UTC))
       cspl.setColor(QColorGroup::Foreground, QColor(196,33,21));
    else cspl.setColor(QColorGroup::Foreground, QColor(42,153,59));
    d->_validFrom->setPalette(cspl);
    d->_validFrom->setText(x->getNotBefore());
 
    cspl = d->_validUntil->palette();
-   if (x->getQDTNotAfter() < QDateTime::currentDateTime())
+   if (x->getQDTNotAfter() < QDateTime::currentDateTime(Qt::UTC))
       cspl.setColor(QColorGroup::Foreground, QColor(196,33,21));
    else cspl.setColor(QColorGroup::Foreground, QColor(42,153,59));
    d->_validUntil->setPalette(cspl);
    d->_validUntil->setText(x->getNotAfter());
 
    cspl = d->_csl->palette();
-   KSSLCertificate::KSSLValidation ksv = x->validate();
-   if (ksv == KSSLCertificate::SelfSigned) {
-	  if (x->getQDTNotAfter() > QDateTime::currentDateTime() &&
-		  x->getQDTNotBefore() < QDateTime::currentDateTime()) {
-	      if (KSSLSigners().useForSSL(*x))
-    	     ksv = KSSLCertificate::Ok;
-	  } else {
-		  ksv = KSSLCertificate::Expired;
-	  }
+
+   KSSLCertificate::KSSLValidation ksv;
+   KSSLCertificate::KSSLValidationList ksvl;
+   if ((x == d->_cert) && !d->_cert_ksvl.isEmpty()) {
+      ksvl = d->_cert_ksvl;
+      ksv = ksvl.first();
+   } else {
+      ksv = x->validate();
+      if (ksv == KSSLCertificate::SelfSigned) {
+         if (x->getQDTNotAfter() > QDateTime::currentDateTime(Qt::UTC) &&
+             x->getQDTNotBefore() < QDateTime::currentDateTime(Qt::UTC)) {
+            if (KSSLSigners().useForSSL(*x))
+               ksv = KSSLCertificate::Ok;
+	 } else {
+            ksv = KSSLCertificate::Expired;
+	 }
+      }
+      ksvl << ksv;
    }
 
-   if (ksv != KSSLCertificate::Ok)
+   if (ksv != KSSLCertificate::Ok) {
       cspl.setColor(QColorGroup::Foreground, QColor(196,33,21));
-   else cspl.setColor(QColorGroup::Foreground, QColor(42,153,59));
+   } else {
+      cspl.setColor(QColorGroup::Foreground, QColor(42,153,59));
+   }
    d->_csl->setPalette(cspl);
 
-   d->_csl->setText(KSSLCertificate::verifyText(ksv));
+   QString errorStr;
+   for(KSSLCertificate::KSSLValidationList::ConstIterator it = ksvl.begin();
+       it != ksvl.end(); ++it) {
+      if (!errorStr.isEmpty())
+         errorStr.append('\n');
+      errorStr += KSSLCertificate::verifyText(*it);
+   }
+
+   d->_csl->setText(errorStr);
+   d->_csl->setMinimumSize(d->_csl->sizeHint());
 
    d->_subject->setValues(x->getSubject());
    d->_issuer->setValues(x->getIssuer());
@@ -319,7 +349,7 @@ KSSLCertBox *KSSLInfoDlg::certInfoWidget(QWidget *parent, const QString &certNam
 }
 
 
-KSSLCertBox::KSSLCertBox(QWidget *parent, const char *name, WFlags f) 
+KSSLCertBox::KSSLCertBox(QWidget *parent, const char *name, WFlags f)
 :            QScrollView(parent, name, f)
 {
     _frame = NULL;
@@ -357,7 +387,7 @@ void KSSLCertBox::setValues(QString certName, QWidget *mailCatcher) {
         new QLabel(tmp, _frame);
     }
     if (!(tmp = cert.getValue("ST")).isEmpty()) {
-        label = new QLabel(i18n("State:"), _frame);
+        label = new QLabel(i18n("Federal State","State:"), _frame);
         label->setAlignment(Qt::AlignLeft | Qt::AlignTop);
         new QLabel(tmp, _frame);
     }
