@@ -1,6 +1,6 @@
 /* This file is part of the KDE libraries
    Copyright (C) 2001-2002 Michael Hckel <haeckel@kde.org>
-   $Id: kdesasl.cpp,v 1.8.4.1 2003/01/29 18:33:50 mcamen Exp $
+   $Id: kdesasl.cpp,v 1.12.2.1 2004/02/10 23:06:28 porten Exp $
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
@@ -17,18 +17,22 @@
    Boston, MA 02111-1307, USA.
 */
 
-#include <stdlib.h>
-#include <qstrlist.h>
+#include "kdesasl.h"
+
 #include <kmdcodec.h>
 #include <kurl.h>
-#include "kdesasl.h"
+
+#include <qstrlist.h>
+
+#include <stdlib.h>
+#include <string.h>
 
 KDESasl::KDESasl(const KURL &aUrl)
 {
   mProtocol = aUrl.protocol();
   mUser = aUrl.user();
   mPass = aUrl.pass();
-  mFirst = TRUE;
+  mFirst = true;
 }
 
 KDESasl::KDESasl(const QString &aUser, const QString &aPass,
@@ -37,7 +41,7 @@ KDESasl::KDESasl(const QString &aUser, const QString &aPass,
   mProtocol = aProtocol;
   mUser = aUser;
   mPass = aPass;
-  mFirst = TRUE;
+  mFirst = true;
 }
 
 KDESasl::~KDESasl() {
@@ -47,15 +51,15 @@ QCString KDESasl::chooseMethod(const QStrIList aMethods)
 {
   if (aMethods.contains("DIGEST-MD5")) mMethod = "DIGEST-MD5";
   else if (aMethods.contains("CRAM-MD5")) mMethod = "CRAM-MD5";
-  else if (aMethods.contains("LOGIN")) mMethod = "LOGIN";
   else if (aMethods.contains("PLAIN")) mMethod = "PLAIN";
+  else if (aMethods.contains("LOGIN")) mMethod = "LOGIN";
   else mMethod = QCString();
   return mMethod;
 }
 
 void KDESasl::setMethod(const QCString &aMethod)
 {
-  mMethod = aMethod;
+  mMethod = aMethod.upper();
 }
 
 QByteArray KDESasl::getPlainResponse()
@@ -64,9 +68,15 @@ QByteArray KDESasl::getPlainResponse()
   QCString pass = mPass.utf8();
   int userlen = user.length();
   int passlen = pass.length();
+  // result = $user\0$user\0$pass (no trailing \0)
   QByteArray result(2 * userlen + passlen + 2);
-  for (int i = 0; i <= userlen; i++) result[i+userlen+1] = result[i] = user[i];
-  for (int i = 0; i < passlen; i++) result[i+2*userlen+2] = pass[i];
+  if ( userlen ) {
+    memcpy( result.data(), user.data(), userlen );
+    memcpy( result.data() + userlen + 1, user.data(), userlen );
+  }
+  if ( passlen )
+    memcpy( result.data() + 2 * userlen + 2, pass.data(), passlen );
+  result[userlen] = result[2*userlen+1] = '\0';
   return result;
 }
 
@@ -87,7 +97,7 @@ QByteArray KDESasl::getCramMd5Response(const QByteArray &aChallenge)
   if (secret.size() > 64)
   {
     KMD5 md5(secret);
-    secret.duplicate((const char*)md5.rawDigest(), 16);
+    secret.duplicate((const char*)(&(md5.rawDigest()[0])), 16);
     len = 16;
   }
   secret.resize(64);
@@ -235,7 +245,7 @@ QByteArray KDESasl::getBinaryResponse(const QByteArray &aChallenge, bool aBase64
   {
     QByteArray ba;
     KCodecs::base64Decode(aChallenge, ba);
-    KCodecs::base64Encode(getBinaryResponse(ba, FALSE), ba);
+    KCodecs::base64Encode(getBinaryResponse(ba, false), ba);
     return ba;
   }
   if (qstricmp(mMethod, "PLAIN") == 0) return getPlainResponse();
@@ -253,3 +263,23 @@ QCString KDESasl::getResponse(const QByteArray &aChallenge, bool aBase64)
   QByteArray ba = getBinaryResponse(aChallenge, aBase64);
   return QCString(ba.data(), ba.size() + 1);
 }
+
+QCString KDESasl::method() const {
+  return mMethod;
+}
+
+bool KDESasl::clientStarts() const {
+  return method() == "PLAIN";
+}
+
+bool KDESasl::dialogComplete( int n ) const {
+  if ( method() == "PLAIN" || method() == "CRAM-MD5" )
+    return n >= 1;
+  if ( method() == "LOGIN" || method() == "DIGEST-MD5" )
+    return n >= 2;
+  return true;
+}
+
+bool KDESasl::isClearTextMethod() const {
+  return method() == "PLAIN" || method() == "LOGIN" ;
+}
