@@ -27,6 +27,9 @@
 #include <ksslcertificatecache.h>
 #include <qstring.h>
 #include <qstringlist.h>
+#include <qvaluelist.h>
+#include <qmap.h>
+#include <qptrvector.h>
 
 
 class KSimpleConfig;
@@ -59,6 +62,7 @@ k_dcop:
   bool cacheSeenCertificate(KSSLCertificate cert);
 
   bool cacheRemoveByCN(QString cn);
+  bool cacheRemoveBySubject(QString subject);
   bool cacheRemoveByCertificate(KSSLCertificate cert);
 	       
   bool cacheIsPermanent(KSSLCertificate cert);
@@ -82,6 +86,7 @@ k_dcop:
   bool cacheRemoveHost(KSSLCertificate cert, QString host);
 
   /* Certificate Authorities */
+  void caVerifyUpdate();
   bool caRegenerate();
 
   QStringList caList();
@@ -100,6 +105,10 @@ k_dcop:
 
   bool caSetUse(QString subject, bool ssl, bool email, bool code);
 
+  QStringList getKDEKeyByEmail(const QString &email);
+
+  KSSLCertificate getCertByMD5Digest(const QString &key);
+
   //
   //  Certificate Home methods
   //
@@ -116,6 +125,13 @@ private:
 
   // Our pointer to OpenSSL
   KOpenSSLProxy *kossl;
+
+  // 
+  void searchAddCert(KSSLCertificate *cert);
+  void searchRemoveCert(KSSLCertificate *cert);
+
+  QMap<QString, QPtrVector<KSSLCertificate> > skEmail;
+  QMap<QString, KSSLCertificate *> skMD5Digest;
 };
 
 
