@@ -19,9 +19,10 @@
    Boston, MA 02111-1307, USA.
 */
 
-// $Id: ktelnetservice.cpp,v 1.3.2.2 2003/03/31 09:13:55 waba Exp $
+// $Id: ktelnetservice.cpp,v 1.7.2.2 2004/05/14 12:37:54 waba Exp $
 
 #include <kapplication.h>
+#include <kmessagebox.h>
 #include <kcmdlineargs.h>
 #include <kdebug.h>
 #include <klocale.h>
@@ -30,22 +31,20 @@
 static const KCmdLineOptions options[] =
 {
 	{"+url", 0, 0},
-	{0, 0, 0}
+	KCmdLineLastOption
 };
 
 int main(int argc, char **argv)
 {
 	KLocale::setMainCatalogue("kdelibs");
-	KCmdLineArgs::init(argc, argv, "ktelnetservice", I18N_NOOP("telnet service"), I18N_NOOP("telnet protocol handler"));
+	KCmdLineArgs::init(argc, argv, "ktelnetservice", I18N_NOOP("telnet service"),
+			   I18N_NOOP("telnet protocol handler"), "unknown");
 	KCmdLineArgs::addCmdLineOptions(options);
 
 	KApplication app;
-	
-	if (!app.authorize("shell_access"))
-		return 3;
 
 	KCmdLineArgs *args = KCmdLineArgs::parsedArgs();
-
+	
 	if (args->count() != 1)
 		return 1;
 
@@ -56,25 +55,48 @@ int main(int argc, char **argv)
 	cmd << "-e";
         if ( url.protocol() == "telnet" )
             cmd << "telnet";
+        else if ( url.protocol() == "ssh" )
+            cmd << "ssh";
         else if ( url.protocol() == "rlogin" )
             cmd << "rlogin";
         else {
             kdError() << "Invalid protocol " << url.protocol() << endl;
             return 2;
         }
+        
+        if (!app.authorize("shell_access"))
+        {
+            KMessageBox::sorry(0, 
+            	i18n("You do not have permission to access the %1 protocol.").arg(url.protocol()));
+            return 3;
+        }
+        
 	if (!url.user().isEmpty())
 	{
 		cmd << "-l";
 		cmd << url.user();
 	}
 
+        QString host;
         if (!url.host().isEmpty())
-           cmd << url.host(); // telnet://host
+           host = url.host(); // telnet://host
         else if (!url.path().isEmpty())
-           cmd << url.path(); // telnet:host
+           host = url.path(); // telnet:host
+
+        if (host.isEmpty() || host.startsWith("-"))
+        {
+            kdError() << "Invalid hostname " << host << endl;
+            return 2;
+        }
+
+        cmd << host;
         
-	if (url.port())
+	if (url.port()){
+            if ( url.protocol() == "ssh" )
+		cmd << "-p" << QString::number(url.port());
+	    else
 		cmd << QString::number(url.port());
+	}
 
 	app.kdeinitExec("konsole", cmd);
 
