@@ -19,8 +19,13 @@
 */
 // -*- mode: c++; c-basic-offset: 4 -*-
 
-#include <qregexp.h>
+#include <qtimer.h>
 
+#include <qregexp.h>
+#include <qheader.h>
+#include <qevent.h>
+
+#include <ksqueezedtextlabel.h>
 #include <kconfig.h>
 #include <kstandarddirs.h>
 #include <kuniqueapplication.h>
@@ -28,13 +33,22 @@
 #include <kcmdlineargs.h>
 #include <kglobal.h>
 #include <klocale.h>
-#include <kiconloader.h>
 #include <dcopclient.h>
 #include <kstatusbar.h>
 #include <kdebug.h>
 #include <kmessagebox.h>
 #include <kdesu/client.h>
 #include <kwin.h>
+#include <kdialog.h>
+#include <ksystemtray.h>
+#include <kpopupmenu.h>
+#include <kaction.h>
+
+#include <qcheckbox.h>
+#include <qlabel.h>
+#include <qlayout.h>
+#include <qpopupmenu.h>
+#include <qheader.h>
 
 #include "observer_stub.h"
 #include "observer.h" // for static methods only
@@ -55,7 +69,7 @@
 UIServer* uiserver;
 
 // ToolBar field IDs
-enum { TOOL_CANCEL };
+enum { TOOL_CANCEL, TOOL_CONFIGURE };
 
 // StatusBar field IDs
 enum { ID_TOTAL_FILES = 1, ID_TOTAL_SIZE, ID_TOTAL_TIME, ID_TOTAL_SPEED };
@@ -74,8 +88,91 @@ static const int defaultColumnWidth[] = { 70,  // SIZE_OPERATION
                                     450  // URL
 };
 
-// number of listview columns
-#define NUM_COLS  9
+class UIServerSystemTray:public KSystemTray
+{
+   public:
+      UIServerSystemTray(UIServer* uis)
+         :KSystemTray(uis)
+      {
+         KPopupMenu* pop= contextMenu();
+         pop->insertItem(i18n("Settings..."), uis, SLOT(slotConfigure()));
+         pop->insertItem(i18n("Remove"), uis, SLOT(slotRemoveSystemTrayIcon()));
+         setPixmap(loadIcon("filesave"));
+         //actionCollection()->action("file_quit")->setEnabled(true);
+         KStdAction::quit(uis, SLOT(slotQuit()), actionCollection());
+      }
+};
+
+class ProgressConfigDialog:public KDialogBase
+{
+   public:
+      ProgressConfigDialog(QWidget* parent);
+      ~ProgressConfigDialog()  {}
+      void setChecked(int i, bool on);
+      bool isChecked(int i) const;
+      friend class UIServer;
+   private:
+      QCheckBox *m_showSystemTrayCb;
+      QCheckBox *m_keepOpenCb;
+      QCheckBox *m_toolBarCb;
+      QCheckBox *m_statusBarCb;
+      QCheckBox *m_headerCb;
+      QCheckBox *m_fixedWidthCb;
+      KListView *m_columns;
+      QCheckListItem *(m_items[ListProgress::TB_MAX]);
+};
+
+ProgressConfigDialog::ProgressConfigDialog(QWidget *parent)
+:KDialogBase(KDialogBase::Plain,i18n("Configure Network Operation Window"),KDialogBase::Ok|KDialogBase::Apply|KDialogBase::Cancel,
+             KDialogBase::Ok, parent, "configprog", false)
+{
+   QVBoxLayout *layout=new QVBoxLayout(plainPage(),spacingHint());
+   m_showSystemTrayCb=new QCheckBox(i18n("Show system tray icon"), plainPage());
+   m_keepOpenCb=new QCheckBox(i18n("Keep network operation window always open"), plainPage());
+   m_headerCb=new QCheckBox(i18n("Show column headers"), plainPage());
+   m_toolBarCb=new QCheckBox(i18n("Show toolbar"), plainPage());
+   m_statusBarCb=new QCheckBox(i18n("Show statusbar"), plainPage());
+   m_fixedWidthCb=new QCheckBox(i18n("Column widths are user adjustable"), plainPage());
+   QLabel *label=new QLabel(i18n("Show information:"), plainPage());
+   m_columns=new KListView(plainPage());
+
+   m_columns->addColumn("info");
+   m_columns->setSorting(-1);
+   m_columns->header()->hide();
+
+   m_items[ListProgress::TB_ADDRESS]        =new QCheckListItem(m_columns, i18n("URL"), QCheckListItem::CheckBox);
+   m_items[ListProgress::TB_REMAINING_TIME] =new QCheckListItem(m_columns, i18n("Rem. Time"), QCheckListItem::CheckBox);
+   m_items[ListProgress::TB_SPEED]          =new QCheckListItem(m_columns, i18n("Speed"), QCheckListItem::CheckBox);
+   m_items[ListProgress::TB_TOTAL]          =new QCheckListItem(m_columns, i18n("Size"), QCheckListItem::CheckBox);
+   m_items[ListProgress::TB_PROGRESS]       =new QCheckListItem(m_columns, i18n("%"), QCheckListItem::CheckBox);
+   m_items[ListProgress::TB_COUNT]          =new QCheckListItem(m_columns, i18n("Count"), QCheckListItem::CheckBox);
+   m_items[ListProgress::TB_RESUME]         =new QCheckListItem(m_columns, i18n("Resume", "Res."), QCheckListItem::CheckBox);
+   m_items[ListProgress::TB_LOCAL_FILENAME] =new QCheckListItem(m_columns, i18n("Local Filename"), QCheckListItem::CheckBox);
+   m_items[ListProgress::TB_OPERATION]      =new QCheckListItem(m_columns, i18n("Operation"), QCheckListItem::CheckBox);
+
+   layout->addWidget(m_showSystemTrayCb);
+   layout->addWidget(m_keepOpenCb);
+   layout->addWidget(m_headerCb);
+   layout->addWidget(m_toolBarCb);
+   layout->addWidget(m_statusBarCb);
+   layout->addWidget(m_fixedWidthCb);
+   layout->addWidget(label);
+   layout->addWidget(m_columns);
+}
+
+void ProgressConfigDialog::setChecked(int i, bool on)
+{
+   if (i>=ListProgress::TB_MAX)
+      return;
+   m_items[i]->setOn(on);
+}
+
+bool ProgressConfigDialog::isChecked(int i) const
+{
+   if (i>=ListProgress::TB_MAX)
+      return false;
+   return m_items[i]->isOn();
+}
 
 ProgressItem::ProgressItem( ListProgress* view, QListViewItem *after, QCString app_id, int job_id,
                             bool showDefault )
@@ -99,17 +196,23 @@ ProgressItem::ProgressItem( ListProgress* view, QListViewItem *after, QCString a
   defaultProgress->setOnlyClean( true );
   connect ( defaultProgress, SIGNAL( stopped() ), this, SLOT( slotCanceled() ) );
   connect ( &m_showTimer, SIGNAL( timeout() ), this, SLOT(slotShowDefaultProgress()) );
-  
+
   if ( showDefault ) {
     m_showTimer.start( 500, true );
   }
 }
 
+bool ProgressItem::keepOpen() const
+{
+    return defaultProgress->keepOpen();
+}
+
+void ProgressItem::finished()
+{
+    defaultProgress->finished();
+}
 
 ProgressItem::~ProgressItem() {
-  if (defaultProgress->onlyClean())
-    defaultProgress->finished();
-  else
     delete defaultProgress;
 }
 
@@ -139,7 +242,7 @@ void ProgressItem::setTotalDirs( unsigned long dirs ) {
 void ProgressItem::setProcessedSize( KIO::filesize_t size ) {
   m_iProcessedSize = size;
 
-  setText( listProgress->lv_size, KIO::convertSize( size ) );
+  setText( ListProgress::TB_TOTAL, KIO::convertSize( size ) );
 
   defaultProgress->slotProcessedSize( 0, size );
 }
@@ -149,7 +252,7 @@ void ProgressItem::setProcessedFiles( unsigned long files ) {
   m_iProcessedFiles = files;
 
   QString tmps = i18n("%1 / %2").arg( m_iProcessedFiles ).arg( m_iTotalFiles );
-  setText( listProgress->lv_count, tmps );
+  setText( ListProgress::TB_COUNT, tmps );
 
   defaultProgress->slotProcessedFiles( 0, m_iProcessedFiles );
 }
@@ -162,7 +265,7 @@ void ProgressItem::setProcessedDirs( unsigned long dirs ) {
 
 void ProgressItem::setPercent( unsigned long percent ) {
   QString tmps = i18n( "%1 % of %2 ").arg( percent ).arg( KIO::convertSize(m_iTotalSize));
-  setText( listProgress->lv_progress, tmps );
+  setText( ListProgress::TB_PROGRESS, tmps );
 
   defaultProgress->slotPercent( 0, percent );
 }
@@ -171,7 +274,7 @@ void ProgressItem::setInfoMessage( const QString & msg ) {
   QString plainTextMsg(msg);
   plainTextMsg.replace( QRegExp( "</?b>" ), QString::null );
   plainTextMsg.replace( QRegExp( "<img.*>" ), QString::null );
-  setText( listProgress->lv_progress, plainTextMsg );
+  setText( ListProgress::TB_PROGRESS, plainTextMsg );
 
   defaultProgress->slotInfoMessage( 0, msg );
 }
@@ -188,76 +291,93 @@ void ProgressItem::setSpeed( unsigned long bytes_per_second ) {
     tmps = i18n( "%1/s").arg( KIO::convertSize( m_iSpeed ));
     tmps2 = m_remainingTime.toString();
   }
-  setText( listProgress->lv_speed, tmps );
-  setText( listProgress->lv_remaining, tmps2 );
+  setText( ListProgress::TB_SPEED, tmps );
+  setText( ListProgress::TB_REMAINING_TIME, tmps2 );
 
   defaultProgress->slotSpeed( 0, m_iSpeed );
 }
 
 
 void ProgressItem::setCopying( const KURL& from, const KURL& to ) {
-  setText( listProgress->lv_operation, i18n("Copying") );
-  setText( listProgress->lv_url, from.url() );
-  setText( listProgress->lv_filename, to.fileName() );
+   setText( ListProgress::TB_OPERATION, i18n("Copying") );
+   setText( ListProgress::TB_ADDRESS, from.url() );
+   setText( ListProgress::TB_LOCAL_FILENAME, to.fileName() );
 
   defaultProgress->slotCopying( 0, from, to );
 }
 
 
 void ProgressItem::setMoving( const KURL& from, const KURL& to ) {
-  setText( listProgress->lv_operation, i18n("Moving") );
-  setText( listProgress->lv_url, from.url() );
-  setText( listProgress->lv_filename, to.fileName() );
+   setText( ListProgress::TB_OPERATION, i18n("Moving") );
+   setText( ListProgress::TB_ADDRESS, from.url() );
+   setText( ListProgress::TB_LOCAL_FILENAME, to.fileName() );
 
   defaultProgress->slotMoving( 0, from, to );
 }
 
 
 void ProgressItem::setCreatingDir( const KURL& dir ) {
-  setText( listProgress->lv_operation, i18n("Creating") );
-  setText( listProgress->lv_url, dir.url() );
-  setText( listProgress->lv_filename, dir.fileName() );
+   setText( ListProgress::TB_OPERATION, i18n("Creating") );
+   setText( ListProgress::TB_ADDRESS, dir.url() );
+   setText( ListProgress::TB_LOCAL_FILENAME, dir.fileName() );
 
   defaultProgress->slotCreatingDir( 0, dir );
 }
 
 
 void ProgressItem::setDeleting( const KURL& url ) {
-  setText( listProgress->lv_operation, i18n("Deleting") );
-  setText( listProgress->lv_url, url.url() );
-  setText( listProgress->lv_filename, url.fileName() );
+   setText( ListProgress::TB_OPERATION, i18n("Deleting") );
+   setText( ListProgress::TB_ADDRESS, url.url() );
+   setText( ListProgress::TB_LOCAL_FILENAME, url.fileName() );
 
   defaultProgress->slotDeleting( 0, url );
 }
 
 void ProgressItem::setTransferring( const KURL& url ) {
-  setText( listProgress->lv_operation, i18n("Loading") );
-  setText( listProgress->lv_url, url.url() );
-  setText( listProgress->lv_filename, url.fileName() );
+   setText( ListProgress::TB_OPERATION, i18n("Loading") );
+   setText( ListProgress::TB_ADDRESS, url.url() );
+   setText( ListProgress::TB_LOCAL_FILENAME, url.fileName() );
 
   defaultProgress->slotTransferring( 0, url );
 }
 
+void ProgressItem::setText(ListProgress::ListProgressFields field, const QString& text)
+{
+  if (listProgress->m_lpcc[field].enabled)
+  {
+     QString t=text;
+     if ((field==ListProgress::TB_ADDRESS) && (listProgress->m_fixedColumnWidths))
+//     if (((field==ListProgress::TB_LOCAL_FILENAME) || (field==ListProgress::TB_ADDRESS)) && (listProgress->m_fixedColumnWidths))
+     {
+        m_fullLengthAddress=text;
+        listProgress->m_squeezer->resize(listProgress->columnWidth(listProgress->m_lpcc[field].index),50);
+        listProgress->m_squeezer->setText(t);
+        t=listProgress->m_squeezer->text();
+     }
+     QListViewItem::setText(listProgress->m_lpcc[field].index,t);
+  }
+}
+
 void ProgressItem::setStating( const KURL& url ) {
-  setText( listProgress->lv_operation, i18n("Examining") );
-  setText( listProgress->lv_url, url.url() );
-  setText( listProgress->lv_filename, url.fileName() );
+   setText( ListProgress::TB_OPERATION, i18n("Examining") );
+   setText( ListProgress::TB_ADDRESS, url.url() );
+   setText( ListProgress::TB_LOCAL_FILENAME, url.fileName() );
 
   defaultProgress->slotStating( 0, url );
 }
 
 void ProgressItem::setMounting( const QString& dev, const QString & point ) {
-  setText( listProgress->lv_operation, i18n("Mounting") );
-  setText( listProgress->lv_url, point ); // ?
-  setText( listProgress->lv_filename, dev ); // ?
+   setText( ListProgress::TB_OPERATION, i18n("Mounting") );
+   setText( ListProgress::TB_ADDRESS, point ); // ?
+   setText( ListProgress::TB_LOCAL_FILENAME, dev ); // ?
 
   defaultProgress->slotMounting( 0, dev, point );
 }
 
 void ProgressItem::setUnmounting( const QString & point ) {
-  setText( listProgress->lv_operation, i18n("Unmounting") );
-  setText( listProgress->lv_url, point ); // ?
-  setText( listProgress->lv_filename, "" ); // ?
+   setText( ListProgress::TB_OPERATION, i18n("Unmounting") );
+   setText( ListProgress::TB_ADDRESS, point ); // ?
+   setText( ListProgress::TB_LOCAL_FILENAME, "" ); // ?
 
   defaultProgress->slotUnmounting( 0, point );
 }
@@ -288,7 +408,7 @@ void ProgressItem::slotShowDefaultProgress() {
     if ( m_visible && m_defaultProgressVisible )
       defaultProgress->show();
     else
-      defaultProgress->hide();      
+      defaultProgress->hide();
   }
 }
 
@@ -323,7 +443,7 @@ void ProgressItem::updateVisibility()
     if ( m_visible && m_defaultProgressVisible )
     {
       m_showTimer.start(250, true); // Show delayed
-    }      
+    }
     else
     {
       m_showTimer.stop();
@@ -334,67 +454,139 @@ void ProgressItem::updateVisibility()
 
 
 //-----------------------------------------------------------------------------
-
 ListProgress::ListProgress (QWidget *parent, const char *name)
-  : KListView (parent, name) {
+: KListView (parent, name)
+{
 
   // enable selection of more than one item
   setMultiSelection( true );
 
   setAllColumnsShowFocus( true );
 
-  lv_operation = addColumn( i18n("Operation") );
-  lv_filename = addColumn( i18n("Local Filename") );
-  //lv_resume = addColumn( i18n("Res.") );
-  lv_count = addColumn( i18n("Count") );
-  lv_progress = addColumn( i18n("%") );
-  //lv_total = addColumn( i18n("Total") );
-  lv_size = addColumn( i18n("Size") );
-  lv_speed = addColumn( i18n("Speed") );
-  lv_remaining = addColumn( i18n("Rem. Time") );
-  lv_url = addColumn( i18n("URL") );
+  m_lpcc[TB_OPERATION].title=i18n("Operation");
+  m_lpcc[TB_LOCAL_FILENAME].title=i18n("Local Filename");
+  m_lpcc[TB_RESUME].title=i18n("Resume", "Res.");
+  m_lpcc[TB_COUNT].title=i18n("Count");
+  m_lpcc[TB_PROGRESS].title=i18n("%");
+  m_lpcc[TB_TOTAL].title=i18n("Size");
+  m_lpcc[TB_SPEED].title=i18n("Speed");
+  m_lpcc[TB_REMAINING_TIME].title=i18n("Rem. Time");
+  m_lpcc[TB_ADDRESS].title=i18n("URL");
+  readSettings();
 
-  readConfig();
+  applySettings();
+
+  //used for squeezing the text in local file name and url
+  m_squeezer=new KSqueezedTextLabel(this);
+  m_squeezer->hide();
+  connect(header(),SIGNAL(sizeChange(int,int,int)),this,SLOT(columnWidthChanged(int)));
 }
 
 
 ListProgress::~ListProgress() {
-  writeConfig();
 }
 
+void ListProgress::applySettings()
+{
+  int iEnabledCols=0;
 
-void ListProgress::readConfig() {
+  // Update listcolumns to show
+  for (int i=0; i<TB_MAX; i++)
+  {
+    if ( !m_lpcc[i].enabled )
+      continue;
+
+    iEnabledCols++;
+
+    // Add new or reuse existing column
+    if ( iEnabledCols > columns() )
+      m_lpcc[i].index=addColumn(m_lpcc[i].title, m_fixedColumnWidths?m_lpcc[i].width:-1);
+    else
+    {
+      m_lpcc[i].index = iEnabledCols - 1;
+      setColumnText(m_lpcc[i].index, m_lpcc[i].title);
+    }
+
+    setColumnWidth(m_lpcc[i].index, m_lpcc[i].width); //yes, this is required here, alexxx
+    if (m_fixedColumnWidths)
+        setColumnWidthMode(m_lpcc[i].index, Manual);
+  }
+
+  // Remove unused columns. However we must keep one column left
+  // Otherwise the listview will be emptied
+  while( iEnabledCols < columns() && columns() > 1 )
+     removeColumn( columns() - 1 );
+
+  if ( columns() == 0 )
+    addColumn( "" );
+
+  if ( !m_showHeader || iEnabledCols == 0 )
+    header()->hide();
+  else
+    header()->show();
+}
+
+void ListProgress::readSettings() {
   KConfig config("uiserverrc");
 
   // read listview geometry properties
   config.setGroup( "ProgressList" );
-  for ( int i = 0; i < NUM_COLS; i++ ) {
-    QString tmps;
-    tmps.sprintf( "Col%d", i );
-    setColumnWidth( i, config.readNumEntry( tmps, defaultColumnWidth[i] ) );
+  for ( int i = 0; i < TB_MAX; i++ ) {
+     QString tmps="Col"+QString::number(i);
+     m_lpcc[i].width=config.readNumEntry( tmps, 0);
+     if (m_lpcc[i].width==0) m_lpcc[i].width=defaultColumnWidth[i];
+
+     tmps="Enabled"+QString::number(i);
+     m_lpcc[i].enabled=config.readBoolEntry(tmps,true);
   }
+  m_showHeader=config.readBoolEntry("ShowListHeader",true);
+  m_fixedColumnWidths=config.readBoolEntry("FixedColumnWidths",false);
+
+  m_lpcc[TB_RESUME].enabled=false;
 }
 
+void ListProgress::columnWidthChanged(int column)
+{
+   //resqueeze if necessary
+   if ((m_lpcc[TB_ADDRESS].enabled) && (column==m_lpcc[TB_ADDRESS].index))
+   {
+      for (QListViewItem* lvi=firstChild(); lvi!=0; lvi=lvi->nextSibling())
+      {
+         ProgressItem *pi=(ProgressItem*)lvi;
+         pi->setText(TB_ADDRESS,pi->fullLengthAddress());
+      }
+   }
+   writeSettings();
+}
 
-void ListProgress::writeConfig() {
-  KConfig config("uiserverrc");
+void ListProgress::writeSettings() {
+   KConfig config("uiserverrc");
 
-  // write listview geometry properties
-  config.setGroup( "ProgressList" );
-  for ( int i = 0; i < NUM_COLS; i++ ) {
-    QString tmps;
-    tmps.sprintf( "Col%d", i );
-    config.writeEntry( tmps, columnWidth( i ) );
-  }
-
-  config.sync();
+   // write listview geometry properties
+   config.setGroup( "ProgressList" );
+   for ( int i = 0; i < TB_MAX; i++ ) {
+      if (!m_lpcc[i].enabled)
+         continue;
+      m_lpcc[i].width=columnWidth(m_lpcc[i].index);
+      QString tmps="Col"+QString::number(i);
+      config.writeEntry( tmps, m_lpcc[i].width);
+   }
+   config.writeEntry("ShowListHeader", m_showHeader);
+   config.writeEntry("FixedColumnWidths", m_fixedColumnWidths);
+   config.sync();
 }
 
 
 //------------------------------------------------------------
 
 
-UIServer::UIServer() : KMainWindow(0, ""), DCOPObject("UIServer")
+UIServer::UIServer()
+:KMainWindow(0, "")
+,DCOPObject("UIServer")
+,m_shuttingDown(false)
+,m_configDialog(0)
+,m_contextMenu(0)
+,m_systemTray(0)
 {
 
   readSettings();
@@ -402,15 +594,18 @@ UIServer::UIServer() : KMainWindow(0, ""), DCOPObject("UIServer")
   // setup toolbar
   toolBar()->insertButton("editdelete", TOOL_CANCEL,
                           SIGNAL(clicked()), this,
-                          SLOT(cancelCurrent()), FALSE, i18n("Cancel"));
+                          SLOT(slotCancelCurrent()), FALSE, i18n("Cancel"));
+  toolBar()->insertButton("configure", TOOL_CONFIGURE,
+                          SIGNAL(clicked()), this,
+                          SLOT(slotConfigure()), true, i18n("Settings..."));
 
   toolBar()->setBarPos( KToolBar::Left );
 
   // setup statusbar
-  statusBar()->insertItem( i18n(" Files: %1 ").arg( 555 ), ID_TOTAL_FILES);
-  statusBar()->insertItem( i18n(" Size: %1 kB ").arg( "134.56" ), ID_TOTAL_SIZE);
+  statusBar()->insertItem( i18n(" Files: %1 ").arg( 0 ), ID_TOTAL_FILES);
+  statusBar()->insertItem( i18n(" Size: %1 kB ").arg( "0" ), ID_TOTAL_SIZE);
   statusBar()->insertItem( i18n(" Time: 00:00:00 "), ID_TOTAL_TIME);
-  statusBar()->insertItem( i18n(" %1 kB/s ").arg("123.34"), ID_TOTAL_SPEED);
+  statusBar()->insertItem( i18n(" %1 kB/s ").arg("0"), ID_TOTAL_SPEED);
 
   // setup listview
   listProgress = new ListProgress( this, "progresslist" );
@@ -421,6 +616,9 @@ UIServer::UIServer() : KMainWindow(0, ""), DCOPObject("UIServer")
            SLOT( slotSelection() ) );
   connect( listProgress, SIGNAL( executed( QListViewItem* ) ),
            SLOT( slotToggleDefaultProgress( QListViewItem* ) ) );
+  connect( listProgress, SIGNAL( contextMenu( KListView*, QListViewItem *, const QPoint &)),
+           SLOT(slotShowContextMenu(KListView*, QListViewItem *, const QPoint&)));
+
 
   // setup animation timer
   updateTimer = new QTimer( this );
@@ -429,17 +627,116 @@ UIServer::UIServer() : KMainWindow(0, ""), DCOPObject("UIServer")
   m_bUpdateNewJob=false;
 
   setCaption(i18n("Progress Dialog"));
-  setMinimumSize( 350, 150 );
-  resize( 460, 150 );
+  setMinimumSize( 150, 50 );
+  resize( m_initWidth, m_initHeight);
 
+  applySettings();
+
+/*  if ((m_bShowList) && (m_keepListOpen))
+  {
+     cerr<<"show() !"<<endl;
+     show();
+  }
+  else*/
   hide();
 }
 
-
 UIServer::~UIServer() {
   updateTimer->stop();
 }
 
+void UIServer::applySettings()
+{
+  if ((m_showSystemTray) && (m_systemTray==0))
+  {
+     m_systemTray=new UIServerSystemTray(this);
+     m_systemTray->show();
+  }
+  else if ((m_showSystemTray==false) && (m_systemTray!=0))
+  {
+     delete m_systemTray;
+     m_systemTray=0;
+  }
+
+  if (m_showStatusBar==false)
+     statusBar()->hide();
+  else
+     statusBar()->show();
+  if (m_showToolBar==false)
+     toolBar()->hide();
+  else
+     toolBar()->show();
+}
+
+void UIServer::slotShowContextMenu(KListView*, QListViewItem* /*item*/, const QPoint& pos)
+{
+   if (m_contextMenu==0)
+   {
+      m_contextMenu=new QPopupMenu(this);
+      m_idCancelItem = m_contextMenu->insertItem(i18n("Cancel Job"), this, SLOT(slotCancelCurrent()));
+//      m_contextMenu->insertItem(i18n("Toggle Progress"), this, SLOT(slotToggleDefaultProgress()));
+      m_contextMenu->insertSeparator();
+      m_contextMenu->insertItem(i18n("Settings..."), this, SLOT(slotConfigure()));
+   }
+   bool enabled = false;
+   QListViewItemIterator it( listProgress );
+   for ( ; it.current(); ++it ) {
+     if ( it.current()->isSelected() ) {
+       enabled = true;
+       break;
+     }
+   }
+   m_contextMenu->setItemEnabled( m_idCancelItem, enabled);
+
+   m_contextMenu->popup(pos);
+}
+
+void UIServer::slotRemoveSystemTrayIcon()
+{
+   m_showSystemTray=false;
+   applySettings();
+   writeSettings();
+}
+
+void UIServer::slotConfigure()
+{
+   if (m_configDialog==0)
+   {
+      m_configDialog=new ProgressConfigDialog(0);
+//      connect(m_configDialog,SIGNAL(cancelClicked()), this, SLOT(slotCancelConfig()));
+      connect(m_configDialog,SIGNAL(okClicked()), this, SLOT(slotApplyConfig()));
+      connect(m_configDialog,SIGNAL(applyClicked()), this, SLOT(slotApplyConfig()));
+   }
+   m_configDialog->m_showSystemTrayCb->setChecked(m_showSystemTray);
+   m_configDialog->m_keepOpenCb->setChecked(m_keepListOpen);
+   m_configDialog->m_toolBarCb->setChecked(m_showToolBar);
+   m_configDialog->m_statusBarCb->setChecked(m_showStatusBar);
+   m_configDialog->m_headerCb->setChecked(listProgress->m_showHeader);
+   m_configDialog->m_fixedWidthCb->setChecked(listProgress->m_fixedColumnWidths);
+   for (int i=0; i<ListProgress::TB_MAX; i++)
+   {
+      m_configDialog->setChecked(i, listProgress->m_lpcc[i].enabled);
+   }
+   m_configDialog->show();
+}
+
+void UIServer::slotApplyConfig()
+{
+   m_showSystemTray=m_configDialog->m_showSystemTrayCb->isChecked();
+   m_keepListOpen=m_configDialog->m_keepOpenCb->isChecked();
+   m_showToolBar=m_configDialog->m_toolBarCb->isChecked();
+   m_showStatusBar=m_configDialog->m_statusBarCb->isChecked();
+   listProgress->m_showHeader=m_configDialog->m_headerCb->isChecked();
+   listProgress->m_fixedColumnWidths=m_configDialog->m_fixedWidthCb->isChecked();
+   for (int i=0; i<ListProgress::TB_MAX; i++)
+      listProgress->m_lpcc[i].enabled=m_configDialog->isChecked(i);
+
+
+   applySettings();
+   listProgress->applySettings();
+   writeSettings();
+   listProgress->writeSettings();
+}
 
 int UIServer::newJob( QCString observerAppId, bool showProgress )
 {
@@ -517,7 +814,10 @@ void UIServer::jobFinished( int id )
 
   // remove item from the list and delete the corresponding defaultprogress
   if ( item ) {
-    delete item;
+    if ( item->keepOpen() )
+      item->finished();
+    else
+      delete item;
   }
 }
 
@@ -719,14 +1019,6 @@ void UIServer::killJob( QCString observerAppId, int progressId )
     observer.killJob( progressId );
 }
 
-
-void UIServer::closeEvent( QCloseEvent * ){
-#ifndef Q_WS_QWS //FIXME(E): Implement for QT Embedded
-   KWin::iconifyWindow(winId());
-#endif
-}
-
-
 void UIServer::slotJobCanceled( ProgressItem *item ) {
   kdDebug(7024) << "UIServer::slotJobCanceled appid=" << item->appId() << " jobid=" << item->jobId() << endl;
   // kill the corresponding job
@@ -738,22 +1030,28 @@ void UIServer::slotJobCanceled( ProgressItem *item ) {
 }
 
 
+void UIServer::slotQuit()
+{
+  m_shuttingDown = true;
+  kapp->quit();
+}
+
 void UIServer::slotUpdate() {
   // don't do anything if we don't have any inserted progress item
   // or if they're all hidden
-  QListViewItemIterator lvit( listProgress );
-  bool visible = false;
-  for ( ; lvit.current(); ++lvit )
-    if ( ((ProgressItem*)lvit.current())->isVisible() ) {
-      visible = true;
-      break;
-    }
+   QListViewItemIterator lvit( listProgress );
+   bool visible = false;
+   for ( ; lvit.current(); ++lvit )
+      if ( ((ProgressItem*)lvit.current())->isVisible() ) {
+         visible = true;
+         break;
+      }
 
-  if ( !visible || !m_bShowList ) {
-    hide();
-    updateTimer->stop();
-    return;
-  }
+   if ( !visible || !m_bShowList ) {
+      if (!m_keepListOpen) hide();
+      updateTimer->stop();
+      return;
+   }
 
   // Calling show() is conditional, so that users can close the window
   // and it only pops up back when a new job is started
@@ -768,7 +1066,7 @@ void UIServer::slotUpdate() {
   }
 
   int iTotalFiles = 0;
-  int iTotalSize = 0;
+  KIO::filesize_t iTotalSize = 0;
   int iTotalSpeed = 0;
   QTime totalRemTime;
 
@@ -884,6 +1182,8 @@ void UIServer::showSSLInfoDialog(const QString &url, const KIO::MetaData &meta)
       if (ncl.count() > 0)
          x->chain().setChain(ncl);
 
+      kdDebug(7024) << "ssl_cert_errors=" << meta["ssl_cert_errors"] << endl;
+      kid->setCertState(meta["ssl_cert_errors"]);
       kid->setup( x,
                   meta["ssl_peer_ip"],
                   url, // the URL
@@ -910,7 +1210,7 @@ KSSLCertDlgRet UIServer::showSSLCertDialog(const QString& host, const QStringLis
    rc.ok = false;
    if (!certList.isEmpty()) {
       KSSLCertDlg *kcd = new KSSLCertDlg(0L, 0L, true);
-      kcd->setup(certList);
+      kcd->setupDialog(certList);
       kcd->setHost(host);
       kdDebug(7024) << "Showing SSL certificate dialog" << endl;
       kcd->exec();
@@ -991,12 +1291,29 @@ int UIServer::open_SkipDlg( int id,
 void UIServer::readSettings() {
   KConfig config("uiserverrc");
   config.setGroup( "UIServer" );
-
+  m_showStatusBar=config.readBoolEntry("ShowStatusBar",false);
+  m_showToolBar=config.readBoolEntry("ShowToolBar",true);
+  m_keepListOpen=config.readBoolEntry("KeepListOpen",false);
+  m_initWidth=config.readNumEntry("InitialWidth",460);
+  m_initHeight=config.readNumEntry("InitialHeight",150);
   m_bShowList = config.readBoolEntry( "ShowList", false );
+  m_showSystemTray=config.readBoolEntry("ShowSystemTray", false);
+}
+
+void UIServer::writeSettings() {
+  KConfig config("uiserverrc");
+  config.setGroup( "UIServer" );
+  config.writeEntry("InitialWidth",width());
+  config.writeEntry("InitialHeight",height());
+  config.writeEntry("ShowStatusBar", m_showStatusBar);
+  config.writeEntry("ShowToolBar", m_showToolBar);
+  config.writeEntry("KeepListOpen", m_keepListOpen);
+  config.writeEntry("ShowList", m_bShowList);
+  config.writeEntry("ShowSystemTray", m_showSystemTray);
 }
 
 
-void UIServer::cancelCurrent() {
+void UIServer::slotCancelCurrent() {
   QListViewItemIterator it( listProgress );
   ProgressItem *item;
 
@@ -1011,9 +1328,29 @@ void UIServer::cancelCurrent() {
   }
 }
 
+void UIServer::resizeEvent(QResizeEvent* e)
+{
+   KMainWindow::resizeEvent(e);
+   writeSettings();
+}
+
+bool UIServer::queryClose()
+{
+  if (( !m_shuttingDown ) && !kapp->sessionSaving()) {
+    hide();
+    return false;
+  }
+  return true;
+}
+
+UIServer* UIServer::createInstance()
+{
+    return new UIServer;
+}
+
 //------------------------------------------------------------
 
-int main(int argc, char **argv)
+extern "C" int kdemain(int argc, char **argv)
 {
     KLocale::setMainCatalogue("kdelibs");
     //  GS 5/2001 - I changed the name to "KDE" to make it look better
@@ -1041,9 +1378,9 @@ int main(int argc, char **argv)
     app.disableSessionManagement();
     app.dcopClient()->setDaemonMode( true );
 
-    uiserver = new UIServer;
+    uiserver = UIServer::createInstance();
 
-    app.setMainWidget( uiserver );
+//    app.setMainWidget( uiserver );
 
     return app.exec();
 }
