@@ -1,3 +1,4 @@
+#include <kmimetype.h>
 #include <kmimemagic.h>
 #include <kinstance.h>
 #include <stdio.h>
@@ -21,5 +22,14 @@ int main( int argc, char** argv )
   else
       printf( "Invalid result\n");
 
+  KMimeType::Format f = KMimeType::findFormatByFileContent( file );
+  if (f.text)
+    printf("Text\n");
+  else
+    printf("Binary\n");
+
+  if (f.compression == KMimeType::Format::GZipCompression)
+    printf("GZipped\n");
+
   return 0;
 }
