@@ -38,7 +38,7 @@ int main( int argc, char** argv )
 
     if ( !tar.open( IO_ReadOnly ) )
     {
-      printf("Could not open %s for reading\n", argv[1] );
+      printf("Could not open %s for reading\n", argv[2] );
       return 1;
     }
 
@@ -119,7 +119,9 @@ int main( int argc, char** argv )
       return 1;
     }
     // Generate long filenames of each possible length bigger than 98...
-    for (int i = 98; i < 500 ; i++ )
+    // Also exceed 512 byte block size limit to see how well the ././@LongLink
+    // implementation fares
+    for (int i = 98; i < 514 ; i++ )
     {
       QString str, num;
       str.fill( 'a', i-10 );
@@ -127,8 +129,8 @@ int main( int argc, char** argv )
       num = num.rightJustify( 10, '0' );
       tar.writeFile( str+num, "testu", "testg", 3, "hum" );
     }
-    // Result of this test : it fails at 482 (instead of 154 previously).
-    // Ok, I think we can do with that :)
+    // Result of this test : works perfectly now (failed at 482 formerly and
+    // before that at 154).
     tar.close();
     printf("Now run 'tar tvzf %s'\n", argv[2]);
     return 0;
@@ -145,7 +147,7 @@ int main( int argc, char** argv )
     {
         QIODevice *dev = static_cast<const KTarFile *>(entry)->device();
         QByteArray contents = dev->readAll();
-        printf("contents='%s'\n", QCString(contents).data());
+        printf("contents='%s'\n", QCString(contents, contents.size()+1 ).data());
     } else
         printf("entry=%p - not found if 0, otherwise not a file\n", (void*)entry);
     return 0;
