@@ -1,4 +1,4 @@
-
+#include <config.h>
 #include <iostream>
 #include <stdlib.h>
 
@@ -90,22 +90,27 @@ int main(int argc, char **argv) {
     // URI that should require no filtering
     filter( "http://www.kde.org", "http://www.kde.org", KURIFilterData::NET_PROTOCOL );
     filter( "http://www.kde.org/developer//index.html", "http://www.kde.org/developer//index.html", KURIFilterData::NET_PROTOCOL );
+        // URL with reference
     filter( "http://www.kde.org/index.html#q8", "http://www.kde.org/index.html#q8", KURIFilterData::NET_PROTOCOL );
+        // local file with reference
     filter( "file:/etc/passwd#q8", "file:/etc/passwd#q8", KURIFilterData::LOCAL_FILE );
+        // hostnames are lowercased by KURL
     filter( "http://www.myDomain.commyPort/ViewObjectRes//Default:name=hello",
-            "http://www.myDomain.commyPort/ViewObjectRes//Default:name=hello", KURIFilterData::NET_PROTOCOL);
+            "http://www.mydomain.commyport/ViewObjectRes//Default:name=hello", KURIFilterData::NET_PROTOCOL);
     filter( "ftp://ftp.kde.org", "ftp://ftp.kde.org", KURIFilterData::NET_PROTOCOL );
     filter( "ftp://username@ftp.kde.org:500", "ftp://username@ftp.kde.org:500", KURIFilterData::NET_PROTOCOL );
 
     // ShortURI tests
     filter( "linuxtoday.com", "http://linuxtoday.com", KURIFilterData::NET_PROTOCOL );
-    filter( "LINUXTODAY.COM", "http://LINUXTODAY.COM", KURIFilterData::NET_PROTOCOL );
+    filter( "LINUXTODAY.COM", "http://linuxtoday.com", KURIFilterData::NET_PROTOCOL );
     filter( "kde.org", "http://kde.org", KURIFilterData::NET_PROTOCOL );
     filter( "mosfet.org", "http://mosfet.org", KURIFilterData::NET_PROTOCOL );
     filter( "/", "/", KURIFilterData::LOCAL_DIR );
     filter( "/", "/", KURIFilterData::LOCAL_DIR, "kshorturifilter" );
     filter( "~/.kderc", QDir::homeDirPath().local8Bit()+"/.kderc", KURIFilterData::LOCAL_FILE, "kshorturifilter" );
     filter( "~", QDir::homeDirPath().local8Bit(), KURIFilterData::LOCAL_DIR, "kshorturifilter", "/tmp" );
+    filter( "~foobar", 0, KURIFilterData::ERROR, "kshorturifilter" );
+    filter( "user@host.domain", "mailto:user@host.domain", KURIFilterData::NET_PROTOCOL ); // new in KDE-3.2
 
     // SMB share test with a specific filter chosen
     // #### Those fail - I see no specific code for them in kshorturifilter !?
@@ -166,6 +171,8 @@ int main(int argc, char **argv) {
 
     // Search Engine tests
     filter( "gg:foo bar", "http://www.google.com/search?q=foo%20bar&ie=UTF-8&oe=UTF-8", KURIFilterData::NET_PROTOCOL );
+    filter( "bug:55798", "http://bugs.kde.org/show_bug.cgi?id=55798", KURIFilterData::NET_PROTOCOL );
+
     // BR ...
     filter( "gg:C++", "http://www.google.com/search?q=C%2B%2B&ie=UTF-8&oe=UTF-8", KURIFilterData::NET_PROTOCOL );
     filter( "ya:foo bar was here", 0, -1 ); // this triggers default search, i.e. google
