@@ -1,7 +1,30 @@
-// $Id: file.cc,v 1.124.2.6 2003/09/01 07:42:28 coolo Exp $
+/*
+   Copyright (C) 2000-2002 Stephan Kulow <coolo@kde.org>
+   Copyright (C) 2000-2002 David Faure <faure@kde.org>
+   Copyright (C) 2000-2002 Waldo Bastian <bastian@kde.org>
+
+   This library is free software; you can redistribute it and/or
+   modify it under the terms of the GNU Library General Public
+   License (LGPL) as published by the Free Software Foundation;
+   either version 2 of the License, or (at your option) any later
+   version.
+
+   This library is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+   Library General Public License for more details.
+
+   You should have received a copy of the GNU Library General Public License
+   along with this library; see the file COPYING.LIB.  If not, write to
+   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+   Boston, MA 02111-1307, USA.
+*/
+
+// $Id: file.cc,v 1.143.2.3 2004/02/15 20:25:17 faure Exp $
 
 #include <config.h>
 
+#include <qglobal.h> //for Q_OS_XXX
 #include <sys/types.h>
 #include <sys/wait.h>
 #include <sys/stat.h>
@@ -9,6 +32,15 @@
 #include <sys/time.h>
 #endif
 
+//sendfile has different semantics in different platforms
+#if defined HAVE_SENDFILE && defined Q_OS_LINUX
+#define USE_SENDFILE 1
+#endif
+
+#ifdef USE_SENDFILE
+#include <sys/sendfile.h>
+#endif
+
 #include <assert.h>
 #include <dirent.h>
 #include <errno.h>
@@ -42,12 +74,11 @@
 #include <kprocess.h>
 
 #ifdef HAVE_VOLMGT
-	#include <volmgt.h>
-	#include <sys/mnttab.h>
-#else
-	#include <kstandarddirs.h>
+#include <volmgt.h>
+#include <sys/mnttab.h>
 #endif
 
+#include <kstandarddirs.h>
 #include <kio/ioslave_defaults.h>
 #include <klargefile.h>
 #include <kglobal.h>
@@ -57,7 +88,7 @@ using namespace KIO;
 
 #define MAX_IPC_SIZE (1024*32)
 
-QString testLogFile( const char *_filename );
+static QString testLogFile( const char *_filename );
 
 extern "C" { int kdemain(int argc, char **argv); }
 
@@ -85,8 +116,8 @@ int kdemain( int argc, char **argv )
 
 FileProtocol::FileProtocol( const QCString &pool, const QCString &app ) : SlaveBase( "file", pool, app )
 {
-    usercache.setAutoDelete( TRUE );
-    groupcache.setAutoDelete( TRUE );
+    usercache.setAutoDelete( true );
+    groupcache.setAutoDelete( true );
 }
 
 void FileProtocol::chmod( const KURL& url, int permissions )
@@ -101,32 +132,35 @@ void FileProtocol::chmod( const KURL& url, int permissions )
 void FileProtocol::mkdir( const KURL& url, int permissions )
 {
     QCString _path( QFile::encodeName(url.path()));
+    
+    kdDebug(7101) << "mkdir(): " << _path << ", permission = " << permissions << endl;
+        
     KDE_struct_stat buff;
     if ( KDE_stat( _path.data(), &buff ) == -1 ) {
-	if ( ::mkdir( _path.data(), 0777 /*umask will be applied*/ ) != 0 ) {
-	    if ( errno == EACCES ) {
-		error( KIO::ERR_ACCESS_DENIED, url.path() );
-		return;
-	    } else if ( errno == ENOSPC ) {
-		error( KIO::ERR_DISK_FULL, url.path() );
-		return;
-	    } else {
-		error( KIO::ERR_COULD_NOT_MKDIR, url.path() );
-		return;
-	    }
-	} else {
-	    if ( permissions != -1 )
-	        chmod( url, permissions );
-	    else
-	        finished();
-	    return;
-	}
+        if ( ::mkdir( _path.data(), 0777 /*umask will be applied*/ ) != 0 ) {
+            if ( errno == EACCES ) {
+          error( KIO::ERR_ACCESS_DENIED, url.path() );
+          return;
+            } else if ( errno == ENOSPC ) {
+          error( KIO::ERR_DISK_FULL, url.path() );
+          return;
+            } else {
+          error( KIO::ERR_COULD_NOT_MKDIR, url.path() );
+          return;
+            }
+        } else {
+            if ( permissions != -1 )
+                chmod( url, permissions );
+            else
+                finished();
+            return;
+        }
     }
 
     if ( S_ISDIR( buff.st_mode ) ) {
-	kdDebug() << "ERR_DIR_ALREADY_EXIST" << endl;
-	error( KIO::ERR_DIR_ALREADY_EXIST, url.path() );
-	return;
+        kdDebug(7101) << "ERR_DIR_ALREADY_EXIST" << endl;
+        error( KIO::ERR_DIR_ALREADY_EXIST, url.path() );
+        return;
     }
     error( KIO::ERR_FILE_ALREADY_EXIST, url.path() );
     return;
@@ -141,32 +175,53 @@ void FileProtocol::get( const KURL& url )
            error( KIO::ERR_ACCESS_DENIED, url.path() );
         else
            error( KIO::ERR_DOES_NOT_EXIST, url.path() );
-	return;
+        return;
     }
 
     if ( S_ISDIR( buff.st_mode ) ) {
-	error( KIO::ERR_IS_DIRECTORY, url.path() );
-	return;
+        error( KIO::ERR_IS_DIRECTORY, url.path() );
+        return;
     }
     if ( !S_ISREG( buff.st_mode ) ) {
-	error( KIO::ERR_CANNOT_OPEN_FOR_READING, url.path() );
-	return;
+        error( KIO::ERR_CANNOT_OPEN_FOR_READING, url.path() );
+        return;
     }
 
     int fd = KDE_open( _path.data(), O_RDONLY);
     if ( fd < 0 ) {
-	error( KIO::ERR_CANNOT_OPEN_FOR_READING, url.path() );
-	return;
+        error( KIO::ERR_CANNOT_OPEN_FOR_READING, url.path() );
+        return;
     }
 
     // Determine the mimetype of the file to be retrieved, and emit it.
     // This is mandatory in all slaves (for KRun/BrowserRun to work).
-    KMimeType::Ptr mt = KMimeType::findByURL( url.path(), buff.st_mode, true /* local URL */ );
+    KMimeType::Ptr mt = KMimeType::findByURL( url, buff.st_mode, true /* local URL */ );
     emit mimeType( mt->name() );
 
-    totalSize( buff.st_size );
     KIO::filesize_t processed_size = 0;
 
+    QString resumeOffset = metaData("resume");
+    if ( !resumeOffset.isEmpty() )
+    {
+        bool ok;
+#if QT_VERSION >= 0x030200
+        KIO::fileoffset_t offset = resumeOffset.toLongLong(&ok);
+#else
+        KIO::fileoffset_t offset = resumeOffset.toULong(&ok);
+#endif
+        if (ok && (offset > 0) && (offset < buff.st_size))
+        {
+            if (KDE_lseek(fd, offset, SEEK_SET) == offset)
+            {
+                canResume ();
+                processed_size = offset;
+                kdDebug( 7101 ) << "Resume offset: " << KIO::number(offset) << endl;
+            }
+        }
+    }
+
+    totalSize( buff.st_size );
+
     char buffer[ MAX_IPC_SIZE ];
     QByteArray array;
 
@@ -190,6 +245,8 @@ void FileProtocol::get( const KURL& url )
 
        processed_size += n;
        processedSize( processed_size );
+
+       //kdDebug( 7101 ) << "Processed: " << KIO::number (processed_size) << endl;
     }
 
     data( QByteArray() );
@@ -222,141 +279,162 @@ void FileProtocol::put( const KURL& url, int _mode, bool _overwrite, bool _resum
 {
     QString dest_orig = url.path();
     QCString _dest_orig( QFile::encodeName(dest_orig));
-    kdDebug( 7101 ) << "Put " << dest_orig << endl;
+
+    kdDebug(7101) << "put(): " << dest_orig << ", mode=" << _mode << endl;
+
     QString dest_part( dest_orig );
     dest_part += QString::fromLatin1(".part");
     QCString _dest_part( QFile::encodeName(dest_part));
 
+    KDE_struct_stat buff_orig;
+    bool orig_exists = (KDE_lstat( _dest_orig.data(), &buff_orig ) != -1);
+    bool part_exists = false;
     bool bMarkPartial = config()->readBoolEntry("MarkPartial", true);
 
-    KDE_struct_stat buff_orig;
-    bool orig_exists = ( KDE_stat( _dest_orig.data(), &buff_orig ) != -1 );
-    if ( orig_exists &&  !_overwrite && !_resume)
-    {
-        if (S_ISDIR(buff_orig.st_mode))
-           error( KIO::ERR_DIR_ALREADY_EXIST, dest_orig );
-        else
-           error( KIO::ERR_FILE_ALREADY_EXIST, dest_orig );
-        return;
-    }
-
-    QString dest;
     if (bMarkPartial)
     {
-        kdDebug(7101) << "Appending .part extension to " << dest_orig << endl;
-        dest = dest_part;
-
         KDE_struct_stat buff_part;
-        bool part_exists = ( KDE_stat( _dest_part.data(), &buff_part ) != -1 );
-        if ( part_exists && !_resume && buff_part.st_size > 0 )
+        part_exists = (KDE_stat( _dest_part.data(), &buff_part ) != -1);
+
+        if (part_exists && !_resume && buff_part.st_size > 0 && S_ISREG(buff_part.st_mode))
         {
-            kdDebug() << "FileProtocol::put : calling canResume with " << KIO::number(buff_part.st_size) << endl;
-             // Maybe we can use this partial file for resuming
-             // Tell about the size we have, and the app will tell us
-             // if it's ok to resume or not.
-             _resume = canResume( buff_part.st_size );
+            kdDebug(7101) << "FileProtocol::put : calling canResume with "
+                          << KIO::number(buff_part.st_size) << endl;
 
-             kdDebug() << "FileProtocol::put got answer " << _resume << endl;
+            // Maybe we can use this partial file for resuming
+            // Tell about the size we have, and the app will tell us
+            // if it's ok to resume or not.
+            _resume = canResume( buff_part.st_size );
 
-             if (!_resume)
-             {
-                 kdDebug(7101) << "Deleting partial file " << dest_part << endl;
-                 if ( ! remove( _dest_part.data() ) ) {
-                     part_exists = false;
-                 } else {
-                     error( KIO::ERR_CANNOT_DELETE_PARTIAL, dest_part );
-                     return;
-                 }
-             }
+            kdDebug(7101) << "FileProtocol::put got answer " << _resume << endl;
         }
     }
-    else
-    {
-       dest = dest_orig;
-       if ( orig_exists && !_resume )
-        {
-             kdDebug(7101) << "Deleting destination file " << dest_part << endl;
-             remove( _dest_orig.data() );
-             // Catch errors when we try to open the file.
-        }
-    }
-    QCString _dest( QFile::encodeName(dest));
-
-    int fd;
-
-    if ( _resume ) {
-        fd = KDE_open( _dest.data(), O_RDWR );  // append if resuming
-        KDE_lseek(fd, 0, SEEK_END); // Seek to end
-    } else {
-        // WABA: Make sure that we keep writing permissions ourselves,
-        // otherwise we can be in for a surprise on NFS.
-        mode_t initialMode;
-        if (_mode != -1)
-           initialMode = _mode | S_IWUSR | S_IRUSR;
-        else
-           initialMode = 0666;
-
-        fd = KDE_open(_dest.data(), O_CREAT | O_TRUNC | O_WRONLY, initialMode);
-    }
-
-    if ( fd < 0 ) {
-	kdDebug(7101) << "####################### COULD NOT WRITE " << dest << _mode << endl;
-	kdDebug(7101) << "errno==" << errno << "(" << strerror(errno) << ")" << endl;
-        if ( errno == EACCES ) {
-            error( KIO::ERR_WRITE_ACCESS_DENIED, dest );
-        } else {
-            error( KIO::ERR_CANNOT_OPEN_FOR_WRITING, dest );
-        }
-        return;
-    }
 
     int result;
+    QString dest;
+    QCString _dest;
+
+    int fd = -1;
+
     // Loop until we got 0 (end of data)
     do
     {
-      QByteArray buffer;
-      dataReq(); // Request for data
-      result = readData( buffer );
-      if (result > 0)
-      {
-         if (write_all( fd, buffer.data(), buffer.size()))
-         {
-            if ( errno == ENOSPC ) // disk full
+        QByteArray buffer;
+        dataReq(); // Request for data
+        result = readData( buffer );
+
+        if (result >= 0)
+        {
+            if (dest.isEmpty())
             {
-              error( KIO::ERR_DISK_FULL, dest_orig);
-              result = -2; // means: remove dest file
+                if ( orig_exists &&  !_overwrite && !_resume)
+                {
+                    if (S_ISDIR(buff_orig.st_mode))
+                      error( KIO::ERR_DIR_ALREADY_EXIST, dest_orig );
+                    else
+                      error( KIO::ERR_FILE_ALREADY_EXIST, dest_orig );
+                    return;
+                }
+
+                if (bMarkPartial)
+                {
+                    kdDebug(7101) << "Appending .part extension to " << dest_orig << endl;
+                    dest = dest_part;
+                    if ( part_exists && !_resume )
+                    {
+                        kdDebug(7101) << "Deleting partial file " << dest_part << endl;
+                        remove( _dest_part.data() );
+                        // Catch errors when we try to open the file.
+                    }
+                }
+                else
+                {
+                    dest = dest_orig;
+                    if ( orig_exists && !_resume )
+                    {
+                        kdDebug(7101) << "Deleting destination file " << dest_orig << endl;
+                        remove( _dest_orig.data() );
+                        // Catch errors when we try to open the file.
+                    }
+                }
+
+                _dest = QFile::encodeName(dest);
+
+                if ( _resume )
+                {
+                    fd = KDE_open( _dest.data(), O_RDWR );  // append if resuming
+                    KDE_lseek(fd, 0, SEEK_END); // Seek to end
+                }
+                else
+                {
+                    // WABA: Make sure that we keep writing permissions ourselves,
+                    // otherwise we can be in for a surprise on NFS.
+                    mode_t initialMode;
+                    if (_mode != -1)
+                        initialMode = _mode | S_IWUSR | S_IRUSR;
+                    else
+                        initialMode = 0666;
+
+                    fd = KDE_open(_dest.data(), O_CREAT | O_TRUNC | O_WRONLY, initialMode);
+                }
+
+                if ( fd < 0 )
+                {
+                    kdDebug(7101) << "####################### COULD NOT WRITE " << dest << _mode << endl;
+                    kdDebug(7101) << "errno==" << errno << "(" << strerror(errno) << ")" << endl;
+                    if ( errno == EACCES )
+                        error( KIO::ERR_WRITE_ACCESS_DENIED, dest );
+                    else
+                        error( KIO::ERR_CANNOT_OPEN_FOR_WRITING, dest );
+                    return;
+                }
             }
-            else
+
+            if (write_all( fd, buffer.data(), buffer.size()))
             {
-              kdWarning(7101) << "Couldn't write. Error:" << strerror(errno) << endl;
-              error( KIO::ERR_COULD_NOT_WRITE, dest_orig);
-              result = -1;
+                if ( errno == ENOSPC ) // disk full
+                {
+                  error( KIO::ERR_DISK_FULL, dest_orig);
+                  result = -2; // means: remove dest file
+                }
+                else
+                {
+                  kdWarning(7101) << "Couldn't write. Error:" << strerror(errno) << endl;
+                  error( KIO::ERR_COULD_NOT_WRITE, dest_orig);
+                  result = -1;
+                }
             }
-         }
-      }
+        }
     }
     while ( result > 0 );
 
-
-    if (result != 0)
+    // An error occurred deal with it.
+    if (result < 0)
     {
-        close(fd);
-	kdDebug(7101) << "Error during 'put'. Aborting." << endl;
-        if (result == -2)
+        kdDebug(7101) << "Error during 'put'. Aborting." << endl;
+
+        if (fd != -1)
         {
-	   remove(_dest.data());
-        } else if (bMarkPartial)
-        {
-           KDE_struct_stat buff;
-           if (( KDE_stat( _dest.data(), &buff ) == -1 ) ||
-               ( buff.st_size < config()->readNumEntry("MinimumKeepSize", DEFAULT_MINIMUM_KEEP_SIZE) ))
-           {
-	       remove(_dest.data());
-           }
+          close(fd);
+
+          KDE_struct_stat buff;
+          if (bMarkPartial && KDE_stat( _dest.data(), &buff ) == 0)
+          {
+            int size = config()->readNumEntry("MinimumKeepSize", DEFAULT_MINIMUM_KEEP_SIZE);
+            if (buff.st_size <  size)
+              remove(_dest.data());
+          }
         }
+
         ::exit(255);
     }
 
+    if ( fd == -1 ) // we got nothing to write out, so we never opened the file
+    {
+        finished();
+        return;
+    }
+
     if ( close(fd) )
     {
         kdWarning(7101) << "Error when closing file descriptor:" << strerror(errno) << endl;
@@ -367,21 +445,29 @@ void FileProtocol::put( const KURL& url, int _mode, bool _overwrite, bool _resum
     // after full download rename the file back to original name
     if ( bMarkPartial )
     {
-       if ( ::rename( _dest.data(), _dest_orig.data() ) )
-       {
-           kdWarning(7101) << " Couldn't rename " << _dest << " to " << _dest_orig << endl;
-           error( KIO::ERR_CANNOT_RENAME_PARTIAL, dest_orig );
-           return;
-       }
+        // If the original URL is a symlink and we were asked to overwrite it,
+        // remove the symlink first. This ensures that we do not overwrite the
+        // current source if the symlink points to it.
+        if( _overwrite && S_ISLNK( buff_orig.st_mode ) )
+          remove( _dest_orig.data() );
+        
+        if ( ::rename( _dest.data(), _dest_orig.data() ) )
+        {
+            kdWarning(7101) << " Couldn't rename " << _dest << " to " << _dest_orig << endl;
+            error( KIO::ERR_CANNOT_RENAME_PARTIAL, dest_orig );
+            return;
+        }
     }
 
-    // set final permissions, if the file was just created
-    if ( _mode != -1 && !orig_exists )
+    // set final permissions
+    if ( _mode != -1 && !_resume )
     {
-       if (::chmod(_dest_orig.data(), _mode) != 0)
-       {
-           warning( i18n( "Could not change permissions for\n%1" ).arg( dest_orig ) );
-       }
+        if (::chmod(_dest_orig.data(), _mode) != 0)
+        {
+            // couldn't chmod. Eat the error if the filesystem apparently doesn't support it.
+            if ( KIO::testFileSystemFlag( _dest_orig, KIO::SupportsChmod ) )
+                 warning( i18n( "Could not change permissions for\n%1" ).arg( dest_orig ) );
+        }
     }
 
     // We have done our job => finish
@@ -392,6 +478,8 @@ void FileProtocol::put( const KURL& url, int _mode, bool _overwrite, bool _resum
 void FileProtocol::copy( const KURL &src, const KURL &dest,
                          int _mode, bool _overwrite )
 {
+    kdDebug(7101) << "copy(): " << src << " -> " << dest << ", mode=" << _mode << endl;
+        
     QCString _src( QFile::encodeName(src.path()));
     QCString _dest( QFile::encodeName(dest.path()));
     KDE_struct_stat buff_src;
@@ -413,7 +501,7 @@ void FileProtocol::copy( const KURL &src, const KURL &dest,
     }
 
     KDE_struct_stat buff_dest;
-    bool dest_exists = ( KDE_stat( _dest.data(), &buff_dest ) != -1 );
+    bool dest_exists = ( KDE_lstat( _dest.data(), &buff_dest ) != -1 );
     if ( dest_exists )
     {
         if (S_ISDIR(buff_dest.st_mode))
@@ -427,6 +515,15 @@ void FileProtocol::copy( const KURL &src, const KURL &dest,
            error( KIO::ERR_FILE_ALREADY_EXIST, dest.path() );
            return;
         }
+        
+        // If the destination is a symlink and overwrite is TRUE,
+        // remove the symlink first to prevent the scenario where 
+        // the symlink actually points to current source!
+        if (_overwrite && S_ISLNK(buff_dest.st_mode))
+        {
+            kdDebug(7101) << "copy(): LINK DESTINATION" << endl;
+            remove( _dest.data() );
+        }
     }
 
     int src_fd = KDE_open( _src.data(), O_RDONLY);
@@ -456,18 +553,43 @@ void FileProtocol::copy( const KURL &src, const KURL &dest,
     }
 
     totalSize( buff_src.st_size );
+
     KIO::filesize_t processed_size = 0;
-
     char buffer[ MAX_IPC_SIZE ];
-    QByteArray array;
-
+    int n;
+#ifdef USE_SENDFILE
+    bool use_sendfile=buff_src.st_size < 0x7FFFFFFF;
+#endif
     while( 1 )
     {
-       int n = ::read( src_fd, buffer, MAX_IPC_SIZE );
+#ifdef USE_SENDFILE
+       if (use_sendfile) {
+            off_t sf = processed_size;
+            n = ::sendfile( dest_fd, src_fd, &sf, MAX_IPC_SIZE );
+            processed_size = sf;
+            if ( n == -1 && errno == EINVAL ) { //not all filesystems support sendfile()
+                kdDebug(7101) << "sendfile() not supported, falling back " << endl;
+                use_sendfile = false;
+            }
+       }
+       if (!use_sendfile)
+#endif
+        n = ::read( src_fd, buffer, MAX_IPC_SIZE );
+
        if (n == -1)
        {
           if (errno == EINTR)
               continue;
+#ifdef USE_SENDFILE
+          if ( use_sendfile ) {
+            kdDebug(7101) << "sendfile() error:" << strerror(errno) << endl;
+            if ( errno == ENOSPC ) // disk full
+            {
+                error( KIO::ERR_DISK_FULL, dest.path());
+                remove( _dest.data() );
+            }
+          } else
+#endif
           error( KIO::ERR_COULD_NOT_READ, src.path());
           close(src_fd);
           close(dest_fd);
@@ -475,25 +597,30 @@ void FileProtocol::copy( const KURL &src, const KURL &dest,
        }
        if (n == 0)
           break; // Finished
+#ifdef USE_SENDFILE
+       if ( !use_sendfile ) {
+#endif
+         if (write_all( dest_fd, buffer, n))
+         {
+           close(src_fd);
+           close(dest_fd);
 
-       if (write_all( dest_fd, buffer, n))
-       {
-          close(src_fd);
-          close(dest_fd);
-          if ( errno == ENOSPC ) // disk full
-          {
+           if ( errno == ENOSPC ) // disk full
+           {
               error( KIO::ERR_DISK_FULL, dest.path());
               remove( _dest.data() );
-          }
-          else
-          {
+           }
+           else
+           {
               kdWarning(7101) << "Couldn't write[2]. Error:" << strerror(errno) << endl;
               error( KIO::ERR_COULD_NOT_WRITE, dest.path());
-          }
-          return;
+           }
+           return;
+         }
+         processed_size += n;
+#ifdef USE_SENDFILE
        }
-
-       processed_size += n;
+#endif
        processedSize( processed_size );
     }
 
@@ -511,7 +638,9 @@ void FileProtocol::copy( const KURL &src, const KURL &dest,
     {
        if (::chmod(_dest.data(), _mode) != 0)
        {
-           warning( i18n( "Could not change permissions for\n%1" ).arg( dest.path() ) );
+        // Eat the error if the filesystem apparently doesn't support chmod.
+        if ( KIO::testFileSystemFlag( _dest, KIO::SupportsChmod ) )
+            warning( i18n( "Could not change permissions for\n%1" ).arg( dest.path() ) );
        }
     }
 
@@ -520,7 +649,9 @@ void FileProtocol::copy( const KURL &src, const KURL &dest,
     ut.actime = buff_src.st_atime;
     ut.modtime = buff_src.st_mtime;
     if ( ::utime( _dest.data(), &ut ) != 0 )
+    {
         kdWarning() << QString::fromLatin1("Couldn't preserve access and modification time for\n%1").arg( dest.path() ) << endl;
+    }
 
     processedSize( buff_src.st_size );
     finished();
@@ -609,7 +740,7 @@ void FileProtocol::symlink( const QString &target, const KURL &dest, bool overwr
         }
         else
         {
-            // Some error occured while we tried to symlink
+            // Some error occurred while we tried to symlink
             error( KIO::ERR_CANNOT_SYMLINK, dest.path() );
             return;
         }
@@ -712,7 +843,7 @@ bool FileProtocol::createUDSEntry( const QString & filename, const QCString & pa
             }
         }
     } else {
-        kdWarning() << "lstat didn't work on " << path.data() << endl;
+        // kdWarning() << "lstat didn't work on " << path.data() << endl;
         return false;
     }
 
@@ -790,20 +921,14 @@ void FileProtocol::stat( const KURL & url )
      * This is the reason for the -1
      */
     QCString _path( QFile::encodeName(url.path(-1)));
-    KDE_struct_stat buff;
-    if ( KDE_lstat( _path.data(), &buff ) == -1 ) {
-	error( KIO::ERR_DOES_NOT_EXIST, url.path(-1) );
-	return;
-    }
 
     QString sDetails = metaData(QString::fromLatin1("details"));
     int details = sDetails.isEmpty() ? 2 : sDetails.toInt();
-    kdDebug() << "FileProtocol::stat details=" << details << endl;
+    kdDebug(7101) << "FileProtocol::stat details=" << details << endl;
 
     UDSEntry entry;
     if ( !createUDSEntry( url.fileName(), _path, entry, details ) )
     {
-	// Should never happen
 	error( KIO::ERR_DOES_NOT_EXIST, url.path(-1) );
 	return;
     }
@@ -1048,7 +1173,7 @@ void FileProtocol::mount( bool _ro, const char *_fstype, const QString& _dev, co
     if(!epath.isEmpty())
         path += QString::fromLatin1(":") + epath;
     QString mountProg = KGlobal::dirs()->findExe("mount", path);
-    if (mountProg.isEmpty()) 
+    if (mountProg.isEmpty())
         mountProg = "mount";
 
     // Two steps, in case mount doesn't like it when we pass all options
@@ -1231,7 +1356,7 @@ void FileProtocol::unmount( const QString& _point )
        path += ":" + epath;
     QString umountProg = KGlobal::dirs()->findExe("umount", path);
 
-    if (umountProg.isEmpty()) 
+    if (umountProg.isEmpty())
         umountProg = "umount";
     buffer.sprintf( "%s %s 2>%s", umountProg.latin1(), QFile::encodeName(KProcess::quote(_point)).data(), tmp );
     system( buffer.data() );
@@ -1250,7 +1375,7 @@ void FileProtocol::unmount( const QString& _point )
  *
  *************************************/
 
-QString testLogFile( const char *_filename )
+static QString testLogFile( const char *_filename )
 {
     char buffer[ 1024 ];
     KDE_struct_stat buff;
