@@ -1,3 +1,4 @@
+// -*- Mode: c++; c-basic-offset: 2; indent-tabs-mode: nil; tab-width: 2; -*-
 /*  This file is part of the KDE libraries
     Copyright (C) 2000 David Faure <faure@kde.org>
 
@@ -17,7 +18,7 @@
     Boston, MA 02111-1307, USA.
 */
 
-// $Id: ftp.h,v 1.45 2002/10/15 10:45:41 faure Exp $
+// $Id: ftp.h,v 1.49.2.2 2004/02/15 21:15:27 faure Exp $
 
 #ifndef __ftp_h__
 #define __ftp_h__
@@ -44,7 +45,7 @@ struct FtpEntry
   QString group;
   QString link;
 
-  long size;
+  KIO::filesize_t size;
   mode_t type;
   mode_t access;
   time_t date;
@@ -68,7 +69,7 @@ public:
 
   /**
    * Connects to a ftp server and logs us in
-   * @ref m_bLoggedOn is set to true if logging on was successfull.
+   * m_bLoggedOn is set to true if logging on was successful.
    * It is set to false if the connection becomes closed.
    *
    */
@@ -103,16 +104,31 @@ private:
   // emit error on error (they are highlevel methods).
 
   /**
-   * Called by @ref openConnection. It opens the control connection to the ftp server.
+   * Connect and login to the FTP server.
+   *
+   * If login is set to false, this function will not attempt
+   * to login to the server.
+   *
+   * @param login if true send login info to the FTP server.
+   */
+  void ftpOpenConnection ( bool login = true );
+
+  /**
+   * Executes any auto login macro's as specified in a .netrc file.
+   */
+  void ftpAutoLoginMacro ();
+
+  /**
+   * Called by openConnection. It opens the control connection to the ftp server.
    *
    * @return true on success.
    */
   bool connect( const QString & host, unsigned short int port = 0 );
 
   /**
-   * Called by @ref openConnection. It logs us in.
-   * @ref m_initialPath is set to the current working directory
-   * if logging on was successfull.
+   * Called by openConnection. It logs us in.
+   * m_initialPath is set to the current working directory
+   * if logging on was successful.
    *
    * @return true on success.
    */
@@ -139,9 +155,9 @@ private:
 
   /**
    * Runs a command on the ftp server like "list" or "retr". In contrast to
-   * @ref ftpSendCmd a data connection is opened. The corresponding socket
-   * @ref sData is available for reading/writing on success.
-   * The connection must be closed afterwards with @ref ftpCloseCommand.
+   * ftpSendCmd a data connection is opened. The corresponding socket
+   * sData is available for reading/writing on success.
+   * The connection must be closed afterwards with ftpCloseCommand.
    *
    * @param mode is 'A' or 'I'. 'A' means ASCII transfer, 'I' means binary transfer.
    * @param errorcode the command-dependent error code to emit on error
@@ -149,10 +165,10 @@ private:
    * @return true if the command was accepted by the server.
    */
   bool ftpOpenCommand( const char *command, const QString & path, char mode,
-                       int errorcode, unsigned long offset = 0 );
+                       int errorcode, KIO::fileoffset_t offset = 0 );
 
   /**
-   * The counterpart to @ref openCommand.
+   * The counterpart to openCommand.
    * Closes data sockets and then reads line sent by server at
    * end of command.
    * @return false on error (line doesn't start with '2')
@@ -164,19 +180,19 @@ private:
   //void ftpAbortTransfer();
 
   /**
-   * Used by @ref openCommand
+   * Used by openCommand
    */
   bool ftpOpenPASVDataConnection();
   /**
-   * Used by @ref openCommand
+   * Used by openCommand
    */
   bool ftpOpenEPSVDataConnection();
   /**
-   * Used by @ref openCommand
+   * Used by openCommand
    */
   bool ftpOpenEPRTDataConnection();
   /**
-   * Used by @ref openCommand
+   * Used by openCommand
    */
   bool ftpOpenDataConnection();
   /**
@@ -187,8 +203,8 @@ private:
    */
   int ftpAcceptConnect();
 
-  size_t ftpRead( void *buffer, long len );
-  size_t ftpWrite( void *buffer, long len );
+  KIO::filesize_t ftpRead( void *buffer, long len );
+  KIO::filesize_t ftpWrite( void *buffer, long len );
 
   bool ftpChmod( const QString & path, int permissions );
 
@@ -241,7 +257,7 @@ private: // data members
   int sDatal;
 
   QString m_host;
-  int m_port;
+  unsigned short int m_port;
   QString m_user;
   QString m_pass;
   /**
@@ -259,8 +275,8 @@ private: // data members
   bool m_bUseProxy;
   bool m_bPersistent;
 
-  size_t m_size;
-  static size_t UnknownSize;
+  KIO::filesize_t m_size;
+  static KIO::filesize_t UnknownSize;
 
   enum
   {
