@@ -23,7 +23,7 @@
 //----------------------------------------------------------------------------
 //
 // KDE File Manager -- HTTP Cookies
-// $Id: kcookiejar.cpp,v 1.89.2.13 2003/08/08 16:14:22 waba Exp $
+// $Id: kcookiejar.cpp,v 1.111.2.3 2004/04/14 11:24:10 waba Exp $
 
 //
 // The cookie protocol is a mess. RFC2109 is a joke since nobody seems to
@@ -60,6 +60,7 @@
 #include <qptrlist.h>
 #include <qptrdict.h>
 #include <qfile.h>
+#include <qdir.h>
 #include <qregexp.h>
 
 #include <kurl.h>
@@ -72,6 +73,13 @@
 
 #define READ_BUFFER_SIZE 8192
 
+// Note with respect to QString::fromLatin1( )
+// Cookies are stored as 8 bit data and passed to kio_http as
+// latin1 regardless of their actual encoding.
+
+// L1 is used to indicate latin1 constants
+#define L1(x) QString::fromLatin1(x)
+
 template class QPtrList<KHttpCookie>;
 template class QPtrDict<KHttpCookieList>;
 
@@ -79,10 +87,10 @@ QString KCookieJar::adviceToStr(KCookieAdvice _advice)
 {
     switch( _advice )
     {
-    case KCookieAccept: return "Accept";
-    case KCookieReject: return "Reject";
-    case KCookieAsk: return "Ask";
-    default: return "Dunno";
+    case KCookieAccept: return L1("Accept");
+    case KCookieReject: return L1("Reject");
+    case KCookieAsk: return L1("Ask");
+    default: return L1("Dunno");
     }
 }
 
@@ -91,7 +99,7 @@ KCookieAdvice KCookieJar::strToAdvice(const QString &_str)
     if (_str.isEmpty())
         return KCookieDunno;
 
-    QString advice = _str.lower();
+    QCString advice = _str.lower().latin1();
 
     if (advice == "accept")
         return KCookieAccept;
@@ -117,7 +125,8 @@ KHttpCookie::KHttpCookie(const QString &_host,
                  time_t _expireDate,
                  int _protocolVersion,
                  bool _secure,
-                 bool _httpOnly) :
+                 bool _httpOnly,
+                 bool _explicitPath) :
        mHost(_host),
        mDomain(_domain),
        mPath(_path.isEmpty() ? QString::null : _path),
@@ -126,7 +135,8 @@ KHttpCookie::KHttpCookie(const QString &_host,
        mExpireDate(_expireDate),
        mProtocolVersion(_protocolVersion),
        mSecure(_secure),
-       mHttpOnly(_httpOnly)
+       mHttpOnly(_httpOnly),
+       mExplicitPath(_explicitPath)
 {
 }
 
@@ -153,12 +163,11 @@ QString KHttpCookie::cookieStr(bool useDOMFormat)
     }
     else
     {
-        result.sprintf("$Version=\"%d\"; ", mProtocolVersion);
-        result += mName + "=" + mValue + "";
-        if (!mPath.isEmpty())
-            result += "; $Path=\""+ mPath + "\"";
+        result = mName + '=' + mValue;
+        if (mExplicitPath)
+            result += L1("; $Path=\"") + mPath + L1("\"");
         if (!mDomain.isEmpty())
-            result += "; $Domain=\""+ mDomain + "\"";
+            result += L1("; $Domain=\"") + mDomain + L1("\"");
     }
     return result;
 }
@@ -180,7 +189,7 @@ bool KHttpCookie::match(const QString &fqdn, const QStringList &domains,
             return false;
 
         // Maybe the domain needs an extra dot.
-        QString domain = "." + mDomain;
+        QString domain = '.' + mDomain;
         if ( !domains.contains( domain ) )
           if ( fqdn != mDomain )
             return false;
@@ -247,37 +256,94 @@ KCookieJar::~KCookieJar()
     // Not much to do here
 }
 
+static void removeDuplicateFromList(KHttpCookieList *list, KHttpCookie *cookiePtr, bool nameMatchOnly=false, bool updateWindowId=false)
+{
+    QString domain1 = cookiePtr->domain();
+    if (domain1.isEmpty())
+       domain1 = cookiePtr->host();
+
+    for ( KHttpCookiePtr cookie=list->first(); cookie != 0; )
+    {
+       QString domain2 = cookie->domain();
+       if (domain2.isEmpty())
+          domain2 = cookie->host();
+
+       if ( 
+            (cookiePtr->name() == cookie->name()) &&
+            (
+              nameMatchOnly ||
+              ( (domain1 == domain2) && (cookiePtr->path() == cookie->path()) )
+            )
+          )
+       {
+          if (updateWindowId)
+          {
+            for(QValueList<long>::ConstIterator it = cookie->windowIds().begin();
+                it != cookie->windowIds().end(); ++it)
+            {
+               long windowId = *it;
+               if (windowId && (cookiePtr->windowIds().find(windowId) == cookiePtr->windowIds().end()))
+               {
+                  cookiePtr->windowIds().append(windowId);
+               }
+            }
+          }
+          KHttpCookiePtr old_cookie = cookie;
+          cookie = list->next();
+          list->removeRef( old_cookie );
+          break;
+       }
+       else
+       {
+          cookie = list->next();
+       }
+    }
+}
+
+
 //
 // Looks for cookies in the cookie jar which are appropriate for _url.
 // Returned is a string containing all appropriate cookies in a format
 // which can be added to a HTTP-header without any additional processing.
 //
-QString KCookieJar::findCookies(const QString &_url, bool useDOMFormat, long windowId)
+QString KCookieJar::findCookies(const QString &_url, bool useDOMFormat, long windowId, KHttpCookieList *pendingCookies)
 {
     QString cookieStr;
     QStringList domains;
     QString fqdn;
     QString path;
     KHttpCookiePtr cookie;
-    int protVersion = 1;
-    int cookieCount = 0;
     KCookieAdvice advice = m_globalAdvice;
 
     if (!parseURL(_url, fqdn, path))
         return cookieStr;
 
-    bool secureRequest = (_url.find( "https://", 0, false) == 0 ||
-                          _url.find( "webdavs://", 0, false) == 0);
+    bool secureRequest = (_url.find( L1("https://"), 0, false) == 0 ||
+                          _url.find( L1("webdavs://"), 0, false) == 0);
 
     extractDomains(fqdn, domains);
+
+    KHttpCookieList allCookies;
+
     for(QStringList::ConstIterator it = domains.begin();
-        it != domains.end();
+        true;
         ++it)
     {
-       QString key = (*it).isNull() ? "" : (*it);
-       KHttpCookieList *cookieList = m_cookieDomains[key];
-       if (!cookieList)
-          continue; // No cookies for this domain
+       KHttpCookieList *cookieList;
+       if (it == domains.end())
+       {
+          cookieList = pendingCookies; // Add pending cookies
+          pendingCookies = 0;
+          if (!cookieList)
+             break;
+       }
+       else
+       {
+          QString key = (*it).isNull() ? L1("") : (*it);
+          cookieList = m_cookieDomains[key];
+          if (!cookieList)
+             continue; // No cookies for this domain
+       }
 
        if (cookieList->getAdvice() != KCookieDunno)
           advice = cookieList->getAdvice();
@@ -287,7 +353,11 @@ QString KCookieJar::findCookies(const QString &_url, bool useDOMFormat, long win
        // session cookies...
        if (advice == KCookieReject &&
            !(m_ignoreCookieExpirationDate && m_autoAcceptSessionCookies))
+       {
+          if (it == domains.end())
+             break; // Finished.
           continue;
+       }
 
        for ( cookie=cookieList->first(); cookie != 0; cookie=cookieList->next() )
        {
@@ -315,36 +385,55 @@ QString KCookieJar::findCookies(const QString &_url, bool useDOMFormat, long win
           {
              cookie->windowIds().append(windowId);
           }
+          
+          if (it == domains.end()) // Only needed when processing pending cookies
+             removeDuplicateFromList(&allCookies, cookie);
 
-          // Use first cookie to determine protocol version
+          allCookies.append(cookie);
+       }
+       if (it == domains.end())
+          break; // Finished.
+    }
+    
+
+    int cookieCount = 0;
+
+    int protVersion=0; 
+    for ( cookie=allCookies.first(); cookie != 0; cookie=allCookies.next() )
+    {
+       if (cookie->protocolVersion() > protVersion)
+          protVersion = cookie->protocolVersion();
+    }
+
+    for ( cookie=allCookies.first(); cookie != 0; cookie=allCookies.next() )
+    {
+       if (useDOMFormat)
+       {
+          if (cookieCount > 0)
+             cookieStr += L1("; ");
+          cookieStr += cookie->cookieStr(true);
+       }
+       else
+       {
           if (cookieCount == 0)
           {
-             protVersion = cookie->protocolVersion();
-          }
-          if (useDOMFormat)
-          {
-             if (cookieCount > 0)
-                cookieStr += "; ";
-             cookieStr += cookie->cookieStr(true);
-          }
-          else if (protVersion == 0)
-          {
-             if (cookieCount == 0)
-                cookieStr += "Cookie: ";
-             else
-                cookieStr += "; ";
-             cookieStr += cookie->cookieStr(false);
+             cookieStr += L1("Cookie: ");
+             if (protVersion > 0)
+             {
+                QString version;
+                version.sprintf("$Version=%d; ", protVersion); // Without quotes
+                cookieStr += version;
+             }
           }
           else
           {
-             if (cookieCount > 0)
-                cookieStr += "\r\n";
-             cookieStr += "Cookie: ";
-             cookieStr += cookie->cookieStr(false);
+             cookieStr += L1("; ");
           }
-          cookieCount++;
+          cookieStr += cookie->cookieStr(false);
        }
+       cookieCount++;
     }
+    
     return cookieStr;
 }
 
@@ -370,9 +459,9 @@ static const char * parseNameValue(const char *header,
         if ((*s=='\0') || (*s==';') || (*s=='\n'))
         {
             // No '=' sign -> use string as the value, name is empty
-            // (behaviour found in Mozilla and IE)
+            // (behavior found in Mozilla and IE)
             Name = "";
-            Value = header;
+            Value = QString::fromLatin1(header);
             Value.truncate( s - header );
             Value = Value.stripWhiteSpace();
             return (s);
@@ -407,12 +496,12 @@ static const char * parseNameValue(const char *header,
             if ((*s=='\0') || (*s=='\n'))
             {
                 // End of Name
-                Value = header;
+                Value = QString::fromLatin1(header);
                 Value.truncate(s - header);
                 return (s);
             }
         }
-        Value = header;
+        Value = QString::fromLatin1(header);
         Value.truncate( s - header );
 
         // *s == '\"';
@@ -431,7 +520,7 @@ static const char * parseNameValue(const char *header,
         while ((*s != '\0') && (*s != ';') && (*s != '\n'))
             s++;
         // End of Name
-        Value = header;
+        Value = QString::fromLatin1(header);
         Value.truncate( s - header );
         Value = Value.stripWhiteSpace();
     }
@@ -461,16 +550,16 @@ bool KCookieJar::parseURL(const QString &_url,
                           QString &_path)
 {
     KURL kurl(_url);
-    if (kurl.isMalformed())
+    if (!kurl.isValid())
        return false;
 
     _fqdn = kurl.host().lower();
     if (kurl.port())
     {
-       if (((kurl.protocol() == "http") && (kurl.port() != 80)) ||
-           ((kurl.protocol() == "https") && (kurl.port() != 443)))
+       if (((kurl.protocol() == L1("http")) && (kurl.port() != 80)) ||
+           ((kurl.protocol() == L1("https")) && (kurl.port() != 443)))
        {
-          _fqdn = QString("%1:%2").arg(kurl.port()).arg(_fqdn);
+          _fqdn = L1("%1:%2").arg(kurl.port()).arg(_fqdn);
        }
     }
 
@@ -484,9 +573,9 @@ bool KCookieJar::parseURL(const QString &_url,
 
     _path = kurl.path();
     if (_path.isEmpty())
-       _path = "/";
+       _path = L1("/");
 
-    QRegExp exp("[\\\\/]\\.\\.[\\\\/]");
+    QRegExp exp(L1("[\\\\/]\\.\\.[\\\\/]"));
     // Weird path, cookie stealing attempt?
     if (exp.search(_path) != -1)
        return false; // Deny everything!!
@@ -543,30 +632,25 @@ void KCookieJar::extractDomains(const QString &_fqdn,
           QCString t = partList[0].lower().utf8();
           if ((t == "com") || (t == "net") || (t == "org") || (t == "gov") || (t == "edu") || (t == "mil") || (t == "int"))
               break;
-
-          // The .name domain uses <name>.<surname>.name
-          // Although the TLD is striclty speaking .name, for our purpose
-          // it should be <surname>.name since people should not be able
-          // to set cookies for everyone with the same surname.
-          // Matches <surname>.name
-          if (partList[1].lower() == "name")
-              break;
        }
-       QString domain = partList.join(".");
-       _domains.append("." + domain);
+
+       // The .name domain uses <name>.<surname>.name
+       // Although the TLD is striclty speaking .name, for our purpose
+       // it should be <surname>.name since people should not be able
+       // to set cookies for everyone with the same surname.
+       // Matches <surname>.name
+       if ((partList.count() == 2)&& (partList[1].lower() == L1("name")))
+          break;
+
+       QString domain = partList.join(L1("."));
+       _domains.append('.' + domain);
        _domains.append(domain);
        partList.remove(partList.begin()); // Remove part
     }
 
-    // Only URLs that would get in here are of type
-    // "host.foo" or "host.co.fo" so simply append
-    // a '.' on top to make sure they are stored under
-    // the proper cookie domain.
-    if (_domains.isEmpty())
-       _domains.append( "." + _fqdn );
-
     // Always add the FQDN at the end of the list for
     // hostname == cookie-domainname checks!
+    _domains.append( '.' + _fqdn );
     _domains.append( _fqdn );
 }
 
@@ -583,6 +667,7 @@ KHttpCookieList KCookieJar::makeCookies(const QString &_url,
                                        long windowId)
 {
     KHttpCookieList cookieList;
+    KHttpCookieList cookieList2;
     KHttpCookiePtr lastCookie = 0;
     const char *cookieStr = cookie_headers.data();
     QString Name;
@@ -610,7 +695,7 @@ KHttpCookieList KCookieJar::makeCookies(const QString &_url,
             cookieStr += 13;
             crossDomain = true;
         }
-        else if (strncasecmp(cookieStr, "Set-Cookie:", 11) == 0)
+        else if (strncasecmp(cookieStr, "Set-Cookie:", 11) == 0) 
         {
             cookieStr = parseNameValue(cookieStr+11, Name, Value, true);
 
@@ -618,7 +703,7 @@ KHttpCookieList KCookieJar::makeCookies(const QString &_url,
             // Default domain = ""
             // Default path according to rfc2109
 
-            KHttpCookie *cookie = new KHttpCookie(fqdn, "", defaultPath, Name, Value);
+            KHttpCookie *cookie = new KHttpCookie(fqdn, L1(""), defaultPath, Name, Value);
             if (windowId)
                cookie->mWindowIds.append(windowId);
             cookie->mCrossDomain = crossDomain;
@@ -627,15 +712,23 @@ KHttpCookieList KCookieJar::makeCookies(const QString &_url,
             cookieList.append(cookie);
             lastCookie = cookie;
         }
-        else if (lastCookie && (strncasecmp(cookieStr, "Set-Cookie2:", 12) == 0))
+        else if (strncasecmp(cookieStr, "Set-Cookie2:", 12) == 0)
         {
-            // Does anyone invent his own headers these days?
-            // Read the fucking RFC guys! This header is not there!
-            //
-            // Update: rfc2965 defines Set-Cookie2: You wonder why they
-            // didn't use the version field instead.
-            cookieStr +=12;
-            // Continue with lastCookie
+            // Attempt to follow rfc2965
+            cookieStr = parseNameValue(cookieStr+12, Name, Value, true);
+
+            // Host = FQDN
+            // Default domain = ""
+            // Default path according to rfc2965
+
+            KHttpCookie *cookie = new KHttpCookie(fqdn, L1(""), defaultPath, Name, Value);
+            if (windowId)
+               cookie->mWindowIds.append(windowId);
+            cookie->mCrossDomain = crossDomain;
+
+            // Insert cookie in chain
+            cookieList2.append(cookie);
+            lastCookie = cookie;
         }
         else
         {
@@ -655,16 +748,16 @@ KHttpCookieList KCookieJar::makeCookies(const QString &_url,
         while ((*cookieStr == ';') || (*cookieStr == ' '))
         {
             cookieStr++;
-
+            
             // Name-Value pair follows
             cookieStr = parseNameValue(cookieStr, Name, Value);
 
-            Name = Name.lower();
-            if (Name == "domain")
+            QCString cName = Name.lower().latin1();
+            if (cName == "domain")
             {
                 lastCookie->mDomain = Value.lower();
             }
-            else if (Name == "max-age")
+            else if (cName == "max-age")
             {
                 int max_age = Value.toInt();
                 if (max_age == 0)
@@ -672,29 +765,30 @@ KHttpCookieList KCookieJar::makeCookies(const QString &_url,
                 else
                     lastCookie->mExpireDate = time(0)+max_age;
             }
-            else if (Name == "expires")
+            else if (cName == "expires")
             {
                 // Parse brain-dead netscape cookie-format
                 lastCookie->mExpireDate = KRFCDate::parseDate(Value);
             }
-            else if (Name == "path")
+            else if (cName == "path")
             {
                 if (Value.isEmpty())
                    lastCookie->mPath = QString::null; // Catch "" <> QString::null
                 else
-                   lastCookie->mPath = Value;
+                   lastCookie->mPath = KURL::decode_string(Value);
+                lastCookie->mExplicitPath = true;
             }
-            else if (Name == "version")
+            else if (cName == "version")
             {
                 lastCookie->mProtocolVersion = Value.toInt();
             }
-            else if ((Name == "secure") ||
-                     (Name.isEmpty() && Value.lower() == "secure"))
+            else if ((cName == "secure") ||
+                     (cName.isEmpty() && Value.lower() == L1("secure")))
             {
                 lastCookie->mSecure = true;
             }
-            else if ((Name == "httponly") ||
-                     (Name.isEmpty() && Value.lower() == "httponly"))
+            else if ((cName == "httponly") ||
+                     (cName.isEmpty() && Value.lower() == L1("httponly")))
             {
                 lastCookie->mHttpOnly = true;
             }
@@ -707,6 +801,13 @@ KHttpCookieList KCookieJar::makeCookies(const QString &_url,
         cookieStr++;
     }
 
+    // RFC2965 cookies come last so that they override netscape cookies.
+    while( !cookieList2.isEmpty() && (lastCookie = cookieList2.take(0)) )
+    {
+       removeDuplicateFromList(&cookieList, lastCookie, true);
+       cookieList.append(lastCookie);
+    }
+
     return cookieList;
 }
 
@@ -759,6 +860,7 @@ KHttpCookieList KCookieJar::makeDOMCookies(const QString &_url,
      return cookieList;
 }
 
+
 //
 // This function hands a KHttpCookie object over to the cookie jar.
 //
@@ -769,9 +871,6 @@ void KCookieJar::addCookie(KHttpCookiePtr &cookiePtr)
     QStringList domains;
     KHttpCookieList *cookieList = 0L;
 
-    QString domain1 = cookiePtr->domain();
-    if (domain1.isEmpty())
-       domain1 = cookiePtr->host();
     // We always need to do this to make sure that the
     // that cookies of type hostname == cookie-domainname
     // are properly removed and/or updated as necessary!
@@ -780,34 +879,15 @@ void KCookieJar::addCookie(KHttpCookiePtr &cookiePtr)
           (it != domains.end() && !cookieList);
           ++it )
     {
-        QString key = (*it).isNull() ? "" : (*it);
+        QString key = (*it).isNull() ? L1("") : (*it);
         KHttpCookieList *list= m_cookieDomains[key];
         if ( !list ) continue;
 
-        for ( KHttpCookiePtr cookie=list->first(); cookie != 0; )
-        {
-            QString domain2 = cookie->domain();
-            if (domain2.isEmpty())
-               domain2 = cookie->host();
-
-            if ( domain1 == domain2 &&
-                 cookiePtr->name() == cookie->name() &&
-                 cookiePtr->path() == cookie->path() )
-            {
-                KHttpCookiePtr old_cookie = cookie;
-                cookie = list->next();
-                list->removeRef( old_cookie );
-                break;
-            }
-            else
-            {
-                cookie = list->next();
-            }
-        }
+        removeDuplicateFromList(list, cookiePtr, false, true);
     }
 
     QString domain = stripDomain( cookiePtr );
-    QString key = domain.isNull() ? "" : domain;
+    QString key = domain.isNull() ? L1("") : domain;
     cookieList = m_cookieDomains[ key ];
     if (!cookieList)
     {
@@ -874,7 +954,7 @@ KCookieAdvice KCookieJar::cookieAdvice(KHttpCookiePtr cookiePtr)
        if (!valid)
        {
           // Maybe the domain doesn't start with a "."
-          QString domain = "."+cookiePtr->domain();
+          QString domain = '.' + cookiePtr->domain();
           if (domains.contains(domain))
              valid = true;
        }
@@ -1129,7 +1209,7 @@ bool KCookieJar::saveCookies(const QString &_filename)
 
     time_t curTime = time(0);
 
-    fprintf(fStream, "# KDE Cookie File\n#\n");
+    fprintf(fStream, "# KDE Cookie File v2\n#\n");
 
     fprintf(fStream, "%-20s %-20s %-12s %-10s %-4s %-20s %-4s %s\n",
                      "# Host", "Domain", "Path", "Exp.date", "Prot",
@@ -1161,18 +1241,18 @@ bool KCookieJar::saveCookies(const QString &_filename)
                     fprintf(fStream, "[%s]\n", domain.local8Bit().data());
                 }
                 // Store persistent cookies
-                QString path("\"");
+                QString path = L1("\"");
                 path += cookie->path();
-                path += "\"";
-                QString domain("\"");
+                path += '"';
+                QString domain = L1("\"");
                 domain += cookie->domain();
-                domain += "\"";
+                domain += '"';
                 fprintf(fStream, "%-20s %-20s %-12s %10lu  %3d %-20s %-4i %s\n",
-                        cookie->host().local8Bit().data(), domain.local8Bit().data(),
-                        path.local8Bit().data(), (unsigned long) cookie->expireDate(),
-                        cookie->protocolVersion()+200, cookie->name().local8Bit().data(),
-                        (cookie->isSecure() ? 1 : 0) + (cookie->isHttpOnly() ? 2 : 0),
-                        cookie->value().local8Bit().data());
+                        cookie->host().latin1(), domain.latin1(),
+                        path.latin1(), (unsigned long) cookie->expireDate(),
+                        cookie->protocolVersion(), cookie->name().latin1(),
+                        (cookie->isSecure() ? 1 : 0) + (cookie->isHttpOnly() ? 2 : 0) + (cookie->hasExplicitPath() ? 4 : 0),
+                        cookie->value().latin1());
                 cookie = cookieList->next();
             }
             else
@@ -1238,7 +1318,18 @@ bool KCookieJar::loadCookies(const QString &_filename)
     bool err = false;
     err = (fgets(buffer, READ_BUFFER_SIZE, fStream) == 0);
 
-    err = err || (strcmp(buffer, "# KDE Cookie File\n") != 0);
+    int version = 1;
+    if (!err)
+    {
+        if (strcmp(buffer, "# KDE Cookie File\n") == 0)
+        {
+          // version 1
+        }
+        else if (sscanf(buffer, "# KDE Cookie File v%d\n", &version) != 1)
+        {
+          err = true;
+        }
+    }
 
     if (!err)
     {
@@ -1262,13 +1353,16 @@ bool KCookieJar::loadCookies(const QString &_filename)
             bool keepQuotes = false;
             bool secure = false;
             bool httpOnly = false;
+            bool explicitPath = false;
             const char *value = 0;
-            if (protVer >= 200)
+            if ((version == 2) || (protVer >= 200))
             {
-                protVer -= 200;
+                if (protVer >= 200)
+                    protVer -= 200;
                 int i = atoi( parseField(line) );
                 secure = i & 1;
                 httpOnly = i & 2;
+                explicitPath = i & 4;
                 line[strlen(line)-1] = '\0'; // Strip LF.
                 value = line;
             }
@@ -1290,9 +1384,13 @@ bool KCookieJar::loadCookies(const QString &_filename)
             if ((expDate == 0) || (expDate < curTime))
                 continue;
 
-            KHttpCookie *cookie = new KHttpCookie(host, domain, path, name,
-                                                  value, expDate, protVer,
-                                                  secure, httpOnly);
+            KHttpCookie *cookie = new KHttpCookie(QString::fromLatin1(host),
+                                                  QString::fromLatin1(domain), 
+                                                  QString::fromLatin1(path), 
+                                                  QString::fromLatin1(name),
+                                                  QString::fromLatin1(value), 
+                                                  expDate, protVer,
+                                                  secure, httpOnly, explicitPath);
             addCookie(cookie);
         }
     }
@@ -1335,6 +1433,7 @@ void KCookieJar::saveConfig(KConfig *_config)
     }
     _config->writeEntry("CookieDomainAdvice", domainSettings);
     _config->sync();
+    m_configChanged = false;
 }
 
 
@@ -1350,12 +1449,13 @@ void KCookieJar::loadConfig(KConfig *_config, bool reparse )
     _config->setGroup("Cookie Dialog");
     m_showCookieDetails = _config->readBoolEntry( "ShowCookieDetails" );
     m_preferredPolicy = _config->readNumEntry( "PreferredPolicy", 0 );
+
     _config->setGroup("Cookie Policy");
     QStringList domainSettings = _config->readListEntry("CookieDomainAdvice");
     m_rejectCrossDomainCookies = _config->readBoolEntry( "RejectCrossDomainCookies", true );
     m_autoAcceptSessionCookies = _config->readBoolEntry( "AcceptSessionCookies", true );
     m_ignoreCookieExpirationDate = _config->readBoolEntry( "IgnoreExpirationDate", false );
-    QString value = _config->readEntry("CookieGlobalAdvice", "Ask");
+    QString value = _config->readEntry("CookieGlobalAdvice", L1("Ask"));
     m_globalAdvice = strToAdvice(value);
 
     // Reset current domain settings first.
@@ -1383,4 +1483,3 @@ void KCookieJar::loadConfig(KConfig *_config, bool reparse )
         setDomainAdvice(domain, advice);
     }
 }
-
