@@ -20,7 +20,7 @@
 //----------------------------------------------------------------------------
 //
 // KDE File Manager -- HTTP Cookies
-// $Id: kcookiejar.h,v 1.28.2.2 2003/02/09 14:09:59 rnolden Exp $
+// $Id: kcookiejar.h,v 1.33 2003/11/22 16:50:45 waba Exp $
 
 #ifndef KCOOKIEJAR_H
 #define KCOOKIEJAR_H
@@ -62,6 +62,7 @@ protected:
     bool    mSecure;
     bool    mCrossDomain;
     bool    mHttpOnly;
+    bool    mExplicitPath;
     QValueList<long> mWindowIds;
 
     QString cookieStr(bool useDOMFormat);
@@ -75,7 +76,8 @@ public:
                 time_t _expireDate=0,
                 int _protocolVersion=0,
                 bool _secure = false,
-                bool _httpOnly = false);
+                bool _httpOnly = false,
+                bool _explicitPath = false);
 
     QString domain(void) { return mDomain; }
     QString host(void) { return mHost; }
@@ -90,6 +92,7 @@ public:
     bool    isExpired(time_t currentDate);
     bool    isCrossDomain(void) { return mCrossDomain; }
     bool    isHttpOnly(void) { return mHttpOnly; }
+    bool    hasExplicitPath(void) { return mExplicitPath; }
     bool    match(const QString &fqdn, const QStringList &domainList, const QString &path);
 };
 
@@ -157,8 +160,11 @@ public:
      *
      * If @p useDOMFormat is true, the string is formatted in a format
      * in compliance with the DOM standard.
+     * @p pendingCookies contains a list of cookies that have not been
+     * approved yet by the user but that will be included in the result
+     * none the less.
      */
-    QString findCookies(const QString &_url, bool useDOMFormat, long windowId);
+    QString findCookies(const QString &_url, bool useDOMFormat, long windowId, KHttpCookieList *pendingCookies=0);
 
     /**
      * This function parses cookie_headers and returns a linked list of
@@ -175,7 +181,7 @@ public:
      * valid KHttpCookie objects for all cookies found in cookie_headers.
      * If no cookies could be found 0 is returned.
      *
-     * cookie_domstr should be a concatenation of "name=value" pairs, seperated
+     * cookie_domstr should be a concatenation of "name=value" pairs, separated
      * by a semicolon ';'.
      */
     KHttpCookieList makeDOMCookies(const QString &_url, const QCString &cookie_domstr, long windowId);
@@ -308,34 +314,34 @@ public:
 
     /** Returns the */
     int preferredDefaultPolicy() const { return m_preferredPolicy; }
-    
+
     /** Returns the */
     bool showCookieDetails () const { return m_showCookieDetails; }
-      
+
      /**
       * Sets the user's default preference cookie policy.
-      */     
+      */
      void setPreferredDefaultPolicy (int value) { m_preferredPolicy = value; }
-     
+
      /**
       * Sets the user's preference of level of detail displayed
       * by the cookie dialog.
-      */     
+      */
      void setShowCookieDetails (bool value) { m_showCookieDetails = value; }
-     
-     
+
+
 protected:
     QStringList m_domainList;
     KCookieAdvice m_globalAdvice;
     QDict<KHttpCookieList> m_cookieDomains;
-   
+
     bool m_configChanged;
-    bool m_cookiesChanged;    
+    bool m_cookiesChanged;
     bool m_showCookieDetails;
     bool m_rejectCrossDomainCookies;
-    bool m_autoAcceptSessionCookies;   
-    bool m_ignoreCookieExpirationDate;    
-    
-    int m_preferredPolicy;    
+    bool m_autoAcceptSessionCookies;
+    bool m_ignoreCookieExpirationDate;
+
+    int m_preferredPolicy;
 };
 #endif
