@@ -41,13 +41,14 @@ static const KCmdLineOptions options[] =
  { "remove <domain>", I18N_NOOP("Remove cookies for domain."), 0 },
  { "remove-all", I18N_NOOP("Remove all cookies."), 0 },
  { "reload-config", I18N_NOOP("Reload configuration file."), 0 },
- { 0, 0, 0 }
+ KCmdLineLastOption
 };
 
-int main(int argc, char *argv[])
+extern "C" int kdemain(int argc, char *argv[])
 {
    KLocale::setMainCatalogue("kdelibs");
-   KCmdLineArgs::init(argc, argv, "kcookiejar", description, version);
+   KCmdLineArgs::init(argc, argv, "kcookiejar", I18N_NOOP("HTTP Cookie Daemon"),
+		      description, version);
 
    KCmdLineArgs::addCmdLineOptions( options );
 
