@@ -2,6 +2,7 @@
 /*
  *  This file is part of the KDE libraries
  *  Copyright (C) 1999-2001 Harri Porten (porten@kde.org)
+ *  Copyright (C) 2003 Apple Computer, Inc.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Lesser General Public
@@ -37,12 +38,21 @@ using namespace KJS;
 // ------------------------------ FunctionPrototypeImp -------------------------
 
 FunctionPrototypeImp::FunctionPrototypeImp(ExecState *exec)
-  : InternalFunctionImp(0)
+  : InternalFunctionImp((FunctionPrototypeImp*)0)
 {
   Value protect(this);
-  put(exec, "toString", Object(new FunctionProtoFuncImp(exec, this, FunctionProtoFuncImp::ToString, 0)), DontEnum);
-  put(exec, "apply",    Object(new FunctionProtoFuncImp(exec, this, FunctionProtoFuncImp::Apply,    2)), DontEnum);
-  put(exec, "call",     Object(new FunctionProtoFuncImp(exec, this, FunctionProtoFuncImp::Call,     1)), DontEnum);
+  putDirect(toStringPropertyName,
+	    new FunctionProtoFuncImp(exec, this, FunctionProtoFuncImp::ToString, 0, toStringPropertyName),
+	    DontEnum);
+  static const Identifier applyPropertyName("apply");
+  putDirect(applyPropertyName,
+	    new FunctionProtoFuncImp(exec, this, FunctionProtoFuncImp::Apply,    2, applyPropertyName),
+	    DontEnum);
+  static const Identifier callPropertyName("call");
+  putDirect(callPropertyName,
+	    new FunctionProtoFuncImp(exec, this, FunctionProtoFuncImp::Call,     1, callPropertyName),
+	    DontEnum);
+  putDirect(lengthPropertyName, 0, DontDelete|ReadOnly|DontEnum);
 }
 
 FunctionPrototypeImp::~FunctionPrototypeImp()
@@ -62,12 +72,13 @@ Value FunctionPrototypeImp::call(ExecState */*exec*/, Object &/*thisObj*/, const
 
 // ------------------------------ FunctionProtoFuncImp -------------------------
 
-FunctionProtoFuncImp::FunctionProtoFuncImp(ExecState *exec,
-                                         FunctionPrototypeImp *funcProto, int i, int len)
+FunctionProtoFuncImp::FunctionProtoFuncImp(ExecState */*exec*/, FunctionPrototypeImp *funcProto,
+					   int i, int len, const Identifier &_ident)
   : InternalFunctionImp(funcProto), id(i)
 {
   Value protect(this);
-  put(exec,"length",Number(len),DontDelete|ReadOnly|DontEnum);
+  putDirect(lengthPropertyName, len, DontDelete|ReadOnly|DontEnum);
+  ident = _ident;
 }
 
 
@@ -83,7 +94,7 @@ Value FunctionProtoFuncImp::call(ExecState *exec, Object &thisObj, const List &a
   switch (id) {
   case ToString: {
     // ### also make this work for internal functions
-    if (thisObj.isNull() || !thisObj.inherits(&InternalFunctionImp::info)) {
+    if (!thisObj.isValid() || !thisObj.inherits(&InternalFunctionImp::info)) {
 #ifndef NDEBUG
       fprintf(stderr,"attempted toString() call on null or non-function object\n");
 #endif
@@ -91,17 +102,19 @@ Value FunctionProtoFuncImp::call(ExecState *exec, Object &thisObj, const List &a
       exec->setException(err);
       return err;
     }
+
     if (thisObj.inherits(&DeclaredFunctionImp::info)) {
        DeclaredFunctionImp *fi = static_cast<DeclaredFunctionImp*>
                                  (thisObj.imp());
-       return String("function " + fi->name() + "(" +
-         fi->parameterString() + ") " + fi->body->toString());
-    } else if (thisObj.inherits(&FunctionImp::info) &&
-        !static_cast<FunctionImp*>(thisObj.imp())->name().isNull()) {
-      result = String("function " + static_cast<FunctionImp*>(thisObj.imp())->name() + "()");
+       return String("function " + fi->name().ustring() + "(" +
+         fi->parameterString() + ") " + fi->body->toCode());
+    } else if (thisObj.inherits(&InternalFunctionImp::info) &&
+        !static_cast<InternalFunctionImp*>(thisObj.imp())->name().isNull()) {
+      result = String("\nfunction " + static_cast<InternalFunctionImp*>(thisObj.imp())->name().ustring() + "() {\n"
+		      "    [native code]\n}\n");
     }
     else {
-      result = String("(Internal function)");
+      result = String("[function]");
     }
     }
     break;
@@ -124,14 +137,14 @@ Value FunctionProtoFuncImp::call(ExecState *exec, Object &thisObj, const List &a
 
     List applyArgs;
     if (!argArray.isA(NullType) && !argArray.isA(UndefinedType)) {
-      if ((argArray.isA(ObjectType) &&
-           Object::dynamicCast(argArray).inherits(&ArrayInstanceImp::info)) ||
-           Object::dynamicCast(argArray).inherits(&ArgumentsImp::info)) {
+      if (argArray.isA(ObjectType) &&
+           (Object::dynamicCast(argArray).inherits(&ArrayInstanceImp::info) ||
+            Object::dynamicCast(argArray).inherits(&ArgumentsImp::info))) {
 
         Object argArrayObj = Object::dynamicCast(argArray);
-        unsigned int length = argArrayObj.get(exec,"length").toUInt32(exec);
+        unsigned int length = argArrayObj.get(exec,lengthPropertyName).toUInt32(exec);
         for (unsigned int i = 0; i < length; i++)
-          applyArgs.append(argArrayObj.get(exec,UString::from(i)));
+          applyArgs.append(argArrayObj.get(exec,i));
       }
       else {
         Object err = Error::create(exec,TypeError);
@@ -158,9 +171,7 @@ Value FunctionProtoFuncImp::call(ExecState *exec, Object &thisObj, const List &a
     else
       callThis = thisArg.toObject(exec);
 
-    List callArgs = args.copy();
-    callArgs.removeFirst();
-    result = func.call(exec,callThis,callArgs);
+    result = func.call(exec,callThis,args.copyTail());
     }
     break;
   }
@@ -170,14 +181,14 @@ Value FunctionProtoFuncImp::call(ExecState *exec, Object &thisObj, const List &a
 
 // ------------------------------ FunctionObjectImp ----------------------------
 
-FunctionObjectImp::FunctionObjectImp(ExecState *exec, FunctionPrototypeImp *funcProto)
+FunctionObjectImp::FunctionObjectImp(ExecState */*exec*/, FunctionPrototypeImp *funcProto)
   : InternalFunctionImp(funcProto)
 {
   Value protect(this);
-  put(exec,"prototype", Object(funcProto), DontEnum|DontDelete|ReadOnly);
+  putDirect(prototypePropertyName, funcProto, DontEnum|DontDelete|ReadOnly);
 
   // no. of arguments for constructor
-  put(exec,"length", Number(1), ReadOnly|DontDelete|DontEnum);
+  putDirect(lengthPropertyName, NumberImp::one(), ReadOnly|DontDelete|DontEnum);
 }
 
 FunctionObjectImp::~FunctionObjectImp()
@@ -207,35 +218,42 @@ Object FunctionObjectImp::construct(ExecState *exec, const List &args)
   }
 
   // parse the source code
-  int sid;
+  SourceCode *source;
   int errLine;
   UString errMsg;
-  ProgramNode *progNode = Parser::parse(body.data(),body.size(),&sid,&errLine,&errMsg);
+  FunctionBodyNode *progNode = Parser::parse(body.data(),body.size(),&source,&errLine,&errMsg);
 
   // notify debugger that source has been parsed
   Debugger *dbg = exec->interpreter()->imp()->debugger();
   if (dbg) {
-    bool cont = dbg->sourceParsed(exec,sid,body,errLine);
+    bool cont = dbg->sourceParsed(exec,source->sid,body,errLine);
     if (!cont) {
+      source->deref();
       dbg->imp()->abort();
+      if (progNode)
+	delete progNode;
       return Object(new ObjectImp());
     }
   }
 
+  exec->interpreter()->imp()->addSourceCode(source);
+
   // no program node == syntax error - throw a syntax error
   if (!progNode) {
     Object err = Error::create(exec,SyntaxError,errMsg.ascii(),errLine);
     // we can't return a Completion(Throw) here, so just set the exception
     // and return it
     exec->setException(err);
+    source->deref();
     return err;
   }
+  source->deref();
 
-  List scopeChain;
-  scopeChain.append(exec->interpreter()->globalObject());
+  ScopeChain scopeChain;
+  scopeChain.push(exec->interpreter()->globalObject().imp());
   FunctionBodyNode *bodyNode = progNode;
 
-  FunctionImp *fimp = new DeclaredFunctionImp(exec, UString::null, bodyNode,
+  FunctionImp *fimp = new DeclaredFunctionImp(exec, Identifier::null(), bodyNode,
 					      scopeChain);
   Object ret(fimp); // protect from GC
 
@@ -247,22 +265,22 @@ Object FunctionObjectImp::construct(ExecState *exec, const List &args)
   while (i < len) {
       while (*c == ' ' && i < len)
 	  c++, i++;
-      if (Lexer::isIdentLetter(c->unicode())) {  // else error
+      if (Lexer::isIdentLetter(c->uc)) {  // else error
 	  param = UString(c, 1);
 	  c++, i++;
-	  while (i < len && (Lexer::isIdentLetter(c->unicode()) ||
-			     Lexer::isDecimalDigit(c->unicode()))) {
+	  while (i < len && (Lexer::isIdentLetter(c->uc) ||
+			     Lexer::isDecimalDigit(c->uc))) {
 	      param += UString(c, 1);
 	      c++, i++;
 	  }
 	  while (i < len && *c == ' ')
 	      c++, i++;
 	  if (i == len) {
-	      fimp->addParameter(param);
+	      fimp->addParameter(Identifier(param));
 	      params++;
 	      break;
 	  } else if (*c == ',') {
-	      fimp->addParameter(param);
+	      fimp->addParameter(Identifier(param));
 	      params++;
 	      c++, i++;
 	      continue;
@@ -275,15 +293,12 @@ Object FunctionObjectImp::construct(ExecState *exec, const List &args)
       return err;
   }
 
-  fimp->put(exec,"length", Number(params),ReadOnly|DontDelete|DontEnum);
   List consArgs;
 
   Object objCons = exec->interpreter()->builtinObject();
   Object prototype = objCons.construct(exec,List::empty());
-  prototype.put(exec, "constructor",
-		Object(fimp), DontEnum|DontDelete|ReadOnly);
-  fimp->put(exec,"prototype",prototype,DontEnum|DontDelete|ReadOnly);
-  fimp->put(exec,"arguments",Null(),DontEnum|DontDelete|ReadOnly);
+  prototype.put(exec, constructorPropertyName, Value(fimp), DontEnum|DontDelete|ReadOnly);
+  fimp->put(exec, prototypePropertyName, prototype, DontEnum|DontDelete|ReadOnly);
   return ret;
 }
 
