@@ -4,21 +4,20 @@
  *  Copyright (C) 1999-2000 Harri Porten (porten@kde.org)
  *
  *  This library is free software; you can redistribute it and/or
- *  modify it under the terms of the GNU Lesser General Public
+ *  modify it under the terms of the GNU Library General Public
  *  License as published by the Free Software Foundation; either
  *  version 2 of the License, or (at your option) any later version.
  *
  *  This library is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- *  Lesser General Public License for more details.
+ *  Library General Public License for more details.
  *
- *  You should have received a copy of the GNU Lesser General Public License
+ *  You should have received a copy of the GNU Library General Public License
  *  along with this library; see the file COPYING.LIB.  If not, write to
  *  the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  *  Boston, MA 02111-1307, USA.
  *
- *  $Id: lexer.cpp,v 1.47 2002/11/19 22:47:27 faure Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -37,7 +36,7 @@
 #include "interpreter.h"
 #include "nodes.h"
 #include "lexer.h"
-#include "ustring.h"
+#include "identifier.h"
 #include "lookup.h"
 #include "internal.h"
 
@@ -68,13 +67,14 @@ Lexer::Lexer()
 #ifndef KJS_PURE_ECMA
     bol(true),
 #endif
-    current(0), next1(0), next2(0), next3(0)
+    current(0), next1(0), next2(0), next3(0),
+    strings(0), numStrings(0), stringsCapacity(0),
+    identifiers(0), numIdentifiers(0), identifiersCapacity(0)
 {
   // allocate space for read buffers
   buffer8 = new char[size8];
   buffer16 = new UChar[size16];
   currLexer = this;
-
 }
 
 Lexer::~Lexer()
@@ -108,6 +108,7 @@ void Lexer::setCode(const UChar *c, unsigned int len)
   eatNextIdentifier = false;
   stackToken = -1;
   lastToken = -1;
+  foundBad = false;
   pos = 0;
   code = c;
   length = len;
@@ -118,10 +119,10 @@ void Lexer::setCode(const UChar *c, unsigned int len)
 #endif
 
   // read first characters
-  current = (length > 0) ? code[0].unicode() : 0;
-  next1 = (length > 1) ? code[1].unicode() : 0;
-  next2 = (length > 2) ? code[2].unicode() : 0;
-  next3 = (length > 3) ? code[3].unicode() : 0;
+  current = (length > 0) ? code[0].uc : 0;
+  next1 = (length > 1) ? code[1].uc : 0;
+  next2 = (length > 2) ? code[2].uc : 0;
+  next3 = (length > 3) ? code[3].uc : 0;
 }
 
 void Lexer::shift(unsigned int p)
@@ -131,7 +132,7 @@ void Lexer::shift(unsigned int p)
     current = next1;
     next1 = next2;
     next2 = next3;
-    next3 = (pos + 3 < length) ? code[pos+3].unicode() : 0;
+    next3 = (pos + 3 < length) ? code[pos+3].uc : 0;
   }
 }
 
@@ -191,7 +192,7 @@ int Lexer::lex()
 
     switch (state) {
     case Start:
-      if (isWhiteSpace()) {
+      if (isWhiteSpace(current)) {
         // do nothing
       } else if (current == '/' && next1 == '/') {
         shift(1);
@@ -285,6 +286,8 @@ int Lexer::lex()
       else if (current == 'u')
         state = InUnicodeEscape;
       else {
+	if (isLineTerminator)
+	  nextLine();
         record16(singleEscape(current));
         state = InString;
       }
@@ -448,19 +451,32 @@ int Lexer::lex()
   fprintf(stderr, "%s ", buffer8);
 #endif
 
-  double dval = 0;
+  long double dval = 0;
   if (state == Number) {
     dval = strtod(buffer8, 0L);
   } else if (state == Hex) { // scan hex numbers
-    // TODO: support long unsigned int
-    unsigned int i;
-    sscanf(buffer8, "%x", &i);
-    dval = i;
+    dval = 0;
+    if (buffer8[0] == '0' && (buffer8[1] == 'x' || buffer8[1] == 'X')) {
+      for (const char *p = buffer8+2; *p; p++) {
+	if (!isHexDigit(*p)) {
+	  dval = 0;
+	  break;
+	}
+	dval = dval * 16 + convertHex(*p);
+      }
+    }
     state = Number;
   } else if (state == Octal) {   // scan octal number
-    unsigned int ui;
-    sscanf(buffer8, "%o", &ui);
-    dval = ui;
+    dval = 0;
+    if (buffer8[0] == '0') {
+      for (const char *p = buffer8+1; *p; p++) {
+	if (*p < '0' || *p > '7') {
+	  dval = 0;
+	  break;
+	}
+	dval = dval * 8 + *p - '0';
+      }
+    }
     state = Number;
   }
 
@@ -509,12 +525,14 @@ int Lexer::lex()
       // Apply anonymous-function hack below (eat the identifier)
       if (eatNextIdentifier) {
         eatNextIdentifier = false;
+#ifdef KJS_VERBOSE
         UString debugstr(buffer16, pos16); fprintf(stderr,"Anonymous function hack: eating identifier %s\n",debugstr.ascii());
+#endif
         token = lex();
         break;
       }
       /* TODO: close leak on parse error. same holds true for String */
-      kjsyylval.ustr = new UString(buffer16, pos16);
+      kjsyylval.ident = makeIdentifier(buffer16, pos16);
       token = IDENT;
       break;
     }
@@ -532,7 +550,7 @@ int Lexer::lex()
       restrKeyword = true;
     break;
   case String:
-    kjsyylval.ustr = new UString(buffer16, pos16);
+    kjsyylval.ustr = makeUString(buffer16, pos16);
     token = STRING;
     break;
   case Number:
@@ -540,7 +558,7 @@ int Lexer::lex()
     token = NUMBER;
     break;
   case Bad:
-    fprintf(stderr, "KJS: yylex: ERROR.\n");
+    foundBad = true;
     return -1;
   default:
     assert(!"unhandled numeration value in switch");
@@ -550,10 +568,10 @@ int Lexer::lex()
   return token;
 }
 
-bool Lexer::isWhiteSpace() const
+bool Lexer::isWhiteSpace(unsigned short c)
 {
-  return (current == ' ' || current == '\t' ||
-          current == 0x0b || current == 0x0c);
+  return (c == ' ' || c == '\t' ||
+          c == 0x0b || c == 0x0c || c == 0xa0);
 }
 
 bool Lexer::isIdentLetter(unsigned short c)
@@ -569,14 +587,14 @@ bool Lexer::isDecimalDigit(unsigned short c)
   return (c >= '0' && c <= '9');
 }
 
-bool Lexer::isHexDigit(unsigned short c) const
+bool Lexer::isHexDigit(unsigned short c)
 {
   return (c >= '0' && c <= '9' ||
           c >= 'a' && c <= 'f' ||
           c >= 'A' && c <= 'F');
 }
 
-bool Lexer::isOctalDigit(unsigned short c) const
+bool Lexer::isOctalDigit(unsigned short c)
 {
   return (c >= '0' && c <= '7');
 }
@@ -819,3 +837,50 @@ bool Lexer::scanRegExp()
 
   return true;
 }
+
+
+void Lexer::doneParsing()
+{
+  for (unsigned i = 0; i < numIdentifiers; i++) {
+    delete identifiers[i];
+  }
+  free(identifiers);
+  identifiers = 0;
+  numIdentifiers = 0;
+  identifiersCapacity = 0;
+
+  for (unsigned i = 0; i < numStrings; i++) {
+    delete strings[i];
+  }
+  free(strings);
+  strings = 0;
+  numStrings = 0;
+  stringsCapacity = 0;
+}
+
+const int initialCapacity = 64;
+const int growthFactor = 2;
+
+Identifier *Lexer::makeIdentifier(UChar *buffer, unsigned int pos)
+{
+  if (numIdentifiers == identifiersCapacity) {
+    identifiersCapacity = (identifiersCapacity == 0) ? initialCapacity : identifiersCapacity *growthFactor;
+    identifiers = (KJS::Identifier **)realloc(identifiers, sizeof(KJS::Identifier *) * identifiersCapacity);
+  }
+
+  KJS::Identifier *identifier = new KJS::Identifier(buffer, pos);
+  identifiers[numIdentifiers++] = identifier;
+  return identifier;
+}
+
+UString *Lexer::makeUString(UChar *buffer, unsigned int pos)
+{
+  if (numStrings == stringsCapacity) {
+    stringsCapacity = (stringsCapacity == 0) ? initialCapacity : stringsCapacity *growthFactor;
+    strings = (UString **)realloc(strings, sizeof(UString *) * stringsCapacity);
+  }
+
+  UString *string = new UString(buffer, pos);
+  strings[numStrings++] = string;
+  return string;
+}
