@@ -2,6 +2,7 @@
 /*
  *  This file is part of the KDE libraries
  *  Copyright (C) 1999-2000 Harri Porten (porten@kde.org)
+ *  Copyright (C) 2003 Apple Computer, Inc.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Lesser General Public
@@ -17,7 +18,6 @@
  *  License along with this library; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
- *  $Id: lookup.cpp,v 1.11 2002/06/19 08:21:00 domi Exp $
  */
 
 #include <stdio.h>
@@ -31,48 +31,46 @@
 
 using namespace KJS;
 
+static bool keysMatch(const UChar *c, unsigned len, const char *s)
+{
+  for (unsigned i = 0; i != len; i++, c++, s++)
+    if (c->uc != (unsigned char)*s)
+      return false;
+  return *s == 0;
+}
+
 const HashEntry* Lookup::findEntry( const struct HashTable *table,
                               const UChar *c, unsigned int len )
 {
+#ifndef NDEBUG
   if (table->type != 2) {
     fprintf(stderr, "KJS: Unknown hash table version.\n");
     return 0;
   }
-  char *ascii = new char[len+1];
-  unsigned int i;
-  for(i = 0; i < len; i++, c++) {
-    if (!c->high())
-      ascii[i] = c->low();
-    else
-      break;
-  }
-  ascii[i] = '\0';
+#endif
 
-  int h = hash(ascii) % table->hashSize;
+  int h = hash(c, len) % table->hashSize;
   const HashEntry *e = &table->entries[h];
 
   // empty bucket ?
-  if (!e->s) {
-    delete [] ascii;
+  if (!e->soffset)
     return 0;
+
+  while(1) {
+    // compare strings
+    if (keysMatch(c, len, &table->sbase[e->soffset]))
+      return e;
+    // try next bucket
+    if(e->next < 0) break;
+
+    e = &table->entries[e->next];
   }
 
-  do {
-    // compare strings
-    if (strcmp(ascii, e->s) == 0) {
-      delete [] ascii;
-      return e;
-    }
-    // try next bucket
-    e = e->next;
-  } while (e);
-
-  delete [] ascii;
   return 0;
 }
 
 const HashEntry* Lookup::findEntry( const struct HashTable *table,
-                                const UString &s )
+                                const Identifier &s )
 {
   return findEntry( table, s.data(), s.size() );
 }
@@ -86,7 +84,7 @@ int Lookup::find(const struct HashTable *table,
   return -1;
 }
 
-int Lookup::find(const struct HashTable *table, const UString &s)
+int Lookup::find(const struct HashTable *table, const Identifier &s)
 {
   return find(table, s.data(), s.size());
 }
@@ -101,7 +99,7 @@ unsigned int Lookup::hash(const UChar *c, unsigned int len)
   return val;
 }
 
-unsigned int Lookup::hash(const UString &key)
+unsigned int Lookup::hash(const Identifier &key)
 {
   return hash(key.data(), key.size());
 }
