@@ -17,7 +17,6 @@
  *  License along with this library; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
- *  $Id: math_object.cpp,v 1.29 2002/06/19 08:21:00 domi Exp $
  */
 
 #include <math.h>
@@ -45,7 +44,7 @@ using namespace KJS;
 const ClassInfo MathObjectImp::info = { "Math", 0, &mathTable, 0 };
 
 /* Source for math_object.lut.h
-@begin mathTable 21
+@begin mathTable 31
   E             MathObjectImp::Euler    DontEnum|DontDelete|ReadOnly
   LN2           MathObjectImp::Ln2      DontEnum|DontDelete|ReadOnly
   LN10          MathObjectImp::Ln10     DontEnum|DontDelete|ReadOnly
@@ -77,12 +76,12 @@ const ClassInfo MathObjectImp::info = { "Math", 0, &mathTable, 0 };
 
 MathObjectImp::MathObjectImp(ExecState * /*exec*/,
                              ObjectPrototypeImp *objProto)
-  : ObjectImp(Object(objProto))
+  : ObjectImp(objProto)
 {
 }
 
 // ECMA 15.8
-Value MathObjectImp::get(ExecState *exec, const UString &propertyName) const
+Value MathObjectImp::get(ExecState *exec, const Identifier &propertyName) const
 {
   return lookupGet<MathFuncImp, MathObjectImp, ObjectImp>( exec, propertyName, &mathTable, this );
 }
@@ -131,7 +130,7 @@ MathFuncImp::MathFuncImp(ExecState *exec, int i, int l)
     ), id(i)
 {
   Value protect(this);
-  put(exec,"length",Number(l),DontDelete|ReadOnly|DontEnum);
+  putDirect(lengthPropertyName, l, DontDelete|ReadOnly|DontEnum);
 }
 
 bool MathFuncImp::implementsCall() const
@@ -201,7 +200,7 @@ Value MathFuncImp::call(ExecState *exec, Object &/*thisObj*/, const List &args)
         result = NaN;
         break;
       }
-      if ( val < result )
+      if ( val < result || (result == 0 && IS_NEGATIVE_ZERO(val)) )
         result = val;
     }
     break;
@@ -234,14 +233,20 @@ Value MathFuncImp::call(ExecState *exec, Object &/*thisObj*/, const List &args)
     result = result / RAND_MAX;
     break;
   case MathObjectImp::Round:
-    if (isNaN(arg))
+    if (isNaN(arg)) {
       result = arg;
-    else if (isInf(arg) || isInf(-arg))
+    }
+    else if (isInf(arg) || isInf(-arg)) {
       result = arg;
-    else if (arg == -0.5)
-      result = 0;
-    else
+    }
+    else if (arg == -0.5 || IS_NEGATIVE_ZERO(arg)) {
+      result = -0.0;
+    }
+    else {
       result = (double)(arg >= 0.0 ? int(arg + 0.5) : int(arg - 0.5));
+      if (result == 0.0 && arg < 0.0)
+	result = -0.0;
+    }
     break;
   case MathObjectImp::Sin:
     result = ::sin(arg);
