@@ -1,20 +1,21 @@
 // -*- c-basic-offset: 2 -*-
 /*
  *  This file is part of the KDE libraries
- *  Copyright (C) 1999-2000 Harri Porten (porten@kde.org)
+ *  Copyright (C) 1999-2000, 2003 Harri Porten (porten@kde.org)
  *  Copyright (C) 2001 Peter Kelly (pmk@post.com)
+ *  Copyright (C) 2003 Apple Computer, Inc.
  *
  *  This library is free software; you can redistribute it and/or
- *  modify it under the terms of the GNU Lesser General Public
+ *  modify it under the terms of the GNU Library General Public
  *  License as published by the Free Software Foundation; either
  *  version 2 of the License, or (at your option) any later version.
  *
  *  This library is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- *  Lesser General Public License for more details.
+ *  Library General Public License for more details.
  *
- *  You should have received a copy of the GNU Lesser General Public License
+ *  You should have received a copy of the GNU Library General Public License
  *  along with this library; see the file COPYING.LIB.  If not, write to
  *  the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  *  Boston, MA 02111-1307, USA.
@@ -27,17 +28,19 @@
 #include "internal.h"
 //#include "debugger.h"
 #ifndef NDEBUG
-#ifndef __osf__
 #include <list>
-#endif
+#include <assert.h>
 #endif
 
 namespace KJS {
 
   class RegExp;
   class SourceElementsNode;
-  class ProgramNode;
+  class ObjectLiteralNode;
+  class PropertyNode;
   class SourceStream;
+  class PropertyValueNode;
+  class PropertyNode;
 
   enum Operator { OpEqual,
 		  OpEqEq,
@@ -74,17 +77,25 @@ namespace KJS {
   public:
     Node();
     virtual ~Node();
+
+    // reusing Value Type here, declare new enum if required
+    virtual Type type() const { return UnspecifiedType; }
+
     /**
      * Evaluate this node and return the result, possibly a reference.
      */
-    virtual Reference2 evaluate(ExecState *exec) const;
+    virtual Reference evaluateReference(ExecState *exec) const;
     /**
      * Returns the value represented by this node. Always dereferenced.
      */
-    virtual Value value(ExecState *exec) const;
-    UString toString() const;
+    virtual Value evaluate(ExecState *exec) const;
+    virtual bool toBoolean(ExecState *exec) const;
+    virtual double toNumber(ExecState *exec) const;
+    virtual UString toString(ExecState *exec) const;
+
+    UString toCode() const;
     virtual void streamTo(SourceStream &s) const = 0;
-    virtual void processVarDecls(ExecState */*exec*/) {}
+    virtual void processVarDecls(ExecState* /*exec*/) {}
     int lineNo() const { return line; }
 
   public:
@@ -102,6 +113,8 @@ namespace KJS {
 #endif
   protected:
     Value throwError(ExecState *exec, ErrorType e, const char *msg) const;
+    Value throwError(ExecState *exec, ErrorType e, const char *msg, Value v, Node *expr) const;
+    Value throwError(ExecState *exec, ErrorType e, const char *msg, Identifier label) const;
     int line;
     unsigned int refcount;
     virtual int sourceId() const { return -1; }
@@ -118,37 +131,44 @@ namespace KJS {
   class StatementNode : public Node {
   public:
     StatementNode();
-    ~StatementNode();
-    void setLoc(int line0, int line1, int sourceId);
+    virtual ~StatementNode();
+    void setLoc(int line0, int line1, SourceCode *src);
     int firstLine() const { return l0; }
     int lastLine() const { return l1; }
-    int sourceId() const { return sid; }
+    int sourceId() const { return sourceCode->sid; }
+    SourceCode *code() const { return sourceCode; }
     bool hitStatement(ExecState *exec);
     bool abortStatement(ExecState *exec);
     virtual Completion execute(ExecState *exec) = 0;
-    void pushLabel(const UString *id) {
-      if (id) ls.push(*id);
-    }
+    void pushLabel(const Identifier &id) { ls.push(id); }
+    virtual void processFuncDecl(ExecState *exec);
   protected:
     LabelStack ls;
   private:
-    Reference2 evaluate(ExecState */*exec*/) const { return Reference2(); }
+    Reference evaluateReference(ExecState* /*exec*/) const { return Reference(0,Identifier::null()); }
     int l0, l1;
-    int sid;
+    SourceCode *sourceCode;
     bool breakPoint;
   };
 
   class NullNode : public Node {
   public:
     NullNode() {}
-    Value value(ExecState *exec) const;
+    virtual Value evaluate(ExecState *exec) const;
+    virtual bool toBoolean(ExecState *exec) const;
+    virtual double toNumber(ExecState *exec) const;
+    virtual UString toString(ExecState *exec) const;
     virtual void streamTo(SourceStream &s) const;
   };
 
   class BooleanNode : public Node {
   public:
     BooleanNode(bool v) : val(v) {}
-    Value value(ExecState *exec) const;
+    virtual Type type() const { return BooleanType; }
+    virtual Value evaluate(ExecState *exec) const;
+    virtual bool toBoolean(ExecState *exec) const;
+    virtual double toNumber(ExecState *exec) const;
+    virtual UString toString(ExecState *exec) const;
     virtual void streamTo(SourceStream &s) const;
   private:
     bool val;
@@ -157,7 +177,11 @@ namespace KJS {
   class NumberNode : public Node {
   public:
     NumberNode(double v) : val(v) { }
-    virtual Value value(ExecState *exec) const;
+    virtual Type type() const { return NumberType; }
+    virtual Value evaluate(ExecState *exec) const;
+    virtual bool toBoolean(ExecState *exec) const;
+    virtual double toNumber(ExecState *exec) const;
+    virtual UString toString(ExecState *exec) const;
     virtual void streamTo(SourceStream &s) const;
   private:
     double val;
@@ -166,7 +190,11 @@ namespace KJS {
   class StringNode : public Node {
   public:
     StringNode(const UString *v) : val(*v) { }
-    Value value(ExecState *exec) const;
+    virtual Type type() const { return StringType; }
+    virtual Value evaluate(ExecState *exec) const;
+    virtual bool toBoolean(ExecState *exec) const;
+    virtual double toNumber(ExecState *exec) const;
+    virtual UString toString(ExecState *exec) const;
     virtual void streamTo(SourceStream &s) const;
   private:
     UString val;
@@ -176,7 +204,8 @@ namespace KJS {
   public:
     RegExpNode(const UString &p, const UString &f)
       : pattern(p), flags(f) { }
-    virtual Value value(ExecState *exec) const;
+    virtual Value evaluate(ExecState *exec) const;
+    virtual bool toBoolean(ExecState *exec) const;
     virtual void streamTo(SourceStream &s) const;
   private:
     UString pattern, flags;
@@ -185,18 +214,18 @@ namespace KJS {
   class ThisNode : public Node {
   public:
     ThisNode() {}
-    virtual Value value(ExecState *exec) const;
+    virtual Value evaluate(ExecState *exec) const;
     virtual void streamTo(SourceStream &s) const;
   };
 
   class ResolveNode : public Node {
   public:
-    ResolveNode(const UString *s) : ident(*s) { }
-    Reference2 evaluate(ExecState *exec) const;
-    virtual Value value(ExecState *exec) const;
+    ResolveNode(const Identifier &s) : ident(s) { }
+    Reference evaluateReference(ExecState *exec) const;
+    virtual Value evaluate(ExecState *exec) const;
     virtual void streamTo(SourceStream &s) const;
   private:
-    UString ident;
+    Identifier ident;
   };
 
   class GroupNode : public Node {
@@ -204,94 +233,88 @@ namespace KJS {
     GroupNode(Node *g) : group(g) { }
     virtual void ref();
     virtual bool deref();
-    virtual ~GroupNode();
-    Reference2 evaluate(ExecState *exec) const;
-    virtual Value value(ExecState *exec) const;
+    Reference evaluateReference(ExecState *exec) const;
+    virtual Value evaluate(ExecState *exec) const;
     virtual void streamTo(SourceStream &s) const;
   private:
     Node *group;
   };
 
-  class ElisionNode : public Node {
-  public:
-    ElisionNode(ElisionNode *e) : elision(e) { }
-    virtual void ref();
-    virtual bool deref();
-    virtual ~ElisionNode();
-    virtual Value value(ExecState *exec) const;
-    virtual void streamTo(SourceStream &s) const;
-  private:
-    ElisionNode *elision;
-  };
-
   class ElementNode : public Node {
   public:
-    ElementNode(ElisionNode *e, Node *n) : list(0L), elision(e), node(n) { }
-    ElementNode(ElementNode *l, ElisionNode *e, Node *n)
-      : list(l), elision(e), node(n) { }
+    // list is circular during construction. cracked in ArrayNode ctor
+    ElementNode(int e, Node *n) : list(this), elision(e), node(n) { }
+    ElementNode(ElementNode *l, int e, Node *n)
+      : list(l->list), elision(e), node(n) { l->list = this; }
     virtual void ref();
     virtual bool deref();
-    virtual ~ElementNode();
-    virtual Value value(ExecState *exec) const;
+    virtual Value evaluate(ExecState *exec) const;
     virtual void streamTo(SourceStream &s) const;
   private:
+    friend class ArrayNode;
     ElementNode *list;
-    ElisionNode *elision;
+    int elision;
     Node *node;
   };
 
   class ArrayNode : public Node {
   public:
-    ArrayNode(ElisionNode *e) : element(0L), elision(e), opt(true) { }
+    ArrayNode(int e) : element(0L), elision(e), opt(true) { }
     ArrayNode(ElementNode *ele)
-      : element(ele), elision(0), opt(false) { }
-    ArrayNode(ElisionNode *eli, ElementNode *ele)
-      : element(ele), elision(eli), opt(true) { }
+      : element(ele->list), elision(0), opt(false) { ele->list = 0; }
+    ArrayNode(int eli, ElementNode *ele)
+      : element(ele->list), elision(eli), opt(true) { ele->list = 0; }
     virtual void ref();
     virtual bool deref();
-    virtual ~ArrayNode();
-    virtual Value value(ExecState *exec) const;
+    virtual Value evaluate(ExecState *exec) const;
     virtual void streamTo(SourceStream &s) const;
   private:
     ElementNode *element;
-    ElisionNode *elision;
+    int elision;
     bool opt;
   };
 
-  class ObjectLiteralNode : public Node {
-  public:
-    ObjectLiteralNode(Node *l) : list(l) { }
-    virtual void ref();
-    virtual bool deref();
-    virtual ~ObjectLiteralNode();
-    virtual Value value(ExecState *exec) const;
-    virtual void streamTo(SourceStream &s) const;
-  private:
-    Node *list;
-  };
-
   class PropertyValueNode : public Node {
   public:
-    PropertyValueNode(Node *n, Node *a, Node *l = 0L)
-      : name(n), assign(a), list(l) { }
+    // list is circular during construction, cut in ObjectLiteralNode ctor
+    PropertyValueNode(PropertyNode *n, Node *a)
+      : name(n), assign(a), list(this) { }
+    PropertyValueNode(PropertyNode *n, Node *a, PropertyValueNode *l)
+      : name(n), assign(a), list(l->list) { l->list = this; }
     virtual void ref();
     virtual bool deref();
-    virtual ~PropertyValueNode();
-    virtual Value value(ExecState *exec) const;
+    virtual Value evaluate(ExecState *exec) const;
     virtual void streamTo(SourceStream &s) const;
   private:
-    Node *name, *assign, *list;
+    friend class ObjectLiteralNode;
+    PropertyNode *name;
+    Node *assign;
+    PropertyValueNode *list;
   };
 
   class PropertyNode : public Node {
   public:
     PropertyNode(double d) : numeric(d) { }
-    PropertyNode(const UString *s) : str(*s) { }
-    virtual Value value(ExecState *exec) const;
+    PropertyNode(const Identifier &s) : str(s) { }
+    virtual Value evaluate(ExecState *exec) const;
     virtual void streamTo(SourceStream &s) const;
   private:
     double numeric;
-    UString str;
+    Identifier str;
+  };
+
+  class ObjectLiteralNode : public Node {
+  public:
+    // empty literal
+    ObjectLiteralNode() : list(0) { }
+    // l points to last list element, get and detach pointer to first one
+    ObjectLiteralNode(PropertyValueNode *l) : list(l->list) { l->list = 0; }
+    virtual void ref();
+    virtual bool deref();
+    virtual Value evaluate(ExecState *exec) const;
+    virtual void streamTo(SourceStream &s) const;
+  private:
+    PropertyValueNode *list;
   };
 
   class AccessorNode1 : public Node {
@@ -299,8 +322,7 @@ namespace KJS {
     AccessorNode1(Node *e1, Node *e2) : expr1(e1), expr2(e2) {}
     virtual void ref();
     virtual bool deref();
-    virtual ~AccessorNode1();
-    Reference2 evaluate(ExecState *exec) const;
+    Reference evaluateReference(ExecState *exec) const;
     virtual void streamTo(SourceStream &s) const;
   private:
     Node *expr1;
@@ -309,39 +331,40 @@ namespace KJS {
 
   class AccessorNode2 : public Node {
   public:
-    AccessorNode2(Node *e, const UString *s) : expr(e), ident(*s) { }
+    AccessorNode2(Node *e, const Identifier &s) : expr(e), ident(s) { }
     virtual void ref();
     virtual bool deref();
-    virtual ~AccessorNode2();
-    Reference2 evaluate(ExecState *exec) const;
+    Reference evaluateReference(ExecState *exec) const;
     virtual void streamTo(SourceStream &s) const;
   private:
     Node *expr;
-    UString ident;
+    Identifier ident;
   };
 
   class ArgumentListNode : public Node {
   public:
-    ArgumentListNode(Node *e);
-    ArgumentListNode(ArgumentListNode *l, Node *e);
+    // list is circular during construction. cracked in ArgumentsNode ctor
+    ArgumentListNode(Node *e) : list(this), expr(e) {}
+    ArgumentListNode(ArgumentListNode *l, Node *e)
+      : list(l->list), expr(e) { l->list = this; }
     virtual void ref();
     virtual bool deref();
-    virtual ~ArgumentListNode();
-    virtual Value value(ExecState *exec) const;
+    virtual Value evaluate(ExecState *exec) const;
     List evaluateList(ExecState *exec) const;
     virtual void streamTo(SourceStream &s) const;
   private:
+    friend class ArgumentsNode;
     ArgumentListNode *list;
     Node *expr;
   };
 
   class ArgumentsNode : public Node {
   public:
-    ArgumentsNode(ArgumentListNode *l);
+    ArgumentsNode() : list(0) {}
+    ArgumentsNode(ArgumentListNode *l) : list(l->list) { l->list = 0; }
     virtual void ref();
     virtual bool deref();
-    virtual ~ArgumentsNode();
-    virtual Value value(ExecState *exec) const;
+    virtual Value evaluate(ExecState *exec) const;
     List evaluateList(ExecState *exec) const;
     virtual void streamTo(SourceStream &s) const;
   private:
@@ -354,8 +377,7 @@ namespace KJS {
     NewExprNode(Node *e, ArgumentsNode *a) : expr(e), args(a) {}
     virtual void ref();
     virtual bool deref();
-    virtual ~NewExprNode();
-    Value value(ExecState *exec) const;
+    virtual Value evaluate(ExecState *exec) const;
     virtual void streamTo(SourceStream &s) const;
   private:
     Node *expr;
@@ -367,8 +389,7 @@ namespace KJS {
     FunctionCallNode(Node *e, ArgumentsNode *a) : expr(e), args(a) {}
     virtual void ref();
     virtual bool deref();
-    virtual ~FunctionCallNode();
-    virtual Value value(ExecState *exec) const;
+    virtual Value evaluate(ExecState *exec) const;
     virtual void streamTo(SourceStream &s) const;
   private:
     Node *expr;
@@ -380,8 +401,7 @@ namespace KJS {
     PostfixNode(Node *e, Operator o) : expr(e), oper(o) {}
     virtual void ref();
     virtual bool deref();
-    virtual ~PostfixNode();
-    Value value(ExecState *exec) const;
+    virtual Value evaluate(ExecState *exec) const;
     virtual void streamTo(SourceStream &s) const;
   private:
     Node *expr;
@@ -393,8 +413,7 @@ namespace KJS {
     DeleteNode(Node *e) : expr(e) {}
     virtual void ref();
     virtual bool deref();
-    virtual ~DeleteNode();
-    virtual Value value(ExecState *exec) const;
+    virtual Value evaluate(ExecState *exec) const;
     virtual void streamTo(SourceStream &s) const;
   private:
     Node *expr;
@@ -405,8 +424,7 @@ namespace KJS {
     VoidNode(Node *e) : expr(e) {}
     virtual void ref();
     virtual bool deref();
-    virtual ~VoidNode();
-    virtual Value value(ExecState *exec) const;
+    virtual Value evaluate(ExecState *exec) const;
     virtual void streamTo(SourceStream &s) const;
   private:
     Node *expr;
@@ -417,8 +435,7 @@ namespace KJS {
     TypeOfNode(Node *e) : expr(e) {}
     virtual void ref();
     virtual bool deref();
-    virtual ~TypeOfNode();
-    virtual Value value(ExecState *exec) const;
+    virtual Value evaluate(ExecState *exec) const;
     virtual void streamTo(SourceStream &s) const;
   private:
     Node *expr;
@@ -429,8 +446,7 @@ namespace KJS {
     PrefixNode(Operator o, Node *e) : oper(o), expr(e) {}
     virtual void ref();
     virtual bool deref();
-    virtual ~PrefixNode();
-    Value value(ExecState *exec) const;
+    virtual Value evaluate(ExecState *exec) const;
     virtual void streamTo(SourceStream &s) const;
   private:
     Operator oper;
@@ -442,8 +458,8 @@ namespace KJS {
     UnaryPlusNode(Node *e) : expr(e) {}
     virtual void ref();
     virtual bool deref();
-    virtual ~UnaryPlusNode();
-    virtual Value value(ExecState *exec) const;
+    virtual Value evaluate(ExecState *exec) const;
+    virtual double toNumber(ExecState *exec) const;
     virtual void streamTo(SourceStream &s) const;
   private:
     Node *expr;
@@ -454,8 +470,8 @@ namespace KJS {
     NegateNode(Node *e) : expr(e) {}
     virtual void ref();
     virtual bool deref();
-    virtual ~NegateNode();
-    virtual Value value(ExecState *exec) const;
+    virtual Value evaluate(ExecState *exec) const;
+    virtual double toNumber(ExecState *exec) const;
     virtual void streamTo(SourceStream &s) const;
   private:
     Node *expr;
@@ -466,8 +482,7 @@ namespace KJS {
     BitwiseNotNode(Node *e) : expr(e) {}
     virtual void ref();
     virtual bool deref();
-    virtual ~BitwiseNotNode();
-    virtual Value value(ExecState *exec) const;
+    virtual Value evaluate(ExecState *exec) const;
     virtual void streamTo(SourceStream &s) const;
   private:
     Node *expr;
@@ -478,8 +493,8 @@ namespace KJS {
     LogicalNotNode(Node *e) : expr(e) {}
     virtual void ref();
     virtual bool deref();
-    virtual ~LogicalNotNode();
-    virtual Value value(ExecState *exec) const;
+    virtual Value evaluate(ExecState *exec) const;
+    virtual bool toBoolean(ExecState *exec) const;
     virtual void streamTo(SourceStream &s) const;
   private:
     Node *expr;
@@ -490,8 +505,7 @@ namespace KJS {
     MultNode(Node *t1, Node *t2, char op) : term1(t1), term2(t2), oper(op) {}
     virtual void ref();
     virtual bool deref();
-    virtual ~MultNode();
-    virtual Value value(ExecState *exec) const;
+    virtual Value evaluate(ExecState *exec) const;
     virtual void streamTo(SourceStream &s) const;
   private:
     Node *term1, *term2;
@@ -501,24 +515,37 @@ namespace KJS {
   class AddNode : public Node {
   public:
     AddNode(Node *t1, Node *t2, char op) : term1(t1), term2(t2), oper(op) {}
+
+    static Node* create(Node *t1, Node *t2, char op);
+
     virtual void ref();
     virtual bool deref();
-    virtual ~AddNode();
-    virtual Value value(ExecState *exec) const;
+    virtual Value evaluate(ExecState *exec) const;
     virtual void streamTo(SourceStream &s) const;
   private:
     Node *term1, *term2;
     char oper;
   };
 
+  class AppendStringNode : public Node {
+  public:
+    AppendStringNode(Node *t, const UString &s) : term(t), str(s) { }
+    virtual void ref();
+    virtual bool deref();
+    virtual Value evaluate(ExecState *exec) const;
+    virtual void streamTo(SourceStream &s) const;
+  private:
+    Node *term;
+    UString str;
+  };
+
   class ShiftNode : public Node {
   public:
     ShiftNode(Node *t1, Operator o, Node *t2)
       : term1(t1), term2(t2), oper(o) {}
     virtual void ref();
     virtual bool deref();
-    virtual ~ShiftNode();
-    virtual Value value(ExecState *exec) const;
+    virtual Value evaluate(ExecState *exec) const;
     virtual void streamTo(SourceStream &s) const;
   private:
     Node *term1, *term2;
@@ -531,8 +558,7 @@ namespace KJS {
       expr1(e1), expr2(e2), oper(o) {}
     virtual void ref();
     virtual bool deref();
-    virtual ~RelationalNode();
-    virtual Value value(ExecState *exec) const;
+    virtual Value evaluate(ExecState *exec) const;
     virtual void streamTo(SourceStream &s) const;
   private:
     Node *expr1, *expr2;
@@ -545,8 +571,7 @@ namespace KJS {
       : expr1(e1), expr2(e2), oper(o) {}
     virtual void ref();
     virtual bool deref();
-    virtual ~EqualNode();
-    virtual Value value(ExecState *exec) const;
+    virtual Value evaluate(ExecState *exec) const;
     virtual void streamTo(SourceStream &s) const;
   private:
     Node *expr1, *expr2;
@@ -559,8 +584,7 @@ namespace KJS {
       expr1(e1), expr2(e2), oper(o) {}
     virtual void ref();
     virtual bool deref();
-    virtual ~BitOperNode();
-    virtual Value value(ExecState *exec) const;
+    virtual Value evaluate(ExecState *exec) const;
     virtual void streamTo(SourceStream &s) const;
   private:
     Node *expr1, *expr2;
@@ -576,8 +600,7 @@ namespace KJS {
       expr1(e1), expr2(e2), oper(o) {}
     virtual void ref();
     virtual bool deref();
-    virtual ~BinaryLogicalNode();
-    virtual Value value(ExecState *exec) const;
+    virtual Value evaluate(ExecState *exec) const;
     virtual void streamTo(SourceStream &s) const;
   private:
     Node *expr1, *expr2;
@@ -593,8 +616,7 @@ namespace KJS {
       logical(l), expr1(e1), expr2(e2) {}
     virtual void ref();
     virtual bool deref();
-    virtual ~ConditionalNode();
-    virtual Value value(ExecState *exec) const;
+    virtual Value evaluate(ExecState *exec) const;
     virtual void streamTo(SourceStream &s) const;
   private:
     Node *logical, *expr1, *expr2;
@@ -605,8 +627,7 @@ namespace KJS {
     AssignNode(Node *l, Operator o, Node *e) : left(l), oper(o), expr(e) {}
     virtual void ref();
     virtual bool deref();
-    virtual ~AssignNode();
-    virtual Value value(ExecState *exec) const;
+    virtual Value evaluate(ExecState *exec) const;
     virtual void streamTo(SourceStream &s) const;
   private:
     Node *left;
@@ -619,8 +640,7 @@ namespace KJS {
     CommaNode(Node *e1, Node *e2) : expr1(e1), expr2(e2) {}
     virtual void ref();
     virtual bool deref();
-    virtual ~CommaNode();
-    virtual Value value(ExecState *exec) const;
+    virtual Value evaluate(ExecState *exec) const;
     virtual void streamTo(SourceStream &s) const;
   private:
     Node *expr1, *expr2;
@@ -628,15 +648,16 @@ namespace KJS {
 
   class StatListNode : public StatementNode {
   public:
-    StatListNode(StatementNode *s) : statement(s), list(0L) { }
-    StatListNode(StatListNode *l, StatementNode *s) : statement(s), list(l) { }
+    // list is circular during construction. cracked in CaseClauseNode ctor
+    StatListNode(StatementNode *s);
+    StatListNode(StatListNode *l, StatementNode *s);
     virtual void ref();
     virtual bool deref();
-    virtual ~StatListNode();
     virtual Completion execute(ExecState *exec);
     virtual void processVarDecls(ExecState *exec);
     virtual void streamTo(SourceStream &s) const;
   private:
+    friend class CaseClauseNode;
     StatementNode *statement;
     StatListNode *list;
   };
@@ -646,8 +667,7 @@ namespace KJS {
     AssignExprNode(Node *e) : expr(e) {}
     virtual void ref();
     virtual bool deref();
-    virtual ~AssignExprNode();
-    Value value(ExecState *exec) const;
+    virtual Value evaluate(ExecState *exec) const;
     virtual void streamTo(SourceStream &s) const;
   private:
     Node *expr;
@@ -655,39 +675,40 @@ namespace KJS {
 
   class VarDeclNode : public Node {
   public:
-    VarDeclNode(const UString *id, AssignExprNode *in);
+    VarDeclNode(const Identifier &id, AssignExprNode *in);
     virtual void ref();
     virtual bool deref();
-    virtual ~VarDeclNode();
-    Value value(ExecState *exec) const;
+    virtual Value evaluate(ExecState *exec) const;
     virtual void processVarDecls(ExecState *exec);
     virtual void streamTo(SourceStream &s) const;
   private:
-    UString ident;
+    Identifier ident;
     AssignExprNode *init;
   };
 
   class VarDeclListNode : public Node {
   public:
-    VarDeclListNode(VarDeclNode *v) : list(0L), var(v) {}
-    VarDeclListNode(Node *l, VarDeclNode *v) : list(l), var(v) {}
+    // list is circular until cracked in VarStatementNode/ForNode ctor
+    VarDeclListNode(VarDeclNode *v) : list(this), var(v) {}
+    VarDeclListNode(VarDeclListNode *l, VarDeclNode *v)
+      : list(l->list), var(v) { l->list = this; }
     virtual void ref();
     virtual bool deref();
-    virtual ~VarDeclListNode();
-    Value value(ExecState *exec) const;
+    virtual Value evaluate(ExecState *exec) const;
     virtual void processVarDecls(ExecState *exec);
     virtual void streamTo(SourceStream &s) const;
   private:
-    Node *list;
+    friend class ForNode;
+    friend class VarStatementNode;
+    VarDeclListNode *list;
     VarDeclNode *var;
   };
 
   class VarStatementNode : public StatementNode {
   public:
-    VarStatementNode(VarDeclListNode *l) : list(l) {}
+    VarStatementNode(VarDeclListNode *l) : list(l->list) { l->list = 0; }
     virtual void ref();
     virtual bool deref();
-    virtual ~VarStatementNode();
     virtual Completion execute(ExecState *exec);
     virtual void processVarDecls(ExecState *exec);
     virtual void streamTo(SourceStream &s) const;
@@ -697,14 +718,13 @@ namespace KJS {
 
   class BlockNode : public StatementNode {
   public:
-    BlockNode(SourceElementsNode *s) : source(s) {}
+    BlockNode(SourceElementsNode *s);
     virtual void ref();
     virtual bool deref();
-    virtual ~BlockNode();
     virtual Completion execute(ExecState *exec);
     virtual void processVarDecls(ExecState *exec);
     virtual void streamTo(SourceStream &s) const;
-  private:
+  protected:
     SourceElementsNode *source;
   };
 
@@ -720,7 +740,6 @@ namespace KJS {
     ExprStatementNode(Node *e) : expr(e) { }
     virtual void ref();
     virtual bool deref();
-    virtual ~ExprStatementNode();
     virtual Completion execute(ExecState *exec);
     virtual void streamTo(SourceStream &s) const;
   private:
@@ -733,7 +752,6 @@ namespace KJS {
       : expr(e), statement1(s1), statement2(s2) {}
     virtual void ref();
     virtual bool deref();
-    virtual ~IfNode();
     virtual Completion execute(ExecState *exec);
     virtual void processVarDecls(ExecState *exec);
     virtual void streamTo(SourceStream &s) const;
@@ -747,7 +765,6 @@ namespace KJS {
     DoWhileNode(StatementNode *s, Node *e) : statement(s), expr(e) {}
     virtual void ref();
     virtual bool deref();
-    virtual ~DoWhileNode();
     virtual Completion execute(ExecState *exec);
     virtual void processVarDecls(ExecState *exec);
     virtual void streamTo(SourceStream &s) const;
@@ -761,7 +778,6 @@ namespace KJS {
     WhileNode(Node *e, StatementNode *s) : expr(e), statement(s) {}
     virtual void ref();
     virtual bool deref();
-    virtual ~WhileNode();
     virtual Completion execute(ExecState *exec);
     virtual void processVarDecls(ExecState *exec);
     virtual void streamTo(SourceStream &s) const;
@@ -774,9 +790,10 @@ namespace KJS {
   public:
     ForNode(Node *e1, Node *e2, Node *e3, StatementNode *s) :
       expr1(e1), expr2(e2), expr3(e3), statement(s) {}
+    ForNode(VarDeclListNode *e1, Node *e2, Node *e3, StatementNode *s) :
+      expr1(e1->list), expr2(e2), expr3(e3), statement(s) { e1->list = 0; }
     virtual void ref();
     virtual bool deref();
-    virtual ~ForNode();
     virtual Completion execute(ExecState *exec);
     virtual void processVarDecls(ExecState *exec);
     virtual void streamTo(SourceStream &s) const;
@@ -788,15 +805,14 @@ namespace KJS {
   class ForInNode : public StatementNode {
   public:
     ForInNode(Node *l, Node *e, StatementNode *s);
-    ForInNode(const UString *i, AssignExprNode *in, Node *e, StatementNode *s);
+    ForInNode(const Identifier &i, AssignExprNode *in, Node *e, StatementNode *s);
     virtual void ref();
     virtual bool deref();
-    virtual ~ForInNode();
     virtual Completion execute(ExecState *exec);
     virtual void processVarDecls(ExecState *exec);
     virtual void streamTo(SourceStream &s) const;
   private:
-    UString ident;
+    Identifier ident;
     AssignExprNode *init;
     Node *lexpr, *expr;
     VarDeclNode *varDecl;
@@ -806,21 +822,21 @@ namespace KJS {
   class ContinueNode : public StatementNode {
   public:
     ContinueNode() { }
-    ContinueNode(const UString *i) : ident(*i) { }
+    ContinueNode(const Identifier &i) : ident(i) { }
     virtual Completion execute(ExecState *exec);
     virtual void streamTo(SourceStream &s) const;
   private:
-    UString ident;
+    Identifier ident;
   };
 
   class BreakNode : public StatementNode {
   public:
     BreakNode() { }
-    BreakNode(const UString *i) : ident(*i) { }
+    BreakNode(const Identifier &i) : ident(i) { }
     virtual Completion execute(ExecState *exec);
     virtual void streamTo(SourceStream &s) const;
   private:
-    UString ident;
+    Identifier ident;
   };
 
   class ReturnNode : public StatementNode {
@@ -828,7 +844,6 @@ namespace KJS {
     ReturnNode(Node *v) : value(v) {}
     virtual void ref();
     virtual bool deref();
-    virtual ~ReturnNode();
     virtual Completion execute(ExecState *exec);
     virtual void streamTo(SourceStream &s) const;
   private:
@@ -840,7 +855,6 @@ namespace KJS {
     WithNode(Node *e, StatementNode *s) : expr(e), statement(s) {}
     virtual void ref();
     virtual bool deref();
-    virtual ~WithNode();
     virtual Completion execute(ExecState *exec);
     virtual void processVarDecls(ExecState *exec);
     virtual void streamTo(SourceStream &s) const;
@@ -851,11 +865,12 @@ namespace KJS {
 
   class CaseClauseNode: public Node {
   public:
-    CaseClauseNode(Node *e, StatListNode *l) : expr(e), list(l) { }
+    CaseClauseNode(Node *e) : expr(e), list(0) { }
+    CaseClauseNode(Node *e, StatListNode *l)
+      : expr(e), list(l->list) { l->list = 0; }
     virtual void ref();
     virtual bool deref();
-    virtual ~CaseClauseNode();
-    Value value(ExecState *exec) const;
+    virtual Value evaluate(ExecState *exec) const;
     Completion evalStatements(ExecState *exec) const;
     virtual void processVarDecls(ExecState *exec);
     virtual void streamTo(SourceStream &s) const;
@@ -866,29 +881,29 @@ namespace KJS {
 
   class ClauseListNode : public Node {
   public:
-    ClauseListNode(CaseClauseNode *c) : cl(c), nx(0L) { }
+    // list is circular during construction. cracked in CaseBlockNode ctor
+    ClauseListNode(CaseClauseNode *c) : cl(c), nx(this) { }
+    ClauseListNode(ClauseListNode *n, CaseClauseNode *c)
+      : cl(c), nx(n->nx) { n->nx = this; }
     virtual void ref();
     virtual bool deref();
-    virtual ~ClauseListNode();
-    ClauseListNode* append(CaseClauseNode *c);
-    Value value(ExecState *exec) const;
+    virtual Value evaluate(ExecState *exec) const;
     CaseClauseNode *clause() const { return cl; }
     ClauseListNode *next() const { return nx; }
     virtual void processVarDecls(ExecState *exec);
     virtual void streamTo(SourceStream &s) const;
   private:
+    friend class CaseBlockNode;
     CaseClauseNode *cl;
     ClauseListNode *nx;
   };
 
   class CaseBlockNode: public Node {
   public:
-    CaseBlockNode(ClauseListNode *l1, CaseClauseNode *d, ClauseListNode *l2)
-      : list1(l1), def(d), list2(l2) { }
+    CaseBlockNode(ClauseListNode *l1, CaseClauseNode *d, ClauseListNode *l2);
     virtual void ref();
     virtual bool deref();
-    virtual ~CaseBlockNode();
-    Value value(ExecState *exec) const;
+    virtual Value evaluate(ExecState *exec) const;
     Completion evalBlock(ExecState *exec, const Value& input) const;
     virtual void processVarDecls(ExecState *exec);
     virtual void streamTo(SourceStream &s) const;
@@ -903,7 +918,6 @@ namespace KJS {
     SwitchNode(Node *e, CaseBlockNode *b) : expr(e), block(b) { }
     virtual void ref();
     virtual bool deref();
-    virtual ~SwitchNode();
     virtual Completion execute(ExecState *exec);
     virtual void processVarDecls(ExecState *exec);
     virtual void streamTo(SourceStream &s) const;
@@ -914,15 +928,14 @@ namespace KJS {
 
   class LabelNode : public StatementNode {
   public:
-    LabelNode(const UString *l, StatementNode *s) : label(*l), statement(s) { }
+    LabelNode(const Identifier &l, StatementNode *s) : label(l), statement(s) { }
     virtual void ref();
     virtual bool deref();
-    virtual ~LabelNode();
     virtual Completion execute(ExecState *exec);
     virtual void processVarDecls(ExecState *exec);
     virtual void streamTo(SourceStream &s) const;
   private:
-    UString label;
+    Identifier label;
     StatementNode *statement;
   };
 
@@ -931,7 +944,6 @@ namespace KJS {
     ThrowNode(Node *e) : expr(e) {}
     virtual void ref();
     virtual bool deref();
-    virtual ~ThrowNode();
     virtual Completion execute(ExecState *exec);
     virtual void streamTo(SourceStream &s) const;
   private:
@@ -940,16 +952,15 @@ namespace KJS {
 
   class CatchNode : public StatementNode {
   public:
-    CatchNode(const UString *i, StatementNode *b) : ident(*i), block(b) {}
+    CatchNode(const Identifier &i, StatementNode *b) : ident(i), block(b) {}
     virtual void ref();
     virtual bool deref();
-    virtual ~CatchNode();
     virtual Completion execute(ExecState *exec);
     Completion execute(ExecState *exec, const Value &arg);
     virtual void processVarDecls(ExecState *exec);
     virtual void streamTo(SourceStream &s) const;
   private:
-    UString ident;
+    Identifier ident;
     StatementNode *block;
   };
 
@@ -958,7 +969,6 @@ namespace KJS {
     FinallyNode(StatementNode *b) : block(b) {}
     virtual void ref();
     virtual bool deref();
-    virtual ~FinallyNode();
     virtual Completion execute(ExecState *exec);
     virtual void processVarDecls(ExecState *exec);
     virtual void streamTo(SourceStream &s) const;
@@ -968,11 +978,14 @@ namespace KJS {
 
   class TryNode : public StatementNode {
   public:
-    TryNode(StatementNode *b, Node *c = 0L, Node *f = 0L)
-      : block(b), _catch((CatchNode*)c), _final((FinallyNode*)f) {}
+    TryNode(StatementNode *b, CatchNode *c)
+      : block(b), _catch(c), _final(0) {}
+    TryNode(StatementNode *b, FinallyNode *f)
+      : block(b), _catch(0), _final(f) {}
+    TryNode(StatementNode *b, CatchNode *c, FinallyNode *f)
+      : block(b), _catch(c), _final(f) {}
     virtual void ref();
     virtual bool deref();
-    virtual ~TryNode();
     virtual Completion execute(ExecState *exec);
     virtual void processVarDecls(ExecState *exec);
     virtual void streamTo(SourceStream &s) const;
@@ -984,109 +997,85 @@ namespace KJS {
 
   class ParameterNode : public Node {
   public:
-    ParameterNode(const UString *i) : id(*i), next(0L) { }
-    ParameterNode *append(const UString *i);
+    // list is circular during construction. cracked in FuncDecl/ExprNode ctor.
+    ParameterNode(const Identifier &i) : id(i), next(this) { }
+    ParameterNode(ParameterNode *list, const Identifier &i)
+      : id(i), next(list->next) { list->next = this; }
     virtual void ref();
     virtual bool deref();
-    virtual ~ParameterNode();
-    Value value(ExecState *exec) const;
-    UString ident() const { return id; }
+    virtual Value evaluate(ExecState *exec) const;
+    Identifier ident() const { return id; }
     ParameterNode *nextParam() const { return next; }
     virtual void streamTo(SourceStream &s) const;
   private:
-    UString id;
+    friend class FuncDeclNode;
+    friend class FuncExprNode;
+    Identifier id;
     ParameterNode *next;
   };
 
-  // inherited by ProgramNode
-  class FunctionBodyNode : public StatementNode {
+  class FunctionBodyNode : public BlockNode {
   public:
     FunctionBodyNode(SourceElementsNode *s);
-    virtual void ref();
-    virtual bool deref();
-    virtual ~FunctionBodyNode();
-    Completion execute(ExecState *exec);
     virtual void processFuncDecl(ExecState *exec);
-    virtual void processVarDecls(ExecState *exec);
-    void streamTo(SourceStream &s) const;
-  protected:
-    SourceElementsNode *source;
+    virtual Completion execute(ExecState *exec);
+    void setProgram(bool _program) { program = _program; }
+    bool isProgram() const { return program; }
+  private:
+    bool program;
   };
 
   class FuncDeclNode : public StatementNode {
   public:
-    FuncDeclNode(const UString *i, ParameterNode *p, FunctionBodyNode *b)
-      : ident(*i), param(p), body(b) { }
+    FuncDeclNode(const Identifier &i, FunctionBodyNode *b)
+      : ident(i), param(0), body(b) { }
+    FuncDeclNode(const Identifier &i, ParameterNode *p, FunctionBodyNode *b)
+      : ident(i), param(p->next), body(b) { p->next = 0; }
     virtual void ref();
     virtual bool deref();
-    virtual ~FuncDeclNode();
-    Completion execute(ExecState */*exec*/)
+    Completion execute(ExecState* /*exec*/)
       { /* empty */ return Completion(); }
     void processFuncDecl(ExecState *exec);
     virtual void streamTo(SourceStream &s) const;
   private:
-    UString ident;
+    Identifier ident;
     ParameterNode *param;
     FunctionBodyNode *body;
   };
 
   class FuncExprNode : public Node {
   public:
+    FuncExprNode(FunctionBodyNode *b)
+      : param(0), body(b) { }
     FuncExprNode(ParameterNode *p, FunctionBodyNode *b)
-	: param(p), body(b) { }
+      : param(p->next), body(b) { p->next = 0; }
     virtual void ref();
     virtual bool deref();
-    virtual ~FuncExprNode();
-    Value value(ExecState *exec) const;
+    virtual Value evaluate(ExecState *exec) const;
     virtual void streamTo(SourceStream &s) const;
   private:
     ParameterNode *param;
     FunctionBodyNode *body;
   };
 
-  class SourceElementNode : public StatementNode {
-  public:
-    SourceElementNode(StatementNode *s) : statement(s), function(0L) { }
-    SourceElementNode(FuncDeclNode *f) : statement(0L), function(f) { }
-    virtual void ref();
-    virtual bool deref();
-    virtual ~SourceElementNode();
-    Completion execute(ExecState *exec);
-    virtual void processFuncDecl(ExecState *exec);
-    virtual void processVarDecls(ExecState *exec);
-    virtual void streamTo(SourceStream &s) const;
-  private:
-    StatementNode *statement;
-    FuncDeclNode *function;
-  };
-
   // A linked list of source element nodes
   class SourceElementsNode : public StatementNode {
   public:
-    SourceElementsNode(SourceElementNode *s1) { element = s1; elements = 0L; }
-    SourceElementsNode(SourceElementsNode *s1, SourceElementNode *s2)
-      { elements = s1; element = s2; }
+    // list is circular until cracked in BlockNode (or subclass) ctor
+    SourceElementsNode(StatementNode *s1);
+    SourceElementsNode(SourceElementsNode *s1, StatementNode *s2);
     virtual void ref();
     virtual bool deref();
-    virtual ~SourceElementsNode();
     Completion execute(ExecState *exec);
     virtual void processFuncDecl(ExecState *exec);
     virtual void processVarDecls(ExecState *exec);
     virtual void streamTo(SourceStream &s) const;
   private:
-    SourceElementNode *element; // 'this' element
+    friend class BlockNode;
+    StatementNode *element; // 'this' element
     SourceElementsNode *elements; // pointer to next
   };
 
-  class ProgramNode : public FunctionBodyNode {
-  public:
-    ProgramNode(SourceElementsNode *s);
-    ~ProgramNode();
-  private:
-    // Disallow copy
-    ProgramNode(const ProgramNode &other);
-  };
-
 } // namespace
 
 #endif
