@@ -3,18 +3,19 @@
  *  This file is part of the KDE libraries
  *  Copyright (C) 1999-2001 Harri Porten (porten@kde.org)
  *  Copyright (C) 2001 Peter Kelly (pmk@post.com)
+ *  Copyright (C) 2003 Apple Computer, Inc.
  *
  *  This library is free software; you can redistribute it and/or
- *  modify it under the terms of the GNU Lesser General Public
+ *  modify it under the terms of the GNU Library General Public
  *  License as published by the Free Software Foundation; either
  *  version 2 of the License, or (at your option) any later version.
  *
  *  This library is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- *  Lesser General Public License for more details.
+ *  Library General Public License for more details.
  *
- *  You should have received a copy of the GNU Lesser General Public License
+ *  You should have received a copy of the GNU Library General Public License
  *  along with this library; see the file COPYING.LIB.  If not, write to
  *  the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  *  Boston, MA 02111-1307, USA.
@@ -29,13 +30,17 @@
 
 #include "value.h"
 #include "types.h"
+#include "reference_list.h"
+#include "identifier.h"
+#include "property_map.h"
+#include "scope_chain.h"
 
 namespace KJS {
 
   class ObjectImpPrivate;
   class PropertyMap;
   class HashTable;
-  class HashEntry;
+  struct HashEntry;
   class ListImp;
 
   // ECMA 262-3 8.6.1
@@ -75,21 +80,19 @@ namespace KJS {
    */
   class Object : public Value {
   public:
-    Object();
+    Object() { }
     explicit Object(ObjectImp *v);
-    Object(const Object &v);
-    virtual ~Object();		// TODO: make non-virtual
 
-    Object& operator=(const Object &v);
+    ObjectImp *imp() const;
 
-    virtual const ClassInfo *classInfo() const; // TODO: make non-virtual
+    const ClassInfo *classInfo() const;
     bool inherits(const ClassInfo *cinfo) const;
 
     /**
      * Converts a Value into an Object. If the value's type is not ObjectType,
      * a null object will be returned (i.e. one with it's internal pointer set
      * to 0). If you do not know for sure whether the value is of type
-     * ObjectType, you should check the @ref isNull() methods afterwards before
+     * ObjectType, you should check the isValid() methods afterwards before
      * calling any methods on the Object.
      *
      * @return The value converted to an object
@@ -127,7 +130,8 @@ namespace KJS {
      *
      * @return The specified property, or Undefined
      */
-    Value get(ExecState *exec, const UString &propertyName) const;
+    Value get(ExecState *exec, const Identifier &propertyName) const;
+    Value get(ExecState *exec, unsigned propertyName) const;
 
     /**
      * Sets the specified property.
@@ -136,10 +140,12 @@ namespace KJS {
      *
      * @param exec The current execution state
      * @param propertyName The name of the property to set
-     * @param propertyValue The value to set
+     * @param value The value to set
      */
-    void put(ExecState *exec, const UString &propertyName,
-             const Value &value, int attr = None);
+    void put(ExecState *exec, const Identifier &propertyName,
+	     const Value &value, int attr = None);
+    void put(ExecState *exec, unsigned propertyName,
+	     const Value &value, int attr = None);
 
     /**
      * Used to check whether or not a particular property is allowed to be set
@@ -151,7 +157,7 @@ namespace KJS {
      * @param propertyName The name of the property
      * @return true if the property can be set, otherwise false
      */
-    bool canPut(ExecState *exec, const UString &propertyName) const;
+    bool canPut(ExecState *exec, const Identifier &propertyName) const;
 
     /**
      * Checks to see whether the object (or any object in it's prototype chain)
@@ -163,7 +169,8 @@ namespace KJS {
      * @param propertyName The name of the property to check for
      * @return true if the object has the property, otherwise false
      */
-    bool hasProperty(ExecState *exec, const UString &propertyName) const;
+    bool hasProperty(ExecState *exec, const Identifier &propertyName) const;
+    bool hasProperty(ExecState *exec, unsigned propertyName) const;
 
     /**
      * Removes the specified property from the object.
@@ -176,7 +183,8 @@ namespace KJS {
      * exist on the object. false if deleting the specified property is not
      * allowed.
      */
-    bool deleteProperty(ExecState *exec, const UString &propertyName);
+    bool deleteProperty(ExecState *exec, const Identifier &propertyName);
+    bool deleteProperty(ExecState *exec, unsigned propertyName);
 
     /**
      * Converts the object into a primitive value. The value return may differ
@@ -216,7 +224,7 @@ namespace KJS {
      * this is discouraged.
      *
      * If an error occurs during construction, the execution state's exception
-     * will be set. This can be tested for with @ref ExecState::hadException().
+     * will be set. This can be tested for with ExecState::hadException().
      * Under some circumstances, the exception object may also be returned.
      *
      * Note: This function should not be called if implementsConstruct() returns
@@ -270,12 +278,12 @@ namespace KJS {
     bool implementsHasInstance() const;
 
     /**
-     * Checks whether value delegates behaviour to this object. Used by the
+     * Checks whether value delegates behavior to this object. Used by the
      * instanceof operator.
      *
      * @param exec The current execution state
      * @param value The value to check
-     * @return true if value delegates behaviour to this object, otherwise
+     * @return true if value delegates behavior to this object, otherwise
      * false
      */
     Boolean hasInstance(ExecState *exec, const Value &value);
@@ -286,6 +294,7 @@ namespace KJS {
      * extra object in it's scope. An example of this is functions declared
      * inside other functions:
      *
+     * \code
      * function f() {
      *
      *   function b() {
@@ -296,16 +305,16 @@ namespace KJS {
      *   // do some stuff
      * }
      * f.prototype = new String();
+     * \endcode
      *
      * When the function f.b is executed, its scope will include properties of
      * f. So in the example above the return value of f.b() would be the new
      * String object that was assigned to f.prototype.
      *
-     * @param exec The current execution state
      * @return The function's scope
      */
-    const List scope() const;
-    void setScope(const List &s);
+    const ScopeChain &scope() const;
+    void setScope(const ScopeChain &s);
 
     /**
      * Returns a List of References to all the properties of the object. Used
@@ -323,7 +332,7 @@ namespace KJS {
      * included in the list.
      * @return A List of References to properties of the object.
      **/
-    List propList(ExecState *exec, bool recursive = true);
+    ReferenceList propList(ExecState *exec, bool recursive = true);
 
     /**
      * Returns the internal value of the object. This is used for objects such
@@ -345,7 +354,10 @@ namespace KJS {
     void setInternalValue(const Value &v);
   };
 
+  inline Object Value::toObject(ExecState *exec) const { return rep->dispatchToObject(exec); }
+
   class ObjectImp : public ValueImp {
+    friend class ObjectProtoFuncImp;
   public:
     /**
      * Creates a new ObjectImp with the specified prototype
@@ -353,6 +365,7 @@ namespace KJS {
      * @param proto The prototype
      */
     ObjectImp(const Object &proto);
+    ObjectImp(ObjectImp *proto);
 
     /**
      * Creates a new ObjectImp with a prototype of Null()
@@ -370,7 +383,7 @@ namespace KJS {
     /**
      * A pointer to a ClassInfo struct for this class. This provides a basic
      * facility for run-time type information, and can be used to check an
-     * object's class an inheritance (see @ref inherits()). This should
+     * object's class an inheritance (see inherits()). This should
      * always return a statically declared pointer, or 0 to indicate that
      * there is no class information.
      *
@@ -381,6 +394,7 @@ namespace KJS {
      * where FooImp inherits from BarImp, you would add the following in your
      * class declarations:
      *
+     * \code
      *   class BarImp : public ObjectImp {
      *     virtual const ClassInfo *classInfo() const { return &info; }
      *     static const ClassInfo info;
@@ -392,11 +406,14 @@ namespace KJS {
      *     static const ClassInfo info;
      *     // ...
      *   };
+     * \endcode
      *
      * And in your source file:
      *
+     * \code
      *   const ClassInfo BarImp::info = {0, 0, 0}; // no parent class
      *   const ClassInfo FooImp::info = {&BarImp::info, 0, 0};
+     * \endcode
      *
      * @see inherits()
      */
@@ -416,7 +433,7 @@ namespace KJS {
      *   }
      *
      * If you have a handle to a statically declared ClassInfo, such as in the
-     * @ref classInfo() example, you can check for inheritance without needing
+     * classInfo() example, you can check for inheritance without needing
      * an instance of the other class:
      *
      *   if (obj1->inherits(FooImp::info)) {
@@ -446,8 +463,8 @@ namespace KJS {
      * Objects)
      *
      * The default implementation uses classInfo().
-     * You should either implement @ref classInfo(), or
-     * if you simply need a classname, you can reimplement @ref className()
+     * You should either implement classInfo(), or
+     * if you simply need a classname, you can reimplement className()
      * instead.
      *
      * @see Object::className()
@@ -461,7 +478,9 @@ namespace KJS {
      * @see Object::get()
      */
     // [[Get]] - must be implemented by all Objects
-    virtual Value get(ExecState *exec, const UString &propertyName) const;
+    virtual Value get(ExecState *exec, const Identifier &propertyName) const;
+    virtual Value getPropertyByIndex(ExecState *exec,
+				     unsigned propertyName) const;
 
     /**
      * Implementation of the [[Put]] internal property (implemented by all
@@ -469,8 +488,10 @@ namespace KJS {
      *
      * @see Object::put()
      */
-    virtual void put(ExecState *exec, const UString &propertyName,
-                     const Value &value, int attr = None);
+    virtual void put(ExecState *exec, const Identifier &propertyName,
+		     const Value &value, int attr = None);
+    virtual void putPropertyByIndex(ExecState *exec, unsigned propertyName,
+				    const Value &value, int attr = None);
 
     /**
      * Implementation of the [[CanPut]] internal property (implemented by all
@@ -478,7 +499,7 @@ namespace KJS {
      *
      * @see Object::canPut()
      */
-    virtual bool canPut(ExecState *exec, const UString &propertyName) const;
+    virtual bool canPut(ExecState *exec, const Identifier &propertyName) const;
 
     /**
      * Implementation of the [[HasProperty]] internal property (implemented by
@@ -487,7 +508,8 @@ namespace KJS {
      * @see Object::hasProperty()
      */
     virtual bool hasProperty(ExecState *exec,
-			     const UString &propertyName) const;
+			     const Identifier &propertyName) const;
+    virtual bool hasPropertyByIndex(ExecState *exec, unsigned propertyName) const;
 
     /**
      * Implementation of the [[Delete]] internal property (implemented by all
@@ -496,14 +518,15 @@ namespace KJS {
      * @see Object::deleteProperty()
      */
     virtual bool deleteProperty(ExecState *exec,
-                                const UString &propertyName);
+				const Identifier &propertyName);
+    virtual bool deletePropertyByIndex(ExecState *exec, unsigned propertyName);
 
     /**
      * Remove all properties from this object.
      * This doesn't take DontDelete into account, and isn't in the ECMA spec.
      * It's simply a quick way to remove everything before destroying.
      */
-    void deleteAllProperties( ExecState * );
+    void deleteAllProperties(ExecState *);
 
     /**
      * Implementation of the [[DefaultValue]] internal property (implemented by
@@ -543,33 +566,45 @@ namespace KJS {
      *
      * @see Object::scope()
      */
-    const List scope() const;
-    void setScope(const List &s);
+    const ScopeChain &scope() const { return _scope; }
+    void setScope(const ScopeChain &s) { _scope = s; }
 
-    List propList(ExecState *exec, bool recursive = true);
+    virtual ReferenceList propList(ExecState *exec, bool recursive = true);
 
     Value internalValue() const;
     void setInternalValue(const Value &v);
+    void setInternalValue(ValueImp *v);
 
     Value toPrimitive(ExecState *exec,
                       Type preferredType = UnspecifiedType) const;
     bool toBoolean(ExecState *exec) const;
     double toNumber(ExecState *exec) const;
-    int toInteger(ExecState *exec) const;
-    int toInt32(ExecState *exec) const;
-    unsigned int toUInt32(ExecState *exec) const;
-    unsigned short toUInt16(ExecState *exec) const;
     UString toString(ExecState *exec) const;
     Object toObject(ExecState *exec) const;
 
-    ValueImp* getDirect(const UString& propertyName) const;
+    // This get method only looks at the property map.
+    // A bit like hasProperty(recursive=false), this doesn't go to the prototype.
+    // This is used e.g. by lookupOrCreateFunction (to cache a function, we don't want
+    // to look up in the prototype, it might already exist there)
+    ValueImp *getDirect(const Identifier& propertyName) const
+        { return _prop.get(propertyName); }
+    void putDirect(const Identifier &propertyName, ValueImp *value, int attr = 0);
+    void putDirect(const Identifier &propertyName, int value, int attr = 0);
+
+    /**
+     * Sets the name of the function, if this is an InternalFunctionImp object.
+     * (calling InternalFunctionImp::setName)
+     */
+    void setFunctionName(const Identifier &propertyName);
+
+  protected:
+    PropertyMap _prop;
   private:
-    const HashEntry* findPropertyHashEntry( const UString& propertyName ) const;
+    const HashEntry* findPropertyHashEntry( const Identifier& propertyName ) const;
     ObjectImpPrivate *_od;
-    PropertyMap *_prop;
     ValueImp *_proto;
     ValueImp *_internalValue;
-    ListImp *_scope;
+    ScopeChain _scope;
   };
 
   /**
@@ -603,11 +638,87 @@ namespace KJS {
                          int sourceId = -1);
 
     /**
-     * Array of error names corresponding to @ref #ErrorType
+     * Array of error names corresponding to ErrorType
      */
     static const char * const * const errorNames;
   };
 
+  inline Object::Object(ObjectImp *v) : Value(v) { }
+
+  inline ObjectImp *Object::imp() const { return static_cast<ObjectImp*>(rep); }
+
+  inline const ClassInfo *Object::classInfo() const
+    { return imp()->classInfo(); }
+
+  inline bool Object::inherits(const ClassInfo *cinfo) const
+    { return imp()->inherits(cinfo); }
+
+  inline Value Object::prototype() const
+    { return Value(imp()->prototype()); }
+
+  inline UString Object::className() const
+    { return imp()->className(); }
+
+  inline Value Object::get(ExecState *exec, const Identifier &propertyName) const
+    { return imp()->get(exec,propertyName); }
+
+  inline Value Object::get(ExecState *exec, unsigned propertyName) const
+    { return imp()->getPropertyByIndex(exec, propertyName); }
+
+  inline void Object::put(ExecState *exec, const Identifier &propertyName, const Value &value, int attr)
+    { imp()->put(exec,propertyName,value,attr); }
+
+  inline void Object::put(ExecState *exec, unsigned propertyName, const Value &value, int attr)
+    { imp()->putPropertyByIndex(exec, propertyName, value, attr); }
+
+  inline bool Object::canPut(ExecState *exec, const Identifier &propertyName) const
+    { return imp()->canPut(exec,propertyName); }
+
+  inline bool Object::hasProperty(ExecState *exec, const Identifier &propertyName) const
+    { return imp()->hasProperty(exec, propertyName); }
+
+  inline bool Object::hasProperty(ExecState *exec, unsigned propertyName) const
+    { return imp()->hasPropertyByIndex(exec, propertyName); }
+
+  inline bool Object::deleteProperty(ExecState *exec, const Identifier &propertyName)
+    { return imp()->deleteProperty(exec,propertyName); }
+
+  inline bool Object::deleteProperty(ExecState *exec, unsigned propertyName)
+    { return imp()->deletePropertyByIndex(exec, propertyName); }
+
+  inline Value Object::defaultValue(ExecState *exec, Type hint) const
+    { return imp()->defaultValue(exec,hint); }
+
+  inline bool Object::implementsConstruct() const
+    { return imp()->implementsConstruct(); }
+
+  inline Object Object::construct(ExecState *exec, const List &args)
+    { return imp()->construct(exec,args); }
+
+  inline bool Object::implementsCall() const
+    { return imp()->implementsCall(); }
+
+  inline bool Object::implementsHasInstance() const
+    { return imp()->implementsHasInstance(); }
+
+  inline Boolean Object::hasInstance(ExecState *exec, const Value &value)
+    { return imp()->hasInstance(exec,value); }
+
+  inline const ScopeChain &Object::scope() const
+    { return imp()->scope(); }
+
+  inline void Object::setScope(const ScopeChain &s)
+    { imp()->setScope(s); }
+
+  inline ReferenceList Object::propList(ExecState *exec, bool recursive)
+    { return imp()->propList(exec,recursive); }
+
+  inline Value Object::internalValue() const
+    { return imp()->internalValue(); }
+
+  inline void Object::setInternalValue(const Value &v)
+    { imp()->setInternalValue(v); }
+
 } // namespace
 
 #endif // _KJS_OBJECT_H_
