@@ -119,9 +119,7 @@ UString RegExp::match(const UString &s, int i, int *pos, int **ovector)
     {
       // We set m_notEmpty ourselves, to look for a non-empty match
       // (see man pcretest or pcretest.c for details).
-      // So this is not the end. We want to try again at i+1.
-      // We won't be at the end of the string - that was checked before setting m_notEmpty.
-      fprintf(stderr, "No match after m_notEmpty. +1 and keep going.\n");
+      // So we don't stop here, we want to try again at i+1.
       m_notEmpty = 0;
       if (pcre_exec(pcregex, NULL, buffer.c_str(), bufferSize, i+1, 0,
                     ovector ? *ovector : 0L, ovecsize) == PCRE_ERROR_NOMATCH)
@@ -131,10 +129,12 @@ UString RegExp::match(const UString &s, int i, int *pos, int **ovector)
       return UString::null;
   }
 
+  // Got a match, proceed with it.
+
   if (!ovector)
     return UString::null; // don't rely on the return value if you pass ovector==0
 #else
-  const int maxMatch = 10;
+  const uint maxMatch = 10;
   regmatch_t rmatch[maxMatch];
 
   char *str = strdup(s.ascii()); // TODO: why ???
@@ -151,11 +151,11 @@ UString RegExp::match(const UString &s, int i, int *pos, int **ovector)
 
   // map rmatch array to ovector used in PCRE case
   nrSubPatterns = 0;
-  for(int j = 1; j < maxMatch && rmatch[j].rm_so >= 0 ; j++)
+  for(uint j = 1; j < maxMatch && rmatch[j].rm_so >= 0 ; j++)
       nrSubPatterns++;
   int ovecsize = (nrSubPatterns+1)*3; // see above
   *ovector = new int[ovecsize];
-  for (int j = 0; j < nrSubPatterns + 1; j++) {
+  for (uint j = 0; j < nrSubPatterns + 1; j++) {
     if (j>maxMatch)
       break;
     (*ovector)[2*j] = rmatch[j].rm_so + i;
@@ -165,10 +165,9 @@ UString RegExp::match(const UString &s, int i, int *pos, int **ovector)
 
   *pos = (*ovector)[0];
 #ifdef HAVE_PCREPOSIX  // TODO check this stuff in non-pcre mode
-  if ( *pos == (*ovector)[1] && (flgs & Global) && *pos != bufferSize )
+  if ( *pos == (*ovector)[1] && (flgs & Global) )
   {
-    // empty match, not at end of string.
-    // Next try will be with m_notEmpty=true
+    // empty match, next try will be with m_notEmpty=true
     m_notEmpty=true;
   }
 #endif
