@@ -17,7 +17,6 @@
  *  License along with this library; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
- *  $Id: string_object.h,v 1.18.2.1 2003/05/17 11:19:25 mueller Exp $
  */
 
 #ifndef _STRING_OBJECT_H_
@@ -30,11 +29,18 @@ namespace KJS {
 
   class StringInstanceImp : public ObjectImp {
   public:
-    StringInstanceImp(const Object &proto);
+    StringInstanceImp(ObjectImp *proto);
+    StringInstanceImp(ObjectImp *proto, const UString &string);
+
+    virtual Value get(ExecState *exec, const Identifier &propertyName) const;
+    virtual void put(ExecState *exec, const Identifier &propertyName, const Value &value, int attr = None);
+    virtual bool hasProperty(ExecState *exec, const Identifier &propertyName) const;
+    virtual bool deleteProperty(ExecState *exec, const Identifier &propertyName);
+    virtual ReferenceList propList(ExecState *exec, bool recursive);
 
     virtual const ClassInfo *classInfo() const { return &info; }
     static const ClassInfo info;
-  };
+ };
 
   /**
    * @internal
@@ -46,7 +52,7 @@ namespace KJS {
   public:
     StringPrototypeImp(ExecState *exec,
                        ObjectPrototypeImp *objProto);
-    Value get(ExecState *exec, const UString &p) const;
+    Value get(ExecState *exec, const Identifier &p) const;
     virtual const ClassInfo *classInfo() const { return &info; }
     static const ClassInfo info;
   };
