@@ -2,23 +2,23 @@
 /*
  *  This file is part of the KDE libraries
  *  Copyright (C) 1999-2000 Harri Porten (porten@kde.org)
+ *  Copyright (C) 2003 Apple Computer, Inc.
  *
  *  This library is free software; you can redistribute it and/or
- *  modify it under the terms of the GNU Lesser General Public
+ *  modify it under the terms of the GNU Library General Public
  *  License as published by the Free Software Foundation; either
  *  version 2 of the License, or (at your option) any later version.
  *
  *  This library is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- *  Lesser General Public License for more details.
+ *  Library General Public License for more details.
  *
- *  You should have received a copy of the GNU Lesser General Public License
+ *  You should have received a copy of the GNU Library General Public License
  *  along with this library; see the file COPYING.LIB.  If not, write to
  *  the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  *  Boston, MA 02111-1307, USA.
  *
- *  $Id: ustring.h,v 1.46 2002/10/25 18:47:33 porten Exp $
  */
 
 #ifndef _KJS_USTRING_H_
@@ -31,7 +31,7 @@
  */
 namespace DOM {
   class DOMString;
-};
+}
 class KJScript;
 class QString;
 class QConstString;
@@ -50,9 +50,11 @@ namespace KJS {
    */
   struct UChar {
     /**
-     * Construct a character with value 0.
+     * Construct a character with uninitialized value.
      */
     UChar();
+    UChar(char u);
+    UChar(unsigned char u);
     /**
      * Construct a character with the value denoted by the arguments.
      * @param h higher byte
@@ -72,7 +74,7 @@ namespace KJS {
     /**
      * @return The lower byte of the character.
      */
-    unsigned char low() const { return uc & 0xFF; }
+    unsigned char low() const { return uc; }
     /**
      * @return the 16 bit Unicode value of the character
      */
@@ -90,33 +92,29 @@ namespace KJS {
      * A static instance of UChar(0).
      */
     static UChar null;
-  private:
-    friend class UCharReference;
-    friend class UString;
-    friend bool operator==(const UChar &c1, const UChar &c2);
-    friend bool operator==(const UString& s1, const char *s2);
-    friend bool operator<(const UString& s1, const UString& s2);
 
     unsigned short uc;
   } KJS_PACKED;
 
-  inline UChar::UChar() : uc(0) { }
+  inline UChar::UChar() { }
   inline UChar::UChar(unsigned char h , unsigned char l) : uc(h << 8 | l) { }
+  inline UChar::UChar(char u) : uc((unsigned char)u) { }
+  inline UChar::UChar(unsigned char u) : uc(u) { }
   inline UChar::UChar(unsigned short u) : uc(u) { }
 
   /**
    * @short Dynamic reference to a string character.
    *
-   * UCharReference is the dynamic counterpart of @ref UChar. It's used when
-   * characters retrieved via index from a @ref UString are used in an
+   * UCharReference is the dynamic counterpart of UChar. It's used when
+   * characters retrieved via index from a UString are used in an
    * assignment expression (and therefore can't be treated as being const):
-   * <pre>
+   * \code
    * UString s("hello world");
    * s[0] = 'H';
-   * </pre>
+   * \endcode
    *
    * If that sounds confusing your best bet is to simply forget about the
-   * existance of this class and treat is as being identical to @ref UChar.
+   * existence of this class and treat is as being identical to UChar.
    */
   class UCharReference {
     friend class UString;
@@ -133,11 +131,11 @@ namespace KJS {
     /**
      * @return Unicode value.
      */
-    unsigned short unicode() const { return ref().unicode(); }
+    unsigned short unicode() const { return ref().uc; }
     /**
      * @return Lower byte.
      */
-    unsigned char low() const { return ref().uc & 0xFF; }
+    unsigned char low() const { return ref().uc; }
     /**
      * @return Higher byte.
      */
@@ -159,13 +157,16 @@ namespace KJS {
     int offset;
   };
 
+  inline UChar::UChar(const UCharReference &c) : uc(c.unicode()) { }
+
   /**
    * @short 8 bit char based string class
    */
   class CString {
   public:
-    CString() : data(0L) { }
+    CString() : data(0L), length(0) { }
     CString(const char *c);
+    CString(const char *c, int len);
     CString(const CString &);
 
     ~CString();
@@ -173,12 +174,13 @@ namespace KJS {
     CString &append(const CString &);
     CString &operator=(const char *c);
     CString &operator=(const CString &);
-    CString &operator+=(const CString &);
+    CString &operator+=(const CString &c) { return append(c); }
 
-    int size() const;
+    int size() const { return length; }
     const char *c_str() const { return data; }
   private:
     char *data;
+    int length;
   };
 
   /**
@@ -187,23 +189,40 @@ namespace KJS {
   class UString {
     friend bool operator==(const UString&, const UString&);
     friend class UCharReference;
+    friend class Identifier;
+    friend class PropertyMap;
+    friend class PropertyMapHashTableEntry;
     /**
      * @internal
      */
     struct Rep {
       friend class UString;
       friend bool operator==(const UString&, const UString&);
-      static Rep *create(UChar *d, int l);
-      inline UChar *data() const { return dat; }
-      inline int size() const { return len; }
 
-      inline void ref() { rc++; }
-      inline int deref() { return --rc; }
+      static Rep *create(UChar *d, int l);
+      void destroy();
+
+      UChar *data() const { return dat; }
+      int size() const { return len; }
+
+      int hash() const { if (_hash == 0) _hash = computeHash(dat, len); return _hash; }
+
+      static unsigned computeHash(const UChar *, int length);
+      static unsigned computeHash(const char *);
+
+      void ref() { ++rc; }
+      void deref() { if (--rc == 0) destroy(); }
 
       UChar *dat;
       int len;
+      int capacity;
       int rc;
+      mutable int _hash;
+
+      enum { capacityForIdentifier = 0x10000000 };
+
       static Rep null;
+      static Rep empty;
     };
 
   public:
@@ -214,7 +233,7 @@ namespace KJS {
     /**
      * Constructs a string from the single character c.
      */
-    UString(char c);
+    explicit UString(char c);
     /**
      * Constructs a string from a classical zero determined char string.
      */
@@ -225,33 +244,37 @@ namespace KJS {
      */
     UString(const UChar *c, int length);
     /**
-     * If copy is false a shallow copy of the string will be created. That
-     * means that the data will NOT be copied and you'll have to guarantee that
-     * it doesn't get deleted during the lifetime of the UString object.
+     * If copy is false the string data will be adopted.
+     * That means that the data will NOT be copied and the pointer will
+     * be deleted when the UString object is modified or destroyed.
      * Behaviour defaults to a deep copy if copy is true.
      */
     UString(UChar *c, int length, bool copy);
     /**
      * Copy constructor. Makes a shallow copy only.
      */
-    UString(const UString &);
+    UString(const UString &s) { attach(s.rep); }
     /**
      * Convenience declaration only ! You'll be on your own to write the
      * implementation for a construction from QString.
      *
      * Note: feel free to contact me if you want to see a dummy header for
-     * your favourite FooString class here !
+     * your favorite FooString class here !
      */
     UString(const QString &);
     /**
-     * Convenience declaration only ! See @ref UString(const QString&).
+     * Convenience declaration only ! See UString(const QString&).
      */
     UString(const DOM::DOMString &);
+    /**
+     * Concatenation constructor. Makes operator+ more efficient.
+     */
+    UString(const UString &, const UString &);
     /**
      * Destructor. If this handle was the only one holding a reference to the
      * string the data will be freed.
      */
-    ~UString();
+    ~UString() { release(); }
 
     /**
      * Constructs a string from an int.
@@ -261,6 +284,10 @@ namespace KJS {
      * Constructs a string from an unsigned int.
      */
     static UString from(unsigned int u);
+    /**
+     * Constructs a string from a long.
+     */
+    static UString from(long l);
     /**
      * Constructs a string from a double.
      */
@@ -272,7 +299,7 @@ namespace KJS {
     UString &append(const UString &);
 
     /**
-     * @return The string converted to the 8-bit string type @ref CString().
+     * @return The string converted to the 8-bit string type CString().
      */
     CString cstring() const;
     /**
@@ -300,14 +327,11 @@ namespace KJS {
      * Assignment operator.
      */
     UString &operator=(const char *c);
-    /**
-     * Assignment operator.
-     */
     UString &operator=(const UString &);
     /**
      * Appends the specified string.
      */
-    UString &operator+=(const UString &s);
+    UString &operator+=(const UString &s) { return append(s); }
 
     /**
      * @return A pointer to the internal Unicode data.
@@ -324,7 +348,7 @@ namespace KJS {
     /**
      * Use this if you want to make sure that this string is a plain ASCII
      * string. For example, if you don't want to lose any information when
-     * using @ref cstring() or @ref ascii().
+     * using cstring() or ascii().
      *
      * @return True if the string doesn't contain any non-ASCII characters.
      */
@@ -347,14 +371,30 @@ namespace KJS {
      * the algorithm will recognize hexadecimal representations (as
      * indicated by a 0x or 0X prefix) and +/- Infinity.
      * Returns NaN if the conversion failed.
-     * @param tolerant if true, toDouble can tolerate garbage after the number.
+     * @param tolerateTrailingJunk if true, toDouble can tolerate garbage after the number.
+     * @param tolerateEmptyString if false, toDouble will turn an empty string into NaN rather than 0.
      */
-    double toDouble(bool tolerant=false) const;
+    double toDouble(bool tolerateTrailingJunk, bool tolerateEmptyString) const;
+    double toDouble(bool tolerateTrailingJunk) const;
+    double toDouble() const;
     /**
      * Attempts an conversion to an unsigned long integer. ok will be set
      * according to the success.
+     * @param tolerateEmptyString if false, toULong will return false for *ok for an empty string.
      */
-    unsigned long toULong(bool *ok = 0L) const;
+    unsigned long toULong(bool *ok, bool tolerateEmptyString) const;
+    unsigned long toULong(bool *ok = 0) const;
+
+    unsigned int toStrictUInt32(bool *ok = 0) const;
+
+    /**
+     * Attempts an conversion to an array index. The "ok" boolean will be set
+     * to true if it is a valid array index according to the rule from
+     * ECMA 15.2 about what an array index is. It must exactly match the string
+     * form of an unsigned integer, and be less than 2^32 - 1.
+     */
+    unsigned toArrayIndex(bool *ok = 0) const;
+
     /**
      * Returns this string converted to lower case characters
      */
@@ -364,16 +404,18 @@ namespace KJS {
      */
     UString toUpper() const;
     /**
-     * @return Position of first occurence of f starting at position pos.
+     * @return Position of first occurrence of f starting at position pos.
      * -1 if the search was not successful.
      */
     int find(const UString &f, int pos = 0) const;
+    int find(UChar, int pos = 0) const;
     /**
-     * @return Position of first occurence of f searching backwards from
+     * @return Position of first occurrence of f searching backwards from
      * position pos.
      * -1 if the search was not successful.
      */
     int rfind(const UString &f, int pos) const;
+    int rfind(UChar, int pos) const;
     /**
      * @return The sub string starting at position pos and length len.
      */
@@ -389,6 +431,7 @@ namespace KJS {
     static void globalClear();
 #endif
   private:
+    UString(Rep *r) { attach(r); }
     void attach(Rep *r);
     void detach();
     void release();
@@ -398,6 +441,9 @@ namespace KJS {
   inline bool operator==(const UChar &c1, const UChar &c2) {
     return (c1.uc == c2.uc);
   }
+  inline bool operator!=(const UChar& c1, const UChar& c2) {
+    return !KJS::operator==(c1, c2);
+  }
   bool operator==(const UString& s1, const UString& s2);
   inline bool operator!=(const UString& s1, const UString& s2) {
     return !KJS::operator==(s1, s2);
@@ -414,8 +460,15 @@ namespace KJS {
     return !KJS::operator==(s1, s2);
   }
   bool operator==(const CString& s1, const CString& s2);
-  UString operator+(const UString& s1, const UString& s2);
+  inline bool operator!=(const CString& s1, const CString& s2) {
+    return !KJS::operator==(s1, s2);
+  }
+  inline UString operator+(const UString& s1, const UString& s2) {
+    return UString(s1, s2);
+  }
 
-}; // namespace
+  int compare(const UString &, const UString &);
+
+} // namespace
 
 #endif
