@@ -19,12 +19,19 @@
 */
 #include "browserextension.h"
 
+#include <qapplication.h>
+#include <qclipboard.h>
 #include <qtimer.h>
 #include <qobjectlist.h>
 #include <qmetaobject.h>
 #include <qstrlist.h>
+#include <qstylesheet.h>
+
 #include <kdebug.h>
+#include <klocale.h>
+#include <kmessagebox.h>
 #include <kstaticdeleter.h>
+#include <kurifilter.h>
 #include <assert.h>
 
 using namespace KParts;
@@ -122,7 +129,8 @@ URLArgs &URLArgs::operator=(const URLArgs &args)
 
 URLArgs::~URLArgs()
 {
-  delete d; d = 0;
+  delete d;
+  d = 0;
 }
 
 void URLArgs::setContentType( const QString & contentType )
@@ -192,6 +200,7 @@ bool URLArgs::newTab() const
     return d ? d->newTab : false;
 }
 
+
 namespace KParts
 {
 
@@ -319,9 +328,9 @@ public:
 }
 
 BrowserExtension::ActionSlotMap * BrowserExtension::s_actionSlotMap = 0L;
-KStaticDeleter<BrowserExtension::ActionSlotMap> actionSlotMapsd;
+static KStaticDeleter<BrowserExtension::ActionSlotMap> actionSlotMapsd;
 BrowserExtension::ActionNumberMap * BrowserExtension::s_actionNumberMap = 0L;
-KStaticDeleter<BrowserExtension::ActionNumberMap> actionNumberMapsd;
+static KStaticDeleter<BrowserExtension::ActionNumberMap> actionNumberMapsd;
 
 BrowserExtension::BrowserExtension( KParts::ReadOnlyPart *parent,
                                     const char *name )
@@ -416,6 +425,44 @@ void BrowserExtension::slotCompleted()
   setURLArgs( URLArgs() );
 }
 
+void BrowserExtension::pasteRequest()
+{
+    QCString plain("plain");
+    QString url = QApplication::clipboard()->text(plain, QClipboard::Selection).stripWhiteSpace();
+
+    // Check if it's a URL
+    QStringList filters = KURIFilter::self()->pluginNames();
+    filters.remove( "kuriikwsfilter" );
+    filters.remove( "localdomainurifilter" );
+    KURIFilterData filterData;
+    filterData.setData( url );
+    filterData.setCheckForExecutables( false );
+    if ( KURIFilter::self()->filterURI( filterData, filters ) )
+    {
+        switch ( filterData.uriType() )
+	{
+	    case KURIFilterData::LOCAL_FILE:
+	    case KURIFilterData::LOCAL_DIR:
+	    case KURIFilterData::NET_PROTOCOL:
+	        slotOpenURLRequest( filterData.uri(), KParts::URLArgs() );
+		break;
+	    case KURIFilterData::ERROR:
+		KMessageBox::sorry( m_part->widget(), filterData.errorMsg() );
+		break;
+	    default:
+		break;
+	}
+    }
+    else if ( KURIFilter::self()->filterURI( filterData, "kuriikwsfilter" ) && url.length() < 250 )
+    {
+        if ( KMessageBox::questionYesNo( m_part->widget(),
+		    i18n( "<qt>Do you want to search the Internet for <b>%1</b>?" ).arg( QStyleSheet::escape(url) ),
+		    i18n( "Internet Search" ), KGuiItem( i18n( "&Search" ), "find"),
+		    KStdGuiItem::cancel(), "MiddleClickSearch" ) == KMessageBox::Yes)
+          slotOpenURLRequest( filterData.uri(), KParts::URLArgs() );
+    }
+}
+
 void BrowserExtension::slotOpenURLRequest( const KURL &url, const KParts::URLArgs &args )
 {
     //kdDebug() << this << " BrowserExtension::slotOpenURLRequest(): url=" << url.url() << endl;
@@ -480,8 +527,7 @@ BrowserExtension::ActionSlotMap * BrowserExtension::actionSlotMapPtr()
 void BrowserExtension::createActionSlotMap()
 {
     assert(!s_actionSlotMap);
-    s_actionSlotMap = new ActionSlotMap;
-    actionSlotMapsd.setObject( s_actionSlotMap );
+    s_actionSlotMap = actionSlotMapsd.setObject( s_actionSlotMap, new ActionSlotMap );
 
     s_actionSlotMap->insert( "cut", SLOT( cut() ) );
     s_actionSlotMap->insert( "copy", SLOT( copy() ) );
@@ -489,7 +535,6 @@ void BrowserExtension::createActionSlotMap()
     s_actionSlotMap->insert( "rename", SLOT( rename() ) );
     s_actionSlotMap->insert( "trash", SLOT( trash() ) );
     s_actionSlotMap->insert( "del", SLOT( del() ) );
-    s_actionSlotMap->insert( "shred", SLOT( shred() ) );
     s_actionSlotMap->insert( "properties", SLOT( properties() ) );
     s_actionSlotMap->insert( "editMimeType", SLOT( editMimeType() ) );
     s_actionSlotMap->insert( "print", SLOT( print() ) );
@@ -501,8 +546,7 @@ void BrowserExtension::createActionSlotMap()
 
     // Create the action-number map
     assert(!s_actionNumberMap);
-    s_actionNumberMap = new ActionNumberMap;
-    actionNumberMapsd.setObject( s_actionNumberMap );
+    s_actionNumberMap = actionNumberMapsd.setObject( s_actionNumberMap, new ActionNumberMap );
     ActionSlotMap::ConstIterator it = s_actionSlotMap->begin();
     ActionSlotMap::ConstIterator itEnd = s_actionSlotMap->end();
     for ( int i=0 ; it != itEnd ; ++it, ++i )
@@ -563,17 +607,10 @@ QStringList BrowserHostExtension::frameNames() const
   return QStringList();
 }
 
-#if QT_VERSION < 300
-const QList<KParts::ReadOnlyPart> BrowserHostExtension::frames() const
-{
-  return QList<KParts::ReadOnlyPart>();
-}
-#else
 const QPtrList<KParts::ReadOnlyPart> BrowserHostExtension::frames() const
 {
   return QPtrList<KParts::ReadOnlyPart>();
 }
-#endif
 
 bool BrowserHostExtension::openURLInFrame( const KURL &, const KParts::URLArgs & )
 {
@@ -612,8 +649,8 @@ bool LiveConnectExtension::put( const unsigned long, const QString &, const QStr
       return false;
 }
 
-bool LiveConnectExtension::call( const unsigned long, const QString &, const QStringList &, Type &, unsigned long &, QString & ) { 
-      return false; 
+bool LiveConnectExtension::call( const unsigned long, const QString &, const QStringList &, Type &, unsigned long &, QString & ) {
+      return false;
 }
 
 void LiveConnectExtension::unregister( const unsigned long ) {}
