@@ -41,7 +41,7 @@ public:
 BrowserRun::BrowserRun( const KURL& url, const KParts::URLArgs& args,
                         KParts::ReadOnlyPart *part, QWidget* window,
                         bool removeReferrer, bool trustedSource )
-    : KRun( url, 0 /*mode*/, false /*is_local_file known*/, false /* no GUI */ ),
+    : KRun( url, window, 0 /*mode*/, false /*is_local_file known*/, false /* no GUI */ ),
       m_args( args ), m_part( part ), m_window( window ),
       m_bRemoveReferrer( removeReferrer ), m_bTrustedSource( trustedSource )
 {
@@ -53,7 +53,7 @@ BrowserRun::BrowserRun( const KURL& url, const KParts::URLArgs& args,
 BrowserRun::BrowserRun( const KURL& url, const KParts::URLArgs& args,
                         KParts::ReadOnlyPart *part, QWidget* window,
                         bool removeReferrer, bool trustedSource, bool hideErrorDialog )
-    : KRun( url, 0 /*mode*/, false /*is_local_file known*/, false /* no GUI */ ),
+    : KRun( url, window, 0 /*mode*/, false /*is_local_file known*/, false /* no GUI */ ),
       m_args( args ), m_part( part ), m_window( window ),
       m_bRemoveReferrer( removeReferrer ), m_bTrustedSource( trustedSource )
 {
@@ -73,7 +73,10 @@ void BrowserRun::init()
     // ### KRun doesn't call a virtual method when it finds out that the URL
     // is either malformed, or points to a non-existing local file...
     // So we need to reimplement some of the checks, to handle m_bHideErrorDialog
-    m_bFault = !m_strURL.isValid();
+    if ( !m_strURL.isValid() ) {
+        redirectToError( KIO::ERR_MALFORMED_URL, m_strURL.url() );
+        return;
+    }
     if ( !m_bIsLocalFile && !m_bFault && m_strURL.isLocalFile() )
       m_bIsLocalFile = true;
 
@@ -81,19 +84,12 @@ void BrowserRun::init()
       struct stat buff;
       if ( stat( QFile::encodeName(m_strURL.path()), &buff ) == -1 )
       {
-        m_bFault = true;
         kdDebug(1000) << "BrowserRun::init : " << m_strURL.prettyURL() << " doesn't exist." << endl;
+        redirectToError( KIO::ERR_DOES_NOT_EXIST, m_strURL.path() );
+        return;
       }
       m_mode = buff.st_mode; // while we're at it, save it for KRun::init() to use it
     }
-
-    if ( m_bFault )
-    {
-      m_bFinished = true;
-      m_timer.start( 0, true );
-      handleError( 0 );
-      return;
-    }
   }
   KRun::init();
 }
@@ -119,15 +115,20 @@ void BrowserRun::scanFile()
 
   if ( m_part )
   {
-      QString proto = m_part->url().protocol();
-      if (proto.find("https", 0, false) == 0) {
+      QString proto = m_part->url().protocol().lower();
+
+      if (proto == "https" || proto == "webdavs") {
           m_args.metaData().insert("main_frame_request", "TRUE" );
           m_args.metaData().insert("ssl_was_in_use", "TRUE" );
           m_args.metaData().insert("ssl_activate_warnings", "TRUE" );
-      } else if (proto.find("http", 0, false) == 0) {
+      } else if (proto == "http" || proto == "webdav") {
           m_args.metaData().insert("ssl_activate_warnings", "TRUE" );
           m_args.metaData().insert("ssl_was_in_use", "FALSE" );
       }
+
+      // Set the PropagateHttpHeader meta-data if it has not already been set...
+      if (!m_args.metaData().contains("PropagateHttpHeader"))
+          m_args.metaData().insert("PropagateHttpHeader", "TRUE");
   }
 
   KIO::TransferJob *job;
@@ -177,7 +178,7 @@ void BrowserRun::slotBrowserScanFinished(KIO::Job *job)
 void BrowserRun::slotBrowserMimetype( KIO::Job *_job, const QString &type )
 {
   Q_ASSERT( _job == m_job );
-  KIO::TransferJob *job = (KIO::TransferJob *) m_job;
+  KIO::TransferJob *job = static_cast<KIO::TransferJob *>(m_job);
   // Update our URL in case of a redirection
   //kdDebug(1000) << "old URL=" << m_strURL.url() << endl;
   //kdDebug(1000) << "new URL=" << job->url().url() << endl;
@@ -238,6 +239,7 @@ BrowserRun::NonEmbeddableResult BrowserRun::handleNonEmbeddable( const QString&
                 KURL destURL;
                 destURL.setPath( tempFile.name() );
                 KIO::Job *job = KIO::file_copy( m_strURL, destURL, 0600, true /*overwrite*/, false /*no resume*/, true /*progress info*/ );
+                job->setWindow (m_window);
                 connect( job, SIGNAL( result( KIO::Job *)),
                          this, SLOT( slotCopyToTempFileResult(KIO::Job *)) );
                 return Delayed; // We'll continue after the job has finished
@@ -269,36 +271,87 @@ bool BrowserRun::allowExecution( const QString &serviceType, const KURL &url )
     return ( KMessageBox::warningYesNo( 0, i18n( "Do you really want to execute '%1'? " ).arg( url.prettyURL() ) ) == KMessageBox::Yes );
 }
 
+static QString makeQuestion( const KURL& url, const QString& mimeType, const QString& suggestedFilename )
+{
+    QString surl = KStringHandler::csqueeze( url.prettyURL() );
+    KMimeType::Ptr mime = KMimeType::mimeType( mimeType );
+    QString comment = mimeType;
+
+    // Test if the mimeType is not recognize as octet-stream.
+    // If so then keep mime-type as comment
+    if (mime->name() != KMimeType::defaultMimeType()) {
+        // The mime-type is known so display the comment instead of mime-type
+        comment = mime->comment();
+    }
+    // The strange order in the i18n() calls below is due to the possibility
+    // of surl containing a '%'
+    if ( suggestedFilename.isEmpty() )
+        return i18n("Open '%2'?\nType: %1").arg(comment).arg(surl);
+    else
+        return i18n("Open '%3'?\nName: %2\nType: %1").arg(comment).arg(suggestedFilename).arg(surl);
+}
+
 //static
 BrowserRun::AskSaveResult BrowserRun::askSave( const KURL & url, KService::Ptr offer, const QString& mimeType, const QString & suggestedFilename )
 {
-    QString surl = KStringHandler::csqueeze( url.prettyURL() );
-    QString question;
-    if ( suggestedFilename.isEmpty() )
-    {
-        question = (offer && !offer->name().isEmpty())
-	       	   ? i18n("Open '%1' using '%2'?").arg(surl).arg(offer->name())
-                   : i18n("Open '%1'?").arg(surl);
-    } else {
-        question = (offer && !offer->name().isEmpty())
-		   ? i18n("Open '%1' (%2) using '%3'?").
-                     arg( surl ).arg(suggestedFilename).arg(offer->name())
-                   : i18n("Open '%1' (%2)?").arg( surl ).arg(suggestedFilename);
-    }
+    QString question = makeQuestion( url, mimeType, suggestedFilename );
+
+    // Text used for the open button
+    QString openText = (offer && !offer->name().isEmpty())
+                       ? i18n("&Open with '%1'").arg(offer->name())
+                       : i18n("&Open with...");
+
     int choice = KMessageBox::questionYesNoCancel(
         0L, question, QString::null,
-        KStdGuiItem::saveAs(), i18n("&Open"),
+        KStdGuiItem::saveAs(), openText,
         QString::fromLatin1("askSave")+ mimeType ); // dontAskAgainName
     return choice == KMessageBox::Yes ? Save : ( choice == KMessageBox::No ? Open : Cancel );
 }
-// Default implementation, overriden in KHTMLRun
+
+//static
+BrowserRun::AskSaveResult BrowserRun::askEmbedOrSave( const KURL & url, const QString& mimeType, const QString & suggestedFilename, int /*flags*/ )
+{
+    KMimeType::Ptr mime = KMimeType::mimeType( mimeType );
+    // Don't ask for:
+    // - html (even new tabs would ask, due to about:blank!)
+    // - dirs obviously (though not common over HTTP :),
+    // - images (reasoning: no need to save, most of the time, because fast to see)
+    // e.g. postscript is different, because takes longer to read, so
+    // it's more likely that the user might want to save it.
+    // - multipart/* ("server push", see kmultipart)
+    // - other strange 'internal' mimetypes like print/manager...
+    if ( mime->is( "text/html" ) ||
+         mime->is( "text/xml" ) ||
+         mime->is( "inode/directory" ) ||
+         mimeType.startsWith( "image" ) ||
+         mime->is( "multipart/x-mixed-replace" ) ||
+         mime->is( "multipart/replace" ) ||
+         mimeType.startsWith( "print" ) )
+        return Open;
+
+    QString question = makeQuestion( url, mimeType, suggestedFilename );
+
+    int choice = KMessageBox::questionYesNoCancel(
+        0L, question, QString::null,
+        KStdGuiItem::saveAs(), KGuiItem( i18n( "&Open" ), "fileopen"),
+        QString::fromLatin1("askEmbedOrSave")+ mimeType ); // dontAskAgainName
+    return choice == KMessageBox::Yes ? Save : ( choice == KMessageBox::No ? Open : Cancel );
+}
+
+// Default implementation, overridden in KHTMLRun
 void BrowserRun::save( const KURL & url, const QString & suggestedFilename )
 {
-    simpleSave( url, suggestedFilename );
+    simpleSave( url, suggestedFilename, m_window );
 }
 
 // static
 void BrowserRun::simpleSave( const KURL & url, const QString & suggestedFilename )
+{
+    simpleSave (url, suggestedFilename, 0);
+}
+
+void BrowserRun::simpleSave( const KURL & url, const QString & suggestedFilename,
+                             QWidget* window )
 {
     // DownloadManager <-> konqueror integration
     // find if the integration is enabled
@@ -316,14 +369,13 @@ void BrowserRun::simpleSave( const KURL & url, const QString & suggestedFilename
             QString errMsg=i18n("The Download Manager (%1) could not be found in your $PATH ").arg(downloadManger);
             QString errMsgEx= i18n("Try to reinstall it  \n\nThe integration with Konqueror will be disabled!");
             KMessageBox::detailedSorry(0,errMsg,errMsgEx);
-            cfg.writeEntry("DownloadManager",QString::null);
+            cfg.writePathEntry("DownloadManager",QString::null);
             cfg.sync ();
         }
         else
         {
             // ### suggestedFilename not taken into account. Fix this (and
-            // the duplicated code) with shiny new KDownload class for 3.2
-            // (pfeiffer)
+            // the duplicated code) with shiny new KDownload class for 3.2 (pfeiffer)
             // Until the shiny new class comes about, send the suggestedFilename
             // along with the actual URL to download. (DA)
             cmd += " " + KProcess::quote(url.url()) + " " + KProcess::quote(suggestedFilename);
@@ -337,7 +389,7 @@ void BrowserRun::simpleSave( const KURL & url, const QString & suggestedFilename
 
     // no download manager available, let's do it ourself
     KFileDialog *dlg = new KFileDialog( QString::null, QString::null /*all files*/,
-                                        0L , "filedialog", true );
+                                        window , "filedialog", true );
     dlg->setOperationMode( KFileDialog::Saving );
     dlg->setCaption(i18n("Save As"));
 
@@ -345,10 +397,11 @@ void BrowserRun::simpleSave( const KURL & url, const QString & suggestedFilename
     if ( dlg->exec() )
     {
         KURL destURL( dlg->selectedURL() );
-        if ( !destURL.isMalformed() )
+        if ( destURL.isValid() )
         {
             KIO::Job *job = KIO::copy( url, destURL );
-	    job->setAutoErrorHandlingEnabled( true );
+            job->setWindow (window);
+            job->setAutoErrorHandlingEnabled( true );
         }
     }
     delete dlg;
@@ -370,11 +423,43 @@ void BrowserRun::handleError( KIO::Job * job )
         return;
     }
 
+    if (d->m_bHideErrorDialog && job->error() != KIO::ERR_NO_CONTENT)
+    {
+        redirectToError( job->error(), job->errorText() );
+        return;
+    }
+
     // Reuse code in KRun, to benefit from d->m_showingError etc.
-    // KHTMLRun and KonqRun reimplement handleError anyway.
     KRun::slotStatResult( job );
 }
 
+void BrowserRun::redirectToError( int error, const QString& errorText )
+{
+    /**
+     * To display this error in KHTMLPart instead of inside a dialog box,
+     * we tell konq that the mimetype is text/html, and we redirect to
+     * an error:/ URL that sends the info to khtml.
+     *
+     * The format of the error:/ URL is error:/?query#url,
+     * where two variables are passed in the query:
+     * error = int kio error code, errText = QString error text from kio
+     * The sub-url is the URL that we were trying to open.
+     */
+    QString errText( errorText );
+    errText.replace( '#', "%23" ); // a # in the error string would really muck things up...
+    KURL newURL(QString("error:/?error=%1&errText=%2")
+                .arg( error ).arg( errText ), 106 );
+    m_strURL.setPass( QString::null ); // don't put the password in the error URL
+
+    KURL::List lst;
+    lst << newURL << m_strURL;
+    m_strURL = KURL::join( lst );
+    //kdDebug(1202) << "BrowserRun::handleError m_strURL=" << m_strURL.prettyURL() << endl;
+
+    m_job = 0;
+    foundMimeType( "text/html" );
+}
+
 void BrowserRun::slotCopyToTempFileResult(KIO::Job *job)
 {
     if ( job->error() ) {
@@ -396,10 +481,7 @@ bool BrowserRun::isTextExecutable( const QString &serviceType )
 
 bool BrowserRun::isExecutable( const QString &serviceType )
 {
-    return ( serviceType == "application/x-desktop" ||
-             serviceType == "application/x-executable" ||
-             serviceType == "application/x-msdos-program" ||
-             serviceType == "application/x-shellscript" );
+    return KRun::isExecutable( serviceType );
 }
 
 bool BrowserRun::hideErrorDialog() const
