@@ -47,7 +47,7 @@ namespace KParts {
          * @param window the mainwindow - passed to KIO::Job::setWindow()
          * @param removeReferrer if true, the "referrer" metadata from @p args isn't passed on
          * @param trustedSource if false, a warning will be shown before launching an executable
-	 * Always pass false for @p trustedSource, except for local directory views.
+         * Always pass false for @p trustedSource, except for local directory views.
          */
         BrowserRun( const KURL& url, const KParts::URLArgs& args,
                     KParts::ReadOnlyPart *part, QWidget *window,
@@ -62,7 +62,7 @@ namespace KParts {
          * @param removeReferrer if true, the "referrer" metadata from @p args isn't passed on
          * @param trustedSource if false, a warning will be shown before launching an executable
          * @param hideErrorDialog if true, no dialog will be shown in case of errors.
-	 * Always pass false for @p trustedSource, except for local directory views.
+         * Always pass false for @p trustedSource, except for local directory views.
          */
         BrowserRun( const KURL& url, const KParts::URLArgs& args,
                     KParts::ReadOnlyPart *part, QWidget *window,
@@ -75,17 +75,44 @@ namespace KParts {
         KURL url() const { return m_strURL; }
 
         bool hideErrorDialog() const;
-
+        
+        // Suggested filename given by the server (e.g. HTTP content-disposition)
+        QString suggestedFilename() const { return m_suggestedFilename; }
+        
         enum AskSaveResult { Save, Open, Cancel };
+        /**
+         * Ask the user whether to save or open a url in another application.
+         * @param url the URL in question
+         * @param offer the application that will be used to open the URL
+         * @param mimeType the mimetype of the URL
+         * @param suggestedFilename optional filename suggested by the server
+         * @return Save, Open or Cancel.
+         */
         static AskSaveResult askSave( const KURL & url, KService::Ptr offer, const QString& mimeType, const QString & suggestedFilename = QString::null );
+        /**
+         * Similar to askSave but for the case where the current application is
+         * able to embed the url itself (instead of passing it to another app).
+         * @param url the URL in question
+         * @param mimeType the mimetype of the URL
+         * @param suggestedFilename optional filename suggested by the server
+         * @param flags reserved for later use
+         * @return Save, Open or Cancel.
+         */
+        static AskSaveResult askEmbedOrSave( const KURL & url, const QString& mimeType, const QString & suggestedFilename = QString::null, int flags = 0 );
 
         // virtual so that KHTML can implement differently (HTML cache)
         virtual void save( const KURL & url, const QString & suggestedFilename );
+
         // static so that it can be called from other classes
+        static void simpleSave( const KURL & url, const QString & suggestedFilename,
+                                QWidget* window );
+
+        /** BIC: Combine with the above function for KDE 4.0. */
         static void simpleSave( const KURL & url, const QString & suggestedFilename );
 
         static bool allowExecution( const QString &serviceType, const KURL &url );
 
+        /** BIC: Obsoleted by KRun::isExecutable( const QString &serviceType ); */
         static bool isExecutable( const QString &serviceType );
         static bool isTextExecutable( const QString &serviceType );
 
@@ -112,9 +139,10 @@ namespace KParts {
          * i.e. launch an external app.
          */
         enum NonEmbeddableResult { Handled, NotHandled, Delayed };
+
         /**
-	 * Helper for foundMimeType: call this if the mimetype couldn't be embedded
-	 */
+         * Helper for foundMimeType: call this if the mimetype couldn't be embedded
+         */
         NonEmbeddableResult handleNonEmbeddable( const QString& mimeType );
 
     protected slots:
@@ -126,12 +154,15 @@ namespace KParts {
     protected:
         KParts::URLArgs m_args;
         KParts::ReadOnlyPart *m_part; // QGuardedPtr?
-        QWidget *m_window;            // QGuardedPtr?
+        QGuardedPtr<QWidget> m_window;
+        // Suggested filename given by the server (e.g. HTTP content-disposition)
+        // When set, we should really be saving instead of embedding
         QString m_suggestedFilename;
         QString m_sMimeType;
         bool m_bRemoveReferrer;
         bool m_bTrustedSource;
     private:
+        void redirectToError( int error, const QString& errorText );
         class BrowserRunPrivate;
         BrowserRunPrivate* d;
 
