@@ -70,7 +70,7 @@ public:
   virtual ~PartBase();
 
   /**
-   *  Internal method. Called by @ref KParts::Part to specify the parent object for plugin objects.
+   *  Internal method. Called by KParts::Part to specify the parent object for plugin objects.
    *
    * @internal
    */
@@ -79,7 +79,7 @@ public:
 
 protected:
   /**
-   * Set the instance (@ref KInstance) for this part.
+   * Set the instance ( KInstance) for this part.
    *
    * Call this *first* in the inherited class constructor,
    * because it loads the i18n catalogues.
@@ -87,21 +87,58 @@ protected:
   virtual void setInstance( KInstance *instance );
 
   /**
-   * Set the instance (@ref KInstance) for this part.
+   * Set the instance ( KInstance) for this part.
    *
    * Call this *first* in the inherited class constructor,
    * because it loads the i18n catalogues.
    */
   virtual void setInstance( KInstance *instance, bool loadPlugins );
 
+  /**
+   * We have three different policies, whether to load new plugins or not. The
+   * value in the KConfig object of the KInstance object always overrides
+   * LoadPlugins and LoadPluginsIfEnabled.
+   */
   enum PluginLoadingMode {
+    /**
+     * Don't load any plugins at all.
+     */
     DoNotLoadPlugins = 0,
-    LoadPlugins = 1,         // load new plugins automatically
-    LoadPluginsIfEnabled = 2 // new plugins are disabled by default
+    /**
+     * Load new plugins automatically. Can be
+     * overridden by the plugin if it sets
+     * EnabledByDefault=false in the corresponding
+     * .desktop file.
+     */
+    LoadPlugins = 1,
+    /**
+     * New plugins are disabled by default. Can be
+     * overridden by the plugin if it sets
+     * EnabledByDefault=true in the corresponding
+     * .desktop file.
+     */
+    LoadPluginsIfEnabled = 2
   };
 
   /**
    * Load the Plugins honoring the PluginLoadingMode.
+   *
+   * If you call this method in an already constructed GUI (like when the user
+   * has changed which plugins are enabled) you need to add the new plugins to
+   * the KXMLGUIFactory:
+   * \code
+   * if( factory() )
+   * {
+   *   QPtrList<KParts::Plugin> plugins = KParts::Plugin::pluginObjects( this );
+   *   QPtrListIterator<KParts::Plugin> it( plugins );
+   *   KParts::Plugin * plugin;
+   *   while( ( plugin = it.current() ) != 0 )
+   *   {
+   *     ++it;
+   *     factory()->addClient(  plugin );
+   *   }
+   * }
+   * \endcode
    */
   void loadPlugins( QObject *parent, KXMLGUIClient *parentGUIClient, KInstance *instance );
 
@@ -124,18 +161,20 @@ private:
  * @li GUI elements that will be merged in the "host" user interface
  * (menubars, toolbars... ).
  *
- * @sect About the widget
- * Note that KParts::Part does not inherit @ref QWidget.
+ * <b>About the widget:</b>\n
+ *
+ * Note that KParts::Part does not inherit QWidget.
  * This is due to the fact that the "visual representation"
- * will probably not be a mere @ref QWidget, but an elaborate one.
+ * will probably not be a mere QWidget, but an elaborate one.
  * That's why when implementing your KParts::Part (or derived)
  * you should call KParts::Part::setWidget() in your constructor.
  *
- * @sect About the GUI elements
- * Those elements trigger actions, defined by the part (@ref action()).
- * The layout of the actions in the GUI is defined by an XML file (@ref setXMLFile()).
+ * <b>About the GUI elements:</b>\n
  *
- * See also @ref ReadOnlyPart and @ref ReadWritePart, which define the
+ * Those elements trigger actions, defined by the part ( action()).
+ * The layout of the actions in the GUI is defined by an XML file ( setXMLFile()).
+ *
+ * See also ReadOnlyPart and ReadWritePart, which define the
  * framework for a "viewer" part and for an "editor"-like part.
  * Use Part directly only if your part doesn't fit into those.
  */
@@ -162,9 +201,9 @@ public:
      * Embed this part into a host widget.
      *
      * You don't need to do this if you created the widget with the
-     * correct parent widget - this is just a @ref QWidget::reparent().
-     * Note that the @ref Part is still the holder
-     * of the @ref QWidget, meaning that if you delete the @ref Part,
+     * correct parent widget - this is just a QWidget::reparent().
+     * Note that the Part is still the holder
+     * of the QWidget, meaning that if you delete the Part,
      * then the widget gets destroyed as well, and vice-versa.
      * This method is not recommended since creating the widget with the correct
      * parent is simpler anyway.
@@ -172,7 +211,7 @@ public:
     virtual void embed( QWidget * parentWidget );
 
     /**
-     * @return The widget defined by this part, set by @ref setWidget().
+     * @return The widget defined by this part, set by setWidget().
      */
     virtual QWidget *widget();
 
@@ -229,30 +268,30 @@ protected:
     virtual void customEvent( QCustomEvent *event );
 
     /**
-     * Convenience method which is called when the Part received a @ref PartActivateEvent .
-     * Reimplement this if you don't want to reimplement @ref event and test for the event yourself
+     * Convenience method which is called when the Part received a PartActivateEvent .
+     * Reimplement this if you don't want to reimplement event and test for the event yourself
      * or even install an event filter.
      */
     virtual void partActivateEvent( PartActivateEvent *event );
 
     /**
      * Convenience method which is called when the Part received a
-     * @ref PartSelectEvent .
-     * Reimplement this if you don't want to reimplement @ref event and
+     * PartSelectEvent .
+     * Reimplement this if you don't want to reimplement event and
      * test for the event yourself or even install an event filter.
      */
     virtual void partSelectEvent( PartSelectEvent *event );
 
     /**
      * Convenience method which is called when the Part received a
-     * @ref GUIActivateEvent .
-     * Reimplement this if you don't want to reimplement @ref event and
+     * GUIActivateEvent .
+     * Reimplement this if you don't want to reimplement event and
      * test for the event yourself or even install an event filter.
      */
     virtual void guiActivateEvent( GUIActivateEvent *event );
 
     /**
-     * Convenience method for @ref KXMLGUIFactory::container.
+     * Convenience method for KXMLGUIFactory::container.
      * @return a container widget owned by the Part's GUI.
      */
     QWidget *hostContainer( const QString &containerName );
@@ -268,6 +307,7 @@ private:
     PartPrivate *d;
 };
 
+class ReadWritePart;
 class ReadOnlyPartPrivate;
 
 /**
@@ -276,24 +316,25 @@ class ReadOnlyPartPrivate;
  * This class takes care of network transparency for you,
  * in the simplest way (synchronously).
  * To use the built-in network transparency, you only need to implement
- * @ref openFile(), not @ref openURL().
+ * openFile(), not openURL().
  * To prevent network transparency, or to implement it another way
  * (e.g. asynchronously), override openURL().
  *
  * KParts Application can use the signals to show feedback while the URL is being loaded.
  *
  * ReadOnlyPart handles the window caption by setting it to the current URL
- * (set in @ref #openURL(), and each time the part is activated).
- * If you want another caption, set it in @ref openFile() and
- * (if the part might ever be used with a part manager) in @ref guiActivateEvent()
+ * (set in openURL(), and each time the part is activated).
+ * If you want another caption, set it in openFile() and
+ * (if the part might ever be used with a part manager) in guiActivateEvent()
  */
 class ReadOnlyPart : public Part
 {
   Q_OBJECT
+  friend class ReadWritePart;
 public:
   /**
    * Constructor
-   * See also @ref Part for the setXXX methods to call.
+   * See also Part for the setXXX methods to call.
    */
   ReadOnlyPart( QObject *parent = 0, const char *name = 0 );
 
@@ -307,7 +348,7 @@ public:
    * the internal KIO job. Use this if you provide another way
    * of displaying progress info (e.g. a statusbar), using the
    * signals emitted by this class, and/or those emitted by
-   * the Job given by @ref started.
+   * the Job given by started.
    */
   void setProgressInfoEnabled( bool show );
 
@@ -324,7 +365,7 @@ public:
 public slots:
   /**
    * Only reimplement openURL if you don't want synchronous network transparency
-   * Otherwise, reimplement @ref openFile() only .
+   * Otherwise, reimplement openFile() only .
    *
    * If you reimplement it, don't forget to set the caption, usually with
    * emit setWindowCaption( url.prettyURL() );
@@ -341,7 +382,7 @@ public:
 
   /**
    * Called when closing the current url (e.g. document), for instance
-   * when switching to another url (note that @ref #openURL() calls it
+   * when switching to another url (note that openURL() calls it
    * automatically in this case).
    * If the current URL is not fully loaded yet, aborts loading.
    * Deletes the temporary file used when the url is remote.
@@ -356,31 +397,31 @@ public:
    * to load the data itself, and send it progressively to the part.
    *
    * @param url the URL representing this data. Although not directly used,
-   * every ReadOnlyPart has a URL (see @ref url()), so this simply sets it.
+   * every ReadOnlyPart has a URL (see url()), so this simply sets it.
    * @param mimeType the type of data that is going to be sent to this part.
    * @return true if the part supports progressive loading and accepts data, false otherwise.
    */
   bool openStream( const QString& mimeType, const KURL& url );
 
   /**
-   * Send some data to the part. @ref openStream must have been called previously,
+   * Send some data to the part. openStream must have been called previously,
    * and must have returned true.
    * @return true if the data was accepted by the part. If false is returned,
-   * the application should stop sending data, and doesn't have to call @ref closeStream.
+   * the application should stop sending data, and doesn't have to call closeStream.
    */
   bool writeStream( const QByteArray& data );
 
   /**
    * Terminate the sending of data to the part.
-   * With some data types (text, html...) @ref closeStream might never actually be called,
-   * in the case of continous streams, for instance plain text or HTML data.
+   * With some data types (text, html...) closeStream might never actually be called,
+   * in the case of continuous streams, for instance plain text or HTML data.
    */
   bool closeStream();
 
 private: // Makes no sense for inherited classes to call those. But make it protected there.
 
   /**
-   * Called by @ref openStream to initiate sending of data.
+   * Called by openStream to initiate sending of data.
    * Parts which implement progress loading should check the @p mimeType
    * parameter, and return true if they can accept a data stream of that type.
    */
@@ -388,12 +429,12 @@ private: // Makes no sense for inherited classes to call those. But make it prot
   /**
    * Receive some data from the hosting application.
    * In this method the part should attempt to display the data progressively.
-   * With some data types (text, html...) @ref closeStream might never actually be called,
-   * in the case of continous streams. This can't happen with e.g. images.
+   * With some data types (text, html...) closeStream might never actually be called,
+   * in the case of continuous streams. This can't happen with e.g. images.
    */
   virtual bool doWriteStream( const QByteArray& /*data*/ ) { return false; }
   /**
-   * This is called by @ref closeStream(), to indicate that all the data has been sent.
+   * This is called by closeStream(), to indicate that all the data has been sent.
    * Parts should ensure that all of the data is displayed at this point.
    * @return whether the data could be displayed correctly.
    */
@@ -415,17 +456,13 @@ signals:
   void completed();
 
   /**
-   * Same as the above signal except besides indicating that the data has
-   * been completely loaded it also informs the host, by setting the flag,
-   * that a pending action has been generated as a result of completing the
-   * requested task (loading the data).  An example of this is meta-refresh
-   * tags on HTML pages which result in the page either being refreshed or
-   * the viewer being redirected to another page.  By emitting this signal
-   * after appropriately setting the flag, the part can tell the host of the
-   * pending scheduled action inorder to give it a chance to accept or cancel
-   * that action.
+   * Same as the above signal except it indicates whether there is
+   * a pending action to be executed on a delay timer. An example of
+   * this is the meta-refresh tags on web pages used to reload/redirect
+   * after a certain period of time. This signal is useful if you want
+   * to give the user the ability to cancel such pending actions.
    *
-   * @p pendingAction  if true, a pending action exists (ex: a scheduled refresh)
+   * @p pendingAction true if a pending action exists, false otherwise.
    */
   void completed( bool pendingAction );
 
@@ -440,7 +477,7 @@ protected slots:
 
 protected:
   /**
-   * If the part uses the standard implementation of @ref #openURL(),
+   * If the part uses the standard implementation of openURL(),
    * it must reimplement this, to open @p m_file.
    * Otherwise simply define it to { return false; }
    */
@@ -454,7 +491,7 @@ protected:
   /**
    * Reimplemented from Part, so that the window caption is set to
    * the current url (decoded) when the part is activated
-   * This is the usual behaviour in 99% of the apps
+   * This is the usual behavior in 99% of the apps
    * Reimplement if you don't like it - test for event->activated() !
    *
    * Technical note : this is done with GUIActivateEvent and not with
@@ -487,13 +524,13 @@ private:
  * Anything that can open a URL, allow modifications, and save
  * (to the same URL or a different one).
  *
- * A read-write part can be set to read-only mode, using @ref setReadWrite().
+ * A read-write part can be set to read-only mode, using setReadWrite().
  *
  * Part writers :
- * Any part inheriting ReadWritePart should check @ref isReadWrite
+ * Any part inheriting ReadWritePart should check isReadWrite
  * before allowing any action that modifies the part.
- * The part probably wants to reimplement @ref setReadWrite, disable those
- * actions. Don't forget to call the parent @ref setReadWrite.
+ * The part probably wants to reimplement setReadWrite, disable those
+ * actions. Don't forget to call the parent setReadWrite.
  */
 class ReadWritePart : public ReadOnlyPart
 {
@@ -507,9 +544,9 @@ public:
   /**
    * Destructor
    * Applications using a ReadWritePart should make sure, before
-   * destroying it, to call @ref #closeURL().
-   * In @ref KMainWindow::queryClose(), for instance, they should allow
-   * closing only if the return value of @ref #closeURL() was true.
+   * destroying it, to call closeURL().
+   * In KMainWindow::queryClose(), for instance, they should allow
+   * closing only if the return value of closeURL() was true.
    * This allows to cancel.
    */
   virtual ~ReadWritePart();
@@ -520,7 +557,7 @@ public:
   bool isReadWrite() const { return m_bReadWrite; }
 
   /**
-   * Changes the behaviour of this part to readonly or readwrite.
+   * Changes the behavior of this part to readonly or readwrite.
    * @param readwrite set to true to enable readwrite mode
    */
   virtual void setReadWrite ( bool readwrite = true );
@@ -532,8 +569,8 @@ public:
 
   /**
    * If the document has been modified, ask the user to save changes.
-   * This method is meant to be called from @ref KMainWindow::queryClose().
-   * It will also be called from @ref closeURL().
+   * This method is meant to be called from KMainWindow::queryClose().
+   * It will also be called from closeURL().
    *
    * @return true if closeURL() can be called without the user losing
    * important data, false if the user chooses to cancel.
@@ -545,12 +582,12 @@ public:
 
   /**
    * Called when closing the current url (e.g. document), for instance
-   * when switching to another url (note that @ref openURL() calls it
+   * when switching to another url (note that openURL() calls it
    * automatically in this case).
    *
    * If the current URL is not fully loaded yet, aborts loading.
    *
-   * If @ref isModified(), @ref queryClose() will be called.
+   * If isModified(), queryClose() will be called.
    *
    * @return false on cancel
    */
@@ -558,11 +595,11 @@ public:
 
   /**
    * Call this method instead of the above if you need control if
-   * the save prompt is shown. For example, if you call @ref queryClose()
-   * from @ref KMainWindow::queryClose(), you would not want to prompt
+   * the save prompt is shown. For example, if you call queryClose()
+   * from KMainWindow::queryClose(), you would not want to prompt
    * again when closing the url.
    *
-   * Equivalent to promptToSave ? @ref closeURL() : @ref ReadOnlyPart::closeURL()
+   * Equivalent to promptToSave ? closeURL() : ReadOnlyPart::closeURL()
    *
    * @since 3.2
    */
@@ -572,7 +609,7 @@ public:
   /**
    * Save the file to a new location.
    *
-   * Calls @ref save(), no need to reimplement
+   * Calls save(), no need to reimplement
    */
   virtual bool saveAs( const KURL &url );
 
@@ -581,9 +618,19 @@ public:
    */
   virtual void setModified( bool modified );
 
+signals:
+   /**
+    * @since 3.2, remove in KDE 4, when queryClose is made virtual
+    *
+    * set handled to true, if you don't want the default handling
+    * set abortClosing to true, if you handled the request, 
+    * but for any reason don't  want to allow closing the document
+    */
+   void sigQueryClose(bool *handled, bool* abortClosing);
+
 public slots:
   /**
-   * Call @ref setModified() whenever the contents get modified.
+   * Call setModified() whenever the contents get modified.
    * This is a slot for convenience, so that you can connect it
    * to a signal, like textChanged().
    */
@@ -592,15 +639,27 @@ public slots:
   /**
    * Save the file in the location from which it was opened.
    * You can connect this to the "save" action.
-   * Calls @ref saveFile() and @ref saveToURL(), no need to reimplement.
+   * Calls saveFile() and saveToURL(), no need to reimplement.
    */
   virtual bool save();
 
+  /**
+   * Waits for any pending upload job to finish and returns whether the
+   * last save() action was successful.
+   */
+  bool waitSaveComplete();
+
 protected:
   /**
    * Save to a local file.
    * You need to implement it, to save to @p m_file.
    * The framework takes care of re-uploading afterwards.
+   *
+   * @return true on success, false on failure.
+   * On failure the function should inform the user about the
+   * problem with an appropriate message box. Standard error
+   * messages can be constructed using KIO::buildErrorString()
+   * in combination with the error codes defined in kio/global.h
    */
   virtual bool saveFile() = 0;
 
@@ -608,8 +667,9 @@ protected:
    * Save the file.
    *
    * Uploads the file, if @p m_url is remote.
-   * This will emit @ref started(), and either @ref completed() or @ref canceled(),
+   * This will emit started(), and either completed() or canceled(),
    * in case you want to provide feedback.
+   * @return true on success, false on failure.
    */
   virtual bool saveToURL();
 
@@ -625,6 +685,6 @@ private:
   bool m_bClosing;
 };
 
-}
+} // namespace
 
 #endif
