@@ -94,7 +94,7 @@ void Shell::slotFileOpen()
 static KCmdLineOptions options[] =
 {
  { "+file(s)",          "Files to load", 0 },
- { 0,0,0 }
+ KCmdLineLastOption
 };
 static const char *version = "v0.0.1 2000 (c) David Faure";
 static const char *description = "This is a test shell for the kghostview part.";
