@@ -30,7 +30,7 @@
    Boston, MA 02111-1307, USA.
 */
 
-// $Id: keramik.cpp,v 1.88.2.9 2003/08/01 01:12:54 orlovich Exp $
+// $Id: keramik.cpp,v 1.121.2.2 2004/03/25 13:35:08 lukas Exp $
 
 #include <qbitmap.h>
 #include <qcheckbox.h>
@@ -39,15 +39,18 @@
 #include <qframe.h>
 #include <qheader.h>
 #include <qintdict.h>
+#include <qlineedit.h>
 #include <qlistbox.h>
 #include <qmenubar.h>
 #include <qpainter.h>
 #include <qpointarray.h>
 #include <qprogressbar.h>
 #include <qpushbutton.h>
+#include <qsettings.h>
 #include <qslider.h>
 #include <qstyleplugin.h>
 #include <qtabbar.h>
+#include <qtimer.h>
 #include <qtoolbar.h>
 #include <qtoolbutton.h>
 
@@ -269,11 +272,40 @@ QPixmap KeramikStyle::stylePixmap(StylePixmap stylepixmap,
 }
 
 
+
+
 KeramikStyle::KeramikStyle()
-	:KStyle( AllowMenuTransparency | FilledFrameWorkaround, ThreeButtonScrollBar ), maskMode(false),
+	:KStyle( AllowMenuTransparency | FilledFrameWorkaround, ThreeButtonScrollBar ),
+		maskMode(false), formMode(false),
 		toolbarBlendWidget(0), titleBarMode(None), flatMode(false), customScrollMode(false), kickerMode(false)
 {
-	hoverWidget = 0;
+	forceSmallMode = false;
+	hoverWidget    = 0;
+	progAnimShift  = 0;
+
+	QSettings settings;
+
+	highlightScrollBar = settings.readBoolEntry("/keramik/Settings/highlightScrollBar", true);
+	animateProgressBar = settings.readBoolEntry("/keramik/Settings/animateProgressBar", false);
+
+	if (animateProgressBar)
+	{
+		QTimer* timer = new QTimer( this );
+		timer->start(50, false);
+		connect(timer, SIGNAL(timeout()), this, SLOT(updateProgressPos()));
+	}
+}
+
+void KeramikStyle::updateProgressPos()
+{
+	progAnimShift++;
+	if (progAnimShift == 28)
+		progAnimShift = 0;
+		
+	//Update the registered progressbars.
+	QMap<QWidget*, bool>::iterator iter;
+	for (iter = progAnimWidgets.begin(); iter != progAnimWidgets.end(); iter++)
+		iter.key()->update();
 }
 
 KeramikStyle::~KeramikStyle()
@@ -292,7 +324,7 @@ void KeramikStyle::polish(QApplication* app)
 void KeramikStyle::polish(QWidget* widget)
 {
 	// Put in order of highest occurrence to maximise hit rate
-	if ( widget->inherits( "QPushButton" )  || widget->inherits( "QComboBox" ) )
+	if ( widget->inherits( "QPushButton" )  || widget->inherits( "QComboBox" ) || widget->inherits("QToolButton") )
 	{
 		widget->installEventFilter(this);
 		if ( widget->inherits( "QComboBox" ) )
@@ -318,11 +350,18 @@ void KeramikStyle::polish(QWidget* widget)
 		widget->installEventFilter(this);
 	}
 
+	if (animateProgressBar && ::qt_cast<QProgressBar*>(widget))
+	{
+		progAnimWidgets[widget] = true;
+		connect(widget, SIGNAL(destroyed(QObject*)), this, SLOT(progressBarDestroyed(QObject*)));
+	}
+
 	KStyle::polish(widget);
 }
 
 void KeramikStyle::unPolish(QWidget* widget)
 {
+	//### TODO: This needs major cleanup (and so does polish() )
 	if ( widget->inherits( "QPushButton" ) || widget->inherits( "QComboBox"  ) )
 	{
 		if ( widget->inherits( "QComboBox" ) )
@@ -347,10 +386,19 @@ void KeramikStyle::unPolish(QWidget* widget)
 		widget->setBackgroundMode( PaletteBackground );
 		widget->removeEventFilter(this);
 	}
+	else if ( ::qt_cast<QProgressBar*>(widget) )
+	{
+		progAnimWidgets.remove(widget);
+	}
 
 	KStyle::unPolish(widget);
 }
 
+void KeramikStyle::progressBarDestroyed(QObject* obj)
+{
+	progAnimWidgets.remove(static_cast<QWidget*>(obj));
+}
+
 
 void KeramikStyle::polish( QPalette& )
 {
@@ -508,18 +556,13 @@ void KeramikStyle::drawPrimitive( PrimitiveElement pe,
 		case PE_ButtonDropDown:
 		case PE_ButtonTool:
 		{
-//			bool sunken = on || down;
-
 			if (titleBarMode)
 			{
 				QRect nr;
 				if (titleBarMode == Maximized)
 				{
 					//### What should we draw at sides?
-					//nr = QRect(r.x(), r.y(),
-					//	QMIN(r.width(), r.height()), r.width() ) );
 					nr = QRect(r.x(), r.y(), r.width()-1, r.height() );
-
 				}
 				else
 				{
@@ -549,9 +592,16 @@ void KeramikStyle::drawPrimitive( PrimitiveElement pe,
 				Keramik::RectTilePainter(keramik_toolbar_clk).draw(p, r, cg.button(), cg.background());
 			}
 			else {
+				if (flags & Style_MouseOver)
+				{
 				Keramik::GradientPainter::renderGradient( p,
 					QRect(r.x(), 0, r.width(), r.height()),
 					Keramik::ColorUtil::lighten(cg.button(), 115), flags & Style_Horizontal, false );
+				}
+				else
+					Keramik::GradientPainter::renderGradient( p,
+						QRect(r.x(), 0, r.width(), r.height()),
+						cg.button(), flags & Style_Horizontal, false );
 
 				p->setPen(cg.button().light(70));
 				p->drawLine(x, y, x2-1, y);
@@ -577,12 +627,13 @@ void KeramikStyle::drawPrimitive( PrimitiveElement pe,
 
 			int  name;
 
-			if ( w <= smallButMaxW || h <= smallButMaxH )
+			if ( w <= smallButMaxW || h <= smallButMaxH || forceSmallMode)
 			{
 				if (sunken)
 					name = keramik_pushbutton_small_pressed;
 				else
 					name =  keramik_pushbutton_small;
+				forceSmallMode = false;
 			}
 			else
 			{
@@ -616,6 +667,7 @@ void KeramikStyle::drawPrimitive( PrimitiveElement pe,
 				p->drawLine(x, y, x, y2);
 				p->drawLine(x, y2, x2, y2);
 				p->drawLine(x2, y, x2, y2);
+				flatMode = false;
 			}
 
 			break;
@@ -711,7 +763,7 @@ void KeramikStyle::drawPrimitive( PrimitiveElement pe,
 
 			QColor col = cg.highlight();
 
-			if (customScrollMode)
+			if (customScrollMode || !highlightScrollBar)
 				col = cg.button();
 
 			if (!active)
@@ -781,7 +833,7 @@ void KeramikStyle::drawPrimitive( PrimitiveElement pe,
 			}
 			break;
 		}
-
+		
 		// CHECKBOX (indicator)
 		// -------------------------------------------------------------------
 		case PE_Indicator:
@@ -809,18 +861,32 @@ void KeramikStyle::drawPrimitive( PrimitiveElement pe,
 			// line edit frame
 		case PE_PanelLineEdit:
 		{
-			p->setPen( cg.dark() );
-			p->drawLine( x, y, x + w - 1, y );
-			p->drawLine( x, y, x, y + h - 1 );
-			p->setPen( cg.mid() );
-			p->drawLine( x + 1, y + 1, x + w - 1, y + 1 );
-			p->drawLine( x + 1, y + 1, x + 1, y + h - 1 );
-			p->setPen( cg.light() );
-			p->drawLine( x + w - 1, y, x + w - 1, y + h - 1 );
-			p->drawLine( x, y + h - 1, x + w - 1, y + h - 1);
-			p->setPen( cg.light().dark( 110 ) );
-			p->drawLine( x + w - 2, y + 1, x + w - 2, y + h - 2 );
-			p->drawLine( x + 1, y + h - 2, x + w - 2, y + h - 2);
+			if (!opt.isDefault() && opt.lineWidth() == 1)
+			{
+				//1-pixel frames can not be simply clipped wider frames, as that would produce too little contrast on the lower border
+				p->setPen( cg.dark() );
+				p->drawLine( x, y, x + w - 1, y );
+				p->drawLine( x, y, x, y + h - 1 );
+				
+				p->setPen( cg.light().dark( 110 ) );
+				p->drawLine( x + w - 1, y, x + w - 1, y + h - 1 );
+				p->drawLine( x, y + h - 1, x + w - 1, y + h - 1);
+			}
+			else
+			{
+				p->setPen( cg.dark() );
+				p->drawLine( x, y, x + w - 1, y );
+				p->drawLine( x, y, x, y + h - 1 );
+				p->setPen( cg.mid() );
+				p->drawLine( x + 1, y + 1, x + w - 1, y + 1 );
+				p->drawLine( x + 1, y + 1, x + 1, y + h - 1 );
+				p->setPen( cg.light() );
+				p->drawLine( x + w - 1, y, x + w - 1, y + h - 1 );
+				p->drawLine( x, y + h - 1, x + w - 1, y + h - 1);
+				p->setPen( cg.light().dark( 110 ) );
+				p->drawLine( x + w - 2, y + 1, x + w - 2, y + h - 2 );
+				p->drawLine( x + 1, y + h - 2, x + w - 2, y + h - 2);
+			}
 			break;
 		}
 
@@ -1209,6 +1275,28 @@ void KeramikStyle::drawKStylePrimitive( KStylePrimitive kpe,
 	}
 }
 
+bool KeramikStyle::isFormWidget(const QWidget* widget) const
+{
+	//Form widgets are in the KHTMLView, but that has 2 further inner levels
+	//of widgets - QClipperWidget, and outside of that, QViewportWidget
+	QWidget* potentialClipPort = widget->parentWidget();
+	if ( !potentialClipPort || potentialClipPort->isTopLevel() )
+	return false;
+
+	QWidget* potentialViewPort = potentialClipPort->parentWidget();
+	if (!potentialViewPort || potentialViewPort->isTopLevel() ||
+			qstrcmp(potentialViewPort->name(), "qt_viewport") )
+		return false;
+
+	QWidget* potentialKHTML  = potentialViewPort->parentWidget();
+	if (!potentialKHTML || potentialKHTML->isTopLevel() ||
+			qstrcmp(potentialKHTML->className(), "KHTMLView") )
+		return false;
+
+
+	return true;
+}
+
 void KeramikStyle::drawControl( ControlElement element,
 								  QPainter *p,
 								  const QWidget *widget,
@@ -1228,6 +1316,10 @@ void KeramikStyle::drawControl( ControlElement element,
 		case CE_PushButton:
 		{
 			const QPushButton* btn = static_cast< const QPushButton* >( widget );
+
+			if (isFormWidget(btn))
+				formMode = true;
+
 			if ( widget == hoverWidget )
 				flags |= Style_MouseOver;
 
@@ -1244,12 +1336,10 @@ void KeramikStyle::drawControl( ControlElement element,
 					toolbarBlendWidget = widget;
 
 				drawPrimitive( PE_ButtonCommand, p, r, cg, flags );
-
 				toolbarBlendWidget = 0;
 			}
 
-			flatMode = false;
-
+			formMode = false;
 			break;
 		}
 
@@ -1350,8 +1440,8 @@ void KeramikStyle::drawControl( ControlElement element,
 								r.x() + r.width() - 1, r.y() + r.height() - 1 );
 					}
 				}
-				else if ( w > smallButMaxW && h > smallButMaxH )
-					nr.setWidth(r.width()-2); //Account for shadow
+				//else if ( w > smallButMaxW && h > smallButMaxH )
+				//	nr.setWidth(r.width()-2); //Account for shadow
 			}
 
 			KStyle::drawControl(element, p, widget, nr, cg, flags, opt);
@@ -1384,13 +1474,13 @@ void KeramikStyle::drawControl( ControlElement element,
 
 				if ( bottom )
 				{
-					Keramik::InactiveTabPainter( mode, bottom ).draw( p, x, y, w, h - 4, cg.button(), cg.background(), disabled, pmode() );
+					Keramik::InactiveTabPainter( mode, bottom ).draw( p, x, y, w, h - 3, cg.button(), cg.background(), disabled, pmode() );
 					p->setPen( cg.dark() );
 					p->drawLine( x, y, x + w, y );
 				}
 				else
 				{
-					Keramik::InactiveTabPainter( mode, bottom ).draw( p, x, y + 4, w, h - 4, cg.button(), cg.background(), disabled, pmode() );
+					Keramik::InactiveTabPainter( mode, bottom ).draw( p, x, y + 3, w, h - 3, cg.button(), cg.background(), disabled, pmode() );
 					p->setPen( cg.light() );
 					p->drawLine( x, y + h - 1, x + w, y + h - 1 );
 				}
@@ -1501,9 +1591,9 @@ void KeramikStyle::drawControl( ControlElement element,
 			if ( mi->isSeparator() )
 			{
 				p->setPen( cg.mid() );
-				p->drawLine( main.x()+5, main.y() + 1, main.width()-10, main.y() + 1 );
+				p->drawLine( main.x()+5, main.y() + 1, main.right()-5, main.y() + 1 );
 				p->setPen( cg.light() );
-				p->drawLine( main.x()+5, main.y() + 2, main.width()-10 , main.y() + 2 );
+				p->drawLine( main.x()+5, main.y() + 2, main.right()-5, main.y() + 2 );
 				break;
 			}
 
@@ -1704,21 +1794,75 @@ void KeramikStyle::drawControl( ControlElement element,
 						pstep = - (pstep - 2 * remWidth );
 					}
 
+					//Store the progress rect.
+					QRect progressRect;
 					if (reverse)
-						Keramik::RowPainter(keramik_progressbar).draw(p, cr.x() + cr.width() - width - pstep, cr.y(), width, cr.height(),
-									 cg.highlight(), cg.background() );
+						progressRect = QRect(cr.x() + cr.width() - width - pstep, cr.y(),
+										width, cr.height());
 					else
-						Keramik::RowPainter(keramik_progressbar).draw(p, cr.x() + pstep, cr.y(), width, cr.height(),
-									cg.highlight(), cg.background() );
+						progressRect = QRect(cr.x() + pstep, cr.y(), width, cr.height());
 
+					Keramik::RowPainter(keramik_progressbar).draw(p, progressRect,
+									 cg.highlight(), cg.background() );
 					return;
 				}
 
+				QRect progressRect;
 
 				if (reverse)
-					Keramik::ProgressBarPainter(keramik_progressbar, reverse).draw( p, cr.x()+(cr.width()-width), cr.y(), width, cr.height(), cg.highlight(), cg.background());
+					progressRect = QRect(cr.x()+(cr.width()-width), cr.y(), width, cr.height());
 				else
-					Keramik::ProgressBarPainter(keramik_progressbar, reverse).draw( p, cr.x(), cr.y(), width, cr.height(), cg.highlight(), cg.background());
+					progressRect = QRect(cr.x(), cr.y(), width, cr.height());
+
+				//Apply the animation rectangle.
+				//////////////////////////////////////
+				if (animateProgressBar)
+				{
+					if (reverse)
+					{
+						//Here, we can't simply shift, as the painter code calculates everything based
+						//on the left corner, so we need to draw the 2 portions ourselves.
+
+						//Start off by checking the geometry of the end pixmap - it introduces a bit of an offset
+						QSize   endDim = loader.size(keramik_progressbar + 3); //3 = 3*1 + 0 = (1,0) = cl
+
+						//We might not have anything to animate at all, though, if the ender is the only thing to paint
+						if (endDim.width() < progressRect.width())
+						{
+							//OK, so we do have some stripes.
+							// Render the endline now - the clip region below will protect it from getting overwriten
+							QPixmap endline = loader.scale(keramik_progressbar + 3, endDim.width(), progressRect.height(),
+															cg.highlight(), cg.background());
+							p->drawPixmap(progressRect.x(), progressRect.y(), endline);
+
+							//Now, calculate where the stripes should be, and set a clip region to that
+							progressRect.setLeft(progressRect.x() + endline.width());
+							p->setClipRect(progressRect, QPainter::CoordPainter);
+
+							//Expand the paint region slightly to get the animation offset.
+							progressRect.setLeft(progressRect.x() - progAnimShift);
+
+							//Paint the stripes.
+							QPixmap stripe = loader.scale(keramik_progressbar + 4, 28, progressRect.height(),
+														  cg.highlight(), cg.background());
+														  //4 = 3*1 + 1 = (1,1) = cc
+
+							p->drawTiledPixmap(progressRect,  stripe);
+							//Exit out here to skip the regular paint path
+							return;
+						}
+					}
+					else
+					{
+						//Clip to the old rectangle
+						p->setClipRect(progressRect, QPainter::CoordPainter);
+						//Expand the paint region
+						progressRect.setLeft(progressRect.x() - 28 + progAnimShift);
+					}
+				}
+
+				Keramik::ProgressBarPainter(keramik_progressbar, reverse).draw( p,
+							progressRect , cg.highlight(), cg.background());
 			}
 			break;
 		}
@@ -1741,6 +1885,18 @@ void KeramikStyle::drawControlMask( ControlElement element,
 	maskMode = false;
 }
 
+bool KeramikStyle::isSizeConstrainedCombo(const QComboBox* combo) const
+{
+	if (combo->width() >= 80)
+		return false;
+	int suggestedWidth = combo->sizeHint().width();
+	
+	if (combo->width() - suggestedWidth < -5)
+		return true;
+
+	return false;
+}
+
 void KeramikStyle::drawComplexControl( ComplexControl control,
                                          QPainter *p,
                                          const QWidget *widget,
@@ -1758,72 +1914,111 @@ void KeramikStyle::drawComplexControl( ComplexControl control,
 		// -------------------------------------------------------------------
 		case CC_ComboBox:
 		{
-			bool toolbarMode = false;
-			const QComboBox* cb = static_cast< const QComboBox* >( widget );
+			bool             toolbarMode = false;
+			const QComboBox* cb          = static_cast< const QComboBox* >( widget );
+			bool             compact     = isSizeConstrainedCombo(cb);
+
+			if (isFormWidget(cb))
+				formMode = true;
+
 			QPixmap * buf = 0;
 			QPainter* p2 = p;
 
 			QRect br = r;
-			
-			if (widget->parent() &&
-					( widget->parent()->inherits("QToolBar")|| !qstrcmp(widget->parent()->name(), kdeToolbarWidget) ) )
-				toolbarMode = true;
 
 			if (controls == SC_All)
 			{
-				buf = new QPixmap( r.width(), r.height() );
-				br.setX(0);
-				br.setY(0);
-				p2 = new QPainter(buf);
-				
-				//Ensure that we have clipping on, and have a sane base.
-				//If need be, Qt will shrink us to the paint region.
-				p->setClipping(true);
-				p->setClipRect(r);
+				//Double-buffer only when we are in the slower full-blend mode
+				if (widget->parent() &&
+						( widget->parent()->inherits("QToolBar")|| !qstrcmp(widget->parent()->name(), kdeToolbarWidget) ) )
+				{
+					buf = new QPixmap( r.width(), r.height() );
+					br.setX(0);
+					br.setY(0);
+					p2 = new QPainter(buf);
+					
+					//Ensure that we have clipping on, and have a sane base.
+					//If need be, Qt will shrink us to the paint region.
+					p->setClipping(true);
+					p->setClipRect(r);
+					toolbarMode = true;
+				}
 			}
 
 
-			if ( br.width() >= 28 && br.height() > 20 ) br.addCoords( 0, -2, 0, 0 );
+			if ( br.width() >= 28 && br.height() > 20 && !compact )
+				br.addCoords( 0, -2, 0, 0 );
+				
+			//When in compact mode, we force the shadow-less bevel mode,
+			//but that also alters height and not just width.
+			//readjust height to fake the other metrics (plus clear 
+			//the other areas, as appropriate). The automasker
+			//will take care of the overall shape.
+			if ( compact )
+			{
+				forceSmallMode = true;
+				br.setHeight( br.height() - 2);
+				p->fillRect ( r.x(), r.y() + br.height(), r.width(), 2, cg.background());
+			}
+				
+				
 			if ( controls & SC_ComboBoxFrame )
 			{
 				if (toolbarMode)
-				{
 					toolbarBlendWidget = widget;
-				}
 
 				if ( widget == hoverWidget )
 					flags |= Style_MouseOver;
-
+					
 				drawPrimitive( PE_ButtonCommand, p2, br, cg, flags );
 
 				toolbarBlendWidget = 0;
 			}
 
 			// don't draw the focus rect etc. on the mask
-			if ( cg.button() == color1 && cg.background() == color0 ) break;
+			if ( cg.button() == color1 && cg.background() == color0 )
+				break;
 
 			if ( controls & SC_ComboBoxArrow )
 			{
-				if ( active ) flags |= Style_On;
+				if ( active )
+					flags |= Style_On;
+					
+				QRect ar = querySubControlMetrics( CC_ComboBox, widget,
+													 SC_ComboBoxArrow );
+				if (!compact)
+				{
+					ar.setWidth(ar.width()-13);
+					QRect rr = visualRect( QRect( ar.x(), ar.y() + 4,
+							loader.size(keramik_ripple ).width(), ar.height() - 8 ),
+							widget );
 
-				QRect ar = querySubControlMetrics( CC_ComboBox, widget, SC_ComboBoxArrow );
-				ar.setWidth(ar.width()-13);
+					ar = visualRect( QRect( ar.x() + loader.size( keramik_ripple ).width() + 4, ar.y(), 
+											11, ar.height() ), 
+									widget );
 
-				QRect rr = visualRect( QRect( ar.x(), ar.y() + 4, loader.size(
-keramik_ripple ).width(), ar.height() - 8 ), widget );
-
-				ar = visualRect( QRect( ar.x() + loader.size( keramik_ripple ).width() +
-4, ar.y(), 11, ar.height() ), widget );
-
-				QPointArray a;
-
-				a.setPoints(QCOORDARRLEN(keramik_combo_arrow), keramik_combo_arrow);
-
-				a.translate( ar.x() + ar.width() / 2, ar.y() + ar.height() / 2 );
-				p2->setPen( cg.buttonText() );
-				p2->drawLineSegments( a );
-
-				Keramik::ScaledPainter( keramik_ripple ).draw( p2, rr, cg.button(), Qt::black, disabled, Keramik::TilePainter::PaintFullBlend );
+					QPointArray a;
+	
+					a.setPoints(QCOORDARRLEN(keramik_combo_arrow), keramik_combo_arrow);
+	
+					a.translate( ar.x() + ar.width() / 2, ar.y() + ar.height() / 2 );
+					p2->setPen( cg.buttonText() );
+					p2->drawLineSegments( a );
+	
+					Keramik::ScaledPainter( keramik_ripple ).draw( p2, rr, cg.button(), Qt::black, disabled, Keramik::TilePainter::PaintFullBlend );
+				}
+				else //Size-constrained combo -- loose the ripple.
+				{
+					ar.setWidth(ar.width() - 7);
+					ar = visualRect( QRect( ar.x(), ar.y(), 11, ar.height() ), widget);
+					QPointArray a;
+	
+					a.setPoints(QCOORDARRLEN(keramik_combo_arrow), keramik_combo_arrow);
+	
+					a.translate( ar.x() + ar.width() / 2, ar.y() + ar.height() / 2 );
+					p2->setPen( cg.buttonText() );
+					p2->drawLineSegments( a );
+				}
 			}
 
 			if ( controls & SC_ComboBoxEditField )
@@ -1840,11 +2035,13 @@ keramik_ripple ).width(), ar.height() - 8 ), widget );
 				else if ( cb->hasFocus() )
 				{
 					QRect re = QStyle::visualRect(subRect(SR_ComboBoxFocusRect, cb), widget);
+					if ( compact )
+						re.addCoords( 3, 3, 0, -3 );
 					p2->fillRect( re, cg.brush( QColorGroup::Highlight ) );
 					drawPrimitive( PE_FocusRect, p2, re, cg,
 					Style_FocusAtBorder, QStyleOption( cg.highlight() ) );
 				}
-				// QComboBox draws the text on its own and uses the painter's current colours
+				// QComboBox draws the text on its own and uses the painter's current colors
 				if ( cb->hasFocus() )
 				{
 					p->setPen( cg.highlightedText() );
@@ -1864,6 +2061,8 @@ keramik_ripple ).width(), ar.height() - 8 ), widget );
 				p->drawPixmap(r.x(), r.y(), *buf);
 				delete buf;
 			}
+
+			formMode = false;
 			break;
 		}
 
@@ -1892,11 +2091,10 @@ keramik_ripple ).width(), ar.height() - 8 ), widget );
 		case CC_ScrollBar:
 		{
 			const QScrollBar* sb = static_cast< const QScrollBar* >( widget );
-			if (sb->parentWidget())
+			if (highlightScrollBar && sb->parentWidget()) //Don't do the check if not highlighting anyway
 			{
-				if (sb->parentWidget()->colorGroup().button() !=
-					sb->colorGroup().button())
-						customScrollMode = true;
+				if (sb->parentWidget()->colorGroup().button() != sb->colorGroup().button())
+					customScrollMode = true;
 			}
 			bool horizontal = sb->orientation() == Horizontal;
 			QRect slider, subpage, addpage, subline, addline;
@@ -1991,6 +2189,10 @@ keramik_ripple ).width(), ar.height() - 8 ), widget );
 			if (active & SC_ToolButtonMenu)
 				mflags |= Style_Down;
 
+			if ( widget == hoverWidget )
+				bflags |= Style_MouseOver;
+
+
 			if (onToolbar &&  static_cast<QToolBar*>(widget->parent())->orientation() == Qt::Horizontal)
 				bflags |= Style_Horizontal;
 
@@ -1998,8 +2200,14 @@ keramik_ripple ).width(), ar.height() - 8 ), widget );
 			{
 				// If we're pressed, on, or raised...
 				if (bflags & (Style_Down | Style_On | Style_Raised) || onControlButtons)
-					drawPrimitive(onToolbar || onControlButtons ? PE_ButtonTool : PE_ButtonCommand, p, button, cg,
+				{
+					//Make sure the standalone toolbuttons have a gradient in the right direction
+					if (!onToolbar && !onControlButtons)
+						bflags |= Style_Horizontal;
+
+					drawPrimitive( PE_ButtonTool, p, button, cg,
 					 bflags, opt);
+				}
 
 				// Check whether to draw a background pixmap
 				else if ( toolbutton->parentWidget() &&
@@ -2095,9 +2303,6 @@ int KeramikStyle::pixelMetric(PixelMetric m, const QWidget *widget) const
 		case PM_ButtonMargin:				// Space btw. frame and label
 			return 4;
 
-		case PM_ButtonDefaultIndicator:
-			return 4;
-
 		case PM_SliderLength:
 			return 12;
 		case PM_SliderControlThickness:
@@ -2139,6 +2344,20 @@ int KeramikStyle::pixelMetric(PixelMetric m, const QWidget *widget) const
 
 		case PM_TabBarTabOverlap:
 			return 0;
+			
+		case PM_TabBarTabShiftVertical:
+		{
+			const QTabBar* tb = ::qt_cast<const QTabBar*>(widget);
+			if (tb)
+			{
+				if (tb->shape() == QTabBar::RoundedBelow || 
+					tb->shape() == QTabBar::TriangularBelow)
+					return 0;
+			}
+			
+			return 2; //For top, or if not sure
+		}
+			
 
 		case PM_TitleBarHeight:
 			return titleBarH;
@@ -2237,7 +2456,7 @@ QSize KeramikStyle::sizeFromContents( ContentsType contents,
 			}
 
 			if ( ! mi->text().isNull() && mi->text().find('\t') >= 0 )
-				w += 12;
+				w += itemHMargin + itemFrame*2 + 7;
 			else if ( mi->popup() )
 				w += 2 * arrowHMargin;
 
@@ -2252,7 +2471,7 @@ QSize KeramikStyle::sizeFromContents( ContentsType contents,
 
 			return QSize( w, h );
 		}
-
+		
 		default:
 			return KStyle::sizeFromContents( contents, widget, contentSize, opt );
 	}
@@ -2286,15 +2505,29 @@ QRect KeramikStyle::querySubControlMetrics( ComplexControl control,
 	{
 		case CC_ComboBox:
 		{
-			int arrow = 11 + loader.size( keramik_ripple ).width();
+			int arrow;
+			bool compact = false;
+			if ( isSizeConstrainedCombo(static_cast<const QComboBox*>(widget) ) ) //### constant
+				compact = true;
+				
+			if ( compact )
+				arrow = 11;
+			else
+				arrow = 11 + loader.size( keramik_ripple ).width();
+				
 			switch ( subcontrol )
 			{
 				case SC_ComboBoxArrow:
-					return QRect( widget->width() - arrow - 14, 0, arrow+13, widget->height() );
+					if ( compact )
+						return QRect( widget->width() - arrow - 7, 0, arrow + 6, widget->height() );
+					else
+						return QRect( widget->width() - arrow - 14, 0, arrow + 13, widget->height() );
 
 				case SC_ComboBoxEditField:
 				{
-					if ( widget->width() < 36 || widget->height() < 22 )
+					if ( compact )
+						return QRect( 2, 4, widget->width() - arrow - 2 - 7, widget->height() - 8 );
+					else if ( widget->width() < 36 || widget->height() < 22 )
 						return QRect( 4, 3, widget->width() - arrow - 20, widget->height() - 6 );
 					else if ( static_cast< const QComboBox* >( widget )->editable() )
 						return QRect( 8, 4, widget->width() - arrow - 26, widget->height() - 11 );
@@ -2304,10 +2537,14 @@ QRect KeramikStyle::querySubControlMetrics( ComplexControl control,
 
 				case SC_ComboBoxListBoxPopup:
 				{
+					//Note that the widget here == the combo, not the completion
+					//box, so we don't get any recursion
+					int suggestedWidth = widget->sizeHint().width(); 
 					QRect def = opt.rect();
-					if ( widget->inherits( "KCompletionBox" ) )
-						def.addCoords( -4, 4, 10, 8 );
-					else def.addCoords( 4, -4, -6, 4 );
+					def.addCoords( 4, -4, -6, 4 );
+					
+					if ((def.width() - suggestedWidth < -12) && (def.width() < 80))
+						def.setWidth(QMIN(80, suggestedWidth - 10));
 
 					return def;
 				}
@@ -2443,11 +2680,16 @@ QRect KeramikStyle::querySubControlMetrics( ComplexControl control,
 
 #include <config.h>
 
+
+#if defined Q_WS_X11 && ! defined K_WS_QTONLY
 #ifdef HAVE_X11_EXTENSIONS_SHAPE_H
 //Xlib headers are a mess -> include them down here (any way to ensure that we go second in enable-final order?)
-#include  <X11/Xlib.h>
-#include <X11/extensions/shape.h>
+#include  <X11/Xlib.h> // schroder
+#include <X11/extensions/shape.h> // schroder
 #endif
+#else
+#undef HAVE_X11_EXTENSIONS_SHAPE_H
+#endif //QTONLY
 
 bool KeramikStyle::eventFilter( QObject* object, QEvent* event )
 {
@@ -2456,25 +2698,29 @@ bool KeramikStyle::eventFilter( QObject* object, QEvent* event )
 
 	if ( !object->isWidgetType() ) return false;
 
-	//Hover highlight on buttons and combos
-	if ( object->inherits("QPushButton") || object->inherits("QComboBox") )
-	{
-		if ( (event->type() == QEvent::Enter) && static_cast<QWidget*>(object)->isEnabled())
+	//Clear hover highlight when needed
+	if ( (event->type() == QEvent::Leave) && (object == hoverWidget) )
 		{
 			QWidget* button = static_cast<QWidget*>(object);
-			hoverWidget = button;
+		hoverWidget = 0;
 			button->repaint( false );
+		return false;
 		}
-		else if ( (event->type() == QEvent::Leave) && (object == hoverWidget) )
+
+	//Hover highlight on buttons, toolbuttons and combos
+	if ( object->inherits("QPushButton") || object->inherits("QComboBox") || object->inherits("QToolButton") )
 		{
-			QWidget* button = static_cast<QWidget*>(object);
-			hoverWidget = 0;
-			button->repaint( false );
+		if (event->type() == QEvent::Enter && static_cast<QWidget*>(object)->isEnabled() )
+		{
+			hoverWidget = static_cast<QWidget*>(object);
+			hoverWidget->repaint( false );
+
 		}
 		return false;
 	}
+
 	//Combo line edits get special frames
-	else if ( event->type() == QEvent::Paint && object->inherits( "QLineEdit" ) )
+	if ( event->type() == QEvent::Paint && object->inherits( "QLineEdit" ) )
 	{
 		static bool recursion = false;
 		if (recursion )
@@ -2582,3 +2828,4 @@ bool KeramikStyle::eventFilter( QObject* object, QEvent* event )
 }
 
 // vim: ts=4 sw=4 noet
+// kate: indent-width 4; replace-tabs off; tab-width 4; space-indent off;
