@@ -27,7 +27,7 @@
    Boston, MA 02111-1307, USA.
 */
 
-// $Id: keramik.h,v 1.18.2.1 2003/06/20 01:29:06 orlovich Exp $
+// $Id: keramik.h,v 1.24.2.1 2004/03/25 13:35:08 lukas Exp $
 
 #ifndef __keramik_h__
 #define __keramik_h__
@@ -114,13 +114,26 @@ public:
 	                              const QWidget* widget,
 	                              SubControl subcontrol,
 	                              const QStyleOption& opt = QStyleOption::Default ) const;
-	// Fix Qt3's wacky image positions
-/*	QPixmap stylePixmap( StylePixmap stylepixmap,
-					const QWidget* widget = 0,
-					const QStyleOption& = QStyleOption::Default ) const;*/
 
-protected:
-	mutable bool maskMode; //Ugly round trip flag to permit masking with little code;
+private slots:
+	//Animation slots.
+	void updateProgressPos();
+	void progressBarDestroyed(QObject* bar);
+
+private:
+
+	bool isSizeConstrainedCombo(const QComboBox* combo) const;
+	bool isFormWidget          (const QWidget*   widget) const;
+
+	///Configuration settings
+	bool animateProgressBar;
+	bool highlightScrollBar;
+
+	//Rendering flags
+	mutable bool forceSmallMode;
+	mutable bool maskMode;   //Ugly round trip flag to permit masking with little code;
+	mutable bool formMode;   //Set when rendering form widgets
+
 	mutable const QWidget* toolbarBlendWidget;  //Ditto for blending with toolbars
 
 	enum TitleBarMode
@@ -129,36 +142,54 @@ protected:
 		Regular,
 		Maximized
 	};
-	
+
 	mutable TitleBarMode titleBarMode; //Set when passing back CC_TilteBar modes to handle
 	//PE_ButtonTool properly for them, as well as when handling CC_ToolButton from
 	//The maximized window controls.
-	
+
 	mutable bool flatMode; //Set when calling PE_PushButton or PE_ButtonDefault
 	// on a flat button.
 
 	mutable bool customScrollMode; //Set when drawing scrollbars with custom colors.
 
-	
+	//Animation support.
+	QMap<QWidget*, bool> progAnimWidgets;
+	int                  progAnimShift;
+
 	bool eventFilter( QObject* object, QEvent* event );
-	
+
 	Keramik::TilePainter::PaintMode pmode() const
 	{
-		return maskMode?Keramik::TilePainter::PaintMask : Keramik::TilePainter::PaintNormal;
+		if (formMode)
+		{
+			//If we're a form widget, we blend on painting, and consider ourselves
+			//not to have a mask (so we don't get clipped to it)
+			if (maskMode)
+				return Keramik::TilePainter::PaintTrivialMask;
+			else
+				return Keramik::TilePainter::PaintFullBlend;
+		}
+		else
+		{
+			if (maskMode)
+				return Keramik::TilePainter::PaintMask;
+			else
+				return Keramik::TilePainter::PaintNormal;
+		}
 	}
-	
+
 	Keramik::TilePainter::PaintMode pmodeFullBlend() const
 	{
 		return maskMode?Keramik::TilePainter::PaintMask : Keramik::TilePainter::PaintFullBlend;
 	}
 
-	
 	QWidget* hoverWidget;
-private:
+
+
 	bool kickerMode;
-	
+
 	QRect subRect(SubRect r, const QWidget *widget) const;
-	
+
 	// Disable copy constructor and = operator
 	KeramikStyle( const KeramikStyle&  );
 	KeramikStyle& operator=( const KeramikStyle&  );
@@ -167,3 +198,4 @@ private:
 #endif
 
 // vim: ts=4 sw=4 noet
+// kate: indent-width 4; replace-tabs off; tab-width 4;
