@@ -1,5 +1,5 @@
 /*
- $Id: kthemestyle.cpp,v 1.34.2.3 2003/07/15 00:40:52 mueller Exp $
+ $Id: kthemestyle.cpp,v 1.43 2004/01/11 20:21:25 orlovich Exp $
 
  This file is part of the KDE libraries
  Copyright (C) 1999 Daniel M. Duley <mosfet@kde.org>
@@ -38,11 +38,8 @@ Port version 0.9.7
  Boston, MA 02111-1307, USA.
 */
 
-#include <qglobal.h>
-
-
-#include <kthemestyle.h>
-#include <kthemebase.h>
+#include "kthemestyle.h"
+#include "kthemebase.h"
 #include <qstyleplugin.h>
 #include <qstylefactory.h>
 #include <kimageeffect.h>
@@ -55,7 +52,6 @@ Port version 0.9.7
 #include <qpalette.h>
 #include <qtabbar.h>
 #include <qtoolbutton.h>
-#include <qglobal.h>
 #include <kglobalsettings.h>
 #include <kdrawutil.h>
 #include <qdrawutil.h>
@@ -346,7 +342,7 @@ int KThemeStyle::pixelMetric ( PixelMetric metric, const QWidget * widget ) cons
                    3 + decoWidth( PushButton ) : 3 + decoWidth( PushButtonDown );
 
         case PM_ScrollBarExtent:
-        case PM_SliderThickness:  //Should this be 16 allways?
+        case PM_SliderThickness:  //Should this be 16 always?
             return getSBExtent();
 
         case PM_ButtonDefaultIndicator:
@@ -398,7 +394,7 @@ int KThemeStyle::pixelMetric ( PixelMetric metric, const QWidget * widget ) cons
 
 KThemeStyle::KThemeStyle( const QString& configDir, const QString &configFile )
         : KThemeBase( configDir, configFile ), paletteSaved( false ), polishLock( false ), menuCache( 0 ), vsliderCache( 0 ),
-         brushHandle( 0 ), brushHandleSet( false )
+         brushHandle( 0 ), brushHandleSet( false ), kickerMode( false )
 {
     mtfstyle = QStyleFactory::create( "Motif" );
     if ( !mtfstyle )
@@ -413,8 +409,11 @@ KThemeStyle::~KThemeStyle()
 }
 
 
-void KThemeStyle::polish( QApplication * /*app*/ )
-{}
+void KThemeStyle::polish( QApplication * app )
+{
+    if (!qstrcmp(app->argv()[0], "kicker"))
+        kickerMode = true;
+}
 
 
 void KThemeStyle::polish( QPalette &p )
@@ -505,7 +504,9 @@ void KThemeStyle::polish( QWidget *w )
          w->installEventFilter(this);
 
 
-    if (w->backgroundPixmap() && !w->isTopLevel())
+    if (w->backgroundPixmap() && !w->isTopLevel() && 
+        (!kickerMode || 
+        (!w->inherits("TaskBar") && !w->inherits("TaskBarContainer") && !w->inherits("TaskbarApplet") && !w->inherits("ContainerArea") && !w->inherits("AppletHandle"))))
     {
         //The brushHandle check verifies that the bg pixmap is actually the brush..
         if (!brushHandleSet || brushHandle ==w->backgroundPixmap()->handle())
@@ -717,7 +718,7 @@ void KThemeStyle::drawPrimitive ( PrimitiveElement pe, QPainter * p, const QRect
                     r.setRect( r.x(), r.y() + ( r.height() - 12 ) / 2, r.width(), 12 );
                 }
                 r.rect( &x, &y, &w, &h );
-                // Handles pixmapped arrows. A little inefficent because you can specify
+                // Handles pixmapped arrows. A little inefficient because you can specify
                 // some as pixmaps and some as default types.
                 WidgetType widget;
                 switch ( pe )
@@ -1658,7 +1659,7 @@ void KThemeStyle::drawControl( ControlElement element,
             {
                 QBrush bg;
                 const QColorGroup * cg2 = colorGroup( cg, ProgressBg );
-                qDrawWinPanel( p, r, *cg2, TRUE );
+                qDrawWinPanel( p, r, *cg2, true );
                 bg.setColor( cg2->color( QColorGroup::Background ) );
                 if ( isPixmap( ProgressBg ) )
                     bg.setPixmap( *uncached( ProgressBg ) );
@@ -2367,10 +2368,5 @@ int KThemeStyle::popupMenuItemHeight( bool /*checkable*/, QMenuItem *mi,
     return ( h );
 }
 
-
-
-
 #include "kthemestyle.moc"
-
-
-
+// kate: indent-width 4; replace-tabs off; tab-width 4; space-indent on;
