@@ -29,6 +29,7 @@
 #include "qframe.h"
 #include "qpushbutton.h"
 #include "qdrawutil.h"
+#include "qprogressbar.h"
 #include "qscrollbar.h"
 #include "qtabbar.h"
 #include "qguardedptr.h"
@@ -140,13 +141,11 @@ void LightStyleV2::drawPrimitive( PrimitiveElement pe,
 	{
 	    flags = ((flags | Style_Sunken) ^ Style_Sunken) | Style_Raised; 
 	    	//Don't show pressed too often (as in light 3)
-	    const QBrush *fill;
-	    if (flags & QStyle::Style_Enabled) {
-		    fill = &cg.brush(QColorGroup::Button);
-	    } else
-		fill = &cg.brush(QColorGroup::Background);
-
-	    drawLightBevel(p, r, cg, flags, fill);
+	    QBrush fill(cg.background());
+	    if (flags & QStyle::Style_Enabled)
+		fill.setColor(cg.button());
+	    
+	    drawLightBevel(p, r, cg, flags, &fill);
 	    p->setPen( cg.buttonText() );
 	    break;
 	}
@@ -292,12 +291,12 @@ void LightStyleV2::drawPrimitive( PrimitiveElement pe,
     case PE_DockWindowHandle:
 	{
 	    QString title;
-	    bool drawTitle = FALSE;
+	    bool drawTitle = false;
 	    if ( p && p->device()->devType() == QInternal::Widget ) {
 		QWidget *w = (QWidget *) p->device();
 		QWidget *p = w->parentWidget();
 		if (p->inherits("QDockWindow") && ! p->inherits("QToolBar")) {
-		    drawTitle = TRUE;
+		    drawTitle = true;
 		    title = p->caption();
 		}
 	    }
@@ -444,7 +443,7 @@ void LightStyleV2::drawPrimitive( PrimitiveElement pe,
 
     case PE_ScrollBarSubLine:
 	{
-	    QRect fr = r, ar = r;;
+	    QRect fr = r, ar = r;
 	    PrimitiveElement pe;
 
 	    p->setPen(cg.dark());
@@ -787,7 +786,7 @@ void LightStyleV2::drawControl( ControlElement control,
 	    }
 
 	    if (flags & Style_Active)
-		qDrawShadePanel(p, r, cg, TRUE, 1,
+		qDrawShadePanel(p, r, cg, true, 1,
 				&cg.brush(QColorGroup::Midlight));
 	    else
 		p->fillRect(r, cg.brush(QColorGroup::Button));
@@ -818,7 +817,7 @@ void LightStyleV2::drawControl( ControlElement control,
 	    if (mi->isChecked() &&
 		! (flags & Style_Active) &
 		(flags & Style_Enabled))
-		qDrawShadePanel(p, cr, cg, TRUE, 1, &cg.brush(QColorGroup::Midlight));
+		qDrawShadePanel(p, cr, cg, true, 1, &cg.brush(QColorGroup::Midlight));
 
 	    if (mi->iconSet()) {
 		QIconSet::Mode mode =
@@ -933,7 +932,7 @@ void LightStyleV2::drawControl( ControlElement control,
     case CE_MenuBarItem:
 	{
 	    if (flags & Style_Active)
-		qDrawShadePanel(p, r, cg, TRUE, 1, &cg.brush(QColorGroup::Midlight));
+		qDrawShadePanel(p, r, cg, true, 1, &cg.brush(QColorGroup::Midlight));
 	    else
 		p->fillRect(r, cg.brush(QColorGroup::Button));
 
@@ -1546,6 +1545,24 @@ QSize LightStyleV2::sizeFromContents( ContentsType contents,
 	    ret = QSize(w, h);
 	    break;
 	}
+    case CT_ProgressBar:
+	{
+	    const QProgressBar* pb = static_cast<const QProgressBar*>(widget);
+	    
+	    //If we have to display the indicator, and we do it on RHS, give some more room
+	    //for it. This tries to match the logic and the spacing in SR_ProgressBarGroove/Contents
+	    //sizing in QCommonStyle.
+	    if (pb->percentageVisible() && 
+	        (pb->indicatorFollowsStyle() || ! pb->centerIndicator()))
+	    {
+		int addw = pb->fontMetrics().width("100%" + 6);
+		return QSize(contentsSize.width() + addw, contentsSize.height());
+	    }
+	    else
+	    	return contentsSize; //Otherwise leave unchanged
+	    
+	    break;
+	}    
 
     default:
 	ret = QCommonStyle::sizeFromContents(contents, widget, contentsSize, data);
