@@ -30,6 +30,7 @@
 #include "qframe.h"
 #include "qpushbutton.h"
 #include "qdrawutil.h"
+#include "qprogressbar.h"
 #include "qscrollbar.h"
 #include "qtabbar.h"
 #include "qguardedptr.h"
@@ -44,7 +45,7 @@
 // The Light Style, 3rd revision
 
 LightStyleV3::LightStyleV3()
-    : QCommonStyle()
+    : KStyle(AllowMenuTransparency)
 {
     basestyle = QStyleFactory::create( "Windows" );
     if ( ! basestyle )
@@ -58,8 +59,9 @@ LightStyleV3::~LightStyleV3()
     delete basestyle;
 }
 
-void LightStyleV3::polishPopupMenu( QPopupMenu * )
+void LightStyleV3::polishPopupMenu( QPopupMenu * menu)
 {
+    KStyle::polishPopupMenu(menu);
     // empty to satisy pure virtual requirements
 }
 
@@ -238,8 +240,8 @@ void LightStyleV3::drawPrimitive( PrimitiveElement pe,
 	// draw the header ( just an etching )
 	if ( ! br.isValid() )
 	    break;
-	drawLightEtch( p, br, ( ( flags & Style_Down ) ? 
-                                cg.midlight() : cg.button() ), 
+	drawLightEtch( p, br, ( ( flags & Style_Down ) ?
+                                cg.midlight() : cg.button() ),
                        ( flags & Style_Down ) );
 	br.addCoords( 1, 1, -1, -1 );
 
@@ -421,12 +423,12 @@ void LightStyleV3::drawPrimitive( PrimitiveElement pe,
     case PE_DockWindowHandle:
 	{
 	    QString title;
-	    bool drawTitle = FALSE;
+	    bool drawTitle = false;
 	    if ( p && p->device()->devType() == QInternal::Widget ) {
 		QWidget *w = (QWidget *) p->device();
 		QWidget *p = w->parentWidget();
 		if (p->inherits("QDockWindow") && ! p->inherits("QToolBar")) {
-		    drawTitle = TRUE;
+		    drawTitle = true;
 		    title = p->caption();
 		}
 	    }
@@ -451,7 +453,7 @@ void LightStyleV3::drawPrimitive( PrimitiveElement pe,
 			p->setPen( cg.midlight() );
 			p->drawLine( i, r.top(), i, r.bottom() );
 			p->setPen( cg.background() );
-			p->drawLine( i + 1, r.top(), i, r.bottom() );
+			p->drawLine( i + 1, r.top(), i + 1, r.bottom() );
 			p->setPen( cg.mid() );
 			p->drawLine( i + 2, r.top(), i + 2, r.bottom() );
 		    }
@@ -466,7 +468,7 @@ void LightStyleV3::drawPrimitive( PrimitiveElement pe,
 			p->setPen( cg.midlight() );
 			p->drawLine( r.left(), i, r.right(), i );
 			p->setPen( cg.background() );
-			p->drawLine( r.left(), i + 1, r.right(), i );
+			p->drawLine( r.left(), i + 1, r.right(), i + 1);
 			p->setPen( cg.mid() );
 			p->drawLine( r.left(), i + 2, r.right(), i + 2 );
 		    }
@@ -834,8 +836,15 @@ void LightStyleV3::drawControl( ControlElement control,
 		p->drawLine( br.topLeft(), br.topRight() );
 		p->setPen( cg.dark() );
 		p->drawLine( br.right(), br.top() + 1, br.right(), br.bottom() );
-		br.addCoords( 1, 1, -1, 0 );
-		p->fillRect( br, cg.brush( QColorGroup::Background ) );
+
+		if ( flags & Style_Selected )
+		{
+                    p->fillRect( br.right() - 3, br.top() + 1, 3, br.height() - 1, cg.brush(QColorGroup::Highlight));
+		    br.addCoords( 1, 1, -4, 0 );
+		}
+		else
+		    br.addCoords( 1, 1, -1, 0 );
+		p->fillRect( br, cg.background() );
 	    } else if ( tb->shape() == QTabBar::RoundedBelow ) {
 		if ( ! ( flags & Style_Selected ) ) {
 		    p->setPen( cg.background() );
@@ -865,7 +874,16 @@ void LightStyleV3::drawControl( ControlElement control,
 		p->drawLine( br.bottomLeft(), br.bottomRight() );
 		p->drawLine( br.right(), br.top(), br.right(), br.bottom() - 1 );
 		br.addCoords( 1, 0, -1, -1 );
-		p->fillRect( br, cg.brush( QColorGroup::Background ) );
+
+		if ( flags & Style_Selected )
+		{
+		    p->fillRect( br.right() - 2, br.top(), 3, br.height(), cg.brush(QColorGroup::Highlight));
+		    br.addCoords( 1, 0, -3, -1 );
+		}
+		else
+		    br.addCoords( 1, 0, -1, -1 );
+
+		p->fillRect( br, cg.background() );
 	    } else
 		QCommonStyle::drawControl( control, p, widget, r, cg, flags, data );
 	    break;
@@ -882,8 +900,10 @@ void LightStyleV3::drawControl( ControlElement control,
 	    int maxpmw = data.maxIconWidth();
 
 	    if ( mi && mi->isSeparator() ) {
-		// draw separator
-		p->fillRect(r, cg.brush(QColorGroup::Button));
+            	if ( widget->erasePixmap() && !widget->erasePixmap()->isNull() )
+                	p->drawPixmap( r.topLeft(), *widget->erasePixmap(), r );
+		else
+			p->fillRect(r, cg.brush(QColorGroup::Button));
 		p->setPen( cg.mid() );
 		p->drawLine(r.left() + 12,  r.top() + 1,
 			    r.right() - 12, r.top() + 1);
@@ -894,8 +914,10 @@ void LightStyleV3::drawControl( ControlElement control,
 	    }
 
 	    if (flags & Style_Active)
-		qDrawShadePanel(p, r, cg, TRUE, 1,
+		qDrawShadePanel(p, r, cg, true, 1,
 				&cg.brush(QColorGroup::Midlight));
+	     else if ( widget->erasePixmap() && !widget->erasePixmap()->isNull() )
+		p->drawPixmap( r.topLeft(), *widget->erasePixmap(), r );
 	    else
 		p->fillRect(r, cg.brush(QColorGroup::Button));
 
@@ -921,11 +943,11 @@ void LightStyleV3::drawControl( ControlElement control,
 		tr = visualRect( tr, r );
 		ir = visualRect( ir, r );
 	    }
-	    
+
 	    if (mi->isChecked() &&
 		! (flags & Style_Active) &
 		(flags & Style_Enabled))
-		qDrawShadePanel(p, cr, cg, TRUE, 1, &cg.brush(QColorGroup::Midlight));
+		qDrawShadePanel(p, cr, cg, true, 1, &cg.brush(QColorGroup::Midlight));
 
 	    if (mi->iconSet()) {
 		QIconSet::Mode mode =
@@ -1024,7 +1046,7 @@ void LightStyleV3::drawControl( ControlElement control,
 			       p, sr, cg, flags);
 	    break;
 	}
-	
+
     case CE_MenuBarEmptyArea:
 	{
 	    p->fillRect(r, cg.brush(QColorGroup::Button));
@@ -1034,7 +1056,7 @@ void LightStyleV3::drawControl( ControlElement control,
     case CE_MenuBarItem:
 	{
 	    if ( flags & Style_Active )
-		qDrawShadePanel(p, r, cg, TRUE, 1, &cg.brush(QColorGroup::Midlight));
+		qDrawShadePanel(p, r, cg, true, 1, &cg.brush(QColorGroup::Midlight));
 	    else
 		p->fillRect( r, cg.brush( QColorGroup::Button ) );
 
@@ -1193,7 +1215,7 @@ void LightStyleV3::drawComplexControl( ComplexControl control,
 					   SC_SpinWidgetFrame, data);
 	    up = spinwidget->upRect();
 	    down = spinwidget->downRect();
-	    
+
 	    if ((controls & SC_SpinWidgetFrame) && frame.isValid())
 		drawPrimitive( PE_Panel, p, frame, cg, flags | Style_Sunken );
 
@@ -1331,9 +1353,9 @@ void LightStyleV3::drawComplexControl( ComplexControl control,
 		QColor grooveColor = cg.midlight();
 		if (!(flags & Style_Enabled))
 		    grooveColor = cg.background();
-            
-            
-		QBrush brush(grooveColor);                            
+
+
+		QBrush brush(grooveColor);
 		drawLightBevel( p, groove, cg,
 				( ( flags | Style_Raised ) ^ Style_Raised ) |
 				( ( flags & Style_Enabled ) ? Style_Sunken :
@@ -1351,10 +1373,10 @@ void LightStyleV3::drawComplexControl( ComplexControl control,
 	    if ((controls & SC_SliderHandle) && handle.isValid()) {
 		QColor sliderColor = cg.highlight();
 		if (!(flags & Style_Enabled))
-		    sliderColor = cg.button();                   
-		
+		    sliderColor = cg.button();
+
 		p->setPen( sliderColor.light() );
-                
+
 		p->drawLine( handle.topLeft(), handle.topRight() );
 		p->drawLine( handle.left(), handle.top() + 1,
 			     handle.left(), handle.bottom() - 1 );
@@ -1760,6 +1782,25 @@ QSize LightStyleV3::sizeFromContents( ContentsType contents,
 	    ret = QSize(w, h);
 	    break;
 	}
+	
+    case CT_ProgressBar:
+	{
+	    const QProgressBar* pb = static_cast<const QProgressBar*>(widget);
+	    
+	    //If we have to display the indicator, and we do it on RHS, give some more room
+	    //for it. This tries to match the logic and the spacing in SR_ProgressBarGroove/Contents
+	    //sizing in QCommonStyle.
+	    if (pb->percentageVisible() && 
+	        (pb->indicatorFollowsStyle() || ! pb->centerIndicator()))
+	    {
+		int addw = pb->fontMetrics().width("100%" + 6);
+		return QSize(contentsSize.width() + addw, contentsSize.height());
+	    }
+	    else
+	    	return contentsSize; //Otherwise leave unchanged
+	    
+	    break;
+	}    
 
     default:
 	ret = QCommonStyle::sizeFromContents(contents, widget, contentsSize, data);
