@@ -33,9 +33,9 @@ class MidiMapper;
  * send MIDI events to any device, as it creates and manages the *Out classes.
  *
  * This class is usually used by creating a DeviceManager object, then call
- * @ref #openDev() and @ref #initDev() . Then, use @ref #numberOfMidiPorts(),
- * @ref #numberOfSynthDevices(), @ref #name() and @ref #type() to choose which
- * device to play MIDI events to and then use @ref defaultDevice() to set the
+ * openDev() and initDev() . Then, use numberOfMidiPorts(),
+ * numberOfSynthDevices(), name() and type() to choose which
+ * device to play MIDI events to and then use defaultDevice() to set the
  * MIDI device to play.
  *
  * @short Manages all MIDI devices and redirects MIDI events to each one as
@@ -163,15 +163,15 @@ class DeviceManager
   public:
     /**
      * Constructor. It just initializes internal variables, before playing any
-     * music, you should call @ref #initManager(), @ref #setMidiMap()
-     * (optional), @ref #openDev(), @ref #initDev(), @ref #setPatchesToUse()
+     * music, you should call initManager(), setMidiMap()
+     * (optional), openDev(), initDev(), setPatchesToUse()
      * (not required, unless you're playing to a GUS device, which must load
-     * the patches), @ref #tmrStart(), and finally, play the music.
+     * the patches), tmrStart(), and finally, play the music.
      */
     DeviceManager(int def=-1);
 
     /**
-     * Destructor. It closes the device (calling @ref #closeDev() ) if it wasn't
+     * Destructor. It closes the device (calling closeDev() ) if it wasn't
      * closed before.
      */
     ~DeviceManager(void);
@@ -190,7 +190,7 @@ class DeviceManager
     int initManager(void);
 
     /**
-     * Checks if the device manager has been initialized (with @initManager),
+     * Checks if the device manager has been initialized (with @p initManager),
      * and in case it wasn't, initializes it.
      *
      * @return 0 if it was (or has just been) correctly initialized, and -1 if
@@ -199,7 +199,7 @@ class DeviceManager
     int checkInit(void);
 
     /**
-     * Obsolete. Please use @ref #deviceForChannel() instead.
+     * \obsolete Please use deviceForChannel() instead.
      *
      */
     MidiOut *chntodev(int chn)
@@ -237,8 +237,8 @@ class DeviceManager
      */
     int usingAlsa(void) { return alsa; };
 
-    // The following funtion are here to emulate a midi, so that the
-    // DeviceManager sends the events to the appropiate devices.
+    // The following function are here to emulate a midi, so that the
+    // DeviceManager sends the events to the appropriate devices.
 
     /**
      * Open the devices. It first initializes the manager it that wasn't done
@@ -246,22 +246,22 @@ class DeviceManager
      * device, as it will be set to an external synth by default, if available).
      *
      * Then /dev/sequencer is opened and the MIDI devices are opened
-     * (calling @ref MidiOut::openDev() ).
-     * @see #ok() to check if there was any problem
-     * @see #closeDev()
-     * @see #initDev()
+     * (calling MidiOut::openDev() ).
+     * @see ok() to check if there was any problem
+     * @see closeDev()
+     * @see initDev()
      */
     void openDev        (void);
 
     /**
      * Closes the devices, and /dev/sequencer.
      *
-     * @see #openDev()
+     * @see openDev()
      */
     void closeDev       (void);
 
     /**
-     * Calls @ref MidiOut::initDev() in turn in each of the available devices.
+     * Calls MidiOut::initDev() in turn in each of the available devices.
      *
      * @see MidiOut::initDev()
      */
@@ -274,7 +274,7 @@ class DeviceManager
      * @param note the key of the note to play (0 to 127).
      * @param vel the velocity of the note (0 to 127).
      *
-     * @see #noteOff()
+     * @see noteOff()
      */
     void noteOn         ( uchar chn, uchar note, uchar vel );
 
@@ -286,7 +286,7 @@ class DeviceManager
      * @param note the key of the note to play (0 to 127).
      * @param vel the velocity of the note (0 to 127).
      *
-     * @see #noteOn()
+     * @see noteOn()
      */
     void noteOff        ( uchar chn, uchar note, uchar vel );
 
@@ -327,6 +327,7 @@ class DeviceManager
      * @param lsb and @p msb the less significant byte and the most significant
      * byte (0 to 127 each) of the number by which notes will be bend. a 0x4000
      * value means not to bend.
+     * @param msb the most significant byte
      */
     void chnPitchBender ( uchar chn, uchar lsb,  uchar msb );
 
@@ -373,12 +374,12 @@ class DeviceManager
      void tmrSetTempo(int v);
 
     /**
-     * Starts the timer. You must call tmrStart before using @ref #wait()
+     * Starts the timer. You must call tmrStart before using wait()
      */
     void tmrStart(long int tpcn);
 
     /**
-     * Stops the timer. This will be called by @ref #closeDev() before closing
+     * Stops the timer. This will be called by closeDev() before closing
      * the device
      */
     void tmrStop(void);
@@ -389,9 +390,14 @@ class DeviceManager
      */
     void tmrContinue(void);
 
+    /**
+     * Sends an all notes off event
+     */
+    void allNotesOff(void);
+
     /**
      * Synchronizes with the MIDI buffer. Midi events are put into a buffer,
-     * along with timer delays (see @ref #wait() ). sync returns when the buffer
+     * along with timer delays (see wait() ). sync returns when the buffer
      * is empty.
      *
      * @param f if false, it syncronizes by waiting for the buffer to be sent.
@@ -417,24 +423,24 @@ class DeviceManager
      * Returns the device to which the MIDI events will be sent.
      * Returns -1 if there's no available device.
      *
-     * @see #setDefaultDevice()
+     * @see setDefaultDevice()
      */
     int defaultDevice(void);
 
     /**
      * Sets the device to send the MIDI events to.
      *
-     * By using @ref #midiPorts(), @ref #synthDevices(), @ref #name() and
-     * @ref #type(), you should choose which device to use (note that they are
+     * By using midiPorts(), synthDevices(), name() and
+     * type(), you should choose which device to use (note that they are
      * numbered with midi ports being first and synth devices next)
      *
-     * @see #defaultDevice()
+     * @see defaultDevice()
      */
     void setDefaultDevice(int i);
 
     /**
      * Loads the patches you're going to use . This has effect only for GUS
-     * cards, although, if you use this function when @ref defaultDevice() is
+     * cards, although, if you use this function when defaultDevice() is
      * not a GUS device, it will be ignored.
      *
      * The parameter is an int [256] array, which contain the following:
@@ -466,17 +472,17 @@ class DeviceManager
      * Returns the filename where the Midi Mapper was loaded from, or "" if no
      * MIDI Mapper is in use.
      *
-     * @see #setMidiMap()
+     * @see setMidiMap()
      */
     const char *midiMapFilename(void);
 
     /**
-     * Sets a @ref MidiMapper object to use. This object should already have
-     * loaded the configuration. See the description of @ref MidiMapper for
+     * Sets a MidiMapper object to use. This object should already have
+     * loaded the configuration. See the description of MidiMapper for
      * more information.
      *
      * @see MidiMapper::MidiMapper()
-     * @see #midiMapFilename()
+     * @see midiMapFilename()
      */
     void setMidiMap(MidiMapper *map);
 
@@ -491,8 +497,8 @@ class DeviceManager
      * connected to these ports, so sending MIDI events to these ports will not
      * produce any music in this case.
      *
-     * @see #synthDevices()
-     * @see #setDefaultDevice()
+     * @see synthDevices()
+     * @see setDefaultDevice()
      */
     int midiPorts(void) { return n_midi; };
 
@@ -501,17 +507,17 @@ class DeviceManager
      * of these devices will need special configuration, for example, to load
      * sound patches.
      *
-     * @see #midiPorts()
-     * @see #setDefaultDevice()
-     * @see #setPatchesToUse()
+     * @see midiPorts()
+     * @see setDefaultDevice()
+     * @see setPatchesToUse()
      */
     int synthDevices(void) { return n_synths; };
 
     /**
      * Returns the name of the @p i-th device . In case the DeviceManager wasn't
-     * yet initialized ( see @ref #checkInit() ), the return value is NULL, and
+     * yet initialized ( see checkInit() ), the return value is NULL, and
      * in case the parameter has a value out of the valid range ( 0 to
-     * @ref midiPorts() + @ref synthDevices() ) it returns an empty string.
+     * midiPorts() + synthDevices() ) it returns an empty string.
      */
     const char *name(int i);
 
