@@ -294,14 +294,14 @@ int
 lt_dladderror (diagnostic)
      const char *diagnostic;
 {
-  int		index	 = 0;
+  int		_index	 = 0;
   int		result	 = -1;
   const char  **temp     = (const char **) 0;
 
   MUTEX_LOCK ();
 
-  index	 = errorcount - LT_ERROR_MAX;
-  temp = LT_DLREALLOC (const char *, user_error_strings, 1 + index);
+  _index	 = errorcount - LT_ERROR_MAX;
+  temp = LT_DLREALLOC (const char *, user_error_strings, 1 + _index);
   if (temp == 0)
     {
       MUTEX_SETERROR (LT_DLSTRERROR (NO_MEMORY));
@@ -309,7 +309,7 @@ lt_dladderror (diagnostic)
   else
     {
       user_error_strings	= temp;
-      user_error_strings[index] = diagnostic;
+      user_error_strings[_index] = diagnostic;
       result			= errorcount++;
     }
 
@@ -319,20 +319,20 @@ lt_dladderror (diagnostic)
 }
 
 int
-lt_dlseterror (index)
-     int index;
+lt_dlseterror (_index)
+     int _index;
 {
   int		errors	 = 0;
 
   MUTEX_LOCK ();
 
-  if (index >= errorcount || index < 0)
+  if (_index >= errorcount || _index < 0)
     {
       /* Ack!  Error setting the error message! */
       MUTEX_SETERROR (LT_DLSTRERROR (INVALID_ERRORCODE));
       ++errors;
     }
-  else if (index < LT_ERROR_MAX)
+  else if (_index < LT_ERROR_MAX)
     {
       /* No error setting the error message! */
       MUTEX_SETERROR (lt_dlerror_strings[errorcount]);
@@ -457,7 +457,7 @@ strrchr(str, ch)
 
 /* NOTE:  Neither bcopy nor the memcpy implementation below can
           reliably handle copying in overlapping areas of memory, so
-          do not rely on this behaviour when invoking memcpy later.  */
+          do not rely on this behavior when invoking memcpy later.  */
 #if ! HAVE_MEMCPY
 
 #  if HAVE_BCOPY
@@ -791,7 +791,7 @@ static struct lt_user_dlloader sys_shl = {
 
 /* --- LOADLIBRARY() INTERFACE LOADER --- */
 
-#ifdef __WINDOWS__
+#if defined(__WINDOWS__) || defined(__CYGWIN__)
 
 /* dynamic linking for Win32 */
 
@@ -1309,7 +1309,7 @@ lt_dlinit ()
 #if HAVE_SHL_LOAD
       errors += lt_dlloader_add (lt_dlloader_next (0), &sys_shl, "dlopen");
 #endif
-#ifdef __WINDOWS__
+#if defined(__WINDOWS__) || defined(__CYGWIN__)
       errors += lt_dlloader_add (lt_dlloader_next (0), &sys_wll, "dlopen");
 #endif
 #ifdef __BEOS__
